#define DEBUG_2 1
#define DEBUG 0
#define DEBUG_large 0
/*
================================================================================
	D e X D D i a l . c
================================================================================

	Methods file for DeXDDial widget.

	Author : Jean-Francois Vibert fraom DeXDial from Denis LAMBOLEZ and JFV
		 b3e CHU ST-ANTOINE

	History: 
		 - Version 1.0: Septembre 1994
		   September, 5th 1994: Creation from DeXDial.c (Denis Lambolez et JFV)
		 - Version 2.0
		   October, 17 1994: Correction de bugs caches pour fonctionnement
					sur Sun solaris (JFV, EB)

================================================================================
*/

/*
================================================================================
	Header files and specials macros
================================================================================
*/

/*Standard library header files*/


#include <stdlib.h>

#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <ctype.h>

/*X11 header files*/
#include <X11/keysym.h>

/*Intrinsic library header files*/
#include <X11/IntrinsicP.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*Xt superclass header files*/
#include <X11/CoreP.h>
#include <X11/Core.h>
#include <X11/CompositeP.h>
#include <X11/Composite.h>
#include <X11/ConstrainP.h>
#include <X11/Constraint.h>

/*Motif library and superclass header files*/
#include <Xm/Xm.h>
#include <Xm/BulletinBP.h>
#include <Xm/BulletinB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>

#ifndef XtInheritGeoMatrixCreate
#define XtInheritGeoMatrixCreate ((XmGeoCreateProc) _XtInherit)
#endif
 
/*DeX library header files*/
#include "DeXInput.h"
#include "DeXDDialP.h"
#include "DeXDDial.h"

#ifndef M_PI
#  define M_PI 3.14159265358979323846
#endif

#define  RADIANS(x)  (M_PI * 2.0 * (x) / 360.0)
#define  DEGREES(x)  ((x) / (M_PI * 2.0) * 360.0)
#define  INDICATOR_WIDTH_RATIO 8
#define  MARKERS_RATIO 10.0
#define  INDICATOR_LENGTH_RATIO (8.0/7.0)
#define  MINIMUM(a,b)    (((a) < (b)) ? (a) :  (b))
#ifndef M_2PI
#	define M_2PI      6.2831853071795864769E0  /*Hex  2^ 2 * 1.921FB54442D18 */
#endif

#ifdef __cplusplus
extern "C"{
#endif
static void compute_indicator_pos(DeXDDialWidget);
/*
================================================================================
	Action function declaration
================================================================================
*/
static void choose_indicator(DeXDDialWidget, XEvent *, char *, int);
static void track_indicator(DeXDDialWidget, XEvent *, char *, int);
static void send_value(DeXDDialWidget, XEvent *, char *, int);
static void change_default(DeXDDialWidget, XEvent *, char *, int);
static void more_precise(DeXDDialWidget, XEvent *, char *, int);
static void less_precise(DeXDDialWidget, XEvent *, char *, int);
/*
================================================================================
	Method function declaration
================================================================================
*/

static void    Initialize(Widget, Widget, ArgList, Cardinal *);
static void    Redisplay(Widget, XEvent *, Region);
static void    Resize(Widget);
static void    Destroy(Widget);
static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
static XtGeometryResult GeometryManager(Widget, XtWidgetGeometry *, XtWidgetGeometry *);
/*
================================================================================
	Internal callback
================================================================================
*/

static void	ok_cb_input(DeXInputWidget, XtPointer, DeXinputCallbackStruct *); 
static void	increase_turn(Widget, XtPointer, XtPointer);
static void	decrease_turn(Widget, XtPointer, XtPointer);

#ifdef __cplusplus
}
#endif

static	float	base_value = 0.,
		old_old_value = -1.,
		old_value = 0.;
static	long	turn_value = 10000;
static	char	LeLabel[60];
static	Boolean	on_left,
		inner_dial,
		redraw = TRUE;
static	Boolean	limit_reached = False;
/*
================================================================================
	Ressource list
================================================================================
*/

static XtResource resources[] = 
{
  	{
	XmNshadowThickness, XmCShadowThickness, XtRDimension, sizeof (Dimension),
    	XtOffset(DeXDDialWidget, manager.shadow_thickness), XtRImmediate, (XtPointer)2
	},
  	{
	DeXNindicatorColor, DeXCIndicatorColor, XtRPixel, sizeof (Pixel),
    	XtOffset(DeXDDialWidget, dial.indicator_color), XtRString, XtExtdefaultforeground
	},
  	{
	DeXNdefaultValue, DeXCDefaultValue, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.default_value_int), XtRImmediate, (XtPointer)100
	},
  	{
	DeXNturnStep, DeXCTurnStep, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.turn_step_int), XtRImmediate, (XtPointer)10000
	},
  	{
	DeXNvalue, DeXCValue, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.value_int), XtRImmediate, (XtPointer)5000
	},
  	{
	DeXNincrement, DeXCIncrement, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.increment_int), XtRImmediate, (XtPointer)100
	},
  	{
	DeXNminimum, DeXCMinimum, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.minimum_int), XtRImmediate,
	(XtPointer)0
	},
  	{
	DeXNmaximum, DeXCMaximum, XtRInt, sizeof (long),
    	XtOffset(DeXDDialWidget, dial.maximum_int), XtRImmediate,
	(XtPointer)10000
	},
  	{
	DeXNmarkersNumber, DeXCMarkersNumber, XtRInt, sizeof (int),
    	XtOffset(DeXDDialWidget, dial.nb_markers), XtRImmediate, (XtPointer)20
	},
  	{
	DeXNdecimalPoint, DeXCDecimalPoint, XtRInt, sizeof (int),
    	XtOffset(DeXDDialWidget, dial.decimal_point), XtRImmediate, (XtPointer)2  
	},
  	{
	DeXNlabelString, DeXCLabelString, XtRString, sizeof (char *), 
/*	DeXNlabelString, DeXCLabelString, XtRString, 60, */
    	XtOffset(DeXDDialWidget, dial.label_string), XtRString,"DeXDDial label"
	},
  	{
	DeXNchangeTurn, DeXCChangeTurn, XtRBoolean,	sizeof (Boolean),
    	XtOffset(DeXDDialWidget, dial.change_turn), XtRBoolean, False 
	},
  	{
	DeXNvalueChangedCallback, DeXCValueChangedCallback, XtRCallback, 
	sizeof (XtPointer),
    	XtOffset(DeXDDialWidget,dial.value_changed), XtRCallback, NULL 
	},
  	{
	DeXNdragCallback, DeXCDragCallback, XtRCallback, 
	sizeof (XtPointer),
    	XtOffset(DeXDDialWidget, dial.drag), XtRCallback, NULL 
	},
};

/*
================================================================================
	Conversion function
================================================================================
*/
static long to_integer_external_format(double value, int decimal_point)
{
	for(; decimal_point > 0; decimal_point--)
		value *= 10.0;
	return((long)value);
}

static double to_float_internal_format(long value, int decimal_point)
{
	double f_value = (double)value;
	
	for(; decimal_point > 0; decimal_point--)
		f_value /= 10.0;
	return(f_value);
}


/*
================================================================================
	Actions table
================================================================================
*/
static XtActionsRec actionList[] =
{
	{"choose_indicator", (XtActionProc) choose_indicator},
	{"track_indicator", (XtActionProc) track_indicator},
	{"send_value", (XtActionProc) send_value},
	{"less_precise",(XtActionProc) less_precise},
	{"more_precise",(XtActionProc) more_precise},
	{"change_default",(XtActionProc) change_default}
};

/*
================================================================================
	Default translation table
================================================================================
*/
static char defaultTranslations[] =
	"<BtnDown>:	choose_indicator() 	\n\
	 <BtnUp>:	send_value()	\n\
	 <BtnMotion>:	track_indicator()	\n\
	 Shift<Btn2Down>:	less_precise()	\n\
	 Shift<Btn3Down>:	more_precise()\n\
	 Shift<Btn1Down>:	change_default()";

/*
================================================================================
	Class record initialization
================================================================================
*/

DeXDDialClassRec  DeXdDialClassRec = 
{
     /* CoreClassPart */
  	{
   	(WidgetClass) &xmBulletinBoardClassRec,	/* superclass            */
   	"DeXDDial",				/* class_name            */
   	sizeof(DeXDDialRec),			/* widget_size           */
   	NULL,					/* class_initialize      */
   	NULL,					/* class_part_initialize */
   	FALSE,					/* class_inited          */
   	Initialize,				/* initialize            */
   	NULL,					/* initialize_hook       */
   	XtInheritRealize,			/* realize               */
   	actionList,				/* actions               */
   	XtNumber(actionList),			/* num_actions           */
   	resources,				/* resources             */
   	XtNumber(resources),			/* num_resources         */
   	NULLQUARK,				/* xrm_class             */
   	TRUE,					/* compress_motion       */
   	TRUE,					/* compress_exposure     */
   	TRUE,					/* compress_enterleave   */
   	TRUE,					/* visible_interest      */
   	Destroy,				/* destroy               */
   	Resize,					/* resize                */
   	Redisplay,				/* expose                */
   	SetValues,				/* set_values            */
   	NULL,					/* set_values_hook       */
   	XtInheritSetValuesAlmost,		/* set_values_almost     */
   	NULL,					/* get_values_hook       */
   	NULL,					/* accept_focus          */
   	XtVersion,				/* version               */
   	NULL,					/* callback private      */
   	defaultTranslations,			/* tm_table              */
   	NULL,					/* query_geometry        */
   	NULL,					/* display_accelerator   */
   	NULL,					/* extension             */
   	},
     /* CompositeClassPart */
  	{
	XtInheritGeometryManager,		/* geometry_manager		*/
	NULL,					/* change_managed		*/
	XtInheritInsertChild,			/* insert_child			*/
	XtInheritDeleteChild,			/* delete_child			*/
	NULL					/* pointer to extension record  */
   	},
     /* ConstraintClassPart */
  	{
	NULL,					/* constraint resource list	*/
	0,					/* number of constraint in list */
	sizeof(XmManagerConstraintRec),		/* size of constraint record    */
	NULL,					/* constraint initialization	*/
	NULL,					/* constraint destroy proc      */
	0,					/* constraint set values proc   */
	NULL					/* pointer to extension record  */
   	},
     /* ManagerClassPart */
  	{
	XtInheritTranslations,			/* translation table			*/
	NULL,					/* synthetic resources			*/
	0,					/* nb of synthetic resources		*/
	NULL,					/* synthetic constraint resources	*/
	0,					/* nb of synthetic constraint resources */
	XmInheritParentProcess,			/* parent_proc				*/
	NULL					/* pointer to extension record		*/
   	},
     /* BulletinBoardClassPart */
  	{
	TRUE,					/* always install accelerator		*/
	XtInheritGeoMatrixCreate,		/* goemetry matrix create procedure	*/
	XmInheritFocusMovedProc,		/* focus moved procedure		*/
	NULL					/* pointer to extension record		*/
   	},
	/* DialClassPart */
  	{
   	0,					/* ignore				*/
   	}
};

/*shadow record*/
WidgetClass DeXdDialWidgetClass = (WidgetClass) &DeXdDialClassRec; 

/*
================================================================================
	Method function definition
================================================================================
*/

static void Initialize (Widget request_in, Widget New_in,
			ArgList arglist, Cardinal *num_arg)
{
	DeXDDialWidget request = (DeXDDialWidget)request_in;
	DeXDDialWidget New = (DeXDDialWidget)New_in;
	XGCValues values;
	XmString xmstring;
	Arg arg[20];
	XtGCMask  valueMask;
	GC gc;
	Pixel background, foreground, top_shadow, bottom_shadow, select_color;
	int argcount;
	char *tmp_buff;
	char string[80], format[80];
/*
Make sure the window size is not zero. The Core 
Initialize() method doesn't do this.
*/
  	if (request->core.width <= 0)
    		New->core.width = 225;
  	if (request->core.height <= 0)
    		New->core.height = 225;

/*
Make sure decimal point is valid
*/
	if(New->dial.decimal_point < 0)
	{
    		XtWarning("DeXDDial: Decimal Point must be positive");
		New->dial.decimal_point = 2;
	}
/*
Make sure there is an increment
*/
	if(New->dial.increment_int == 0)
	{
		New->dial.increment_int = 100;
		sprintf(string,"DeXDDial: increment must be non zero: set to %d\n",
		       New->dial.increment_int / pow(10., (double)(New->dial.decimal_point)));
    		XtWarning(string);
	}

/*
Make sure the value, the min and max dial settings are valid.
*/
		
  	if (New->dial.minimum_int >= New->dial.maximum_int) 
	{
    		XtWarning("DeXDDial: Maximum must be greater than the Minimum");
    		New->dial.minimum_int = New->dial.maximum_int - 10;
 	}

  	if (New->dial.value_int > New->dial.maximum_int) 
    		New->dial.value_int =  New->dial.maximum_int;
  	if (New->dial.value_int < New->dial.minimum_int) 
    		New->dial.value_int =  New->dial.minimum_int;

/*
Make sure turn step is valid
*/
	if(New->dial.turn_step_int <= 0)
	{
    		XtWarning("DeXDDial: Turn Step must be positive");
		New->dial.turn_step_int = 10000;
	}

/*
Make sure markers number is valid
*/
	if(New->dial.nb_markers <= 0)
	{
    		XtWarning("DeXDDial: Markers number must be positive");
		New->dial.nb_markers = 20;
	}

/*
compute angle increment for markers
*/
  	New->dial.angle_increment = M_2PI / (float)(New->dial.nb_markers);
  	New->dial.large_angle_increment = (3. * M_2PI / 4.) / (float)(New->dial.nb_markers);

/*
compute normalized_pos 
*/

	New->dial.normalized_pos  = 0.;
/*
Convert to internal format
*/

	New->dial.minimum = to_float_internal_format(New->dial.minimum_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("Initialize: w->dial.minimum = %f\n", New->dial.minimum);
#endif						     

	New->dial.maximum = to_float_internal_format(New->dial.maximum_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("Initialize: w->dial.maximum = %f\n", New->dial.maximum);
#endif

	New->dial.turn_step = to_float_internal_format(New->dial.turn_step_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("Initialize: w->dial.turn_step = %f\n", New->dial.turn_step);
#endif

	New->dial.value = to_float_internal_format(New->dial.value_int,
						     New->dial.decimal_point);

	New->dial.one_turn_value = to_float_internal_format(New->dial.turn_step_int,
						     New->dial.decimal_point);

	New->dial.default_value = to_float_internal_format(New->dial.default_value_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("Initialize: w->dial.default_value = %f\n", New->dial.default_value);
#endif						     

	New->dial.increment = to_float_internal_format(New->dial.increment_int,
						     New->dial.decimal_point);

	if(New->dial.turn_step + New->dial.minimum > New->dial.maximum)
	{
		New->dial.turn_step = New->dial.maximum - New->dial.minimum;
		New->dial.turn_step_int = to_integer_external_format(New->dial.turn_step,
							       New->dial.decimal_point);
	}
/*
Allocs memory for markers
*/
  	if((New->dial.segments = (XSegment *)calloc((New->dial.nb_markers * 2 + 1),
	   sizeof(XSegment))) == NULL)
    		XtError ("DeXDDial: Too many markers, can't allocate memory.");

/*
Position the Indicator at the bottom
*/

	New->dial.angle = 1.5000001;
	New->dial.large_angle = 1.250000001;
	New->dial.large_normalized_pos = .125;
	New->dial.normalized_pos = 0.;
/*
Create childs
*/
	/* XmLabel*/
	
	if(New->dial.label_string == NULL)
	{
		New->dial.label_string = LeLabel; 
		/* default value for the label string */
		strcpy(New->dial.label_string, "DeXDDial label"); 
	}
	else
	{
		/* store in a static area the label string */

		if ( strlen(New->dial.label_string) > 58)
			strncpy(LeLabel,New->dial.label_string,58);
		else
			strcpy(LeLabel, New->dial.label_string); 

		New->dial.label_string = LeLabel;
	}
	LeLabel[59] = '\0';
	xmstring = XmStringCreateSimple(New->dial.label_string);
	New->dial.label_height = XmStringHeight(BB_LabelFontList(New), xmstring);


	argcount = 0;
	if(New->dial.label_string != NULL)
	{
		XtSetArg(arg[argcount], XmNlabelString, xmstring); argcount++;
		XtSetArg(arg[argcount], XmNrecomputeSize, True); argcount++;
		XtSetArg(arg[argcount], XmNheight, New->dial.label_height); argcount++;
	}
	XtSetArg(arg[argcount], XmNlabelType, XmSTRING); argcount++;
	XtSetArg(arg[argcount], XmNborderWidth, 0); argcount++;
	XtSetArg(arg[argcount], XmNborderColor, New->core.background_pixel); argcount++;
	XtSetArg(arg[argcount], XmNwidth, New->core.width - 2 * BB_MarginWidth(New)); argcount++;
	XtSetArg(arg[argcount], XmNx, BB_MarginWidth(New) * 2); argcount++;
	XtSetArg(arg[argcount], XmNy, (BB_MarginHeight(New) - 5)); argcount++;
	XtSetArg(arg[argcount], XmNalignment, XmALIGNMENT_BEGINNING); argcount++;
	XtSetArg(arg[argcount], XmNfontList, BB_LabelFontList(New)); argcount++;
	XtSetArg(arg[argcount], XmNforeground, New->manager.foreground); argcount++;
	XtSetArg(arg[argcount], XmNbackground, New->core.background_pixel); argcount++;
	New->dial.children[DEXDDIAL_LABEL] = (Widget) XmCreateLabel((Widget)New, "DeXDDial_Label",
							arg, argcount);
	XmStringFree(xmstring);		

	/* DeXInput */
	xmstring = XmStringCreateSimple("12345678901234567890");
	New->dial.text_height = XmStringHeight(BB_TextFontList(New), xmstring) + 66; 
	XmStringFree(xmstring);
	sprintf(format, "Turn = %%.%df", New->dial.decimal_point);
	sprintf(string,format, New->dial.one_turn_value);

	argcount = 0;
	XtSetArg(arg[argcount], XmNverifyBell, FALSE); argcount++;
	XtSetArg(arg[argcount], XmNcolumns, 1); argcount++;
	XtSetArg(arg[argcount], XmNx, BB_MarginWidth(New)); argcount++;
	XtSetArg(arg[argcount], XmNalignment, XmALIGNMENT_BEGINNING); argcount++;
	XtSetArg(arg[argcount], XmNy, New->core.height - BB_MarginHeight(New) -
			        New->dial.text_height + 10); argcount++;
	XtSetArg(arg[argcount], XmNheight, New->dial.text_height); argcount++;
	XtSetArg(arg[argcount], XmNwidth, New->core.width - 2 * BB_MarginWidth(New)); argcount++;
	XtSetArg(arg[argcount], XmNbackground, New->core.background_pixel); argcount++;
/*
	XtSetArg(arg[argcount], XmNhighlightColor, New->manager.foreground); argcount++;
*/
	XtSetArg(arg[argcount], DeXNlabelString, string); argcount++;
	XtSetArg(arg[argcount], DeXNdecimalPoint, New->dial.decimal_point); argcount++;
	XtSetArg(arg[argcount], DeXNvalue, New->dial.value_int); argcount++;
	XtSetArg(arg[argcount], DeXNdefaultValue, New->dial.default_value_int); argcount++;
	XtSetArg(arg[argcount], DeXNincrement, New->dial.increment_int); argcount++;

	New->dial.children[DEXDDIAL_INPUT] = (Widget) XtCreateWidget("DeXDDial_Input",
								   DeXinputWidgetClass,
								   (Widget) New, 
								   arg, argcount);

/*
Initialize button multiply child
*/

/*
Create GC and Pixmaps for buttons
*/
	gc = XCreateGC(XtDisplay(New), RootWindowOfScreen(DefaultScreenOfDisplay(XtDisplay(New))),
		       0, NULL);
	New->dial.label_pixmap_mult_off = XCreatePixmap(XtDisplay(New),
				     RootWindowOfScreen(DefaultScreenOfDisplay(XtDisplay(New))),
				     16, 16,
				     DefaultDepthOfScreen(DefaultScreenOfDisplay(XtDisplay(New))));
	New->dial.label_pixmap_div_off = XCreatePixmap(XtDisplay(New),
				     RootWindowOfScreen(DefaultScreenOfDisplay(XtDisplay(New))),
				     16, 16,
				     DefaultDepthOfScreen(DefaultScreenOfDisplay(XtDisplay(New))));
	New->dial.label_pixmap_mult_on= XCreatePixmap(XtDisplay(New),
				     RootWindowOfScreen(DefaultScreenOfDisplay(XtDisplay(New))),
				     16, 16,
				     DefaultDepthOfScreen(DefaultScreenOfDisplay(XtDisplay(New))));
	New->dial.label_pixmap_div_on = XCreatePixmap(XtDisplay(New),
				     RootWindowOfScreen(DefaultScreenOfDisplay(XtDisplay(New))),
				     16, 16,
				     DefaultDepthOfScreen(DefaultScreenOfDisplay(XtDisplay(New))));
	
#if DEBUG
	printf("Pixmap label_pixmap_mult_off : 0x%x\nPixmap label_pixmap_div_off : 0x%x\n\
Pixmap label_pixmap_mult_on : 0x%x\nPixmap label_pixmap_div_on : 0x%x\n",
	New->dial.label_pixmap_mult_off, New->dial.label_pixmap_div_off,
	New->dial.label_pixmap_mult_on, New->dial.label_pixmap_div_on);
#endif
	/* get background color from parent */
	argcount = 0;
	XtSetArg(arg[argcount], XmNbackground, &background); argcount++;
	XtGetValues(New->core.parent, arg, argcount);
	/* compute other colors */
	XmGetColors(DefaultScreenOfDisplay(XtDisplay(New)), New->core.colormap, background,
		    &foreground, &top_shadow, &bottom_shadow, &select_color);

	if (New->dial.change_turn)
	{
		/* XmPushButton Mult */
		XSetLineAttributes(XtDisplay(New), gc, 2, LineSolid, CapButt, JoinMiter);
		XSetForeground(XtDisplay(New), gc, background);
		XFillRectangle(XtDisplay(New), New->dial.label_pixmap_mult_off, gc, 0, 0, 16, 16);
		XSetForeground(XtDisplay(New), gc, foreground);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_mult_off, gc, 4, 4, 12, 12);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_mult_off, gc, 4, 12, 12, 4);
		XSetForeground(XtDisplay(New), gc, select_color);
		XFillRectangle(XtDisplay(New), New->dial.label_pixmap_mult_on, gc, 0, 0, 16, 16);
		XSetForeground(XtDisplay(New), gc, foreground);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_mult_on, gc, 4, 4, 12, 12);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_mult_on, gc, 4, 12, 12, 4);
		argcount = 0;
		XtSetArg(arg[argcount], XmNx, (BB_MarginWidth(New) * 2)); argcount++;
		XtSetArg(arg[argcount], XmNy, (New->dial.center_y - 25)); argcount++;
		XtSetArg(arg[argcount], XmNheight, 20); argcount++;
		XtSetArg(arg[argcount], XmNwidth, 20); argcount++;
		XtSetArg(arg[argcount], XmNlabelType, XmPIXMAP); argcount++;
		XtSetArg(arg[argcount], XmNlabelPixmap, New->dial.label_pixmap_mult_off); argcount++;
		XtSetArg(arg[argcount], XmNarmPixmap, New->dial.label_pixmap_mult_on); argcount++;
		XtSetArg(arg[argcount], XmNfontList, BB_TextFontList(New)); argcount++;
		New->dial.children[DEXDDIAL_BUT_MULT] = (Widget) XmCreatePushButton((Widget)New, "DeXDDial_Button_Mult",
									   arg, argcount);

		/* XmPushButton Div */
		XSetForeground(XtDisplay(New), gc, background);
		XFillRectangle(XtDisplay(New), New->dial.label_pixmap_div_off, gc, 0, 0, 16, 16);
		XSetForeground(XtDisplay(New), gc, foreground);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_off, gc, 4, 8, 12, 8);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_off, gc, 7, 11, 9, 11);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_off, gc, 7, 5, 9, 5);
		XSetForeground(XtDisplay(New), gc, select_color);
		XFillRectangle(XtDisplay(New), New->dial.label_pixmap_div_on, gc, 0, 0, 16, 16);
		XSetForeground(XtDisplay(New), gc, foreground);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_on, gc, 4, 8, 12, 8);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_on, gc, 7, 11, 9, 11);
		XDrawLine(XtDisplay(New), New->dial.label_pixmap_div_on, gc, 7, 5, 9, 5);
		argcount = 0;
		XtSetArg(arg[argcount], XmNx, (BB_MarginWidth(New) * 2)); argcount++;
		XtSetArg(arg[argcount], XmNy, (New->dial.center_y + 5)); argcount++;
		XtSetArg(arg[argcount], XmNheight, 20); argcount++;
		XtSetArg(arg[argcount], XmNwidth, 20); argcount++;
		XtSetArg(arg[argcount], XmNlabelType, XmPIXMAP); argcount++;
		XtSetArg(arg[argcount], XmNlabelPixmap, New->dial.label_pixmap_div_off); argcount++;
		XtSetArg(arg[argcount], XmNarmPixmap, New->dial.label_pixmap_div_on); argcount++;
		New->dial.children[DEXDDIAL_BUT_DIV] = (Widget) XmCreatePushButton((Widget)New, "DeXDDial_Button_Div",
									   arg, argcount);
		XtAddCallback(New->dial.children[DEXDDIAL_BUT_MULT], XmNactivateCallback, (XtCallbackProc) increase_turn, NULL);
		XtAddCallback(New->dial.children[DEXDDIAL_BUT_DIV], XmNactivateCallback, (XtCallbackProc) decrease_turn, NULL);
	} /* fin du if */
	

/*
Add Internals Callbacks
*/
	XtAddCallback(New->dial.children[DEXDDIAL_INPUT], DeXNokCallback, (XtCallbackProc) ok_cb_input, NULL);

/*
compute bottom and top shadow colors
*/
	XmGetColors(DefaultScreenOfDisplay(XtDisplay(New)), New->core.colormap,
		    New->dial.indicator_color, NULL, &top_shadow, &bottom_shadow, NULL);
/*
Create the graphics contexts used for the dial face 
and the indicator.
*/
  	valueMask = GCForeground | GCBackground;
  	values.foreground = New->manager.foreground;
  	values.background = New->core.background_pixel;
  	New->dial.dial_GC = XtGetGC ((Widget)New, valueMask, &values);  

	New->dial.inverse_dial_GC = XtGetGC ((Widget)New, valueMask, &values);  

  	valueMask = GCForeground | GCBackground;
  	values.foreground = New->dial.indicator_color;
  	values.background = New->core.background_pixel;
  	New->dial.indicator_GC = XtGetGC ((Widget)New, valueMask, &values);  

  	valueMask = GCForeground | GCBackground | GCLineWidth;
  	values.foreground = top_shadow;
  	values.background = New->core.background_pixel;
	values.line_width = New->manager.shadow_thickness;
	New->dial.top_shadow_GC = XtGetGC ((Widget)New, valueMask, &values);  

	valueMask = GCForeground | GCBackground | GCLineWidth;
  	values.foreground = bottom_shadow;
  	values.background = New->core.background_pixel;
	values.line_width = New->manager.shadow_thickness;
  	New->dial.bottom_shadow_GC = XtGetGC ((Widget)New, valueMask, &values);  

	valueMask = GCForeground | GCBackground | GCLineWidth;
  	values.foreground = New->core.background_pixel;
  	values.background = New->core.background_pixel;
	values.line_width = New->manager.shadow_thickness;
  	New->dial.inverse_GC = XtGetGC ((Widget)New, valueMask, &values);   
  	New->dial.inverse_shadow_GC = XtGetGC ((Widget)New, valueMask, &values);   

/*
Manage childs
*/
	XtManageChild(New->dial.children[DEXDDIAL_LABEL]);
	if (New->dial.change_turn)
	{
		XtManageChild(New->dial.children[DEXDDIAL_BUT_MULT]);
		XtManageChild(New->dial.children[DEXDDIAL_BUT_DIV]);
	} /* fin du if */
	XtManageChild(New->dial.children[DEXDDIAL_INPUT]);

	Resize ((Widget)New);
#if 0
/*
Initialize DeXInput child
*/
	sprintf(format, "Turn = %%.%df           ", New->dial.decimal_point);
	sprintf(string,format, New->dial.one_turn_value);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNdecimalPoint, New->dial.decimal_point, NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNvalue, New->dial.value_int, NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNdefaultValue, New->dial.default_value_int, NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNincrement, New->dial.increment_int, NULL);
#endif
/*
	compute_indicator_pos(New);
	Redisplay((Widget)New, NULL, NULL);
*/
#if DEBUG
	printf("Dial Widget initialized\n"); 
#endif
}

/*============================================================================*/
/*	Now we setup the values read in the ressources values                 */
/*============================================================================*/

static Boolean SetValues (Widget current_in, Widget request_in, 
			  Widget New_in, ArgList arglist, Cardinal *num_arg)
{
	DeXDDialWidget current = (DeXDDialWidget)current_in;
	DeXDDialWidget request = (DeXDDialWidget)request_in;
	DeXDDialWidget New = (DeXDDialWidget)New_in;
	XmString	xmstring;
	Arg arg[10];
	Pixel  top_shadow, bottom_shadow;
	XGCValues  values;
  	XtGCMask   valueMask;
	DeXdDialCallbackStruct cb;
	int argcount, nb_tours;
	float	New_normalized_pos;
	char *tmp_buff, string[80], format[80];
	static char	old_label[60],
		new_label[60];
/*
Make sure decimal point is valid
*/
	if(New->dial.decimal_point < 0)
	{
    		XtWarning("DeXDDial: Decimal Point must be positive");
		New->dial.decimal_point = 2;
	}


	New->dial.default_value = to_float_internal_format(New->dial.default_value_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("Initialize: w->dial.default_value = %f\n", New->dial.default_value);
#endif						     

	New->dial.increment = to_float_internal_format(New->dial.increment_int,
						     New->dial.decimal_point);
/*
Make sure the value, the min and max dial settings are valid.
*/
		
  	if (New->dial.minimum_int >= New->dial.maximum_int) 
	{
    		XtWarning("DeXDDial: Maximum must be greater than the Minimum");
    		New->dial.minimum_int = New->dial.maximum_int - 1;
 	}

  	if (New->dial.value_int > New->dial.maximum_int) 
    		New->dial.value_int =  New->dial.maximum_int;
  	if (New->dial.value_int < New->dial.minimum_int) 
    		New->dial.value_int =  New->dial.minimum_int;

/*
Make sure turn step is valid
*/
	if(New->dial.turn_step_int <= 0)
	{
    		XtWarning("DeXDDial: Turn Step must be positive");
		New->dial.turn_step_int = 10000;
	}
	New->dial.one_turn_value = to_float_internal_format(New->dial.turn_step_int,
						     New->dial.decimal_point);
/*
Make sure markers number is valid
*/
	if(New->dial.nb_markers <= 0)
	{
    		XtWarning("DeXDDial: Markers number must be positive");
		New->dial.nb_markers = 20;
	}

/*
compute angle increment for markers if needed and realloc segments
*/
	if(New->dial.nb_markers != current->dial.nb_markers)
	{
		New->dial.angle_increment = M_2PI / (float)(New->dial.nb_markers);
		New->dial.large_angle_increment = (3. * M_2PI / 4.) / (float)(New->dial.nb_markers);
		free(current->dial.segments);
		/* Allocs memory for markers */
		if((New->dial.segments = (XSegment *)calloc((New->dial.nb_markers * 2 + 1),
							    sizeof(XSegment))) == NULL)
			XtError ("DeXDDial: Too many markers, can't allocate memory.");
		redraw = TRUE;
	}



/*	BUG: pas possible de changer le label une fois le widget realise */

	/* if the label has been modified */
	if(strcmp(current->dial.label_string, New->dial.label_string))
	{

		/* store in a static area the label string */

		strcpy(old_label, current->dial.label_string);
		current->dial.label_string = old_label;

		strcpy(new_label, New->dial.label_string);
		New->dial.label_string = new_label;
		new_label[59] = '\0';

/*
		printf("1 Changing %s to %s\n", current->dial.label_string, New->dial.label_string); 

		strcpy(current->dial.label_string, New->dial.label_string);

		printf("2: Changing %s to %s\n", current->dial.label_string, New->dial.label_string); 
*/
		/* change the label */
		xmstring = XmStringCreateLocalized(New->dial.label_string);
		New->dial.label_height = XmStringHeight(BB_LabelFontList(New), xmstring);

/*		printf("avant XtVaSetValues\n"); */
		
		XtVaSetValues(New->dial.children[DEXDDIAL_LABEL],
			      XmNlabelType, XmSTRING,
			      XmNlabelString, xmstring,
			      NULL);
		XmStringFree(xmstring);
	}

/*
Convert to internal format
*/

	New->dial.minimum = to_float_internal_format(New->dial.minimum_int,
						     New->dial.decimal_point);
	New->dial.maximum = to_float_internal_format(New->dial.maximum_int,
						     New->dial.decimal_point);
	New->dial.value = to_float_internal_format(New->dial.value_int,
						     New->dial.decimal_point);

	if(New->dial.turn_step + New->dial.minimum > New->dial.maximum)
	{
		New->dial.turn_step = New->dial.maximum - New->dial.minimum;
		New->dial.turn_step_int = to_integer_external_format(New->dial.turn_step,
							       New->dial.decimal_point);
	}
	New->dial.one_turn_value = to_float_internal_format(New->dial.turn_step_int,
						     New->dial.decimal_point);
#if DEBUG
	printf("SetValue: w->dial.minimum = %f\n", New->dial.minimum);
	printf("SetValue: w->dial.maximum = %f\n", New->dial.maximum);
	printf("SetValue: w->dial.one_turn_value = %f\n", New->dial.one_turn_value);
#endif						     

/* if decimal point or value have changed.*/
	if(current->dial.decimal_point != New->dial.decimal_point ||
	   current->dial.value != New->dial.value)
	{
#if DEBUG_2
		/* construct the format */
		sprintf(format, "%%.%df", New->dial.decimal_point);
		sprintf(string, format, New->dial.value);
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNvalue, 0, NULL);
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNvalue, New->dial.value_int, NULL);
		sprintf(format, "Turn = %%.%df", New->dial.decimal_point);
		sprintf(string,format, New->dial.one_turn_value);

/*		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
			      New->manager.foreground);
*/
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNdecimalPoint,
			      New->dial.decimal_point, NULL);
#endif
	}
/*
If the indicator color or background color 
has changed, generate the GC's.
*/
 	if(New->dial.indicator_color!=current->dial.indicator_color ||
  	   New->core.background_pixel != current->core.background_pixel ||
	   New->manager.shadow_thickness != current->manager.shadow_thickness)
	{
		/*compute bottom and top shadow colors*/
		XmGetColors(DefaultScreenOfDisplay(XtDisplay(New)), New->core.colormap,
			    New->dial.indicator_color, NULL, &top_shadow, &bottom_shadow, NULL);
		
		valueMask = GCForeground | GCBackground;
		values.foreground = New->dial.indicator_color;
		values.background = New->core.background_pixel;
    		XtReleaseGC((Widget)New, New->dial.indicator_GC);
		New->dial.indicator_GC = XtGetGC ((Widget)New, valueMask, &values);  
		
		valueMask = GCForeground | GCBackground | GCLineWidth;
		values.foreground = top_shadow;
		values.background = New->core.background_pixel;
		values.line_width = New->manager.shadow_thickness;
    		XtReleaseGC((Widget)New, New->dial.top_shadow_GC);
		New->dial.top_shadow_GC = XtGetGC ((Widget)New, valueMask, &values);  
		
		valueMask = GCForeground | GCBackground | GCLineWidth;
		values.foreground = bottom_shadow;
		values.background = New->core.background_pixel;
		values.line_width = New->manager.shadow_thickness;
    		XtReleaseGC((Widget)New, New->dial.bottom_shadow_GC);
		New->dial.bottom_shadow_GC = XtGetGC ((Widget)New, valueMask, &values);  

		valueMask = GCForeground | GCBackground | GCLineWidth;
		values.foreground = New->core.background_pixel;
		values.background = New->core.background_pixel;
		values.line_width = New->manager.shadow_thickness;
    		XtReleaseGC((Widget)New, New->dial.inverse_GC);
		New->dial.inverse_GC = XtGetGC ((Widget)New, valueMask, &values);   

		redraw = TRUE;     
  	}

/*
If the marker color has changed, regenerate the GC.
*/
  	if (New->manager.foreground != current->manager.foreground ||
  	    New->core.background_pixel != current->core.background_pixel)
	{
		argcount = 0;
		XtSetArg(arg[argcount], XmNforeground, New->manager.foreground); argcount++;
		XtSetArg(arg[argcount], XmNbackground, New->core.background_pixel); argcount++;
		XtSetValues(New->dial.children[DEXDDIAL_INPUT], arg, argcount);
		XtSetValues(New->dial.children[DEXDDIAL_LABEL], arg, argcount);
		
		valueMask = GCForeground | GCBackground;
    		values.foreground = New->manager.foreground;
    		values.background = New->core.background_pixel;
    		XtReleaseGC((Widget)New, New->dial.dial_GC);
    		New->dial.dial_GC = XtGetGC ((Widget)New, valueMask, &values);   
    		redraw = TRUE;     
  	}


	New->dial.value_int = to_integer_external_format(New->dial.value, New->dial.decimal_point);
	nb_tours = (int)(New->dial.value_int / New->dial.turn_step_int);
	base_value = New->dial.one_turn_value * (float)nb_tours;
			
	/* compute indicator position and redisplay it*/

	New_normalized_pos = ((New->dial.value - (base_value)) /
			     New->dial.one_turn_value);
	if(New_normalized_pos > 1.0)
			New_normalized_pos -= 1.0;
	New->dial.normalized_pos = New_normalized_pos;

	New->dial.large_normalized_pos = (((New->dial.value - New->dial.minimum) * 0.75)
					  / (New->dial.maximum - New->dial.minimum))
					+ 0.125;

/*
If the indicator value has changed, or if the min/max
values have changed, recompute the indicator coordinates.
*/

	if (New->dial.value != current->dial.value ||
      	    New->dial.minimum != current->dial.minimum ||
            New->dial.maximum != current->dial.maximum ||
	    New->dial.turn_step != current->dial.turn_step)
	{
    		compute_indicator_pos(New);
    		redraw = TRUE;
  	}

/*
If the value or decimal point have changed call the changed_value callback
*/
	if (New->dial.value != current->dial.value ||
	    New->dial.decimal_point != current->dial.decimal_point)
	{
		/* construct the callback structure */
		cb.reason = DEX_VALUE_CHANGED;
		cb.event = 0;
		cb.value = New->dial.value_int;
		cb.converted_value = New->dial.value;
		cb.decimal_point = New->dial.decimal_point;
		
		/* call ok callback */
		XtCallCallbacks((Widget)New, DeXNvalueChangedCallback, &cb);
	}
	
/* if width, height or margin have been modified */
	if(New->dial.label_height != current->dial.label_height   ||
	   BB_MarginWidth(New) != BB_MarginWidth(current)	  ||
	   BB_MarginHeight(New) != BB_MarginHeight(current)	  ||
	   New->core.width != current->core.width		  ||
	   New->core.height != current->core.height)
	{
		Resize((Widget)New);
		redraw = TRUE;
	}
	return(True);
} 

/*============================================================================*/

static void Destroy (Widget w_in)
{
	DeXDDialWidget w = (DeXDDialWidget)w_in;

	XtReleaseGC ((Widget)w, w->dial.inverse_GC);
  	XtReleaseGC ((Widget)w, w->dial.dial_GC);
  	XtReleaseGC ((Widget)w, w->dial.indicator_GC);
  	XtReleaseGC ((Widget)w, w->dial.bottom_shadow_GC);
  	XtReleaseGC ((Widget)w, w->dial.top_shadow_GC);

	XFreePixmap(XtDisplay(w), w->dial.label_pixmap_mult_on);
	XFreePixmap(XtDisplay(w), w->dial.label_pixmap_div_on);
	XFreePixmap(XtDisplay(w), w->dial.label_pixmap_mult_off);
	XFreePixmap(XtDisplay(w), w->dial.label_pixmap_div_off);

/*	free(w->dial.label_string); */
	free(w->dial.segments);
}

/*============================================================================*/

static void Resize (Widget w_in)
{
	DeXDDialWidget w = (DeXDDialWidget)w_in;
	Arg arg[10];
	char	string[80], format[80];
	float    angle, cosine, sine;
  	int       i;
	int argcount;
  	XSegment  *ptr;
/*
compute the center of the widget
*/
  	w->dial.center_x = w->core.width/2; 
  	w->dial.center_y = (w->core.height - w->dial.label_height -  w->dial.text_height -
			    3 * BB_MarginHeight(w))/2
			   + w->dial.label_height + BB_MarginHeight(w) + 10; /*+10 pour le descendre*/   
/* 
Generate the segment array containing the    
face of the dial, only if redraw is necessary.
*/ 
	if (redraw)
	{

		w->dial.outer_radius = MINIMUM(w->core.width, (w->core.height - w->dial.label_height -
							   w->dial.text_height
							   - 3 * BB_MarginHeight(w))) / 2;
		w->dial.inner_radius = w->dial.outer_radius -
				   MINIMUM(w->core.width, (w->core.height - w->dial.label_height -
							   w->dial.text_height
							   - 3 * BB_MarginHeight(w))) /
							   MARKERS_RATIO;
		w->dial.large_radius = w->dial.outer_radius +
					(w->dial.outer_radius - w->dial.inner_radius);

		/* inside markers*/
		for (i = 0, ptr = w->dial.segments, angle = 0.;
		     i < w->dial.nb_markers;
		     i++, ptr++, angle += w->dial.angle_increment)
		{   
			cosine = cos(angle);   
			sine   = sin(angle); 
			ptr->x1 = w->dial.center_x + w->dial.outer_radius * cosine; 
			ptr->y1 = w->dial.center_y - w->dial.outer_radius * sine;
			ptr->x2 = w->dial.center_x + w->dial.inner_radius * cosine; 
			ptr->y2 = w->dial.center_y - w->dial.inner_radius * sine;
		}  
		/* outside markers*/
		for (i = 0, angle = (-(M_PI/4.));
		     i < (w->dial.nb_markers + 1);
		     i++, ptr++, angle += w->dial.large_angle_increment)
		{   
			cosine = cos(angle);  
			sine   = sin(angle);
			ptr->x1 = w->dial.center_x + w->dial.large_radius * cosine; 
			ptr->y1 = w->dial.center_y - w->dial.large_radius * sine;
			ptr->x2 = w->dial.center_x + w->dial.outer_radius * cosine; 
			ptr->y2 = w->dial.center_y - w->dial.outer_radius * sine;
		}  

	}


/*
resize child
*/
	/* DeXInputWidget */
	argcount = 0;
	XtSetArg(arg[argcount], XmNx, BB_MarginWidth(w)); argcount++;
	XtSetArg(arg[argcount], XmNy, w->core.height - BB_MarginHeight(w) -
		 w->dial.text_height + 10); argcount++;
	XtSetArg(arg[argcount], XmNwidth, w->core.width - 2 * BB_MarginWidth(w)); argcount++;
	sprintf(format, "Turn = %%.%df", w->dial.decimal_point);
	sprintf(string,format, w->dial.one_turn_value);
/*	XtSetArg(arg[argcount], XmNhighlightColor, w->manager.foreground); argcount++; */
	XtSetArg(arg[argcount], DeXNlabelString, string); argcount++;
	XtSetValues(w->dial.children[DEXDDIAL_INPUT], arg, argcount);

	/* XmLabel */
	argcount = 0;
	XtSetArg(arg[argcount], XmNx, BB_MarginWidth(w) * 2); argcount++;
	XtSetArg(arg[argcount], XmNy, (BB_MarginHeight(w) - 5)); argcount++;
	XtSetArg(arg[argcount], XmNwidth, w->core.width - 2 * BB_MarginWidth(w)); argcount++;
	XtSetArg(arg[argcount], XmNheight, w->dial.label_height); argcount++;
	XtSetValues(w->dial.children[DEXDDIAL_LABEL], arg, argcount);

	if (w->dial.change_turn)
	{
		/* XmPushButton Mult */
		argcount = 0;
		XtSetArg(arg[argcount], XmNx, BB_MarginWidth(w) * 2); argcount++;
		XtSetArg(arg[argcount], XmNy, (w->dial.center_y - 25)); argcount++;
		XtSetValues(w->dial.children[DEXDDIAL_BUT_MULT], arg, argcount);


		/* XmPushButton Div */
		argcount = 0;
		XtSetArg(arg[argcount], XmNx, BB_MarginWidth(w) * 2); argcount++;
		XtSetArg(arg[argcount], XmNy, (w->dial.center_y + 5)); argcount++;
		XtSetValues(w->dial.children[DEXDDIAL_BUT_DIV], arg, argcount);
	} /* fin du if */

 	compute_indicator_pos(w);

} 

/*============================================================================*/

static void compute_indicator_pos(DeXDDialWidget w)
{

	float	angle,
		large_angle,
		cos_angle,
		sin_angle;
	int	nb_tours;
  	Dimension large_indicator_length, indicator_length,
		indicator_width, small_part_indicator_length;

/*
Make the indicator two pixels shorter than the  
inner edge of the markers.
*/
	small_part_indicator_length = w->dial.inner_radius - 1;
  	indicator_length = w->dial.inner_radius / INDICATOR_LENGTH_RATIO;
/*
Make the indicator width tenth of the inner diameter
*/
	indicator_width = w->dial.inner_radius / INDICATOR_WIDTH_RATIO;

/*
Normalize the indicator value to lie between zero
and 1, and then convert it to an angle.
*/
		angle = RADIANS(-360. * w->dial.normalized_pos - 90.); 
		cos_angle = cos(angle);
		sin_angle = sin(angle);
			
		/*
		Find the x,y coordinates of the shape of the indicator.   
		*/
		w->dial.indicator[0].x = w->dial.center_x + indicator_width *
					sin_angle;
		w->dial.indicator[0].y = w->dial.center_y + indicator_width *
					cos_angle;
		w->dial.indicator[1].x = w->dial.center_x + indicator_length * 
					cos_angle; 
		w->dial.indicator[1].y = w->dial.center_y - indicator_length * 
					sin_angle;
		w->dial.indicator[2].x = w->dial.center_x - indicator_width *
					sin_angle;
		w->dial.indicator[2].y = w->dial.center_y - indicator_width *
					cos_angle;
		w->dial.indicator[3].x = w->dial.indicator[0].x;
		w->dial.indicator[3].y = w->dial.indicator[0].y;

		/*small part of the indicator*/
		w->dial.indicator[4].x = w->dial.center_x + indicator_length * 
					cos_angle; 
		w->dial.indicator[4].y = w->dial.center_y - indicator_length * 
					sin_angle;
		w->dial.indicator[5].x = w->dial.center_x + small_part_indicator_length * 
					cos_angle; 
		w->dial.indicator[5].y = w->dial.center_y - small_part_indicator_length * 
					sin_angle;
		large_angle = RADIANS(-360. * w->dial.large_normalized_pos - 90.); 

		cos_angle = cos(large_angle);
		sin_angle = sin(large_angle);
		large_indicator_length = w->dial.outer_radius - w->dial.inner_radius;
		/*
		Find the x,y coordinates of the shape of the large indicator.   
		*/
		w->dial.large_indicator[0].x = (w->dial.center_x + w->dial.outer_radius * 
						cos_angle) + indicator_width * sin_angle;
		w->dial.large_indicator[0].y = (w->dial.center_y - w->dial.outer_radius * 
						sin_angle) + indicator_width * cos_angle;
		w->dial.large_indicator[1].x = w->dial.center_x + (w->dial.outer_radius +
								   large_indicator_length) * cos_angle;
		w->dial.large_indicator[1].y = w->dial.center_y - (w->dial.outer_radius +
								   large_indicator_length) * sin_angle;
		w->dial.large_indicator[2].x = (w->dial.center_x + w->dial.outer_radius * 
						cos_angle) - indicator_width * sin_angle;
		w->dial.large_indicator[2].y =  (w->dial.center_y - w->dial.outer_radius * 
						 sin_angle) - indicator_width * cos_angle;
		w->dial.large_indicator[3].x = w->dial.large_indicator[0].x;
		w->dial.large_indicator[3].y = w->dial.large_indicator[0].y;


} 

/*============================================================================*/

static void Redisplay (Widget w_in, XEvent *event, Region region)
{
	char string[80], format[80];

	DeXDDialWidget w = (DeXDDialWidget)w_in;

	if(w->core.visible)
	{
		XClearWindow(XtDisplay(w), XtWindow(w));
		/*
		 * Draw the markers used for the dial face.
		 */
		XDrawSegments(XtDisplay(w), XtWindow(w),
		      w->dial.dial_GC, 
		      w->dial.segments,
		      (w->dial.nb_markers * 2 + 1));

		/* Cercle entre les deux jeux de ticks */

		XDrawArc(XtDisplay(w), XtWindow(w),
			 w->dial.dial_GC,
			 (w->dial.center_x - w->dial.outer_radius),
			 (w->dial.center_y - w->dial.outer_radius),
			 (w->dial.outer_radius * 2),
			 (w->dial.outer_radius * 2),
			 0, 23040);
		XDrawArc(XtDisplay(w), XtWindow(w),
			 w->dial.dial_GC,
			 (w->dial.center_x - w->dial.large_radius),
			 (w->dial.center_y - w->dial.large_radius),
			 (w->dial.large_radius * 2),
			 (w->dial.large_radius * 2),
			 -2881, 17281);
		/*
		Clear indicator at old value
		*/
		/* foreground of the indicator */
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.indicator, 4,
			     Convex, CoordModeOrigin);
		/* shadow of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.indicator, 4,
			   CoordModeOrigin);
		/* nose of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_dial_GC,
			   &(w->dial.indicator[4]), 2,
			   CoordModeOrigin);

		/* foreground of the large indicator */
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.large_indicator, 4,
			     Convex, CoordModeOrigin);
		/* shadow of the large indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.large_indicator, 4,
			   CoordModeOrigin);
		/* nose of the large indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_dial_GC,
			   &(w->dial.large_indicator[4]), 2,
			   CoordModeOrigin);
		/* Min et Max */

		sprintf(format, "%%.%df", w->dial.decimal_point);
		sprintf(string, format, w->dial.minimum);
		XDrawString(XtDisplay(w), XtWindow(w),
			    w->dial.inverse_dial_GC,
			    (w->dial.center_x - w->dial.outer_radius - 20),
			    (w->dial.center_y + w->dial.outer_radius),
			    string,strlen(string));
		sprintf(format, "%%.%df", w->dial.decimal_point);
		sprintf(string, format, w->dial.maximum);
		XDrawString(XtDisplay(w), XtWindow(w),
			    w->dial.inverse_dial_GC,
			    (w->dial.center_x + w->dial.outer_radius),
			    (w->dial.center_y + w->dial.outer_radius),
			    string,strlen(string));
		/* Cercle entre les deux jeux de ticks */

		XDrawArc(XtDisplay(w), XtWindow(w),
			 w->dial.inverse_dial_GC,
			 (w->dial.center_x - w->dial.outer_radius),
			 (w->dial.center_y - w->dial.outer_radius),
			 (w->dial.outer_radius * 2),
			 (w->dial.outer_radius * 2),
			 -0, 23040);
		XDrawArc(XtDisplay(w), XtWindow(w),
			 w->dial.inverse_dial_GC,
			 (w->dial.center_x - w->dial.large_radius),
			 (w->dial.center_y - w->dial.large_radius),
			 (w->dial.large_radius * 2),
			 (w->dial.large_radius * 2),
			 -2881, 17281);

/*
Draw the indicator at its current value.
*/

		/* foreground of the indicator */
    		XFillPolygon(XtDisplay(w), XtWindow(w),
              	  	      w->dial.indicator_GC, 
                  	      w->dial.indicator, 4,
			      Convex, CoordModeOrigin);

		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.indicator_GC, 
			     w->dial.large_indicator, 4,
			     Convex, CoordModeOrigin);

		/* shadow of the inside (small) indicator */
		if((w->dial.normalized_pos >= 0. && w->dial.normalized_pos < 0.5) ||
		   (w->dial.normalized_pos < -0.5 && w->dial.normalized_pos > -1.0))
		{
			/*
			 * small indicator
			 */
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.indicator[0].x, w->dial.indicator[0].y,
				  w->dial.indicator[1].x, w->dial.indicator[1].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.indicator[1].x, w->dial.indicator[1].y,
				  w->dial.indicator[2].x, w->dial.indicator[2].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.indicator[2].x, w->dial.indicator[2].y,
				  w->dial.indicator[3].x, w->dial.indicator[3].y);
		}
		else
		{
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.indicator[0].x, w->dial.indicator[0].y,
				  w->dial.indicator[1].x, w->dial.indicator[1].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.indicator[1].x, w->dial.indicator[1].y,
				  w->dial.indicator[2].x, w->dial.indicator[2].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.indicator[2].x, w->dial.indicator[2].y,
				  w->dial.indicator[3].x, w->dial.indicator[3].y);
		}
		/* shadow of the outside (large) indicator */
		if((w->dial.large_normalized_pos >= 0. && w->dial.large_normalized_pos < 0.5) ||
		   (w->dial.large_normalized_pos < -0.5 && w->dial.large_normalized_pos > -1.0))
		{
			/*
			 * large indicator
			 */
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.large_indicator[0].x, w->dial.large_indicator[0].y,
				  w->dial.large_indicator[1].x, w->dial.large_indicator[1].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.large_indicator[1].x, w->dial.large_indicator[1].y,
				  w->dial.large_indicator[2].x, w->dial.large_indicator[2].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.large_indicator[2].x, w->dial.large_indicator[2].y,
				  w->dial.large_indicator[3].x, w->dial.large_indicator[3].y);
		}
		else
		{
			/*
			 * large indicator
			 */
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.large_indicator[0].x, w->dial.large_indicator[0].y,
				  w->dial.large_indicator[1].x, w->dial.large_indicator[1].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.top_shadow_GC,
				  w->dial.large_indicator[1].x, w->dial.large_indicator[1].y,
				  w->dial.large_indicator[2].x, w->dial.large_indicator[2].y);
			XDrawLine(XtDisplay(w), XtWindow(w),
				  w->dial.bottom_shadow_GC,
				  w->dial.large_indicator[2].x, w->dial.large_indicator[2].y,
				  w->dial.large_indicator[3].x, w->dial.large_indicator[3].y);
		}
		
		/* nose of the indicator */
		XDrawLine(XtDisplay(w), XtWindow(w),
			  w->dial.dial_GC,
			  w->dial.indicator[4].x, w->dial.indicator[4].y,
			  w->dial.indicator[5].x, w->dial.indicator[5].y);

		/* construct the format */
		sprintf(format, "%%.%df", w->dial.decimal_point);
		sprintf(string, format, w->dial.value);

		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, 0, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNdefaultValue, w->dial.default_value_int, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNincrement, w->dial.increment_int, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNdecimalPoint, w->dial.decimal_point, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, w->dial.value_int, NULL);

		if (limit_reached)
		{
			XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], XmNhighlightColor, w->manager.highlight_color, NULL);
			if (w->dial.value >= w->dial.maximum)
				XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, w->dial.value_int - 1, NULL);
			if (w->dial.value <= w->dial.minimum)
				XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, w->dial.value_int + 1, NULL);
			sprintf(string,"At limit: turn back");
		}
		else
		{
			XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
				      w->manager.foreground, NULL);
			sprintf(format, "Turn = %%.%df", w->dial.decimal_point);
			sprintf(string,format, w->dial.one_turn_value);
		} /* fin du else */
#if DEBUG_2
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
#endif
	}
} 


/*
================================================================================
	Action function definition
================================================================================
*/
/**********************************************************/

static void choose_indicator(DeXDDialWidget w, XEvent *event, char *args,
			    int n_args)
{

	Arg arglist[1];
  	float angle;
	Position rayon;
  	float New_value, New_normalized_pos;
	DeXdDialCallbackStruct cb;
	int n, nb_tours;
	Boolean  previously_on_left;

/*
Get the angle in degrees and normalize it.
*/
   	angle = atan2((double)(event->xbutton.y - w->dial.center_y),
		    (double)(event->xbutton.x - w->dial.center_x));
	if (sin(angle))
	{
		rayon = (Position)((float)(event->xbutton.y - w->dial.center_y)/ sin(angle));
		inner_dial = (abs(rayon) > w->dial.outer_radius)? False: True;
	} /* fin du if */
	else
		inner_dial = True;

} /* choose_indicator */

/***********************************************************/

static void track_indicator(DeXDDialWidget w, XEvent *event, char *args,
			    int n_args)
{

	Arg arglist[1];
  	float angle;
	Position rayon;
  	float New_value, New_normalized_pos;
	DeXdDialCallbackStruct cb;
	int n, nb_tours;
	Boolean  previously_on_left;

	limit_reached = False;
/*
Get the angle in degrees and normalize it.
*/
   	angle = atan2((double)(event->xbutton.y - w->dial.center_y),
		    (double)(event->xbutton.x - w->dial.center_x));
	
#if DEBUG
	printf("*********************\nangle in radians: %f \n",angle);
#endif

	angle = (angle < 0.) ? DEGREES(2 * M_PI + angle): DEGREES(angle);

	if (inner_dial)
	{
		w->dial.angle = angle;
	} /* fin du if */
	else
	{
		w->dial.large_angle = angle;
	} /* fin du if */

	New_normalized_pos = angle / 360.0;
	New_normalized_pos -= 0.25;
	if(New_normalized_pos < 0.0)
		New_normalized_pos += 1.0;

#if DEBUG || DEBUG_large
	printf("angle : %f \nNew Pos : %f\nold Pos %f\n",
	       angle, New_normalized_pos, w->dial.normalized_pos);
#endif

	if (inner_dial)
	{
		if (New_normalized_pos != w->dial.normalized_pos)
		{
			previously_on_left = on_left;
			old_old_value = old_value;
			old_value = w->dial.normalized_pos;
		} /* fin du if */
		/*
		 * test if the New position is on the right or on the left of
		 * the old position
		 */

		if (New_normalized_pos >= 0.0 && New_normalized_pos < 0.5)
			on_left = TRUE;
		else
			on_left = FALSE;
#if DEBUG
		if(on_left)
		{
			printf("A gauche\n");
		} /* fin du if */
		else
		{
			printf("A droite\n");
		} /* fin du else */
#endif
	} /* fin du if */
#if DEBUG
	else
	{
		if (New_normalized_pos != w->dial.large_normalized_pos)
		{
			previously_on_left = on_left;
			old_old_value = old_value;
			old_value = w->dial.large_normalized_pos;
		} /* fin du if */
	} /* fin du else */
#endif
/*  
Convert the angle to a value. 
*/

#define	ON_RIGHT	(!on_left)
#define	PREVIOUSLY_ON_RIGHT	(!previously_on_left)

	if (inner_dial)
	{
		New_value = base_value + (New_normalized_pos * (w->dial.one_turn_value));
#if DEBUG

		printf("old_old_value : %f old_value : %f New_pos %f\n",
		       old_old_value, old_value, New_normalized_pos);
		printf(" : %f old_value : %f New_pos %f\n",
		       old_old_value, old_value, New_normalized_pos);


		if (New_normalized_pos != w->dial.normalized_pos)
		{
			printf("On a bouge, (previously at %s)\n",
			       (PREVIOUSLY_ON_RIGHT)?"droite":"gauche");
			if (previously_on_left && ON_RIGHT)
				printf("G->D\n");
			if (PREVIOUSLY_ON_RIGHT && on_left)
				printf("D->G\n");
		} /* fin du if */

#endif

		if ((New_normalized_pos != w->dial.normalized_pos)
		    && (New_value > w->dial.minimum)
		    && (previously_on_left && ON_RIGHT)
		    && (old_old_value > old_value)
		    && (old_value < New_normalized_pos))
		{
			/*
			 * We pass the bottom from left to right, thus substract a turn
			 */
#if DEBUG
			printf("-1 tour\n");
#endif
			base_value -= w->dial.one_turn_value;
			New_value -= w->dial.one_turn_value;
		} /* fin du if */
		if ((New_normalized_pos != w->dial.normalized_pos)
		    && (New_value < w->dial.maximum)
		    && (PREVIOUSLY_ON_RIGHT && on_left)
		    && (old_old_value < old_value)
		    && (old_value > New_normalized_pos))
		{
			/*
			 * We pass the bottom from right to left, thus add a turn
			 */
#if DEBUG
			printf("+1 tour\n");
#endif
			base_value += w->dial.one_turn_value;
			New_value += w->dial.one_turn_value;
		} /* fin du if */

		if (New_value < w->dial.minimum)
		{
			w->dial.value = w->dial.minimum - 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
			limit_reached = True;
		} /* fin du if */
		else if (New_value > w->dial.maximum)
		{
			w->dial.value = w->dial.maximum + 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)nb_tours;
					
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
					      w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
			limit_reached = True;
		} /* fin du if */
		else
		{
			w->dial.value = New_value;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
		}
	} /* fin du if */
	else
	{
		New_value = w->dial.minimum + ((New_normalized_pos - 0.125) *
				((w->dial.maximum - w->dial.minimum)) / 0.75);
#if DEBUG_large
		printf("Large dial: old_old_value : %f old_value : %f New_pos %f valeur: %f\n",
		       old_old_value, old_value, New_normalized_pos, New_value);
#endif
		if (New_value < w->dial.minimum)
		{
			w->dial.value = w->dial.minimum - 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								w->dial.decimal_point);
			/* set up indicator position to redisplay it*/

			w->dial.large_normalized_pos = .125; /* 45 degres */
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			limit_reached = True;
		} /* fin du if */
		else if (New_value > w->dial.maximum)
		{
			w->dial.value = w->dial.maximum + 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			/* set up indicator position to redisplay it*/

			w->dial.large_normalized_pos = .875;	/* -45 degres */
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			limit_reached = True;
		} /* fin du if */
		else
		{
			w->dial.value = New_value;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			w->dial.large_normalized_pos = New_normalized_pos;
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
		}
	} /* fin du else */
#if DEBUG
	printf("New value Angle = %f (%d)\n",New_value,w->dial.value_int);
#endif
/*
Clear indicator at old value
*/
	if(w->core.visible)
	{
		/* foreground of the indicator */
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.indicator, 4,
			     Convex, CoordModeOrigin);
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.large_indicator, 4,
			     Convex, CoordModeOrigin);
		/* shadow of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.indicator, 4,
			   CoordModeOrigin);
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.large_indicator, 4,
			   CoordModeOrigin);
		/* nose of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_dial_GC,
			   &(w->dial.indicator[4]), 2,
			   CoordModeOrigin);
	}
/*
recompute New position
*/
		compute_indicator_pos(w);
/*
redisplay indicator
*/
		Redisplay((Widget)w, event, NULL);

} /* track_indicator */



/*
================================================================================
	Internal callback definition
================================================================================
*/

/*============================================================================*/

static void increase_turn(Widget w, XtPointer tag, XtPointer cb)
{
	char	format[80],
		string[80];
	DeXDDialWidget	New;

	New = (DeXDDialWidget)XtParent(w);

	if (((New->dial.maximum - New->dial.minimum) / (New->dial.turn_step * 10.)) <= 1.)
	{
		XBell(XtDisplay(w),50);
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
			      New->manager.highlight_color, NULL);
		sprintf(string,"Too large!");
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
		return;
	} /* fin du if */
	New->dial.turn_step *= 10.;

	New->dial.one_turn_value = (float)New->dial.turn_step;
	New->dial.turn_step_int = to_integer_external_format(New->dial.turn_step,
							       New->dial.decimal_point);
	sprintf(format, "Turn = %%.%df", New->dial.decimal_point);
	sprintf(string,format, New->dial.one_turn_value);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
		      New->manager.foreground, NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
}
/*============================================================================*/

static void decrease_turn(Widget w, XtPointer tag, XtPointer cb)
{
	char	format[80],
		string[80];
	DeXDDialWidget	New;

	New = (DeXDDialWidget)XtParent(w);
		
	if (pow(10., (double)(-1. * (New->dial.decimal_point - 1))) < New->dial.turn_step)
		New->dial.turn_step /= 10.;
	else
	{
		XBell(XtDisplay(w),50);
		sprintf(string,"Too small!");
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
			      New->manager.highlight_color, NULL);
		XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
		return;
	} /* fin du if */

	New->dial.one_turn_value = (float)New->dial.turn_step;
	New->dial.turn_step_int = to_integer_external_format(New->dial.turn_step,
							       New->dial.decimal_point);
	sprintf(format, "Turn = %%.%df", New->dial.decimal_point);
	sprintf(string,format, New->dial.one_turn_value);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
		      New->manager.foreground, NULL);
	XtVaSetValues(New->dial.children[DEXDDIAL_INPUT], DeXNlabelString, string , NULL);
}
/*============================================================================*/

static void ok_cb_input(DeXInputWidget w, XtPointer tag, DeXinputCallbackStruct * cb)
{
	DeXDDialWidget parent;
	double tmp_double;
	DeXdDialCallbackStruct cb_struct;

	char	string[80],
		format[80];
	int	nb_tours;
	float	New_angle,
		New_normalized_pos;

	
	/* get back parent id */
	parent = (DeXDDialWidget)XtParent(w);

	tmp_double = (double)(cb->value / pow(10.0,(double)cb->decimal_point));


	if(tmp_double > parent->dial.maximum || tmp_double < parent->dial.minimum)
	{
		/* reset the previous value */
/*		XBell(XtDisplay(w), 50); */
		/* construct the format */
		sprintf(format, "%%.%df", parent->dial.decimal_point);
		if (tmp_double > parent->dial.maximum)
		{
			parent->dial.value = parent->dial.maximum;
		} /* fin du if */
		if (tmp_double < parent->dial.minimum)
		{
			parent->dial.value = parent->dial.minimum;
		} /* fin du if */
		sprintf(string, format, parent->dial.value);
		parent->dial.value_int = to_integer_external_format(parent->dial.value, parent->dial.decimal_point);
		limit_reached = True;
	}
	else
	{
		parent->dial.value = tmp_double;
		/* set value ressource */
		parent->dial.value_int = to_integer_external_format(parent->dial.value, parent->dial.decimal_point);

		nb_tours = (int)(parent->dial.value_int / parent->dial.turn_step_int);

		base_value = parent->dial.one_turn_value * (float)nb_tours;
			
		/* compute indicator position and redisplay it*/

		New_normalized_pos = ((parent->dial.value - (base_value)) /
				     parent->dial.one_turn_value);
		if(New_normalized_pos > 1.0)
			New_normalized_pos -= 1.0;
		parent->dial.normalized_pos = New_normalized_pos;
		parent->dial.large_normalized_pos = (((parent->dial.value - parent->dial.minimum) * 0.75)
				  / (parent->dial.maximum - parent->dial.minimum)) + 0.125;
		limit_reached = False;
#if DEBUG
		printf("ok_cb_input: Nb tours = %d (base value = %f) New angle: %f Pos: %f\n",
		       nb_tours, base_value,
		       New_angle, New_normalized_pos);
#endif
	}
	compute_indicator_pos(parent);
	Redisplay((Widget)parent, NULL, NULL);

	/* construct the callback structure */
	cb_struct.reason = DEX_VALUE_CHANGED;
	cb_struct.event = ((XmAnyCallbackStruct *)cb)->event;
	cb_struct.value = parent->dial.value_int;
	cb_struct.converted_value = parent->dial.value;
	cb_struct.decimal_point = parent->dial.decimal_point;
		
		/* call ok callback */
	XtCallCallbacks((Widget)parent, DeXNvalueChangedCallback, &cb_struct);
} /* ok_cb_input */
/*============================================================================*/

static void send_value(DeXDDialWidget w, XEvent *event, char *args,
			    int n_args)
{

	Arg arglist[1];
  	float angle;
  	float New_value, New_normalized_pos;
	DeXdDialCallbackStruct cb;
	char	string[80];
	int n, nb_tours;
	Boolean  previously_on_left;

/*
Get the angle in degrees and normalize it.
*/
   	angle = atan2((double)(event->xbutton.y - w->dial.center_y),
		    (double)(event->xbutton.x - w->dial.center_x));
	w->dial.angle = angle;
#if DEBUG
	printf("*********************\nangle in radians: %f \n",angle);
#endif

	angle = (angle < 0.) ? DEGREES(2 * M_PI + angle): DEGREES(angle);

	New_normalized_pos = angle / 360.0;
	New_normalized_pos -= 0.25;
	if(New_normalized_pos < 0.0)
		New_normalized_pos += 1.0;

#if DEBUG
	printf("angle : %f \nNew Pos : %f\nold Pos %f\n",
	       angle, New_normalized_pos, w->dial.normalized_pos);
#endif

	if (inner_dial)
	{
		if (New_normalized_pos != w->dial.normalized_pos)
		{
			previously_on_left = on_left;
			old_old_value = old_value;
			old_value = w->dial.normalized_pos;
		} /* fin du if */
	} /* fin du if */
	else
	{
		if (New_normalized_pos != w->dial.large_normalized_pos)
		{
			previously_on_left = on_left;
			old_old_value = old_value;
			old_value = w->dial.large_normalized_pos;
		} /* fin du if */
	} /* fin du else */

/*
test if the New position is on the right or on the left of the old position
*/

	if (New_normalized_pos >= 0.0 && New_normalized_pos < 0.5)
		on_left = TRUE;
	else
		on_left = FALSE;

#if DEBUG
	if(on_left)
	{
		printf("A gauche\n");
	} /* fin du if */
	else
	{
		printf("A droite\n");
	} /* fin du else */
#endif

/*  
Convert the angle to a value. 
*/

/*  
Convert the angle to a value. 
*/

#define	ON_RIGHT	(!on_left)
#define	PREVIOUSLY_ON_RIGHT	(!previously_on_left)

	if (inner_dial)
	{
		New_value = base_value + (New_normalized_pos * (w->dial.one_turn_value));
#if DEBUG

		printf("old_old_value : %f old_value : %f New_pos %f\n",
		       old_old_value, old_value, New_normalized_pos);
		printf(" : %f old_value : %f New_pos %f\n",
		       old_old_value, old_value, New_normalized_pos);


		if (New_normalized_pos != w->dial.normalized_pos)
		{
			printf("On a bouge, (previously at %s)\n",
			       (PREVIOUSLY_ON_RIGHT)?"droite":"gauche");
			if (previously_on_left && ON_RIGHT)
				printf("G->D\n");
			if (PREVIOUSLY_ON_RIGHT && on_left)
				printf("D->G\n");
		} /* fin du if */

#endif

		if ((New_normalized_pos != w->dial.normalized_pos)
		    && (New_value > w->dial.minimum)
		    && (previously_on_left && ON_RIGHT)
		    && (old_old_value > old_value)
		    && (old_value < New_normalized_pos))
		{
			/*
			 * We pass the bottom from left to right, thus substract a turn
			 */
#if DEBUG
			printf("-1 tour\n");
#endif
			base_value -= w->dial.one_turn_value;
			New_value -= w->dial.one_turn_value;
		} /* fin du if */
		if ((New_normalized_pos != w->dial.normalized_pos)
		    && (New_value < w->dial.maximum)
		    && (PREVIOUSLY_ON_RIGHT && on_left)
		    && (old_old_value < old_value)
		    && (old_value > New_normalized_pos))
		{
			/*
			 * We pass the bottom from right to left, thus add a turn
			 */
#if DEBUG
			printf("+1 tour\n");
#endif
			base_value += w->dial.one_turn_value;
			New_value += w->dial.one_turn_value;
		} /* fin du if */

		if (New_value < w->dial.minimum)
		{
			w->dial.value = w->dial.minimum - 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
			limit_reached = True;
		} /* fin du if */
		else if (New_value > w->dial.maximum)
		{
			w->dial.value = w->dial.maximum + 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)nb_tours;
					
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
					      w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
			limit_reached = True;
		} /* fin du if */
		else
		{
			w->dial.value = New_value;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			w->dial.normalized_pos = New_normalized_pos;
			w->dial.large_normalized_pos = (((w->dial.value - w->dial.minimum) * 0.75)
					  / (w->dial.maximum - w->dial.minimum)) + 0.125;
		}
	} /* fin du if */
	else
	{
		New_value = w->dial.minimum + ((New_normalized_pos - 0.125) *
				((w->dial.maximum - w->dial.minimum)) / 0.75);
#if DEBUG_large
		printf("Large dial: old_old_value : %f old_value : %f New_pos %f valeur: %f\n",
		       old_old_value, old_value, New_normalized_pos, New_value);
#endif
		if (New_value < w->dial.minimum)
		{
			w->dial.value = w->dial.minimum - 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								w->dial.decimal_point);
			/* set up indicator position to redisplay it*/

			w->dial.large_normalized_pos = .125; /* 45 degres */
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			limit_reached = True;
		} /* fin du if */
		else if (New_value > w->dial.maximum)
		{
			w->dial.value = w->dial.maximum + 0.0000000001;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			/* set up indicator position to redisplay it*/

			w->dial.large_normalized_pos = .875;	/* -45 degres */
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
			limit_reached = True;
		} /* fin du if */
		else
		{
			w->dial.value = New_value;
			w->dial.value_int = to_integer_external_format(w->dial.value,
								       w->dial.decimal_point);
			w->dial.large_normalized_pos = New_normalized_pos;
			nb_tours = (int)(w->dial.value_int / w->dial.turn_step_int);
			base_value = w->dial.one_turn_value * (float)(nb_tours - 1);
				
			/* compute indicator position to redisplay it*/

			New_normalized_pos = ((w->dial.value - (base_value)) /
				     w->dial.one_turn_value);
			if(New_normalized_pos > 1.0)
				New_normalized_pos -= 1.0;
			w->dial.normalized_pos = New_normalized_pos;
		}
	} /* fin du else */
#if DEBUG
	printf("New value Angle = %f (%d)\n",New_value,w->dial.value_int);
#endif
/*
Clear indicator at old value
*/
	if(w->core.visible)
	{
		/* foreground of the indicator */
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.indicator, 4,
			     Convex, CoordModeOrigin);
		XFillPolygon(XtDisplay(w), XtWindow(w),
			     w->dial.inverse_shadow_GC, 
			     w->dial.large_indicator, 4,
			     Convex, CoordModeOrigin);
		/* shadow of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.indicator, 4,
			   CoordModeOrigin);
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_shadow_GC,
			   w->dial.large_indicator, 4,
			   CoordModeOrigin);
		/* nose of the indicator */
		XDrawLines(XtDisplay(w), XtWindow(w),
			   w->dial.inverse_dial_GC,
			   &(w->dial.indicator[4]), 2,
			   CoordModeOrigin);
	}
/*
recompute New position
*/
	compute_indicator_pos(w);
/*
redisplay indicator
*/
	Redisplay((Widget)w, event, NULL);

/*
Store value in the callback structure
*/
/*
	XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], XmNhighlightColor, w->manager.foreground, NULL);
*/
#if DEBUG_2
	XtVaSetValues(w->dial.children[DEXDDIAL_INPUT],
		      DeXNvalue, w->dial.value_int , NULL);
#endif		     

	cb.event = event;
	cb.reason = DEX_VALUE_CHANGED; 
	cb.converted_value = w->dial.value;
	cb.value = w->dial.value_int;
	cb.decimal_point = w->dial.decimal_point;
	XtCallCallbacks((Widget)w, DeXNvalueChangedCallback, &cb);
} /*  send_value */

/*============================================================================*/

static void change_default(DeXDDialWidget w, XEvent *event,
					     char *args, int n_args)
{
	char	format[80],
		string[80];

	if (w->dial.children[DEXDDIAL_INPUT])
	{
/*
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], XmNhighlightColor,
			      w->manager.highlight_color, NULL);
*/
		sprintf(format, "New Default = %%.%df", w->dial.decimal_point);
		sprintf(string,format, w->dial.value);
		string[79] = '\0';
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT],
			      DeXNlabelString, (String)string ,
			      NULL
			      );
		w->dial.default_value_int = w->dial.value_int;
		w->dial.default_value = w->dial.value;
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT],
			      DeXNdefaultValue, w->dial.default_value_int, NULL);

	} /* fin du if */
}

/*============================================================================*/

static void more_precise(DeXDDialWidget w, XEvent *event,
			 char *args, int n_args)
{
	if(w->dial.children[DEXDDIAL_INPUT])
	{
		w->dial.decimal_point ++;
		if (w->dial.decimal_point > 10 )
			w->dial.decimal_point = 10;
		w->dial.value_int *= 10;
		w->dial.value = to_float_internal_format(w->dial.value_int,
						     w->dial.decimal_point);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, w->dial.value_int, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNdecimalPoint,
			      w->dial.decimal_point , NULL);
	} /* fin du if */
}
/*============================================================================*/

static void less_precise(DeXDDialWidget w, XEvent *event,
			 char *args, int n_args)
{
	if(w->dial.children[DEXDDIAL_INPUT])
	{
		w->dial.decimal_point --;
		if (w->dial.decimal_point < 0 )
			w->dial.decimal_point = 0;
		w->dial.value_int /= 10;
		w->dial.value = to_float_internal_format(w->dial.value_int,
						     w->dial.decimal_point);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNvalue, w->dial.value_int, NULL);
		XtVaSetValues(w->dial.children[DEXDDIAL_INPUT], DeXNdecimalPoint,
			      w->dial.decimal_point , NULL);
	} /* fin du if */
}
/*===================================END=========================================*/

/*
================================================================================
	Convenience functions
================================================================================
*/
Widget DeXCreateDDial (Widget parent, char *name,
		       ArgList arglist, Cardinal argcount)
{
	return ( XtCreateWidget(name, DeXdDialWidgetClass, parent, arglist,
				argcount));
}
