/*
================================================================================
	D e X D i a l S c a l e P . h
================================================================================

	Private header file for DeXDialScale widget.

	Author : Jean-Francois VIBERT a partir de DeXDial de Denis Lambolez
		 b3e CHU ST-ANTOINE

	History: 
		 - Version 1.0: Septembre 1994
		   August, 23th 1993: Creation from DeXDial.c (Denis Lambolez et JFV)
================================================================================
*/

#ifndef DeXDDialP_h
#define DeXDDialP_h

#define DEXDDIAL_LABEL		0
#define DEXDDIAL_INPUT		1
#define DEXDDIAL_BUT_MULT		2
#define DEXDDIAL_BUT_DIV		3

#define NB_CHILDREN			4

/*
================================================================================
	Class Structure
================================================================================
*/

/*DeXDDial widget class part*/
typedef struct _DeXDDialClassPart
{
       int ignore;
} DeXDDialClassPart;

/*Class structure with DeXDDial part, Core part and Primitive part*/
typedef struct _DeXDDialClassRec
{
   	CoreClassPart			core_class;
	CompositeClassPart		composite_class;
	ConstraintClassPart		constraint_class;
	XmManagerClassPart		manager_class;
	XmBulletinBoardClassPart	bulletin_board_class;
   	DeXDDialClassPart		dial_class;
} DeXDDialClassRec;

/*Class structure initialized in DeXDDial.c*/
#ifdef __cplusplus
extern "C"{
#endif
extern DeXDDialClassRec DeXdDialClassRec; 
#ifdef __cplusplus
}
#endif


/*
================================================================================
	Instance Structure
================================================================================
*/

/*DeXDDial instance part*/
typedef struct _DeXDDialPart
{
	XtCallbackList value_changed;	/* callback list			*/
	XtCallbackList drag;		/* callback list			*/
   	XSegment  *segments;		/* markers shapes			*/
   	GC        dial_GC;          	/* assorted gc's			*/
	GC	  inverse_GC;
   	GC        top_shadow_GC;
	GC	  bottom_shadow_GC;
	GC	  inverse_shadow_GC;
	GC	  inverse_dial_GC;
	GC	  indicator_GC;
	Widget	  children[NB_CHILDREN];/* array of children ids		*/
	String	  label_string;		/* string for label			*/
/*	char	  label_string[60]; */	/* string for label			*/
	double	  default_value;	/* defaultvalue for DeXinput		*/
	double    value;         	/* indicator position			*/
   	double    minimum;          	/* minimum value			*/
   	double    maximum;          	/* maximum value			*/
	double	  turn_step;		/* increment value for a turn		*/
	double	  increment;		/* increment for DeXinput buttons	*/
	float	  angle;		/* current angle			*/
	float	  large_angle;		/* current angle			*/
	float	  angle_increment;	/* increment to draw markers		*/
	float	  large_angle_increment;/* increment to draw markers		*/
	float	  normalized_pos;	/* position of the indicator		*/
	float	  one_turn_value;	/* size of a turn 			*/
	long	  default_value_int;	/* defaultvalue for DeXinput		*/
	float	  large_normalized_pos;	/* position of the indicator		*/
	long	  value_int;         	/* indicator position			*/
   	long      minimum_int;        	/* minimum value			*/
   	long      maximum_int;        	/* maximum value			*/
	long	  turn_step_int;	/* increment value for a turn		*/
	long	  increment_int;	/* increment for DeXinput buttons	*/
	Pixel	  indicator_color;	/* foreground color for the indicator	*/
	int	  label_height;		/* height of the label string		*/
	int	  text_height;		/* height of the text_field		*/
	int	  decimal_point;	/* decimal point			*/
	int	  nb_markers;		/* number of markers			*/
	Boolean	  change_turn;		/* flag to put or not buttons x and /	*/
	XPoint    indicator[6];		/* indicator shape			*/
	XPoint    large_indicator[6];	/* large indicator shape		*/
	Pixmap	  label_pixmap_mult_off;/* the label pixmap for plus button in non arm state	*/
	Pixmap	  label_pixmap_div_off;	/* the label pixmap for minus button in non arm state	*/
	Pixmap	  label_pixmap_mult_on;	/* the label pixmap for plus button in arm state	*/
	Pixmap	  label_pixmap_div_on;	/* the label pixmap for minus button in arm state	*/
   	Position  center_x;         	/* coordinates of the dial center	*/
   	Position  center_y;         	
    	Position  inner_radius;      	/* inside of markers			*/
  	Position  outer_radius;       	/* outside of markers			*/
  	Position  large_radius;       	/* outside of markers			*/
} DeXDDialPart;

/*DeXDialScale instance structure*/
typedef struct _DeXDDialRec 
{
   	CorePart		core;
	CompositePart		composite;
	ConstraintPart		constraint;
	XmManagerPart		manager;
	XmBulletinBoardPart	bulletin_board;
   	DeXDDialPart		dial;
} DeXDDialRec;

#endif /* DeXDDialP_h*/
