/*
================================================================================
	D e X I n p u t P . h
================================================================================

	Private header file for DeXInput widget.

	Author : Denis LAMBOLEZ
		 b3e CHU ST-ANTOINE

	History: 
		 - Version 1.0: January 1994
		   January, 3th 1994: Creation from DeXDialP.h
================================================================================
*/

#ifndef DeXInputP_h
#define DeXInputP_h

#define DEXINPUT_LABEL		0
#define DEXINPUT_BUTTON_DEFAULT	1
#define DEXINPUT_TEXT		2
#define DEXINPUT_BUTTON_PLUS	3
#define DEXINPUT_BUTTON_MINUS	4
#define DEXINPUT_BUTTON_OK	5

#define NUM_CHILDREN		6

#define PLUS  10
#define MINUS 11
#define NONE  12

/*
================================================================================
	Class Structure
================================================================================
*/

/*DeXInput widget class part. this class add nothing*/
typedef struct _DeXInputClassPart
{
       int ignore;
} DeXInputClassPart;

/*DeXInput Class structure*/
typedef struct _DeXInputClassRec
{
   	CoreClassPart			core_class;
	CompositeClassPart		composite_class;
	ConstraintClassPart		constraint_class;
	XmManagerClassPart		manager_class;
	XmBulletinBoardClassPart	bulletin_board_class;
	DeXInputClassPart		input_class;
} DeXInputClassRec;

/*Class structure initialized in DeXInput.c*/
#ifdef __cplusplus
extern "C"{
#endif
extern DeXInputClassRec DeXinputClassRec; 
#ifdef __cplusplus
}
#endif

/*
================================================================================
	Instance Structure
================================================================================
*/

/*DeXInput instance part*/
typedef struct _DeXInputPart
{
	XtCallbackList ok_callback;			/* callback list					*/
/*	String	       label_string;	*/		/* the label						*/
	char	       label_string[60];		/* the label						*/
	Widget         children[NUM_CHILDREN];		/* child  widgets					*/
	Pixmap	       label_pixmap_plus_off;		/* the label pixmap for plus button in non arm state	*/
	Pixmap	       label_pixmap_minus_off;		/* the label pixmap for minus button in non arm state	*/
	Pixmap	       label_pixmap_plus_on;		/* the label pixmap for plus button in arm state	*/
	Pixmap	       label_pixmap_minus_on;		/* the label pixmap for minus button in arm state	*/
	double	       prev_values[10];			/* Roll buffer to save previous values			*/
	long	       timer_count;			/* to count the number of call of the timer		*/
	XtIntervalId   time_proc_id;			/* Id of the time procedure for incrementation		*/
	Dimension      label_height;			/* height of the label				        */
	int	       nb_prev_values;			/* number of previous values				*/
	int 	       increment;			/* value increment					*/
   	int            value;				/* value						*/
	int	       default_value;			/* default value					*/
	int	       decimal_point;			/* position of the decimal point			*/
	int	       which_time_proc;			/* PLUS == incrementation, MINUS == decrementation	*/
	int	       prev_values_index;		/* store the current position in the prev_values array  */
} DeXInputPart;

/*DeXInput instance structure*/
typedef struct _DeXInputRec 
{
   	CorePart		core;
	CompositePart		composite;
	ConstraintPart		constraint;
	XmManagerPart		manager;
	XmBulletinBoardPart	bulletin_board;
   	DeXInputPart		input;
} DeXInputRec;

#endif /*DeXInputP_h*/
