
#define DEBUG_UPDT_1 0
/* --------------------------------------------------------------------------------------
			G_EQUATIONS_PHENOM.C
   ------------------------------------------------------------------------------------*/
#include <stdlib.h>
#include<stdio.h>
#include<math.h>

#undef	MAIN
#include "G_neuron.h"

#include "G_synap.h"

#ifndef TRUE
#define	TRUE	1
#endif


#ifndef FALSE
#define FALSE	0
#endif

#define	SPIKE		1
#define	DEBUT_INP_CUR	20.
#define	FIN_INP_CUR	150

#define M1 259200
#define IA1 7141
#define IC1 54773
#define RM1 (1.0/M1)
#define M2 134456
#define IA2 8121
#define IC2 28411
#define RM2 (1.0/M2)
#define M3 243000
#define IA3 4561
#define IC3 51349

#ifndef M_E
#	define M_E 2.7182818284590452354
#endif 


#define MAX_MEM             200


static	float	I_total;
void	update_expon(int);
void	update_r_k(int);
float	ran1(int *);		
float	gasdev(int *);
void	nrerror(char *); 
void    reactualise_E_Na_K(void );

/*******************************************************************************
	Function for Summing the inputs that are in the queues 
*******************************************************************************/
float suminputs(float *ip_tc, float *im_tc, float * i_amp,
		       int * i_time, short head, short tail, float t_nb_it)
/*
float	ip_tc:	 cste temps attack PSP
	im_tc:	 cste temps decay PSP 
float 	*i_amp:	 file d'attente des amplitudes des events
int	*i_time: file d'attente des moments de survenue des events
short	head:  tete et queue de la file d'attente 
	tail:
float   t_nb_it: nb iterations

*/
{
	short	curr_input;
	float	curr_time;
	float   delta_time;
	double	sum_i;

	/* If head == tail, the queues are empty */
	
	if (head == tail) return(0.);
	
	
	/*
	 * Loop for each input, sum their current values together
	 */
	
	sum_i = 0.0;
	for (curr_input = head; curr_input != tail; curr_input = (curr_input+1)%MAX_MEM)
	{
		curr_time = (float)i_time[curr_input];		/* moment de l'arrivee de l'input */
		delta_time = (t_nb_it - curr_time);

		if(!ip_tc[curr_input] || !im_tc[curr_input])
			continue;
		
		if ( t_nb_it < (curr_time + ip_tc[curr_input]) && t_nb_it > curr_time)
			sum_i += i_amp[curr_input] * delta_time/ip_tc[curr_input] *
				exp((-delta_time/ip_tc[curr_input])+1.);
		if ( t_nb_it >= (curr_time + ip_tc[curr_input]))
			sum_i += (i_amp[curr_input] 
				  * (delta_time + im_tc[curr_input] - ip_tc[curr_input])/im_tc[curr_input] *
				exp(((-delta_time - im_tc[curr_input] + ip_tc[curr_input])/im_tc[curr_input])+1.));

	}
	return(sum_i);
}


/*
*	UPDATE_XNBC() calcule le potentiel a un moment pour le modele phenomenologique xnbc
*/
void update_xnbc(int nb_it)
{
	register	i, j, k, nb_r;	        /* indices */
	int	set, d_lag;
	float	K_epsp,			/* constantes pour le calcul de reversion */
		K_ipsp,
		K_noise,
		gasdev(),
		delta_ipsp,		
		delta_epsp,		
	        duree,
		exp_x,			/* exponentielle de delta */
		t_i, 			/* Elapsed time */
		somme_psp,		/* epsp et ipsp venant des
					 * autres neurones et leur somme
					 */
		delta,			/* pot mb - rev pot */
                seuil_increase,
		net_input,		/* somme de ce qui arrive (intermediaire) */
	        white_noise,            /* le bruit non filtre */
	        sum_inputs, 
	        curr_time,
		noise = 0.;		/* background noise */	


	short	nb_it_s;        	/* Current iteration in short */


	set = (int)time(0) & 077;	/* nb aleatoire */
	
	t_i = (float)nb_it;
	nb_it_s = (short)nb_it;
	net_input = somme_psp = 0.;

	/*
	 * Calcul du voltage intra somatique
	 */
	if (backgrd_noise_sd)
	{
		white_noise = gasdev(&set);
		white_noise *= (backgrd_noise_sd * sqrt(time_step)) ;
		if (filter_noise)
		{
		  center_noise = center_noise * filter_a + filter_b * white_noise; 
		  noise = center_noise + (backgrd_noise_m * time_step);
		}
		else
		  noise = white_noise + (backgrd_noise_m * time_step);
	} /* fin du if */
	else	
	  noise = (backgrd_noise_m * time_step);

	soma = (overshoot
			  * exp((-decharge_lag)
				* time_step * i_tau_mb_pot)) /* deja un inverse */
			 + (soma_post_spike - overshoot); 
#if DEBUG_UPDT_1
	printf("%d:  soma au depart: %f (overshoot= %f) ",
	       nb_it,soma,
	       overshoot);
	printf("soma post_spike: %f Noise: %f\n",
	       soma_post_spike,noise);
#endif
	/*
	 * Calcul des inputs venant des autres cellules
	 */



#if DEBUG_UPDT_1
	printf("%d:  soma au depart: %f (overshoot= %f) ",
	       nb_it,soma,
	       overshoot);
	printf("soma post_spike: %f Noise: %f\n",
	       soma_post_spike,noise);
#endif
	        


  /***********************************************************/
  /*--- calcul du courant synaptique   ----*/
  /***********************************************************/

  I_Syn_pos = 0.;
  
  sum_inputs = 0.;
  
#if DEBUG
  for (nb_r = 0; nb_r < nb_receptors; nb_r++)
  {
    printf("Recp %d: %d\n", nb_r, Receptors[nb_r].transmetteur);
  }
#endif

  for (j = 0; j < nb_spikes; j++)
  {
      curr_time = tab_syn[j].t_arrivee / time_step;
      if ((t_i < curr_time))
      {
	  tab_syn[j].syn_pos = tab_syn[j].syn_neg = 0.;
      }
      else
      {
	if (tab_syn[j].weight) /* le poids ne sert pas a definir E ou I, c'est le rev_pot qui le fait */
	{
	    for (nb_r = 0; nb_r < nb_receptors; nb_r++)
	    {
#if DEBUG_V8
		   printf("update_xnbc: Recp %d: %d; transm: %d ANY: %d\n", nb_r, Receptors[nb_r].transmetteur ,tab_syn[j].transmetteur, ANY);
#endif   
	      if ((Receptors[nb_r].transmetteur == NO_TRANSMISSION) || (tab_syn[j].transmetteur == NO_TRANSMISSION))
		continue;
	      if ((Receptors[nb_r].transmetteur & tab_syn[j].transmetteur) 
		  || (tab_syn[j].transmetteur == ANY)
		  || (Receptors[nb_r].transmetteur == ANY))
	      {
		   if ( t_i < (curr_time + Receptors[nb_r].tau_a_psp) && t_i > curr_time)
		   {
		     sum_inputs += (t_i - curr_time) / Receptors[nb_r].tau_a_psp *
		                  exp(((curr_time - t_i) / Receptors[nb_r].tau_a_psp) + 1.);
		   }
		   if ( t_i >= (curr_time + Receptors[nb_r].tau_a_psp))
		   {
		     sum_inputs += (t_i - curr_time + Receptors[nb_r].tau_d_psp - Receptors[nb_r].tau_a_psp)
		                  / Receptors[nb_r].tau_d_psp *
		                  exp(((curr_time - t_i - Receptors[nb_r].tau_d_psp + Receptors[nb_r].tau_a_psp)
				       / Receptors[nb_r].tau_d_psp) + 1.);
		   }
		   tab_syn[j].syn_pos = Receptors[nb_r].g_Syn * sum_inputs;
		   G_Syn_pos = tab_syn[j].weight * tab_syn[j].syn_pos;
		   I_Syn_pos += G_Syn_pos * (soma - Receptors[nb_r].rev_pot);
#if DEBUG
		   printf("%d (spike %d): PSP [%d]: %f %f %f %f\n", nb_it, j, nb_r, tab_syn[j].weight, (soma - Receptors[nb_r].rev_pot), G_Syn_pos, I_Syn_pos);
#endif		   
	      } /* fin du if */
	    } /* fin du for */
	  } /* fin du if */
      } /* fin du else */
  }

	/*
	 * Le probleme du bruit: le bruit est lui aussi multiplie par
	 * le facteur positif s'il est positif (il a la valeur d'un
	 * epsp) et par le facteur negatif sinon (si 0 il est
	 * considere comme positif)
	 */
  /* printf("input_voltage = %f\n",input_voltage); */

	net_input =  input_voltage + noise - I_Syn_pos;

	/*
	 * Maintenant on peut calculer le soma
	 * Calcul du voltage intra somatique
	 * et tenir compte du shunt de la mb qui varie en f(t)
	 */

	d_lag = (int)(decharge_lag);
	if (d_lag < MAX_SHUNT / time_step)
	{
		if (shunt_tau / time_step < .1)
			soma += net_input;
		else
			soma += (net_input
					  * ( 1. - exp(-decharge_lag * time_step/shunt_tau)));
	} /* fin du if */
	else
		soma += net_input;

	/*
	 * ca decharge ou pas?
	 */

      /*
       * Calcul du seuil de decharge
       */
      threshold =   delta_seuil 
			* exp((-decharge_lag) 
			* time_step  * i_tau_seuil)
			+ seuil;

      /*
      soma -= p_rest_pot;
      soma = v_plus * 
		       (exp(A * soma) - 1.) 
			/ (exp(A * soma) - v_rap);
      soma += p_rest_pot;
      */

     if (soma >= threshold)
      {				/* on a un spike */
	    la_spike = 1;
	    spike = TRUE;
	    decharge_lag = 0.;
#if DEBUG_UPDT_1
	    printf("soma avt spike: %f\n",
		   soma);
#endif
	    soma_post_spike = /* soma + overshoot; */
	    soma = pot_post_spike;

	    /* calcul de la valeur absolue du seuil apres ce spike (seuil_peak) */

	    if (less_fatigue)
	    {
	      delta = (seuil_peak - threshold);
	      seuil_increase = (seuil_post_spike - delta)* fatigue + delta;
	    }
	    else
	    {
	      seuil_increase = seuil_post_spike * 
			     (fatigue * (threshold - seuil) + 1.);
	    }
	    seuil_peak = seuil_increase + threshold;
	    delta_seuil = seuil_peak - seuil;
#if DEBUG_BURST
	    printf("SPIKE: iter: %d (spike suivante: %d)\n", nb_it, spike_suivante);
#endif
      } /* fin du if */
      else			/* pas de spike */
      {
	    la_spike = 0;
	    spike = FALSE;
	    decharge_lag += 1.;
      } /* fin du else */
     le_potentiel = soma;

} /* fin du update_xnbc */


/*
*	UPDATE_LEAKY() calcule le potentiel a un moment pour le leaky integrator
*/
void update_leaky(int nb_it)
{
	register	i, j, k, nb_r;	/* indices */
	int	set;
	float	K_epsp,			/* constantes pour le calcul de reversion */
		K_ipsp,
		K_noise,

		gasdev(),
	        duree,
		exp_x,			/* exponentielle de delta */
		t_i, 			/* Elapsed time */
		somme_psp,		/* epsp et ipsp venant des
					 * autres neurones et leur somme
					 */
		delta,			/* pot mb - rev pot */
                seuil_increase,
		net_input,		/* somme de ce qui arrive (intermediaire) */
	        white_noise,            /* le bruit non filtre */
	        center_noise,           /* le bruit centre filtre */
	        sum_inputs,
	        curr_time,
		noise = 0.;		/* background noise */	
	short	nb_it_s;        	/* Current iteration in short */


	set = (int)time(0) & 077;	/* nb aleatoire */
	
	t_i = (float)nb_it;
	nb_it_s = (short)nb_it;
	net_input = somme_psp = 0.;

	/*
	 * Calcul du voltage intra somatique
	 */
	if (backgrd_noise_sd)
	{
		white_noise = gasdev(&set);
		white_noise *= (backgrd_noise_sd * sqrt(time_step)) ;
		if (filter_noise)
		{
		  center_noise = center_noise * filter_a + filter_b * white_noise; 
		  noise = center_noise + backgrd_noise_m;
		}
		else
		  noise = white_noise + backgrd_noise_m;
	} /* fin du if */
	else	
	  noise = backgrd_noise_m;


        soma = soma * ( 1. - (time_step * i_tau_mb_pot)) + 
	                 (time_step * i_tau_mb_pot * p_rest_pot);


#if DEBUG_UPDT_1
	printf("%d:  soma au depart: %f ",
	       nb_it,soma);
	printf("Noise: %f\n", noise);
#endif
		/*
		 * Calcul des inputs venant des autres cellules
		 */


  /***********************************************************/
  /*--- calcul du courant synaptique   ----*/
  /***********************************************************/

  I_Syn_pos = 0.;
  
  sum_inputs = 0.;
  
#if DEBUG
  for (nb_r = 0; nb_r < nb_receptors; nb_r++)
  {
    printf("Recp %d: %d\n", nb_r, Receptors[nb_r].transmetteur);
  }
#endif

  for (j = 0; j < nb_spikes; j++)
  {
      curr_time = tab_syn[j].t_arrivee / time_step;
      if ((t_i < curr_time))
      {
	  tab_syn[j].syn_pos = tab_syn[j].syn_neg = 0.;
      }
      else
      {
	if (tab_syn[j].weight) /* le poids ne sert pas a definir E ou I, c'est le rev_pot qui le fait */
	{
	    for (nb_r = 0; nb_r < nb_receptors; nb_r++)
	    {
#if DEBUG_V8
		   printf("update_xnbc: Recp %d: %d; transm: %d ANY: %d\n", nb_r, Receptors[nb_r].transmetteur ,tab_syn[j].transmetteur, ANY);
#endif   
	      if ((Receptors[nb_r].transmetteur == NO_TRANSMISSION) || (tab_syn[j].transmetteur == NO_TRANSMISSION))
		continue;
	      if ((Receptors[nb_r].transmetteur & tab_syn[j].transmetteur) 
		  || (tab_syn[j].transmetteur == ANY)
		  || (Receptors[nb_r].transmetteur == ANY))
	      {
		   if ( t_i < (curr_time + Receptors[nb_r].tau_a_psp) && t_i > curr_time)
		   {
		     sum_inputs += (t_i - curr_time) / Receptors[nb_r].tau_a_psp *
		                  exp(((curr_time - t_i) / Receptors[nb_r].tau_a_psp) + 1.);
		   }
		   if ( t_i >= (curr_time + Receptors[nb_r].tau_a_psp))
		   {
		     sum_inputs += (t_i - curr_time + Receptors[nb_r].tau_d_psp - Receptors[nb_r].tau_a_psp)
		                  / Receptors[nb_r].tau_d_psp *
		                  exp(((curr_time - t_i - Receptors[nb_r].tau_d_psp + Receptors[nb_r].tau_a_psp)
				       / Receptors[nb_r].tau_d_psp) + 1.);
		   }
		   tab_syn[j].syn_pos = Receptors[nb_r].g_Syn * sum_inputs;
		   G_Syn_pos = tab_syn[j].weight * tab_syn[j].syn_pos;
		   I_Syn_pos += G_Syn_pos * (soma - Receptors[nb_r].rev_pot);
#if DEBUG
		   printf("%d (spike %d): PSP [%d]: %f %f %f %f\n", nb_it, j, nb_r, tab_syn[j].weight, (soma - Receptors[nb_r].rev_pot), G_Syn_pos, I_Syn_pos);
#endif		   
	      } /* fin du if */
	    } /* fin du for */
	  } /* fin du if */
      } /* fin du else */
  }

	/*
	 * Le probleme du bruit: le bruit est lui aussi multiplie par
	 * le facteur positif s'il est positif (il a la valeur d'un
	 * epsp) et par le facteur negatif sinon (si 0 il est
	 * considere comme positif)
	 */
  /* printf("input_voltage = %f\n",input_voltage); */

	net_input =  input_voltage + noise - I_Syn_pos;


	/*
	 * Maintenant on peut calculer le soma
	 * Calcul du voltage intra somatique
	 * et tenir compte du shunt de la mb qui varie en f(t)
	 */

	soma += net_input;

	/*
	 * ca decharge ou pas?
	 */

	/*
	 * Calcul du seuil de decharge
	 */
	threshold =  delta_seuil 
	             * exp((-decharge_lag) 
			   * time_step  * i_tau_seuil)
	             + seuil;

   
	if (soma >= threshold)
	{				/* on a un spike */
	    la_spike = 1;
	    spike = TRUE;
	    decharge_lag = 0.;
#if DEBUG_UPDT_1
	    printf("soma avt spike: %f\n",
		   soma);
#endif
	    soma_post_spike = /* soma + overshoot; */
	      soma = pot_post_spike;
	    
	    /* calcul de la valeur absolue du seuil apres ce spike (seuil_peak) */

	    if (less_fatigue)
	    {
	      delta = (seuil_peak - threshold);
	      seuil_increase = (seuil_post_spike - delta)* fatigue + delta;
	    }
	    else
	    {
	      seuil_increase = seuil_post_spike * 
		(fatigue * (threshold - seuil) + 1.);
	    }
	    seuil_peak = seuil_increase + threshold;
	    delta_seuil = seuil_peak - seuil;
#if DEBUG_BURST
	    printf("SPIKE: iter: %d (spike suivante: %d)\n", nb_it, spike_suivante);
#endif
	} /* fin du if */
	else			/* pas de spike */
	{
	  la_spike = 0;
	  spike = FALSE;
	  decharge_lag += 1.;
	} /* fin du else */
	le_potentiel = soma;
} /* fin du update_leaky */


/*
*	UPDATE_COND() calcule le potentiel a un moment pour le modele conditional burster
*/
void update_cond(int nb_it)
{
	register	i, j, k, nb_r;	/* indices */
	int	set, d_lag;
	float	K_epsp,			/* constantes pour le calcul de reversion */
		K_ipsp,
		K_noise,
		gasdev(),
		delta_ipsp,		
		delta_epsp,		
	        duree,
		exp_x,			/* exponentielle de delta */
		t_i, 			/* Elapsed time */
		somme_psp,		/* epsp et ipsp venant des
					 * autres neurones et leur somme
					 */
		delta,			/* pot mb - rev pot */
                seuil_increase,
		net_input,		/* somme de ce qui arrive (intermediaire) */
	        white_noise,            /* le bruit non filtre */
	        sum_inputs,
	        curr_time,
		noise = 0.;		/* background noise */	
	short	nb_it_s;        	/* Current iteration in short */


	set = (int)time(0) & 077;	/* nb aleatoire */
	
	t_i = (float)nb_it;
	nb_it_s = (short)nb_it;
	net_input = somme_psp = 0.;

	/* Gestion du sablier */

	for (i = 0; i < nb_sabliers; i++)
	{
	       sablier[i]--;
	       if (!sablier[i])
	       {
		   /* 
		    *  si le sablier est vide, on l'enleve en avancant les
		    *  autres d un cran 
		    */
		   for (j = i+1; j < nb_sabliers; j++)
		   {
		       sablier[j-1] = sablier[j];
		   }
		   nb_sabliers--;
		   frequence = (freq_spike * (compose_fact * nb_sabliers));
		   if (frequence > freq_max)
		     frequence = freq_max;
		   i--;
	       }
	}
	if (!nb_sabliers)
		in_burst = FALSE;

	/*
	 * Calcul du voltage intra somatique
	 */
	if (backgrd_noise_sd)
	{
		white_noise = gasdev(&set);
		white_noise *= (backgrd_noise_sd * sqrt(time_step)) ;
		if (filter_noise)
		{
		  center_noise = center_noise * filter_a + filter_b * white_noise; 
		  noise = center_noise + backgrd_noise_m;
		}
		else
		  noise = white_noise + backgrd_noise_m;
	} /* fin du if */
	else	
	  noise = backgrd_noise_m;

	if (in_burst)
	{
	        soma = (burst_overshoot
				  * exp((-decharge_lag)
					* time_step * i_tau_mb_pot))
		                 + (soma_post_spike - burst_overshoot); 
#if DEBUG_UPDT_1
		printf("%d:  soma au depart: %f (overshoot= %f) ",
		       nb_it,soma,
		       burst_overshoot);
		printf("soma post_spike: %f Noise: %f\n",
		       soma_post_spike,noise);
#endif

		/* Implementation du Reversal Potential (V 4.4) */

		/*
		 *	Le prb des sommes de PSP est plus delicat que celui 
		 * d'un seul PSP. Nous avons estime qu'il faut d'abord faire 
		 * la somme de tout ce qui est + (epsp) et - (ipsp), 
		 * separement, puis de calculer son facteur de dependance par 
		 * rapport au rev pot.
		 * On estime que plus on se rapproche du reversal potential,
		 * moins l'effet d'un PSP est important donc, cela depend de
		 * (soma - rev_pot). Si on est au pot de mb, le facteur
		 * multiplicateur est 1, et si on est au rev pot il est 0.
		 * Donc, le PSP sera multiplie par un facteur qui devient
		 * lineairement de plus en plus petit au fur et a mesure
		 * que l'on se rapporche du Rev pot (V4.10):
		 *   K_rev_[pn] = 1 / (Pot_repos - reversal)
		 *   K_[ei]psp = (soma - reversal) * K_rev_[pn]
		 *   [ei]psp *= K_[ei]psp   
		 */
		/* On ne se lance dans les calculs que s'il y a lieu */



#if DEBUG_UPDT_1
		printf("%d:  soma au depart: %f (overshoot= %f) ",
		       nb_it,soma,
		       burst_overshoot);
		printf("soma post_spike: %f Noise: %f\n",
		       soma_post_spike,noise);
#endif
	}
	else
	{
	        soma = (overshoot
				  * exp((-decharge_lag)
					* time_step * i_tau_mb_pot)) /* deja un inverse */
		                 + (soma_post_spike - overshoot); 
#if DEBUG_UPDT_1
		printf("%d:  soma au depart: %f (overshoot= %f) ",
		       nb_it,soma,
		       overshoot);
		printf("soma post_spike: %f Noise: %f\n",
		       soma_post_spike,noise);
#endif
		/*
		 * Calcul des inputs venant des autres cellules
		 */

		delta_epsp = rev_pot_pos - soma;
		delta_ipsp = rev_pot_neg - soma;
#if DEBUG_UPDT
		printf(" delta_epsp = %f  delta_ipsp = %f\n",
			delta_epsp, delta_ipsp);
#endif		       


		/* Implementation du Reversal Potential (V 4.4) */

		/*
		 *	Le prb des sommes de PSP est plus delicat que celui 
		 * d'un seul PSP. Nous avons estime qu'il faut d'abord faire 
		 * la somme de tout ce qui est + (epsp) et - (ipsp), 
		 * separement, puis de calculer son facteur de dependance par 
		 * rapport au rev pot.
		 * On estime que plus on se rapproche du reversal potential,
		 * moins l'effet d'un PSP est important donc, cela depend de
		 * (soma - rev_pot). Si on est au pot de mb, le facteur
		 * multiplicateur est 1, et si on est au rev pot il est 0.
		 * Donc, le PSP sera multiplie par un facteur qui devient
		 * lineairement de plus en plus petit au fur et a mesure
		 * que l'on se rapporche du Rev pot (V4.10):
		 *   K_rev_[pn] = 1 / (Pot_repos - reversal)
		 *   K_[ei]psp = (soma - reversal) * K_rev_[pn]
		 *   [ei]psp *= K_[ei]psp   
		 */
		/* On ne se lance dans les calculs que s'il y a lieu */



#if DEBUG_UPDT_1
		printf("%d:  soma au depart: %f (overshoot= %f) ",
		       nb_it,soma,
		       overshoot);
		printf("soma post_spike: %f Noise: %f\n",
		       soma_post_spike,noise);
#endif
	        
	}
		/*
		 * Calcul des inputs venant des autres cellules
		 */

  /***********************************************************/
  /*--- calcul du courant synaptique   ----*/
  /***********************************************************/

  I_Syn_pos = 0.;
  
  sum_inputs = 0.;
  
#if DEBUG
  for (nb_r = 0; nb_r < nb_receptors; nb_r++)
  {
    printf("Recp %d: %d\n", nb_r, Receptors[nb_r].transmetteur);
  }
#endif

  for (j = 0; j < nb_spikes; j++)
  {
      curr_time = tab_syn[j].t_arrivee / time_step;
      if ((t_i < curr_time))
      {
	  tab_syn[j].syn_pos = tab_syn[j].syn_neg = 0.;
      }
      else
      {
	if (tab_syn[j].weight) /* le poids ne sert pas a definir E ou I, c'est le rev_pot qui le fait */
	{
	    for (nb_r = 0; nb_r < nb_receptors; nb_r++)
	    {
#if DEBUG_V8
		   printf("update_xnbc: Recp %d: %d; transm: %d ANY: %d\n", nb_r, Receptors[nb_r].transmetteur ,tab_syn[j].transmetteur, ANY);
#endif   
	      if ((Receptors[nb_r].transmetteur == NO_TRANSMISSION) || (tab_syn[j].transmetteur == NO_TRANSMISSION))
		continue;
	      if ((Receptors[nb_r].transmetteur & tab_syn[j].transmetteur) 
		  || (tab_syn[j].transmetteur == ANY)
		  || (Receptors[nb_r].transmetteur == ANY))
	      {
		   if ( t_i < (curr_time + Receptors[nb_r].tau_a_psp) && t_i > curr_time)
		   {
		     sum_inputs += (t_i - curr_time) / Receptors[nb_r].tau_a_psp *
		                  exp(((curr_time - t_i) / Receptors[nb_r].tau_a_psp) + 1.);
		   }
		   if ( t_i >= (curr_time + Receptors[nb_r].tau_a_psp))
		   {
		     sum_inputs += (t_i - curr_time + Receptors[nb_r].tau_d_psp - Receptors[nb_r].tau_a_psp)
		                  / Receptors[nb_r].tau_d_psp *
		                  exp(((curr_time - t_i - Receptors[nb_r].tau_d_psp + Receptors[nb_r].tau_a_psp)
				       / Receptors[nb_r].tau_d_psp) + 1.);
		   }
		   tab_syn[j].syn_pos = Receptors[nb_r].g_Syn * sum_inputs;
		   G_Syn_pos = tab_syn[j].weight * tab_syn[j].syn_pos;
		   I_Syn_pos += G_Syn_pos * (soma - Receptors[nb_r].rev_pot);
#if DEBUG
		   printf("%d (spike %d): PSP [%d]: %f %f %f %f\n", nb_it, j, nb_r, tab_syn[j].weight, (soma - Receptors[nb_r].rev_pot), G_Syn_pos, I_Syn_pos);
#endif		   
	      } /* fin du if */
	    } /* fin du for */
	  } /* fin du if */
      } /* fin du else */
  }

	/*
	 * Le probleme du bruit: le bruit est lui aussi multiplie par
	 * le facteur positif s'il est positif (il a la valeur d'un
	 * epsp) et par le facteur negatif sinon (si 0 il est
	 * considere comme positif)
	 */
  /* printf("input_voltage = %f\n",input_voltage); */

	net_input =  input_voltage + noise - I_Syn_pos;

	if ((net_input + soma) >= seuil_burst)
	{
	        duree = ran1(&set)*burst_range;
		sablier[nb_sabliers] = (duree_burst_min + duree)/time_step;
		nb_sabliers++;
		in_burst = TRUE;
#if DEBUG_BURST
		printf("ran1: %f (%f range)\n",ran1(&set),burst_range);
		printf("Duree: %f ms (%d it) nb_sabliers: %d\n",duree,sablier[nb_sabliers-1], nb_sabliers);
#endif
		if (frequence < freq_max)
		{
		      frequence = (compose_fact * nb_sabliers) * freq_spike;
		      if (frequence > freq_max)
			   frequence = freq_max;
		}
		if (frequence)
		      spike_suivante = nb_it;
#if DEBUG_BURST
		      printf("iter: %d (spike suivante: %d)\n", nb_it, spike_suivante);
#endif

	}
        if (I_Syn_neg)
        {
	        if (nb_sabliers)
		{
		      frequence = (compose_fact * nb_sabliers-1) * freq_spike;
		      if (frequence < 0.)
		      {
			frequence = 0.;
			nb_sabliers--;
			in_burst = FALSE;
		      }
		}
	}
#if 0

	un_epsp *= input_e_fact;
	un_ipsp *= input_i_fact;

	if (un_epsp || un_ipsp)
	{
		/*
		 * on met le resultat epsp+ipsp dans la file d'attente
		 * temporelle
		 */
		inp_t[tail] = nb_it;

		if (un_epsp)
		{
			if (bum_type && ((un_epsp + soma) >= seuil_burst))
			{
				duree = ran1(&set)*burst_range;
				sablier[nb_sabliers] = (duree_burst_min + duree)/time_step;
				if (un_epsp1)
				{
				     le_epsp1[1].x = le_epsp1[0].x + (int)(t_i * x_echelle);
				     le_epsp1[1].y = le_epsp1[0].y;
				     le_epsp1[2].x = le_epsp1[1].x;
				     le_epsp1[2].y = le_epsp1[1].y - 3;
				     le_epsp1[3].x = le_epsp1[2].x + (int)((float)sablier[nb_sabliers] * x_echelle);
				     le_epsp1[3].y = le_epsp1[2].y;
				     le_epsp1[4].x = le_epsp1[3].x;
				     le_epsp1[4].y = le_epsp1[3].y + 3;
				     le_epsp1[5].x = le_epsp1[0].x + (int)((float)nb_points * x_echelle);
				     le_epsp1[5].y = le_epsp1[4].y;
				     un_epsp1 = FALSE;
				}
				if (un_epsp2)
				{
				     le_epsp2[1].x = le_epsp2[0].x + (int)(t_i * x_echelle);
				     le_epsp2[1].y = le_epsp2[0].y;
				     le_epsp2[2].x = le_epsp2[1].x;
				     le_epsp2[2].y = le_epsp2[1].y - 3;
				     le_epsp2[3].x = le_epsp2[2].x + (int)((float)sablier[nb_sabliers] * x_echelle);
				     le_epsp2[3].y = le_epsp2[2].y;
				     le_epsp2[4].x = le_epsp2[3].x;
				     le_epsp2[4].y = le_epsp2[3].y + 3;
				     le_epsp2[5].x = le_epsp2[0].x + (int)((float)nb_points * x_echelle);
				     le_epsp2[5].y = le_epsp2[4].y;
				     un_epsp2 = FALSE;
				}
				if (un_epsp3)
				{
				     le_epsp3[1].x = le_epsp3[0].x + (int)(t_i * x_echelle);
				     le_epsp3[1].y = le_epsp3[0].y;
				     le_epsp3[2].x = le_epsp3[1].x;
				     le_epsp3[2].y = le_epsp3[1].y - 3;
				     le_epsp3[3].x = le_epsp3[2].x + (int)((float)sablier[nb_sabliers] * x_echelle);
				     le_epsp3[3].y = le_epsp3[2].y;
				     le_epsp3[4].x = le_epsp3[3].x;
				     le_epsp3[4].y = le_epsp3[3].y + 3;
				     le_epsp3[5].x = le_epsp3[0].x + (int)((float)nb_points * x_echelle);
				     le_epsp3[5].y = le_epsp3[4].y;
				     un_epsp3 = FALSE;
				}
				nb_sabliers++;
				in_burst = TRUE;
#if DEBUG_BURST
				printf("ran1: %f (%f range)\n",ran1(&set),burst_range);
				printf("Duree: %f ms (%d it) nb_sabliers: %d\n",duree,sablier[nb_sabliers-1], nb_sabliers);
#endif
				if (frequence < freq_max)
				{
				      frequence = (compose_fact * nb_sabliers) * freq_spike;
				      if (frequence > freq_max)
					   frequence = freq_max;
				}
				if (frequence)
				      spike_suivante = nb_it;
#if DEBUG_BURST
				printf("iter: %d (spike suivante: %d)\n", nb_it, spike_suivante);
#endif
			}
		}
		if (un_ipsp)
		{
			if (un_ipsp1)
			{
			     le_ipsp1[1].x = le_ipsp1[0].x + (int)(t_i * x_echelle);
			     le_ipsp1[1].y = le_ipsp1[0].y;
			     le_ipsp1[2].x = le_ipsp1[1].x;
			     le_ipsp1[2].y = le_ipsp1[1].y + 3;
			     le_ipsp1[3].x = le_ipsp1[2].x + 2;
			     le_ipsp1[3].y = le_ipsp1[2].y;
			     le_ipsp1[4].x = le_ipsp1[3].x;
			     le_ipsp1[4].y = le_ipsp1[3].y - 3;
			     le_ipsp1[5].x = le_ipsp1[0].x + (int)((float)nb_points * x_echelle);
			     le_ipsp1[5].y = le_ipsp1[4].y;
			     un_ipsp1 = FALSE;
       			}
			if (un_ipsp2)
			{
			     le_ipsp2[1].x = le_ipsp2[0].x + (int)(t_i * x_echelle);
			     le_ipsp2[1].y = le_ipsp2[0].y;
			     le_ipsp2[2].x = le_ipsp2[1].x;
			     le_ipsp2[2].y = le_ipsp2[1].y + 3;
			     le_ipsp2[3].x = le_ipsp2[2].x + 3;
			     le_ipsp2[3].y = le_ipsp2[2].y;
			     le_ipsp2[4].x = le_ipsp2[3].x;
			     le_ipsp2[4].y = le_ipsp2[3].y - 3;
			     le_ipsp2[5].x = le_ipsp2[0].x + (int)((float)nb_points * x_echelle);
			     le_ipsp2[5].y = le_ipsp2[4].y;
 			     un_ipsp2 = FALSE;
      			}
			if (un_ipsp3)
			{
			     le_ipsp3[1].x = le_ipsp3[0].x + (int)(t_i * x_echelle);
			     le_ipsp3[1].y = le_ipsp3[0].y;
			     le_ipsp3[2].x = le_ipsp3[1].x;
			     le_ipsp3[2].y = le_ipsp3[1].y + 3;
			     le_ipsp3[3].x = le_ipsp3[2].x + 3;
			     le_ipsp3[3].y = le_ipsp3[2].y;
			     le_ipsp3[4].x = le_ipsp3[3].x;
			     le_ipsp3[4].y = le_ipsp3[3].y - 3;
			     le_ipsp3[5].x = le_ipsp3[0].x + (int)((float)nb_points * x_echelle);
			     le_ipsp3[5].y = le_ipsp3[4].y;
			     un_ipsp3 = FALSE;
       			}
			if (nb_sabliers)
			{
			      frequence = (compose_fact * nb_sabliers-1) * freq_spike;
			      if (frequence < 0.)
			      {
				   frequence = 0.;
				   nb_sabliers--;
				   in_burst = FALSE;
			      }
			}
		}
		somme_psp = un_epsp - un_ipsp;

#if DEBUG_BURST
		printf("inp_max= %f inp_t= %d\n",
		       inp_max_pos[tail],
		       inp_t[tail]);
#endif
		inp_max_pos[tail] = somme_psp;

		if(somme_psp > 0.0)
		{
			inp_ta[tail] = tau_attack_epsp / time_step ;
			inp_td[tail] = tau_decay_epsp / time_step ;
		}
		else if(somme_psp < 0.0)
		{
			inp_ta[tail] = tau_attack_ipsp / time_step;
			inp_td[tail] = tau_decay_ipsp / time_step;
		}

		tail = (tail + 1) % MAX_MEM;
		if (tail == head)
			head = (head + 1) % MAX_MEM;
	} /* fin du if */

	/*
	 * Le probleme du bruit: le bruit est lui aussi multiplie par
	 * le facteur positif s'il est positif (il a la valeur d'un
	 * epsp) et par le facteur negatif sinon (si 0 il est
	 * considere comme positif)
	 */

	net_input =  suminputs(inp_ta, inp_td,
				   inp_max_pos,
				   inp_t,
				   head,
				   tail, t_i) + noise;

#endif
	/*
	 * Maintenant on peut calculer le soma
	 * Calcul du voltage intra somatique
	 * et tenir compte du shunt de la mb qui varie en f(t)
	 */
	d_lag = (int)(decharge_lag);
	if (d_lag < MAX_SHUNT / time_step)
	{
		if (shunt_tau / time_step < .1)
			soma += net_input;
		else
			soma += (net_input
					  * ( 1. - exp(-decharge_lag * time_step/shunt_tau)));
	} /* fin du if */
	else
		soma += net_input;

	/*
	 * ca decharge ou pas?
	 */

	if (in_burst)     /* on est dans un burst */
	{
	  /*
	      soma -= seuil_burst;
	      soma = b_v_plus * 
			       (exp(b_A * soma) - 1.) 
				/ (exp(b_A * soma) - b_v_rap);
	      soma += seuil_burst;
	  */
	      if (nb_it == spike_suivante )
	      {
		    spike = TRUE;
		    decharge_lag = 0.;
		    soma_post_spike = /* soma + burst_overshoot; */
		    soma = burst_post_spike;

		    if (frequence)
		      spike_suivante = nb_it + (int)((1000./ frequence) / time_step);
	      }
	      else
	      {
		    spike = FALSE;
		    decharge_lag += 1.;
	      }
	}
	else                         /* on est hors burst */
        {
	      /*
	       * Calcul du seuil de decharge
	       */
	      threshold =   delta_seuil 
				* exp((-decharge_lag) 
				* time_step  * i_tau_seuil)
				+ seuil;

	      /*
    	      soma -= rest_pot;
	      soma = v_plus * 
			       (exp(A * soma) - 1.) 
				/ (exp(A * soma) - v_rap);
	      soma += rest_pot;
	      */

	      if (soma >= threshold)
	      {				/* on a un spike */
		    la_spike = 1;
		    spike = TRUE;
		    decharge_lag = 0.;
#if DEBUG_UPDT_1
		    printf("soma avt spike: %f\n",
			   soma);
#endif
		    soma_post_spike = /* soma + overshoot; */
		    soma = pot_post_spike;

		    /* calcul de la valeur absolue du seuil apres ce spike (seuil_peak) */

		    if (less_fatigue)
		    {
		      delta = (seuil_peak - threshold);
		      seuil_increase = (seuil_post_spike - delta)* fatigue + delta;
		    }
		    else
		    {
		      seuil_increase = seuil_post_spike * 
		                     (fatigue * (threshold - seuil) + 1.);
		    }
		    seuil_peak = seuil_increase + threshold;
		    delta_seuil = seuil_peak - seuil;
#if DEBUG_BURST
		    printf("SPIKE: iter: %d (spike suivante: %d)\n", nb_it, spike_suivante);
#endif
	      } /* fin du if */
	      else			/* pas de spike */
	      {
		    la_spike = 0;
		    spike = FALSE;
		    decharge_lag += 1.;
	      } /* fin du else */
	}
	le_potentiel = soma;
} /* fin du update_cond */






