#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* 
 *	Modify this according to your system
 */

#define	UNIX	1
#define PC	0

/*
*	askval.c - Prompt for a number with default value and line edition
*
*	Author: J-F VIBERT 
*		CHU St-Antoine PARIS
*
*	Dates:
*		29-aug-85	Version 1.0	default value
*		26-juillet-89   Version 2.0	line edition
*               29-aout-89      Version 2.1	line recalls 
*               5-sept-89      	Version 2.2	domains added 
*		12-sept-89	Version 2.3	change stdout dynamically
*		9-oct-89	Version 2.4	support 8 bits CSI prefix
*		14-janvier-89	Version 2.5	bug si redirection corrige
*		19-mars-1990	Version 2.6	modification de position 
*						du (? for help)
*		28-juin-91	Version 2.7	8-bits clean
*		9-juillet-94	Version 2.8	chgt du setcbreak pour
*						compatibilite Systeme V
*		21-oct-95	Version 2.9	askvalfree() added
*		11-aug-97	Version 2.10	Increase of buffer size because
*						screens are now larger...
*		25-nov-99	Version 2.11	VMS and RAINBOW code removed...
*		2-dec-99	Version 2.12	pb de std_ask et std_err sur Linux
*						corrige provisoirement...
*
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/


#ifdef	DOCUMENTATION

title	askval - Prompt for a value with default value and line edition
index		 Prompt for a value with default value and line edition

Synopsis


	#include <askval.h>

	/*  ask*val simple form */

	int valeur;
	valeur = askival(prompt, default);
	char    *prompt;
	int     default;

	char valeur;
	valeur = askcval(prompt, default);
	char    *prompt;
	char    default;

	float valeur, askfval();
	valeur = askfval(prompt, default);
	char    *prompt;
	float   default;

	int nb_car;
	nb_car = asksval(prompt, default, reponse);
	char    *prompt,
	        *default,
	        *reponse;

	/*  ask*valh help form */

	int valeur;
	valeur = askivalh(prompt, default, help);
	char    *prompt;
	int     default;
	char    *help;

	char valeur;
	valeur = askcvalh(prompt, default, help);
	char    *prompt;
	char    default;
	char    *help;

	float valeur, askfval();
	valeur = askfvalh(prompt, default, help);
	char    *prompt;
	float   default;
	char    *help;

	int nb_car;
	nb_car = asksvalh(prompt, default, reponse, help);
	char    *prompt,
	        *default,
	        *reponse,
	        *help;

	/*  ask*valf full form */

	int valeur;
	valeur = askivalf(prompt,default,field,
	                        ligne,col,help,h_ligne,h_col);
	char    *prompt;
	int     default,
	        field,
	        ligne,
	        col;
	char    *help;
	int     h_ligne,
	        h_col;

	char valeur;
	valeur = askcvalf(prompt, default,ligne,col,
                        help,h_ligne,h_col);
	char    *prompt;
	char    default;
	int     ligne,
	        col;
	char    *help;
	int     h_ligne,
	        h_col;

	float valeur, askfval();
	valeur = askfvalf(prompt, default,field,
	                        ligne,col,help,h_ligne,h_col);
	char    *prompt;
	float   default;
	int     field,
	        ligne,
	        col;
	char    *help;
	int     h_ligne,
	        h_col;

	int nb_car;
	nb_car = asksvalf(prompt, default, reponse,field,ligne,col,
	                          help,h_ligne,h_col);
	char    *prompt,
	        *default,
	        *reponse;
	int     field,
	        ligne,
	        col;
	char    *help;
	
	/*	related fonctions */

	askidomain(min,max)
	int     min,
	        max;

	askfdomain(min,max)
	float   min,
	        max;

	ask_speak(langue)
	char    langue;

	ask_tirets(ok)
	int     ok;

	ask_out(def_std)
	FILE	*def_std;

	void
	askvalfree(void)
	
Description

	{\bf WARNING}: To use askval, the programs must be linked or compiled
		with the {\tt -laskval} option on UNIX, the ASKVAL.LIB 
		library on MS/DOS.

	ASKVAL V2 is a collection of C functions allowing to get integers,
	floats, characters and strings from the keyboard with editing,
	history and help possibilities. It should be installed as a library.

		Askival() prompts on stderr for an integer value on stdin,
	and returns the given value. If no value is entered (only the
	carriage return key depressed), the default value is returned.
	The min and max limits are tested, and the value refused if not 
	inside the right domain.

	Askfval() follows the same principle, but waits for a real number. 

	Askcval() follows the same principle, but waits for a character.

	In both cases, no control is performed concerning the range of entered
	values, only the validity of characters is checked.

	Asksval() follows  the same principle, but waits for a string.  The
        number of characters  in  the "reponse" string is returned, or NULL
        on error or end of file.

	At the end of the program, askvalfree() frees all memory dynamically
	allocated by askval, in order to avoid memory leaks.

	The forms ask*valh() are the same functions but
	with the possibility to provide an help message that will be printed
	if needed on the first top line of the screen. The existence of an
	help possibility is flagged by a small message at the end of the
	line. By default this message is in english, but the langage can be
	choosen by ask_speak().

	The forms ask*valf() are the same functions but in full form with the
	possibility to indicate  the field length
	(indicated physically on the screen by underscores), the position of
	the prompt beginning in line and column on the screen, an help
	message and its position (line and column) on the screen.  If lines
	and columns are both 0, the prompt is written at the current
	position. If the line and columns corresponding to the help message
	are both 0, the help message is written at the current position, but
	is not flagged as existing, otherwise it behaves as in ask*valh().

	
		The forms fask*val() are the same functions reading from a
	file and thus without prompt.

		Askval sends to the stderr the prompt followed by the
	default value. 
	
	An immediate $<$CR$>$ sends this default value.

	A movement with one of the editing keys (see below) allows to edit
	this default value in order to change it.

	Any other new entry annulates the default value and takes the new
	key strokes as the beginning of a new value.

	Editing keys can be used at any moment.

	The authorized field length can be indicated by underscore
	characters (9 for int -6 on MS/DOS- and 14 for float), by a call to
	ask_tirets(1) [to get the undersores] or to ask_tirets(0) [to
	eliminate them].  By default, the underscores are off for ask*val()
	forms and ask*valh() forms.  For ask*valf() forms, underlines
	are always on.

	For ask*valh() and ask*valf(), a message flagging the existence of
	an help message is printed. The language of this message can be 
	choosen by ask_speak(langue). The possible values of langue are:

		{\obeylines
		a for american (default)
		f for french
		e for spanish (espa\~{n}a)
		d for deutsh
		i for italian
		}

	The domain limits can be modified by askidomain() and askfdomain().
	The domain defaults are set according to the word length. The global
	variables ask_i_min, ask_i_max, ask_f_min and ask_f_max hold the
	current int and float domains. To enable a new domain, ask*domain()
	must be called before the ask*val(), and becomes the new default
	domain.  To reset the default value to the full range, ask*domain()
	must be call with two identical arguments as in askidomain(0,0).

	Normally the default output is on stderr, but can
	be reset on stdout if needed using ask_out(stdout);
	
	\underline{Editing keys and external user manual.}
	
	For all the functions, askval provides an on-screen editor. The
	arrow keys have their natural role: left arrow to go backward
	character by character, rigtharrow to go forward character by
	character. The special VT100/VT2xx/VT3xx PFn keys are significant:

	{\obeylines
	LEFT ARROW:	backward one character
	RIGHT ARROW:	forward one character
	PF1:	backward to beginning of line (HOME key on PCs)
	PF2:	backward one word	(CTRL LEFT ARROW key on PCs)
	PF3:	forward	one word	(CTRL RIGHT ARROW key on PCs)
	PF4:	forward to end of line	(END key on PCs)
	UP ARROW:	backward in history
	DOWN ARROW:	forward in history
	CTRL R or Find: find previous string (F2 key on PCs)
	? or Help:	help		(F1 key on PCs)
	CTRL G:	abort entry
	CTRL L: repaint entry
	}

	PF2 and PF3 are only valid when entering strings.
	
		Literal characters are refused (with beep) when integer or
	floating values are awaited (eE are permitted for floats). 
	
		Integer or float values are refused if they are external to
	given limits. If so, a message is given with the authorized limits,
	and a replacement value awaited.

		CTRL G aborts the entry and calls back the default value.

		CTRL L repaints the line if an external message is received.

                The  10 last values entered for int, float  or  string  (no
        single characters) are memorized in a ring buffer.
        
		UP ARROW recalls back in the backward direction the strings
	memorized in the ring buffer corresponding  to  the  same  type  of
	askval function.

		DOWN  ARROW  recalls  back  in  the forward  direction  the
	strings memorized in the ring buffer corresponding to the same type
	of askval function.

		CTRL R or the Find Key (or F2 on PCs)  considers  the string
	already  typed  as the beginning of a memorized string, and recalls
	the full string if any, or beeps if not.

	?  or the HELP key (or F1 on PCs) gives an help text in reverse video
	characters, generaly at the top  of  the  screen,  or a very fugace
	message {\bf No Help} at the current position.  


	ASKVAL V2 is fully compatible with ASKVAL V1, thus any program
	previously compiled with ASKVAL V1 can be recompiled with ASKVAL V2
	in order to get the editing facilities, without any code
	modification. The behavior is the same, without help, and without
	any place holders.

Bugs

		If a string was read from stdin by using scanf("%s",string)
	before using one  of  the  askval()  fonctions, the LF character is
	left in stdin buffer,  and is considered as the terminator (default
	value) if askival(), askcval() or  askfval()  are  called after the
	scanf(),  while gets(string) remove the LF  and  thus  empties  the
	stdin buffer.  It is thus necessary  to read in string using gets()
	instead  of  scanf().    Moreover, gets() is more  performant  than
	scanf().  

Author

		Dr Jean-Francois Vibert

		CHU Saint-Antoine Paris 12
		
#endif

#define	NB_HIST	10


/* macros et definitions */

#	define CTRL_L	('\014')
#	define CTRL_G	('\007')
#	define CTRL_R	('\022')
#	define ESC	('\033')
#	define QUESTION	('?')
#	define REVERSE	("\033[7m")
#	define NORMAL	("\033[m")
#	define REDRAW	CTRL_L
#	define BEEP	putchar('\007')
#	define A_GAUCHE		putc('\b',std_ask)
#	define a_droite(nb)	fprintf(std_ask,"\033[%dC",nb)
#	define ask_tirets(ok)	ask_traits = ok;
#	define curseur(l,c)	fprintf(std_ask,"\033[0%d;%dH",l,c)

#if UNIX
#	define GETCH	getchar()
#	define ASK_I_MIN	(-2147483648)
#	define ASK_I_MAX	(2147483647)
#	define ASK_F_MIN	((float)-1.701411733192644299e+38)
#	define ASK_F_MAX	((float)1.701411733192644299e+38)
#endif

#if PC 
#	define GETCH	getch()
#	define ASK_I_MIN	(-32768)
#	define ASK_I_MAX	(32767)
#	define ASK_F_MIN	((float)-1.701411733192644299e+38)
#	define ASK_F_MAX	((float)1.701411733192644299e+38)
#endif

#if UNIX
#	define HAUT	('A')		/* ^ */
#	define BAS	('B')		/* v */
#	define DROITE	('C')		/* -> */
#	define GAUCHE	('D')		/* <- */
#	define DEBUT	('P')		/* PF1 */
#	define FIN	('S')		/* PF4 */
#	define AVANCE_MOT	('R')	/* PF3 */
#	define RECULE_MOT	('Q')	/* PF2 */
#	define TROUVE	('1')		/* Touche Find : Esc[1~ */
#	define HELP	('2')		/* Touche Help : Esc[28~ */
#	define save_curs()	fputs("\0337",std_ask)
#	define rest_curs()	fputs("\0338",std_ask)
#endif

#if PC
#	define HAUT	(72)		/* ^ */
#	define BAS	(80)		/* v */
#	define DROITE	(77)		/* -> */
#	define GAUCHE	(75)		/* <- */
#	define DEBUT	(71)		/* home */
#	define FIN	(79)		/* end */
#	define AVANCE_MOT	(116)	/* ctrl -> */
#	define RECULE_MOT	(115)	/* ctrl -> */
#	define HELP	(59)		/* F1 */
#	define TROUVE	(60)		/* F2 */
#	define save_curs()	fputs("\033[s",std_ask)
#	define rest_curs()	fputs("\033[u",std_ask)
#endif

#if UNIX
#	define I_MAX	9
#	define SLEEP		sleep(1)
#endif

#if PC
#	define I_MAX	6
#	define SLEEP		for(i = 0; i < 100; i++);
#endif

char	*ask_i_previous[NB_HIST],	/* Les chaines i d'avant */
	*ask_f_previous[NB_HIST],	/* Les chaines f d'avant */
	*ask_s_previous[NB_HIST],	/* Les chaines s d'avant */
	with_help[20] = {' ','(','?',' ','f','o','r',' ','h','e','l','p',')',0},
					/* la chaine d'help par defaut */
	no_help[15] = {'N','o',' ','h','e','l','p','\007',0},
					/* la chaine no help */
	bad_val[30] = {'B','a','d',' ','v','a','l','u','e',0};
					/*
					 * la chaine de mauvaise valeur par
					 * defaut
					 */
int	ask_i_min = ASK_I_MIN,	/* le domaine des int */
	ask_i_max = ASK_I_MAX;	

float	ask_f_min = ASK_F_MIN,	/* le domaine des float */
	ask_f_max = ASK_F_MAX;	

int	nb_i_hist,	/* le nb de chaines i memorisees */
	nb_f_hist,	/* le nb de chaines f memorisees */
	nb_s_hist,	/* le nb de chaines s memorisees */
	p_i_stack,	/* et leurs "pointeurs" */
	p_f_stack,
	p_s_stack;

int	ask_traits = 0;		/* faut-il ou non des tirets	*/
				/* les fonctions		*/


/* FILE	*std_ask = stderr; */		/* The default output */
#define std_ask stderr

static	int	max_texte,	/* longueur max de texte ecrit */
		len_texte;	/* longueur de texte ecrit */

static	ask_edite(),
	re_ecrit();

/*
*	ASKVALFREE()	permet de liberer la memoire de l'histoire
*/
void	askvalfree()
{
	int	i;
	
	for (i = 0; i < NB_HIST; i++)
	{
		if (ask_i_previous[i])
			free(ask_i_previous[i]);
		if (ask_i_previous[i])
			free(ask_f_previous[i]);
		if (ask_i_previous[i])
			free(ask_s_previous[i]);
	}
} /* fin du askfdomain */

/*
*	ASKIDOMAIN()	permet de mettre le domaine a une nouvelle valeur
*/
askidomain(min,max)
int	min,
	max;
{
	if (min == max)		/* reset aux valeurs par defaut */
	{
		ask_i_min = ASK_I_MIN;
		ask_i_max = ASK_I_MAX;
	} /* fin du if */
	else
	{
		ask_i_min = min;
		ask_i_max = max;
	}
} /* fin du askidomain */

/*
*	ASKFDOMAIN()	permet de mettre le domaine a une nouvelle valeur
*/
askfdomain(min,max)
float	min,
	max;
{
	if (min == max)		/* reset aux valeurs par defaut */
	{
		ask_f_min = ASK_F_MIN;
		ask_f_max = ASK_F_MAX;
	} /* fin du if */
	else
	{
		ask_f_min = min;
		ask_f_max = max;
	}
} /* fin du askfdomain */

/*
*	ASK_SPEAK() permet de choisir la langue utilisee
*		a = anglais (defaut)
*		f = francais
*		i = italien
*		e = espagnol
*		d = allemand (deutsh)
*/
ask_speak(langue)
char	langue;
{
	switch (langue)
	{
		case 'f':
			strcpy(with_help," (? pour aide)");
			strcpy(no_help,"Pas d'aide\007");
			strcpy(bad_val,"Mauvaise valeur");
			break;
		case 'i':
			strcpy(with_help," (? per aiuto)");
			strcpy(no_help,"No aiuto\007");
			strcpy(bad_val,"Cattiva valora");
			break;
		case 'd':
			strcpy(with_help," (? fur Hilfe");
			strcpy(no_help,"Nicht Hilfe\007");
			strcpy(bad_val,"Nicht Gut");
			break;
		case 'e':
			strcpy(with_help," (? por ayudad)");
			strcpy(no_help,"No ayudad\007");
			strcpy(bad_val,"No bueno");
			break;
		case 'a':
		default:
			strcpy(with_help," (? for help)");
			strcpy(no_help,"No help\007");
			strcpy(bad_val,"Bad value");
	} /* fin du switch (langue) */
} /* fin du ask_speak */

/*
*	ASK_OUT() permet de rediriger la sortie vers stderr ou stdout
*/
ask_out(std)
FILE	*std;
{
/*
	std_ask = std;
*/
	return;
} /* fin du ask_out */

/*
*
*	ASKIVAL & FASKIVAL & ASKIVALF & ASKIVALH
*		prompte pour une valeur entiere sur stdin
*		retourne cette valeur entiere, ou bien une
*		valeur par defaut si return seul est entre
*/
askival(prompt,defaut)
char *prompt;
int  defaut;
{
	char	ch_recu[20],
		hors_lim[50];
	int	recu;
	
	do
	{
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%d",defaut);
		ask_edite(I_MAX,ch_recu,'i',ask_i_previous,no_help,0,0);
		if (strlen(ch_recu)) recu = atoi(ch_recu);
		else recu = defaut;
		if (ask_i_min == ask_i_max) break;
		if (recu < ask_i_min || recu > ask_i_max)
		{
			sprintf(hors_lim,"%s (%d-%d)",bad_val,ask_i_min,ask_i_max);
			a_droite(254-strlen(hors_lim));
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (recu < ask_i_min || recu > ask_i_max);
	return(recu);
}

askivalh(prompt,defaut,help)
char *prompt;
int  defaut;
char *help;
{
	char	ch_recu[20],
		hors_lim[50];
	int	recu;
	
	do
	{
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%d",defaut);
		ask_edite(I_MAX,ch_recu,'i',ask_i_previous,help,1,1);
		if (strlen(ch_recu)) recu = atoi(ch_recu);
		else recu = defaut;
		if (ask_i_min == ask_i_max) break;
		if (recu < ask_i_min || recu > ask_i_max)
		{
			sprintf(hors_lim,"%s (%d-%d)",bad_val,ask_i_min,ask_i_max);
			a_droite(254-strlen(hors_lim));
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (recu < ask_i_min || recu > ask_i_max);
	return(recu);
}

askivalf(prompt,defaut,field,ligne,col,help,h_l,h_c)
char	*prompt;
int 	defaut,
	field,
	ligne,
	col;
char	*help;
int	h_l,
	h_c;
{
	char	ch_recu[20],
		hors_lim[50];
	int	recu;
	
	ask_traits = 1;
	do
	{
		if (ligne && col) curseur(ligne,col);
		if (field > I_MAX) field = I_MAX;
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%d",defaut);
		ask_edite(field,ch_recu,'i',ask_i_previous,help,h_l,h_c);
		if (strlen(ch_recu)) recu = atoi(ch_recu);
		else recu = defaut;
		if (ask_i_min == ask_i_max) break;
		if (recu < ask_i_min || recu > ask_i_max)
		{
			if (ligne && col) curseur(ligne,col);
			sprintf(hors_lim,"%s (%d-%d)",bad_val,ask_i_min,ask_i_max);
			a_droite(254-strlen(hors_lim)-col);
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (recu < ask_i_min || recu > ask_i_max);
	return(recu);
}

faskival(pf,defaut)
FILE *pf;
int  defaut;
{
	char ch_recu[25];

	fgets(ch_recu,25,pf);
	if (strlen(ch_recu)) return(atoi(ch_recu));
	return(defaut);
}
/*
*
*	ASKCVAL & FASKCVAL & ASKCVALF & ASKFVALH
*		prompte pour un caractere ou un string sur stdin
*		retourne ce caractere ou la 1ere lettre du string,
*		ou bien une valeur par defaut si return seul est entre
*/

char askcval(prompt,defaut)
char *prompt;
char  defaut;
{
	char ch_recu[3];

	fputs(prompt,std_ask);
	ch_recu[0] = defaut;
	ch_recu[1] = 0;
	ask_edite(1,ch_recu,'c',0,no_help,0,0);
	if (strlen(ch_recu)) return(ch_recu[0]);
	return(defaut);
}

char askcvalh(prompt,defaut,help)
char *prompt;
char  defaut;
char *help;
{
	char ch_recu[3];

	fputs(prompt,std_ask);
	ch_recu[0] = defaut;
	ch_recu[1] = 0;
	ask_edite(1,ch_recu,'c',0,help,1,1);
	if (strlen(ch_recu)) return(ch_recu[0]);
	return(defaut);
}

char askcvalf(prompt,defaut,ligne,col,help,h_l,h_c)
char *prompt;
char  defaut;
int	ligne,
	col;
char *help;
int	h_l,
	h_c;
{
	char ch_recu[3];

	ask_traits = 1;
	if (ligne &&  col) curseur(ligne,col);	
	fputs(prompt,std_ask);
	ch_recu[0] = defaut;
	ch_recu[1] = 0;
	ask_edite(1,ch_recu,'c',0,help,h_l,h_c);
	if (strlen(ch_recu)) return(ch_recu[0]);
	return(defaut);
}

char faskcval(pf,defaut)
FILE *pf;
char  defaut;
{
	char ch_recu[25];
	fgets(ch_recu,25,pf);
	if (strlen(ch_recu)) return(ch_recu[0]);
	return(defaut);
}
/*
*
*	ASKFVAL & FASKFVAL & ASKFVALF &ASKFVALH
*		prompte pour une valeur reelle sur stdin
*		retourne cette valeur reelle, ou bien une
*		valeur par defaut si return seul est entre.
*
*/

float askfval(prompt,defaut)
char *prompt;
float  defaut;
{
	float	f_val;
	char	ch_recu[25],
		hors_lim[50];

	do
	{
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%f",defaut);
		ask_edite(24,ch_recu,'f',ask_f_previous,no_help,0,0);
		if (strlen(ch_recu))
			sscanf(ch_recu,"%f",&f_val);
		else f_val = defaut;
		if (ask_f_min == ask_f_max) break;
		if (f_val < ask_f_min || f_val > ask_f_max)
		{
			sprintf(hors_lim,"%s (%.1f-%.1f)",bad_val,ask_f_min,ask_f_max);
			a_droite(254-strlen(hors_lim));
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (f_val < ask_f_min || f_val > ask_f_max);
	return(f_val);
}

float askfvalh(prompt,defaut,help)
char *prompt;
float  defaut;
char *help;
{
	float	f_val;
	char	ch_recu[25],
		hors_lim[50];

	do
	{
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%f",defaut);
		ask_edite(24,ch_recu,'f',ask_f_previous,help,1,1);
		if (strlen(ch_recu))
			sscanf(ch_recu,"%f",&f_val);
		else f_val = defaut;
		if (ask_f_min == ask_f_max) break;
		if (f_val < ask_f_min || f_val > ask_f_max)
		{
			sprintf(hors_lim,"%s (%.1f-%.1f)",bad_val,ask_f_min,ask_f_max);
			a_droite(254-strlen(hors_lim));
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (f_val < ask_f_min || f_val > ask_f_max);
	return(f_val);
}

float askfvalf(prompt,defaut,field,ligne,col,help,h_l,h_c)
char	*prompt;
float 	defaut;
int	field,
	ligne,
	col;
char	*help;
int	h_l,
	h_c;
{
	float	f_val;
	char	ch_recu[25],
		hors_lim[50];

	ask_traits = 1;
	do
	{
		if (ligne &&  col) curseur(ligne,col);
		if (field > 24) field = 24;
		fputs(prompt,std_ask);
		sprintf(ch_recu,"%f",defaut);
		ask_edite(field,ch_recu,'f',ask_f_previous,help,h_l,h_c);
		if (strlen(ch_recu))
			sscanf(ch_recu,"%f",&f_val);
		else f_val = defaut;
		if (ask_f_min == ask_f_max) break;
		if (f_val < ask_f_min || f_val > ask_f_max)
		{
			if (ligne && col) curseur(ligne,col);
			sprintf(hors_lim,"%s (%.1f-%.1f)",bad_val,ask_f_min,ask_f_max);
			a_droite(254-strlen(hors_lim)-col);
			fprintf(std_ask,"\007%s%s%s\r",REVERSE,hors_lim,NORMAL);
		} /* fin du if */
	} while (f_val < ask_f_min || f_val > ask_f_max);
	return(f_val);
}

float faskfval(pf,defaut)
FILE *pf;
float  defaut;
{
	float f_val;
	char ch_recu[25];

	fgets(ch_recu,25,pf);
	if (strlen(ch_recu))
	{
		sscanf(ch_recu,"%f",&f_val);
		return(f_val);
	}
	return(defaut);
}

/*
*
*	ASKSVAL  & FASKSVAL & ASKSVALF & ASKSVALH
*		prompte pour une chaine de caracteres sur stdin
*		retourne le buffer, ou bien une
*		valeur par defaut si return seul est entre
*/

asksval(prompt,defaut,renvoi)
char *prompt, *defaut, *renvoi;
{
	char ch_recu[256];
	int nb_car;

	fputs(prompt,std_ask);
	sprintf(ch_recu,"%s",defaut);
	ask_edite(254 - strlen(prompt),ch_recu,'s',ask_s_previous,no_help,0,0);
	if (nb_car=strlen(ch_recu))
	{
		strcpy(renvoi,ch_recu);
		return(nb_car);
	}
	strcpy(renvoi,defaut);
	nb_car=strlen(defaut);
	return(nb_car);
}

asksvalh(prompt,defaut,renvoi,help)
char *prompt, *defaut, *renvoi, *help;
{
	char ch_recu[256];
	int nb_car;

	fputs(prompt,std_ask);
	sprintf(ch_recu,"%s",defaut);
	ask_edite(254-strlen(prompt)-strlen(with_help),ch_recu,'s',ask_s_previous,help,1,1);
	if (nb_car=strlen(ch_recu))
	{
		strcpy(renvoi,ch_recu);
		return(nb_car);
	}
	strcpy(renvoi,defaut);
	nb_car=strlen(defaut);
	return(nb_car);
}

asksvalf(prompt,defaut,renvoi,field,ligne,col,help,h_l,h_c)
char *prompt, *defaut, *renvoi;
int	field,
	ligne,
	col;
char *help;
int	h_l,
	h_c;
{
	char ch_recu[256];
	int nb_car;

	ask_traits = 1;
	if (ligne &&  col) curseur(ligne,col);
	if (field > 255) field = 255;
	fputs(prompt,std_ask);
	sprintf(ch_recu,"%s",defaut);
	ask_edite(field,ch_recu,'s',ask_s_previous,help,h_l,h_c);
	if (nb_car=strlen(ch_recu))
	{
		strcpy(renvoi,ch_recu);
		return(nb_car);
	}
	strcpy(renvoi,defaut);
	nb_car=strlen(defaut);
	return(nb_car);
}

fasksval(pf,defaut,renvoi)
FILE *pf;
char *defaut, *renvoi;
{
	char ch_recu[256];
	int nb_car;

	fgets(ch_recu,255,pf);
	if (nb_car=strlen(ch_recu))
	{
		strcpy(renvoi,ch_recu);
		return(nb_car);
	}
	strcpy(renvoi,defaut);
	nb_car=strlen(defaut);
	return(nb_car);
}

#if UNIX

#include <termio.h>

static	struct termio tbuf; 
static	struct termio otbuf; /* old values */

static void setcbreak()
{

	
	if(ioctl(0,TCGETA, (char *) &otbuf) == -1)
	{
		printf("askval: Impossible to get the terminal characteristics\n");
		printf("You probably redirect the stdin... (not allowed)\n");
		exit(1);
	}

	tbuf = otbuf;
	tbuf.c_lflag &= ~(ECHO | ECHOK | ECHONL | ICANON);
	tbuf.c_oflag &= ~(OPOST);
	tbuf.c_iflag &= ~(INLCR | IGNCR | ICRNL);
	tbuf.c_cc[VMIN] = 1;
	tbuf.c_cc[VTIME] = 1;

	if (ioctl(0,TCSETAW,&tbuf) == -1)
	{
		printf("askval: Impossible to put the terminal in CBREAK mode\n");
		exit(1);
	}
	
}

static void restore()
{

	if (ioctl(0,TCSETAW,&otbuf) == -1)
		printf("Impossible to restore the terminal characteristics\n");
		
}
#endif

/*
*	ASK_EDITE() permet de faire de l'edition pleine page de la reponse
*	avant de la sauver.
*/
static ask_edite(field_length,texte,type,history,aide,h_ligne,h_col)
int     field_length;   /* length of the field to edit */
char	*texte,		/* the filled text */
	type,		/* i, f, c, s */
	*history[NB_HIST];
char	*aide;		/* texte du help */
int	h_ligne,	/* ligne et colonne du help */
	h_col;
{

	int	col_crte,	/* colonne courante */
		keep = 1,	/* flag */
		deplcmt = 0,	/* flag */
		hist_ptr,	/* pointeur sur l'historique */
		current_stack,	/* indice du ring buffer en cours */
		nb_hist,	/* nb de chaines stockees */
		help,		/* is help needed */
		i, k;		/* indices */
		
	char 	c,		/* le charactere lu */
		place,	/*
				 * ce qui s'imprime a la place d'un caractere
				 * vide si ask_traits est a 1
				 */
		defaut[256],	/* la valeur par defaut */
		corrige[256];	/* buffer de correction */

	if (!isatty(0))	/* si c'est redirige, l'edition n'a pas de sens */
	{
		fgets(texte,255,stdin);
		return;
	} /* fin du if */

#if UNIX
	setcbreak();		/* met le terminal en cbreak */
#endif
	texte[field_length] = 0;	/* limite le champ ecrit au champ alloue */
	max_texte = 0;
	save_curs();
	col_crte = 0;		/* On ecrit d'abord le valeur par defaut */
	strcpy(defaut,texte);
	strcpy(corrige,texte);
	fprintf(std_ask,"%s",texte);
		/* suivie ou non de place holders */
	len_texte = strlen(texte);
	if ( len_texte > max_texte) max_texte = len_texte;
	if (ask_traits)	place = '_';
	else 		place = ' ';
	if (h_ligne && h_col) help = 1;
	else help = 0;
	if (help || ask_traits) for (i = strlen(texte); i < field_length; i++)
			putc(place,std_ask);
	if (help)
	{
		curseur(h_ligne,h_col);
		fputs(with_help,std_ask);
	} /* fin du if */
	rest_curs();

	/* on pointe sur la bonne reponse en fonction du type */
	switch(type)
	{
		case 'i':
			current_stack = p_i_stack;
			nb_hist = nb_i_hist;
			break;
		case 'f':
			current_stack = p_f_stack;
			nb_hist = nb_f_hist;
			break;
		case 's':
			current_stack = p_s_stack;
			nb_hist = nb_s_hist;
	} /* fin du switch(type) */
	if (type != 'c') hist_ptr = current_stack;
	
	while (1)
	{
		c = GETCH;
		if (c == '\015')	/* c'est fini, on a fait CR */
		{
			fputs("\n\r",std_ask);
			break;
		} /* fin du if */
#if UNIX
		if ((c & 0177) == ESC)		/* c'est le debut dune touche speciale */
		{
			if ( (c & 0377 == '\233') || (c = GETCH) == '[' || c == 'O')
			{
#endif
#if PC
		if (c == 0)		/* c'est le debut dune touche speciale */
		{
#endif
				c = GETCH;
				switch (c)
				{
					case HAUT:
						if (type == 'c' || !nb_hist) break;
						if (hist_ptr) hist_ptr--;
						else hist_ptr = nb_hist-1;
						strcpy(texte,history[hist_ptr]);
						re_ecrit(field_length,texte,place,help,h_ligne,h_col);
						col_crte = 0;
						break;
					case BAS:
						if (type == 'c' || !nb_hist) break;
						if (hist_ptr < nb_hist-1) hist_ptr++;
						else hist_ptr = 0;
						strcpy(texte,history[hist_ptr]);
						re_ecrit(field_length,texte,place,help,h_ligne,h_col);
						col_crte = 0;
						break;
					case DROITE:
						col_crte++;
						if (col_crte == strlen(texte)+1)
						{
							BEEP;
							col_crte--;
							break;
						}
						a_droite(1);
						deplcmt = 1;
						break;
					case GAUCHE:
						col_crte--;
						if (col_crte < 0)
						{
							BEEP;
							col_crte++;
							break;
						}
						A_GAUCHE;
						deplcmt = 1;
						break;
					case DEBUT:	/* debut de ligne */
						rest_curs();
						col_crte = 0;
						deplcmt = 1;
						break;
					case RECULE_MOT:	/* back par mot */
						if (type != 's')
						{
							BEEP;
							break;
						} /* fin du if */
						while(texte[col_crte] != ' ')
						{
							if (col_crte)
							{
								col_crte--;
							} /* fin du if */
							else
							{
								BEEP;
								break;
							} /* fin du else */
						} /* fin du while */
						rest_curs();
						if (col_crte)
						{
							col_crte--;
							a_droite(col_crte);
						} /* fin du if */
						deplcmt = 1;
						break;
					case AVANCE_MOT:	/* forward par mot */
						if (type != 's')
						{
							BEEP;
							break;
						} /* fin du if */
						while(texte[col_crte] != ' ')
						{
							if (col_crte < strlen(texte))
							{
								col_crte++;
							} /* fin du if */
							else
							{
								BEEP;
								break;
							} /* fin du else */
						} /* fin du while( */
						col_crte++;
						rest_curs();
						a_droite(col_crte);
						deplcmt = 1;
						break;
					case FIN:	/* fin de ligne */
						a_droite(strlen(texte)-col_crte);
						col_crte = strlen(texte);
						deplcmt = 1;
						break;
#if PC
					case HELP:	/* aide */
						if (help)
						{
							curseur(h_ligne,h_col);
							fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
							rest_curs();
						} /* fin du if */
						else
						{
							rest_curs();
							fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
							rest_curs();
							len_texte = strlen(texte);
							if ( len_texte > max_texte) max_texte = len_texte;
							fprintf(std_ask,"%s",texte);
							len_texte = strlen(texte);
							if ( len_texte > max_texte) max_texte = len_texte;
							for(i = 0; i < max_texte; i++) putc(' ',std_ask);
							rest_curs();
							if (ask_traits) for (i = strlen(texte); i < field_length; i++)
								putc(place,std_ask);
							rest_curs();
							if (col_crte) a_droite(col_crte);
						} /* fin du else */
						break;
					case TROUVE:	/* recherche dans l'histoire */
						if (type == 'c' || !nb_hist) break;
						for (i = 0; i < nb_hist; i++)
						{
							if (hist_ptr) hist_ptr--;
							else hist_ptr = nb_hist-1;
							if (strncmp(history[hist_ptr],texte,strlen(texte))) continue;
							else
							{
								strcpy(texte,history[hist_ptr]);
								re_ecrit(field_length,texte,place,help,h_ligne,h_col);
								col_crte = 0;
								break;
							} /* fin du else */
						} /* fin du for */
						BEEP;	/* pas trouve */
						break;
#endif
#if UNIX
					case TROUVE:
						c = GETCH;
						if ( c == '~')	/* touche Find */
						{
							if (type == 'c' || !nb_hist) break;
							for (i = 0; i < nb_hist; i++)
							{
								if (hist_ptr) hist_ptr--;
								else hist_ptr = nb_hist-1;
								if (strncmp(history[hist_ptr],texte,strlen(texte))) continue;
								else
								{
									strcpy(texte,history[hist_ptr]);
									re_ecrit(field_length,texte,place,help,h_ligne,h_col);
									col_crte = 0;
									break;
								} /* fin du else */
							} /* fin du for */
							BEEP;	/* pas trouve */
						} /* fin du if */
						break;
					case HELP:	/* aide */
						c = GETCH;
						if ( c == '8')	/* touche Help ? */
						{
							c = GETCH;
							if ( c == '~')	/* touche Help */
							{
								if (help)
								{
									curseur(h_ligne,h_col);
									fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
									rest_curs();
								} /* fin du if */
								else
								{
									rest_curs();
									fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
									fflush(std_ask);
									rest_curs();
									len_texte = strlen(texte);
									if ( len_texte > max_texte) max_texte = len_texte;
									SLEEP;
									len_texte = strlen(texte);
									if ( len_texte > max_texte) max_texte = len_texte;
									for(i = 0; i < max_texte; i++) putc(' ',std_ask);
									rest_curs();
									fprintf(std_ask,"%s",texte);
									if(ask_traits)
										for (i = strlen(texte); i < field_length; i++)
															putc(place,std_ask);
									rest_curs();
									if (col_crte) a_droite(col_crte);
								} /* fin du else */
							} /* fin du if */
						} /* fin du if */
						break;
#endif
					default:
						BEEP;
				}
#if UNIX
			}
#endif
		}
		else
		{
			if ( c == '\177' || c == '\010')	/* delete ou BS */
			{
				if (col_crte == 0)
				{
					BEEP;
					continue;
				}
				for (i = 0; i < 255; i++) corrige[i] = 0;
				k = col_crte;
				strncpy(corrige,texte,k-1);
				strcat(corrige,&texte[k]);
				strcpy(texte,corrige);
				col_crte--;
				fprintf(std_ask,"\b%s%c",&texte[k-1],(place)?place:' ');
				rest_curs();
				if (col_crte) a_droite(col_crte);
				continue;	
			}
			/* On annule et on reprend le defaut */
			if ( c == CTRL_G )	
			{
				BEEP;
				keep = 1;
				rest_curs();
				col_crte = 0;
				strcpy(texte,defaut);
				strcpy(corrige,texte);
				fprintf(std_ask,"%s",texte);
				if (ask_traits || help)
					for (i = strlen(texte); i < field_length; i++)
						putc(place,std_ask);
				if (help)
				{
					curseur(h_ligne,h_col);
					fputs(with_help,std_ask);
				}
				rest_curs();
			} /* fin du if */
			if ( c == REDRAW )
			{
				BEEP;
				re_ecrit(field_length,texte,place,help,h_ligne,h_col);
				if (col_crte) a_droite(col_crte);
				continue;
			} /* fin du if */
			if (c == QUESTION )
			{
				if (help)
				{
					curseur(h_ligne,h_col);
					fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
					rest_curs();
					if (col_crte) a_droite(col_crte);
				} /* fin du if */
				else
				{
					rest_curs();
					fprintf(std_ask,"%s\007%s%s",REVERSE,aide,NORMAL);
					fflush(std_ask);
					rest_curs();
					SLEEP;
					len_texte = strlen(texte);
					if ( len_texte > max_texte) max_texte = len_texte;
					for(i = 0; i < max_texte; i++) putc(' ',std_ask);
					rest_curs();
					fprintf(std_ask,"%s",texte);
					if (ask_traits) 
						for (i = strlen(texte); i < field_length; i++)
							putc(place,std_ask);
					rest_curs();
					if (col_crte) a_droite(col_crte);
				} /* fin du else */
				continue;
			} /* fin du if */
			if (c == CTRL_R )
			{
				if (type == 'c' || !nb_hist) continue;
				for (i = 0; i < nb_hist; i++)
				{
					if (hist_ptr) hist_ptr--;
					else hist_ptr = nb_hist-1;
					if (strncmp(history[hist_ptr],texte,strlen(texte))) continue;
					else
					{
						strcpy(texte,history[hist_ptr]);
						re_ecrit(field_length,texte,place,help,h_ligne,h_col);
						col_crte = 0;
						break;
					} /* fin du else */
				} /* fin du for */
				BEEP;	/* pas trouve */
				continue;
			} /* fin du if */

			if ( c < ' ' && c >= 0) /* entre le debut et le space */
			{
				BEEP;
				continue;
			}
			if (keep && !deplcmt)
			{
				keep = 0;
				rest_curs();
				for(i = 0; i < max_texte; i++) putc(' ',std_ask);
				rest_curs();
				if (ask_traits || help)
					for (i = 0; i < field_length; i++)
						putc(place,std_ask);
				if (help)
				{
					curseur(h_ligne,h_col);
					fputs(with_help,std_ask);
				} /* fin du if */
				rest_curs();
				col_crte = 0;
				texte[0] = 0; /* on elimine le defaut */
			} /* fin du if */
			if (col_crte == field_length)
			{
				BEEP;
				continue;
			} /* fin du if */
			if( type == 'i')
			{
				if (!strchr("1234567890-+",c))
				{
					BEEP;
					continue;
				} /* fin du if */
			}
			if( type == 'f')
			{
				if (!strchr("1234567890-+.eE",c))
				{
					BEEP;
					continue;
				} /* fin du if */
			}
			for (i = 0; i < 255; i++) corrige[i] = 0;
			strncpy(corrige,texte,col_crte);
			corrige[col_crte] = c;
			corrige[col_crte+1] = 0;
			strcat(corrige,&texte[col_crte]);
			corrige[field_length] = 0;
			strcpy(texte,corrige);
			fprintf(std_ask,"%s",&texte[col_crte]);
			col_crte++;
			rest_curs();
			a_droite(col_crte);
			if (col_crte > field_length)
			{
				col_crte--;
				A_GAUCHE;
				BEEP;
			} /* fin du if */
		}
	}
#if UNIX
	restore();	/* repasse en mode cooked */
#endif
	if (type != 'c')
	{
		history[current_stack] = (char *)malloc((strlen(texte)+1));
		strcpy(history[current_stack],texte);
		if (current_stack < NB_HIST-1) current_stack++;
		else current_stack = 0;
		if (nb_hist < NB_HIST) nb_hist ++;

		/* on restocke  en fonction du type */
		switch(type)
		{
			case 'i':
				p_i_stack = current_stack;
				nb_i_hist = nb_hist;
				break;
			case 'f':
				p_f_stack = current_stack;
				nb_f_hist = nb_hist;
				break;
			case 's':
				p_s_stack = current_stack;
				nb_s_hist = nb_hist;
		} /* fin du switch(type) */
	} /* fin du if */
}
/*
*	RE_ECRIT() reecrit la ligne et se met au debut
*/
static re_ecrit(field_length,texte,place,help,h_ligne,h_col)
int	field_length;
char	*texte,
	place;
int	help,
	h_ligne,
	h_col;
{
	int	i;
	
	rest_curs();
	len_texte = strlen(texte);
	if (len_texte > max_texte) max_texte = len_texte;
	for(i = 0; i < max_texte; i++) putc(' ',std_ask);
	rest_curs();
	fprintf(std_ask,"%s",texte);
	if (ask_traits || help)
		for (i = strlen(texte); i < field_length; i++)
			putc(place,std_ask);
	if (help)
	{
		curseur(h_ligne,h_col);
		fputs(with_help,std_ask);
	} /* fin du if */
	rest_curs();
	fflush(std_ask);
}
