/*****************************************************************************
*                              HEADER MODULE                              
******************************************************************************
* APPLICATION  : MENU	                                             
* NAME         : MENU_MNGT_PROCESS.c	                                             
* GOAL         : This module groups the callbacks which run new process.
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* COPYRIGHT    : Copyright Dr J-F VIBERT
*		 Laboratoire de Biomathematiques
*		 URBB B3E INSERM U263
*		 Faculte de Medecine Saint-Antoine
*		 27 rue Chaligny
*		 75571 Paris cedex 12
*	         FRANCE
*	         
*                e-mail vibert@b3e.jussieu.fr
*
*                All rights reserved
*	         Access limited to authorized persons	
*	         This software was developped only for resarch purpose
*	         and is not, in any way, usable for commercial purpose.
*****************************************************************************/


/*****************************************************************************
*                       INCLUDE LIBRARY                                     
*****************************************************************************/

#include <stdio.h> 		/***     ***/
#include <unistd.h> 		/*** for exec fork ***/
#include <stdlib.h> 		/* for putenv ***/
#include <string.h> 		/* for str ***/

/*****************************************************************************
*                       INCLUDE SOURCE MODULE                               
*****************************************************************************/

#include "MENU_constante.h"	/*** ctes definition of the application ***/


/*****************************************************************************
*                       DEFINE SECTION                               
*****************************************************************************/


/*****************************************************************************
*                       LIST OF TYPES USED                               
*****************************************************************************/

/*********************** LIST OF EXTERN TYPE DECLARATIONS USED            ***/
                                
/*********************** LIST OF EXPORTABLE TYPES CREATED                 ***/ 
                         
/*********************** LIST OF LOCAL TYPE CREATED                       ***/ 

/*********************** see also include files                           ***/
 
                      
/*****************************************************************************
*                       LIST OF VARIABLES USED                               
*****************************************************************************/

/*********************** LIST OF EXTERN VARIABLE DECLARATIONS USED        ***/
extern int mode_verbose;
extern char *xnbc_bin_ref_xterm;
extern char *current_path;
extern char *current_experiment;
extern char **envg;
extern int  on_garde;
extern struct stool 	{
			char name[80];
			int  emulator;
			int  available;
			int  pid;
			int  path;
			} 		tool[NB_TOOLS] ;

	
/*********************** LIST OF EXPORTABLE VARIABLES CREATED             ***/ 

/*********************** LIST OF LOCAL VARIABLES CREATED                  ***/

/*********************** see also include files                           ***/

                         
/*****************************************************************************
*                       LIST OF FUCNTIONS USED                               
*****************************************************************************/

/*********************** LIST OF EXTERN FUNCTIONS USED                    ***/ 
extern void     MENU_LIB_alloc();
extern void     MENU_LIB_free();
extern  void MENU_LIB_erreur();
extern  void MENU_WIN_DIALOG_create_message();

/*********************** LIST OF EXPORTABLE FUCNTIONS CREATE              ***/                          

void MENU_MNGT_PROCESS_spawn(int flag_xterm,char *path_prg,int indice_tool);


/*********************** LIST OF LOCAL FUNCTIONS IMPLENENTED              ***/ 
static void MENU_MNGT_PROCESS_change_env();

                      

/*****************************************************************************
*                              HEADER FUNCTION                              
******************************************************************************
* NAME	       : MENU_MNGT_PROCESS_spawn	
* GOAL         : run a process corresponding to the tool specifications passed
*                via the parameters.
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* SCOPE        : local of the module. 	
* INPUT        : ref	: the path and the name of the tool
*                argf   : the name of the work file if it is necessary
*                arg1   : the path and the name of the tool if it is in a xterm
*                         process that the tool is running.
*                indice_tool : the index of the tool in the array where the pid
*                              of the process is stored.	
* OUTPUT       : none	
* RETURN       : none	
*****************************************************************************/

void	MENU_MNGT_PROCESS_spawn(int flag_xterm,char *path_prg,int indice_tool)
  {
  int pid;
  char generic_name[512];
  char path_xterm[512];
  char keep_ps[10];
  char work_name[512];
  char *pc;
#ifdef linux
  char link_edit;
  char buffer[256];
#endif
  MACRO_TRACE(("---> start  : MENU_MNGT_PROCESS_spawn\n"))
    
    
  strcpy(generic_name,current_experiment);
  pc = strrchr(generic_name,'_');
  *pc = 0;


      
#ifdef linux
/*
 * Je ne sais pas pourquoi, mais sous linux, link_edt se termine des qu'on
 * l'appelle via le bouton, donc on le traite differemment
 */

	if (!strncmp(strrchr(path_prg,'/'),"/link_edit",10))
	{
		if (mode_verbose==ON)
		        printf("+---> Link_edit running...\n\n");
		switch(flag_xterm)
		{
			case 3: 
				sprintf(buffer,"%s -e %s",xnbc_bin_ref_xterm, path_prg);
				if (mode_verbose==ON)
			               printf("---> The child process\n%s\nis running...\n\n",buffer);
				system(buffer);
				break;
			case 2: if (mode_verbose==ON)
			               printf("---> The child process\n%s\nis running...\n\n",path_prg);

				sprintf(buffer,"%s",path_prg);
				system(buffer);
				break;
			case 1: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
				sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
				sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
				if (mode_verbose==ON)
					printf("---> The child process\n%s -e %s %s %s\nis running...\n\n",path_xterm,path_prg,keep_ps,work_name); 
				if (on_garde)
				{
					sprintf(buffer,"%s %s %s",path_prg,keep_ps,work_name);
					system(buffer);
				} 
				else
				{
					sprintf(buffer,"%s %s",path_prg,work_name);
					system(buffer);
				} /* fin du else */
				break;
			case 0:
				sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
				sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
				if (mode_verbose==ON)
					printf("---> The child process\n%s %s %s\nis running...\n\n",path_prg,keep_ps,work_name);
				if (on_garde)
				{
					sprintf(buffer,"%s %s %s",path_prg,keep_ps,work_name);
					system(buffer);
				} 
				else
				{
					sprintf(buffer,"%s %s",path_prg,work_name);
					system(buffer);
				} 
				break; 
		}
	}
	else
	{
		/*** create a fork ***/
		pid=fork();
		MACRO_TRACE(("     in the processus %d : fork = %d\n",getpid(),pid));

		if (pid == 0)			/*** child processus ***/
		{

			MENU_MNGT_PROCESS_change_env();
		
			switch(flag_xterm)
			{
			        case 3: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
					if (mode_verbose==ON)
						printf("---> The child process\n%s -e %s %s %s\nis running...\n\n",path_xterm,path_prg); 

					if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,NULL,envg) == -1) 
						MENU_LIB_erreur();
				       break;
				case 2: if (mode_verbose==ON)
					printf("---> The child process\n%s\nis running...\n\n",path_prg);
					if (execle(path_prg,path_prg,NULL,envg)==-1)
						MENU_LIB_erreur();
					break;
				case 1: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
					sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
					sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
					if (mode_verbose==ON)
						printf("---> The child process\n%s -e %s %s %s\nis running...\n\n",path_xterm,path_prg,keep_ps,work_name); 
					if (on_garde)
					{
						if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,keep_ps,work_name,NULL,envg) == -1) 
						MENU_LIB_erreur();
					} 
					else
					{
						if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,work_name,NULL,envg) == -1) 
						MENU_LIB_erreur();
					} /* fin du else */
					break;
		
			        case 4: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
				        if (mode_verbose==ON)
					  printf("---> The child process\n%s -e %s\nis running...\n\n",path_xterm,path_prg); 
					if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,NULL,envg)==-1) 
					  MENU_LIB_erreur();
					break;
	
				case 0:
					sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
/*					sprintf(verbose,   "%s",(mode_verbose)?" -v ":""); */
					sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
					if (mode_verbose==ON)
						printf("---> The child process\n%s %s %s\nis running...\n\n",path_prg,keep_ps,work_name);
					if (on_garde)
					{
						if (execle(path_prg,path_prg,keep_ps,work_name,NULL,envg)==-1) 
						MENU_LIB_erreur();
					} 
					else
					{
						if (execle(path_prg,path_prg,work_name,NULL,envg)==-1) 
						MENU_LIB_erreur();
					} /* fin du else */
					break; 
			}
		}
		else 				/*** parent processus ***/
		{				/*** stored the pid of the child processus ***/
			tool[indice_tool].pid=pid;
			MACRO_TRACE(("     the child process %d is stored\n",pid))
		
		}  

	} /* fin du else */
#else
  /*** create a fork ***/
  pid=fork();
  MACRO_TRACE(("     in the processus %d : fork = %d\n",getpid(),pid));

  if (pid == 0)			/*** child processus ***/
  {
     MENU_MNGT_PROCESS_change_env();
     switch(flag_xterm)
     {
      case 3: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
	      if (mode_verbose==ON)
		printf("3 ---> The child process\n%s -e %s %s %s\nis running...\n\n",path_xterm,path_prg); 
	      
	      if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,NULL,envg) == -1) 
		MENU_LIB_erreur();
	      break;
      case 2: if (mode_verbose==ON)
                  printf("2 ---> The child process\n%s\nis running...\n\n",path_prg); 
              if (execle(path_prg,path_prg,NULL,envg)==-1)
		MENU_LIB_erreur();
              break;
      case 1: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
	      sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
/*
	      sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
*/
	      sprintf(work_name, "%s",current_experiment);	      
	      if (mode_verbose==ON)
	          printf("1 ---> The child process\n%s -e %s %s %s\nis running...\n\n",path_xterm,path_prg,keep_ps,work_name); 
	      if (on_garde)
	      {
	         if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,keep_ps,work_name,NULL,envg)==-1) 
	         MENU_LIB_erreur();
	      } 
	      else
	      {
	         if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,work_name,NULL,envg)==-1) 
	         MENU_LIB_erreur();
	      } 
	      break;
	
      case 4: sprintf(path_xterm,"%s",xnbc_bin_ref_xterm);
	      if (mode_verbose==ON)
	          printf("4 ---> The child process\n%s -e %s\nis running...\n\n",path_xterm,path_prg); 
              if (execle(path_xterm,path_xterm,"-T","XNBC 8","-e",path_prg,NULL,envg)==-1) 
	         MENU_LIB_erreur();
	      break;
	
      case 0:
	      sprintf(keep_ps,   "%s",(on_garde)?"-k":"");
	      sprintf(work_name, "%s%s/%s",current_path,current_experiment,generic_name);
	      if (mode_verbose==ON)
	         printf("0 ---> The child process\n%s %s %s\nis running...\n\n",path_prg,keep_ps,work_name);
              if (on_garde)
	      {
		 if (execle(path_prg,path_prg,keep_ps,work_name,NULL,envg)==-1) 
			MENU_LIB_erreur();
		
	      } 
	      else
	      {
		 if (execle(path_prg,path_prg,work_name,NULL,envg)==-1) 
			MENU_LIB_erreur();
	      }
	      break; 
      }     
  }
  else 				/*** parent processus ***/
  {				/*** stored the pid of the child processus ***/
     tool[indice_tool].pid=pid;
     MACRO_TRACE(("     the child process %d is stored (%d)\n",pid,flag_xterm))
  }  
#endif
  
     
  MACRO_TRACE(("---> end    : MENU_MNGT_PROCESS_spawn\n"))
}


/*****************************************************************************
*                              HEADER FUNCTION                              
******************************************************************************
* NAME	       : MENU_MNGT_PROCESS_change_env	
* GOAL         : change the environment variable PWD.
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* SCOPE        : local of the module. 	
* INPUT        : ref	: the path and the name of the tool
*                argf   : the name of the work file if it is necessary
*                arg1   : the path and the name of the tool if it is in a xterm
*                         process that the tool is running.
*                indice_tool : the index of the tool in the array where the pid
*                              of the process is stored.	
* OUTPUT       : none	
* RETURN       : none	
*****************************************************************************/
static void MENU_MNGT_PROCESS_change_env()
{
    char *buffer;

    MACRO_TRACE(("---> start  : MENU_MNGT_PROCESS_change_env\n"))
    MENU_LIB_alloc(TYPE_char*255,&buffer);
    sprintf(buffer,"PWD=%s",current_path);
    *(buffer+strlen(buffer)-1)='\0';
    putenv(buffer);  
    MENU_LIB_free(buffer);
    MACRO_TRACE(("---> end    : MENU_MNGT_PROCESS_change_env\n"))
}


/*****************************************************************************
*                       END OF THE MODULE                               
*****************************************************************************/
