/*****************************************************************************
*                              HEADER MODULE                              
******************************************************************************
* APPLICATION  : MENU	                                             
* NAME         : MENU_constante.h	                                             
* GOAL         : define the constantes used by the application.                                
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* COPYRIGHT    : Copyright Dr J-F VIBERT
*		 Laboratoire de Biomathematiques
*		 URBB B3E INSERM U263
*		 Faculte de Medecine Saint-Antoine
*		 27 rue Chaligny
*		 75571 Paris cedex 12
*	         FRANCE
*	         
*                e-mail vibert@b3e.jussieu.fr
*
*                All rights reserved
*	         Access limited to authorized persons	
*	         This software was developped only for resarch purpose
*	         and is not, in any way, usable for commercial purpose.
*****************************************************************************/

				/*** used to allow for each module .c of the
				application to include this module .h without
				redefinition or not declaration of the 
				constantes defined in this current module 
				when the compilator and editor work ***/
				
#ifndef MENU_constante		/*** ifndef 01 ***/	
#define MENU_constante


/*****************************************************************************
*                       INCLUDE LIBRARY                                     
*****************************************************************************/

				/*** no include library for this module the
				stdio.h library is achieved by default ***/
				

/*****************************************************************************
*                       INCLUDE SOURCE MODULE                               
*****************************************************************************/

				/*** in this module, we have only the 
				definitions of the constantes which are 
				differently valuated according to the 
				specifications given during the compilation
				phase. See this module, to have a total
				overview of the constantes used in the
				whole application ***/
#include "MENU_variability.h" 

/*****************************************************************************
*                       DEFINE SECTION                               
*****************************************************************************/

/***************IDENTIFICATION OF THE APPLICATION****************************/

#ifdef HAVE_CONFIG_H
#	include <config_xnbc.h>
#endif

#define TITLE_APPLICATION       "XNBC 9"
				
#define VERSION_APPLICATION	"9.8"

#ifndef VERSION
#	define VERSION VERSION_APPLICATION
#endif

/***************WIDGETS ID OF THE APPLICATION****************************/
				
				/*** the application nbc must work with 
				particular widgets to get and to change
				the values of their attributes.
				So, the application nbc must stored
				the identification of the widgets. 
				The identification of theses widgets is
				stored in an array of integers. The size of 
				this array, so the number of widgets that
				the application nbc must stored, is given by
				this following variable ***/
				
#define NUMBER_WIDGETS_NAMED 			62


				/*** 				  So when
				a widget is created by the window manager,
				we must get the identification. 
				So, after each creation, we can run a
				callback and pass to this callback a
				mnemonic code.
				Indeed, when a callback is called, the
				identification of the widget is automatically
				passed to the callback. Moreover, we can pass
				other parameters into the callback. 
				So, we pass a code : the indice of the array
				where the identification of the widgets are
				stored. In order to improve the readability
				of the implementation, the indices of this
				array have a mnemonic representation thanks
				to the following list of constantes.
				Theses constantes are also used by the
				creation.c module and the callback.c module.
				Moreover, in order to improve the understand-
				ability theses constantes give informations
				about the widget that they referenced.
				
				Theses indices are used as indices in the
				array widget_array_di 			***/
				

#define WIDGET_LABEL_HELPONLINE 		 0

#define WIDGET_FRAME_PHENOMENOLOGICAL 		 1
#define WIDGET_BULLETIN_PHENOMENOLOGICAL 	 2
#define WIDGET_LABEL1_PHENOMENOLOGICAL		 3
#define WIDGET_LABEL2_PHENOMENOLOGICAL		 4

#define WIDGET_FRAME_CONDUCTANCE 		 5
#define WIDGET_BULLETIN_CONDUCTANCE 		 6
#define WIDGET_LABEL1_CONDUCTANCE		 7
#define WIDGET_LABEL2_CONDUCTANCE		 8

#define WIDGET_FRAME_VIRTUAL		 	 9
#define WIDGET_BULLETIN_VIRTUAL		 	10
#define WIDGET_LABEL1_VIRTUAL			11
#define WIDGET_LABEL2_VIRTUAL			12
	 
#define WIDGET_FRAME_NETWORK		 	13 
#define WIDGET_BULLETIN_NETWORK		 	14
#define WIDGET_LABEL_NETWORK			15

#define WIDGET_FRAME_SIMULATOR			16
#define WIDGET_BULLETIN_SIMULATOR	 	17 
#define WIDGET_LABEL_SIMULATOR			18

#define WIDGET_FRAME_EXPERT			19
#define WIDGET_BULLETIN_EXPERT			20
#define WIDGET_LABEL_EXPERT			21

#define WIDGET_BULLETIN_RESULT			22
#define WIDGET_LABEL_RESULT			23

#define WIDGET_FRAME_VISUNBC			24
#define WIDGET_BULLETIN_VISUNBC			25 
#define WIDGET_LABEL_VISUNBC			26

#define WIDGET_FRAME_XTMS			27
#define WIDGET_BULLETIN_XTMS			28
#define WIDGET_LABEL_XTMS			29

#define WIDGET_FRAME_XCAA			30
#define WIDGET_BULLETIN_XCAA			31
#define WIDGET_LABEL_XCAA			32

#define WIDGET_CASCADEBUTTON_FILE		33
#define WIDGET_PUSHBUTTON_CURRENT		34
#define WIDGET_PUSHBUTTON_NEW			35
#define WIDGET_PUSHBUTTON_OPEN			36
#define WIDGET_PUSHBUTTON_QUIT			38

#define WIDGET_CASCADEBUTTON_GRAPHIC		39
#define WIDGET_PUSHBUTTON_GRAPH			40
#define WIDGET_PUSHBUTTON_DOT			41
#define WIDGET_PUSHBUTTON_PMB			42
#define WIDGET_PUSHBUTTON_SPK			43
#define WIDGET_PUSHBUTTON_3D			44
#define WIDGET_PUSHBUTTON_PS			45

#define WIDGET_CASCADEBUTTON_HELP		46
#define WIDGET_PUSHBUTTON_HELPCONTEXT		47
#define WIDGET_PUSHBUTTON_HELPWINDOW		48

#define WIDGET_MAIN				50

#define WIDGET_BUTTON_KEEP			51
#define WIDGET_BUTTON_NBC_P			52
#define WIDGET_BUTTON_NBC_G			53
#define WIDGET_BUTTON_NBC_PG			54

#define WIDGET_PUSHBUTTON_HELPONPANEL		55
#define WIDGET_BUTTON_BROWSE			56

#define WIDGET_PUSHBUTTON_XGRAPH		57
#define WIDGET_PUSHBUTTON_GNUPLOT		58
#define WIDGET_PUSHBUTTON_RASTER		59

#define WIDGET_CASCADEBUTTON_OPTIONS		60
#define WIDGET_PUSHBUTTON_PRINTER		61

/***********************default values of the environment variables**********/
/*
#define XNBC_BIN_DIR_TOOLS_DEFAULT	"/usr/local/bin/xnbc.bin/V9/"
*/

#define XNBC_BIN_DIR_TOOLS_DEFAULT	"."


#ifdef __osf__
#  define XNBC_BIN_REF_XTERM_DEFAULT	"/usr/bin/X11/dxterm"
#else
#  ifdef __linux__
#     define XNBC_BIN_REF_XTERM_DEFAULT	"/usr/bin/konsole" 
#  else
#     define XNBC_BIN_REF_XTERM_DEFAULT	"/usr/bin/X11/xterm"
#  endif
#endif

#define XNBC_LOCAL_BIN_DIR              "/usr/local/bin/"
/*************************** name of the tools ******************************/
/*
#define TOOL_P_NEURON_EDITOR		"G_neuronP"
*/
#define TOOL_P_NEURON_EDITOR		"P_neuron"
#define TOOL_G_NEURON_EDITOR		"G_neuron"
#define TOOL_V_NEURON_EDITOR		"none"
#define TOOL_NETWORK_EDITOR		"none"
#define TOOL_X_SIMULATOR		"nbc_x"
#define TOOL_S_LINK_EDIT		"link_edit"
#define TOOL_F_NETW_EDIT		"network_editor"
#define TOOL_P_G_SIMULATOR		"none"
#define TOOL_EXPERT_SYSTEM		"EXPERT_main"
#define TOOL_DRUG_EDITOR		"drug_editor"
#define TOOL_VISUALIZATION		"visu_nbc"
#define TOOL_XTMS			"xtms"
#define TOOL_XCAA			"xcaa"
#define TOOL_GRAPH			"graph_n_b_c"
#define TOOL_XGRAPH			"xgraph_n_b_c"
#define TOOL_GNUPLOT			"gnuplot"
#define TOOL_RASTER			"raster_viewer"
#define TOOL_DOT			"dot_displ_n_b_c"
#define TOOL_PMB			"pmb_cell_n_b_c"
#define TOOL_SPK			"spk_cell_n_b_c"
#define TOOL_3D				"pmb_3d_n_b_c"
#define TOOL_PSVIEWER			"none"
#define FILE_HELP_CONTEXT		"nbc_help.txt"


/******************************************************************************/

 				/*** the number of tools that the nbc 
				application propose. Note, that theses tools
				are ofen presented by a button with its
				representation, or are available in the
				menu Graphics in the menu-bar option  ***/
				
#define NB_TOOLS			21

#define INDICE_P_NEURON_EDITOR		0
#define INDICE_G_NEURON_EDITOR		1
#define INDICE_V_NEURON_EDITOR		2
#define INDICE_NETWORK_EDITOR		3
#define INDICE_X_SIMULATOR		4
#define INDICE_S_LINK_EDIT		5
#define INDICE_F_NETW_EDIT		6
#define INDICE_P_G_SIMULATOR		7
#define INDICE_EXPERT_SYSTEM		8
#define INDICE_VISUALIZATION		9
#define INDICE_XTMS			10
#define INDICE_XCAA			11
#define INDICE_GRAPH			12
#define INDICE_DOT			13
#define INDICE_PMB			14
#define INDICE_SPK			15
#define INDICE_3D			16
#define INDICE_PSVIEWER			17
#define INDICE_XGRAPH			18
#define INDICE_GNUPLOT			19
#define INDICE_RASTER			20


/******************************************************************************/

#define XTERM_NEEDED	0
#define AUTONOME	1


/******************************************************************************/

#define NBC_BINDIR	0
#define OTHER_PATH	1
#define NONE_NBC	2


/******************************************************************************/

#define TYPE_char sizeof(char)		
#define TYPE_FILE sizeof(int)	
#define TYPE_int   sizeof(float)
#define TYPE_float sizeof(FILE)	


/******************************************************************************/

#define ON 1
#define OFF 0


#endif				/***endif 01 ***/


/*****************************************************************************
*                       END OF THE MODULE                               
*****************************************************************************/

