/*****************************************************************************
*                              CONTROL PANEL                              
******************************************************************************
* APPLICATION  : XNBC Control panel	                                             
* NAME         : cp_callback.c	                                             
* GOAL         : This module propose to the user the different tools used by
*                the nbc application. This proposition is presented into the
*                screen thanks to a visual representation via the X motif   
*                interface. The goal of this module is also to take into    
*                account the choices of the user, so to run and manage the  
*                tools chosen by the user.                                  
******************************************************************************
* AUTHOR       : Jean-Francois VIBERT and Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : JF Vib******************************************************************************
* APPLICATION  : MENU	                                             
* NAME         : MENU_MAIN.c	                                             
* GOAL         : This module propose to the user the different tools used by
*                the nbc application. This proposition is presented into the
*                screen thanks to a visual representation via the X motif   
* ******************************************************************************
* APPLICATION  : XNBC V9 Control Panel	                                             
* NAME         : cp_main.c	                                             
* GOAL         : This module propose to the user the different tools used by
*                the nbc application. This proposition is presented into the
*                screen thanks to a visual representation via the X motif   
* ert	
* MODIFICATION : adapted to find_files on dec 1997	
*              : V8.24  (gnuplot menu) by JFV 24 mar 1999
*              : V8.25  (Lesstif compatibility + menu raster) 
*                                                       by JFV 16 juillet 1999
*	       : V8.26  (Option menu for editors+gnu configure distribution) 
*							by JFV 14 sept 1999
*	       : V8.27  (Important modifications in nbc_x and the control
*			panel allows to run several tools simutaneously)
*							by JFV 29 nov 1999
*              : V8.27-1 Virtual menu disappeared under Linux. Fixed
*                                                       by JFV 3 dec 1999
*
*              : V8.27-2 Keep button put in option menu + bug correction 
*                        in printer definition          by JFV 4 dec 1999
*              : V8.27-3 Virtual button Linux revealed bug correction 
*							by JFV 25 july 2000
*              : V8.27-4 All printf set to verbose mode ti fit with Cygwin
*                                                       by JFV 14 august 2001
*              : V8-27-5 If .xnbcrc not existing and not creatable, still go on 
*                                                       by JFV 16 august 2001 
*              : V9.0 beta Version 9
*                                                       by JFV 30 august 2001
*              : Control Panel V9 with glade            by JFV 5 sept 2001
******************************************************************************
* COPYRIGHT    : Copyright Dr J-F VIBERT
*		 Laboratoire de Biomathematiques
*		 ex URBB B3E INSERM U263
*                ESI B3E INSERM U444
*		 Faculte de Medecine Saint-Antoine
*		 27 rue Chaligny
*		 75571 Paris cedex 12
*	         FRANCE
*	         
*                e-mail vibert@u444.jussieu.fr
*
*                All rights reserved
*                Distributed under the GPL
*	         This software was developped only for resarch purpose.
*****************************************************************************/

/*
 * main.c initial gnr par Glade. diter ce fichier  votre
 * convenance. Glade n'crira plus dans ce fichier.
 */

#define True 1
#define False 0

#ifdef HAVE_CONFIG_H
#  include <config_xnbc.h>
#endif

#define PACKAGE_DATA_DIR "."
#define PACKAGE_SOURCE_DIR "."

#include <signal.h>		/*** to use signal routines ***/
#include <stdio.h> 		/***  stdandard   ***/
#include <sys/stat.h> 		/*** for struct stat_sbuf    ***/
#include <stdlib.h> 		/***  for exit   ***/
#include <sys/wait.h> 		/***  for wait   ***/
#include <unistd.h> 		/***  for acess   ***/
#include <string.h>
#include <gtk/gtk.h>

#include "cp_interface.h"
#include "support.h"
/*****************************************************************************
*                       INCLUDE SOURCE MODULE                               
*****************************************************************************/

#include "MENU_constante.h"	/*** ctes definition of the application ***/
#include "find_files.h"         /*** header pour .xnbcrc                ***/

extern void 	MENU_CB_INIT_store_widget_id();   /***  callback procedure ***/
extern void	MENU_MNGT_SENSITIVE_put_sensitive(); /***  callback procedure ***/
extern void     MENU_MNGT_SENSITIVE_maj_button();
void     MENU_LIB_init_alloc();
int      MENU_LIB_alloc(int, char **);
void     MENU_LIB_free(char *);
void     MENU_LIB_error();
void     MENU_CB_EXPERIMENT_menu_quit();
void     MENU_MAIN_title_application();


char *xnbc_bin_dir;			
char *xnbc_bin_ref_xterm;
char *xnbc_bin_ref_psviewer;
char *current_path;
char *current_experiment;

char *home_pathg;
char **envg;
char *strtext;

int errno;	
int on_garde;
int current_net_editor;
int flag_fsb;
int flag_xterm;
int indice_tool;
int mode_verbose;
int code_help_context;
int nb_max_recent;

char recent_exp[20][256];
char file_name_cprc[256];
char cmd_lpr[256];
char xnbcrc_dir[256];

struct stool {
  char name[80];
  int  emulator;
  int available;
  int pid;
  int path;
} tool[NB_TOOLS]={
    {TOOL_P_NEURON_EDITOR,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_G_NEURON_EDITOR,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_V_NEURON_EDITOR,	NONE_NBC,	-1,	0,	NONE_NBC  },
    {TOOL_NETWORK_EDITOR,	NONE_NBC,	-1,	0,	NONE_NBC  },
    {TOOL_X_SIMULATOR,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_S_LINK_EDIT,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_F_NETW_EDIT,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_P_G_SIMULATOR,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_DRUG_EDITOR,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_VISUALIZATION,	AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_XTMS,		AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_XCAA,		AUTONOME,	-1,	0,	NBC_BINDIR},
    {TOOL_GRAPH,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_DOT,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_PMB,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_SPK,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_3D,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_PSVIEWER,		AUTONOME,	-1,	0,	OTHER_PATH},
    {TOOL_XGRAPH,		XTERM_NEEDED,	-1,	0,	NBC_BINDIR},
    {TOOL_GNUPLOT,		XTERM_NEEDED,	-1,	0,	OTHER_PATH},
    {TOOL_RASTER,		AUTONOME,	-1,	0,	NBC_BINDIR}
};

static void 	MENU_MAIN_signal_usr();	     /*** function signal SIGCHLD ***/
static int      MENU_MAIN_recup_variable_environment_tools();
static int      MENU_MAIN_recup_variable_environment_terminal_emulator();
static int      MENU_MAIN_recup_variable_environment_psviewer();
static int 	MENU_MAIN_verif_item(char *item);



GtkWidget *main_window;
GtkWidget *about_window;
GtkWidget *fileselection_exp;
GtkWidget *dialog_current;
GtkWidget *dialog_help_current;
GtkWidget *dialog_previous;
GtkWidget *dialog_printer;
GtkWidget *fileselection_new;
GtkWidget *dialog_message;
GtkWidget *dialog_virtual;

void show_message(char *);

int
main (int argc, char *argv[], char *env[])
{

  GtkWidget *keep_postscript_files1;
  GtkWidget *verbose_mode1;
  GtkWidget *spinbutton1;

  FILE *fpt;
  FILE *fpt_cprc;
  char *home_path;
  char *item;
  int  i;
  char field[100];
  char value[256];
  char printer[100];
  char file_name[256];
  char buffer[256];
  char first_time = 0;

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif
  /********************************************************** variable ***/
  
  
  /************************************************ trace for debug ***/
  MACRO_TRACE(("--> start : main\n"))
  
  /************************************************ memory alloc ***/
  MENU_LIB_init_alloc();
  
  
  /*********** recup the verbos mode ****/
  mode_verbose=OFF;
  if (argc>=2)
    if ((strcmp(argv[1],"-v")==0) || (strcmp(argv[1],"-V")==0))
      mode_verbose=ON;
  
  /************************************************ recup the var environment ***/
  MENU_MAIN_recup_variable_environment_tools();
  MENU_MAIN_recup_variable_environment_terminal_emulator();
  MENU_MAIN_recup_variable_environment_psviewer();
  on_garde = ON;
  nb_max_recent = 5;

  /*** recup the home_pathg ***/
  home_path=(char *)getenv("HOME"); 
  sprintf(home_pathg,"%s/",home_path);
  envg=env;
 
  strcpy(current_experiment,get_current_experiment());
  printf("Current experiment : %s\n",current_experiment);

  /********* recup the item in the .xnbcrc and .xnbc_cprc files  *****/
#ifdef __CYGWIN__
  sprintf(file_name,"%s\\xnbcrc", current_experiment);
  sprintf(file_name_cprc,"C:\\xnbc_cprc");
#else
  sprintf(file_name,"%s.xnbcrc", current_experiment);
  sprintf(file_name_cprc,"%s.xnbc_cprc", home_pathg);
#endif
  MENU_LIB_alloc(TYPE_FILE,&fpt);
  fpt = fopen(file_name_cprc,"r");
  if (fpt == NULL) 
  {
     if (mode_verbose==ON)
     {
        printf("\n\nXNBC APPLICATION REPORT :\n");
        printf("\nThe configuration file of the XNBC application is not available !\n");
        printf("This file (.xnbc_cprc) must be stored in your experiment directory.\n");
        printf("It will be created now.\n");
     }

     if ((fpt = fopen(file_name_cprc,"w")) == NULL)
     {
	printf("\n\nImpossible to write (%s) in your home directory. Please correct this (necessary to memorize your XNBC parameters)... and try again.\n",file_name_cprc);
	exit(1);
     } 

     first_time = 1;
     /*
       Le premier run chez un nouvel utilisateur
     
     */
  }

  fpt_cprc = fopen(file_name_cprc,"r");
  if (fpt_cprc == NULL) 
  {
     if (mode_verbose == ON)
     {
        printf("\n\nXNBC APPLICATION REPORT :\n");
        printf("\nThe memory file of the XNBC control panel is not available !\n");
        printf("This file (.xnbc_cprc) is stored in your home directory.\n");
        printf("It will be created now.\n");
     }

     if ((fpt_cprc = fopen(file_name_cprc,"w")) == NULL)
     {
	printf("\n\nImpossible to create a ~/.xnbc_cprc file (%s)\nOnly your options and old files will not be memorized\nYou can nevertheless continue.", file_name_cprc);
     } /* fin du if */
  }
  else
  {
    i = 0;
     while(fgets(buffer,255,fpt_cprc))
       {
	 sscanf(buffer,"%s %s %s",field,value,printer);
		
	 if(!strcmp("cmd_lpr:",field))
	   {
	     sscanf(value,"%s %s",cmd_lpr,printer);
	     strcat(cmd_lpr," ");
	     strcat(cmd_lpr,printer);
	   }
	 if(!strcmp("keep_PS:",field))
	   {
	     sscanf(value,"%d",&on_garde);
	   }
	 if(!strcmp("mode_verbose:",field))
	   {
	     sscanf(value,"%d",&mode_verbose);
	   }
	 if(!strcmp("nb_max_recent:",field))
	   {
	     sscanf(value,"%d",&nb_max_recent);
	   }
	 
	 /* Now the experiments recently opened */
	 if (i < nb_max_recent)
	   {
	     if(!strcmp("experiment:",field))
	       {
		 sscanf(value,"%s",recent_exp[i]);
		 i++;
	       }
	   }
       }
     fclose(fpt_cprc);
  }
  strcpy(current_path, get_dir_name());
  strcpy(file_name, get_project_name());
  item = strrchr(file_name,'/');

  printf("Current path : %s\n",current_path);
  printf("Project name : %s\n",file_name);

#if 0
  if (!item)
  {
     if (mode_verbose==ON)
     {
        printf("\n\nXNBC APPLICATION REPORT :\n");
        printf("\nThe configuration file of the XNBC application is not available !\n");
        printf("This file (.xnbc_cprc) must be stored in your home directory.\n");
        printf("So, your default experiment will be nammed by 'no_experiment_xnbc'.\n");
        printf("And, a directory 'no_experiment_xnbc' will be created on your home directory.\n");
     }
     MENU_LIB_alloc(TYPE_char*512,&item);
     
     strcpy(item,"WARNING : No experiment name!!");

  }
  else
  {
    item++;
    strcat(item, "_xnbc");
  }
  strcpy(current_experiment, item);
#endif

  /********** verif si le texte est correcte ***/ 
  /*  MENU_MAIN_verif_item(item);*/

  
  /*** trace for verbos mode ***/
  if (mode_verbose==ON)
  {
     printf("XNBC current experiment= %s\n",current_experiment); 
     printf("XNBC current path= %s\n",current_path); 
  }

  

  /*** install the SIGCHLD signal ***/
  /*** the end of running of a child processus give a signal
  SIGCHLD into the parent processus. By default, this signal do not
  operate the end of this parent processus as the others signals.
  According to the behavior of the parent processus when the signal
  SIGCHLD arrives, a zombie processus can be create, and the wait primitive
  can have a different action.
  Indeed, if the SIGCHLD signal is ignored by the parent processus, the end
  of a child processus if the parent processus is not in the "wait" mode do not
  give the creation of a zombie processus.
  Nevertheless, the SIGCHLD signal can be receipt, the child processus which ends
  change also of its state : it becomes a zombie processus whit the same pid.
  Then, with a wait primitive we can get the zombie processus (stored in the
  processus table managed by the kernel) and of course get its pid. ***/
  
  MACRO_TRACE(("install the SIGCHLD signal\n"))
  signal(SIGCHLD,MENU_MAIN_signal_usr);


  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /*
  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");
  */

  add_pixmap_directory (PACKAGE_DATA_DIR "/");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  main_window = create_main_window ();
  about_window = create_about_window();
  fileselection_exp = create_fileselection_exp();
  dialog_current = create_dialog_current();
  dialog_help_current = create_dialog_help_current();
  dialog_previous = create_dialog_previous();
  dialog_printer = create_dialog_printer();
  fileselection_new = create_fileselection_new();
  dialog_message = create_dialog_message();
  dialog_virtual = create_dialog_virtual();

  keep_postscript_files1 = lookup_widget(GTK_WIDGET(main_window),"keep_postscript_files1");
  verbose_mode1 = lookup_widget(GTK_WIDGET(main_window),"verbose_mode1");
  spinbutton1 = lookup_widget(GTK_WIDGET(dialog_previous),"spinbutton1");
  gtk_spin_button_set_value(spinbutton1,(gfloat)nb_max_recent);

  MENU_MAIN_title_application();

  if (on_garde)
    {
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(keep_postscript_files1),TRUE);
    }
  else
    {
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(keep_postscript_files1),FALSE);
    }

  if (mode_verbose)
    {
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(verbose_mode1),TRUE);
    }
  else
    {
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(verbose_mode1),FALSE);
    }

  gtk_widget_show (main_window);

  if (first_time)
  {
    gtk_widget_show (fileselection_new);
    show_message("This is your first use of XNBC.\nYou must provide an experiment name to be able to begin."); 
    first_time = 0;
  }

  gtk_main ();

  return 0;
}


/*****************************************************************************
*                              HEADER FUNCTION                              
******************************************************************************
* NAME	       : signal_usr	
* GOAL         : this routine is running when a child of the parent has been
*                terminated. The goal of this routine is to take into account
*                this information and to tell to the user what are the
*                modifications. In practise, the buttons corresponding to a tool
*                are either accessible, either not, according to the rules 
*                management of the whole set of the tools.	
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* SCOPE        : local into this module	
* INPUT        : none	
* OUTPUT       : none	
* RETURN       : none	
*****************************************************************************/

static void MENU_MAIN_signal_usr()
{
int 	pid_child_processus,
	end_statut_child_processus,
	loop_indice;

  /*** trace for debug ***/
  MACRO_TRACE(("--> start : MENU_MAIN_signal_usr\n"))
  MACRO_TRACE(("the parent is awake by the dead of one of its childs\n"))
  
  /*** get the pid of the last process which has already terminated 
  In fact when a process terminates, either normally or abnormally, the parent
  is notified by the kernel sending the parent SIGCHLD signal. Since the termination
  of a child is an asynchronous event, it can happen at any time while the parent
  is running, this signal is the asynchronous notification from the kernel to the
  parent. The parent can choose to ignore this signal, or it can provide a function
  that is call when the signal occurs : a signal handler. In the application MENU_main
  the parent provide the signal_usr function when the SIGCHLD signal is sending.
  So this fucntion which calls wait can block if all of its children are running, or
  it can return immediately with the termination status of a child (if a child has
  terminated and is waiting for its termination status to be fetched), or it can
  return immediately with an error (if it doesn't have any child processes).
  
  Here, the parent process is calling wait because it received the SIGCHLD signal.
  so we espect wait to return immediately.
  
  Thanks to this routine, we can always tell which child terminated because the 
  process ID is returned by the function. This ID is stored in the  pid_child_processus
  local variable.
  
  The termination status of the terminated process is stored in the location pointed to by
  the argument passed into the wait routine. Here, we don't care about the termination
  status, so we can jsut pass a null pointer as this assignment. Nevertheless, for debug
  reason and test status we choice to get the termination status. So, it is at the  
  end_statut_child_processus adress that this data is stored. Thanks to it, we can display
  the signal number when the child has terminated      ***/
  
  pid_child_processus=wait(&end_statut_child_processus);
  
  /*** trace to display information about the last child which has terminated ***/
  MACRO_TRACE(("the process %d is ended\n",pid_child_processus))
  MACRO_TRACE(("the termination status of the terminated process is %d\n",
               end_statut_child_processus))
               
  /*** now, we can take into account that a child has terminated so, first we identified
  the tool corresponding of the child pid which has terminated, then we notified in the
  array MENU_MAIN_tool_is_used that this tool is now not runnning ***/
  for (loop_indice=0;loop_indice<NB_TOOLS;loop_indice++)
 
  if (tool[loop_indice].pid==pid_child_processus)
      tool[loop_indice].pid=0;


  /*** then, we operate a feedback to the screen of the user.  So it is achieved
  thanks to the management rules about the availabality of the tools defined in 
  the put_sensitive routine  ***/ 


  MACRO_TRACE(("install the SIGCHLD signal\n"))
  signal(SIGCHLD,MENU_MAIN_signal_usr);
  
  /*** trace for debug ***/ 
  MACRO_TRACE(("--> end   : MENU_MAIN_signal_usr\n"))

}


/**********************************************************/
static int MENU_MAIN_recup_variable_environment_psviewer()
{
char *var;
char *xnbc_var;
struct stat s_buf;
int code=1;

MACRO_TRACE(("--> start : recup_variable_environment_psviewer\n"))

/** recup the var ***/
var=(char *)getenv("XNBC_BIN_REF_PSVIEWER");
MENU_LIB_alloc(TYPE_char*255,&xnbc_var);
if (var==NULL) strcpy(xnbc_var,"");
else strcpy(xnbc_var,var);


if (strcmp(xnbc_var,"")==0)
   {  
   if (mode_verbose==ON)     
   printf("\nThe environment variable XNBC_BIN_REF_PSVIEWER is not initialized in your environment.\n");
   code=(-1);
   }
if (code!=-1)
   {
   /*** tester que c'est bien un exe, accessible ***/
   s_buf.st_mode=0;
   stat(xnbc_var,&s_buf);
   if ((s_buf.st_mode & S_IFMT) != S_IFREG) 
       {  
       if (mode_verbose==ON)      
          {
          printf("\nThe environment variable XNBC_BIN_REF_PSVIEWER exists in your environment.\n");
          printf("The value of this variable is '%s'.\n",xnbc_var);
          printf("Nevertheless, it does not pointed to an accessible runnable file.\n");
          }
       code=(-1);
       }
    }
if (code==-1) 
   {   
   if (mode_verbose==ON)
      {
      printf("Then, in the menu 'Graphics' , the 'View PostScript file' fucntionnality\nwill be not available.\n");
      printf("Please check...and run again xnbc if you want all functionnalities...\n");
      }
   strcpy(xnbc_bin_ref_psviewer,"");
   tool[INDICE_PSVIEWER].available=0;
   }
else
   {
   strcpy(xnbc_bin_ref_psviewer,xnbc_var);
   tool[INDICE_PSVIEWER].available=1;
   MACRO_TRACE(("the xnbc_bin_ref_psviewer is %s.\n",xnbc_bin_ref_psviewer))
   }
MENU_LIB_free(xnbc_var);
MACRO_TRACE(("--> end : recup_variable_environment_psviewer\n"))

return(code);
}   
   
   
/**********************************************************/
static int MENU_MAIN_recup_variable_environment_terminal_emulator()
{
char *var;
char *xnbc_var;
int fact=0;
struct stat s_buf;
int code=1;
int i;

MACRO_TRACE(("--> start : recup_variable_environment_terminal_emulator\n"))

/** recup the var ***/
var=(char *)getenv("XNBC_BIN_REF_XTERM");
MENU_LIB_alloc(TYPE_char*255,&xnbc_var);

if (var==NULL) {
               strcpy(xnbc_var,XNBC_BIN_REF_XTERM_DEFAULT);
               fact=1;
               }
else           {
               strcpy(xnbc_var,var);
               fact=2;
               }
/*** tester que c'est bien un exe, accessible ***/
s_buf.st_mode=0;
stat(xnbc_var,&s_buf);
if ((s_buf.st_mode & S_IFMT) != S_IFREG) 
   {
   if (mode_verbose == ON)
	printf("\nMessages from XNBC below:\n");
   if (fact==2) 
       {
       if (mode_verbose==ON)
          {
          printf("\nThe environment variable XNBC_BIN_REF_XTERM exists in your environment.\n");
          printf("So, it is it which is used and not the default value.\n");
          printf("The value of this variable is '%s'.\n",xnbc_var);
          printf("Nevertheless, it does not pointed to an accessible runnable file.\n");
          }
       code=(-1);
       }
   if (fact==1)
       { 
       if (mode_verbose==ON)
          {      
          printf("\nThe environment variable XNBC_BIN_REF_XTERM does not exist in your environment.\n");
          printf("So, it is the default value '%s' which is used.\n",XNBC_BIN_REF_XTERM_DEFAULT);
          printf("Nevertheless, it does not pointed to an accessible runnable file.\n");
          }
       code=(-1);
       }
    }
if (code==-1) 
   {
   if (mode_verbose==ON)
   printf("Then, all tools which requires a terminal emulator will be not available...\n");
   for(i=0;i<NB_TOOLS;i++)
       if (tool[i].emulator==XTERM_NEEDED) 
          {
          if (mode_verbose==ON)
             printf("The '%s' will be not available.\n",tool[i].name);
          tool[i].available=0;
          }
   strcpy(xnbc_bin_ref_xterm,"");
   if (mode_verbose==ON)
   printf("Please check...and run again xnbc if you want all functionnalities...\n");
   }
else
   {
   strcpy(xnbc_bin_ref_xterm,xnbc_var);
   }

MENU_LIB_free(xnbc_var);
MACRO_TRACE(("--> end   : recup_variable_environment_terminal_emulator\n"))

return(code);
}


/**********************************************************/
static int MENU_MAIN_recup_variable_environment_tools()
{
    int code;
    int i;
    char *var;
    char *xnbc_var;
    char *tool_exe;
    struct stat s_buf;
    int fact=0;
    int etat;
    MACRO_TRACE(("--> start : recup_variable_environment_tools\n"))

    code=1;
    MENU_LIB_alloc(TYPE_char*255,&xnbc_var);

    /*** recup the var ***/
    var=(char *)getenv("XNBC_BIN_DIR_TOOLS");
    if (var==NULL) {
		   strcpy(xnbc_var,XNBC_BIN_DIR_TOOLS_DEFAULT);
		   fact=1;
		   }
    else           {
		   strcpy(xnbc_var,var);
		   fact=2;
		   }
    /**** tester que c'est bien un path ****/
    s_buf.st_mode=0;
    stat(xnbc_var,&s_buf);
    if ((s_buf.st_mode & S_IFMT) != S_IFDIR) 
       {
       printf("\nMessages from XNBC below:\n");
       if (fact==2) 
	   {
	   if (mode_verbose==ON)
	      {
	      printf("\nThe environment variable XNBC_BIN_DIR_TOOLS exist in your environment.\n");
	      printf("So, it is it which is used and not the default value.\n");
	      printf("The value of this variable is '%s'\n",xnbc_var);
	      printf("Nevertheless, it does not point to an accessible directory\n");
	      }
	   code=(-1);
	   }
       if (fact==1)
	   { 
	   if (mode_verbose==ON)
	      {      
	      printf("\nThe environment variable XNBC_BIN_DIR_TOOLS does not exist in your environment.\n");
	      printf("So, it is the default value '%s' which is used.\n",XNBC_BIN_DIR_TOOLS_DEFAULT);
	      printf("Nevertheless, it does not point to an accessible directory\n");
	      }
	   code=(-1);
	   }
	}
    if (code==-1) strcpy(xnbc_bin_dir,"");   
    else  strcpy(xnbc_bin_dir,xnbc_var);

    /**** tester que les outils exe existent dans ce repertoire ****/
    if (code!=-1)
    {
       MENU_LIB_alloc(TYPE_char*255,&tool_exe);
       etat=0;
       for (i=0;i<NB_TOOLS;i++)
       { 
	   if (tool[i].path==NBC_BINDIR)
	   {      
	      s_buf.st_mode=0;
	      sprintf(tool_exe,"%s%s",xnbc_var,tool[i].name);
	      stat(tool_exe,&s_buf);
	      if (((s_buf.st_mode & S_IFMT) != S_IFREG)   ||  (access(tool_exe,X_OK)==-1) )
	      {
		 if (etat==0) 
		 {
		    if (mode_verbose==ON)
		    printf("\nThe XNBC_BIN_DIR_TOOLS is '%s'.\n",xnbc_bin_dir);
		    etat=1;
		 }
		 if (mode_verbose==ON)
		 printf("The tool '%s' is not runnable in the XNBC_BIN_DIR_TOOLS directory.\n",tool[i].name);
		 tool[i].available=0;
		 code=(-1);
	      }
	      else 
	      {
		 if (tool[i].available==-1) tool[i].available=1;
	      }
	   }
       }
       MENU_LIB_free(tool_exe);   
    }
    else
    {
       for (i=0;i<NB_TOOLS;i++) if (tool[i].path==NBC_BINDIR) tool[i].available=0;
    }

    /*** current simulator ***/
    if (tool[INDICE_S_LINK_EDIT].available==1) current_net_editor=1;
    else if (tool[INDICE_F_NETW_EDIT].available==1) current_net_editor=2;
    else current_net_editor=0;

    /***********************************/
     MENU_LIB_alloc(TYPE_char*255,&tool_exe);
     s_buf.st_mode=0;
     sprintf(tool_exe,"%s%s",xnbc_var,FILE_HELP_CONTEXT);
     stat(tool_exe,&s_buf);
     if (((s_buf.st_mode & S_IFMT) != S_IFREG)   ||  (access(tool_exe,R_OK)==-1) )
	{
	if (mode_verbose==ON)
	printf("The file '%s' is not readable in the XNBC_BIN_DIR_TOOLS directory.\n",FILE_HELP_CONTEXT);
	code_help_context=(-1);
	code=(-1);
	}
     else
	code_help_context=1;
     MENU_LIB_free(tool_exe);   

     /* traitement de gnuplot */

     /*** recup the var ***/

     var=(char *)getenv("XNBC_BIN_GNUPLOT");
     if (var==NULL)
     {
	 strcpy(xnbc_var,XNBC_LOCAL_BIN_DIR);
     }
     else
     {
         strcpy(xnbc_var,var);
     }

     s_buf.st_mode=0;
     /*     sprintf(tool_exe,"%s/%s",xnbc_var,tool[INDICE_GNUPLOT].name); */
     sprintf(tool_exe,"%s",xnbc_var);
     stat(tool_exe,&s_buf);
     if (((s_buf.st_mode & S_IFMT) != S_IFREG)   ||  (access(tool_exe,X_OK)==-1) )
     {
       if (mode_verbose==ON)
	 printf("The tool %s is not found or not runnable\n",tool[INDICE_GNUPLOT].name);
       tool[INDICE_GNUPLOT].available=0;
       code=(-1);
     }
     else 
     {
       if (mode_verbose==ON)
	 /* printf("\nGnuplot is in %s/%s\n",xnbc_var,tool[INDICE_GNUPLOT].name); */
	 printf("\nGnuplot is in %s\n",xnbc_var);
       if (tool[INDICE_GNUPLOT].available==-1) 
	 tool[INDICE_GNUPLOT].available=1;
     }

     /* traitement de raster */

     /*** recup the var ***/

     var=(char *)getenv("XNBC_BIN_RASTER");
     if (var==NULL)
     {
	 strcpy(xnbc_var,XNBC_LOCAL_BIN_DIR);
     }
     else
     {
         strcpy(xnbc_var,var);
     }
     s_buf.st_mode=0;
     sprintf(tool_exe,"%s/%s",xnbc_var,tool[INDICE_RASTER].name);
     stat(tool_exe,&s_buf);
     if (((s_buf.st_mode & S_IFMT) != S_IFREG)   ||  (access(tool_exe,X_OK)==-1) )
     {
       if (mode_verbose==ON)
	 printf("The tool %s is not found or not runnable\n",tool[INDICE_RASTER].name);
       tool[INDICE_RASTER].available=0;
       code=(-1);
     }
     else 
     {
       if (mode_verbose==ON)
	 printf("\nRaster is in %s/%s\n",xnbc_var,tool[INDICE_RASTER].name);
       if (tool[INDICE_RASTER].available==-1) 
	 tool[INDICE_RASTER].available=1;
     }

    /**** quit ***/
    if (code==-1) 
    if (mode_verbose==ON) 
    printf("Please check...and run again xnbc if you want all functionalities...\n");

    MENU_LIB_free(xnbc_var);

    MACRO_TRACE(("--> end : recup_variable_environment_tools\n"))

    return(code);
}

#if 0
/**************************************************************************************************/
static int MENU_MAIN_verif_item(char *item)
{
int code;
int i;
char *lettre;
char *path;
char *experiment;
struct stat s_buf;
MACRO_TRACE(("--> start : MENU_MAIN_verif_item\n"))

MENU_LIB_alloc(TYPE_char*255,&experiment);
MENU_LIB_alloc(TYPE_char*255,&path);

  /********** verif si le texte est correcte ***/ 
  /*************************************/
   code=True;
 i=(int)strlen(item);
 if ((*(item+i-1))=='/') *(item+i-1)='\0';
 if (code)  { if (strcmp(item,"")==0) code=False; }
 if (code)  { if (strcmp(item,"/")==0) code=False; }
  
 if (code)  { /*** verif le .xnbc ***/
             /**** extract the current_path ***/
             i=1;
             lettre=item+strlen(item)-i;
             while((*lettre!='/')&&(i<=strlen(item))) 
                  {
                  i++;
                  lettre=item+strlen(item)-i;
                  }
             strncpy(path,item,strlen(item)-i+1);
             *(path+ strlen(item)-i+1)='\0';
             strcpy(experiment,item+strlen(item)-i+1);
             lettre=experiment;
                         
                
             i=strlen(lettre)-1;
             while( (*(lettre+i)!='_')&&(i>0) )
                  {
                  i--;
                  }
             lettre=lettre+i;
             if (strcmp(lettre,"_xnbc")!=0) {code=False;} 

             }   
 if (code)  {
               stat(item,&s_buf);
               if ((s_buf.st_mode & S_IFMT) != S_IFDIR) code=False; 
            } 
                            
if (!code)    
    {	    
    strcpy(current_experiment,"no_experiment_xnbc");
    strcpy(current_path,home_pathg);
    sprintf(item,"%s%s",current_path,current_experiment);
    mkdir(item,0755);
    }
 else
    {     
    strcpy(current_experiment,experiment);
    strcpy(current_path,path);
    } 
MENU_LIB_free(experiment);
MENU_LIB_free(path);  
MACRO_TRACE(("--> end   : MENU_MAIN_verif_item\n"))

}
#endif

/***************************************************************/
void MENU_MAIN_title_application()
{
  char title[256];
  
  sprintf(title,_("%s    V%s     -     Experiment : ~/%s"),TITLE_APPLICATION,VERSION,current_experiment);
  gtk_window_set_title (GTK_WINDOW (main_window), title);

}

/****************************************************************/
int MENU_LIB_alloc(int nb_elems,char **ad_pt)
{
int return_value;

MACRO_TRACE(("--> start : MENU_LIB_alloc\n"))

*(ad_pt)=(char *)malloc(nb_elems);

if ((*(ad_pt))==NULL)
   {
   printf("\n\nREPORT STATUT ERROR : 1\n");
   printf("The software requires memory.\n");
   printf("There is no available memory !\n");
   printf("The software cannot continue execution.\n");
   printf("Please, give more memory before rerun this software...\n");
   return_value=(-1);
   }
else
   {
   return_value=1;
   }
MACRO_TRACE(("--> end   : MENU_LIB_alloc\n"))
return(return_value);
}

/****************************************************************/
void MENU_LIB_free(char *pt)
{ 
MACRO_TRACE(("--> start : MENU_LIB_free\n"))
free(pt); 
MACRO_TRACE(("--> end   : MENU_LIB_free\n"))
}

/****************************************************************/
void MENU_LIB_init_alloc()
{
 MACRO_TRACE(("--> start : MENU_LIB_init_alloc\n"))
 MENU_LIB_alloc(TYPE_char*255,&xnbc_bin_dir);
 MENU_LIB_alloc(TYPE_char*255,&xnbc_bin_ref_xterm);
 MENU_LIB_alloc(TYPE_char*255,&xnbc_bin_ref_psviewer);
 MENU_LIB_alloc(TYPE_char*255,&current_path);
 MENU_LIB_alloc(TYPE_char*255,&current_experiment);
 MENU_LIB_alloc(TYPE_char*255,&home_pathg);
 MACRO_TRACE(("--> end   : MENU_LIB_init_alloc\n"))
}

void MENU_LIB_deinit_alloc()
{
MACRO_TRACE(("--> start : MENU_LIB_deinit_alloc\n"))
if (xnbc_bin_dir!=NULL) MENU_LIB_free(xnbc_bin_dir);
if (xnbc_bin_ref_xterm!=NULL) MENU_LIB_free(xnbc_bin_ref_xterm);
if (xnbc_bin_ref_psviewer!=NULL) MENU_LIB_free(xnbc_bin_ref_psviewer);
if (current_path!=NULL) MENU_LIB_free(current_path);
if (current_experiment!=NULL) MENU_LIB_free(current_experiment);
if (home_pathg!=NULL) MENU_LIB_free(home_pathg);
MACRO_TRACE(("--> end   : MENU_LIB_deinit_alloc\n"))
}

/*****************************************************************************
*                              HEADER FUNCTION                              
******************************************************************************
* NAME	       : MENU_CALLBACKS_erreur	
* GOAL         : display the system error messages when an error occurs.
******************************************************************************
* AUTHOR       : Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* SCOPE        : local into the module. 	
* INPUT        : none	
* OUTPUT       : none	
* RETURN       : none	
*****************************************************************************/

void MENU_LIB_erreur()
{  
  MACRO_TRACE(("---> start  : MENU_CALLBACKS_erreur\n"))

  printf("errno %d\n",errno);

  MACRO_TRACE(("---> end    : MENU_CALLBACKS_erreur\n"))
}

void show_message(char *texte)
{
 GtkWidget *label;
  
 gtk_widget_show(dialog_message);

 label = lookup_widget(GTK_WIDGET(dialog_message),"label_message");

 gtk_label_set_text(GTK_LABEL(label),texte);
 
}


/*****************************************************************************
*                              HEADER FUNCTION                              
******************************************************************************
* NAME	       : MENU_get_files	
* GOAL         :  
******************************************************************************
* AUTHOR       : JFV after Marc TISSERANT                                             
* DATE WRITTEN : 1995, March	
* TESTS        : level 0 - no specific tests tool is used for the development.	
* REVIEWER     : ...	
* MODIFICATION : none	
******************************************************************************
* SCOPE        : local in the module.	
* INPUT        : none
* OUTPUT       : none	
* RETURN       : none	
*****************************************************************************/
void MENU_get_files(char *extension)
{
#include <dirent.h>

     DIR *dp;
     int i;
     int nb_ext;
     char buffer[512];
     char directory[512];
     struct dirent *dirp;

     MACRO_TRACE(("---> start  : MENU_WIN_VIRTUAL_alloc_text\n"))

     nb_ext = strlen(extension);
     MENU_LIB_alloc(TYPE_char*(10000),&strtext);
     strtext[0] = 0;
     sprintf(directory,"%s",current_path);
     /*  printf("%s\n",directory); */
     if (dp=opendir(directory))
     {

       while((dirp=readdir(dp)) != NULL)
       {
	  if (strlen(dirp->d_name) > nb_ext)
	  {
	    i = strlen(dirp->d_name) - nb_ext;
	    /*
    printf("%s\n",dirp->d_name);
	    */
	    if (strcmp( (dirp->d_name)+i,extension)==0)
	    {
	      sprintf(buffer,dirp->d_name);
	      *(buffer+i)='\0';
	      sprintf(buffer,"%s\n",buffer);
	      strcat(strtext,buffer);
	    }
	  }
       }    
       closedir(dp);
     } /* fin du  if */

     MACRO_TRACE(("---> end    : MENU_WIN_VIRTUAL_alloc_text\n"))
}

/*****************************************************************************
*                       END OF THE MODULE                               
*****************************************************************************/
