#ifndef _FIND_FILES_
#define _FIND_FILES_
#define DEBUG_FILES	0

/* 
 *  Functions pour la gestion des fichiers de xnbc
 *
 *  JF Vibert 1 sept 1997
 *
 *	modifications:
 *		9 sept 97: renvoit noname si pas de fichier.xnbcrc
 *              20 sept 2001 : .xnbcrc in the project dir instead of home dir
 *                             only relative path is kept (JFV)
 *		29 oct 2003 : extension of network file becomes .ntw instead of .lnk
 *              18 dec 2003 : modification de put_* et de save_xnbcrc pour XNBC V9 (JFV)
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libgen.h>
#include "find_files.h"

static  char actual_experiment[256];
static  char simulation_name[256];
static  char network_name[256];
static  char lnk_name[256];
static  char pmb_name[256];
static  char tms_name[256];
static  char sim_name[256];
static  char abe_name[256];
static  char prj_name[256];
static  char dir_name[256];


/* Ces fonctions permettent de recuperer les noms des fichiers */

/******************************************************************************
	Function to obtain the current (or the last used simulation name
	There is only one line in the ~/.xnbc_current:
	the fully qualified path of the simulation directory, ended by a /
	All other files will be relative to this, in order to be able to 
	move the entire directory elsewhere.
*******************************************************************************/
char *get_current_experiment(void)
{
	FILE	*p_rc;
	char	*home;
	char    *p_c;
	char	buffer[256];

	home = NULL;
	home = getenv("HOME");
	if (!home)
	{
	  home = getenv("PWD");
	}
	if(home)
	{
		strcpy(buffer,home);
		strcat(buffer,"/.xnbc_current");

		if (p_rc = fopen(buffer,"r"))
		{
			if(!fgets(actual_experiment,256,p_rc))
			{
			  /*
			  strcpy(actual_experiment,home);
			  strcat(actual_experiment,"/noname_nbc/");
			  printf("(current 1) No experiment yet... Create one\n");
			  */
			}
			fclose(p_rc);
		}
		else
		{
		  /*
		  strcpy(actual_experiment,home);
		  strcat(actual_experiment,"/noname_nbc/");
		  printf("(current 2) No experiment yet... Create one\n");
		  */
		}
	}
	else
	{
	  /*
	   strcpy(actual_experiment,"./noname_nbc/");
	  printf("(current 3) No experiment yet... Create one\n");
	  */
	}
	/*
	printf("find_files.c: Current experiment: %s\n",actual_experiment);
	*/
	while (p_c = strrchr(actual_experiment,'\n'))
	  *p_c = 0; /* removes the final return if any */
	return(actual_experiment);
}


/******************************************************************************
	Function to obtain the generic simulation name
*******************************************************************************/
char *get_simulation_name(void)
{
	FILE	*p_rc;
	char	*home;
	char	buffer[256];
	char	value[256];
	char	parametre[256];


	strcpy(buffer,get_current_experiment());
	strcat(buffer,"/.xnbcrc");
	
	if (p_rc = fopen(buffer,"r"))
	  {
	    while(fgets(buffer,256,p_rc))
	      {
		sscanf(buffer,"%s %s",parametre,value);
		if (!strcmp(parametre,"Simulation:"))		
		  {
		    sscanf(value,"%s",simulation_name);
		    break;
		  }
	      }
	    fclose(p_rc);
	  }
	else
	  {
	  /*
	    strcpy(simulation_name,"./noname_nbc/noname");
	  printf("(simul 2) No experiment yet... Create one\n");
	  */
	  }

	return(simulation_name);
}

/******************************************************************************
	Function to obtain the project name (*_nbc)
*******************************************************************************/
char *get_project_name(void)
{
	FILE	*p_rc;
	char	*home;
	char	buffer[256];
	char	fichier[256];
	char	value[256];
	char	parametre[256];

	strcpy(buffer,get_current_experiment());
	strcat(buffer,"/.xnbcrc");
	
	if (p_rc = fopen(buffer,"r"))
	  {
	    while(fgets(buffer,256,p_rc))
	      {
		sscanf(buffer,"%s %s",parametre,value);
		if (!strcmp(parametre,"Project:"))		
		  {
		    sscanf(value,"%s",prj_name);
		    break;
		  }
	      }
	    fclose(p_rc);
	  }
	else
	  {
	  /*
	    strcpy(simulation_name,"./noname_nbc/noname");
	  printf("(project 2) No project yet... Create one\n");
	  */
	  }
	return(prj_name);
}

/******************************************************************************
	Function to obtain the project directory name (*_nbc)
*******************************************************************************/
char *get_dir_name(void)
{
	FILE	*p_rc;
	char	*home;
	char	buffer[256];
	char	fichier[256];
	char	value[256];
	char	parametre[256];


		strcpy(fichier,get_current_experiment());
		strcat(fichier,"/.xnbcrc");

		printf("Reading %s...\n",fichier);
		if (p_rc = fopen(fichier,"r"))
		{
			while(fgets(buffer,256,p_rc))
			{
				sscanf(buffer,"%s %s",parametre,value);
				if (!strcmp(parametre,"Directory:"))		
				{
					sscanf(value,"%s",dir_name);
					break;
				}
			}
			fclose(p_rc);
		}
		else
		  {
		  /*
		    strcpy(simulation_name,"./noname_nbc/noname");
		  printf("(dir 1) No experiment yet... Create one\n");
		  */
		  }

	return(dir_name);
}


/******************************************************************************
	Function to obtain the network name (lnk)
*******************************************************************************/
char *get_network_name(void)
{
	FILE	*p_rc;
	char	*home;
	char	fichier[256];
	char	buffer[256];
	char	value[256];
	char	parametre[256];

		strcpy(fichier,get_current_experiment());
		strcat(fichier,"/.xnbcrc");

		if (p_rc = fopen(fichier,"r"))
		{
			while(fgets(buffer,256,p_rc))
			{
				sscanf(buffer,"%s %s",parametre,value);
				if (!strcmp(parametre,"Network:"))
				{
					sscanf(value,"%s",lnk_name);
					break;
				}
			}
			fclose(p_rc);
		}
		else
		  {
		  /*
		    strcpy(simulation_name,"./noname_nbc/noname");
		  printf("(network 1) No experiment yet... Create one\n");
		  */
		  }


	return(lnk_name);
}


/******************************************************************************
	Function to obtain the pmb name
*******************************************************************************/
char *get_pmb_name(void)
{

	strcpy(pmb_name, get_simulation_name());
	strcat(pmb_name,".pmb");
	return(pmb_name);
}

/******************************************************************************
	Function to obtain the lnk name
*******************************************************************************/
char *get_lnk_name(void)
{

	strcpy(pmb_name, get_network_name());
	strcat(pmb_name,".ntw");
	return(pmb_name);
}

/******************************************************************************
	Function to obtain the tms name
*******************************************************************************/
char *get_tms_name(void)
{

	strcpy(tms_name, get_simulation_name());
	strcat(tms_name,".tms");
	return(tms_name);
}

/******************************************************************************
	Function to obtain the abe name
*******************************************************************************/
char *get_abe_name(void)
{

	strcpy(abe_name, get_simulation_name());
	strcat(abe_name,".abe");
	return(abe_name);
}

/******************************************************************************
	Function to obtain the sim name
*******************************************************************************/
char *get_sim_name(void)
{

	strcpy(sim_name, get_simulation_name());
	strcat(sim_name,".sim");
	return(sim_name);
}
/******************************************************************************
	Function to obtain the current (or the last used simulation name
	There is only one line in the ~/.xnbc_current:
	the fully qualified path of the simulation directory, ended by a /
	All other files will be relative to this, in order to be able to
	move the entire directory elsewhere.
*******************************************************************************/
int put_current_experiment(char *new_dir)
{
	FILE	*p_rc;
	char	*home;
	char	buffer[256];
	char    base[256];
	char    exp[256];

	home = NULL;
	home = getenv("HOME");
	if (!home)
	{
	  home = getenv("PWD");
	}
	if(home)
	{
		strcpy(buffer,home);
		strcat(buffer,"/.xnbc_current");

		if (p_rc = fopen(buffer,"w"))
		{

		  fprintf(p_rc,"%s",new_dir);
		  /*
		  printf("find_files.c: put_current_experiment : %s \n",base);
		  */
		  strcpy(actual_experiment,new_dir);
		  fclose(p_rc);
		}
		else
		{
		  printf("Impossible to save %s\n",buffer);
		  return(0);
		}
	}
	else
	{
	  printf("Impossible to get the HOME directory and, consequently, to save %s\n",buffer);
	  return(0);
	}

	return(1);
}

/******************************************************************************
	Function to update (or create) the .xnbcrc file
*******************************************************************************/
int save_xnbcrc(char *exp_dir, char *network, char *simulation, char *project, char *dir)
{
	FILE	*p_rc;
	char	*home,
	        *p_c_dot,
	        *p_c_slash;
	char	buffer[256],
	        fichier[256],
	        sim[256],
	        lnk[256],
	        old_sim[256],
                old_lnk[256],
                old_dir[256],
                old_prj[256];
	char	value[256];
	char	parametre[256];


        buffer[0] = 0;
	fichier[0] = 0;
	/*
	strcpy(buffer, actual_experiment);
	*/
	strcpy(buffer, exp_dir);

	if(buffer)
	{
	  strcpy(fichier,buffer);
	  strcat(fichier,"/.xnbcrc");
	  /*
	  printf("find_files : **%s**\n",fichier);
	  */
	  if (p_rc = fopen(fichier,"r"))
	  {
	      while(fgets(buffer,256,p_rc))
	      {
		  sscanf(buffer,"%s %s",parametre,value);
		  if (!strcmp(parametre,"Network:"))
		    {
		      sscanf(value,"%s",old_lnk);
		    }
		  if (!strcmp(parametre,"Simulation:"))
		    {
		      sscanf(value,"%s",old_sim);
		    }
		  if (!strcmp(parametre,"Project:"))
		    {
		      sscanf(value,"%s",old_prj);
		    }
		  if (!strcmp(parametre,"Directory:"))
		    {
		      sscanf(value,"%s",old_dir);
		    }
	      }
	      fclose(p_rc);
	  }
	  /* et on le refait */
	  if (p_rc = fopen(fichier,"w"))
	  {
	    printf("Creating %s\n",fichier);
	    if (network)
	    {
	      strcpy(lnk, network);
	      if (p_c_dot = strrchr(lnk,'.'))
	      {
		*p_c_dot = 0;
	      }
	      fprintf(p_rc,"Network: %s\n",lnk);
	    }
	    else
	      fprintf(p_rc,"Network: %s\n",old_lnk);
	    if (simulation)
	    {
	      strcpy(sim, simulation);
	      if (p_c_dot = strrchr(sim,'.'))
	      {
		*p_c_dot = 0;
	      }
	      fprintf(p_rc,"Simulation: %s\n",sim);
	    }
	    else
	      fprintf(p_rc,"Simulation: %s\n",old_sim);
	    if (project)
	      fprintf(p_rc,"Project: %s\n",project);
	    else
	      fprintf(p_rc,"Project: %s\n",old_prj);
	    if (dir)
	      fprintf(p_rc,"Directory: %s\n",dir);
	    else
	      fprintf(p_rc,"Directory: %s\n",old_dir);
	    fclose(p_rc);
	  }
	  else
	    return (0);
	}
	return (1);
}
/*******************************************************************************
       Function for saving the network name
*******************************************************************************/

put_network_name(char *exp_dir, char *network)
{
  return(save_xnbcrc(exp_dir,network,NULL,NULL,NULL));
}

/*******************************************************************************
       Function for saving the simulation name
*******************************************************************************/

put_simulation_name(char *exp_dir, char *simulation)
{
  return(save_xnbcrc(exp_dir,NULL,simulation,NULL,NULL));
}

/*******************************************************************************
       Function for saving the project name
*******************************************************************************/

put_project_name(char *exp_dir, char *project)
{
  return(save_xnbcrc(exp_dir,NULL,NULL,project,NULL));
}

/*******************************************************************************
       Function for saving the network name
*******************************************************************************/

put_dir_name(char *exp_dir, char *dir)
{
  return(save_xnbcrc(exp_dir,NULL,NULL,NULL,dir));
}

/*******************************************************************************
       Function for browsing the html manual
*******************************************************************************/

void browse_manual(char *section)
{
	char	*browser;
	char	*root_html;
	char	xnbc_browser[256];
	char	xnbc_root_html[256];
	char	buffer[256];

	browser = root_html = NULL;
	browser = getenv("XNBC_BROWSER");
	if(browser)
	{
		strcpy(xnbc_browser,browser);
	}
	else
	{
		strcpy(xnbc_browser,DEF_XNBC_BROWSER);
	} /* fin du else */

	root_html = getenv("XNBC_ROOT_HTML");
	if(root_html)
	{
		strcpy(xnbc_root_html,root_html);
	}
	else
	{
		strcpy(xnbc_root_html,DEF_XNBC_ROOT_HTML);
	} /* fin du else */

	sprintf(buffer,"%s %s%s&",xnbc_browser,xnbc_root_html,section);

	system(buffer);
}




#endif




