#define _MEMSPY_ 0
#define DEBUG 0

/*---------------------------------------------------------------------------
	A V L . C : Porogramme pour gerer les diifrentes matrices de connexion
---------------------------------------------------------------------------*/
/*
*
*	Author: Yann HERRY
*		Laboratoire de Biomathematiques
*		URBB B3E INSERM U263
*		Faculte de Medecine Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert, Mai 1993 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h> /* pour time dans modif_avl */

#include "le_alea.h" /* pour ran1 */
#include "le_avl.h"

#if _MEMSPY_
#include "/u/sa2/boussard/MEMORY/memspy.h"
#endif

/*                                                                          					    */
/*            DECLARATION DES TYPES ET DES VARIABLES GLOBALES			    */
/*                                                                         					    */

extern ARBRE_AVL LIENS;     		 /* ARBRE CONTENANT TOUS LES LIENS */
extern struct NOEUD *Find_noeud; 	 /* NOEUD DE TRAVAIL COURANT             */


/*			  PROTOTYPES  													*/

static void RG(ARBRE_AVL *);
static void RD(ARBRE_AVL *);
static void RGD(ARBRE_AVL *);
static void RDG(ARBRE_AVL *);
static int  sup(int ,int);
static int  hauteur(ARBRE_AVL );
static void reequilibrage(ARBRE_AVL *);
static void MOVEMAX(struct element *, ARBRE_AVL *, int );



/* ---------------- PROCEDURES DE ROTATION D'AVL -----------------------

 *       A =  5       ------>    rd(A)=3
 *        ___/ \___                        ___/ \___
 *        3         6                          1       5
 *     __/ \__                                   __/ \__
 *     1     4                                     4       6

 *       A =  3       ------>    rg(A)= 5
 *        ___/ \___                        ___/ \___
 *        1       5                           3          6
 *             __/ \__                  __/ \__
 *             4     6                  1     4


 *                     7
 *            _____/ \_____    ----> rdg(A)=
 *       A =  3           8                      5
 *        ___/ \___                        ___/ \___
 *          1       5                         3           7
 *             __/ \__                  __/ \__  __/ \__
 *              4     6                   1      4    6     8


 *                     4
 *            _____/ \_____    ----> rgd(A)=
 *       A =  1           7                           5
 *                    ___/ \___                ___/ \___
 *                    5          8              4              7
 *                 __/ \__                  __/ \__  __/ \__
 *                 3       6                  1      3    6     8

 *---------------------------------------------------------------------------*/

static void RG(ARBRE_AVL *A)
{
	 ARBRE_AVL Aux;

	 Aux=(*A)->d;
	 (*A)->d=Aux->g;
	 Aux->g=(*A);
	 (*A)=Aux;

}  /* RG */
/*---------------------------------------------------------------------------*/
static void RD(ARBRE_AVL *A)
{

	 ARBRE_AVL Aux;

	 Aux=(*A)->g;
	 (*A)->g=Aux->d;
	 Aux->d=(*A);
	 (*A)=Aux;

}  /* RD */
/*---------------------------------------------------------------------------*/
static void RGD(ARBRE_AVL *A)
{
	 RG(&(*A)->g);
	 RD(A);

} /* RGD */
/*---------------------------------------------------------------------------*/
static void RDG(ARBRE_AVL *A)
{
	 RD(&(*A)->d);
	 RG(A);

} /* RDG */

/* ---------------- FONCTIONS HAUTEUR & REEQUILIBRAGE --------------*/

static int sup(int x,int y)
{
	if (x > y) return(x+1);
	else return(y+1);
}

/*---------------------------------------------------------------------------*/
static int hauteur(ARBRE_AVL A)
{
	int inter=0;
	if (A==NULL) return(0);
	else
	{
		inter=sup(hauteur(A->g),hauteur(A->d));
		return(inter);
	}
}
/*---------------------------------------------------------------------------*/
static void reequilibrage(ARBRE_AVL *T)
{
	 int hg,hd;
	 switch((*T)->deseq)
	 {
		case  0: break;
		case  1: break ;
		case -1: break ;
		case   2:
			  switch((*T)->g->deseq)
			  {
				  case  0 :
				  case  1 :  /* idem */
						RD(T);
						hg=hauteur((*T)->g);
						hd=hauteur((*T)->d);
						(*T)->deseq=hg-hd;
						hg=hauteur((*T)->d->g);
						hd=hauteur((*T)->d->d);
						(*T)->d->deseq=hg-hd;
						break;
				  case -1 :
						RGD(T);
						switch((*T)->deseq)
						{
						   case 1 :
								  (*T)->g->deseq=0;
								  (*T)->d->deseq = -1;
								  break;
						   case  -1 :
								  (*T)->g->deseq=1;
								  (*T)->d->deseq=0;
								  break;
						   case 0  : /* cas ou A=Y */
								  (*T)->g->deseq=0;
								  (*T)->d->deseq=0;
								  break;

						};
						(*T)->deseq=0;
						break;

			  };
			  break;

		case -2 : /* cas symetrique du cas +2 */
			  switch((*T)->d->deseq)
			  {
				   case	 0 :
				   case -1 : /* idem */
				   {
						 RG(T);
						 hg=hauteur((*T)->g);
						 hd=hauteur((*T)->d);
						 (*T)->deseq=hg-hd;
						 hg=hauteur((*T)->g->g);
						 hd=hauteur((*T)->g->d);
						 (*T)->g->deseq=hg-hd;
						 break;
				   };
				   case 1 :
				   {
						 RDG(T);
						 switch((*T)->deseq)
						 {
							 case   1 :
								   (*T)->g->deseq=0;
								   (*T)->d->deseq = -1;
								   break;
							  case -1 :
								   (*T)->g->deseq=1;
								   (*T)->d->deseq=0;
									 break;
							  case 0  :
								   (*T)->g->deseq=0;
								   (*T)->d->deseq=0;
								   break;
						 }
						 (*T)->deseq=0;
				   }
				   break;
			  }
			  break;
	 }
}

/* ----------------- PROCEDURES DE GESTION DES AVL -------------------*/

struct NOEUD *INSERER(struct element *elem, ARBRE_AVL *A, int type)
{
	ARBRE_AVL New,T,P,TT,PP, Inter;

/* Creation du noeud a ajouter, Test taille memoire */

	if ( (New=(ARBRE_AVL)calloc( 1, sizeof(struct NOEUD)  ) )== NULL) return(NULL);


	New->elem.Cle	=	elem->Cle;
	New->elem.NbNeurL	=	elem->NbNeurL;
	New->elem.NbNeurH	=	elem->NbNeurH;
	New->elem.Type	=	elem->Type;
	New->deseq	=	0;
	New->g		=	NULL; 
	New->d		=	NULL;
	New->pred		=	NULL; 
	New->suiv		=	NULL;


/* Insertion du noeud dans l'arbre */
	if (*A==NULL)  *A=New;
	else
	{
		T = *A;
		TT=NULL;
		P = *A;
		PP=NULL;
		/* TT est le pere de T et PP est le pere de P */
		while (P != NULL)
		{
			 /* Descente a la recherche de la feuille, en memorisant */
			 /*  le dernier noeud pointe par T dont le desequilbre   */
			 /*  est +/- 1                                           */

				if (P->deseq != 0) {  T=P; TT=PP; };

				PP=P;

				if (elem->Cle <= P->elem.Cle)	P=P->g;
				else P=P->d;

		};
		/* Adjonction */
		switch(type)
		{
			case AVL_SIMPLE :
				if (elem->Cle <= PP->elem.Cle)
					PP->g=New;
				else
					PP->d=New;
				break;
			case AVL_DCHAINE :
				if (elem->Cle <= PP->elem.Cle)
				{
					PP->g=New;
					/* On Ajoute a Gauche : MAJ Pred et Suiv */
					if (PP->pred != NULL) PP->pred->suiv=New;
					Inter = PP->pred;
					PP->pred=New;
					New->suiv=PP;
					New->pred=Inter;
				}
				else
				{
					PP->d=New;
					/* On Ajoute a Droite : MAJ Pred et Suiv */
					if (PP->suiv != NULL) PP->suiv->pred=New;
					Inter = PP->suiv;
					PP->suiv=New;
					New->pred=PP;
					New->suiv=Inter;

				}
				break;
		}

		/* modification du desequilibre sur le chemin de T a New */
		P=T;

		while( P != New)
		{

			if (elem->Cle <= P->elem.Cle)
			{
				P->deseq=P->deseq+1;
				P=P->g;
			}
			else
			{
				P->deseq=P->deseq-1;
				P=P->d;
			}
		}
		reequilibrage(&T);

		/* mise a jour des pointeurs apres une rotation */
		if (TT == NULL) *A=T;
		else
		{
			if (T->elem.Cle <= TT->elem.Cle) TT->g=T;
			else TT->d=T;

		}  /* descente */
	}
	return(New);
} /* INSERER */

/*---------- Algorithme de suppression dans un arbre binaire -----------*/

static void MOVEMAX(struct element *MAX, ARBRE_AVL *A, int type)
{
	if ((*A)->d==NULL)
	{
		 *MAX=(*A)->elem;
		 if (type == AVL_DCHAINE)
		 {
				if((*A)->pred != NULL) (*A)->pred->suiv = (*A)->suiv; /* MAJ PRED & SUIV */
				if((*A)->suiv != NULL) (*A)->suiv->pred = (*A)->pred;
		 }
		
		 *A=(*A)->g;
		 if ((*A) != NULL) (*A)->deseq=(*A)->deseq-1;
	}
	else
	{
		 (*A)->deseq=(*A)->deseq+1;
		 MOVEMAX(MAX,&(*A)->d,type);
		 reequilibrage(A);
	 };
}
/*---------------------------------------------------------------------------*/
void ENLEVER(struct element *elem, ARBRE_AVL *A, int type)
{
	struct element MAX;
	int hg, hd;
	ARBRE_AVL inter;

	MAX.Cle=0; /* Initialisation */

	if ((*A)!=NULL)
	{
		if (elem->Cle < (*A)->elem.Cle )
		{
			ENLEVER(elem, &(*A)->g,type);
			hg=hauteur((*A)->g);
			hd=hauteur((*A)->d);
			(*A)->deseq=hg-hd;
			reequilibrage(A);
		}
		else
			if (elem->Cle > (*A)->elem.Cle )

			{
				ENLEVER(elem,&(*A)->d,type);
				hg=hauteur((*A)->g);
				hd=hauteur((*A)->d);
				(*A)->deseq=hg-hd;
				reequilibrage(A);
			}
			else /* A->elem.Cle = elem.Cle */
				if ((*A)->g==NULL)
				{
					if (type == AVL_DCHAINE)
					{
						if((*A)->pred != NULL) (*A)->pred->suiv = (*A)->suiv; /* MAJ PRED & SUIV */
						if((*A)->suiv != NULL) (*A)->suiv->pred = (*A)->pred;
					}

					if ((*A) != NULL) 
					{
						inter = (*A);
						(*A)=(*A)->d;
						/* Liberer la memoire */
						if (inter != NULL)
						{ 
						free(inter->elem.Matrix);
						free(inter);
						}
					}
				}
				else
					if ((*A)->d==NULL)
					{
						if (type == AVL_DCHAINE)
						{
							if((*A)->pred != NULL) (*A)->pred->suiv = (*A)->suiv; /* MAJ PRED & SUIV */
							if((*A)->suiv != NULL) (*A)->suiv->pred = (*A)->pred;
						}

						if ((*A) != NULL)
						{
						    inter = (*A);
						    (*A)=(*A)->g;

						    /* Liberer la memoire */
						    if (inter != NULL)
						    {
							free(inter->elem.Matrix);
							free(inter);
						    }
						}

					}
					 else /* A->d != NULL et A->g != NULL */
					 {
						if ((*A)->g->d==NULL)  (*A)->deseq=(*A)->deseq-1;
						MOVEMAX(&MAX, &(*A)->g, type);
						(*A)->elem=MAX;
						reequilibrage(A);
					 };
	}
} /* ENLEVER */

/*----------------- Fonctions sur l'AVL -----------------------------*/

struct element MIN(ARBRE_AVL A)
{

	if (A->g == NULL)
		return(A->elem);
	else
		return(MIN(A->g));

}
/*-------------------------------------------------------------------*/
void parcours(ARBRE_AVL A)
{

	int i;
	if (A!=NULL)
	{
		parcours(A->g);
		printf("CLE : %2d NbL : %2d NbH : %2d\n",A->elem.Cle, A->elem.NbNeurL, A->elem.NbNeurH);

		for(i=0; i<  A->elem.NbNeurL*A->elem.NbNeurH; i++)
		{
			printf("(%d) OnOff : %d Etat : %d\n",i, A->elem.Matrix[i].OnOff,
			A->elem.Matrix[i].Etat);
		}

		parcours(A->d);
	}
}
/*-------------------------------------------------------------------*/
void liste(ARBRE_AVL A)
{
	int i;
	ARBRE_AVL pt;
	pt=A;

	if(A == NULL) return;
	while (pt->pred!=NULL) pt=pt->pred;

	while (pt!=NULL)
	{
		printf("CLE : %2d NbL : %2d NbH : %2d\n",pt->elem.Cle, pt->elem.NbNeurL, pt->elem.NbNeurH);
/*
		for(i=0; i<  A->elem.NbNeurL*A->elem.NbNeurH; i++)
		{
			printf("(%d) OnOff : %d Etat : %d\n",i, A->elem.Matrix[i].OnOff,
			A->elem.Matrix[i].Etat);
		}
*/
		pt=pt->suiv;
	}
}
/*----------------------------------------------------------------------*/

struct NOEUD *search_listeD(int Cle, ARBRE_AVL *A)
{
	ARBRE_AVL pt = *A;

	if (pt == NULL) return(NULL);

	while (pt->pred!=NULL) pt=pt->pred;

	while ( (pt != NULL)&&(pt->elem.Cle != Cle) ) pt=pt->suiv;

	return(pt);
}

/*----------------------------------------------------------------------*/

struct NOEUD *search_avl(int Cle, ARBRE_AVL *A)
{
	ARBRE_AVL pt = *A;
	
	
	if (pt == NULL) return(NULL);
	else
		if(pt->elem.Cle == Cle) return(pt);
		else
			if (Cle < pt->elem.Cle) return(search_avl(Cle,&pt->g));
			else  return(search_avl(Cle,&pt->d));
}

/*----------------------------------------------------------------------
	Retourner l'element X en tete de l'arbre avl
*/

struct element tete(ARBRE_AVL *X)
{
	return(MIN((*X)));
}
/*---------------------------------------------------------------*/
int modif_avl(int orig, int fin, int nbL, int nbH, int type)
{
	int Cle;
	int i, nbcase = nbL*nbH;
	struct element New_elem;
	int	set = (int) time (0) & 077;
	float   result; 
	
	Cle=Create_Cle(orig,fin); 

/*---- Premier Cas de figure : nouveau lien cree entre 2 clusters ------*/
	if ((Find_noeud = search_listeD(Cle, &LIENS)) == NULL)
	{
#if DEBUG
		printf("NOUVEAU LIEN\n");
#endif
		New_elem.Cle=Cle;
		New_elem.Type = type;
		New_elem.NbNeurL = nbL;
		New_elem.NbNeurH = nbH;

		if( (Find_noeud = INSERER(&New_elem,&LIENS,AVL_DCHAINE)) == NULL)
			return(MEMORY_AVL);

		/* Allocation memoire pour la matrice */
		if (type == MANUEL)
		{
			if ( (Find_noeud->elem.Matrix = (MAT *)calloc(nbL*nbH, sizeof(MAT)) )== NULL)
			return(MEMORY_MATRIX);
			for (i=0; i < nbcase; i++)
			{
				Find_noeud->elem.Matrix[i].OnOff = 0;
				Find_noeud->elem.Matrix[i].Etat = NEANT;
			}
	
		}
		else if (type == EXCIT)
		{
			if ( (Find_noeud->elem.Matrix = (MAT *)calloc(nbL*nbH, sizeof(MAT)) )== NULL)
			return(MEMORY_MATRIX);
			for (i=0; i < nbcase; i++)
			{
				Find_noeud->elem.Matrix[i].OnOff = 1;
				Find_noeud->elem.Matrix[i].Etat = EXCIT;
			}
	
		}
		else if (type == INHIB)
		{
			if ( (Find_noeud->elem.Matrix = (MAT *)calloc(nbL*nbH, sizeof(MAT)) )== NULL)
			return(MEMORY_MATRIX);
			for (i=0; i < nbcase; i++)
			{
				Find_noeud->elem.Matrix[i].OnOff = 1;
				Find_noeud->elem.Matrix[i].Etat = INHIB;
			}
	
		}
		else if (type == ALEAT)
		{
			if ( (Find_noeud->elem.Matrix = (MAT *)calloc(nbL*nbH, sizeof(MAT)) )== NULL)
			return(MEMORY_MATRIX);
			for (i=0; i < nbcase; i++)
			{
				result = ran1(&set);
				Find_noeud->elem.Matrix[i].OnOff = 1;
				Find_noeud->elem.Matrix[i].Etat = (result < 0.5 ? INHIB : EXCIT);
			}
		
		}
		
	}
/*-- Deuxieme Cas de figure : le lien existe mais la taille des clusters a change ---*/
	else
	{
		if ((Find_noeud->elem.NbNeurL != nbL)||(Find_noeud->elem.NbNeurH != nbH))
		{

#if DEBUG
			printf("MODIFICATION TAILLE\n");
#endif

			Find_noeud->elem.NbNeurL = nbL;
			Find_noeud->elem.NbNeurH = nbH;
			free(Find_noeud->elem.Matrix);
			if ( (Find_noeud->elem.Matrix = (MAT *)calloc(nbL*nbH, sizeof(MAT)) ) == NULL)
				return(MEMORY_MATRIX);
			
			if (type == MANUEL)
			{
				

			}
			else if (type == EXCIT)
			{
				
				for (i=0; i < nbcase; i++)
				{
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = EXCIT;
				}

			}
			else if (type == INHIB)
			{
				
				for (i=0; i < nbcase; i++)
				{
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = INHIB;
				}

			}
			else if (type == ALEAT)
			{
				
				for (i=0; i < nbcase; i++)
				{
					result = ran1(&set);
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = (result < 0.5 ? INHIB : EXCIT);
				}

			}

		}
/*-- Troisieme Cas de figure : lien existe, taille identique mais modification du lien ---*/
		if (Find_noeud->elem.Type != type)
		{
#if DEBUG
			printf("TYPE DIFFERENT\n");
#endif
		
			Find_noeud->elem.Type = type;
			if (type == MANUEL)
			{
				

			}
			else if (type == EXCIT)
			{
				
				for (i=0; i < nbcase; i++)
				{
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = EXCIT;
				}

			}
			else if (type == INHIB)
			{
				
				for (i=0; i < nbcase; i++)
				{
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = INHIB;
				}

			}
			else if (type == ALEAT)
			{
				
				for (i=0; i < nbcase; i++)
				{
					result = ran1(&set);
					Find_noeud->elem.Matrix[i].OnOff = 1;
					Find_noeud->elem.Matrix[i].Etat = (result < 0.5 ? INHIB : EXCIT);
				}

			}

		}
	}
		

	return(NOPROBLEM);

}
/*---------------------------------------------------------------*/
void modif_matrix(void)
{
	int nbL, nbH, etat, OnOff;
	int i, taille;

	nbL = Find_noeud->elem.NbNeurL;
	nbH = Find_noeud->elem.NbNeurH;
	taille = nbL * nbH;

	for(i = 0; i<taille;  i++)
	{
		printf("OnOff:"); scanf("%d",&OnOff);
		printf("Etat :"); scanf("%d",&etat);
		Find_noeud->elem.Matrix[i].OnOff = OnOff;
		Find_noeud->elem.Matrix[i].Etat = etat;
	}
}
/*---------------------------------------------------------------*/
int sup_avl(int orig, int fin)
{
	int Cle;

	Cle=Create_Cle(orig,fin);


	if ((Find_noeud = search_listeD(Cle, &LIENS)) == NULL) return(0);
	else
		ENLEVER(&Find_noeud->elem,&LIENS,AVL_DCHAINE);

	return(1);

}

/*---------------------------------------------------------------*/
int Create_Cle(int orig, int fin)
{
	char inter[20];
	int Cle;

	sprintf(inter,"%d%d",orig,fin);

	Cle = atoi(inter);

	return(Cle);
}
/*---------------------------------------------------------------*/
void clear_avl(ARBRE_AVL *A)
{
	int i;
	ARBRE_AVL pt;
	pt= *A;

	if(pt == NULL) return;
	while (pt->pred!=NULL) pt=pt->pred;

	do
	{
		ENLEVER(&pt->elem, &LIENS, AVL_DCHAINE);
		pt = pt->suiv;
	}while( pt );
}
/*======================================================================*/

