#define _MEMSPY_ 0
#define DEBUG 0

/*----------------------------------------------------------------------------
 *	D I A L O G _ I N P U T S . C
 *--------------------------------------------------------------------------*/

/*
*
*	Author: Patrick GLANDAZ
*		Laboratoire de Biomathematiques
*		URBB B3E INSERM U263
*		Faculte de Medecine Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert, July 1992 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*/


#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Mrm/MrmAppl.h>

#include "PGIntrinsic.h"
#include "le_vms_or_unix.h"
#include "le_strdup.h"
#include "le_typedefs.h"
#include "cluster_link.h"
#include "le_messbox.h"
#include "dialog_inputs.h"
#include "matrix.h"
#include "le_avl.h"

#if _MEMSPY_
#include "/u/sa2/boussard/MEMORY/memspy.h"
#endif

#define	 WAITING_ANSWER_ACTION		1
#define	 ANSWER_RECEIVED_ACTION		2
#define	 NO_WAITING_ACTION		3

int changed;
int source,target;

extern  int current_cluster;
extern  int max_link_para;
extern  XtAppContext app_context;
extern	ARBRE_AVL     	LIENS;      	/* ARBRE CONTENANT TOUS LES LIENS */
extern	struct NOEUD 	*Find_noeud;		/* NOEUD DE TRAVAIL COURANT       */

extern	CLUSTER_WIDGET	*clust_wid;
extern	LINK_PARAMETERS	*link_para;

	Widget		widget_array_di[DI_MAX_WIDGETS];
extern  GC		contexte_drawing;
extern  Pixmap		DrawPixmap;

static	Widget		inputlink_dialog = (Widget)NULL;
static	Widget		inputcluster_dialog = (Widget)NULL;
static	Widget		link_neuron_dialog = (Widget)NULL;
static	Widget		link_Drawing_dialog = (Widget)NULL;

static Widget          openConnect;

/*------------------------------------------------------------------------*/

static	Display		*the_display_di;

static	LINK_PARAMETERS	inputlink_return;
static	CLUSTER_WIDGET	inputcluster_return;
static	LINK_NEURON	link_neuron_return;
static	LINK_DRAWING	link_Drawing_return;

static	int		inputlink_action;
static	int		inputlink_answer;
static	int		inputcluster_action;
static	int		inputcluster_answer;
static	int		link_neuron_action;
static	int		link_neuron_answer;
static	int		link_Drawing_action;
static	int		link_Drawing_answer;

static	char		clust_name_buffer[20];

static XmString MASK = NULL;

static Widget TOP_LEVEL;

static int  di_apply_proc(void);

/*----------------------------------------------------------------------------
 *	I N I T _ D I A L O G _ I N P U T S
 *--------------------------------------------------------------------------*/

void init_dialog_inputs (toplevel_wid)
Widget	toplevel_wid;
{

	
	changed = 0; /* pour la matrice */

	the_display_di = XtDisplay (toplevel_wid);

	TOP_LEVEL = toplevel_wid;


	/*
	 * Create classes and widgets used in this program. 
	 */

	/* Begin user code block <create_dialogShell3> */
	/* End user code block <create_dialogShell3> */

	inputcluster_dialog = (Widget)CreateformD_InputCluster_leic(toplevel_wid);
	/*
	XtManageChild(formD_InputCluster_leic);
	*/
	/* Begin user code block <create_dialogShell2> */
	/* End user code block <create_dialogShell2> */

	inputlink_dialog = (Widget)CreateformD_InputLink_leil(toplevel_wid);
	/*
	XtManageChild(formD_InputLink_leil);
	*/
	/* Begin user code block <create_dialogShell1> */
	/* End user code block <create_dialogShell1> */

	link_neuron_dialog = (Widget)Createwindow_Grille(toplevel_wid);
	/*
	XtManageChild(window_Grille);
	*/

	/* Begin user code block <create_dialogShell> */
	/* End user code block <create_dialogShell> */

	/*    
	openConnect = (Widget)CreateopenConnect(toplevel_wid);
	*/
	/*
	  XtManageChild(openConnect);
	*/



}

/*----------------------------------------------------------------------------
 *	E X I T _ D I A L O G _ I N P U T S
 *--------------------------------------------------------------------------*/

void exit_dialog_inputs ()
{

	XtDestroyWidget (inputlink_dialog);
	inputlink_dialog = (Widget)NULL;
	
	XtDestroyWidget (inputcluster_dialog);
	inputcluster_dialog = (Widget)NULL;
}
 			    
/*----------------------------------------------------------------------------
 *	D I _ C R E A T E _ P R O C
 *--------------------------------------------------------------------------*/

void	di_create_proc (Widget	w, XtPointer tag, XtPointer reason)
/*
Widget	w;
int	*tag;
XmAnyCallbackStruct *reason;
*/
{
	int	widget_num = (int)tag;

	widget_array_di[widget_num] = w;
	
}


/*---------------------------------------------------------------------------
 *	D I _ O K _ P R O C
 *--------------------------------------------------------------------------*/

void	di_ok_proc (Widget w, XtPointer tag, XtPointer reason)
/*
Widget	w;
int	*tag;
XmAnyCallbackStruct *reason;
*/
{
	int	widget_num = (int)tag;
	XmFileSelectionBoxCallbackStruct *Fichier;
	char *Nom_Fichier;
	XmString Pattern;
	int n;
	static char last;
	Arg arglist[3];

	switch (widget_num)
	{
		case k_PushBG_Ok_leil:
			if (di_apply_proc())
			{
				inputlink_action = ANSWER_RECEIVED_ACTION;
				inputlink_answer = OK_ANSWER;
			}
			break;

		case k_PushBG_Apply_leil:
			di_apply_proc();
			break;
			
		case k_PushBG_Ok_leic:
		        if (verify_cluster_inputs()) 
			{
				inputcluster_action = ANSWER_RECEIVED_ACTION;
				inputcluster_answer = OK_ANSWER;
		       	}
			break;
			
		case k_toggleB_no_auto:
			if (XmToggleButtonGetState (widget_array_di[k_toggleB_no_auto]))
			{
				inputcluster_return.no_auto = FALSE;
			}
			else
			{
				inputcluster_return.no_auto = TRUE;
			}
			break;
			
		case k_toggleB_send:
			verify_link_inputs ();
			inputlink_return.deconnect = _PART_SEND_;
#if 0
			if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
	 		{
	 			
	 			inputlink_return.inter_cluster = '+';
				Notify_AVL(EXCIT);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
			{
				
				inputlink_return.inter_cluster = '-';
				Notify_AVL(INHIB);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
			{
				
				inputlink_return.inter_cluster = 'r';
				Notify_AVL(ALEAT);
			}
			else 
			{
				inputlink_return.inter_cluster = 'm';
				Notify_AVL(MANUEL);
			}
			deconnect(_PART_SEND_);
#endif

			break;

		case k_toggleB_receive:
			verify_link_inputs ();
			inputlink_return.deconnect = _PART_RECEIVE_;
#if 0
			if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
	 		{
	 			
	 			inputlink_return.inter_cluster = '+';
				Notify_AVL(EXCIT);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
			{
				
				inputlink_return.inter_cluster = '-';
				Notify_AVL(INHIB);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
			{
				
				inputlink_return.inter_cluster = 'r';
				Notify_AVL(ALEAT);
			}
			else 
			{
				inputlink_return.inter_cluster = 'm';
				Notify_AVL(MANUEL);
			}

			deconnect(_PART_RECEIVE_);
#endif
			break;

		case k_toggleB_average:
			verify_link_inputs ();
			inputlink_return.deconnect = _PART_AVERAGE_;
#if 0
			if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
	 		{
	 			
	 			inputlink_return.inter_cluster = '+';
				Notify_AVL(EXCIT);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
			{
				
				inputlink_return.inter_cluster = '-';
				Notify_AVL(INHIB);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
			{
				
				inputlink_return.inter_cluster = 'r';
				Notify_AVL(ALEAT);
			}
			else 
			{
				inputlink_return.inter_cluster = 'm';
				Notify_AVL(MANUEL);
			}

			deconnect(_PART_AVERAGE_);
#endif
			break;


		case k_ToggleBG_Excit_leil:
		case k_ToggleBG_Inhib_leil:
		case k_ToggleBG_Random_leil:
		case k_ToggleBG_Matrix_manual:
			verify_link_inputs ();
			break;

		case k_toggleB_full_connections:
			verify_link_inputs ();
			inputlink_return.deconnect = _FULLY_CONNECTED_;
#if 0
			deconnect(_FULLY_CONNECTED_);
#endif
			break;

		case k_PushBG_Details_leil :

		        di_apply_proc();
#if DEBUG	
			printf("Affiche la matrice de connection\n");

			if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
	 		{
	 			
	 			inputlink_return.inter_cluster = '+';
				Notify_AVL(EXCIT);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
			{
				
				inputlink_return.inter_cluster = '-';
				Notify_AVL(INHIB);
			}
			else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
			{
				
				inputlink_return.inter_cluster = 'r';
				Notify_AVL(ALEAT);
			}
			else 
			{
				inputlink_return.inter_cluster = 'm';
				Notify_AVL(MANUEL);
			}
#endif		

			/*---- Procedure de Gestion de la matrice des connexions ----*/

			Display_Matrice(widget_array_di[k_Window_Grid], widget_array_di[k_Draw_Matrix_leil]);
			break;

		case k_PushBG_Matrix_leil :
#if DEBUG
			printf("Validation de la Grille et Quitter\n");
#endif
	
			XFreeGC(XtDisplay(widget_array_di[k_Window_Grid]),
				contexte_drawing);
			XFreePixmap(XtDisplay(widget_array_di[k_Window_Grid]),
				DrawPixmap);
			/*
			if(changed)
			{
				XmToggleButtonSetState (
				widget_array_di[k_ToggleBG_Matrix_manual],
				True, True);
				inputlink_return.inter_cluster = 'm';
				Notify_AVL(MANUEL);
				changed = 0;
			}
			*/
			break;
		
		case k_ToggleBG_Matrix_excit :
			change_couleur(EXCIT);
			break;

		case k_ToggleBG_Matrix_nocon :
			change_couleur(NEANT);
			break;
			
		case k_ToggleBG_Matrix_inhib :
			change_couleur(INHIB);
			break;

		case k_PushBG_Matrix_record :
			gest_motif(RECORD);
			break;

		case k_PushBG_Matrix_place :
			gest_motif(PLACE);
			break;

		case k_PushBG_Matrix_stop :
			gest_motif(STOP);
			break;

		case k_PushBG_Matrix_clear :
			gest_motif(CLEAR);
			break;

		default:
			printf ("Appel de di_ok_proc avec un mauvais argument : %d\n", widget_num);
			break;
	}
}


/*---------------------------------------------------------------------------
 *	D I _ A P P L Y _ P R O C
 *--------------------------------------------------------------------------*/

int	di_apply_proc (void)
{

	if (verify_link_inputs ())
	{
         	/* Autoconexion ou non */

		if (XmToggleButtonGetState (widget_array_di[k_toggleB_no_auto]))
		{
			inputcluster_return.no_auto = FALSE;
		}
		else
		{
			inputcluster_return.no_auto = TRUE;
		}

		/* partiel ou non */

		if (XmToggleButtonGetState (widget_array_di[k_toggleB_send]))
		{
			inputlink_return.deconnect = _PART_SEND_;
			deconnect(_PART_SEND_);
		}
		else if (XmToggleButtonGetState (widget_array_di[k_toggleB_receive]))
		{
			inputlink_return.deconnect = _PART_RECEIVE_;
			deconnect(_PART_RECEIVE_);
		}
		else if (XmToggleButtonGetState (widget_array_di[k_toggleB_average]))
		{
			inputlink_return.deconnect = _PART_AVERAGE_;
			deconnect(_PART_AVERAGE_);
		}
		else if (XmToggleButtonGetState (widget_array_di[k_toggleB_full_connections]))
		{
			inputlink_return.deconnect = _FULLY_CONNECTED_;
			deconnect(_FULLY_CONNECTED_);
		}

		/* type de connexion */

		if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
		{

			inputlink_return.inter_cluster = '+';
			Notify_AVL(EXCIT);
		}
		else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
		{

			inputlink_return.inter_cluster = '-';
			Notify_AVL(INHIB);
		}
		else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
		{

			inputlink_return.inter_cluster = 'r';
			Notify_AVL(ALEAT);
		}
		else 
		{
			inputlink_return.inter_cluster = 'm';
			Notify_AVL(MANUEL);
		}

		return(TRUE);
	}
	else
	  return(FALSE);
}

/*---------------------------------------------------------------------------
 *	D I _ C A N C E L _ P R O C
 *--------------------------------------------------------------------------*/

void	di_cancel_proc (Widget w, XtPointer tag, XtPointer reason)
/*
Widget	w;
int	*tag;
XmAnyCallbackStruct *reason;
*/
{
	int	widget_num = (int)tag;

	switch (widget_num)
	{
		case k_PushBG_Cancel_leil:
			inputlink_action = ANSWER_RECEIVED_ACTION;
			inputlink_answer = CANCEL_ANSWER;
			break;
		case k_PushBG_Cancel_leic:
			inputcluster_action = ANSWER_RECEIVED_ACTION;
			inputcluster_answer = CANCEL_ANSWER;
			break;
		default:
			printf ("Appel de di_cancel_proc avec un mauvais argument.\n");
			break;
	}
}

/*---------------------------------------------------------------------------
 *	H E L P _ C B
 *--------------------------------------------------------------------------*/


/* ARGSUSED */
void help_CB(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
{

	void	display_help(int);
	char def[255];
	char *topic;
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
	int message = (int)(unsigned long)client;
	
	topic = NULL;
	if(!XtIsManaged(help_form))
		XtManageChild(help_form);
	else
		XtPopup(XtParent(help_form),XtGrabNone);
			

	display_help(message);
}


/*******************************************************************************
	Function for managing and displaying the help
*******************************************************************************/
void display_help(int flag)
{
	
	static char **help_files;
	static char **topics;
	static int nb_items;
	int count;
	int *pos;
	FILE *fp;
	char *pt;
	char file_name[256];
	XmString string;
	char buffer[256];
	Widget w;

	void help_CB();
	char *text;
	int i;
	long l;

#if DEBUG
	printf("In display_help\n");
#endif
#ifdef __CYGWIN__
#	define SEP_DIR "\\"
#	define CR 2
#else
#	define SEP_DIR "/"
#	define CR 1
#endif

	if(flag == -1)
	{
		for(i=0 ; i<nb_items ; i++)
		{
			free(help_files[i]);
			free(topics[i]);
		}
		free(help_files);
		free(topics);
		return;
	}


	if(!nb_items)
	{
		strcpy(file_name,help_directory);
		strcat(file_name,SEP_DIR);
		strcat(file_name,"link_hlprc");
		
		if(!(fp = fopen(file_name,"r")))
		{
			sprintf(buffer,"Sorry, file %s not found...\nNo Help Available !",file_name);
			display_error(help_form,buffer);
			return;
		}
		
		nb_items = 0;
		
		while(fgets(buffer,255,fp))
		{
			if (buffer[0] == '#')		/* commented out */
				continue;

			pt = strrchr(buffer,':');
			if(!pt)
				continue;
			nb_items++;
			*pt = '\0';
			pt += 2;
			if(nb_items == 1)
			{
				help_files = (char **)calloc(1,nb_items*sizeof(char *));
				topics = (char **)calloc(1,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}
			else
			{
				help_files = (char **)realloc(help_files,nb_items*sizeof(char *));
				topics = (char **)realloc(topics,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}	
		}
		
		for(i=0 ; i<nb_items ; i++)
		{
			string = XmStringCreateLocalized(topics[i]);
			w = XtVaCreateWidget(topics[i],
				xmPushButtonWidgetClass,
				topic_pulldownMenu,NULL);
		
  			XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)help_CB, (XtPointer)(i+1));
			XtManageChild(w);
			XmStringFree(string);
		}
		flag = 1;	
	}
	
	strcpy(file_name,help_directory);
	strcat(file_name,SEP_DIR);
	strcat(file_name,help_files[flag-1]);
		
	if(!(fp = fopen(file_name,"r")))
	{
		sprintf(buffer,"Sorry, file %s not found...\nTopic Not Available !",file_name);
		printf("%s",buffer);
		display_error(help_form,buffer); 
		return;
	}	
	
	fseek(fp, 0L, SEEK_END);
	l = ftell(fp)+1;
	text = (char *)calloc(1,l*sizeof(char));
	fseek(fp, 0L, SEEK_SET);
	fread(text, l*sizeof(char), 1, fp);
	text[l-1] = '\0';
	
	XmTextSetString(help_text,text);
	
	string = XmStringCreateLocalized(topics[flag-1]);
	XtVaSetValues(topic_label,XmNlabelString,string,NULL);
	XmStringFree(string);
	
	
	free(text);		
#if DEBUG
	printf("sortie de display_help\n");
#endif

}


/*---------------------------------------------------------------------------
 *	D I _ H E L P _ P R O C
 *--------------------------------------------------------------------------*/

void	di_help_proc (Widget w, XtPointer tag, XtPointer reason)
/*
Widget	w;
int	*tag;
XmAnyCallbackStruct *reason;
*/
{
	int	widget_num = (int)tag;

#if DEBUG
	printf("In di_help_proc avec %d\n", widget_num);
#endif	

	switch (widget_num)
	{
		case k_PushBG_Help_leil:
			Createhelp_form(w);
			break;
		case k_PushBG_Help_leic:
			Createhelp_form(w);
			break;
		default:
			printf ("Appel de di_help_proc avec un mauvais argument.\n");
			break;
	}
}

/*-----------------------------------------------------------------------------
 *			DI_CREATE_HELP
 *----------------------------------------------------------------------------*/
 
void di_create_help (Widget w, XtPointer tag, XtPointer reason)
/*
Widget		w;
int		*tag;
unsigned long	*reason;
*/
{
	int widget_num = (int)tag;

	if(widget_num != k_Draw_Matrix_leil)
	{	
		XtAddEventHandler(w,EnterWindowMask,FALSE,fill_label_help,
		(XtPointer)(unsigned long)widget_num);
		XtAddEventHandler(w,LeaveWindowMask,FALSE,clear_label_help,
		(XtPointer)(unsigned long)widget_num); 
	}
	else
	{
		XtAddEventHandler(w,LeaveWindowMask,FALSE,clear_label_help,
		(XtPointer)(unsigned long)widget_num); 
		XtAddEventHandler(w,PointerMotionMask,FALSE,display_coordinates,
		(XtPointer)(unsigned long)widget_num);
	}
}
/*----------------------------------------------------------------------------
 *		FILL_LABEL_HELP
 *---------------------------------------------------------------------------*/
 void fill_label_help(Widget w,int widget_num, XEvent *evt)
 {
 XmString string;
 
 XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],XmNrecomputeSize,
 	FALSE,NULL);
 
 switch(widget_num)
 	{	
	case k_PushBG_Matrix_leil :
			string = XmStringCreateLocalized(
	"Update Matrix and quit the connections editing area...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		
		case k_PushBG_Matrix_record :
			string = XmStringCreateLocalized(
	"Repetitives connections: start recording a new motif of connections...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;

		case k_PushBG_Matrix_place :
			string = XmStringCreateLocalized(
	"Paste the last motif you've recorded each time you click on the Matrix...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;

		case k_PushBG_Matrix_stop :
			string = XmStringCreateLocalized(
	"Restore regular editing mode...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		
		case k_PushBG_Matrix_clear :
			string = XmStringCreateLocalized(
	"WARNING! Arm the clearing procedure of all connections...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		case k_ToggleBG_Matrix_excit:
			string = XmStringCreateLocalized(
	"Establishing excitation connections when selected...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
			
		case k_ToggleBG_Matrix_nocon:
			string = XmStringCreateLocalized(
	"Disable clicked connection when selected...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
				
		case k_ToggleBG_Matrix_inhib:
			string = XmStringCreateLocalized(
	"Establishing inhibation connections when selected...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
				
		case k_Draw_Matrix_leil:
			string = XmStringCreateLocalized(
	"Click on wanted localization to establish or to clear connection...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
			
			
		default:
			printf ("Appel de fill_label_help avec un mauvais argument.\n");
			break;
	}
 }

/*---------------------------------------------------------------------------
 *		CLEAR_LABEL_HELP
 *---------------------------------------------------------------------------*/
 void clear_label_help(Widget w,int widget_num,XEvent *evt)
{
	XmString string;
	string = XmStringCreateLocalized(
	"                                                    ");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
	
} 

/*---------------------------------------------------------------------------
 *		DI_HELP_ON_LINE
 *---------------------------------------------------------------------------*/
void di_help_on_line (Widget w, XtPointer tag, XtPointer reason)
/*
Widget		w;
int		*tag;
unsigned long	*reason;
*/
{
	int widget_num = (int)tag;
	XmString string;

	switch(widget_num)
	{
		case k_PushBG_Matrix_leil :
			string = XmStringCreateLocalized(
	"Update Matrix and quit the connections editing area...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		
		case k_PushBG_Matrix_record :
			string = XmStringCreateLocalized(
	"Repetitives connections: start recording a new motif of connections...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;

		case k_PushBG_Matrix_place :
			string = XmStringCreateLocalized(
	"Paste the last motif you've recorded each time you click on the Matrix...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;

		case k_PushBG_Matrix_stop :
			string = XmStringCreateLocalized(
	"Restore regular editing mode...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		
		case k_PushBG_Matrix_clear :
			string = XmStringCreateLocalized(
	"WARNING! Arm the clearing procedure of all connections...");
			XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		default:
			/*printf ("Appel de di_help_on_line avec un mauvais
			argument.\n");*/
			break;
	}
}		

/*--------------------------------------------------------------------------
 *		DI_CLEAR_HELP_ON_LINE
 *--------------------------------------------------------------------------*/
 void di_clear_help_on_line (Widget w, XtPointer tag, XtPointer reason)
/*
Widget		w;
int		*tag;
unsigned long	*reason;
*/
{
	
}


/****************************************************************************
 *									    *
 *	I N P U T   L I N K						    *
 *									    *
 ****************************************************************************/

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ I N P U T L I N K
 *--------------------------------------------------------------------------*/

LINK_PARAMETERS	*display_inputlink (struct_link_para, clust1, clust2)
LINK_PARAMETERS *struct_link_para;
int		clust1, clust2;
{
	int		i;
	char string[256];

/*--- Remplissage de la fenetre --------------------------------------------*/

	if (XtIsManaged (inputlink_dialog))
	{
		XtUnmanageChild (inputlink_dialog);
	}

	if (struct_link_para == NULL)
	{
		default_link_inputs (clust1, clust2); 
	}
	else
	{
		full_link_inputs (struct_link_para);
	}

	sprintf(string,"XNBC Link Editor : Link Parameters");
	
	XtVaSetValues(XtParent(inputlink_dialog),XmNtitle,string,NULL);


	if(source != clust1)
		source = clust1;
	if(target != clust2)
		target = clust2;
		
	XtManageChild (inputlink_dialog);

/*--- Boucle d'evenements --------------------------------------------------*/

	while (inputlink_action != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent (app_context, XtIMAll);
		XSync (the_display_di, False);
	}

	XtUnmanageChild (inputlink_dialog);
	XSync (the_display_di, False);

/*--- Sortie ---------------------------------------------------------------*/

	inputlink_action = NO_WAITING_ACTION;

	if (inputlink_answer == CANCEL_ANSWER)
	{
		return (LINK_PARAMETERS *)NULL;
	}
	else if (inputlink_answer == OK_ANSWER)
	{
		return &inputlink_return;
	}
	return (LINK_PARAMETERS *)NULL;
}

/*----------------------------------------------------------------------------
 *	F U L L _ L I N K _ I N P U T S
 *--------------------------------------------------------------------------*/

static	void	full_link_inputs (struct_link_para)
LINK_PARAMETERS *struct_link_para;
{
	char		char_buffer[256];
	XmString	xm_string;

	sprintf (char_buffer, "From nucleus %d to nucleus %d",
	    struct_link_para->cluster1, struct_link_para->cluster2);
	xm_string = XmStringCreateSimple (char_buffer);
	XtVaSetValues (widget_array_di[k_LabelG_ClustToClust_leil],
	    XmNlabelString, xm_string,
	    NULL);
	XmStringFree (xm_string);

	sprintf (char_buffer, "%d", struct_link_para->axone_mean);
	XmTextSetString (widget_array_di[k_TextF_AxoneMean_leil], char_buffer);

	sprintf (char_buffer, "%d", struct_link_para->axone_sd);
	XmTextSetString (widget_array_di[k_TextF_AxoneSd_leil], char_buffer);

	sprintf (char_buffer, "%d", struct_link_para->born_min);
	XmTextSetString (widget_array_di[k_TextF_BornMin_leil], char_buffer);

	sprintf (char_buffer, "%d", struct_link_para->born_max);
	XmTextSetString (widget_array_di[k_TextF_BornMax_leil], char_buffer);

	sprintf (char_buffer, "%f", struct_link_para->K_connections);
	XmTextSetString (widget_array_di[k_textF_K_connections], char_buffer);

        XtSetSensitive(widget_array_di[k_toggleB_full_connections], False);
        XtSetSensitive(widget_array_di[k_toggleB_send], False);
        XtSetSensitive(widget_array_di[k_toggleB_receive], False);
        XtSetSensitive(widget_array_di[k_toggleB_average], False);
	switch (struct_link_para->deconnect)
	{
		case _PART_SEND_:
		        XtSetSensitive(widget_array_di[k_toggleB_send], True);
			XmToggleButtonSetState (
			    widget_array_di[k_toggleB_send], True, True);
			break;
		case _PART_RECEIVE_:
		        XtSetSensitive(widget_array_di[k_toggleB_receive], True);
			XmToggleButtonSetState (
			    widget_array_di[k_toggleB_receive], True, True);
			break;
		case _PART_AVERAGE_:
		        XtSetSensitive(widget_array_di[k_toggleB_average], True);
			XmToggleButtonSetState (
			    widget_array_di[k_toggleB_average], True, True);
			break;
		case _FULLY_CONNECTED_:
	        default:
		        XtSetSensitive(widget_array_di[k_toggleB_full_connections], True);
			XmToggleButtonSetState (
			    widget_array_di[k_toggleB_full_connections], True, True);
			break;
	}

	switch (struct_link_para->inter_cluster)
	{
		case '+':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Excit_leil], True, True);
			break;
		case '-':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Inhib_leil], True, True);
			break;
		case 'r':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Random_leil], True, True);
			break;
		case 'm':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Matrix_manual], True, True);
			break;

		default:
			break;
	}
	inputlink_return.cluster1 = struct_link_para->cluster1;
	inputlink_return.cluster2 = struct_link_para->cluster2;
/*
	XtVaSetValues(widget_array_di[k_toggleB_send],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_receive],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_average],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_full_connections],XmNsensitive,True,NULL);
	XmToggleButtonSetState (widget_array_di[k_toggleB_full_connections], True, True);
*/
}


/*----------------------------------------------------------------------------
 *	D E F A U L T _ L I N K _ I N P U T S
 *--------------------------------------------------------------------------*/

static	void	default_link_inputs (clust1, clust2)
int	clust1, clust2;
{
	char		char_buffer[256];
	XmString	xm_string;


	sprintf (char_buffer, "From nucleus %d to nucleus %d", clust1, clust2);
	xm_string = XmStringCreateSimple(char_buffer);
	XtVaSetValues (widget_array_di[k_LabelG_ClustToClust_leil],
	    XmNlabelString, xm_string,
	    NULL);
	XmStringFree (xm_string);

	sprintf (char_buffer, "1");
	XmTextSetString (widget_array_di[k_TextF_AxoneMean_leil], char_buffer);
	XmTextSetString (widget_array_di[k_TextF_BornMin_leil], char_buffer);
	XmTextSetString (widget_array_di[k_TextF_BornMax_leil], char_buffer);

	sprintf (char_buffer, "0");
	XmTextSetString (widget_array_di[k_TextF_AxoneSd_leil], char_buffer);
	sprintf (char_buffer, "0.0");
	XmTextSetString (widget_array_di[k_textF_K_connections], char_buffer);
	XtVaSetValues(widget_array_di[k_toggleB_send],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_receive],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_average],XmNsensitive,False,NULL);
	XtVaSetValues(widget_array_di[k_toggleB_full_connections],XmNsensitive,True,NULL);

	/*
	XmToggleButtonSetState (widget_array_di[k_ToggleBG_Excit_leil], True, True);
	XmToggleButtonSetState (widget_array_di[k_toggleB_full_connections], True, True);
	*/
	inputlink_return.cluster1 = clust1;
	inputlink_return.cluster2 = clust2;
}

/*----------------------------------------------------------------------------
 *	V E R I F Y _ L I N K _ I N P U T S
 *--------------------------------------------------------------------------*/

static	int	verify_link_inputs ()
{
	float	float_buffer = 0.0;
	char	*char_pointer;
	int	int_buffer = 0;
	int	yes_or_no;
	static	int	axone_mean_accepted;
	static	int	axone_sd_accepted;
	static	int	born_min_accepted;
	static	int	born_max_accepted;
	static	int	axone_mean_is_accepted = FALSE;
	static	int	axone_sd_is_accepted = FALSE;
	static	int	born_min_is_accepted = FALSE;
	static	int	born_max_is_accepted = FALSE;
	int	born_min_buffer;
	char	char_buffer[100];

/*--- Verifie la valeur d'axone mean ---------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_AxoneMean_leil]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_AxoneMean_leil],
	    char_buffer);

	if (int_buffer <= 0)
	{
		sprintf (char_buffer,
		    "Axone mean = %d is an impossible value.\nPlease input a value > 0",
		    int_buffer);
		display_error (inputlink_dialog, char_buffer);
		return FALSE;
	}
	else if (int_buffer > 1000 &&
	    !(axone_mean_is_accepted && axone_mean_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "Axone mean = %d is a great value.\nAre you sure ?",
		    int_buffer);
		yes_or_no = display_question (inputlink_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			axone_mean_accepted = int_buffer;
			axone_mean_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputlink_return.axone_mean = int_buffer;

/*--- Verifie la valeur d'axone sd -----------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_AxoneSd_leil]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_AxoneSd_leil],
	    char_buffer);

	if (int_buffer < 0)
	{		
		sprintf (char_buffer,
		    "Axone sd. = %d is an impossible value.\nPlease input a value >= 0.",
		    int_buffer);
		display_error (inputlink_dialog, char_buffer);
		return	FALSE;
	}
	else if (int_buffer > 1000 && 
	    !(axone_sd_is_accepted && axone_sd_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "Axone sd. = %d is a great value.\nAre you sure ?",
		    int_buffer);
		yes_or_no = display_question (inputlink_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			axone_sd_accepted = int_buffer;
			axone_sd_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputlink_return.axone_sd = int_buffer;

/*--- Verifie la valeur de born min ----------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_BornMin_leil]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_BornMin_leil],
	    char_buffer);

	if (int_buffer < 0)
	{		
		sprintf (char_buffer,
		    "Born min. = %d is an impossible value.\nPlease input a value >= 0.",
		    int_buffer);
		display_error (inputlink_dialog, char_buffer);
		return FALSE;
	}
	else if (int_buffer > 1000 &&
	    !(born_min_is_accepted && born_min_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "Born min. = %d is a great value.\nAre you sure ?",
		    int_buffer);
		yes_or_no = display_question (inputlink_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			born_min_accepted = int_buffer;
			born_min_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}
	born_min_buffer = int_buffer;

	inputlink_return.born_min = int_buffer;

/*--- Verifie la valeur de born max ----------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_BornMax_leil]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_BornMax_leil],
	    char_buffer);

	if (int_buffer <= 0)
	{		
		sprintf (char_buffer,
		    "Born max. = %d is an impossible value.\nPlease input a value > 0",
		    int_buffer);
		display_error (inputlink_dialog, char_buffer);
		return FALSE;
	}
	else if (int_buffer < born_min_buffer)
	{
		display_error (inputlink_dialog, 
		    "Born max. < born min. is impossible.\nPlease input another value");
		return FALSE;
	}
	else if (int_buffer > 1000 &&
	    !(born_max_is_accepted && born_max_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "Born max. = %d is a great value.\nAre you sure ?",
		    int_buffer);
		yes_or_no = display_question (inputlink_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			born_max_accepted = int_buffer;
			born_max_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputlink_return.born_max = int_buffer;

/*--- Verifie la valeur de K_connections ----------------------------------------*/

	if (XmToggleButtonGetState (widget_array_di[k_toggleB_full_connections]))
	{
		inputlink_return.deconnect = _FULLY_CONNECTED_;
	}
	if (XmToggleButtonGetState (widget_array_di[k_toggleB_send]))
	{
		inputlink_return.deconnect = _PART_SEND_;
	}
	else if (XmToggleButtonGetState (widget_array_di[k_toggleB_receive]))
	{
		inputlink_return.deconnect = _PART_RECEIVE_;
	}
	else if (XmToggleButtonGetState (widget_array_di[k_toggleB_average]))
	{
		inputlink_return.deconnect = _PART_AVERAGE_;
	}

	char_pointer = XmTextGetString (
	    widget_array_di[k_textF_K_connections]);
	sscanf(char_pointer,"%f",&float_buffer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%f", float_buffer);
	XmTextSetString (widget_array_di[k_textF_K_connections],
	    char_buffer);


	if (float_buffer == 0.0)
	{		
		XtVaSetValues(widget_array_di[k_toggleB_send],XmNsensitive,False,NULL);
		XtVaSetValues(widget_array_di[k_toggleB_receive],XmNsensitive,False,NULL);
		XtVaSetValues(widget_array_di[k_toggleB_average],XmNsensitive,False,NULL);
		XtVaSetValues(widget_array_di[k_toggleB_full_connections],XmNsensitive,True,NULL);
	}
	else if (float_buffer < 0.0)
	{		
		sprintf (char_buffer,
		    "K connections = %f is an impossible value.\nPlease input a value >= 0",
		    float_buffer);
		display_error (inputlink_dialog, char_buffer);
		return FALSE;
	}
	else if (float_buffer > (float)clust_wid[inputlink_return.cluster2].nb_neurons)
	{
		display_error (inputlink_dialog, 
		    "Connection number > neuron number is impossible.\nPlease input another value");
		float_buffer = 0.;
		return FALSE;
	}

	if (float_buffer > 0.0)
	{
		XtVaSetValues(widget_array_di[k_toggleB_send],XmNsensitive,True,NULL);
		XtVaSetValues(widget_array_di[k_toggleB_receive],XmNsensitive,True,NULL);
		if (float_buffer > 1.) /* Sinon, division par 0.! */
		  XtVaSetValues(widget_array_di[k_toggleB_average],XmNsensitive,True,NULL);
		else 
		  XtVaSetValues(widget_array_di[k_toggleB_average],XmNsensitive,False,NULL);
		XtVaSetValues(widget_array_di[k_toggleB_full_connections],XmNsensitive,True,NULL);

		switch (inputlink_return.deconnect)
		{
			case _PART_SEND_:
				XtSetSensitive(widget_array_di[k_toggleB_send], True);
				XmToggleButtonSetState (
				    widget_array_di[k_toggleB_send], True, True);
				break;
			case _PART_RECEIVE_:
				XtSetSensitive(widget_array_di[k_toggleB_receive], True);
				XmToggleButtonSetState (
				    widget_array_di[k_toggleB_receive], True, True);
				break;
			case _PART_AVERAGE_:
				XtSetSensitive(widget_array_di[k_toggleB_average], True);
				XmToggleButtonSetState (
				    widget_array_di[k_toggleB_average], True, True);
				break;
			case _FULLY_CONNECTED_:
				XtSetSensitive(widget_array_di[k_toggleB_full_connections], True);
				XmToggleButtonSetState (
				    widget_array_di[k_toggleB_full_connections], True, True);
				break;
		}
		/*
		XmToggleButtonSetState (widget_array_di[k_toggleB_full_connections], True, True);
		 */
	} /* fin du if */

	inputlink_return.K_connections = float_buffer;

/*--- Tout est bon. On reinitialise tout et on ressort ---------------------*/

	if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Excit_leil]))
	{
		inputlink_return.inter_cluster = '+';
	}
	else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Inhib_leil]))
	{
		inputlink_return.inter_cluster = '-';
	}
	else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Random_leil]))
	{
		inputlink_return.inter_cluster = 'r';
	}
	else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Matrix_manual]))
	{
		inputlink_return.inter_cluster = 'm';
	}
	
	axone_mean_is_accepted = FALSE;
	axone_sd_is_accepted = FALSE;
	born_min_is_accepted = FALSE;
	born_max_is_accepted = FALSE;
	
	born_min_buffer = 0;

#if DEBUG
	printf("deconnect: %d\n",inputlink_return.deconnect);
	printf("K_connections: %f\n",inputlink_return.K_connections);
	printf("Borne max: %d\n",inputlink_return.born_max);
	printf("Borne min: %d\n",inputlink_return.born_min);
	printf("Inter_nucleus: %c\n",inputlink_return.inter_cluster);
#endif

	return TRUE;
}


/*----------------------------------------------------------------------------
 *	D E C O N N E C T
 *--------------------------------------------------------------------------*/
	/* modifie la matrice de connexions selon le type de connectivite:
						1 : emet K,
						2 : recoit K,
						3 : P(wij = 1) = K / (N - 1)
						(N = nombre de neurones)
	*/

void deconnect(int type_K)
{
	NEURON_CONNECTIONS	*connect = (NEURON_CONNECTIONS *)NULL;
	time_t	debut;

	float	fk,
		K,
		x;

	int	compt,
	        auto_cnx,
		N, N_1, N_2,
		i,
		j,
		enough,
		k;

	int 		Cle, Compt, Type, OnOff, Etat, no_auto;

	time(&debut);
	srand(debut);

	no_auto = FALSE;
	K = inputlink_return.K_connections;
	N_1 = clust_wid[inputlink_return.cluster1].nb_neurons;
	N_2 = clust_wid[inputlink_return.cluster2].nb_neurons;

	if (inputlink_return.cluster1 == inputlink_return.cluster2)
	{
	  auto_cnx = True;
	  no_auto = clust_wid[inputlink_return.cluster1].no_auto;
	}
	else
	{
	   auto_cnx = False;
	}
#if DEBUG
	printf("K: %d \nCluster 1: %d: N_1: %d\n", K, inputlink_return.cluster1, N_1);
	printf("Cluster 2: %d: N_2: %d\n", inputlink_return.cluster2, N_2);
#endif

	N = (N_1 >= N_2)? N_1: N_2;

	if ( !(connect = (NEURON_CONNECTIONS *) calloc (N,
	    sizeof (NEURON_CONNECTIONS))) )
	{
		display_message (inputlink_dialog, "Not enough memory. Try later.");
		return;
	}
		
	for (i = 0; i < N; i++)
	{
		if ( !(connect[i].synaptic_knobs = (short *) calloc (N,
		    sizeof (short))) )
		{
			display_message (inputlink_dialog, "Not enough memory. Try later.");
			return;
		}
	}


	Cle = Create_Cle(inputlink_return.cluster1,inputlink_return.cluster2);				
	Find_noeud = search_listeD(Cle, &LIENS);

	/* On part d'une matrice a 0 */

	switch(type_K)
	{
		case _PART_SEND_:
			for (i = 0; i < N_1; i++)
				for (j = 0; j < N_2; j++)
					(connect[i].synaptic_knobs)[j] = 0;
			for (i = 0; i < N_1 ; i++)
			{
				compt = 0;
				while(compt != (int)K)
				{
					fk = (float) rand() / (float) RAND_MAX;
					fk *= (float) N_2;
					k = (int) fk;
					if (k == N_2)
						k = N_2 - 1;
					if (((connect[i].synaptic_knobs)[k] == 0) && (k != i))
					{
						(connect[i].synaptic_knobs)[k] = 1;
						compt++;
					}
				}
			}
			/* on enleve les autoconnexions si necessaire */
			if (auto_cnx)
			{
			  for (i = 0; i < N_1; i++)
			    for (j = 0; j < N_2; j++)
			      if ((i == j) && no_auto)
				(connect[i].synaptic_knobs)[j] = 0;
			}			
			/*
			 * Mise a jour de Find_Noeud et Liens
			 */
			Compt = 0;
			for (j = 0; j < N_1; j++)
			{
				for (k = 0; k < N_2; k++)
				{
					if (Find_noeud != NULL)
					{
						Find_noeud->elem.Matrix[Compt].OnOff = (connect[j].synaptic_knobs)[k];
					}
					Compt++;
				}
			}
			break;
			
		case _PART_RECEIVE_:
			for (i = 0; i < N_2; i++)
				for (j = 0; j < N_1; j++)
					(connect[i].synaptic_knobs)[j] = 0;
			for (i = 0; i < N_2; i++)
			{
				compt = 0;
				while(compt != (int)K)
				{
					fk = (float) rand() / (float) RAND_MAX;
					fk *= (float) N_1;
					k = (int) fk;
					if (k == N_1)
						k = N_1 - 1;
					if (((connect[k].synaptic_knobs)[i] == 0) && (k != i))
					{
						(connect[k].synaptic_knobs)[i] = 1;
						compt++;
					}
				}
			}
			/* on enleve les autoconnexions si necessaire */
			if (auto_cnx)
			{
			  for (i = 0; i < N_1; i++)
			    for (j = 0; j < N_2; j++)
			      if ((i == j) && no_auto)
				(connect[i].synaptic_knobs)[j] = 0;
			}
			/*
			 * Mise a jour de Find_Noeud et Liens
			 */
			Compt = 0;
			for (j = 0; j < N_2; j++)
			{
				for (k = 0; k < N_1; k++)
				{
					if (Find_noeud != NULL)
					{
						Find_noeud->elem.Matrix[Compt].OnOff = (connect[j].synaptic_knobs)[k];
					}
					Compt++;
				}
			}
			break;
			
		case _PART_AVERAGE_:
			x = K / (float) (N_2 - 1);		
			for (i = 0; i < N_1; i++)
				for (j = 0; j < N_2; j++)
				{
					fk = (float) rand() / (float) RAND_MAX;
					if (fk <= x)
						(connect[i].synaptic_knobs)[j] = 1;
				}
			/* on enleve les autoconnexions si necessaire */
			if (auto_cnx)
			{
			  for (i = 0; i < N_1; i++)
			    for (j = 0; j < N_2; j++)
			      if ((i == j) && no_auto)
				(connect[i].synaptic_knobs)[j] = 0;
			}
			/*
			 * Mise a jour de Find_Noeud et Liens
			 */
			Compt = 0;
			for (j = 0; j < N_1; j++)
			{
				for (k = 0; k < N_2; k++)
				{
					if (Find_noeud != NULL)
					{
						Find_noeud->elem.Matrix[Compt].OnOff = (connect[j].synaptic_knobs)[k];
					}
					Compt++;
				}
			}
			break;
		case _FULLY_CONNECTED_:
			for (i = 0; i < N_1; i++)
				for (j = 0; j < N_2; j++)
				{
				       (connect[i].synaptic_knobs)[j] = 1;
				}
			/* on enleve les autoconnexions si necessaire */
			if (auto_cnx)
			{
			  for (i = 0; i < N_1; i++)
			    for (j = 0; j < N_2; j++)
			      if ((i == j) && no_auto)
				(connect[i].synaptic_knobs)[j] = 0;
			}
			/*
			 * Mise a jour de Find_Noeud et Liens
			 */
			Compt = 0;
			for (j = 0; j < N_1; j++)
			{
				for (k = 0; k < N_2; k++)
				{
					if (Find_noeud != NULL)
					{
						Find_noeud->elem.Matrix[Compt].OnOff = (connect[j].synaptic_knobs)[k];
#if DEBUG
	      printf("Liaison %d -> %d: neuron %d -> %d : %d (Compt= %d)\n",
		     inputlink_return.cluster1,	     
		     inputlink_return.cluster2,
		     j, k,  Find_noeud->elem.Matrix[Compt].OnOff, Compt);
#endif
					}
					Compt++;
				}
			}
			break;
			
		default:
			break;
			
	}
#if DEBUG
	Compt = 0;
	for (j = 0; j < N_1; j++)
	{
	  for (k = 0; k < N_2; k++)
	  {
	    if (Find_noeud != NULL)
	    {
	      Find_noeud->elem.Matrix[Compt].OnOff = (connect[j].synaptic_knobs)[k];
#if DEBUG
	      printf("Liaison %d -> %d: neuron %d -> %d : %d\n",
		     inputlink_return.cluster1,	     
		     inputlink_return.cluster2,
		     j, k,  Find_noeud->elem.Matrix[Compt].OnOff);
#endif
	    }
	    Compt++;
	  }
	}
#endif
} /* fin du deconnect */

/****************************************************************************
 *									    *
 *	I N P U T   C L U S T E R					    *
 *									    *
 ****************************************************************************/

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ I N P U T C L U S T E R
 *--------------------------------------------------------------------------*/

CLUSTER_WIDGET	*display_inputcluster (struct_cluster)
CLUSTER_WIDGET  *struct_cluster;
{
	char	*char_pointer;

/*--- Remplissage de la fenetre --------------------------------------------*/

	if (XtIsManaged (inputcluster_dialog))
	{
		XtUnmanageChild (inputcluster_dialog);
	}

	if (struct_cluster == NULL)
	{
		default_cluster_inputs ();
	}
	else
	{
		full_cluster_inputs (struct_cluster);
	}

	char_pointer = XmTextGetString (widget_array_di[k_TextF_ClusterName_leic]);
	strcpy(clust_name_buffer ,char_pointer);
	XtFree(char_pointer);

/*--- Boucle d'evenements --------------------------------------------------*/

	XtManageChild (inputcluster_dialog);

	while (inputcluster_action != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent (app_context, XtIMAll);
		XSync (the_display_di, False);
	}

	XtUnmanageChild (inputcluster_dialog);
	XSync (the_display_di, False);

/*--- Sortie ---------------------------------------------------------------*/

	inputcluster_action = NO_WAITING_ACTION;

	if (inputcluster_answer == CANCEL_ANSWER)
	{
		return (CLUSTER_WIDGET *)NULL;
	}
	else if (inputcluster_answer == OK_ANSWER)
	{
		return &inputcluster_return;
	}
	return (CLUSTER_WIDGET *)NULL;
}

/*----------------------------------------------------------------------------
 *	S E L E C T _ N E U R O N _ F I L E
 *--------------------------------------------------------------------------*/

void	select_neuron_file (Widget w, XtPointer tag, XmAnyCallbackStruct *reason)
/*
Widget	w;
char	*tag;
XmAnyCallbackStruct	*reason;
*/
{
	char		*char_pointer;
	int message=(int)(tag);
	XmString	xm_string;
	XmTextPosition	position;
	char mask[50];

	switch(message)
	{
#if ADD_NEW_MODEL
/*************** TO ADD A MODEL COPY THIS AND ADAPT *************/
		case k_PushB_X_unit:
			strcpy(mask,"*.X_unit");
			break;
/************************ STOP **********************************/
#endif
		case k_PushB_L_unit:
			strcpy(mask,"*.L_unit");
			break;
		case k_PushB_G_unit:
			strcpy(mask,"*.G_unit");
			break;
		case k_PushB_P_unit:
			strcpy(mask,"*.P_unit");
			break;
		case k_PushB_B_unit:
			strcpy(mask,"*.B_unit");
			break;
		case k_PushB_virtual:
			strcpy(mask,"*.tms");
			break;
		default:
			strcpy(mask,"*.*");
			break;
	}

	char_pointer = display_fileselect (inputcluster_dialog, 4, mask,
	    "Select the neuron file :", NULL);

	if (char_pointer)
	{
		XmTextSetString (widget_array_di[k_TextF_NeuronFile_leic],
		    char_pointer);
		position = XmTextGetLastPosition (
		    widget_array_di[k_TextF_NeuronFile_leic]);
		XmTextSetInsertionPosition (
		    widget_array_di[k_TextF_NeuronFile_leic], position);
	}
}

/*----------------------------------------------------------------------------
 *	F U L L _ C L U S T E R _ I N P U T S
 *--------------------------------------------------------------------------*/

static	void	full_cluster_inputs (struct_cluster)
CLUSTER_WIDGET	*struct_cluster;
{
	char		char_buffer[256];
	XmTextPosition	position;

	XmTextSetString (widget_array_di[k_TextF_ClusterName_leic],
	    struct_cluster->ch_name);

	XmTextSetString (widget_array_di[k_TextF_NeuronFile_leic],
	    struct_cluster->neuron_file);
	position = XmTextGetLastPosition (
	    widget_array_di[k_TextF_NeuronFile_leic]);
	XmTextSetInsertionPosition (
	    widget_array_di[k_TextF_NeuronFile_leic], position);

	sprintf (char_buffer, "%d", struct_cluster->nb_neurons);
	XmTextSetString (widget_array_di[k_TextF_NbNeurons_leic], char_buffer);
	
	sprintf (char_buffer, "%d", struct_cluster->position_H);
	XmTextSetString (widget_array_di[k_TextF_PositionH_leic], char_buffer);

	sprintf (char_buffer, "%d", struct_cluster->position_L);
	XmTextSetString (widget_array_di[k_TextF_PositionL_leic], char_buffer);

	sprintf (char_buffer, "%d", struct_cluster->position_AP);
	XmTextSetString (widget_array_di[k_TextF_PositionAP_leic], char_buffer);

	sprintf (char_buffer, "%d", struct_cluster->delta_H);
	XmTextSetString (widget_array_di[k_TextF_DeltaH_leic], char_buffer);

	sprintf (char_buffer, "%d", struct_cluster->delta_L);
	XmTextSetString (widget_array_di[k_TextF_DeltaL_leic], char_buffer);

	sprintf (char_buffer, "%d", struct_cluster->delta_AP);
	XmTextSetString (widget_array_di[k_TextF_DeltaAP_leic], char_buffer);

	XmToggleButtonSetState(widget_array_di[k_toggleB_no_auto], (!struct_cluster->no_auto), True);
	
	switch (struct_cluster->shape)
	{
		case 'e':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Ellipsoid_leic], True, True);
			break;
		case 'b':
			XmToggleButtonSetState (
			    widget_array_di[k_ToggleBG_Brick_leic], True, True);
			break;
		default:
			break;
	}
}

/*----------------------------------------------------------------------------
 *	D E F A U L T _ C L U S T E R _ I N P U T S
 *--------------------------------------------------------------------------*/

static	void	default_cluster_inputs ()
{
	char		char_buffer[256];
	int		i = 0, found = FALSE;

	while (!found)
	{
		i++;
		sprintf (char_buffer, "Name_%d", i);
		found = !search_clust_wid (char_buffer);
	}

	XmTextSetString (widget_array_di[k_TextF_ClusterName_leic], char_buffer);

#ifdef	UNIX
	XmTextSetString (widget_array_di[k_TextF_NeuronFile_leic], "./default.P_unit");
#elif	VMS
	XmTextSetString (widget_array_di[k_TextF_NeuronFile_leic], "[]default.unit");
#endif

	sprintf (char_buffer, "1");
	XmTextSetString (widget_array_di[k_TextF_NbNeurons_leic], char_buffer);

	sprintf (char_buffer, "0");
	XmTextSetString (widget_array_di[k_TextF_PositionH_leic], char_buffer);

	XmTextSetString (widget_array_di[k_TextF_PositionL_leic], char_buffer);

	XmTextSetString (widget_array_di[k_TextF_PositionAP_leic], char_buffer);

	XmTextSetString (widget_array_di[k_TextF_DeltaH_leic], char_buffer);

	XmTextSetString (widget_array_di[k_TextF_DeltaL_leic], char_buffer);

	XmTextSetString (widget_array_di[k_TextF_DeltaAP_leic], char_buffer);

	XmToggleButtonSetState (widget_array_di[k_ToggleBG_Ellipsoid_leic], True, True);

	XmToggleButtonSetState (widget_array_di[k_toggleB_no_auto], True, True);
}

/*----------------------------------------------------------------------------
 *	V E R I F Y _ C L U S T E R _ I N P U T S
 *--------------------------------------------------------------------------*/

static	int	verify_cluster_inputs ()
{
	char		char_buffer[512];
	char		*char_pointer;
	XmString	xm_string;
	int		int_buffer = 0;
	int		yes_or_no, i, k;
	int             numero;
	int             current_number;

	static	int	H_accepted;
	static	int	L_accepted;
	static	int	AP_accepted;
	static	int	dH_accepted;
	static	int	dL_accepted;
	static	int	dAP_accepted;
	static  int	H_is_accepted = FALSE;
	static  int	L_is_accepted = FALSE;
	static  int	AP_is_accepted = FALSE;
	static  int	dH_is_accepted = FALSE;
	static  int	dL_is_accepted = FALSE;
	static  int	dAP_is_accepted = FALSE;

/*--- Cluster Name ---------------------------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_ClusterName_leic]);

	sprintf (char_buffer, "%.10s", char_pointer);
	XtFree (char_pointer);
	for (i = 0; i <= 10; i++)
	{
		if (char_buffer[i] == ' ') char_buffer[i] = '\0';
	}

	numero = search_clust_wid (char_buffer);

	XmTextSetString (widget_array_di[k_TextF_ClusterName_leic],
	    char_buffer);

	if (numero && strcmp (clust_name_buffer, char_buffer))
	{
		sprintf (char_buffer,
		    "The cluster %s already exists.\nTry another one.",
		    char_pointer);
		display_error (inputcluster_dialog, char_buffer);
		return FALSE;
	}

	strcpy (inputcluster_return.ch_name, char_buffer);

/*--- Neuron File ----------------------------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_NeuronFile_leic]);
	inputcluster_return.neuron_file = char_pointer;
	

/*--- Number of neurons ----------------------------------------------------*/

	current_number = get_nb_neurons(numero);

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_NbNeurons_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_NbNeurons_leic],
	    char_buffer);

	if (int_buffer <= 0)
	{
		sprintf (char_buffer,
		    "%d is an impossible value.\nInput a positive value.",
		    int_buffer);
		display_error (inputcluster_dialog, char_buffer);
		return FALSE;
	}

	if (current_number && current_number != int_buffer)
	{
		sprintf (char_buffer,
			 "Cluster %d\nNumber of neuron changed from %d to %d\nThis will destroy all links from and to this cluster\nYou will have to redefine them!\nAre you sure?\n",
			 numero, current_number,
			 int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
		    for (k = 1; k <= max_link_para; k++)
		    {
#if DEBUG
		      printf("cluster[%d] (%s): cluster1: %d et cluster2: %d\n",
			     k, clust_wid[k].ch_name, link_para[k].cluster1, link_para[k].cluster2);
#endif
		          if (link_para[k].cluster1 == numero  
			      || link_para[k].cluster2 ==  numero)
			  {
#if DEBUG
			    printf("\tDeleting %d to %d\n",
				   link_para[k].cluster1, link_para[k].cluster2);
#endif
			    if (link_para[k].cluster1 && link_para[k].cluster2)
			      delete_link_para_silent(link_para[k].cluster1,link_para[k].cluster2);
			  }
		    }

		}
		else
		{
		  sprintf (char_buffer, "%d", current_number);
		  XmTextSetString (widget_array_di[k_TextF_NbNeurons_leic],
				   char_buffer);
		  int_buffer =  current_number;
		}
	}
	inputcluster_return.nb_neurons = int_buffer;

/*--- Verifie la valeur de la position H -----------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_PositionH_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_PositionH_leic],
	    char_buffer);

	if ( (int_buffer >= 100 || int_buffer <= -100)   &&
	    !(H_is_accepted && H_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "H = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			H_accepted = int_buffer;
			H_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.position_H = int_buffer;

/*--- Verifie la valeur de dH ----------------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_DeltaH_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_DeltaH_leic],
	    char_buffer);

	if (int_buffer < 0)
	{
		sprintf (char_buffer,
		    "%d is an impossible value.\nInput a positive value.",
		    int_buffer);
		display_error (inputcluster_dialog, char_buffer);
		return FALSE;
	}
	else if ( (int_buffer >= 100 ) &&
	    !(dH_is_accepted && dH_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "dH = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			dH_accepted = int_buffer;
			dH_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.delta_H = int_buffer;

/*--- Verifie la valeur de la position L -----------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_PositionL_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_PositionL_leic],
	    char_buffer);

	if ( (int_buffer >= 100 || int_buffer <= -100)   &&
	    !(L_is_accepted && L_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "L = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			L_accepted = int_buffer;
			L_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.position_L = int_buffer;

/*--- Verifie la valeur de dL ----------------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_DeltaL_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_DeltaL_leic],
	    char_buffer);

	if (int_buffer < 0)
	{
		sprintf (char_buffer,
		    "%d is an impossible value.\nInput a positive value.",
		    int_buffer);
		display_error (inputcluster_dialog, char_buffer);
		return FALSE;
	}
	else if ( (int_buffer >= 100 ) &&
	    !(dL_is_accepted && dL_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "dL = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			dL_accepted = int_buffer;
			dL_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.delta_L = int_buffer;

/*--- Verifie la valeur de la position AP ----------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_PositionAP_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_PositionAP_leic],
	    char_buffer);

	if ( (int_buffer >= 100 || int_buffer <= -100)   &&
	    !(AP_is_accepted && AP_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "AP = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			AP_accepted = int_buffer;
			AP_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.position_AP = int_buffer;

/*--- Verifie la valeur de dAP ----------------------------------------------*/

	char_pointer = XmTextGetString (
	    widget_array_di[k_TextF_DeltaAP_leic]);
	int_buffer = atoi (char_pointer);
	XtFree (char_pointer);

	sprintf (char_buffer, "%d", int_buffer);
	XmTextSetString (widget_array_di[k_TextF_DeltaAP_leic],
	    char_buffer);

	if (int_buffer < 0)
	{
		sprintf (char_buffer,
		    "%d is an impossible value.\nInput a positive value.",
		    int_buffer);
		display_error (inputcluster_dialog, char_buffer);
		return FALSE;
	}
	else if ( (int_buffer >= 100 ) &&
	    !(dAP_is_accepted && dAP_accepted == int_buffer) )
	{
		sprintf (char_buffer,
		    "dAP = %d is a great value.\nAre you sure ?", int_buffer);
		yes_or_no = display_question (inputcluster_dialog, char_buffer);
		if (yes_or_no == YES_ANSWER)
		{
			dAP_accepted = int_buffer;
			dAP_is_accepted = TRUE;
		}
		else if (yes_or_no == NO_ANSWER)
		{
			return FALSE;
		}
	}

	inputcluster_return.delta_AP = int_buffer;

/*--- Tout est bon. On reinitialise tout et on ressort ---------------------*/

	if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Ellipsoid_leic]))
	{
		inputcluster_return.shape = 'e';
	}
	else if (XmToggleButtonGetState (widget_array_di[k_ToggleBG_Brick_leic]))
	{
		inputcluster_return.shape = 'b';
	}

	H_is_accepted = FALSE;
	L_is_accepted = FALSE;
	AP_is_accepted = FALSE;
	dH_is_accepted = FALSE;
	dL_is_accepted = FALSE;
	dAP_is_accepted = FALSE;

	return TRUE;
}
