#define DEBUG 0

#define _MEMSPY_ 0

/*----------------------------------------------------------------------------
 *	M A T R I X . C : Programme pour Connecter les neurones de 2 Clusters
 *--------------------------------------------------------------------------*/

/*
*
*	Author: Yann HERRY
*		Laboratoire de Biomathematiques
*		URBB B3E INSERM U263
*		Faculte de Medecine Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert, Mai 1993 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <Xm/Label.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/BulletinB.h>
#include <Xm/FileSB.h>
#include <Mrm/MrmAppl.h>


#include "le_vms_or_unix.h"
#include "le_messbox.h"
#include "put_extension.h"
#include "le_strdup.h"
#include "le_avl.h"
#include "le_typedefs.h"

#if _MEMSPY_
#include "/u/sa2/boussard/MEMORY/memspy.h"
#endif

static int	TMINCASE  = 16 ;/* Taille minimale d'une case */ 

#define 	OFF 		0
#define 	ON  		1


int test_alloc;

/*---- Pour le widget enregistrant les motifs pour la gestion des connexions --*/
#define 	STOP     	0
#define 	RECORD 		1
#define 	PLACE    	2
#define		REPEAT		3
#define		CLEAR		4
	/* pour afficher les coordonnees de la liaison */
#define 	k_Label_Matrix_HoLine 47
	/* dfn dans dialog_inputs.h */
#define		k_label_current_mode	48  
#define		k_ToggleBG_Matrix_nocon 49
#define		k_Window_Grid		33

/*---- Nombre de cases pour l'enregistrement d'un motif ---*/
#define NBCASE 400


typedef struct Coord
{
	int x, y;
} COORD;

typedef struct enreg
{
	MAT Case;
	COORD Pos;	
} ENREG;

/*----Prototype des fonctions ----*/

static void Affiche_Cases(Widget);

static void ChangeMatrice( Widget, int, int, int, int); 

static void Draw_Libel(Widget, int, int, int); 

static int TailleCase(double, double);

static int mon_min(double, double);

static void draw(Widget, int, int, int, int);


/*---- Variable Globale ----------*/

ENREG Anim[NBCASE];
Pixmap DrawPixmap;
extern Cursor working_cursor;
extern ARBRE_AVL  LIENS;      /* ARBRE CONTENANT TOUS LES LIENS */

extern struct NOEUD *Find_noeud;     /* NOEUD DE TRAVAIL COURANT       */
extern  CLUSTER_WIDGET  *clust_wid;
extern	int nb_clusters_in_array;

unsigned long	couleur[4];
XGCValues 	val_drawing;
GC		contexte_drawing;
Colormap		colmapid;
XColor 		Colors, RGB;
static int TCase, 
      X1, Y1, X2, Y2, 
     couleur_case = EXCIT, 
     domotif, ind_motif, 
     ORIG, FINAL, NbNeurL, NbNeurH;

static  int	save_V7 = False;

Display		*disp_drawing;	
static Widget 	drawing_area;
static long LDraw_glb;
static long HDraw_glb;
extern  Widget widget_array_di[];
extern int changed;
extern int source;
extern int target;

extern XtAppContext app_context;

#define LE_MAX(a,b) (a < b ? b : a)


int err_alloc_pixmap(Display *disp,XErrorEvent *evt)
{
	
#if DEBUG	
	printf("\n X pixmap allocation error :\n");
#endif
	
	test_alloc = 0;
	return test_alloc;
	
}


/*----------------------------------------------------------------------------
 * Fonction D I S P L A Y _ M A T R I C E : initialise la grille de connexions
 *--------------------------------------------------------------------------*/

int	Display_Matrice (Widget mere, Widget drawing_grid)
{
	long LFen, HFen, LDraw, HDraw;
	int i, n;	
	double LCase, HCase;
	Arg arglist[5];
	int width,height,depth,blanc,noir;
	int max_neuron;
	int x,y;
	int matrix;
	unsigned int largeur,hauteur,bordure,profondeur;
	char string[255];
	Status	status;

 	drawing_area = drawing_grid;	
	
/* # Prendre la taille de la fenetre */

	LFen = 700; HFen = 700; 

/*---- Calcul des parametre pour la fenetre : taille de la Drawing, taille des cases ------*/

	disp_drawing = XtDisplay (drawing_grid);

	/*-- Taille d'une Case ---*/

	LCase = LFen / Find_noeud->elem.NbNeurL; HCase = HFen / Find_noeud->elem.NbNeurH;
	
	max_neuron = LE_MAX(Find_noeud->elem.NbNeurL,Find_noeud->elem.NbNeurH);
	
	
	/*--- Recalcul de la Drawing : si la Drawing est plus grand que la fenetre alors ascenseurs ---*/
	
	sprintf(string,
		"XNBC Link Editor : Connection Matrix from cluster %d to cluster %d",
		source,target);
	
	XtVaSetValues(XtParent(widget_array_di[k_Window_Grid]),XmNtitle,string,NULL);
		
	XtManageChild (widget_array_di[k_Window_Grid]);
	
	colmapid = DefaultColormapOfScreen(DefaultScreenOfDisplay(disp_drawing));
	XAllocNamedColor(disp_drawing, colmapid, "gainsboro", &Colors, &RGB);
      	couleur[NEANT] = Colors.pixel;
      	val_drawing.foreground = couleur[NEANT];
      	val_drawing.background = noir;
    	contexte_drawing = XCreateGC(XtDisplay(drawing_grid),XtWindow(drawing_grid)
     	 	,GCBackground | GCForeground,
				&val_drawing);
	XtVaGetValues(drawing_grid,XmNwidth,&width,XmNheight,&height,NULL);
	XFillRectangle(disp_drawing,XtWindow(drawing_grid),
		contexte_drawing
		,0 , 0 ,width,height);
	
	TCase = TMINCASE;
	if(max_neuron > 50)
		TCase /= 2;
	else if(max_neuron > 100)
		TCase /= 4;
	else if(max_neuron > 300)
		TCase /= 8;
	else if(max_neuron > 500)
		TCase /= 16;

#if DEBUG
	printf("TCase = %d\n",TCase);
	printf("max_neuron = %d\n", max_neuron);
#endif

	test_alloc = 0;
	matrix = 1;
	TCase++;
	
	XSetErrorHandler((XtPointer)(&err_alloc_pixmap));
	XSynchronize(XtDisplay(drawing_grid),1);
	
	while(!test_alloc)
	{
		TCase--;
		if((TCase) && (max_neuron < 1311))
		{
			test_alloc = 1;
		
			LDraw = (Find_noeud->elem.NbNeurL) * TCase;
			HDraw = (Find_noeud->elem.NbNeurH) * TCase;
			
      			/* creation de la pixmap de background */
      				
      			XtVaGetValues(drawing_grid,XmNdepth,&depth,NULL);
      			DrawPixmap = XCreatePixmap(XtDisplay(drawing_grid),
      				XtWindow(drawing_grid),LDraw,HDraw,depth);
      		}
      		else 
      		{
      			test_alloc = 1;
      			matrix = 0;	
      		}
	}

	
	XSetErrorHandler(NULL);
	XSynchronize(XtDisplay(drawing_grid),0);
	
	
	if(!matrix)
	{
		display_message(widget_array_di[k_Window_Grid],
"Sorry, the matrix is too big and can't be edited.\n\
You have to go on without editing it.\n");
		XtUnmanageChild (widget_array_di[k_Window_Grid]);
		return (1);
	}
	
	
	LDraw_glb = LDraw;
	HDraw_glb = HDraw;
	
        /*----------- Reactualise la taille de la Drawing ---------------------------------*/

	n=0;
        XtSetArg(arglist[n], XmNwidth, LDraw); n++;
        XtSetArg(arglist[n], XmNheight, HDraw); n++;
       	XtSetValues(drawing_grid, arglist, n);

      /*------------ Initialise la color map -------------------------------------*
      /* Cherche Bleu = Inhib & Rouge = Excit & Blanc = Rien*/

      status = XAllocNamedColor(disp_drawing, colmapid, "blue", &Colors, &RGB);
      couleur[INHIB] = Colors.pixel;

#if DEBUG
	printf("status for blue = %d\n",status);
#endif

      status = XAllocNamedColor(disp_drawing, colmapid, "red", &Colors, &RGB);
      couleur[EXCIT] = Colors.pixel;

#if DEBUG
	printf("status for red = %d\n",status);
#endif
  
      status = XAllocNamedColor(disp_drawing, colmapid, "black", &Colors, &RGB);
      couleur[BLACK] = Colors.pixel;
#if DEBUG
	printf("status for black = %d\n",status);
#endif
      
      blanc = WhitePixel(XtDisplay(drawing_grid), DefaultScreen(XtDisplay(drawing_grid)) );
      noir =  BlackPixel(XtDisplay(drawing_grid), DefaultScreen(XtDisplay(drawing_grid)) );
	      
      XDefineCursor(XtDisplay(widget_array_di[k_Window_Grid])
		    ,XtWindow(widget_array_di[k_Window_Grid]),
		    working_cursor);

      XSetForeground(disp_drawing, contexte_drawing, couleur[NEANT]);
      XFillRectangle(disp_drawing, DrawPixmap,contexte_drawing ,0 , 0 ,LDraw,HDraw);


      XtVaSetValues(drawing_grid,XmNbackgroundPixmap,DrawPixmap,NULL);

      draw(drawing_grid, 0, 0,
	   Find_noeud->elem.NbNeurL,
	   Find_noeud->elem.NbNeurH);
      
      XDefineCursor(XtDisplay(widget_array_di[k_Window_Grid])
		    ,XtWindow(widget_array_di[k_Window_Grid]),
		    None);
		return (0);
}
/*---------------------------------------------------------------------------
 * Procedure G E S T _ S C R O L L I N G : Rafraichir la fenetre
 *--------------------------------------------------------------------------*/

void gest_scrolling(Widget drawing_grid,int *tag, XmDrawingAreaCallbackStruct *call)
{
	XClearWindow(disp_drawing,XtWindow(drawing_grid));
}
/*---------------------------------------------------------------------------------------------
 * Procedure A F F I C H E _ C A S E S : Construit toutes les cases de la matrice (demarrage) 
 *---------------------------------------------------------------------------------------------*/

static void Affiche_Cases(Widget drawing_grid)
{
	draw(drawing_grid, 0, 0, Find_noeud->elem.NbNeurL, Find_noeud->elem.NbNeurH);
}

/*---------------------------------------------------------------------------
 * Procedure D R A W : Remplir la grille
 *--------------------------------------------------------------------------*/

static void draw(Widget drawing_grid, int X, int Y, int L, int H)
{
	int  i, j, jj, k, OrigX, OrigY, pasi = X, pasj = Y, ifin = (L)*TCase, jfin = (H)*TCase;
	int blanc, noir, Compt, CaseOnOff, Etat;

	XSegment *Segm;
		
	/*-- Boucle d'affichage des CASES  PLEINES ---*/
	for(j=Y; j < H; j++)
	{
	    for(i=X; i < L; i++)
	    {
		if ((i == j) && clust_wid[source].no_auto)
			continue;
			
	    	OrigX = (i)*TCase; OrigY = (j)*TCase;
	
		/*-- Dessine les Cases de Connexion --*/
		if (Find_noeud != NULL)
		{
			Etat = Find_noeud->elem.Matrix[i+ j * Find_noeud->elem.NbNeurL].Etat; 
			CaseOnOff= Find_noeud->elem.Matrix[i+  j* Find_noeud->elem.NbNeurL].OnOff;
			if (CaseOnOff == ON)
			{
#if DEBUG
				printf("Case %d,%d: %d (%d)\n",j,i,Etat,CaseOnOff);
#endif
				XSetForeground(disp_drawing, contexte_drawing, couleur[Etat]);
		       		       
		       		XFillRectangle(disp_drawing, DrawPixmap,
		       		  contexte_drawing ,OrigX , OrigY ,
		       		   TCase , TCase);
			}
		}

	    }
	} /* fin du for */
	/*-- Contexte Graphique ---*/
	XSetForeground(disp_drawing, contexte_drawing, couleur[BLACK]);

	/*-- Dessine la grille de Connexion --*/
	
	if (Find_noeud != NULL)
	{
		if ( ( Segm = (XSegment *)calloc(  (Find_noeud->elem.NbNeurL + 2)* 
				                 (Find_noeud->elem.NbNeurH + 2) , 
				                  sizeof(XSegment))
                                                ) == NULL) exit(1);
		Compt = 0;

		Segm[Compt].x1 = 0; 
		Segm[Compt].y1 = 0;  

		Segm[Compt].x2 = 0; 
		Segm[Compt].y2 = jfin; Compt++;
		
		Segm[Compt].x1 = ifin; 
		Segm[Compt].y1 = 0;  

		Segm[Compt].x2 = ifin; 
		Segm[Compt].y2 = jfin; Compt++;
		
		Segm[Compt].x1 = 0; 
		Segm[Compt].y1 = 0;  

		Segm[Compt].x2 = ifin; 
		Segm[Compt].y2 = 0; Compt++;
		
		Segm[Compt].x1 = 0; 
		Segm[Compt].y1 = jfin;  

		Segm[Compt].x2 = ifin; 
		Segm[Compt].y2 = jfin; Compt++;
		
		XDrawSegments(disp_drawing,DrawPixmap,contexte_drawing,Segm,Compt);
		XFlush(disp_drawing);

		free(Segm);
	}
}
/*---------------------------------------------------------------------------------------
 *          DISPLAY_COORDINATES
  *---------------------------------------------------------------------------------------*/
   
void display_coordinates(Widget w,int widget_num, XEvent *evt)
{
static int lastX=0;
static int lastY=0;
int PosX,PosY,nX,nY,X,Y;
XmString string;
char coordonnees[100];

PosX = evt->xbutton.x; 
PosY = evt->xbutton.y;
nX = PosX / TCase ; 
nY = PosY / TCase ; 

X = (int) (nX)+1;
Y = (int) (nY)+1;
if ((X != lastX) || (Y != lastY))
{
	sprintf(coordonnees,
	"Inter-neuron Link : from neuron %d (cluster %d) to neuron %d (cluster %d)",
		Y,source,X,target);
	string = XmStringCreateLocalized(coordonnees);
	XtVaSetValues(widget_array_di[k_Label_Matrix_HoLine],
				XmNlabelString,string,NULL);
	XmStringFree(string);
	lastX =X;lastY=Y;
}

}

/*---------------------------------------------------------------------------
 * Procedure D R A W _ M A T R I X : CallBack pour cocher une case de la matrice 
 *--------------------------------------------------------------------------*/
void	draw_matrix (Widget drawing_grid, XtPointer tag, XmDrawingAreaCallbackStruct *call)
{

	int LFen, HFen, LDraw, HDraw, PosX, PosY, X, Y, NewX, NewY, Etat = couleur_case;
        	double nX, nY;
 
	int i, j, ClicONOFF;
	
	static int x_ref,y_ref;	/* pour l'enregistrement */
		
	/*-- Gestion du Clic de la Souris pour la selection d'une case --*/

	PosX = call->event->xbutton.x; PosY = call->event->xbutton.y;

	ClicONOFF = call->event->xany.type;

	/*--- Recherche de la case correspondant --*/

	nX = PosX / TCase ; nX = floor(nX); nY = PosY / TCase ; nY = floor(nY);
	X = (int) (nX); Y = (int) (nY);

	if (ClicONOFF == ButtonPress)
	{
		X1 = (int) nX; Y1 = (int) nY;
		
	} 

	if (ClicONOFF == ButtonRelease)
	{
		X2 = (int) nX; Y2 = (int) nY;
	}

	/*--- Test de la selection par la souris ---*/
	switch(domotif)
	{
		case RECORD	:
			if(!ind_motif)
				{
					x_ref=X;
					y_ref=Y;
				}
			if ((Find_noeud !=NULL)&&(couleur_case != NEANT))
			{
				if (   (ind_motif < NBCASE)&&
			        	(X >= 0)&&
			         	(Y >= 0)&&
			         	(X < Find_noeud->elem.NbNeurL + 1)&&
			         	(Y < Find_noeud->elem.NbNeurH + 1)&&
			         	(ClicONOFF == ButtonRelease)
			   	){
					ChangeMatrice(drawing_grid, X, Y, ON, Etat); 
					Anim[ind_motif].Case.OnOff = ON;
					Anim[ind_motif].Case.Etat = couleur_case;
					Anim[ind_motif].Pos.x = X - x_ref;
					Anim[ind_motif].Pos.y = Y - y_ref;
					ind_motif++;
	
				}
			}
			else if((Find_noeud !=NULL)&&(couleur_case == NEANT))
			{
				if (   (ind_motif < NBCASE)&&
			        	(X >= 0)&&
			         	(Y >= 0)&&
			         	(X < Find_noeud->elem.NbNeurL + 1)&&
			         	(Y < Find_noeud->elem.NbNeurH + 1)&&
			         	(ClicONOFF == ButtonRelease)
			   	){
					ChangeMatrice(drawing_grid, X, Y, OFF, Etat); 
					Anim[ind_motif].Case.OnOff = OFF;
					Anim[ind_motif].Case.Etat = couleur_case;
					Anim[ind_motif].Pos.x = X - x_ref;
					Anim[ind_motif].Pos.y = Y - y_ref;
					ind_motif++;
	
				}
			}
			break;
			
		case PLACE	:
			for (i = 0; i < ind_motif; i++)
			{
				NewX = X + (Anim[i].Pos.x ); 
				NewY = Y + (Anim[i].Pos.y  );  
				Etat = Anim[i].Case.Etat;
				if ((NewX >= 0)&&(NewY >= 0)&&(NewX < Find_noeud->elem.NbNeurL + 1)&&(NewY < Find_noeud->elem.NbNeurH + 1)&&(ClicONOFF == ButtonRelease))
					{
					if(Etat != NEANT) ChangeMatrice(drawing_grid, NewX, NewY, ON, Etat);
					else ChangeMatrice(drawing_grid, NewX, NewY, OFF, Etat);
					}
			}
			break;
		case STOP  : 
			
			if(ClicONOFF == ButtonRelease)	
			{
			 for(i=(X2<X1?X2:X1);i<=(X1<X2?X2:X1);i++)
				 {
					 for(j=(Y2<Y1?Y2:Y1);j<=	(Y1<Y2?Y2:Y1);j++)
					 {
						 Find_noeud->elem.Matrix[i + j * Find_noeud->elem.NbNeurL].Etat 
							 = couleur_case;
						 if(couleur_case == NEANT)
							 Find_noeud->elem.Matrix[i + j *
							 Find_noeud->elem.NbNeurL].OnOff = OFF;
						 else
							 Find_noeud->elem.Matrix[i + j *
							 Find_noeud->elem.NbNeurL].OnOff = ON;
					 }	
				 }
			 XSetForeground(disp_drawing, contexte_drawing, couleur[couleur_case]);

			 XFillRectangle(disp_drawing, DrawPixmap, contexte_drawing,
				 (X2<X1?X2:X1)*TCase,(Y2<Y1?Y2:Y1)*TCase,abs(X2-X1)*TCase+TCase,abs(Y2-Y1)*TCase+TCase);
			 XClearArea(disp_drawing,XtWindow(drawing_grid),(X2<X1?X2:X1)*TCase,(Y2<Y1?Y2:Y1)*TCase,
				 abs(X2-X1)*TCase+TCase,abs(Y2-Y1)*TCase+TCase,TRUE);
			 changed = 1;		
			}		  
			break;
			  
		case CLEAR :
			  
			break;
		
		
		default :
			printf("Traitment on motif unrecognized\n");
	}
	
}

/*--------------------------------------------------------------------------------------------------------
 *  Procedure C H A N G E M A T R I C E : Il s'agit de faire un OU-EXCLUSIF sur la case (i,j) selectionne 
 *------------------------------------------------------------------------------------------------------*/

static void ChangeMatrice(Widget drawing_grid,  int i, int j, int ONOFF, int Etat)
{
	int OrigX, OrigY, CaseONOFF;
	int blanc, noir;
	

	/* Gestion des Cases de Connexion */

	/*
	if (domotif != PLACE) CaseONOFF = Find_noeud->elem.Matrix[i + j * Find_noeud->elem.NbNeurL].OnOff ^= ONOFF;
	else CaseONOFF = Find_noeud->elem.Matrix[i + j * Find_noeud->elem.NbNeurL].OnOff = ONOFF;
	*/
	
	CaseONOFF = Find_noeud->elem.Matrix[i + j * Find_noeud->elem.NbNeurL].OnOff = ONOFF;
	
	/* Affichage */

	OrigX = (i)*TCase; OrigY = (j)*TCase;	
	
	
	if (CaseONOFF == ON) 
	{
		Find_noeud->elem.Matrix[i + j * Find_noeud->elem.NbNeurL].Etat = Etat;

		XSetForeground(disp_drawing, contexte_drawing, couleur[Etat]);
	}
	else
		XSetForeground(disp_drawing, contexte_drawing, couleur[NEANT]);

	
	
	XFillRectangle(disp_drawing, DrawPixmap, contexte_drawing,
		OrigX,OrigY,TCase,TCase);
	XClearArea(disp_drawing,XtWindow(drawing_grid),OrigX,OrigY,TCase,TCase,TRUE);
	changed = 1;	
}
/*----------------------------------------------------------------------------
 * Procedure D R A W _ L I B E L : Construit un libelle de la matrice
 *---------------------------------------------------------------------------*/

static void Draw_Libel(Widget drawing_grid, int X, int Y, int indice) 
{
        char mot[100]; int size_mot;
	XGCValues 	val_drawing;

	sprintf(mot,"%d",indice); size_mot = strlen(mot);
	
	XDrawString(disp_drawing,DrawPixmap,contexte_drawing,X,Y,mot,size_mot);	

}
/*----------------------------------------------------------------------------
 * Fonction C H A N G E _ C O U L E U R : enregistre la couleur de la case (excit/inhib) 
 *---------------------------------------------------------------------------*/

void change_couleur(int color)
{
	couleur_case = color;
}
/*----------------------------------------------------------------------------
 * Fonction G E S T _ M O T I F : enregistre le mode de reprodution de Motifs (record, place, stop) 
 *---------------------------------------------------------------------------*/

void gest_motif(int mode)
{
    int X,Y;
    long i, nbcase;
     XmString string;
     char cur_mode[20];
     
     *cur_mode = '\0';

    if (Find_noeud != NULL) nbcase = (Find_noeud->elem.NbNeurL ) * (Find_noeud->elem.NbNeurH);
   switch(mode)
    {
        case RECORD	:
#if DEBUG
	printf("RECORD\n");
#endif
	
	strcpy(cur_mode,"record");
	
	for(i=0;i<ind_motif;i++)
	{
		 
		Anim[i].Case.OnOff = OFF;
		Anim[i].Case.Etat = NEANT;
		Anim[i].Pos.x = 0;
		Anim[i].Pos.y = 0;
	}
			
	ind_motif = 0;
	break;

        case PLACE	:
#if DEBUG
	printf("PLACE\n");
#endif
	
	strcpy(cur_mode,"paste");
	
	break;
     
        case STOP	:

#if DEBUG
	printf("STOP\n");
#endif

	strcpy(cur_mode,"regular");
	
	break;

       
        case CLEAR	:
#if DEBUG
	printf("CLEAR\n");
#endif

	for (i=0; i < nbcase; i++)
	{
		Find_noeud->elem.Matrix[i].OnOff = OFF;
		Find_noeud->elem.Matrix[i].Etat = NEANT;
		
	}
	XSetForeground(disp_drawing, contexte_drawing, couleur[NEANT]);
				
	
	XFillRectangle(disp_drawing, DrawPixmap,
		       contexte_drawing ,0, 0 ,
		       	LDraw_glb , HDraw_glb);
	XClearWindow(disp_drawing,XtWindow(drawing_area));	       
	
	
	break;
    }

    if(mode != CLEAR)
    	{
    		string = XmStringCreateLocalized(cur_mode);
    		XtVaSetValues(widget_array_di[k_label_current_mode],
    		XmNlabelString,string,NULL);
    		XmStringFree(string);
    		domotif = mode;
    	}
}
/*----------------------------------------------------------------------------
 * Procedure N O T I F Y _ M A T R I X : enregistre nb_neurone et init matrix 
 *---------------------------------------------------------------------------*/

void Notify_Matrix(  int orig, int final,  int nb_neur_clust2, int nb_neur_clust1)
{
	ORIG = orig;
	FINAL = final;
	NbNeurL = nb_neur_clust1;
	NbNeurH = nb_neur_clust2;

}
/*----------------------------------------------------------------------------
 * Procedure N O T I F Y _ A V L : enregistre le type de la connexion 
 *---------------------------------------------------------------------------*/

void Notify_AVL(int type)
{
	int ERREUR;
	ERREUR = modif_avl(ORIG, FINAL, NbNeurL, NbNeurH, type);

#if DEBUG
	printf("modif_avl: %d * %d  ERREUR: %d\n", NbNeurL, NbNeurH, ERREUR);
#endif

	switch(ERREUR)
	{
		case NOPROBLEM    	:
/*
			liste(LIENS);
*/
			break;
		case  MEMORY_AVL 	:
			printf("Plus de memoire pour la creation d'un nouveau lien\n");
			break;
		case  MEMORY_MATRIX:
			printf("Plus de memoire pour la creation d'une matrice de connexion\n");
			break;
	}
}
/*----------------------------------------------------------------------------
 * Fonction T A I L L E C A S E : retourne une valeur pour avoir une Case Carre
 *---------------------------------------------------------------------------*/
static int TailleCase(double LCase, double HCase)
{
	if ((LCase < TMINCASE)||(HCase < TMINCASE)) 
	  return(TMINCASE);
	return(mon_min(LCase, HCase));	
}
/*----------------------------------------------------------------------------
 * Fonction M I N : retourne le min de deux valeurs 
 *---------------------------------------------------------------------------*/
static int mon_min(double A, double B)
{
	int Ival; double Dval;

	if (A > B) Dval = floor(B); 
 	else Dval = floor(A);

	Ival =(int) Dval;
	return(Ival);
}
/*----------------------------------------------------------------------------
 * Fonction R E A D _ C N X  : retourne OK ou PROBLEM
 *---------------------------------------------------------------------------*/
int read_cnx(char *cnx_name)
{
	float un_float;

	int *ComptMat;

	short *CONNEXION_MATRIX;
	short *MATRIX_WGT;
	short *MATRIX_LEN;
	short *NB_NEURONS;
	char *len_name, *wgt_name;

	FILE *INPUT_CNX_FILE;
	FILE *INPUT_WGT_FILE;
	FILE *INPUT_LEN_FILE;

	int i, j, k, n, m, Compti, Comptj, ni, nj, lg_axone, poids, clust1, clust2, ind1, ind2, Cle;
	long carre_TOTAL;

	short NB_CLUSTER;
	short TOTAL_NEURONS = 0;
	short un_short;
	short nb_short;
	short neurone_i;
	short neurone_j;


	if (save_V7)
	{
	    if ((INPUT_CNX_FILE = fopen(cnx_name, "r") ) == NULL) return(0);

	    if (fread(&NB_CLUSTER, sizeof(short), 1, INPUT_CNX_FILE) == 0) return(0);

	    NB_NEURONS = (short *)calloc( (size_t)NB_CLUSTER ,  sizeof(short) );

	    ComptMat = (int *)calloc(NB_CLUSTER*NB_CLUSTER, sizeof(int));

	    if (fread(NB_NEURONS, sizeof(short), (size_t)NB_CLUSTER, INPUT_CNX_FILE) == 0) return(0) ;

	    for(i=0; i<NB_CLUSTER; TOTAL_NEURONS += *(NB_NEURONS+(i++)));
		    CONNEXION_MATRIX = (short *)calloc( (size_t)(2*TOTAL_NEURONS*TOTAL_NEURONS), sizeof(short));

	    if (fread(CONNEXION_MATRIX, sizeof(short), (size_t)(2*TOTAL_NEURONS*TOTAL_NEURONS),
		  INPUT_CNX_FILE) == 0) return(0);

	    fclose(INPUT_CNX_FILE);
	}
	else
	{
		wgt_name = put_extension (cnx_name, "wgt");	
		len_name = put_extension (cnx_name, "len");	
/*------------------------------------------------------------------------*
  		The semantic of the len ver2 file is as it follows:  															
				-it's made of decimal short integers (except the length, a float)
				-first token is the number of nucleus in the network      1 short
				-then, for each nucleus,follows its number of neurons     nb_nucl * 1 short
				-at this point starts the connections area:
					for each nucleus,
						for each neuron, are sequentially specified
						- neuron id                                      1 short
							- cluster id                             1 short
							- number of neurons connected to the specified  
								neuron (in fact, the number of connections) nb_con * 1 short
							- for each target neuron is specified:
								-the target id                              1 short
								-the axone length in ms                     1 float
								
*-------------------------------------------------------------------------*/
	    if ((INPUT_LEN_FILE = fopen(len_name, "r") ) == NULL)
	    {
		free(wgt_name);
		free(len_name);
		return(0);
	    }
	    if (fread(&NB_CLUSTER, sizeof(short), 1, INPUT_LEN_FILE) == 0)
	    {
		free(wgt_name);
		free(len_name);
		return(0);
	    }

#if DEBUG
	    printf("%d nucleus\n",NB_CLUSTER);
#endif

	    NB_NEURONS = (short *)calloc( (size_t)NB_CLUSTER ,  sizeof(short) );

	    ComptMat = (int *)calloc(NB_CLUSTER*NB_CLUSTER, sizeof(int));

	    for (i=0; i<NB_CLUSTER; i++)
	    {
	      fread(&NB_NEURONS[i], sizeof(short), 1, INPUT_LEN_FILE) ;
#if DEBUG
	      printf("cluster %d: %d neurons\n",i,NB_NEURONS[i]);
#endif
	    }

	    for(i=0; i<NB_CLUSTER; i++)
	    {
	      TOTAL_NEURONS += NB_NEURONS[i];
	    }
#if DEBUG
	      printf("Total: %d neurons\n",TOTAL_NEURONS);
#endif

	    carre_TOTAL = TOTAL_NEURONS*TOTAL_NEURONS;

	    MATRIX_LEN = (short *)calloc( (size_t)(TOTAL_NEURONS*TOTAL_NEURONS), sizeof(short));

    	    for(i = 0; i < TOTAL_NEURONS; i++)
	    {
	      fread(&neurone_i, sizeof(short), 1, INPUT_LEN_FILE); /* Le neurone i */
	      fread(&un_short, sizeof(short), 1, INPUT_LEN_FILE); /* son cluster (pas utilise) */
	      fread(&nb_short, sizeof(short), 1, INPUT_LEN_FILE); /* le nb de neurones qu'il connecte */
	      for (j = 0; j < nb_short; j++)
	      {    
		fread(&neurone_j, sizeof(short), 1, INPUT_LEN_FILE); /* le neurone j */
		fread(&un_float, sizeof(float), 1, INPUT_LEN_FILE); /* la longueur de la connexion */

		MATRIX_LEN[(neurone_i-1)*TOTAL_NEURONS + (neurone_j-1)] = (short)un_float;
	      }

	    }

	    fclose(INPUT_LEN_FILE);


/*------------------------------------------------------------------------*
  		The semantic of the wgt ver2 file is as it follows:
  															
				-it's made of decimal short integers
				-first token is the number of nucleus in the network
				-then, for each nucleus,follows its number of neurons
				-at this point starts the connections area:
					for each nucleus,
						for each neuron, are sequentially specified
							- neuron id
							- cluster id 
							- number of neurons connected to the specified
								neuron (in fact, the number of connections)
							- for each target neuron is specified:
								-the target id
								-the nmda flag
								-the synaptic weight (nb knobs),that's 
									algebric (can be positive or negative)
								
								
*-------------------------------------------------------------------------*/
	    if ((INPUT_WGT_FILE = fopen(wgt_name, "r") ) == NULL)
	    {
		free(wgt_name);
		free(len_name);
		return(0);
	    }

	    if (fread(&NB_CLUSTER, sizeof(short), 1, INPUT_WGT_FILE) == 0)
	    {
		free(wgt_name);
		free(len_name);
		return(0);
	    }

	    for (i=0; i < NB_CLUSTER; i++)
	    {
	      fread(&un_short, sizeof(short), 1, INPUT_WGT_FILE) ; /* Lecture fictive */
	    }


	    MATRIX_WGT = (short *)calloc( (size_t)(TOTAL_NEURONS*TOTAL_NEURONS), sizeof(short));


	    for (i=0; i < TOTAL_NEURONS; i++)
	    {
		fread(&neurone_i, sizeof(short), 1, INPUT_WGT_FILE); /* Le neurone i */
		fread(&un_short, sizeof(short), 1, INPUT_WGT_FILE); /* son cluster (pas utilise) */
		fread(&nb_short, sizeof(short), 1, INPUT_WGT_FILE); /* le nb de neurones qu'il connecte */
		for (j = 0; j < nb_short; j++)
		{    
		  fread(&neurone_j, sizeof(short), 1, INPUT_WGT_FILE); /* le neurone j */
		  fread(&un_short, sizeof(short), 1, INPUT_WGT_FILE); /* la nmda flag */
		  fread(&un_short, sizeof(short), 1, INPUT_WGT_FILE); /* le poids de la connexion */
		  MATRIX_WGT[(neurone_i-1)*(TOTAL_NEURONS)+ (neurone_j-1)] = un_short;
		}
	    }
#if DEBUG
	    for (i=0; i < TOTAL_NEURONS; i++)
	    {
	      for (j=0; j < TOTAL_NEURONS; j++)
		printf("%3d ",MATRIX_WGT[(i)*(TOTAL_NEURONS)+ (j)]);
	      printf("\n");
	    }
#endif
	    fclose(INPUT_WGT_FILE);

	    free(wgt_name);
	    free(len_name);
	}

#if DEBUG
	printf("Nombre de cluster : %d\n", NB_CLUSTER);

	for(i=0; i<NB_CLUSTER; i++)
		printf("Nombre de neurones pour le cluster %d = %d\n", i, *(NB_NEURONS+i));
	printf("Total de neurones : %d\n", TOTAL_NEURONS);

	printf("Matrice de connexion :\n");
#endif

	Compti = 0; 
	ni = NB_NEURONS[Compti];
	for (i = 0; i < TOTAL_NEURONS; i++)
	{
		if (i == ni) 
		{
			Compti++;
			ni += NB_NEURONS[Compti];
		}
		clust1 = Compti + 1;

		Comptj = 0; 
		nj = NB_NEURONS[Comptj]; 
		j = 0; 
		for(k=0; k < NB_CLUSTER; k++)
		{
			for(n=0; n < NB_NEURONS[k]; n++)
			{
				if (j == nj) 
				{
					Comptj++;
					nj += NB_NEURONS[Comptj];
				}
				clust2 = Comptj + 1;

				if (save_V7)
				{
				  poids = *(CONNEXION_MATRIX+(i*2*TOTAL_NEURONS)+j);
				  lg_axone = *(CONNEXION_MATRIX+(i*2*TOTAL_NEURONS)+j+TOTAL_NEURONS);		
				}
				else
				{
				  poids = MATRIX_WGT[(i)*TOTAL_NEURONS+(j)];
				  lg_axone = MATRIX_LEN[i*TOTAL_NEURONS+j];
				}

				Cle = Create_Cle(clust1, clust2);
				m = ComptMat[Compti + NB_CLUSTER*Comptj];

				if ((Find_noeud = search_listeD(Cle, &LIENS)) != NULL)
				{
					if (poids < 0) Find_noeud->elem.Matrix[m].Etat = INHIB ;
					else Find_noeud->elem.Matrix[m].Etat = EXCIT;			
					if (lg_axone == 0) Find_noeud->elem.Matrix[m].OnOff  = OFF;
					else Find_noeud->elem.Matrix[m].OnOff  = ON;
#if DEBUG
					printf("Cle : %d i : %d j : %d Matrix : %d poids : %d length : %d \n",
					       Cle,  i, j, m, poids, lg_axone);
#endif				

				}
				j++; 
				ComptMat[Compti + NB_CLUSTER*Comptj]++;
			}

		}
	}
	free(ComptMat);
	free(NB_NEURONS);
	if (save_V7)
	  free(CONNEXION_MATRIX);
	else
	{
	  free(MATRIX_LEN);
	  free(MATRIX_WGT);
	}
	return(1);
} 
/*-----------------------------------------------------------------------*/
void Place_Connect(char *Ligne, int Num, char lien)
{

	char *inter;
	int i, Val, Etat;

	/* printf("%s avec %d en %c\n", Ligne, Num, lien); */

	inter = (char *)calloc(200,sizeof(char));

	while(inter != NULL)
	{

		inter = strtok(Ligne,".");

		if (inter != NULL)
		{
			for (i=0; i< strlen(inter)+1; i++) Ligne++;

			Val = atoi(inter);
			if (lien == 'e') Etat = EXCIT; else Etat = INHIB;
			Find_noeud->elem.Matrix[(Num - 1) +  (Val - 1)* Find_noeud->elem.NbNeurL].OnOff = ON;
			Find_noeud->elem.Matrix[(Num - 1) + (Val - 1)* Find_noeud->elem.NbNeurL].Etat = Etat;
		}
	}
	free(inter);
}


/*----------------------------------------------------------------------------
 * Procedure  Lect_Fich_MAT
 *---------------------------------------------------------------------------*/

void Lect_Fich_MAT( char *Nom_Fichier )
{
	FILE *ptfic;
	int i, TailleA, TailleB, indB;

	char *Ligne, lien, Orient;

	Ligne = (char *)calloc(500, sizeof(char));


	ptfic = fopen(Nom_Fichier,"r");

	fscanf(ptfic,"%d %d %c\n",&TailleA, &TailleB, &Orient);


	for (i=1; i <= TailleB; i++)
	{
			fscanf(ptfic, "%s %c %d\n", Ligne, &lien, &indB);
			Place_Connect(Ligne, indB, lien);
			free(Ligne);
			Ligne = (char *)calloc(500, sizeof(char));
	}

	fclose(ptfic);

}

