/*
*       Neuro_Bio_Clusters_Simulator
*
*       CLUSTER.H
*
*       Header file pour le simulateur de reseaux neuronaux a haute
*       plausibilite biologique.
*
*
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not in any way usable for commercial purpose.
*
*/

#ifdef MAIN
#	define CLASS
#else
#	define CLASS	extern
#endif


CLASS
struct	bornes
{
	short	min,
		max;
};

/*
 * structure pour lcontenir les parametres des chgts de
 * longueur des axones
 */
CLASS
struct	chgt	
{
	int	fin;	/* arret du chgt */
	short	old_mean;	/* longeur au debut du chgt */
	float	pente;	/* increment de longuer total / duree de chgt */
} chgt_length[SMALL_NET][SMALL_NET];	/* les chgt de longueur ([SMALL_NET][SMALL_NET]) */

CLASS
struct cluster
{
	struct	bornes *weight_borns;	/* les familles de clusters ([NB_CLUSTER]) */
	float	input_mean,		/* input moyen		*/
		input_sd,		/* et sa variabilite	*/
		backgrd_noise_m,	/* input de fond permanent moyen */
		backgrd_noise_sd,	/* variabilite de l'input de fond */
		rest_pot,		/* resting  potential du cluster	*/
		pot_post_spike,		/* Pot mb post spike */
		overshoot,		/* overshoot apres spike (rest - post_spike) */
		tau_mb_pot,		/* cste de temps de la mb */
		i_tau_mb_pot,		/* 1/cste de temps de la mb */
		seuil,			/* firing threshold par cluster */
		seuil_post_spike,	/* seuil apres un spike */
		E_K,			/* Potentiel d'equilibre du potassium */
		E_Na,			/* Potentiel d'equilibre du sodium */
		E_Cl,			/* Potentiel d'equilibre du chlore */
		E_Ca,			/* Potentiel d'equilibre du calcium */
		rev_pot_pos,		/* Reversal Potential positif (pour epsp) */
		rev_pot_neg,		/* Resersal Potential negatif (pour ipsp) */
		X_x3,			/* Reserve */
		tau_seuil,		/* cste de temps du seuil apres un spike */
		i_tau_seuil,		/* 1/cste de temps du seuil apres un spike */
		psp,			/* taille d'un psp (e ou i) */
		tau_psp_plus,		/* conste de temps de montee */
		tau_psp_moins,		/* conste de temps de descente */
		i_tau_psp_plus,		/* 1/conste de temps de montee */
		i_tau_psp_moins,	/* 1/conste de temps de descente */
		input_fact,		/* facteur additionel */
		psp_scale,		/* facteur d'echelle des PSP */
		shunt_values[MAX_SHUNT],/* valeurs du shunt de la membrane */
		shunt_tau,		/* constante de temps du shunt de la mb */
		fluctuation,		/* hebb */
		hebb;			/* coeff de la loi d'apprentissage	*/

	short	numero,			/* le numero (de 1 a NB_CLUSTERS +1) */
		*axone_mean,		/* longueur moyenne par cluster ([NB_CLUSTER])*/
		*axone_sd,		/* variable longueur  par cluster ([NB_CLUSTER])*/
		cluster_size,		/* taille des clusters */
		soma_diameter,		/* diametre des cellules */
		nb_act_clust,		/* nb de cellules actives par cluster */ 
		reset_psp,		/* burst autorise (flag) */
		first_neuron,		/* numero du premier neurone du cluster */
		X_x6,			/* Reserve */
/****************************************************************************/
		f_factor_scale,         /* facteur multiplicatif des inputs de bruit */
		presence_bruit_sup,	/* flag rajout 28/6/90 */
		num,			/* numero de fonction de bruit sup */
/****************************************************************************/	
		ExternIntern;		/* Cluster Interne = INTERN = 0, Externe = EXTERN = 1 */	

	void	(*learning)();		/* pointeur de fonction pour la loi choisie */

	char	clust_names[15],	/* le nom du cluster */
		file_name[60],		/* le nom du fichier *.P_unit (V6.0)	*/
		tms_name[60],		/* le nom du fichier *.tms (V6.0)	*/
		*inter_cluster,		/* connexions entre clusters ([NB_CLUSTER])*/
		no_autoconnexions,	/* flag */
		with_input,		/* flag */
		input_type,		/* [0](rien), [r]egular,[n]ormal,[g]amma,[e]xpon */
		loi;			/* loi d'apprentissage */

	FILE	*pt_tms;

} **cluster;				/* Dimension: ([NB_CLUSTER]) */

