/***********************************************************************
*
*		D O T _ D I S P L _ N _ B _ C. C
*
*	Plot l'activite du reseau en fonction du temps. 
*	Lit le fichier *.spk venant de neuro_bio_clusters
*	Cree un fichier PostScript
*	
*	21 fevrier 1990
*
*	Modifs:
*		JFV:	Mise au niveau V5	17/12/91
*			Correction de bugs	21/12/92
*		V2	lecteur de *.prm
*			au lieu de *.def	2/11/93
*
*
*	Authors:Drs J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*=========================================================================
*
*	Must be loaded with -lplot_ps
*
*
*=========================================================================
*
*	This program reads file * and creates a file *.PS to be
*	spooled on the LN03R
*
***************************************************************************/ 
#ifdef DOCUMENTATION

title	DOT_DISPL_N_B_C	Plotting dot display from neuro_bio_clusters
index			Plotting dot display from neuro_bio_clusters

Synopsis

	dot_displ_n_b_c [-v] [-d] [-k] [file_in]

Description

	DOT_DISPL_N_B_C reads in the file "file_in.spk" et "file_in.prm"
	created by neuro_bio_clusters providing a file of the following
	format:

	Etat du reseau au depart:

	{\tt
	\nofill
	    +<--------------1-------------><--------------2------------->
	    |         |         |         |         |         |         |
	    0    5    10   15   20   25   30   35   40   45   50   55   60
	    |    |    |    |    |    |    |    |    |    |    |    |    |
	   0|--11---1---1--1-1--11-1--111-12---222---2-----22-2--2---2---
	   1|                                                            
	   2|                                                      2     
	   3|                                2               2        22 
	   4| 1  1       1    1               2     2  2  2    2  2 2   2
	   5|1                      1    1  2     22                     
	   6|     1  1    1          1                          2        
	   7|      1  11                                2                
	   8|                              2     2         22        2   
	   9|                         1        2      2  2               
		...
	\fill
	}
	
	It plots the graph for a PostScript printer. 

	It is possible to choose the way these plots are drawn (scales,
	begining and end of times), by answering a series of
	questions.
	

	{\tt -v} switch gives a verbose output on stdout

	{\tt -d} switch uses the defaults values silently
	
	{\tt -k} is to keep the PostScript file instead of spooling it
	
	The output file file_in.PS is spooled on the PostScript printer, and
	then deleted (if not -k).

	Users of workstation should use visu_n_b_c to get X visualisation in
	color

Files	

	Input files: file_in.spk file_in.prm

	Output files: file_in.ps

See also

	graph_n_b_c, visu_n_b_c
	
Diagnostics

	Self explaining diagnostics are provided on fatal errors.

Authors

	Dr Jean-Francois Vibert 

	CHU Saint-Antoine Univ. P. & M. Curie PARIS

Bugs/deficiencies

	No known bugs.
#endif

#include <stdio.h>
#include <askval.h>
#include <rsx.h>
#include <plot_ps.h>
#include <math.h>

/*
#define TRACE
#define DEBUG
*/

/*	Pour la Laser */

#define B_PAP 30.
#define ABS_HAUTE 0
#define ABS_BASSE 1
#define REL_HAUTE 2
#define REL_BASSE 3

#define NB_MAX 100
#define PT_MAX 32000
#define	TON_BLANC	1.
#define	TON_GRIS	0.7
#define	NEUR_BY_SHEET	200

#define MAIN
#	include "cstes_nbc.h"
#undef MAIN

char	*usage = "%s [?] [-v] [-d] [-k] [file]\n",
	defaut,		/* flag "default" */
	G_sim,		/* flag */
	verbose,	/* flag bavard */
	keep = 'd',	/* flag keep the file (don't spool it) */
	reponse[50],	/* buffer pour les legendes */
	file[60],	/* generic file name */
	f_ps[60],	/* PS  file (*.ps) */
	f_data[60],	/* simulation at the end (*.spk) */
	f_param[60],	/* parameters  (*.prm) */
	f_modif[60],	/* fichier des modifs   (*.mod) */
	m_legend[30],	/* buffer pour les modifs */
	fix_scal,	/* flag ps_echelle fixe */
	prcntg,		/* flag ps_echelle en pct */
	*legend[NB_MAX],	/* les titres de graphs */
	buffer[140],	/* intermediaire */
	type_ps_echelle[140],	/* legende pour l'echelle de Y */
	fichier[60],	/* nom du fichier traite */
	date[20], heure[20], prompt[81],
	clust_names[NB_MAX+1][15], /* nom des clusters*/
	ligne[2000];	/* une ligne du fichier SPK */
	
short	nb_clusters;

int	nb_plot,
	cluster_size[NB_MAX],
	cluster_lim[NB_MAX+1],
	neur_droite,	/* le neurone a droite de la page */
	neur_gauche,	/* celui a gauche de la page */
	start,
	nb_neur,	/* compteur de neurones */
	nb_iter,	/* nb d'iteration */
	iter_deb,	/* iter du debut de la feuille */
	num_iter,	/* numero de l'iteration en cours */
	modif,		/* flag */
	m_dd,		/* parametres modifies au vol: m_* */
	m_d,
	m_iter,
	m_clust,
	m_cl_to,
	duree;
	
float	x_max,
	y_max[NB_MAX],
	x_min,
	y_min[NB_MAX],
	nb_data[NB_MAX],
	plot_height,
	plot_width,
	x_axis_size = 16.,
	le_tick,
	x[PT_MAX],
	y[NB_MAX][PT_MAX],
	m_M,			/* parametres modifies au vol: m_* */
	m_S,
	m_ii,
	m_i,
	f_debut, f_fin;


FILE	*p_data,	/* experimental file (*.sim) */
	*p_ps,		/* ps file at the end (*.ps) */
	*p_modif,	/* fichier des modifs */
	*p_prm;		/* parameter file (*.prm) */

main(argc,argv)
int argc;
char **argv;
{

	int	i, j, k,	/* un indice bon a tout */
		f_n_l,
		clust_gauche,	/* le cluster le plus a gauche */
		clust_droite,	/* le cluster le plus a droite */
		neur_lim_gauche,/* celui de gauche du cluster */
		neur_lim_droite,/* celui de droite du cluster */
		neur_restants,	/* nb de neurones a prendre en compte apres */
		nb_neur_en_cours, /* le neur en cours */
		num_clust_en_cours, /* le cluster laisse en plan */
		num_neur_en_cours, /* le neurone en cours laisse en plan */
		nb_page,	/* le numero de la page en cours */
		nb_msec,	/* un compteur de temps */
		bout_x = 160,	/* bout de l'axe des x */ 
		nb_sheets,	/* nb prevu de pages */
		nb_pg_cl,	/* nb de page en largeur */
		is_modif,	/* modifs were done */
		duree_feuille,	/* nb de ms par feuille	*/
		debut,		/* limites temporelles du plot */
		fin;

	char	*la_ligne;

	float	x_scale, y_scale,	/* origine */
		bout, facteur,
		le_bout,		/* extremite de l'axe en ms/10 */
		x[10], y[10],		/* coordonees */	
		x_dot, y_dot,		/* idem   */
		x_pt[4],y_pt[4],	/* un ps_point */
		gris,			/* couleur du grise */
		big, medium, little;


		/* traitement de la ligne de commande */

	strcpy(file,"");
	defaut = verbose = FALSE;
	if (argc == 1)
		asksval("\nName of the file *.spk to draw (without extension): ",
				"", file);
	if (argc > 1)
	{
		for (i=1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf(usage,argv[0]);
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'd':
						defaut = TRUE;
						break;
					case 'k':
						keep = 'f';
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
				}
			}
			else strcpy(file,argv[i]);
		}
		if (streq(file,""))
			asksval("\nName of the file *.spk to draw (without extension): ",
				"", file);
	}


	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}
	else strcpy(f_data,file);
		/* on a dans file le nom generique */

	strcat(f_data,".spk");
	strcpy(f_modif,file);
	strcat(f_modif,".mod");
	strcpy(f_param,file);
	strcat(f_param,".prm");
	
        if ((p_data=fopen(f_data,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_data);
		exit(1);
	}
	printf("%s opened...\n",f_data);

	modif = TRUE;
        if ((p_modif=fopen(f_modif,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_modif);
		modif = FALSE;
	}
	else printf("%s opened...\n",f_modif);
        if ((p_prm=fopen(f_param,"r")) == NULL)
	{
		printf("the file %s cannot be opened...\n ",f_param);
		exit(1);
	}
	printf("%s opened...\n",f_param);
		/*
		 * lecture des donnees originales pour connaitre le nb de data
		 * et les max et min
		 */
	if (modif)
	{
		is_modif = TRUE;	 
		if ( fgets(buffer,80,p_modif) == 0)
		{
			printf("They were not on flight parameters modifications\n");
			is_modif = FALSE;
		} /* fin du if */
		else rewind(p_modif);
	} /* fin du if */

	/* Recherche du nombre d'iterations */

	printf("Computing the simulation duration...\n");
	while (fgets(ligne,2000,p_data))
	{
		if (ligne[4] == '|' && ligne[3] == '0') break;
	} /* fin du while */
	nb_iter = 0;
	while (fgets(ligne,2000,p_data)) nb_iter++;
	rewind(p_data);
	ligne[3] = ligne[4] = 0; /* remise a zero du test de ligne */
	
	num_iter = 0;

	/*
	 * lecture des donnees originales pour connaitre le nb de data
	 * et les max et min
	 */
	read_param();		/* lectures de tailles des clusters */
		 
	if (G_sim)
		printf("Conductance simulation (nbc_g)\n");
	else
		printf("Phenomenologic simulation (nbc_p)\n");
	
	for(i = 1; i <= nb_clusters; i++)
	{
		if (verbose)
		{
			printf("Cluster %d: %d neurons\n",
			       i, cluster_size[i]);
		} /* fin du if */
	}
	cluster_lim[0] = 0;
	for(i = 1; i <= nb_clusters; i++)
	{
		cluster_lim[i] = cluster_lim[i-1] + cluster_size[i];
	}

	cluster_size[0] = nb_neur;
	printf("There are %d neurons in %d cluster%c\n",
	       cluster_size[0],
	       nb_clusters,
	       ((nb_clusters > 1)?'s':' '));
	nb_sheets = (cluster_size[0] / NEUR_BY_SHEET) + 1;
	
	
	if (nb_sheets > 1) printf("They will be split in %d sheets\n",nb_sheets);
		
	if(verbose) printf("File to be plot: %s\n",f_data);

	duree_feuille = 1000;
	if (!defaut)
	{

		sprintf(buffer,"Beginning of the plot [0] ms: ",0);
		debut = askival(buffer,0);
		do
		{
			sprintf(buffer,"End of the plot [%d] ms: ",nb_iter);
			fin = askival(buffer,nb_iter);
		} while (fin <= debut); /* fin du do */
		duree_feuille = askival("Duration of a sheet in ms: ",duree_feuille);
		
/*
		x_axis_size = askfval("Time axis length in cm: ",x_axis_size);
		bout_x = askival("Time axis length in ms: ",fin);
*/
		bout_x = duree_feuille;
		le_bout = (float)bout_x / 10.;
	}
	else
	{
		debut = 0;
		fin = nb_iter - 1;
	} /* fin du else */
	duree = fin - debut;

	f_fin = (float)fin;
	f_debut = (float)debut;


		/* creation du fichier de sortie 
			et initialisation Benson */

	strcpy(f_ps,file);
	strcat(f_ps,".dot_disp.ps");
	if(verbose) printf("PostScript file to be produced: %s\n",f_ps);

	ps_iniplot(f_ps,B_PAP);

	plot_height = 26.;
	plot_width = 18.;
	rewind(p_data);
	ligne[3] = ligne[4] = 0; /* remise a zero du test de ligne */

	ps_echel((x_axis_size/le_bout),1.,0.,0.,1.,0.);
	ps_pen(NOIR);
	do_dot();
	ps_user();
	
	num_clust_en_cours = 1;
	nb_neur_en_cours = num_neur_en_cours = 0;
	neur_gauche = neur_droite = 0;
	gris = TON_BLANC;
	start = FALSE;
	num_iter = 0;
	la_ligne = &ligne[5];
	neur_restants = 0; /*
			    * nb de neurones a prendre en compte pour un
			    * cluster coupe en 2
			    */
	for (nb_pg_cl = 0; nb_pg_cl < nb_sheets; nb_pg_cl++)
	{
		neur_droite = neur_gauche + NEUR_BY_SHEET;
		if (neur_droite > cluster_size[0]) 
			neur_droite = cluster_size[0];
			
		while (fgets(ligne,2000,p_data))
		{
			if (verbose) printf("**>%s",ligne);
			if (ligne[4] == '|' && ligne[3] == '0') break;
		} /* fin du while */
		nb_page = 0;
		if (start) ps_newpage();
		else start = TRUE;
		
		ps_origin(2.,7.,0.,0.);

		ps_movea(0.,0.);
		ps_drawa(le_bout,0.);
			/* Trace des axes */
		printf("Drawing sheet %d, page %d...\n",
				nb_pg_cl + 1, nb_page + 1);

		/* Dans quels clusters sommes nous? */

		for (i = 1; i <= nb_clusters; i++)
		{
			if (neur_gauche >= cluster_lim[i-1]
			    && neur_gauche < cluster_lim[i])
				clust_gauche = i;
			if (neur_droite >= cluster_lim[i-1]
			    && neur_droite <= cluster_lim[i])
				clust_droite = i;
		} /* fin du for */
		if (verbose) printf("Left Neuron %d -> %d Right Neuron\n",
				    neur_gauche, neur_droite);
		if (verbose) printf("Left Cluster: %d Right Cluster: %d\n",
				clust_gauche, clust_droite);

		neur_lim_gauche = 0;
		for (i = clust_gauche; i <= clust_droite; i ++)
		{
			nb_neur_en_cours += (cluster_size[i] - neur_restants);
			neur_lim_droite = neur_lim_gauche + 
					  cluster_size[i] - neur_restants;
			neur_restants = 0;
			if (neur_lim_droite > neur_droite)
			{
				neur_lim_droite = neur_droite;
				neur_restants =  cluster_lim[i] - neur_droite;
				nb_neur_en_cours = neur_droite;
			} /* fin du if */
			num_clust_en_cours = i;
			if ( num_clust_en_cours % 2) gris = TON_BLANC;
			else gris = TON_GRIS;
			x[0] = 0.;	y[0] = 0.1 * neur_lim_gauche;
			x[1] = x[0];;	y[1] = 0.1 * neur_lim_droite;
			x[2] = le_bout;	y[2] = y[1];
			x[3] = x[2];	y[3] = y[0];
			ps_movea(x[0],y[0]);
			ps_closed_curve(x,y,4,FALSE,TRUE,gris);
			ps_movea(x[0],y[0]);
			traa(-0.5,0.,PL_RB);
			sprintf(buffer,"%d ",neur_lim_gauche+1);
			ps_chara(-0.8,0.1,PL_RH,buffer,0.4,0);
			sprintf(buffer,"%d %s",
				num_clust_en_cours,
				clust_names[num_clust_en_cours]);
			ps_chara(x[0]-0.5,y[0]+1.,PL_AH,buffer,0.4,90);
			ps_movea(x[1],y[1]);
			traa(-0.5,0.,PL_RB);
			sprintf(buffer,"%d ",neur_lim_droite);
			ps_chara(-0.8,-0.5,PL_RH,buffer,0.4,0);
			neur_lim_gauche = neur_lim_droite;
			printf("cluster %d done\n",num_clust_en_cours);
		} /* fin du for */

		nb_msec = 0;
		iter_deb = debut;

		/* retour au 0 */

		ps_movea(0.,0.);

		le_tick = (float)(bout_x - iter_deb) / 10.;

		y_tick((float)iter_deb,(float)bout_x,le_tick,(le_tick/2.),(le_tick/10.),"Time (msec)");

		ps_movea(0.,0.);
		ps_drawa(le_bout,0.);

		if (verbose) printf("1er fond dessine\n");

		do
		{
			ligne[4] = 0;	/* un EOS */
			num_iter = atoi(ligne) + 1;
			if (verbose) printf("%d -> |%s",num_iter, la_ligne+1);
			for (i = neur_gauche; i < neur_droite; i++)
			{
				if (la_ligne[i] == ' ' || la_ligne[i] == '-')
					continue;
				x_dot = 0.1 * (num_iter - iter_deb - 1);
				y_dot = 0.1 * (i - neur_gauche + 1);
				fputs("newpath\n",pl_spl);
				ps_movea((x_dot),(y_dot));
				do_the_point();
			} /* fin du for */

			if (num_iter > fin) break;

			nb_msec ++;
			if (verbose) printf("%d ms\n",nb_msec);
			if (is_modif) modif = TRUE;

			if (modif)
			{
				int	nb_leg, fleche_done;
				
				fleche_done = FALSE;
				while (modif && (m_iter < num_iter)) get_modif();
				nb_leg = 1;
				while (modif && (m_iter == num_iter))
				{
					if (!fleche_done)
					{
						ps_movea(0.1 * (m_iter - iter_deb),0.);
						ps_cms();
						ps_drawr(0.15,-0.3);
						ps_mover(-0.3,0.);
						ps_drawr(0.15,0.3);
						ps_drawr(0.,-1.5);
						ps_user();
						fleche_done = TRUE;
					} /* fin du if */
					if (verbose)
						printf("Modif on cluster #%d at %d msec\n",
							m_clust, m_iter);
					if (m_clust >= clust_gauche && m_clust <= clust_droite)
					{
						int bout;
						
						bout = cluster_lim[m_clust-1];
						if (bout < neur_gauche)
							bout = neur_gauche;
						x[0] = 0.1 * bout;
						bout = cluster_lim[m_clust];
						if (bout > neur_droite)
							bout = neur_droite;
						x[1] = 0.1 * bout;
						ps_movea(0.1 * (m_iter - iter_deb),x[0]);
						fprintf(pl_spl,"[4 4] 0 setdash\n");
						ps_drawa(0.1 * (m_iter - iter_deb),x[1]);
						ps_movea(0.1 * (m_iter - iter_deb),-1.5);
						ps_chara(0.,(-0.3 * nb_leg),REL_HAUTE,m_legend,.2,0);
						nb_leg++;
					} /* fin du if */
					get_modif();
				} /* fin du if */
				fprintf (pl_spl,"[] 0 setdash\n");
			} /* fin du if */

			if (nb_msec > bout_x)
			{
				titre();
				nb_msec = 0;
				iter_deb = num_iter;
				ps_newpage();
				nb_page ++;
				ps_origin(2.,7.,0.,0.);

				ps_movea(0.,0.);
				ps_drawa(le_bout,0.);
					/* Trace des axes */
				printf("Drawing number %d, page %d at %d ms...\n",nb_pg_cl + 1, nb_page + 1, iter_deb);


				/*
				 * Axe des effectifs (dans le sens Y benson)
				 */

				neur_lim_gauche = 0;
				for (i = clust_gauche; i <= clust_droite; i ++)
				{
					nb_neur_en_cours += (cluster_size[i] - neur_restants);
					neur_lim_droite = neur_lim_gauche + 
							  cluster_size[i] - neur_restants;
					neur_restants = 0;
					if (neur_lim_droite > neur_droite)
					{
						neur_lim_droite = neur_droite;
						neur_restants =  cluster_lim[i] - neur_droite;
						nb_neur_en_cours = neur_droite;
					} /* fin du if */
					num_clust_en_cours = i;
					if ( num_clust_en_cours % 2) gris = TON_BLANC;
					else gris = TON_GRIS;
					x[0] = 0.;	y[0] = 0.1 * neur_lim_gauche;
					x[1] = x[0];;	y[1] = 0.1 * neur_lim_droite;
					x[2] = le_bout;	y[2] = y[1];
					x[3] = x[2];	y[3] = y[0];
					ps_movea(x[0],y[0]);
					ps_closed_curve(x,y,4,FALSE,TRUE,gris);
					ps_movea(x[0],y[0]);
					traa(-0.5,0.,PL_RB);
					sprintf(buffer,"%d ",neur_lim_gauche+1);
					ps_chara(-0.8,0.1,PL_RH,buffer,0.4,0);
					neur_lim_gauche = neur_lim_droite;
					sprintf(buffer,"%d %s",num_clust_en_cours,clust_names[num_clust_en_cours]);
					ps_chara(x[0]-0.5,y[0]+1.,PL_AH,buffer,0.4,90);
					ps_movea(x[1],y[1]);
					traa(-0.5,0.,PL_RB);
					sprintf(buffer,"%d ",neur_lim_droite);
					ps_chara(-0.8,-0.5,PL_RH,buffer,0.4,0);
					neur_lim_gauche = neur_lim_droite;
				} /* fin du for */

				/* retour au 0 */
				ps_movea(0.,0.);
				
				y_tick((float)iter_deb,(float)bout_x,10.,5.,1.,"Time (msec)");

				ps_movea(0.,0.);
				ps_drawa(le_bout,0.);
			} /* fin du if */
		} while (fgets(ligne,2000,p_data));

		titre();

		rewind(p_data);
		ligne[3] = ligne[4] = 0; /* remise a zero du test de ligne */

		if (is_modif) rewind(p_modif);
	}
	ps_endplot(f_ps,keep);
}
/*
*	GET_MODIF() lit les modif et les dispatch
*/
get_modif()
{
	char	m_buffer[81], to, m_code; 
	float	m_old_f, m_old_d;

	if (fgets(m_buffer,80,p_modif) == 0)
	{
		modif = FALSE;
		return(modif);
	} /* fin du if */
	sscanf(m_buffer,"%d %d %c",&m_iter,&m_clust,&m_code);
	switch (m_code)
	{
		case 'M':
			sscanf(m_buffer,"%d %d %c %f %c %f",
					&m_iter,&m_clust,
					&m_code,&m_old_f,&to,&m_M);
			sprintf(m_legend,"Rest: %.2f mV",m_M);
			break;			
		case 'S':
			sscanf(m_buffer,"%d %d %c %f %c %f",
					&m_iter,&m_clust,
					&m_code,&m_old_f,&to,&m_S);
			sprintf(m_legend,"Thr: %.2f mV",m_S);
			break;			
		case 'I':
			sscanf(m_buffer,"%d %d %c %f %c %f",
					&m_iter,&m_clust,
					&m_code,&m_old_f,&to,&m_ii);
			sprintf(m_legend,"In m: %.2f mV",m_ii);
			break;			
		case 'i':
			sscanf(m_buffer,"%d %d %c %f %c %f",
					&m_iter,&m_clust,
					&m_code,&m_old_f,&to,&m_i);
			sprintf(m_legend,"In s: %.2f",m_i);
			break;
		case 'D':
			sscanf(m_buffer,"%d %d %c %d %c %d %d",
					&m_iter,&m_clust,
					&m_code,&m_old_d,&to,&m_dd,&m_cl_to);
			sprintf(m_legend,"Ax %d m: %d",m_cl_to,m_dd);
			break;			
		case 'd':
			sscanf(m_buffer,"%d %d %c %d %c %d %d",
					&m_iter,&m_clust,
					&m_code,&m_old_d,&to,&m_d,&m_cl_to);
			sprintf(m_legend,"Ax %d s: %d",m_cl_to,m_d);
			break;			
	} /* fin du switch */
	return(modif);
} /* fin du get */
/*
*
*	TITRE() ecrit la legende generale du dessin 
*
*/
titre()
{
	static	char leg_txt[150];
	int	i;

	sprintf(buffer,"%d cluster%c", nb_clusters,(nb_clusters)?'s':' ');
	traa(-2.,-3.5,ABS_HAUTE);
	sprintf(leg_txt,"File: %s    %s, neurons %d to %d, from %d to %d ms.",
				f_data, buffer,
				neur_gauche, neur_droite,
				iter_deb, num_iter-1);
	if (strlen(leg_txt) > 149 ) leg_txt[149]='\0';
	ps_chara(-2.,-4.1,ABS_HAUTE,leg_txt,.4,0);
	traa(-2.,-4.5,ABS_HAUTE);
	if((p_prm = fopen(f_param,"r")) == NULL)
	{
		printf("Parameter file %s impossible to open as read\n", f_param);
		return;
	}
	while(fgets(buffer,80,p_prm))
	{
		if (buffer[0] == 'M' && buffer[1] == 'a')
		{
			fgets(buffer,132,p_prm);
			break;
		} /* fin du if */
	} /* fin du while(fgets(buffer */
	ps_font("Courier",.2);
	for (i = 0; i < nb_clusters+1; i++)
	{
		fgets(buffer,142,p_prm);
		buffer[142] = 0;
		buffer[strlen(buffer)-1] = 0;
		if (strlen(buffer) > 79)
		{
			buffer[7] = 0;
			strcat(buffer,&buffer[49]);
		} /* fin du if */
		ps_chara(-2.,-4.6 - ((float)i * .3) ,ABS_HAUTE,buffer,.3,0);
	} /* fin du for */
	fgets(buffer,132,p_prm);
	buffer[strlen(buffer)-1] = 0;
	ps_chara(-2.,-4.6 - ((float)(nb_clusters+1) * .3) ,ABS_HAUTE,buffer,.3,0);
	ps_font("Times-Roman",.4);
	fclose(p_prm);
}
/*
*
*	Y_TICK(debut,dmax,d_big,d_medium,d_little,texte)
*
*	met des tick-marks dans le sens x de d_big en d_big,
*	avec des traits a d_medium et d_little
*	jusqu'a dmin.
*
*/
y_tick(debut,dmax,d_big,d_medium,d_little,texte)
float 	debut,dmax, d_big, d_medium, d_little;
char 	*texte;
{
	float hx, where; /* Laser */
	int i, i_dmax, i_little, i_medium, i_big, i_deb;
	char modulo, nb_ech[10];

	dmax /= 10.; /* car on le donne en ms et il le faut en "cm" */
	hx = 0.4;
	traa(0.,0.,ABS_HAUTE);
	i_dmax = (int)(dmax + debut);
	i_little = (int)d_little;
	i_medium = (int)d_medium;
	i_big = (int)d_big;
	i_deb = (int)debut;
	for ( where = 0. ; where <= dmax;  )
	{
		i = (int) where;
		modulo = 'l';
		if ( ! (i % i_medium) ) modulo = 'm';
		if ( ! (i % i_big) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(0.,-0.5,REL_BASSE);
				sprintf(nb_ech,"%d",((i * 10)+i_deb));
				traa(0.0,-0.6,REL_HAUTE);
				ps_chara(0.,0.,REL_HAUTE,nb_ech,hx,0);
				traa(0.0,1.1,REL_HAUTE);
				break;
			case 'm':
				traa(0.,-0.3,REL_BASSE);
				traa(0.,0.3,REL_HAUTE);
				break;
			case 'l':
				traa(0.,-0.2,REL_BASSE);
				traa(0.,0.2,REL_HAUTE);
		}
		where += d_little;
		ps_movea(where,0.);
  	}
	ps_chara((x_axis_size - 3.),-1.4,ABS_HAUTE,texte,0.4,0);
}
/*
*	do_the_point() demarre un panneau avec un pattern gris
*/
do_the_point()
{
	
	sprintf(pl_buf,"DoDot\n");
	fputs(pl_buf,pl_spl);

	draw = FALSE;
}
/*
 * DO_DOT() defini une nouvelle fonction PS
 */
do_dot()
{

	fputs("/DoDot {\n",pl_spl);
	fputs("-1 -1 rmoveto\n",pl_spl);
	fputs("0 2 rlineto\n",pl_spl);
	fputs("2 0 rlineto\n",pl_spl);
	fputs("0 -2 rlineto\n",pl_spl);
	fputs("-2 0 rlineto\n",pl_spl);
	
	fputs("closepath\n",pl_spl);

	draw = FALSE;

	sprintf(pl_buf,"gsave 0. setgray fill grestore} def\n");
	fputs(pl_buf,pl_spl);
	
} /* fin du do */



/*
*
*	READ_PARAM Relecture des parametres de la simulation dans f_param
*
*/
read_param()
{
	int	i,
		k;
		
	char	inutile[80];
	
	if((p_prm = fopen(f_param,"r")) == NULL)
	{
		printf("Parameter file %s impossible to open as read\n", f_param);
		return;
	}
	/* nb de neurones */
	G_sim = FALSE;
	fgets(buffer,80,p_prm);
	if (buffer[0] == 'G')
	{
		G_sim = TRUE;
		fgets(buffer,80,p_prm);
	} /* fin du if */
	if (buffer[0] == 'P')
	{
		G_sim = FALSE;
		fgets(buffer,80,p_prm);
	} /* fin du if */
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_neur);
	       
	/* nb de clusters */
	fgets(buffer,80,p_prm);
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_clusters);

	k = 1;
	while (fgets(buffer,80,p_prm))
	{
		sscanf(buffer,"%s",inutile);
		if (!strcmp(inutile,"CLUSTER"))
		{
			fgets(buffer,80,p_prm);
			sscanf(buffer,"%s %s %s %d",
			       inutile,
			       inutile,
			       inutile,
			       &(cluster_size[k]));
			k++;
		} /* fin du if */
	} /* fin du while(fgets(buffer */
	fclose(p_prm);
} /* fin du read_param() */

