/*
*       Neuro_Bio_Clusters_Simulator
*
*       N_B_C.H
*
*       Header file pour le simulateur de reseaux neuronaux a haute
*       plausibilite biologique.
*
*
*	ULTRIX 	2 avril 1990
*	
*	Modifications:
*			15 mai 1990:	Reversal Potential (V4.4)
*	Modifications:
*			2 jullet 90:	Sauvegarde et reprise (V4.6)
*			2 juillet 1990 : bruit supplementaire (V4.7)
*                                        par A. Rudnichi
*                                       rajout de num dans struct cluster
*                                      rajout de presence_bruit_sup dans struct cluster
*                                     a la place de X_x8 et X_x9 en short
*				    definitions supplementaires :
*                                    f_fonction,p_fonction et with_bruit.
*
*			19 fevrier 91:	Nouvelle fonction du
*					reversal pot. (V 4.9)
*			9 avril 91:	Possibiliter de couper delimiter le nb de
*					connexions par neurone (V 4.10)
*			22 mai 91	File d'attente des spikes sur l'axone
*					(permet que  l'intervalle interspike
*					soit plus grand que la longueur de l'axone)
*
*	Versions:	V1	n_c						juin 88
*			V2	n_c	delais inter neurones			juin 88
*			V3	n_c	parallelisme stat + phase refract	juill 88
*			V4	n_b_c	refonte profonde et structuration	2 avril 90
*			V4.0	Mb pot en exponentielles			2 avril 90
*			V4.1	ajout des seuils variables			5 avril 90
*			V4.2	ajout des E_K et E_Na				12 avril 90
*			V4.3	Ajout des time series et correction de bugs	fin avril 90
*			V4.4	Ajout des reversal potentials			15 mai 90
*			V4.5	traitement des inputs + corrct bugs		juin 90
*			V4.6	sauvetage et reprise de simulation		2 juillet 90	
*        		V4.7	inputs de bruit supp  (ar)			2 juillet 90
*			V4.8	generalized modification enabled		nov 90
*			V4.9	Nouvelle fonction de reversal pot. (jfv+sh)	19 fevrier 91
*			V4.10	Conservation de n connexions par neurones 
*				+ clusters ayant ou non une histoire		10-Avril-91
*				+ choose_file
*				+ Z compression on fly
*			V4.11	File d'attente des spikes sur l'axone		22 mai 91
*				(permet que  l'intervalle interspike
*				soit plus grand que la longueur de l'axone)
*			V4.12	Prise en compte du shunt de la mb 		30 mai 91
*			V4.13   Affinement du calcul des amplitudes des PSPs	7 juin 91
*			V4.14	Numero d'iteration si non dot display		22-juillet-91
*			V4.15   Modif fichier *.pmb pour pmb_*_n_b_c		1-aout-91
*			V5.0	Allocation dynamique des structures afin de	8 juin 91
*				gagner de la place.
*			V5.1	Allocation dynamique niveau 4.15		7 nov 91
*			V5.2	Chgt structure de Neurone
*					(+ appartenance symbolique
*					 + localisation spatiale)		18 dec 91
*			V5.3	Modification du comportement de l'option	12 nov 92
*				de chgt de delay au vol
*				+ ajout du chgt de periode refractaire au vol
*			V5.4	Modification du calcul des PSP (KP+JFV)		23 Dec 92
*			V5.5	Traitement de la ligne de commande (KP+JFV)	26 Jan 93
*				+ manuel
*			V5.6	Chgt en rampe des longueurs d'axone (KP+JFV)	19 Fev 93
*			V6.1	Lecture des fichier *.unit et *.ntw		25 mai 93
*			V6.2	Sous menu des parametres de cellules		1 juin 93
*				+ chargement du fichier *.tms (cluster virtuel)
*			V6.3	Ajout du switch de travail en mode batch	31 juillet 93
*								(JFV+KP)
*			V6.4	Modification des allongements	(JFV)		21 septembre 93
*						(supression de variabilite)
*			V6.5	Mise en conformite avec nbc_g	(JFV)		3 Nov 93
*
*
*
*	Auteur: Dr J-F VIBERT
*		ex	L.I.O.N.	Laboratoire de Physiologie
*		B3E URBB U263
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not in any way usable for commercial purpose.
*
*/

#if UNIX
#	include <sys/types.h>
#	include <sys/timeb.h>
#endif
#if VMS
#	include <types.h>
#	include <timeb.h>
#endif
#include <signal.h>
#include <math.h>

#ifdef MAIN
#	define CLASS
#else
#	define CLASS	extern
#endif

#include "version_nbc.h"

/*
 * les constantes et le systeme d'exploitation
 */
#include "cstes_nbc.h"

#if VMS
#	define	srandom	srand
#	define	random	rand
#	define	unlink	delete
#endif

/*	definitions de macros */

/*
*
*	SCROLL() defini la scrolling region 
*
*/
#define	scroll(haut,bas) printf("\033[%d;%dr",haut,bas)
/*
*
*	curseur() positione le curseur
*
*/
#define	curseur(ligne,colonne)	printf("\033[%d;%dH",ligne,colonne)
/*
*
*	SAVE_CURS() sauvegarde la position du curseur
*
*/
#define	save_curs()	printf("\0337")
/*
*
*	REST_CURS() sauvegarde la position du curseur
*
*/
#define	rest_curs()	printf("\0338")
/*
*
*	CLR_SCROLL() efface la scrolling region
*
*/
#define	clr_scroll()	printf("\033[18M")

CLASS
/*************************************************************************/
struct identif_fonct	/* rajout pour input AR 28/06/90*/
{
	char	nom[60],
		typ_fonc[60];
	int	duree1,
		amplitude,
		frequence,
		phase1,
		sens1;	/*
			 * rajout le 18 sept pour sens positif ou negatif de
			 * l'input
			 */
	float	val[T_MAX];
}param_fonctions[MAX],*p_identif;

#include "cluster.h"
#include "neurone.h"


/*--------------------------------------------------------------------------------------
 * Gestion des clusters externes : liste ou sont tries tous les numero de neurone du cluster k
----------------------------------------------------------------------------------------*/
struct noeud_tms
{
        struct noeud_tms *suiv;
        int  num_neur;
};
typedef struct noeud_tms *Node;
/*---------------------------------------------------------------------------------------*/


CLASS
char	*etat_avant,		/* etat du resau a t-1 (dimension: NB_NEUR) */
	*etat_apres;		/* etat du resau a t (dimension: NB_NEUR) */

CLASS
short	*tab_short,		/* pointeur de short pour fwrite */
	*tab_con,		/* pointeur de short pour fwrite */
				/* flags */
	verbose,		/* verbeux  */
	batch,			/* pas de questions */
	debug,			/* visualisation de certaines variables */
	keep,			/* keep postscript file */
	output,			/* on demande une sortie */
	voir_neurones,		/* on regarde le dot display */
	new_matrix,		/* on fabrique une nouvelle matrice */
	exam_soma,		/* on conserve le pmb */
	modif_param,		/* les parametres ont ete modifies */
	from_lnk,		/* reseau venant de link_edit */
/****************************************************************************/
	with_bruit,		/* bruit supplementaire */
/***************************************************************************/
	Z_compress,			/* flag pour comprimer la mat. connex. */
	restart_old,			/* reprend une ancienne simulation */
	bell_input,			/* flag pour sonner a chaque input */
	chg_dist,			/* flags de chgt de longueur	*/
	chg_dist_P,			/* cght de longuer en douceur */
	P_neuron_running,		/* L'editeur P_neuron is running.... */
	spy,				/* numero du neurone a surveiller */
	last, first,			/* 1er et dernier neurone d'un cluster */
	nb_actifs,			/* nb de neurones qui dechargent */
	nb_clusters,			/* nb de clusters de neurons */
	nb_courant,			/* un entier a tout faire */
	nb_neuron;			/* nb general de neurones */




CLASS
char	/* les noms des fichiers */
	f_param[60],			/* les parametres e la simlation */
	f_pre_defaut[60],		/* les parametres par defaut en entree */
	f_defaut[60],			/* les parametres par defaut en sortie */
	f_simul[60],			/* nb de neurones actifs */
	f_modif[60],			/* fichier des modifs au vol */
	f_connex[60],			/* matrice des connexions output */
	f_old_connex[60],		/* matrice des connexions input */
	f_pmb[60],			/* potentiel membranaire */
	f_spk[60],			/* decharges des neurones */
	f_tms[60],			/* time series */
	f_cluster[60],			/* fichier temporaire */
	f_lnk_file_name[60],		/* buffer pour un nom de fichier .ntw */
/****************************************************************************/
	f_fonction[60],			/* fichier des inputs sup rajout le26 */
/***************************************************************************/
	name[60],			/* nom generique */
	buffer[100],			/* ca sert toujours!! */
	*tab_char;			/* pointeur de char pour fwrite */

CLASS
int	/* des entiers pour les iteration, pour avoir le temps... */
	nb_it,				/* numero de l'iteration courante */
	next_stop,			/* prochain arret au cours de la simulation */
	last_it,			/* very last iteration (inconditional stop) */
	nb_next_it,			/* increment entre arrets */
	duree,				/* nb d'iterations */
	debut_chgt,			/* moment du debut de cght de longueur */
	duree_chgt;			/* nb d'iterations du chgt */

CLASS
FILE	*p_param,			/* fichier de parametre */
	*p_soma,			/* le fichier de surveillance de soma */
	*p_seuil,			/* le fichier de surveillance du seuil */
	*p_intra,			/* le fichier de surveillance du soma sans inputs */
	*p_defaut,			/* fichier des parametres ant */
	*p_simul,			/* fichier de resultats */
	*p_modif,			/* fichier des modifs au vol */
	*p_connex,			/* fichier des connexions */
	*p_pmb,				/* potentiel membranaire */
	*p_spk,				/* decharges des neurones */
	*p_tms,				/* time series */
	*p_old_connex,			/* fichier des connexions anciennes */
/****************************************************************************/
	*p_fonction,			/* rajout 28/06 fonctions input sup */
/***************************************************************************/
	**p_cluster;			/* fichiers temporaires (dimension: NB_CLUSTER) */

CLASS
void	arret(),
	nrerror(),
	hebb_learn(),
	widrow_learn(),
	read_lnk_file(char *);	

CLASS
int	read_unit_file(char *, int);

/****************************************************************************/
CLASS
float	tab[NB_CLUST_MAX][T_MAX];		/*  valeurs de bkgrnd noise sup ([NB_CLUSTER][T_MAX])*/
CLASS
int	tab_norme[NB_CLUST_MAX][T_MAX];	/*  valeurs normees de bkgrnd noise sup ([NB_CLUSTER][T_MAX])*/
/****************************************************************************/

CLASS
float	*K_rev_n,
	*K_rev_p;	/*
			 * constantes de reversal pot.
			 */

/*---------------------------------------------
 * Variables et fonctions pour lire le fichier tms
----------------------------------------------*/
typedef struct lect_tms
{
	int tms_time;
	int num_neur;
} read_tms;

CLASS
read_tms *tab_tms;


int ouvre_tms_file(void);
int ferme_tms_file(void);



