/***********************************************************************
*
*		P M B _ 3 D _ N _ B _ C . C
*
*	Plot le potentiels de membrane en 3D
*	avec le meme format que xgraph.
*	Cree un fichier PostScript
*	Le fichier doit provenir de neuro_bio_clusters
*	
*		start:	2 novembre 1989
*			2 may 1990
*
*		V2 8 aout 97	propose le nom de fichier meme si
*				donne en ligne de commande
*		V3 12 aout 97	Nouveau dessin de la forme des spikes
*
*               V4 26 aout 97   Titre en clair
*
*	Authors:Drs J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*=========================================================================
*
*	Must be loaded with -lplot_ps -lm 
*
*
*=========================================================================
*
*	This program reads file *.pmb and creates a file *_3d.ps to be
*	spooled on the LN03R
*
***************************************************************************/ 
#ifdef DOCUMENTATION

title	PMB_3D_N_B_C     Plotting membrane potential data
index			Plotting membrane potential data

Synopsis

	pmb_3d_n_b_c [-v] [-d] [-k] [file_in[.pmb]]

Description

	PMB_3D_N_B_C reads in the file "file_in.pmb" et "file_in.prm" 
	created by neuro_bio_clusters
	providing a file of the following format:

	{\tt
	\nofill

		"Iteration 1"
		1 -20.5
		2 -10.0
		3 -40.2
		4 -50.1

		"Iteration 2"
		1 -80.2
		2 -30.3
		4 -50.3
		6 -70.0
		...
	\fill
	}
	
	The 1st number is the neuron number and the second the membrane potentiol.

	It plots the graph for a PostScript printer. 

	It is possible to choose the way these plots are drawn (scales,
	lines, dots, begining and end of times), by answering a series of
	questions.
	

	{\tt -v} switch gives a verbose output on stdout

	{\tt -d} switch uses the defaults values silently
	
	{\tt -k} is to keep the PostScript file instead of spooling it
	
	The output file file_in_3d.ps is spooled on the PostScript printer, and
	then deleted (if not {\tt -k}).
Files	 

	Input files: file_in.sim file_in.prm

	Output files: file_in_3d.ps

See also

	visu_n_b_c
	
Diagnostics

	Self explaining diagnostics are provided on fatal errors.

Authors

	Dr Jean-Francois Vibert 

	CHU Saint-Antoine Univ. P. & M. Curie PARIS

Bugs/deficiencies

	No known bugs.
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <askval.h>
#include <rsx.h>
#include <plot_ps.h>
#include <math.h>
#include "find_files.h"

/* pour la fonction psplot3d */

int	plot_start = TRUE,	/* flag d'initialisation */
	plot_nb_line;		/* numero de trace courant */

float	x_extr, y_extr,
	offset,
	s1, s2, s3, s4, s5, s6,	/* les parametres des rotations precalcules */
	c1, c2;


/* fin de la partie specifique a psplot3d */

/*
#define TRACE
#define DEBUG
*/

/*	Pour la Laser */

#define B_PAP 21.
#define ABS_HAUTE 0
#define ABS_BASSE 1
#define REL_HAUTE 2
#define REL_BASSE 3
#define TOP_SPIKE 100.0

#define NB_MAX 100

#define NB_ITER_MAX 5000

#define MAIN
#	include "cstes_nbc.h"
#undef MAIN


char	*usage = "%s [?] [-v] [-d] [-k] [file]\n",
	defaut,		/* flag "default" */
	G_sim,		/* flag */
	spike,		/* flag */
	verbose,	/* flag bavard */
	keep = 'd',	/* flag keep the file (don't spool it) */
	reponse[50],	/* buffer pour les legendes */
	file[256],	/* generic file name */
	f_ps[256],	/* experimental file (*.bsp) */
	f_data[256],	/* simulation at the end (*.pmb) */
	f_param[256],	/* parameters  (*.prm) */
	f_rot_def[256],	/* fichier des rotations */
	fix_scal,	/* flag ps_echelle fixe */
	prcntg,		/* flag ps_echelle en pct */
	*legend[NB_MAX],	/* les titres de graphs */
	buffer[256],	/* intermediaire */
	type_ps_echelle[140],	/* legende pour l'echelle de Y */
	fichier[256],	/* nom du fichier traite */
	date[20], heure[20], prompt[81];

int	nb_plot,
	nb_iter,
	nb_neur,
	nb_clusters,
	cluster_size[NB_MAX+1],
	cluster_lim[NB_MAX+1],
	start,
	duree;
	
float	temps_max, temps_min,
        p_mb_max, p_mb_min, p_mb_dyn,
	nb_data[NB_MAX],
	plot_height,
	plot_width,
        x[NB_ITER_MAX+2],
        y[NB_ITER_MAX+2],
        z[NB_ITER_MAX+2],
        temps[NB_ITER_MAX+2],
	p_mb[NB_NEUR_MAX][NB_ITER_MAX+2],
	f_debut, f_fin,
	x_posi(),
	y_posi();

char	w[NB_ITER_MAX+2],
	p_spk[NB_NEUR_MAX][NB_ITER_MAX+2];

FILE	*p_data,	/* experimental file (*.sim) */
	*p_ps,		/* ps file at the end (*.ps) */
	*p_prm,		/* parameter file (*.prm) */
	*p_rot;		/* fichier des rotations */
	
main(argc,argv)
int argc;
char **argv;
{

	int	i, j, k,	/* un indice bon a tout */
		f_n_l,
		cluster_current,
		cpt_z, z_tick, z_tick_def = 10,/*
					        * tick mark sur les Z
					        * (clusters limites)
					        */
		nb_traces,
		nb_ps_points,
		debut,		/* limites temporelles du plot */
		fin;

	char	choix_y;
	char    rep;
	char    *p_c;		/* pointeur */

	float	x_scale, y_scale,	/* origine */
		x_gen_scale, x_gen_scale_def,	/* ps_echelle globale */
		y_gen_scale, y_gen_scale_def,	/* ps_echelle globale */
		bout, facteur, x_fin, real_z,
		x_l, y_l, x_f, y_f, z_f,		/* intermediaires */
		big, medium, little,
		x_rot, y_rot,		/* valeurs des rotations */
		x_rot_def = 0., y_rot_def = 20.,	/* valeurs des rotations */
		z_decal, z_decal_def,	/* espace netre courbes */
		yeux, yeux_def = 0.,
		fenetre, fenetre_def = 0.,		/* parametres de la perspective */
		trame_light, trame_light_def = 1.,	/* grises */
		trame_dark, trame_dark_def = 0.5,
		trame_his, trame_def, 
		trame, old_trame;


		/* traitement de la ligne de commande */

	strcpy(file,get_simulation_name());

	printf("Simulation name: %s\n",file);
	G_sim = defaut = verbose = FALSE;

#if 0
	if (argc == 1)
	{
	        if (file)
		  asksval("\nName of the *.pmb file to draw [.sim]: \n",
			  file, file);
		else
		  asksval("\nName of the *.pmb file to draw [.sim]: \n",
			  "* to choose an existing file",file);

		while (tolower(file[0]) == '*')
		{
			choose_file(".","pmb",file);
			p_c = strrchr(file,'.');
			if (p_c) *p_c = 0;			/* supprime l'extension */
			asksval("\nName of the file: ",file,file);
		} /* fin du if */
	}
#endif

	if (argc > 1)
	{
		for (i=1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf(usage,argv[0]);
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'd':
						defaut = TRUE;
						break;
					case 'k':
						keep = 's';
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
				}
			}
			/*
			else strcpy(file,argv[i]);
			*/
		}
#if 0
		if (!file)
		{
			asksval("\nName of the *.pmb file to draw [.pmb]: ",
				"* to choose an existing file",file);

			while (tolower(file[0]) == '*')
			{
				choose_file(".","pmb",file);
				p_c = strrchr(file,'.');
				if (p_c) *p_c = 0;			/* supprime l'extension */
				asksval("\nName of the file: \n",file,file);
			} /* fin du if */
		}
#endif
	}
        if (file)
	  asksval("\nName of the *.pmb file to draw [.pmb]: \n",
		  file, file);
	else
	  asksval("\nName of the *.pmb file to draw [.pmb]: \n",
		  "* to choose an existing file",file);
	while (tolower(file[0]) == '*')
	{
		choose_file(".","pmb",file);
		p_c = strrchr(file,'.');
		if (p_c) *p_c = 0;			/* supprime l'extension */
		asksval("\nName of the file: ",file,file);
	} /* fin du if */

	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}

	/* on a dans file le nom generique */
#if 0

	
	asksval("\nName of the *.pmb file to draw [.pmb] (* to change): \n"
		,file,file);
	while (tolower(file[0]) == '*')
	{
		choose_file(".","pmb",file);
		p_c = strrchr(file,'.');
		if (p_c) *p_c = 0;			/* supprime l'extension */
		asksval("\nName of the file: \n",file,file);
	} /* fin du if */
	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}
	else strcpy(f_data,file);
	/* on a dans file le nom generique */

#endif
        strcpy(f_data,file);
	strcat(f_data,".pmb");

        while ((p_data=fopen(f_data,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_data);
		rep = askcval("Would you like to choose an other file ([y] or n): ",'y');
		if (strchr("yYoOsS",rep))
		{
			asksval("\nName of the *.sim file to draw [.pmb]: ",
				"* to choose an existing file",file);

			while (tolower(file[0]) == '*')
			{
				choose_file(".","pmb",file);
				p_c = strrchr(file,'.');
				if (p_c) *p_c = 0;			/* supprime l'extension */
				asksval("\nName of the file: \n",file,file);
			} /* fin du if */
			f_n_l=inchr(file,'.');
			if (f_n_l != -1)
			{
				printf("\007Extension is removed...\n");
				strncpy(file,file,f_n_l);
			}
			else strcpy(f_data,file);
			/* on a dans file le nom generique */

			strcat(f_data,".pmb");
		} /* fin du if */
		else
		{
			exit(1);
		} /* fin du else */
	}

	strcpy(f_param,file);
	strcat(f_param,".prm");
	strcpy(f_rot_def,file);
	strcat(f_rot_def,".rot");
	
	printf("%s opened...\n",f_data);
        if ((p_prm=fopen(f_param,"r")) == NULL)
	{
		printf("the file %s cannot be opened...\n ",f_param);
		exit(1);
	}
	printf("%s opened...\n",f_param);
        if ((p_rot=fopen(f_rot_def,"r")) == NULL)
	{
		x_rot_def = 30.;
		y_rot_def = 50.;
		z_decal_def = 0.5;
		z_tick_def = 5;
		yeux_def = fenetre_def = 0.;
		trame_def = 'n';
		trame_dark_def = 0.5;
		trame_light_def = 0.9;
		x_gen_scale_def = y_gen_scale_def = 1.;
		printf("the file %s cannot be opened... \n",f_rot_def);
	} /* fin du if */
	else
	{
		x_rot_def = faskfval(p_rot,30.);
		y_rot_def = faskfval(p_rot,50.);
		z_decal_def = faskfval(p_rot,0.5);
		z_tick_def = faskival(p_rot,5);
		yeux_def = faskfval(p_rot,0.);
		fenetre_def = faskfval(p_rot,0.);
		trame_dark_def = faskfval(p_rot,0.4);
		trame_light_def = faskfval(p_rot,0.9);
		x_gen_scale_def = faskfval(p_rot,1.);
		y_gen_scale_def = faskfval(p_rot,1.);
		fclose(p_rot);
	} /* fin du else */


	/*
	 * lecture des donnees originales pour connaitre le nb de data
	 * et les max et min
	 */
	read_param();		/* lectures de tailles des clusters */

	for(i = 1; i <= nb_clusters; i++)
	{
		if (verbose)
		{
			printf("Cluster %d: %d neurons\n",
			       i, cluster_size[i]);
		} /* fin du if */
	}
	cluster_lim[0] = 0;
	for(i = 1; i <= nb_clusters; i++)
	{
		cluster_lim[i] = cluster_lim[i-1] + cluster_size[i];
	}
		
	if(verbose) printf("File to be plot: %s\n",f_data);

	if (!defaut)
	{
		sprintf(buffer,"Beginning of the plot [0] ms: ",0);
		debut = askival(buffer,0);
		do
		{
			sprintf(buffer,"End of the plot [the end] ms: ",NB_ITER_MAX);
			fin = askival(buffer,NB_ITER_MAX);
		} while (fin <= debut); /* fin du do */
		puts("Rotations:\n");
		/*
		*	On reclame les parametres 3D 
		*/
	
		printf("\n");
		askfdomain(-90.,90.);
		x_rot = askfval("\tGive an X-rotation angle (-90 to 90) : ",x_rot_def);
		y_rot = askfval("\tGive an Y-rotation angle (-90 to 90) : ",y_rot_def);
		askfdomain(0.,20.);
		z_decal = askfval("\tGive a Z offset in cm : ",z_decal_def);
		askfdomain(0.,0.);
		yeux = askfval("\tGive the distance eye <-> plot origin : ",yeux_def);
		fenetre = askfval("\tGive the distance eye <-> screen [infinite (no perspective)]: ",fenetre_def);
		z_tick = askival("Number of skipped traces for Z-scale : ",z_tick_def);

		x_gen_scale = askfval("General size X scaling of the figure : ",x_gen_scale_def); 
		y_gen_scale = askfval("General size Y scaling of the figure : ",y_gen_scale_def); 

		askfdomain(0.,1.);
		trame_light = askfval("Light value (0.(black)-1.(white)) : ",trame_light_def);
		trame_dark = askfval("Dark value (0.(black)-1.(white)) : ",trame_dark_def);

		trame_def = trame_his;

	        if ((p_rot=fopen(f_rot_def,"w")) == NULL)
		{
			printf("\007Cannot write the parameter file %s, sorry...\n",f_rot_def);
		} /* fin du if */
		else
		{
			fprintf(p_rot,"%f\n%f\n%f\n%d\n%f\n%f\n",
				x_rot, y_rot,
				z_decal, z_tick, yeux, fenetre);
			fprintf(p_rot,"%f\n%f\n%f\n%f\n",
				trame_dark, trame_light,
				x_gen_scale, y_gen_scale);
			fclose(p_rot);
		} /* fin du else */
	}
	else
	{
		debut = 0;
		fin = NB_ITER_MAX - 1;
		printf ("Rotation X: %.2f  Rotation Y: %.2f\n", x_rot, y_rot);
		z_decal = z_decal_def;
		yeux = fenetre = 0.;
		z_tick = z_tick_def;
		trame_light = trame_dark = trame_light_def;
	} /* fin du else */
	duree = fin - debut;
	if (duree >= NB_ITER_MAX - 1 )
	{
		printf("\007WARNING: no more than %d ms for 1 plot\n\
If you need more you must draw several plots\n", NB_ITER_MAX);
		fin = debut + NB_ITER_MAX -1;
	} /* fin du if */
	f_fin = (float)fin;
	f_debut = (float)debut;

	nb_iter = read_data();
	if (verbose) printf("Number of iterations: %d\n",nb_iter);

	if ( temps_max < f_fin)
	{
		fin = (int)temps_max;
		f_fin = (float)fin;
		duree = fin - debut;
	}

		/* creation du fichier de sortie 
			et initialisation PS */

	strcpy(f_ps,file);
	strcat(f_ps,"_pmb_3d.ps");
	if(verbose) printf("PostScript file to be produced: %s\n",f_ps);

	if (x_rot >= 0.)
	{
		x_f = temps_max - f_debut;
		y_f = 0.;
		z_f = (float)nb_neur * (-z_decal);
		x_fin = x_posi(x_f,y_f,z_f,x_rot,(y_rot+90.),yeux,fenetre);
		offset = 0.;
	}
	else
	{	
		x_f = 0.;
		y_f = 0.;
		z_f = (float)nb_neur * (-z_decal);
		offset = x_posi(x_f,y_f,z_f,x_rot,(y_rot+90.),yeux,fenetre);
		offset *= (-1.);
		x_f = (float)nb_iter;
		z_f = 0.;
		x_fin = x_posi(x_f,y_f,z_f,x_rot,(y_rot+90.),yeux,fenetre);
	}

	if (verbose) printf("Offset -> %f\n", offset);
	
	ps_iniplot(f_ps,B_PAP);

	titre(x_rot,y_rot,z_decal,x_gen_scale,y_gen_scale);
	
	ps_origin(5.,9.,0.,0.);

	ps_scale(x_gen_scale,y_gen_scale);
	
	plot_height = 23./3.;
	plot_width = 16./1.5;
	fclose(p_data);
	start = FALSE;
	p_mb_dyn = p_mb_max - p_mb_min;
	if (! duree)
	{
		printf("Null duration, sorry...\n");
		exit(1);
	} /* fin du if */
	if (!(p_mb_dyn))
	{
		printf("Pot mb min = pot mb max, sorry...\n");
		exit(1);
	} /* fin du if */

	y_l = y_posi(0.,p_mb_dyn,0.,x_rot,(y_rot+90.),yeux,fenetre);

	x_scale = plot_width / (float)(duree);	/*  ps_echelle des x */
	y_scale = (plot_height) / (p_mb_dyn)  ;	/* ps_echelle des y en mV */
	if(verbose) printf("X_scale -> %f   Y_scale -> %f\n",x_scale,y_scale);

	ps_echel(x_scale,y_scale,0.,0.,1.,0.);
	ps_user();

	nb_traces = nb_neur;
	cpt_z = nb_traces;
	trame = trame_light;	/* on commence par le fond fonce */
	old_trame = trame_dark;
	
                /* le soubassement */
        temps[nb_iter] = temps[nb_iter-1];
        temps[nb_iter+1] = temps[0];
	for (j =0; j < nb_iter + 2; j++)
		temps[j] -= f_debut;
	
	cluster_current = nb_clusters;
	real_z = z_decal;
	nb_ps_points = nb_iter + 2;

	if (verbose)
		printf("Nb of neuron in cluster %d: %d\n", cluster_current, nb_neur); 

	for (i = nb_neur-1; i >= 0 ; i--)
	{
		if (i+1 == cluster_lim[cluster_current])
		{
			float inter;
			
			inter = trame;
			trame = old_trame;
			old_trame = inter;
			printf("\nPlotting Cluster %d\n",cluster_current);
			cluster_current--;
		} /* fin du if */
		printf("\tunit %3d...\r",i+1);

                p_mb[i][nb_iter] = p_mb[i][nb_iter+1] = p_mb_min; 

                for (j = 0; j < nb_ps_points; j++)
		{
			x[j] = temps[j];
			y[j] = p_mb[i][j] - p_mb_min;
			z[j] = (float)i * real_z;
		} /* fin du for */

		if (verbose)
		{
			for (j =0; j < nb_ps_points; j++)
				printf("x: %f y: %f z: %f\n",
					x[j],y[j],z[j]);
		} /* fin du if */
                psplot3d(x,y,z,x_rot,(y_rot+90.),yeux,fenetre,plot_width,nb_ps_points,trame);
#if 0
		if (cpt_z % z_tick == 0)
		{
			x_l = x_posi(temps_max,0.,((float)i * real_z),x_rot,(y_rot+90.),yeux,fenetre);
			y_l = y_posi(temps_max,0.,((float)i * real_z),x_rot,(y_rot+90.),yeux,fenetre);
			ps_drawa(x_l+offset,y_l);
			ps_chara(0.,-.2,REL_HAUTE,&start[9],0.4,0); 
			cpt_z = 0;
		}
#endif
		cpt_z--;
		nb_traces --;
        }
	if (verbose)
		printf("Plotting scales...\n");

	printf("\n");

	ps_chara(2.,-0.5,ABS_HAUTE,type_ps_echelle,0.4,0);
	ps_endplot(f_ps,keep);
	exit(0);
}
/*
*
*	READ_DATA() lit les donnees 
*	sur le fichier pour connaitre le nombre de 
*	donnees et les max et min et les stocker dans le tqblequ p_mb 
*
*/
read_data()
{
	float	x_i, y_i, y_s;
	int	nb_unit, nb_pt, num;
	char	started = 0;
		
	nb_unit = 0;
        nb_iter = -1;
	nb_pt = -1;
	temps_max = 0.;
        p_mb_max = -1e15;
        p_mb_min = 1e15;
	started = 0;
        
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[strlen(buffer)-1] = 0;
			nb_iter++;
			if (nb_iter > (int)f_fin) break;
			if (nb_iter <= (int)f_debut)
				continue;
			else
				started = 1;
                        nb_unit = 0;
			nb_pt ++;
			temps[nb_pt] = (float)nb_iter;
			if (!(nb_iter % 100)) printf("Reading iteration %s ms...\r",&buffer[1]);
			fflush(stdout);
		} /* fin du if */
		else
		{
			if (!started)
				continue;
			if (buffer[0] == '\n') continue;
			if (strstr(buffer," 100.0 "))
			{
				sscanf(buffer,"%d %f %f %f",&num, &y_i, &y_s, &x_i);
				p_mb[nb_unit][nb_pt] = y_i;
				p_spk[nb_unit][nb_pt] = 'S';
			} /* fin du if */
			else
			{
				sscanf(buffer,"%d %f",&num, &y_i);
				p_mb[nb_unit][nb_pt] = y_i;
				p_spk[nb_unit][nb_pt] = 'R';
			} /* fin du else */
			if (p_mb[nb_unit][nb_pt] >  p_mb_max)
                                p_mb_max = p_mb[nb_unit][nb_pt];
			if (p_mb[nb_unit][nb_pt] < p_mb_min)
                                p_mb_min = p_mb[nb_unit][nb_pt];
			if (verbose) printf(" c # %d -> %f mV\n",
                                nb_unit,p_mb[nb_unit][nb_pt]);
			nb_unit++;
		} /* fin du else */
	} /* fin du while */
	temps_max = (float)nb_iter+1.;
	if (verbose) printf("Number of iterations: %d\nMin = %f Max = %f\n",
		nb_pt+1, p_mb_min, p_mb_max);
	if (!G_sim)
		p_mb_max = TOP_SPIKE;
	return(nb_pt+1);

#if OLD

	float	x_i, y_i;
	int	nb_unit, nb_pt;
		
	nb_unit = 0;
        nb_iter = -1;
	nb_pt = -1;
	temps_max = 0.;
        p_mb_max = -1e15;
        p_mb_min = 1e15;
        
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[strlen(buffer)-1] = 0;
			nb_iter++;
			if (nb_iter > (int)f_fin) break;
			if (nb_iter <= (int)f_debut) continue;
                        nb_unit = 0;
			nb_pt ++;
			temps[nb_pt] = (float)nb_iter;
			if (!(nb_iter%100))
				printf("Reading iteration %s ms...\n",&buffer[1]);
		} /* fin du if */
		else
		{
			if (buffer[0] == '\n') continue;
			if (strchr(buffer,'A'))
			{
				sscanf(buffer,"%f %f %c",&x_i,&y_i,&spike);
			} /* fin du if */
			else
			{
				sscanf(buffer,"%f %f",&x_i,&y_i);
			} /* fin du else */
			p_mb[nb_unit][nb_pt] = y_i;
			if (p_mb[nb_unit][nb_pt] >  p_mb_max)
                                p_mb_max = p_mb[nb_unit][nb_pt];
			if (p_mb[nb_unit][nb_pt] < p_mb_min)
                                p_mb_min = p_mb[nb_unit][nb_pt];
			if (verbose) printf(" c # %d -> %f mV\n",
                                nb_unit,p_mb[nb_unit][nb_pt]);
			nb_unit++;
		} /* fin du else */
	} /* fin du while */
	temps_max = (float)nb_iter+1.;
	if (verbose) printf("Number of iterations: %d\nMin = %f Max = %f\n",
		nb_pt+1, p_mb_min, p_mb_max);
	return(nb_pt+1);
#endif
}
/*
*
*	TITRE() ecrit la legende generale du dessin 
*
*/
titre(x_rot,y_rot,z_decal,x_gen_scale,y_gen_scale)
float	x_rot,
	y_rot,
	z_decal,
	x_gen_scale,
	y_gen_scale;
{
	static	char leg_txt[256];
	int	i;

	traa(0.,0.,ABS_HAUTE);
	sprintf(leg_txt,"%d cluster%c", nb_clusters,(nb_clusters)?'s':' ');
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,2.6,ABS_HAUTE,leg_txt,.4,0);
	sprintf(leg_txt,"File: %s", f_data);
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,2.1,ABS_HAUTE,leg_txt,.3,0);
	sprintf(leg_txt,"PS file: %s", f_data);
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,1.6,ABS_HAUTE,leg_txt,.3,0);
	sprintf(leg_txt,"Rot: X %.1f Y %.1f Z %.2f Scal: x %.2f y %.2f", 
			x_rot, y_rot, z_decal, x_gen_scale, y_gen_scale);
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.5,0.,REL_HAUTE,leg_txt,.3,0);
	traa(0.,0.,ABS_HAUTE);
}


/*
*
*	READ_PARAM Relecture des parametres de la simulation dans f_param
*
*/
read_param()
{
	int	i,
		k;
		
	char	inutile[80];
	
	if((p_prm = fopen(f_param,"r")) == NULL)
	{
		printf("Parameter file %s impossible to open as read\n", f_param);
		return;
	}
	/* nb de neurones */
	G_sim = FALSE;
	fgets(buffer,80,p_prm);
#if OLD
	if (buffer[0] == 'G')
	{
		G_sim = TRUE;
	} /* fin du if */
	if (buffer[0] == 'P')
	{
		G_sim = FALSE;
	} /* fin du if */
#endif
       	fgets(buffer,80,p_prm);
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_neur);
	       
	/* nb de clusters */
	fgets(buffer,80,p_prm);
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_clusters);

	k = 1;
	while (fgets(buffer,80,p_prm))
	{
		sscanf(buffer,"%s",inutile);
		if (!strcmp(inutile,"CLUSTER"))
		{
			fgets(buffer,80,p_prm);
			sscanf(buffer,"%s %s %s %d",
			       inutile,
			       inutile,
			       inutile,
			       &(cluster_size[k]));
			k++;
		} /* fin du if */
	} /* fin du while(fgets(buffer */
	fclose(p_prm);
} /* fin du read_param() */


/*
*
*	PSPLOT3D()
*
*	dessine des courbes en 3d en partant de la fin pour PostScript
*	a linker avec -lm et -lplot_ps
*
*	Attention, pour etre dans l'espace DIRECT, 
*		la profondeur est X
*		la largeur est Y
*		la hauteur est Z
*/


#define TRUE 1
#define FALSE 0



/*
#define DEBUG
#define DEBUG1
#define DEBUG3
#define DEBUG4
*/

#define	DEG_RAD	0.0174532925


psplot3d(y_coord,z_coord,x_coord,theta,phi,oeil,ecran,max_width,nb_xy,gris)
float	y_coord[],
	z_coord[],
	x_coord[],
	theta,
	phi,
	oeil,
	ecran,
	max_width;
int	nb_xy;
float	gris;
{

	int 	i, k;		/* des indices */
	int	perspective;	/* des flag */
	
	float	x_new, y_new, z_new, 	/* les coordonees apres rotation */
		x_crt, y_crt, horiz,
		pente, ord_orig,
		x_avt, y_avt;

	ps_user();
	if (ecran == 0.0) perspective = FALSE;
	else perspective = TRUE;

	if (plot_start)
	{
		/* calcul de  la matrice des rotations */

		theta *= DEG_RAD;
		phi *= DEG_RAD;
		s1 = sin(theta);
		s2 = sin(phi);
		c1 = cos(theta);
		c2 = cos(phi);
		s3 = c1 * s2;
		s4 = s1 * s2;
		s5 = c1 * c2;
		s6 = s1 * c2;
#ifdef DEBUG
	printf("s1 %f\ns2 %f\ns3 %f\ns4 %f\ns5 %f\ns16 %f\nc1 %f\nc2 %f\n",
				s1,s2,s3,s4,s5,s6,c1,c2);
#endif
	}

	for(i = 0; i < nb_xy; i++)
	{
#ifdef DEBUG
		printf("\n%d:\tx -> %f y -> %f\n", i, y_coord[i], z_coord[i]);
#endif

		x_new = -x_coord[i] * s1 + y_coord[i]  * c1;
		y_new = -x_coord[i] * s5 - y_coord[i]  * s6 + z_coord[i]  * s2;
		z_new = -x_coord[i] * s3 - y_coord[i]  * s4 - z_coord[i]  * c2 + oeil;
		y_coord[i] = (perspective)? (ecran * x_new / z_new) : x_new;
		z_coord[i] = (perspective)? (ecran * y_new / z_new) : y_new;
		/* on recale au centre pour etre tjs dans le masque */
		y_coord[i] += offset;
#ifdef DEBUG
		printf(">>>>>>>\tx -> %20.10f y -> %20.10f (offset = %f)\n", 
					y_coord[i], z_coord[i],offset);
#endif
	}

	if (plot_start)
	{
		plot_start = FALSE;
		plot_nb_line = 1;
	}
	else
	{
		plot_nb_line ++;
	}

		/* Trace de la courbe fermee plan */
	ps_movea(y_coord[0],z_coord[0]);
	ps_closed_curve(y_coord,z_coord,nb_xy,TRUE,TRUE,gris);
		
	x_extr = y_coord[0];
	y_extr = z_coord[0];

}
/*
*	X_POSI() renvoie la position du point en argument
*/
float x_posi(y_coord,z_coord,x_coord,theta,phi,oeil,ecran)
float	y_coord,
	z_coord,
	x_coord,
	theta,
	phi,
	oeil,
	ecran;
{

	int	perspective;	/* des flag */

	float	x_new, z_new; 	/* les coordonees apres rotation */

	if (ecran == 0.0) perspective = FALSE;
	else perspective = TRUE;

		/* calcul de  la matrice des rotations */

	theta *= DEG_RAD;
	phi *= DEG_RAD;
	s1 = sin(theta);
	s2 = sin(phi);
	c1 = cos(theta);
	c2 = cos(phi);
	s3 = c1 * s2;
	s4 = s1 * s2;
	s5 = c1 * c2;
	s6 = s1 * c2;

#ifdef DEBUG
	printf("s1 %f\ns2 %f\ns3 %f\ns4 %f\ns5 %f\ns16 %f\nc1 %f\nc2 %f\n",
				s1,s2,s3,s4,s5,s6,c1,c2);
#endif

#ifdef DEBUG
	printf("\n\tx -> %f y -> %f\n", y_coord, z_coord);
#endif
	x_new = -x_coord * s1 + y_coord  * c1;
	z_new = -x_coord * s3 - y_coord  * s4 - z_coord  * c2 + oeil;
	y_coord = (perspective)? (ecran * x_new / z_new) : x_new;
	return(y_coord);
}

/*
*	Y_POSI() renvoie la position du point en argument
*/
float y_posi(y_coord,z_coord,x_coord,theta,phi,oeil,ecran)
float	y_coord,
	z_coord,
	x_coord,
	theta,
	phi,
	oeil,
	ecran;
{

	int	perspective;	/* des flag */

	float	x_new, y_new, z_new; 	/* les coordonees apres rotation */

	if (ecran == 0.0) perspective = FALSE;
	else perspective = TRUE;

		/* calcul de  la matrice des rotations */

	theta *= DEG_RAD;
	phi *= DEG_RAD;
	s1 = sin(theta);
	s2 = sin(phi);
	c1 = cos(theta);
	c2 = cos(phi);
	s3 = c1 * s2;
	s4 = s1 * s2;
	s5 = c1 * c2;
	s6 = s1 * c2;

#ifdef DEBUG
	printf("s1 %f\ns2 %f\ns3 %f\ns4 %f\ns5 %f\ns16 %f\nc1 %f\nc2 %f\n",
				s1,s2,s3,s4,s5,s6,c1,c2);
#endif

#ifdef DEBUG
	printf("\n\tx -> %f y -> %f\n", y_coord, z_coord);
#endif
	x_new = -x_coord * s1 + y_coord  * c1;
	y_new = -x_coord * s5 - y_coord  * s6 + z_coord * s2;
	z_new = -x_coord * s3 - y_coord  * s4 - z_coord  * c2 + oeil;
	y_coord = (perspective)? (ecran * x_new / z_new) : x_new;
	z_coord = (perspective)? (ecran * y_new / z_new) : y_new;
	return(z_coord);
}
