/***********************************************************************
*
*		P M B _ C E L L _ N _ B _ C . C
*
*	Plot le potentiels de membrane d'unee cellule au choix
*	avec le meme format que splot.
*	Cree un fichier PostScript
*	Le fichier doit provenir de neuro_bio_clusters
*	
*		start:	29 may 1990
*
*	Modifs:
*		V1 29 jul 91	redessin des spikes pour pouvoir
*				dessiner des bursts
*		V2 2 nov 93	lit le *.prm au lieu du *.def
*				pour compatibilite avec nbc_g
*		V3 28 nov 95	lit 10000 ms pour 100 neurones
*
*		V4 8 aout 97	propose le nom de fichier meme si
*				donne en ligne de commande
*		V5 12 aout 97	Nouveau dessin de la fome des spikes
*
*		V6 26 aout 97	Nom des fichiers en clair
*
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Biomathematiques
*		URBB B3E ESI U444
*		Faculte de Medecine Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*
*=========================================================================
*
*	Must be loaded with -lm -lplot_ps and psplot3d.o
*
*
*=========================================================================
*
*	This program reads file *.pmb and creates a file *.PS to be
*	spooled on the LN03R
*
***************************************************************************/ 
#ifdef DOCUMENTATION

title	PMB_CELL_N_B_C     Plotting membrane potential data
index			Plotting membrane potential data

Synopsis

	pmb_cell_n_b_c [-v] [-d] [-k] [file_in[.pmb]]

Description

	PMB_CELL_N_B_C reads in the file {\tt file_in.pmb} et {\tt file_in.prm}
	created by neuro_bio_clusters providing a file of the following
	format:

	{\tt

	\nofill
		
		"Iteration 1"
		1 -70.0
		2 -80.0
		3 -50.5
		4 -80.0

		"Iteration 2"
		1 -72.0
		2 -78.0
		3 -60.5
		4 -80.0
		...
	\fill
	}
	
	The 1st number is the neuron number and the second the membrane
	potential in mV.

	It plots the graph for a PostScript printer. 

	It is possible to choose the way these plots are drawn (scales,
	lines, dots, begining and end of times), by answering a series of
	questions.
	

	{\tt -v} switch gives a verbose output on stdout

	{\tt -d} switch uses the defaults values silently
	
	{\tt -k} is to keep the PostScript file instead of spooling it
	
	The output file file_in.PS is spooled on the PostScript printer, and
	then deleted (if not {\tt -k}).

	Users of workstation should use visu_n_b_c for X colors
	visualisation and plots

Files	

	Input files: file_in.pbm file_in.prm file_in.def

	Output files: file_in.ps

See also

	visu_n_b_c
	
Diagnostics

	Self explaining diagnostics are provided on fatal errors.

Authors

	Dr Jean-Francois Vibert 

	CHU Saint-Antoine Univ. P. & M. Curie PARIS

Bugs/deficiencies

	No known bugs.
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <askval.h>
#include <rsx.h>
#include <plot_ps.h>
#include <math.h>
#include "find_files.h"

#undef CLASS

/*
#define TRACE
#define DEBUG
*/

/*	Pour la Laser */

#define B_PAP 21.
#define ABS_HAUTE 0
#define ABS_BASSE 1
#define REL_HAUTE 2
#define REL_BASSE 3

#define NB_ITER_MAX 2000
#define TOP_SPIKE   100.0

#define MAIN
#	include "cstes_nbc.h"
#undef MAIN

/* Car non dynamiquement alloue pour l'instant! */
/*
#define NB_CLUST_MAX	10
#define NB_NEUR_MAX	10
*/

#define NB_MAX NB_CLUST_MAX


char	*usage = "%s [?] [-v] [-d] [-k] [file]\n",
	defaut,		/* flag "default" */
	spike,		/* flag, c'est un spike */
	G_sim,		/* ca vient de nbc_g */
	verbose,	/* flag bavard */
	keep = 'd',	/* flag keep the file (don't spool it) */
	reponse[256],	/* buffer pour les legendes */
	file[256],	/* generic file name */
	f_ps[256],	/* experimental file (*.bsp) */
	f_data[256],	/* simulation at the end (*.pmb) */
	f_param[256],	/* parameters  (*.prm) */
	f_defaut[256],	/* fichier des parametres */
	f_rot_def[256],	/* fichier des rotations */
	fix_scal,	/* flag ps_echelle fixe */
	prcntg,		/* flag ps_echelle en pct */
	*legend[NB_MAX],	/* les titres de graphs */
	buffer[256],	/* intermediaire */
	type_ps_echelle[256],	/* legende pour l'echelle de Y */
	fichier[256],	/* nom du fichier traite */
	date[20], heure[20], prompt[256];

int	nb_plot,
	nb_iter,
	nb_neur,	/* nb de neurones */
	nb_clusters,
	cluster_size[NB_MAX+1],
	cluster_lim[NB_MAX+1],
	nb_cell,		/* numero de la cellule a tracer */
	les_cells[100],		/* les numeros des cellules a dessiner */
	num_cell,		/* le nombre de cellules */
	start,
	duree;
	
float	temps_max, temps_min,
        p_mb_max, p_mb_min, p_mb_dyn,
	nb_data[NB_MAX],
	plot_height,
	zoom,
	plot_width,
	demi_it = 0.5,		/* demi iteration (nlt a 0.5 ms) */
        x[NB_ITER_MAX+2],
        y[NB_ITER_MAX+2],
        temps[NB_ITER_MAX+2],
	p_mb[NB_NEUR_MAX][NB_ITER_MAX+2],
	p_post[NB_NEUR_MAX][NB_ITER_MAX+2],
	f_debut, f_fin;

char	p_spk[NB_NEUR_MAX][NB_ITER_MAX+2],
        z[NB_ITER_MAX+2];

FILE	*p_data,	/* experimental file (*.sim) */
	*p_ps,		/* ps file at the end (*.ps) */
	*p_prm;		/* parameter file (*.prm) */
	
main(argc,argv)
int argc;
char **argv;
{

	int	i, j, k,	/* un indice bon a tout */
		f_n_l,
		cluster_current,
		nb_traces,
		nb_ps_points,
		debut,		/* limites temporelles du plot */
		fin;

	char	choix_y;
	char    rep;
	char    *p_c;		/* pointeur */

	float	x_scale, y_scale,	/* origine */
		x_gen_scale, x_gen_scale_def,	/* ps_echelle globale */
		y_gen_scale, y_gen_scale_def,	/* ps_echelle globale */
		bout, facteur, x_fin, real_z,
		x_l, y_l, x_f, y_f, z_f,		/* intermediaires */
		big, medium, little, volt;

		/* traitement de la ligne de commande */

	strcpy(file,get_simulation_name());

	printf("Simulation name: %s\n",file);
	G_sim = defaut = verbose = FALSE;

#if 0
	if (argc == 1)
	{
	        if (file)
		  asksval("\nName of the *.pmb file to draw [.sim]: \n",
			  file, file);
		else
		  asksval("\nName of the *.pmb file to draw [.sim]: \n",
			  "* to choose an existing file",file);

		while (tolower(file[0]) == '*')
		{
			choose_file(".","pmb",file);
			p_c = strrchr(file,'.');
			if (p_c) *p_c = 0;			/* supprime l'extension */
			asksval("\nName of the file: ",file,file);
		} /* fin du if */
	}
#endif

	if (argc > 1)
	{
		for (i=1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf(usage,argv[0]);
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'd':
						defaut = TRUE;
						break;
					case 'k':
						keep = 's';
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
				}
			}
			/*
			else strcpy(file,argv[i]);
			*/
		}
#if 0
		if (!file)
		{
			asksval("\nName of the *.pmb file to draw [.pmb]: ",
				"* to choose an existing file",file);

			while (tolower(file[0]) == '*')
			{
				choose_file(".","pmb",file);
				p_c = strrchr(file,'.');
				if (p_c) *p_c = 0;			/* supprime l'extension */
				asksval("\nName of the file: \n",file,file);
			} /* fin du if */
		}
#endif
	}
        if (file)
	  asksval("\nName of the *.pmb file to draw [.pmb]: \n",
		  file, file);
	else
	  asksval("\nName of the *.pmb file to draw [.pmb]: \n",
		  "* to choose an existing file",file);
	while (tolower(file[0]) == '*')
	{
		choose_file(".","pmb",file);
		p_c = strrchr(file,'.');
		if (p_c) *p_c = 0;			/* supprime l'extension */
		asksval("\nName of the file: ",file,file);
	} /* fin du if */

	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}

	/* on a dans file le nom generique */
#if 0

	
	asksval("\nName of the *.pmb file to draw [.pmb] (* to change): \n"
		,file,file);
	while (tolower(file[0]) == '*')
	{
		choose_file(".","pmb",file);
		p_c = strrchr(file,'.');
		if (p_c) *p_c = 0;			/* supprime l'extension */
		asksval("\nName of the file: \n",file,file);
	} /* fin du if */
	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}
	else strcpy(f_data,file);
	/* on a dans file le nom generique */

#endif
        strcpy(f_data,file);
	strcat(f_data,".pmb");
	
        while ((p_data=fopen(f_data,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_data);
		rep = askcval("Would you like to choose an other file ([y] or n): ",'y');
		if (strchr("yYoOsS",rep))
		{
			asksval("\nName of the *.pmb file to draw [.pmb]: ",
				"* to choose an existing file",file);

			while (tolower(file[0]) == '*')
			{
				choose_file(".","pmb",file);
				p_c = strrchr(file,'.');
				if (p_c) *p_c = 0;			/* supprime l'extension */
				asksval("\nName of the file: \n",file,file);
			} /* fin du if */
			f_n_l=inchr(file,'.');
			if (f_n_l != -1)
			{
				printf("\007Extension is removed...\n");
				strncpy(file,file,f_n_l);
			}
			else strcpy(f_data,file);
			/* on a dans file le nom generique */

			strcat(f_data,".pmb");
		} /* fin du if */
		else
		{
			exit(1);
		} /* fin du else */
	}


	strcpy(f_param,file);
	strcat(f_param,".prm");
	
	printf("%s opened...\n",f_data);
        if ((p_prm=fopen(f_param,"r")) == NULL)
	{
		printf("the file %s cannot be opened...\n ",f_param);
		exit(1);
	}
	printf("%s opened...\n",f_param);

	/*
	 * lecture des donnees originales pour connaitre le nb de data
	 * et les max et min
	 */
	read_param();		/* lectures de tailles des clusters */
		 
	if (G_sim)
		printf("Conductance simulation\n");
	else
		printf("Phenomenologic simulation\n");
	
	for(i = 1; i <= nb_clusters; i++)
	{
		if (verbose)
		{
			printf("Cluster %d: %d neurons\n",
			       i, cluster_size[i]);
		} /* fin du if */
	}
	cluster_lim[0] = 0;
	for(i = 1; i <= nb_clusters; i++)
	{
		cluster_lim[i] = cluster_lim[i-1] + cluster_size[i];
	}

		
	if(verbose) printf("File to be plot: %s\n",f_data);

	if (!defaut)
	{
		sprintf(buffer,"Beginning of the plot [0] ms: ",0);
		debut = askival(buffer,0);
		do
		{
			sprintf(buffer,"End of the plot [the end] ms: ");
			fin = askival(buffer,NB_ITER_MAX);
		} while (fin <= debut); /* fin du do */
	}
	else
	{
		debut = 0;
		fin = NB_ITER_MAX - 1;
	} /* fin du else */
	duree = fin - debut;
	if (duree >= NB_ITER_MAX - 1 )
	{
		printf("\007WARNING: no more than %d ms for 1 plot\n\
If you need more you must draw several plots\n", NB_ITER_MAX);
		fin = debut + NB_ITER_MAX - 1;
	} /* fin du if */
	f_fin = (float)fin;
	f_debut = (float)debut;

	num_cell = 0;
	do
	{
		nb_cell = askival("Number of the cell to draw (End = [0]): ",0);
		les_cells[num_cell] = nb_cell -1; /* Donc -1 indique la fin */
		num_cell++;
	} while(nb_cell); /* fin du do */

	zoom = askfval("Y amplification factor: ",1.);
	nb_iter = read_data();
	if (verbose) printf("Number of iterations: %d\n",nb_iter);

	if ( temps_max < f_fin)
	{
		fin = (int)temps_max;
		f_fin = (float)fin;
		duree = fin - debut;
	}

		/* creation du fichier de sortie 
			et initialisation PS */

	strcpy(f_ps,file);
	strcat(f_ps,"_intra_cell.ps");
	if(verbose) printf("PostScript file to be produced: %s\n",f_ps);


	ps_iniplot(f_ps,B_PAP);

	titre();
	
	ps_origin(1.5,4.,0.,0.);

	
	plot_height = 22./(num_cell + 1);
	plot_width = 16.;
	fclose(p_data);
	start = FALSE;


	if (G_sim)
		p_mb_dyn = p_mb_max - p_mb_min;
	else
		p_mb_dyn = TOP_SPIKE - p_mb_min;

	if (! duree)
	{
		printf("Duree nulle, sorry...\n");
		exit(1);
	} /* fin du if */
	if (!(p_mb_dyn))
	{
		printf("Pot mb min = pot mb max, sorry...\n");
		exit(1);
	} /* fin du if */

	plot_height *= zoom;
	x_scale = plot_width / (float)(duree);	/*  ps_echelle des x */
	y_scale = plot_height / (p_mb_dyn)  ;	/* ps_echelle des y en mV */
	if(verbose) printf("X_scale -> %f   Y_scale -> %f\n",x_scale,y_scale);

	ps_echel(x_scale,y_scale,0.,0.,1.,0.);
	ps_user();

	/* Axe des Temps (dans le sens X benson) */

	/* retour au 0 */

	ps_movea(0.,0.);
	ps_drawa((float)(duree),0.);
	if  (duree <= 10) 
	{
		big = 5.;
		medium = 1.;
		little = 1.;
	}
	if (duree > 10 && duree <= 100) 
	{
		big = 10.;
		medium = 5.;
		little = 1.;
	}
	if (duree > 100 && duree <=750) 
	{
		big = 50.;
		medium = 25.;
		little = 5.;
	}
	if (duree > 750 && duree <= 1000) 
	{
		big = 100.;
		medium = 50.;
		little = 10.;
	}
	if (duree > 1000 && duree <= 2000) 
	{
		big = 500.;
		medium = 100.;
		little = 50.;
	}
	if (duree > 2000 && duree <= 5000) 
	{
		big = 1000.;
		medium = 500.;
		little = 250.;
	}
	if (duree > 5000 && duree <= 10000) 
	{
		big = 5000.;
		medium = 1000.;
		little = 500.;
	}
	
	x_tick((float)debut,(float)fin,big,medium,little,"Time (ms)");

	nb_traces = num_cell;

	for (j =0; j < nb_iter + 2; j++)
		temps[j] -= f_debut;
	
	nb_ps_points = nb_iter;
	for (i = 0; i < num_cell-1 ; i++)
	{
		sprintf(buffer,"Unit %d",les_cells[i]+1);
		ps_chara(-1.5,0.,ABS_HAUTE,buffer,.4,90);

		/* Axe des Y */

		big = 20.;
		ps_movea(0.,0.);
		traa(0.,plot_height,ABS_BASSE);
		traa(-0.2,0.,REL_BASSE);
		ps_movea(0.,0.);
		traa(-0.2,0.,REL_BASSE);
		ps_movea(0.,0.);
		for (volt = 0.; volt < p_mb_max; volt += big)
		{
			ps_movea(0.,volt - p_mb_min);
			traa(-0.2,0.,REL_BASSE);
			sprintf(buffer,"%.0f",volt);
			ps_chara(-0.5,0.,REL_HAUTE,buffer,0.3,0);
		} /* fin du for */
		for (volt = -big; volt > p_mb_min; volt -= big)
		{
			ps_movea(0.,volt - p_mb_min);
			traa(-0.2,0.,REL_BASSE);
			sprintf(buffer,"%.0f",volt);
			ps_chara(-0.5,0.,REL_HAUTE,buffer,0.3,0);
		} /* fin du for */

                for (j = 0; j < nb_ps_points; j++)
		{
			x[j] = temps[j];
			y[j] = p_mb[les_cells[i]][j] - p_mb_min;
			z[j] = p_spk[les_cells[i]][j];
		} /* fin du for */

		if (verbose)
		{
			for (j =0; j < nb_ps_points-1; j++)
				printf("x: %f y: %f\n",
					x[j],y[j]);
		} /* fin du if */
		/*
		 * Dessin proprement dit
		 */
		ps_movea(x[0],y[0]);
		for (j =1; j < nb_ps_points-1; j++)
		{
			if (!G_sim && z[j] == 'S') /* on a un spike, on le dessine bien */
			{
				ps_drawa((x[j]-(2./3.)),y[j]);	/* monte en 0.3 ms */
				ps_drawa((x[j]-(1./3.)),TOP_SPIKE - p_mb_min);	/* monte en 0.3 ms */
				ps_drawa(x[j],p_post[les_cells[i]][j] - p_mb_min);	/* descend en 0.3 ms */
			} /* fin du else */
			else	
				ps_drawa(x[j],y[j]);
		} /* fin du for */
		ps_origin(0.,plot_height+.5,0.,0.);
        }
	printf("\n");
	ps_chara(2.,-0.5,ABS_HAUTE,type_ps_echelle,0.4,0);
	ps_endplot(f_ps,keep);
}
/*
*
*	READ_DATA() lit les donnees 
*	sur le fichier pour connaitre le nombre de 
*	donnees et les max et min et les stocker dans le tqblequ p_mb 
*
*/
read_data()
{
	float	x_i, y_i, y_s;
	int	nb_unit, nb_pt, num;
	char	started = 0;
		
	nb_unit = 0;
        nb_iter = -1;
	nb_pt = -1;
	temps_max = 0.;
        p_mb_max = -1e15;
        p_mb_min = 1e15;
	started = 0;
        
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[strlen(buffer)-1] = 0;
			nb_iter++;
			if (nb_iter > (int)f_fin) break;
			if (nb_iter <= (int)f_debut)
				continue;
			else
				started = 1;
                        nb_unit = 0;
			nb_pt ++;
			temps[nb_pt] = (float)nb_iter;
			if (!(nb_iter % 100)) printf("Reading iteration %s ms...\r",&buffer[1]);
			fflush(stdout);
		} /* fin du if */
		else
		{
			if (!started)
				continue;
			if (buffer[0] == '\n') continue;
			if (strstr(buffer," 100.0 "))
			{
				sscanf(buffer,"%d %f %f %f",&num, &y_i, &y_s, &x_i);
				p_mb[nb_unit][nb_pt] = y_s;
				p_spk[nb_unit][nb_pt] = 'S';
				p_post[nb_unit][nb_pt] = x_i;
			} /* fin du if */
			else
			{
				sscanf(buffer,"%d %f",&num, &y_i);
				p_mb[nb_unit][nb_pt] = y_i;
				p_spk[nb_unit][nb_pt] = 'R';
			} /* fin du else */
			if (p_mb[nb_unit][nb_pt] >  p_mb_max)
                                p_mb_max = p_mb[nb_unit][nb_pt];
			if (p_mb[nb_unit][nb_pt] < p_mb_min)
                                p_mb_min = p_mb[nb_unit][nb_pt];
			if (verbose) printf(" c # %d -> %f mV\n",
                                nb_unit,p_mb[nb_unit][nb_pt]);
			nb_unit++;
		} /* fin du else */
	} /* fin du while */
	temps_max = (float)nb_iter+1.;
	if (verbose) printf("Number of iterations: %d\nMin = %f Max = %f\n",
		nb_pt+1, p_mb_min, p_mb_max);
	if (!G_sim)
		p_mb_max = TOP_SPIKE;
	return(nb_pt+1);
}
/*
*
*	TITRE() ecrit la legende generale du dessin 
*
*/
titre()
{
	static	char leg_txt[256];
	int	i;

	traa(0.,0.,ABS_HAUTE);
	sprintf(leg_txt,"%d nucl%s", nb_clusters,(nb_clusters > 1)?"ei":"us");
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,2.6,ABS_HAUTE,leg_txt,.4,0);
	sprintf(leg_txt,"File: %s", f_data);
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,2.1,ABS_HAUTE,leg_txt,.3,0);
	sprintf(leg_txt,"PS file: %s", f_ps);
	if (strlen(leg_txt) > 255 ) leg_txt[255]='\0';
	ps_chara(0.,1.6,ABS_HAUTE,leg_txt,.3,0);
	traa(0.,0.,ABS_HAUTE);
}
/*
*
*	X_TICK(dmin,dmax,big,medium,little,libelle)
*
*	met des tick-marks dans le sens x de d_big en d_big,
*	avec des traits a d_medium et d_little
*	jusqu'a dmin.
*
*/
x_tick(dmin,dmax,big,medium,little,libelle)
float	dmin, dmax,
	big,medium,little;
char libelle[];
{
	float hx, hy, cosa, sina, d_little, where; /* Benson */
	float	f_duree;
	int i, i_dmax, i_little, i_medium, i_big;
	char modulo, nb_ech[10];

	ps_pen(0);
	tras(0.,0.,PL_AH); /* retour a Y0 */
	hy = 0.4; hx = 0.4; cosa = 1.; sina = 0.;
	traa(0.,0.,PL_AB);
	f_duree = dmax - dmin;
	i_dmax = (int)dmax;
	i_big = big; i_medium = medium; i_little = little;
	where = 0.;

	while( where <= f_duree )
	{
		i = where + dmin;
		modulo = 'l';
		if ( ! (i % i_medium) ) modulo = 'm';
		if ( ! (i % i_big) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(0.,-0.5,PL_RB);
				sprintf(nb_ech,"%d",i);
				traa(0.,-0.4,PL_RH);
				ps_chara(0.,0.,PL_RH,nb_ech,hx,0);
				traa(0.,0.5,PL_RH);
				traa(0.,0.4,PL_RH);
				break;
			case 'm':
				traa(0.,-0.3,PL_RB);
				traa(0.,0.3,PL_RH);
				break;
			case 'l':
				traa(0.,-0.2,PL_RB);
				traa(0.,0.2,PL_RH);
		}
		where += little;
		ps_movea(where,0.);
	}
	if (where > 15.) ps_chara(15.,-1.4,ABS_HAUTE,libelle,0.4,0);
	else ps_chara((where - 0.5),-1.4,PL_RH,libelle,0.4,0);
}


/*
*
*	READ_PARAM Relecture des parametres de la simulation dans f_param
*
*/
read_param()
{
	int	i,
		k;
		
	char	inutile[80];
	
	if((p_prm = fopen(f_param,"r")) == NULL)
	{
		printf("Parameter file %s impossible to open as read\n", f_param);
		return;
	}
	/* nb de neurones */
	G_sim = FALSE;
	fgets(buffer,80,p_prm);
	if (buffer[0] == 'G')
	{
		G_sim = TRUE;
		fgets(buffer,80,p_prm);
	} /* fin du if */
	if (buffer[0] == 'P')
	{
		G_sim = FALSE;
		fgets(buffer,80,p_prm);
	} /* fin du if */
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_neur);
	       
	/* nb de clusters */
	fgets(buffer,80,p_prm);
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_clusters);

	k = 1;
	while (fgets(buffer,80,p_prm))
	{
		sscanf(buffer,"%s",inutile);
		if (!strcmp(inutile,"CLUSTER"))
		{
			fgets(buffer,80,p_prm);
			sscanf(buffer,"%s %s %s %d",
			       inutile,
			       inutile,
			       inutile,
			       &(cluster_size[k]));
			k++;
		} /* fin du if */
	} /* fin du while(fgets(buffer */
	fclose(p_prm);
} /* fin du read_param() */

