/***********************************************************************
*
*		X G R A P H _ N _ B _ C . C
*
*	Plot des couples de donnees sur la laser
*	avec le meme format que graph_n_b_c.
*       ouvre une fenetre X
*	Cree un fichier PostScript
*	Le fichier doit provenir de neuro_bio_clusters
*	
*	V1 9 avril 1990
*
*	V2 16 nov 92	Decoupe automatique en plusieurs pages
*	V3 23 nov 92	Sortie de simulations non limitees dans le temps
*			(3 sec par page max)
*	V4 18 dec 92	Suppression de la trace globale si 1 cluster
*			BoundingBox ajoutee
*			Delimitation des pages
*	V5 30 oct 93	Lecture du .prm au lieu du .def pour pouvoir accepter
*			ce qui vient de nbc_p et nbc_g
*	V6 24 nov 95	Chgt des poids des psp
*       V8  6 dec 96    Adaptation a XNBC V8 (fichie *.mod)
*       V9  2 mar 97    Possibilite de changer de fichier si non trouve (JFV)
*       V10 11 aout 97	Possibilite de changer de fichier si non trouve
*			meme si donne sur ligne de commande(JFV)
*       V11 3 oct 97	bug corrige (lecture de fichier) (JFV)
*       V12 17 fev 99   bug corrige si pas de fichier .mod (JFV)
*     
*       V1 de xgraph    22 fev 99 compilation avec plot_xps (JFV)
*       V2 de xgraph    17 juin 99 version X de graph_n_b_c V14 (JFV)
*
*	Authors:Dr J-F VIBERT
*		B3E URBB INSERM U263
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*=========================================================================
*
*	Must be loaded with -lplot_ps et -lm
*
*
*=========================================================================
*
*	This program reads file * and creates a file *.PS to be
*	spooled on the LN03R
*
***************************************************************************/ 
#ifdef DOCUMENTATION

title	XGRAPH_N_B_C	Plotting activity of clusters from neuro_bio_clusters
index			Plotting activity of clusters from neuro_bio_clusters

Synopsis

	xgraph_n_b_c [-v] [-d] [-k] [file_in]

Description

	XGRAPH_N_B_C reads in the file "file_in.sim" et "file_in.prm" created
	by neuro_bio_clusters providing a file of the following format:

	{\tt
	\nofill
		"legend 1"
		1 2 0
		3 4 0
		4 5 9

		"legend 2"
		2 3 9
		4 5 8
		6 7 0
		...
	\fill
	}
	
	The 1st number is X (the time) and the second Y (the cluster
	activity), the third is the supplemental background noise and is
	optional. Modifications made during the simulation are indicated as dashed line with
	the following abbreviations:

	\nofill
	Rest:	modification of the resting potential
	Thr:	modification of the thrshold value
	In m:	modification of the mean of the background noise
	In s:	modification of the sd of the background noise
	Ax m:	modification of the mean of the axone length
	Ax s:	modification of the sd of the background noise
	\fill
	
	It plots the graph for a PostScript printer. 

	It is possible to choose the way these plots are drawn (begining and
	end of times, page size), by answering a series of questions.
	

	{\tt -v} switch gives a verbose output on stdout

	{\tt -d} switch uses the defaults values silently
	
	{\tt -k} is to keep the PostScript file instead of spooling it
	
	The output file file_in.PS is spooled on the PostScript printer, and
	then deleted (if not {\tt -k}).

Files	 

	Input files: file_in.sim file_in.prm file_in.mod

	Output files: file_in.ps

See also

	psgraph
	
Diagnostics

	Self explaining diagnostics are provided on fatal errors.

Authors

	Dr Jean-Francois Vibert 

	CHU Saint-Antoine Univ. P. & M. Curie PARIS

Bugs/deficiencies

	No known bugs.
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <askval.h>
#include <rsx.h>
#include <plot_xps.h>
#include <math.h>
#include "find_files.h"

#define TRACE	0
#define DEBUG	0
#define DEBUG_2	0
#define DEBUG_3	0

/*	Pour la Benson */

#define B_PAP 30.
#define ABS_HAUTE 0
#define ABS_BASSE 1
#define REL_HAUTE 2
#define REL_BASSE 3

#define PT_MAX 3000

#define MAIN
#include "cstes_nbc.h"
#undef MAIN

char	*usage = "%s [?] [-v] [-d] [-k] [file]\n",
	defaut,		/* flag "default" */
	verbose,	/* flag bavard */
	keep = 'd',	/* flag keep the file (don't spool it) */
	to_draw,	/* flag plot a dessiner */
	reponse[50],	/* buffer pour les legendes */
	file[256],	/* generic file name */
	f_ps[256],	/* experimental file (*.bsp) */
	f_data[256],	/* simulation at the end (*.pha) */
	f_param[256],	/* parameters  (*.prm) */
	f_modif[256],	/* fichier des modifs   (*.mod) */
	m_legend[256],	/* buffer pour les modifs */
	fix_scal,	/* flag ps_echelle fixe */
	prcntg,		/* flag ps_echelle en pct */
	*legend[NB_CLUST_MAX],	/* les titres de graphs */
	clust_names[NB_CLUST_MAX][15],	/* les noms des clusters */
	buffer[256],	/* intermediaire */
	type_ps_echelle[140],	/* legende pour l'echelle de Y */
	fichier[256],	/* nom du fichier traite */
	date[20], heure[20],
	rep,
	prompt[81];

short	nb_clusters,
	c_est_fini;	/* flag */
	
int	nb_plot,
	nb_plot_drawn,
        nb_groupes,
	duree_feuille,
	nb_neur,
	cluster_size[NB_CLUST_MAX],
	plot_to_draw[NB_CLUST_MAX],
        groupes[5][NB_CLUST_MAX],
	start,
	modif,			/* flag */
	m_dd,			/* parametres modifies au vol: m_* */
	m_d,
	m_clust,
        m_nuc,
	m_cl_to,
	num_page,
	i_plot,
	vraie_fin,
	vrai_debut,
	duree;
	
float	x_max,
	y_max[NB_CLUST_MAX],
	x_min,
	y_min[NB_CLUST_MAX],
	m_iter,
	bkg_fact,		/* facteur de mise a l'echelle du bruit supp */
	nb_data[NB_CLUST_MAX],
	temps[PT_MAX],
	bruit[PT_MAX],
	plot_height,
	plot_width,
	x[PT_MAX],
	y[NB_CLUST_MAX][PT_MAX],
	bkg[NB_CLUST_MAX][PT_MAX],
	m_M,			/* parametres modifies au vol: m_* */
	m_S,
	m_ii,
	m_i,
	m_epsp,
	m_ipsp,
	f_debut, f_fin;


FILE	*p_data,	/* experimental file (*.sim) */
	*p_ps,		/* ps file at the end (*.ps) */
	*p_modif,	/* fichier des modifs */
	*p_prm;		/* parameter file (*.prm) */

main(argc,argv)
int argc;
char **argv;
{

	int	i, j, k,	/* un indice bon a tout */
		f_n_l,
		nb_leg,
		is_modif,	/* modifs were done */
		debut,		/* limites temporelles du plot */
		fin;

	char	choix_y;
	char    *p_c;		/* pointeur */

	float	x_scale, y_scale,	/* origine */
		bout, facteur,
		big, medium, little;


		/* traitement de la ligne de commande */

	strcpy(file,get_simulation_name());

	printf("Simulation name: %s\n",file);
	defaut = verbose = FALSE;


	if (argc > 1)
	{
		for (i=1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf(usage,argv[0]);
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'd':
						defaut = TRUE;
						break;
					case 'k':
						keep = 's';
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
				}
			}
			/*
			else strcpy(file,argv[i]);
			*/
		}
	}
        if (file)
	  asksval("\nName of the *.sim file to draw [.sim]: \n",
		  file, file);
	else
	  asksval("\nName of the *.sim file to draw [.sim]: \n",
		  "* to choose an existing file",file);
	while (tolower(file[0]) == '*')
	{
		choose_file(".","sim",file);
		p_c = strrchr(file,'.');
		if (p_c) *p_c = 0;			/* supprime l'extension */
		asksval("\nName of the file: ",file,file);
	} /* fin du if */

	f_n_l=inchr(file,'.');
	if (f_n_l != -1)
	{
		printf("\007Extension is removed...\n");
		strncpy(file,file,f_n_l);
	}

	/* on a dans file le nom generique */

        strcpy(f_data,file);
	strcat(f_data,".sim");
		
        while ((p_data=fopen(f_data,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_data);
		rep = askcval("Would you like to choose an other file ([y] or n): ",'y');
		if (strchr("yYoOsS",rep))
		{
			asksval("\nName of the *.sim file to draw [.sim]: ",
				"* to choose an existing file",file);

			while (tolower(file[0]) == '*')
			{
				choose_file(".","sim",file);
				p_c = strrchr(file,'.');
				if (p_c) *p_c = 0;			/* supprime l'extension */
				asksval("\nName of the file: ",file,file);
			} /* fin du if */
			f_n_l=inchr(file,'.');
			if (f_n_l != -1)
			{
				printf("\007Extension is removed...\n");
				strncpy(file,file,f_n_l);
			}
			else strcpy(f_data,file);
			/* on a dans file le nom generique */

			strcat(f_data,".sim");
		} /* fin du if */
		else
		{
			exit(1);
		} /* fin du else */
	}

	printf("%s opened...\n",f_data);

	strcpy(f_modif,file);
	strcat(f_modif,".mod");
	strcpy(f_param,file);
	strcat(f_param,".prm");

	is_modif = modif = TRUE;
        if ((p_modif=fopen(f_modif,"r")) == NULL)
	{
		printf("the file %s cannot be opened... \n",f_modif);
		is_modif = modif = FALSE;
	}
	else
	{
	        printf("%s opened...\n",f_modif);

		/*
		 * lecture des donnees originales pour connaitre le n de data
		 * et les max et min
		 */
		is_modif = TRUE;	 
		if ( fgets(buffer,80,p_modif) == 0)
		{
		  printf("They were not on flight parameters modifications\n");
		  is_modif = FALSE;
		} /* fin du if */
		else
		  rewind(p_modif);
	}
	read_param();		/* lectures de tailles des clusters */
	
	for(i = 1; i <= nb_clusters; i++)
	{
		if (verbose)
		{
			printf("Cluster %d: %d neurons\n",i,
			       cluster_size[i]);
		} /* fin du if */
	}

	cluster_size[0] = 0;
	strcpy(clust_names[i],"Total");
	for ( i = 1; i <= nb_clusters; i++)
	{
		cluster_size[0] += cluster_size[i];	/*
							 * dans cluster_size
							 * [0] on a le nb de
							 * neurones total
							 */
	} /* fin du for */

		
	if(verbose) printf("File to be plot: %s\n",f_data);

	printf("Computing the simulation duration...\n");


	nb_plot = -1;
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[(strlen(buffer)-1)] = 0;
			nb_plot++;
			strcpy(clust_names[nb_plot],(buffer+1));
			i = 0;
			printf("Cluster %d: %s\n",nb_plot,clust_names[nb_plot]);
		} /* fin du if */
		else
		{
			if (buffer[0] == '\n') continue;
			i++;
		} /* fin du else */
	} /* fin du while */
	nb_plot++;
	x_max = (float)i - 1.;

	rewind(p_data);

	
	printf("\nSimulation was done with %d cluster%c and finished at %.0f ms\n\n",
	       nb_plot-1,(nb_plot>2)?'s':' ',x_max);

	if (!defaut)
	{
		sprintf(buffer,"Beginning of the plot [0] ms: ",0);
		debut = askival(buffer,0);
		do
		{
			sprintf(buffer,"End of the plot [%d ms]: ",(int)x_max);
			fin = askival(buffer,(int)x_max);
		} while (fin <= debut); /* fin du do */
		if (fin > (int)x_max)
		  {
		    printf("\007No data after %.2f ms, end of plot set at %.2f ms\n",x_max,x_max);
		    fin = (int)x_max;
		  }
		puts("Representation of the data (scaling):");
		puts("\t- p - as Percentage of the cluster size");
		puts("\t- t - in True number of active units");
		puts("\t- w - as Percentage of the whole neural network size");
		puts("\t- n - as Percentage of the normalized whole neural network size");
		choix_y = askcval("\n\tYour choice [p]: ",'p');
		choix_y = tolower(choix_y);
		duree_feuille = askival("Duration of a sheet in ms: ",1000);
	}
	else
	{
		debut = 0;
		fin = (int)x_max - 1;
		choix_y = 'p';
		duree_feuille = 1000;
	} /* fin du else */
	f_debut = (float)debut;
	f_fin = (float)fin;
	vraie_fin = fin;
	vrai_debut = debut;

	if ( x_max < vraie_fin)
	{
		vraie_fin = x_max;
	}

	duree = vraie_fin - vrai_debut;
	if (duree_feuille > duree)
		duree_feuille = duree;
	if (vraie_fin > duree_feuille)
		fin = duree_feuille + vrai_debut;
	if (duree_feuille > PT_MAX )
	{
		printf("\007WARNING: no more than %d ms for 1 sheet\n\
If you need more you must draw several plots\n",PT_MAX);
		exit(1);
	} /* fin du if */
	f_fin = (float)fin;

	/* choix des clusters */
	
	fprintf(stderr,"\007Choose the cluster plots to draw:\n");
	j = 0;
	if (nb_plot == 2)
	{
		rep = askcval("\tTotal activity (y or [n]): ",'n');
		if (strchr("yYoO",rep))
		{
			plot_to_draw[0] = 0;
			j++;
		} /* fin du if */
	} /* fin du if */
	for (i = 1; i < nb_plot; i++)
	{
		sprintf(buffer,"\tCluster %d (%s) ([y] or n): ",
			i, clust_names[i]);
		rep = askcval(buffer,'y');
		if (strchr("yYoO",rep))
		{
			plot_to_draw[j] = i;
			j++;
		} /* fin du if */
	} /* fin du for */
	nb_plot_drawn = j;

	/* definition des groupes */

	k = 0;
	do
	{
	  rep = askcval("Do you need to group clusters [n]?: ",'n');
	  if (strchr("yYsSoO",rep))
	  {
	    int ki;

	    printf("Group %d:\n",k+1);
	    ki = 0;
	    for (i = 1; i < nb_plot; i++)
	    {
		sprintf(buffer,"\tCluster %d (%s) ([y] or n): ",
			i, clust_names[i]);
		rep = askcval(buffer,'y');
		if (strchr("yYoO",rep))
		{
			groupes[k][ki] = i;
			ki++;
		} /* fin du if */
	    } /* fin du for */
	    k++;
	  }
	  else
	    break;
	} while (TRUE);

	nb_groupes = k;
	nb_plot_drawn += nb_groupes;
	
	if (nb_plot_drawn == 1) plot_height = 11.;
	if (nb_plot_drawn > 1)  plot_height = 22./nb_plot_drawn;
	plot_width = 16.;
	if (!defaut)
	{
		plot_height = askfval("Plot height (cm): ",plot_height);
		plot_width = askfval("Plot width (cm): ",plot_width);
	} /* fin du if */

	for (i = 0; i < nb_plot_drawn; i++)
	{
		printf("We draw cluster %d (%s)\n",
			plot_to_draw[i],
			(plot_to_draw[i])?
				clust_names[plot_to_draw[i]]:
				"Total Act");
	} /* fin du for */

		/* creation du fichier de sortie 
			et initialisation PostScript */

	strcpy(f_ps,file);
	strcat(f_ps,"_act_clust.ps");
	if(verbose)
		printf("PostScript file to be produced: %s (%d to %.0f, sheet= %d)\n",
		       f_ps, debut,vraie_fin,duree_feuille);

	ps_iniplot(NULL,f_ps,0,0, 21., 29., 0.75, "xgraph_n_n_c V1");

	titre();
	
	sprintf(buffer,"%%%%\n%%%%BoundingBox: %.1f %.1f %.1f %.1f\n%%%%\n",
		.8*X_PAS, 4.3*X_PAS,
		(plot_width+.8+2.+1.)*X_PAS,
		((plot_height+1.5)*nb_plot_drawn)*X_PAS + 4.3*X_PAS);
	ps_code(buffer);

	rewind(p_data);
	start = FALSE;
	x_scale = plot_width / (float)(duree_feuille);	/*  ps_echelle des x */

	ps_origin(2.,4.,0.,0.);

	if (modif)
	{
	  is_modif = TRUE;	 
	  if ( fgets(buffer,80,p_modif) == 0)
	  {
	      printf("They were not on flight parameters modifications\n");
	      is_modif = FALSE;
	  } /* fin du if */
	  else rewind(p_modif);
	}
	num_page = 0;
	c_est_fini = FALSE;
	while (TRUE)
	{
		if (debut >= vraie_fin)
			break;
		read_data(f_debut,f_fin);

		/* calcul de groupes */

		for (i = 0; i < nb_groupes; i++)
		{
		  int ii, jj, kk;

		  k = nb_clusters+i+1;
		  y_max[k] = 0;
		  cluster_size[k] = 0;
		  /* legende */
		  sprintf(buffer,"Group %d (", i+1);
		  strcpy(clust_names[k],buffer);
		  legend[k] = (char *)malloc(strlen(buffer)+1);
		  strcpy(legend[k],buffer);

		  for (jj = 0; jj < nb_clusters; jj++)
		  {
#if DEBUG
		      printf("groupes[%d][%d]: %d\n",i,jj,groupes[i][jj]);	    
#endif
		      if (groupes[i][jj])
		      {
			kk = groupes[i][jj];
			cluster_size[k] += cluster_size[kk];
			nb_data[k] = nb_data[kk];
#if DEBUG
			printf("groupe %d (k= %d): taille: %d\n", i+1,k, cluster_size[k]);
#endif			
			strcat(legend[k],clust_names[kk]);
			strcat(legend[k]," ");

			for (ii = 0; ii < x_max; ii++)
			{
#if DEBUG
			  if (ii < 10) 
			    printf("%d: y[%d][%d]: %f + y[%d][%d]: %f -> ",kk, k, ii, y[k][ii], kk, ii, y[kk][ii]);
#endif
			  y[k][ii] += y[kk][ii];
#if DEBUG
			  if (ii < 10) printf(" y[%d][%d]: %f\n", k, ii, y[k][ii]);
#endif
			  if (y[k][ii] > y_max[k])
			    y_max[k] = y[k][ii];
			}
		      }
		  }
		  strcat(legend[k],")");
		  plot_to_draw[nb_plot-1] = nb_clusters+i+1;
#if DEBUG
		  printf("plot_to_draw[%d]: %d \n", groupes[i][jj], plot_to_draw[groupes[i][jj]] );
#endif			
		  nb_plot++;
		}

		  
		i_plot = 0;
		printf("Plotting from %d to %d ms\n",debut,fin);
		for (i = 0; i < nb_plot; i++)
		{
			to_draw = FALSE;
			for (i_plot = 0; i_plot < nb_plot_drawn; i_plot++)
			{
			  printf("plot_to_draw[%d] = %d\n",i_plot, plot_to_draw[i_plot]);
				if (plot_to_draw[i_plot] == i)
				{
					to_draw = TRUE;
				} /* fin du if */
			} /* fin du for */
			if (!to_draw)
				continue;
			printf("Plotting cluster %d (%s)\n",
			       i,(i)?clust_names[i]:"Total Act");

			sprintf(buffer,"%%%%\n%%%% Cluster %d (%s)\n%%%%\n",
				plot_to_draw[i_plot],clust_names[plot_to_draw[i_plot]]);
			ps_code(buffer);
			switch (choix_y)
			{
				case 'p':
					facteur = 100./cluster_size[i]; 
					strcpy(type_ps_echelle,"Y scale as Percentage of the cluster size");
					bout = 100.;
					big = 20.;
					medium = 10.;
					little = 5.;
					break;
				case 't':
					facteur = 1.; 
					strcpy(type_ps_echelle,"Y scale as True number of active units\n");
					bout = cluster_size[i];
					big = 20.;
					medium = 10.;
					little = 5.;
					break;
				case 'n':
					facteur = 100./cluster_size[0]; 
					strcpy(type_ps_echelle,"Y scale as Percentage of the whole neural network size\n");
					bout = y_max[0]*facteur;;
					big = 20.;
					medium = 10.;
					little = 5.;
					break;
				case 'w':
				default :
					facteur = 100./cluster_size[0]; 
					strcpy(type_ps_echelle,"Y scale as Percentage of the normalized whole neural network size\n");
					bout = 100.;
					big = 20.;
					medium = 10.;
					little = 5.;
			} /* fin du switch */
			y_scale = (plot_height - 1.) / bout ;	/* ps_echelle des y en % */
			if(verbose) printf("X_scale -> %f   Y_scale -> %f\n",x_scale,y_scale);

			ps_echel(x_scale,y_scale,0.,0.,1.,0.);

#if DEBUG_2
			printf("Facteur = %f",facteur);
#endif
			if (is_modif)
			  modif = TRUE;

			printf("Plotting %s...\n",legend[i]);
			

				/* Trace des axes */
			if(verbose) printf("y_max = %f\n",y_max[i]);

			ps_pen(0);
			ps_user();

			/* Axe des Temps (dans le sens X benson) */

			/* retour au 0 */

			ps_movea(0.,0.);

			/* Axe des effectifs (dans le sens Y benson) */
/*
			ps_drawa(0.,cluster_size[i]);
*/
			/* retour au 0 */

			ps_movea(0.,0.);

			if (plot_height < 4.)
			{
				big = 100;
				medium = 50;
				little = 10;
			} /* fin du if */

			y_tick(bout,big,medium,little,legend[i]);

			ps_movea(0.,0.);

			ps_drawa((float)(duree_feuille),0.);

#if OLD
			big = (float)duree_feuille / 10.;
			medium = big / 2.;
			little = big / 10.;


			x_tick(f_debut,f_fin,big,medium,little,"Time (ms)");
#endif
			/* Axe des Temps (dans le sens X benson) */
			
			/* retour au 0 */
			
			ps_movea(0.,0.);
			ps_drawa((float)(duree_feuille),0.);
			if  (duree_feuille <= 10) 
			  {
			    big = 5.;
			    medium = 1.;
			    little = 1.;
			  }
			if (duree_feuille > 10 && duree_feuille <= 100) 
			  {
			    big = 10.;
			    medium = 5.;
			    little = 1.;
			  }
			if (duree_feuille > 100 && duree_feuille <=750) 
			  {
			    big = 50.;
			    medium = 25.;
			    little = 5.;
			  }
			if (duree_feuille > 750 && duree_feuille <= 1000) 
			  {
			    big = 100.;
			    medium = 50.;
			    little = 10.;
			  }
			if (duree_feuille > 1000 && duree_feuille <= 2000) 
			  {
			    big = 500.;
			    medium = 100.;
			    little = 50.;
			  }
			if (duree_feuille > 2000 && duree_feuille <= 5000) 
			  {
			    big = 1000.;
			    medium = 500.;
			    little = 250.;
			  }
			if (duree_feuille > 5000 && duree_feuille <= 10000) 
			  {
			    big = 5000.;
			    medium = 1000.;
			    little = 500.;
			  }
			
			x_tick((float)debut,(float)fin,big,medium,little,"Time (ms)");
			
			ps_movea(0.,0.);
			sprintf(buffer,"%d N.",cluster_size[i]);
			ps_chara(-2.,-0.5,ABS_HAUTE,buffer,0.3,0);
			ps_movea(0.,0.);
/*			
			ps_drawa(0.,bout);
*/
			ps_movea(0.,y[i][0]*facteur);
			ps_line_width(1.);
			
			if (is_modif) get_modif();

			for( j = 0; j < nb_data[i]; j++)
			{
				if (x[j] < f_debut)
					continue;
				if (x[j] > f_fin)
					break;
#if DEBUG_2
				printf("x = %f y = %f \n", x[j]-f_debut,y[i][j]);
#endif
				ps_drawa(x[j]-f_debut,y[i][j]*facteur);
				if (is_modif)
				{
					while (modif && (m_iter < x[j]))
						get_modif();
					nb_leg = 1;
#if DEBUG_3
					printf("modif = %d m_iter = %.1f m_iter = %.1f j = %d\n",
					       modif,m_iter,(m_iter - f_debut),j);
#endif
					while (modif && (m_iter  == x[j]))
					{
						if (m_clust == i)
						{
#if DEBUG_3						
							printf("cluster %d: x[%d]= %f x[%d]-f_debut= %f\n",
							       i,j,x[j],j,(x[j]-f_debut));
#endif
							ps_movea((x[j]-f_debut),0.);
							ps_code("[10 10] 0 setdash\n");
							/* fprintf(pl_spl,"[10 10] 0 setdash\n"); */
							ps_drawr(0.,bout);
							ps_movea((x[j]-f_debut),bout);
							ps_chara(0.1,(-0.3 * nb_leg),REL_HAUTE,m_legend,.2,0);
							nb_leg++;
						} /* fin du if */
						if (m_nuc == i)
						{
#if DEBUG_3						
							printf("cluster %d: x[%d]= %f x[%d]-f_debut= %f\n",
							       i,j,x[j],j,(x[j]-f_debut));
#endif
							ps_movea((x[j]-f_debut),0.);
							ps_code("[2 2] 0 setdash\n");
							/* fprintf(pl_spl,"[2 2] 0 setdash\n"); */
							ps_drawr(0.,bout);
							ps_movea((x[j]-f_debut),bout);
							ps_chara(0.1,(-0.3 * nb_leg),REL_HAUTE,m_legend,.2,0);
							nb_leg++;
						} /* fin du if */
						get_modif();
					} /* fin du while */
					ps_code("[] 0 setdash\n");
					/* fprintf(pl_spl,"[] 0 setdash\n"); */
					ps_movea((x[j]-f_debut),y[i][j]*facteur);
				} /* fin du if */
			} /* fin du for */
			ps_origin(0.,plot_height,0.,0.);
			if (is_modif)
			{
				rewind(p_modif);
				modif = TRUE;
			} /* fin du if */
		}
		ps_chara(2.,-0.5,ABS_HAUTE,type_ps_echelle,0.4,0);
		debut += duree_feuille;
		fin += duree_feuille;
		f_fin = (float)fin;
		f_debut = (float)debut;
		if (c_est_fini)
			break;
		if (fin > (int)x_max)
		{
			fin = (int)x_max;
			f_fin = x_max;
			c_est_fini = TRUE;
		} /* fin du if */
		if (f_fin > vraie_fin)
		{
			fin = vraie_fin;
			f_fin = (float)vraie_fin;
			c_est_fini = TRUE;
		} /* fin du if */
		num_page++;
		if (num_page >= (int)x_max / duree_feuille)
			break;

		ps_newpage();
		sprintf(buffer,"%%%%\n%%%% From %d to %d ms (page %d)\n%%%%\n",
			debut,fin,num_page);
		ps_code(buffer);
		if (is_modif)
		{
			rewind(p_modif);
			modif = TRUE;
		} /* fin du if */
		ps_origin(2.,4.,0.,0.);

		/* Remise a zero des tableaux */

		for (i = 0; i < nb_plot; i++)
		  for (j = 0; j < nb_data[i]; j ++)
		    y[i][j] = 0.;

	} /* fin du while */

	keep = askcval("\007Type \n\tp to exit and print, \n\tx to exit and discard \nAnswer [p]:",'p');
	
	switch(keep)
	{
	case 'p':
	  ps_endplot('s'); break;
	case 'x':
	  ps_endplot('f'); break;
	}
}
/*
*	GET_MODIF() lit les modif et les dispatch
*/
get_modif()
{
	char	m_buffer[81], on[5], *blanc; 
	int     m_numero;
	unsigned int l_iter;


	if (fgets(m_buffer,80,p_modif) == 0)
	{
		modif = FALSE;
		return(modif);
	} /* fin du if */
	sscanf(m_buffer,"%lu %s %d ",&l_iter,on,&m_numero);
	m_iter	= (float)l_iter;
	/* remise a zero afin de n'avoir que celui qui est modifie */ 
	m_nuc = m_clust = 0;      
	if (!strcmp(on,"nuc"))
	  m_nuc = m_numero;
	if (!strcmp(on,"clu"))
	  m_clust = m_numero;
	/* il faut sauter 3 blancs */
	blanc = strchr(m_buffer,' '); /* 123*clu 1 CB=noise... */
	blanc = strchr(blanc,' ');    /* 123 clu*1 CB=noise... */
	blanc = strchr(blanc,' ');    /* 123 clu 1*CB=noise... */
	strcpy(m_legend,(++blanc));           /* CB=noise... */

	return(modif);
} /* fin du get */


/*
*
*	READ_DATA() lit les donnees 
*	sur le fichier pour connaitre le nombre de 
*	donnees et les max et min
*	Le bruit sup est egalement lu en 3 eme position s'il y a lieu
*
*/
read_data(de_tant,a_tant)
float	de_tant,	/* limites a conserver */
	a_tant;
{
	int	i, j, nb_graph;
	float	x_i, y_i, bkg_i;

	bkg_i = 0.;
	nb_graph = -1;
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[strlen(buffer)-1] = 0;
			i = 0;
			nb_graph++;
			legend[nb_graph] = (char *)malloc(strlen(buffer));
			strcpy(legend[nb_graph],&buffer[1]);
			printf("Reading %s...\n",legend[nb_graph]);
		} /* fin du if */
		else
		{
			if (buffer[0] == '\n') continue;
			bkg_i = -1.;
			sscanf(buffer,"%f %f %f",&x_i,&y_i,&bkg_i);
			if (x_i <  de_tant|| x_i > a_tant) continue;
			x[i] = x_i;
			y[nb_graph][i] = y_i;
			bkg[nb_graph][i] = bkg_i;	/* deja normes entre 0 et 100 */
			if (x[i] > x_max) x_max = x[i];
			if (y[nb_graph][i] > y_max[nb_graph]) y_max[nb_graph] = y[nb_graph][i];
			if (x[i] < x_min) x_min = x[i];
			if (y[nb_graph][i] < y_min[nb_graph]) y_min[nb_graph] = y[nb_graph][i];
			nb_data[nb_graph]++;
			i++;
		} /* fin du else */
	} /* fin du while */
	nb_graph++;
	rewind(p_data);
}

#if 0
/*
*
*	READ_DATA() lit les donnees 
*	sur le fichier pour connaitre le nombre de 
*	donnees et les max et min
*	Le bruit sup est egalement lu en 3 eme position s'il y a lieu
*
*/

read_data(de_tant,a_tant)
float	de_tant,	/* limites a conserver */
	a_tant;
{
	int	i, j, nb_graph, k;
	float	x_i, y_i, bkg_i;

	bkg_i = 0.;
	nb_graph = -1;
	i = 0;
	while (fgets(buffer,80,p_data))
	{
		if (buffer[0] == '\"')
		{
			buffer[strlen(buffer)-1] = 0;
			nb_graph++;
			legend[nb_graph] = (char *)malloc(strlen(buffer));
			strcpy(legend[nb_graph],&buffer[1]);
			printf("Reading %s...\n",legend[nb_graph]);
		} /* fin du if */
		else
		{
			if (buffer[0] == '\n') continue;
			sscanf(buffer,"%f %f %f",&x_i,&y_i,&bkg_i);
			if (x_i <  de_tant|| x_i > a_tant)
			  continue;
			else
			{
			  k = 0;
			  x[i] = x_i;
			  y[k][i] = y_i;
			  bkg[k][i] = bkg_i;	/* deja normes entre 0 et 100 */
			  if (x[i] > x_max) x_max = x[i];
			  if (y[k][i] > y_max[k]) y_max[k] = y[k][i];
			  if (x[i] < x_min) x_min = x[i];
			  if (y[k][i] < y_min[k]) y_min[k] = y[k][i];
			  nb_data[k]++;

			  for (k = 1; k <= nb_clusters; k++)
			  {
			    fgets(buffer,80,p_data);
			    sscanf(buffer,"%f %f %f",&x_i,&y_i,&bkg_i);
			    x[i] = x_i;
			    y[k][i] = y_i;
			    bkg[k][i] = bkg_i;	/* deja normes entre 0 et 100 */
			    if (x[i] > x_max) x_max = x[i];
			    if (y[k][i] > y_max[k]) y_max[k] = y[k][i];
			    if (x[i] < x_min) x_min = x[i];
			    if (y[k][i] < y_min[k]) y_min[k] = y[k][i];
			    nb_data[k]++;
			  }
			i++;
			}
		} /* fin du else */
	} /* fin du while */
	nb_graph++;
	rewind(p_data);
}
#endif
/*
*
*	TITRE() ecrit la legende generale du dessin 
*
*/
titre()
{
	char leg_txt[80];
	int	i;

	traa(0.,0.,ABS_HAUTE);
	sprintf(leg_txt,"File: %s", f_data);
	if (strlen(leg_txt) > 79 ) leg_txt[79]='\0';
	ps_chara(3.,2.,ABS_HAUTE,leg_txt,.4,0);
	sprintf(leg_txt,"%d nucl%s", nb_plot - 1,(nb_plot)?"ei":"us");
	if (strlen(leg_txt) > 79 ) leg_txt[79]='\0';
	ps_chara(0.,2.,ABS_HAUTE,leg_txt,.4,0);
	traa(0.,0.,ABS_HAUTE);
}
/*
*
*	Y_TICK(dmin,d_big,d_medium,d_little)
*
*	met des tick-marks dans le sens x de d_big en d_big,
*	avec des traits a d_medium et d_little
*	jusqu'a dmin.
*
*/
y_tick(dmax,d_big,d_medium,d_little,texte)
float 	dmax, d_big, d_medium, d_little;
char 	*texte;
{
	float hx, hy, cosa, sina, where; /* Benson */
	int i, i_dmax, i_little, i_medium, i_big;
	char modulo, nb_ech[10];

	hx = 0.4; hy = 0.4; cosa = 0.; sina = 1.;
	traa(0.,0.,ABS_HAUTE);
	i_dmax = (int)dmax;
	i_little = (int)d_little;
	i_medium = (int)d_medium;
	i_big = (int)d_big;
	where = 0.;

	for ( where = 0. ; where <= dmax;  )
	{
		i = where;
		modulo = 'l';
		if ( ! (i % i_medium) ) modulo = 'm';
		if ( ! (i % i_big) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(-0.5,0.,REL_BASSE);
				sprintf(nb_ech,"%d",i);
				traa(-0.6,0.0,REL_HAUTE);
				ps_chara(0.,0.,REL_HAUTE,nb_ech,hx,0);
				traa(0.6,-0.0,REL_HAUTE);
				traa(0.5,0.,REL_HAUTE);
				break;
			case 'm':
				traa(-0.3,0.,REL_BASSE);
				traa(0.3,0.,REL_HAUTE);
				break;
			case 'l':
				traa(-0.2,0.,REL_BASSE);
				traa(0.2,0.,REL_HAUTE);
		}
		where += d_little;
		ps_movea(0.,where);
  	}
	ps_chara(-1.4,0.,ABS_HAUTE,texte,0.4,90);
	traa(0.,0.,ABS_HAUTE);
	ps_drawa(0.,(where - d_little));
}

/*
*
*	X_TICK(dmin,dmax,big,medium,little,libelle)
*
*	met des tick-marks dans le sens x de d_big en d_big,
*	avec des traits a d_medium et d_little
*	jusqu'a dmin.
*
*/
x_tick(dmin,dmax,big,medium,little,libelle)
float	dmin, dmax,
	big,medium,little;
char libelle[];
{
	float hx, hy, cosa, sina, d_little, where; /* Benson */
	float	f_duree;
	int i, i_dmax, i_little, i_medium, i_big;
	char modulo, nb_ech[10];

	ps_pen(0);
	tras(0.,0.,PL_AH); /* retour a Y0 */
	hy = 0.4; hx = 0.4; cosa = 1.; sina = 0.;
	traa(0.,0.,PL_AB);
	f_duree = dmax - dmin;
	i_dmax = (int)dmax;
	i_big = big; i_medium = medium; i_little = little;
	where = 0.;

	while( where <= f_duree )
	{
		i = where + dmin;
		modulo = 'l';
		if ( ! (i % i_medium) ) modulo = 'm';
		if ( ! (i % i_big) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(0.,-0.5,PL_RB);
				sprintf(nb_ech,"%d",i);
				traa(0.,-0.4,PL_RH);
				ps_chara(0.,0.,PL_RH,nb_ech,hx,0);
				traa(0.,0.5,PL_RH);
				traa(0.,0.4,PL_RH);
				break;
			case 'm':
				traa(0.,-0.3,PL_RB);
				traa(0.,0.3,PL_RH);
				break;
			case 'l':
				traa(0.,-0.2,PL_RB);
				traa(0.,0.2,PL_RH);
		}
		where += little;
		ps_movea(where,0.);
	}
	if (where > 15.) ps_chara(15.,-1.4,ABS_HAUTE,libelle,0.4,0);
	else ps_chara((where - 0.5),-1.4,PL_RH,libelle,0.4,0);
}


/*
*
*	READ_PARAM Relecture des parametres de la simulation dans f_param
*
*/
read_param()
{
	int	i,
		k;
		
	char	inutile[80];
	
	if((p_prm = fopen(f_param,"r")) == NULL)
	{
		printf("Parameter file %s impossible to open as read\n", f_param);
		return;
	}
	/* nb de neurones */

	fgets(buffer,80,p_prm);
	fgets(buffer,80,p_prm);	
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_neur);

	printf("Total number of neurons: %d\n",nb_neur); 
	       
	/* nb de clusters */
	fgets(buffer,80,p_prm);
	sscanf(buffer,"%s %s %s %s %s %s %d",
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       inutile,
	       &nb_clusters);

	printf("Total number of clusters: %d\n",nb_clusters); 

	k = 1;
	while (fgets(buffer,80,p_prm))
	{
		sscanf(buffer,"%s",inutile);
		if (!strcmp(inutile,"CLUSTER"))
		{
			fgets(buffer,80,p_prm);
			sscanf(buffer,"%s %s %s %d",
			       inutile,
			       inutile,
			       inutile,
			       &(cluster_size[k]));
			printf("Number of neurons in cluster %d: %d\n",k,cluster_size[k]); 
			k++;
		} /* fin du if */
	} /* fin du while(fgets(buffer */
	fclose(p_prm);
} /* fin du read_param() */

