#ifndef _MAIN_
#define _MAIN_
 
/*
*
*	NEURO_BIO_CLUSTERS.C implemente un reseau de clusters de neurones 
*	biologiques dont on peut parametrer le nombre, les tailles, les 
*	connexions intra et interclusters, et les caracteristiques de mb.
*	Au sein d'un cluster, les neurones ne sont pas tous synchronises.
*
*	Donne une sortie utilisable par xgraph avec les outputs des
*	clusters et l'output global, et l'output de groupes de clusters.
*
*
*	ULTRIX 		2 avril 90
*
*
*	Versions
*	:	V1	n_c							juin 88
*		V2	n_c	delais inter neurones			juin 88
*		V3	n_c	parallelisme stat + phase refract	juill 88
*
*		V4	n_b_c	refonte profonde et structuration	2 avril 90
*		V4.0	Mb pot en exponentielles			2 avril 90
*		V4.1	ajout des seuils variables			5 avril 90
*		V4.2	ajout des E_K et E_Na				12 avril 90
*		V4.3	Ajout des time series et correction de bugs	fin avril 90
*		V4.4	Ajout des reversal potentials			15 mai 90
*		V4.5	traitement des inputs + corrct bugs		juin 90
*		V4.6	sauvetage et reprise de simulation		2 juillet 90	
*        	V4.7	inputs de bruit supp  (ar)			2 juillet 90
*		V4.8	generalized modification enabled		nov 90
*		V4.9	Nouvelle fonction de reversal pot. (jfv+sh)	19 fevrier 91
*		V4.10	Conservation de n connexions par neurones 
*				+ clusters ayant ou non une histoire		10-Avril-91
*				+ choose_file
*				+ Z compression on fly
*		V4.11	File d'attente des spikes sur l'axone		22 mai 91
*				(permet que  l'intervalle interspike
*				soit plus grand que la longueur de l'axone)
*		V4.12	Prise en compte du shunt de la mb 		30 mai 91
*		V4.13   Affinement du calcul des amplitudes des PSPs	7 juin 91
*		V4.14	Numero d'iteration si non dot display		22-juillet-91
*		V4.15   Modif fichier *.pmb pour pmb_*_n_b_c		1-aout-91
*
*		V5.0	Allocation dynamique des structures afin de	8 juin 91
*				gagner de la place.
*		V5.1	Allocation dynamique niveau 4.15		7 nov 91
*		V5.2	Chgt structure de Neurone
*					(+ appartenance symbolique
*					 + localisation spatiale)		18 dec 91
*		V5.3	Modification du comportement de l'option	12 nov 92
*				de chgt de delay au vol
*				+ ajout du chgt de periode refractaire au vol
*		V5.4	Modification du calcul des PSP (KP+JFV)		23 Dec 92
*		V5.5	Traitement de la ligne de commande (KP+JFV)	26 Jan 93
*				+ manuel
*		V5.6	Chgt en rampe des longueurs d'axone (KP+JFV)	19 Fev 93
*
*		V6.0	Chgt de strcture des  fichiers cnx  (JFV + YH)	24 Avr 93
*					(ajout nb + tailles clusters)
*					(relecture des fichiers de link_edit)
*					Chgt de structure de la struct clusters
*					(relecture des fichiers de P_neuron)
*		V6.1	Lecture des fichier *.unit et *.lnk		25 mai 93
*		V6.2	Sous menu des parametres de cellules		1 juin 93
*				+ chargement du fichier *.tms (cluster virtuel)
*		V6.3	Ajout du switch de travail en mode batch	31 juillet 93
*								(JFV+KP)
*		V6.4	Modification des allongements	(JFV)		21 septembre 93
*						(supression de variabilite)
*		V6.5	Mise en conformite avec nbc_g	(JFV)		3 Nov 93
*                    V6.6    Lecture automatique des fichiers unit nommes    21 jan 94
*                               de link_nbc + sauvegarde du .pmb par defaut (JFV)
*
*		V7.0	Pas d'iteration de moins de 1 ms (JFV)		11 fev 94
*				Supression de nbs fichiers (.spk, .sim, ...)
*		V7.1	nbc_x (extended) pour multi-modelling		5 mars 94
*		V7.2	files groupees dans un sous directory		2 mai 95
*		V7.3	Alignement sur P_neuron V4,5			5 mai 95
*		V7.4	Pb aligenment de data				20 oct 95
*		V7.5	Remplacement de bruit_sup (v4.7) par modulation	21 oct 95
*				Supression des pauses et bell inutiles
*
*		V8.0	Remplacement de l'interface ASCII par un	oct 1995
*				interface X (St. Rimbaud)
*		V8.1	Adjonction de l'appel des autres outils (JFV)	jan 1996
*		V8.2	Coherence avec G_Neurone V5.7	(JFV)		fev 1996
*		V8.3-6	Correction de bugs		(JFV)		mars 1996
*		V8.7	Pointage sur le bon directory	(JFV)		1 aout 96
*		V8.8	Filtrage a la Andre Fabio Kohn	(JFV)		5 sept 96
*		V8.9	Adjonction du modele conditional burster (JFV)	11 sept 96
*                              Mise en plce des indication ADD_NEW_MODEL
*                   V8.10   Mise en place de la fatigue     (JFV+KP)        19 sept 96
*		V8.11   Mise en place des bursts        (JFV)		21 oct  96
*                    V8.12   Update de neurone et network    (GS)            9 mars 97
*		V8.13   Optimisations diverses          (JFV+JP)	20 mars 97
*		V8.14   Prise en compte de G_neuron V7  (JFV)	        15 avr 97
*                    V8.15   Correction de bugs de pas d'iteration (JFV)     11 mai 97
*                    V8.16   Chargement du bruit avec les clusters (JFV)     20 mai 97
*                    V8.17   Correction de bug dans les inputs (JFV)         26 mai 97
*                    V8.18   Correction de bug si time step petit (JFV)      27 mai 97
*                    V8.19   Chgt de la surface en surface relative (JFV)    13 juin 97
*                    V8.20   Load avec le .lnk + filtered noise  (JFV)       11 aout 97
*
*                    V9.00   Ajout d`un panneau pour prendre en compte les 
*                               drogues (GS)                                    25 juillet 97
*                    V9.1    Prise en compte du Leaky Integrator 
*                              (model LIM) et appel de pheno PUM (JFV+KP)      19 aout 97
*                    V9.2    Supression de la bufferisation pmb et tms (JFV) 22 aout 97 
*                    V9.3    Ajout de la liste des drogues (GS)		25 aout 97 
*                    V9.4    Gestion de .xnbcrc  (JFV)		        1 sept 97
*                    V9.5    Ajout de l appel au grug editor (GS)            4 sept 97
*                    V9.6    Rearrangement du .sim pour xcaa et graph (JFV)  5 sept 97
*		V9.7	Correction pb decharge initiale si mort (JFV)	17 mars 98
*		V9.8	Sauvegarde des parametres de simulation (JFV)	16 oct 1998
*		V9.9	Creation d'un fichier xyz pour network_viewer (JFV) 4 dec 1998
*
*                     V10.0   Bouton pour les stimulations (JFV)              13 dec 1998
*                     V10.1   Panneau de controle des stimulations (JFV)      16 dec 1998
*                     V10.2   Sauvegarde du fichier au format Abeles (JFV)    19 dec 1998
*                               pour pouvoir utiliser Spanake de A. Villa (UCL)
*			V10.3	Gestion des boutons de stimulation en fonction	23 dec 1998
*			des parametres des noyaux (JFV)
*                     V10.4   Boutons Stim All ON, All OFF, stop at (JFV)     26 dec 1998
*                     V10.5   remaniement de l'ergonomie des drogues (JFV)    9 jan 1999
*                     V10.6   MAXFLOAT non defini dans red hat 5.2 (JFV)      25 jan 1999
*                     V10.7   mod file and sim files not corrects 
*                               if name changed (JFV)                           18 fev 1999
*                     V10.8   verification que le fichier n'est pas ecrase,    
*                               Gestion de la fermeture par le window manager (JFV) 18 fev 1999
*
*                    V11.0   Ajout du menu Outputs (JFV)                     6 mars 1999
*                    V11.1   Ajout du fichier pour Gnuplot (JFV)             17 mars 1999
*                    V11.2   Ajout de la stimulation par single pulses (JFV) 25 mars 1999
*                    V11.3   Modification de l entete des fichiers abe (JFV) 28 avril 1999
*		V11.4	Ajout de HAVE_CONFIG_H (JFV) 16 sep 1999
*		V11.5	Ajout de XNBC_DRUG_DIR (JFV) 6 oct 1999
*		V11.6	Sauvegarde des parametres de stimulation (JFV)  29 nov 1999
*                    V11.7   Nombreux changements: compteur visible en permanence
*                                                     suppression des messages peu utiles
*                                                     corrections d'un bug sur .nbc_xrc
*                                                                        (JFV+FA) 5 dec 1999
*                    V11.8   Prise en compte du changement de next stop par le bouton run
*                                                                        (JFV+FA) 8 dec 1999
*                    V11.9   Suppression de nb_step_ms pour eviter des erreurs 
*                               d'arrondi (importants si 1/time_step non entier).
*                               Changement de next_stop en temps entre 2 stops.
*                               Creation de la boite de dialogue display_question_no
*                               Mise avec le flag debug de la question skip neuron 
*                               description.   
*                               Corrections de plusieurs castings (inutiles ou erronnes).
*                                                                        (JFV+FA) 8 dec 1999
*                    V11.10  Correction de bug (False announce of bad value of next stop)
*                                                                        (JFV) 15 fev 2000
*                    V11.11  Implementation de la stimulation par des trains de pulses
*                                                                        (JFV) 18 fev 2000
*                    V11.12  Plante si *.nucl non existant. Corrected.  (JFV) 21 fev 2000
*                    V11.13  Option menu to choose the stimulation  (JFV) 22 fev 2000
*                    V11.14  Deplacement des fenetres lors de l'appel de stimulation (JFV) 25 fev 2000
*                    V11.15  Ajout du bouton quit dans la fenetre de nom de simulation (JFV) 3 avr 2000
*		V11.16	Nb de it/ms mall affiche. corrige (JFV) 19 juin 2000
*		V11.17	Panneau de visualisation graphique (JFV) 19 decembre 2000
*		V11.18	Supression des messages dans le panneau central (JFV) 29 decembre 2000
*		V11.19	Resizing de la drawing area (JFV) 2 janvier 2001
*		V11.20	Control of random initialization of alive nuclei (JFV+FA) 5 janvier 2001
*		V11.21	Supress useless messages, and a bug in cluster of LIM (JFV+FA) 11 janvier 2001
*		V11.22	Changing the behavior of Cancel button of simulation name +
*                              	Updating the ran1 and gasdev functions (JFV+FA) 16 janvier 2001
*		V11.23	Control of random initialization of mb pot of alive nuclei (JFV+FA) 19 janvier 2001
*		V11.24	Update network menu had a wrong behavior -fixed- (JFV+FA) 23 janvier 2001
*		V11.25	Random initialization of PUM had a wrong behavior -fixed- (JFV+FA) 26 janvier 2001
*		V11.26	Close Nucleus parameter instaed og OK button (JFV+FA) 31 janvier 2001
*                    V11.27  	Adaptation to Windows. .lnk replaced by .ntw (JFV) 13 sept 2001
*                    V11.28 	MAXFLOAT too large for 64 bit machines replaced by 10000. (JFV) 16 nov 2001
*		V11.29	Adaptation  la nouvelle structure des bouons synatiques de G_neuron

*		V12.0	Adaptation  la nouvelle structure remanie des boutons synatiques de G_neuron
*		V12.1	Correction d'un pb au niveau de la saauvegarde du pmb pour le modle G
*		V12.2	Ajout d'un fichier log qui sauve ce qui est crit dans la fenetre du simulateur
*		V12.3	version intermdiaire, mise enplace du modle HH simmple	
*		V12.4	Ajout de fonctions spcialises pour HH_SIMPLE et HH_FULL
*
*	Auteur: Dr J-F VIBERT
*		ex	L.I.O.N.	Laboratoire de Physiologie
*		ex 	B3E ESI U444
*		ESIM INSERM UMR-S 707
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	License GPL
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is distributed under the GPL License
*
*/

/*
*	MAIN() part of neuro_bio_clusters
*/


#ifdef DOCUMENTATION

title	NBC_X	Simulation of HH + P biological neural networks
index		Simulation of HH + P biological neural networks

Synopsis

	nbc_x [\\?|-h] [-v] [-b] [file_in] [-o file_out]

Description

	NEURO_BIO_CLUSTERS is a rather general purpose tool designed to
	simulate several interconnected clusters of neurons. Neurons sharing
	the same features are gathered in a cluster. This clusterization
	facilitates the entry of parameters since the neuron
	description parameters are set once at the cluster level.

	NEURO_BIO_CLUSTERS produces a large number of output files differing
	only by their extension. It can read some of these output files
	as input files.

	NEURO_BIO_CLUSTERS can read a previously stored neuron description in
	a name_1.x_def file. This file can be modified and either kept with the
	same name (keeping a backup copy {\tt name_1.x_def.bak} or stored in a
	new {\tt file name_2.x_def}. Matrix files ({\tt name_1.cnx} and {\tt
	name_1.lng} can also be read and modified or not. If needed, the
	modified matrix can be stored in a new {\tt name_2.cnx} file.
	
	Several command line options are available to the user:

	
	{\tt ?} or {\tt -h} gives the usage

	{\tt -v} switch gives a verbose output on stdout

	{\tt -b} switch to run in batch mode (no questions are asked)

	{\tt -k} to keep the PostScript file instead of spooling it

	{\tt file_in}	definition input file without extension

	{\tt -o file_out}	generic name of output files


	During the simulation it is possible to choose to see only the
	iteration count running or a simplified dot display on the standard
	output. It is possible to stop and restart the simulation
	asynchroneously at any time in order to adjust some parameters
	through a menu.

	At the end of the simulation, it is possible to keep the actual
	state in order to continue the stopped simulation at a latter
	convenient time.

	
Files	 
	\nofill
	Input files:
	    
		file_in.x_def:	simulation parameters (binary file)
		file_in.len:	connection matrix for axone length (binary file)
		file_in.wgt:	connection matrix for synaptic weight (binary file)
		file_in.ntw:	connection matrix in symbolic form (ascii file from link_edit or network_editor)
		file_in.neur:	state of a stopped simulation (binary file)
		file.P_unit:	description of a neuron (from P_neuron)
		file.B_unit:	description of a neuron (from B_neuron)
		file.G_unit:	description of a neuron (from G_neuron)
	        
	Output files:
	     
		file_in.x_def.bak: backup of file_in.x_def
		file_out.x_def:	simulation parameters (binary file)
		file_out.len:	length matrix (binary file)
		file_out.wgt:	length matrix (binary file)
		file_out.neur:	state of a stopped simulation (binary file)
		file_out.prm:	simulation parameters (text file)
		file_out.mod:	on line modifications (text file)
		file_out.tms:	time series (text file)
		file_out.pmb:	membrane potential values (text file)
		file_out.log:	log file (text file)
		file_out.xyz:   position des neurones (text file)
		file_out.abe:   spike description file from Moshe Abeles (text file)
	\fill

See also

	xnbc 
	
Diagnostics

	Self explaining diagnostics are provided on errors.

Authors

	Dr Jean-Francois Vibert and Stephane Rimbaud

	CHU Saint-Antoine Univ. P. & M. Curie PARIS

Bugs/deficiencies

	No known bugs. A lot of deficiencies progressively removed...
#endif


/*
 * Globally included information.
 */

#include "creation-simu.h"
#include "sim_init.h"


int main(int argc, char **argv)
{
    /* Begin user code block <declarations> */

    unsigned long rouge;
    Visual        *visual;
    XtAppContext app;
    Arg          args[256];
    Cardinal     ac;
    int		i;
    Boolean     argok=False;
    char	buffer[256];
    char	msg_error[256];
    char	*pt;
    char        *home;
   
    /* End user code block <declarations> */
    
	 /*
	     * The applicationShell is created as an unrealized
	     * parent for multiple topLevelShells.  The topLevelShells
	     * are created as popup children of the applicationShell.
	     * This is a recommendation of Paul Asente & Ralph Swick in
	     * _X_Window_System_Toolkit_ p. 677.
	     */
    
    XtToolkitInitialize();
    app = XtCreateApplicationContext();
    app_context = app;
    
    
    seed = time(0);
    set = time(0);

    *input_file = '\0';
    *output_file = '\0';
    
    /* arguments analysis */
	for (i = 1; i < argc; i++)
	{
		switch(argv[i][0])
		{
			case	'-':
				switch(argv[i][1])
				{
					case	'?':
					case	'h':
						printf("usage: nbc_x [? or -h] [-v [debug.out]] [-b] [-d display] [[-in] file_in] [-out file_out]\n");
						printf("switches:\n");
						printf("\t1) ? or -h to print this.\n");
						printf("\t2) -v for debug mode in default stderr ou specified file.\n");
						printf("\t3) -b for batch mode: requires in and out file names arguments.\n");
						printf("\t4) -d for specifying a particular display, default is local display.\n");
						printf("\t5) -in for input_file file (*.x_def or *.ntw or *.neur): without extension is assumed as *.x_def.\n");
						printf("\t6) -out for output_file simulation files (with or without any extention).\n"); 
						printf("\t\tREM: 1) 2) and 3) are exclusives.\n");
						exit(0);

					case	'v':
						debug = True;
						fdbg = stderr;
						break;
					case	'b':
						batch = True;
						break;
					case	'd':
						i++;
						if((the_display = XOpenDisplay(argv[i])) == NULL)
						{
							fprintf(stderr,"Can't open display %s for NBC Simulator\nOpen default display...\n",
							       argv[i]);
							if((the_display = XOpenDisplay(NULL)) == NULL)
							{
								printf("FATAL ERROR : Can't open default display for NBC Simulator\n");
								exit(0);
							}
						}
						break;
					case	'i':
						i++;
						strcpy(input_file,argv[i]);
						break;
					case	'o':
						i++;
						strcpy(output_file,argv[i]);
						break;
					case	'k':
						break;
					default	:
						printf("usage: nbc_x [? or -h] [-v [debug.out]] [-b] [-d display] [[-i] file_in] [-out file_out]\n");
						printf("switches:\n");
						printf("\t1) ? or -h to print this.\n");
						printf("\t2) -v for debug mode in default stderr or specified file.\n");
						printf("\t3) -b for batch mode: requires in and out file names arguments.\n");
						printf("\t4) -d for specifying a particular display, default is local display.\n");
						printf("\t5) -i for input_file file (*.ntw or *.neur): without extension is assumed as *.ntw.\n");
						printf("\t6) -o for output_file simulation files (with or without any extention).\n"); 
						printf("\t\tREM: 1) 2) and 3) are exclusives.\n");
						exit(0);
				} /* fin du switch */
			default	:
				strcpy(input_file,argv[i]);
		} /* fin du switch */
	} /* fin du for */

	if(!batch && !the_display)
   	{
	    if((the_display = XOpenDisplay(NULL)) == NULL)
	    {
		XtWarning("Sorry, cannot open display");
		exit(1);
	    }
   	}

	strcpy(input_file,get_lnk_name());
	previous_event = 0L;

#if 0
						debug = True;
						fdbg = stderr;
#endif

 	init_directories();

 	if ((simu = (SIMULATION *)calloc(1,sizeof(SIMULATION))) == NULL)
	{
		XtWarning("Sorry, cannot allocate enough memory for simulation...");
		exit(1);
	} /* fin du if */
 	activity_buffers = NULL;

	default_sim_values();

	/* les fichiers importants */

#if BUFFERIZE_AND_FLUSH
	pmb_fp = NULL;
	tms_fp = NULL;
	sim_fp = NULL;
#endif

 	if(!batch)
 	{
 		/* interactive */
 		
	    working_cursor = XCreateFontCursor(the_display,150);

	    /* 150 c'est l'id du curseur "montre" */
		
	    XtInitializeWidgetClass(applicationShellWidgetClass);
	    /*
	     * Install the tearOffModel resource converter.
	     */
#if (XmVersion >= 1002)
	    XmRepTypeInstallTearOffModelConverter();
#endif

	    /*
	     * Register special BuilderXcessory converters.
	     */
	    RegisterBxConverters(app);
	    
	    XtDisplayInitialize(app, the_display, BX_APP_NAME ,
		                    NULL, NULL, 0, &argc, argv);
		
	    the_colormap = DefaultColormapOfScreen(DefaultScreenOfDisplay(the_display));
		
		/* create the icon pixmap */
	    pixicon  = XCreateBitmapFromData (the_display, DefaultRootWindow(the_display),
		    nbc_icon_bits, nbc_icon_width, nbc_icon_height);  

	    /* Gestion des couleurs */
	    visual = DefaultVisual(the_display, DefaultScreen(the_display));
	    switch(visual->class)
	      {
	      case GrayScale:
	      case PseudoColor: 
		cree_palette();
		colormap = TRUE;
		break;
	      case DirectColor:	
	      case TrueColor: 
		couleurs();
		break;
	      case StaticColor:	
	      case StaticGray:	
		break;
	      } 

	    
	    ac = 0;
	    XtSetArg(args[ac], XmNtitle, "XNBC Simulator"); ac++;
	    XtSetArg(args[ac], XmNiconPixmap, pixicon);  ac++;
	    XtSetArg(args[ac], XmNiconName, "XNBC Simulator"); ac++;
	    XtSetArg(args[ac], XmNx, 0); ac++;
	    XtSetArg(args[ac], XmNy, 0); ac++;
	    XtSetArg(args[ac], XmNwidth, 720); ac++;
	    XtSetArg(args[ac], XmNheight, 300); ac++;
	    AppShell = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
	            applicationShellWidgetClass, the_display, args, ac);

	   
	 	/* windows creation */   
	 
	    sim_mainW = (Widget)Createsim_mainW(AppShell);
	    XtManageChild(sim_mainW);
	    XtPopup(XtParent(sim_mainW), XtGrabNone);

	    XMoveWindow(the_display, XtWindow(AppShell),50,50);
	    
	    form_cluster_parameters = (Widget)Createform_cluster_parameters(AppShell);
	    form_nucleus_parameters = (Widget)Createform_nucleus_parameters(AppShell);
	    form_help = (Widget)Createform_help(AppShell);
	    nucl_w = NULL;
	    clu_w = NULL;

	    if(!(*output_file))
	    {
	    	XtSetSensitive(widget_array[ID_pushB_save_state],False);
			XtSetSensitive(widget_array[ID_pushB_continue],False);
			XtSetSensitive(widget_array[ID_pushB_close],False);
			sprintf(buffer,"XNBC Simulator V%s: [no name]",NBC_X_VERSION);
	    	XtVaSetValues(AppShell,XmNtitle,buffer,NULL);
	    }

	    /* reading the simulation parameters saved the last time nbc_x was run */

	    home = getenv("HOME");
	    if(home)
	    {
			strcpy(buffer,home);
			strcat(buffer,"/.nbc_xrc");
	    }
	    else
			strcpy(buffer,"./.nbc_xrc");


	    if (!read_x_sim_file(buffer))
	    {

		    printf("Impossible to open the parameter file (%s)...\n",buffer);
		    sprintf(msg_error,"Impossible to open the parameter file (%s)...",buffer);
			display_error(widget_array[ID_sim_mainW],msg_error,"Using the default parameters!");
		}
	    else
	    {
		    status("Previous parameters read successfully");
	    }
	    if(simu->use_ntr)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_use_ntr],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_use_ntr],False,True);
	    }
	    if(simu->output_sim_f)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_sim_f],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_sim_f],False,True);
	    }
	    if(simu->output_pmb_f)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_pmb_f],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_pmb_f],False,True);
	    }
	    if(simu->output_abe_f)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_abe_f],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_abe_f],False,True);
	    }
	    if(simu->output_tms_f)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_tms_f],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_tms_f],False,True);
	    }
	    if(simu->output_gnp_f)
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_gnp_f],True,True);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_pushB_gnp_f],False,True);
	    }

	    simu->stop_at_ms = (float)simu->next_stop * simu->time_step;
	    simu->end_at_ms = (float)simu->last_it * simu->time_step;
	    simu->look_at_it_count = TRUE;
	    simu->look_at_ms_count = FALSE;
	    simu->look_at_glob_act = TRUE;
	    simu->look_at_dot_display = TRUE;
		simu->use_ntr = FALSE;

	    display_help(1);
	    
	    if(!(*input_file))
	    {
	    	strcpy(status_info,"No Loaded Network (*.ntw)");
	    }
	    else
	    {
	    	status("Loading...");
	    	XFlush(the_display);
	    	if(!load_simulator())
	    		init_simulation();
	    	else
	    		status("No Loaded Network (*.ntw)");
	    }
	    
		running = stopped = False;
	    
	    if (simu->watch_sim)
	    {
	      form_graph = (Widget)Createform_graph(AppShell);

	      XtManageChild(widget_array[ID_graph_form]);

	      XMoveWindow(the_display, XtWindow(form_graph),300,330);

	      the_drawable = XtWindow(graph_area);

	      if (gc_black)
			XFreeGC(the_display,gc_black);
	      if (gc_white)
			XFreeGC(the_display,gc_white);
	      if (gc_dot)
			XFreeGC(the_display,gc_dot);
	      if (gc_red)
			XFreeGC(the_display,gc_red);

	      gc_black = XCreateGC(the_display, the_drawable, 0, NULL);
	      XSetForeground(the_display,gc_black, BlackPixelOfScreen(DefaultScreenOfDisplay(the_display)));
	      XSetBackground(the_display,gc_black, WhitePixelOfScreen(DefaultScreenOfDisplay(the_display)));

	      gc_white = XCreateGC(the_display, the_drawable, 0, NULL);
	      XSetForeground(the_display,gc_white, WhitePixelOfScreen(DefaultScreenOfDisplay(the_display)));

	      gc_red = XCreateGC(the_display, the_drawable, 0, NULL);
	      XSetForeground(the_display,gc_red,BlackPixelOfScreen(DefaultScreenOfDisplay(the_display)) );
	      XSetLineAttributes(the_display,gc_red,5,LineSolid,CapButt,JoinMiter);

	      gc_dot = XCreateGC(the_display, the_drawable, 0, NULL);
	      XSetForeground(the_display,gc_dot, BlackPixelOfScreen(DefaultScreenOfDisplay(the_display)));
	      XSetBackground(the_display,gc_dot, WhitePixelOfScreen(DefaultScreenOfDisplay(the_display)));
	      XSetLineAttributes(the_display,gc_dot,2,LineSolid,CapRound,JoinRound);

	      XmToggleButtonSetState(widget_array[ID_toggleB_display],True,False);
	      XmToggleButtonSetState(widget_array[ID_toggle_global],simu->look_at_glob_act,False);
	      XmToggleButtonSetState(widget_array[ID_toggle_dot],simu->look_at_dot_display,False);
	    }
	    else
	    {
	      XmToggleButtonSetState(widget_array[ID_toggleB_display],False,False);
	    }

	    XtAppMainLoop(app);   
	    
	 }
	 else
	 {
	 	/* batch */
	 	
	 	launch_batch_simulation();
	 	
	 }	
	 
	 exit(0); 
}
/**********************************************
      Valeurs par defaut pour la simulation
*******************************************/
void default_sim_values()
{
	/* flags et valeurs par defaut */
	
	simu->watch_sim = TRUE;
	simu->saved = FALSE;
	simu->started = FALSE;

	simu->look_at_dot_display = TRUE;
	simu->look_at_glob_act = TRUE;
	simu->look_at_it_count = FALSE;
	simu->look_at_ms_count = FALSE;
	simu->use_ntr = FALSE;

	simu->time_step = TIME_STEP;
	simu->spike_width = SPIKE_WIDTH;

	simu->nb_step_spk = (unsigned long)(simu->spike_width/TIME_STEP);
	simu->nb_step_save_it = (unsigned long)nintf(1./TIME_STEP);
	simu->nb_step_save_ms = 1.;
	simu->nb_step_watch_it = (unsigned long)nintf(1./TIME_STEP);
	simu->nb_step_watch_ms = 1.;
	simu->count_show_it = NB_STEP_SHOW;
	simu->count_show_ms = (float)NB_STEP_SHOW / (float)NB_STEP_MS;
    simu->nb_it = 0;
    simu->time_ms = 0.;
    simu->it_to_save = 0;
    simu->it_to_watch = 0;
    simu->it_to_show = 0;
	simu->next_stop = NEXT_STOP;
	simu->last_it = LAST_IT;	
	simu->nb_next_it = NB_NEXT_IT;
	simu->output_tms_f = 1;
	simu->output_abe_f = 0;
	simu->output_sim_f = 1;
	simu->output_pmb_f = 1;
	simu->output_gnp_f = 0;
	simu->stop_at_ms = (float)simu->next_stop * simu->time_step;
	simu->end_at_ms = (float)simu->last_it * simu->time_step;
}
#endif




