#ifndef _NBC_COMPUTING_
#define _NBC_COMPUTING_

#define NBC_X_LIM_DEBUG 0
#define NBC_X_DEBUG 0
#define DEBUG 0
#define DEBUG_1 0
 
#include "creation-simu.h"


#ifndef PL_Cl
#	define PL_Cl 	   1.1e-9         /* m/s (=1.1e-7 cm/s)                */
#endif
#ifndef PL_Na
#	define PL_Na      5.0e-10         /* m/s (=5.0e-8 cm/s)                */
#endif
#ifndef PL_K
#	define PL_K       1.8e-8          /* m/s (=1.8e-6 cm/s)                */
#endif
#ifndef Jbar_p_Na
#	define Jbar_p_Na  6.0e-6   
#endif
#ifndef Km
#	define Km          13.4            /* mM (=13.4 mol/m^3)                */
#endif

#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
/*******************************************************************************
	Function for one XXM cluster iteration computation
*******************************************************************************/
#include "update_X_res.inc"

#endif

/*******************************************************************************
	Function for one virtual cluster iteration computation
*******************************************************************************/
#include "update_V_res.inc"

/*******************************************************************************
	Function for one CBM cluster iteration computation with transmitors
*******************************************************************************/
#include "update_G_res_use_ntr_simple.inc"

/*******************************************************************************
	Function for one CBM cluster iteration computation with transmitors
*******************************************************************************/
#include "update_G_res_use_ntr_full.inc"

/*******************************************************************************
	Function for one CBM cluster iteration computation without transmitors
*******************************************************************************/
#include "update_G_res_no_ntr_simple.inc"


/*******************************************************************************
	Function for one CBM cluster iteration computation without transmitors
*******************************************************************************/
#include "update_G_res_no_ntr_full.inc"

/*******************************************************************************
	Function for one PUM cluster iteration computation
*******************************************************************************/
#include "update_P_res.inc"


/*******************************************************************************
	Function for one LIM cluster iteration computation
*******************************************************************************/
#include "update_L_res.inc"

/*******************************************************************************
	Function for one BUM cluster iteration computation
*******************************************************************************/
#include "update_B_res.inc"

/*******************************************************************************
	Function for weight fluctuation according to hebb
*******************************************************************************/
void hebb_learn(int i, int k)
{
	int j;
	int l;

#if DEBUG_HEBB
	printf("Dans Hebb\n");
#endif
	for (j = 0; j < simu->nb_neurons; j++)
	{
		for(l=0 ; l<sim_neurons[j]->connected_neurons ; l++)
		{

			if((sim_neurons[i]->id == sim_neurons[j]->targets[l])
				&&((simu->nb_it 
				- sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]])
				 == sim_neurons[j]->axone_step[l]))
			{
#if DEBUG_HEBB
				printf("Dans chgt\n");
					sim_neurons[j]->connections[l] ++;
#endif
				if (sim_neurons[j]->connections[l] > 0) 
					sim_neurons[j]->connections[l] +=
						 (sim_neurons[i]->its_nucleus->hebb * 1. / 
						 (simu->nb_it - sim_neurons[i]->its_nucleus->date + 1)) ;
				if (sim_neurons[j]->connections[l] < 0)
					sim_neurons[j]->connections[l] -= 
					 (sim_neurons[i]->its_nucleus->hebb * 1. / 
					 (simu->nb_it - sim_neurons[i]->its_nucleus->date + 1)) ;
			}
		}
	}
}

/*******************************************************************************
	Function for Summing the inputs that are in the queues 
*******************************************************************************/
float suminputs(float *ip_tc, float *im_tc, float * i_amp,
		       unsigned long * i_time, short head, short tail)
/*
float	ip_tc:	 cste temps attack PSP
	im_tc:	 cste temps decay PSP 
float 	*i_amp:	 file d'attente des amplitudes des events
unsigned long	*i_time: file d'attente des moments de survenue des events
short	head:  tete et queue de la file d'attente 
	tail:

*/
{
	short	curr_input;
	float	curr_time;
	float	sum_i;

	/* If head == tail, the queues are empty */
	
	if (head == tail) return(0.);
	
	
	/*
	 * Loop for each input, sum their current values together
	 */
	
	sum_i = 0.0;
	for (curr_input = head; curr_input != tail; curr_input = (curr_input+1)%MAX_SPK)
	{
		
		curr_time = (float)i_time[curr_input];		/* moment de l'arrivee de l'input */

		if(!ip_tc[curr_input] || !im_tc[curr_input])
			continue;
		
		if ( simu->t_nb_it < (curr_time + ip_tc[curr_input]) && simu->t_nb_it > curr_time)
			sum_i += i_amp[curr_input] * (simu->t_nb_it - curr_time)/ip_tc[curr_input] *
				exp(((curr_time - simu->t_nb_it)/ip_tc[curr_input])+1.);
		if ( simu->t_nb_it >= (curr_time + ip_tc[curr_input]))
			sum_i += i_amp[curr_input] * (simu->t_nb_it - curr_time + im_tc[curr_input] - ip_tc[curr_input])/im_tc[curr_input] *
				exp(((curr_time - simu->t_nb_it - im_tc[curr_input] + ip_tc[curr_input])/im_tc[curr_input])+1.);

	}
	return(sum_i);
}

/*******************************************************************************
	Function for Summing the inputs that are in the queues 
*******************************************************************************/
float suminputs_G(float *ip_tc, float *im_tc, float * i_amp,
		       unsigned long * i_time, short head, short tail)
/*
float	ip_tc:	 cste temps attack PSP
	im_tc:	 cste temps decay PSP 
float 	*i_amp:	 file d'attente des amplitudes des events
unsigned long	*i_time: file d'attente des moments de survenue des events
short	head:  tete et queue de la file d'attente 
	tail:

*/
{
	short	curr_input;
	float	curr_time;
	float	sum_i;

	/* If head == tail, the queues are empty */
	
	if (head == tail) return(0.);
	
	
	/*
	 * Loop for each input, sum their current values together
	 */
	
	sum_i = 0.0;
	for (curr_input = head; curr_input != tail; curr_input = (curr_input+1)%MAX_MEM)
	{
		curr_time = (float)i_time[curr_input];		/* moment de l'arrivee de l'input */

		if(!ip_tc[curr_input] || !im_tc[curr_input])
			continue;
		
		if ( simu->t_nb_it < (curr_time + ip_tc[curr_input]) && simu->t_nb_it > curr_time)
			sum_i += i_amp[curr_input] * (simu->t_nb_it - curr_time)/ip_tc[curr_input] *
				exp(((curr_time - simu->t_nb_it)/ip_tc[curr_input])+1.);
		if ( simu->t_nb_it >= (curr_time + ip_tc[curr_input]))
			sum_i += i_amp[curr_input] * (simu->t_nb_it - curr_time + im_tc[curr_input] - ip_tc[curr_input])/im_tc[curr_input] *
				exp(((curr_time - simu->t_nb_it - im_tc[curr_input] + ip_tc[curr_input])/im_tc[curr_input])+1.);

	}
	return(sum_i);
}


/*******************************************************************************
	Function for summing NMDA inputs
*******************************************************************************/
float sum_NMDA(float coeff_NMDA, float t, float ip_tc, float im_tc, float * i_amp,
		       unsigned long * i_time, short head, short tail)
/*
float	coeff_NMDA:	coefficient modulant la conductance NMDA
	t:	 temps absolu de la manip
	ip_tc:	 cste temps attack NMDA
	im_tc:	 cste temps decay NMDA
float 	*i_amp:	 file d'attente des amplitudes des events
unsigned long	*i_time: file d'attente des moments de survenue des events
short	head:  tete et queue de la file d'attente 
	tail:

*/
{

	float		curr_time;
	short		curr_input;
	double	 	sum_i;


	/* If head == tail, the queues are empty */
	if (head == tail) return(0.); 
	/*
	 * Loop for each input, sum their current values together
	 */
	sum_i =  0.0;
	for (curr_input = head; curr_input != tail; curr_input = (curr_input+1)%MAX_MEM)
	{
		curr_time = (float)i_time[curr_input];		/* moment de l'arrivee de l'input */

		if ( curr_time <= t) 
			sum_i += coeff_NMDA * i_amp[curr_input]
				* (exp((curr_time - t)/ip_tc)
				- exp((curr_time - t)/im_tc));
	}
	return(sum_i);
}
/*******************************************************************************
	Function for getting next input time
*******************************************************************************/
/*
 * Renvoie l'intervalle pour l'input suivant. ATTENTION: ca renvoie un
 * intervalle et non une date
 */
unsigned long GetInpInterval(int i_neur)
{
	float	next_input;
	
	switch (sim_neurons[i_neur]->its_nucleus->input_type)
	{

		case 'n':		/* Normal */
			next_input = N(sim_neurons[i_neur]->its_nucleus->input_mean,
				sim_neurons[i_neur]->its_nucleus->input_sd);
			break;
		
		case 'p':		/* Poisson */
			next_input = f_poisson(sim_neurons[i_neur]->its_nucleus->input_mean);
			break;
		
		case 'g':		/* Gamma */
			next_input = f_gamma();
			break;

		case 'r':		/* Regular */
		default:
			next_input = sim_neurons[i_neur]->its_nucleus->input_mean;
			break;
	}
	return((unsigned long)nintf(next_input / simu->time_step));
}
/*******************************************************************************
	Function for initilizing next input computation
*******************************************************************************/
void init_inp_interval(int k_nucl)
{
	seed = (long)time(0);
	if (sim_nucleus[k_nucl]->input_type == 'g')
		initgamma(sim_nucleus[k_nucl]->input_mean,
				sim_nucleus[k_nucl]->input_sd);
}

/*******************************************************************************
	Function for reactualizing E_Na and E_K next input computation
*******************************************************************************/
void  reactualise_E_Na_K(CBM *cbm)
{
	
  cbm->E_K  = R_temp_ZF *
			log( cbm->K_ext / cbm->K_int);
  

  cbm->E_Na = R_temp_ZF *
			log( cbm->Na_ext / cbm->Na_int);


  cbm->E_Cl = R_temp_ZF *
			log( cbm->Cl_ext / cbm->Cl_int);
/*
  E_Goldman  = 1000.* (float)((float)(R*(Temp+273.15)/(Z*F)) *
			(float)(log((K_ext + (0.01 * Na_ext))/(K_int + (0.01 * Na_int)))));
  */
  
  cbm->E_Goldman  = R_temp_ZF *
			log(((PL_K * cbm->K_ext) + (PL_Na * (0.01 * cbm->Na_ext)) + (PL_Cl * cbm->Cl_int))/((PL_K * cbm->K_int) + (PL_Na * (0.01 * cbm->Na_int)) + (PL_Cl * cbm->Cl_ext)));

  cbm->E_fuite = cbm->E_Goldman;

#if DEBUG
  printf("cbm->E_K : %f cbm->K_ext : %f cbm->K_int : %f\n", 
	 cbm->E_K, cbm->K_ext, cbm->K_int);
  printf("cbm->E_Na : %f cbm->Na_ext : %f cbm->Na_int : %f\n", 
	 cbm->E_Na, cbm->Na_ext, cbm->Na_int);
  printf("cbm->E_Cl : %f cbm->Cl_ext : %f cbm->Cl_int : %f\n", 
	 cbm->E_Cl, cbm->Cl_ext, cbm->Cl_int);
  printf("cbm->E_fuite = cbm->E_Goldman : %f \n", 
	 cbm->E_fuite);
#endif
			
}


/*******************************************************************************
	Function for reactualizing E_Na and E_K next input computation
*******************************************************************************/
void  reactualise_E_Na_K_simple(CBM *cbm)
{
/*
	
  cbm->E_K  = R_temp_ZF *
			log( cbm->K_ext / cbm->K_int);
  

  cbm->E_Na = R_temp_ZF *
			log( cbm->Na_ext / cbm->Na_int);


  cbm->E_Goldman  = R_temp_ZF *
                        log((cbm->K_ext + (0.01 * cbm->Na_ext))/(cbm->K_int + (0.01 * cbm->Na_int)));

*/
  cbm->E_Goldman = cbm->E_fuite;
#if DEBUG
  printf("cbm->E_K : %f \n", 
	 cbm->E_K);
  printf("cbm->E_Na : %f \n", 
	 cbm->E_Na);
  printf("cbm->E_fuite = cbm->E_Goldman : %f \n", 
	 cbm->E_fuite);
#endif

}



#endif
