#ifndef _SIM_TYPES_
#define _SIM_TYPES_

#define MAX_BURST  20
#define MAX_DROGUE  20
#define MAX_CRT  20

#ifndef R
#define R 8232.0819112627         /* constante des gaz parfaits (* 1000) */
#endif
#ifndef Z
#define Z 1.                      /* valence des ions Na+ et K+ */
#endif
#ifndef F
#define F 9.648456e4              /* constante de Faraday en C/mole */
#endif
#ifndef ZF
#define ZF 9.648456e4              /* constante de Faraday en C/mole */
#endif

#ifndef M_PI 
#define M_PI	3.14159265358979323836
#endif

#define EXPONENTIELLE             1
#define RUNGE_KUTTA               2
#define RUNGE_KUTTA_VARIABLE_STEP 3
#define EULER                     4
#ifdef ADD_INTEGRATION_METHOD
/* Adapt here with the new method */
#define INT_METHOD                5
#endif

#define MICHAELIS_MENTON_MODEL    1
#define HH_SIMPLE_MODEL           2
#define HH_FULL_MODEL             3
#define HH_PUMP_MODEL             4
#ifdef ADD_A_MODEL
/* Adapt here with the new method */
#define NEW_MODEL                 5
#endif


typedef struct neuron
{
	unsigned long id;
	unsigned long number;
	float decharge_lag; 	/* last spike iteration time */
	float HC_ap;
	float HC_lat;
	float HC_vert;
	float radius;
	struct cluster *its_cluster;
	struct nucleus *its_nucleus;
	void *neuron_data;		/* LIM_DYNAMIC or PUM_DYNAMIC or CBM_DYNAMIC or BUM_DYNAMIC */
	
	float *axone_length;
	short *targets;
	short *connections;
	short *next_spike;
	short *nmda;

	unsigned long *axone_step;
	unsigned long connected_neurons;
	
	unsigned long t_next_i;
	unsigned long t_last_i;
	unsigned long spike_time[MAX_SPK];
	unsigned long inp_t[MAX_MEM];
	
	short spike_en_cours;
	short last_state;
	short new_state;
	short spike;
	short tete;
	short queue;
	short head;
	short tail;
	
} NEURON;

#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/

/********************* Modele X Neuron ************************/ 

typedef struct xxm
{
	short noise_active;                   /* bruit actif, non filtre */
	short filter_noise;             /* parametre a du filtre de Andre Fabio Kohn */
	float backgrd_noise_m;		/* noise input mean */
	float backgrd_noise_sd;		/* noise input standard deviation */

	float rest_pot;				/* resting potential */
	float pot_post_spike;		/* pot mb post spike */
	float overshoot;			/* post spike overshoot (rest - post_spike) */
	float tau_mb_pot;			/* mb time constant */
	float i_tau_mb_pot;			/* 1/mb time constant */
	float seuil;				/* firing threshold */
	float seuil_post_spike;		/* post spike firing threshold */
	float rev_pot_pos;			/* positive reversal potential (epsp) */
	float rev_pot_neg;			/* negative reversal potential (ipsp) */
	float tau_seuil;			/* firing threshold time constant post spike */
	float i_tau_seuil;			/* 1/firing threshold time constant post spike */
	float epsp;					/* epsp size */
	float ipsp;					/* ipsp size */
	float E_K;					/* potassium resting potential */
	float E_Na;					/* sodium resting potential */
	float E_Cl;					/* chlore resting potential */
	float E_Ca;					/* calcium resting potential */
	float tau_attack_epsp;		/* epsp attack time constant */
	float tau_decay_epsp;		/* epsp decay time constant */
	float tau_attack_ipsp;		/* ipsp attack time constant */	
	float tau_decay_ipsp;		/* ipsp decay time constant */
	float i_tau_attack_epsp;	/* 1/epsp attack time constant */
	float i_tau_decay_epsp;		/* 1/epsp decay time constant */
	float i_tau_attack_ipsp;	/* 1/ipsp attack time constant */	
	float i_tau_decay_ipsp;		/* 1/ipsp decay time constant */
	float shunt_tau;			/* mb shunt time constant */
	float K_rev_p;
	float K_rev_n;
	float tau_a_epsp;			/* time cst in iteration steps */
	float tau_d_epsp;
	float tau_a_ipsp;
	float tau_d_ipsp;
	float tau_i_seuil;			/* firing threshold time constant post spike */
	float tau_i_mb_pot;			/* mb time constant */
	float shunt_i_tau;			/* mb shunt time constant */
	
	float soma_diam_m;
	float soma_diam_sd;
	float capacite;                  /* capacite */
	float i_capacite;                /* 1/capacite */
	
}	XXM;

typedef struct xxm_dynamic
{
	float soma;
	float threshold;
	float seuil_peak;
	float delta_seuil;
	float soma_post_spike;
	float e_K;			/* Potassium resting potential */
	float e_Na;    			/* Sodium resting potential */
	float center_noise;             /* bruit centre filtre */
	
	float inp_max_pos[MAX_MEM];
	float inp_max_neg[MAX_MEM];
	float inp_nmda[MAX_MEM];
	float inp_ta[MAX_MEM];
	float inp_td[MAX_MEM];

} XXM_DYNAMIC;


/***********************       STOP     ***********************/
#endif

/********************* Modele B Neuron ************************/ 

typedef struct bum
{
	float backgrd_noise_m;		/* noise input mean */
	float backgrd_noise_sd;		/* noise input standard deviation */
	float backgrd_n_sd;		/* noise input standard deviation (apres time_step)*/

	float rest_pot;			/* resting potential */
	float pot_post_spike;		/* pot mb post spike */
	float burst_post_spike;		/* pot mb post spike pendant le burst */
	float overshoot;	       	/* post spike overshoot (rest - post_spike) */
	float burst_overshoot;	       	/* post spike overshoot (rest - post_spike) */
	float tau_mb_pot;	       	/* mb time constant */
	float i_tau_mb_pot;	       	/* 1/mb time constant */
	float seuil_post_spike;		/* post spike firing threshold */
	float rev_pot_pos;	       	/* positive reversal potential (epsp) */
	float rev_pot_neg;	       	/* negative reversal potential (ipsp) */
	float tau_seuil;	       	/* firing threshold time constant post spike */
	float i_tau_seuil;	       	/* 1/firing threshold time constant post spike */
	float epsp;		      	/* epsp size */
	float ipsp;	       		/* ipsp size */
	float E_K;     			/* potassium resting potential */
	float E_Na;			/* sodium resting potential */
	float freq_spike;               /* frequence intra burst de base (spk/s) */
	float freq_max;                 /* frequence max si plusieurs inouts (spk/s) */    
	float freq_b_spike;               /* frequence intra burst de base (spk/s) */
	float compose_fact;             /* facteur de composition des bursts */
	float duree_burst_min;          /* duree min des bursts (ms) */
	float duree_burst_max;          /* duree max des bursts (ms) */
	float duree_b_min;          /* duree min des bursts (ms) */
	float duree_b_max;          /* duree max des bursts (ms) */
	float burst_range;              /* duree max-min des bursts (ms) */
	float seuil;                    /* seuil de declenchement des spikes normaux */
	float seuil_burst;              /* seuil de declenchement des burst */           
	float tau_attack_epsp;		/* epsp attack time constant */
	float tau_decay_epsp;		/* epsp decay time constant */
	float tau_attack_ipsp;		/* ipsp attack time constant */	
	float tau_decay_ipsp;		/* ipsp decay time constant */
	float i_tau_attack_epsp;	/* 1/epsp attack time constant */
	float i_tau_decay_epsp;		/* 1/epsp decay time constant */
	float i_tau_attack_ipsp;	/* 1/ipsp attack time constant */	
	float i_tau_decay_ipsp;		/* 1/ipsp decay time constant */
	float shunt_tau;		/* mb shunt time constant */
	float K_rev_p;
	float K_rev_n;
	float tau_a_epsp;		/* time cst in iteration steps */
	float tau_d_epsp;
	float tau_a_ipsp;
	float tau_d_ipsp;
	float tau_i_seuil;			/* firing threshold time constant post spike */
	float tau_i_mb_pot;			/* mb time constant */
	float shunt_i_tau;			/* mb shunt time constant */

	float b_A;                      /* pour le calcul du soma dans les bursts */
	float b_v_plus;
	float b_v_rap;
	
	float soma_diam_m;
	float soma_diam_sd;
	float capacite;                 /* capacite du filtre */
	float i_capacite;               /* 1/capacite du filtre (evite un calcul) */
	float res_capa;                 /* resistance * capacite (evite un calcul) */
	float resistance;               /* resistance du filtre */
	float fatigue;                  /* fatigue du seuil */

	short noise_active;             /* bruit actif, non filtre */
	short filter_noise;             /* parametre a du filtre de Andre Fabio Kohn */
	short less_fatigue;             /* fatigue du seuil scd modele */
	
}	BUM;

typedef struct bum_dynamic
{
	float soma;
	float threshold;
	float seuil_peak;
	float delta_seuil;
	float soma_post_spike;
	float e_K;			/* Potassium resting potential */
	float e_Na;    			/* Sodium resting potential */
	float center_noise;             /* bruit centre filtre */
	float frequence;                /* frequence en cours */
	float periode;                  /* periode en cours */
	float decharge_lag;		/* depuis combien de temps ca a decharge */
	
	float inp_max_pos[MAX_MEM];
	float inp_max_neg[MAX_MEM];
	float inp_nmda[MAX_MEM];
	float inp_ta[MAX_MEM];
	float inp_td[MAX_MEM];

	int   spike_suivante;           /* moment de la prochaine spike */
	int   sablier[MAX_BURST];       /* le temps qui reste a chaque burst */
	int   nb_sabliers;              /* nombre de sabliers (nb de bursts imbriques) */
	char  in_burst;                 /* flag */
} BUM_DYNAMIC;

/********************* Modele P Neuron ************************/ 

typedef struct pum
{
	float backgrd_noise_m;		/* noise input mean */
	float backgrd_noise_sd;		/* noise input standard deviation */
	float backgrd_n_sd;		/* noise input standard deviation (apres time_step)*/

	float rest_pot;			/* resting potential */
	float pot_post_spike;		/* pot mb post spike */
	float overshoot;	       	/* post spike overshoot (rest - post_spike) */
	float tau_mb_pot;	       	/* mb time constant */
	float i_tau_mb_pot;	       	/* 1/mb time constant */
	float seuil;		       	/* firing threshold */
	float seuil_post_spike;		/* post spike firing threshold */
	float rev_pot_pos;	       	/* positive reversal potential (epsp) */
	float rev_pot_neg;	       	/* negative reversal potential (ipsp) */
	float tau_seuil;	       	/* firing threshold time constant post spike */
	float i_tau_seuil;	       	/* 1/firing threshold time constant post spike */
	float epsp;	       		/* epsp size */
	float ipsp;	       		/* ipsp size */
	float E_K;	       		/* potassium resting potential */
	float E_Na;	       		/* sodium resting potential */
	float E_Cl;	       		/* chlore resting potential */
	float E_Ca;		       	/* calcium resting potential */
	float tau_attack_epsp;		/* epsp attack time constant */
	float tau_decay_epsp;		/* epsp decay time constant */
	float tau_attack_ipsp;		/* ipsp attack time constant */	
	float tau_decay_ipsp;		/* ipsp decay time constant */
	float i_tau_attack_epsp;	/* 1/epsp attack time constant */
	float i_tau_decay_epsp;		/* 1/epsp decay time constant */
	float i_tau_attack_ipsp;	/* 1/ipsp attack time constant */	
	float i_tau_decay_ipsp;		/* 1/ipsp decay time constant */
	float shunt_tau;	       	/* mb shunt time constant */
	float K_rev_p;
	float K_rev_n;
	float tau_a_epsp;	       	/* time cst in iteration steps */
	float tau_d_epsp;
	float tau_a_ipsp;
	float tau_d_ipsp;
	float tau_i_seuil;			/* firing threshold time constant post spike */
	float tau_i_mb_pot;			/* mb time constant */
	float shunt_i_tau;			/* mb shunt time constant */
	float A;                        /* pour le calcul du soma hors burst */
	float v_plus;
	float v_rap;
	
	float soma_diam_m;
	float soma_diam_sd;
	float capacite;                 /* capacite du filtre */
	float i_capacite;               /* 1/capacite du filtre (evite un calcul) */
	float res_capa;                 /* resistance * capacite (evite un calcul) */
	float resistance;               /* resistance du filtre */
	float fatigue;                  /* fatigue du seuil */

	short noise_active;             /* bruit actif, non filtre */
	short filter_noise;             /* parametre a du filtre de Andre Fabio Kohn */
	short less_fatigue;             /* fatigue du seuil scd modele */
	
}	PUM;

typedef struct pum_dynamic
{
	float soma;
	float threshold;
	float seuil_peak;
	float delta_seuil;
	float soma_post_spike;
	float e_K;			/* Potassium resting potential */
	float e_Na;    			/* Sodium resting potential */
	float center_noise;             /* bruit centre filtre */
	
	float inp_max_pos[MAX_MEM];
	float inp_max_neg[MAX_MEM];
	float inp_nmda[MAX_MEM];
	float inp_ta[MAX_MEM];
	float inp_td[MAX_MEM];

} PUM_DYNAMIC;

/********************* Modele L Neuron ************************/ 

typedef struct lim
{
	float backgrd_noise_m;		/* noise input mean */
	float backgrd_noise_sd;		/* noise input standard deviation */
	float backgrd_n_sd;		/* noise input standard deviation (apres time_step)*/
	float backgrd_n_m;		/* noise input standard deviation (apres time_step)*/

	float rest_pot;			/* resting potential */
	float pot_post_spike;		/* pot mb post spike */
	float tau_mb_pot;	       	/* mb time constant */
	float i_tau_mb_pot;	       	/* 1/mb time constant */
	float i_tau_i_mb_pot;	       	/* intermediaire constante */
	float seuil;		       	/* firing threshold */
	float seuil_post_spike;		/* post spike firing threshold */
	float tau_seuil;	       	/* firing threshold time constant post spike */
	float i_tau_seuil;	       	/* intermediaire constant */
	float tau_i_seuil;	       	/* firing threshold time constant post spike */
	float epsp;	       		/* epsp size */
	float ipsp;	       		/* ipsp size */
	
	float capacite;                 /* capacite du filtre */
	float i_capacite;               /* 1/capacite du filtre (evite un calcul) */
	float res_capa;                 /* resistance * capacite (evite un calcul) */
	float resistance;               /* resistance du filtre */
	float fatigue;                  /* fatigue du seuil */

	short noise_active;             /* bruit actif, non filtre */
	short filter_noise;             /* parametre a du filtre de Andre Fabio Kohn */
	short less_fatigue;             /* fatigue du seuil scd modele */
	
}	LIM;

typedef struct lim_dynamic
{
	float soma;
	float threshold;
	float seuil_peak;
	float delta_seuil;
	float soma_post_spike;
	float center_noise;             /* bruit centre filtre */
	
} LIM_DYNAMIC;

/********************* Modele G Neuron ************************/ 

typedef struct cbm
{
	float backgrd_noise_m;		/* noise input mean */
	float backgrd_noise_sd;		/* noise input standard deviation */
	float backgrd_n_sd;		/* noise input standard deviation (apres time_step)*/
	
	float rest_pot;			/* resting potential */
	float decay_mb_pot;		/* decroissance du pot de mb */
	float g_Na,		/* conductance Na */
		g_K,		/* conductance K */
		g_Ca,		/* conductance Ca */
		g_Cl,		/* conductance Cl */
		g_A,		/* conductance K pour le courant Ia */
		g_M,		/* conductance k pour le courant Im */
		g_C,		/* conductance K pour le courant Ic */
		g_H,		/* conductance K pour le courant Ih */
		g_ahp,		/* conductance k pour le courant Iahp */
		g_Nap,		/* conductance Na persistant */
		g_Cab,		/* conductance Ca LVA */
		g_fuite,	/* conductance du courant de fuite */
		g_fuite_Na,	/* conductance du courant de fuite sodique */
		g_fuite_K,	/* conductance du courant de fuite potassique */
		g_Syn_pos,	/* conductance du courant synaptique*/
		g_Syn_neg,	/* conductance du courant synaptique*/
		K_ext,		/* concentration K extracellulaire */
		K_int,		/* concentration K intracellulaire */
		Na_ext,		/* concentration Na extracellulaire */
		Na_int,		/* concentration Na intracellulaire */
		Cl_ext,     /* concentration extracellulaire de Chlore */
		Cl_int,     /* concentration intracellulaire de Chlore */
		E_K,		/* Potentiel d'equilibre du potassium */
		K_rest,		/* Concentration de K au repos */
		E_Na,		/* Potentiel d'equilibre du sodium */
		E_Cl,		/* Potentiel d'equilibre du chlore */
		E_Ca,		/* Potentiel d'equilibre du calcium */
		E_H,		/* Potentiel d'equilibre du courant H */
		E_Goldman,	/* Potentiel de repos calcul avec l'equation de Goldman */
		E_fuite,	/* Potentiel de fuite */
		Ca_Influx,	/* Vitesse d'entree du calcium */
		Ca_Removal,	/* Vitesse de sortie du calcium */
		Ca_Kd,		/* Constante de dissociation du calcium */
		K_Ca_Kd,	/* Constante de dissociation du calcium pour l'activation de I_K */
		lambda_Ik,	/* Parametre de controle de Tau maximal pout K */
		lambda_Im,	/* Parametre de controle de Tau maximal pout Im */
		lambda_Ic,	/* Parametre de controle de Tau maximal pout Ic */
		lambda_Ih,	/* Parametre de controle de Tau maximal pout Ih */
		lambda_Inap,	/* Parametre de controle de Tau maximal pout Inap */
		lambda_Icab,	/* Parametre de controle de Tau maximal pout Icab */
		tau_m_cab,	/* Tau de l'activation de ICab */
		capacite,	/* capacite de la mb */
		E_Syn_pos,	/* Potentiel d'equilibre du courant synaptique */
		E_Syn_neg,	/* Potentiel d'equilibre du courant synaptique */
		g_NMDA,		/* conductance NMDA */
		tau_attack_NMDA,/* constante de temps 1 */
		tau_decay_NMDA,	/* constante de temps 2 */
		eta_NMDA,	/* coefficient liant la conductance des canaux
				   sensibles au NMDA a la concentration en magnesium */
		gamma_NMDA,	/* coefficient de dependance au voltage des canaux NMDA */
		magnesium,	/* concentration en Mg++ extracellulaire */
		rev_pot_NMDA;	/* potentiel d'inversion du courant NMDA */

	float i_capacite;               /* 1/capacite du filtre (evite un calcul) */
	float res_capa;                 /* resistance * capacite (evite un calcul) */
	float resistance;               /* resistance du filtre */


	float rev_pot_pos;		/* positive reversal potential (epsp) */
	float rev_pot_neg;	       	/* negative reversal potential (ipsp) */
	float tau_attack_epsp;		/* epsp attack time constant */
	float tau_decay_epsp;		/* epsp decay time constant */
	float tau_attack_ipsp;		/* ipsp attack time constant */	
	float tau_decay_ipsp;		/* ipsp decay time constant */
	float i_tau_attack_epsp;	/* 1/epsp attack time constant */
	float i_tau_decay_epsp;		/* 1/epsp decay time constant */
	float i_tau_attack_ipsp;	/* 1/ipsp attack time constant */	
	float i_tau_decay_ipsp;		/* 1/ipsp decay time constant */
	float tau_a_epsp;	       	/* time cst in iteration steps */
	float tau_d_epsp;
	float tau_a_ipsp;
	float tau_d_ipsp;
	float tau_a_NMDA;
	float tau_d_NMDA;
	
	float soma_diam_m;
	float soma_diam_sd;
	float spike_slope;		/* pente de detection des spikes */
	float pente_spike;              /* adapte au time step */

	float	Ca_i_init,	        /* Valeurs initiales */
		K_ext_init,
		m_ahp_init,
		m_K_init,
		m_Nap_init,
		le_potentiel_init,
		le_potentiel_E_K_init,
		le_potentiel_E_Na_init,
		tau_Ca_init,
		m_Ca_init,
		m_Ih_init,
		m_Cab_init,
		h_Cab_init,
		h_Na_init,
		h_A_init;

				/*
				 * variable globales pour la sigmoide
				 * d'activation/inactivation
				 */

	float	V_Na,		/* V 1/2 pour la Na */
		A_Na,		/* coeff a pour le Na */
		Pow_Na,		/* Exposant pour le Na */

		V_K,		/* V 1/2 pour la K */
		A_K,		/* coeff a pour le K */
		Pow_K,		/* Exposant pour le K */

		V_INa_A,	/* V 1/2 pour INa activation */
		A_INa_A,	/* coeff a pour INa activation */
		Pow_INa_A,	/* Exposant pour INa activation */
		V_INa_B,	/* V 1/2 pour INa inactivation */
		A_INa_B,	/* coeff a pour INa inactivation */
		Pow_INa_B,	/* Exposant pour INa inactivation */
		Tau_INa_B,	/* Cste de temps pour INa inactivation */
		Tau_INa_A,      /* Cste de temps pour INa activation */
		lambda_INa,	/* Parametre de controle de Tau maximal pour Na */

		V_Ca,		/* V 1/2 pour la Ca */
		A_Ca,		/* coeff a pour le Ca */
		Pow_Ca,		/* Exposant pour le Ca */
		Tau_Ca,		/* Cste de temps pour le Ca */

		V_Ia_A,		/* V 1/2 pour Ia activation */
		A_Ia_A,		/* coeff a pour Ia activation */
		Pow_Ia_A,	/* Exposant pour Ia activation */
		V_Ia_B,		/* V 1/2 pour Ia inactivation */
		A_Ia_B,		/* coeff a pour Ia inactivation */
		Pow_Ia_B,	/* Exposant pour Ia inactivation */
		Tau_Ia_B,	/* Cste de temps pour Ia inactivation */

		V_Im,		/* V 1/2 pour Im  (potentiel du point d'inflexion) */
		A_Im,		/* coeff a pour Im (pente de la sigmoide) */

		V_Ic,		/* V 1/2 pour Ic (potentiel du point d'inflexion) */
		A_Ic,		/* coeff a pour Ic (pente de la sigmoide) */

		V_Inap,		/* V 1/2 pour INap (potentiel du point d'inflexion) */
		A_Inap,		/* coeff a pour INap (pente de la sigmoide) */

		Pow_m_cab,	/* Exposant pour ICab activation */
		Vm_Icab,	/* V 1/2 pour INap (potentiel du point d'inflexion) */
		Am_Icab,	/* coeff a pour INap (pente de la sigmoide) */
		Vh_Icab,	/* V 1/2 pour Icab (potentiel du point d'inflexion) */
		Ah_Icab,	/* coeff a pour Icab (pente de la sigmoide) */

		V_Ih,		/* V 1/2 pour Ih (potentiel du point d'inflexion) */
		A_Ih;		/* coeff a pour Ih (pente de la sigmoide) */
		

	short neurotransmitor;
	short nb_receptors;
	
	short filter_noise;             /* parametre a du filtre de Andre Fabio Kohn */
	short noise_active;
	short model_type;
	short integration_type;

	RECEPTOR Receptors[NB_MAX_RECEPTORS];

	
}	CBM;

typedef struct cbm_dynamic
{
	float soma;

	float	g_tot,		/* Conductance totale de la mb */
		e_K,		/* potentiel d'equilibre de K */
		e_Na,		/* potentiel d'equilibre de Na */
#if 0		
		K_ext,		/* concentration K extracellulaire */
		K_int,		/* concentration K intracellulaire */
		Na_ext,		/* concentration Na extracellulaire */
		Na_int,		/* concentration Na intracellulaire */
		Cl_ext,     /* concentration extracellulaire de Chlore */
		Cl_int,     /* concentration intracellulaire de Chlore */
#endif
		Ca_i,	        /* concentration en calcium intracellulaire */
		m_Na,		/* variable d'ouverture des canaux Na */
		h_Na,		/* variable de fermeture des canaux Na */
		tau_Na,		/* cste de temps de g_Na */
		m_K,		/* variable d'ouverture des canaux K */
		h_K,		/* variable de fermeture des canaux K*/
		tau_K,		/* cste de temps de g_K */
		m_Ca,		/* variable d'ouverture des canaux Ca */
		h_Ca ,		/* variable de fermeture des canaux Ca */
		tau_Ca,		/* cste de temps de g_Ca */
		m_Cl,		/* variable d'ouverture des canaux Cl */
		h_Cl,		/* variable de fermeture des canaux Cl */
		tau_Cl,		/* cste de temps de g_Cl */
		m_A,		/* variable d'ouverture des canaux K pour le courant Ia */
		h_A,		/* variable de fermeture des canaux K pour courant Ia */
		tau_A,		/* cste de temps de g_A */
		m_M,		/* variable d'ouverture des canaux k pour le courant Im */
		tau_M,		/* cste de temps de g_M */
		m_C,		/* variable d'ouverture des canaux k pour le courant Ic */
		tau_C,		/* cste de temps de g_C */
		m_Ih,		/* variable d'ouverture des canaux k pour le courant Ic */
		tau_Ih,		/* cste de temps de g_C */
		m_ahp,		/* variable d'ouverture des canaux k pour le courant Iahp */
		tau_ahp,	/* cste de temps de g_ahp */
		m_Nap,		/* variable d'ouverture des canaux Na persistants */
		m_Cab,		/* variable d'ouverture des canaux Ca LVA */
		h_Cab,		/* variable de fermeture des canaux Ca LVA */
		e_fuite,	/* Potentiel d'equilibre du aux	courants de fuite */
		e_Goldman,	/* Potentiel de repos calcul avec l'equation de Goldman */
		
		courant_tot,	/* courant total traversant la mb */
		rev_pot_pos,	/* potentiel de reversion positif */
		rev_pot_neg,	/* potentiel de reversion negatif */
		input_fact,	/* facteur additionel          */
		fact_echelle;   /* facteur pour amperage */

	
	float inp_max_pos[MAX_MEM];
	float inp_max_neg[MAX_MEM];
	float inp_nmda[MAX_MEM];
	float inp_ta[MAX_MEM];
	float inp_td[MAX_MEM];
	float center_noise;             /* bruit centre filtre */
	float diametre;		/* diametre de la cellule en microm */
	float surface ;		/* surface de la cellule en cm2 */
	float surface_normalisee ;	/* rapport de la surface parrapport a une cellule de 20 u de diametre */

} CBM_DYNAMIC;

typedef struct neur_virtual
{
	FILE *fp_tms;
		
} VIRTUAL;

typedef struct cluster
{
	short number;		        /* cluster number (from 1 to NB_CLUSTERS +1) */
	short cluster_size;		/* size in neurons */
	short spike_duration;		/* spike duration in nb iterations */
	unsigned long nb_act_clu;
	char name[256];			/* name of the cluster */
	char file_name[256];	        /* file name *.unit ou .tms	*/
	char model;			/* P->pum, G->cbm, V->virtual */
	char not_used;			/* struct alignement */
	void *clu_data;			/* pointer to cluster characteristics,
								according to the model */
	struct neuron **neuron_array;	/* reference array to access cluster neurons */
	
	short nb_nucleus;
	struct nucleus **nucleus_array;
	
	void (*update_soma)();	        /* update mb pot function according to the model */
	short glutamate;		/* flag interacting with nmda */

}	CLUSTER;


typedef struct nucleus
{
	
	float input_mean;
	float input_sd;
	float scale_f;
	float backgrd_noise_m;
	float backgrd_noise_sd;
	float backgrd_n_sd;		/* noise input standard deviation (apres time_step)*/
	float modul_period;
	float modul_amp;
	float modul_phase;
	float modul_value;
	float fluctuation;
	float low_mb_pot;
	float up_mb_pot;
	float hebb;
	unsigned long date;		/* beginning of learning */
	unsigned long pulse_up;	        /* beginning of pulses */
	unsigned long pulse_down;	/* end of pulses */

	struct neuron **neuron_array;
	struct cluster **cluster_array;
	void (*learning)();		/* connection weights fluctuation law */

	int nb_neurons;
	int nucleus_size;
	int nb_act_nucl;
	int nb_pulses;
	int num_pulse;
	int pulses_restants;
	int recent_last_spike;
	int old_last_spike;
	int prob_spike;

	short id;
	short number;
	short nb_clusters;
	short inhib;
	short noise_active;

	char drogue[MAX_DROGUE];
	char crt[MAX_CRT];
	char name[256];
	char modul_type;		/* 0,s,t,r,p */
	char input_type;		/* 0,r,n,g,e */
	char law;			/* h for Hebb */	
	char alive;			/* y or n */
	char drug_enabled;		/* y or n */
	char filter_noise;              /* y or n */

}	NUCLEUS;

typedef struct simulation
{
	float nb_step_save_ms;
        float nb_step_watch_ms;
	float count_show_ms;
	float time_step;
	float spike_width;
	float t_nb_it;
	float stop_at_ms;
	float end_at_ms;
	float time_ms;
	float Temp;

	unsigned long nb_it;
	unsigned long it_to_save;
	unsigned long it_to_watch;
	unsigned long it_to_show;
	unsigned long next_stop;
	unsigned long last_it;
	unsigned long nb_next_it;
	unsigned long nb_step_save_it;
	unsigned long nb_step_watch_it;
	unsigned long count_show_it;
	unsigned long nb_step_spk;

	int total_active_neurons;
	int nb_neurons;

	short nb_nucleus;
	short nb_clusters;
	short parameters_loaded;
	short look_at_dot_display;
	short look_at_glob_act;
	short look_at_it_count;
	short look_at_ms_count;
	short started;
	short saved;
	short watch_sim;
	short use_ntr;

	char name[256];
	char prm_name[256];
	char sim_name[256];
	char mod_name[256];
	char tms_name[256];
	char abe_name[256];
	char pmb_name[256];
	char gnp_name[256];
	char cnx_name[256];
#if 0
	char x_def_name[256];
#endif
	char x_sim_name[256];
	char neur_name[256];
	char lnk_name[256];
	char wgt_name[256];
	char len_name[256];
	char clu_name[256];
	char nucl_name[256];
	char anat_name[256];

	short   output_abe_f;
	short   output_tms_f;
	short   output_pmb_f;
	short   output_sim_f;
	short   output_gnp_f;
		
}	SIMULATION;

typedef struct clu
{
	char *file;
	short *refs;
	short nb_refs;
}	CLU;

#endif






