/*******************************************************************************
	Function for one CBM cluster iteration computation without transmitors
*******************************************************************************/

void update_G_res_no_ntr_simple(int pos)
{

	int	i, j, k, l, n, d_l, i_r, ntrm;
	
	int	flag_psp[NB_MAX_RECEPTORS];
	float	diff_volt[NB_MAX_RECEPTORS];

	float	
		white_noise,
		t_i, 			/* Elapsed time */
		psp, epsp, ipsp, nmda, somme_psp,	
		delta_ipsp,		
		delta_epsp,
		net_input_pos,		/* somme de ce qui arrive positif (intermediaire) */
		net_input_neg,		/* somme de ce qui arrive negatif (intermediaire) */
		net_input_nmda,		/* somme de ce qui arrive en NMDA (intermediaire) */
		coeff_NMDA,		/* intermediaire de calcul */
		noise = 0.,		/* background noise */	
		clu_noise = 0.,		/* cluster background noise */	
		nucl_noise = 0.,	/* nucleus background noise */
		sum_i_pos,		/* pour la somme des inputs */
		sum_i_neg,		/* pour la somme des inputs */

		G_Na,		/* conductance totale du courant sodique */
		I_Na,		/* Fast Sodium Current*/
		G_K,		/* conductance totale de la rectification retardee*/
		I_K,		/* courant K de la rectification retardee */
		G_Syn_pos,	/* conductance totale pour le courant synaptique */
		G_Syn_neg,	/* conductance totale pour le courant synaptique */
		I_Syn_pos,	/* courant synaptique */
		I_Syn_neg,	/* courant synaptique */

		tau_m_Na,       /* cste de temps de l'activation des canaux Na  */
		m_1_Na,         /* facteur intervenant dans le calcul de m_Na*/
		alpha_m_Na,     /* facteur intervenant dans le calcul de m_Na et tau_m_Na */
		beta_m_Na,      /* facteur intervenant dans le calcul de m_Na et tau_m_Na */
		tau_h_Na,       /* cste de temps d'inactivation des canaux Na */
		h_1_Na,         /* facteur intervenant dans le calcul de h_na */
		alpha_h_Na,     /* facteur intervenant dans le calcul de h_Na et tau_h_Na */
		beta_h_Na,      /* facteur intervenant dans le calcul de m_Na et tau_h_Na */
		tau_m_K,        /* cste de temps de l'activation des canaux K */
		m_1_K,          /* facteur intervenant dans le calcul de m_K */
		alpha_m_K,      /* facteur intervenant dans le calcul de tau_m_K */
		beta_m_K,       /* facteur intervenant dans le calcul de tau_m_K */
		alpha_m1_K,     /* facteur intervenant dans le calcul de m1_k */
		beta_m1_K,      /* facteur intervenant dans le calcul de m1_k */
		tau_h_K,        /* cste de temps d'inactivation des canaux K */
		h_1_K,          /* facteur intervenant dans le calcul de h_K */

		soma_avant,	/* pour calculer la pente */
		input_sup,	/* voltage du a l'input supplementaire */
		delta_potentiel,/* cgt local de potentiel */
		decay,		/* variable intermediaire */
		decay_mb_pot,		/* variable intermediaire */
		curr_time;	/* variable intermediaire */

	char buffer[256];

	CBM *clu_cbm;
	CBM *clu_cnx;
	CBM_DYNAMIC *cbm_dyn;

	NEURON *neuron_src;
	NEURON *neuron_tgt;
	int knobs, jj;

	
	if((pos<0) || (pos>=simu->nb_neurons))
	{
		fprintf(stderr,"i is out of range in update_G_res_no_ntr!!!!\n");
		exit(-1);
	}
	i = pos;

	k = (sim_neurons[i]->its_cluster->number)-1;
	n = (sim_neurons[i]->its_nucleus->number)-1;
	clu_cbm = (CBM *)sim_neurons[i]->its_cluster->clu_data;
	cbm_dyn = (CBM_DYNAMIC *)sim_neurons[i]->neuron_data;

	t_i = (float)(simu->nb_it);


	soma_avant = cbm_dyn->soma;

#if DEBUG_1
	printf("\n*****************\tentree dans update_G_res_no_ntr_simple neurone %d ***************\nIteration : %f\nsoma : %f\n",i,t_i,soma_avant);
#endif
#if DEBUG
	printf("time step: %f m (%.3f it/ms)\nspike duration: %f m (%lu i)\nsaved: %f m (%lu i)\n",
	       simu->time_step,
	       nintf(1./simu->time_step),
	       simu->spike_width,
	       simu->nb_step_spk,
	       simu->nb_step_save_ms,
	       simu->nb_step_save_it);
#endif
/*************** TIRE DIRECTEMENT DE G_NEURON V7.7 ***************************/

  /******************************************************************/
  /*----  Calcul de la conductance au k pour le courant
          de rectification retardee                     ------------*/
  /******************************************************************/

      /****      variable d'activation des canaux K         ****/
	alpha_m_K =  exp(clu_cbm->A_K * (cbm_dyn->soma - clu_cbm->V_K));
	beta_m_K = exp(-clu_cbm->A_K * (cbm_dyn->soma - clu_cbm->V_K));
	tau_m_K = (1. / ( clu_cbm->lambda_Ik * (alpha_m_K + beta_m_K)));
	m_1_K = alpha_m_K / ( alpha_m_K + beta_m_K);
	decay = (exp (-simu->time_step / tau_m_K));
	cbm_dyn->m_K = cbm_dyn->m_K * decay + m_1_K * (1. - decay);
      
      /****         Conductance et courant K                 ****/
	if (stim_drug_f[n] && sim_nucleus[n]->drogue[drug_K]) 
		G_K = 0.;
	else
		G_K = clu_cbm->g_K * pow(cbm_dyn->m_K,clu_cbm->Pow_K);
#if DEBUG
      printf("K  -> m_K: %f A_K: %f V_K: %f G_K = %f g_K = %f E_K = %f\n",cbm_dyn->m_K,  clu_cbm->A_K, clu_cbm->V_K, G_K, clu_cbm->g_K, clu_cbm->E_K);
#endif
 
  
  /******************************************************************/
  /*---------     calcul de la conductance au Na     ---------------*/
  /******************************************************************/
  
  /*---------------------------------------------------------------*/ 
	/****      variable d'activation des canaux Na          ****/
	m_1_Na = 1. /(1. + exp (-2.0 * clu_cbm->A_INa_A * (cbm_dyn->soma - clu_cbm->V_INa_A)));

	/****    variable d'inactivation des canaux Na         ****/
	h_1_Na = 1. /(1. + exp (-2.0 * clu_cbm->A_INa_B * (cbm_dyn->soma - clu_cbm->V_INa_B)));
	decay = exp ( -simu->time_step / clu_cbm->Tau_INa_B);
	cbm_dyn->h_Na = cbm_dyn->h_Na * decay + h_1_Na * (1. - decay);

	/****        Conductance et Courant Na                ****/ 
	if (stim_drug_f[n] && sim_nucleus[n]->drogue[drug_Na]) 
		G_Na = 0.;
	else
		G_Na = clu_cbm->g_Na * pow(m_1_Na,clu_cbm->Pow_INa_A) * pow(cbm_dyn->h_Na,clu_cbm->Pow_INa_B) ;

#if DEBUG
	printf("Na -> V: %f A: %f V: %f\n",cbm_dyn->soma,  clu_cbm->A_Na, clu_cbm->V_Na);
	printf("Na -> G_Na: %f g_Na: %f m_1_Na: %f E_Na: %f\n", G_Na, clu_cbm->g_Na, m_1_Na, clu_cbm->E_Na);
#endif


  /*---------------------------------------------------------------*/ 
     

	if (clu_cbm->noise_active)
	{
      		seed = (int)time(0) & 077;
		clu_noise = gasdev(&seed);
		clu_noise *= clu_cbm->backgrd_n_sd;
#if KOHN_DEBUG		
		if (clu_cbm->filter_noise)
		{
		  cbm_dyn->center_noise = cbm_dyn->center_noise * exp(-(simu->time_step) / (clu_cbm->res_capa)) 
		                          + clu_cbm->i_capacite * clu_noise; 
		  clu_noise =  cbm_dyn->center_noise + clu_cbm->backgrd_noise_m;		  
		}
		else
#endif
		 clu_noise += clu_cbm->backgrd_noise_m;
	} 
#if KOHN_DEBUG
	printf("clu_cbm->noise_active: %d, clu_cbm->filter_noise: %d\n",
	       clu_cbm->noise_active, clu_cbm->filter_noise);
	printf("cbm_dyn->center_noise: %f\nclu_cbm->res_capa: %f\nclu_cbm->i_capacite: %f\nclu_noise:%f\n*****\n",
	       cbm_dyn->center_noise, clu_cbm->res_capa,
	       clu_cbm->i_capacite, clu_noise); 
#endif
	if(stim_noise_f[n] && sim_nucleus[n]->noise_active)
	{
		nucl_noise = gasdev(&seed);
		nucl_noise *= sim_nucleus[n]->backgrd_n_sd;
		nucl_noise += sim_nucleus[n]->backgrd_noise_m;		
	} 
		
		
	if (stim_modul_f[n])
	{
	  nucl_noise += sim_nucleus[n]->modul_value;
	}

	noise = clu_noise + nucl_noise;
	
#if NBC_X_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Noise for neuron %lu: total %f, nucl %f, clu %f\n",
			sim_neurons[i]->id,noise,nucl_noise,clu_noise);
	}
#endif

	/*
	 *  Inputs from other neurons
	 */

  /***********************************************************/
  /*--- calcul de la conductance du courant synaptique   ----*/
  /***********************************************************/


/* Si NON prise en compte des neurotransmetteurs */
	/*
	 *  Inputs from other neurons
	 */
	epsp = ipsp = nmda = somme_psp = 0.;
	delta_epsp = clu_cbm->rev_pot_pos - cbm_dyn->soma;
	delta_ipsp = clu_cbm->rev_pot_neg - cbm_dyn->soma;
			   
	for (j = 0; j < simu->nb_neurons; j++) 
	{
#if NBC_X_DEBUG
		printf("\a CONNECTION from neuron %lu (j = %d) \n",
			sim_neurons[j]->id, j);
#endif
				
#if 0
		neuron_src = sim_neurons[i];
		for(jj=0 ; jj<simu->nb_neurons ; jj++)
		{
			neuron_tgt = sim_neurons[jj];
			knobs = 0;
			for(k=0 ; k<neuron_src->connected_neurons ;k++)
			{
				if(neuron_tgt == sim_neurons[which_neuron(neuron_src->targets[k])])
				{
					knobs = neuron_src->connections[k];
					printf("\a to neuron %lu (l %d):  %d\n",
					neuron_tgt->id, jj, knobs);

				}	
			}
		}
/***************************/
#endif
		for (l = 0 ; l < sim_neurons[j]->connected_neurons ; l++)
		{
#if NBC_X_DEBUG
			printf("\a to neuron %lu (l %d):  %d\n",
				sim_neurons[j]->targets[l], l, sim_neurons[j]->connections[l]);
#endif
			if((sim_neurons[j]->targets[l] == sim_neurons[i]->id) &&
			  (sim_neurons[j]->connections[l]))
			{
#if DEBUG_1
				printf("****\nsimu->nb_it = %d\n sim_neurons[%d]->spike_time[sim_neurons[%d]->next_spike[%d]] = %d\n sim_neurons[%d]->axone_step[%d] = %d (diff = %d)\n****\n",simu->nb_it,j,j,l, sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]], j,l, sim_neurons[j]->axone_step[l], (simu->nb_it - sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]]) );
#endif

				if ((simu->nb_it - sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]]) ==
				  sim_neurons[j]->axone_step[l]) 
				{
#if DEBUG_1
					printf("sim_neurons[%d]->connections[%d] = %d\n",j,l,sim_neurons[j]->connections[l]);
#endif
					if (sim_neurons[j]->connections[l] > 0)
					{
						epsp += clu_cbm->g_Syn_pos * sim_neurons[j]->connections[l];
#if DEBUG_1
						printf("epsp = %f\n",epsp);
#endif
					}
					else
					{
						ipsp += clu_cbm->g_Syn_neg * sim_neurons[j]->connections[l];
#if DEBUG_1
						printf("ipsp = %f\n",ipsp);
#endif
					}
					sim_neurons[j]->next_spike[l] = (sim_neurons[j]->next_spike[l] + 1) % MAX_SPK;
						
#if NBC_X_DEBUG
					if(debug &&(sim_neurons[j]->connections[l] > 0))
					{
						fprintf(fdbg,"Epsp contribution from neuron %lu to neuron %lu:  %f( new epsp:%f)\n",
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_cbm->g_Syn_pos * sim_neurons[j]->connections[l],epsp);
					}
					if(debug &&(sim_neurons[j]->connections[l] < 0))
					{
						fprintf(fdbg,"Ipsp contribution from neuron %lu to neuron %lu:  %f ( new ipsp:%f)\n",
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_cbm->g_Syn_neg * sim_neurons[j]->connections[l],ipsp);
					}
#endif
				} 
					
			} 
		}
	}

	
	/*
	 *	Is it time for input?
	 */
	if (sim_nucleus[n]->input_type && stim_pulse_f[n] 
		&& (simu->nb_it >= sim_neurons[i]->t_next_i))
	{
		if (!sim_nucleus[n]->inhib)
		{
			epsp += (clu_cbm->g_Syn_pos  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input epsp contribution to neuron %lu:  %f( new epsp:%f)\n",
					sim_neurons[i]->id,
					clu_cbm->g_Syn_pos * sim_nucleus[n]->scale_f,epsp);
			}
#endif
		} 
		else
		{
			ipsp += -(clu_cbm->g_Syn_neg  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input ipsp contribution to neuron %lu:  -%f( new ipsp:%f)\n",
					sim_neurons[i]->id,
					clu_cbm->g_Syn_neg * sim_nucleus[n]->scale_f,ipsp);
			}
#endif
		}

		/*
		 * stimulation (last and next)
		 */
		
		sim_neurons[i]->t_last_i = sim_neurons[i]->t_next_i;
		sim_neurons[i]->t_next_i += GetInpInterval(i);
		
		/*
		 * Sauvegarde de la serie temporelle
		 *    Save time of input 
		 * (valeur negative = external input)
		 */
		
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f -%d\n",
			simu->time_ms, sim_nucleus[n]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f -%d\n",
			  simu->time_ms, sim_nucleus[n]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"A,%d,%lu\n", n+1, simu->nb_it - previous_event);
		previous_event = simu->nb_it;

#endif
	}

	if (ipsp)
	{
		/*
		* on met le resultat ipsp dans la file d'attente
		* temporelle
		*/
		
		sim_neurons[i]->inp_t[sim_neurons[i]->tail] = simu->nb_it;
		
		/*
		* Afin de ne pas depasser les pot de reversion, on
		* doit verifier que ce que l'on met dans la file
		* d'attente n'es pas plus grand que la difference
		* entre le soma et le rev pot.  Ca pose pas mal de
		* problemes tordus de signes....
		*/
		
		
#if DEBUG
		if (ipsp < delta_ipsp)
			ipsp = delta_ipsp;
#endif
		
#if DEBUG_1
		printf("\t Somme IPSP for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			ipsp);
#endif
#if  NBC_X_DEBUG 
		if(debug)
		{
			fprintf(fdbg,"\to Somme IPSP for neuron %lu:  %f\n",
				sim_neurons[i]->id,
				ipsp);
		}  
#endif
	
		cbm_dyn->inp_max_neg[sim_neurons[i]->tail] = ipsp;
		cbm_dyn->inp_ta[sim_neurons[i]->tail] = clu_cbm->tau_a_ipsp;
		cbm_dyn->inp_td[sim_neurons[i]->tail] = clu_cbm->tau_d_ipsp;
#if DEBUG_1
		printf("\tipsp asc : cbm_dyn->inp_td[sim_neurons[%d]->tail] :  %f\n",i,cbm_dyn->inp_ta[sim_neurons[i]->tail]);
		printf("\tipsp desc : cbm_dyn->inp_td[sim_neurons[%d]->tail] :  %f\n",i,cbm_dyn->inp_td[sim_neurons[i]->tail]);
#endif
	
		sim_neurons[i]->tail = (sim_neurons[i]->tail + 1) % MAX_MEM;
		if (sim_neurons[i]->tail == sim_neurons[i]->head)
			sim_neurons[i]->head = (sim_neurons[i]->head + 1) % MAX_MEM;
	} /* fin du if */
	if (epsp)
	{
		/*
		* on met le resultat epsp dans la file d'attente
		* temporelle
		*/
		
		sim_neurons[i]->inp_t[sim_neurons[i]->tail] = simu->nb_it;
		
		/*
		* Afin de ne pas depasser les pot de reversion, on
		* doit verifier que ce que l'on met dans la file
		* d'attente n'es pas plus grand que la difference
		* entre le soma et le rev pot.  Ca pose pas mal de
		* problemes tordus de signes....
		*/
		
		
#if DEBUG
		if (epsp > delta_epsp)
			epsp = delta_epsp;
#endif
		
#if DEBUG_1
		printf("\t Somme EPSP for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			epsp);
#endif
#if  NBC_X_DEBUG 
		if(debug)
		{
			fprintf(fdbg,"\to Somme EPSP for neuron %lu:  %f\n",
				sim_neurons[i]->id,
				epsp);
		}  
#endif
	
		cbm_dyn->inp_max_pos[sim_neurons[i]->tail] = epsp;
		cbm_dyn->inp_ta[sim_neurons[i]->tail] = clu_cbm->tau_a_epsp;
		cbm_dyn->inp_td[sim_neurons[i]->tail] = clu_cbm->tau_d_epsp;
#if DEBUG_1
		printf("\tepsp asc : cbm_dyn->inp_ta[sim_neurons[%d]->tail] :  %f\n",i,cbm_dyn->inp_ta[sim_neurons[i]->tail]);
		printf("\tepsp desc : cbm_dyn->inp_td[sim_neurons[%d]->tail] :  %f\n",i,cbm_dyn->inp_td[sim_neurons[i]->tail]);
#endif
	
		sim_neurons[i]->tail = (sim_neurons[i]->tail + 1) % MAX_MEM;
		if (sim_neurons[i]->tail == sim_neurons[i]->head)
			sim_neurons[i]->head = (sim_neurons[i]->head + 1) % MAX_MEM;
	} /* fin du if */
	
	/*
	 * si NMDA il y a aussi du AMPA, donc il ne peut y avoir de
	 * nmda sans epsp
	 */

	net_input_pos =  suminputs_G(cbm_dyn->inp_ta,
				   cbm_dyn->inp_td,
				   cbm_dyn->inp_max_pos,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail);

#if DEBUG_1
	printf("net_input_pos  -> %f\n",net_input_pos);
#endif

	net_input_neg =  suminputs_G(cbm_dyn->inp_ta,
				   cbm_dyn->inp_td,
				   cbm_dyn->inp_max_neg,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail);

#if DEBUG_1
	printf("net_input_neg  -> %f\n",net_input_neg);
#endif

	coeff_NMDA = clu_cbm->g_NMDA
			/ (1. + clu_cbm->eta_NMDA
			* clu_cbm->magnesium
			* exp( - clu_cbm->gamma_NMDA
			* cbm_dyn->soma));

	net_input_nmda = sum_NMDA(coeff_NMDA, t_i,
				   clu_cbm->tau_a_NMDA,
				   clu_cbm->tau_d_NMDA,
				   cbm_dyn->inp_nmda,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail);

#if DEBUG_1
	printf("net_input_nmda  -> %f\n",net_input_nmda);
#endif

	/*
	 * Maintenant on peut calculer le soma
	 * Calcul du voltage intra somatique realise par update_neuron()
	 */

	/*--- Calcul du potentiel  de mb                         ------- */


	cbm_dyn->g_tot = G_K + G_Na + clu_cbm->g_fuite 
		+ net_input_pos - net_input_neg + net_input_nmda ;
#if DEBUG_1
	printf("g_tot  -> %f\n",cbm_dyn->g_tot);
#endif

	decay_mb_pot = exp (-(cbm_dyn->g_tot) * simu->time_step / clu_cbm->capacite);

	cbm_dyn->g_tot = cbm_dyn->g_tot * cbm_dyn->surface_normalisee;

#if DEBUG_1
	printf("g_tot  normalis -> %f\n",cbm_dyn->g_tot);
#endif

	delta_potentiel = 
			(
				(G_K * clu_cbm->E_K)
				+ (G_Na * clu_cbm->E_Na)
				+ (clu_cbm->g_fuite * clu_cbm->E_fuite)
				+ (net_input_pos * clu_cbm->rev_pot_pos)
				- (net_input_neg * clu_cbm->rev_pot_neg)
				+ (net_input_nmda * clu_cbm->rev_pot_NMDA)
			) 
			* cbm_dyn->surface_normalisee;
			    
	delta_potentiel /= cbm_dyn->g_tot;
	delta_potentiel *= (1. - decay_mb_pot);
	delta_potentiel +=  noise;
#if DEBUG
	printf("delta_potentiel  -> %f\ncbm_dyn->surface_normalisee %f\n,decay_mb_pot : %f",delta_potentiel,cbm_dyn->surface_normalisee,decay_mb_pot);
#endif
	
	cbm_dyn->soma = cbm_dyn->soma * decay_mb_pot + delta_potentiel;

#if DEBUG_1
	printf("\ncbm_dyn->soma   -> %f\n\n",cbm_dyn->soma);
#endif

	/*---  ca decharge ou pas?                              -------*/

	/*
	 * ca decharge ou pas? Critere de pente du signal.
	 */

/*	
	if (cbm_dyn->soma - soma_avant >= clu_cbm->pente_spike
	    && !sim_neurons[i]->spike_en_cours)
*/
	if (cbm_dyn->soma >= -10. && !sim_neurons[i]->spike_en_cours)
	{				/* on a un spike */
		sim_neurons[i]->new_state = SPIKE;

		sim_neurons[i]->decharge_lag = 0.;
	
		sim_neurons[i]->spike_en_cours = TRUE;
	
#if DEBUG_1
		printf("SPIKE\n");
#endif

		/*
		 * Mise a jour de la file d'attente et waited_spike:
		 * la date d'emission du premier spike a venir
		 * est en tete de file d'attente. Il a donc decharge
		 * depuis (maintenant  - date d'emission)
		 */
		 
		/* le spike est ajoute a la file de ceux qui sont deja sur l'axone */

		sim_neurons[i]->spike_time[sim_neurons[i]->queue] = simu->nb_it;
		sim_neurons[i]->queue = (sim_neurons[i]->queue + 1) % MAX_SPK;
		if (sim_neurons[i]->queue == sim_neurons[i]->tete)
			sim_neurons[i]->tete = (sim_neurons[i]->tete + 1) % MAX_SPK;

		if (sim_nucleus[n]->law != 0)
			(*sim_nucleus[n]->learning)(i,(sim_nucleus[n]->number - 1));
		
		/*
		* sauvegarde de la serie temporelle
		*    Save time of spike 
		*/
	 
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
						 strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"%d,%lu,%lu\n", n+1, sim_neurons[i]->id, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif
	} /* fin du if */
	else			/* pas de spike */
	{
		sim_neurons[i]->decharge_lag += 1.;
	} /* fin du else */
/*
	if (sim_neurons[i]->spike_en_cours && (cbm_dyn->soma - soma_avant <= 0))
*/	
	if (sim_neurons[i]->spike_en_cours && (cbm_dyn->soma <= -10))
	{
		sim_neurons[i]->spike_en_cours = FALSE;
	} /* fin du if */
} /* fin du update_G_res _no_tr */

