/*******************************************************************************
	Function for one CBM cluster iteration computation with transmitors
*******************************************************************************/
void update_G_res_use_ntr_simple(int pos)
{

	int	i, j, k, l, n, d_l, i_r, ntrm;
	
	int		flag_psp[NB_MAX_RECEPTORS];
	float	diff_volt[NB_MAX_RECEPTORS];

	float	
	    white_noise,
		t_i, 			/* Elapsed time */
		psp, epsp, ipsp, nmda,		
		net_input,		/* somme de ce qui arrive  (intermediaire) */
		net_input_nmda,		/* somme de ce qui arrive en NMDA (intermediaire) */
		coeff_NMDA,		/* intermediaire de calcul */
		noise = 0.,		/* background noise */	
		clu_noise = 0.,		/* cluster background noise */	
		nucl_noise = 0.,	/* nucleus background noise */
		sum_i_pos,		/* pour la somme des inputs */
		sum_i_neg,		/* pour la somme des inputs */

		G_Na,		/* conductance totale du courant sodique */
		I_Na,		/* Fast Sodium Current*/
		G_K,		/* conductance totale de la rectification retardee*/
		I_K,		/* courant K de la rectification retardee */
		G_Ca,		/* conductance totale du courant calcique */
		I_Ca,		/* Fast Calcium Current*/
		G_Cl,		/* conductance totale du courant Chlore */
		I_Cl,		/* courant Cl */
		G_A,		/* conductance totale de K pour le courant Ia */
		I_A,		/* courant Ia*/
		G_M,		/* conducatnce totale de K pour le courant Im */
		I_M,		/* courant Im non inactivable*/
		G_C,		/* courant k calcium dependant non inactivable */
		I_C,		/* courant K calcium et voltage dependant non inactivable*/
		G_H,		/* courant k rectification entrante Ih */
		I_H,		/* courant K calcium et voltage dependant non inactivable*/
		G_ahp,		/* conductance totale du K pour le courant Iahp */
		I_ahp,		/* courant K calcium dependant voltage independant*/
		G_Nap,		/* conductance totale du courant sodique persistant */
		I_Nap,		/* Persistant Sodium Current*/
		G_Cab,		/* conductance totale du courant Ca LVA */
		I_Cab,		/* Ca LVA Current*/
		G_Syn_pos,	/* conductance totale pour le courant synaptique */
		G_Syn_neg,	/* conductance totale pour le courant synaptique */
		I_Syn_pos,	/* courant synaptique */
		I_Syn_neg,	/* courant synaptique */
		G_NMDA,		/* conductance du NMDA */

		tau_m_Na,       /* cste de temps de l'activation des canaux Na  */
		m_1_Na,         /* facteur intervenant dans le calcul de m_Na*/
		alpha_m_Na,     /* facteur intervenant dans le calcul de m_Na et tau_m_Na */
		beta_m_Na,      /* facteur intervenant dans le calcul de m_Na et tau_m_Na */
		tau_h_Na,       /* cste de temps d'inactivation des canaux Na */
		h_1_Na,         /* facteur intervenant dans le calcul de h_na */
		alpha_h_Na,     /* facteur intervenant dans le calcul de h_Na et tau_h_Na */
		beta_h_Na,      /* facteur intervenant dans le calcul de m_Na et tau_h_Na */
		tau_m_Ca,       /* cste de temps d'activation des canaux Ca */
		m_1_Ca,         /* facteur intervenant dans le calcu de m_Ca */
		tau_m_K,        /* cste de temps de l'activation des canaux K */
		m_1_K,          /* facteur intervenant dans le calcul de m_K */
		alpha_m_K,      /* facteur intervenant dans le calcul de tau_m_K */
		beta_m_K,       /* facteur intervenant dans le calcul de tau_m_K */
		alpha_m1_K,     /* facteur intervenant dans le calcul de m1_k */
		beta_m1_K,      /* facteur intervenant dans le calcul de m1_k */
		tau_h_K,        /* cste de temps d'inactivation des canaux K */
		h_1_K,          /* facteur intervenant dans le calcul de h_K */

		soma_avant,	/* pour calculer la pente */
		input_sup,	/* voltage du a l'input supplementaire */
		delta_potentiel,/* cgt local de potentiel */
		decay,		/* variable intermediaire */
		decay_mb_pot,		/* variable intermediaire */
		curr_time;	/* variable intermediaire */

	char buffer[256];

	CBM *clu_cbm;
	CBM *clu_cnx;
	CBM_DYNAMIC *cbm_dyn;

#if DEBUG
	printf("time step: %f m (%d i)\nspike duration: %f m (%d i)\nsaved: %f m (%d i)\n",
	       simu->time_step,
	       nintf(1./simu->time_step),
	       simu->spike_width,
	       simu->nb_step_spk,
	       simu->nb_step_save_ms,
	       simu->nb_step_save_it);
#endif
	
	if((pos<0) || (pos>=simu->nb_neurons))
	{
		fprintf(stderr,"i is out of range in update_G_res_use_ntr!!!!\n");
		exit(-1);
	}
	i = pos;

	k = (sim_neurons[i]->its_cluster->number)-1;
	n = (sim_neurons[i]->its_nucleus->number)-1;
	clu_cbm = (CBM *)sim_neurons[i]->its_cluster->clu_data;
	cbm_dyn = (CBM_DYNAMIC *)sim_neurons[i]->neuron_data;

	t_i = (float)(simu->nb_it);




       	/* noise computation (methode de Andre Fabio Kohn (USP, San Paolo, Bresil) */
	/*
         *         filter_a = exp(-time_step / (resistance * capacite));
	 *	  filter_b = 1./capacite;
	 *	  center_noise = center_noise * filter_a + filter_b * white_noise; 
	 *	  noise = center_noise + backgrd_noise_m;
	 */

	if (clu_cbm->noise_active)
	{
		clu_noise = gasdev(&seed);
		clu_noise *= clu_cbm->backgrd_n_sd;
		if (clu_cbm->filter_noise)
		{
		  cbm_dyn->center_noise = cbm_dyn->center_noise * exp(-(simu->time_step) / (clu_cbm->res_capa)) 
		                          + clu_cbm->i_capacite * clu_noise; 
		  clu_noise =  cbm_dyn->center_noise + clu_cbm->backgrd_noise_m;		  
		}
		else
		  clu_noise += clu_cbm->backgrd_noise_m;		
	} 
#if KOHN_DEBUG
	printf("clu_cbm->noise_active: %d, clu_cbm->filter_noise: %d\n",
	       clu_cbm->noise_active, clu_cbm->filter_noise);
	printf("cbm_dyn->center_noise: %f\nclu_cbm->res_capa: %f\nclu_cbm->i_capacite: %f\nclu_noise:%f\n*****\n",
	       cbm_dyn->center_noise, clu_cbm->res_capa,
	       clu_cbm->i_capacite, clu_noise); 
#endif
	if(stim_noise_f[n] && sim_nucleus[n]->noise_active)
	{
		nucl_noise = gasdev(&seed);
		nucl_noise *= sim_nucleus[n]->backgrd_n_sd;
		nucl_noise += sim_nucleus[n]->backgrd_noise_m;		
	} 
		
		
	if (stim_modul_f[n])
	{
	  nucl_noise += sim_nucleus[n]->modul_value;
	}

	noise = clu_noise + nucl_noise;
	
#if NBC_X_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Noise for neuron %lu: total %f, nucl %f, clu %f\n",
			sim_neurons[i]->id,noise,nucl_noise,clu_noise);
	}
#endif

	/*
	 *	intra somatic potential
	 */
	 
	/*
	 *  Inputs from other neurons
	 */

  /***********************************************************/
  /*--- calcul de la conductance du courant synaptique   ----*/
  /***********************************************************/

/* 
	Si prise en compte des neurotransmetteurs 
		i est le neurone simul
		j est neurone qui le connecte
*/
/*
  I_Syn_pos = 0.;
*/  
	for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
	{
		clu_cbm->Receptors[i_r].G_syn = 0.;
		diff_volt[i_r] = clu_cbm->Receptors[i_r].g_Syn * (cbm_dyn->soma - clu_cbm->Receptors[i_r].rev_pot);
		flag_psp[i_r] = 0;
	}
  
/*
#if DEBUG_V8
  for (nb_r = 0; nb_r < nb_receptors; nb_r++)
  {
    printf("1 - Recp %d: %d (%d) (%s)\n", nb_r, Receptors[nb_r].transmetteur, Receptors[nb_r].num_transm, Receptors[nb_r].nom);
  }
#endif
*/
	for (j = 0; j < simu->nb_neurons; j++) 
	{
		clu_cnx = (CBM *)sim_neurons[j]->its_cluster->clu_data;
		ntrm = clu_cnx->neurotransmitor;
		for (l = 0 ; l < sim_neurons[j]->connected_neurons ; l++)
		{
			if((sim_neurons[j]->targets[l] == sim_neurons[i]->id) && (sim_neurons[j]->connections[l]))
			{
				if ((simu->nb_it - sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]]) == sim_neurons[j]->axone_step[l]) 
				{				
					for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
					{
						if (ntrm == NO_TRANSMISSION)
						{
							continue;
						}
						if (ntrm == clu_cbm->Receptors[i_r].num_transm)
						{
							clu_cbm->Receptors[i_r].G_syn += diff_volt[i_r] * sim_neurons[j]->connections[l];
							flag_psp[i_r] = 1;
						}
					}
					sim_neurons[j]->next_spike[l] = (sim_neurons[j]->next_spike[l] + 1) % MAX_SPK;
				}
			}
		}
	}
					

	
	/*
	 *	Is it time for input?
	 */

	epsp = ipsp = 0.;
	if (sim_nucleus[n]->input_type && stim_pulse_f[n] 
		&& (simu->nb_it >= sim_neurons[i]->t_next_i))
	{
		if (!sim_nucleus[n]->inhib)
		{
			epsp += (clu_cbm->g_Syn_pos  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input epsp contribution to neuron %lu:  %f( new epsp:%f)\n",
					sim_neurons[i]->id,
					clu_cbm->g_Syn_pos * sim_nucleus[n]->scale_f,epsp);
			}
#endif
		} 
		else
		{
			ipsp += -(clu_cbm->g_Syn_neg  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input ipsp contribution to neuron %lu:  -%f( new ipsp:%f)\n",
					sim_neurons[i]->id,
					clu_cbm->g_Syn_neg * sim_nucleus[n]->scale_f,ipsp);
			}
#endif
		}

		/*
		 * stimulation (last and next)
		 */
		
		sim_neurons[i]->t_last_i = sim_neurons[i]->t_next_i;
		sim_neurons[i]->t_next_i += GetInpInterval(i);
		
		/*
		 * Sauvegarde de la serie temporelle
		 *    Save time of input 
		 * (valeur negative = external input)
		 */
		
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f -%d\n",
			simu->time_ms, sim_nucleus[n]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f -%d\n",
			  simu->time_ms, sim_nucleus[n]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"A,%d,%lu\n", n+1, simu->nb_it - previous_event);
		previous_event = simu->nb_it;

#endif
	}

	/*
	 * si NMDA il y a aussi du AMPA, donc il ne peut y avoir de
	 * nmda sans epsp
	 */
	for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
	{
		if (flag_psp[i_r])
		{
			if (clu_cbm->Receptors[i_r].G_syn  > 0.)
			{
				/*
				 * on met le resultat epsp+ipsp dans la file d'attente
				 * temporelle (on utilise la file d'attente max_pos car elle existe, en fit ce sont les PSP dans ce cas)
				 */
				sim_neurons[i]->inp_t[sim_neurons[i]->tail] = simu->nb_it;

				cbm_dyn->inp_max_pos[sim_neurons[i]->tail] = clu_cbm->Receptors[i_r].G_syn;
				cbm_dyn->inp_nmda[sim_neurons[i]->tail] = nmda ;
				cbm_dyn->inp_ta[sim_neurons[i]->tail] = clu_cbm->Receptors[i_r].tau_a_psp;
				cbm_dyn->inp_td[sim_neurons[i]->tail] = clu_cbm->Receptors[i_r].tau_d_psp;
#if DEBUG
				printf("cbm_dyn[%d]->inp_max_pos= %f sim_neurons[%d]->inp_t= %d\n",
					   i+1,cbm_dyn->inp_max_pos[sim_neurons[i]->tail],
					   i+1,sim_neurons[i]->inp_t[sim_neurons[i]->tail]);
				printf("cbm_dyn[%d]->inp_nmda= %f sim_neurons[%d]->inp_t= %d\n",
					   i+1,cbm_dyn->inp_nmda[sim_neurons[i]->tail],
					   i+1,sim_neurons[i]->inp_t[sim_neurons[i]->tail]);
#endif
				sim_neurons[i]->tail = (sim_neurons[i]->tail + 1) % MAX_MEM;
				if (sim_neurons[i]->tail == sim_neurons[i]->head)
					sim_neurons[i]->head = (sim_neurons[i]->head + 1) % MAX_MEM;
			} /* fin du if */
		} /* fin du if */
	} /* fin du for */

	net_input =  suminputs(cbm_dyn->inp_ta,
				   cbm_dyn->inp_td,
				   cbm_dyn->inp_max_pos,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail);

				   
	coeff_NMDA = clu_cbm->g_NMDA
			/ (1. + clu_cbm->eta_NMDA
			* clu_cbm->magnesium
			* exp( - clu_cbm->gamma_NMDA
			* cbm_dyn->soma));

	net_input_nmda = sum_NMDA(coeff_NMDA, t_i,
				   clu_cbm->tau_a_NMDA,
				   clu_cbm->tau_d_NMDA,
				   cbm_dyn->inp_nmda,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail);
	/*
	 * Maintenant on peut calculer le soma
	 * Calcul du voltage intra somatique realise par update_neuron()
	 */

	soma_avant = cbm_dyn->soma;


#if DEBUG
	printf("\tentree dans update neurone %d\n",i);
#endif
/*************** TIRE DIRECTEMENT DE G_NEURON V7.7 ***************************/

  /******************************************************************/
  /*----  Calcul de la conductance au k pour le courant
          de rectification retardee                     ------------*/
  /******************************************************************/

      /****      variable d'activation des canaux K         ****/
      alpha_m_K =  exp(clu_cbm->A_K * (cbm_dyn->soma - clu_cbm->V_K));
      beta_m_K = exp(-clu_cbm->A_K * (cbm_dyn->soma - clu_cbm->V_K));
      tau_m_K = (1. / ( clu_cbm->lambda_Ik * (alpha_m_K + beta_m_K)));
      m_1_K = alpha_m_K / ( alpha_m_K + beta_m_K);
      decay = (exp (-simu->time_step / tau_m_K));
      cbm_dyn->m_K = cbm_dyn->m_K * decay +
	    m_1_K * (1. - decay);
      
      /****         Conductance et courant K                 ****/
      if (stim_drug_f[n] && sim_nucleus[n]->drogue[drug_K]) 
	G_K = 0.;
      else
	  G_K = clu_cbm->g_K * pow(cbm_dyn->m_K,clu_cbm->Pow_K);
#if DEBUG
      printf("K  -> m_K: %f A_K: %f V_K: %f I_K: %f\n",cbm_dyn->m_K,  clu_cbm->A_K, clu_cbm->V_K);
#endif

  
  /******************************************************************/
  /*---------     calcul de la conductance au Na     ---------------*/
  /******************************************************************/
  
	/*
	printf("HH\n");
	*/
	/****      variable d'activation des canaux Na          ****/
	m_1_Na = 1. /(1. + exp (-2.0 * clu_cbm->A_INa_A * (cbm_dyn->soma - clu_cbm->V_INa_A)));

	/****    variable d'inactivation des canaux Na         ****/
	h_1_Na = 1. /(1. + exp (-2.0 * clu_cbm->A_INa_B * (cbm_dyn->soma - clu_cbm->V_INa_B)));
	decay = exp ( -simu->time_step / clu_cbm->Tau_INa_B);
	cbm_dyn->h_Na = cbm_dyn->h_Na * decay +
	h_1_Na * (1. - decay);

	/****        Conductance et Courant Na                ****/ 
	if (stim_drug_f[n] && sim_nucleus[n]->drogue[drug_Na]) 
		G_Na = 0.;
	else
		G_Na = clu_cbm->g_Na * pow(m_1_Na,clu_cbm->Pow_INa_A) * pow(cbm_dyn->h_Na,clu_cbm->Pow_INa_B) ;


  /*---------------------------------------------------------------*/ 

  /**************************************************************/
  /*---  calcul de E_K car accumulation de K              ------*/
  /* d(Ko)/dt = IK_total/(Vperi*F) - (Ko - Krest)/(tauK-diff)   */
  /* from Methods in Neural modeling eds Koch and Segev pg. 115 */
  /* Krest = 2.5 mM, tauK-diff = 7msec                          */
  /* require radius of cell, in um                              */
  /* assumes extracellular space is 70nm radius                 */
  /**************************************************************/

  /*	radius = (diametre/2.);
	decay = exp(-simu->time_step/7.);
        K_ext = K_ext * decay +
	7. * (((I_K + I_A + I_M + I_C + I_ahp)
	* (3.11e-10*pow(radius,2.0))/ 
	(pow(radius+0.07,3.0)-pow(radius,3.0)))
	+ (K_rest/7.)) * (1. - decay);
	
        E_K = 25.* (float) log(K_ext/140.);


  
  
  */

  /*---------------------------------------------------------------*/ 

	/*--- Calcul du potentiel  de mb                         ------- */
/* ATTENTION  net_input est un voltage... */

/* dans nbc_x sans ntr
	cbm_dyn->g_tot = G_K + G_Na + G_A +  G_Ca + G_ahp + G_C + G_M + G_Nap + G_Cab + G_H
		+ clu_cbm->g_fuite
		+ net_input_pos + net_input_nmda ;
*/
/* Dans G_neuron :

  g_tot = G_K + G_Na + G_A +  G_Ca + G_ahp + G_C + G_M + G_Nap + G_Cab + G_H + g_fuite + g_fuite_Na + g_fuite_K
		  + G_NMDA + GL_Na + GL_K + GL_Cl;
  for (j = 0; j < nb_spikes; j++)
  {g
    	g_tot +=  tab_syn[j].G_syn;
  }

  if (g_tot == 0.) g_tot = 0.0000000001;

  decay_mb_pot = exp ((-(g_tot) * time_step / capacite));

  g_tot = g_tot * surface_normalisee;
  
  */

	cbm_dyn->g_tot = G_K + G_Na + clu_cbm->g_fuite;


	for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
	{
		if (flag_psp[i_r])
		{
		     cbm_dyn->g_tot +=  clu_cbm->Receptors[i_r].G_syn;
		}
	}
/*
	for (j = 0; j < nb_spikes; j++)
	{
	    	tab_syn[j].G_syn;
	}
*/
	if (cbm_dyn->g_tot == 0.)
		cbm_dyn->g_tot = 0.0000000001;


        decay_mb_pot = exp (-(cbm_dyn->g_tot) * simu->time_step / clu_cbm->capacite);

	cbm_dyn->g_tot = cbm_dyn->g_tot * cbm_dyn->surface_normalisee;

	delta_potentiel = 
			    (
				(G_K * clu_cbm->E_K)
				+ (G_Na  * clu_cbm->E_Na)
			    	+ (clu_cbm->g_fuite * clu_cbm->E_fuite)
			    	+ (net_input_nmda * clu_cbm->rev_pot_NMDA)
			    )
			    * cbm_dyn->surface_normalisee;
	for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
	{
		if (flag_psp[i_r])
		{
		  delta_potentiel += ((clu_cbm->Receptors[i_r].G_syn *  clu_cbm->Receptors[i_r].rev_pot)
			* cbm_dyn->surface_normalisee);
		}
	}
	delta_potentiel /= cbm_dyn->g_tot;
	delta_potentiel *= (1. - decay_mb_pot);

	delta_potentiel += (noise + epsp - ipsp);
	cbm_dyn->soma = cbm_dyn->soma * decay_mb_pot + delta_potentiel;
	cbm_dyn->courant_tot = (cbm_dyn->g_tot * cbm_dyn->soma);
	
/* Dans G_neuron :

     delta_potentiel = 
			  (G_K + G_ahp + G_C + G_A + G_M) * E_K
			  + (G_Na * E_Na) + (G_Ca * E_Ca)
			  + (G_Nap * E_Na) + (G_Cab * E_Ca)
			  + (G_H * E_K)
			  + (g_fuite * E_fuite)
			  + (g_fuite_Na * E_Na)
			  + (g_fuite_K * E_K)

			  + (GL_Na * E_Na)
			  + (GL_K * E_K)
			  + (GL_Cl * E_Cl)
//			
//			  + (G_Syn_pos * rev_pot_pos)
//
			  + (G_NMDA * rev_pot_NMDA)
			  + input_current;
  		for (j = 0; j < nb_spikes; j++)
  		{
				delta_potentiel += (tab_syn[j].G_syn * tab_syn[j].rev_pot);
  		}
		delta_potentiel /= g_tot;
	    delta_potentiel *= (1. - decay_mb_pot);
	    delta_potentiel += noise;
        le_potentiel = le_potentiel * decay_mb_pot + delta_potentiel;
        courant_tot = (g_tot * le_potentiel);

*/
	
	/*---  ca decharge ou pas?                              -------*/

	/*
	 * ca decharge ou pas? Critere de pente du signal.
	 */

	if (cbm_dyn->soma - soma_avant >= clu_cbm->pente_spike
	    && !sim_neurons[i]->spike_en_cours)
	{				/* on a un spike */
		sim_neurons[i]->new_state = SPIKE;

		sim_neurons[i]->decharge_lag = 0.;
	
		sim_neurons[i]->spike_en_cours = TRUE;

		/*
		 * Mise a jour de la file d'attente et waited_spike:
		 * la date d'emission du premier spike a venir
		 * est en tete de file d'attente. Il a donc decharge
		 * depuis (maintenant  - date d'emission)
		 */
		 
		/* le spike est ajoute a la file de ceux qui sont deja sur l'axone */

		sim_neurons[i]->spike_time[sim_neurons[i]->queue] = simu->nb_it;
		sim_neurons[i]->queue = (sim_neurons[i]->queue + 1) % MAX_SPK;
		if (sim_neurons[i]->queue == sim_neurons[i]->tete)
			sim_neurons[i]->tete = (sim_neurons[i]->tete + 1) % MAX_SPK;

		if (sim_nucleus[n]->law != 0)
			(*sim_nucleus[n]->learning)(i,(sim_nucleus[n]->number - 1));
		
		/*
		* sauvegarde de la serie temporelle
		*    Save time of spike 
		*/
	 
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
						 strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"%d,%lu,%lu\n", n+1, sim_neurons[i]->id, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif
	} /* fin du if */
	else			/* pas de spike */
	{
		sim_neurons[i]->decharge_lag += 1.;
	} /* fin du else */
	if (sim_neurons[i]->spike_en_cours && (cbm_dyn->soma - soma_avant <= 0))
	{
		sim_neurons[i]->spike_en_cours = FALSE;
	} /* fin du if */
} /* fin du update_G_res_use_ntr  */
