/*******************************************************************************
	Function for one LIM cluster iteration computation
*******************************************************************************/
void update_L_res(int i)
{

	int	j, k, l,n, d_l;

	float	
	        seuil_increase,
	        delta,
		epsp, ipsp,		
		somme_psp,		/* epsp et ipsp venant des
					 * autres neurones et leur somme
					 */
		net_input,		/* somme de ce qui arrive (intermediaire) */
		clu_noise = 0.;		/* cluster background noise */	

	float nucl_noise = 0.;
	float noise = 0.;
	char buffer[256];


	LIM *clu_lim;
	LIM_DYNAMIC *lim_dyn;
	
	if (sim_neurons[i]->spike)
	{
	        /*
		 * on est encore pendant le spike precedent, on ne fait rien
		 */

#if NBC_X_LIM_DEBUG
		if(debug)
		{
			fprintf(fdbg,"neuron %lu still in spike.\n",sim_neurons[i]->id);
		}
#endif
		sim_neurons[i]->spike--;
		return;
	} 
	
	n = (sim_neurons[i]->its_nucleus->number)-1;
	clu_lim = (LIM *)sim_neurons[i]->its_cluster->clu_data;
	lim_dyn = (LIM_DYNAMIC *)sim_neurons[i]->neuron_data;
	
       	/* noise computation (methode de Andre Fabio Kohn (USP, San Paolo, Bresil) */
	/*
         *         filter_a = exp(-time_step / (resistance * capacite));
	 *	  filter_b = 1./capacite;
	 *	  center_noise = center_noise * filter_a + filter_b * white_noise; 
	 *	  noise = center_noise + backgrd_noise_m;
	 */

	if (clu_lim->noise_active)
	{
		clu_noise = gasdev(&seed);
		clu_noise *= clu_lim->backgrd_n_sd;
		if (clu_lim->filter_noise)
		{
		  lim_dyn->center_noise = lim_dyn->center_noise * exp(-(simu->time_step) / (clu_lim->res_capa)) 
		                          + clu_lim->i_capacite * clu_noise; 
		  clu_noise =  lim_dyn->center_noise + clu_lim->backgrd_noise_m;		  
		}
		else
		  clu_noise += clu_lim->backgrd_noise_m;		
	} 
#if KOHN_DEBUG
	printf("clu_lim->noise_active: %d, clu_lim->filter_noise: %d\n",
	       clu_lim->noise_active, clu_lim->filter_noise);
	printf("lim_dyn->center_noise: %f\nclu_lim->res_capa: %f\nclu_lim->i_capacite: %f\nclu_noise:%f\n*****\n",
	       lim_dyn->center_noise, clu_lim->res_capa,
	       clu_lim->i_capacite, clu_noise); 
#endif
	if(stim_noise_f[n] && sim_nucleus[n]->noise_active)
	{
		nucl_noise = gasdev(&seed);
		nucl_noise *= sim_nucleus[n]->backgrd_n_sd;
		nucl_noise += sim_nucleus[n]->backgrd_noise_m;		
	} 
		
	nucl_noise += (sim_nucleus[n]->modul_value * simu->time_step);
	
	if (stim_modul_f[n] || stim_pulse_f[n])
	  nucl_noise += sim_nucleus[n]->modul_value;
	
	noise = clu_noise + nucl_noise;
	
	
#if NBC_X_LIM_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Noise for neuron %lu: total %f, nucl %f, clu %f\n",
			sim_neurons[i]->id,noise,nucl_noise,clu_noise);
	}
#endif

	/*
	 *	intra somatic potential
	 */

#if NBC_X_LIM_DEBUG
	printf("Intra somatic potential for neuron %lu (avant):  %f\n",
			sim_neurons[i]->id,lim_dyn->soma);
	printf("clu_lim->i_tau_mb_pot: %f; clu_lim->i_tau_i_mb_pot: %f\n",
	       clu_lim->i_tau_mb_pot, clu_lim->i_tau_i_mb_pot);
	if(debug)
	{
		fprintf(fdbg,"Intra somatic potential for neuron %lu:  %f\n",
			sim_neurons[i]->id,lim_dyn->soma);
	}
#endif
        lim_dyn->soma = lim_dyn->soma * clu_lim->i_tau_mb_pot + clu_lim->i_tau_i_mb_pot;

#if NBC_X_LIM_DEBUG
	printf("Intra somatic potential for neuron %lu (apres et avant bruit):  %f\n",
			sim_neurons[i]->id,lim_dyn->soma);
	if(debug)
	{
		fprintf(fdbg,"Intra somatic potential for neuron %lu:  %f\n",
			sim_neurons[i]->id,lim_dyn->soma);
	}
#endif

	/*
	 *  Inputs from other neurons
	 */
	epsp = ipsp = somme_psp = 0.;
		       
	for (j = 0; j < simu->nb_neurons; j++) 
	{
		for(l = 0 ; l < sim_neurons[j]->connected_neurons ; l++)
		{
			if((sim_neurons[j]->targets[l] == sim_neurons[i]->id) &&
			  (sim_neurons[j]->connections[l]))
			{
				if ((simu->nb_it - sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]]) ==
				  sim_neurons[j]->axone_step[l]) 
				{
					if (sim_neurons[j]->connections[l] > 0)
						epsp += clu_lim->epsp * sim_neurons[j]->connections[l];
					else
						ipsp += clu_lim->ipsp * sim_neurons[j]->connections[l];
					
					sim_neurons[j]->next_spike[l] = 
						(sim_neurons[j]->next_spike[l] + 1) % MAX_SPK;
#if NBC_X_LIM_DEBUG 
					if(debug &&(sim_neurons[j]->connections[l] > 0))
					{
						fprintf(fdbg,"(%f %f) Epsp contribution from neuron %lu to neuron %lu:  %f( new epsp:%f)\n",
							simu->nb_it, sim_neurons[j]->axone_step[l],
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_lim->epsp * sim_neurons[j]->connections[l],epsp);
					}
					if(debug &&(sim_neurons[j]->connections[l] < 0))
					{
						fprintf(fdbg,"Ipsp contribution from neuron %lu to neuron %lu:  %f( new ipsp:%f)\n",
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_lim->ipsp * sim_neurons[j]->connections[l],ipsp);
					}
#endif
				} 
					
			} 
		}
	} 
	
	/*
	 *	Is it time for input?
	 */
	if (sim_nucleus[n]->input_type && stim_pulse_f[n]
		&& (simu->nb_it >= sim_neurons[i]->t_next_i))
	{
		if (sim_nucleus[n]->inhib)
		{
			ipsp -= (clu_lim->ipsp  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_LIM_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input ipsp contribution to neuron %lu:  -%f( new ipsp:%f)\n",
					sim_neurons[i]->id,
					clu_lim->ipsp * sim_nucleus[n]->scale_f,ipsp);
			}
#endif
		} 
		else
		{
			epsp += (clu_lim->epsp  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_LIM_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input epsp contribution to neuron %lu:  %f( new epsp:%f)\n",
					sim_neurons[i]->id,
					clu_lim->epsp * sim_nucleus[n]->scale_f,epsp);
			}
#endif
		}

		/*
		 * stimulation (last and next)
		 */
		
		sim_neurons[i]->t_last_i = sim_neurons[i]->t_next_i;
		sim_neurons[i]->t_next_i += GetInpInterval(i);
		
		/*
		 * Sauvegarde de la serie temporelle
		 *    Save time of input 
		 * (valeur negative = external input)
		 */
#if BUFFERIZE_AND_FLUSH		
		sprintf(buffer,"%.3f -%d\n", simu->time_ms, sim_nucleus[n]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f -%d\n", simu->time_ms, sim_nucleus[n]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"A,%d,%lu\n", n+1, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif	
	}


       	somme_psp = epsp + ipsp;
	     
#if  NBC_X_LIM_DEBUG 
        printf("\to Somme PSP for neuron %lu:  %f\n",
	       sim_neurons[i]->id,
	       somme_psp);
        if(debug)
        {
		fprintf(fdbg,"\to Somme PSP for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			somme_psp);
	}  
#endif

	/*
	* Maintenant on peut calculer le soma
	* Calcul du voltage intra somatique
	* et tenir compte du shunt de la mb qui varie en f(t)
	*/


	lim_dyn->soma += (somme_psp + noise);

#if NBC_X_LIM_DEBUG
	printf("\to+o SOMA for neuron %lu:  %f\n",
		sim_neurons[i]->id,
		lim_dyn->soma);

	if(debug)
	{
		fprintf(fdbg,"\to SOMA for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			lim_dyn->soma);
	}  
#endif

	
	/*
	 * Calcul du seuil de decharge
	 */
	if (clu_lim->seuil_post_spike)
	{
		lim_dyn->threshold =   lim_dyn->delta_seuil 
					* exp((-sim_neurons[i]->decharge_lag) 
					      * clu_lim->i_tau_seuil) 
					+ clu_lim->seuil;
	}
#if NBC_X_LIM_DEBUG
	printf("Threshold for neuron %lu:  %f\n",
		sim_neurons[i]->id,
		lim_dyn->threshold);
	if(debug)
	{
		fprintf(fdbg,"Threshold for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			lim_dyn->threshold);
	} 
#endif


	/*
	 * ca decharge ou pas?
	 */
	 
	if (lim_dyn->soma >= lim_dyn->threshold)
	{				/* on a un spike */
		sim_neurons[i]->new_state = SPIKE;

		sim_neurons[i]->decharge_lag = 1.;
		sim_neurons[i]->spike = sim_neurons[i]->its_cluster->spike_duration;
		
		/*
		 * Mise a jour de la file d'attente et waited_spike:
		 * la date d'emission du premier spike a venir
		 * est en tete de file d'attente. Il a donc decharge
		 * depuis (maintenant  - date d'emission)
		 */
		/* le spike est ajoute a la file de ceux qui sont deja sur l'axone */

		sim_neurons[i]->spike_time[sim_neurons[i]->queue] = simu->nb_it;
		sim_neurons[i]->queue = (sim_neurons[i]->queue + 1) % MAX_SPK;
		if (sim_neurons[i]->queue == sim_neurons[i]->tete)
			sim_neurons[i]->tete = (sim_neurons[i]->tete + 1) % MAX_SPK;

		/*
		 * Mais elle tient quand meme compte de sa position
		 * moyenne actuelle qui depend en fait du rapport des
		 * ions de chaque cote de celle-ci
		 */

		lim_dyn->soma = lim_dyn->soma_post_spike = clu_lim->pot_post_spike;

		/* calcul de la valeur absolue du seuil apres ce spike (seuil_peak) */

		if (clu_lim->seuil_post_spike)
		{
			if (clu_lim->less_fatigue)
			{
			  delta = (lim_dyn->seuil_peak - lim_dyn->threshold);
			  seuil_increase = (clu_lim->seuil_post_spike - delta)* clu_lim->fatigue + delta;
			}
			else
			{
			  seuil_increase = clu_lim->seuil_post_spike * 
					 (clu_lim->fatigue * (lim_dyn->threshold - clu_lim->seuil) + 1.);
			}
			lim_dyn->seuil_peak = seuil_increase + lim_dyn->threshold;
			lim_dyn->delta_seuil = lim_dyn->seuil_peak - clu_lim->seuil;
		} /* fin du if */
		else
		{
			lim_dyn->seuil_peak = lim_dyn->threshold;
			lim_dyn->delta_seuil = 0.;
		}


		if (sim_nucleus[n]->law != 0)
			(*sim_nucleus[n]->learning)(i,(sim_nucleus[n]->number - 1));
			
		/*
		 * sauvegarde de la serie temporelle
		 *    Save time of spike 
		 */
#if BUFFERIZE_AND_FLUSH		 
		sprintf(buffer,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"%d,%lu,%lu\n", n+1, sim_neurons[i]->id, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif
	}
	else			
	{
		sim_neurons[i]->decharge_lag += 1.;		
	} 
}

