/*******************************************************************************
	Function for one PUM cluster iteration computation
*******************************************************************************/
void update_P_res(int i)
{

	int	j, k, l,n, d_l;

	float	
		K_epsp,			/* constantes pour le calcul de reversion */
		K_ipsp,
		K_noise,
		delta_ipsp,		
		delta_epsp,
		seuil_increase,
		delta,
		epsp, ipsp,		
		somme_psp,	/* epsp et ipsp venant des
					 * autres neurones et leur somme
					 */
		net_input,		/* somme de ce qui arrive (intermediaire) */
		clu_noise = 0.;		/* cluster background noise */	

	float nucl_noise = 0.;
	float noise = 0.;
	char buffer[256];


	PUM *clu_pum;
	PUM_DYNAMIC *pum_dyn;
	
	if (sim_neurons[i]->spike)
	{
	        /*
		 * on est encore pendant le spike precedent, on ne fait rien
		 */

#if NBC_X_DEBUG
		if(debug)
		{
			fprintf(fdbg,"neuron %lu still in spike.\n",sim_neurons[i]->id);
		}
#endif
		sim_neurons[i]->spike--;
		return;
	} 
	
	n = (sim_neurons[i]->its_nucleus->number)-1;
	clu_pum = (PUM *)sim_neurons[i]->its_cluster->clu_data;
	pum_dyn = (PUM_DYNAMIC *)sim_neurons[i]->neuron_data;
	
       	/* noise computation (methode de Andre Fabio Kohn (USP, San Paolo, Bresil) */
	/*
         *         filter_a = exp(-time_step / (resistance * capacite));
	 *	  filter_b = 1./capacite;
	 *	  center_noise = center_noise * filter_a + filter_b * white_noise; 
	 *	  noise = center_noise + backgrd_noise_m;
	 */

	if (clu_pum->noise_active)
	{
		clu_noise = gasdev(&seed);
		clu_noise *= clu_pum->backgrd_n_sd;
		if (clu_pum->filter_noise)
		{
		  pum_dyn->center_noise = pum_dyn->center_noise * exp(-(simu->time_step) / (clu_pum->res_capa)) 
		                          + clu_pum->i_capacite * clu_noise; 
		  clu_noise =  pum_dyn->center_noise + clu_pum->backgrd_noise_m;		  
		}
		else
		  clu_noise += clu_pum->backgrd_noise_m;		
	} 
#if KOHN_DEBUG
	printf("clu_pum->noise_active: %d, clu_pum->filter_noise: %d\n",
	       clu_pum->noise_active, clu_pum->filter_noise);
	printf("pum_dyn->center_noise: %f\nclu_pum->res_capa: %f\nclu_pum->i_capacite: %f\nclu_noise:%f\n*****\n",
	       pum_dyn->center_noise, clu_pum->res_capa,
	       clu_pum->i_capacite, clu_noise); 
#endif
	if(stim_noise_f[n] && sim_nucleus[n]->noise_active)
	{
		nucl_noise = gasdev(&seed);
		nucl_noise *= sim_nucleus[n]->backgrd_n_sd;
		nucl_noise += sim_nucleus[n]->backgrd_noise_m;		
	} 
		
	if (stim_modul_f[n] || stim_pulse_f[n])
	  nucl_noise += sim_nucleus[n]->modul_value;
	
	noise = clu_noise + nucl_noise;
	
#if NBC_X_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Noise for neuron %lu: total %f, nucl %f, clu %f\n",
			sim_neurons[i]->id,noise,nucl_noise,clu_noise);
	}
#endif

	/*
	 *	intra somatic potential
	 */

	pum_dyn->soma = (clu_pum->overshoot
		* exp((-sim_neurons[i]->decharge_lag) 
		* clu_pum->i_tau_mb_pot)) 
		+ clu_pum->rest_pot; 


#if NBC_X_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Intra somatic potential for neuron %lu:  %f\n",
			sim_neurons[i]->id,pum_dyn->soma);
	}
#endif


	/*
	 *  Inputs from other neurons
	 */
	epsp = ipsp = somme_psp = 0.;
	delta_epsp = clu_pum->rev_pot_pos - pum_dyn->soma;
	delta_ipsp = clu_pum->rev_pot_neg - pum_dyn->soma;
		       
	for (j = 0; j < simu->nb_neurons; j++) 
	{
		for(l = 0 ; l < sim_neurons[j]->connected_neurons ; l++)
		{
			if((sim_neurons[j]->targets[l] == sim_neurons[i]->id) &&
			  (sim_neurons[j]->connections[l]))
			{
				if ((simu->nb_it - sim_neurons[j]->spike_time[sim_neurons[j]->next_spike[l]]) ==
				  sim_neurons[j]->axone_step[l]) 
				{
					if (sim_neurons[j]->connections[l] > 0)
						epsp += clu_pum->epsp * sim_neurons[j]->connections[l];
					else
						ipsp += clu_pum->ipsp * sim_neurons[j]->connections[l];
					
					sim_neurons[j]->next_spike[l] = 
						(sim_neurons[j]->next_spike[l] + 1) % MAX_SPK;
#if NBC_X_DEBUG 
					if(debug &&(sim_neurons[j]->connections[l] > 0))
					{
						fprintf(fdbg,"(%f %f) Epsp contribution from neuron %lu to neuron %lu:  %f( new epsp:%f)\n",
							simu->nb_it, sim_neurons[j]->axone_step[l],
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_pum->epsp * sim_neurons[j]->connections[l],epsp);
					}
					if(debug &&(sim_neurons[j]->connections[l] < 0))
					{
						fprintf(fdbg,"Ipsp contribution from neuron %lu to neuron %lu:  %f( new ipsp:%f)\n",
							sim_neurons[j]->id,sim_neurons[i]->id,
							clu_pum->ipsp * sim_neurons[j]->connections[l],ipsp);
					}
#endif
				} 
					
			} 
		}
	} 
	
	
	/*
	 *	Is it time for input?
	 */
	if (sim_nucleus[n]->input_type && stim_pulse_f[n]
		&& (simu->nb_it >= sim_neurons[i]->t_next_i))
	{
		if (sim_nucleus[n]->inhib)
		{
			ipsp -= (clu_pum->ipsp  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input ipsp contribution to neuron %lu:  -%f( new ipsp:%f)\n",
					sim_neurons[i]->id,
					clu_pum->ipsp * sim_nucleus[n]->scale_f,ipsp);
			}
#endif
		} 
		else
		{
			epsp += (clu_pum->epsp  * 
				sim_nucleus[n]->scale_f);
#if NBC_X_DEBUG
			if(debug)
			{
				fprintf(fdbg,"Input epsp contribution to neuron %lu:  %f( new epsp:%f)\n",
					sim_neurons[i]->id,
					clu_pum->epsp * sim_nucleus[n]->scale_f,epsp);
			}
#endif
		}

		/*
		 * stimulation (last and next)
		 */
		
		sim_neurons[i]->t_last_i = sim_neurons[i]->t_next_i;
		sim_neurons[i]->t_next_i += GetInpInterval(i);
		
		/*
		 * Sauvegarde de la serie temporelle
		 *    Save time of input 
		 * (valeur negative = external input)
		 */
		
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f -%d\n", simu->time_ms, sim_nucleus[n]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f -%d\n", simu->time_ms, sim_nucleus[n]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"A,%d,%lu\n", n+1, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif
	}
	/* Implementation du Reversal Potential (V 4.4) */

	/*
	 *	Le prb des sommes de PSP est plus delicat que celui 
	 * d'un seul PSP. Nous avons estime qu'il faut d'abord faire 
	 * la somme de tout ce qui est + (epsp) et - (ipsp), 
	 * separement, puis de calculer son facteur de dependance par 
	 * rapport au rev pot.
	 * On estime que plus on se rapproche du reversal potential,
	 * moins l'effet d'un PSP est important donc, cela depend de
	 * (soma - rev_pot). Si on est au pot de mb, le facteur
	 * multiplicateur est 1, et si on est au rev pot il est 0.
	 * Donc, le PSP sera multiplie par un facteur qui devient
	 * lineairement de plus en plus petit au fur et a mesure
	 * que l'on se rapporche du Rev pot (V4.10):
	 *   K_rev_[pn] = 1 / (Pot_repos - reversal)
	 *   K_[ei]psp = (soma - reversal) * K_rev_[pn]
	 *   [ei]psp *= K_[ei]psp   
	 */
	/* On ne se lance dans les calculs que s'il y a lieu */

	if (epsp)
	{
		/* K_rev_p doit etre positif dans un cas naturel */
		
		K_epsp = delta_epsp * clu_pum->K_rev_p;
		epsp *= K_epsp;
		
#if NBC_X_DEBUG
		if(debug)
		{
			fprintf(fdbg,"Final epsp due to K_epsp for neuron %lu:  %f(K_epsp :%f)\n",
				sim_neurons[i]->id,
				epsp,K_epsp);
		}
#endif

	} 
	if (ipsp)
	{
		/* K_rev_n doit etre negatif dans un cas naturel */

		K_ipsp = delta_ipsp * clu_pum->K_rev_n;
		ipsp *= K_ipsp;
		
#if NBC_X_DEBUG
		if(debug)
		{
			fprintf(fdbg,"Final ipsp due to K_ipsp for neuron %lu:  %f(K_ipsp :%f)\n",
				sim_neurons[i]->id,
				ipsp,K_ipsp);
		}
#endif
	} 
	if (epsp || ipsp)
	{
		/*
		 * on met le resultat epsp+ipsp dans la file d'attente
		 * temporelle
		 */
		
		sim_neurons[i]->inp_t[sim_neurons[i]->tail] = simu->nb_it;
		
		/*
		 * Afin de ne pas depasser les pot de reversion, on
		 * doit verifier que ce que l'on met dans la file
		 * d'attente n'es pas plu grand que la difference
		 * entre le soma et le rev pot.  Ca pose pas mal de
		 * problemes tordus de signes....
		 */
		
		somme_psp = epsp + ipsp;
		if (somme_psp > delta_epsp)
			somme_psp = delta_epsp;
		if (somme_psp < delta_ipsp)
			somme_psp = delta_ipsp;
	     
#if  NBC_X_DEBUG 
	        printf("\to Somme PSP for neuron %lu:  %f\n",
	      		sim_neurons[i]->id,
	       		somme_psp);
		if(debug)
		{
			fprintf(fdbg,"\to Somme PSP for neuron %lu:  %f\n",
				sim_neurons[i]->id,
				somme_psp);
		}  
#endif

		pum_dyn->inp_max_pos[sim_neurons[i]->tail] = somme_psp;
		if(somme_psp > 0.0)
		{
			pum_dyn->inp_ta[sim_neurons[i]->tail] = clu_pum->tau_a_epsp;
			pum_dyn->inp_td[sim_neurons[i]->tail] = clu_pum->tau_d_epsp;
		}
		else
		{
			pum_dyn->inp_ta[sim_neurons[i]->tail] = clu_pum->tau_a_ipsp;
			pum_dyn->inp_td[sim_neurons[i]->tail] = clu_pum->tau_d_ipsp;
		}

		sim_neurons[i]->tail = (sim_neurons[i]->tail + 1) % MAX_MEM;
		if (sim_neurons[i]->tail == sim_neurons[i]->head)
			sim_neurons[i]->head = (sim_neurons[i]->head + 1) % MAX_MEM;
	} /* fin du if */
	/*
	 * Le probleme du bruit: le bruit est lui aussi multiplie par
	 * le facteur positif s'il est positif (il a la valeur d'un
	 * epsp) et par le facteur negatif sinon (si 0 il est
	 * considere comme positif)
	 */
	
	if (noise)
	{
		noise *= ((noise > 0.)? 
			delta_epsp * clu_pum->K_rev_p:
			delta_ipsp * clu_pum->K_rev_n);
		
#if NBC_X_DEBUG
		if(debug)
		{
			fprintf(fdbg,"Noise modification due to K_noise for neuron %lu:  %f\n",
				sim_neurons[i]->id,
				noise);
		}  
#endif
	} 

#if NBC_X_DEBUG
       	printf("\to Somme PSP for neuron %lu:  %f\n",
       		sim_neurons[i]->id,
       		somme_psp);
#endif
	net_input = suminputs(pum_dyn->inp_ta, pum_dyn->inp_td,
				   pum_dyn->inp_max_pos,
				   sim_neurons[i]->inp_t,
				   sim_neurons[i]->head,
				   sim_neurons[i]->tail) + noise;

#if NBC_X_DEBUG
	printf("\to NET INPUT for neuron %lu:  %f (noise: %f)\n",
		sim_neurons[i]->id,
		net_input, noise);

	if(debug)
	{
		fprintf(fdbg,"\to NET INPUT for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			net_input);
	}  
#endif
	/*
	* Maintenant on peut calculer le soma
	* Calcul du voltage intra somatique
	* et tenir compte du shunt de la mb qui varie en f(t)
	*/

	if (clu_pum->shunt_i_tau && sim_neurons[i]->decharge_lag)
	        pum_dyn->soma += (net_input *
				  (1. - exp(-(sim_neurons[i]->decharge_lag)/clu_pum->shunt_i_tau)));
	else
		pum_dyn->soma += net_input;

#if NBC_X_DEBUG
		printf("\to+o SOMA for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			pum_dyn->soma);

	if(debug)
	{
		fprintf(fdbg,"\to SOMA for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			pum_dyn->soma);
	}  
#endif

/*
*		 necessaire, a cause du bruit rajoute qui peut depasser
*/
	if (pum_dyn->soma > clu_pum->rev_pot_pos)
		pum_dyn->soma = clu_pum->rev_pot_pos; 
	if (pum_dyn->soma < clu_pum ->rev_pot_neg)
		pum_dyn->soma = clu_pum->rev_pot_neg;
	
	/*
	 * Calcul du seuil de decharge
	 */
	if (clu_pum->seuil_post_spike)
	{
		pum_dyn->threshold =   pum_dyn->delta_seuil 
					* exp((-sim_neurons[i]->decharge_lag) 
					      * clu_pum->i_tau_seuil) 
					+ clu_pum->seuil;
	}
#if NBC_X_DEBUG
	if(debug)
	{
		fprintf(fdbg,"Threshold for neuron %lu:  %f\n",
			sim_neurons[i]->id,
			pum_dyn->threshold);
	} 
#endif


	pum_dyn->soma -= clu_pum->rest_pot;
	pum_dyn->soma = clu_pum->v_plus * 
	                 (exp(clu_pum->A * pum_dyn->soma) - 1.) 
			 / (exp(clu_pum->A * pum_dyn->soma) - clu_pum->v_rap);
	pum_dyn->soma += clu_pum->rest_pot;

	/*
	 * ca decharge ou pas?
	 */
	 
	if (pum_dyn->soma >= pum_dyn->threshold)
	{				/* on a un spike */
		sim_neurons[i]->new_state = SPIKE;

		sim_neurons[i]->decharge_lag = 1.;
		sim_neurons[i]->spike = sim_neurons[i]->its_cluster->spike_duration;
		
		/*
		 * Mise a jour de la file d'attente et waited_spike:
		 * la date d'emission du premier spike a venir
		 * est en tete de file d'attente. Il a donc decharge
		 * depuis (maintenant  - date d'emission)
		 */
		/* le spike est ajoute a la file de ceux qui sont deja sur l'axone */

		sim_neurons[i]->spike_time[sim_neurons[i]->queue] = simu->nb_it;
		sim_neurons[i]->queue = (sim_neurons[i]->queue + 1) % MAX_SPK;
		if (sim_neurons[i]->queue == sim_neurons[i]->tete)
			sim_neurons[i]->tete = (sim_neurons[i]->tete + 1) % MAX_SPK;

		/*
		 * Mais elle tient quand meme compte de sa position
		 * moyenne actuelle qui depend en fait du rapport des
		 * ions de chaque cote de celle-ci
		 */

		pum_dyn->soma = pum_dyn->soma_post_spike = clu_pum->pot_post_spike;

		/* calcul de la valeur absolue du seuil apres ce spike (seuil_peak) */
/****/
		if (clu_pum->seuil_post_spike)
		{
			if (clu_pum->less_fatigue)
			{
			  delta = (pum_dyn->seuil_peak - pum_dyn->threshold);
			  seuil_increase = (clu_pum->seuil_post_spike - delta)* clu_pum->fatigue + delta;
			}
			else
			{
			  seuil_increase = clu_pum->seuil_post_spike * 
					 (clu_pum->fatigue * (pum_dyn->threshold - clu_pum->seuil) + 1.);
			}
			pum_dyn->seuil_peak = seuil_increase + pum_dyn->threshold;
			pum_dyn->delta_seuil = pum_dyn->seuil_peak - clu_pum->seuil;
		} /* fin du if */
		else
		{
			pum_dyn->seuil_peak = pum_dyn->threshold;
			pum_dyn->delta_seuil = 0.;
		}

		/* La file d'attente des epsp est videe puisque ca a decharge */

		sim_neurons[i]->tail = sim_neurons[i]->head = 0;

		if (sim_nucleus[n]->law != 0)
			(*sim_nucleus[n]->learning)(i,(sim_nucleus[n]->number - 1));
			
		/*
		 * sauvegarde de la serie temporelle
		 *    Save time of spike 
		 */
		 
#if BUFFERIZE_AND_FLUSH
		sprintf(buffer,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		tms_buffer = bufferize_and_flush(tms_buffer,buffer,&tms_buf_size,
			strlen(buffer),&tms_tmpfp);
		tms_size += strlen(buffer);
#else
		if (simu->output_tms_f)
		  fprintf(tms_fp,"%.3f %lu\n",simu->time_ms,sim_neurons[i]->id);
		if (simu->output_abe_f)
		  fprintf(abe_fp,"%d,%lu,%lu\n", n+1, sim_neurons[i]->id, simu->nb_it - previous_event);
		previous_event = simu->nb_it;
#endif	
	}
	else			
	{
		sim_neurons[i]->decharge_lag += 1.;		
	} 
}


