#define DEBUG1 0
#define DEBUG1 0
#define DEBUG_UPDT 0
/*
		UTIL_N_B_C.C regroupe les fonctions utiles diverses

		JFV		4 avril 90
*/
	
/*
*
*	NEURO_BIO_CLUSTERS.C implemente un reseau de clusters de neurones 
*	biologiaues dont on peut parametrer le nombre, les tailles, les 
*	connexions intra et interclusters.
*	Au sein d'un cluster, les neurones ne sont pas tous synchronises.
*
*	Donne une sortie utilisable par xgraph avec les outputs des
*	clusters et l'output global, et l'output de groupes de clusters.
*
*
*	ULTRIX 		4 avril 1990
*
*	Modifs:
*		Modifs en vol:		15 decembre 89	JFV
*		Arret prevus:		4 janvier 90	JFV
*		Sauvegarde de l'etat:	2 juillet 90	JFV
*               fonction calcul_intermediaire 4 aout 90	A.Rudnichi
*		V5.1	Mise a niveau	6 novembre 91	JFV
*		V7.0	Mise a niveau	18 fev 94	JFV
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "creation-simu.h"


/*
 * generateur de nb aleatoires uniforme (recipes in C)
 */

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;

	if (*idum <= 0 || !iy) {
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if (*idum < 0) *idum += IM;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0) *idum += IM;
	j=iy/NDIV;
	iy=iv[j];
	iv[j] = *idum;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX
/* (C) Copr. 1986-92 Numerical Recipes Software 6)A24. */

#ifndef M_E
#   define M_E 2.7182818284590452354
#endif

/*
 * generateur de nb aleatoires a distribution gaussienne  (recipes in C)
 */

#include <math.h>

float gasdev(long *idum)
{
	float ran1(long *idum);
	static int iset=0;
	static float gset;
	float fac,rsq,v1,v2;

	if  (iset == 0) {
		do {
			v1=2.0*ran1(idum)-1.0;
			v2=2.0*ran1(idum)-1.0;
			rsq=v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	} else {
		iset=0;
		return gset;
	}
}
/* (C) Copr. 1986-92 Numerical Recipes Software 6)A24. */

void 
nrerror(char *error_text)
{

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}
/*
 *	Fonctions de distributions d'input
 */

#ifndef RAND_MAX
#	define RAND_MAX 32767.0
#endif

/* The exponential generator, as a macro */
#define EXPONENTIAL(beta) (-(beta) * log(ran1(&seed)))

/*          This function generates exponentially distributed
 * random numbers from Turbo C's uniform random number generator.
 * The inverse transform method is used, where, if U is uniformly
 * distributed from 0 to 1, X = -beta * ln(U) is exponentially
 * distributed.
 */

float 
exponential(float beta)
{
	return(-beta * log(ran1(&seed)));
}

/*               Function for generating gamma
 *               distributed random numbers.
 *
 *     This function generates gamma distributed random numbers
 * with parameters alpha (shape parameter) and beta (scale parameter)
 * (mean is alpha * beta, variance is alpha * beta**2).  Note that,
 * if X = gamma(alpha,1) then X' = gamma(alpha, beta) = beta * X.  Also,
 * gamma(1, 1) = exponential(1).
 *
 *     The algorithm is as follows:
 *
 * For 0 < alpha < 1:
 * 1. Compute b = (e + alpha)/e beforehand.
 * 2. Generate a uniformly distributed random number, U1, and let
 *    P = bU1.  If P > 1, go to step 4.
 * 3. Let Y = P ** (1/alpha).  Generate another uniformly distributed
 *    random number, U2.  If U2 <= exp(-Y), set X = Y and return beta * X.
 *    Otherwise, go back to step 2.
 * 4. Let Y = -log((b - P)/alpha).  Generate another uniformly distributed
 *    random number, U2.  If U2 <= Y ** (alpha - 1), set X = Y and
 *    return beta * X.  Otherwise, go back to step 2.
 *
 * For alpha = 1:
 * 1. Generate X = expontial(1), and return beta * X.
 *
 * For alpha > 1:
 * 1. Compute a = 1/sqrt(2*alpha - 1), b = alpha - log(4),
 *    q = alpha + 1/alpha, theta = 4.5, d = 1 + log(theta).
 * 2. Generate two uniformly distributed random numbers, U1 and U2.
 * 3. Let V = a*log(U1/(1 - U1)), Y = alpha*exp(V), Z = U1**2 * U2, and
 *    W = b + q * V - Y.
 * 4. If W + d - theta * Z >= 0, set X = Y and return beta * X.
 * 5. If W >= log(Z), set X = Y and return beta * X.  Otherwise, go to
 *    step 2.
 *
 *     Because of the complexity of the algorithm, two routines are
 * visible to the user.  initgamma() specifies the parameters of the
 * desired gamma distribution.  It computes any constants needed later, and
 * determines which algorithm to use.  gamma() does the actual generation,
 * based on the parameters to initgamma.
 *
 */


/* Static precomputed constants */
static float alpha, beta, a, b, q, theta, d;

/* The actual gamma function */
static float (*ProperGamma)();

static float 
NullFunc(void)
{
	return(0.0);
}

static float 
LessThanOne(void)
{
	float U1, U2;   /* Two uniformly distributed random numbers */
	float P, Y;     /* Used in the computation                  */

	while (1)
	{
		U1 = ran1(&seed);           /* Step 2 */
		P = b * U1;

		if (P <= 1)
		{                    /* Step 3 */
		       Y = pow(P, (1.0/alpha));
		       U2 = ran1(&seed);
		       if (U2 <= exp(-Y)) return(Y);
		}
		else
		{                         /* Step 4 */
		       Y = -log((b - P) / alpha);
		       U2 = ran1(&seed);
		       if (U2 <= pow(Y, (alpha - 1.0))) return(Y);
		}
	} /* endwhile */
}


static float 
One(void)
{
	return(exponential(1.0));
}

static float 
GreaterThanOne(void)
{
	float	U1, U2,
		V, Y, z, w;
	
	while (1)
	{
		U1 = 0.0;
		while(!U1 || (U1 == 1.0))
		 	U1 = ran1(&seed);                     /* Step 2 */
		U2 = ran1(&seed);

		V = a * log(U1 / (1.0 - U1));              /* Step 3 */
		Y = alpha * exp(V);
		z = U1 * U1 * U2;
		w = b + q * V - Y;

		if ((w + d - theta * z) >= 0.0) return(Y); /* Step 4 */
		if (w >= log(z)) return(Y);                /* Step 5 */
	} /* endwhile */
}

int 
initgamma(float alph, float bet)
{
	if (alph <= 0.0)
	{
		ProperGamma = NullFunc;
		return(-1);
	}
	if (alph < 1.0)
	{
		ProperGamma = LessThanOne;
		b = (M_E + alph)/M_E;
		alpha = alph;
		beta = bet;
	}
	if (alph == 1.0)
	{
		ProperGamma = One;
		alpha = alph;
		beta = bet;
	}
	if (alph > 1.0)
	{
		ProperGamma = GreaterThanOne;
		a = 1.0 / sqrt(2.0 * alph - 1.0);
		b = alph - log(4.0);
		q = alph + 1.0/alph;
		theta = 4.5;
		d = 1.0 + log(theta);
		alpha = alph;
		beta = bet;
	}
	return(0);
}

double 
f_gamma(void)
{
	return(beta * (*ProperGamma)());
}

/*            Function for generating normally
 *            distributed random numbers.
 *
 *     This function generates normally distributed random numbers
 * with mean of mu and standard deviation of sigma, using the
 * polar method of Marsaglia and Bray, "A Convenient Method for
 * Generating Normal Variables", _SIAM Rev._, 6: 260-264 (1964).
 *
 *     The algorithm is as follows:
 *
 * 1. Generate two uniformly distributed numbers, U1 and U2.
 *    Let Vi = 2*Ui-1 for i=1,2, and let W = V1*V1 + V2*V2.
 * 2. If W > 1, go back to step 1.  Otherwise, let
 *    Y = sqrt(-2*ln(W)/W), X1 = V1*Y, and X2 = V2*Y.  Then
 *    X1 and X2 are normally distributed with mean of 0 and
 *    variance of 1.
 * 3. Random numbers with mean of mu and standard deviation of sigma
 *    are calculated by: X_prime = mu + sigma * X.
 *
 * Note that numbers are generated in pairs.  On odd-numbered calls
 * To this function, pairs are calculated.  On even-numbered calls,
 * the second value is returned.
 *
 *     The user assumes responsibility for initializing the random
 * number generator.
 *
 */

float 
N(float mu, float sigma)
{

	static int odd = TRUE;     /* Says whether we should calculate this time */
	static float X2;          /* Second value from last odd call.           */
	float 	U1, U2,             /* Uniformly distributed.                     */
	        V1, V2, w, Y,       /* Work variables (see above).                */
	        X1; 
	                   /* First value computed (returned immediately)*/
	
	
/* Check to see if we need to compute anything, complement indicator. */
	if (odd = !odd)
		return(mu + sigma * X2);

/* Do the computation step 1 (until W <= 1) */
	do
	{
		U1 = ran1(&seed); ;  /* Generate U(0,1) */
		U2 = ran1(&seed);
		V1 = 2 * U1 - 1;
		V2 = 2 * U2 - 1;
		w = V1 * V1 + V2 * V2;
	} while (w > 1);

/* Do the computation step 2 */
	Y = sqrt(-2 * log(w) / w);
	X1 = V1 * Y;
	X2 = V2 * Y;

/* Return X1 this time, X2 next time */
	return(mu + sigma * X1);
}
/*
 * Arrondi les flottants au lieu de les tronquer
 */
int 
arrondi(float f)
{
	int	inter;

	inter = f;
	if ((f - inter) > .5)
	inter = inter + 1;
	return(inter);
} /* fin du float */


/*
 * MODULATION permet de calculer la valeur modulee a ajouter au bruit de fond
 */
#undef M_PI
#undef M_DEUX_PI
#define M_PI	3.14159265358979323846
#define M_DEUX_PI 6.28318530717958647692

float	
modulation(NUCLEUS *noyau)
{
	float	amp,
		place,
	        period,
	        phase,
		demi_period,
	        pulse_duration,
	        inter_pulse,
		f_nb_it;
	unsigned long pulse_up;
	unsigned long pulse_down;

	if (!stim_modul_f[(noyau->number)-1])
	  return(0.);

	period = noyau->modul_period * simu->time_step;
	inter_pulse = (unsigned long) period;
	phase = noyau->modul_phase * simu->time_step;
	pulse_duration =  (unsigned long) phase;
	amp = noyau->modul_amp;

	f_nb_it = (float)simu->nb_it;

	switch(noyau->modul_type)
	{
		case 's':
			return(sin((M_DEUX_PI * (f_nb_it/period) + phase)) * amp);
		case 't':
			demi_period = period / 2.;
			place = (f_nb_it < phase)?
					period + f_nb_it - phase:
					f_nb_it - phase -
					(float)((int)((f_nb_it - phase) / period)) * period;
			return((place < demi_period)? (amp / demi_period) * place: ((-amp / demi_period) * place) + (2. * amp) );
		case 'r':
			place = (f_nb_it < phase)?
					period + f_nb_it - phase:
					f_nb_it - phase -
					(float)((int)((f_nb_it - phase) / period)) * period;
			return((place < period/2.)? amp: 0. );
		case 'p':
		        if (noyau->pulse_up)
			{
			   noyau->pulse_up --;
			   if (noyau->pulse_up)
			   {
#if DEBUG
			     printf("up: %d\n",noyau->pulse_up);
#endif
			     return(amp);
			   }
			   else
			   {
			     if (noyau->pulses_restants)
			     {
			       noyau->pulse_down = pulse_duration;
			       noyau->pulses_restants--;
			     }
			     else
			     {
			       noyau->pulse_down = inter_pulse;
			       noyau->pulses_restants = noyau->nb_pulses;
			     }
#if DEBUG
			     printf("down: %d\n",noyau->pulse_down);
#endif
			     return(0.);
			   }
			}
			else
			{
			   noyau->pulse_down --;
			   if (noyau->pulse_down)
			   {
#if DEBUG
			     printf("down: %d\n",noyau->pulse_down);
#endif
			     return(0.);
			   }
			   else
			   {
			     if (noyau->pulses_restants)
			     {
			       noyau->pulse_up = pulse_duration;
			     }
			     else
			     {
			       noyau->pulse_down = inter_pulse;
			       noyau->pulses_restants = noyau->nb_pulses;
			     }
#if DEBUG
			     printf("up: %d\n",noyau->pulse_up);
#endif
			     return(amp);
			   }
			}
	}
	return(0.);
} 

void	
init_poisson(void)
{
	time_t	seed;		/*--- random nb seed		---*/
				/*----------------------------------------------*/
				/* P(R <= r) = r (r -> [0;1])			*/
				/* X = -(1/lambda) * log(1 - R)			*/
				/* P(X <= x) = 1 - exp(-lambda * x) = F(x)	*/
				/* f(x) = dF(x)/dx = lambda * exp(-lambda * x)	*/
				/*----------------------------------------------*/
	time(&seed);
	srand(seed);
}

float	
f_poisson(float lambda_poisson)
{
	
	float	r;		/*--- inter-event interval	---*/

	r = 0.;
	while ((r == 0.) || (r == 1.))
		r = (float) rand() / (float) RAND_MAX;
	return( (-lambda_poisson) * log(1. - r));
}
