#ifndef _CREATING_ANATOMY_FILE_
#define _CREATING_ANATOMY_FILE_

#define _DEBUG_ 0



/*-------------------------------------------------------------------------*
 * The aim of this file is to create an anatomy file for NBC Network Editor*
 * version2 from a network structure                                       *                        *
 *-------------------------------------------------------------------------*/


/* general remarks about saving an whole network:
			the lnk, anat and cnx files can be created in  any order */


 
/* The semantic of the .anat file is quite simple:
			<short>	number of nucleus in network
			<short> number of neurons in nucleus 1
			<short> number of neurons in nucleus 2
			...
			<short> number of neurons in cluster "n"
		then for each neuron:	
			<NEURON_ANATOMY> -> float x 3: H , L , AP 	(12 bytes) */
 
 
 
/* standard include files */
#include <Xm/Xm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif

/* prototypes of functions implemented in this file */
int create_anat_file();

/*-------------------------------------------------------------------------*
	C R E A T E _ A N A T _ F I L E
 *-------------------------------------------------------------------------*/
int create_anat_file(Network *network)
{
	int i,j,k,p,counter;
	short total_nucleus;
	short *total_neurons;
	Nucleus_widget **all_nucleus;
	FILE *fp;
	char buffer[255];
	char tmp[256];
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array, *cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	

	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad Network ! can't create anat file !\n");
#endif
		return -1;
	}	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nCurrent Network hasn't any name ! can't create anat file !\n");
#endif
		return -1;
	}
	
	/* get the anat file name */

	strcpy(buffer,project_directory);
	strcat(buffer,"/");
	strcat(buffer,network->name);
	strcat(buffer,".anat");

	/* test the network structure */	

	if(!network->nb_nucleus || !network->nb_neurons)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad nucleus or neuron number: can't create anat file...\n");
#endif
		return -1;
	} 	 

	nucleus_array = network->nucleus_array;
	

	if(!nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus array empty: can't create anat file...\n");
#endif
		return -1;
	}

	/* now let's count the total number of clusters in the network */

	total_nucleus = network->nb_nucleus;
	
	/* get the total neuron per nucleus... reference all nucleus in local array... */
	
	total_neurons = (short *)calloc(1,total_nucleus*sizeof(short));	
	all_nucleus = network->nucleus_array;
	counter = 0;

	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		total_neurons[counter++] = nucleus_tmp->nb_neurons;
	}

	/* ok , it's time to open the anat file...*/

	if(!(fp=fopen(buffer,"wb")))
	{
#if _DEBUG_
		fprintf(stderr,"\nUnable to open %s in write mode !\nCan't create anat file...\n",
			buffer);
#endif
		
		sprintf(tmp,"Sorry, can't write %s.\nCheck authorizations or disk quotas...",
			buffer);
		display_error(widget_array[ID_MainW_network],tmp,
			"Write Error !");
		free(total_neurons);
		fclose(fp);
		return -1;
	}


	/*--- Now let's really do it: writing the output anat file... ---*/

	fwrite(&total_nucleus,sizeof(short),1,fp);
	fwrite(total_neurons,total_nucleus*sizeof(short),1,fp);
		
	for(i=0 ; i<total_nucleus ; i++)
	{
		nucleus_tmp = all_nucleus[i];
		for(k=0 ; k<nucleus_tmp->nb_clusters ; k++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[k];
			neuron_array = cluster_tmp->neuron_array;

			for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
			{
				neuron_tmp = neuron_array[j];
				fwrite(&neuron_tmp->neuron_id,sizeof(short),1,fp);
				fwrite(&cluster_tmp->cluster_id,sizeof(short),1,fp);
				fwrite(&neuron_tmp->anatomy,sizeof(NEURON_ANATOMY),1,fp);
				
			}
		}
	}


	/* well...it seems to be ok...*/

	free(total_neurons);
	fclose(fp);

#if _DEBUG_
	printf("\nOk: %s has been created.\n",buffer);
#endif
	return 0;


}	/* end of create_anat_file */

#endif
