#ifndef _CREATING_CLUSTER_LIST_FILE_
#define _CREATING_CLUSTER_LIST_FILE_

#define _DEBUG_ 0

/*****************************************************************************

	Object: creates from a network structure a cluster list file 
		 for NBC Network Editor
				
******************************************************************************/
/* usual includes */
#include <Xm/Xm.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif

/* prototypes of functions implemented in this file */
int create_clu_file();

/*----------------------------------------------------------------------------
 *	C R E A T E_ C L U _ F I L E
 *--------------------------------------------------------------------------*/

int	create_clu_file (Network *network)
{

	int i,j,k,counter;
	short total_clusters;
	short *total_neurons;
	CLUSTER_WIDGET **all_clusters;
	FILE *fp;
	char buffer[255];
	char tmp[256];
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array, *cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	int nb_lines;
	typedef struct {
		char *file;
		short *refs;
		short nb_refs;
		}	CLU;
	CLU *clu_lines;
	CLU *clu_tmp;
	
	

	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad Network ! can't create clu file !\n");
#endif
		return -1;
	}	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nCurrent Network hasn't any name ! can't create cnx file !\n");
#endif
		return -1;
	}
	
	

	/* test the network structure */	

	if(!network->nb_nucleus || !network->nb_neurons)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad nucleus or neuron number: can't create clu file...\n");
#endif
		return -1;
	} 	 

	nucleus_array = network->nucleus_array;
	

	if(!nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus array empty: can't create cnx file...\n");
#endif
		return -1;
	}

	/* now let's count the total number of clusters in the network */

	total_clusters = 0;
	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"Unexpected empty nucleus no%d: can't create cnx file...\n",
				i);
#endif
			return -1;
		}
		total_clusters += nucleus_tmp->nb_clusters;			
	}

	/* get the total neuron per cluster... reference all clusters in local array... */
	
	total_neurons = (short *)calloc(1,total_clusters*sizeof(short));	
	all_clusters = (CLUSTER_WIDGET **)calloc(1,total_clusters*sizeof(CLUSTER_WIDGET *));
	counter = 0;

	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		cluster_array = nucleus_tmp->cluster_array;

		for(j=0 ; j<nucleus_tmp->nb_clusters ; j++)
		{
			cluster_tmp = all_clusters[counter] = cluster_array[j];
			if(!cluster_tmp)
			{
#if _DEBUG_
				fprintf(stderr,
				"Unexpected empty cluster no%d, nucleus no%d: can't create clu file...\n",
				j,i);
#endif
				free(total_neurons);
				free(all_clusters);
				return -1;
			}
			total_neurons[counter++] = cluster_tmp->nb_neurons;
		}	
	}


	nb_lines = total_clusters;
	clu_tmp = (CLU *)calloc(1,total_clusters*sizeof(CLU));
	for(i=0 ; i<total_clusters ;i++)
	{
		/* fill the clu lines */
		clu_tmp[i].file = (char *)calloc(1,(strlen(all_clusters[i]->neuron_file)+1)*
			sizeof(char));
		strcpy(clu_tmp[i].file,all_clusters[i]->neuron_file);
		clu_tmp[i].nb_refs = 1;
		clu_tmp[i].refs = (short *)calloc(1,clu_tmp[i].nb_refs*sizeof(short));
		clu_tmp[i].refs[clu_tmp[i].nb_refs - 1] = all_clusters[i]->cluster_id;
	
	}
	
	for(i=0 ; i<total_clusters ; i++)
	{
		if(!clu_tmp[i].nb_refs)
				continue;
		strcpy(buffer,clu_tmp[i].file);
		counter = 0;
		for(j=0 ; j<total_clusters ; j++)
		{
			if(!clu_tmp[j].nb_refs)
				continue;
			if(!strcmp(buffer,clu_tmp[j].file))
			{
				counter++;
				if(counter > 1)
				{
					clu_tmp[i].nb_refs = counter;
					clu_tmp[i].refs = (short *)realloc(clu_tmp[i].refs,
						clu_tmp[i].nb_refs*sizeof(short));
					clu_tmp[i].refs[clu_tmp[i].nb_refs - 1] = 
						clu_tmp[j].refs[0];
					free(clu_tmp[j].file);
					free(clu_tmp[j].refs);
					clu_tmp[j].nb_refs = 0;
					nb_lines--;
				}
			}
		}	
	}
	
	clu_lines = (CLU *)calloc(1,nb_lines*sizeof(CLU));
	j = 0;
	for(i=0 ; i<total_clusters ; i++)
	{
		if(!clu_tmp[i].nb_refs)
				continue;
				
		memcpy(&clu_lines[j],&clu_tmp[i],sizeof(CLU));
		j++;
	}
	
	/* get the clu file name */

	strcpy(buffer,project_directory);
	strcat(buffer,"/");
	strcat(buffer,network->name);
	strcat(buffer,".clu");
	
	if(!(fp=fopen(buffer,"w")))
	{
#if _DEBUG_
		fprintf(stderr,"\nUnable to open %s in write mode !\nCan't create clu file...\n",
			buffer);
#endif
		sprintf(tmp,"Sorry, can't write %s.\nCheck authorizations or disk quotas...",
			buffer);
		display_error(widget_array[ID_MainW_network],tmp,
			"Write Error !");
		free(total_neurons);
		free(all_clusters);
		fclose(fp);
		return -1;
	}
	
	for(i=0 ; i<nb_lines ; i++)
	{
		fprintf(fp,"%s :",clu_lines[i].file);
		for(j=0 ; j<clu_lines[i].nb_refs ; j++)
		{
			fprintf(fp," %d",clu_lines[i].refs[j]);
		}
		fprintf(fp,"\n");
	}
	
	free(total_neurons);
	free(all_clusters);
	fclose(fp);
	
	free(clu_tmp);
	for(i=0 ; i<nb_lines ; i++)
	{
		free(clu_lines[i].file);
		free(clu_lines[i].refs);
		clu_lines[j].nb_refs = 0;
	}
	free(clu_lines);
	
	return 0;	

}

#endif
