#ifndef _CREATING_LEN_FILE_
#define _CREATING_LEN_FILE_

#define _DEBUG_ 0

/*-------------------------------------------------------------------------*
 *	The aim of this file is to create a len file format NBC Network Editor *	
 *	version 2 , from the specified ver2 network structure                  *
 *-------------------------------------------------------------------------*/
 
 
/* general remarks about saving an whole network:
			the lnk, anat, wgt and len files can be created in  any order */
  
 
/*------------------------------------------------------------------------*
  		The semantic of the len ver2 file is as it follows:
  															
				-it's made of decimal short integers
				-first token is the number of nucleus in the network
				-then, for each nucleus,follows its number of neurons
				-at this point starts the connections area:
					for each nucleus,
						for each neuron, are sequentially specified
							- neuron id
							- cluster id 
							- number of neurons connected to the specified
								neuron (in fact, the number of connections)
							- for each target neuron is specified:
								-the target id
								-the axone length in ms
								
*-------------------------------------------------------------------------*/
 
/* standard include files */
#include <Xm/Xm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif


/* prototypes of functions implemented in this file */
int create_len_file();


/*-------------------------------------------------------------------------*
	C R E A T E _ L E N _ F I L E
 *-------------------------------------------------------------------------*/
int create_len_file(Network *network)
{
	int i,j,k,p,counter;
	short total_nucleus;
	short *total_neurons;
	Nucleus_widget **all_nucleus;
	FILE *fp;
	char buffer[255];
	char tmp[256];
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array, *cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	

	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad Network ! can't create len file !\n");
#endif
		return -1;
	}	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nCurrent Network hasn't any name ! can't create len file !\n");
#endif
		return -1;
	}
	
	/* get the len file name */

	strcpy(buffer,project_directory);
	strcat(buffer,"/");
	strcat(buffer,network->name);
	strcat(buffer,".len");

	/* test the network structure */	

	if(!network->nb_nucleus || !network->nb_neurons)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad nucleus or neuron number: can't create len file...\n");
#endif
		return -1;
	} 	 

	nucleus_array = network->nucleus_array;
	

	if(!nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus array empty: can't create len file...\n");
#endif
		return -1;
	}

	/* now let's count the total number of clusters in the network */

	total_nucleus = network->nb_nucleus;
	
	/* get the total neuron per nucleus... reference all nucleus in local array... */
	
	total_neurons = (short *)calloc(1,total_nucleus*sizeof(short));	
	all_nucleus = network->nucleus_array;
	counter = 0;

	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		total_neurons[counter++] = nucleus_tmp->nb_neurons;
	}

	/* ok , it's time to open the len file...*/

	if(!(fp=fopen(buffer,"wb")))
	{
#if _DEBUG_
		fprintf(stderr,"\nUnable to open %s in write mode !\nCan't create len file...\n",
			buffer);
#endif
		
		sprintf(tmp,"Sorry, can't write %s.\nCheck authorizations or disk quotas...",
			buffer);
		display_error(widget_array[ID_MainW_network],tmp,
			"Write Error !");
		free(total_neurons);
		fclose(fp);
		return -1;
	}


	/*--- Now let's really do it: writing the output len file... ---*/

	fwrite(&total_nucleus,sizeof(short),1,fp);
	fwrite(total_neurons,total_nucleus*sizeof(short),1,fp);
		
	for(i=0 ; i<total_nucleus ; i++)
	{
		nucleus_tmp = all_nucleus[i];
		for(k=0 ; k<nucleus_tmp->nb_clusters ; k++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[k];
			neuron_array = cluster_tmp->neuron_array;

			for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
			{
				neuron_tmp = neuron_array[j];
				fwrite(&neuron_tmp->neuron_id,sizeof(short),1,fp);
				fwrite(&cluster_tmp->cluster_id,sizeof(short),1,fp);
				fwrite(&neuron_tmp->connected_neurons,sizeof(short),1,fp);
				
				for(p=0 ; p<neuron_tmp->connected_neurons ; p++)
				{
					fwrite(&neuron_tmp->connections[p].target_id,
						sizeof(short),1,fp);
					fwrite(&neuron_tmp->connections[p].axone_length,
						sizeof(float),1,fp);
				}
				
			}
		}
	}


	/* well...it seems to be ok...*/

	free(total_neurons);
	fclose(fp);

#if _DEBUG_
	printf("\nOk: %s has been created.\n",buffer);
#endif
	return 0;

} /* end of create_len_file */ 	


#endif
