#ifndef _CREATING_LINK_CONFIGURATION_FILE_VER2
#define _CREATING_LINK_CONFIGURATION_FILE_VER2

#ifndef _DEBUG_
#define _DEBUG_ 0
#endif

/*****************************************************************************

	Object: creates from a network structure a link configution file format 2.0
		 for NBC Network Editor
				
******************************************************************************/

/* general remarks about saving an whole network:
			the lnk, anat and cnx files can be created in  any order */




/* usual includes */
#include <Xm/Xm.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif

/* prototypes of functions implemented in this file */
int create_link_file();
void append_nucleus_in_lnk();
void append_link_in_lnk();



/*----------------------------------------------------------------------------
 *	C R E A T E_ L N K_ F I L E
 *--------------------------------------------------------------------------*/

int	create_lnk_file (Network *network)
{
	int	i, j;
	FILE *file_p;
	char lnk_name[255];
	char buffer[256];
	Nucleus_widget *nucleus_tmp;
	LINK_PARAMETERS *link_tmp;
	
	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nThe network structure is empty...\n");
#endif	
		return -1;
	}
	
	/* recovering lnk file name from the network structure */
	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nNo lnk file name was present in the structure...\n");
#endif
		return -1;
	}
	else
	{
		
		strcpy(lnk_name,network->name);
		strcat(lnk_name,".ntw");	
	}
	
	if(file_exists(lnk_name,project_directory))
	{
		sprintf(buffer,"Do you really want to update old file\n%s/%s ?",
			project_directory,lnk_name);
		if(display_question(widget_array[ID_MainW_network],
			buffer,"Confirmation needed...")!= YES_ANSWER)
			return -1;
		
	}
	strcpy(lnk_name,project_directory);
	strcat(lnk_name,"/");
	strcat(lnk_name,network->name);
	strcat(lnk_name,".ntw");
	
	/* opening lnk file */
	
	file_p = fopen (lnk_name, "w");
	
	if(!file_p)
	{
		sprintf(buffer,"Sorry, can't write %s.\nCheck authorizations or disk quotas...",
			lnk_name);
		display_error(widget_array[ID_MainW_network],buffer,
			"Write Error !");
		return -1;
	}

	fprintf (file_p, "TypeOfFile: LinkConfigurationFileVer2\n");
	fprintf (file_p, "Creator: FullEditor\n");

	/* lnk file header */

	fprintf(file_p, "NumberOfNucleus: %d\n", network->nb_nucleus);
	
	fprintf(file_p, "NumberOfInterNucleusLink: %d\n", network->nb_links);
	
	fprintf(file_p, "TotalNeuronsInNetwork: %d\n", network->nb_neurons);
	

	for(i=0;i<network->nb_nucleus;i++)
	{
		nucleus_tmp = (Nucleus_widget *)network->nucleus_array[i];
		if(nucleus_tmp)
			append_nucleus_in_lnk(file_p,nucleus_tmp);	
	}
	
	for(i=0;i<network->nb_links;i++)
	{
		link_tmp = (LINK_PARAMETERS *)network->link_array[i];
		if(link_tmp)
			append_link_in_lnk(file_p,link_tmp,network->nucleus_array,NULL);
	}

	fclose(file_p);
	
	/* Ok, seems to be 'well done' */
	
	return 0;
	
}	/* fin de create_lnk_file */ 
	
/**************************************************************************
 *	APPEND NUCLEUS IN LNK
 ***************************************************************************/
void append_nucleus_in_lnk(FILE *fp,Nucleus_widget *nucleus)
{
	long i,j;
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	LINK_PARAMETERS **link_array;
	LINK_PARAMETERS *link_tmp;
	
	
	if (!nucleus)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus empty...\n");
#endif	
		return;
	}
	
	cluster_array = (CLUSTER_WIDGET **)nucleus->cluster_array;
	link_array = (LINK_PARAMETERS **)nucleus->link_array;
	
	if(!cluster_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus empty...\n");
#endif	
		return; 
	}
	fprintf (fp, "BeginNucleus: *\n");
	
	fprintf (fp, "NucleusName: %s\n", nucleus->nucleus_name);
	
	fprintf (fp, "NucleusId: %d\n", nucleus->nucleus_id);
	
	fprintf (fp, "NumberOfClusters: %d\n", nucleus->nb_clusters);
	
	fprintf (fp, "NumberOfInterClusterLink: %d\n", nucleus->nb_links);
	
	fprintf (fp, "TotalNeuronsInNucleus: %d\n", nucleus->nb_neurons);
		
	
	fprintf (fp, "NucleusPositionH: %f\n", nucleus->H);
	fprintf (fp, "NucleusDeltaH: %f\n", nucleus->delta_H);
	fprintf (fp, "NucleusPositionL: %f\n", nucleus->L);
	fprintf (fp, "NucleusDeltaL: %f\n", nucleus->delta_L);
	fprintf (fp, "NucleusPositionAP: %f\n", nucleus->AP);
	fprintf (fp, "NucleusDeltaAP: %f\n", nucleus->delta_AP);
			
	for (i = 0; i < nucleus->nb_clusters; i++)
	{
		if(cluster_array[i])
		{
			fprintf (fp, "BeginCluster: *\n");
			fprintf (fp, "ClusterName: %s\n", cluster_array[i]->ch_name);
			fprintf (fp, "ClusterId: %d\n", cluster_array[i]->cluster_id);
			fprintf (fp, "PositionH: %f\n", cluster_array[i]->H);
			fprintf (fp, "DeltaH: %f\n", cluster_array[i]->delta_H);
			fprintf (fp, "PositionL: %f\n", cluster_array[i]->L);
			fprintf (fp, "DeltaL: %f\n", cluster_array[i]->delta_L);
			fprintf (fp, "PositionAP: %f\n", cluster_array[i]->AP);
			fprintf (fp, "DeltaAP: %f\n", cluster_array[i]->delta_AP);
			fprintf (fp, "NbNeurons: %d\n", cluster_array[i]->nb_neurons);		
			fprintf (fp, "NeuronFile: %s\n", cluster_array[i]->neuron_file);

			fprintf (fp, "EndCluster: *\n");
		}
	}

	for (i = 0; i < nucleus->nb_links ; i++)
	{
		link_tmp = (LINK_PARAMETERS *)link_array[i];
		if(link_tmp)
			append_link_in_lnk(fp,link_tmp,NULL,cluster_array);	
	}

	fprintf (fp, "EndNucleus: *\n");
	
}	/* fin de append_nucleus_in_lnk */

/**************************************************************************
 *	APPEND LINK IN LNK
 ***************************************************************************/

void append_link_in_lnk(FILE *fp,LINK_PARAMETERS *link,
		Nucleus_widget *nucleus[],CLUSTER_WIDGET *cluster[])
{

	CLUSTER_WIDGET *cluster_tmp;
	Nucleus_widget *nucleus_tmp;

	if(!nucleus && !cluster)
	{
#if _DEBUG_
		fprintf(stderr,"\nLink can't be dumped: nucleus and cluster arrays empty...\n");
#endif	
		return;
	}
	else if (cluster)
	  {
		  fprintf (fp, "BeginLink: *\n");
		  
		  fprintf (fp, "Cluster1: %d\n",
			 link->source_id );
		  
		  fprintf (fp, "Cluster2: %d\n",
			  link->target_id);
		  fprintf (fp, "AxoneMean: %f\n",
			  link->axone_mean);
		  fprintf (fp, "AxoneSd: %f\n",
			  link->axone_sd);
		  fprintf (fp, "BornMin: %d\n",
			  link->min_knobs);
		  fprintf (fp, "BornMax: %d\n",
			  link->max_knobs);
		  fprintf (fp, "Type: %d\n",
			  link->type);
		  fprintf (fp, "Nmda: %d\n",
			  link->nmda);  
		  fprintf (fp, "EndLink: *\n");
	  }
	else if(nucleus)
	{
		  fprintf (fp, "BeginLinkInterNucleus: *\n");
		 
		  fprintf (fp, "Nucleus1: %d\n",
			  link->source_id);
		   
		  fprintf (fp, "Nucleus2: %d\n",
			  link->target_id);
		  fprintf (fp, "INAxoneMean: %f\n",
			  link->axone_mean);
		  fprintf (fp, "INAxoneSd: %f\n",
			  link->axone_sd);
		  fprintf (fp, "INBornMin: %d\n",
			  link->min_knobs);
		  fprintf (fp, "INBornMax: %d\n",
			  link->max_knobs);
		  fprintf (fp, "INType: %d\n",
			  link->type);
		  fprintf (fp, "INNmda: %d\n",
			  link->nmda);
		  fprintf (fp, "EndLinkInterCluster: *\n");
	  }
	  
}	/* fin de append_link_in_lnk */


#endif

