#define _MEMSPY_ 0 

/*----------------------------------------------------------------------------
 *	M E S S A G E _ B O X E S . C
 *--------------------------------------------------------------------------*/

/*
*
*	Author: Patrick GLANDAZ
*		Laboratoire de Biomathematiques
*		URBB B3E INSERM U263
*		Faculte de Medecine Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert, July 1992 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is not, in any way, usable for commercial purpose.
*
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <Xm/XmP.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/BulletinB.h>
#include <Xm/FileSB.h>

#include "ne_vms_or_unix.h"
#include "messbox.h"
#include "ne_strdup.h"

#if _MEMSPY_
#include "/u/sa2/boussard/MEMORY/memspy.h"
#endif

#define	   WAITING_ANSWER_ACTION	1
#define	   ANSWER_RECEIVED_ACTION	2
#define	   NO_WAITING_ACTION		3

static	char	prompt_value[64];
static	char	fileselect_value[256];
static	int	action_mb;
static	int	answer_mb;
static	char	dir_tab[NB_DIR_TAB][256];
static	int	first_fileselect = TRUE;

static	Widget	message_dialog = (Widget)NULL;
static	Widget	prompt_dialog = (Widget)NULL;
static	Widget	error_dialog = (Widget)NULL;
static	Widget	question_dialog = (Widget)NULL;
static	Widget	fileselect_dialog = (Widget)NULL;

extern Colormap the_colormap;
extern Display * the_display;

extern XtAppContext app_context;

void	mb_ok_proc ();
void	mb_cancel_proc ();

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ M E S S A G E
 *--------------------------------------------------------------------------*/

int	display_message (caller, msg,title)
Widget	caller;
char	*msg;
char *title;
{
	XmString	xm_string;
	Display		*disp = XtDisplay (caller);

/*--- Message Dialog -------------------------------------------------------*/

	message_dialog = XmCreateMessageDialog (caller,
	    "message_dialog", NULL, 0);
	XtVaSetValues (message_dialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL);
	XtUnmanageChild (
	    XmMessageBoxGetChild (message_dialog, XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild (
	    XmMessageBoxGetChild (message_dialog, XmDIALOG_HELP_BUTTON));
	XtAddCallback (message_dialog, XmNokCallback, mb_ok_proc,
	(XtPointer)(unsigned long) k_MessageD_mb);

/*--- Remplissage de la fenetre --------------------------------------------*/

	xm_string = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);

	XtVaSetValues(XtParent(message_dialog),XmNtitle,title,NULL);
		
	XtVaSetValues (message_dialog,
	    XmNmessageString, xm_string,
	    XmNx, 150,
	    XmNy, 200,
	    NULL);

	XtManageChild (message_dialog);
	XSetWindowColormap(the_display, XtWindow(XtParent(message_dialog)),
						the_colormap);
	XmStringFree (xm_string);

/*--- Boucle d'evenements --------------------------------------------------*/

	action_mb = WAITING_ANSWER_ACTION;
	
	while (action_mb != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent(app_context, XtIMAll);
		XSync (disp, False);
	}
	
	XtUnmanageChild (message_dialog);
	XSync (disp, False);
	XmUpdateDisplay (message_dialog);

	action_mb = NO_WAITING_ACTION;
	XtDestroyWidget (message_dialog);
	return	answer_mb;
}

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ Q U E S T I O N
 *--------------------------------------------------------------------------*/

int	display_question (caller, msg,title)
Widget	caller;
char	*msg;
char *title;
{
	XmString	xm_string, yes, no;
	Display		*disp = XtDisplay (caller);

/*--- Question Dialog ------------------------------------------------------*/

	yes = XmStringCreateSimple ("Yes");
	no = XmStringCreateSimple ("No");

	question_dialog = XmCreateQuestionDialog (caller,
	    "question_dialog", NULL, 0);
	XtVaSetValues (question_dialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    XmNokLabelString, yes,
	    XmNcancelLabelString, no,
	    NULL);
	XtUnmanageChild (
	    XmMessageBoxGetChild (question_dialog, XmDIALOG_HELP_BUTTON));
	XtAddCallback (question_dialog, XmNokCallback, mb_ok_proc,
	    (XtPointer)(unsigned long)k_QuestionD_mb);
	XtAddCallback (question_dialog, XmNcancelCallback,
	    mb_cancel_proc, (XtPointer)(unsigned long)k_QuestionD_mb);

	XmStringFree (yes);
	XmStringFree (no);

/*--- Remplissage de la fenetre --------------------------------------------*/

	xm_string = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);

	if (XtIsManaged (question_dialog))
	{
		XtUnmanageChild (question_dialog);
	}

	XtVaSetValues (question_dialog,
	    XmNmessageString, xm_string,
	    XmNx, 200,
	    XmNy, 250,
	    NULL);
	XmStringFree (xm_string);

	XtVaSetValues(XtParent(question_dialog),XmNtitle,title,NULL);
	
	XtManageChild (question_dialog);
	XSetWindowColormap(the_display, XtWindow(XtParent(question_dialog)),
						the_colormap);

/*--- Boucle d'evenements --------------------------------------------------*/

	action_mb = WAITING_ANSWER_ACTION;
	while (action_mb != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent(app_context, XtIMAll);
		XSync (disp, False);	
	}
	
	XtUnmanageChild (question_dialog);
	XSync (disp, False);
	XmUpdateDisplay (question_dialog);	

/*--- Sortie ---------------------------------------------------------------*/

	action_mb = NO_WAITING_ACTION;
	XtDestroyWidget (question_dialog);

	if (answer_mb == OK_ANSWER)
	{
		return YES_ANSWER;
	}
	else if (answer_mb == CANCEL_ANSWER)
	{
		return NO_ANSWER;
	}
}

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ E R R O R
 *--------------------------------------------------------------------------*/

int	display_error (caller, msg,title)
Widget	caller;
char	*msg;
char	*title;
{
	XmString	xm_string;
	Display		*disp = XtDisplay (caller);

/*--- Error Dialog ---------------------------------------------------------*/

	error_dialog = XmCreateErrorDialog (caller,
	    "error_dialog", NULL, 0);
	XtVaSetValues (error_dialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL);
	XtUnmanageChild (
	    XmMessageBoxGetChild (error_dialog, XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild (
	    XmMessageBoxGetChild (error_dialog, XmDIALOG_HELP_BUTTON));
	XtAddCallback (error_dialog, XmNokCallback, mb_ok_proc, 
	(XtPointer)(unsigned long)k_ErrorD_mb);

/*--- Remplissage de la fenetre --------------------------------------------*/

	xm_string = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);

	XtVaSetValues (error_dialog,
	    XmNmessageString, xm_string,
	    XmNx, 250,
	    XmNy, 200,
	    NULL);
	    
        XtVaSetValues(XtParent(error_dialog),XmNtitle,title,NULL);
	
	XtManageChild (error_dialog);
	XSetWindowColormap(the_display, XtWindow(XtParent(error_dialog)),
						the_colormap);
	XmStringFree (xm_string);

/*--- Boucle d'evenements --------------------------------------------------*/

	action_mb = WAITING_ANSWER_ACTION;
	
	while (action_mb != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent(app_context, XtIMAll);
		XSync (disp, False);
	}
	
	XtUnmanageChild (error_dialog);
	XSync (disp, False);
	XmUpdateDisplay (error_dialog);

	action_mb = NO_WAITING_ACTION;
	XtDestroyWidget (error_dialog);
	return	answer_mb;
}

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ P R O M P T
 *--------------------------------------------------------------------------*/

char	*display_prompt (caller, msg, defaut)
Widget	caller;
char	*msg, *defaut;
{
	XmString	xm_string;
	char		vide[2];
	Display		*disp = XtDisplay (caller);

/*--- Prompt Dialog --------------------------------------------------------*/

	prompt_dialog = XmCreatePromptDialog (caller,
	    "Help on context", NULL, 0);
	XtVaSetValues (prompt_dialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL);
	XtUnmanageChild (
	    XmSelectionBoxGetChild (prompt_dialog, XmDIALOG_HELP_BUTTON));
	XtAddCallback (prompt_dialog, XmNokCallback,
	    mb_ok_proc,(XtPointer)(unsigned long) k_PromptD_mb);
	XtAddCallback (prompt_dialog, XmNcancelCallback,
	    mb_cancel_proc, (XtPointer)(unsigned long)k_PromptD_mb);

/*--- Remplissage de la fenetre --------------------------------------------*/

	sprintf	(vide, "");

	xm_string = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);
	XtVaSetValues (prompt_dialog,
	    XmNselectionLabelString, xm_string,
	    XmNx, 200,
	    XmNy, 150,
	    NULL);

	XmStringFree (xm_string);
	if (!defaut)
	{
		defaut = vide;
	}
	xm_string = XmStringCreateLtoR (defaut, XmSTRING_DEFAULT_CHARSET);
	XtVaSetValues (prompt_dialog,
	    XmNtextString, xm_string,
	    NULL);


	XtManageChild (prompt_dialog);
	XSetWindowColormap(the_display, XtWindow(XtParent(prompt_dialog)),
						the_colormap);
	XmStringFree (xm_string);

/*--- Boucle d'evenements --------------------------------------------------*/

	action_mb = WAITING_ANSWER_ACTION;
	
	while (action_mb != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent(app_context, XtIMAll);
		XSync (disp, False);
	}
	
	XtUnmanageChild (prompt_dialog);
	XSync (disp, False);
	XmUpdateDisplay (prompt_dialog);

/*--- Sortie ---------------------------------------------------------------*/

	action_mb = NO_WAITING_ACTION;
	XtDestroyWidget (prompt_dialog);
	
	if (answer_mb == CANCEL_ANSWER)
	{
		return (char *) NULL;
	}
	else if (answer_mb == OK_ANSWER)
	{
		return prompt_value;
	}	
}

/*----------------------------------------------------------------------------
 *	D I S P L A Y _ F I L E S E L E C T
 *--------------------------------------------------------------------------*/

char	*display_fileselect (caller, numero, dirmask, msg, directory)
Widget	caller;
int	numero;
char	*dirmask, *msg, *directory;
{
	XmString	xm_string1, xm_string2, xm_string3, xm_string4;
	char		*slash_position;
	Display		*disp = XtDisplay (caller);
	static char	dir[255];
	char *pt;
	Arg		args[15];
	int		i, n = 0;
	
/*--- Fileselect Dialog ----------------------------------------------------*/

	if (first_fileselect)
	{
		for (i = 0; i < NB_DIR_TAB; i++)
		{
#ifdef	UNIX
			strcpy(dir_tab[i], "./");
#elif	VMS
			strcpy(dir_tab[i], "[]");
#endif
		}
		first_fileselect = FALSE;
	}

/*--- Remplissage de la fenetre --------------------------------------------*/

	xm_string1 = XmStringCreateSimple ("File Selection");
	xm_string2 = XmStringCreateSimple (dirmask);
	xm_string3 = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);

	if (numero == 0)
	{
		xm_string4 = XmStringCreateSimple (directory);
	}
	else
	{
		xm_string4 = XmStringCreateSimple (dir_tab[numero]);
	}

	XtSetArg (args[n], XmNdialogTitle, xm_string1); n++;
	XtSetArg (args[n], XmNdirMask, xm_string2); n++;
	XtSetArg (args[n], XmNselectionLabelString, xm_string3); n++;
	XtSetArg (args[n], XmNdirectory, xm_string4); n++;
	XtSetArg (args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
	XtSetArg (args[n], XmNdefaultPosition, False); n++;
	XtSetArg (args[n], XmNautoUnmanage, False); n++;
	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
	XtSetArg (args[n], XmNwidth, 400); n++;
	XtSetArg (args[n], XmNheight, 500); n++;
	XtSetArg (args[n], XmNx, 200); n++;
	XtSetArg (args[n], XmNy, 120); n++;

	fileselect_dialog = XmCreateFileSelectionDialog (caller,
	    "fileselect_dialog", args, n);
	    
	XtUnmanageChild (XmSelectionBoxGetChild (
	    fileselect_dialog, XmDIALOG_HELP_BUTTON));
	XtAddCallback (fileselect_dialog, XmNokCallback,
	    mb_ok_proc, (XtPointer)(unsigned long)k_FileSelectD_mb);
	XtAddCallback (fileselect_dialog, XmNcancelCallback,
	    mb_cancel_proc, (XtPointer)(unsigned long)k_FileSelectD_mb);

	
	XtVaSetValues(XtParent(fileselect_dialog),XmNtitle,"file selection",NULL);

	XtManageChild (fileselect_dialog);
	XSetWindowColormap(the_display, XtWindow(XtParent(fileselect_dialog)),
						the_colormap);
	
	XmStringFree (xm_string1);	
	XmStringFree (xm_string2);	
	XmStringFree (xm_string3);	
	XmStringFree (xm_string4);	

/*--- Boucle d'evenements --------------------------------------------------*/

	action_mb = WAITING_ANSWER_ACTION;
	
	while (action_mb != ANSWER_RECEIVED_ACTION)
	{
		XtAppProcessEvent(app_context, XtIMAll);
		XSync (disp, False);
	}

	XtUnmanageChild (fileselect_dialog);
	XSync (disp, False);
	XmUpdateDisplay (fileselect_dialog);

/*--- Sortie ---------------------------------------------------------------*/

	action_mb = NO_WAITING_ACTION;
	XtDestroyWidget (fileselect_dialog);

	if (answer_mb == CANCEL_ANSWER)
	{
		return (char *) NULL;
	}
	else if (answer_mb == OK_ANSWER)
	{
		if (!strlen (fileselect_value))
		{
			return (char *)NULL;
		}
		if (numero != 0)
		{
			strcpy(dir,pt = str_dup (fileselect_value));
			free(pt);
			strcpy(dir_tab[numero], dir);
#ifdef	UNIX
			slash_position = strrchr (dir_tab[numero], '/');
#elif	VMS
			slash_position = strrchr (dir_tab[numero], ']');
#endif
			*(++slash_position) = '\0';
			
		}
		return fileselect_value;
	}	
}

/*----------------------------------------------------------------------------
 *	M B _ O K _ P R O C
 *--------------------------------------------------------------------------*/

void	mb_ok_proc (w, tag, reason)
Widget	w;
int	tag;
XmAnyCallbackStruct *reason;
{
	char	*value;

	answer_mb = OK_ANSWER;

	switch (tag)
	{
		case k_QuestionD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_ErrorD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_PromptD_mb:
			XmStringGetLtoR (
			   ((XmSelectionBoxCallbackStruct *) reason)->value,
			   XmSTRING_DEFAULT_CHARSET, &value);
			sprintf (prompt_value, "%s", value);
			XtFree (value);
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_MessageD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_FileSelectD_mb:
			XmStringGetLtoR (
			    ((XmFileSelectionBoxCallbackStruct *) reason)->value,
			    XmSTRING_DEFAULT_CHARSET, &value);
			sprintf (fileselect_value, "%s", value);
			XtFree (value);
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		default:
			break;
	}
}

/*----------------------------------------------------------------------------
 *	M B _ C A N C E L _ P R O C
 *--------------------------------------------------------------------------*/

void	mb_cancel_proc (w, tag, reason)
Widget	w;
int	tag;
XmAnyCallbackStruct *reason;
{
	answer_mb = CANCEL_ANSWER;

	switch (tag)
	{
		case k_QuestionD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_PromptD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		case k_FileSelectD_mb:
			action_mb = ANSWER_RECEIVED_ACTION;
			break;
		default:
			break;
	}
}

