/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	THIS C SOURCE CODE FILE CONTAINS FUNCTIONS FOR MANAGING THE NETWORK
	EDITOR AND SOME OTHER NETWORK FEATURES.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifndef _NETWORK_FUNCTIONS_
#define _NETWORK_FUNCTIONS_

#define _DEBUG_ 0

/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include "find_files.h"

/* application globals include files */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif



/*******************************************************************************
	Function for initializing global directory variables
*******************************************************************************/
void init_directories(void)
{
	char *home;
	char	*network_ed_help;
	char buffer[256];
	
	home = getenv("HOME");
	if(home)
	{
		strcpy(working_directory,home);
		strcpy(project_directory,home);
		sprintf(buffer,"%s/noname_nbc",home);
		if(!dir_exists(buffer))
		{
			sprintf(buffer,"mkdir %s/noname_nbc",home);
			system(buffer);
		}
		strcpy(project_directory,home);
		strcat(project_directory,"/noname_nbc");
	}
	else
	{
		strcpy(working_directory,".");
		strcpy(project_directory,".");
		sprintf(buffer,"%s/noname_nbc",".");
		if(!dir_exists(buffer))
		{
			sprintf(buffer,"mkdir %s/noname_nbc",".");
			system(buffer);
		}
		strcpy(project_directory,".");
		strcat(project_directory,"/noname_nbc");
	}
	network_ed_help = getenv("XNBC_HELP_DIR");
	if(network_ed_help)
	{
		strcpy(help_directory,network_ed_help);
		strcat(help_directory,"/network_editor");
	}
	else
	{
		strcpy(help_directory,"/usr/local/bin/xnbc.bin/V8/help");
		fprintf(stderr,"XNBC_HELP_DIR not defined, using %s\n",
			help_directory);
		strcat(help_directory,"/network_editor");
	}
}
/*******************************************************************************
	Function for updating directories
*******************************************************************************/
void update_directories(char *full_lnk_name)
{
	char *pt;
	char buffer[256];
	char tmp[256];
	
	strcpy(buffer,full_lnk_name);
	
	if((pt = strstr(buffer,".ntw"))!=NULL)
			*pt='\0';
			
	if((pt = strrchr(buffer,'/'))!= NULL)
	{
		*pt = '\0';
		pt++;
		strcpy(project_directory,buffer);
		if(!strstr(project_directory,"_nbc"))
		{
			sprintf(tmp,"%s.ntw",pt);
			if(!file_exists(tmp,buffer))
			{
				strcat(project_directory,"/");
				strcat(project_directory,pt);
				strcat(project_directory,"_nbc");
			}
		}
		if(!dir_exists(project_directory))
		{
			sprintf(tmp,"mkdir %s",project_directory);
			system(tmp);
		}
		
	}
	else
	{
		strcpy(project_directory,working_directory);
		if(!strstr(project_directory,"_nbc"))
		{
			sprintf(tmp,"%s.ntw",buffer);
			if(!file_exists(tmp,working_directory))
			{
				strcat(project_directory,"/");
				strcat(project_directory,buffer);
				strcat(project_directory,"_nbc");
			}
		}
		if(!dir_exists(project_directory))
		{
			sprintf(tmp,"mkdir %s",project_directory);
			system(tmp);
		}
		
	}
			
	

}
/*******************************************************************************
	Function for searching for a file in the specified directory
*******************************************************************************/
int file_exists(char *file_name,char *dir_name)
{
	int len;
	struct dirent *dp;
	DIR *dir_pointer;
	
	len = strlen(file_name);
	dir_pointer = opendir(dir_name);
	if(!dir_pointer)
		return False;
	for (dp = readdir(dir_pointer); dp != NULL; dp =
	       readdir(dir_pointer))
	       if ((strlen(dp->d_name)) == len && !strcmp(dp->d_name, file_name))
	       {
	               closedir(dir_pointer);
	               return True;
	       }
	closedir(dir_pointer);
	return False;

}
/*******************************************************************************
	Function for testing if directory exists
*******************************************************************************/
int dir_exists(char *dir_name)
{
	DIR *dir_pointer;

	dir_pointer = opendir(dir_name);
	if(!dir_pointer)
		return False;
	closedir(dir_pointer);
	return True;	
}

/*******************************************************************************
	Function for saving customizations
*******************************************************************************/
void save_customizations(void)
{
	int i;
	FILE *fp;
	char file_name[256];
	char buffer[256];
	
	strcpy(file_name,working_directory);
	strcat(file_name,"/.net_customrc");
	if(!(fp = fopen(file_name,"w")))
	{
		sprintf(buffer,"Can't write %s...",file_name);
		display_error(widget_array[ID_form_custom],
			buffer,
			"Write Error !");
		return;
	}
	
	fprintf(fp,"NetworkEditor:\n");
	
	for(i=0 ; i<15 ; i++)
	{
		if(funcs_net[i].set)
			fprintf(fp,"\t[%s]\n",funcs_net[i].func);
	}
	
	fprintf(fp,"NucleusEditor:\n");
	
	for(i=0 ; i<14 ; i++)
	{
		if(funcs_nucl[i].set)
			fprintf(fp,"\t[%s]\n",funcs_nucl[i].func);
	}
	
	fclose(fp);
	
}
/*******************************************************************************
	Function for reading customizations
*******************************************************************************/
void read_customizations(void)
{
	int i;
	FILE *fp;
	char buffer[256];
	char file_name[256];
	char pt[255];
	char *p;
	int level;
	
	strcpy(file_name,working_directory);
	strcat(file_name,"/.net_customrc");
	if(!(fp = fopen(file_name,"r")))
	{
		sprintf(buffer,"File %s doesn't exist...",file_name);
		display_message(widget_array[ID_form_custom],
			buffer,
			"No Saved Customizations");
		return;
	}
	
	
	while(fgets(buffer,255,fp))
	{
		if(!strcmp(buffer,"NetworkEditor:\n"))
		{
			level = _NETWORK_;
		}
		else if(!strcmp(buffer,"NucleusEditor:\n"))
		{
			level = _NUCLEUS_;	
		}
		else
		{
			p = strchr(buffer,'[');
			if(!p)
				continue;
			p++;
			i=0;
			while(((*p)!=']')&&(i<100))
			{	
				pt[i] = *p;
				p++;
				i++;
			}	
			pt[i] = '\0';
			switch(level)
			{
				case _NETWORK_:
					for(i=0 ; i<15 ; i++)
					{
						if(!strcmp(pt,funcs_net[i].func))
							funcs_net[i].set = True;
					}
					break;
					
				case _NUCLEUS_:
					for(i=0 ; i<14 ; i++)
					{
						if(!strcmp(pt,funcs_nucl[i].func))
							funcs_nucl[i].set = True;
					}
					break;
					
				default:
					break;
			}	
		}
	}
	
	
	fclose(fp);
	
}


/*******************************************************************************
	Function for initializing customizations
*******************************************************************************/
void initialize_customizations(level)
{

	
		
	static FUNC net_funcs[15] = { {"New file",ID_PushB_net_file_new,0,0} , 
						   {"Open file...",ID_PushB_net_file_open,0,0},
						   {"Save file",ID_PushB_net_file_save,0,0},
						   {"Save file as...",ID_PushB_net_file_save_as,0,0},
						   {"Exit",ID_PushB_net_file_exit,0,0},
						   {"New nucleus",ID_PushB_net_nucleus_new,0,0},
						   {"Delete nucleus",ID_PushB_net_nucleus_delete,0,0},
						   {"Modify nucleus...",ID_PushB_net_nucleus_modify,0,0},
						   {"New neuron",ID_PushB_net_neuron_new,0,0},
						   {"Copy neuron",ID_PushB_net_neuron_copy,0,0},
						   {"Delete neuron",ID_PushB_net_neuron_delete,0,0},
						   {"Modify neuron...",ID_PushB_net_neuron_modify,0,0},
						   {"New link",ID_PushB_net_link_new,0,0},
						   {"Delete link",ID_PushB_net_link_delete,0,0},
						   {"Modify link...",ID_PushB_net_link_modify,0,0} }; 
	
	static FUNC nucl_funcs[14] = { {"Add cluster...",ID_PushB_composition_add,0,0} , 
						   {"Remove cluster",ID_PushB_composition_remove,0,0},
						   {"Modify cluster...",ID_PushB_composition_modify,0,0},
						   {"Place neurons...",ID_PushB_generic_place_neurons,0,0},
						   {"Create intra-link...",ID_PushB_generic_create_il,0,0},
						   {"Remove intra-link",ID_PushB_generic_remove_il,0,0},
						   {"Modify intra-link...",ID_PushB_generic_modify_il,0,0},
						   {"New neuron",ID_PushB_neuron_in_nucleus_new,0,0},
						   {"Delete neuron",ID_PushB_neuron_in_nucleus_dele,0,0},
						   {"Modify neuron...",ID_PushB_neuron_in_nucleus_modi,0,0},
						   {"New link",ID_PushB_link_in_nucleus_new,0,0},
						   {"Delete link",ID_PushB_link_in_nucleus_delete,0,0},
						   {"Modify link...",ID_PushB_link_in_nucleus_modify,0,0},
						   {"Nucleus parameters...",ID_PushB_nucleus_para_edit,0,0} }; 
	
	int i;
	int first,p;
	XmString string;
	
	
	which_list = level;
	
	XmListDeleteAllItems(widget_array[ID_list_func]);
	XmListDeleteAllItems(widget_array[ID_list_custom]);
	
	funcs_net = net_funcs;
	funcs_nucl = nucl_funcs;
	
	switch(which_list)
	{
	
		case _NETWORK_:
			
			
			first = ID_PushB_net_f1;
			for(i=0 ; i< 5 ; i++)
			{
				if(XtIsManaged(widget_array[first+i]))
				{
					XtRemoveAllCallbacks(widget_array[first+i],
						XmNactivateCallback);
					XtUnmanageChild(widget_array[first+i]);
					XFlush(the_display);
				}
				
				
			}
			p = 0;
			for(i=0 ; i< 15 ; i++)
			{
				
				if(net_funcs[i].set && (p < 5) )
				{
					XtSetSensitive(widget_array[ID_list_func],True);
					XtAddCallback(widget_array[first+p],XmNactivateCallback,
							net_activate_CB,(XtPointer)net_funcs[i].offset);
					
					string = XmStringCreateLocalized(net_funcs[i].func);
					XtVaSetValues(widget_array[first+p],XmNlabelString,
						string,NULL);	
					XmListAddItem(widget_array[ID_list_custom],string,0);
					XmStringFree(string);
					XtManageChild(widget_array[first+p]);
					p++;
					net_funcs[i].pos = p;	
					
				}
				else if(p >=5)
				{
					XtSetSensitive(widget_array[ID_list_func],False);
					string = XmStringCreateLocalized(net_funcs[i].func);	
					XmListAddItem(widget_array[ID_list_func],string,0);
					XmStringFree(string);
					
				}
				else
				{
					net_funcs[i].pos = 0;
					string = XmStringCreateLocalized(net_funcs[i].func);	
					XmListAddItem(widget_array[ID_list_func],string,0);
					XmStringFree(string);
					
				}
			}
			if(!p)
			{
				XtSetSensitive(widget_array[ID_list_custom],False);
				string = XmStringCreateLocalized("[no customizations]");	
				XmListAddItem(widget_array[ID_list_custom],string,0);
				XmStringFree(string);	
			}
			else
				XtSetSensitive(widget_array[ID_list_custom],True);
			break;
			
		case _NUCLEUS_:
			
			
			first = ID_PushB_nucleus_f1;
			for(i=0 ; i<5 ; i++)
			{
				if(XtIsManaged(widget_array[first+i]))
				{
					XtRemoveAllCallbacks(widget_array[first+i],
						XmNactivateCallback);
					XtUnmanageChild(widget_array[first+i]);
					XFlush(the_display);
				}
					
			}
			p = 0;
			for(i=0 ; i< 14 ; i++)
			{
	
				if(nucl_funcs[i].set && (p < 5) )
				{
					XtSetSensitive(widget_array[ID_list_func],True);
					XtAddCallback(widget_array[first+p],XmNactivateCallback,
							nucl_activate_CB,(XtPointer)nucl_funcs[i].offset);
					string = XmStringCreateLocalized(nucl_funcs[i].func);	
					XtVaSetValues(widget_array[first+p],XmNlabelString,
						string,NULL);
					XmListAddItem(widget_array[ID_list_custom],string,0);
					XmStringFree(string);
					XtManageChild(widget_array[first+p]);
					p++;
					nucl_funcs[i].pos = p;	
					
				}
				else if(p >=5)
				{
					XtSetSensitive(widget_array[ID_list_func],False);
					string = XmStringCreateLocalized(nucl_funcs[i].func);	
					XmListAddItem(widget_array[ID_list_func],string,0);
					XmStringFree(string);
					
				}
				else
				{
					nucl_funcs[i].pos = 0;
					string = XmStringCreateLocalized(nucl_funcs[i].func);	
					XmListAddItem(widget_array[ID_list_func],string,0);
					XmStringFree(string);
					
				}
			}
			if(!p)
			{
				XtSetSensitive(widget_array[ID_list_custom],False);
				string = XmStringCreateLocalized("[no customizations]");	
				XmListAddItem(widget_array[ID_list_custom],string,0);
				XmStringFree(string);	
			}
			else
				XtSetSensitive(widget_array[ID_list_custom],True);
			break;
			
		default:
		
			break;
	
	}
	

}

/*******************************************************************************
	Function for updating customizations
*******************************************************************************/
void update_customizations(int message)
{

	int pos_count;
	int *pos_list;
	Widget list;
	int set;
	int i;
	int max;
	XmString *all_items;
	char *pt;
	FUNC *funcs;
	
	switch(message)
	{
		case ID_arrowButton_on:
			list = widget_array[ID_list_func];
			set = True;
			break;

		case ID_arrowButton_off:
			list = widget_array[ID_list_custom];
			set = False;
			break;

		default:
			break;
	}

	XmListGetSelectedPos(list,&pos_list,&pos_count);
	
	if(!pos_count)
		return;
	
	XtVaGetValues(list,XmNitems,&all_items,NULL);
	
	XmStringGetLtoR(all_items[pos_list[0]-1],XmSTRING_DEFAULT_CHARSET,
				&pt);
	
	
	
	switch(which_list)
	{
		case _NETWORK_:
			funcs = funcs_net;
			for(i=0 ; i<15 ; i++)
			{
				if(!strcmp(pt,funcs[i].func))
				{
					funcs[i].set = set;
					break;
				}
					
			}
			break;

		case _NUCLEUS_:
			funcs = funcs_nucl;
			for(i=0 ; i<14 ; i++)
			{
				if(!strcmp(pt,funcs[i].func))
				{
					funcs[i].set = set;
					break;
				}
					
			}
			break;

		default:
			break;
	}

	initialize_customizations(which_list);

}
/*******************************************************************************
	Functions for verifying exiting  nucleus name or not
*******************************************************************************/
int nucleus_name_already_exists(char *name)
{
	Nucleus_widget *nucleus_tmp;
	int i;
	
	if(!current_network)
	{
		fprintf(stderr,"current network is NULL in nucleus_name_already_exists!\n");
		exit(-1);
	}

	for(i=0 ; i<current_network->nb_nucleus	; i++)
	{
		nucleus_tmp = current_network->nucleus_array[i];
		if(!strcmp(nucleus_tmp->nucleus_name,name))
		{
			if(!current_nucleus || (current_nucleus != nucleus_tmp))
				return True;
		}
	}
	
	return False;	

}
/*******************************************************************************
	Functions for verifying exiting  cluster name or not
*******************************************************************************/
int cluster_name_already_exists(char *name)
{
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	
	if(!current_network)
	{
		fprintf(stderr,"current network is NULL in nucleus_name_already_exists!\n");
		exit(-1);
	}

	for(i=0 ; i<current_network->nb_nucleus	; i++)
	{
		nucleus_tmp = current_network->nucleus_array[i];
		for(j=0 ; j<nucleus_tmp->nb_clusters ; j++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[j];
			if(!strcmp(cluster_tmp->ch_name,name))
			{
				if(cluster_tmp->cluster_id != current_cluster_id)
					return True;
			}
		}
	}
	
	return False;	

}

/*******************************************************************************
	Function for updating the nucleus,clusters and neurons numbers
*******************************************************************************/
void update_numbers(void)
{
	short nucleus_cnt;
	short cluster_cnt;
	short neuron_cnt;
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_tmp;
	int i,j,k;

	nucleus_cnt = cluster_cnt = neuron_cnt = 0;

	if(!current_network)
	{
		fprintf(stderr,"current network is NULL in nucleus_name_already_exists!\n");
		exit(-1);
	}
	
	for(i=0 ; i<current_network->nb_nucleus	; i++)
	{
		nucleus_cnt++;
		nucleus_tmp = current_network->nucleus_array[i];
		nucleus_tmp->number = nucleus_cnt;
		
		for(j=0 ; j<nucleus_tmp->nb_clusters ; j++)
		{
			
			cluster_cnt++;
			cluster_tmp = nucleus_tmp->cluster_array[j];
			cluster_tmp->number = cluster_cnt;
			
			if(!cluster_tmp->neuron_array)
				continue;
			
			for(k=0 ; k<cluster_tmp->nb_neurons ; k++)
			{
			
				neuron_cnt++;
				neuron_tmp = cluster_tmp->neuron_array[k];
				neuron_tmp->number = neuron_cnt;
				if(!k)
				{
					cluster_tmp->first_neuron = neuron_cnt;
					if(!j)
						nucleus_tmp->first_neuron = neuron_cnt;
				}
				if(k == cluster_tmp->nb_neurons - 1)
				{
					cluster_tmp->last_neuron = neuron_cnt;
					if(j == nucleus_tmp->nb_clusters - 1)
						nucleus_tmp->last_neuron = neuron_cnt;	
				}
			}	
		}
		
	}

	
}


/*******************************************************************************
	Function that generates a unique neuron id
*******************************************************************************/
int manage_neuron_id(int neuron_id, int force)
{
	static int *pile = NULL;
	static last_neuron_id = 0;
	int *pile_tmp;
	int i;
	
	
	if(!neuron_id)
	{
		if(!last_neuron_id)
		{
			pile = (int *)calloc(1,sizeof(int));
			pile[0] = True;
			last_neuron_id++;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",last_neuron_id);
#endif

			return last_neuron_id;
		}
		else if(pile)
		{
			i = 0;
			while(i<last_neuron_id)
			{
				if(pile[i] == False)
				{
					pile[i] = True;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",i+1);
#endif
					return i+1;
				}
				i++;
			}	
			last_neuron_id++;
			pile_tmp = (int *)realloc(pile,last_neuron_id*sizeof(int));
			if(!pile_tmp)
			{
				printf(stderr,"pile is NULL in unique neuron_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[last_neuron_id-1] = True;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",last_neuron_id);
#endif			
			return last_neuron_id;
		}
	}
	else if(neuron_id == -1)
	{
		if(pile)
			free(pile);
		pile = NULL;
		last_neuron_id = 0;
#if _DEBUG_
			fprintf(stderr,"\n ALL Neuron Id Freed...\n");
#endif
		return 0;
	}
	else if(force)
	{
		if(!last_neuron_id)
		{
			pile_tmp = (int *)calloc(1,neuron_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in unique neuron_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[neuron_id-1] = True;
			last_neuron_id = neuron_id;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",last_neuron_id);
#endif
			return last_neuron_id;
		}
		else if(neuron_id > last_neuron_id)
		{
			pile_tmp = (int *)realloc(pile,neuron_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in unique neuron_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[neuron_id-1] = True;
			for(i=last_neuron_id ; i<neuron_id-1 ; i++)
				pile[i] = False;
			last_neuron_id = neuron_id;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",last_neuron_id);
#endif
			return last_neuron_id;
		}
		else
		{
			if(pile[neuron_id-1] == True)
			{
				fprintf(stderr,"pile[neuron_id-1] is True in manage_neuron_id!!!!\n");
				exit(-1);
			}
			pile[neuron_id-1] = True;
#if _DEBUG_
			fprintf(stderr,"neuron id %d loaded...\n",neuron_id);
#endif
			return neuron_id;
		}
	}	
	else 
	{
		pile[neuron_id-1] = False;
#if _DEBUG_
			fprintf(stderr,"neuron id %d is now FREE...\n",neuron_id);
#endif
		return 0;
	}
}
/*******************************************************************************
	Function that generates a unique cluster id
*******************************************************************************/
int manage_cluster_id(int cluster_id, int force)
{
	static int *pile = NULL;
	static last_cluster_id = 0;
	int *pile_tmp;
	int i;
	
	if(!cluster_id)
	{
		if(!last_cluster_id)
		{
			pile = (int *)calloc(1,sizeof(int));
			pile[0] = True;
			last_cluster_id++;
			return last_cluster_id;
		}
		else if(pile)
		{
			i = 0;
			while(i<last_cluster_id)
			{
				if(pile[i] == False)
				{
					pile[i] = True;
					return i+1;
				}
				i++;
			}	
			last_cluster_id++;
			pile_tmp = (int *)realloc(pile,last_cluster_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile id is NULL in unique cluster_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[last_cluster_id-1] = True;
			return last_cluster_id;
		}
	}
	else if(cluster_id == -1)
	{
		if(pile)
			free(pile);
		pile = NULL;
		last_cluster_id = 0;
	}
	else if(force)
	{
		if(!last_cluster_id)
		{
			pile_tmp = (int *)calloc(1,cluster_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in manage_cluster_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[cluster_id-1] = True;
			last_cluster_id = cluster_id;
			return last_cluster_id;
		}
		else if(cluster_id > last_cluster_id)
		{
			pile_tmp = (int *)realloc(pile,cluster_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in manage_cluster_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[cluster_id-1] = True;
			for(i=last_cluster_id ; i<cluster_id-1 ; i++)
				pile[i] = False;
			last_cluster_id = cluster_id;
			return last_cluster_id;
		}
		else
		{
			if(pile[cluster_id-1] == True)
			{
				fprintf(stderr,"pile[cluster_id-1] is True in manage_cluster_id!!!!\n");
				exit(-1);
			}
			pile[cluster_id-1] = True;
			return cluster_id;
		}
	}		
	else 
	{
		pile[cluster_id-1] = False;
		return 0;
	}
}
/*******************************************************************************
	Function that generates a unique nucleus id
*******************************************************************************/
int manage_nucleus_id(int nucleus_id, int force)
{
	static int *pile = NULL;
	static last_nucleus_id = 0;
	int *pile_tmp;
	int i;
	
	if(!nucleus_id)
	{
		if(!last_nucleus_id)
		{
			pile = (int *)calloc(1,sizeof(int));
			pile[0] = True;
			last_nucleus_id++;
			return last_nucleus_id;
		}
		else if(pile)
		{
			i = 0;
			while(i<last_nucleus_id)
			{
				if(pile[i] == False)
				{
					pile[i] = True;
					return i+1;
				}
				i++;
			}	
			last_nucleus_id++;
			pile_tmp = (int *)realloc(pile,last_nucleus_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile id is NULL in unique nucleus_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[last_nucleus_id-1] = True;
			return last_nucleus_id;
		}
	}
	else if(nucleus_id == -1)
	{
		if(pile)
			free(pile);
		pile = NULL;
		last_nucleus_id = 0;
	}
	else if(force)
	{
		if(!last_nucleus_id)
		{
			pile_tmp = (int *)calloc(1,nucleus_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in manage_nucleus_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[nucleus_id-1] = True;
			last_nucleus_id = nucleus_id;
			return last_nucleus_id;
		}
		else if(nucleus_id > last_nucleus_id)
		{
			pile_tmp = (int *)realloc(pile,nucleus_id*sizeof(int));
			if(!pile_tmp)
			{
				fprintf(stderr,"pile is NULL in manage_nucleus_id!!!!\n");
				exit(-1);
			}
			pile = pile_tmp;
			pile[nucleus_id-1] = True;
			for(i=last_nucleus_id ; i<nucleus_id-1 ; i++)
				pile[i] = False;
			last_nucleus_id = nucleus_id;
			return last_nucleus_id;
		}
		else
		{
			if(pile[nucleus_id-1] == True)
			{
				fprintf(stderr,"pile[nucleus_id-1] is True in manage_nucleus_id!!!!\n");
				exit(-1);
			}
			pile[nucleus_id-1] = True;
			return nucleus_id;
		}
	}		
	else
	{
		pile[nucleus_id-1] = False;
		return 0;
	}
}

/******************************************************************************
	Function that deselects all the selected nucleus in the network
******************************************************************************/
void deselect_all_nucleus(void)
{
	Nucleus_widget *nucleus_tmp;
	int i;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current_nucleus is NULL in deselect_all_nucleus!!!\n");
		exit(-1);
	}
	
	nucleus_tmp = current_nucleus;
	
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		if(current_network->nucleus_array[i]->selected)
			select_nucleus(current_network->nucleus_array[i]->form,
			 current_network->nucleus_array[i]->nucleus_id,NULL);	
	}
	
	current_nucleus = nucleus_tmp;
}


/*****************************************************************************
	Function giving default names depending on the context
******************************************************************************/
char *get_name(int *id,int context,int type,int reset)
{
	static int net_number=1;
	static int nucl_number=1;
	char buffer[255];
	char number[10];
	char *name;

	
	if(reset)
	{
		manage_nucleus_id(-1,0);
		manage_cluster_id(-1,0);
	}
	
	if(context == _NETWORK_)
	{
		switch(type)
		{
			case _NEURON_:
				strcpy(buffer,"Neuron_");
				break;
			case _NUCLEUS_:
				strcpy(buffer,"Nucleus_");
			
			default:
				break;
		}
		sprintf(number,"%d",*id = manage_nucleus_id(0,0));
	}
	else if(context == _NUCLEUS_)
	{
		switch(type)
		{
			case _NEURON_:
				strcpy(buffer,"Neuron_");
				break;
			case _CLUSTER_:
				strcpy(buffer,"Cluster_");
			
			default:
				break;
		}
		sprintf(number,"%d",*id = manage_cluster_id(0,0));
	}
	strcat(buffer,number);
	name = (char *)calloc(1,sizeof(buffer)+1); 
	strcpy(name,buffer);
	
	return (char *)name;
		
}
/*****************************************************************************
	Function for allocating a nucleus in the network 
******************************************************************************/
Nucleus_widget *alloc_nucleus_net(void)
{
	Nucleus_widget **nucleus_array;
	int i;
	Nucleus_widget *retval;
	char *name;
	
	if(!current_network)
		return NULL;
	
	if(current_network->nucleus_array)
	{
		nucleus_array = (Nucleus_widget **)calloc(1,
			(current_network->nb_nucleus+1)*sizeof(Nucleus_widget *));
		for(i=0 ; i<current_network->nb_nucleus ; i++)
			nucleus_array[i] = current_network->nucleus_array[i];
				
		retval = nucleus_array[current_network->nb_nucleus] = (Nucleus_widget *)
			calloc(1,sizeof(Nucleus_widget));
		
		for(i=0 ; i<3 ; i++)
		{
			retval->work_width[i] = AREA_SIZE;
			retval->work_height[i] = AREA_SIZE;
			retval->origin_x[i] = AREA_SIZE/2;
			retval->origin_y[i] = AREA_SIZE/2;
			
		}		
		
		name =	get_name(&retval->nucleus_id,_NETWORK_,_NUCLEUS_,False);
		strcpy(retval->nucleus_name,name);
		free(name);
		retval->selected = False;
			
		retval->number = current_network->nb_nucleus + 1;
		
		
		free(current_network->nucleus_array);
		
		current_network->nucleus_array = nucleus_array;
	}
	else
	{
		nucleus_array = (Nucleus_widget **)calloc(1,
			1*sizeof(Nucleus_widget *));
			
		retval = nucleus_array[0] = (Nucleus_widget *)
			calloc(1,sizeof(Nucleus_widget));
		
		for(i=0 ; i<3 ; i++)
		{
			retval->work_width[i] = AREA_SIZE;
			retval->work_height[i] = AREA_SIZE;
			retval->origin_x[i] = AREA_SIZE/2;
			retval->origin_y[i] = AREA_SIZE/2;
			
		}
		name =	get_name(&retval->nucleus_id,_NETWORK_,_NUCLEUS_,False);
		strcpy(retval->nucleus_name,name);	
		free(name);
		retval->number =  1;
		
		current_network->nucleus_array = nucleus_array;
		
	}
	retval->color = grey;
	
	current_network->nb_nucleus++;
	return (Nucleus_widget *)retval;
	

}
/*****************************************************************************
	Function for allocating a neuron in the network (1 neuron = nucleus)
******************************************************************************/
Nucleus_widget *alloc_neuron_net(void)
{
	Nucleus_widget **nucleus_array;
	int i;
	Nucleus_widget *retval;
	char *name;
	
	if(!current_network)
		return NULL;
	
	if(current_network->nucleus_array)
	{
		nucleus_array = (Nucleus_widget **)calloc(1,
			(current_network->nb_nucleus+1)*sizeof(Nucleus_widget *));
		for(i=0 ; i<current_network->nb_nucleus ; i++)
			nucleus_array[i] = current_network->nucleus_array[i];
			
		retval = nucleus_array[current_network->nb_nucleus] = (Nucleus_widget *)
			calloc(1,sizeof(Nucleus_widget));
		
		name = get_name(&retval->nucleus_id,_NETWORK_,_NEURON_,False);	
		strcpy(retval->nucleus_name,name);
		free(name); 
		
		retval->number = current_network->nb_nucleus + 1;
		retval->nb_clusters = 1;
		retval->nb_links = 0;
		retval->nb_neurons = 1;
		retval->selected = False;
		retval->cluster_array = (CLUSTER_WIDGET **)calloc(1,
			1*sizeof(CLUSTER_WIDGET *));
		retval->cluster_array[0] = (CLUSTER_WIDGET *)calloc(1,
			sizeof(CLUSTER_WIDGET));
			
		
		name =	get_name(&retval->cluster_array[0]->cluster_id,_NUCLEUS_,_NEURON_,False);
		strcpy(retval->cluster_array[0]->ch_name, name);
		free(name);
		retval->cluster_array[0]->neuron_array = (NEURON **)calloc(1,
			1*sizeof(NEURON *));
						
		retval->cluster_array[0]->neuron_array[0] =(NEURON *)calloc(1,
			sizeof(NEURON));
		
		retval->cluster_array[0]->nb_neurons++;
		
		retval->cluster_array[0]->neuron_array[0]->neuron_id =
			manage_neuron_id(0,0);
		
		
		free(current_network->nucleus_array);
		
		current_network->nucleus_array = nucleus_array;
		
		
	}
	else
	{
		nucleus_array = (Nucleus_widget **)calloc(1,
			1*sizeof(Nucleus_widget *));
			
		retval = nucleus_array[0] = (Nucleus_widget *)
			calloc(1,sizeof(Nucleus_widget));
		
		name = get_name(&retval->nucleus_id,_NETWORK_,_NEURON_,False);	
		strcpy(retval->nucleus_name,name);
		free(name);	
		retval->number =  1;
		retval->nb_clusters = 1;
		retval->nb_links = 0;
		retval->nb_neurons = 1;
		retval->cluster_array = (CLUSTER_WIDGET **)calloc(1,
			1*sizeof(CLUSTER_WIDGET *));
		retval->cluster_array[0] = (CLUSTER_WIDGET *)calloc(1,
			sizeof(CLUSTER_WIDGET));
		
		name = get_name(&retval->cluster_array[0]->cluster_id,_NUCLEUS_,_NEURON_,False);
		strcpy(retval->cluster_array[0]->ch_name, name);
		free(name);
		
		retval->cluster_array[0]->nb_neurons++;
		
		retval->cluster_array[0]->neuron_array = (NEURON **)calloc(1,
			1*sizeof(NEURON *));
		
		retval->cluster_array[0]->neuron_array[0] =(NEURON *)calloc(1,
			sizeof(NEURON));
			
		retval->cluster_array[0]->neuron_array[0]->neuron_id = 
			 manage_neuron_id(0,0);
			
		current_network->nucleus_array = nucleus_array;
	}
	retval->color = grey;
	
	current_network->nb_nucleus++;
	current_network->nb_neurons++;
	
	update_numbers();
	
	return (Nucleus_widget *)retval;
	
	
}

/*****************************************************************************
	Function for creation of a new nucleus
******************************************************************************/
int new_nucleus(void)
{	
	
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		PointerMotionMask,FALSE,move_nucleus,
			NULL); 
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		ButtonPressMask,FALSE,set_nucleus,
			NULL); 
			
	XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LHview],True,True);
	
	current_nucleus = alloc_nucleus_net();		
	
	return 0;
}

/*****************************************************************************
	Function for creation of a new neuron
******************************************************************************/
int new_neuron(void)
{
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		PointerMotionMask,FALSE,move_neuron,
			NULL); 
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		ButtonPressMask,FALSE,set_neuron,
			NULL);
	
	 
	XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LHview],True,True);
	
	current_nucleus = alloc_neuron_net();
	
	return 0;
}
/*****************************************************************************
	Function for moving a dashed neuron in the network area
******************************************************************************/
void move_neuron(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	static int lastX=0;
	static int lastY=0;
	
	enable_label_net = 1;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	if((posX==lastX) && (posY==lastY))
		return;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	
	XDrawArc(the_display,XtWindow(widget_array[ID_DrawingA_net]),
						the_GC,posX-NEURON_WIDTH/2,posY-NEURON_HEIGHT/2,
						NEURON_WIDTH,NEURON_HEIGHT,
						0*64,360*64);
						
	lastX = posX;
	lastY = posY;
	
	
}
/******************************************************************************
	Function for controling placement of a new neuron and then creating it
*******************************************************************************/
void set_neuron(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	static int click_count=0;
	int width,height;
	char buffer[255];
	CLUSTER_WIDGET *current_cluster;
	NEURON *current_neuron;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	XtVaGetValues(widget_array[ID_DrawingA_net],
		XmNwidth,&width,XmNheight,&height,NULL);
	
	enable_label_net = 0;
	
	click_count++;
	
	if(click_count % 2)
	{
		current_nucleus->L = (float)(posX - current_network->origin_x[0])
			/network_scale[0];
		current_nucleus->posX[0] = (int)(network_scale[0]*current_nucleus->L)
			+ current_network->origin_x[0] - NEURON_WIDTH/2;
		current_nucleus->posX[1] = (int)(network_scale[1]*current_nucleus->L)
			+ current_network->origin_x[1] - NEURON_WIDTH/2;
		
		current_nucleus->H = (float)(current_network->origin_y[0] - posY)
			/network_scale[0];
		current_nucleus->posY[0] = (int)(-network_scale[0]*current_nucleus->H)
			+ current_network->origin_y[0] - NEURON_HEIGHT/2;
		current_nucleus->posY[2] =  (int)(-network_scale[2]*current_nucleus->H)
			+ current_network->origin_y[2] - NEURON_HEIGHT/2;
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LAPview],True,True);
		
		string = XmStringCreateLocalized(
		"Now evaluate AP position of this neuron in the horizontal view...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
	}
	else 
	{ 
		current_nucleus->AP = (float)(current_network->origin_y[1] - posY)
			/network_scale[1];
		
		current_nucleus->posY[1] = (int)(-network_scale[1]*current_nucleus->AP)
			+ current_network->origin_y[1] - NEURON_HEIGHT/2;
		current_nucleus->posX[2] = (int)(network_scale[2]*current_nucleus->AP)
			+ current_network->origin_x[2] - NEURON_WIDTH/2;
		
		current_nucleus->form = XtVaCreateWidget (current_nucleus->nucleus_name,
		    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
		    XmNwidth, NEURON_WIDTH,
		    XmNheight, NEURON_HEIGHT,
		    XmNborderWidth, 0,
		    XmNrecomputeSize, False,
		    NULL);
		   
		current_nucleus->bgnd = XCreatePixmap(the_display,
		 	XtWindow(widget_array[ID_DrawingA_net]),NEURON_WIDTH,NEURON_HEIGHT,
		 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_net])));
		 	
		XSetForeground(the_display,the_GC,white);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display, current_nucleus->bgnd,the_GC,
			0,0,NEURON_WIDTH,NEURON_HEIGHT);
		 
		XtVaSetValues(current_nucleus->form,XmNbackgroundPixmap,
			current_nucleus->bgnd,NULL);
		
		string = XmStringCreateLocalized(
		"Ok, the neuron is placed. Let's give it a name and define its type...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
		XtRemoveEventHandler(widget_array[ID_DrawingA_net],
			XtAllEvents,True,(XtEventHandler)move_neuron,NULL);
			
		XtRemoveEventHandler(widget_array[ID_DrawingA_net],
			XtAllEvents,True,(XtEventHandler)set_neuron,NULL);
		
		enable_label_net = 1;
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LHview],True,True);
			
		current_cluster = current_nucleus->cluster_array[0];
		current_neuron = current_nucleus->cluster_array[0]->neuron_array[0];
		current_neuron->anatomy.L = 
			current_cluster->L = current_nucleus->L;
		current_neuron->anatomy.H = 
			current_cluster->H = current_nucleus->H;
		current_neuron->anatomy.AP = 
			current_cluster->AP = current_nucleus->AP;	
		
		
		XmTextReplace(widget_array[ID_Text_neuron_edit_name],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_name]),
			current_nucleus->nucleus_name);
			
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			last_neuron_file);
			
		XmTextRemove(widget_array[ID_Text_neuron_edit_l]);
		sprintf(buffer,"%.3f",current_nucleus->L);
		XmTextReplace(widget_array[ID_Text_neuron_edit_l],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_l]),
			buffer);
		
		XmTextRemove(widget_array[ID_Text_neuron_edit_h]);	
		sprintf(buffer,"%.3f",current_nucleus->H);
		XmTextReplace(widget_array[ID_Text_neuron_edit_h],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_h]),
			buffer);
			
		XmTextRemove(widget_array[ID_Text_neuron_edit_ap]);
		sprintf(buffer,"%.3f",current_nucleus->AP);
		XmTextReplace(widget_array[ID_Text_neuron_edit_ap],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_ap]),
			buffer);
		
		if(!XtIsManaged(widget_array[ID_FormD_neuron_edit]))
		{	
			XtManageChild(widget_array[ID_FormD_neuron_edit]);
			XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_neuron_edit])),
						the_colormap);
		}
	}
			
}
/*****************************************************************************
	Function for moving a dashed nucleus in the network area
******************************************************************************/
void move_nucleus(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	static lastX=0;
	static lastY=0;
	
	enable_label_net = 1;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	if((posX==lastX) && (posY==lastY))
		return;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	
	XDrawRectangle(the_display,XtWindow(widget_array[ID_DrawingA_net]),
						the_GC,posX-NUCLEUS_WIDTH/2,posY-NUCLEUS_HEIGHT/2,
						NUCLEUS_WIDTH,NUCLEUS_HEIGHT);
	lastX = posX;
	lastY = posY;
	
	
}
/******************************************************************************
	Function for controling placement of a new nucleus and then creating it
*******************************************************************************/
void set_nucleus(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	Widget label_net_info = widget_array[ID_Label_net_information];
	static int click_count=0;
	int width,height;
	XmString string;
	char buffer[255];
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	XtVaGetValues(widget_array[ID_DrawingA_net],
		XmNwidth,&width,XmNheight,&height,NULL);
	
	click_count++;
	
	enable_label_net = 0;
	
	if(click_count % 2)
	{	
		current_nucleus->L = (float)(posX - current_network->origin_x[0])
			/network_scale[0];
		current_nucleus->posX[0] = (int)(network_scale[0]*current_nucleus->L)
			+ current_network->origin_x[0] - NUCLEUS_WIDTH/2;
		current_nucleus->posX[1] = (int)(network_scale[1]*current_nucleus->L)
			+ current_network->origin_x[1] - NUCLEUS_WIDTH/2;
		
		current_nucleus->H = (float)(current_network->origin_y[0] - posY)
			/network_scale[0];
		current_nucleus->posY[0] = (int)(-network_scale[0]*current_nucleus->H)
			+ current_network->origin_y[0] - NUCLEUS_HEIGHT/2;
		current_nucleus->posY[2] =  (int)(-network_scale[2]*current_nucleus->H)
			+ current_network->origin_y[2] - NUCLEUS_HEIGHT/2;
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LAPview],True,True);
		
		string = XmStringCreateLocalized(
		"Now evaluate AP position of this nucleus in the horizontal view...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
	}
	else 
	{ 
		current_nucleus->AP = (float)(current_network->origin_y[1] - posY)
			/network_scale[1];
		
		current_nucleus->posY[1] =  (int)(-network_scale[1]*current_nucleus->AP)
			+ current_network->origin_y[1] - NUCLEUS_HEIGHT/2;
		current_nucleus->posX[2] = (int)(network_scale[2]*current_nucleus->AP)
			+ current_network->origin_x[2] - NUCLEUS_WIDTH/2;
		
		current_nucleus->form = XtVaCreateWidget (current_nucleus->nucleus_name,
		    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
		    XmNwidth, NUCLEUS_WIDTH,
		    XmNheight, NUCLEUS_HEIGHT,
		    XmNborderWidth, 0,
		    XmNrecomputeSize, False,
		    NULL);
		
		current_nucleus->bgnd = XCreatePixmap(the_display,
		 	XtWindow(widget_array[ID_DrawingA_net]),NUCLEUS_WIDTH,NUCLEUS_HEIGHT,
		 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_net])));
		 	
		XSetForeground(the_display,the_GC,white);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display, current_nucleus->bgnd,the_GC,
			0,0,NUCLEUS_WIDTH,NUCLEUS_HEIGHT);
		
		XtVaSetValues(current_nucleus->form,XmNbackgroundPixmap,
			current_nucleus->bgnd,NULL);
			   
		string = XmStringCreateLocalized(
		"Ok, now let's define this nucleus a little bit more...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
		XtRemoveEventHandler(widget_array[ID_DrawingA_net],
			XtAllEvents,True,(XtEventHandler)move_nucleus,NULL);
			
		XtRemoveEventHandler(widget_array[ID_DrawingA_net],
			XtAllEvents,True,(XtEventHandler)set_nucleus,NULL);
		
		
		enable_label_net = 1;
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_popup_LHview],True,True);
		
		sprintf(buffer,"%d neurons",current_nucleus->nb_neurons);
		string = XmStringCreateLocalized(buffer);
		XtVaSetValues(widget_array[ID_LabelG_nbneuron_in_nucleus],
			XmNlabelString,string,NULL);
		XmStringFree(string);
		
		XmTextRemove(widget_array[ID_TextF_AP_nucleusC]);
		sprintf(buffer,"%.2f",current_nucleus->AP);
		XmTextReplace(widget_array[ID_TextF_AP_nucleusC],0,
			XmTextGetLastPosition(widget_array[ID_TextF_AP_nucleusC]),
			buffer);
		XmTextRemove(widget_array[ID_TextF_AP_nucleusR]);
		sprintf(buffer,"%.2f",1.8);
		XmTextReplace(widget_array[ID_TextF_AP_nucleusR],0,
			XmTextGetLastPosition(widget_array[ID_TextF_AP_nucleusR]),
			buffer);
			
		XmTextRemove(widget_array[ID_TextF_H_nucleusC]);
		sprintf(buffer,"%.2f",current_nucleus->H);
		XmTextReplace(widget_array[ID_TextF_H_nucleusC],0,
			XmTextGetLastPosition(widget_array[ID_TextF_H_nucleusC]),
			buffer);
		XmTextRemove(widget_array[ID_TextF_H_nucleusR]);
		sprintf(buffer,"%.2f",1.8);
		XmTextReplace(widget_array[ID_TextF_H_nucleusR],0,
			XmTextGetLastPosition(widget_array[ID_TextF_H_nucleusR]),
			buffer);	
			
		XmTextRemove(widget_array[ID_TextF_L_nucleusC]);
		sprintf(buffer,"%.2f",current_nucleus->L);
		XmTextReplace(widget_array[ID_TextF_L_nucleusC],0,
			XmTextGetLastPosition(widget_array[ID_TextF_L_nucleusC]),
			buffer);
		XmTextRemove(widget_array[ID_TextF_L_nucleusR]);
		sprintf(buffer,"%.2f",1.8);
		XmTextReplace(widget_array[ID_TextF_L_nucleusR],0,
			XmTextGetLastPosition(widget_array[ID_TextF_L_nucleusR]),
			buffer);	
		
		XmTextReplace(widget_array[ID_TextF_nucleus_name],0,
			XmTextGetLastPosition(widget_array[ID_TextF_nucleus_name]),
			current_nucleus->nucleus_name);
			
		XtManageChild(widget_array[ID_FormD_nucleus_parameters]);
		XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_nucleus_parameters])),
						the_colormap);
		
	}
}
/*****************************************************************************
	Function who calculates the nucleus positions when the scale of
	the displayed view changed
******************************************************************************/
int calculate_positions(void)
{
	Nucleus_widget *nucleus_tmp;
	int i;
	int width,height;
	int view;
	
	if(!current_network)
		return -1;
	if(!current_network->nucleus_array)
		return -1;
	
	view = which_network_view();
	
	switch(view)
	{
		case 0:
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				
				nucleus_tmp->posX[0] = (int)(network_scale[0]* nucleus_tmp->L)
					+ current_network->origin_x[0] - width/2;
				nucleus_tmp->posX[1] = 	(int)(network_scale[1]* nucleus_tmp->L)
					+ current_network->origin_x[1] - width/2;
				nucleus_tmp->posY[0] = (int)(-network_scale[0]* nucleus_tmp->H)
					+ current_network->origin_y[0] - height/2;
				nucleus_tmp->posY[2] = (int)(-network_scale[2]* nucleus_tmp->H)
					+ current_network->origin_y[2] - height/2;			
			}
			break;
	
		case 1:
			
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				nucleus_tmp->posX[0] = (int)(network_scale[0]* nucleus_tmp->L)
					+ current_network->origin_x[0] - width/2;
				nucleus_tmp->posX[1] = 	(int)(network_scale[1]* nucleus_tmp->L)
					+ current_network->origin_x[1] - width/2;
				nucleus_tmp->posY[1] = (int)(-network_scale[1]* nucleus_tmp->AP)
					+ current_network->origin_y[1] - height/2;
				nucleus_tmp->posX[2] = (int)(network_scale[2]* nucleus_tmp->AP)
					+ current_network->origin_x[2] - width/2;
			}
			break;
	
		case 2:
			
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				
				nucleus_tmp->posY[1] = (int)(-network_scale[1]* nucleus_tmp->AP)
					+ current_network->origin_y[1] - height/2;
				nucleus_tmp->posX[2] = 	(int)(network_scale[2]* nucleus_tmp->AP)
					+ current_network->origin_x[2] - width/2;
				nucleus_tmp->posY[0] = (int)(-network_scale[0]* nucleus_tmp->H)
					+ current_network->origin_y[0] - height/2;
				nucleus_tmp->posY[2] = (int)(-network_scale[2]* nucleus_tmp->H)
					+ current_network->origin_y[2] - height/2;
	
				
			}
			break;
	
		default:
			break;
	}	
	
	return 0;
}
/*****************************************************************************
	Function for redisplaying the network when the view changed
******************************************************************************/
int redisplay_network(void)
{
	int width,height;

	XtVaGetValues(widget_array[ID_DrawingA_net],XmNwidth,&width,
		XmNheight,&height,NULL);
	
	XSetForeground(the_display,the_GC,color_view[which_network_view()]);
	XSetBackground(the_display,the_GC,black);
	XFillRectangle(the_display,network_pixmap,the_GC,0,0,width,height);
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			working_cursor);
	
		
	calculate_positions();
	
	draw_net_axes();
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	draw_nucleus();
	
	draw_network_links();
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
	
	return 0;
}
/******************************************************************************
	Function for drawing the nucleus in the network
*******************************************************************************/
int draw_nucleus(void)
{
	Nucleus_widget *nucleus_tmp;
	Nucleus_widget **nucleus_array;
	int i,j;
	Widget form;
	Pixmap bgnd;
	char buffer[255];
	
	if(!current_network)
		return -1;
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nCan't draw nucleus because the nucleus array is empty!\n");
#endif	
		return -1;
	}
	
	nucleus_array = (Nucleus_widget **)calloc(1,current_network->nb_nucleus*
		sizeof(Nucleus_widget *));
		
	for(i=0 ; i<current_network->nb_nucleus ; i++)
		nucleus_array[i] = current_network->nucleus_array[i];
	
	sort_nucleus_by_depth(nucleus_array);	
	
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nCan't draw nucleus because one of them is empty!\n");
#endif	
			return -1;
		}
		else if(!nucleus_tmp->form)
		{
#if _DEBUG_
			fprintf(stderr,"\nCan't draw nucleus: no form !\n");
#endif	
			return -1;
		}
		else
		{
			
			XtVaSetValues(nucleus_tmp->form,XmNbackgroundPixmap,None,NULL);
			XtDestroyWidget(nucleus_tmp->form);
			if(nucleus_tmp->nb_neurons ==1)
			{
				nucleus_tmp->form = XtVaCreateWidget (nucleus_tmp->nucleus_name,
			    	xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
			    	XmNwidth, NEURON_WIDTH,
			    	XmNheight, NEURON_HEIGHT,
			    	XmNborderWidth, 0,
			    	XmNrecomputeSize, False,
			    	NULL);
			    XtVaSetValues(nucleus_tmp->form,XmNbackgroundPixmap,
			    	nucleus_tmp->bgnd,NULL);
			}
			else
			{
				nucleus_tmp->form = XtVaCreateWidget (nucleus_tmp->nucleus_name,
			    	xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
			    	XmNwidth, NUCLEUS_WIDTH,
			    	XmNheight, NUCLEUS_HEIGHT,
			    	XmNborderWidth, 0,
			    	XmNrecomputeSize, False,
			    	NULL);
			    XtVaSetValues(nucleus_tmp->form,XmNbackgroundPixmap,
			    	nucleus_tmp->bgnd,NULL);
			}
			XtAddEventHandler(nucleus_tmp->form,ButtonPressMask,False,
				(XtEventHandler)select_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonReleaseMask,False,
				(XtEventHandler)release_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonMotionMask,False,
				(XtEventHandler)chplace_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonMotionMask,False,
				(XtEventHandler)info_nucleus,nucleus_tmp->nucleus_id);	
			XtAddEventHandler(nucleus_tmp->form,EnterWindowMask,False,
				(XtEventHandler)info_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,LeaveWindowMask,False,
				(XtEventHandler)clear_label_net_info,
				nucleus_tmp->nucleus_id);
				
			draw_one_nucleus(nucleus_tmp);
		}
		
			
	}
	free(nucleus_array);
}

/*******************************************************************************
	Function for redrawing one nucleus
*******************************************************************************/
void draw_one_nucleus(Nucleus_widget *nucleus_tmp)
{
	Widget form = nucleus_tmp->form;
	Pixmap bgnd = nucleus_tmp->bgnd;
	char buffer[255];
	
	if(nucleus_tmp->nb_neurons == 1)
	{
		if(XtIsManaged(form))
			XtUnmanageChild(form);

		if((nucleus_tmp->posX[which_network_view()]<0)
			 || (nucleus_tmp->posX[which_network_view()]>=AREA_SIZE)
			 ||	(nucleus_tmp->posY[which_network_view()]<0)
			 ||	(nucleus_tmp->posY[which_network_view()]>=AREA_SIZE) )
			 return;

		XtVaSetValues(form,XmNx,nucleus_tmp->posX[which_network_view()],
			XmNy,nucleus_tmp->posY[which_network_view()],NULL);

		XtManageChild(form);

		sprintf(buffer,"%d",nucleus_tmp->nucleus_id);
		XSetForeground(the_display,the_GC,black);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawArc(the_display,bgnd,
			the_GC,1,1,NEURON_WIDTH-2,NEURON_HEIGHT-2,
			0*64,360*64);
		if(nucleus_tmp->selected == False)
		{
			XSetForeground(the_display,the_GC,nucleus_tmp->color);
		}
		else
		{
			XSetForeground(the_display,the_GC,white);
		}	
		XFillArc(the_display,bgnd,
			the_GC,2,2,NEURON_WIDTH-4,NEURON_HEIGHT-4,
			0*64,360*64);
		XSetForeground(the_display,the_GC,black);	
		XDrawString(the_display,bgnd,the_GC,NEURON_WIDTH/2-1,
			NEURON_HEIGHT/2,buffer,strlen(buffer));

		XClearWindow(the_display,XtWindow(form));

	}
	else
	{
		if(XtIsManaged(form))
			XtUnmanageChild(form);


		if((nucleus_tmp->posX[which_network_view()]<0)
			 || (nucleus_tmp->posX[which_network_view()]>=AREA_SIZE)
			 ||	(nucleus_tmp->posY[which_network_view()]<0)
			 ||	(nucleus_tmp->posY[which_network_view()]>=AREA_SIZE) )
			 return;

		XtVaSetValues(form,XmNx,nucleus_tmp->posX[which_network_view()],
			XmNy,nucleus_tmp->posY[which_network_view()],NULL);

		XtManageChild(form);

		sprintf(buffer,"%d",nucleus_tmp->nucleus_id);
		XSetForeground(the_display,the_GC,black);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawRectangle(the_display,bgnd,
			the_GC,1,1,NUCLEUS_WIDTH-2,NUCLEUS_HEIGHT-2);
		if(nucleus_tmp->selected == False)
		{
			XSetForeground(the_display,the_GC,nucleus_tmp->color);
		}
		else
		{
			XSetForeground(the_display,the_GC,white);	
		}	
		XFillRectangle(the_display,bgnd,
			the_GC,2,2,NUCLEUS_WIDTH-4,NUCLEUS_HEIGHT-4);
		XSetForeground(the_display,the_GC,black);	
		XDrawString(the_display,bgnd,the_GC,NUCLEUS_WIDTH/2-2,
			NUCLEUS_HEIGHT/2-10,buffer,strlen(buffer));
		sprintf(buffer,"%d neurons",nucleus_tmp->nb_neurons);
		XDrawString(the_display,bgnd,the_GC,2,
			NUCLEUS_HEIGHT/2+10,buffer,strlen(buffer));

		XClearWindow(the_display,XtWindow(form));	
	}

}

/*******************************************************************************
	Function for drawing only one network link
*******************************************************************************/
void draw_one_network_link(LINK_PARAMETERS *link_tmp)
{
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int view = which_network_view();
	float X1,X2,Y1,Y2;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;
	double x_mid,y_mid,d,dmin;
	
	
	nucleus_src = search_nucleus(link_tmp->source_id);
	nucleus_tgt = search_nucleus(link_tmp->target_id);
	if(!nucleus_src || !nucleus_tgt)
		return;
	if(nucleus_src == nucleus_tgt)
	{
		switch(link_tmp->type)
		{

			case _FULL_EXCITATION_:
			case _PART_EXCITATION_:
				color = red[4];
				break;

			case _FULL_INHIBITION_:
			case _PART_INHIBITION_:
				color = blue[4];
				break;

			case _FULL_RANDOMIZED_:
			case _PART_RANDOMIZED_:
				color = green;
				break;

			default:
				color = grey;
				break;
		}
		nucleus_src->color = color;
		draw_one_nucleus(nucleus_src);

	}
	else
	{
		switch(view)
		{
			case 0:
				X1 = nucleus_src->L;
				Y1 = nucleus_src->H;
				X2 = nucleus_tgt->L;
				Y2 = nucleus_tgt->H;
				break;

			case 1:
				X1 = nucleus_src->L;
				Y1 = nucleus_src->AP;
				X2 = nucleus_tgt->L;
				Y2 = nucleus_tgt->AP;
				break;

			case 2:
				X1 = nucleus_src->AP;
				Y1 = nucleus_src->H;
				X2 = nucleus_tgt->AP;
				Y2 = nucleus_tgt->H;
				break;

			default:

				break;
		}
		posX1 =	(int)(X1*network_scale[view]) 
			+ current_network->origin_x[view];
		posX2 =	(int)(X2*network_scale[view]) 
			+ current_network->origin_x[view];	
		posY1 =	(int)(-Y1*network_scale[view]) 
			+ current_network->origin_y[view];
		posY2 =	(int)(-Y2*network_scale[view]) 
			+ current_network->origin_y[view];
		
		switch(link_tmp->type)
		{

			case _FULL_EXCITATION_:
			case _PART_EXCITATION_:
				color = red[4];
				over_x = over_y = +3;
				break;

			case _FULL_INHIBITION_:
			case _PART_INHIBITION_:
				color = blue[4];
				over_x = over_y = -3;
				break;

			case _FULL_RANDOMIZED_:
			case _PART_RANDOMIZED_:
				color = green;
				over_x = over_y = 0;
				break;

			default:
				return;
		}
		XSetForeground(the_display,the_GC,color);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawLine(the_display, network_pixmap,the_GC,
			posX1+over_x,posY1+over_y,posX2+over_x,posY2+over_y);

		if((posX2<0)
			 || (posX2>=AREA_SIZE)
			 ||	(posY2<0)
			 ||	(posY2>=AREA_SIZE) )
			 return;
		
		
		
		if(nucleus_tgt->nb_neurons == 1)
		{
			
			

			dmin = (double)(NEURON_WIDTH*NEURON_WIDTH+NEURON_HEIGHT*NEURON_HEIGHT)
					/4.0;

			
			d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
			x_mid = posX1+over_x;
			y_mid = posY1+over_y;
			while(d>1.1*dmin)
			{
				x_mid = (9*x_mid + posX2+over_x)/10;
				y_mid = (9*y_mid + posY2+over_y)/10;
				d = (double)(posX2-x_mid)*(posX2-x_mid)
					+(double)(posY2-y_mid)*(posY2-y_mid);
			}
			
			Xmid = (int)x_mid - NEURON_WIDTH/4;
			Ymid = (int)y_mid - NEURON_HEIGHT/4;
		
			XDrawArc(the_display,network_pixmap,
				the_GC,Xmid ,
				Ymid,
				12,12,
				0*64,360*64);
			XFillArc(the_display,network_pixmap,
				the_GC,Xmid ,
				Ymid,
				12,12,
				0*64,360*64);
		}
		else
		{
			
#define DELTA(a,b) (a>b ? a-b : b-a)			



			dmin = (double)(NUCLEUS_WIDTH*NUCLEUS_WIDTH+NUCLEUS_HEIGHT*NUCLEUS_HEIGHT)
						/(4.0*(0.8+0.75*(1.0-(double)(MIN(DELTA(posY2,posY1),DELTA(posX2,posX1)))
							/(double)(MAX(DELTA(posY2,posY1),DELTA(posX2,posX1))))));	
		
			d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
			x_mid = posX1+over_x;
			y_mid = posY1+over_y;
			while(d>1.0*dmin)
			{
				x_mid = (19*x_mid + posX2 + over_x)/20;
				y_mid = (19*y_mid + posY2 + over_y)/20;
				d = (double)(posX2-x_mid)*(posX2-x_mid)
					+(double)(posY2-y_mid)*(posY2-y_mid);
			}
			
			Xmid = (int)x_mid - NEURON_WIDTH/4;
			Ymid = (int)y_mid - NEURON_HEIGHT/4;
		
			XDrawArc(the_display,network_pixmap,
				the_GC,Xmid ,
				Ymid,
				12,12,
				0*64,360*64);
			XFillArc(the_display,network_pixmap,
				the_GC,Xmid ,
				Ymid,
				12,12,
				0*64,360*64);
		}

	}
	

}

/******************************************************************************
	Function for drawing the links between nucleus in the network
*******************************************************************************/
int draw_network_links(void)
{
	LINK_PARAMETERS *link_tmp;
	int i;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int view = which_network_view();
	float X1,X2,Y1,Y2;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;

	
	if(!current_network->nb_links)
		return 0;
	
	for(i=0 ; i<current_network->nb_links ; i++)
	{
		link_tmp = current_network->link_array[i];
		if(link_tmp->level != _NETWORK_)
		{
			fprintf(stderr,"Bad level link in draw_network_links!!\n");
			exit(-1);
		}
		draw_one_network_link(link_tmp);
		
	}
	
	return 0;
}

/******************************************************************************
	Function for redefining the coordinates of the origin in the current view
	in the network
*******************************************************************************/
int redefine_origin(void)
{
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		PointerMotionMask,FALSE,move_origin,
			(XtPointer)(unsigned long)which_network_view()); 
	
	XtAddEventHandler(widget_array[ID_DrawingA_net],
		ButtonPressMask,FALSE,set_new_origin,
			(XtPointer)(unsigned long)which_network_view()); 
	
	return 0;
}
/*****************************************************************************
	Function for setting origin of the network axes
******************************************************************************/
void set_new_origin(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	Nucleus_widget *nucleus_tmp;
	int i;
	int width,height;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	
	XSetFunction(the_display,the_GC,GXcopy);
		
	current_network->origin_x[message] = posX;
	current_network->origin_y[message] = posY;
	
	switch(message)
	{
		case 0:
			if(!current_network->nucleus_array)
				break;
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				
				nucleus_tmp->posX[0] = (int)(network_scale[0]* nucleus_tmp->L)
					+ current_network->origin_x[0] - width/2;
				nucleus_tmp->posX[1] = 	(int)(network_scale[1]* nucleus_tmp->L)
					+ current_network->origin_x[1] - width/2;
				nucleus_tmp->posY[0] = (int)(-network_scale[0]* nucleus_tmp->H)
					+ current_network->origin_y[0] - height/2;
				nucleus_tmp->posY[2] = (int)(-network_scale[2]* nucleus_tmp->H)
					+ current_network->origin_y[2] - height/2;			
			}
			break;
	
		case 1:
			if(!current_network->nucleus_array)
				break;
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				nucleus_tmp->posX[0] = (int)(network_scale[0]* nucleus_tmp->L)
					+ current_network->origin_x[0] - width/2;
				nucleus_tmp->posX[1] = 	(int)(network_scale[1]* nucleus_tmp->L)
					+ current_network->origin_x[1] - width/2;
				nucleus_tmp->posY[1] = (int)(-network_scale[1]* nucleus_tmp->AP)
					+ current_network->origin_y[1] - height/2;
				nucleus_tmp->posX[2] = (int)(network_scale[2]* nucleus_tmp->AP)
					+ current_network->origin_x[2] - width/2;
			}
			break;
	
		case 2:
			if(!current_network->nucleus_array)
				break;
			for(i=0 ; i<current_network->nb_nucleus ; i++)
			{
				nucleus_tmp = current_network->nucleus_array[i];
				if(!nucleus_tmp)
					break;
				
				if(nucleus_tmp->nb_neurons == 1)
				{
					width = NEURON_WIDTH;
					height = NEURON_HEIGHT;
				}
				else
				{
					width = NUCLEUS_WIDTH;
					height = NUCLEUS_HEIGHT;
				}
				
				nucleus_tmp->posY[1] = (int)(-network_scale[1]* nucleus_tmp->AP)
					+ current_network->origin_y[1] - height/2;
				nucleus_tmp->posX[2] = 	(int)(network_scale[2]* nucleus_tmp->AP)
					+ current_network->origin_x[2] - width/2;
				nucleus_tmp->posY[0] = (int)(-network_scale[0]* nucleus_tmp->H)
					+ current_network->origin_y[0] - height/2;
				nucleus_tmp->posY[2] = (int)(-network_scale[2]* nucleus_tmp->H)
					+ current_network->origin_y[2] - height/2;
	
				
			}
			break;
	
		default:
			break;
	}
	
	redisplay_network();
	
	XtRemoveEventHandler(widget_array[ID_DrawingA_net],
		PointerMotionMask,True,(XtEventHandler)move_origin,(XtPointer)(unsigned long)message);
	
	XtRemoveEventHandler(widget_array[ID_DrawingA_net],
		ButtonPressMask,True,(XtEventHandler)set_new_origin,(XtPointer)(unsigned long)message);
		
	
	
}

/*****************************************************************************
	Function for moving network axes
******************************************************************************/
void move_origin(Widget w,int message, XEvent *evt)
{
	int old_x;
	int old_y;
	int posX,posY;
	static int lastX=0;
	static int lastY=0;
	
	old_x = current_network->origin_x[message];
	old_y = current_network->origin_y[message];
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	if((posX == old_x) && (posY == old_y))
		return;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	
	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_net]),the_GC,
		0,posY,
		current_network->work_width[message],
		posY);
	
	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_net]),the_GC,
		posX,0,
		posX,
		current_network->work_height[message]);
		
	lastX = posX;
	lastY = posY;
}

/******************************************************************************
	Function that simply draws the axes	of the network area, using the global
	Graphic Context
*******************************************************************************/
int draw_net_axes(void)
{
	int which_view = which_network_view();
	int i,j,x,y,counter;
	char buffer[255];
	
	XSetFunction(the_display,the_GC,GXcopy);
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
	
	XDrawLine(the_display,network_pixmap,the_GC,
		0,current_network->origin_y[which_view],
		current_network->work_width[which_view],
		current_network->origin_y[which_view]);
	
	XDrawLine(the_display,network_pixmap,the_GC,
		current_network->origin_x[which_view],0,
		current_network->origin_x[which_view],
		current_network->work_height[which_view]);
		
	
	/* x positifs */	
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x < current_network->work_width[which_view])
	{
		i = x - current_network->origin_x[which_view]
			 - counter*network_scale[which_view];
		if((int)(i/network_scale[which_view]) == 1)
		{
			counter++;
			XDrawLine(the_display,network_pixmap,the_GC,
				x,y,x,y-5);
			sprintf(buffer,"+%d",counter);
			XDrawString(the_display,network_pixmap,the_GC,
				x-5,y+20,buffer,strlen(buffer));
		}
		x++; 
	}
	/* x negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x >= 0)
	{
		i = current_network->origin_x[which_view] - x
			 - counter*network_scale[which_view];
		if((int)(i/network_scale[which_view]) == 1)
		{
			counter++;
			XDrawLine(the_display,network_pixmap,the_GC,
				x,y,x,y-5);
			sprintf(buffer,"-%d",counter);
			XDrawString(the_display,network_pixmap,the_GC,
				x-5,y+20,buffer,strlen(buffer));
		}
		x--; 
	}
	/* y negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y < current_network->work_height[which_view])
	{
		j = y - current_network->origin_y[which_view]
			 - counter*network_scale[which_view];
		if((int)(j/network_scale[which_view]) == 1)
		{
			counter++;
			XDrawLine(the_display,network_pixmap,the_GC,
				x,y,x+5,y);
			sprintf(buffer,"-%d",counter);
			XDrawString(the_display,network_pixmap,the_GC,
				x-20,y+5,buffer,strlen(buffer));
		}
		y++; 
	}
	/* y positifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y >= 0)
	{
		j = current_network->origin_y[which_view] - y
			 - counter*network_scale[which_view];
		if((int)(j/network_scale[which_view]) == 1)
		{
			counter++;
			XDrawLine(the_display,network_pixmap,the_GC,
				x,y,x+5,y);
			sprintf(buffer,"+%d",counter);
			XDrawString(the_display,network_pixmap,the_GC,
				x-20,y+5,buffer,strlen(buffer));
		}
		y--; 
	}	
	return 0;
}
/*****************************************************************************
	Function that return the number of the current net view
******************************************************************************/
int which_network_view(void)
{
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup_LHview]))
			return 0;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup_LAPview]))
			return 1;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup_APHview]))
			return 2;
}
/*****************************************************************************
	Function that return the scale number of the current net view
******************************************************************************/
int which_network_scale(void)
{
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup2_huge]))
			return _HUGE;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup2_large]))
			return LARGE;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup2_medium]))
			return MEDIUM;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup2_small]))
			return SMALL;		
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_popup2_tiny]))
			return TINY;		
}

/*****************************************************************************
	Function that forces the scale toggles states
******************************************************************************/
void force_scale(int view)
{
	switch(network_scale[view])
	{
		case _HUGE:
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_huge],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_tiny],False,False);
			break;
		case LARGE:
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_large],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_tiny],False,False);
			break;
		case MEDIUM:
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_medium],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_tiny],False,False);
			break;
		case SMALL:
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_small],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_tiny],False,False);
			break;
		case TINY:
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_popup2_tiny],True,False);
			break;
		default:
			break;
	}
}

/****************************************************************************
	Function for allocating and initializing the network structure
******************************************************************************/
int init_network(void)
{
	int i;
	
	if(current_network)
		return -1;
		
	current_nucleus = NULL;
	previous_selected_nucleus = NULL;
	
	
	
	current_network = (Network *)calloc(1,sizeof(Network));
	for(i=0 ; i<3 ; i++)
	{
		current_network->work_width[i] = AREA_SIZE;
		current_network->work_height[i] = AREA_SIZE;
		current_network->origin_x[i] = AREA_SIZE/2;
		current_network->origin_y[i] = AREA_SIZE/2;
		network_scale[i] = MEDIUM;
		nucleus_scale[i] = NUCLEUS_HUGE;
	}
		
	return 0;
}

/****************************************************************************
	Function for activating or deactivating the net menus buttons
*****************************************************************************/
int enable_net_menu(int flag)
{
	XtVaSetValues(widget_array[ID_PushB_net_nucleus_delete],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_nucleus_modify],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_neuron_copy],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_neuron_delete],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_neuron_modify],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_file_save],XmNsensitive,False,NULL);
	
	if(*current_network->name)
		XtVaSetValues(widget_array[ID_PushB_net_file_save],XmNsensitive,flag,NULL);
	
	XtVaSetValues(widget_array[ID_PushB_net_link_new],XmNsensitive,flag,NULL);	
	XtVaSetValues(widget_array[ID_PushB_net_link_delete],XmNsensitive,flag,NULL);
	XtVaSetValues(widget_array[ID_PushB_net_link_modify],XmNsensitive,flag,NULL);
	
	
	return 0;
}

/**************************************************************************
	Function for exiting nbc network editor
****************************************************************************/

int exit_network(Widget w,XtPointer client,XtPointer call)
{

	unsigned long used[NB_COLORS];
	int i;
#if 0	
	plot_network("essai_plot.ps","lp4");
#endif	
	save_neuron_file_list(True);

	destroy_network();

	manage_neuron_id(-1,0);
	manage_cluster_id(-1,0);
	manage_nucleus_id(-1,0);
	
	if(network_pixmap)
		XFreePixmap(the_display,network_pixmap);
		
	if(nucleus_pixmap)
		XFreePixmap(the_display,nucleus_pixmap);
	
	if(curve_pixmap)
		XFreePixmap(the_display,curve_pixmap);
		
	if(matrix_pixmap)
		XFreePixmap(the_display,matrix_pixmap);	
	
	network_pixmap = (Pixmap)NULL;
	nucleus_pixmap = (Pixmap)NULL;
	curve_pixmap = (Pixmap)NULL;
	matrix_pixmap = (Pixmap)NULL;
	
	if(private)
		XFreeColormap(the_display,the_colormap);
	else
	{
		used[0] = colors[C_GREY];
		for(i=0 ; i<NB_COLORS ; i++)
			used[i] = used[0] + i;
		XFreeColors(the_display,
				the_colormap,
				used,
				NB_COLORS,
				AllPlanes);
	
	}
	
	display_help(-1);

#if _MEMSPY_
	memspy_scan();
    memspy_status();
#endif
	
	exit(0);

}

/****************************************************************************
	Function for destroying and freeing the network structure
*****************************************************************************/
int destroy_network(void)
{
	reset_network();
	if(current_network)
		free(current_network);
	
	current_network = (Network *)NULL;
	
	return 0;
}

/****************************************************************************
		Function for destroying explicitly the nucleus,link and tree
*****************************************************************************/
int reset_network(void)
{
	int i,j,k;
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array,*cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	LINK_PARAMETERS **link_array,*link_tmp;
	
	if(!current_network)
		return -1;
	
	nucleus_array = current_network->nucleus_array;
	if(nucleus_array)
	{
		for(i=0 ; i<current_network->nb_nucleus ; i++)
		{
			nucleus_tmp = nucleus_array[i];
			if(nucleus_tmp)
			{
				if(nucleus_tmp->form)
				{
					XtUnmanageChild(nucleus_tmp->form);
					XtDestroyWidget(nucleus_tmp->form);
					if(nucleus_tmp->bgnd)
						XFreePixmap(the_display,nucleus_tmp->bgnd);
				}
				cluster_array = nucleus_tmp->cluster_array;
				if(cluster_array)
				{
					for(j=0 ; j<nucleus_tmp->nb_clusters ; j++)
					{
						cluster_tmp = cluster_array[j];
						if(cluster_tmp)
						{
							if(cluster_tmp->neuron_file)
								free(cluster_tmp->neuron_file);
							neuron_array = cluster_tmp->neuron_array;
							if(neuron_array)
							{
								for(k=0 ; k<cluster_tmp->nb_neurons ; k++)
								{
									neuron_tmp = neuron_array[k];
									if(neuron_tmp->form)
									{
										XtDestroyWidget(neuron_tmp->form);
										if(neuron_tmp->bgnd)
											XFreePixmap(the_display,
												neuron_tmp->bgnd);
									}
									if(neuron_tmp->connections)
										free(neuron_tmp->connections);
									manage_neuron_id(neuron_tmp->neuron_id,0);
									free(neuron_tmp);
								}
								free(neuron_array);
							}
							manage_cluster_id(cluster_tmp->cluster_id,0);
							free(cluster_tmp);
						}
						
					}
					free(cluster_array);
				}
				link_array = nucleus_tmp->link_array;
				if(link_array)
				{
					for(j=0 ; j<nucleus_tmp->nb_links ; j++)
					{
						link_tmp = link_array[j];
						if(link_tmp)
						{

							free(link_tmp);
						}
					}
					free(link_array);
				}
				manage_nucleus_id(nucleus_tmp->nucleus_id,0);
				free(nucleus_tmp);
			}
		}
		free(nucleus_array);
		
	}	
	
	link_array = current_network->link_array;
	if(link_array)
	{
		for(j=0 ; j<current_network->nb_links ; j++)
		{
			link_tmp = link_array[j];
			if(link_tmp)
			{
				free(link_tmp);
			}
		}
		free(link_array);
	}
		
	current_network->nucleus_array = (Nucleus_widget **)NULL;
	current_network->link_array = (LINK_PARAMETERS **)NULL;
	
	free(current_network);
	current_network = (Network *)calloc(1,sizeof(Network));
	for(i=0 ; i<3 ;i++)
	{
		current_network->work_width[i] = AREA_SIZE;
		current_network->work_height[i] = AREA_SIZE;
		current_network->origin_x[i] = AREA_SIZE/2;
		current_network->origin_y[i] = AREA_SIZE/2;
	}
	
	return 0;
}

/***************************************************************************
		Function providing the help on line in the main window
****************************************************************************/

void fill_label_net_info(Widget w,int message, XEvent *evt)
{
XmString string;
Widget label_net_info = widget_array[ID_Label_net_information];
static int lastX=0;
static int lastY=0;
static lastView=0;
int posX,posY,which_view;
float X,Y;
char x_axe[3][3]={"L","L","AP"};
char y_axe[3][3]={"H","AP","H"};
char view_name[3][18]={"<1> Frontal","<2> Horizontal","<3> Sagittal"};
char buffer[255];

if(!enable_label_net)
	return;

switch(message)
	{
	case ID_ToggleB_popup_LHview:
		string = XmStringCreateLocalized(
		"Set Frontal view mode...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_ToggleB_popup_LAPview:
		string = XmStringCreateLocalized(
		"Set Horizontal view mode...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_ToggleB_popup_APHview:
		string = XmStringCreateLocalized(
		"Set Sagittal view mode...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;	
	case ID_PushB_popup_origin:
		string = XmStringCreateLocalized(
		"Set the origin coordinates in the current view...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;		
	case ID_PushB_net_file_new:
		string = XmStringCreateLocalized(
		"Clear current network in order to build an new one...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_plot:
		string = XmStringCreateLocalized(
		"Plot the current network view in Postscript format...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_file_open:
		string = XmStringCreateLocalized(
		"Open a network file without saving the current one...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_file_save:
		string = XmStringCreateLocalized(
		"Save the network in current file...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_file_save_as:
		string = XmStringCreateLocalized(
		"Get the network saved in an other file...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_file_exit:
		string = XmStringCreateLocalized(
		"Quit without saving the network...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_nucleus_new:
		string = XmStringCreateLocalized(
		"Create a new NUCLEUS");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_nucleus_copy:
		string = XmStringCreateLocalized(
		"First, select the NUCLEUS you want to copy...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_nucleus_delete:
		string = XmStringCreateLocalized(
		"First, select the NUCLEUS you want to delete...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_nucleus_modify:
		string = XmStringCreateLocalized(
		"First, select the NUCLEUS you want to modify...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_neuron_new:
		string = XmStringCreateLocalized(
		"Create a new NEURON");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_neuron_copy:
		string = XmStringCreateLocalized(
		"First, select the NEURON you want to copy...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_neuron_delete:
		string = XmStringCreateLocalized(
		"First, select the NEURON you want to delete...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_neuron_modify:
		string = XmStringCreateLocalized(
		"First, select the NEURON you want to modify...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_link_new:
		string = XmStringCreateLocalized(
		"First, select SOURCE and TARGET of the link you want to create...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_link_delete:
		string = XmStringCreateLocalized(
		"First, select SOURCE and TARGET of the link you want to delete...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_link_modify:
		string = XmStringCreateLocalized(
		"First, select SOURCE and TARGET of the link you want to modify...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_preferences:
		string = XmStringCreateLocalized("Set your preferences: accelerators,colors...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_help_on_context:
		string = XmStringCreateLocalized("Get help on a particular context");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_net_help_on_window:
		string = XmStringCreateLocalized("Get help on this window");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_DrawingA_net:
		posX = evt->xbutton.x; 
		posY = evt->xbutton.y;
		which_view = which_network_view();
		if((lastX == posX)&&(lastY == posY)&&(which_view == lastView))
			return;
		lastX = posX;
		lastY = posY;
		lastView = which_view;
		X = (float)(posX - current_network->origin_x[which_view])/network_scale[which_view];
		Y = (float)(current_network->origin_y[which_view] - posY)/network_scale[which_view];
		sprintf(buffer,"Network %s View: current position (%s;%s) = (%.3f;%.3f) [in mm]",
			view_name[which_view],x_axe[which_view],
			y_axe[which_view],X,Y);
		
		string = XmStringCreateLocalized(buffer);
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;	
	
	case ID_PushB_neuron_file_other:
		string = XmStringCreateLocalized(
		"To choose an other neuron type file...Right button for types...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	defaults:
		string = XmStringCreateLocalized("Sorry, no help on line available for this...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	}
}
/*****************************************************************************
	Function for clearing the help on line label 
*****************************************************************************/
void clear_label_net_info(Widget w,int message, XEvent *evt)
{
XmString string;
Widget label_net_info = widget_array[ID_Label_net_information];

string = XmStringCreateLocalized("                                    ");
XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
XmStringFree(string);
		

}
/*****************************************************************************
	Function providing a warning before resetting the network,and call reset
******************************************************************************/
int clear_network(void)
{
	char buffer[255];
	int width,height;
	int i;

	if(display_question(widget_array[ID_MainW_network],
		"\tAre you sure ? \nThis will clear the current network.",
		"Confirmation needed...")!= YES_ANSWER)
		return -1;

	while(current_neuron)
		select_neuron(current_neuron->form,
			current_neuron->neuron_id,NULL);	
	while(current_nucleus)
		select_nucleus(current_nucleus->form,
			current_nucleus->nucleus_id,NULL);
		
	reset_network();

	XFlush(the_display);
	
	XtVaGetValues(widget_array[ID_DrawingA_net],XmNwidth,&width,
		XmNheight,&height,NULL);
	
	XSetForeground(the_display,the_GC,color_view[which_network_view()]);
	XSetBackground(the_display,the_GC,black);
	XFillRectangle(the_display,network_pixmap,the_GC,0,0,width,height);
	
	draw_net_axes();
	
	sprintf(buffer,"XNBC Network Editor V%s: [no name]",VERSION);
	XtVaSetValues(XtParent(widget_array[ID_MainW_network]),XmNtitle,
		buffer,NULL);
	
	enable_net_menu(False);
	
	network_modified = False;
	*working_file='\0';
	
	return 0;

}
/******************************************************************************
	Function for opening a new network file
*******************************************************************************/
int open_network(void)
{
	int i;
	char *file_selected;
	char *pt;
	char buffer[255];
	
	if(current_network->nb_links)
		if(display_question(widget_array[ID_MainW_network],
			"Are you sure you've saved current network ?",
			"Confirmation needed...")!= YES_ANSWER)
			return;

	file_selected = display_fileselect (widget_array[ID_MainW_network],
	    3, "*.ntw", "Select the file to be opened :", NULL);

	if(!file_selected)
		return -1;

	if(!(pt=strstr(file_selected,".ntw")))
	{
		sprintf(buffer,"Sorry, can't open %s \nas a link configuration file...",
			file_selected);
		display_error(widget_array[ID_MainW_network],buffer,
			"File Mismatch Error !");
		return -1;
	}
	
	update_directories(file_selected);
	
	while(current_neuron)
		select_neuron(current_neuron->form,
			current_neuron->neuron_id,NULL);	
	while(current_nucleus)
		select_nucleus(current_nucleus->form,
			current_nucleus->nucleus_id,NULL);
		

	
	destroy_network();
	
	init_network();
		
	strcpy(working_file,file_selected);
	

#ifdef _UNIX_
	pt = strrchr(file_selected,'/');
	if(pt)
	{
		pt++;
		strcpy(current_network->name,pt);
	}
	else
		strcpy(current_network->name,file_selected);
#endif
#ifdef _VMS_
	pt = strrchr(file_selected,']');
	if(pt)
	{
		pt++;
		strcpy(current_network->name,pt);
	}
	else
		strcpy(current_network->name,file_selected);
#endif	

	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			working_cursor);
	
	
	if(read_network())
	{	
		enable_net_menu(False);
		
		strcpy(buffer,current_network->name);
		if(pt=strrchr(buffer,'.'))
			*pt = '\0';
		
		destroy_network();
	
		init_network();
		
		strcpy(current_network->name,buffer);
		
		sprintf(buffer,"XNBC Network Editor V%s: %s",VERSION,current_network->name);
		XtVaSetValues(XtParent(widget_array[ID_MainW_network]),XmNtitle,
			buffer,NULL);
		redisplay_network();
			
		XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
		return;
		
	}
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
	
	update_numbers();
	
	for(i=0 ; i<3 ; i++)
	{
		current_network->work_width[i] = AREA_SIZE;
		current_network->work_height[i] = AREA_SIZE;
		current_network->origin_x[i] = AREA_SIZE/2;
		current_network->origin_y[i] = AREA_SIZE/2;
	}

	sprintf(buffer,"XNBC Network Editor V%s: %s",VERSION,current_network->name);
	XtVaSetValues(XtParent(widget_array[ID_MainW_network]),XmNtitle,
		buffer,NULL);
	
	enable_net_menu(True);
	
	redisplay_network();
	
	sprintf(working_file,"%s/%s.ntw",project_directory,current_network->name);
	sprintf(buffer,"network read from file \n%s...",working_file);
	display_message(widget_array[ID_MainW_network],buffer,
		"Done...");
	network_modified = False;
	
	return 0;
			
}
/*****************************************************************************
	Function for loading the network from the different files
******************************************************************************/
int read_network(void)
{

	char *pt;
	int	status;
	
	if(pt = strrchr(current_network->name,'.'))
		*pt = '\0';
	
	if(!read_lnk_file(current_network))
	{
		if(!read_wgt_file(current_network))
			if(!read_len_file(current_network))
				if(!read_anat_file(current_network))
				{
					update_numbers();
					changed = False;
					return 0;	
				}
	} /* fin du if */
	
	changed = False;
	return -1;
}
/*****************************************************************************
	Function for saving the network into specific files
******************************************************************************/
int save_network(int mode)
{
	char *file_selected;
	char buffer[255];

	if(!current_network->nb_neurons)
	{
		sprintf(buffer,"Sorry, can't save network yet, \nit is empty..."
				);
		display_error(widget_array[ID_MainW_network],buffer,
				"Nothing to Save !");
			return;
	}
	
	if(!*working_file || mode)
	{
		file_selected = display_fileselect (widget_array[ID_MainW_network],
		    3, "*.ntw", "Please enter name of link\nconfiguration file :", NULL);

		if(!file_selected)
		{
			display_message(widget_array[ID_MainW_network],
				"Can't save network without\nlink configuration file name...",
				"Missing File Name");
			return;
		}
		
		if(!strstr(file_selected,".ntw"))
		{
			sprintf(buffer,"Sorry, can't save %s \nas a link configuration file...",
				file_selected);
			display_error(widget_array[ID_MainW_network],buffer,
				"File Mismatch Error");
			return;
		}	
		strcpy(working_file,file_selected);
		update_directories(working_file);

	}
	
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			working_cursor);
	
			
	if(!write_network())
	{
		XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
		sprintf(working_file,"%s/%s.ntw",project_directory,current_network->name);
		sprintf(buffer,"wrote network into file \n%s...",working_file);
		display_message(widget_array[ID_MainW_network],buffer,"Done...");
		put_network_name(get_dir_name(),working_file);
		return 0;
	
	}

	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
	
	return -1;
}
/******************************************************************************
	Function that creates explicitly the different files for saving the net
*******************************************************************************/
int write_network(void)
{
	char *pt;
	char buffer[255];
	
	strcpy(current_network->name,working_file);
	if(pt = strrchr(current_network->name,'.'))
		*pt = '\0';
		
	if(pt = strrchr(current_network->name,'/'))
	{
		pt++;
	}
	else 
		pt = current_network->name;
		
	XtSetSensitive(widget_array[ID_PushB_net_file_save],True);
	sprintf(buffer,"XNBC Network Editor V%s: %s",VERSION,pt);
	XtVaSetValues(toplevel,XmNtitle,buffer,NULL);
	
	strcpy(buffer,pt);
	strcpy(current_network->name,buffer);
	
	if(!create_lnk_file(current_network))
		if(!create_wgt_file(current_network))
			if(!create_len_file(current_network))
				if(!create_anat_file(current_network))
					if(!create_clu_file(current_network))
						return 0;
	return -1;
}


#endif
