/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	THIS C SOURCE CODE FILE CONTAINS FUNCTIONS FOR MANAGING SPECIFIC 
	NEURONS FEATURES.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifndef _NEURON_FUNCTIONS_
#define _NEURON_FUNCTIONS_

#define _DEBUG_ 0

/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* application globals include files */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif


/*****************************************************************************
	Function for saving the neuron file list
*****************************************************************************/
int save_neuron_file_list(flag)
{
	FILE *rc;
	char	buffer[255];
	char file_name[256];
	char *text;
	XmString string;
	char *pt;
	XmString *all_items;
	int i,nb_items;
	
	if(flag)
		if(display_question(widget_array[ID_MainW_network],
			"Do you wish to save the neuron files list \n for next session ?",
			"Neuron Files List") == NO_ANSWER)
			return 0;
	
	XtVaGetValues(widget_array[ID_List_neuron_edit_file],
		XmNitemCount,&nb_items,XmNitems,&all_items,NULL);
	
	if(!nb_items)
		return -1;
	
	strcpy(file_name,working_directory);
	strcat(file_name,"/.x_unitrc");
	if((rc=fopen(file_name,"w"))!= NULL)
	{
		for(i=nb_items-1 ; i>=0 ; i--)
		{
			string = all_items[i];
			XmStringGetLtoR(string,XmSTRING_DEFAULT_CHARSET,
				&text);
			fprintf(rc,"%s\n",text);
			free(text);
		}
		
		
		XtVaGetValues(widget_array[ID_List_cluster_neuron_file],
		XmNitemCount,&nb_items,XmNitems,&all_items,NULL);
	
		if(!nb_items)
		return -1;
		
		
		for(i=nb_items-1 ; i>=0 ; i--)
		{
			string = all_items[i];
			XmStringGetLtoR(string,XmSTRING_DEFAULT_CHARSET,
			&text);
			if(!XmListItemExists(widget_array[ID_List_neuron_edit_file],
				string))
			{
				fprintf(rc,"%s\n",text);
			}
			free(text);

		}
		
		fclose(rc);
		return 0;
		
	}
	
	return -1;
	 
}

/*****************************************************************************
	Function for initializing the neuron file list
******************************************************************************/
int init_neuron_file_list(void)
{
	FILE *rc;
	char buffer[255];
	char file_name[256];
	XmString string;
	char *pt;
	
	
	strcpy(file_name,working_directory);
	strcat(file_name,"/.x_unitrc");
	if((rc=fopen(file_name,"r"))!= NULL)
	{
		while(fgets(buffer,255,rc))
		{
			if(pt = strrchr(buffer,'\n'))
				*pt = '\0';
			string = XmStringCreateLocalized(buffer);
			XmListAddItem(widget_array[ID_List_neuron_edit_file],string,1);
			XmListAddItem(widget_array[ID_List_cluster_neuron_file],string,1);
			XmStringFree(string);
			XmListSelectPos(widget_array[ID_List_neuron_edit_file],1,True);
			XmListSelectPos(widget_array[ID_List_cluster_neuron_file],1,True);
		}
		strcpy(last_neuron_file,buffer);
		fclose(rc);
	}
	else
	{
		strcpy(last_neuron_file,"./default.P_unit");
		strcpy(buffer,"./default.tms");
		string = XmStringCreateLocalized(buffer);
		XmListAddItem(widget_array[ID_List_neuron_edit_file],string,1);
		XmListSelectPos(widget_array[ID_List_neuron_edit_file],1,True);
		XmListAddItem(widget_array[ID_List_cluster_neuron_file],string,1);
		XmListSelectPos(widget_array[ID_List_cluster_neuron_file],1,True);
		XmStringFree(string);
		
		strcpy(buffer,"./default.G_unit");
		string = XmStringCreateLocalized(buffer);
		XmListAddItem(widget_array[ID_List_neuron_edit_file],string,1);
		XmListAddItem(widget_array[ID_List_cluster_neuron_file],string,1);
		XmStringFree(string);
		XmListSelectPos(widget_array[ID_List_neuron_edit_file],1,True);
		XmListSelectPos(widget_array[ID_List_cluster_neuron_file],1,True);
					
		strcpy(buffer,"./default.P_unit");
		string = XmStringCreateLocalized(buffer);
		XmListAddItem(widget_array[ID_List_neuron_edit_file],string,1);
		XmListAddItem(widget_array[ID_List_cluster_neuron_file],string,1);
		XmStringFree(string);			
		XmListSelectPos(widget_array[ID_List_neuron_edit_file],1,True);
		XmListSelectPos(widget_array[ID_List_cluster_neuron_file],1,True);
		
	}
	return 0;
}

/******************************************************************************
	Function for checking the neuron editor fields and register modifications
	if any
*******************************************************************************/
int apply_neuron(void)
{
	char *buffer;
	XmString string;
	char *pt;
	float dim_max = (float)AREA_SIZE / (float)(2*_HUGE);
	
	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_name]);
	if(nucleus_name_already_exists(buffer))
	{
		display_message(widget_array[ID_MainW_network],
		"Sorry, this name already exists...",
	"Invalid Neuron Name !");
		XtFree(buffer);
		return -1;
	}
	if(strlen(buffer)>30)
		buffer[30] = '\0';
	strcpy(current_nucleus->nucleus_name,buffer);
	XtFree(buffer);
	
	buffer = XmTextFieldGetString(widget_array[ID_TextF_neuron_edit_file]);
	if(!strstr(buffer,".P_unit") && !strstr(buffer,".G_unit")
		&& !strstr(buffer,".tms"))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Invalid Neuron type file...","Bad File Name !");
		return -1;
	}
	if(current_nucleus->cluster_array[0]->neuron_file)
		free(current_nucleus->cluster_array[0]->neuron_file); 
	pt = (char *)calloc(1,strlen(buffer)+1);
	strcpy(pt,buffer);
	current_nucleus->cluster_array[0]->neuron_file = pt;
	XtFree(buffer);
	
	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_l]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the L dimension is too large...","Bad Dimension !");
		return -1;
	}
	
	current_nucleus->L = atof(buffer);
	XtFree(buffer);

	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_h]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the H dimension is too large...","Bad Dimension !");
		return -1;
	}
	current_nucleus->H = atof(buffer);
	XtFree(buffer);	

	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_ap]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the AP dimension is too large...","Bad Dimension !");
		return -1;
	}
	
	current_nucleus->AP = atof(buffer);
	XtFree(buffer);
	
	current_nucleus->posX[0] = (int)(network_scale[0]* current_nucleus->L)
					+ current_network->origin_x[0] - NEURON_WIDTH/2;
	current_nucleus->posX[1] = 	(int)(network_scale[1]* current_nucleus->L)
					+ current_network->origin_x[1] - NEURON_WIDTH/2;
	current_nucleus->posY[0] = (int)(-network_scale[0]* current_nucleus->H)
					+ current_network->origin_y[0] - NEURON_HEIGHT/2;
	current_nucleus->posY[2] = (int)(-network_scale[2]* current_nucleus->H)
					+ current_network->origin_y[2] - NEURON_HEIGHT/2;
	current_nucleus->posY[1] = (int)(-network_scale[1]* current_nucleus->AP)
					+ current_network->origin_y[1] - NEURON_HEIGHT/2;
	current_nucleus->posX[2] = (int)(network_scale[2]* current_nucleus->AP)
					+ current_network->origin_x[2] - NEURON_WIDTH/2;
	
	deselect_all_nucleus();
	
	select_nucleus(current_nucleus->form,current_nucleus->nucleus_id,NULL);
	
	
		
	return 0;
}

/******************************************************************************
	Function for selecting the neuron type file
*******************************************************************************/
void select_neuron_type(Widget w,int message, XEvent *evt)
{

	if (evt->xany.type == ButtonPress )
        		
        {
        	/* Position the menu where the event occurred */
    		XmMenuPosition(widget_array[ID_Popup_other],
    		 (XButtonPressedEvent *)(evt));
    		XtManageChild(widget_array[ID_Popup_other]);
        }
			
}
/******************************************************************************
	Function for editing and modifying a selected neuron
*******************************************************************************/
void modify_neuron(void)
{
	char buffer[255];
	Nucleus_widget *nucleus_tmp;
	
	if((!current_nucleus)|| (current_nucleus->nb_neurons!=1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons!=1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which neuron? No neuron was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;

	XmTextReplace(widget_array[ID_Text_neuron_edit_name],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_name]),
			nucleus_tmp->nucleus_name);
	
	if(current_nucleus->cluster_array[0]->neuron_file)	
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			nucleus_tmp->cluster_array[0]->neuron_file);
	else
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			last_neuron_file);

	XmTextRemove(widget_array[ID_Text_neuron_edit_l]);
	sprintf(buffer,"%.3f",nucleus_tmp->L);
	XmTextReplace(widget_array[ID_Text_neuron_edit_l],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_l]),
		buffer);

	XmTextRemove(widget_array[ID_Text_neuron_edit_h]);	
	sprintf(buffer,"%.3f",nucleus_tmp->H);
	XmTextReplace(widget_array[ID_Text_neuron_edit_h],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_h]),
		buffer);

	XmTextRemove(widget_array[ID_Text_neuron_edit_ap]);
	sprintf(buffer,"%.3f",nucleus_tmp->AP);
	XmTextReplace(widget_array[ID_Text_neuron_edit_ap],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_ap]),
		buffer);

	XtManageChild(widget_array[ID_FormD_neuron_edit]);
	XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_neuron_edit])),
						the_colormap);
	
}
/******************************************************************************
	Function for copying an existing selected neuron
*******************************************************************************/
void copy_neuron(void)
{
	Nucleus_widget *nucleus_tmp;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	
	if((!current_nucleus)|| (current_nucleus->nb_neurons!=1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons!=1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which neuron? No neuron was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;
	
	string = XmStringCreateLocalized(
		"Click where the new neuron should be in the Frontal view...");
	XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
	XmStringFree(string);
	
	select_nucleus(nucleus_tmp->form,nucleus_tmp->nucleus_id,NULL);
	
	new_neuron();
	
	XtAddEventHandler(widget_array[ID_FormD_neuron_edit],ExposureMask,False,
		(XtEventHandler)set_neuron_editor_fields_for_copy,
		(XtPointer)nucleus_tmp->nucleus_id);
		
	

}
/******************************************************************************
	Function for setting the neuron editor fields when copying
*******************************************************************************/
void set_neuron_editor_fields_for_copy(Widget w,int message,XEvent *evt)
{
	Nucleus_widget *nucleus_tmp;
	
	nucleus_tmp = search_nucleus(message);
	if(nucleus_tmp->cluster_array[0]->neuron_file)
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			nucleus_tmp->cluster_array[0]->neuron_file);
			
	XtRemoveEventHandler(widget_array[ID_FormD_neuron_edit],
		XtAllEvents,True,(XtEventHandler)set_neuron_editor_fields_for_copy,
		(XtPointer)message);	
	
}


/******************************************************************************
	Function for deleting an existing selected neuron
*******************************************************************************/
void delete_neuron(void)
{
	Nucleus_widget *nucleus_tmp;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	char buffer[255];
	int pos;
	
	if((!current_nucleus)|| (current_nucleus->nb_neurons!=1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons!=1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which neuron? No neuron was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;
	
	sprintf(buffer,
"Do you really want to delete the neuron\n that's named %s with the Id=%d ?",
	nucleus_tmp->nucleus_name,nucleus_tmp->nucleus_id);
		
	if(display_question(widget_array[ID_MainW_network],
		buffer,
		"Confirmation needed...") == NO_ANSWER)
		return;
		
	pos = nucleus_tmp->number;
	
	select_nucleus(nucleus_tmp->form,nucleus_tmp->nucleus_id,NULL);
	
	if(XtIsManaged(nucleus_tmp->form))
			XtUnmanageChild(nucleus_tmp->form);
			
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	if(pos == current_network->nb_nucleus)
	{
		destroy_nucleus(pos);
		current_network->nb_nucleus--;
		current_network->nb_neurons--;
		if(!current_network->nb_nucleus)
		{
			free(current_network->nucleus_array);
			current_network->nucleus_array = NULL;
			enable_net_menu(False);
			
		}
		
		return;
	}
	else
	{
		
		destroy_nucleus(pos);
		
		current_network->
			nucleus_array[current_network->nb_nucleus-1]->number = pos;
	
		current_network->nucleus_array[pos-1] = 
			current_network->nucleus_array[current_network->nb_nucleus-1];
		
		current_network->nucleus_array[current_network->nb_nucleus-1] = NULL;
	
		current_network->nb_nucleus--;
		current_network->nb_neurons--;
		
	}
	
}

/*****************************************************************************
	Function for selecting a neuron in the nucleus area
******************************************************************************/
void select_neuron(Widget w,int message, XEvent *evt)
{
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	char buffer[255];
	static int nb_selected=0;
	Widget label_info = widget_array[ID_Label_nucleus_information];
	XmString string;
	
	if(XtIsManaged(widget_array[ID_FormD_neuron_edit]))
		return;
	if(XtIsManaged(widget_array[ID_FormD_link_simple]))
		return;	
		
	neuron_tmp = search_neuron(message,&cluster_tmp);
	if(!neuron_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNeuron not found in select_neuron ! \n");
#endif		
		exit(-1);	
	}
	
	
	if((nb_selected == 2) && (neuron_tmp->selected == False))
	{
		string = XmStringCreateLocalized(
			"Please note: only two selections are permitted...");
		XtVaSetValues(label_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		 
		return;
	}
	
	
	sprintf(buffer,"%d",neuron_tmp->neuron_id);
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetLineAttributes(the_display,the_GC,2,LineSolid,
		CapRound,JoinRound);
	XDrawArc(the_display,neuron_tmp->bgnd,
		the_GC,1,1,NEURON_WIDTH-2,NEURON_HEIGHT-2,
		0*64,360*64);

	if(neuron_tmp->selected == True)
	{
		neuron_tmp->selected = False;
		nb_selected--;
		if(nb_selected == 1)
		{
			current_neuron = get_first_selected_neuron(&cluster_tmp);
			previous_selected_neuron = current_neuron;
		}	
		else if(nb_selected == 0)
		{
			previous_selected_neuron = NULL;
			current_neuron = NULL;
		}
		XSetForeground(the_display,the_GC,neuron_tmp->color);
	}
	else
	{
		neuron_tmp->selected = True;
		nb_selected++;
		if(nb_selected == 1)
		{
			previous_selected_neuron = neuron_tmp;
			current_neuron = neuron_tmp;
		}
		else if(nb_selected == 2)
		{
			current_neuron = neuron_tmp;
		}
		XSetForeground(the_display,the_GC,white);
	}

	XFillArc(the_display,neuron_tmp->bgnd,
		the_GC,2,2,NEURON_WIDTH-4,NEURON_HEIGHT-4,
		0*64,360*64);
	XSetForeground(the_display,the_GC,black);	
	XDrawString(the_display,neuron_tmp->bgnd,the_GC,NEURON_WIDTH/2-10,
		NEURON_HEIGHT/2+2,buffer,strlen(buffer));

	XClearWindow(the_display,XtWindow(neuron_tmp->form));

	
}
/*****************************************************************************
	Function for searching the neuron that corresponds to the given id
******************************************************************************/
NEURON *search_neuron(int id,CLUSTER_WIDGET **cluster)
{
	NEURON *neuron_tmp;
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_neuron! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_neuron ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_neuron ! \n");
#endif		
			exit(-1);
		}
		j=0;
		while(j<nucleus_tmp->nb_clusters)
		{
			cluster_tmp = nucleus_tmp->cluster_array[j++];
			if(!cluster_tmp)
			{
#if _DEBUG_
				fprintf(stderr,"\nEmpty cluster in search_neuron ! \n");
#endif		
				exit(-1);
			}
			k=0;
			while(k<cluster_tmp->nb_neurons)
			{
				neuron_tmp = (NEURON *)cluster_tmp->neuron_array[k++];
				if(!cluster_tmp)
				{
#if _DEBUG_
					fprintf(stderr,"\nEmpty neuron in search_neuron ! \n");
#endif		
					exit(-1);
				}
				if(neuron_tmp && (neuron_tmp->neuron_id == id))
				{
					*cluster = (CLUSTER_WIDGET *)cluster_tmp;
					return (NEURON *)neuron_tmp;
				}
			}
		}
			
	}	

	return (NEURON *)NULL;		
}
/*****************************************************************************
	Function that returns the fisrt selected neuron
******************************************************************************/
NEURON *get_first_selected_neuron(CLUSTER_WIDGET **cluster)
{
 
 	NEURON *neuron_tmp;
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_neuron! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_neuron ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_neuron ! \n");
#endif		
			exit(-1);
		}
		j=0;
		while(j<nucleus_tmp->nb_clusters)
		{
			cluster_tmp = nucleus_tmp->cluster_array[j++];
			if(!cluster_tmp)
			{
#if _DEBUG_
				fprintf(stderr,"\nEmpty cluster in search_neuron ! \n");
#endif		
				exit(-1);
			}
			k=0;
			while(k<cluster_tmp->nb_neurons)
			{
				neuron_tmp = cluster_tmp->neuron_array[k++];
				if(!cluster_tmp)
				{
#if _DEBUG_
					fprintf(stderr,"\nEmpty neuron in search_neuron ! \n");
#endif		
					exit(-1);
				}
				if(neuron_tmp->selected == True)
				{
					*cluster = cluster_tmp;
					return (NEURON *)neuron_tmp;
				}
			}
		}
			
	}	

	return (NEURON *)NULL;	
}
/******************************************************************************
	Function for releasing and redrawing one neuron after move
*******************************************************************************/
void release_neuron(Widget w,int message, XEvent *evt)
{
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	short width,height;
	short posX,posY,posRelX,posRelY;
	float old_l,old_h,old_ap;
	Widget hBar,vBar;
	short min,max,size,value;
	
	if(evt->type != ButtonRelease)
		return;
	
	neuron_tmp = search_neuron(message,&cluster_tmp);
	if(!neuron_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNeuron not found in release_neuron ! \n");
#endif		
		exit(-1);	
	}
		
	old_l = neuron_tmp->anatomy.L;
	old_h = neuron_tmp->anatomy.H;
	old_ap = neuron_tmp->anatomy.AP;
	
	XtVaGetValues(neuron_tmp->form,XmNx,&posRelX,XmNy,&posRelY,NULL);


	
	posX = evt->xbutton.x + posRelX; 
	posY = evt->xbutton.y + posRelY;
	
	
	
	switch(which_nucleus_view())
	{
		case 0:
			
			neuron_tmp->anatomy.L = (float)(posX - current_nucleus->origin_x[0])
				/nucleus_scale[0] + current_nucleus->L;
			
			neuron_tmp->anatomy.H = (float)(current_nucleus->origin_y[0] - posY)
				/nucleus_scale[0] + current_nucleus->H;
			break;
			
		case 1:
			
			neuron_tmp->anatomy.L = (float)(posX - current_nucleus->origin_x[1])
				/nucleus_scale[1] + current_nucleus->L;
			neuron_tmp->anatomy.AP = (float)(current_nucleus->origin_y[1] - posY)
				/nucleus_scale[1] + current_nucleus->AP;
				
			break;
			
		
		case 2:
		
			neuron_tmp->anatomy.AP = (float)(posX - current_nucleus->origin_x[2])
				/nucleus_scale[2] + current_nucleus->AP;
			neuron_tmp->anatomy.H = (float)(current_nucleus->origin_y[2] - posY)
				/nucleus_scale[2] + current_nucleus->H;
			
			break;	
		
		default:
			break;
	}
	
	if((abs(evt->xbutton.x)>NEURON_WIDTH/2+1) || (abs(evt->xbutton.y)>NEURON_HEIGHT/2+1))
	{
		if((neuron_tmp->anatomy.L - current_nucleus->L) > current_nucleus->delta_L) 
			 	current_nucleus->delta_L = neuron_tmp->anatomy.L 
			 		- current_nucleus->L;
		else if ((neuron_tmp->anatomy.L - current_nucleus->L) < -current_nucleus->delta_L)
			 	current_nucleus->delta_L = -(neuron_tmp->anatomy.L 
			 		- current_nucleus->L);
			 				
		if((neuron_tmp->anatomy.H - current_nucleus->H) > current_nucleus->delta_H) 
			 	current_nucleus->delta_H = neuron_tmp->anatomy.H 
			 		- current_nucleus->H;
		else if ((neuron_tmp->anatomy.H - current_nucleus->H) < -current_nucleus->delta_H)
			 	current_nucleus->delta_H = -(neuron_tmp->anatomy.H 
			 		- current_nucleus->H);
		
		if((neuron_tmp->anatomy.AP - current_nucleus->AP) > current_nucleus->delta_AP) 
			 	current_nucleus->delta_AP = neuron_tmp->anatomy.AP 
			 		- current_nucleus->AP;
		else if ((neuron_tmp->anatomy.AP - current_nucleus->AP) < -current_nucleus->delta_AP)
			 	current_nucleus->delta_AP = -(neuron_tmp->anatomy.AP 
			 		- current_nucleus->AP);	 		
			 		
					  
		select_neuron(w,message,evt);
		redraw_one_neuron(neuron_tmp);
		
		XSetForeground(the_display,the_GC,color_view[which_nucleus_view()]);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display,nucleus_pixmap,the_GC,0,0,AREA_SIZE,AREA_SIZE);
		draw_nucleus_axes();
		draw_neuron_links();
		XtVaSetValues(widget_array[ID_DrawingA_nucleus_view],XmNwidth,
				AREA_SIZE,XmNheight,AREA_SIZE,NULL);	
		XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));
		

	
	}
	else
		XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));
}

/******************************************************************************
	Function for moving a neuron on the nucleus area
******************************************************************************/
void chplace_neuron(Widget w,int message, XEvent *evt)
{	
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	short width,height;
	short posX,posY,posRelX,posRelY;
	float old_l,old_h,old_ap;
	static lastX=0;
	static lastY=0;
	
	if((evt->type != MotionNotify) || (!evt->xmotion.state && Button1Mask))
		return;
	
	neuron_tmp = search_neuron(message,&cluster_tmp);
	if(!neuron_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNeuron not found in release_neuron ! \n");
#endif		
		exit(-1);	
	}
		
	old_l = neuron_tmp->anatomy.L;
	old_h = neuron_tmp->anatomy.H;
	old_ap = neuron_tmp->anatomy.AP;
	
	XtVaGetValues(neuron_tmp->form,XmNx,&posRelX,XmNy,&posRelY,NULL);
	
	posX = evt->xbutton.x + posRelX; 
	posY = evt->xbutton.y + posRelY;	
	
	
	if((posX==lastX) && (posY==lastY))
		return;
		
	switch(which_nucleus_view())
	{
		case 0:
			
			neuron_tmp->anatomy.L = (float)(posX - current_nucleus->origin_x[0])
				/nucleus_scale[0] + current_nucleus->L;
			
			neuron_tmp->anatomy.H = (float)(current_nucleus->origin_y[0] - posY)
				/nucleus_scale[0] + current_nucleus->H;
			break;
			
		case 1:
			
			neuron_tmp->anatomy.L = (float)(posX - current_nucleus->origin_x[1])
				/nucleus_scale[1] + current_nucleus->L;
			neuron_tmp->anatomy.AP = (float)(current_nucleus->origin_y[1] - posY)
				/nucleus_scale[1] + current_nucleus->AP;
				
			break;
			
		
		case 2:
		
			neuron_tmp->anatomy.AP = (float)(posX - current_nucleus->origin_x[2])
				/nucleus_scale[2] + current_nucleus->AP;
			neuron_tmp->anatomy.H = (float)(current_nucleus->origin_y[2] - posY)
				/nucleus_scale[2] + current_nucleus->H;
			
			break;	
		
		default:
			break;
	}
		
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));

	XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);

	XDrawArc(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]),
						the_GC,posX-NEURON_WIDTH/2,posY-NEURON_HEIGHT/2,
						NEURON_WIDTH,NEURON_HEIGHT,
						0*64,360*64);
	
	
	lastX = posX;
	lastY = posY;
	
}

/******************************************************************************
	Function for info on a neuron on the nucleus area
******************************************************************************/
void info_neuron(Widget w,int message, XEvent *evt)
{
	Widget label_info = widget_array[ID_Label_nucleus_information];
	XmString string;
	char buffer[255];
	CLUSTER_WIDGET *cluster_tmp;
	
	NEURON *neuron_tmp;
	
	neuron_tmp = search_neuron(message,&cluster_tmp);
	
	if(!neuron_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNeuron not found in info_neuron ! \n");
#endif		
		exit(-1);	
	}
	
	sprintf(buffer,
"ID:%d CLUSTER:%s /REL to Nucleus Center L=%.3f H=%.3f AP=%.3f/%s" ,
		neuron_tmp->neuron_id,
		cluster_tmp->ch_name,
		neuron_tmp->anatomy.L - current_nucleus->L,
		neuron_tmp->anatomy.H - current_nucleus->H,
		neuron_tmp->anatomy.AP - current_nucleus->AP,
		(neuron_tmp->selected==True)?"<Selected>":"");
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(label_info,XmNlabelString,string,NULL);
	XmStringFree(string);
		
}
/*****************************************************************************
	Function for creation of a new specific neuron in the current nucleus
******************************************************************************/
int new_neuron_in_nucleus(void)
{

	XtAddEventHandler(widget_array[ID_DrawingA_nucleus_view],
		PointerMotionMask,FALSE,move_neuron_in_nucleus,
			NULL); 
	
	XtAddEventHandler(widget_array[ID_DrawingA_nucleus_view],
		ButtonPressMask,FALSE,set_neuron_in_nucleus,
			NULL);
	
	 
	XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_LHview],True,True);
	
	current_neuron = alloc_new_neuron();
	
	return 0;
}
/*****************************************************************************
	Function for moving a dashed neuron in the nucleus area
******************************************************************************/
void move_neuron_in_nucleus(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	static lastX=0;
	static lastY=0;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	if((posX==lastX) && (posY==lastY))
		return;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));
	
	XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	
	XDrawArc(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]),
						the_GC,posX-NEURON_WIDTH/2,posY-NEURON_HEIGHT/2,
						NEURON_WIDTH,NEURON_HEIGHT,
						0*64,360*64);
						
	lastX = posX;
	lastY = posY;
	
	
}
/******************************************************************************
	Function for controling placement of a new neuron and then creating it
	in the nucleus area
*******************************************************************************/
void set_neuron_in_nucleus(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	XmString string;
	Widget label_info = widget_array[ID_Label_nucleus_information];
	static int click_count=0;
	short width,height;
	char buffer[255];
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_tmp;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	
	click_count++;
	
	if(click_count % 2)
	{
		current_neuron->anatomy.L = (float)(posX - current_nucleus->origin_x[0])
			/nucleus_scale[0] + current_nucleus->L;
		
		current_neuron->anatomy.H = (float)(current_nucleus->origin_y[0] - posY)
			/nucleus_scale[0] + current_nucleus->H;
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_LAPview],True,True);
		
		string = XmStringCreateLocalized(
		"Now evaluate AP position of this neuron in the horizontal view...");
		XtVaSetValues(label_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
	}
	else 
	{ 
		current_neuron->anatomy.AP = (float)(current_nucleus->origin_y[1] - posY)
			/nucleus_scale[1] + current_nucleus->AP;
	
		search_neuron(current_neuron->neuron_id,&cluster_tmp);		
		sprintf(buffer,"%s.neuron_%d",cluster_tmp->ch_name,
				current_neuron->neuron_id);
		
		current_neuron->color = grey;
		
		current_neuron->form = XtVaCreateWidget (buffer,
		    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_nucleus_view],
		    XmNwidth, NEURON_WIDTH,
		    XmNheight, NEURON_HEIGHT,
		    XmNborderWidth, 0,
		    XmNrecomputeSize, False,
		    NULL);
		   
		current_neuron->bgnd = XCreatePixmap(the_display,
		 	XtWindow(widget_array[ID_DrawingA_nucleus_view]),NEURON_WIDTH,NEURON_HEIGHT,
		 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_nucleus_view])));
		 	
		XSetForeground(the_display,the_GC,white);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display, current_neuron->bgnd,the_GC,
			0,0,NEURON_WIDTH,NEURON_HEIGHT);
		 
		XtVaSetValues(current_neuron->form,XmNbackgroundPixmap,
			current_neuron->bgnd,NULL);
		
		redraw_one_neuron(current_neuron);
		
		string = XmStringCreateLocalized(
		"Ok, the neuron is placed. Let's give it a name and define its type...");
		XtVaSetValues(label_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		
		XtRemoveEventHandler(widget_array[ID_DrawingA_nucleus_view],
			XtAllEvents,True,(XtEventHandler)move_neuron_in_nucleus,NULL);
			
		XtRemoveEventHandler(widget_array[ID_DrawingA_nucleus_view],
			XtAllEvents,True,(XtEventHandler)set_neuron_in_nucleus,NULL);
			
		
		XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_LHview],True,True);
		
		XmTextReplace(widget_array[ID_Text_neuron_edit_name],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_name]),
			cluster_tmp->ch_name);
			
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			last_neuron_file);
			
		XmTextRemove(widget_array[ID_Text_neuron_edit_l]);
		sprintf(buffer,"%.3f",current_neuron->anatomy.L);
		XmTextReplace(widget_array[ID_Text_neuron_edit_l],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_l]),
			buffer);
		
		XmTextRemove(widget_array[ID_Text_neuron_edit_h]);	
		sprintf(buffer,"%.3f",current_neuron->anatomy.H);
		XmTextReplace(widget_array[ID_Text_neuron_edit_h],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_h]),
			buffer);
			
		XmTextRemove(widget_array[ID_Text_neuron_edit_ap]);
		sprintf(buffer,"%.3f",current_neuron->anatomy.AP);
		XmTextReplace(widget_array[ID_Text_neuron_edit_ap],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_ap]),
			buffer);
		
		refresh_composition();
		
		if(!XtIsManaged(widget_array[ID_FormD_neuron_edit]))
		{	
			XtManageChild(widget_array[ID_FormD_neuron_edit]);
			XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_neuron_edit])),
						the_colormap);
		}
	}
			
}

/*******************************************************************************
	Function for allocating a new specific neuron
*******************************************************************************/
NEURON *alloc_new_neuron(void)
{
	CLUSTER_WIDGET *cluster_tmp;
	CLUSTER_WIDGET **cluster_array
		=	(CLUSTER_WIDGET **)current_nucleus->cluster_array;
	CLUSTER_WIDGET **new_cluster_array;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int i;
	char *name;
	
		
	/* first if the nucleus has already some clusters... */
	if(cluster_array)
	{
		new_cluster_array = (CLUSTER_WIDGET **)calloc(1,
			(current_nucleus->nb_clusters + 1)*sizeof(CLUSTER_WIDGET *));
		
		for(i=0 ; i<current_nucleus->nb_clusters ; i++)
		{
			new_cluster_array[i] = (CLUSTER_WIDGET *)calloc(1,
				sizeof(CLUSTER_WIDGET));
			memcpy(new_cluster_array[i],cluster_array[i],sizeof(CLUSTER_WIDGET));
			free(cluster_array[i]);
		}
		
		cluster_tmp = new_cluster_array[current_nucleus->nb_clusters] = 
			(CLUSTER_WIDGET *)calloc(1,sizeof(
			CLUSTER_WIDGET));
		
		current_nucleus->nb_clusters++;
		current_nucleus->nb_neurons++;
		current_network->nb_neurons++;
		
		
		
		/* now copy the name, neuron file name and the number of neurons for
		this new cluster */
		
		name =	get_name(&cluster_tmp->cluster_id,_NUCLEUS_,_NEURON_,False);
		if(strlen(name) > 19)
			name[19] = '\0';
		strcpy(cluster_tmp->ch_name,name);	
		free(name);
		
		cluster_tmp->neuron_file = (char *)calloc(1,strlen(last_neuron_file)+1);
		strcpy(cluster_tmp->neuron_file,last_neuron_file);
		
		cluster_tmp->nb_neurons = 1;

		/* allocate the neurons... */
		neuron_array = cluster_tmp->neuron_array = (NEURON **)calloc(1,
			sizeof(NEURON *));
		
		neuron_tmp = neuron_array[0] = (NEURON *)calloc(1,
			sizeof(NEURON));
			
		neuron_tmp->neuron_id = manage_neuron_id(0,0);
		
		free(cluster_array);
		current_nucleus->cluster_array = new_cluster_array;
		
	}
	/* if new nucleus with no clusters yet... */
	else
	{
		cluster_array = (CLUSTER_WIDGET **)calloc(1,sizeof(CLUSTER_WIDGET *));
		cluster_tmp = cluster_array[0] = (CLUSTER_WIDGET *)calloc(1,sizeof(
			CLUSTER_WIDGET));
		
		current_nucleus->nb_clusters++;
		current_nucleus->nb_neurons++;
		current_network->nb_neurons++;
		
		
		/* now copy the name, neuron file name and the number of neurons for
		this new cluster */
		
		name =	get_name(&cluster_tmp->cluster_id,_NUCLEUS_,_NEURON_,False);
		if(strlen(name) > 19)
			name[19] = '\0';
		strcpy(cluster_tmp->ch_name,name);	
		free(name);
		
		cluster_tmp->neuron_file = (char *)calloc(1,strlen(last_neuron_file)+1);
		strcpy(cluster_tmp->neuron_file,last_neuron_file);
		
		cluster_tmp->nb_neurons = 1;

		/* allocate the neurons... */
		neuron_array = cluster_tmp->neuron_array = (NEURON **)calloc(1,
			sizeof(NEURON *));
					
		
		neuron_tmp = neuron_array[0] = (NEURON *)calloc(1,
				sizeof(NEURON));
			
		neuron_tmp->neuron_id = manage_neuron_id(0,0);
		
		
		current_nucleus->cluster_array = cluster_array;
	}
	
	update_numbers();
	
	return (NEURON *)neuron_tmp;

}
/******************************************************************************
	Function for checking the neuron editor fields and register modifications
	if any , but on the nucleus level.
*******************************************************************************/
int apply_neuron_in_nucleus(void)
{
	char *buffer;
	XmString string;
	char *pt;
	CLUSTER_WIDGET *cluster_tmp;
	float dim_max = (float)AREA_SIZE / (float)(2*_HUGE);
	
	if(!current_neuron)
	{
		fprintf(stderr,"current neuron is NULL in apply_neuron_in_nucleus \n");
		exit(-1);
	}
	
	
	search_neuron(current_neuron->neuron_id,&cluster_tmp);
	
	current_cluster_id = cluster_tmp->cluster_id;
	
	if(cluster_tmp->nb_neurons == 1)
	{
		buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_name]);
		if(cluster_name_already_exists(buffer))
		{
			display_message(widget_array[ID_FormD_Nucleus],
		"Sorry, this name already exists...",
		"Invalid Neuron Name !");
			XtFree(buffer);
			return -1;
		}
		if(strlen(buffer)>19)
			buffer[19] = '\0';
		strcpy(cluster_tmp->ch_name,buffer);
		XtFree(buffer);
		
		buffer = XmTextFieldGetString(widget_array[ID_TextF_neuron_edit_file]);
		if(!strstr(buffer,".P_unit") && !strstr(buffer,".G_unit")
			&& !strstr(buffer,".tms"))
		{
			display_error(widget_array[ID_FormD_neuron_edit],
				"Invalid Neuron type file...","Bad File Name !");
			return -1;
		}
		if(cluster_tmp->neuron_file)
			free(cluster_tmp->neuron_file); 
		pt = (char *)calloc(1,strlen(buffer)+1);
		strcpy(pt,buffer);
		cluster_tmp->neuron_file = pt;
		XtFree(buffer);
	}
	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_l]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the L dimension is too large...","Bad Dimension !");
		return -1;
	}
	
	current_neuron->anatomy.L = atof(buffer);
	XtFree(buffer);

	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_h]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the H dimension is too large...","Bad Dimension !");
		return -1;
	}
	current_neuron->anatomy.H = atof(buffer);
	XtFree(buffer);	

	buffer = XmTextGetString(widget_array[ID_Text_neuron_edit_ap]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_neuron_edit],
			"Sorry, the AP dimension is too large...","Bad Dimension !");
		return -1;
	}
	
	current_neuron->anatomy.AP = atof(buffer);
	XtFree(buffer);
	
	deselect_all_neuron();
	
	select_neuron(current_neuron->form,current_neuron->neuron_id,NULL);
	
	
		
	return 0;
}
/*****************************************************************************
	Function for searching the neuron that corresponds to the name and file
******************************************************************************/
NEURON *identify_neuron(char *cluster_name,char *neuron_file)
{
	NEURON *neuron_tmp;
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_neuron! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_neuron ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_neuron ! \n");
#endif		
			exit(-1);
		}
		j=0;
		while(j<nucleus_tmp->nb_clusters)
		{
			cluster_tmp = nucleus_tmp->cluster_array[j++];
			if(!cluster_tmp)
			{
#if _DEBUG_
				fprintf(stderr,"\nEmpty cluster in search_neuron ! \n");
#endif		
				exit(-1);
			}
			
			if(!strcmp(cluster_tmp->ch_name,cluster_name) && 
				!(strcmp(cluster_tmp->neuron_file,neuron_file)))
				return (NEURON *)cluster_tmp->neuron_array[0];
			
		}
			
	}	

	return (NEURON *)NULL;		
}
/******************************************************************************
	Function for editing and modifying a selected neuron in the nucleus area
*******************************************************************************/
void modify_neuron_in_nucleus(void)
{
	char buffer[255];
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	
	if((!current_neuron)
		|| (current_neuron->selected == False))
	{
		
		if((!previous_selected_neuron)
			|| (previous_selected_neuron->selected == False))
		{
			display_error(widget_array[ID_DrawingA_nucleus_view],
				"Which neuron? No neuron was selected...","Can't do that !");
			return;
		}
		else
			neuron_tmp = previous_selected_neuron;
	}	
	else
		neuron_tmp = current_neuron;

	search_neuron(neuron_tmp->neuron_id,&cluster_tmp);
	
	if(cluster_tmp->nb_neurons == 1)
	XmTextReplace(widget_array[ID_Text_neuron_edit_name],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_name]),
			cluster_tmp->ch_name);
	else
	{
		sprintf(buffer,"%s.neuron_%d",cluster_tmp->ch_name,neuron_tmp->neuron_id);
		XmTextReplace(widget_array[ID_Text_neuron_edit_name],0,
			XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_name]),
			buffer);
	}
	
	if(cluster_tmp->neuron_file)	
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			cluster_tmp->neuron_file);
	else
		XmTextFieldReplace(widget_array[ID_TextF_neuron_edit_file],0,
			XmTextGetLastPosition(widget_array[ID_TextF_neuron_edit_file]),
			last_neuron_file);

	XmTextRemove(widget_array[ID_Text_neuron_edit_l]);
	sprintf(buffer,"%.3f",neuron_tmp->anatomy.L);
	XmTextReplace(widget_array[ID_Text_neuron_edit_l],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_l]),
		buffer);

	XmTextRemove(widget_array[ID_Text_neuron_edit_h]);	
	sprintf(buffer,"%.3f",neuron_tmp->anatomy.H);
	XmTextReplace(widget_array[ID_Text_neuron_edit_h],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_h]),
		buffer);

	XmTextRemove(widget_array[ID_Text_neuron_edit_ap]);
	sprintf(buffer,"%.3f",neuron_tmp->anatomy.AP);
	XmTextReplace(widget_array[ID_Text_neuron_edit_ap],0,
		XmTextGetLastPosition(widget_array[ID_Text_neuron_edit_ap]),
		buffer);

	XtManageChild(widget_array[ID_FormD_neuron_edit]);
	XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_neuron_edit])),
						the_colormap);	
}

/*******************************************************************************
	Function for deleting a neuron form the nucleus area
*******************************************************************************/
void delete_neuron_in_nucleus(void)
{
	char buffer[255];
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int pos;
	
	if(creating)
		neuron_tmp = current_neuron;
	else if((!current_neuron)
		|| (current_neuron->selected == False))
	{
		
		if((!previous_selected_neuron)
			|| (previous_selected_neuron->selected == False))
		{
			display_error(widget_array[ID_DrawingA_nucleus_view],
				"Which neuron? No neuron was selected...","Can't do that !");
			return;
		}
		else
			neuron_tmp = previous_selected_neuron;
	}	
	else
		neuron_tmp = current_neuron;

	search_neuron(neuron_tmp->neuron_id,&cluster_tmp);
	
	sprintf(buffer,"Do you really want to delete the neuron\n with the ID %d from the cluster %s ?",
		neuron_tmp->neuron_id, cluster_tmp->ch_name);
	if(display_question(widget_array[ID_FormD_Nucleus],
			buffer,
			"Confirmation needed...") == NO_ANSWER)
			return;
	
	
	if(cluster_tmp->nb_neurons <= 2)
	{
		if(cluster_tmp->nb_neurons == 2)
		{
			sprintf(buffer,"Do you really want to delete the cluster\n with the ID %d and name %s ?",
				cluster_tmp->cluster_id, cluster_tmp->ch_name);
			if(display_question(widget_array[ID_FormD_Nucleus],
				buffer,
				"Confirmation needed...") == NO_ANSWER)
				return;
		}
		delete_cluster(cluster_tmp->ch_name,cluster_tmp->neuron_file,
			cluster_tmp->nb_neurons);
	}
	else 
	{
		if(neuron_tmp == cluster_tmp->neuron_array[cluster_tmp->nb_neurons-1])
		{
			delete_single_neuron(
				&(cluster_tmp->neuron_array[cluster_tmp->nb_neurons-1]));
		}
		else
		{
			pos = 0;
			while(neuron_tmp != cluster_tmp->neuron_array[pos])
				pos++;
			delete_single_neuron(
				&(cluster_tmp->neuron_array[pos]));
			cluster_tmp->neuron_array[pos] = 
				cluster_tmp->neuron_array[cluster_tmp->nb_neurons-1];
			cluster_tmp->neuron_array[cluster_tmp->nb_neurons-1] = NULL;
			
		}	
		cluster_tmp->nb_neurons--;
		current_nucleus->nb_neurons--;
		current_network->nb_neurons--;
	}
	
	refresh_composition();
	
}


/*******************************************************************************
	Function for deleting a single neuron structure
*******************************************************************************/
void delete_single_neuron(NEURON **neuron)
{
	
	notify_neuron_deletion(*neuron);
	verify_extremities(*neuron);
	if((*neuron)->connections)
	{
		free((*neuron)->connections);
		(*neuron)->connections = NULL;
	}
	if((*neuron)->selected == True)
		select_neuron((*neuron)->form,(*neuron)->neuron_id,NULL);
	
	manage_neuron_id((*neuron)->neuron_id,0);
	
	if((*neuron)->form)
	{
		XtUnmanageChild((*neuron)->form);
		XtDestroyWidget((*neuron)->form);
		(*neuron)->form = NULL;
	}
	if((*neuron)->bgnd)
	{
		XFreePixmap(the_display,(*neuron)->bgnd);
		(*neuron)->bgnd = 0;
	}
	free(*neuron);
	*neuron = NULL;

}

#endif
