/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	THIS C SOURCE CODE FILE CONTAINS FUNCTIONS FOR MANAGING THE NUCLEUS
	EDITOR, AND OTHER NUCLEUS SPECIFIC FUNCTIONS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifndef _NUCLEUS_FUNCTIONS_
#define _NUCLEUS_FUNCTIONS_

#define _DEBUG_ 0

/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* application globals include files */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif




/****************************************************************************
	Function for redisplaying the current selected nucleus that is edited
	in the nucleus editor
*****************************************************************************/
int redisplay_nucleus(void)
{
	short width,height;

	wdebug = (Widget)widget_array[ID_DrawingA_nucleus_view];
		
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_Nucleus]),
			working_cursor);
	
	if(!nucleus_pixmap)
	{			
		nucleus_pixmap = XCreatePixmap(the_display,
			XtWindow(widget_array[ID_DrawingA_nucleus_view]),
			AREA_SIZE,AREA_SIZE,
			DefaultDepth(the_display,DefaultScreen (the_display)));
		
		XSetForeground(the_display,the_GC,color_view[which_nucleus_view()]);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display,nucleus_pixmap,the_GC,0,0,AREA_SIZE,AREA_SIZE);
		
		XtVaSetValues(widget_array[ID_DrawingA_nucleus_view],
			XmNbackgroundPixmap,nucleus_pixmap,
			XmNresizePolicy,XmRESIZE_NONE,NULL);
		
	
	
	}
	else
	{
					
		XSetForeground(the_display,the_GC,color_view[which_nucleus_view()]);
		XSetBackground(the_display,the_GC,black);
		XFillRectangle(the_display,nucleus_pixmap,the_GC,0,0,AREA_SIZE,AREA_SIZE);
	
	}	
	
	
	
	draw_nucleus_axes();
	
	
	draw_neurons();
	
	
	draw_neuron_links();


	XtVaSetValues(widget_array[ID_DrawingA_nucleus_view],XmNwidth,
				AREA_SIZE,XmNheight,AREA_SIZE,XmNresizePolicy,XmRESIZE_NONE,NULL);
	

	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));

	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_Nucleus]),
			NULL);
	
	return 0;
}

/*****************************************************************************
	Function that draws the neurons in the current nucleus area
******************************************************************************/
int draw_neurons(void)
{
	
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int counter;
	char buffer[256];
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in draw_neurons\n");
		exit(-1);
	}
	
	neuron_array = (NEURON **)calloc(1,current_nucleus->nb_neurons*sizeof(NEURON *));
	counter = 0;
	
	cluster_array = current_nucleus->cluster_array;
	for(i=current_nucleus->nb_clusters-1 ; i>=0 ; i--)
	{
		cluster_tmp = cluster_array[i];
		for(j=cluster_tmp->nb_neurons-1 ; j>=0 ; j--)
		{
			neuron_array[counter++] = cluster_array[i]->neuron_array[j];
			    
		}
	}		
			
	sort_neurons_by_depth(neuron_array);
	
	for(i=0 ; i<current_nucleus->nb_neurons ; i++)
	{
		
		neuron_tmp = neuron_array[i];
		if(!neuron_tmp->form)
			continue;
		XtVaSetValues(neuron_tmp->form,XmNbackgroundPixmap,None,NULL);
		strcpy(buffer,XtName(neuron_tmp->form));
		XtDestroyWidget(neuron_tmp->form);
		
		neuron_tmp->form = XtVaCreateWidget (buffer,
				    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_nucleus_view],
				    XmNwidth, NEURON_WIDTH,
				    XmNheight, NEURON_HEIGHT,
				    XmNborderWidth, 0,
				    XmNrecomputeSize, False,
				    NULL);
		XtVaSetValues(neuron_tmp->form,XmNbackgroundPixmap,neuron_tmp->bgnd,NULL);
				    
		XtAddEventHandler(neuron_tmp->form,ButtonPressMask,False,
			(XtEventHandler)select_neuron,neuron_tmp->neuron_id);
		XtAddEventHandler(neuron_tmp->form,ButtonReleaseMask,False,
			(XtEventHandler)release_neuron,neuron_tmp->neuron_id);
		XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
			(XtEventHandler)chplace_neuron,neuron_tmp->neuron_id);
		XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
			(XtEventHandler)info_neuron,neuron_tmp->neuron_id);	
		XtAddEventHandler(neuron_tmp->form,EnterWindowMask,False,
			(XtEventHandler)info_neuron,neuron_tmp->neuron_id);
		XtAddEventHandler(neuron_tmp->form,LeaveWindowMask,False,
			(XtEventHandler)clear_label_nucl_info,
			neuron_tmp->neuron_id);		
		
		redraw_one_neuron(neuron_tmp);
		
	}
	free(neuron_array);
	return 0;
}
/*******************************************************************************
	Function for redrawing only one specified neuron 
*******************************************************************************/
int redraw_one_neuron(NEURON *neuron)
{
	char buffer[255];
	float X,Y,Xref,Yref;
	int pos_x,pos_y;
	int view = which_nucleus_view();
	short width,height;
	

	if(!neuron->form)
		return -1;

	if(XtIsManaged(neuron->form))
		XtUnmanageChild(neuron->form);

	switch(view)
	{
		case 0:
			X = neuron->anatomy.L;
			Y = neuron->anatomy.H;
			Xref = current_nucleus->L;
			Yref = current_nucleus->H;
			break;

		case 1:
			X = neuron->anatomy.L;
			Y = neuron->anatomy.AP;
			Xref = current_nucleus->L;
			Yref = current_nucleus->AP;
			break;

		case 2:
			X = neuron->anatomy.AP;
			Y = neuron->anatomy.H;
			Xref = current_nucleus->AP;
			Yref = current_nucleus->H;
			break;

		default:
			break;
	}


	pos_x =	(int)(X*nucleus_scale[view] - Xref*nucleus_scale[view])
		+ current_nucleus->origin_x[view] - NEURON_WIDTH/2;
	pos_y = (int)(Yref*nucleus_scale[view] - Y*nucleus_scale[view])
		+ current_nucleus->origin_y[view] - NEURON_HEIGHT/2;
	

	if((pos_x <= 0) || (pos_x >= AREA_SIZE) || (pos_y <= 0) ||(pos_y >= AREA_SIZE))
		return -1; 

	XtVaSetValues(neuron->form,XmNx,pos_x,XmNy,pos_y,NULL);


	XtManageChild(neuron->form);



	sprintf(buffer,"%d",neuron->neuron_id);
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetLineAttributes(the_display,the_GC,2,LineSolid,
		CapRound,JoinRound);



	XDrawArc(the_display,neuron->bgnd,
		the_GC,1,1,NEURON_WIDTH-2,NEURON_HEIGHT-2,
		0*64,360*64);
	if(neuron->selected == False)
	{
		XSetForeground(the_display,the_GC,neuron->color);
	}
	else
	{
		XSetForeground(the_display,the_GC,white);
	}	

	XFillArc(the_display,neuron->bgnd,
		the_GC,2,2,NEURON_WIDTH-4,NEURON_HEIGHT-4,
		0*64,360*64);
	XSetForeground(the_display,the_GC,black);	
	XDrawString(the_display,neuron->bgnd,the_GC,NEURON_WIDTH/2-10,
		NEURON_HEIGHT/2+2,buffer,strlen(buffer));

	XClearWindow(the_display,XtWindow(neuron->form));
	
	return 0;
}

/*****************************************************************************
	Function that forces the nucleus scale toggles states
******************************************************************************/
void force_nucleus_scale(int view)
{
	switch(nucleus_scale[view])
	{
		case NUCLEUS_HUGE:
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_huge],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_tiny],False,False);
			break;
		case NUCLEUS_LARGE:
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_large],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_tiny],False,False);
			break;
		case NUCLEUS_MEDIUM:
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_medium],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_tiny],False,False);
			break;
		case NUCLEUS_SMALL:
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_small],True,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_tiny],False,False);
			break;
		case NUCLEUS_TINY:
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_huge],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_large],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_medium],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_small],False,False);
			XmToggleButtonSetState(widget_array[ID_ToggleB_nucl_tiny],True,False);
			break;
		default:
			break;
	}
}
/*****************************************************************************
	Function that redefines the nucleus origin
******************************************************************************/
int redefine_nucleus_origin(void)
{
	XtAddEventHandler(widget_array[ID_DrawingA_nucleus_view],
		PointerMotionMask,FALSE,move_nucleus_origin,
			(XtPointer)(unsigned long)which_nucleus_view()); 
	
	XtAddEventHandler(widget_array[ID_DrawingA_nucleus_view],
		ButtonPressMask,FALSE,set_nucleus_new_origin,
			(XtPointer)(unsigned long)which_nucleus_view());
			
	return 0;
}
/*****************************************************************************
	Function for moving nucleus axes
******************************************************************************/
void move_nucleus_origin(Widget w,int message, XEvent *evt)
{
	int old_x;
	int old_y;
	int posX,posY;
	static int lastX=0;
	static int lastY=0;
	
	old_x = current_nucleus->origin_x[message];
	old_y = current_nucleus->origin_y[message];
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	if((posX == old_x) && (posY == old_y))
		return;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]));
	
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	
	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]),the_GC,
		0,posY,
		current_nucleus->work_width[message],
		posY);
	
	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_nucleus_view]),the_GC,
		posX,0,
		posX,
		current_nucleus->work_height[message]);
		
	lastX = posX;
	lastY = posY;
}

/*****************************************************************************
	Function for setting origin of the nucleus axes
******************************************************************************/
void set_nucleus_new_origin(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	Nucleus_widget *nucleus_tmp;
	int i;
	short width,height;
	
	posX = evt->xbutton.x; 
	posY = evt->xbutton.y;
	
	
	XSetFunction(the_display,the_GC,GXcopy);
		
	current_nucleus->origin_x[message] = posX;
	current_nucleus->origin_y[message] = posY;
	
	redisplay_nucleus();
	
	XtRemoveEventHandler(widget_array[ID_DrawingA_nucleus_view],
		PointerMotionMask,True,(XtEventHandler)move_nucleus_origin,(XtPointer)(unsigned long)message);
	
	XtRemoveEventHandler(widget_array[ID_DrawingA_nucleus_view],
		ButtonPressMask,True,(XtEventHandler)set_nucleus_new_origin,(XtPointer)(unsigned long)message);
		
	
	
}
/*****************************************************************************
	Function that draws the axes in the current view of the current selected
	nucleus.
******************************************************************************/
int	draw_nucleus_axes()
{
	int which_view = which_nucleus_view();
	int i,j,x,y,counter;
	char buffer[255];
	float factor;
	int scale = nucleus_scale[which_view];
	short width,height;
	
	
	switch(scale)
	{
		case NUCLEUS_HUGE:
			factor = 1.0;
		
		case NUCLEUS_LARGE:
			factor = 2.5;
			break;
		
		case NUCLEUS_MEDIUM:
			factor = 5.0;
			break;
			
		case NUCLEUS_SMALL:
			factor = 10.0;
			break;
			
		case NUCLEUS_TINY:
			factor = 10.0;
			break;
			
		default:
			break;
	}
	
	scale = (int)((float)scale/factor);
	
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
	
	XDrawLine(the_display,nucleus_pixmap,the_GC,
		0,current_nucleus->origin_y[which_view],
		AREA_SIZE,
		current_nucleus->origin_y[which_view]);
	
	XDrawLine(the_display,nucleus_pixmap,the_GC,
		current_nucleus->origin_x[which_view],0,
		current_nucleus->origin_x[which_view],
		AREA_SIZE);
		
	
	/* x positifs */	
	x = current_nucleus->origin_x[which_view];
	y = current_nucleus->origin_y[which_view];
	counter = 0;	
	while(x < AREA_SIZE)
	{
		i = x - current_nucleus->origin_x[which_view]
			 - counter*scale;
		if((int)(i/scale) == 1)
		{
			counter++;
			XDrawLine(the_display,nucleus_pixmap,the_GC,
				x,y,x,y-5);
			sprintf(buffer,"+%.1f",(float)counter/factor);
			XDrawString(the_display,nucleus_pixmap,the_GC,
				x-10,y+20,buffer,strlen(buffer));
		}
		x++; 
	}
	/* x negatifs */
	x = current_nucleus->origin_x[which_view];
	y = current_nucleus->origin_y[which_view];
	counter = 0;	
	while(x >= 0)
	{
		i = current_nucleus->origin_x[which_view] - x
			 - counter*scale;
		if((int)(i/scale) == 1)
		{
			counter++;
			XDrawLine(the_display,nucleus_pixmap,the_GC,
				x,y,x,y-5);
			sprintf(buffer,"-%.1f",(float)counter/factor);
			XDrawString(the_display,nucleus_pixmap,the_GC,
				x-10,y+20,buffer,strlen(buffer));
		}
		x--; 
	}
	/* y negatifs */
	x = current_nucleus->origin_x[which_view];
	y = current_nucleus->origin_y[which_view];
	counter = 0;	
	while(y < AREA_SIZE)
	{
		j = y - current_nucleus->origin_y[which_view]
			 - counter*scale;
		if((int)(j/scale) == 1)
		{
			counter++;
			XDrawLine(the_display,nucleus_pixmap,the_GC,
				x,y,x+5,y);
			sprintf(buffer,"-%.1f",(float)counter/factor);
			XDrawString(the_display,nucleus_pixmap,the_GC,
				x-25,y+5,buffer,strlen(buffer));
		}
		y++; 
	}
	/* y positifs */
	x = current_nucleus->origin_x[which_view];
	y = current_nucleus->origin_y[which_view];
	counter = 0;	
	while(y >= 0)
	{
		j = current_nucleus->origin_y[which_view] - y
			 - counter*scale;
		if((int)(j/scale) == 1)
		{
			counter++;
			XDrawLine(the_display,nucleus_pixmap,the_GC,
				x,y,x+5,y);
			sprintf(buffer,"+%.1f",(float)counter/factor);
			XDrawString(the_display,nucleus_pixmap,the_GC,
				x-25,y+5,buffer,strlen(buffer));
		}
		y--; 
	}
	
	return 0;

}


/*****************************************************************************
	Function that draws the neuron links
******************************************************************************/
int draw_neuron_links(void)
{
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	Connection connection;
	int i,j,k;
	
	if(!current_nucleus->nb_links)
		return 0;
	
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_src = current_nucleus->cluster_array[i];
		for(j=0 ; j<cluster_src->nb_neurons ; j++)
		{
			neuron_src = cluster_src->neuron_array[j];
			for(k=0 ; k<neuron_src->connected_neurons ; k++)
			{
				connection = neuron_src->connections[k];
				neuron_tgt = search_neuron(connection.target_id,&cluster_tgt);
				search_cluster(cluster_tgt->cluster_id,&nucleus_tmp);
				if(nucleus_tmp != current_nucleus)
					continue;
				else
					draw_one_connection(&connection,neuron_src->neuron_id,
						neuron_tgt->neuron_id);
			}
		}
		
		
	}
	return 0;
}
/*******************************************************************************
	Function for drawing only one connection in a nucleus
*******************************************************************************/
void draw_one_connection(Connection *connection,int source_id,int target_id)
{
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	CLUSTER_WIDGET *cluster_tmp;
	int view = which_nucleus_view();
	float X1,X2,Y1,Y2,Xref,Yref;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;
	double x_mid,y_mid,d,dmin;
	
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current_nucleus is NULL in draw_one_connection!!!!\n");
		exit(-1);
	}
	
	neuron_src = search_neuron(source_id,&cluster_tmp);
	neuron_tgt = search_neuron(target_id,&cluster_tmp);
	if(neuron_src == neuron_tgt)
	{
		if(connection->synaptic_knobs)
			color = eval_color(connection->synaptic_knobs);
		else		
			color = grey;
		neuron_src->color = color;
		redraw_one_neuron(neuron_src);

	}
	else
	{
		switch(view)
		{
			case 0:
				X1 = neuron_src->anatomy.L;
				Y1 = neuron_src->anatomy.H;
				X2 = neuron_tgt->anatomy.L;
				Y2 = neuron_tgt->anatomy.H;
				Xref = current_nucleus->L;
				Yref = current_nucleus->H;
				break;

			case 1:
				X1 = neuron_src->anatomy.L;
				Y1 = neuron_src->anatomy.AP;
				X2 = neuron_tgt->anatomy.L;
				Y2 = neuron_tgt->anatomy.AP;
				Xref = current_nucleus->L;
				Yref = current_nucleus->AP;
				break;

			case 2:
				X1 = neuron_src->anatomy.AP;
				Y1 = neuron_src->anatomy.H;
				X2 = neuron_tgt->anatomy.AP;
				Y2 = neuron_tgt->anatomy.H;
				Xref = current_nucleus->AP;
				Yref = current_nucleus->H;
				break;

			default:

				break;
		}
		posX1 =	(int)(X1*nucleus_scale[view] - Xref*nucleus_scale[view]) 
			+ current_nucleus->origin_x[view];
		posX2 =	(int)(X2*nucleus_scale[view] - Xref*nucleus_scale[view]) 
			+ current_nucleus->origin_x[view];	
		posY1 =	(int)(-Y1*nucleus_scale[view] + Yref*nucleus_scale[view]) 
			+ current_nucleus->origin_y[view];
		posY2 =	(int)(-Y2*nucleus_scale[view] + Yref*nucleus_scale[view]) 
			+ current_nucleus->origin_y[view];
		
		
		if(connection->synaptic_knobs > 0)
		{
			color = eval_color(connection->synaptic_knobs);
			over_x = over_y = +3;
		}
		else if	(connection->synaptic_knobs < 0)	
		{
			color = eval_color(connection->synaptic_knobs);
			over_x = over_y = -3;
		}
		else		
		{
			color = grey;
			return;
		}	
		XSetForeground(the_display,the_GC,color);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawLine(the_display, nucleus_pixmap,the_GC,
			posX1+over_x,posY1+over_y,posX2+over_x,posY2+over_y);

		if((posX2<0)
			 || (posX2>=AREA_SIZE)
			 ||	(posY2<0)
			 ||	(posY2>=AREA_SIZE) )
			 return;
		
		
		

		dmin = (double)(NEURON_WIDTH*NEURON_WIDTH+NEURON_HEIGHT*NEURON_HEIGHT)
				/4.0;
		d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
		x_mid = posX1+over_x;
		y_mid = posY1+over_y;
		while(d>1.1*dmin)
		{
			x_mid = (9*x_mid + posX2+over_x)/10;
			y_mid = (9*y_mid + posY2+over_y)/10;
			d = (double)(posX2-x_mid)*(posX2-x_mid)
				+(double)(posY2-y_mid)*(posY2-y_mid);
		}

		Xmid = (int)x_mid - NEURON_WIDTH/4;
		Ymid = (int)y_mid - NEURON_HEIGHT/4;

		XDrawArc(the_display,nucleus_pixmap,
			the_GC,Xmid ,
			Ymid,
			12,12,
			0*64,360*64);
		XFillArc(the_display,nucleus_pixmap,
			the_GC,Xmid ,
			Ymid,
			12,12,
			0*64,360*64);

		

	}
	

}

/*****************************************************************************
	Function that return the number of the current nucleus view
******************************************************************************/
int which_nucleus_view(void)
{
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_nucl_LHview]))
			return 0;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_nucl_LAPview]))
			return 1;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_nucl_APHview]))
			return 2;
}

/***************************************************************************
		Function providing the help on line for the nucleus editor
****************************************************************************/

void fill_label_nucl_info(Widget w,int message, XEvent *evt)
{
XmString string;
Widget label_nucl_info = widget_array[ID_Label_nucleus_information];
static int lastX=0;
static int lastY=0;
static lastView=0;
int posX,posY,which_view;
float X,Y;
char x_axe[3][3]={"L","L","AP"};
char y_axe[3][3]={"H","AP","H"};
char view_name[3][18]={"<1> Frontal","<2> Horizontal","<3> Sagittal"};
char buffer[255];


switch(message)
	{
	case ID_FormD_Nucleus:
		string = XmStringCreateLocalized(
		"Editing selected nucleus...");
		XtVaSetValues(widget_array[ID_Label_net_information],XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_nucleus_file_new:
		string = XmStringCreateLocalized(
		"Clear current nucleus: erase composition and intra links...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	case ID_PushB_nucleus_plot:
		string = XmStringCreateLocalized(
		"Plot the current nucleus view in Postscript format...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_nucleus_file_open:
		string = XmStringCreateLocalized(
		"Open an old saved nucleus configuration file...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_nucleus_file_save:
		string = XmStringCreateLocalized(
		"Save current nucleus in configuration file...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_nucleus_file_save_as:
		string = XmStringCreateLocalized(
		"Save current nucleus in an other configuration file...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_nucleus_exit:
		string = XmStringCreateLocalized(
		"Quit without taking modifications into account...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_composition_add:
		string = XmStringCreateLocalized(
		"Add a cluster in the generic composition...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_composition_remove:
		string = XmStringCreateLocalized(
		"Remove selected cluster from the generic composition...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_composition_modify:
		string = XmStringCreateLocalized(
		"Modify selected cluster...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_generic_place_neurons:
		string = XmStringCreateLocalized(
		"Update generic placement calculation of all neurons...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_generic_create_il:
		string = XmStringCreateLocalized(
		"Create nucleus' generic intra link...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_generic_remove_il:
		string = XmStringCreateLocalized(
		"Remove nucleus' generic intra link...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_generic_modify_il:
		string = XmStringCreateLocalized(
		"Modify nucleus' generic intra link...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_neuron_in_nucleus_new:
		string = XmStringCreateLocalized(
		"Create a NEURON...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	
	case ID_PushB_neuron_in_nucleus_dele:
		string = XmStringCreateLocalized(
		"Delete the last SELECTED neuron...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	
	case ID_PushB_neuron_in_nucleus_modi:
		string = XmStringCreateLocalized(
		"Modify the last SELECTED neuron...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_link_in_nucleus_new:
		string = XmStringCreateLocalized(
		"Create a SINGLE link between two selected neurons...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_link_in_nucleus_delete:
		string = XmStringCreateLocalized(
		"Delete a SINGLE link between two selected neurons...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_link_in_nucleus_modify:
		string = XmStringCreateLocalized(
		"Modify a SINGLE link between two selected neurons...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_nucleus_para_edit:
		string = XmStringCreateLocalized(
		"Edit the nucleus parameters...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_options_nucleus_prefer:
		string = XmStringCreateLocalized(
		"Set your preferences: accelerators, colors...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_nucleus_help_context:
		string = XmStringCreateLocalized(
		"If you need some help on a particular context...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushB_nucleus_help_window:
		string = XmStringCreateLocalized(
		"You don't know well the nucleus editor ?");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushBG_nucleus_ok:
		string = XmStringCreateLocalized(
		"Update nucleus modifications and leave the nucleus editor...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushBG_nucleus_apply:
		string = XmStringCreateLocalized(
		"Update nucleus modifications...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushBG_nucleus_cancel:
		string = XmStringCreateLocalized(
		"Do not update nucleus modifications and leave...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_PushBG_nucleus_help:
		string = XmStringCreateLocalized(
		"You don't know well the nucleus editor ?");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_DrawingA_nucleus_view:
	
		posX = evt->xbutton.x; 
		posY = evt->xbutton.y;
		which_view = which_nucleus_view();
		if((lastX == posX)&&(lastY == posY)&&(which_view == lastView))
			return;
		lastX = posX;
		lastY = posY;
		lastView = which_view;
		X = (float)(posX - current_nucleus->origin_x[which_view])/nucleus_scale[which_view];
		Y = (float)(current_nucleus->origin_y[which_view] - posY)/nucleus_scale[which_view];
		sprintf(buffer,"Nucleus %s View: (%s;%s) = (%.3f;%.3f) [in mm]",
			view_name[which_view],x_axe[which_view],
			y_axe[which_view],X,Y);
		
		string = XmStringCreateLocalized(buffer);
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	case ID_List_composition:
		string = XmStringCreateLocalized(
		"Select a component of the list to modify, to remove or add a new one");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
		
	case ID_PushB_cluster_other_file:
		string = XmStringCreateLocalized(
		"Select the neuron file type");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
			
	defaults:
		string = XmStringCreateLocalized("Sorry, no help on line available for this...");
		XtVaSetValues(label_nucl_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		break;
	
	}

}
/*****************************************************************************
	Function for clearing the help on line label in the nucleus editor 
*****************************************************************************/
void clear_label_nucl_info(Widget w,int message, XEvent *evt)
{
XmString string;
Widget label_net_info = widget_array[ID_Label_nucleus_information];

string = XmStringCreateLocalized("                                    ");
XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
XmStringFree(string);
		

}
/*****************************************************************************
	Function for deleting a nucleus
******************************************************************************/
void delete_nucleus()
{
	Nucleus_widget *nucleus_tmp;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	char buffer[255];
	int pos;
	
	if(creating)
		nucleus_tmp = current_nucleus;
	else if((!current_nucleus)|| (current_nucleus->nb_neurons==1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons==1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which nucleus? No nucleus was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;
	
	sprintf(buffer,
"Do you really want to delete the nucleus\n that's named %s with the Id=%d ?",
	nucleus_tmp->nucleus_name,nucleus_tmp->nucleus_id);
		
	if(display_question(widget_array[ID_MainW_network],
		buffer,
		"Confirmation needed...") == NO_ANSWER)
		return;
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
					working_cursor);
		
	pos = nucleus_tmp->number;
	
	if(!creating)
		select_nucleus(nucleus_tmp->form,nucleus_tmp->nucleus_id,NULL);
	
	if(XtIsManaged(nucleus_tmp->form))
			XtUnmanageChild(nucleus_tmp->form);
			
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
	
	if(pos == current_network->nb_nucleus)
	{
		current_network->nb_neurons -= 
			current_network->nucleus_array[pos-1]->nb_neurons;
		destroy_nucleus(pos);
		current_network->nb_nucleus--;
		if(!current_network->nb_nucleus)
		{
			free(current_network->nucleus_array);
			current_network->nucleus_array = NULL;
			enable_net_menu(False);
			
		}
		
		return;
	}
	else
	{
		current_network->nb_neurons -= 
			current_network->nucleus_array[pos-1]->nb_neurons;
		
		destroy_nucleus(pos);
		
		current_network->
			nucleus_array[current_network->nb_nucleus-1]->number = pos;
	
		current_network->nucleus_array[pos-1] = 
			current_network->nucleus_array[current_network->nb_nucleus-1];
		
		current_network->nucleus_array[current_network->nb_nucleus-1] = NULL;
	
		current_network->nb_nucleus--;
		
		
	}
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
					NULL);
	
}

/******************************************************************************
	Function for copying a nucleus	-> unused
******************************************************************************/
void copy_nucleus()
{
	Nucleus_widget *nucleus_tmp;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	char buffer[255];
	int pos;
	
	if((!current_nucleus)|| (current_nucleus->nb_neurons==1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons==1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which nucleus? No nucleus was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;
}

/******************************************************************************
	Function for modifying a nucleus	
******************************************************************************/
void modify_nucleus()
{
	Nucleus_widget *nucleus_tmp;
	XmString string;
	Widget label_net_info = widget_array[ID_Label_net_information];
	char buffer[255];
	int pos;
	
	if((!current_nucleus)|| (current_nucleus->nb_neurons==1)
		|| (current_nucleus->selected == False))
	{
		
		if((!previous_selected_nucleus)
			|| (previous_selected_nucleus->nb_neurons==1)
			|| (previous_selected_nucleus->selected == False))
		{
			display_error(widget_array[ID_DrawingA_net],
				"Which nucleus? No nucleus was selected...","Can't do that !");
			return;
		}
		else
			nucleus_tmp = previous_selected_nucleus;
	}	
	else
		nucleus_tmp = current_nucleus;
	
	update_center();
	update_radius();
	
	sprintf(buffer,"%d neurons",current_nucleus->nb_neurons);
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(widget_array[ID_LabelG_nbneuron_in_nucleus],
		XmNlabelString,string,NULL);
	XmStringFree(string);

	XmTextRemove(widget_array[ID_TextF_AP_nucleusC]);
	sprintf(buffer,"%.2f",current_nucleus->AP);
	XmTextReplace(widget_array[ID_TextF_AP_nucleusC],0,
		XmTextGetLastPosition(widget_array[ID_TextF_AP_nucleusC]),
		buffer);
	XmTextRemove(widget_array[ID_TextF_AP_nucleusR]);
	sprintf(buffer,"%.2f",current_nucleus->delta_AP);
	XmTextReplace(widget_array[ID_TextF_AP_nucleusR],0,
		XmTextGetLastPosition(widget_array[ID_TextF_AP_nucleusR]),
		buffer);

	XmTextRemove(widget_array[ID_TextF_H_nucleusC]);
	sprintf(buffer,"%.2f",current_nucleus->H);
	XmTextReplace(widget_array[ID_TextF_H_nucleusC],0,
		XmTextGetLastPosition(widget_array[ID_TextF_H_nucleusC]),
		buffer);
	XmTextRemove(widget_array[ID_TextF_H_nucleusR]);
	sprintf(buffer,"%.2f",current_nucleus->delta_H);
	XmTextReplace(widget_array[ID_TextF_H_nucleusR],0,
		XmTextGetLastPosition(widget_array[ID_TextF_H_nucleusR]),
		buffer);	

	XmTextRemove(widget_array[ID_TextF_L_nucleusC]);
	sprintf(buffer,"%.2f",current_nucleus->L);
	XmTextReplace(widget_array[ID_TextF_L_nucleusC],0,
		XmTextGetLastPosition(widget_array[ID_TextF_L_nucleusC]),
		buffer);
	XmTextRemove(widget_array[ID_TextF_L_nucleusR]);
	sprintf(buffer,"%.2f",current_nucleus->delta_L);
	XmTextReplace(widget_array[ID_TextF_L_nucleusR],0,
		XmTextGetLastPosition(widget_array[ID_TextF_L_nucleusR]),
		buffer);	

	XmTextReplace(widget_array[ID_TextF_nucleus_name],0,
		XmTextGetLastPosition(widget_array[ID_TextF_nucleus_name]),
		current_nucleus->nucleus_name);

	XtManageChild(widget_array[ID_FormD_nucleus_parameters]);
	XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_nucleus_parameters])),
						the_colormap);
	
}
/*****************************************************************************
	Function for applying generic components from a nucleus composition
******************************************************************************/
int apply_cluster(void)
{
	char *name;
	char *file;
	int nb_neurons;
	char tmp[255];
	XmString string;
	char *pt;
	int i;
	
	name = XmTextGetString(widget_array[ID_Text_cluster_name]);
	if(strlen(name)>19)
		name[19] = '\0';
		
	file = XmTextFieldGetString(widget_array[ID_TextF_cluster_neuron_file]);
	if(!strstr(file,".G_unit") && !strstr(file,".P_unit") && 
		!strstr(file,".tms"))
	{
		display_error(widget_array[ID_FormD_nucleus_composition],
			"Sorry, can't take your neuron file choice into account...","Bad Neuron File !");
		return -1;
	} 
	
	pt = XmTextGetString(widget_array[ID_Text_nbneuron_in_cluster]);
	nb_neurons = atoi(pt);
	
	if(nb_neurons < 2)
	{
		display_error(widget_array[ID_FormD_nucleus_composition],
			"Sorry, Minimum number of neuron by cluster, is 2...","Invalid Neuron Number!");
		return -1;
	}
	
	if(nb_neurons > 1000)
	{
		display_error(widget_array[ID_FormD_nucleus_composition],
			"Sorry, Maximum number of neuron by cluster, is 1000...","Invalid Neuron Number!");
		return -1;
	}
	
	/* so far we can say this cluster is ok. let's fill composition list */
	
	sprintf(tmp,"generic> 	NAME:%s		TYPE:%s 	SIZE:%d	",name,file,nb_neurons);
	string = XmStringCreateLocalized(tmp);
	if(!XmListItemExists(widget_array[ID_List_composition],
					string))
	{
		if(add_new_cluster_in_array(name,file,nb_neurons))
			return -1;
		XmListAddItem(widget_array[ID_List_composition],string,1);
		XmListSelectPos(widget_array[ID_List_composition],1,True);
	}
	XmStringFree(string);
	XtFree(name);
	XtFree(file);
	XtFree(pt);
	
	
		
	return 0;
}

/*****************************************************************************
	Function for applying nucleus parameters changes
******************************************************************************/
int apply_nucleus(void)
{
	char *buffer;
	char tmp[255];
	XmString string;
	char *pt;
	float old_l,old_h,old_ap;
	float dim_max = (float)AREA_SIZE / (float)(2*_HUGE);
	
	old_l = current_nucleus->L;
	old_h = current_nucleus->H;
	old_ap = current_nucleus->AP;
	
	buffer = XmTextGetString(widget_array[ID_TextF_nucleus_name]);
	if(nucleus_name_already_exists(buffer))
	{
		display_message(widget_array[ID_MainW_network],
		"Sorry, this name already exists...",
	"Invalid Nucleus Name !");
		XtFree(buffer);
		return -1;
	}
	
	if(strlen(buffer)>30)
		buffer[30] = '\0';
	strcpy(current_nucleus->nucleus_name,buffer);
	XtFree(buffer);
	
	sprintf(tmp,"Nucleus Editor : %s",current_nucleus->nucleus_name);
	XtVaSetValues(XtParent(widget_array[ID_FormD_Nucleus]),XtNtitle,tmp,NULL);
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_generic]))
		current_nucleus->specific = False;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_specific]))
		current_nucleus->specific = True;
		
	buffer = XmTextGetString(widget_array[ID_TextF_L_nucleusC]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the L center coordinate is too large...","Bad Dimension !");
		return -1;
	}
	
	current_nucleus->L = atof(buffer);
	XtFree(buffer);

	buffer = XmTextGetString(widget_array[ID_TextF_H_nucleusC]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the H center coordinate is too large...","Bad Dimension !");
		return -1;
	}
	current_nucleus->H = atof(buffer);
	XtFree(buffer);	

	buffer = XmTextGetString(widget_array[ID_TextF_AP_nucleusC]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the AP center coordinate is too large...","Bad Dimension !");
		return -1;
	}
	current_nucleus->AP = atof(buffer);
	XtFree(buffer);

	buffer = XmTextGetString(widget_array[ID_TextF_L_nucleusR]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the L radius value is too high...","Bad Dimension !");
		return -1;
	}
	
	current_nucleus->delta_L = atof(buffer);
	XtFree(buffer);

	buffer = XmTextGetString(widget_array[ID_TextF_H_nucleusR]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the H radius value is too high...","Bad Dimension !");
		return -1;
	}
	current_nucleus->delta_H = atof(buffer);
	XtFree(buffer);
	
	buffer = XmTextGetString(widget_array[ID_TextF_AP_nucleusR]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_FormD_nucleus_parameters],
			"Sorry, the AP radius value is too high...","Bad Dimension !");
		return -1;
	}
	current_nucleus->delta_AP = atof(buffer);
	XtFree(buffer);
	
	current_nucleus->posX[0] = (int)(network_scale[0]* current_nucleus->L)
					+ current_network->origin_x[0] - NUCLEUS_WIDTH/2;
	current_nucleus->posX[1] = 	(int)(network_scale[1]* current_nucleus->L)
					+ current_network->origin_x[1] - NUCLEUS_WIDTH/2;
	current_nucleus->posY[0] = (int)(-network_scale[0]* current_nucleus->H)
					+ current_network->origin_y[0] - NUCLEUS_HEIGHT/2;
	current_nucleus->posY[2] = (int)(-network_scale[2]* current_nucleus->H)
					+ current_network->origin_y[2] - NUCLEUS_HEIGHT/2;
	current_nucleus->posY[1] = (int)(-network_scale[1]* current_nucleus->AP)
					+ current_network->origin_y[1] - NUCLEUS_HEIGHT/2;
	current_nucleus->posX[2] = (int)(network_scale[2]* current_nucleus->AP)
					+ current_network->origin_x[2] - NUCLEUS_WIDTH/2;
	
	correct_neurons_position(current_nucleus,old_l,old_h,old_ap);
	
	deselect_all_nucleus();
	
	select_nucleus(current_nucleus->form,current_nucleus->nucleus_id,NULL);
	
		
	
	return 0;
}

/*****************************************************************************
	Function for destroying the position specified nucleus
******************************************************************************/
void destroy_nucleus(int pos)
{
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET **cluster_array,*cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	LINK_PARAMETERS **link_array,*link_tmp;
	int i,j,k;

	nucleus_tmp = current_network->nucleus_array[pos-1];
	if(nucleus_tmp)
	{
		notify_nucleus_deletion(nucleus_tmp);
		
		if(nucleus_tmp->form)
		{
			XtDestroyWidget(nucleus_tmp->form);
			if(nucleus_tmp->bgnd)
				XFreePixmap(the_display,nucleus_tmp->bgnd);
		}
		cluster_array = nucleus_tmp->cluster_array;
		if(cluster_array)
		{
			for(j=0 ; j<nucleus_tmp->nb_clusters ; j++)
			{
				cluster_tmp = cluster_array[j];
				if(cluster_tmp)
				{
					if(cluster_tmp->neuron_file)
						free(cluster_tmp->neuron_file);
					neuron_array = cluster_tmp->neuron_array;
					if(neuron_array)
					{
						for(k=0 ; k<cluster_tmp->nb_neurons ; k++)
						{
							neuron_tmp = neuron_array[k];
							if(neuron_tmp->form)
							{
								XtDestroyWidget(neuron_tmp->form);
								if(neuron_tmp->bgnd)
									XFreePixmap(the_display,
										neuron_tmp->bgnd);
							}
							if(neuron_tmp->connections)
								free(neuron_tmp->connections);
							
							manage_neuron_id(neuron_tmp->neuron_id,0);
							
							free(neuron_tmp);
						}
						free(neuron_array);
					}
					manage_cluster_id(cluster_tmp->cluster_id,0);
					
					free(cluster_tmp);
				}

			}
			free(cluster_array);
		}
		link_array = nucleus_tmp->link_array;
		if(link_array)
		{
			for(j=0 ; j<nucleus_tmp->nb_links ; j++)
			{
				link_tmp = link_array[j];
				if(link_tmp)
				{

					free(link_tmp);
				}
			}
			free(link_array);
		}
		
		manage_nucleus_id(current_network->nucleus_array[pos-1]->nucleus_id,0);
		
		free(current_network->nucleus_array[pos-1]);
	}
	 current_network->nucleus_array[pos-1] = NULL;
}

/*****************************************************************************
	Function for selecting a nucleus
******************************************************************************/
void select_nucleus(Widget w,int message, XEvent *evt)
{
	Nucleus_widget *nucleus_tmp;
	int type;
	char buffer[255];
	static int nb_selected=0;
	Widget label_net_info = widget_array[ID_Label_net_information];
	XmString string;
	
	if(XtIsManaged(widget_array[ID_FormD_Nucleus]))
		return;
	if(XtIsManaged(widget_array[ID_FormD_link_simple]))
		return;
	if(XtIsManaged(widget_array[ID_FormD_generic_link_edit]))
		return;
			
	nucleus_tmp = search_nucleus(message);
	if(!nucleus_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus not found in select_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	
	if((nb_selected == 2) && (nucleus_tmp->selected == False))
	{
		string = XmStringCreateLocalized(
			"Please note: only two selections are permitted...");
		XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
		XmStringFree(string);
		 
		return;
	}
	
	if(nucleus_tmp->nb_neurons == 1)
	{
		type = _NEURON_;
	}
	else
		type = _NUCLEUS_;
	

	if(type == _NUCLEUS_)
	{
		sprintf(buffer,"%d",nucleus_tmp->nucleus_id);
		XSetForeground(the_display,the_GC,black);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawRectangle(the_display,nucleus_tmp->bgnd,
			the_GC,1,1,NUCLEUS_WIDTH-2,NUCLEUS_HEIGHT-2);
		
		if(nucleus_tmp->selected == True)
		{
			nucleus_tmp->selected = False;
			nb_selected--;
			if(nb_selected == 1)
			{
				current_nucleus = get_first_selected_nucleus();
				previous_selected_nucleus = current_nucleus;
			}
			else if(nb_selected == 0)
			{
				current_nucleus = NULL;
				previous_selected_nucleus = NULL;
			}
			XSetForeground(the_display,the_GC,nucleus_tmp->color);
		}
		else
		{
			nucleus_tmp->selected = True;
			nb_selected++;
			if(nb_selected == 1)
			{
				previous_selected_nucleus = nucleus_tmp;
				current_nucleus = nucleus_tmp;
			}
			else if(nb_selected == 2)
			{
				current_nucleus = nucleus_tmp;
			}
			XSetForeground(the_display,the_GC,white);	
		}
		
		XFillRectangle(the_display,nucleus_tmp->bgnd,
			the_GC,2,2,NUCLEUS_WIDTH-4,NUCLEUS_HEIGHT-4);
		XSetForeground(the_display,the_GC,black);	
		XDrawString(the_display,nucleus_tmp->bgnd,the_GC,NUCLEUS_WIDTH/2-2,
			NUCLEUS_HEIGHT/2-10,buffer,strlen(buffer));
		sprintf(buffer,"%d neurons",nucleus_tmp->nb_neurons);
		XDrawString(the_display,nucleus_tmp->bgnd,the_GC,2,
			NUCLEUS_HEIGHT/2+10,buffer,strlen(buffer));

		XClearWindow(the_display,XtWindow(nucleus_tmp->form));		
	}
	else
	{
		sprintf(buffer,"%d",nucleus_tmp->nucleus_id);
		XSetForeground(the_display,the_GC,black);
		XSetBackground(the_display,the_GC,white);
		XSetLineAttributes(the_display,the_GC,2,LineSolid,
			CapRound,JoinRound);
		XDrawArc(the_display,nucleus_tmp->bgnd,
			the_GC,1,1,NEURON_WIDTH-2,NEURON_HEIGHT-2,
			0*64,360*64);
		
		if(nucleus_tmp->selected == True)
		{
			nucleus_tmp->selected = False;
			nb_selected--;
			if(nb_selected == 1)
			{
				current_nucleus = get_first_selected_nucleus();
				previous_selected_nucleus = current_nucleus;
			}	
			else if(nb_selected == 0)
			{
				previous_selected_nucleus = NULL;
				current_nucleus = NULL;
			}
			XSetForeground(the_display,the_GC,nucleus_tmp->color);
		}
		else
		{
			nucleus_tmp->selected = True;
			nb_selected++;
			if(nb_selected == 1)
			{
				previous_selected_nucleus = nucleus_tmp;
				current_nucleus = nucleus_tmp;
			}
			else if(nb_selected == 2)
			{
				current_nucleus = nucleus_tmp;
			}
			XSetForeground(the_display,the_GC,white);
		}
			
		XFillArc(the_display,nucleus_tmp->bgnd,
			the_GC,2,2,NEURON_WIDTH-4,NEURON_HEIGHT-4,
			0*64,360*64);
		XSetForeground(the_display,the_GC,black);	
		XDrawString(the_display,nucleus_tmp->bgnd,the_GC,NEURON_WIDTH/2-1,
			NEURON_HEIGHT/2,buffer,strlen(buffer));

		XClearWindow(the_display,XtWindow(nucleus_tmp->form));
			
	}
}

/******************************************************************************
	Function for releasing and redrawing the nucleus after one move
*******************************************************************************/
void release_nucleus(Widget w,int message, XEvent *evt)
{
	Nucleus_widget *nucleus_tmp;
	short width,height,type;
	int posX,posY;
	float old_l,old_h,old_ap;
	
	if(evt->type != ButtonRelease)
		return;
	
	nucleus_tmp = search_nucleus(message);
	if(!nucleus_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus not found in chplace_nucleus ! \n");
#endif		
		exit(-1);	
	}
		
	old_l = nucleus_tmp->L;
	old_h = nucleus_tmp->H;
	old_ap = nucleus_tmp->AP;
	
	if(nucleus_tmp->nb_neurons == 1)
	{
		width = NEURON_WIDTH;
		height = NEURON_HEIGHT;
	}
	else
	{
		width = NUCLEUS_WIDTH;
		height = NUCLEUS_HEIGHT;
	}
	
	posX = evt->xbutton.x + nucleus_tmp->posX[which_network_view()]; 
	posY = evt->xbutton.y + nucleus_tmp->posY[which_network_view()];
	
	
	
	switch(which_network_view())
	{
		case 0:
			
			nucleus_tmp->L = (float)(posX - current_network->origin_x[0])
				/network_scale[0];
			
			nucleus_tmp->H = (float)(current_network->origin_y[0] - posY)
				/network_scale[0];
			break;
			
		case 1:
			
			nucleus_tmp->L = (float)(posX - current_network->origin_x[1])
				/network_scale[1];
			nucleus_tmp->AP = (float)(current_network->origin_y[1] - posY)
				/network_scale[1];
				
			break;
			
		
		case 2:
		
			nucleus_tmp->AP = (float)(posX - current_network->origin_x[2])
				/network_scale[2];
			nucleus_tmp->H = (float)(current_network->origin_y[2] - posY)
				/network_scale[2];
			
			break;	
		
		default:
			break;
	}
	
	if((abs(evt->xbutton.x)>width/2+1) || (abs(evt->xbutton.y)>height/2+1))
	{
		correct_neurons_position(nucleus_tmp,old_l,old_h,old_ap);
		if(XtIsManaged(widget_array[ID_FormD_nucleus_parameters]))
			nucl_activate_CB(w,ID_PushB_nucleus_para_edit,NULL);
		select_nucleus(w,message,evt);
		calculate_positions();
		redisplay_network();
		
	}
	else
		XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
}

/******************************************************************************
	Function for moving a nucleus on the network area
******************************************************************************/
void chplace_nucleus(Widget w,int message, XEvent *evt)
{
	Nucleus_widget *nucleus_tmp;
	int type;
	int posX,posY;
	static lastX=0;
	static lastY=0;
	float old_l,old_h,old_ap;
	
	if((evt->type != MotionNotify) || (!evt->xmotion.state && Button1Mask))
		return;
		
	nucleus_tmp = search_nucleus(message);
	if(!nucleus_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus not found in chplace_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	old_l = nucleus_tmp->L;
	old_h = nucleus_tmp->H;
	old_ap = nucleus_tmp->AP;
		
	
	posX = evt->xbutton.x + nucleus_tmp->posX[which_network_view()]; 
	posY = evt->xbutton.y + nucleus_tmp->posY[which_network_view()];
	
	if((posX==lastX) && (posY==lastY))
		return;
		
	switch(which_network_view())
	{
		case 0:
			
			nucleus_tmp->L = (float)(posX - current_network->origin_x[0])
				/network_scale[0];
			
			nucleus_tmp->H = (float)(current_network->origin_y[0] - posY)
				/network_scale[0];
			break;
			
		case 1:
			
			nucleus_tmp->L = (float)(posX - current_network->origin_x[1])
				/network_scale[1];
			nucleus_tmp->AP = (float)(current_network->origin_y[1] - posY)
				/network_scale[1];
				
			break;
			
		
		case 2:
		
			nucleus_tmp->AP = (float)(posX - current_network->origin_x[2])
				/network_scale[2];
			nucleus_tmp->H = (float)(current_network->origin_y[2] - posY)
				/network_scale[2];
			
			break;	
		
		default:
			break;
	}
		
	if(nucleus_tmp->nb_neurons == 1)
	{
		type = _NEURON_;
	}
	else
		type = _NUCLEUS_;
	
	if(type == _NUCLEUS_)
	{
		XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
		
		XSetFunction(the_display,the_GC,GXcopy);
		XSetBackground(the_display,the_GC,white);
		XSetForeground(the_display,the_GC,black);
		XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
		
		XDrawRectangle(the_display,XtWindow(widget_array[ID_DrawingA_net]),
							the_GC,posX-NUCLEUS_WIDTH/2,posY-NUCLEUS_HEIGHT/2,
							NUCLEUS_WIDTH,NUCLEUS_HEIGHT);
	}
	else
	{
		XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_net]));
		
		XSetFunction(the_display,the_GC,GXcopy);
		XSetBackground(the_display,the_GC,white);
		XSetForeground(the_display,the_GC,black);
		XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
		
		XDrawArc(the_display,XtWindow(widget_array[ID_DrawingA_net]),
							the_GC,posX-NEURON_WIDTH/2,posY-NEURON_HEIGHT/2,
							NEURON_WIDTH,NEURON_HEIGHT,
							0*64,360*64);
	}
	
	lastX = posX;
	lastY = posY;
	
	correct_neurons_position(nucleus_tmp,old_l,old_h,old_ap);
}

/******************************************************************************
	Function for info on a nucleus on the network area
******************************************************************************/
void info_nucleus(Widget w,int message, XEvent *evt)
{
	Widget label_net_info = widget_array[ID_Label_net_information];
	XmString string;
	char buffer[255];
	
	Nucleus_widget *nucleus_tmp;
	
	nucleus_tmp = search_nucleus(message);
	
	if(!nucleus_tmp)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus not found in info_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	sprintf(buffer,
	"Id : %d    Name : %s   at   L=%.3f  ,  H=%.3f  ,  AP=%.3f  %s" ,
		nucleus_tmp->nucleus_id,
		nucleus_tmp->nucleus_name,nucleus_tmp->L,
		nucleus_tmp->H,	nucleus_tmp->AP,
		(nucleus_tmp->selected==True)?"<Selected>":"");
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(label_net_info,XmNlabelString,string,NULL);
	XmStringFree(string);
		
}
/*****************************************************************************
	Function for searching the nucleus that corresponds to the given id
******************************************************************************/
Nucleus_widget *search_nucleus(int id)
{
	Nucleus_widget *nucleus_tmp;
	int i;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_nucleus ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_nucleus ! \n");
#endif		
			exit(-1);
		}
		if(nucleus_tmp->nucleus_id == id)
			return (Nucleus_widget *)nucleus_tmp;	
	}	

	return (Nucleus_widget *)NULL;		
}

/*****************************************************************************
	Function for searching the cluster that corresponds to the given id
******************************************************************************/
CLUSTER_WIDGET *search_cluster(int id,Nucleus_widget **nucleus)
{
	CLUSTER_WIDGET *cluster_tmp;
	Nucleus_widget *nucleus_tmp;
	int i,j;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_nucleus ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_nucleus ! \n");
#endif		
			exit(-1);
		}
		for(j=0 ; j<nucleus_tmp->nb_clusters ;j++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[j];
			if(cluster_tmp->cluster_id == id)
			{
				*nucleus = nucleus_tmp;
				return (CLUSTER_WIDGET *)cluster_tmp;
			}
		}
		
			
	}	

	return (CLUSTER_WIDGET *)NULL;		
}
/*****************************************************************************
	Function that returns the fisrt selected nucleus
******************************************************************************/
Nucleus_widget *get_first_selected_nucleus(void)
{
	int i;
	Nucleus_widget *nucleus_tmp;
	
	if(!current_network)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty network in search_nucleus ! \n");
#endif		
		exit(-1);	
	}
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nEmpty nucleus_array in search_nucleus ! \n");
#endif		
		exit(-1);
	}
	i=0;
	while(i<current_network->nb_nucleus)
	{
		nucleus_tmp = current_network->nucleus_array[i++];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nEmpty nucleus in search_nucleus ! \n");
#endif		
			exit(-1);
		}
		if(nucleus_tmp->selected == True)
			return (Nucleus_widget *)nucleus_tmp;	
	}	

	return (Nucleus_widget *)NULL;		

}

/******************************************************************************
	Function for setting generic or specific menu in the nucleus editor
*******************************************************************************/
void set_generic_menu(int flag)
{
	XtSetSensitive(widget_array[ID_PushB_composition_add],flag);
	XtSetSensitive(widget_array[ID_PushB_composition_remove],flag);
	XtSetSensitive(widget_array[ID_PushB_composition_modify],flag);
	XtSetSensitive(widget_array[ID_PushB_generic_place_neurons],flag);
	XtSetSensitive(widget_array[ID_PushB_generic_create_il],flag);
	XtSetSensitive(widget_array[ID_PushB_generic_remove_il],flag);
	XtSetSensitive(widget_array[ID_PushB_generic_modify_il],flag);
	XtSetSensitive(widget_array[ID_PushB_neuron_in_nucleus_new],!flag);
	XtSetSensitive(widget_array[ID_PushB_neuron_in_nucleus_dele],!flag);
	XtSetSensitive(widget_array[ID_PushB_neuron_in_nucleus_modi],!flag);
	XtSetSensitive(widget_array[ID_PushB_link_in_nucleus_new],!flag);
	XtSetSensitive(widget_array[ID_PushB_link_in_nucleus_delete],!flag);
	XtSetSensitive(widget_array[ID_PushB_link_in_nucleus_modify],!flag);

}

/*******************************************************************************
	Function for adding one new cluster in the current nucleus in both
	specific and generic mode
*******************************************************************************/
int add_new_cluster_in_array(char *name, char *neuron_file, int nb_neurons) 
{
	CLUSTER_WIDGET *cluster_tmp,*cluster;
	CLUSTER_WIDGET **cluster_array
		=	(CLUSTER_WIDGET **)current_nucleus->cluster_array;
	CLUSTER_WIDGET **new_cluster_array;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int i;
	int nf_changed,nb_n_changed,new,old;
	
	if(nb_neurons < 1)
		return -1;
	
	nf_changed = nb_n_changed = False;
	new = True;
	
	/* verify if an other cluster in the nucleus has same name */
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster = current_nucleus->cluster_array[i];
		if(!strcmp(name,cluster->ch_name))
		{
			if(strcmp(neuron_file,cluster->neuron_file))
				nf_changed = True;
			
			if(nb_neurons != cluster->nb_neurons)
				nb_n_changed = True;
			
			new = False;
			old = i;	
			break;	
			
		}
	}
	
	if(!new)
	{
		if(!nf_changed && !nb_n_changed)
			return 0;
			
		if(nf_changed)
		{
			free(cluster->neuron_file);
			cluster->neuron_file = (char *)calloc(1,strlen(neuron_file)+1);
			strcpy(cluster->neuron_file,neuron_file);		
		}
		if(nb_n_changed)
		{
			if(nb_neurons > cluster->nb_neurons)
			{
				neuron_array = (NEURON **)calloc(1,nb_neurons*sizeof(NEURON *));
				for(i=0 ; i<cluster->nb_neurons ; i++)
				{
					neuron_array[i] = cluster->neuron_array[i];
				}
				for(i=cluster->nb_neurons ; i<nb_neurons ; i++)
				{
					neuron_array[i] = (NEURON *)calloc(1,sizeof(NEURON));
					neuron_array[i]->neuron_id = manage_neuron_id(0,0);
				}
				free(cluster->neuron_array);
				current_nucleus->nb_neurons += nb_neurons - cluster->nb_neurons;
				current_network->nb_neurons += nb_neurons - cluster->nb_neurons;
				cluster->neuron_array = neuron_array;
				cluster->nb_neurons = nb_neurons;
				update_numbers();
			}
			else if(nb_neurons < cluster->nb_neurons)
			{
				for(i=nb_neurons ; i<cluster->nb_neurons ; i++)
				{
					delete_single_neuron(
						&(current_nucleus->cluster_array[old]->neuron_array[i]));
				
				}
				current_nucleus->nb_neurons += nb_neurons - cluster->nb_neurons;
				current_network->nb_neurons += nb_neurons - cluster->nb_neurons;
				cluster->nb_neurons = nb_neurons;
				update_numbers();
			}
			generic_place_neurons(False);
		}
		return 0;
		
	}
	
		
	/* first if the nucleus has already some clusters... */
	if(cluster_array)
	{
		new_cluster_array = (CLUSTER_WIDGET **)calloc(1,
			(current_nucleus->nb_clusters + 1)*sizeof(CLUSTER_WIDGET *));
		
		for(i=0 ; i<current_nucleus->nb_clusters ; i++)
		{
			new_cluster_array[i] = (CLUSTER_WIDGET *)calloc(1,
				sizeof(CLUSTER_WIDGET));
			memcpy(new_cluster_array[i],cluster_array[i],sizeof(CLUSTER_WIDGET));
			free(cluster_array[i]);
		}
		
		cluster_tmp = new_cluster_array[current_nucleus->nb_clusters] = 
			(CLUSTER_WIDGET *)calloc(1,sizeof(
			CLUSTER_WIDGET));
		
		current_nucleus->nb_clusters++;
		current_nucleus->nb_neurons += nb_neurons;
		current_network->nb_neurons += nb_neurons;
		
		
		cluster_tmp->cluster_id = current_cluster_id;
		
		
		
		/* now copy the name, neuron file name and the number of neurons for
		this new cluster */
		
		if(strlen(name) > 19)
			name[19] = '\0';
		strcpy(cluster_tmp->ch_name,name);	
		
		cluster_tmp->neuron_file = (char *)calloc(1,strlen(neuron_file)+1);
		strcpy(cluster_tmp->neuron_file,neuron_file);
		
		cluster_tmp->nb_neurons = nb_neurons;

		/* allocate the neurons... */
		neuron_array = cluster_tmp->neuron_array = (NEURON **)calloc(1,
			nb_neurons*sizeof(NEURON *));
					
		for(i=0 ; i<nb_neurons ; i++)
		{
			neuron_tmp = neuron_array[i] = (NEURON *)calloc(1,
				sizeof(NEURON));
			
			neuron_tmp->neuron_id = manage_neuron_id(0,0);
		}
		
		free(cluster_array);
		current_nucleus->cluster_array = new_cluster_array;
		
	}
	/* if new nucleus with no clusters yet... */
	else
	{
		cluster_array = (CLUSTER_WIDGET **)calloc(1,sizeof(CLUSTER_WIDGET *));
		cluster_tmp = cluster_array[0] = (CLUSTER_WIDGET *)calloc(1,sizeof(
			CLUSTER_WIDGET));
		
		current_nucleus->nb_clusters++;
		current_nucleus->nb_neurons += nb_neurons;
		current_network->nb_neurons += nb_neurons;
		
		
		
		cluster_tmp->cluster_id = current_cluster_id;
		
		
		/* now copy the name, neuron file name and the number of neurons for
		this new cluster */
		
		if(strlen(name) > 19)
			name[19] = '\0';
		strcpy(cluster_tmp->ch_name,name);	
		
		cluster_tmp->neuron_file = (char *)calloc(1,strlen(neuron_file)+1);
		strcpy(cluster_tmp->neuron_file,neuron_file);
		
		cluster_tmp->nb_neurons = nb_neurons;

		/* allocate the neurons... */
		neuron_array = cluster_tmp->neuron_array = (NEURON **)calloc(1,
			nb_neurons*sizeof(NEURON *));
					
		for(i=0 ; i<nb_neurons ; i++)
		{
			neuron_tmp = neuron_array[i] = (NEURON *)calloc(1,
				sizeof(NEURON));
			
			neuron_tmp->neuron_id = manage_neuron_id(0,0);
		}
		
		current_nucleus->cluster_array = cluster_array;
	}
	
	update_numbers();
	
	return 0;
}
/*******************************************************************************
	Function for canceling cluster when newly created
*******************************************************************************/
void cancel_cluster(void)
{	
	char buffer[255];
	char *neuron_file;
	char *name;
	char *text;
	int nb_neurons;
	
	name = XmTextGetString(widget_array[ID_Text_cluster_name]);

	neuron_file = XmTextFieldGetString(widget_array[ID_TextF_cluster_neuron_file]);

	text = XmTextGetString(widget_array[ID_Text_nbneuron_in_cluster]);
	nb_neurons = atoi(text);

	manage_cluster_id(current_cluster_id,0);
	
	free(text);
	free(name);
	free(neuron_file);

}

/*******************************************************************************
	Function for extracting cluster characteristics from a component of 
	the list nucleus composition
*******************************************************************************/
void get_cluster_characteristics(char *text,char **name_pt,char **file_pt,
	int *nb_neurons_pt)
{
	char *pt;
	char buffer[255];
	char tmp[255];
	int i;
	
	strcpy(buffer,text);
	
	if(!(pt = strstr(buffer,"NAME:")))
	{
		fprintf(stderr,"invalid cluster text in get_cluster_characteristics!!!\n");
		exit(-1);
	}
	pt+=5;
	i=0;
	while(!isspace(*pt))
		tmp[i++] = *pt++;
	tmp[i] = '\0';
	*name_pt = (char *)calloc(1,strlen(tmp)+1);
	strcpy(*name_pt,tmp);

	if(!(pt = strstr(buffer,"TYPE:")))
	{
		fprintf(stderr,"invalid cluster text in get_cluster_characteristics!!!\n");
		exit(-1);
	}
	pt+=5;
	i=0;
	while(!isspace(*pt))
		tmp[i++] = *pt++;
	tmp[i] = '\0';
	*file_pt = (char *)calloc(1,strlen(tmp)+1);
	strcpy(*file_pt,tmp);
	
	if(!(pt = strstr(buffer,"generic>")))
	{
		*nb_neurons_pt = 1;
	}
	else if(!(pt = strstr(buffer,"SIZE:")))
	{
		fprintf(stderr,"invalid cluster text in get_cluster_characteristics!!!\n");
		exit(-1);
	}
	else
	{
		pt+=5;
		i=0;
		while(!isspace(*pt))
			tmp[i++] = *pt++;
		tmp[i] = '\0';
		*nb_neurons_pt = atoi(tmp);
	}
}

/*******************************************************************************
	Function for deleting a named cluster from the current nucleus
*******************************************************************************/
void delete_cluster(char *name,char *file,int nb_neurons)
{

	int i;
	int pos;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	
	/* find wich cluster to delete first... */
	cluster_tmp = NULL;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = current_nucleus->cluster_array[i];
		if(!strcmp(name,cluster_tmp->ch_name) &&
			!strcmp(file,cluster_tmp->neuron_file) &&
			(nb_neurons == cluster_tmp->nb_neurons))
		{	
		
			pos = i;
			break;
		}
		cluster_tmp = NULL;
	}
	
	if(!cluster_tmp)
	{
		fprintf(stderr,"no cluster matched in delete cluster !!!\n");
		exit(-1);
	}

	/* Ok. Let's begin delete procedure... */
	
	notify_cluster_deletion(cluster_tmp);
	neuron_array = cluster_tmp->neuron_array;
	for(i=0 ; i<cluster_tmp->nb_neurons ; i++)
	{
		
		delete_single_neuron(
			&(current_nucleus->cluster_array[pos]->neuron_array[i]));
	}
	free(neuron_array);
	
	current_nucleus->nb_neurons -= cluster_tmp->nb_neurons;
	current_network->nb_neurons -= cluster_tmp->nb_neurons;

	free(cluster_tmp->neuron_file);
	
	manage_cluster_id(cluster_tmp->cluster_id,0);
	
	if(pos+1 != current_nucleus->nb_clusters) 
	{
		memcpy(current_nucleus->cluster_array[pos],
				current_nucleus->cluster_array[current_nucleus->nb_clusters-1],
					sizeof(CLUSTER_WIDGET));
		free(current_nucleus->cluster_array[current_nucleus->nb_clusters-1]);
	}
	else
		free(current_nucleus->cluster_array[pos]);
		
	current_nucleus->nb_clusters--;
		
}

/*******************************************************************************
	Function for scanning the composition of the current nucleus and 
	display it in the composition list
*******************************************************************************/
void refresh_composition(void)
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i;
	XmString string;
	char buffer[255];
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in refresh_composition !!!\n");
		exit(-1);
	}

	XmListDeleteAllItems(widget_array[ID_List_composition]);

	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ;i++)
	{
		cluster_tmp = cluster_array[i];
		if(cluster_tmp->nb_neurons == 1)
		{
			sprintf(buffer,"specific> 	NAME:%s 	TYPE:%s 	SIZE:%d	",
			cluster_tmp->ch_name,cluster_tmp->neuron_file,cluster_tmp->nb_neurons);
		
		}
		else
		{
			sprintf(buffer,"generic> 	NAME:%s 	TYPE:%s 	SIZE:%d	",
			cluster_tmp->ch_name,cluster_tmp->neuron_file,cluster_tmp->nb_neurons);
		}
		string = XmStringCreateLocalized(buffer);
		if(!XmListItemExists(widget_array[ID_List_composition],
						string))
		{
			XmListAddItem(widget_array[ID_List_composition],string,1);
			XmListSelectPos(widget_array[ID_List_composition],1,True);
		}
		XmStringFree(string);
	}		
}


/*******************************************************************************
	Function for apply placement in the neuron placement generator box
*******************************************************************************/
int apply_placement()
{
	
	char tmp[255];
	XmString string;
	char *pt;
	char *buffer;
	int i;
	int type;
	int dim;
	float dim_max = (float)AREA_SIZE / (float)(2*_HUGE);
	
	buffer = XmTextFieldGetString(widget_array[ID_textF_L_radius]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_n_p_generator_form],
			"Sorry, the L radius value is too high...","Bad Dimension !");
		return -1;
	}
	
	current_nucleus->delta_L = atof(buffer);
	XtFree(buffer);

	buffer = XmTextFieldGetString(widget_array[ID_textF_H_radius]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_n_p_generator_form],
			"Sorry, the H radius value is too high...","Bad Dimension !");
		return -1;
	}
	current_nucleus->delta_H = atof(buffer);
	XtFree(buffer);
	
	buffer = XmTextFieldGetString(widget_array[ID_textF_AP_radius]);
	if(pt=strrchr(buffer,','))
		*pt = '.';
	if((atof(buffer)>dim_max)||(atof(buffer)<-dim_max))
	{
		display_error(widget_array[ID_n_p_generator_form],
			"Sorry, the AP radius value is too high...","Bad Dimension !");
		return -1;
	}
	current_nucleus->delta_AP = atof(buffer);
	XtFree(buffer);
	
	if(XmToggleButtonGetState(widget_array[ID_toggleB_random]))
		type = 0;
	else 
		type = 1;
		
	switch(type)
	{
	
		case 1:
			if(XmToggleButtonGetState(widget_array[ID_toggleB_3D]))
			{
				if(current_nucleus->delta_L && current_nucleus->delta_AP
					&& current_nucleus->delta_H)
				{
					if(current_nucleus->nb_neurons < 5)
					{
						display_error(widget_array[ID_n_p_generator_form],
							"Sorry, 5 Neurons minimum for 3D...","Missing Neurons !");
						return -1;
					}
					place_neurons_3D_equidistant();
					refresh_composition();
					redisplay_nucleus();
				}
				else
				{
					display_error(widget_array[ID_n_p_generator_form],
						"Sorry, the 3 radius value must be non zero for 3D...","Bad Dimension !");
					return -1;
				}	
			}
			else if(XmToggleButtonGetState(widget_array[ID_toggleB_2D]))
			{
				dim = 0;
				if(current_nucleus->delta_L)
					dim++;
				if(current_nucleus->delta_H)
					dim++;
				if(current_nucleus->delta_AP)
					dim++;
				if(dim == 2)
				{
					if(current_nucleus->nb_neurons < 3)
					{
						display_error(widget_array[ID_n_p_generator_form],
							"Sorry, 3 Neurons minimum for 2D...","Missing Neurons !");
						return -1;
					}
					place_neurons_2D_equidistant();
					refresh_composition();
					redisplay_nucleus();
				}
				else
				{
					display_error(widget_array[ID_n_p_generator_form],
						"Sorry, only 2 radius value must be non zero for 2D...","Bad Dimension !");
					return -1;
				}	
			}
			else
			{
				dim = 0;
				if(current_nucleus->delta_L)
					dim++;
				if(current_nucleus->delta_H)
					dim++;
				if(current_nucleus->delta_AP)
					dim++;
				if(dim == 1)
				{
					if(current_nucleus->nb_neurons < 2)
					{
						display_error(widget_array[ID_n_p_generator_form],
							"Sorry, 2 Neurons minimum for 1D...","Missing Neurons !");
						return -1;
					}
					place_neurons_1D_equidistant();
					refresh_composition();
					redisplay_nucleus();
				}
				else
				{
					display_error(widget_array[ID_n_p_generator_form],
						"Sorry, only 1 radius value must be non zero for 1D...","Bad Dimension !");
					return -1;
				}	
			}
			break;
			
		case 0:
			generic_place_neurons(True);
			refresh_composition();
			redisplay_nucleus();
			break;
			
		default:
			break;
	}
		
		
	
	
	
	return 0;
}


/*******************************************************************************
	Function for  3D calculation of neurons equidistant placement
*******************************************************************************/
void place_neurons_3D_equidistant()
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int	set;
	float dh,dl,dap;
	char buffer[255];
	short width,height;
	int nb_neurons;
	int p;
	int q;
	int r;
	int per_plan;
	int extra;
	NEURON_ANATOMY *positions;
	int counter;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in generic_place_neurons\n");
		exit(-1);
	}
	
	
	set = (int) time (0) & 077;
			
	dh = (float) current_nucleus->delta_H;
	dl = (float) current_nucleus->delta_L;
	dap = (float) current_nucleus->delta_AP;
	nb_neurons = current_nucleus->nb_neurons;

	
	
	p = 0;
	while((p*p*p)<nb_neurons)
		p++;

	positions = (NEURON_ANATOMY *)calloc(1,p*p*p*sizeof(NEURON_ANATOMY));
	counter = 0;
	for(i=0 ; i<p ; i++)
	{
		for(j=0 ; j<p ; j++)
		{
			for(k=0 ; k<p ; k++)
			{
				positions[counter].L = current_nucleus->L -dl + i*(2*dl/p);
				positions[counter].H = current_nucleus->H -dh + j*(2*dh/p);
				positions[counter].AP = current_nucleus->AP -dap + k*(2*dap/p);
				counter++;
			}
		}
	}			

	r = (int)nb_neurons/p;
	q = (int)(nb_neurons-r*p); 

			
	neuron_array = (NEURON **)calloc(1,current_nucleus->nb_neurons*sizeof(NEURON *));
			
	k = 0;
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			
			neuron_array[k++] = cluster_tmp->neuron_array[j];
				
		}
	}	
	
	
	counter = 0;
	extra = q;
	for(i=0 ; i<p ; i++)
	{
		per_plan = r;
		for(j=0 ; j<p ; j++)
		{
			
			if(per_plan)
			{
				for(k=0 ; k<p ; k++)
				{
					if(per_plan)
					{
						memcpy(&neuron_array[counter]->anatomy,&positions[i*p*p+j*p+k],
							sizeof(NEURON_ANATOMY));
						counter++;
						per_plan--;
						
					}
					else if(extra)
					{
						memcpy(&neuron_array[counter]->anatomy,&positions[i*p*p+j*p+k],
							sizeof(NEURON_ANATOMY));
						counter++;
						extra--;
						break;
					}
					else
						break;
				}
			}
			else if(extra)
			{
				memcpy(&neuron_array[counter]->anatomy,&positions[i*p*p+j*p+k],
					sizeof(NEURON_ANATOMY));
				counter++;
				extra--;
				break;
			}
			else 
				break;
			
		}
	}
		
	free(neuron_array);
	free(positions);
}

/*******************************************************************************
	Function for  2D calculation of neurons equidistant placement
*******************************************************************************/
void place_neurons_2D_equidistant()
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int	set;
	float dh,dl,dap;
	char buffer[255];
	short width,height;
	int nb_neurons;
	int p;
	int q;
	int r;
	int per_plan;
	int extra;
	NEURON_ANATOMY *positions;
	int counter;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in generic_place_neurons\n");
		exit(-1);
	}
	
	
	set = (int) time (0) & 077;
			
	dh = (float) current_nucleus->delta_H;
	dl = (float) current_nucleus->delta_L;
	dap = (float) current_nucleus->delta_AP;
	nb_neurons = current_nucleus->nb_neurons;

	
	
	p=0;
	while((p*p)<nb_neurons)
		p++;

	positions = (NEURON_ANATOMY *)calloc(1,p*p*sizeof(NEURON_ANATOMY));
	counter = 0;
	if(!dl)
	{
		for(j=0 ; j<p ; j++)
		{
			for(k=0 ; k<p ; k++)
			{
				positions[counter].L = current_nucleus->L;
				positions[counter].H = current_nucleus->H -dh + j*(2*dh/p);
				positions[counter].AP = current_nucleus->AP -dap + k*(2*dap/p);
				counter++;
			}
		}
	}			
	else if(!dh)
	{
		for(j=0 ; j<p ; j++)
		{
			for(k=0 ; k<p ; k++)
			{
				positions[counter].L = current_nucleus->L -dl + j*(2*dl/p);
				positions[counter].H = current_nucleus->H;
				positions[counter].AP = current_nucleus->AP -dap + k*(2*dap/p);
				counter++;
			}
		}
	}
	else if(!dap)
	{
		for(j=0 ; j<p ; j++)
		{
			for(k=0 ; k<p ; k++)
			{
				positions[counter].L = current_nucleus->L -dl + k*(2*dl/p);
				positions[counter].H = current_nucleus->H -dh + j*(2*dh/p);
				positions[counter].AP = current_nucleus->AP;
				counter++;
			}
		}
	}	
	r = (int)nb_neurons/p;
	q = (int)(nb_neurons-r*p); 

			
	neuron_array = (NEURON **)calloc(1,current_nucleus->nb_neurons*sizeof(NEURON *));
			
	k = 0;
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			
			neuron_array[k++] = cluster_tmp->neuron_array[j];
				
		}
	}	
	
	
	counter = 0;
	extra = q;
	for(i=0 ; i<p ; i++)
	{
		per_plan = r;
		
		for(k=0 ; k<p ; k++)
		{
			if(per_plan)
			{
				memcpy(&neuron_array[counter]->anatomy,&positions[i*p+k],
					sizeof(NEURON_ANATOMY));
				counter++;
				per_plan--;

			}
			else if(extra)
			{
				memcpy(&neuron_array[counter]->anatomy,&positions[i*p+k],
					sizeof(NEURON_ANATOMY));
				counter++;
				extra--;
				break;
			}
			else
				break;
		}
			
			
	}
		
	free(neuron_array);
	free(positions);
}


/*******************************************************************************
	Function for  1D calculation of neurons equidistant placement
*******************************************************************************/
void place_neurons_1D_equidistant()
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j,k;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int	set;
	float dh,dl,dap;
	char buffer[255];
	short width,height;
	int nb_neurons;
	int p;
	int q;
	int r;
	int per_plan;
	int extra;
	NEURON_ANATOMY *positions;
	int counter;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in generic_place_neurons\n");
		exit(-1);
	}
	
	
	set = (int) time (0) & 077;
			
	dh = (float) current_nucleus->delta_H;
	dl = (float) current_nucleus->delta_L;
	dap = (float) current_nucleus->delta_AP;
	nb_neurons = current_nucleus->nb_neurons;

	
	
	
	

	positions = (NEURON_ANATOMY *)calloc(1,nb_neurons*sizeof(NEURON_ANATOMY));
	counter = 0;
	if(!dl && !dh)
	{
		
		for(k=0 ; k<nb_neurons ; k++)
		{
			positions[counter].L = current_nucleus->L;
			positions[counter].H = current_nucleus->H;
			positions[counter].AP = current_nucleus->AP -dap + k*(2*dap/nb_neurons);
			counter++;
		}

	}			
	else if(!dh && !dap)
	{
		for(k=0 ; k<nb_neurons ; k++)
		{
			positions[counter].L = current_nucleus->L -dl + k*(2*dl/nb_neurons);
			positions[counter].H = current_nucleus->H;
			positions[counter].AP = current_nucleus->AP;
			counter++;
		}
		
	}
	else if(!dap && !dl)
	{
		
		for(k=0 ; k<nb_neurons ; k++)
		{
			positions[counter].L = current_nucleus->L;
			positions[counter].H = current_nucleus->H -dh + k*(2*dh/nb_neurons);
			positions[counter].AP = current_nucleus->AP;
			counter++;
		}
		
	}	
	
			
	neuron_array = (NEURON **)calloc(1,current_nucleus->nb_neurons*sizeof(NEURON *));
			
	k = 0;
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			
			neuron_array[k++] = cluster_tmp->neuron_array[j];
				
		}
	}	
	
	
	
	
	for(i=0 ; i<nb_neurons ; i++)
	{
		memcpy(&neuron_array[i]->anatomy,&positions[i],
			sizeof(NEURON_ANATOMY));
		
	}
		
	free(neuron_array);
	free(positions);
}

/*******************************************************************************
	Function for generic 3D calculation of neurons placement
*******************************************************************************/

float y_1(float x)
{
	return (float)x;
}

float y_2(float x)
{
	return (float)(0.5+4*(x-0.5)*(x-0.5)*(x-0.5));
}

float y_3(float x)
{
	float y1;
	float y2;
	float r;
	float q;

	y1 = 0.0;
	y2 = 1.0;


	while(y2-y1 > 0.001)
	{
		r = (y1+y2)/2;
		if((q = (0.5+4*(r-0.5)*(r-0.5)*(r-0.5))) == x)
			return (float)r;
		else if(q < x)
			y1 = r;
		else if(q > x)
			y2 = r;

	}
	return (float)r;

}



void generic_place_neurons(int flag)
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int	set;
	float dh,dl,dap;
	char buffer[255];
	short width,height;
	float (*func)(float x);
	
	
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in generic_place_neurons\n");
		exit(-1);
	}
	
	if(XmToggleButtonGetState(widget_array[ID_toggleB_constant]))
		func = &y_1;
	else if(XmToggleButtonGetState(widget_array[ID_toggleB_center]))
		func = &y_2;
	else
		func = &y_3;
	
	set = (int) time (0) & 077;
			
	dh = (float) current_nucleus->delta_H;
	dl = (float) current_nucleus->delta_L;
	dap = (float) current_nucleus->delta_AP;

	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		if(cluster_tmp->nb_neurons == 1)
			continue;
		neuron_array = cluster_tmp->neuron_array;
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			
			neuron_tmp = neuron_array[j];
			
			if(!neuron_tmp->form || flag)
			{
				neuron_tmp->anatomy.H = current_nucleus->H 
					+ (2. * func(ran1 (&set)) - 1.)*dh;
				neuron_tmp->anatomy.L = current_nucleus->L
					+ (2. * func(ran1 (&set)) - 1.)*dl;
				neuron_tmp->anatomy.AP = current_nucleus->AP
					+ (2. * func(ran1 (&set)) - 1.)*dap;
			}	
				
			if(!neuron_tmp->form)	
			{
				neuron_tmp->color = grey;
				
				sprintf(buffer,"%s.neuron_%d",cluster_tmp->ch_name,neuron_tmp->neuron_id);
				neuron_tmp->form = XtVaCreateWidget (buffer,
				    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_nucleus_view],
				    XmNwidth, NEURON_WIDTH,
				    XmNheight, NEURON_HEIGHT,
				    XmNborderWidth, 0,
				    XmNrecomputeSize, False,
				    NULL);
				    
				XtAddEventHandler(neuron_tmp->form,ButtonPressMask,False,
					(XtEventHandler)select_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonReleaseMask,False,
					(XtEventHandler)release_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
					(XtEventHandler)chplace_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
					(XtEventHandler)info_neuron,neuron_tmp->neuron_id);	
				XtAddEventHandler(neuron_tmp->form,EnterWindowMask,False,
					(XtEventHandler)info_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,LeaveWindowMask,False,
					(XtEventHandler)clear_label_nucl_info,
					neuron_tmp->neuron_id);
				
			}
			else if(flag)
			{
				XtUnmanageChild(neuron_tmp->form);
			}
			if(!neuron_tmp->bgnd)
			{
				neuron_tmp->bgnd = XCreatePixmap(the_display,
		 	XtWindow(widget_array[ID_DrawingA_nucleus_view]),NEURON_WIDTH,NEURON_HEIGHT,
		 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_nucleus_view])));
				XSetForeground(the_display,the_GC,white);
				XSetBackground(the_display,the_GC,black);
				XFillRectangle(the_display, neuron_tmp->bgnd,the_GC,
				0,0,NEURON_WIDTH,NEURON_HEIGHT);
				XtVaSetValues(neuron_tmp->form,XmNbackgroundPixmap,
				neuron_tmp->bgnd,NULL);
			}
			
		  
		}
	}	
}

/****************************************************************************
	Function for cleanning the nucleus editor drawing area
****************************************************************************/
void clear_nucleus_view(void)
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int i,j;
	
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in clear_nucleus_view\n");
		exit(-1);
	}
	
	while(current_neuron)
		select_neuron(current_neuron->form,
			current_neuron->neuron_id,NULL);
	
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		neuron_array = cluster_tmp->neuron_array;
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_tmp = neuron_array[j];
			if(neuron_tmp->form)	
			{	
				if(XtIsManaged(neuron_tmp->form))
					XtUnmanageChild(neuron_tmp->form);
			}
			  
		}
	}	

}

/****************************************************************************
	Function for applying nucleus center moves on neuron positions
****************************************************************************/
void correct_neurons_position(Nucleus_widget *nucleus_tmp,
	float old_l,float old_h,float old_ap)
{
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int i,j;
	
	
	if(!nucleus_tmp)
	{
		fprintf(stderr,"nucleus_tmp is NULL in correct_neurons_pos\n");
		exit(-1);
	}
	
	cluster_array = nucleus_tmp->cluster_array;
	for(i=0 ; i<nucleus_tmp->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		neuron_array = cluster_tmp->neuron_array;
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_tmp = neuron_array[j];
			neuron_tmp->anatomy.L -= old_l;
			neuron_tmp->anatomy.H -= old_h;
			neuron_tmp->anatomy.AP -= old_ap;
			neuron_tmp->anatomy.L += nucleus_tmp->L;
			neuron_tmp->anatomy.H += nucleus_tmp->H;
			neuron_tmp->anatomy.AP += nucleus_tmp->AP;
			  
		}
	}	
}
/*****************************************************************************
	Function that calculates the radius valus for the nucleus
******************************************************************************/
int update_radius(void)
{
	
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	float Lmax,Hmax,APmax,LCmax,HCmax,APCmax;

	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in update_radius\n");
		exit(-1);
	}
	
	Lmax = Hmax = APmax = 0.0;

			
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		cluster_tmp->delta_L = LCmax = 0.0;
		cluster_tmp->delta_H = HCmax = 0.0;
		cluster_tmp->delta_AP = APCmax = 0.0;
		neuron_array = cluster_tmp->neuron_array;
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_tmp = neuron_array[j];
			if((neuron_tmp->anatomy.L - current_nucleus->L) > Lmax)
			 	Lmax = neuron_tmp->anatomy.L - current_nucleus->L;
			else if ((neuron_tmp->anatomy.L - current_nucleus->L) < -Lmax)
			 	Lmax = -(neuron_tmp->anatomy.L - current_nucleus->L);
			 		
			if((neuron_tmp->anatomy.L - cluster_tmp->L) > LCmax)
			 	LCmax = neuron_tmp->anatomy.L - cluster_tmp->L;
			else if ((neuron_tmp->anatomy.L - cluster_tmp->L) < -LCmax)
			 	LCmax = -(neuron_tmp->anatomy.L - cluster_tmp->L);
			
			if((neuron_tmp->anatomy.H - current_nucleus->H) > Hmax) 
			 	Hmax = neuron_tmp->anatomy.H - current_nucleus->H;
			else if ((neuron_tmp->anatomy.H - current_nucleus->H) < -Hmax)
			 	Hmax = -(neuron_tmp->anatomy.H - current_nucleus->H);
			
			if((neuron_tmp->anatomy.H - cluster_tmp->H) > HCmax)
			 	HCmax = neuron_tmp->anatomy.H - cluster_tmp->H;
			else if ((neuron_tmp->anatomy.H - cluster_tmp->H) < -HCmax)
			 	HCmax = -(neuron_tmp->anatomy.H - cluster_tmp->H);	
			 
			 		
			if((neuron_tmp->anatomy.AP - current_nucleus->AP) > APmax) 
			 	APmax = neuron_tmp->anatomy.AP - current_nucleus->AP;  
		    else if ((neuron_tmp->anatomy.AP - current_nucleus->AP) < -APmax)
			 	APmax = -(neuron_tmp->anatomy.AP - current_nucleus->AP); 
			 	
			if((neuron_tmp->anatomy.AP - cluster_tmp->AP) > APCmax)
			 	APCmax = neuron_tmp->anatomy.AP - cluster_tmp->AP;
			else if ((neuron_tmp->anatomy.AP - cluster_tmp->AP) < -APCmax)
			 	APCmax = -(neuron_tmp->anatomy.AP - cluster_tmp->AP);	 	
		}
		
		cluster_tmp->delta_L =  LCmax;
		cluster_tmp->delta_H =  HCmax;
		cluster_tmp->delta_AP =  APCmax;
	}
	
	current_nucleus->delta_L = Lmax;
	current_nucleus->delta_H = Hmax;
	current_nucleus->delta_AP = APmax;
			
	return 0;
}


/*****************************************************************************
	Function that calculates the center valus for the nucleus 
******************************************************************************/
int update_center(void)
{
	
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	float Lmean,Hmean,APmean;

	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in update_center\n");
		exit(-1);
	}
	
	Lmean = Hmean = APmean = 0.0;

	if(current_nucleus->nb_neurons < 2)
		return 0;
			
	cluster_array = current_nucleus->cluster_array;
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = cluster_array[i];
		cluster_tmp->L = cluster_tmp->H = cluster_tmp->AP = 0.0;
		neuron_array = cluster_tmp->neuron_array;
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_tmp = neuron_array[j];
			Lmean += neuron_tmp->anatomy.L;
			Hmean += neuron_tmp->anatomy.H;
			APmean += neuron_tmp->anatomy.AP;
			cluster_tmp->L += neuron_tmp->anatomy.L;	
			cluster_tmp->H += neuron_tmp->anatomy.H;
			cluster_tmp->AP += neuron_tmp->anatomy.AP;
		}
		cluster_tmp->L /= cluster_tmp->nb_neurons;
		cluster_tmp->H /= cluster_tmp->nb_neurons;
		cluster_tmp->AP /= cluster_tmp->nb_neurons;
	}
	
	Lmean /= (float)current_nucleus->nb_neurons;
	Hmean /= (float)current_nucleus->nb_neurons;
	APmean /= (float)current_nucleus->nb_neurons;
	
	current_nucleus->L = Lmean;
	current_nucleus->H = Hmean;
	current_nucleus->AP = APmean;
			
	return 0;
}
/******************************************************************************
	Function that deselects all the selected neurons in the nucleus
******************************************************************************/
void deselect_all_neuron(void)
{
	NEURON *neuron_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current_nucleus is NULL in deselect_all_nucleus!!!\n");
		exit(-1);
	}
	
	
	
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_tmp = current_nucleus->cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_tmp = cluster_tmp->neuron_array[j];
			if(neuron_tmp->selected)
				select_neuron(neuron_tmp->form,
			 		neuron_tmp->neuron_id,NULL);
		}	
	}
	
}
#endif
