/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	THIS C SOURCE CODE FILE CONTAINS FUNCTIONS FOR MANAGING THE NETWORK
	PLOTTINGS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifndef _PLOTTING_FUNCTIONS_
#define _PLOTTING_FUNCTIONS_

#define _DEBUG_ 0

/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>

/* application globals include files */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif


#include <plot_ps.h>

/*****************************************************************************
	Function for applying plot parameters
*****************************************************************************/
int apply_plot(void)
{
	char *field;
	int view;
	
	if(XtIsManaged(widget_array[ID_FormD_generic_link_matrix]))
	{
		view = _MATRIX_;
	}
	else if(XtIsManaged(widget_array[ID_FormD_Nucleus]))
	{
		view = _NUCLEUS_;
	}
	else
	{
		view = _NETWORK_;
	}
	
	switch(view)
	{
	
		case _NETWORK_:
			field = XmTextFieldGetString(widget_array[ID_textF_queue]);
			if(!(*field))
			{
				display_error(widget_array[ID_form_plot],
				"Sorry, can't plot network without valid queue name...",
				"Invalid Queue Name !");
				return -1;
			}
			strcpy(default_queue,field);
			XtFree(field);
			field = XmTextFieldGetString(widget_array[ID_textF_file]);
			if(!(*field) || !strstr(field,".ps"))
			{
				display_error(widget_array[ID_form_plot],
				"Sorry, can't plot network without valid ps file name...",
				"Invalid PS File Name !");
				return -1;
			}
			strcpy(default_ps_file,field);
			XtFree(field);
			if(plot_network(default_ps_file,default_queue))
				return -1;
			break;
			
		case _NUCLEUS_:
			field = XmTextFieldGetString(widget_array[ID_textF_queue]);
			if(!(*field))
			{
				display_error(widget_array[ID_form_plot],
				"Sorry, can't plot network without valid queue name...",
				"Invalid Queue Name !");
				return -1;
			}
			strcpy(default_queue,field);
			XtFree(field);
			field = XmTextFieldGetString(widget_array[ID_textF_file]);
			if(!(*field) || !strstr(field,".ps"))
			{
				display_error(widget_array[ID_form_plot],
				"Sorry, can't plot network without valid ps file name...",
				"Invalid PS File Name !");
				return -1;
			}
			strcpy(default_ps_file,field);
			XtFree(field);
			if(plot_current_nucleus(default_ps_file,default_queue))
				return -1;
			break;
			
		default:
			return -1;
	}
	
	return 0;
}
/*******************************************************************************
	Function for plotting the current selected nucleus 
*******************************************************************************/
int plot_current_nucleus(char *ps_file_name,char *queue_name)
{
	
	char buffer[256];
	float x[4];
	float y[4];
	
	if(!ps_file_name)
	{
		display_error(widget_array[ID_FormD_Nucleus],
			"Sorry, the network can't be plotted without a ps file name..."
			,"Invalid PS File Name !");
		return -1;
		
	}
	
	if(!queue_name)
	{
		display_error(widget_array[ID_FormD_Nucleus],
			"Sorry, the network can't be plotted without a valid queue name..."
			,"Invalid Queue Name !");
		return -1;
		
	}
	
	ps_iniplot(ps_file_name,21.);
/*
	{
		sprintf(buffer,"Can't open file %s.\nCan't plot network...",ps_file_name);
		display_error(widget_array[ID_FormD_Nucleus],buffer
			,"Can't Open PS File !");
		return -1;
	}
*/
	
	ps_origin(0.5,23.,0.,0.);
	
	ps_echel(18./(float)AREA_SIZE,-18./(float)AREA_SIZE,0.,0.,1.,0.);
	
	ps_user();
	
	ps_font("Helvetica-Bold",0.5);
	
	ps_movea((float)AREA_SIZE/2.,(float)AREA_SIZE/2.);
	
	switch(which_nucleus_view())
	{
		case 0:
			ps_set_rgb_color(255./255.,255./255.,224./255.);	
			break;
		case 1:
			ps_set_rgb_color(225./255.,252./255.,241./255.);
			break;
		case 2:
			ps_set_rgb_color(255./255.,235./255.,249./255.);
			break;
		default:
			break;
	}
	
	x[0] = 0.;
	y[0] = 0.;
	x[1] = (float)AREA_SIZE;
	y[1] = 0.;
	x[2] = (float)AREA_SIZE;
	y[2] = (float)AREA_SIZE;
	x[3] = 0.;
	y[3] = (float)AREA_SIZE;
	
	ps_filled_area(x,y,4);
	
	ps_set_rgb_color(0.,0.,0.);
	
	ps_frame((float)(AREA_SIZE),(float)(AREA_SIZE));
	
	ps_movea((float)(AREA_SIZE/2),(float)(AREA_SIZE+120));
	
	ps_movea((float)(AREA_SIZE/2)-400.,(float)(AREA_SIZE+110));
	sprintf(buffer,"Nucleus Name: %s (network %s) , %d neurons",
		current_nucleus->nucleus_name,current_network->name,
			current_nucleus->nb_neurons);
	ps_write(buffer);
	ps_movea((float)(AREA_SIZE/2)-150.,(float)(AREA_SIZE+140));
	switch(which_nucleus_view())
	{
		case 0:
			sprintf(buffer,"FRONTAL VIEW (ref in mm)");
			break;
		case 1:
			sprintf(buffer,"HORIZONTAL VIEW (ref in mm)");
			break;
		case 2:
			sprintf(buffer,"SAGITTAL VIEW (ref in mm)");
			break;
		default:
			break;
	}
	ps_write(buffer);
	ps_line_width(2);
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_Nucleus]),
			working_cursor);
	
	ps_font("Helvetica",0.3);
	
	plot_nucleus_axes();
	
	plot_neuron_links(); 
	
	plot_neurons();
	
	ps_queue_name(queue_name);

	if (keep_ps)
		ps_endplot(ps_file_name,'s');
	else
		ps_endplot(ps_file_name,'d');
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_Nucleus]),
			NULL);
	
	return 0;
}
/******************************************************************************
	Function for sorting neurons by depth
******************************************************************************/
void sort_neurons_by_depth(NEURON **neuron_array)
{
	int view = which_nucleus_view();
	NEURON *neuron_3;
	NEURON *neuron_1;
	NEURON *neuron_2;
	int i,j;
	
	
	for(i=0 ; i<current_nucleus->nb_neurons-1 ; i++)
	{
		switch(view)
		{
			case 0:
				while((i>1) && (neuron_array[i-2]->anatomy.AP < neuron_array[i-1]->anatomy.AP))
					 i -= 2; 
				break;
				
			case 1:
				while((i>1) && (neuron_array[i-2]->anatomy.H < neuron_array[i-1]->anatomy.H))
					 i -= 2; 
				break;
			case 2:
				while((i>1) && (neuron_array[i-2]->anatomy.L < neuron_array[i-1]->anatomy.L))
					 i -= 2; 
				break;
			default:
				break;
		
		}
		neuron_1 = neuron_array[i];
		j = i+1;
		neuron_2 = neuron_array[j];
		switch(view)
		{
			case 0:
				while((j<current_nucleus->nb_neurons) && 
					(neuron_1->anatomy.AP >= neuron_2->anatomy.AP))
				{
					j++;
					if(j<current_nucleus->nb_neurons)
						neuron_2 = neuron_array[j];
					
				}
				if(neuron_1->anatomy.AP < neuron_2->anatomy.AP)
				{
					neuron_array[i] = neuron_2;
					neuron_array[j] = neuron_1;
				}
				break;
				
				
			case 1:
				while((j<current_nucleus->nb_neurons) && 
					(neuron_1->anatomy.H >= neuron_2->anatomy.H))
				{
					j++;
					if(j<current_nucleus->nb_neurons)
						neuron_2 = neuron_array[j];
					
				}
				if(neuron_1->anatomy.H < neuron_2->anatomy.H)
				{
					neuron_array[i] = neuron_2;
					neuron_array[j] = neuron_1;
				}
				break;
			case 2:
				while((j<current_nucleus->nb_neurons) && 
					(neuron_1->anatomy.L >= neuron_2->anatomy.L))
				{
					j++;
					if(j<current_nucleus->nb_neurons)
						neuron_2 = neuron_array[j];
					
				}
				if(neuron_1->anatomy.L < neuron_2->anatomy.L)
				{
					neuron_array[i] = neuron_2;
					neuron_array[j] = neuron_1;
				}
				break;
			default:
				break;
		
		}
	
	}
	
	
}
/*****************************************************************************
	Function that plots the neurons in the current nucleus area
******************************************************************************/
int plot_neurons(void)
{
	
	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	int i,j;
	NEURON **neuron_array;
	NEURON *neuron_tmp;
	int counter;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current nucleus is NULL in draw_neurons\n");
		exit(-1);
	}
	
	neuron_array = (NEURON **)calloc(1,current_nucleus->nb_neurons*sizeof(NEURON *));
	counter = 0;
	
	cluster_array = current_nucleus->cluster_array;
	for(i=current_nucleus->nb_clusters-1 ; i>=0 ; i--)
	{
		cluster_tmp = cluster_array[i];
		for(j=cluster_tmp->nb_neurons-1 ; j>=0 ; j--)
		{
			neuron_array[counter++] = cluster_array[i]->neuron_array[j];
			    
		}
	}		
			
	sort_neurons_by_depth(neuron_array);
	
	for(i=0 ; i<current_nucleus->nb_neurons ; i++)
	{
		
		neuron_tmp = neuron_array[i];

		plot_one_neuron(neuron_tmp);

		
	}
	free(neuron_array);
	
			
	return 0;
}
/*******************************************************************************
	Function for plotting only one specified neuron 
*******************************************************************************/
int plot_one_neuron(NEURON *neuron)
{
	char buffer[255];
	float X,Y,Xref,Yref;
	int pos_x,pos_y;
	int view = which_nucleus_view();
	short width,height;
	

	if(!neuron->form)
		return -1;

	ps_font("Helvetica",0.3);
	
	switch(view)
	{
		case 0:
			X = neuron->anatomy.L;
			Y = neuron->anatomy.H;
			Xref = current_nucleus->L;
			Yref = current_nucleus->H;
			break;

		case 1:
			X = neuron->anatomy.L;
			Y = neuron->anatomy.AP;
			Xref = current_nucleus->L;
			Yref = current_nucleus->AP;
			break;

		case 2:
			X = neuron->anatomy.AP;
			Y = neuron->anatomy.H;
			Xref = current_nucleus->AP;
			Yref = current_nucleus->H;
			break;

		default:
			break;
	}


	pos_x =	(int)(X*nucleus_scale[view] - Xref*nucleus_scale[view])
		+ current_nucleus->origin_x[view];
	pos_y = (int)(Yref*nucleus_scale[view] - Y*nucleus_scale[view])
		+ current_nucleus->origin_y[view];
	

	if((pos_x <= 0) || (pos_x >= AREA_SIZE) || (pos_y <= 0) ||(pos_y >= AREA_SIZE))
		return -1; 

	
	ps_movea((float)(pos_x),
			(float)(pos_y));


	sprintf(buffer,"%d",neuron->neuron_id);

	if(neuron->selected == False)
	{
		if(neuron->color == grey)
		{
			ps_set_rgb_color(199./255.,199./255.,199./255.);
			sprintf(buffer,"%d",neuron->neuron_id);
		}
		else if(neuron->color == blue[0])
		{
			ps_set_rgb_color(227./255.,227./255.,255./255.);
			sprintf(buffer,"%d-",neuron->neuron_id);
		}
		else if(neuron->color == blue[1])
		{
			ps_set_rgb_color(199./255.,199./255.,255./255.);
			sprintf(buffer,"%d-",neuron->neuron_id);
		}
		else if(neuron->color == blue[2])
		{
			ps_set_rgb_color(170./255.,170./255.,255./255.);
			sprintf(buffer,"%d-",neuron->neuron_id);
		}
		else if(neuron->color == blue[3])
		{
			ps_set_rgb_color(113./255.,113./255.,255./255.);		
			sprintf(buffer,"%d-",neuron->neuron_id);
		}
		else if(neuron->color == blue[4])
		{
			ps_set_rgb_color(0./255.,0./255.,255./255.);
			sprintf(buffer,"%d-",neuron->neuron_id);
		}
		else if(neuron->color == red[0])
		{		
			ps_set_rgb_color(255./255.,227./255.,227./255.);
			sprintf(buffer,"%d+",neuron->neuron_id);
		}
		else if(neuron->color == red[1])
		{		
			ps_set_rgb_color(255./255.,199./255.,199./255.);
			sprintf(buffer,"%d+",neuron->neuron_id);
		}
		else if(neuron->color == red[2])	
		{	
			ps_set_rgb_color(255./255.,170./255.,170./255.);
			sprintf(buffer,"%d+",neuron->neuron_id);
		}	
		else if(neuron->color == red[3])
		{		
			ps_set_rgb_color(255./255.,142./255.,142./255.);
			sprintf(buffer,"%d+",neuron->neuron_id);
		}
		else if(neuron->color == red[4])
		{		
			ps_set_rgb_color(255./255.,0./255.,0./255.);
			sprintf(buffer,"%d+",neuron->neuron_id);
		}
	}
	else
	{
		ps_set_rgb_color(255./255.,255./255.,255./255.);
	}
	ps_disk((float)(NEURON_WIDTH-1)*18./(float)AREA_SIZE,-1.);
	ps_set_rgb_color(0.,0.,0.);
	ps_circle((float)(NEURON_WIDTH)*18./(float)AREA_SIZE);
	ps_mover(-(float)NEURON_WIDTH/3,(float)NEURON_WIDTH/4);
	ps_write(buffer);
	
	return 0;
}
/*****************************************************************************
	Function that plots the axes in the current view of the current selected
	nucleus.
******************************************************************************/
int	plot_nucleus_axes()
{
	int which_view = which_nucleus_view();
	int i,j,x,y,counter;
	char buffer[255];
	float factor;
	int scale = nucleus_scale[which_view];
	short width,height;
	
	
	switch(scale)
	{
		case NUCLEUS_HUGE:
			factor = 0.5;
			break;
		
		case NUCLEUS_LARGE:
			factor = 1.0;
			break;
		
		case NUCLEUS_MEDIUM:
			factor = 2.5;
			break;
			
		case NUCLEUS_SMALL:
			factor = 5.0;
			break;
			
		case NUCLEUS_TINY:
			factor = 10.0;
			break;
			
		default:
			break;
	}
	
	scale = (int)((float)scale/factor);
	
	ps_set_rgb_color(0.,0.,0.);
	
	ps_movea(0.,(float)current_nucleus->origin_y[which_view]);
	ps_drawa((float)AREA_SIZE,(float)current_nucleus->origin_y[which_view]);
	
	ps_movea((float)current_nucleus->origin_x[which_view],0.);
	ps_drawa((float)current_nucleus->origin_x[which_view],(float)AREA_SIZE);
		
	
	/* x positifs */	
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x < current_network->work_width[which_view])
	{
		
		i = x - current_nucleus->origin_x[which_view]
			 - counter*scale;
		if((int)(i/scale) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)x,(float)(y-5));
			
			sprintf(buffer,"+%.1f",(float)counter/factor);
			ps_movea((float)(x-5),(float)(y+20));
			ps_write(buffer);
		}
		x++; 
	}
	/* x negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x >= 0)
	{
		
		i = current_nucleus->origin_x[which_view] - x
			 - counter*scale;
		if((int)(i/scale) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)x,(float)(y-5));
	
			sprintf(buffer,"-%.1f",(float)counter/factor);
			ps_movea((float)(x-5),(float)(y+20));
			ps_write(buffer);
		}
		x--; 
	}
	/* y negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y < current_network->work_height[which_view])
	{
		
		j = y - current_nucleus->origin_y[which_view]
			 - counter*scale;
		if((int)(j/scale) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)(x+5),(float)y);
	
			sprintf(buffer,"-%.1f",(float)counter/factor);
			ps_movea((float)(x-20),(float)(y+5));
			ps_write(buffer);
			
		}
		y++; 
	}
	/* y positifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y >= 0)
	{
		
		j = current_nucleus->origin_y[which_view] - y
			 - counter*scale;
		if((int)(j/scale) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)(x+5),(float)y);
	
			sprintf(buffer,"+%.1f",(float)counter/factor);
			ps_movea((float)(x-25),(float)(y+5));
			ps_write(buffer);
			
		}
		y--; 
	}	
	return 0;
}




/*****************************************************************************
	Function that draws the neuron links
******************************************************************************/
int plot_neuron_links(void)
{
	Nucleus_widget *nucleus_tmp;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	Connection connection;
	int i,j,k;
	
	if(!current_nucleus->nb_links)
		return 0;
	
	for(i=0 ; i<current_nucleus->nb_clusters ; i++)
	{
		cluster_src = current_nucleus->cluster_array[i];
		for(j=0 ; j<cluster_src->nb_neurons ; j++)
		{
			neuron_src = cluster_src->neuron_array[j];
			for(k=0 ; k<neuron_src->connected_neurons ; k++)
			{
				connection = neuron_src->connections[k];
				neuron_tgt = search_neuron(connection.target_id,&cluster_tgt);
				search_cluster(cluster_tgt->cluster_id,&nucleus_tmp);
				if(nucleus_tmp != current_nucleus)
					continue;
				else
					plot_one_connection(&connection,neuron_src->neuron_id,
						neuron_tgt->neuron_id);
			}
		}
		
		
	}
	return 0;
}
/*******************************************************************************
	Function for drawing only one connection in a nucleus
*******************************************************************************/
void plot_one_connection(Connection *connection,int source_id,int target_id)
{
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	CLUSTER_WIDGET *cluster_tmp;
	int view = which_nucleus_view();
	float X1,X2,Y1,Y2,Xref,Yref;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;
	double x_mid,y_mid,d,dmin;
	
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current_nucleus is NULL in draw_one_connection!!!!\n");
		exit(-1);
	}
	
	neuron_src = search_neuron(source_id,&cluster_tmp);
	neuron_tgt = search_neuron(target_id,&cluster_tmp);
	if(neuron_src == neuron_tgt)
	{
		if(connection->synaptic_knobs)
			color = eval_color(connection->synaptic_knobs);
		else		
			color = grey;
		neuron_src->color = color;
		
	}
	else
	{
		switch(view)
		{
			case 0:
				X1 = neuron_src->anatomy.L;
				Y1 = neuron_src->anatomy.H;
				X2 = neuron_tgt->anatomy.L;
				Y2 = neuron_tgt->anatomy.H;
				Xref = current_nucleus->L;
				Yref = current_nucleus->H;
				break;

			case 1:
				X1 = neuron_src->anatomy.L;
				Y1 = neuron_src->anatomy.AP;
				X2 = neuron_tgt->anatomy.L;
				Y2 = neuron_tgt->anatomy.AP;
				Xref = current_nucleus->L;
				Yref = current_nucleus->AP;
				break;

			case 2:
				X1 = neuron_src->anatomy.AP;
				Y1 = neuron_src->anatomy.H;
				X2 = neuron_tgt->anatomy.AP;
				Y2 = neuron_tgt->anatomy.H;
				Xref = current_nucleus->AP;
				Yref = current_nucleus->H;
				break;

			default:

				break;
		}
		posX1 =	(int)(X1*nucleus_scale[view] - Xref*nucleus_scale[view]) 
			+ current_nucleus->origin_x[view];
		posX2 =	(int)(X2*nucleus_scale[view] - Xref*nucleus_scale[view]) 
			+ current_nucleus->origin_x[view];	
		posY1 =	(int)(-Y1*nucleus_scale[view] + Yref*nucleus_scale[view]) 
			+ current_nucleus->origin_y[view];
		posY2 =	(int)(-Y2*nucleus_scale[view] + Yref*nucleus_scale[view]) 
			+ current_nucleus->origin_y[view];
		
		
		if(connection->synaptic_knobs > 0)
		{
			color = eval_color(connection->synaptic_knobs);
			over_x = over_y = +3;
		}
		else if	(connection->synaptic_knobs < 0)	
		{
			color = eval_color(connection->synaptic_knobs);
			over_x = over_y = -3;
		}
		else		
		{
			color = grey;
			return;
		}	
		
		if(color == grey)
			ps_set_rgb_color(199./255.,199./255.,199./255.);
		else if(color == blue[0])
			ps_set_rgb_color(227./255.,227./255.,255./255.);
		else if(color == blue[1])
			ps_set_rgb_color(199./255.,199./255.,255./255.);
		else if(color == blue[2])
			ps_set_rgb_color(170./255.,170./255.,255./255.);
		else if(color == blue[3])
			ps_set_rgb_color(113./255.,113./255.,255./255.);		
		else if(color == blue[4])
			ps_set_rgb_color(0./255.,0./255.,255./255.);
		else if(color == red[0])		
			ps_set_rgb_color(255./255.,227./255.,227./255.);
		else if(color == red[1])		
			ps_set_rgb_color(255./255.,199./255.,199./255.);
		else if(color == red[2])		
			ps_set_rgb_color(255./255.,170./255.,170./255.);
		else if(color == red[3])		
			ps_set_rgb_color(255./255.,142./255.,142./255.);
		else if(color == red[4])		
			ps_set_rgb_color(255./255.,0./255.,0./255.);
		
		
		
		ps_movea((float)(posX1+over_x),(float)(posY1+over_y));
		ps_drawa((float)(posX2+over_x),(float)(posY2+over_y));
		
		if((posX2<0)
			 || (posX2>=AREA_SIZE)
			 ||	(posY2<0)
			 ||	(posY2>=AREA_SIZE) )
			 return;

	
		dmin = (double)(NEURON_WIDTH*NEURON_WIDTH+NEURON_HEIGHT*NEURON_HEIGHT)
					/1.0;
		d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
		x_mid = posX1+over_x;
		y_mid = posY1+over_y;
		while(d>0.8*dmin)
		{
			x_mid = (19*x_mid + posX2+over_x)/20;
			y_mid = (19*y_mid + posY2+over_y)/20;
			d = (double)(posX2-x_mid)*(posX2-x_mid)
				+(double)(posY2-y_mid)*(posY2-y_mid);
		}

		Xmid = (int)x_mid;
		Ymid = (int)y_mid;

		ps_movea((float)Xmid,(float)Ymid);
		if(connection->synaptic_knobs > 0)
			ps_disk(6.*18./(float)AREA_SIZE,-1.);
		else if	(connection->synaptic_knobs < 0)
		{	
			ps_circle(6.*18./(float)AREA_SIZE);
			ps_disk(6.*18./(float)AREA_SIZE,-1.);
			ps_disk(2.*18./(float)AREA_SIZE,1.);
		}

	}
	

}

/*****************************************************************************
	Function for plotting the network 
******************************************************************************/
int plot_network(char *ps_file_name,char *queue_name)
{
	
	char buffer[256];
	float x[4];
	float y[4];
	
	if(!ps_file_name)
	{
		display_error(widget_array[ID_MainW_network],
			"Sorry, the network can't be plotted without a ps file name..."
			,"Invalid PS File Name !");
		return -1;
		
	}
	
	if(!queue_name)
	{
		display_error(widget_array[ID_MainW_network],
			"Sorry, the network can't be plotted without a valid queue name..."
			,"Invalid Queue Name !");
		return -1;
		
	}
	
	ps_iniplot(ps_file_name,21.);
/*
	{
		sprintf(buffer,"Can't open file %s.\nCan't plot network...",ps_file_name);
		display_error(widget_array[ID_MainW_network],buffer
			,"Can't Open PS File !");
		return -1;
	}
*/
	ps_origin(0.5,23.,0.,0.);
	
	ps_echel(18./(float)AREA_SIZE,-18./(float)AREA_SIZE,0.,0.,1.,0.);
	
	ps_user();
	
	ps_movea((float)AREA_SIZE/2.,(float)AREA_SIZE/2.);
	
	switch(which_network_view())
	{
		case 0:
			ps_set_rgb_color(255./255.,255./255.,224./255.);	
			break;
		case 1:
			ps_set_rgb_color(225./255.,252./255.,241./255.);
			break;
		case 2:
			ps_set_rgb_color(255./255.,235./255.,249./255.);
			break;
		default:
			break;
	}
	
	x[0] = 0.;
	y[0] = 0.;
	x[1] = (float)AREA_SIZE;
	y[1] = 0.;
	x[2] = (float)AREA_SIZE;
	y[2] = (float)AREA_SIZE;
	x[3] = 0.;
	y[3] = (float)AREA_SIZE;
	
	ps_filled_area(x,y,4);
	
	ps_set_rgb_color(0.,0.,0.);
	
	ps_font("Helvetica-Bold",0.5);
	
	ps_frame((float)(AREA_SIZE),(float)(AREA_SIZE));
	
	ps_movea((float)(AREA_SIZE/2),(float)(AREA_SIZE+120));
	
	ps_movea((float)(AREA_SIZE/2)-300.,(float)(AREA_SIZE+110));
	sprintf(buffer,"Network Name: %s , %d nucleus, %d neurons",
		current_network->name,current_network->nb_nucleus,
			current_network->nb_neurons);
	ps_write(buffer);
	ps_movea((float)(AREA_SIZE/2)-150.,(float)(AREA_SIZE+140));
	switch(which_network_view())
	{
		case 0:
			sprintf(buffer,"FRONTAL VIEW (ref in mm)");
			break;
		case 1:
			sprintf(buffer,"HORIZONTAL VIEW (ref in mm)");
			break;
		case 2:
			sprintf(buffer,"SAGITTAL VIEW (ref in mm)");
			break;
		default:
			break;
	}
	ps_write(buffer);
	ps_line_width(2);
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			working_cursor);
	
		
	calculate_positions();
	
	plot_net_axes(); 
	
	plot_network_links();
	
	plot_nucleus();
	
	ps_queue_name(queue_name);
	if (keep_ps)
		ps_endplot(ps_file_name,'s');
	else
		ps_endplot(ps_file_name,'d');
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_MainW_network]),
			NULL);
	
	return 0;
}

/******************************************************************************
	Function for sorting nucleus by depth
******************************************************************************/
void sort_nucleus_by_depth(Nucleus_widget **nucleus_array)
{
	int view = which_network_view();
	Nucleus_widget *nucleus_3;
	Nucleus_widget *nucleus_1;
	Nucleus_widget *nucleus_2;
	int i,j;
	
	
	for(i=0 ; i<current_network->nb_nucleus-1 ; i++)
	{
		switch(view)
		{
			case 0:
				while((i>1) && (nucleus_array[i-2]->AP < nucleus_array[i-1]->AP))
					 i -= 2; 
				break;
				
			case 1:
				while((i>1) && (nucleus_array[i-2]->H < nucleus_array[i-1]->H))
					 i -= 2; 
				break;
			case 2:
				while((i>1) && (nucleus_array[i-2]->L < nucleus_array[i-1]->L))
					 i -= 2; 
				break;
			default:
				break;
		
		}
		nucleus_1 = nucleus_array[i];
		j = i+1;
		nucleus_2 = nucleus_array[j];
		switch(view)
		{
			case 0:
				while((j<current_network->nb_nucleus) && 
					(nucleus_1->AP >= nucleus_2->AP))
				{
					j++;
					if(j<current_network->nb_nucleus)
						nucleus_2 = nucleus_array[j];
					
				}
				if(nucleus_1->AP < nucleus_2->AP)
				{
					nucleus_array[i] = nucleus_2;
					nucleus_array[j] = nucleus_1;
				}
				break;
				
				
			case 1:
				while((j<current_network->nb_nucleus) && 
					(nucleus_1->H >= nucleus_2->H))
				{
					j++;
					if(j<current_network->nb_nucleus)
						nucleus_2 = nucleus_array[j];
					
				}
				if(nucleus_1->H < nucleus_2->H)
				{
					nucleus_array[i] = nucleus_2;
					nucleus_array[j] = nucleus_1;
				}
				break;
			case 2:
				while((j<current_network->nb_nucleus) && 
					(nucleus_1->L >= nucleus_2->L))
				{
					j++;
					if(j<current_network->nb_nucleus)
						nucleus_2 = nucleus_array[j];
					
				}
				if(nucleus_1->L < nucleus_2->L)
				{
					nucleus_array[i] = nucleus_2;
					nucleus_array[j] = nucleus_1;
				}
				break;
			default:
				break;
		
		}
	
	}
	
	
}

/******************************************************************************
	Function for plotting the nucleus in the network
*******************************************************************************/
int plot_nucleus(void)
{
	Nucleus_widget *nucleus_tmp;
	Nucleus_widget **nucleus_array;
	int i,j;
	Widget form;
	Pixmap bgnd;
	char buffer[255];
	
	if(!current_network)
		return -1;
	
	if(!current_network->nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nCan't draw nucleus because the nucleus array is empty!\n");
#endif	
		return -1;
	}
	
	
	nucleus_array = (Nucleus_widget **)calloc(1,current_network->nb_nucleus*
		sizeof(Nucleus_widget *));
		
	for(i=0 ; i<current_network->nb_nucleus ; i++)
		nucleus_array[i] = current_network->nucleus_array[i];
	
	sort_nucleus_by_depth(nucleus_array);		
	
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		if(!nucleus_tmp)
		{
#if _DEBUG_
			fprintf(stderr,"\nCan't draw nucleus because one of them is empty!\n");
#endif	
			return -1;
		}
		else if(!nucleus_tmp->form)
		{
#if _DEBUG_
			fprintf(stderr,"\nCan't draw nucleus: no form !\n");
#endif	
			return -1;
		}
		else
		{
			plot_one_nucleus(nucleus_tmp);
		}
		
			
	}
	free(nucleus_array);
	
}

/*******************************************************************************
	Function for redrawing one nucleus
*******************************************************************************/
int plot_one_nucleus(Nucleus_widget *nucleus_tmp)
{
	Widget form = nucleus_tmp->form;
	Pixmap bgnd = nucleus_tmp->bgnd;
	char buffer[255];
	float x[4];
	float y[4];
	
	ps_font("Helvetica",0.3);
	
	if(nucleus_tmp->nb_neurons == 1)
	{
		ps_font("Helvetica",0.2);
		
		if((nucleus_tmp->posX[which_network_view()]<0)
			 || (nucleus_tmp->posX[which_network_view()]>=AREA_SIZE)
			 ||	(nucleus_tmp->posY[which_network_view()]<0)
			 ||	(nucleus_tmp->posY[which_network_view()]>=AREA_SIZE) )
			 return;


		ps_movea((float)(nucleus_tmp->posX[which_network_view()]+NEURON_WIDTH/2),
			(float)(nucleus_tmp->posY[which_network_view()]+NEURON_HEIGHT/2));

		ps_set_rgb_color(0.,0.,0.);
		ps_circle((float)(NEURON_WIDTH)*18./(float)AREA_SIZE);
		
		sprintf(buffer,"%d:%s",nucleus_tmp->nucleus_id,nucleus_tmp->nucleus_name);
		
		if(nucleus_tmp->selected == False)
		{
			if(nucleus_tmp->color == grey)
				ps_set_rgb_color(199./255.,199./255.,199./255.);
			else if(nucleus_tmp->color == blue[4])
				ps_set_rgb_color(115./255.,115./255.,255./255.);
			else if(nucleus_tmp->color == red[4])		
				ps_set_rgb_color(255./255.,0./255.,0./255.);
			else if(nucleus_tmp->color == green)		
				ps_set_rgb_color(0./255.,255./255.,0./255.);
			
		}
		else
		{
			ps_set_rgb_color(255./255.,255./255.,255./255.);
		}
		ps_disk((float)(NEURON_WIDTH-1)*18./(float)AREA_SIZE,-1.);
		
		ps_set_rgb_color(0.,0.,0.);
		ps_mover(-(float)NEURON_WIDTH,(float)NEURON_WIDTH/4);
		ps_write(buffer);
		ps_movea((float)(nucleus_tmp->posX[which_network_view()]+NEURON_WIDTH/2),
			(float)(nucleus_tmp->posY[which_network_view()]+NEURON_HEIGHT/2));
		ps_mover(-(float)NEURON_WIDTH/2,(float)2*NEURON_HEIGHT/3);
		
		ps_size(0.25);
						
		if(nucleus_tmp->color == blue[4])
		{
			sprintf(buffer,"inhib");
			ps_write(buffer);
		}	
		else if(nucleus_tmp->color == red[4])
		{		
			sprintf(buffer,"excit");
			ps_write(buffer);
		}
		else if(nucleus_tmp->color == green)		
		{		
			sprintf(buffer,"random");
			ps_write(buffer);
		}	

		

	}
	else
	{
		ps_font("Helvetica",0.3);

		if((nucleus_tmp->posX[which_network_view()]<0)
			 || (nucleus_tmp->posX[which_network_view()]>=AREA_SIZE)
			 ||	(nucleus_tmp->posY[which_network_view()]<0)
			 ||	(nucleus_tmp->posY[which_network_view()]>=AREA_SIZE) )
			 return;

		
		ps_movea((float)(nucleus_tmp->posX[which_network_view()]+NUCLEUS_WIDTH/2),
			(float)(nucleus_tmp->posY[which_network_view()]+NUCLEUS_HEIGHT/2));


		sprintf(buffer,"%d:%s",nucleus_tmp->nucleus_id,nucleus_tmp->nucleus_name);
		ps_set_rgb_color(0.,0.,0.);
		
		x[0] = (float)(nucleus_tmp->posX[which_network_view()] - NUCLEUS_WIDTH/2);
		y[0] = (float)(nucleus_tmp->posY[which_network_view()] - NUCLEUS_HEIGHT/2);
		x[1] = (float)(nucleus_tmp->posX[which_network_view()] + 2*NUCLEUS_WIDTH 
			- NUCLEUS_WIDTH/2);
		y[1] = (float)(nucleus_tmp->posY[which_network_view()] - NUCLEUS_HEIGHT/2);
		x[2] = (float)(nucleus_tmp->posX[which_network_view()] + 2*NUCLEUS_WIDTH
			- NUCLEUS_WIDTH/2);
		y[2] = (float)(nucleus_tmp->posY[which_network_view()] + 2*NUCLEUS_HEIGHT
			- NUCLEUS_HEIGHT/2);
		x[3] = (float)(nucleus_tmp->posX[which_network_view()] - NUCLEUS_WIDTH/2);
		y[3] = (float)(nucleus_tmp->posY[which_network_view()] + 2*NUCLEUS_HEIGHT
			- NUCLEUS_HEIGHT/2);
		
		ps_closed_curve(x,y,4,1,0,1.);
		
		if(nucleus_tmp->selected == False)
		{
			if(nucleus_tmp->color == grey)
				ps_set_rgb_color(199./255.,199./255.,199./255.);
			else if(nucleus_tmp->color == blue[4])
				ps_set_rgb_color(115./255.,115./255.,255./255.);
			else if(nucleus_tmp->color == red[4])		
				ps_set_rgb_color(255./255.,0./255.,0./255.);
			else if(nucleus_tmp->color == green)		
				ps_set_rgb_color(0./255.,255./255.,0./255.);
					
			
		}
		else
		{
			ps_set_rgb_color(255./255.,255./255.,255./255.);
		}
		x[0] += 1.0;
		y[0] += 1.0;
		x[1] -= 1.0;
		y[1] += 1.0;
		x[2] -= 1.0;
		y[2] -= 1.0;
		x[3] += 1.0;
		y[3] -= 1.0;
		
		ps_filled_area(x,y,4);
		ps_set_rgb_color(0./255.,0./255.,0./255.);
		ps_mover(5.0,-(float)3*NUCLEUS_HEIGHT/2);
		ps_write(buffer);
		ps_movea((float)(nucleus_tmp->posX[which_network_view()]+NUCLEUS_WIDTH/2),
			(float)(nucleus_tmp->posY[which_network_view()]+NUCLEUS_HEIGHT/2));
		ps_mover(-(float)NUCLEUS_WIDTH+5.0,0.0);
		sprintf(buffer,"%d neurons",nucleus_tmp->nb_neurons);
		ps_write(buffer);
		
		ps_movea((float)(nucleus_tmp->posX[which_network_view()]+NUCLEUS_WIDTH/2),
			(float)(nucleus_tmp->posY[which_network_view()]+NUCLEUS_HEIGHT/2));
		ps_mover(-(float)NUCLEUS_WIDTH+15.0,(float)NUCLEUS_HEIGHT/2);
		
		ps_size(0.3);
						
		if(nucleus_tmp->color == blue[4])
		{
			sprintf(buffer,"inhib");
			ps_write(buffer);
		}	
		else if(nucleus_tmp->color == red[4])
		{		
			sprintf(buffer,"excit");
			ps_write(buffer);
		}
		else if(nucleus_tmp->color == green)		
		{		
			sprintf(buffer,"random");
			ps_write(buffer);
		}	
	}

}

/*******************************************************************************
	Function for drawing only one network link
*******************************************************************************/
void plot_one_network_link(LINK_PARAMETERS *link_tmp)
{
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int view = which_network_view();
	float X1,X2,Y1,Y2;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;
	double x_mid,y_mid,d,dmin;
	int type;
	
	
	
	nucleus_src = search_nucleus(link_tmp->source_id);
	nucleus_tgt = search_nucleus(link_tmp->target_id);
	if(nucleus_src == nucleus_tgt)
	{
		switch(link_tmp->type)
		{

			case _FULL_EXCITATION_:
			case _PART_EXCITATION_:
				color = red[4];
				type = 1;
				break;

			case _FULL_INHIBITION_:
			case _PART_INHIBITION_:
				color = blue[4];
				type = -1;
				break;

			case _FULL_RANDOMIZED_:
			case _PART_RANDOMIZED_:
				color = green;
				type = 0;
				break;

			default:
				color = grey;
				break;
		}
		nucleus_src->color = color;

	}
	else
	{
		switch(view)
		{
			case 0:
				X1 = nucleus_src->L;
				Y1 = nucleus_src->H;
				X2 = nucleus_tgt->L;
				Y2 = nucleus_tgt->H;
				break;

			case 1:
				X1 = nucleus_src->L;
				Y1 = nucleus_src->AP;
				X2 = nucleus_tgt->L;
				Y2 = nucleus_tgt->AP;
				break;

			case 2:
				X1 = nucleus_src->AP;
				Y1 = nucleus_src->H;
				X2 = nucleus_tgt->AP;
				Y2 = nucleus_tgt->H;
				break;

			default:

				break;
		}
		posX1 =	(int)(X1*network_scale[view]) 
			+ current_network->origin_x[view];
		posX2 =	(int)(X2*network_scale[view]) 
			+ current_network->origin_x[view]; 
		posY1 =	(int)(-Y1*network_scale[view]) 
			+ current_network->origin_y[view];	
		posY2 =	(int)(-Y2*network_scale[view]) 
			+ current_network->origin_y[view];
		
		
		
		switch(link_tmp->type)
		{

			case _FULL_EXCITATION_:
			case _PART_EXCITATION_:
				color = red[4];
				type = 1;
				over_x = over_y = +3;
				ps_set_rgb_color(1.,0.,0.);
				break;

			case _FULL_INHIBITION_:
			case _PART_INHIBITION_:
				color = blue[4];
				type = -1;
				over_x = over_y = -3;
				ps_set_rgb_color(0.,0.,1.);
				break;

			case _FULL_RANDOMIZED_:
			case _PART_RANDOMIZED_:
				color = green;
				type = 0;
				over_x = over_y = 0;
				ps_set_rgb_color(0.,1.,0.);
				break;

			default:
				color = black;
				ps_set_rgb_color(0.,0.,0.);
				break;
		}
		
		
		ps_movea((float)(posX1+over_x),(float)(posY1+over_y));
		ps_drawa((float)(posX2+over_x),(float)(posY2+over_y));
		
		if((posX2<0)
			 || (posX2>=AREA_SIZE)
			 ||	(posY2<0)
			 ||	(posY2>=AREA_SIZE) )
			 return;
		
		
		
		if(nucleus_tgt->nb_neurons == 1)
		{
			dmin = (double)(NEURON_WIDTH*NEURON_WIDTH+NEURON_HEIGHT*NEURON_HEIGHT)
					/1.0;
			d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
			x_mid = posX1+over_x;
			y_mid = posY1+over_y;
			while(d>0.8*dmin)
			{
				x_mid = (19*x_mid + posX2+over_x)/20;
				y_mid = (19*y_mid + posY2+over_y)/20;
				d = (double)(posX2-x_mid)*(posX2-x_mid)
					+(double)(posY2-y_mid)*(posY2-y_mid);
			}
			
			Xmid = (int)x_mid;
			Ymid = (int)y_mid;
		
			ps_movea((float)Xmid,(float)Ymid);
			if(type > 0)
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
			else if (type < 0)
			{
				ps_circle(12.*18./(float)AREA_SIZE);
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
				ps_disk(4.*18./(float)AREA_SIZE,1.);
			}
			else
			{
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
				ps_set_rgb_color(1.,1.,1.);
				ps_circle(6.*18./(float)AREA_SIZE);
			}
				
			
		}
		else
		{
	
	
#define DELTA(a,b) (a>b ? a-b : b-a)			



			dmin = (double)(NUCLEUS_WIDTH*NUCLEUS_WIDTH+NUCLEUS_HEIGHT*NUCLEUS_HEIGHT)
						/(0.8+0.75*(1.0-(double)(MIN(DELTA(posY2,posY1),DELTA(posX2,posX1)))
							/(double)(MAX(DELTA(posY2,posY1),DELTA(posX2,posX1)))));	
			d = (double)(posX2-posX1)*(posX2-posX1)+(double)(posY2-posY1)*(posY2-posY1);
			x_mid = posX1+over_x;
			y_mid = posY1+over_y;
			while(d>1.1*dmin)
			{
				x_mid = (19*x_mid + posX2 + over_x)/20;
				y_mid = (19*y_mid + posY2 + over_y)/20;
				d = (double)(posX2-x_mid)*(posX2-x_mid)
					+(double)(posY2-y_mid)*(posY2-y_mid);
			}
			
			Xmid = (int)x_mid;
			Ymid = (int)y_mid;
			
			ps_movea((float)Xmid,(float)Ymid);
			if(type > 0)
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
			else if (type < 0)
			{
				ps_circle(12.*18./(float)AREA_SIZE);
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
				ps_disk(4.*18./(float)AREA_SIZE,1.);
			}
			else
			{
				ps_disk(12.*18./(float)AREA_SIZE,-1.);
				ps_set_rgb_color(1.,1.,1.);
				ps_circle(6.*18./(float)AREA_SIZE);
			}
			
			
		}

	}
	

}


/******************************************************************************
	Function for drawing the links between nucleus in the network
*******************************************************************************/
int plot_network_links(void)
{
	LINK_PARAMETERS *link_tmp;
	int i;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int view = which_network_view();
	float X1,X2,Y1,Y2;
	int	posX1,posX2,posY1,posY2;
	int Xmid,Ymid;
	unsigned long color;
	int over_x,over_y;

	
	if(!current_network->nb_links)
		return 0;
	
	for(i=0 ; i<current_network->nb_links ; i++)
	{
		link_tmp = current_network->link_array[i];
		if(link_tmp->level != _NETWORK_)
		{
			fprintf(stderr,"Bad level link in draw_network_links!!\n");
			exit(-1);
		}
		plot_one_network_link(link_tmp);
		
	}
	
	return 0;
}
/******************************************************************************
	Function that simply plots the axes	of the network 
*******************************************************************************/
int plot_net_axes(void)
{
	int which_view = which_network_view();
	int i,j,x,y,counter;
	char buffer[255];
	
	ps_font("Helvetica",0.3);
	
	ps_set_rgb_color(0.,0.,0.);
	
	ps_movea(0.,(float)current_network->origin_y[which_view]);
	ps_drawa((float)AREA_SIZE,(float)current_network->origin_y[which_view]);
	
	ps_movea((float)current_network->origin_x[which_view],0.);
	ps_drawa((float)current_network->origin_x[which_view],(float)AREA_SIZE);
		
	
	/* x positifs */	
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x < current_network->work_width[which_view])
	{
		i = x - current_network->origin_x[which_view]
			 - counter*network_scale[which_view];
		if((int)(i/network_scale[which_view]) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)x,(float)(y-5));
			
			sprintf(buffer,"+%d",counter);
			ps_movea((float)(x-5),(float)(y+20));
			ps_write(buffer);
		}
		x++; 
	}
	/* x negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(x >= 0)
	{
		i = current_network->origin_x[which_view] - x
			 - counter*network_scale[which_view];
		if((int)(i/network_scale[which_view]) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)x,(float)(y-5));
	
			sprintf(buffer,"-%d",counter);
			ps_movea((float)(x-5),(float)(y+20));
			ps_write(buffer);
		}
		x--; 
	}
	/* y negatifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y < current_network->work_height[which_view])
	{
		j = y - current_network->origin_y[which_view]
			 - counter*network_scale[which_view];
		if((int)(j/network_scale[which_view]) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)(x+5),(float)y);
	
			sprintf(buffer,"-%d",counter);
			ps_movea((float)(x-20),(float)(y+5));
			ps_write(buffer);
			
		}
		y++; 
	}
	/* y positifs */
	x = current_network->origin_x[which_view];
	y = current_network->origin_y[which_view];
	counter = 0;	
	while(y >= 0)
	{
		j = current_network->origin_y[which_view] - y
			 - counter*network_scale[which_view];
		if((int)(j/network_scale[which_view]) == 1)
		{
			counter++;
			ps_movea((float)x,(float)y);
			ps_drawa((float)(x+5),(float)y);
	
			sprintf(buffer,"+%d",counter);
			ps_movea((float)(x-25),(float)(y+5));
			ps_write(buffer);
			
		}
		y--; 
	}	
	return 0;
}

#endif
