#ifndef _READING_ANATOMY_FILE_
#define _READING_ANATOMY_FILE_

#define _DEBUG_ 0

/*--------------------------------------------------------------------------*
 *	The aim of this file is to read a anat file format ver2 NBC Network     *
 *  Editor , and store it in the specified network ver2 structure.          *
 *--------------------------------------------------------------------------*/
 
/* Remarks: the network structure must have been loaded first with the
corresponding lnk file and most important is that cluster elements of
the network have to know their number since the anat file will be read 
sequentially. */


/* general remarks about reading an whole network structure:
		there is a special order to read the different types of file,
		first the lnk file must be read, then the cnx file and finaly
		the anat file.
		The reason for this is the order of allocating the internal
		network structures such as the nucleus, cluster or neuron 
		structures.					*/


/* standard include files */
#include <Xm/Xm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif

/* prototypes of functions implemented in this file */
int read_anat_file();

/*--------------------------------------------------------------------------*
 *   	R E A D _ A N A T _ F I L E	                                        *
 *--------------------------------------------------------------------------*/
int read_anat_file(Network *network)
{
	int i,j,k,p,counter;
	short total_nucleus;
	short anat_total_nucleus;
	short *total_neurons;
	short *anat_total_neurons;
	Nucleus_widget **all_nucleus;
	FILE *fp;
	char buffer[255];
	char tmp[256];
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array, *cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	short cluster_id;
	short value;
	
	
	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad Network ! can't read cnx file !\n");
#endif
		return -1;
	}	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nCurrent Network hasn't any name ! can't read cnx file !\n");
#endif
		return -1;
	}
	
	/* get the cnx file name */

	strcpy(buffer,project_directory);
	strcat(buffer,"/");
	strcat(buffer,network->name);
	strcat(buffer,".anat");

	/* test the network: it must already have the informations from 
	the lnk file, only the neuron array from each cluster has to be filled */
	
	if(!network->nb_nucleus || !network->nb_neurons)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad nucleus or neuron number: can't read cnx file...\n");
#endif
		return -1;
	} 	 

	nucleus_array = network->nucleus_array;
	
	if(!nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus array empty: can't read cnx file...\n");
#endif
		return -1;
	}

	/* now let's count the total number of nucleus in the network */

	total_nucleus = network->nb_nucleus;

	/* get the total neuron per nucleus... reference all nucleus in local array... */
	
	total_neurons = (short *)calloc(1,total_nucleus*sizeof(short));	
	all_nucleus = network->nucleus_array;
	counter = 0;
#if _DEBUG_
			fprintf(stderr,"total_neurons: %x\n",total_neurons);
			fprintf(stderr,"all_nucleus: %x\n",all_nucleus);
#endif
	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		
		total_neurons[counter++] = nucleus_tmp->nb_neurons;
			
	}
	
	/* ok , it's time to open the anat file...*/
	
	if(!(fp=fopen(buffer,"rb")))
	{
#if _DEBUG_
		fprintf(stderr,"\nUnable to open %s in read mode !\nCan't read cnx file...\n",
			buffer);
#endif
		
		sprintf(tmp,"File %s doesn't exist yet.\nCan't read network...",
			buffer);
		display_error(widget_array[ID_MainW_network],tmp,
			"Read Error !");
		free(total_neurons);
		free(anat_total_neurons);
		fclose(fp);
		return -1;
	}
	
	
	/* As we read the header of the cnx file, we check that the data correspond
	to the one from the lnk file */
	
	fread(&anat_total_nucleus,sizeof(short),1,fp);
	
	if(anat_total_nucleus != total_nucleus)
	{
#if _DEBUG_
		fprintf(stderr,"\nSorry, bad number of nucleus: can't read anat file\n");
#endif
		free(total_neurons);
		fclose(fp);
		return -1;
	}

#if _DEBUG_
	printf("\nAmount of clusters in this network: %d\n",cnx_total_clusters);
#endif

	anat_total_neurons = (short *)calloc(1,anat_total_nucleus*sizeof(short));

	fread(anat_total_neurons,anat_total_nucleus*sizeof(short),1,fp);


	for(i=0 ; i<anat_total_nucleus ; i++)
	{
		if(anat_total_neurons[i] != total_neurons[i])
		{
#if _DEBUG_
			fprintf(stderr,
			"\nSorry, bad number of neurons,nucleus %d: can't read anat file\n",
				i);
#endif
			free(total_neurons);
			free(anat_total_neurons);
			fclose(fp);
			return -1;
		}
#if _DEBUG_
		printf("cluster %d: %d neurons\n",i,cnx_total_neurons[i]); 
#endif

	}
	/* Now we can read the neuron structures */

	counter = 0;

	for(i=0 ; i<anat_total_nucleus ; i++)
	{
		nucleus_tmp = all_nucleus[i];
		
		for(k=0 ; k<nucleus_tmp->nb_clusters ; k++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[k];
			neuron_array = cluster_tmp->neuron_array;
#if _DEBUG_
			fprintf(stderr,"neuron_array: %x\n",neuron_array);
			
#endif
			for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
			{
				neuron_tmp = neuron_array[j]; 
#if _DEBUG_
				fprintf(stderr,"neuron_tmp: %x\n",neuron_tmp);
				
#endif
				counter++;
				
				fread(&value,sizeof(short),1,fp);
				if(value != neuron_tmp->neuron_id)
				{
					fprintf(stderr,"bad neuron id in read anat !!!!\n");
					exit(-1);
				
				}
				
				fread(&cluster_id,sizeof(short),1,fp);
				
				if(cluster_id != cluster_tmp->cluster_id)
				{
					fprintf(stderr,"bad cluster id in read anat !!!!\n");
					exit(-1);
				
				}
				
				
				fread(&neuron_tmp->anatomy,sizeof(NEURON_ANATOMY),1,fp);
				

					
#if _DEBUG_
				printf("anatomy read...\n");
#endif				
							
		
			}
		}
	}	 	
	


	/* well...it seems to be ok...*/

	fclose(fp);
	free(total_neurons);
	free(anat_total_neurons);
	

#if _DEBUG_
	printf("\nOk: %s has been read.\n",network->name);
#endif
	return 0;
	
	
} /* end of read_anat_file */
 
 
 
#endif 
