#ifndef _READING_LINK_CONFIGURATION_FILE_VER2
#define _READING_LINK_CONFIGURATION_FILE_VER2

#ifndef _DEBUG_
#define _DEBUG_ 0
#endif


/*****************************************************************************

	Object: reads from a link configution file format 2.0 into a network
		struture from NBC Network Editor
				
******************************************************************************/


/* general remarks about reading an whole network structure:
		there is a special order to read the different types of file,
		first the lnk file must be read, then the cnx file and finaly
		the anat file.
		The reason for this is the order of allocating the internal
		network structures such as the nucleus, cluster or neuron 
		structures.					*/



/* usual includes */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif


/* prototypes of functions implemented in this file */
int read_lnk_file();



/*----------------------------------------------------------------------------
 *  R E A D _L N K _ F I L E 
 *---------------------------------------------------------------------------*/

int read_lnk_file(Network *network)
{
	int	i, j;
	FILE *file_p;
	char lnk_name[255];
	char buffer[255];
	char field[255];
	char value[255];
	Nucleus_widget *nucleus_tmp;
	LINK_PARAMETERS *IC_link_tmp,*IN_link_tmp;
	CLUSTER_WIDGET *cluster_tmp;
	int new_nucleus;
	int new_cluster;
	int new_IC_link;
	int new_IN_link;
	Nucleus_widget **nucleus_array;
	CLUSTER_WIDGET **cluster_array;
	LINK_PARAMETERS **IC_link_array;
	LINK_PARAMETERS **IN_link_array;
	int num;
	
	/* the network structure must be allocated and contain the lnk file name */
	
	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nNetwork structure must be allocated before reading lnk file...\n");
#endif		
		return -1;
	}
	
	/* recovering the lnk file name from the network structure */
	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nLnk file name must be present in the network structure...\n");
#endif	
		return -1;
	}
	else
	{
		strcpy(lnk_name,project_directory);
		strcat(lnk_name,"/");
		strcat(lnk_name,network->name);
		strcat(lnk_name,".ntw");	
	}
	
	/* opening the lnk file in read character mode */
	
	file_p = fopen (lnk_name, "r");
	
	/* testing file pointer , in case of failure -> return -1*/
	
	if(!file_p)
	{
#if _DEBUG_
		fprintf(stderr,"\nCan't open %s...\n",lnk_name);
#endif	
		sprintf(buffer,"File %s doesn't exist yet.\nEdit and save network to create it...",
			lnk_name); 
		display_message(widget_array[ID_MainW_network],buffer,"New File");
		return -1;
	}

	/* before filling the network structure , this one must be empty
		but it has to be done before calling this function, so
		as we want the program to be clean with allocating, if the
		network isn't empty yet, -> exiting with an error message */
		
	if(network->nucleus_array || network->link_array)
	{
		fprintf(stderr,"\nSorry, there is a bug: network structure isn't empty...\nLoading network in 'read_lnk_file' fails...\n");
		exit(-1);
	}	

	/* type of file */
	fgets(buffer,255,file_p);
	sscanf(buffer,"%s %s",field,value);
	if(strcmp("TypeOfFile:",field))
	{
#if _DEBUG_
		fprintf(stderr,"\n%s isn't a XNBC link configuration file...\n",lnk_name);
#endif	
		display_message(widget_array[ID_MainW_network],buffer,"This is not a XNBC link configuration file...");
		return -1;
	}
	if(strcmp("LinkConfigurationFileVer2",value))
	{
#if _DEBUG_
		fprintf(stderr,"\n%s isn't lnk format 2.0 version...\n",lnk_name);
#endif	
		display_message(widget_array[ID_MainW_network],buffer,"This is not a  lnk format 2.0 version...");
		return -1;	
	}
	fgets(buffer,255,file_p);
	sscanf(buffer,"%s %s",field,value);
	if(!strcmp("Creator:",field))
	{
		if(!strcmp("SimpleEditor",value))
		{
			display_message(widget_array[ID_MainW_network],
					"This file was created by the Simple Network Editor\nUse the Simple Network Editor to modify it!",
					"WARNING");
			return -1;
		}
	}

	/* now let's read the lnk file */

#if _DEBUG_
			fprintf(stderr,"net_activate_CB: %x\n\n",net_activate_CB);
#endif
	
	while(fgets(buffer,255,file_p))
	{
		sscanf(buffer,"%s %s",field,value);
		
/*************** first testing network specs ***************************/

		if(!strcmp("NumberOfNucleus:",field))
		{
			sscanf(value,"%d",&num);
			network->nb_nucleus = (short)num;
			network->nucleus_array = (Nucleus_widget **)
				calloc(1,network->nb_nucleus*sizeof(Nucleus_widget *));
			nucleus_array = network->nucleus_array;
#if _DEBUG_
			fprintf(stderr,"network->nucleus_array: %x\n",nucleus_array);
#endif
			
			new_nucleus = 0;
		}
		else if(!strcmp("NumberOfInterNucleusLink:",field))
		{
			sscanf(value,"%d",&num);
			network->nb_links = (short)num;
			network->link_array = (LINK_PARAMETERS **)
				calloc(1,network->nb_links*sizeof(LINK_PARAMETERS *));
			IN_link_array = network->link_array;
#if _DEBUG_
			fprintf(stderr,"network->link_array: %x\n",IN_link_array);
#endif			
			
			new_IN_link = 0; 
		}
		else if(!strcmp("TotalNeuronsInNetwork:",field))
		{
			sscanf(value,"%d",&num);
			network->nb_neurons = (short)num;
		}
		

/******************* nucleus specs *************************/

		else if(!strcmp("BeginNucleus:",field))
		{
			nucleus_tmp = nucleus_array[new_nucleus] = (Nucleus_widget *)
				calloc(1,sizeof(Nucleus_widget));
			nucleus_tmp->number = new_nucleus+1;
			
			for(i=0 ; i<3 ; i++)
			{
				nucleus_tmp->work_width[i] = AREA_SIZE;
				nucleus_tmp->work_height[i] = AREA_SIZE;
				nucleus_tmp->origin_x[i] = AREA_SIZE/2;
				nucleus_tmp->origin_y[i] = AREA_SIZE/2;
			}
			
#if _DEBUG_
			fprintf(stderr,"nucleus_tmp: %x\n",nucleus_tmp);
#endif			
			
			 
		}
		else if(!strcmp("NucleusName:",field))
		{
			strcpy(nucleus_tmp->nucleus_name,value);
		}
		else if(!strcmp("NucleusId:",field))
		{
			sscanf(value,"%d",&num);
			nucleus_tmp->nucleus_id = manage_nucleus_id(num,True);
		}
		else if(!strcmp("NumberOfClusters:",field))
		{
			sscanf(value,"%d",&num);
			nucleus_tmp->nb_clusters = (short)num;
			
			nucleus_tmp->cluster_array = (CLUSTER_WIDGET **)
				calloc(1,nucleus_tmp->nb_clusters*sizeof(CLUSTER_WIDGET *));
			cluster_array = nucleus_tmp->cluster_array;
			new_cluster = 0;
#if _DEBUG_
			fprintf(stderr,"nucleus_tmp->cluster_array: %x\n",nucleus_tmp->cluster_array);
#endif			
			
			
		}
		else if(!strcmp("NumberOfInterClusterLink:",field))
		{
			sscanf(value,"%d",&num);
			nucleus_tmp->nb_links = (short)num;
			
		    nucleus_tmp->link_array = (LINK_PARAMETERS **)
				calloc(1,nucleus_tmp->nb_links*sizeof(LINK_PARAMETERS *));
			IC_link_array = nucleus_tmp->link_array;
			new_IC_link = 0;
			
#if _DEBUG_
			fprintf(stderr,"nucleus_tmp->link_array: %x\n",nucleus_tmp->link_array);
#endif			
		}
		else if(!strcmp("TotalNeuronsInNucleus:",field))
		{
			sscanf(value,"%d",&num);
			nucleus_tmp->nb_neurons = (short)num;
			if(nucleus_tmp->nb_neurons == 1)
			{
				nucleus_tmp->form = XtVaCreateWidget (nucleus_tmp->nucleus_name,
					    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
					    XmNwidth, NEURON_WIDTH,
					    XmNheight, NEURON_HEIGHT,
					    XmNborderWidth, 0,
					    XmNrecomputeSize, False,
					    NULL);
				
#if _DEBUG_
			fprintf(stderr,"nucleus_tmp->form: %x\n",nucleus_tmp->form);
#endif				
					   
				nucleus_tmp->bgnd = XCreatePixmap(the_display,
				 	XtWindow(widget_array[ID_DrawingA_net]),NEURON_WIDTH,NEURON_HEIGHT,
				 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_net])));
				 	
				XSetForeground(the_display,the_GC,white);
				XSetBackground(the_display,the_GC,black);
				XFillRectangle(the_display, nucleus_tmp->bgnd,the_GC,
					0,0,NEURON_WIDTH,NEURON_HEIGHT);
				 
				XtVaSetValues(nucleus_tmp->form,XmNbackgroundPixmap,
					nucleus_tmp->bgnd,NULL);
			}
			else
			{
				nucleus_tmp->form = XtVaCreateWidget (nucleus_tmp->nucleus_name,
					    xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_net],
					    XmNwidth, NUCLEUS_WIDTH,
					    XmNheight, NUCLEUS_HEIGHT,
					    XmNborderWidth, 0,
					    XmNrecomputeSize, False,
					    NULL);
#if _DEBUG_
			fprintf(stderr,"nucleus_tmp->form: %x\n",nucleus_tmp->form);
#endif					   
					   
					   
				nucleus_tmp->bgnd = XCreatePixmap(the_display,
				 	XtWindow(widget_array[ID_DrawingA_net]),NUCLEUS_WIDTH,NUCLEUS_HEIGHT,
				 	DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_net])));
				 	
				XSetForeground(the_display,the_GC,white);
				XSetBackground(the_display,the_GC,black);
				XFillRectangle(the_display, nucleus_tmp->bgnd,the_GC,
					0,0,NUCLEUS_WIDTH,NUCLEUS_HEIGHT);
				 
				XtVaSetValues(nucleus_tmp->form,XmNbackgroundPixmap,
					nucleus_tmp->bgnd,NULL);
			}

			XtAddEventHandler(nucleus_tmp->form,ButtonPressMask,False,
				(XtEventHandler)select_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonReleaseMask,False,
				(XtEventHandler)release_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonMotionMask,False,
				(XtEventHandler)chplace_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,ButtonMotionMask,False,
				(XtEventHandler)info_nucleus,nucleus_tmp->nucleus_id);	
			XtAddEventHandler(nucleus_tmp->form,EnterWindowMask,False,
				(XtEventHandler)info_nucleus,nucleus_tmp->nucleus_id);
			XtAddEventHandler(nucleus_tmp->form,LeaveWindowMask,False,
				(XtEventHandler)clear_label_net_info,
				nucleus_tmp->nucleus_id);
			nucleus_tmp->color = grey;

		}
		else if(!strcmp("NucleusPositionH:",field))
		{
		
		      sscanf(value,"%f",&nucleus_tmp->H);
		}
		else if(!strcmp("NucleusDeltaH:",field))
		{
		      sscanf(value,"%f",&nucleus_tmp->delta_H);
		}
		else if(!strcmp("NucleusPositionL:",field))
		{
		     sscanf(value,"%f",&nucleus_tmp->L);
		}
		else if(!strcmp("NucleusDeltaL:",field))
		{
		     sscanf(value,"%f",&nucleus_tmp->delta_L);
		}
		else if(!strcmp("NucleusPositionAP:",field))
		{
		     sscanf(value,"%f",&nucleus_tmp->AP);
		}
		else if(!strcmp("NucleusDeltaAP:",field))
		{
		     sscanf(value,"%f",&nucleus_tmp->delta_AP);
		}
 
/**************** cluster specs ***********************************/

		else if(!strcmp("BeginCluster:",field))
		{
		    cluster_tmp = cluster_array[new_cluster] = (CLUSTER_WIDGET *)
			calloc(1,sizeof(CLUSTER_WIDGET));
			cluster_tmp->number = new_cluster+1;
#if _DEBUG_
			fprintf(stderr,"cluster_tmp: %x\n",cluster_tmp);
#endif
			
		}
		else if(!strcmp("ClusterName:",field))
		{
		        strcpy(cluster_tmp->ch_name,value);
		}
		else if(!strcmp("ClusterId:",field))
		{
		     sscanf(value,"%d",&num);  
		     cluster_tmp->cluster_id = manage_cluster_id(num,True);
		     
		}
		else if(!strcmp("PositionH:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->H);
		}
		else if(!strcmp("DeltaH:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->delta_H);
		}
		else if(!strcmp("PositionL:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->L);
		}
		else if(!strcmp("DeltaL:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->delta_L);
		}
		else if(!strcmp("PositionAP:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->AP);
		}
		else if(!strcmp("DeltaAP:",field))
		{
		        sscanf(value,"%f",&cluster_tmp->delta_AP);
		}
		else if(!strcmp("NbNeurons:",field))
		{
				sscanf(value,"%d",&num);
		        cluster_tmp->nb_neurons = (short)num;
		}
		else if(!strcmp("NeuronFile:",field))
		{
		        cluster_tmp->neuron_file = (char *)calloc(1,strlen(value)+1);
			strcpy(cluster_tmp->neuron_file,value); 
#if _DEBUG_
			fprintf(stderr,"cluster_tmp->neuron_file: %x\n",cluster_tmp->neuron_file);
#endif 
		}
		else if(!strcmp("EndCluster:",field))
		{
		         new_cluster++;
		}

/****************************** ICluster link specs *************************/

		else if(!strcmp("BeginLink:",field))
		{
		        IC_link_tmp = IC_link_array[new_IC_link] = (LINK_PARAMETERS *)
			      calloc(1,sizeof(LINK_PARAMETERS));
			    IC_link_tmp->level = _NUCLEUS_;
			      
#if _DEBUG_
			fprintf(stderr,"IC_link_tmp: %x\n",IC_link_tmp);
#endif			     
		}
		else if(!strcmp("Cluster1:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->source_id = (short)num;
		}
		else if(!strcmp("Cluster2:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->target_id = (short)num;
		}
		else if(!strcmp("AxoneMean:",field))
		{
		        sscanf(value,"%f",&IC_link_tmp->axone_mean);
		}
		else if(!strcmp("AxoneSd:",field))
		{
		        sscanf(value,"%f",&IC_link_tmp->axone_sd);
		}
		else if(!strcmp("BornMin:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->min_knobs = (short)num;
		}
		else if(!strcmp("BornMax:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->max_knobs = (short)num;
		}
		else if(!strcmp("Type:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->type = (short)num;
		}
		else if(!strcmp("Nmda:",field))
		{
		        sscanf(value,"%d",&num);
		        IC_link_tmp->nmda = (short)num;
		}
		else if(!strcmp("EndLink:",field))
		{
		        new_IC_link++;
		}
		else if(!strcmp("EndNucleus:",field))
		{
		        new_nucleus++;
		}

/*******************INucleus link specs **********************************/

		else if(!strcmp("BeginLinkInterNucleus:",field))
		{
		       IN_link_tmp  = IN_link_array[new_IN_link] = (LINK_PARAMETERS *)
			      calloc(1,sizeof(LINK_PARAMETERS));
			   IN_link_tmp->level = _NETWORK_;
#if _DEBUG_
			fprintf(stderr,"IN_link_tmp: %x\n",IN_link_tmp);
#endif			      
			      
		}
		else if(!strcmp("Nucleus1:",field))
		{
		       sscanf(value,"%d",&num);
		       IN_link_tmp->source_id = (short)num;
		}
		else if(!strcmp("Nucleus2:",field))
		{
		       sscanf(value,"%d",&num);
		       IN_link_tmp->target_id = (short)num;
		}
		else if(!strcmp("INAxoneMean:",field))
		{
		        sscanf(value,"%f",&IN_link_tmp->axone_mean);
		}
		else if(!strcmp("INAxoneSd:",field))
		{
		        sscanf(value,"%f",&IN_link_tmp->axone_sd);
		}
		else if(!strcmp("INBornMin:",field))
		{
		        sscanf(value,"%d",&num);
		        IN_link_tmp->min_knobs = (short)num;
		}
		else if(!strcmp("INBornMax:",field))
		{
		        sscanf(value,"%d",&num);
		        IN_link_tmp->max_knobs = (short)num;
		}
		else if(!strcmp("INType:",field))
		{
		        sscanf(value,"%d",&num);
		        IN_link_tmp->type = (short)num;
		}
		else if(!strcmp("INNmda:",field))
		{
		        sscanf(value,"%d",&num);
		        IN_link_tmp->nmda = (short)num;
		}
		else if(!strcmp("EndLinkInterCluster:",field))
		{
		        new_IN_link++;
		}
		
	}
	

	/* Ok seems to be 'well done' */
	return 0;
	

} /* fin de read_lnk_file */


#endif
