#ifndef _READING_WGT_FILE_
#define _READING_WGT_FILE_

#define _DEBUG_ 0

/*--------------------------------------------------------------------------*
 *	The aim of this file is to read a wgt file format ver2 NBC Network      *
 *  Editor , and store it in the specified network ver2 structure.          *
 *--------------------------------------------------------------------------*/
 
/* Remarks: the network structure must have been loaded first with the
corresponding lnk file and most important is that cluster elements of
the network have to know their number since the wgt file will be read 
sequentially. */


/* general remarks about reading an whole network structure:
		there is a special order to read the different types of file,
		first the lnk file must be read, then the wgt and len files and finaly
		the anat file.
		The reason for this is the order of allocating the internal
		network structures such as the nucleus, cluster or neuron 
		structures.					*/


/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* application specific includes */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif


/* prototypes of functions implemented in this file */
int read_wgt_file();

/*--------------------------------------------------------------------------*
 *   	R E A D _ W G T _ F I L E	                                        *
 *--------------------------------------------------------------------------*/
int read_wgt_file(Network *network)
{
	int i,j,k,p,counter;
	short total_nucleus;
	short wgt_total_nucleus;
	short *total_neurons;
	short *wgt_total_neurons;
	Nucleus_widget **all_nucleus;
	FILE *fp;
	char buffer[255];
	char tmp[256];
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array, *cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	short cluster_id;
	
	
	if(!network)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad Network ! can't read cnx file !\n");
#endif
		return -1;
	}	
	if(!*network->name)
	{
#if _DEBUG_
		fprintf(stderr,"\nCurrent Network hasn't any name ! can't read cnx file !\n");
#endif
		return -1;
	}
	
	/* get the cnx file name */

	strcpy(buffer,project_directory);
	strcat(buffer,"/");
	strcat(buffer,network->name);
	strcat(buffer,".wgt");


	/* test the network: it must already have the informations from 
	the lnk file, only the neuron array from each cluster has to be filled */
	
	if(!network->nb_nucleus || !network->nb_neurons)
	{
#if _DEBUG_
		fprintf(stderr,"\nBad nucleus or neuron number: can't read cnx file...\n");
#endif
		return -1;
	} 	 

	nucleus_array = network->nucleus_array;
	
	if(!nucleus_array)
	{
#if _DEBUG_
		fprintf(stderr,"\nNucleus array empty: can't read cnx file...\n");
#endif
		return -1;
	}

	/* now let's count the total number of nucleus in the network */

	total_nucleus = network->nb_nucleus;

	/* get the total neuron per nucleus... reference all nucleus in local array... */
	
	total_neurons = (short *)calloc(1,total_nucleus*sizeof(short));	
	all_nucleus = network->nucleus_array;
	counter = 0;
#if _DEBUG_
			fprintf(stderr,"total_neurons: %x\n",total_neurons);
			fprintf(stderr,"all_nucleus: %x\n",all_nucleus);
#endif
	for(i=0 ; i<network->nb_nucleus ; i++)
	{
		nucleus_tmp = nucleus_array[i];
		
		total_neurons[counter++] = nucleus_tmp->nb_neurons;
			
	}
	
	/* ok , it's time to open the wgt file...*/
	
	if(!(fp=fopen(buffer,"rb")))
	{
#if _DEBUG_
		fprintf(stderr,"\nUnable to open %s in read mode !\nCan't read cnx file...\n",
			buffer);
#endif
		
		sprintf(tmp,"File %s doesn't exist yet.\nCan't read network...",
			buffer);
		display_error(widget_array[ID_MainW_network],tmp,
			"Read Error !");
		free(total_neurons);
		free(wgt_total_neurons);
		fclose(fp);
		return -1;
	}
	
	
	/* As we read the header of the cnx file, we check that the data correspond
	to the one from the lnk file */
	
	fread(&wgt_total_nucleus,sizeof(short),1,fp);
	
	if(wgt_total_nucleus != total_nucleus)
	{
#if _DEBUG_
		fprintf(stderr,"\nSorry, bad number of nucleus: can't read wgt file\n");
#endif
		free(total_neurons);
		fclose(fp);
		return -1;
	}

#if _DEBUG_
	printf("\nAmount of clusters in this network: %d\n",cnx_total_clusters);
#endif

	wgt_total_neurons = (short *)calloc(1,wgt_total_nucleus*sizeof(short));

	fread(wgt_total_neurons,wgt_total_nucleus*sizeof(short),1,fp);


	for(i=0 ; i<wgt_total_nucleus ; i++)
	{
		if(wgt_total_neurons[i] != total_neurons[i])
		{
#if _DEBUG_
			fprintf(stderr,
			"\nSorry, bad number of neurons,nucleus %d: can't read wgt file\n",
				i);
#endif
			free(total_neurons);
			free(wgt_total_neurons);
			fclose(fp);
			return -1;
		}
#if _DEBUG_
		printf("cluster %d: %d neurons\n",i,cnx_total_neurons[i]); 
#endif

	}
	/* Now we can read the neuron structures */

	counter = 0;

	for(i=0 ; i<wgt_total_nucleus ; i++)
	{
		nucleus_tmp = all_nucleus[i];
		
		for(k=0 ; k<nucleus_tmp->nb_clusters ; k++)
		{
			cluster_tmp = nucleus_tmp->cluster_array[k];
			neuron_array = cluster_tmp->neuron_array = (NEURON **)calloc(1,
				cluster_tmp->nb_neurons*sizeof(NEURON *));
#if _DEBUG_
			fprintf(stderr,"neuron_array: %x\n",neuron_array);
			
#endif
			for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
			{
				neuron_tmp = neuron_array[j] = (NEURON *)calloc(1,
					sizeof(NEURON));
#if _DEBUG_
				fprintf(stderr,"neuron_tmp: %x\n",neuron_tmp);
				
#endif
				counter++;
				
				fread(&neuron_tmp->neuron_id,sizeof(short),1,fp);
				neuron_tmp->neuron_id = manage_neuron_id(neuron_tmp->neuron_id,True);
				
				fread(&cluster_id,sizeof(short),1,fp);
				
				if(cluster_id != cluster_tmp->cluster_id)
				{
					fprintf(stderr,"bad cluster id in read wgt !!!!\n");
					exit(-1);
				
				}
				
				sprintf(buffer,"Neuron%d",neuron_tmp->neuron_id);	
				neuron_tmp->form = XtVaCreateWidget (buffer,
						xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_nucleus_view],
						XmNwidth, NEURON_WIDTH,
						XmNheight, NEURON_HEIGHT,
						XmNborderWidth, 0,
						XmNrecomputeSize, False,
						NULL);
#if _DEBUG_
				fprintf(stderr,"neuron_tmp->form: %x\n",neuron_tmp->form);
				
#endif


				neuron_tmp->bgnd = XCreatePixmap(the_display,
					XtWindow(widget_array[ID_DrawingA_net]),NEURON_WIDTH,NEURON_HEIGHT,
					DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_nucleus_view])));

				XSetForeground(the_display,the_GC,white);
				XSetBackground(the_display,the_GC,black);
				XFillRectangle(the_display, neuron_tmp->bgnd,the_GC,
					0,0,NEURON_WIDTH,NEURON_HEIGHT);

				XtVaSetValues(neuron_tmp->form,XmNbackgroundPixmap,
					neuron_tmp->bgnd,NULL);
					
				XtAddEventHandler(neuron_tmp->form,ButtonPressMask,False,
					(XtEventHandler)select_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonReleaseMask,False,
					(XtEventHandler)release_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
					(XtEventHandler)chplace_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,ButtonMotionMask,False,
					(XtEventHandler)info_neuron,neuron_tmp->neuron_id);	
				XtAddEventHandler(neuron_tmp->form,EnterWindowMask,False,
					(XtEventHandler)info_neuron,neuron_tmp->neuron_id);
				XtAddEventHandler(neuron_tmp->form,LeaveWindowMask,False,
					(XtEventHandler)clear_label_nucl_info,
					neuron_tmp->neuron_id);
					
				neuron_tmp->color = grey;

			
#if _DEBUG_
				printf("\nneuron id: %d\n",neuron_tmp->neuron_id);
#endif

				
				/* read the number of connected neurons and the 
					following connections */

				
				fread(&neuron_tmp->connected_neurons,sizeof(short),1,fp);
				
#if _DEBUG_
				printf("neurons connected: %d\n",neuron_tmp->connected_neurons);
#endif
				neuron_tmp->connections = (Connection *)calloc(1,
					neuron_tmp->connected_neurons*sizeof(Connection));

				for(p=0 ; p<neuron_tmp->connected_neurons ; p++)
				{
					fread(&neuron_tmp->connections[p].target_id,
						sizeof(short),1,fp);
					fread(&neuron_tmp->connections[p].nmda,
						sizeof(short),1,fp);
					fread(&neuron_tmp->connections[p].synaptic_knobs,
						sizeof(short),1,fp);
				}
					
#if _DEBUG_
				printf("connections read...\n");
#endif				
							
		
			}
		}
	}	 	
	


	/* well...it seems to be ok...*/

	fclose(fp);
	free(total_neurons);
	free(wgt_total_neurons);
	

#if _DEBUG_
	printf("\nOk: %s has been read.\n",network->name);
#endif
	return 0;
	
	
} /* end of read_wgt_file */
 
 
 
#endif 
