#define PSFIG   1
/*
                        P  L  O  T  1  _  P  S
                        ======================

                   Routines de dessin sur la LN03R PostScript.

           NIVEAU 1 : Ce jeu de routine regroupe les routines de base
                appelables par l'utilisateur de base

                Mise en forme Version 2
                        Juillet 90
                Version 3       Nouveaux noms de fonctions ps_*
                        31 juillet 90
                Version 4.2
                        11 oct 91
                Version 4.3
                        11 aout 97
		Version 6.0	prototypage
			19 jan 99
		Version 6.1	prise en compte de PRINT_PLOT_PS
			1 sept 99
		Version 6.2	redefinition de M_PI (pas defini sur HPUX...)
			6 oct 99

*
*       Auteur: Dr J-F VIBERT
*               Laboratoire de Physiologie
*               CHU Saint-Antoine
*               27 rue Chaligny
*               75571 Paris cedex 12
*                       France
*
*       Copyright Dr JF Vibert 
*       All rights reserved
*       Access limited to authorized persons    
*       This software was developped only for resarch purpose
*       and is in any way usable for commercial purpose.
*
title   PLOT1   Niveau 1 des routines PLOT_PS pour C
index           Niveau 1 des routines PLOT_PS pour C

description

                   Routines de dessin sur la PLOT_PS.

        NIVEAU 1 : Ce jeu de routine regroupe les routines 
                appelables par l'utilisateur


author

        Jean-Francois Vibert

        CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

        PLOT0 et PLOT2

newpage
*/

#include <stdlib.h>
#include <stdio.h>

#include "plot_ps.h"

#ifdef  DOCUMENTATION


title   PS_INIPLOT      Initialize the PostScript plotter 
index	   Initialize the PostScript plotter 


synopsis
	#include <plot_ps.h>

	ps_iniplot(plot_file,paper);
	char   *plot_file;      /* File for the plot */
	float   paper;	  /* Paper width in cm */

description

	PS_INIPLOT opens the PostScript file and draws a frame corresponding
	to the flag page of a line-printer. The date and time of the file
	production, the file name and the user name are indicated as well as
	a page number. This banner is output on each page by pl_newpage().
	It is drawn at the bottom of the page and is 1 cm wide.

author

	Dr Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	plopen() and plclose()

newpage
	

title   PS_ENDPLOT      Close and spools the PostScript file
index	   Close and spools the PostScript file


synopsis
	#include <plot_ps.h>

	ps_endplot(file,action);
	char    *file;  /* nom du fichier dessin */
		action; /* Action to take about the plot file
				s: spool and keep
				d: spool and delete
				f: freeze (no spool) */
				
	ps_queue_name(nom_de_la_queue)
	char    *nom_de_la_queue; /* queue name where the plot
				     will be sent */

description

	PS_ENDPLOT closes the PostScript file and process it according
	to the action indicated by the action argument:

		s: spool and keep

		d: spool and delete

		f: freeze (no spool)

	PS_QUEUE_NAME allows to change the name of the queue where the plot
	will be sent. It default to the name defined in plot_ps.h as
	QUEUE_NAME, or the environment variable PRINT_PLOT_PS if it exists.

author

	Dr Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	plopen() and plclose()
newpage


title   PS_NEWPAGE      Go to the next page
index	   Go to the next page


synopsis
	#include <plot_ps.h>

	ps_newpage()
description

	PS_NEWPAGE go to the next page and redraws the banner with the page
	number.

author

	Dr Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also


newpage


title   PS_BANNER      Put or remove a banner on each printed page
index	   Put or remove a banner on each printed page

synopsis
	#include <plot_ps.h>

	ps_banner()

	#include <plot_ps.h>

	ps_no_banner()

description

	PS_BANNER puts a banner on each printed page. 
	PS_NO_BANNER remove the banner on each printed page. 
	The banner has the page number.

        WARNING: If no banner is needed, even on the first page, 
        ps_no_banner must be called before ps_iniplot.

        the banner is written by default, except if at the building 
        time the PUT_BANNER_ON_PAGES option was set. This building flag 
        only changes the default value of the flag controlling the banner. 

author

	Dr Jean-Francois Vibert

	CHU Saint-Antoine INSERM U444 Universite Paris VI

See also


newpage

title   PS_ORIGIN       Define new origine and move the pen
index	   Define new origine and move the pen

synopsis

	#include <plot_ps.h>

	ps_origin(xf,yf,xa,ya);
	float   xf, yf, /*
			 * Absolute coordinate of the new position relative to
			 * the old one, in cm.
			 */
		xa, ya; /*
			 * Absolute coordinates of this new position
			 * in the new coordinates, en cm.
			 */

description

	PS_ORIGIN defines a new absolute origine. The pen if first moved to xf
	and yf coordinates. xa and ya corespond to the new coordinates of
	the pen.

	All coordinates are given in centimeters (also refered as PostScipt
	space).

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_ECHEL	Define scale of drawings (subject versus cms)
index	   Define scale of drawings (subject versus cms)

synopsis
	#include <plot_ps.h>

	ps_echel(ex,ey,xs,ys,ca,sa);
	float   ex, ey, /* Number of cms by user unit   */
		xs, ys, /* Absolute (en cm) coordinates 
					of user origine */
		ca, sa; /* Cos and Sin of the drawing
				       (rotation)       */
description

	PS_ECHEL Define the scale of the drawing, and its origin in the
	PostScript coordinates (relative to the last call to ps_origin). Its
	allows also rotation of the entire drawing (but does not rotate texts).

		The scales must be given in centimeters by user (or subject)
	unit. They must have a value greater than 0.008 since  PostScript
	draws one centimeter using 118 steps. If not, the function returns
	FALSE. 

		The coordinates of the new origine must be given in
	PostScript units, i.e. in centimeters.

		Cosinus and Sinus of the angle of the drawing relative to
	the X axis gives a rotation possibility. These parameters are here
	more for compatibility reason than for real use, since {\tt
	ps_rotate} is more general. Use these parameters only when the
	rotation is needed for drawings only.
		
	\nofill
	Horizontal head up: cosa=1. sina=0.
	Horizontal head down: cosa=-1. sina=0.
	Verticalement ascending: cosa=0. sina=1.
	Verticalement descending: cosa=0. sina=-1.
	\fill

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_SCALE Scale of drawings
index	    Scale of drawings

synopsis
	#include <plot_ps.h>

	ps_scale(x_scl,y_scl)
	float   x_scl,
		y_scl;
description

	PS_SCALE set a new global  scale of the drawing, with uniform (if 
	x_scl == y_scl) or non-uniform (if x_scl != y_scl) scaling. It acts 
	on both the drawing and texts. it allows to change the proportion of
	the drawing as well as its size. To come back to the original 
	scale, ps_scale must be called with 1./x_scl and 1./y_scl.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_PEN  Change the drawing pen 
index	   Change the drawing pen 

synopsis
	#include <plot_ps.h>

	ps_pen(k);
	int k;    /* pen number (0, 1 or 2) */
description

	PS_PEN selects the drawing pen. If the pen number (k) is not one of 
	0, 1 or 2, the function returns FALSE, else it returns TRUE.
	If ps_pen() is not called, the default pen is 0 (BLACK).

	\nofill
	0 gives a black
	1 gives a gray at 0.3
	2 a light gray at 0.7.
	\fill

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
	
newpage

title   PS_ROTATE       Allows a rotation in degres
index	   Allows a rotation in degres

synopsis
	#include <plot_ps.h>

	ps_rotate(rot);
	int      rot;  /* Rotation in degres */
description

	PS_ROTATE() Allows a rotation in degres of the graph and the texts.
	Only the subsequent drawings are rotated. This rotation is more
	general that those obtained with the COS and Sin parameters of {\tt
	ps_echel}.
	
author

	Jean-Francois

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_FONT Allows to change the font
index	   Allows to change the font

synopsis

	#include <plot_ps.h>

	ps_font(font,hy);
	char    *font;  /* font name */
	float   hy;     /* height of the font in cm */

description

	PS_FONT allows to choose a new font as the current font. The first
	argument is the PostScript build in font name. The size of the font 
	is given in centimer (hy). It can be changed alone by ps_size().
	A size value of 0. keeps the current size of he font.

	Legal font names are:

	\nofill
		AvantGarde-Book
		AvantGarde-BookOblique
		AvantGarde-Demi
		AvantGarde-DemiOblique
		Courier
		Courier-Bold
		Courier-Oblique
		Courier-BoldOblique
		Helvetica
		Helvetica-Bold
		Helvetica-Oblique
		Helvetica-BoldOblique
		LubalinGraph-Book
		LubalinGraph-BookOblique
		LubalinGraph-Demi
		LubalinGraph-DemiOblique
		Times-Roman
		Times-Bold
		Times-Italic
		Times-BoldItalic
		NewCenturySchlbk-Roman
		NewCenturySchlbk-Bold
		NewCenturySchlbk-Italic
		NewCenturySchlbk-BoldItalic
		Souvenir-Demi
		Souvenir-DemiItalic
		Souvenir-Light
		Souvenir-LightItalic
		Symbol
	\fill
author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_SIZE Define size (in cm)  of characters 
index	   Define size (in cm)  of characters 

synopsis
	#include <plot_ps.h>

	ps_size(hx);
	float   hx;    /* Size of height 
			of characters given 
			in centimeters */
description

	PS_SIZE defines the size of characters, given in centimeters,
	and acts on he current font choosen by ps_police().

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage

title   PS_CHARA	Write a string in PostScipt space, stays at the end
index	   Write a string in PostScipt space, stays at the end

synopsis
	#include <plot_ps.h>

	ps_chara(x,y,j,str,hy,rot);
	float   x, y;    /* x and y coordinates */
			 /*    given in PostScipt units (cm) */ 
	int     j;       /* the "Benson" j parameter */ 
	char    *str;    /* the string to write */ 
	float   hy;      /* character height (in cm)  */
	int     rot;     /* rotation in degres */
description

	PS_CHARA writes a string at the position indicated by x,y and j, 
	with the rotation given by cosa and sina, and stays at the end
	of the string.

	PS_CHARA replicates the old Benson pcara fortran routine.

	PS_WRITE\ldots should now be used.
	
		X and Y are the coordinates of the target given in PostScipt
	units (in centimeters).
	
		These coordinates are considered as absolute (j=0 or j=1) or
	relative (j=2 or j=3) according to the value of j.      

		During the move, the pen is up (j=0 or j=2) or down
	(j=1 or j=3) according to the value of j.

	The following constants are defined in PLOT_PS.H:

	Symbolic codes for the pen movements
	 
	\nofill
	\#define PL_AH 0		Absolu Plume Haute      
	\#define PL_AB 1		Absolu Plume Basse      
	\#define PL_RH 2		Relatif Plume Haute     
	\#define PL_RB 3		Relatif Plume Basse     

	\#define PEN_AU 0	       Absolute Pen Up
	\#define PEN_AD 1	       Absolute Pen Down       
	\#define PEN_RU 2	       Relative Pen Up 
	\#define PEN_RD 3	       Relative Pen Down       
	\fill

	The size of characters is given in centimeters by  hy 
	(height) of the characters.

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	All text writing functions
 
newpage

title   PS_CHARS	Write a string in user space, stays at the end
index	   Write a string in user space, stays at the end

synopsis
	#include <plot_ps.h>

	ps_chars(x,y,j,str,hy,rot);
	float   x, y;   /* x and y coordinates */
			/*     given in user units */ 
	int     j;      /* the "Benson" j parameter */
	char    *str;   /* the string to write */ 
	float   hy;     /* character height (in cm) */ 
	int     rot;    /* rotation in degres */
description

	PS_CHARS writes a string at the position indicated by x,y and j, 
	with the rotation given in degre, and stays at the end
	of the string.

	PS_CHARS almost replicates the opld Benson pcars fortran routine.

	PS_WRITE\ldots should now be used.

	X and Y are the coordinates of the target given in user
	units (NOT in centimeters).

		These coordinates are considered as absolute (j=0 or j=1) or
	relative (j=2 or j=3) according to the value of j. 

		During the move, the pen is up (j=0 or j=2) or down
	(j=1 or j=3) according to the value of j.

	The following constants are defined in PLOT.H:
	
	Symbolic codes for the pen movements

	\nofill
	\#define PL_AH 0		Absolu Plume Haute      
	\#define PL_AB 1		Abslou Plume Basse      
	\#define PL_RH 2		Relatif Plume Haute     
	\#define PL_RB 3		Relatif Plume Basse     

	\#define PEN_AU 0	       Absolute Pen Up
	\#define PEN_AD 1	       Absolute Pen Down       
	\#define PEN_RU 2	       Relative Pen Up 
	\#define PEN_RD 3	       Relative Pen Down       
	\fill

	The size of characters is given in centimeters by hy (height) of the
	characters.
 

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	All text writing functions PS_WRITE*
 

newpage

title   PS_GRAY Define a new default gray level
index	   Define a new default gray level

synopsis
	#include <plot_ps.h>

	ps_gray(gray);
	float     gray; /* gray for the subsequent drawings */
description

	PS_GRAY() defines a new gray value for the subsequent drawings.
	{\tt gray} must be between 0 and 1 (BLACK or NOIR = 0., WHITE or
	BLANC = 1. are \#defined in PLOT_PS.H).

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_LINE_WIDTH   Defines a new line width for the lines
index	   Defines a new line width for the lines

synopsis
	#include <plot_ps.h>

	ps_line_width(width);
	float     width; /* line width in picas */

description

	PS_LINE_WIDTH defines a new line width for the lines subsequently
	drawn. The width is given in PostScript units (picas).

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage

title   PS_DASHED_LINE  Defines a new dashed line pattern
index	   Defines a new dashed line pattern

synopsis
	#include <plot_ps.h>

	ps_dashed_line(dash,gap,offset)
	float     dash,   /* pica units */
		  gap;    /* pica units */
		  offset; /* pica units */

description

	PS_DASHED_LINE  defines a new dashed line pattern for the subsequent lines
	drawn. {\tt dash} is the length of the dash, {\tt gap}  is the length of the
	gap between dashes and {\tt ofsett} is the phase of the first dash start
	relatively to the line start.  All values are given in PotScript pica units.

author

	Jean-Francois VIBERT et Eric BOUSSARD

	CHU Saint-Antoine INSERM U263 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage

title   PS_SOLID_LINE   Start the solid line pattern
index	   Start the solid line patter

synopsis
	#include <plot_ps.h>

	ps_solid_line()


description

	PS_SOLID_LINE  returns back to the solid line pattern after a call to PS_DASHED_LINE.

author

	Jean-Francois VIBERT et Eric BOUSSARD

	CHU Saint-Antoine INSERM U263 Universite Paris VI

See also

	PLOT1 et PLOT2

newpage

title   PS_ARC  Draws an arc clockwise
index	   Draws an arc clockwise

synopsis
	#include <plot_ps.h>
	
	ps_arc(r,theta_1,theta_2)
	float   r;      /* radius always in cms */
	int     theta_1,/* begining angle */
		theta_2;/* ending angle */
description

	PS_ARC Draws an arc clockwise begining at the current position, and
	coms back to the same position.  The radius must be given in cms.
	Begining and ending arcs are given in degres clockwise from the
	horizontal line.
	
author

	Jean-Francois VIBERT and Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et Ecole Normale Superieure

See also
	PS_CIRCLE, PS_DISK, PS_ELLIPSE, PS_ELLIP_DISK
	
newpage

title   PS_CIRCLE       Draws an empty circle
index		   Draws an empty circle

synopsis
	#include <plot_ps.h>
	
	ps_circle(r)
	float   r;   /* radius always in cms */
description

	PS_CIRCLE draws an empty circle whose the center is located at the
	current position. It comes back at the same current position.

author

	Jean-Francois VIBERT et Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et Ecole Normale
	Superieure

See also

	PLOT1 et PLOT2
newpage
title   PS_DISK Draws a filled disk
index	   Draws a filled disk

synopsis
	#include <plot_ps.h>
	
	ps_disk(r,gray)
	float   r,   /* radius always in cms */
		gray;/* shading 0-1, -1 color */
description

	PS_DISK draws an filled disk whose the center is located at the
	current position. It comes back at the same current position.  The
	shading is between 0 (black) and 1 (white). If gray is negative, the disk is
	filled with the current color.
author

	Jean-Francois VIBERT et Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et Ecole Normale
	Superieure

See also

	PLOT1 et PLOT2
	
newpage
title   PS_ELLIPSE      Draws an empty ellipse
index		   Draws an empty ellipse

synopsis
	#include <plot_ps.h>
	
	ps_ellipse(h_d,v_d)
	float   h_d,    /* horizontal diameter in cms */
		v_d;    /* vertical diameter in cms */
description

	PS_ELLIPSE draws an empty ellipse whose the center is located at the
	current position. The parameters are the horizontal and vertical
	diameters (not radius) in cms.  It comes back at the same current
	position.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage
title   PS_ELLIP_DISK   Draws a filled ellipse
index		   Draws a filled ellipse

synopsis
	#include <plot_ps.h>
	
	ps_ellip_disk(h_d,v_d,gray)
	float   h_d,    /* horizontal diameter in cms */
		v_d,    /* vertical diameter in cms */
		gray;   /* shading 0-1 */
description

	PS_ELLIP_DISK draws a filled ellipse whose the center is located at
	the current position. The parameters are the horizontal and vertical
	diameters (not radius) in cms.  It comes back at the same current
	position. The shading is between 0 (black) and 1 (white). If gray is
	negative, the ellipse is filled with the current color.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
#endif

/*
*
*       PS_INIPLOT() permet d'initialiser la benson. Il appelle plopen()
*       et dessine la baniere
*
*/

#if UNIX
#ifdef HAVE_CONFIG_H
/* changed by BaCh (bach@mwgdna.com) 
   xnbc is not part of plot_ps
 */
/*#	include <config_xnbc.h>*/
#	if HAVE_LIMITS_H
#		include <limits.h>
#	endif
#	if HAVE_UNISTD_H
#		include <unistd.h>
#	else
#		include <sys/utsname.h>
#	endif
#	include <sys/types.h>
#	include <time.h>
#else
#	include <limits.h>
#	include <sys/utsname.h>
#	include <sys/types.h>
#	include <time.h>
#endif
#endif

#if vms
#       include <libdef.h>
#       include <descrip.h>
#endif


/*
 *      PS_SIZE est une routine qui permet de definir un facteur d'echelle
 *      pour les differentes routines travaillant sur des caracteres.
 *
 *      ATTENTION: hx  est une hauteur exprimee en
 *      centimetres.
 */
void ps_size(float hhx)
{
	float h_lettre;

	h_lettre = hhx * 37.5;  /* 37.5 = nb de pica/cm */
	sprintf(pl_buf,"/%s findfont %d scalefont setfont\n",
				pl_police,(int)h_lettre);
	fputs(pl_buf,pl_spl);
	hl_crte = hhx;

}

/*
 *      PS_CHARA
 *
 *      Routine PostScipt d'ecriture d'un texte dans l'espace Absolu
 *      avec deplacement initial et precision d'un format.
 *      Ce format replace celui precedament defini.
 *			      Il n'y a pas de recalage.
 */
void ps_chara(float x, float y, int cd, char *strg, float hx, int rot)
{
	int l;

	l=strlen(strg);		 /* Longueur du string	   */
	traa (x,y,cd);		  /* Deplacement		  */
	if (hx != hl_crte) ps_size(hx);
	if (rot)
	{
		sprintf(pl_buf,"gsave\n %d rotate\n",rot);
		fputs(pl_buf,pl_spl);
	}
	p_puts(strg);		   /* Ecriture de la chaine	*/
	if (rot)
	{
		sprintf(pl_buf,"grestore\n");
		fputs(pl_buf,pl_spl);
	}
}
/*
 *      PS_CHARS
 *
 *      Routine PostScipt d'ecriture d'un texte dans l'espace sujet
 *      avec deplacement initial et precision d'un format.
 *      Ce format replace celui precedament defini.
 *			      Il n'y a pas de recalage.
 */
void ps_chars(float x,float y,int cd,char *strg,float hx,int rot)
{
	int l;

	l=strlen(strg);		 /* Longueur du string	   */
	tras(x,y,cd);		   /* Deplacement		  */
	if (hx != hl_crte) ps_size(hx);
	if (rot)
	{
		sprintf(pl_buf,"gsave\n %d rotate\n",rot);
		fputs(pl_buf,pl_spl);
	}
	p_puts(strg);		   /* Ecriture de la chaine	*/
	if (rot)
	{
		sprintf(pl_buf,"grestore\n");
		fputs(pl_buf,pl_spl);
	}
}

/*
*
*       PS_FONT() permet de choisir une police de caracteres
*/
void ps_font(char *txt_police,float hhx)
{
	float   h_lettre;

	if (hhx == 0.)
		hhx = hl_crte;
	strcpy(pl_police,txt_police);
	h_lettre = hhx * 37.5;  /* 37.5 = nb de pica/cm */
	sprintf(pl_buf,"/%s findfont %d scalefont setfont\n",
			txt_police,(int)h_lettre);
	fputs(pl_buf,pl_spl);
	hl_crte = hhx;
}
/*
 *      PS_ORIGIN permet de modifier l'origine Absolue, d'effectuer un
 *	      deplacement et de terminer un record.
 *
 */
void ps_origin(float xf,float yf,float xa,float ya)
{
	traa (xf,yf,PL_AH);	     /* Deplacement initial	  */
	pl_X_crt = xa;
	pl_Y_crt = ya;
	if (xa < (-32767/PL_PAS) || xa > (32767/PL_PAS))
		fprintf(stderr,"PLOT_PS: ps_origin warning: XA exceeding limits\n");
	else
	if (ya < (-32767/PL_PAS) || ya > (32767/PL_PAS))
		fprintf(stderr,"PLOT_PS: ps_origin warning: YA exceeding limits\n");
	else
	{
		pl_x = xa * X_PAS;      /* Redefinition de l'adresse de	*/
		pl_y = ya * X_PAS;      /* la plume		     */
	}
	sprintf(pl_buf,"%.1f %.1f translate\n",ex_x,ex_y);
	fputs(pl_buf,pl_spl);
}

/* Write a banner on each page */
void ps_banner(void)
{
  ps_page_banner = TRUE;
}

/* Avoid the banner on each page */

void ps_no_banner(void)
{
  ps_page_banner = FALSE;
}

void ps_iniplot(char *la_plot_file, float paper)
/* File for the plot */
	    /* Paper width in cm */
{
	char proprio[256], moment[50];
	char *p_q_n;
	char page[5];

#if UNIX
	extern char *getlogin(void);
#if HAVE_UNISTD_H
	char sys_name[100];
	time_t tim, *p_tim;

	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
	gethostname(sys_name,100);
	/* changed by Bastien Chevreux (bach@mwgdna.com)
	   This fails miserably when started in a cron job, as 
	   getlogin() can return a NULL pointer

	   The canonical way for this should be getpwuid(geteuid())  
	   (NOT!!! cuserid()), but I am too lazy

	   sprintf(proprio,"%s@%s",getlogin(),sys_name);
	*/
	if (getlogin()!=NULL) {
	  sprintf(proprio,"%s@%s",getlogin(),sys_name);
	} else {
	  sprintf(proprio,"unknown@%s",sys_name);
	}


#else
#  ifndef __hpux
	struct utsname sys_name, *p_sys_name;
#  endif
	time_t tim, *p_tim;
#  ifndef __hpux
	p_sys_name = &sys_name;
#  endif
	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
#  ifndef __hpux
	uname(p_sys_name);
	/* changed by Bastien Chevreux (bach@mwgdna.com)
	   This fails miserably when started in a cron job, as 
	   getlogin() can return a NULL pointer

	   sprintf(proprio,"%s@%s",getlogin(),sys_name);
	*/
	if (getlogin() != NULL)
	{
	  sprintf(proprio,"%s@%s",getlogin(),p_sys_name->nodename);
	}
	else
	{
	  sprintf(proprio,"unknown@%s",p_sys_name->nodename);
	}
#  endif

#endif
#endif

#if VMS
	$DESCRIPTOR(dscDate, "12345678901234567890123");
	/* la chaine doit faire 23 caracteres de long */
	lib$date_time(&dscDate);
	dscDate.dsc$a_pointer[17]='\0';
	/* on tronque les secondes et les centiemes */
	strcpy(moment, dscDate.dsc$a_pointer);
	strcpy(proprio, "VAX/VMS");
#endif
#if MS_DOS
	strcpy(moment,"now...");
	strcpy(proprio,"MS/DOS");
#endif

/*
 * Initialisations generales
 */

	if ((p_q_n = getenv("PRINT_PLOT_PS")) == NULL)
		strcpy(queue_name,PLOT_PS_QUEUE_NAME);
	else
		strcpy(queue_name,p_q_n);
		
	pl_ex = pl_ey = PL_PAS;
	pl_rex = pl_rey = X_PAS;
	pl_ps_page = pl_cms = 1;
	pl_ca = 1.;
	pl_sa = 0.;
	
	strcpy(plot_file,la_plot_file); /* pour pl_newpage */

	plopen(plot_file);
	fprintf(pl_spl,"%%!PS-Adobe-2.0 EPSF-1.2\n");
	fprintf(pl_spl,"%%PLOT_PS V%s\n%%%%Creator: %s\n",PS_VERSION,proprio);
	fprintf(pl_spl,"%%%%Title: %s\n",plot_file);
	fprintf(pl_spl,"%%%%CreationDate: %s\n",moment);
	/*
	 * ecriture de la BoundingBox correspondant a la page entiere.
	 */
#if PSFIG == 0
	fprintf(pl_spl,"%%%%BoundingBox: (atend)\n");
	/*
	 * Supression de cette ligne car psfig ne sait pas en tenir compte, ce
	 * qui oblige a l'enlever ou a la remplacer
	 */
#endif
	fprintf(pl_spl,"%%%%Pages: (atend)\n");
	fprintf(pl_spl,"%%%%EndComments\n");
	fprintf(pl_spl,"save\n");
	fprintf(pl_spl,"/M { moveto } bind def\n");
	fprintf(pl_spl,"/rM { rmoveto } bind def\n");
	fprintf(pl_spl,"/L { lineto } bind def\n");
	fprintf(pl_spl,"/rL { rlineto } bind def\n");
	fprintf(pl_spl,"/S { show } bind def\n");
	fprintf(pl_spl,"/GSG { gsave stroke grestore } bind def\n");
	

	/* Changed by BaCh (bach@mwgdna.com): 
	   I don't want banners on the pages */
	/* and then JFV with the ps_page_banner flag */

	if (ps_page_banner)
	{
	  fprintf(pl_spl,"%%%%Beginning of the flag page\n");

	  /*      Le cadre de flag-page */

	  traa(0.8,0.8,PL_AH);
	  traa(19.5,0.,PL_RB);
	  traa(0.,1.,PL_RB);
	  traa(-19.5,0.,PL_RB);
	  traa(0.,-1.,PL_RB);
	  traa(18.5,0.,PL_RH);
	  traa(0.,1.,PL_RB);
	
	  traa(1.,1.15,PL_AH);

	  /* le texte de la baniere */

	  ps_font("Times-Roman",0.4);
	  ps_chara(0.,0.,PL_RH,moment,0.4,0);
	  traa(8.,1.1,PL_AH);
	  ps_font("NewCenturySchlbk-BoldItalic",0.5);
	  ps_chara(0.,0.,PL_RH,plot_file,0.5,0);
	  traa(16.,1.15,PL_AH);
	  ps_font("Times-Roman",0.4);
	  ps_chara(0.,0.,PL_RH,proprio,0.4,0);
	  traa(19.5,1.15,PL_AH);
	  sprintf(page,"%2d",pl_ps_page);
	  ps_chara(0.,0.,PL_RH,page,0.4,0);

	  traa(0.,0.,PL_AH);

	  fprintf(pl_spl,"%%%%End of the flag page\n");
	}
	/* Changed by BaCh (bach@mwgdna.com): 
	   I don't want banners on the pages */
	/* and then JFV with the ps_page_banner flag */


	/* Mise en place pour le dessin */

	ps_origin(1.,2.,0.,0.);

	/* Initialisation de la Bounding Box aux valeurs inverses */

	b_b_ll_x = UR_X;
	b_b_ll_y = UR_Y;
	b_b_ur_x = LL_X;
	b_b_ur_y = LL_Y;

}


/*
*
*       PS_ENDPLOT() permet de terminer et spooler la benson. Il appelle plclose()
*       et spoole eventuellement en l'effacant, le dessin 
*
*/

extern void plclose();

void ps_endplot(char *file_sp, char action)
/* Action to take about the plot file
			s: spool and keep
			d: spool and delete
			f: freeze (no spool)
		*/ 
{
	char ligne[256];


	fprintf(pl_spl,"GSG\n");	/* stroke les derniers dessins */
	plclose(file_sp);       /* fermeture du fichier */

	if (!strlen(file_sp)) strcpy(file_sp,plot_file);
#if UNIX
	sprintf(ligne,"%s %s",queue_name,file_sp);
#endif
#if VMS
	sprintf(ligne,"%s %s",queue_name,file_sp);
#endif
#if MS_DOS
	sprintf(ligne,"PRINT %s",file_sp);
#endif

	switch (action)
	{

		case 'd':
			system(ligne);
#if UNIX
			sprintf(ligne,"rm %s",file_sp);
#endif
#if VMS
			sprintf(ligne,"DELETE %s;*",file_sp);
#endif
#if MS_DOS
			sprintf(ligne,"DEL %s",file_sp);
#endif
			system(ligne);
			break;

		case 'f':
			return;

		case 's':
		default:
			system(ligne);
	}
}

/*
 * PS_QUEUE_NAME permet de changer de queue par defaut
 */

void ps_queue_name(char *nom_de_la_queue)
{
	strcpy(queue_name,nom_de_la_queue);
} /* fin du char */

/*
*
*       PS_NEWPAGE permet de changer de page.
*       il redessine dessine la baniere et numerote les pages
*
*/

void ps_newpage(void)
{
	char proprio[256], moment[50];
	char page[5];

#if UNIX
	extern char *getlogin(void);
#if HAVE_UNISTD_H
	char sys_name[100];
	time_t tim, *p_tim;

	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
	gethostname(sys_name,100);
	/* changed by Bastien Chevreux (bach@mwgdna.com)
	   This fails miserably when started in a cron job, as 
	   getlogin() can return a NULL pointer

	   sprintf(proprio,"%s@%s",getlogin(),sys_name);
	*/
	if (getlogin()!=NULL) {
	  sprintf(proprio,"%s@%s",getlogin(),sys_name);
	} else {
	  sprintf(proprio,"unknown@%s",sys_name);
	}

#else
#  ifndef __hpux
	struct utsname sys_name, *p_sys_name;
#  endif
	time_t tim, *p_tim;
#  ifndef __hpux
	p_sys_name = &sys_name;
#  endif
	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
#  ifndef __hpux
	uname(p_sys_name);
	/* changed by Bastien Chevreux (bach@mwgdna.com)
	   This fails miserably when started in a cron job, as 
	   getlogin() can return a NULL pointer
	*/
	if (getlogin()!=NULL) {
	  sprintf(proprio,"%s@%s",getlogin(),p_sys_name->nodename);
	} else {
	  sprintf(proprio,"unknown@%s",p_sys_name->nodename);
	}
#  endif

#endif
#endif

#if VMS
	$DESCRIPTOR(dscDate, "12345678901234567890123");
	/* la chaine doit faire 23 caracteres de long */
	lib$date_time(&dscDate);
	dscDate.dsc$a_pointer[17]='\0';
	/* on tronque les secondes et les centiemes */
	strcpy(moment, dscDate.dsc$a_pointer);
	strcpy(proprio, "VAX/VMS");
#endif
#if MS_DOS
	strcpy(moment,"now...");
	strcpy(proprio,"MS/DOS");
#endif

	fprintf(pl_spl,"showpage\n");
	pl_ps_page++;


	/* Changed by BaCh (bach@mwgdna.com): 
	   I don't want banners on the pages */
	/* and then JFV with the ps_page_banner flag */

	if (ps_page_banner)
	{
	  fprintf(pl_spl,"%%%%Beginning of the flag page\n");

	  /*      Le cadre de flag-page */

	  traa(0.8,0.8,PL_AH);
	  traa(19.5,0.,PL_RB);
	  traa(0.,1.,PL_RB);
	  traa(-19.5,0.,PL_RB);
	  traa(0.,-1.,PL_RB);
	  traa(18.5,0.,PL_RH);
	  traa(0.,1.,PL_RB);
	
	  traa(1.,1.15,PL_AH);

	  /* le texte de la baniere */

	  ps_font("Times-Roman",0.4);
	  ps_chara(0.,0.,PL_RH,moment,0.4,0);
	  traa(8.,1.1,PL_AH);
	  ps_font("NewCenturySchlbk-BoldItalic",0.5);
	  ps_chara(0.,0.,PL_RH,plot_file,0.5,0);
	  traa(16.,1.15,PL_AH);
	  ps_font("Times-Roman",0.4);
	  ps_chara(0.,0.,PL_RH,proprio,0.4,0);
	  traa(19.5,1.15,PL_AH);
	  sprintf(page,"%2d",pl_ps_page);
	  ps_chara(0.,0.,PL_RH,page,0.4,0);

	  traa(0.,0.,PL_AH);

	  fprintf(pl_spl,"%%%%End of the flag page\n");
	}
	/* Changed by BaCh (bach@mwgdna.com)
	   I don't want banner on the pages */
	/* and then JFV with the ps_page_banner flag */

	/* Mise en place pour le dessin */

	ps_origin(1.,2.,0.,0.);
}

/*
 *      PS_ECHEL : Definition de l'Espace Sujet par rapport a l'Espace Absolu
 *
 *	      (ex ,ey) : Nombre de centimetre par U.U. (multiple de 200)
 *	      (xs, ys) : Coordonees ABSOLUES de l'origine SUJET
 *	      (ca, sa) : Cosinus et sinus du dessin
 *      retourne TRUE si l'echelle est licite, et FALSE sinon
 */
int ps_echel( float ex,float ey,float xs,float ys,float ca,float sa)
{
	pl_rex = ex * X_PAS;
	pl_rey = ey * X_PAS;    /* Echelle en reel      */
	pl_ca = ca;
	pl_sa = sa;	     /* Rotation de l'espace Sujet	*/

	pl_xs = xs * X_PAS;
	pl_ys = ys * X_PAS;     /* Origine Sujet	*/
	pl_ex = pl_rex; 
	pl_ey = pl_rey;	 /* Echelle entiere      */

	if((pl_rex - pl_ex) != 0.0 ||
		((pl_rey - pl_ey) != 0.0 ) )
	  return(FALSE);

	if (pl_ex == 0. || pl_ey == 0.) 
	{
		pl_cms = TRUE;
		return(FALSE);
	}
	else
	{
		pl_cms=FALSE;
		return(TRUE);
	}
}
/*
 *      PS_PEN : Selectionne la plume desiree. Le systeme ne connait que
 *	      les plumes 0, 1, 2
 */
int ps_pen(int k) 
{
	float   gris = 0.;
	
	if (k<0 || k>2) return(FALSE);
	pl_plume=k;

	switch(k)
	{
		case 0:
			gris = 0.;
			break;
		case 1:
			gris = 0.3;
			break;
		case 2:
			gris = 0.7;
	}
	
	sprintf(pl_buf,"%3.1f setgray\n",gris);
	fputs(pl_buf,pl_spl);
	return(TRUE);
}
/*
 *      PS_SCALE est une routine qui permet de definir un facteur d'echelle
 *      global en X et Y portant sur dessin et texte, X et Y peuvent etre 
 *      differents
 */
void ps_scale(float x_scl,float y_scl)
{
	
	sprintf(pl_buf,"%.1f %.1f scale\n",x_scl,y_scl);
	fputs(pl_buf,pl_spl);
}
/*
*       PS_LINE_WIDTH() permet de choisir une epaisseur de trait
*/
void ps_line_width(float largeur)
{
	sprintf(pl_buf,"GSG\n%.2f setlinewidth\n",largeur);
	fputs(pl_buf,pl_spl);
}

/*
*       PS_DASHED_LINE()	permet de choisir un pattern de ligne tiretee
*/
void ps_dashed_line(float dash,  float gap, float  offset)
{
	sprintf(pl_buf,"[%.2f %.2f] %.2f setdash\n", dash, gap, offset);
	fputs(pl_buf,pl_spl);
}
/*
*       PS_SOLID_LINE() permet de remettre la ligne en continue apres un setdash
*/
void ps_solid_line(void)
{
	fputs("[] 0 setdash\n",pl_spl);
}
/*
*       PS_GRAY() permet de definir le niveau de gris
*/
void ps_gray(float gris)
/* entre 0 et 1 */
{
	if (gris < 0.) gris = 0.;       /* 0 --> NOIR   */
	if (gris > 1.) gris = 1.;       /* 1 --> BLANC  */

	sprintf(pl_buf,"%f setgray\n",gris);
	fputs(pl_buf,pl_spl);
}
/*
*       PS_ROTATE() permet de definir une rotation 
*/
void ps_rotate(int angle)
{
	sprintf(pl_buf,"%d rotate\n",angle);
	fputs(pl_buf,pl_spl);
}
/*
 * ps_arc dessine un arc de cercle en absolu au point courant et y revient
 */
void ps_arc(float r, int theta_1,int theta_2)
/* radius always in cms */
/* begining angle */
/* ending angle */
{

	float ri;
	
	ri = r * X_PAS;
	fprintf(pl_spl,"%.1f %.1f %.1f %d %d arc\n",ex_x, ex_y, ri, theta_1, theta_2);
	fprintf(pl_spl,"%.1f %.1f M\n",ex_x,ex_y);
	
} /* fin du ps_arc */

/*
 * ps_circle dessine un cercle en absolu au point courant et y revient
 */
void ps_circle(float r)
{

	float ri;
	
	ri = r * X_PAS;
	fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc stroke \n",ex_x, ex_y, ri);
	fprintf(pl_spl,"%.1f %.1f M\n",ex_x,ex_y);
	draw = FALSE;
	
} /* fin du ps_circle */
/*
 * ps_circle dessine un cercle en absolu au point courant et y revient
 */
void ps_disk(float r,float gray)
{

	float ri;

	ri = r * X_PAS;
	if (gray >= 0.)
	{
		if (gray > 1.) gray = 1.;       /* white */
		fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc %f setgray fill stroke \n",
			ex_x, ex_y, ri, gray);
	} /* fin du if */
	else
	{
		fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc fill stroke \n",
			ex_x, ex_y, ri);
	} /* fin du else */
	fprintf(pl_spl,"%.1f %.1f M\n",ex_x,ex_y);
	draw = FALSE;
	
} /* fin du ps_disk */
/*
 * ps_ellipse dessine une ellipse en absolu au point courant et y revient
 */
void ps_ellipse(float h_d,float v_d)
/* horizontal diameter in cms */
/* vertical diameter in cms */
{

	float ri;
	float   ellipse_scale;

	ellipse_scale = (h_d / v_d);
	ri = (h_d / 2.) * X_PAS;
	fprintf(pl_spl,"1 %f scale\n",ellipse_scale);
	fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc stroke \n",ex_x, ex_y, ri);
	ellipse_scale = (v_d / h_d);
	fprintf(pl_spl,"1 %f scale\n",ellipse_scale);
	fprintf(pl_spl,"%.1f %.1f M\n",ex_x,ex_y);
	draw = FALSE;
	
} /* fin du ps_ellipse */
/*
 * ps_ellip_disk dessine une ellipse pleine en absolu au point courant
 * et y revient
 */
void ps_ellip_disk(float h_d,float v_d,float gray)
/* horizontal diameter in cms */
/* vertical diameter in cms */
/* shading 0-1 */
{

	float ri;
	float   ellipse_scale;

	ellipse_scale = (h_d / v_d);
	ri = (h_d / 2.) * X_PAS;
	fprintf(pl_spl,"1 %f scale\n",ellipse_scale);
	if (gray >= 0.)
	{
		if (gray > 1.) gray = 1.;       /* white */
		fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc %f setgray fill stroke \n",
			ex_x, ex_y, ri, gray);
	} /* fin du if */
	else
	{
		fprintf(pl_spl,"newpath %.1f %.1f %.1f 0 360 arc fill stroke \n",
			ex_x, ex_y, ri);
	} /* fin du else */
	ellipse_scale = (v_d / h_d);
	fprintf(pl_spl,"1 %f scale\n",ellipse_scale);
	fprintf(pl_spl,"%.1f %.1f M\n",ex_x,ex_y);
	draw = FALSE;
	
} /* fin du ps_ellip_disk */
















