/*
			P  L  O  T  2  _  P  S 
			======================

		   Routines de dessin sur la LN03R PostScript.

	NIVEAU 2 : Ce jeu de routine regroupe les routines de mouvement
			de plume, de niveau plus eleve

		Mise en forme Version 2
			Juillet 90
		V4.2	oct 91
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/

/****************************************************************
*
*	Routines de mouvement de la plume
*
*		b -> espace benson (en cm)
*		s -> espace utilisateur c-a-d sujet (a l'echelle)
*		a -> mouvement en absolu 
*		r -> mouvement en relatif
*
*	xxxxba	-> espace benson en absolu
*	xxxxsa	-> espace utilisateur en absolu
*	xxxxbr	-> espace benson en relatif
*	xxxxsr	-> espace utilisateur en relatif
*
*	P pour plume
*	MOV pour deplacement plume haute (sans trace)
*	LIN pour deplacement plume basse (trace de trait)
*
********************************************************************/
        

#ifdef	DOCUMENTATION

title	PS_MOVEMENTS	Functions to move pen and to draw
index     		Functions to move pen and to draw


synopsis
	#include <plot_ps.h>

	ps_user()

	ps_cms()

	ps_mover(x,y)
	float x, y;

	ps_movea(x,y)
	float x, y;

	ps_drawr(x,y)
	float x, y;

	ps_drawa(x,y)
	float x, y;

	ps_home()

description

	{\tt ps_user()} indicates that all subsequent pen movements will be
	considered with the coordinates now expressed in user units and not
	in centimeters.

	{\tt ps_cms()} indicates that all subsequent pen movements will be
	considered with the coordinates now expressed in centimeters and not
	in user units.

	With the following functions,you are able to draw in the current
	space: {\tt ps_move} move without drawing ,{\tt ps_draw} move the
	pen but draws too .The coordinates are absolutes ({\tt a} suffixe) or
	relatives ({\tt r} suffixe).

	{\tt ps_movea(x,y)} moves to the absolute (x,y) point.

	{\tt ps_mover(x,y)} moves to the relative (x,y) point.

	{\tt ps_drawa(x,y)} draws a line to the absolute (x,y) point.

	{\tt ps_drawr(x,y)} draws a line to the relative (x,y) point.

	{\tt ps_home()} goes back to the origin of the graph.

author
	Dr Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	ps_where() 

newpage

title	PS_WHERE	Get the current X-Y position
index     		Get the current X-Y position


synopsis
	#include <plot_ps.h>

	int     ps_where(x,y)
		float *x, *y;

	int     ps_is_cms();

description

	{\tt ps_where()} return in x and y arguments the value of the
	current position in the current units. If units are cms, it return
	TRUE (1), FALSE (0) if units are in units of the user.

	{\bf WARNING:} results are false if {\tt ps_where} is call after a
	text writing function for wich the pen stays at the end of the
	string. It returns the position of the string beginning. But it can
	be used to know where to go to be at the string begining.


	{\tt ps_is_cms()} return TRUE (1) if the current space is expressed
	as cms, and FALSE (0) if the current space is in units of the user.

author
	Dr Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	ps_cms() and ps_user(), 

#endif

#include <stdlib.h>
#include <stdio.h>

#include "plot_ps.h"

/*
	Les fonctions suivantes ne devraient jamais etre appelees:
	Elles permettent eventuellement de faire un deplacement dans un
	autre espace que l'espace courant.

	Routines de mouvement de la plume

	\nofill
		b -> espace papier (en cm) (b pour Benson, raison historique)
		s -> espace utilisateur c-a-d sujet (a l'echelle)
		a -> mouvement en absolu 
		r -> mouvement en relatif

	xxxxba	-> espace papier en absolu
	xxxxsa	-> espace utilisateur en absolu
	xxxxbr	-> espace benson en relatif
	xxxxsr	-> espace utilisateur en relatif
	\fill
	
	PS pour plume
	MOV pour deplacement plume haute (sans trace)
	LIN pour deplacement plume basse (trace de trait)

	{\tt pmovsa(x,y)} se deplace au point x,y en absolu dans l'espace
	sujet

	{\tt pmovba(x,y)} se deplace au point x,y en absolu dans l'espace
	papier (papier)

	{\tt pmovsr(x,y)} se deplace au point x,y en relatif dans l'espace
	sujet

	{\tt pmovbr(x,y)} se deplace au point x,y en relatif dans l'espace
	papier

	{\tt plinsa(x,y)} trace jusqu'au point x,y en absolu dans l'espace
	sujet

	{\tt plinba(x,y)} trace jusqu'au point x,y en absolu dans l'espace
	papier

	{\tt plinsr(x,y)} trace jusqu'au point x,y en relatif dans l'espace
	sujet

	{\tt plinbr(x,y)} trace jusqu'au point x,y en relatif dans l'espace
papier
*/
/*
 *
 *	PMOVBA(x,y) * *	va au point nomme plume haute en absolu
 *
 */
void pmovba(float x, float y)
{
	traa(x,y,PL_AH);
}
/*
*
*	  PMOVSA(x,y)
*
*	va au point nomme plume haute en absolu
*	espace sujet a l'echelle  (non en cm)
*
*/
void pmovsa(float x, float y)
{
	tras(x,y,PL_AH);
}
/*
*
*	  PMOVBR(x,y)
*
*	va au point nomme plume haute en relatif
*	espace benson (en cm)
*
*/
void pmovbr(float x, float y)
{
	traa(x,y,PL_RH);
}
/*
*
*	  PMOVSR(x,y)
*
*	va au point nomme plume haute en relatif
*	espace sujet a l'echelle  (non en cm)
*
*/
void pmovsr(float x, float y)
{
	tras(x,y,PL_RH);
}
/*
*
*	  PLINBA(x,y)
*
*	va au point nomme plume haute en absolu
*
*/
void plinba(float x, float y)
{
	traa(x,y,PL_AB);
}
/*
*
*	  PLINSA(x,y)
*
*	va au point nomme plume haute en absolu
*	espace sujet a l'echelle  (non en cm)
*
*/
void plinsa(float x, float y)
{
	tras(x,y,PL_AB);
}
/*
*
*	  PLINBR(x,y)
*
*	va au point nomme plume haute en relatif
*	espace benson (en cm)
*
*/
void plinbr(float x, float y)
{
	traa(x,y,PL_RB);
}
/*
*
*	  PLINSR(x,y)
*
*	va au point nomme plume haute en relatif
*	espace sujet a l'echelle  (non en cm)
*
*/
void plinsr(float x, float y)
{
	tras(x,y,PL_RB);
}
/*
*
*	PBENSON() ou PCMS() switch le travail en espace Benson ou Laser (en cm)
*
*/
void ps_cms(void)
{
	pl_cms = TRUE;
}
void ps_benson(void)
{
	pl_cms = TRUE;
}
/*
*
*	PUSER() switch le travail en espace utilisateur (NON en cm)
*
*/
void ps_user(void)
{
	pl_cms = FALSE;
}
/* pour savoir en quoi on est, quand on est perdu... (dans une fonction par ex) */
int ps_is_cms(void)
{
	return(pl_cms);
}
/*
*
*	PS_WHERE(&x,&y) renvoie les valeur de la position courante en absolu
*	tient compte de,  et renvoie  l'espace de travail
*/
int ps_where(float *x, float *y)
{

	if (pl_cms)
	{
		*x = pl_X_crt;
		*y = pl_Y_crt;
	} /* fin du 	if */
	else
	{
		*x = (pl_X_crt + pl_xs/X_PAS ) * pl_rex/X_PAS;
		*y = (pl_Y_crt + pl_ys/X_PAS ) * pl_rey/X_PAS;
	} /* fin du 	else */
	return(pl_cms);
} /* fin du ps_where */
/*
*
*	PDRAWA(x,y) trace une ligne (x et y en absolu)
*	tient compte de l'espace de travail
*/
void ps_drawa(float x, float y)
{
	if (pl_cms) traa(x,y,PL_AB);
	else tras(x,y,PL_AB);
}
/*
*
*	PDRAWR(x,y) trace une ligne (x et y en relatif)
*	tient compte de l'espace de travail
*/
void ps_drawr(float x, float y)
{
	if (pl_cms) traa(x,y,PL_RB);
	else tras(x,y,PL_RB);
}
/*
*
*	PMOVEA(x,y) deplace la plume (x et y en absolu)
*	tient compte de l'espace de travail
*/
void ps_movea(float x, float y)
{
	if (pl_cms) traa(x,y,PL_AH);
	else tras(x,y,PL_AH);
}
/*
*
*	PMOVER(x,y) deplace la plume (x et y en relatif)
*	tient compte de l'espace de travail
*/
void ps_mover(float x, float y)
{
	if (pl_cms) traa(x,y,PL_RH);
	else tras(x,y,PL_RH);
}
/*
*
*	PHOME(x,y) deplace la plume a l'origine (0.,0.)
*	tient compte de l'espace de travail
*/
void ps_home(void)
{
	if (pl_cms) traa(0.,0.,PL_AH);
	else tras(0.,0.,PL_AH);
}
