/*
			P  L  O  T  3  _  P  S 
			======================

		   Routines de dessin sur la LN03R PostScript.

	NIVEAU 3 : Ce jeu de routine regroupe d'autres routines du
		logiciel plot_ps, de niveau plus eleve

		Version 1 et 2
			Juillet 90
		Version 3
			31 juillet 90
		Version 3.1
			8 octobre 1990
		Version 4.2
			11 octobre 1991
		Version 6.3
			22 mars 2001
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/

/*
title	PLOT3_PS Fonctions de dessins complexes
index		Fonctions de dessins complexes


description

	           Routines de dessin sur la PLOT_PS.

	   NIVEAU 3 : Ce jeu de routine regroupe les routines
	                   appelables par l'utilisateur permettant
			   des fonctions de niveau eleve.


author

	Jean-Francois Vibert, Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	plot0_ps, plot1_ps et plot2_ps
			
newpage
*/

#ifdef	DOCUMENTATION

title	PS_POLYLINE Draws a polyline
index		Draws a polyline

synopsis
	#include <plot_ps.h>

	ps_polyline(x,y,nb_pts,dot,dot_type,dot_size)
	float   x[],	/* array with x coordinates */
		y[];	/* array with y coordinates */
	int     nb_pts,     /* nb of values in x (and y) */
		dot,	/* TRUE if dots must be drawn */
		dot_type;   /* dot type from a list of 16 */
	float   dot_size;   /* dot size in cms */
description

	PS_POLYLINE() draws a polygonal line whose absolute coordinates are
	in 2 arrays x and y, beginning at the first X,Y coordinates.  If {\tt
	dot} is TRUE, a dot is drawn at each articulation of the polyline
	(included extremities). The dot size is given in cms by {\tt
	dot_size}. The {\tt dot_type} must be choosen among the following
	values:

		\nofill
		value 1 and 2: square empty and filled
		value 3 and 4: losange  empty and filled
		value 5 and 6: up triangle empty and filled
		value 7 and 8: down triangle empty and filled
		value 9 and 10:  circle empty and filled
		value 11 and 12: left triangle empty and filled
		value 13 and 14: right triangle empty and filled
		value 15: x (cross)
		value 16: + (plus sign)
		default: . (dot)
	\fill


author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage
title	PS_POLYDOTS Draws a series of dots
index		Draws a series of dots

synopsis
	#include <plot_ps.h>

	ps_polyline(x,y,nb_pts,dot_type,dot_size)
	float   x[],	/* array with x coordinates */
		y[];	/* array with y coordinates */
	int     nb_pts,     /* nb of values in x (and y) */
		dot_type;   /* dot type from a list of 16 */
	float   dot_size;   /* dot size in cms */
description

	PS_POLYDOTS() draws a series of dots whose absolute coordinates are
	in 2 arrays x and y, beginning at the first X,Y coordinates.
	A dot is drawn at each coordinate (included extremities).
	The dot size is given in cms by {\tt dot_size}.
	The {\tt dot_type} must be choosen among the following values:

		\nofill
		value 1 and 2: square empty and filled
		value 3 and 4: losange  empty and filled
		value 5 and 6: up triangle empty and filled
		value 7 and 8: down triangle empty and filled
		value 9 and 10:  circle empty and filled
		value 11 and 12: left triangle empty and filled
		value 13 and 14: right triangle empty and filled
		value 15: x (cross)
		value 16: + (plus sign)
		default: . (dot)
	\fill


author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage
title	PS_CLOSED_CURVE and PS_FILLED_AREA Draws a possibly filled surface
index		Draws a possibly filled surface

synopsis
	#include <plot_ps.h>

	ps_closed_curve(x,y,nb_pts,contour,fill,gray)
	float   x[],	/* array with x coordinates */
		y[];	/* array with y coordinates */
	int     nb_pts,     /* nb of values in x (and y) */
		contour,    /* t or f line around or not */
		fill;       /* t or f filled or not */
	float   gray;       /* gray filling color */

	ps_filled_area(x_pts,y_pts,nb_pts)
	float   x_pts[],
		y_pts[];
	int     nb_pts;

description

	PS_CLOSED_CURVE() draws polygonal surface whose absolute coordinates
	are in 2 arrays x and y, beginning at the first X,Y coordinates. The
	curve will be close automatically to the first point. It is possible
	to decide to draw a contour around the surface. This surface can be
	filled in gray, whose color is given by gray.

	PS_FILLED_AREA() draws polygonal surface whose absolute coordinates
	are in 2 arrays x and y, beginning at the first X,Y coordinates. The
	curve will be close automatically to the first point. It will be
	closed and filled by the current color (gray or color).
	
author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage

title	PS_BEZIER Draws a Bezier curve
index		Draws a Bezier curve

synopsis
	#include <plot_ps.h>

	ps_bezier(x,y,dot,dot_type,dot_size)
	float   x[],	/* array with x coordinates */
		y[];	/* array with y coordinates */
	int     dot,	/* TRUE if dots must be drawn */
		dot_type;   /* dot type from a list of 16 */
	float   dot_size;   /* dot size in cms */
description

	PS_BEZIER() draws a Bezier cubic curve whose the 3 absolute
	coordinates are in 2 arrays x and y, beginning at the current
	position.

	WARNING: the ps_bezier() draws only a small bezier curve on four (4)
	control points: the current position and the 3 control points in the
	x and y arrays.

	If {\tt dot} is TRUE, a dot is drawn at each Bezier cubic control
	point (included extremities). The dot size is given in cms by {\tt
	dot_size}. The dot size is given in cms by {\tt dot_size}. The {\tt
	dot_type} must be choosen among the following values:

		\nofill
		value 1 and 2: square empty and filled
		value 3 and 4: losange  empty and filled
		value 5 and 6: up triangle empty and filled
		value 7 and 8: down triangle empty and filled
		value 9 and 10:  circle empty and filled
		value 11 and 12: left triangle empty and filled
		value 13 and 14: right triangle empty and filled
		value 15: x (cross)
		value 16: + (plus sign)
		default: . (dot)
	\fill
author

	Jean-Francois VIBERT et Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et
	Ecole Normale Superieur

See also

	PLOT1 et PLOT2
newpage

title	PS_POINT Draws a point centered on the current position
index		Draws a point or dot centered on the current position

synopsis

	#include <plot_ps.h>

	ps_point(dot_type,dot_size)
	int     dot_type;	/* type of the dot (1-16) */
	float   dot_size;	/* height of the symbol in cms */

description

	PS_POINT draws a point centered at the current position, whose
	height is given by dot_size (in cms). 16 different types are available:

	\nofill
		value 1 and 2: square empty and filled
		value 3 and 4: losange  empty and filled
		value 5 and 6: up triangle empty and filled
		value 7 and 8: down triangle empty and filled
		value 9 and 10:  circle empty and filled
		value 11 and 12: left triangle empty and filled
		value 13 and 14: right triangle empty and filled
		value 15: x (cross)
		value 16: + (plus sign)
		default: . (dot)
	\fill

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT1 et PLOT2
newpage
title	PS_FRAME	Draws a frame at the current position
index	Draws a frame at the current position

synopsis

	#include <plot_ps.h>

	ps_frame(width,height)
	float   width,	/* of the rectangle */
		height;       /* of the rectangle */

description

	PS_FRAME draws an empty transparent frame whose position of the
	lower-left angle is at the current position.

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI & 
	Ecole Normale Superieure


see also

	PLOT1 and PLOT2
newpage
title	PS_RECTANGLE	Draws a rectangle at a given position
index	Draws a rectangle at a given position

synopsis

	#include <plot_xps.h>

	ps_rectangle(x_ll,y_ll,width,height)
	float   x_ll,	 /* x and y coordinate of the */ 
		y_ll,	 /* lower left corner of the rectangle */
		width,	/* of the rectangle */
		height;       /* of the rectangle */

description

	PS_RECTANGLE draws a filled rectangle whose position of the
	lower-left corner is at the x_ll,y_ll absolute position.
	The color or grey is the current colr or grey value.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine INSERM U444 Universite Paris VI


see also

	PLOT1 and PLOT2
newpage
	
title	PS_WRITE	Writes a character string or a number
index		Writes a character string or a number

synopsis
	#include <plot_ps.h>

	ps_write(string)
	char    *string;/* string to write  */

	ps_writea(x,y,string,rotation)
	float   x,      /* X coordinate     */
		y;      /* Y coordinate     */
	char    *string;/* string to write  */
	int	rotation;/* rotation in degrees */
	
	ps_writer(x,y,string,rotation)
	float   x,      /* X coordinate     */
		y;      /* Y coordinate     */
	char    *string;/* string to write  */
	int	rotation;/* rotation in degrees */

	ps_writeah(x,y,string,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		taille; /* character height */
	char    *string;/* string to write  */

			
	ps_writeav(x,y,string,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		taille; /* character height */
	char    *string;/* string to write  */

	ps_writefloatav(x,y,number,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		number, /* number to write  */
		taille; /* character height */

	ps_writefloatah(x,y,number,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		number, /* number to write  */
		taille; /* character height */

	ps_writeintav(x,y,number,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		taille; /* character height */
	int     number; /* number to write  */

	ps_writeintah(x,y,number,taille)
	float   x,      /* X coordinate     */
		y,      /* Y coordinate     */
		taille; /* character height */
	int     number; /* number to write  */
description

	PS_WRITE..A..() with the following code:

	\nofill
		nothing	-> string
		int	-> integer
		float	-> float

		h	-> horizontal
		v	-> vertical
	\fill
	
	{\tt ps_write} writes a character string horizontally, at the
	current position, with the current font size.
	
	{\tt ps_writea} writes a character string horizontally, with x and y
	as absolute coordinates, with a given rotation, and with the current
	font size.
	
	{\tt ps_writer} writes a character string horizontally, with x and y
	as relative coordinates, with a given rotation, and with the current
	font size.
	
	{\tt ps_writeav} writes a character string vertically, with x and y as
	absolute positions

	{\tt ps_writefloatah} writes a float horizontally, with x and y as
	absolute positions

	{\tt ps_writefloatav} writes a float vertically, with x and y as
	absolute positions

	{\tt ps_writeintah} Writes an integer horizontally, with x and y as
	absolute positions

	{\tt ps_writeintav} Writes an integer vertically, with x and y as
	absolute positions

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI & 
	Ecole Normale Superieure

see also

	PLOT1 and PLOT2
newpage
	
title	PS_AXES	Draws a two axes centered on the extremal values
index		Draws a two axes centered on the extremal values

synopsis
	#include <plot_ps.h>
	ps_axes(taillex,tailley,minx,maxx,miny,maxy,legende_x,legende_y,hy,cadre)
	float taillex,tailley,      /* axes lenghts in cm */
	      minx,maxx,miny,maxy;  /* maximal values */
	char  *legende_x,	   /* X axis legend */
	      *legende_y;	   /* Y axis legend */
	float hy;		    /* letters height */
	int   cadre;		/* flag */
description

	PS_AXES draws two axes centered on the maximal and minimal values of
	the axes(without tick marks).The absolute values of the maxima and
	minima are written at the bottom of the axes.If the flag is TRUE, a
	frame is drawn around the axes.
	
author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI & 
	Ecole Normale Superieure

see also

	PLOT1 and PLOT2
newpage

title	PS_INCLUDE	Includes a PostScript file
index		Includes a PostScript file

synopsis
	#include <plot_ps.h>

	ps_include(file_name)
	char     *file_name;	/* name of the PS file */
description

	PS_INCLUDE includes a PostScript file in the currently opened PS
	file. It is the responsability of the user to be sure of the PS
	content of the file, and of its coherence (scale, units, position,
	page number, etc\ldots).

	PS_INCLUDE returns TRUE if the file exits and FALSE if a problem was
	encountered during the file opening.

author

	Jean-Francois VIBERT et Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et
	Ecole Normale Superieur

See also

	PLOT1 et PLOT2
	
newpage

title	PS_CODE	Includes  PostScript code 
index		Includes PostScript code

synopsis
	#include <plot_ps.h>

	ps_code(ps_code)
	char     *ps_code;	/* name of the PS file */
description

	PS_CODE includes PostScript code in the currently opened PS file. It
	is the responsability of the user to be sure of the PS content of
	the included code, and of its coherence (scale, units, position,
	page number, etc\ldots).

author

	Jean-Francois VIBERT et Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI et
	Ecole Normale Superieur

See also

	PLOT1 et PLOT2

newpage

title	PS_TICKED_AXES Draws a ticked reference
index		 Draws a ticked reference

synopsis
	#include <plot_ps.h>

	ps_ticked_axes(pos_x,pos_y,mnx,mxx,b_tick_x,e_tick_x,
		       mny,mxy,b_tick_y,e_tick_y,htick,
		       legendx,fmtx,legendy,fmty,h_lettre,cadre)
	float   pos_x,pos_y,/* position of the two axes crossing
			       in your units (user) */
		mnx,mxx,  /* X minima and maxima in your units */
		b_tick_x, /* beginning and end of the X ticks  */
		e_tick_x, /*   in your units (user)	    */
		mny,mxy,  /* Y minima and maxima in your units */
		b_tick_y, /* beginning and end of the Y ticks  */
		e_tick_y, /*   in your units (user)	    */
		htick;    /* length of the smallest tick mark, in cms */
	char    *legendx, /* legend on the X horizontal axis   */
		*fmtx,    /* graduations format (deflt: "%.2f") */
		*legendy, /* legend on the Y vertical axis     */
		*fmty;    /* graduations format (deflt: "%.2f") */
	float   h_txt;    /* heigh of the legend letters in cms */
	int     cadre;    /* flag = 1 if you want a
			   * frame around the axe
			   */

	ps_X_axis(mnx,mxx,htick,b_tick,e_tick,legendx,fmtx,h_txt)
	float   mnx, mxx, /* X minima and maxima in your units  */
		htick,    /* length of the smallest tick mark, in cms */
		b_tick,   /* beginning and end of the X ticks   */
		e_tick;   /*   in your units (user)	     */
	char    *legendx, /* legend on the X horizontal axis    */
		*fmtx;    /* graduations format (deflt: "%.2f") */
	float   h_txt;    /* heigh of the legend letters in cms */

	ps_Y_axis(mny,mxy,htick,b_tick,e_tick,legendy,fmty,h_txt)
	float   mny, mxy, /* X minima and maxima in your units  */
		htick,    /* length of the smallest tick mark, in cms */
		b_tick,   /* beginning and end of the Y ticks   */
		e_tick;   /*   in your units (user)	     */
	char    *legendy, /* legend on the Y vertical axis      */
		*fmty;    /* graduations format (deflt: "%.2f") */
	float   h_txt;    /* heigh of the legend letters in cms */

description

	PS_TICKED_AXES draws a reference with two ticked axes.  The axes are
	drawn from the current position, X axis first, thus, the minimum
	value called {\tt mnx} is always considered as beeing at the current
	position. When the tick marks are not asked from the minimum value of
	the axes, this minimum value is the axis begining. This allows to
	draw graphs slightly shift from the axes. A frame can be drawn around
	the graph, that appears to be drawn inside a rectangle with the
	lower and left sides ticked.  The ticked marks are automatically
	computed to put small ticks on units, medium ticks on 5 units
	multiples and large ticks on 10 units multiples.  Figures are
	annexed to the big ticks.  The axes can cross at other values than
	0.,0., indicated as pos_x and pos_y, in your units.  Ticked part can
	be shorter than the axis itself. Tick and letter sizes are under
	control. Figures annexed to big ticks can be formatted using a
	standard C printf format. It defaults (if {\tt fmt*} is 0), to {\tt
	"\\%.2f"}.

	If the flag {\tt cadre} is true a frame is drawn around
	the reference.

	PS_X_AXIS and PS_Y_AXIS do the same things but in one direction
	only, x or y. They begin always at the current position.
	
	To choose the font of the legends, call {\tt ps_font()} before the
	axis routines.
	 
	All values are considered to be given in user space values, except
	the size ot tick marks and letters, given in cms.

author

	Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

see also


newpage
	
title	PS_HISTO   Draws an histogramm
index		 Draws an histogramm

synopsis
	#include <plot_ps.h>

	ps_histo(popu,tx,ty,gap,nb_class,flag)
	int   popu[MAX_DATA], /* array of values to be drawn */
	      nb_class,	   /* number of classes */
	      flag; /* 0 no calcul 1 vertical 2 horizontal other = all */
	float tx,ty, /* length of the axes */
	      gap;   /* width of a class in user units */	
description

	PS_HISTO draws the histogramm of the given array (integer values).
	The flag FLAG indicates if you want or not indicate statistical
	reference on the drawing. In this case, the mathematic library must
	be loaded when linking the task.

	 If the flag is 0 no indications are given. If the flag is 1 the
	mean value of the histogramm is indicated with a horizontal
	gray line. If the flag is 2 then the expected value is
	indicated with a vertical line. If flag takes another value, both
	are indicated.
author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

	
See also


#endif

#include <stdio.h>
#include <math.h>

#include "plot_ps.h"

#ifdef	M_PI
#	define	PI	M_PI
#else
#	define	PI	3.1415926535
#endif

/*
*	PS_POLYLINE() trace une ligne brisee
*/
void ps_polyline(float x_pts[], float y_pts[], int nb_pts, int dot, int dot_type, float size)
     /*
	dot,	 TRUE if dots must be drawn 
	dot_type;    dot type from a list of 16 
	size;	     size of the dot in cms 
     */
{
	int	i;

	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}

	if (pl_cms) traa(x_pts[0],y_pts[0],PL_AH);
	else tras(x_pts[0],y_pts[0],PL_AH);

	for (i = 0; i < nb_pts; i++) 
	{
		if (pl_cms) traa(x_pts[i],y_pts[i],PL_AB);
		else tras(x_pts[i],y_pts[i],PL_AB);
		if (dot) ps_point(dot_type,size);
	}
	if (!dot) fputs("GSG\n",pl_spl);

	draw = FALSE;

}

/*
*	PS_POLYDOTS() trace une serie de points
*/
void ps_polydots(float x_pts[], float y_pts[], int nb_pts, int dot_type, float size)
     /*
	dot_type;    dot type from a list of 16 
	size;	     size of the dot in cms 
     */
{
	int	i;

	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}

	if (pl_cms) traa(x_pts[0],y_pts[0],PL_AH);
	else tras(x_pts[0],y_pts[0],PL_AH);

	for (i = 0; i < nb_pts; i++) 
	{
		if (pl_cms) traa(x_pts[i],y_pts[i],PL_AH);
		else tras(x_pts[i],y_pts[i],PL_AH);
		ps_point(dot_type,size);
	}

	draw = FALSE;

}
/*
*	PS_CLOSED_CURVE() demarre un panneau avec un pattern gris
*/
void ps_closed_curve(float x_pts[], float y_pts[], int nb_pts, int contour, int rempli, float gris)
{
	int	i;

	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}

	if (pl_cms) traa(x_pts[0],y_pts[0],PL_AH);
	else tras(x_pts[0],y_pts[0],PL_AH);

	for (i = 0; i < nb_pts; i++) 
	{
		if (pl_cms) traa(x_pts[i],y_pts[i],PL_AB);
		else tras(x_pts[i],y_pts[i],PL_AB);
	}
	
	sprintf(pl_buf,"closepath\n");
	fputs(pl_buf,pl_spl);

	draw = FALSE;

	if (contour) 
	{
		sprintf(pl_buf,"gsave\nstroke\ngrestore\n");
		fputs(pl_buf,pl_spl);
	}
	if (rempli)
	{
		sprintf(pl_buf,"gsave %f setgray fill grestore\n",gris);
		fputs(pl_buf,pl_spl);
	}
	/*	reactualisation de ex_x t ex_y */
	
	if (pl_cms)
	{
		ex_x = x_pts[0] * X_PAS;
		ex_y = y_pts[0] * X_PAS;

	} /* fin du if */
	else
	{
		float	xs, xa, ys, ya;
		
		xs = x_pts[0] * pl_ca; ys = y_pts[0] * pl_sa;	/* Projection
								sur X */
		xa = xs - ys;
		xs = x_pts[0] * pl_sa; ys = y_pts[0] * pl_ca;	/* Projection
								sur Y */
		ya = xs + ys;

		ex_x = (xa * pl_rex) + pl_xs;	/*
							 * Mise a l'echelle
							 */
		ex_y = (ya * pl_rey) + pl_ys;

	} /* fin du else */
}
/*
*	PS_FILLED_AREA() demarre un panneau rempli avec la couleur courante (gris ou couleur)
*/
void ps_filled_area(float x_pts[], float y_pts[], int nb_pts)
{
	int	i;

	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}

	if (pl_cms) traa(x_pts[0],y_pts[0],PL_AH);
	else tras(x_pts[0],y_pts[0],PL_AH);

	for (i = 0; i < nb_pts; i++) 
	{
		if (pl_cms) traa(x_pts[i],y_pts[i],PL_AB);
		else tras(x_pts[i],y_pts[i],PL_AB);
	}
	
	draw = FALSE;


	fputs("gsave fill grestore\n",pl_spl);

	/*	reactualisation de ex_x t ex_y */
	
	if (pl_cms)
	{
		ex_x = x_pts[0] * X_PAS;
		ex_y = y_pts[0] * X_PAS;

	} /* fin du if */
	else
	{
		float	xs, xa, ys, ya;
		
		xs = x_pts[0] * pl_ca; ys = y_pts[0] * pl_sa;	/* Projection
								sur X */
		xa = xs - ys;
		xs = x_pts[0] * pl_sa; ys = y_pts[0] * pl_ca;	/* Projection
								sur Y */
		ya = xs + ys;

		ex_x = (xa * pl_rex) + pl_xs;	/*
							 * Mise a l'echelle
							 */
		ex_y = (ya * pl_rey) + pl_ys;

	} /* fin du else */
}
/*
*	PS_RECTANGLE() demarre un rectangle rempli avec la couleur courante (gris ou couleur)
*/

void ps_rectangle(float x_orig, float y_orig, 
		  float x_length, float y_length)
{	
	int	i; 
	float	x_points[4], y_points[4],
		xs, xa, ys, ya;

	ps_movea(x_orig,y_orig);

	x_points[0] = x_orig;		y_points[0] = y_orig + y_length;
	x_points[1] = x_orig + x_length;	y_points[1] = y_orig + y_length;
	x_points[2] = x_orig + x_length; y_points[2] = y_orig;
	x_points[3] = x_orig;		y_points[3] = y_orig; 
	
	
	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}

	if (pl_cms) traa(x_points[0],y_points[0],PL_AH);
	else tras(x_points[0],y_points[0],PL_AH);

	for (i = 0; i < 4; i++) 
	{
		if (pl_cms) traa(x_points[i],y_points[i],PL_AB);
		else tras(x_points[i],y_points[i],PL_AB);
	}
	
	draw = FALSE;


	fputs("gsave fill grestore\n",pl_spl);

	/*	reactualisation de ex_x t ex_y */
	
	if (pl_cms)
	{
		ex_x = x_points[0] * X_PAS;
		ex_y = y_points[0] * X_PAS;

	} /* fin du if */
	else
	{
		xs = x_points[0] * pl_ca; ys = y_points[0] * pl_sa;	/* Projection
								sur X */
		xa = xs - ys;
		xs = x_points[0] * pl_sa; ys = y_points[0] * pl_ca;	/* Projection
								sur Y */
		ya = xs + ys;

		ex_x = (xa * pl_rex) + pl_xs;	/*
						 * Mise a l'echelle
						 */
		ex_y = (ya * pl_rey) + pl_ys;

	} /* fin du else */
}
/*
*	PS_BEZIER() trace une ligne brisee avec 3 pts de controle
*/
void ps_bezier(
	  float	x_pts[],
	  float	y_pts[],
	  int	dot,	/* TRUE if dots must be drawn */
	  int	dot_type,   /* dot type from a list of 16 */
	  float	dot_size)   /* dot size in cms */
{
	int i;
	float	x_i[3], y_i[3];

	if (draw)
	{
		fputs("GSG\n",pl_spl);
		draw = FALSE;
	}
	for (i = 0; i < 3; i++) 
	{
		if (dot)
		{
			ps_movea(x_pts[i],y_pts[i]);
			ps_point(dot_type,dot_size);
		} /* fin du if */
		if (pl_cms)
		{
			x_i[i] = x_pts[i] * X_PAS;
			y_i[i] = y_pts[i] * X_PAS;
		} /* fin du if */
		else
		{
			float	xs, xa, ys, ya;
			
			xs = x_pts[i] * pl_ca; ys = y_pts[i] * pl_sa;	/* Projection
									sur X */
			xa = xs - ys;
			xs = x_pts[i] * pl_sa; ys = y_pts[i] * pl_ca;	/* Projection
								sur Y */
			ya = xs + ys;

			x_i[i] = (xa * pl_rex) + pl_xs;	/*
								* Mise a l'echelle
								*/
			y_i[i] = (ya * pl_rey) + pl_ys;
			
		} /* fin du else */
	}
	fprintf(pl_spl,"%.1f %.1f %.1f %.1f %.1f %.1f curveto\n",
		x_i[0], y_i[0],
		x_i[1], y_i[1],
		x_i[2], y_i[2]);
		
	ex_x = x_i[2];		/* update du pt courant */
	ex_y = y_i[2];
	fputs("GSG\n",pl_spl);
	draw = FALSE;
} /* end of ps_besier */

/*
*
*	dessine un point centre sur le point courant
*
*/
void ps_point(int typ_pt, float	dot_size)
{
	float	x[8], y[8];
#define PLEIN   1,0.
#define VIDE    0,1.

	if (dot_size == 0.) dot_size = 0.2; /* default value */
	switch (typ_pt)
	{
		case 1:			/* carre vide */
			traa(-(dot_size/2.),-(dot_size/2.),PL_RH);
			x[0] = 0.;	y[0] = dot_size; 
			x[1] = dot_size; 	y[1] = 0.; 
			x[2] = 0.; 	y[2] = -dot_size; 
			x[3] = -dot_size; 	y[3] = 0.; 
			do_point(x,y,4,TRUE,VIDE);
			traa((dot_size/2.),(dot_size/2.),PL_RH);
			break;
		case 2:			/* carre plein */
			traa(-(dot_size/2.),-(dot_size/2.),PL_RH);
			x[0] = 0.;	y[0] = dot_size; 
			x[1] = dot_size; 	y[1] = 0.; 
			x[2] = 0.; 	y[2] = -dot_size; 
			x[3] = -dot_size; 	y[3] = 0.; 
			do_point(x,y,4,TRUE,PLEIN);
			traa((dot_size/2.),(dot_size/2.),PL_RH);
			break;
		case 3:			/* losange vide */
			traa(-(dot_size/2.),0.,PL_RH);
			x[0] = (dot_size/2.);	y[0] = (dot_size/2.); 
			x[1] = (dot_size/2.); 	y[1] = -(dot_size/2.); 
			x[2] = -(dot_size/2.); 	y[2] = -(dot_size/2.); 
			x[3] = -(dot_size/2.); 	y[3] = (dot_size/2.); 
			do_point(x,y,4,TRUE,VIDE);
			traa((dot_size/2.),0.,PL_RH);
			break;
		case 4:			/* losange plein */
			traa(-(dot_size/2.),0.,PL_RH);
			x[0] = (dot_size/2.);	y[0] = (dot_size/2.); 
			x[1] = (dot_size/2.); 	y[1] = -(dot_size/2.); 
			x[2] = -(dot_size/2.); 	y[2] = -(dot_size/2.); 
			x[3] = -(dot_size/2.); 	y[3] = (dot_size/2.); 
			do_point(x,y,4,TRUE,PLEIN);
			traa((dot_size/2.),0.,PL_RH);
			break;
		case 5:			/* triangle ^ vide */
			traa(-(dot_size/2.),-(dot_size/2.),PL_RH);
			x[0] = (dot_size/2.);	y[0] = dot_size; 
			x[1] = (dot_size/2.); 	y[1] = -dot_size; 
			x[2] = -dot_size; 	y[2] = 0.; 
			do_point(x,y,3,TRUE,VIDE);
			traa((dot_size/2.),(dot_size/2.),PL_RH);
			break;
		case 6:			/* triangle ^ plein */
			traa(-(dot_size/2.),-(dot_size/2.),PL_RH);
			x[0] = (dot_size/2.);	y[0] = dot_size; 
			x[1] = (dot_size/2.); 	y[1] = -dot_size; 
			x[2] = -dot_size; 	y[2] = 0.; 
			do_point(x,y,3,TRUE,PLEIN);
			traa((dot_size/2.),(dot_size/2.),PL_RH);
			break;
		case 7:			/* triangle v vide */
			traa(-(dot_size/2.),(dot_size/2.),PL_RH);
			x[0] = (dot_size/2.);	y[0] = -dot_size; 
			x[1] = (dot_size/2.); 	y[1] = dot_size; 
			x[2] = -dot_size; 	y[2] = 0.; 
			do_point(x,y,3,TRUE,VIDE);
			traa((dot_size/2.),-(dot_size/2.),PL_RH);
			break;
		case 8:			/* triangle v plein */
			traa(-(dot_size/2.),(dot_size/2.),PL_RH);
			x[0] = (dot_size/2.);	y[0] = -dot_size; 
			x[1] = (dot_size/2.); 	y[1] = dot_size; 
			x[2] = -dot_size; 	y[2] = 0.; 
			do_point(x,y,3,TRUE,PLEIN);
			traa((dot_size/2.),-(dot_size/2.),PL_RH);
			break;
		case 9:			/* rond vide */
			traa(-(dot_size/2.),-(dot_size/4.),PL_RH);
			x[0] = 0.;	y[0] = (dot_size/2.); 
			x[1] = (dot_size/4.); 	y[1] = (dot_size/4.); 
			x[2] = (dot_size/2.); 	y[2] = 0.; 
			x[3] = (dot_size/4.); 	y[3] = -(dot_size/4.); 
			x[4] = 0.; 	y[4] = -(dot_size/2.); 
			x[5] = -(dot_size/4.); 	y[5] = -(dot_size/4.); 
			x[6] = -(dot_size/2.); 	y[6] = 0.; 
			x[7] = -(dot_size/4.); 	y[7] = (dot_size/4.); 
			do_point(x,y,8,TRUE,VIDE);
			traa((dot_size/2.),(dot_size/4.),PL_RH);
			break;
		case 10:		/* rond plein */
			traa(-(dot_size/2.),-(dot_size/4.),PL_RH);
			x[0] = 0.;	y[0] = (dot_size/2.); 
			x[1] = (dot_size/4.); 	y[1] = (dot_size/4.); 
			x[2] = (dot_size/2.); 	y[2] = 0.; 
			x[3] = (dot_size/4.); 	y[3] = -(dot_size/4.); 
			x[4] = 0.; 	y[4] = -(dot_size/2.); 
			x[5] = -(dot_size/4.); 	y[5] = -(dot_size/4.); 
			x[6] = -(dot_size/2.); 	y[6] = 0.; 
			x[7] = -(dot_size/4.); 	y[7] = (dot_size/4.); 
			do_point(x,y,8,TRUE,PLEIN);
			traa((dot_size/2.),(dot_size/4.),PL_RH);
			break;
		case 11:			/* triangle < vide */
			traa(-(dot_size/2.),0.,PL_RH);
			x[0] = dot_size;	y[0] = (dot_size/2.); 
			x[1] = 0.; 	y[1] = -dot_size; 
			x[2] = -dot_size; 	y[2] = (dot_size/2.); 
			do_point(x,y,3,TRUE,VIDE);
			traa((dot_size/2.),0.,PL_RH);
			break;
		case 12:			/* triangle < plein */
			traa(-(dot_size/2.),0.,PL_RH);
			x[0] = dot_size;	y[0] = (dot_size/2.); 
			x[1] = 0.; 	y[1] = -dot_size; 
			x[2] = -dot_size; 	y[2] = (dot_size/2.); 
			do_point(x,y,3,TRUE,PLEIN);
			traa((dot_size/2.),0.,PL_RH);
			break;
		case 13:			/* triangle > vide */
			traa((dot_size/2.),0.,PL_RH);
			x[0] = -dot_size;	y[0] = (dot_size/2.); 
			x[1] = 0.; 	y[1] = -dot_size; 
			x[2] = dot_size; 	y[2] = (dot_size/2.); 
			do_point(x,y,3,TRUE,VIDE);
			traa(-(dot_size/2.),0.,PL_RH);
			break;
		case 14:			/* triangle > plein */
			traa((dot_size/2.),0.,PL_RH);
			x[0] = -dot_size;	y[0] = (dot_size/2.); 
			x[1] = 0.; 	y[1] = -dot_size; 
			x[2] = dot_size; 	y[2] = (dot_size/2.); 
			do_point(x,y,3,TRUE,PLEIN);
			traa(-(dot_size/2.),0.,PL_RH);
			break;
		case 15:			/* croix X */
			traa(-(dot_size/2.),(dot_size/2.),PL_RH);
			traa(dot_size,-dot_size,PL_RB);
			traa(-dot_size,0.,PL_RH);
			traa(dot_size,dot_size,PL_RB);
			traa(-(dot_size/2.),-(dot_size/2.),PL_RH);
			break;
		case 16:			/* croix + */
			traa(0.,(dot_size/2.),PL_RH);
			traa(0.,-dot_size,PL_RB);
			traa(-(dot_size/2.),(dot_size/2.),PL_RH);
			traa(dot_size,.0,PL_RB);
			traa(-(dot_size/2.),0.,PL_RH);
			break;
		default:			/* petit carre */
			traa(-(dot_size/4.),(dot_size/4.),PL_RH);
			traa((dot_size/2.),0.,PL_RB);
			traa(0.,-(dot_size/2.),PL_RB);
			traa(-(dot_size/2.),.0,PL_RB);
			traa(0.,(dot_size/2.),PL_RB);
			traa((dot_size/4.),-(dot_size/4.),PL_RH);
	}
}
/*
*	do_point() demarre un panneau avec un pattern gris
*/
void do_point(float *x_pts, float *y_pts, int nb_pts, int contour, int rempli, float gris)
{
	int	i;

	for (i = 0; i < nb_pts; i++) 
		traa(x_pts[i],y_pts[i],PL_RB);
	
	sprintf(pl_buf,"closepath\n");
	fputs(pl_buf,pl_spl);

	draw = FALSE;

	if (contour) 
	{
		sprintf(pl_buf,"gsave\nstroke\ngrestore\n");
		fputs(pl_buf,pl_spl);
	}
	if (rempli)
	{
		sprintf(pl_buf,"gsave %f setgray fill grestore\n",gris);
		fputs(pl_buf,pl_spl);
	}
	/*	reactualisation de ex_x t ex_y non utile */
}
/*
 * Draws a frame whose position of the infer-left angle is at the current
 * position (sh)
 */
void ps_frame(float width,float height)
	/* of the rectangle */
       	/* of the rectangle */
{
	ps_drawr(0.,height) ;
	ps_drawr(width,0.) ;
	ps_drawr(0.,-height);
	ps_drawr(-width,0.);
	
} /* fin du ps_frame */
/*
 * Writes a character string horizontally, at current position
 * (jfv)
 */
void ps_write(char *string)
{
	ps_chara(0.,0.,PL_RH,string,hl_crte,0);
} /* fin du ps_write */
/*
 * Writes a character string, with x and y as relative positions
 * (sh)
 */
void ps_writea(float x, float y, char *string, int rotation)
{
	ps_chara(x,y,PL_AH,string,hl_crte,rotation);

} /* fin du ps_writea */
/*
 * Writes a character string, with x and y as relative positions
 * (sh)
 */
void ps_writer(float x, float y, char *string, int rotation)
{
	ps_chara(x,y,PL_AH,string,hl_crte,rotation);
} /* fin du ps_writer */
/*
 * Writes a character string horizontally, with x and y as absolute positions
 * (sh)
 */
void ps_writeah(float x, float y, char *string, float taille)
{
	ps_chara(x,y,PL_AH,string,taille,0);
} /* fin du pwriteah */
/*
 * Writes a character string vertically, with x and y as absolute positions
 * (sh)
 */

void ps_writeav(float x, float y, char *string, float taille)
{
	ps_chara(x,y,PL_AH,string,taille,90);
} /* fin du ps_writeav */
/*
 * Writes a float vertically, with x and y as absolute positions
 * (sh)
 */

void ps_writefloatav(float x, float y, float number, float taille)
{
	char buffer[20] ;

	sprintf(buffer,"%f",number) ;
	ps_chara(x,y,PL_AH,buffer,taille,90);
	ps_home();
} /* fin du ps_write */

/*
 * Writes a float horizontally, with x and y as absolute positions
 * (sh)
 */
void ps_writefloatah(float x, float y, float number, float taille)
{
	char buffer[20] ;

	sprintf(buffer,"%f",number) ;
	ps_chara(x,y,PL_AH,buffer,taille,0);
} /* fin du pwrite */
/*
 * Writes an integer vertically, with x and y as absolute positions
 * (sh)
 */

void ps_writeintav(float x, float y, int number, float taille)
{
	char buffer[20] ;

	sprintf(buffer,"%d",number) ;
	ps_chara(x,y,PL_AH,buffer,taille,90);
} /* fin du ps_write */

/*
 * Writes an integer horizontally, with x and y as absolute positions
 * (sh)
 */
void ps_writeintah(float x, float y, int number, float taille)
{
	char buffer[20] ;

	sprintf(buffer,"%d",number) ;
	ps_chara(x,y,PL_AH,buffer,taille,0);
} /* fin du ps_write */

/*
 * PS_INCLUDE pour inclure un fichier postscript encapsule
 */
int ps_include(char *file_name)
{
	FILE	*ps_file;
	char	c;

	if((ps_file = fopen(file_name,"r")) == NULL)
	{
		fprintf(stderr,"\007PLOT_PS: ps_include Warning: Impossible to open the file %s\n",file_name);
		return(FALSE);
	}
	else
	{
		fprintf(pl_spl,"%%%%\n%%%% PLOT_PS beginning of ps_include of file %s\n%%%%\n",file_name);
		while((c = getc(ps_file)) != EOF) fputc(c,pl_spl);
		fprintf(pl_spl,"%%%%\n%%%% PLOT_PS end of ps_include of file %s\n%%%%\n",file_name);
		fclose(ps_file);
		return(TRUE);
	} /* fin du else */
} /* fin du ps_include */
/*
 * PS_CODE pour inclure du code postscript
 */
void ps_code(char *ps_code)
{

	fprintf(pl_spl,"%%%% PLOT_PS beginning of ps_code inclusion\n");
	fprintf(pl_spl,"%s\n",ps_code);
	fprintf(pl_spl,"%%%% PLOT_PS end of ps_code inclusion\n");
} /* fin du ps_include */

   /************************
   * AXES sans graduations *
   ************************/
/*
 * Trace 2 axes X and Y, without tick marks nor legend except max and min (sh)
 */
   
void ps_axes(float tx,float ty,float mnx,float mxx,float mny,
	float mxy,char *legendx,char *legendy,float hy, int cadre)
{
	float px,py;
	
   /* mise a l'echelle */
	px = tx/(mxx-mnx);
	py = ty/(mxy-mny);
	mnx=mnx*px;
	mxx=mxx*px;
	mny=mny*py;
	mxy=mxy*py;

  /* pose des legendes */
	ps_writeah(tx/4.,-hy-.6,legendx,hy) ;
	ps_writeav(-hy-.6,ty/4.,legendy,hy);
	ps_home();

   /* trace du cadre si demande */
	if (cadre != 0 )
	{
		ps_home() ;
		ps_drawa(0.,mxy+mny)   ;
		ps_drawa(mnx+mxx,mxy+mny)   ;
		ps_drawa((mxx+mnx),0.);
		ps_drawa(0.,0.);
		
	} /* fin du if */

   /* trace des axes */
	ps_movea(0.,-mny * (mny<=0.) );
	ps_drawa((mxx-mnx),-mny * (mny<=0.));
	ps_movea(-mnx * ( mnx<= 0.),0.);
	ps_drawa(-mnx * ( mnx<= 0.),-mny+mxy);
	ps_home();

} /* fin du ps_axes */

   /************************
   * AXES avec graduations *
   ************************/
void ps_ticked_axes(float pos_x, float pos_y, float mnx, float mxx, 
	       float b_tick_x, float e_tick_x, 
	       float mny, float mxy, 
	       float b_tick_y, float e_tick_y,
	       float htick, char *legendx, 
	       char *fmtx, char *legendy, char *fmty, 
	       float h_txt, int cadre)
{

	ps_X_axis(mnx,mxx,htick,b_tick_x,e_tick_x,legendx,fmtx,h_txt);
	ps_mover(pos_x-mnx,-(pos_y-mny));
	ps_Y_axis(mny,mxy,htick,b_tick_y,e_tick_y,legendy,fmty,h_txt);

  /* trace du cadre si demande */
	if (cadre)
	{
		ps_mover(0.,mxy-mny) ;
		ps_drawr(mxx-mnx,0.)   ;
		ps_drawr(0.,-(mxy-mny)) ;
		ps_mover(-(mxx-mnx),0.)   ;
	} /* fin du if */
	
} /* fin du ps_ticked_axes */


void ps_X_axis(float mnx, float mxx, float htick, 
	  float b_tick, float e_tick, 
	  char *legendx, char *fmt, float hy)
{
	float	duree,
		k1,
		pas1, val,
		ancien_x, ancien_y,
		x_crt,	y_crt,
		where;
	int	i, l_legend;

	char	buffer[20], modulo;
	
	if (fmt[0] == EOF) 
	  strcpy(fmt,"%.2f"); 

	duree = mxx - mnx;
	if (duree <= 0.)
	{
		fprintf(stderr,"\007PLOT_PS: ps_X_axis: null axis length! (min = max)\n");
		return;
	}

	/* on se rappelle de la position courante */

	x_crt = ex_x;
	y_crt = ex_y;

   /* trace des axes */
	ps_drawr(duree,0.);
	ps_mover(-duree,0.);

  /* Trace des graduations */
	/* horizontalement */
	
	k1 = duree;
	if (k1 < 1.)  while ( k1 < 1.) k1 *= 10. ;
	if (k1 > 10.)  while( k1 > 10.) k1 /= 10. ;

	pas1 = duree / (k1 * 10.) ;

	where = 0.;
	ps_mover((b_tick-mnx),0.);
	i = 0;
	ancien_x = ex_x;
	ancien_y = ex_y;
	while( where < (e_tick - b_tick))
	{
		val = where + b_tick;
		modulo = 'l';
		if ( ! (i % 5) ) modulo = 'm';
		if ( ! (i % 10) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(0.,-3.*htick,PL_RB);
				sprintf(buffer,fmt,val);
				traa(0.,-hy,PL_RH);
				ps_chara(0.,0.,PL_RH,buffer,hy,0);
				break;
			case 'm':
				traa(0.,-2.*htick,PL_RB);
				break;
			case 'l':
				traa(0.,-htick,PL_RB);
		}
		p_depl(ancien_x,ancien_y,PL_AH);
		where += pas1;
		ps_mover(where,0.);
		i++;
	}
	p_depl(x_crt,y_crt,PL_AH);
	ps_mover(duree,0.);
	l_legend = strlen(legendx);
	traa(-hy*l_legend*.6,-(4*htick)-2*hy,PL_RH);
	ps_chara(0.1,0.,PL_RH,legendx,hy,0);
	p_depl(x_crt,y_crt,PL_AH);
} /* fin du ps_X_axis */

void ps_Y_axis( float mny, float mxy, float htick, 
	   float b_tick, float e_tick, 
	   char *legendy, char *fmt, float hy)
{
	float	duree,
		k2,
		pas2, val,
		ancien_x, ancien_y,
		x_crt,	y_crt,
		where;
	int	i, l_legend;
	char	buffer[20], modulo;

	if (*fmt == 0) strcpy(fmt,"%.2f"); 

	duree = mxy - mny;

	if (duree <= 0.)
	{
		fprintf(stderr,"\007PLOT_PS: ps_Y_axis: null axis length! (min = max)\n");
		return;
	}

	/* on se rappelle de la position courante */

	x_crt = ex_x;
	y_crt = ex_y;

   /* trace des axes */
	ps_drawr( 0.,duree);
	ps_mover( 0.,-duree);


  /* Trace des graduations */
	/* verticalement */

	k2 = duree ;
	if (k2 < 1.)  while (k2 < 1.) k2 *= 10. ;
	if (k2 > 10.)  while (k2 > 10.) k2 /= 10. ;
	pas2 = duree / (k2 * 10.) ;

	where = 0.;
	ps_mover(0.,(b_tick-mny));
	i = 0;
	ancien_x = ex_x;
	ancien_y = ex_y;
	while( where < (e_tick-b_tick))
	{
		val = where + b_tick;
		modulo = 'l';
		if ( ! (i % 5) ) modulo = 'm';
		if ( ! (i % 10) ) modulo = 'b';
		switch (modulo)
		{
			case 'b':
				traa(-3.*htick,0.,PL_RB);
				sprintf(buffer,fmt,val);
				l_legend = strlen(buffer);
				traa(-(hy*l_legend*.6),0.0,PL_RH);
				ps_chara(0.,0.,PL_RH,buffer,hy,0);
				break;
			case 'm':
				traa(-2.*htick,0.,PL_RB);
				break;
			case 'l':
				traa(-htick,0.,PL_RB);
		}
		p_depl(ancien_x,ancien_y,PL_AH);
		where += pas2;
		ps_mover(0.,where);
		i++;
  	}
	p_depl(x_crt,y_crt,PL_AH);
	ps_mover( 0.,duree);
	l_legend = strlen(legendy);
/*	traa(-hy*l_legend*.6,hy,PL_RH); */
	traa(-1.5,hy,PL_RH); 
	ps_chara(0.,0.,PL_RH,legendy,hy,0);
	p_depl(x_crt,y_crt,PL_AH);
} /* fin du ps_Y_axis */

   /************
   *  PS_HISTO *
   ************/

void ps_histo(int *popu, float tx, float ty, 
	 float gap, int nb_class, int flag)
     /* flag: 0 pas de calcul 1 vertical 2 horizontal autre = tout */
{
	int i;
	float px,py,
	      maxx,minx,
	      maxy,miny,
	      inter_popu,
	      ev,vv,etv,
	      eh,vh,eth;

	if (nb_class == 0)
	{
		fprintf(stderr,"\007PLOT_PS: ps_histo: Class number = 0\n");
	} /* fin du if */
	/* calcul des maxi et mini */
	minx = miny = maxx = maxy = 0. ;
	i = 0 ;
	while (i<nb_class)
	{
		inter_popu = (float)(popu[i]) ;
		if (inter_popu>maxy) maxy = inter_popu ;
		if (inter_popu<miny) miny = inter_popu ;
		i++ ;
	} /* fin du while */
	maxx = gap * nb_class ;
	if (miny > 0.) miny = 0. ;

	/* changement d'echelle */
	px = tx / (maxx-minx) ;
	py = ty / (maxy-miny) ;

	/* trace de l'histogramme */
	i=0 ;
	while ( (i+1) < nb_class )
	{
		inter_popu = (float)(popu[i]) ;
		ps_movea((float)(i) * gap * px, inter_popu * py) ;
		ps_drawa((float)(i+1) * gap * px, inter_popu * py) ;
		ps_drawa((float)(i+1) * gap * px, (float)(popu[i+1]) * py) ;
		i++ ;
	} /* fin du while */
	ps_drawa((float)(i+1) * gap * px, (float)(popu[i]) * py) ;
	ps_drawa((float)(i+1) * gap * px, 0.) ;	

   /* donnees statistiques */
	if (flag != 0)
	{
		
	stat_histo(&eh,&vh,&eth,&ev,&vv,&etv,popu,nb_class) ;

	if (flag !=2)
		
		{
			ps_movea(0.,0.);
			ps_gray(.3) ;
			ps_movea(eh*gap*px,0.) ;
			ps_drawa(eh*gap*px,ty) ;
			ps_writeav(eh*gap*px+.3,ty-.9,"Mean",.2) ;
			ps_gray(.7) ;
			ps_movea((eh+(1.96*eth))*px*gap,0.) ;
			if ( (eh+(1.96*eth))*px*gap < maxx*px ) ps_drawa((eh+(1.96*eth))*px*gap,6.) ;
			ps_movea((eh-(1.96*eth))*px*gap,0.) ;
			if ((eh-(1.96*eth))>0.) ps_drawa((eh-(1.96*eth))*px*gap,6.) ;	
		} /* fin du if */

	if (flag != 1)
		{
		
			ps_gray(.3);
			ps_movea(0.,ev*py) ;
			ps_drawa(maxx*px,ev*py) ;
			ps_writeah(maxx*px-.9,ev*py+.1,"Mean",.2) ;
			ps_gray(.7);
			ps_movea(0.,(ev+1.96*etv)*py) ;
			if ((ev+1.96*etv)*py < ty) ps_drawa(maxx*px,(ev+1.96*etv)*py) ;
			ps_movea(0.,(ev-1.96*etv)*py) ;
			if (ev-1.96*etv >0.) ps_drawa(maxx*px,(ev-1.96*etv)*py) ;	
			ps_movea(0.,0.) ;
		} /* fin du if */

	} /* fin du if */
	ps_gray(0.) ;

} /* fin du ps_histo */

   /*******************************************
   * Calculs statistiques: esperance,variance *
   *******************************************/

void stat_histo(float *esperanceh,float *varianceh,float *ecart_typeh,
	   float *esperancev,float *variancev,float *ecart_typev,
	   int *histo,int nb_max)
{
	int i, total;
	int maxy, miny; 
	float esperance_x2h = 0.,esperance_x2v = 0.;

	maxy = miny = histo[0];
	*esperancev = 0.;
	*esperanceh = 0.;
	total = 0 ;
	for (i=0;i<nb_max;i++)
	{
		if (histo[i]<miny) miny=histo[i] ;
		if (histo[i]>maxy) maxy=histo[i] ;
		total += histo[i] ;
	} /* fin du for */
	if (miny>0) miny = 0;
	
	for (i=0;i<nb_max;i++)
	{
		*esperanceh += (float)(histo[i]*(i+1)) ;
		esperance_x2h += (float)(histo[i]*(i+1)*(i+1));
		*esperancev += (float)(histo[i]) ;
		esperance_x2v += ((float)(histo[i])*(float)(histo[i]) );
	} /* fin du for */

	*esperanceh = *esperanceh / (float)(total) ;
	*esperancev = *esperancev / (float)(nb_max) ;
	esperance_x2h = esperance_x2h / (float)(total) ;
	esperance_x2v = esperance_x2v / (float)(nb_max) ;
	*variancev = esperance_x2v - (*esperancev * *esperancev) ;
	*varianceh = esperance_x2h - (*esperanceh * *esperanceh) ;	
	*ecart_typeh = sqrt(*varianceh) ;
	*ecart_typev = sqrt(*variancev) ;
	
} /* fin du stat_histo */


