/*
			P  L  O  T  3 D _  P  S 
			======================

		   Routines de dessin sur la LN03R PostScript.

	NIVEAU 3 : Ce jeu de routine regroupe les routines 3D  du
		logiciel plot_ps. Elle demande a etre chargees avec la
		librairie mathematique.

		Version 1 et 2
			Juillet 90
		Version 3
			31 juillet 90
		Version 4.2
			11 octobre 91
			
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/

/*
title	PLOT3_PS Fonctions de dessins complexes
index		Fonctions de dessins complexes


description

	           Routines de dessin sur la PLOT_PS.

	   NIVEAU 3 : Ce jeu de routine regroupe les routines
	                   appelables par l'utilisateur permettant
			   des fonctions de niveau eleve.


author

	Jean-Francois Vibert, Sylvain Hanneton

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	plot0_ps, plot1_ps, plot2_ps et plot3_ps
			
newpage
*/
#ifdef	DOCUMENTATION

title	PS_3DSETUP Set up a 3D reference
index		 Set up a 3D reference

synopsis
	#include <plot_ps.h>
	ps_3Dsetup(taillex,tailley,taillez,
		minx,maxx,miny,maxy,minz,maxz,
		angle,phi,dist,dist2);
	float taillex,tailley,taillez,       /* axes lenghts in cm*/
	      minx,maxx,miny,maxy,minz,maxz, /* minima and maxima
						of user values */
	      angle,phi,		     /* angular coord of 3D ref */
	      dist,			     /* distance reference-screen */
	      dist2;			     /* distance eye-screen */	
description

	ps_3Dsetup must be call at the very begining of a 3D drawing.

	The reference is oriented as the following if the angles are 0:
	
	\nofill
		- y axe is upward
		- x axe is rightward
		- z axe is frontward (toward the observer)
	\fill

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

See also

	PLOT1 et PLOT2,p3Dbar,p3Dframe,p3Dbox,p3Dcalcul...
newpage
	
title	PS_3DCALCUL Computes projection and rotation from 3D to 2D space
index		    computes projection and rotation from 3D to 2D space

synopsis
	#include <plot_ps.h>

	ps_3Dcalcul(y1,z1,x1,x,y)
	float    x1,y1,z1,    /* 3D coordinates */
		*x,*y;	/* 2D coordinates */

description

	ps_3Dcalcul translates the coordinates from the 3D reference (drawn by
	p3Daxes for example) to a 2D reference x,y of the corresponding
	projection on the screen.

	The reference is oriented as the following if the angles are 0:
	
	\nofill
		- y axe is upward
		- x axe is rightward
		- z axe is frontward (toward the observer)
	\fill
author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI & 
	Ecole Normale Superieure

See also

	PLOT1 et PLOT2,ps_3Dbar,ps_3Dframe,ps_3Dbox,ps_3Daxes...
newpage
title	PS_3DAXES Draws a 3D reference
index		 Draws a 3D reference

synopsis
	#include <plot_ps.h>
	ps_3Daxes(boite,outline);
	int   boite,
	      outline;     /* true if a box is needed */
description

	ps_3Daxes draws a reference triaxes in 3D whoses axes have not tick
	marks. If the BOITE flag is TRUE, then, an open box is drawn in the
	background in order to add depth feeling to the drawing. If the out
	line flag is TRUE, the sides of the box are outlined.

	The reference is oriented as the following if the angles are 0:
	
	\nofill
		- y axe is upward
		- x axe is rightward
		- z axe is frontward (toward the observer)
	\fill

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

See also

	PLOT1 et PLOT2,ps_3Dbar,ps_3Dframe,ps_3Dbox,ps_3Dcalcul...
newpage
	
title	PS_3DBOX   Draws a box in 3D (only 3 faces are represented)
index		 Draws a box in 3D

synopsis
	#include <plot_ps.h>
	
	ps_3Dbox(k,l,m,la,ep,lo,outline)
	float   k,l,m,     /* coordinates of the origin point of the box */
		la,ep,lo;  /* dimensions of the box */
	int     outline;   /* outline flag */
	
description

	PS_3DBOX draws the back sides of a box in a way to always get visible
	the inside of the box. The given coordinates are those of the bottom,
	left and back corner. Then are width, height and thickness of the
	box. Hidden surfaces are correctly processed. It draws a box alone,
	without axes, and the box can be drawn anywhere, at the difference
	of ps_3Daxes. If the flag {\tt outline} is true, the visible sides of
	the box are outlined. 

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

	
See also

	PLOT1 et PLOT2,ps_3Dbar,ps_3Dframe,ps_3Dbox,ps_3Dcalcul...
newpage
	
	
title	PS_3DBAR   Draws a bar in 3D
index		 Draws a bar in 3D

synopsis
	#include <plot_ps.h>

	ps_3Dbar(k,l,m,la,ep,lo,outline)
	float   k,l,m,     /* coordinates of the origin point of the bar */
		la,ep,lo;  /* dimensions of the bar */
	int     outline ;  /* outline flag */
description

	ps_3Dbar draws the front sides of a bar in a way to always get visible
	the outside of the bar. The given coordinates are those of the
	bottom, left and back corner. Then are width, height and thickness
	of the bar. Hidden surfaces are correctly processed. It draws a bar
	alone, without axes, and the bar can be drawn anywhere. If the {\tt
	outline} flag is true, the sides of the bar are outlined.

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI
	ENS
	
See also
	PLOT1 et PLOT2,ps_3Daxes,ps_3Dframe,ps_3Dbox,ps_3Dcalcul...
newpage

title	PS_3DFRAME   Draws a bar skeleton in 3D 
index		   Draws a bar skeleton in 3D

synopsis
	#include <plot_ps.h>
	
	ps_3Dframe(k,l,m,la,ep,lo)
	float   k,l,m,     /* coordinates of the origin point of the bar */
		la,ep,lo;  /* dimensions of the box */
description

	ps_3Dframe draws the skeleton of a bar. The given coordinates are
	those of the bottom, left and back corner. Then are width, height
	and thickness of the bar.  It draws a bar alone, without axes, and
	the bar frame can be drawn anywhere.

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

See also

	PLOT1 et PLOT2,ps_3Dbar,ps_3Dframe,ps_3Dbox,ps_3Dcalcul...
newpage

title	PS_3DCLOSEBOX   Close a box with skeleton in 3D 
index		      Close a box with skeleton in 3D

synopsis
	#include <plot_ps.h>
	
	ps_3Dclosebox(k,l,m,la,ep,lo)
	float   k,l,m,     /* coordinates of the origin point of the bar */
		la,ep,lo;  /* dimensions of the box */
description

	ps_3Dclosebox closes a box (drawn with ps_3Dbox for example) with the
	corresponding front skeleton. The given coordinates are
	those of the bottom, left and back corner. Then are width, height
	and thickness of the box. It allows to enclose items in this box.
	With this function you can also outline the front sides of a bar
	drawn with ps_3Dbar for example.

	ps_3Dclosebox must be invoked after all the objects enclosed in the
	box are drawn. 

author

	Sylvain HANNETON

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI &
	Ecole Normale Superieure

See also

	PLOT1 et PLOT2,ps_3Dbar,ps_3Dframe,ps_3Dbox,ps_3Dcalcul...
	
#endif

#include <stdio.h>
#include <math.h>

#include "plot_ps.h"

#ifdef	M_PI
#	define	PI	M_PI
#else
#	define	PI	3.1415926535
#endif


/**************************************
* FONCTIONS UTILISEES par troisDstate *
**************************************/
static   float	kx,
		ky,
		kz,
		minx,maxx,
		miny,maxy,
		minz,maxz,
		taillex,
		tailley,
		taillez;
/*
* Computes the 3D reference
*/
void ps_3Dsetup(float tx, float ty, float tz, float ax, float bx, 
	   float ay, float by, float az, float bz, 
	   float angle, float phi, float d1, float d2)
{
	int inter;

	taillex = tx ;
	tailley = ty ;
	taillez = tz ;
	maxx = bx;
	maxy = by;
	maxz = bz;
	minx = ax;
	miny = ay;
	minz = az;
	kx = taillex/(bx-ax) ;
	ky = tailley/(by-ay) ;
	kz = taillez/(bz-az) ;
	ps_3D_dist_1 = d1 ;
	ps_3D_dist_2 = d2 ;
	ps_3D_angle = angle ;
	ps_3D_phi = phi;
	inter = ps_3D_angle/(2.*PI) ;
	ps_3D_angle = ( ps_3D_angle - inter*(2.*PI)) ;
	inter = ps_3D_phi/(2.*PI) ;
	ps_3D_phi = (ps_3D_phi - inter*(2.*PI)) ;
	cps_3D_angle = cos(ps_3D_angle);
	sps_3D_angle = sin(ps_3D_angle);
	cps_3D_phi = cos(ps_3D_phi) ;
	sps_3D_phi = sin(ps_3D_phi);
	


} /* fin du ps_3Dsetup */


/*
 * Traces a triaxe repere with rotation (sh)
 */
 
void ps_3Daxes(int boite, int outline)
{
	float	x[8],y[8],
		a,b;

	int	i,j,
		indps_3D_angle,
		indps_3D_phi,
		tab[5][5];
	
	/* calcul des indices */
	indps_3D_angle = (ps_3D_angle/(PI/2.)) ;
	indps_3D_phi = (ps_3D_phi/(PI/2.)) ;

	/* Initialisation du tableau */
	for (i=0; i < 5 ; i++ ) for (j=0; j < 5 ; j++ ) tab[i][j] = 0 ;
	tab[indps_3D_angle][indps_3D_phi] = 1 ;
	
	/* trace de la boite */
	
   if (boite != 0 )
   {
	/* Face 4 */
	if ((tab[0][0] || tab[1][0] || tab[0][1] || tab[1][1] ||
	     tab[2][2] || tab[3][2] || tab[2][3] || tab[3][3]))
	{
	ps_home();
	ps_3Dcalcul(minx,miny,maxz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(minx,maxy,maxz,x+1,y+1);
	ps_3Dcalcul(minx,maxy,minz,x+2,y+2);
	ps_3Dcalcul(minx,miny,minz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.6);
	} /* fin du if */

	/* Face 5 */
	if (tab[0][0] || tab[1][0] || tab[2][0] || tab[3][0] ||
	    tab[0][3] || tab[1][3] || tab[2][3] || tab[3][3])
	{
	ps_3Dcalcul(minx,miny,maxz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(maxx,miny,maxz,x+1,y+1);
	ps_3Dcalcul(maxx,miny,minz,x+2,y+2);
	ps_3Dcalcul(minx,miny,minz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.5);
	} /* fin du if */

	/* Face 3 */
	if (tab[0][0] || tab[3][0] || tab[0][1] || tab[3][1] ||
	    tab[1][2] || tab[2][2] || tab[1][3] || tab[2][3])
	{
	ps_3Dcalcul(minx,maxy,minz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(maxx,miny,minz,x+2,y+2);
	ps_3Dcalcul(maxx,maxy,minz,x+1,y+1);
	ps_3Dcalcul(minx,miny,minz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.4);
	} /* fin du if */

	/* Face 2 */
	if (tab[2][0] || tab[3][0] || tab[2][1] || tab[3][1] ||
	    tab[0][2] || tab[1][2]|| tab[0][3] || tab[1][3])
	{
	ps_3Dcalcul(maxx,maxy,maxz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(maxx,miny,maxz,x+1,y+1);
	ps_3Dcalcul(maxx,miny,minz,x+2,y+2);
	ps_3Dcalcul(maxx,maxy,minz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.7);
	} /* fin du if */

	/* Face 6 */
	if ((indps_3D_phi==2) || (indps_3D_phi==1))
	{
	ps_3Dcalcul(maxx,maxy,maxz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(minx,maxy,maxz,x+1,y+1);
	ps_3Dcalcul(minx,maxy,minz,x+2,y+2);
	ps_3Dcalcul(maxx,maxy,minz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.8);
	} /* fin du if */

	/* Face 1 */
	if (tab[1][0] || tab[2][0] || tab[1][1] || tab[2][1] ||
	    tab[0][2] || tab[3][2]||tab[0][3]||tab[3][3])
	{
	ps_3Dcalcul(maxx,maxy,maxz,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(minx,maxy,maxz,x+1,y+1);
	ps_3Dcalcul(minx,miny,maxz,x+2,y+2);
	ps_3Dcalcul(maxx,miny,maxz,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.9);
	} /* fin du if */

   } /* fin du if */

	/* trace des axes */
	ps_home();
	ps_3Dcalcul(maxx,0.,0.,&a,&b);
	ps_drawa(a,b);/* axe horizontal */
	ps_writeah(a+.1,b+.1,"X",.2);
	ps_home();
	ps_3Dcalcul(0.,maxy,0.,&a,&b);
	ps_drawa(a,b);/* axe vertical */
	ps_writeah(a+.1,b+.1,"Y",.2);
	ps_home();
	ps_3Dcalcul(0.,0.,maxz,&a,&b);
	ps_drawr(a,b);/* axe des profondeurs */
	ps_writeah(a+.1,b+.1,"Z",.2);	
	ps_home();
} /* fin du ps_3Daxes */


/*
 * computes the projection on the screen of points in the 3D space (sh)
 */
void ps_3Dcalcul( float y1, float z1, float x1, float *x, float *y)
{
	float a,b,c;
	
	a = -x1 * sps_3D_angle * kz + y1 * kx * cps_3D_angle ;
	b = -x1 * kz * cps_3D_angle * cps_3D_phi
		+ z1 * ky * sps_3D_phi -y1 * kx * sps_3D_angle * cps_3D_phi;
	c = -z1 * ky * cps_3D_phi - x1 * kz * cps_3D_angle * sps_3D_phi
		- y1 * kx * sps_3D_phi * sps_3D_angle + ps_3D_dist_1;
	*x = ps_3D_dist_2 * (a / c) ;
	*y = ps_3D_dist_2 * (b / c) ;
	
} /* fin du ps_3Dcalcul */

void ps_3Dframe(float x, float y, float z, float la, float ep, float lo)
{
	float a,b;
	
	ps_3Dcalcul(x,y,z,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y,z,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x+la,y,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x,y,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x,y,z,&a,&b);
	ps_drawa(a,b);
	
	ps_3Dcalcul(x,y+lo,z,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y+lo,z,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x+la,y+lo,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x,y+lo,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x,y+lo,z,&a,&b);
	ps_drawa(a,b);
	
	ps_3Dcalcul(x,y,z,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x+la,y,z,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y+lo,z,&a,&b);
	ps_drawa(a,b);
	
	ps_3Dcalcul(x,y,z+ep,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x,y+lo,z+ep,&a,&b);
	ps_drawa(a,b);

	ps_3Dcalcul(x+la,y,z+ep,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y+lo,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x,y,z,&a,&b);
	ps_movea(a,b);
	
	
} /* fin du p3Dframe */
 
void ps_3Dbar(float k, float l, float m, float la, 
	 float ep, float lo, int outline)
{
	float x[8],y[8];
	int	i,j,
		indps_3D_angle,indps_3D_phi,
		tab[5][5];
	
	/* calcul des indices */
	indps_3D_angle = (ps_3D_angle/(PI/2.)) ;
	indps_3D_phi = (ps_3D_phi/(PI/2.)) ;

	/* Initialisation du tableau */
	for (i=0; i < 5 ; i++ ) for (j=0; j < 5 ; j++ ) tab[i][j] = 0 ;
	tab[indps_3D_angle][indps_3D_phi] = 1 ;
			
	/* trace de la boite */
	/* Face 4 */
	if (!(tab[0][0] || tab[1][0] || tab[0][1] || tab[1][1] ||
	      tab[2][2] || tab[3][2] || tab[2][3] || tab[3][3]))
	{
	ps_3Dcalcul(k,l,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l+lo,m,x+2,y+2);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.6);
	} /* fin du if */

	/* Face 5 */
	if (!(tab[0][0] || tab[1][0] || tab[2][0] || tab[3][0] ||
	      tab[0][3] || tab[1][3] || tab[2][3] || tab[3][3]))
	{
	ps_3Dcalcul(k,l,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m+ep,x+1,y+1);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.5);
	} /* fin du if */

	/* Face 3 */
	if (!(tab[0][0] || tab[3][0] || tab[0][1] || tab[3][1] ||
	      tab[1][2] || tab[2][2] || tab[1][3] || tab[2][3]))
	{
	ps_3Dcalcul(k,l+lo,m,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+1,y+1);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.4);
	} /* fin du if */

	/* Face 2 */
	if (!(tab[2][0] || tab[3][0] || tab[2][1] || tab[3][1] ||
	      tab[0][2] || tab[1][2]|| tab[0][3] || tab[1][3]))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m+ep,x+1,y+1);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.7);
	} /* fin du if */

	/* Face 6 */
	if (!((indps_3D_phi==2) || (indps_3D_phi==1)))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l+lo,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.8);
	} /* fin du if */

	/* Face 1 */
	if (!(tab[1][0] || tab[2][0] || tab[1][1] || tab[2][1] ||
	      tab[0][2] || tab[3][2]||tab[0][3]||tab[3][3]))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l,m+ep,x+2,y+2);
	ps_3Dcalcul(k+la,l,m+ep,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.9);
	} /* fin du if */



} /* fin du p3Daxes */

void ps_3Dbox(float k, float l, float m, float la, 
	 float ep, float lo, int outline)
{
	float x[8],y[8];
	int i,j,tab[5][5];
	int	indps_3D_angle,indps_3D_phi;

	/* calcul des indices */
	indps_3D_angle = (ps_3D_angle/(PI/2.)) ;
	indps_3D_phi = (ps_3D_phi/(PI/2.)) ;

	/* Initialisation du tableau */
	for (i=0; i < 5 ; i++ ) for (j=0; j < 5 ; j++ ) tab[i][j] = 0 ;
	tab[indps_3D_angle][indps_3D_phi] = 1 ;
	
	
	/* trace de la boite */
	/* Face 4 */
	if ( (tab[0][0] || tab[1][0] || tab[0][1] || tab[1][1] ||
	      tab[2][2] || tab[3][2] || tab[2][3] || tab[3][3]))
	{
	ps_3Dcalcul(k,l,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l+lo,m,x+2,y+2);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.6);
	} /* fin du if */

	/* Face 5 */
	if ( (tab[0][0] || tab[1][0] || tab[2][0] || tab[3][0] ||
	      tab[0][3] || tab[1][3] || tab[2][3] || tab[3][3]))
	{
	ps_3Dcalcul(k,l,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m+ep,x+1,y+1);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.5);
	} /* fin du if */

	/* Face 3 */
	if ( (tab[0][0] || tab[3][0] || tab[0][1] || tab[3][1] ||
	      tab[1][2] || tab[2][2] || tab[1][3] || tab[2][3]))
	{
	ps_3Dcalcul(k,l+lo,m,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+1,y+1);
	ps_3Dcalcul(k,l,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.4);
	} /* fin du if */

	/* Face 2 */
	if ( (tab[2][0] || tab[3][0] || tab[2][1] || tab[3][1] ||
	      tab[0][2] || tab[1][2]|| tab[0][3] || tab[1][3]))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k+la,l,m+ep,x+1,y+1);
	ps_3Dcalcul(k+la,l,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.7);
	} /* fin du if */

	/* Face 6 */
	if ( ((indps_3D_phi==2) || (indps_3D_phi==1)))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l+lo,m,x+2,y+2);
	ps_3Dcalcul(k+la,l+lo,m,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.8);
	} /* fin du if */

	/* Face 1 */
	if ( (tab[1][0] || tab[2][0] || tab[1][1] || tab[2][1] ||
	      tab[0][2] || tab[3][2]||tab[0][3]||tab[3][3]))
	{
	ps_3Dcalcul(k+la,l+lo,m+ep,x,y);
	ps_movea(x[0],y[0]);
	ps_3Dcalcul(k,l+lo,m+ep,x+1,y+1);
	ps_3Dcalcul(k,l,m+ep,x+2,y+2);
	ps_3Dcalcul(k+la,l,m+ep,x+3,y+3);
	ps_closed_curve(x,y,4,outline,1,.9);
	} /* fin du if */

} /* fin du p3Daxes */

void ps_3Dclosebox(float x, float y, float z, float la, float ep, float lo)
{
	float a,b;
	
	ps_3Dcalcul(x+la,y+lo,z+ep,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x+la,y+lo,z+ep,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x,y+lo,z+ep,&a,&b);
	ps_drawa(a,b);
	ps_3Dcalcul(x+la,y+lo,z+ep,&a,&b);
	ps_movea(a,b);
	ps_3Dcalcul(x+la,y+lo,z,&a,&b);
	ps_drawa(a,b);


	ps_3Dcalcul(x,y,z,&a,&b);
	ps_movea(a,b);
	
} /* fin du p3Dclosebox */
