#define DPS_DEBUG	0
/*
*	                P  L  O  T  1  _  D  P  S
*	                ======================
*
*		   Routines de dessin sur la LN03R PostScript.
*
*	   NIVEAU 1 : Ce jeu de routine regroupe les routines de base
*		appelables par l'utilisateur de base
*
*		Mise en forme Version 2
*			Juillet 90
*		Version 3	Nouveaux noms de fonctions ps_*
*			31 juillet 90
*		Version 4
*			21 mars 91 / plot_dps V0
*		Version 4.2
*			11 oct 91
*		PLOT_DPS
*		Version 1.0
*			3 Avril 1991
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
title	PLOT1	Niveau 1 des routines PLOT_DPS pour C
index		Niveau 1 des routines PLOT_DPS pour C

description

		   Routines de dessin sur la PLOT_DPS.

	NIVEAU 1 : Ce jeu de routine regroupe les routines 
		appelables par l'utilisateur


author

	Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT0_DPS, PLOT2_DPS, PLOT3_DPS et PLOT3D_DPS

newpage
*/

#include <stdio.h>

#include "plot_ps.h"
	
#ifdef  DOCUMENTATION

title	PS_SET_RGB_COLOR	Set a current RGB color
index			Set a current RGB color

synopsis
	#include <plot_ps.h>

	ps_set_rgb_color(red,green,blue);
	float     red,   /* red for the subsequent drawings */
		  green, /* green for the subsequent drawings */
		  blue;  /* blue for the subsequent drawings */
description

	PS_SET_RGB_COLOR() defines a new color value in the RGB space for
	the subsequent drawings.

	0.0 is no contribution of the corresponding color.
	
	1.0 is full contribution of the corresponding color.

	Equal values of each color gives a gray shade: all 0.0 gives black,
	all 1.0 gives white.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	ps_set_hsb_color, ps_set_cmyk_color

newpage

title	PS_SET_HSB_COLOR	Set a current HSB color
index			Set a current HSB color

synopsis
	#include <plot_ps.h>

	ps_set_hsb_color(hue,saturation,brightness);
	float     hue,   /* hue for the subsequent drawings */
		  saturation, /* saturation for the subsequent drawings */
		  brightness;  /* brightness for the subsequent drawings */
description

	PS_SET_HSB_COLOR() defines a new color value in the HSB space for
	the subsequent drawings.

	For Hue: 0. or 1. is pure red, 1/3 is pure green and 2/3 is pure blue.

	For Saturation: 0. is no color (only a shade of gray), 1. is maximum
	concentration (no white light mixed)

	For Brightness: 0. is black and 1. is maximum intensity.

author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	ps_set_rgb_color, ps_set_cmyk_color
	
newpage

title	PS_SET_CMYK_COLOR	Set a current CMYK color
index			Set a current CMYK color

synopsis
	#include <plot_ps.h>

	ps_set_cmyk_color(cyan,magenta,yellow,back)
	float     cyan,   /* cyan for the subsequent drawings */
		  magenta, /* magenta for the subsequent drawings */
		  yellow, /* yellow for the subsequent drawings */
		  black;  /* black for the subsequent drawings */
description

	PS_SET_CMYK_COLOR() defines a new color value in the CMYK space for
	the subsequent drawings.

	0.0 is no contribution of the corresponding color.
	
	1.0 is full contribution of the corresponding color.

	Equal values of each color gives a gray shade: all 0.0 gives black,
	all 1.0 gives white.

	ps_set_cmyk_color() has effect only in the printing environment. If
	in the window environement, it has no effect.
		
author

	Jean-Francois VIBERT

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	ps_set_rgb_color, ps_set_hsb_color
	
#endif


/*
*
*	PS_SET_RGB_COLOR() definie une nouvelle couleur dans l'espace RGB
*
*/
void ps_set_rgb_color(float red,float green,float blue)
/* red for the subsequent drawings */
/* green for the subsequent drawings */
/* blue for the subsequent drawings */
{
	fprintf(pl_spl,"%.3f %.3f %.3f setrgbcolor\n",red,green,blue);
} /* fin du ps_set_rgb_color */

/*
*
*	PS_SET_HSB_COLOR() definie une nouvelle couleur dans l'espace HSB
*
*/
void ps_set_hsb_color(float hue,float saturation,float brightness)
/* hue for the subsequent drawings */
/* saturation for the subsequent drawings */
/* brightness for the subsequent drawings */
{
	fprintf(pl_spl,"%.3f %.3f %.3f sethsbcolor\n",hue,saturation,brightness);
} /* fin du ps_set_hsb_color */

/*
*
*	PS_SET_CMYK_COLOR() definie une nouvelle couleur dans l'espace CMYK
*
*/
void ps_set_cmyk_color(float cyan,float magenta,float yellow,float black)
/* cyan for the subsequent drawings */
/* magenta for the subsequent drawings */
/* yellow for the subsequent drawings */
/* black for the subsequent drawings */
{
	fprintf(pl_spl,"%.3f %.3f %.3f %.3f setcymkcolor\n",cyan,magenta,yellow,black);
} /* fin du ps_set_cmyk_color */


