
/*
*                        P  L  O  T  0  _ D  P  S
*                        ======================
*			
*		Routines de dessin sur la LN03R PostScript.
*
*           NIVEAU 0 : Ce jeu de routine regroupe les routines internes
*		theoriquement non appelables par l'utilisateur
*
*		Mise en forme version 2
*			juillet 90
*		Version 3	Nouveaux noms de fonctions ps_*
*			31 juillet 90
*		Version 4
*			21 mars 91 / plot_dps V0
*		PLOT_DPS
*		Version 1.0
*			3 avril 1991
*		Version 1.2
*			30 oct 1991
*		PLOT_XPS
*		Version 1.0
*			3 decembre 1992
*		Version 2.0
*			11 juillet 1994
*		Version 2.14
*			11 avril 2001
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/
/*
*	PLOT0_PS.C --> librairies en mode POSTSCRIPT 
*	pour la LN03R

title	PLOT0S	Niveau 0 des routines PostScript pour C
index		Niveau 0 des routines PostScript pour C

description

		   Routines de dessin sur la Laser PostScript
		   en mode symbolique.

	NIVEAU 0 : Ce jeu de routine regroupe les routines les plus
		elementaires qui controlent directement la laser
		sans pratiquement d'aide a l'utilisateur.

	ATTENTION : Toute les variables de travail des modules commencent
			par 'pl_'
		    Tous les sous programmes de travail commencent
		    	par 'p_'

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT2_PS
newpage
title	ROUTINES_INTERNES Routines internes de travail
index		Routines internes de travail

description

		   Routines internes de travail pour
	creer le fichier de dessin pour l'imprimante PostScipt.

	{\em Ces routines ne devraient jamais etre appel\'{e}s}

	NIVEAU 0 : Ce jeu de routine regroupe les routines les plus
		elementaires qui controlent directement le traceur
		sans pratiquement d'aide a l'utilisateur.

	ATTENTION : Toute les variables de travail des modules commencent
			par 'pl_'
		    Tous les sous programmes de travail commencent
			par 'p_'

	P_DEPL
	Envoi un groupe de 2 valeurs x,y au format PostScipt avec gestion 
	du deplacement et de la plume

	P_PUTS
	Envoi une chaine de caracteres avec le dernier format (hx,hy,cosa,sina) 
	utilise	

	PLOT
	Deplace la plume au point indique, actualise la position courante


author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT2_PS

newpage
title	TRAA	Move the pen in PostScipt space (in cm)
index		Move the pen in PostScipt space (in cm)

synopsis
	#include <plot_xps.h>

	traa(x,y,j);
	float x, y;     x and y coordinates
	                given in cm 
	int j;          the Benson j parameter 
description

	TRAA replicates the Benson traa fortran routine, and normally has
	not to be used.

		X and Y are the coordinates of the target given in PostScipt
	space (in centimeters). 

		These coordinates are considered as absolute (j=0 or j=1) or
	relative (j=2 or j=3) according to the value of j. 
	
		During the move, the pen is up (j=0 or j=2) or down
	(j=1 or j=3) according to the value of j.

		The following constants are defined in PLOT_PS.H:

	
	\nofill
		Code des deplacements de la plume

	\#define PL_AH 0		Absolu Plume Haute	
	\#define PL_AB 1		Absolu Plume Basse	
	\#define PL_RH 2		Relatif Plume Haute	
	\#define PL_RB 3		Relatif Plume Basse	
	\fill

		Normally it is not necessary to call traa since puser, pcms,
	pmover, pmovea, pdrawr and pdrawa are also defined and easier to use.

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PMOVER and PMOVEA to move the pen
	PDRAWR and PDRAWA to draw a line
newpage

title	TRAS	Move the pen in subject space (not in cm)
index		Move the pen in subject space (not in cm)

synopsis
	#include <plot_xps.h>

	tras(x,y,j);
	float x, y;     x and y coordinates
	                given in sbject units m 
	int j;          the Benson j parameter 
description

	TRAS replicates the Benson tras fortran routine.

		X and Y are the coordinates of the target given in subject 
	units (NOT in centimeters).

		These coordinates are considered as absolute (j=0 or j=1) or
	relative (j=2 or j=3) according to the value of j. 
	
		During the move, the pen is up (j=0 or j=2) or down
	(j=1 or j=3) according to the value of j.

	The following constants are defined in PLOT.H:

	\nofill
	 Code des deplacements de la plume
	 
	\#define PL_AH 0		Absolu Plume Haute	
	\#define PL_AB 1		Abslou Plume Basse	
	\#define PL_RH 2		Relatif Plume Haute	
	\#define PL_RB 3		Relatif Plume Basse	
	\fill

	Normally it is not necessary to call traa since puser, pcms, pmover,
	pmovea, pdrawr and pdrawa are also defined and easier to use.

author

	Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PMOVER and PMOVEA to move the pen
	PLINER and PLINEA to draw a line
newpage
title	PLOPEN	Initialization of the PostScipt file
index		Initialization of the PostScipt file

synopsis
	#include <plot_xps.h>

	plopen(name);
	char *name;        
description

	PLOPEN: Open the file in wich the plot will be send,
	and initialize the plotter.

	It must be called once at the very before using any plotter statement
	The working buffer beginning is defined in PLOT.H and has a fixed
	length, (that can be modified).

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	INIPLOT and PLCLOSE

newpage

title	PLCLOSE	Close the PostScipt file
index			Close the PostScipt file

synopsis
	#include <plot_xps.h>

	plclose(nom)
	char  *nom; 
description

	PLCLOSE: Stops the plotter and closes the file "nom" in wich the 
	plot has been send.

	The PostScipt file must be send to the plotter in order to be drawn.
	The file can be spooled as for any ascii file.

author

	Jean-Francois VIBERT a partir de PLOT0.C de Jean-Noel Albert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOPEN and ENDPLOT

*/

#ifdef	DOCUMENTATION

first-page-title	PLOT_XPS
first-page-sub-title A PostScript graphic library for C
first-page-user-manual Reference manual for V 2.14 
first-page-author Jean-Fran\c{c}ois VIBERT
first-page-author Eric BOUSSARD
first-page-author Sylvain HANNETON
first-page-address ESI
first-page-address Facult de Mdecine Saint-Antoine
first-page-address Universite Pierre et Marie Curie
first-page-address INSERM U444
first-page-address PARIS
first-page-address FRANCE
first-page-address vibert@u444.jussieu.fr

title	PLOT_PS Generalities on PLOT_PS

description

	PLOT_XPS is a set of C functions designed to build PostScript files
	and to draw DisplayPostScript drawings in X windows from C programs.

	They all begin with the prefix {\tt ps_} and need to have the {\tt
	plot_xps.h} header file included.

	The very first function to be called is {\tt ps_iniplot}. It opens
	both the postscript file and the first X window. If necessary, both can
	be dissociated, getting only PS file or only X window. {\tt ps_iniplot}
	can be called from a Xwindow program or not.

	PS_MAX_WINDOW_NUMBER (\#defined as 10 at library building time) can
	be opened.

	New windows and the corresponding PS files can be open or closed
	during the drawing session using {\tt ps_create_window} and {\tt
	ps_close_window}.
	
	Windows are always refered by their name, since their number can be
	changed according to the successive creations and closings of windows.

	All operations are always performed on the current window selected
	by {\em ps_use_window(window_name)}.

	Windows can be hidden momentarily, drwing or printing can be disbled
	and renabled during the session.

	Several fonctions give informations on the current window state
	({\em ps_is_*()}).
	
	PLOT_XPS allows to work in both absolute (in cms) and user or subject
	space. To be able to work in the user space, the function {\tt
	ps_echel} must be called to define this user space.

	Inclusion of PS file or of plain PostScript code is also possible
	with PLOT_XPS. The file inclusion possibility allows the inclusion of
	a drawing obtained with any wysiwyg interactive drawing program.

	The very last function call must be {\tt ps_endplot}. It closes the
	postscript files and spool them or not, and closes all remaining windows.

	PLOT_XPS allows 3D drawings. Before to use any 3D function, the {\tt
	ps_3Dsetup} must be called in order to setup all the necessary
	parameters. These functions requires to be linked with the
	mathematics library.

	PLOT_XPS allows also the use of color PostScript printers

	Some functions are rather high level, as the axis and 3D functions,
	and it is advised to understand the way they work by exercising on
	some small exemples.

example

	The order of function calls is important at the beginning:

	1/ {\tt ps_iniplot} initializes the plot_ps system

	2/ {\tt ps_origin} set up the physical and logical origine of the
	graph (x = 0., y = 0.)
	
	3/ {\tt ps_echel} initializes the  space of the user, and allows to work
	in user units instead of cms.

	4/ {\tt ps_user} or {\tt ps_cms} chooses the working space, {\em
	i.e.} says if the x,y subsequent coordinates are to be considered as
	user units or cms.
	
	5/ {\tt ps_movea} and {\tt ps_drawa}, {\tt ps_mover} and {\tt
	ps_drawr} either move the pen without drawing (move) or with drawing
	(draw). teh last letter {\tt a}, for absolute or {\tt r}, for
	relative, indicates if the x,y coordinates are to be considered as
	absolute coordinates relative to the last call to {\tt ps_origin} or
	relative to the last pen position.

	6/ {\tt ps_endplot} terminates and cleans-up the plot_ps system,
	closes the file and spoole it or not accordind to the given function
	argument.

synopsis
	#include <plot_xps.c>

	...
	ps_iniplot(NULL,"my_file.ps",/* open a file my_file.ps and a window	*/
	           10,10,            /* called my_window located at 10,10 on	*/
	           21.,29.7,         /* the screen, with an A4 sheet size,	*/
	           .5,               /* scaled at half size, without previous	*/
	           "My_window");     /* display					*/
	ps_origin(10.,10.,0.,0.);     /*
	                               * move the pen at 10.,10. cms relative
	                               * to the lower left corner of the page
	                               * (+ 2cm in Y direction due to the
	                               * banner), and set this position as the
	                               * new origin
	                               */
	ps_echel(0.5,0.5,0.,0.,1.,0.); /*
	                                * set the scale as 1 user unit for 2
	                                * cms, drawing head up
	                                */
	ps_user();                      /* work in user space */
	ps_movea(1.,3.);		/* go to x=1., y=3. */
	ps_drawr(2.,0.);                /* and draws a 2 units sided square */
	ps_drawr(0.,-2.);
	ps_drawr(-2.,0.);
	ps_drawa(1.,3.);
	ps_endplot('d');   /* close, spoole and delete the file */
	...

description

	The program using PLOT_XPS must be linked with the plot_xps library,
	the xps and X11 libraries and THEN the math library if the 3D
	functions are used.


	Under unix for exemple:

	{\tt cc -O -o my_prog my_prog.c -lplot_xps -lX11 -lm}

	The Makefile should include:

	\begin{verbatim}
	CFLAGS = -O
	LIBS =  -lplot_xps \
	        -lX11 \
	        -lm
	\bin\cc -o my_program my_program.c $(LIBS)
	\end{verbatim}
author

	Jean-Francois VIBERT, Eric Boussard et Sylvain HANNETON

	CHU Saint-Antoine INSERM URBB U263 Universite Paris VI
#endif

#include <stdio.h>
#define	PLOT_XPS
#include "plot_xps.h"
#undef	PLOT_XPS

/*
 * Routines pour faciliter le portage venant de RSX
 */

#define CR  '\015'
#define LF  '\012'


/*
	ROUTINES INTERNES de travail
*/
/*
 *	Sort un groupe de deux valeurs au format de deplacement
 */

/*	 
 *	TRAS est la traduction de PLOTS dans la norme PostScipt
 */
void tras(float x, float y, int cd)
{
	float xa, ya;			/* Calcul de la rotation Sujet	*/
	float xs, ys;
	float xi, yi;

	xs = x * xps_dessin[ps_current_window]->pl_ca;
	ys = y * xps_dessin[ps_current_window]->pl_sa;	/* Projection sur X	*/
	xa = xs - ys;
	xs = x * xps_dessin[ps_current_window]->pl_sa;
	ys = y * xps_dessin[ps_current_window]->pl_ca;	/* Projection sur Y	*/
	ya = xs + ys;

	xi = xa * xps_dessin[ps_current_window]->pl_rex;		/* Mise a l'echelle 	*/
	yi = ya * xps_dessin[ps_current_window]->pl_rey;

	if (cd <= 1)
	{
		xi += xps_dessin[ps_current_window]->pl_xs;
		yi += xps_dessin[ps_current_window]->pl_ys;
	}

	if(! p_depl(xi,yi,cd)) fprintf(stderr,"\007Illegal code %d in TRAS\n",cd);

}
/*
 *	TRAA est tout a fait semblable a TRAS,si ce n'est que le facteur
 *	d'echelle est pris egale au nombre de pas par centimetre ET qu'il
 *	n'y a pas de probleme d'origine.
 */
void traa(float x, float y, int cd)
{
	float i_x, i_y;

	i_x = x * X_PAS;
	i_y = y * X_PAS;
	
	if(! p_depl(i_x,i_y,cd)) fprintf(stderr,"\007Illegal code %d dans TRAA\n",cd);;
}
/*
*
*	P_DEPL() genere les deplacement PostScript
*/
/*
 *	Deplacement sur le traceur, en absolue ou en relatif,
 *		plume haute ou basse
 *	   Cd = 0 : Absolu haut
 *		1 : Absolu bas
 *		2 : Relatif haut
 *		3 : Relatif bas
 *	Si ce code est farfelu, il n'y a rien de fait.
 *
 *	REMARQUE : Cette routine travaille a l'echelle d'un pas
 *		   elementaire.
 */
int p_depl(float i_x, float i_y, int cd)
{

	if (cd < 0 || cd > 3)
		return(FALSE);		/* Status			*/

	switch(cd)
	{
		case PL_AH:
			if (xps_dessin[ps_current_window]->draw)
			{
				if (xps_dessin[ps_current_window]->ps_hard_copy)
					fputs("GSG\n",xps_dessin[ps_current_window]->pl_spl);
				xps_dessin[ps_current_window]->draw = FALSE;
			}
		  	if (xps_dessin[ps_current_window]->ps_hard_copy)
				fprintf(xps_dessin[ps_current_window]->pl_spl,"%.1f %.1f M\n",i_x,i_y);
			if (xps_dessin[ps_current_window]->ps_show)
				PSmoveto(i_x,i_y);
			xps_dessin[ps_current_window]->ex_x = i_x;
			xps_dessin[ps_current_window]->ex_y = i_y;
			break;
		case PL_AB:
			if (!xps_dessin[ps_current_window]->draw)
			{
				if (xps_dessin[ps_current_window]->ps_hard_copy)
					fprintf(xps_dessin[ps_current_window]->pl_spl,"newpath\n%.1f %.1f M\n",
						xps_dessin[ps_current_window]->ex_x,
						xps_dessin[ps_current_window]->ex_y);
				if (xps_dessin[ps_current_window]->ps_show)
				{
					PSnewpath();
					PSmoveto(xps_dessin[ps_current_window]->ex_x,
						 xps_dessin[ps_current_window]->ex_y);
				} 
				xps_dessin[ps_current_window]->draw = TRUE;
			}
			if (xps_dessin[ps_current_window]->ps_hard_copy)
				fprintf(xps_dessin[ps_current_window]->pl_spl,"%.1f %.1f L\n",
						i_x,i_y);
			if (xps_dessin[ps_current_window]->ps_show)
				PSlineto(i_x,i_y);
			xps_dessin[ps_current_window]->ex_x = i_x;
			xps_dessin[ps_current_window]->ex_y = i_y;
			break;
		case PL_RH:
		  	if (xps_dessin[ps_current_window]->ps_hard_copy)
				fprintf(xps_dessin[ps_current_window]->pl_spl,"%.1f %.1f rM\n",i_x,i_y);
			if (xps_dessin[ps_current_window]->ps_show)
				PSrmoveto(i_x,i_y);
			if (xps_dessin[ps_current_window]->draw)
			{
				if (xps_dessin[ps_current_window]->ps_hard_copy)
					fputs("GSG\n",xps_dessin[ps_current_window]->pl_spl);
				xps_dessin[ps_current_window]->draw = FALSE;
			}
			xps_dessin[ps_current_window]->ex_x += i_x;
			xps_dessin[ps_current_window]->ex_y += i_y;
			break;
		case PL_RB:
			if (!xps_dessin[ps_current_window]->draw)
			{
				if (xps_dessin[ps_current_window]->ps_hard_copy)
					fprintf(xps_dessin[ps_current_window]->pl_spl,"newpath\n%.1f %.1f M\n",
						xps_dessin[ps_current_window]->ex_x,
						xps_dessin[ps_current_window]->ex_y);
				if (xps_dessin[ps_current_window]->ps_show)
				{
					PSnewpath();
					PSmoveto(xps_dessin[ps_current_window]->ex_x,
						 xps_dessin[ps_current_window]->ex_y);
				} 
				xps_dessin[ps_current_window]->draw = TRUE;
			}
			if (xps_dessin[ps_current_window]->ps_hard_copy)
				fprintf(xps_dessin[ps_current_window]->pl_spl,"%.1f %.1f rL\n",
					i_x,i_y);
			if (xps_dessin[ps_current_window]->ps_show)
				PSrlineto(i_x,i_y);
			xps_dessin[ps_current_window]->ex_x += i_x;
			xps_dessin[ps_current_window]->ex_y += i_y;
			break;
	}
	/*
	 * Calcul pour la Bounding_box
	 */
	if (xps_dessin[ps_current_window]->ex_x > xps_dessin[ps_current_window]->b_b_ur_x)
		xps_dessin[ps_current_window]->b_b_ur_x = xps_dessin[ps_current_window]->ex_x;
	if (xps_dessin[ps_current_window]->ex_x < xps_dessin[ps_current_window]->b_b_ll_x)
		xps_dessin[ps_current_window]->b_b_ll_x = xps_dessin[ps_current_window]->ex_x;
	if (xps_dessin[ps_current_window]->ex_y > xps_dessin[ps_current_window]->b_b_ur_y)
		xps_dessin[ps_current_window]->b_b_ur_y = xps_dessin[ps_current_window]->ex_y;
	if (xps_dessin[ps_current_window]->ex_y < xps_dessin[ps_current_window]->b_b_ll_y)
		xps_dessin[ps_current_window]->b_b_ll_y = xps_dessin[ps_current_window]->ex_y;

	return (TRUE);
}
/*
 *	Sort sur le traceur une chaine de caracteres avec le DERNIER format
 *	defini.
 */
int p_puts(char *strg, int rot)
{

	if (strlen(strg) >= MAX_PLT)
	{
		fprintf(stderr,
			"\007PLOT_XPS warning: String too long:\n %s\nIgnored\n",
			strg);
		return(0);
	}

	if (xps_dessin[ps_current_window]->ps_hard_copy)
		fprintf(xps_dessin[ps_current_window]->pl_spl,"(%s) S\n",strg);
	if (xps_dessin[ps_current_window]->ps_show)
		PSshow(strg,rot);
	return(1);
}
/***********************************************************************
*
*	Routines theoriquement appelables par l'utilisateur
*
*		PLOPEN	-> en 1er pour ouvrir
*		PLCLOSE	-> en dernier pour fermer
*
*
*************************************************************************/

 
/*
 *	PLOPEN	: Permet l'initialisation du traceur
 *			Appelee une seule fois
 *			Le buffer de travail est defini dans PLOT.H
 *				et a une longueur fixee (mais modifiable)
 */
void plopen(char *nom)
{
/*
 * 		Ouvre le fichier resultat
 */
	if ((xps_dessin[ps_current_window]->pl_spl = fopen(nom,"w")) == NULL)
	{
		fprintf(stderr,"\007PLOT_XPS error: Impossible to open the PostScript file %s \n",nom);
		exit(1);
	}
}

/*
 *	PLCLOSE : Ferme le fichier
 */
void plclose(char *nom)
{
	if(xps_dessin[ps_current_window]->draw)
	{
		if (xps_dessin[ps_current_window]->ps_hard_copy)
			fputs("stroke\n",xps_dessin[ps_current_window]->pl_spl);
	} /* fin du if */
	if (xps_dessin[ps_current_window]->ps_hard_copy)
		fputs("showpage\nrestore\n",xps_dessin[ps_current_window]->pl_spl);
	/*
	 * reecriture de la BoundingBox correspondant aux mvts de plume.
	 * ATTENTION: des chaines de caracteres peuvent depasser!!
	 */
	if (xps_dessin[ps_current_window]->ps_hard_copy)
	{
		
		fprintf(xps_dessin[ps_current_window]->pl_spl,
			"%%%%BoundingBox: %.1f %.1f %.1f %.1f\n",
			xps_dessin[ps_current_window]->b_b_ll_x,
			xps_dessin[ps_current_window]->b_b_ll_y,
			xps_dessin[ps_current_window]->b_b_ur_x,
			xps_dessin[ps_current_window]->b_b_ur_y);
		fprintf(xps_dessin[ps_current_window]->pl_spl,
			"%%%%Pages: %d\n",
			xps_dessin[ps_current_window]->pl_ps_page);
		fprintf(xps_dessin[ps_current_window]->pl_spl,
			"%%%%End of File %s\n",nom);
	} /* fin du if */


						/* Ferme tout		*/
	if (fclose(xps_dessin[ps_current_window]->pl_spl) == -1)
	{
		fprintf(stderr,"\007PLOT_XPS error: Problem when closing the %s PostScript file\n", nom);
	}
}

