#define XPS_DEBUG	0

/*
 *			P L O T X _ X P S
 *			=================
 *
 *		Routines d'affichages du dessin sur ecran X
 *		Ces routines sont invisibles a l'utilisateur
 *
 *		Version 1.0
 *			3 decembre 1992
 *
 *	Auteurs: Eric BOUSSARD & Dr J-F VIBERT
 *		 B3E
 *		 CHU Saint-Antoine
 *		 27 rue Chaligny
 *		 75571 Paris cedex 12
 *
 *	Copyright Dr JF VIBERT
 *	All rights reserved 
 *	Access limited to authorized persons
 *	This software was developped only for research purpose
 *	and is in any way usable for commercial purpose
 */


/* Inclusion des fichiers header */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <X11/Xlib.h>
#define PLOT_XPS
#include "plot_xps.h"
#undef  PLOT_XPS
#include "plot_xps.fonts"

/* Macros */
#define ABSOL(a)	((a<0.)?-a:a)
#define MAX(a, b)	((a>b)?a:b)
#define DIFF(a, b, c)	(MAX(ABSOL(a-b),c))

/****************************************
 *					*
 *		couleur			*
 *					*
 ****************************************
 * Defini une colormap pour un ecran    *
 * a 8 plans graphiques                 *
 ****************************************/
 
void couleur()
{
	int		ncouleurs = DisplayCells(xps_display,xps_screen);
	int		i,j,k;
	Colormap	paldef = DefaultColormap(xps_display,xps_screen);
	XColor		coul[256];

#if XPS_DEBUG
	printf("couleur %d\n",ncouleurs);
#endif
	if (!xps_display)
		return;
#if 1	
	if (ncouleurs > 256)
		ncouleurs = 256;
	if (ncouleurs < 256)
	{
		type_ecran = 0;
		return;
	}

	colormap = XCreateColormap(xps_display,
				   DefaultRootWindow(xps_display),
				   DefaultVisual(xps_display,xps_screen),
				   AllocAll);
	for(i=0; i<ncouleurs; i++)
	{
		coul[i].pixel = i;
		coul[i].flags = DoRed|DoGreen|DoBlue;
	} 
	/* Pour conserver les couleurs d'environnement */
	XQueryColors(xps_display,paldef,coul,ncouleurs);

	/* Definition d'un jeu de 180 couleurs accessibles */
	for (i=0; i<6; i++)
		for (j=0; j<6; j++)
			for (k=0; k<6; k++)
			{
				coul[BASE_COUL+i*30+j*5+k].red = i*13107;
				coul[BASE_COUL+i*30+j*5+k].green = j*13107;
				coul[BASE_COUL+i*30+j*5+k].blue = (int)((float)(k)*16383.25);
			}

	/* Definition d'un jeu de 16 niveaux de gris accessibles */
	for (i=0; i<16; i++)
	{
		coul[BASE_GRIS+i].red   = i*4369;
		coul[BASE_GRIS+i].green = i*4369;
		coul[BASE_GRIS+i].blue  = i*4369;
	}

	/* Enregistrement de la nouvelle palette */
	XStoreColors(xps_display,colormap,coul,ncouleurs);
#endif
#if XPS_DEBUG
	printf("... done \n");
#endif
}


/****************************************/
/*					*
 *		xps_pixel		*
 *					*
 ****************************************
 * Calcul des coordonnees ecrans a      *
 * partir des coordonnees ps en pas     *
 ****************************************/

static void xps_pixel(float x, float y, int *pt)
{
#if XPS_DEBUG
	printf("xps_pixel \n");
#endif
#if 1
	pt[0] = (int)( x*xps_dessin[ps_current_window]->cmt[0][0]
		   + y*xps_dessin[ps_current_window]->cmt[0][1]
		   +   xps_dessin[ps_current_window]->cmt[0][2]);
	pt[1] = (int)( x*xps_dessin[ps_current_window]->cmt[1][0]
		   + y*xps_dessin[ps_current_window]->cmt[1][1]
		   +   xps_dessin[ps_current_window]->cmt[1][2]);
#endif
}



/****************************************/
/*					*
 *		PSflush			*
 *					*
 ****************************************
 * Actualise les derniers traces a	*
 * l'ecran				*
 ****************************************/

void PSflush()
{
#if 0
	printf("PSflush \n");
#endif
	if (!xps_display)
		return;
	if (xps_dessin[ps_current_window]->ps_auto_flush)
	{
		XClearWindow(xps_display,xps_dessin[ps_current_window]->xps_window);
		XFlush(xps_display);
	}
}



/****************************************/
/*					*
 *		PSlineto		*
 *					*
 ****************************************
 * Trace une ligne en coord absolues    *
 ****************************************/

void PSlineto(float x, float y)
{
	int	pt1[2],
		pt2[2];
#if XPS_DEBUG
	printf("PSlineto \n");
#endif
#if 1
	if (!xps_display)
		return;
	
	xps_pixel((float)xps_dessin[ps_current_window]->ex_x,
		  (float)xps_dessin[ps_current_window]->ex_y,
		  pt1);
	xps_pixel(x,y,pt2);

	XDrawLine(xps_display,
		  xps_dessin[ps_current_window]->pixmap,
		  xps_dessin[ps_current_window]->xps_gc,
		  pt1[0],pt1[1],
		  pt2[0],pt2[1]);

	xps_dessin[ps_current_window]->xder = x;
	xps_dessin[ps_current_window]->yder = y;

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSrlineto		*
 *					*
 ****************************************
 * Trace une ligne en coord relatives   *
 ****************************************/

void PSrlineto(float x, float y)
{
	int	pt1[2],pt2[2];

#if XPS_DEBUG
	printf("PSrlineto \n");
#endif
	if (!xps_display)
		return;
#if 1	
	xps_pixel((float)xps_dessin[ps_current_window]->ex_x,
		  (float)xps_dessin[ps_current_window]->ex_y,
		  pt1);

	xps_pixel((float)(xps_dessin[ps_current_window]->ex_x) + x,
		  (float)(xps_dessin[ps_current_window]->ex_y) + y,
		  pt2);

	XDrawLine(xps_display,
		  xps_dessin[ps_current_window]->pixmap,
		  xps_dessin[ps_current_window]->xps_gc,
		  pt1[0], pt1[1],
		  pt2[0], pt2[1]);

	xps_dessin[ps_current_window]->xder =
		x + (float)(xps_dessin[ps_current_window]->ex_x);
	xps_dessin[ps_current_window]->yder =
		y + (float)(xps_dessin[ps_current_window]->ex_y);

	PSflush();
#endif
}

/****************************************/
/*					*
 *		PSmoveto		*
 *					*
 ****************************************
 * Deplacement du point courant en      *
 * coord absolues                       *
 ****************************************/

void PSmoveto(float x, float y)
{
#if XPS_DEBUG
	printf("PSmoveto \n");
#endif
#if 1
	xps_dessin[ps_current_window]->xder = x;
	xps_dessin[ps_current_window]->yder = y;
#endif
}


/****************************************/
/*					*
 *		PSrmoveto		*
 *					*
 ****************************************
 * Deplacement du point courant en      *
 * coord relatives                      *
 ****************************************/

void PSrmoveto(float x, float y)
{
#if XPS_DEBUG
	printf("PSrmoveto \n");
#endif
#if 1
	xps_dessin[ps_current_window]->xder += x;
	xps_dessin[ps_current_window]->yder += y;
#endif
}


/****************************************/
/*					*
 *		PScurveto		*
 *					*
 ****************************************
 * Courbes de Bezier                    *
 ****************************************/

void PScurveto(float x1, float y1,
	       float x2, float y2,
	       float x3, float y3)
{
#if 0
	float	x0, y0,
		x, y,
		t,inc,
		ax, ay,
		bx, by,
		cx, cy,
		dx, dy;

	int	pt1[2],
		pt2[2];
#endif		
#if XPS_DEBUG
	printf("PScurveto \n");
#endif
	if (!xps_display)
		return;
	
#if 0
	x0 = xps_dessin[ps_current_window]->xder;
	y0 = xps_dessin[ps_current_window]->yder;

	dx = 0.;
	dy = 0.;
	DIFF(x0,x1,dx);
	DIFF(y0,y1,dy);
	DIFF(x0,x2,dx);
	DIFF(y0,y2,dy);
	DIFF(x0,x3,dx);
	DIFF(y0,y3,dy);
	DIFF(x1,x2,dx);
	DIFF(y1,y2,dy);
	DIFF(x1,x3,dx);
	DIFF(y1,y3,dy);
	DIFF(x2,x3,dx);
	DIFF(y2,y3,dy);

	dx *= pl_conv;
	dy *= pl_conv;
	if (dx > dy)
		inc = 1. / dx;
	else
		inc = 1. / dy;
	
	cx = (x1 - x0) / 3.;
	cy = (y1 - y0) / 3.;
	bx = ((x2 - x1) / 3.) - cx;
	by = ((y2 - y1) / 3.) - cy;
	ax = x3 - x0 - cx - bx;
	ay = y3 - y0 - cy - by;

	xps_pixel(x0,y0,pt1);
	for (t=0.; t<=1.; t+=inc)
	{
		 x = ax*t*t*t + bx*t*t + cx*t + x0;
		 y = ay*t*t*t + by*t*t + cy*t + y0;
		 xps_pixel(x,y,pt2);
		 XDrawLine(xps_display,
			   xps_dessin[ps_current_window]->pixmap,
			   xps_dessin[ps_current_window]->xps_gc,
			   pt1[0],pt1[1],
			   pt2[0],pt2[1]);
		pt1[0] = pt2[0];
		pt1[1] = pt2[1];
	}
	
	xps_dessin[ps_current_window]->xder = x3;
	xps_dessin[ps_current_window]->yder = y3;
#endif
	fprintf(stderr,"PLOT_XPS Warning: ps_bezier not yet implemented for X\n");
}


/****************************************/
/*					*
 *		PSshow			*
 *					*
 ****************************************
 * Ecriture de texte suivant un angle   *
 ****************************************/

void PSshow(char *strg, int rot)
{
	int		pt[2],
			x,y,
			i,j;
	unsigned long	pixel,
			fg;
	double		sina,cosa;		/* rotation */
	float		dx;			/* agrandissement */
	double		alpha;			/* angle de rotation */
	Pixmap		pixmap_tampon1,		/* texte normal */
			pixmap_tampon2;		/* texte agrandit */
	XImage		*image1,		/* texte normal */
			*image2;		/* texte agrandit */
	int		largeur,hauteur,
			l,h,
			nchar;
	int		dir,ascent,descent;
	XCharStruct	overall;
	XGCValues	values;
	
#if XPS_DEBUG
	printf("PSshow \n");
#endif
#if 1
	if (!xps_display)
		return;

	xps_pixel((float)xps_dessin[ps_current_window]->ex_x,
		  (float)xps_dessin[ps_current_window]->ex_y,
		  pt);

	XGetGCValues(xps_display,
	             xps_dessin[ps_current_window]->xps_gc,
		     GCForeground,&values);
	fg = values.foreground;

	dx = (xps_dessin[ps_current_window]->hl_crte * X_PAS)
	     / xps_dessin[ps_current_window]->h_font;
	nchar = (int)strlen(strg);

	XTextExtents(xps_dessin[ps_current_window]->fontinfo,
		     strg,nchar,
		     &dir,&ascent,&descent,&overall);
	largeur = XTextWidth(xps_dessin[ps_current_window]->fontinfo,
			     strg,nchar);
	hauteur = ascent + descent;

	/* texte normal */
	pixmap_tampon1 = XCreatePixmap(xps_display,
				      xps_dessin[ps_current_window]->xps_window,
				      largeur, hauteur,
				      DefaultDepth(xps_display,xps_screen));
	XSetForeground(xps_display,
		       xps_dessin[ps_current_window]->xps_gc,
		       0);
	XFillRectangle(xps_display,
		       pixmap_tampon1,
		       xps_dessin[ps_current_window]->xps_gc,
		       0,0,
		       largeur,hauteur);
	XSetForeground(xps_display,
		       xps_dessin[ps_current_window]->xps_gc,
		       1);
	XDrawString(xps_display,
		    pixmap_tampon1,
		    xps_dessin[ps_current_window]->xps_gc,
		    0, xps_dessin[ps_current_window]->fontinfo->ascent,
		    strg, (int)strlen(strg));
	image1 = XGetImage(xps_display,
			   pixmap_tampon1,
			   0,0,
			   largeur, hauteur,
			   !0, ZPixmap);
	XFreePixmap(xps_display,pixmap_tampon1);

	/* cadre pour l'agrandissement de ce texte */
	l = largeur;
	h = hauteur;
	largeur = (int)(dx * (float)l);
	hauteur = (int)(dx * (float)h);
	pixmap_tampon2 = XCreatePixmap(xps_display,
				       xps_dessin[ps_current_window]->xps_window,
				       largeur, hauteur,
				       DefaultDepth(xps_display,xps_screen));
	image2 = XGetImage(xps_display,
			  pixmap_tampon2,
			  0,0,
			  largeur, hauteur,
			  !0, ZPixmap);
	XFreePixmap(xps_display,pixmap_tampon2);

	/* agrandissement du texte */
	for (i=0; i<largeur; i++)
		for (j=0; j<hauteur; j++)
		{
			XPutPixel(image2,i,j,
				  XGetPixel(image1,
					    (int)((float)(i) / dx),
					    (int)((float)(j) / dx)));
		}

	/* rotation et ecriture */
	alpha = -(double)(rot+xps_dessin[ps_current_window]->alpha) * M_PI / 180.;
	sina = sin(alpha);
	cosa = cos(alpha);

	XSetForeground(xps_display,
		       xps_dessin[ps_current_window]->xps_gc,
		       fg);
	
	for (i=0; i<largeur; i++)
		for (j=0; j<hauteur; j++)
		{
			pixel = XGetPixel(image2,i,j);
			if (pixel)
			{
				x = (int)floor((double)(i)*cosa - (double)(j)*sina + .5) + pt[0];
				y = (int)floor((double)(i)*sina + (double)(j)*cosa + .5) + pt[1] - hauteur;
				XDrawPoint(xps_display,
					   xps_dessin[ps_current_window]->pixmap,
					   xps_dessin[ps_current_window]->xps_gc,
					   x,y);
			}
		}
	
	XDestroyImage(image1);
	XDestroyImage(image2);

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSfindfont		*
 *					*
 ****************************************
 * Cherche la fonte la plus proche de   *
 * celle demandee                       *
 ****************************************/

void PSfindfont(char *police, float h_lettre)
{
	int taille, offset;
	char fonte[256];
	
#if XPS_DEBUG
	printf("PSfindfont \n");
#endif
#if 1
	if (!xps_display)
		return;
	
	if (xps_dessin[ps_current_window]->fontinfo != NULL)
		XFreeFont(xps_display,xps_dessin[ps_current_window]->fontinfo);

	taille = (int)(h_lettre * X_PAS);
	if (taille <= 9)
	{
		offset = 0;
		xps_dessin[ps_current_window]->h_font = 8;
	}
	else if (taille <= 11)
	{
		offset = 1;
		xps_dessin[ps_current_window]->h_font = 10;
	}
	else if (taille <= 13)
	{
		offset = 2;
		xps_dessin[ps_current_window]->h_font = 12;
	}
	else if (taille <= 16)
	{
		offset = 3;
		xps_dessin[ps_current_window]->h_font = 14;
	}
	else if (taille <= 20)
	{
		offset = 4;
		xps_dessin[ps_current_window]->h_font = 18;
	}
	else
	{
		offset = 5;
		xps_dessin[ps_current_window]->h_font = 24;
	}

	if (!strcmp(police,"AvantGarde-Book"))
		strcpy(fonte,fontelist[AVANTGARDE+BOOK+offset]);
	else if (!strcmp(police,"AvantGarde-BookOblique"))
		strcpy(fonte,fontelist[AVANTGARDE+BOOK+OBLIQUE+offset]);
	else if (!strcmp(police,"AvantGarde-Demi"))
		strcpy(fonte,fontelist[AVANTGARDE+DEMI+offset]);
	else if (!strcmp(police,"AvantGarde-DemiOblique"))
		strcpy(fonte,fontelist[AVANTGARDE+DEMI+OBLIQUE+offset]);
	else if (!strcmp(police,"Courier"))
		strcpy(fonte,fontelist[COURIER+offset]);
	else if (!strcmp(police,"Courier-Bold"))
		strcpy(fonte,fontelist[COURIER+BOLD+offset]);
	else if (!strcmp(police,"Courier-Oblique"))
		strcpy(fonte,fontelist[COURIER+OBLIQUE+offset]);
	else if (!strcmp(police,"Courier-BoldOblique"))
		strcpy(fonte,fontelist[COURIER+BOLD+OBLIQUE+offset]);
	else if (!strcmp(police,"Helvetica"))
		strcpy(fonte,fontelist[HELVETICA+offset]);
	else if (!strcmp(police,"Helvetica-Bold"))
		strcpy(fonte,fontelist[HELVETICA+BOLD+offset]);
	else if (!strcmp(police,"Helvetica-Oblique"))
		strcpy(fonte,fontelist[HELVETICA+OBLIQUE+offset]);
	else if (!strcmp(police,"Helvetica-BoldOblique"))
		strcpy(fonte,fontelist[HELVETICA+BOLD+OBLIQUE+offset]);
	else if (!strcmp(police,"LubalinGraph-Book"))
		strcpy(fonte,fontelist[LULABIN+BOOK+offset]);
	else if (!strcmp(police,"LubalinGraph-BookOblique"))
		strcpy(fonte,fontelist[LULABIN+BOOK+OBLIQUE+offset]);
	else if (!strcmp(police,"LubalinGraph-Demi"))
		strcpy(fonte,fontelist[LULABIN+DEMI+offset]);
	else if (!strcmp(police,"LubalinGraph-DemiOblique"))
		strcpy(fonte,fontelist[LULABIN+DEMI+OBLIQUE+offset]);
	else if (!strcmp(police,"Times-Roman"))
		strcpy(fonte,fontelist[TIMES+ROMAN+offset]);
	else if (!strcmp(police,"Times-Bold"))
		strcpy(fonte,fontelist[TIMES+BOLD+offset]);
	else if (!strcmp(police,"Times-Italic"))
		strcpy(fonte,fontelist[TIMES+ITALIC+offset]);
	else if (!strcmp(police,"Times-BoldItalic"))
		strcpy(fonte,fontelist[TIMES+BOLD+ITALIC+offset]);
	else if (!strcmp(police,"NewCenturySchlbk-Roman"))
		strcpy(fonte,fontelist[NEWCENTURY+ROMAN+offset]);
	else if (!strcmp(police,"NewCenturySchlbk-Bold"))
		strcpy(fonte,fontelist[NEWCENTURY+BOLD+offset]);
	else if (!strcmp(police,"NewCenturySchlbk-Italic"))
		strcpy(fonte,fontelist[NEWCENTURY+ITALIC+offset]);
	else if (!strcmp(police,"NewCenturySchlbk-BoldItalic"))
		strcpy(fonte,fontelist[NEWCENTURY+BOLD+ITALIC+offset]);
	else if (!strcmp(police,"Souvenir-Demi"))
		strcpy(fonte,fontelist[SOUVENIR+DEMI+offset]);
	else if (!strcmp(police,"Souvenir-DemiItalic"))
		strcpy(fonte,fontelist[SOUVENIR+DEMI+ITALIC+offset]);
	else if (!strcmp(police,"Souvenir-Light"))
		strcpy(fonte,fontelist[SOUVENIR+LIGHT+offset]);
	else if (!strcmp(police,"Souvenir-LightItalic"))
		strcpy(fonte,fontelist[SOUVENIR+LIGHT+ITALIC+offset]);
	else if (!strcmp(police,"Symbol"))
		strcpy(fonte,fontelist[SYMBOL+offset]);
	else
	{
		strcpy(fonte,"6x10");
		fprintf(stderr,"PLOT_XPS Warning: Can't find font <%s>, use default\n",police);
		xps_dessin[ps_current_window]->fontinfo =
			XLoadQueryFont(xps_display,"6x10");
	}

	if ((xps_dessin[ps_current_window]->fontinfo =
		XLoadQueryFont(xps_display,fonte)) == NULL)
	{
		fprintf(stderr,"PLOT_XPS Warning: Can't find font <%s>, use default\n",police);
		xps_dessin[ps_current_window]->fontinfo =
			XLoadQueryFont(xps_display,"6x10");
	}

	XSetFont(xps_display,
		 xps_dessin[ps_current_window]->xps_gc,
		 xps_dessin[ps_current_window]->fontinfo->fid);

#endif
}


/****************************************/
/*					*
 *		PSscale			*
 *					*
 ****************************************
 * Change la matrice de transformation  *
 * courante pour un scaling             *
 ****************************************/

void PSscale(float x_scl, float y_scl)
{
#if XPS_DEBUG
	printf("PSscale x: %f  y: %f\n",x_scl,y_scl);
#endif
#if 1
	xps_dessin[ps_current_window]->cmt[0][0] *= x_scl;
	xps_dessin[ps_current_window]->cmt[1][0] *= x_scl;
	xps_dessin[ps_current_window]->cmt[0][1] *= y_scl;
	xps_dessin[ps_current_window]->cmt[1][1] *= y_scl;

	xps_dessin[ps_current_window]->xder = 0.;
	xps_dessin[ps_current_window]->yder = 0.;
	xps_dessin[ps_current_window]->xini = 0.;
	xps_dessin[ps_current_window]->yini = 0.;
#endif
}


/****************************************/
/*					*
 *		PSrotate		*
 *					*
 ****************************************
 * Change la matrice de transformation  *
 * courante pour une rotation           *
 ****************************************/

void PSrotate(int beta)
{
	float a,b,c,d;
	double alpha;
	
#if XPS_DEBUG
	printf("PSrotate %d deg\n",beta);
#endif
#if 1
	alpha = (double)(beta) * M_PI / 180.;
	a =  xps_dessin[ps_current_window]->cmt[0][0] * (float)cos(alpha)
	   + xps_dessin[ps_current_window]->cmt[0][1] * (float)sin(alpha);
	b =  xps_dessin[ps_current_window]->cmt[1][0] * (float)cos(alpha)
	   + xps_dessin[ps_current_window]->cmt[1][1] * (float)sin(alpha);
	c =  xps_dessin[ps_current_window]->cmt[0][0] * (float)(-sin(alpha))
	   + xps_dessin[ps_current_window]->cmt[0][1] * (float)cos(alpha);
	d =  xps_dessin[ps_current_window]->cmt[1][0] * (float)(-sin(alpha))
	   + xps_dessin[ps_current_window]->cmt[1][1] * (float)cos(alpha);
	xps_dessin[ps_current_window]->cmt[0][0] = a;
	xps_dessin[ps_current_window]->cmt[1][0] = b;
	xps_dessin[ps_current_window]->cmt[0][1] = c;
	xps_dessin[ps_current_window]->cmt[1][1] = d;

	xps_dessin[ps_current_window]->xder = 0.;
	xps_dessin[ps_current_window]->yder = 0.;
	xps_dessin[ps_current_window]->xini = 0.;
	xps_dessin[ps_current_window]->yini = 0.;

	xps_dessin[ps_current_window]->alpha += beta;

#endif
}

/****************************************/
/*					*
 *		PStranslate		*
 *					*
 ****************************************
 * Change la matrice de transformation  *
 * courante pour une translation        *
 ****************************************/

void PStranslate(float x, float y)
{
#if XPS_DEBUG
	printf("PStranslate x: %f  y: %f\n",x,y);
#endif
#if 1
	xps_dessin[ps_current_window]->cmt[0][2] +=
		  x * xps_dessin[ps_current_window]->cmt[0][0]
		+ y * xps_dessin[ps_current_window]->cmt[0][1];
	xps_dessin[ps_current_window]->cmt[1][2] +=
		  x * xps_dessin[ps_current_window]->cmt[1][0]
		+ y * xps_dessin[ps_current_window]->cmt[1][1];

	xps_dessin[ps_current_window]->xder = 0.;
	xps_dessin[ps_current_window]->yder = 0.;
	xps_dessin[ps_current_window]->xini = 0.;
	xps_dessin[ps_current_window]->yini = 0.;
#endif
}


/****************************************/
/*					*
 *		PSsetlinewidth		*
 *					*
 ****************************************
 * Regle la largeur du trace            *
 ****************************************/

void PSsetlinewidth(float largeur)
{
#if XPS_DEBUG
	printf("PSsetlinewidth \n");
#endif
	if (!xps_display)
		return;
#if 1	
	XSetLineAttributes(xps_display,
		      xps_dessin[ps_current_window]->xps_gc,
		      (int)(largeur*pl_conv),
		      LineSolid,CapButt,JoinMiter);
#endif
}

/****************************************/
/*					*
 *		PSsetdash		*
 *					*
 ****************************************
 * Regle le trace en pointille          *
 ****************************************/

void PSsetdash(float dash, float gap, float offset)
{
	char dash_list[2];
	XGCValues values;
#if XPS_DEBUG
	printf("PSsetdash \n");
#endif
	if (!xps_display)
		return;
#if 1
	dash_list[1] = (int)(gap*pl_conv);
	dash_list[0] = (int)(dash*pl_conv);
	XSetDashes(xps_display,
		   xps_dessin[ps_current_window]->xps_gc,
		   (int)(offset*pl_conv),
		   dash_list,
		   2);
	values.line_style = LineOnOffDash;
	XChangeGC(xps_display,
		  xps_dessin[ps_current_window]->xps_gc,
		  GCLineStyle,
		  &values);
#endif
}

/****************************************/
/*					*
 *		PSsetsolid		*
 *					*
 ****************************************
 * Regle le trace en ligne pleine       *
 ****************************************/

void PSsetsolid()
{
	XGCValues values;
#if XPS_DEBUG
	printf("PSsetdash \n");
#endif
	if (!xps_display)
		return;
#if 1
	values.line_style = LineSolid;
	XChangeGC(xps_display,
		  xps_dessin[ps_current_window]->xps_gc,
		  GCLineStyle,
		  &values);
#endif
}

/****************************************/
/*					*
 *		PSsetgray		*
 *					*
 ****************************************
 * Regle la couleur du trace en niveau  *
 * de gris                              *
 ****************************************/

void PSsetgray(float gris)
{
	int index;
#if XPS_DEBUG
	printf("PSsetgray \n");
#endif
	if (!xps_display)
		return;
#if 1	
	index = 0;
	switch(type_ecran)
	{
		case 0 : return;
		case 1 : 
		case 2 : index = BASE_GRIS + (int)(gris*NB_GRIS); break;
#if 0
		case 3 : index = (int)(gris*255.)*256*256
				+(int)(gris*255.)*256
				+(int)(gris*255.); break;
#endif
		case 3 :
		{
			XColor	hw_rgb;
			Colormap	colourmap =
					DefaultColormap(xps_display,xps_screen);
			 hw_rgb.red = gris * 0xffff ;
			 hw_rgb.green = gris * 0xffff ;
			 hw_rgb.blue = gris * 0xffff ;
			 XAllocColor (xps_display, colourmap, &hw_rgb);

			 index = hw_rgb.pixel ;
			 break;
		} /* fin du case */
	}
	XSetForeground(xps_display,
		       xps_dessin[ps_current_window]->xps_gc,
		       index);
#endif
}


/****************************************/
/*					*
 *		PSsetrgbcolor		*
 *					*
 ****************************************
 * Regle la couleur du trace par le     *
 * mode Red Green Blue                  * 
 ****************************************/

void PSsetrgbcolor(float red, float green, float blue)
{
	int index;
#if XPS_DEBUG
	printf("PSsetrgbcolor \n");
#endif
	if (!xps_display)
		return;
#if 1	
	index = 0;
	switch(type_ecran)
	{
		case 0:
			return;
		case 1:
			;
		case 2:
			index = BASE_COUL + (int)(red*5.)*30
					 + (int)(green*5.)*5
					 + (int)(blue*4.);
			break;
#if 0
		case 3:
			index =  (int)(red*255.)*256*256
			      + (int)(green*255.)*256
			      + (int)(blue*255.);
			break;
#endif
		case 3 :
		{
			XColor	hw_rgb;
			Colormap	colourmap =
					DefaultColormap(xps_display,xps_screen);
			hw_rgb.red = red * 0xffff ;
			hw_rgb.green = green * 0xffff ;
			hw_rgb.blue = blue * 0xffff ;
			XAllocColor (xps_display, colourmap, &hw_rgb);

			index = hw_rgb.pixel ;
			break;
		} /* fin du case */
	}
	XSetForeground(xps_display,
		       xps_dessin[ps_current_window]->xps_gc,
		       index);
	xps_dessin[ps_current_window]->red   = red;
	xps_dessin[ps_current_window]->green = green;
	xps_dessin[ps_current_window]->blue  = blue;
#endif
}


/****************************************/
/*					*
 *		PSsethsbcolor		*
 *					*
 ****************************************
 * Regle la couleur du trace par le     *
 * mode Hue Saturation Brightness       *
 ****************************************/

void PSsethsbcolor(float hue, float saturation, float brightness)
{
#if XPS_DEBUG
	printf("PSsethsbcolor \n");
#endif
	if (!xps_display)
		return;
	
	fprintf(stderr,"PLOT_XPS Warning: ps_set_hsb_color not yet implemented for X, use rgb\n");
}


/****************************************/
/*					*
 *		PSsetcmykcolor		*
 *					*
 ****************************************
 * Regle la couleur du trace par le     *
 * mode  Cyan Magenta Yellow Black      *
 ****************************************/

void PSsetcmykcolor(float cyan, float magenta, float yellow, float black)
{
#if XPS_DEBUG
	printf("PSsetcmykcolor \n");
#endif
	if (!xps_display)
		return;
	
	fprintf(stderr,"PLOT_XPS Warning: ps_set_cmyk_color not yet implemented for X, use rgb\n");
}


/****************************************/
/*					*
 *		PScurrentrgbcolor	*
 *					*
 ****************************************
 * Renvoie la couleur courante par le   *
 * mode Red Green Blue                  *
 ****************************************/

void PScurrentrgbcolor(float *red, float *green, float *blue)
{
#if XPS_DEBUG
	printf("PScurrentrgbcolor \n");
#endif
#if 1
	*red   = xps_dessin[ps_current_window]->red;
	*green = xps_dessin[ps_current_window]->green;
	*blue  = xps_dessin[ps_current_window]->blue;
#endif
}


/****************************************/
/*					*
 *		PScurrenthsbcolor	*
 *					*
 ****************************************
 * Renvoie la couleur courante par le   *
 * mode Hue Saturation Brightness       *
 ****************************************/

void PScurrenthsbcolor(float *hue, float *saturation, float *brightness)
{
#if XPS_DEBUG
	printf("PScurrenthsbcolor \n");
#endif
	fprintf(stderr,"PLOT_XPS Warning: ps_get_hsb_color not yet implemented for X, use rgb\n");
}


/****************************************/
/*					*
 *		PScurrentcmykcolor	*
 *					*
 ****************************************
 * Renvoie la couleur courante par le   *
 * mode Cyan Magenta Yellow Black       *
 ****************************************/

void PScurrentcmykcolor(float *cyan, float *magenta, float *yellow, float *black)
{
#if XPS_DEBUG
	printf("PScurrentcmykcolor \n");
#endif
	fprintf(stderr,"PLOT_XPS Warning: ps_get_hsb_color not yet implemented for X, use rgb\n");
}


/****************************************/
/*					*
 *		PSarcn			*
 *					*
 ****************************************
 * Permet de tracer arc cercle ellipse  *
 * angle dans le sens horaire           *
 ****************************************/

void PSarcn(float x, float y,
	    float rx, float ry,
	    float theta_1, float theta_2)
{
	int	centre[2],
		rayon[2];

#if XPS_DEBUG
	printf("PSarcn \n");
#endif
	if (!xps_display)
		return;
#if 1	
	theta_1 *= -1;
	theta_2 *= -1;
	xps_pixel(x,y,centre);
	rayon[0] = rx * pl_conv;
	rayon[1] = ry * pl_conv;
	centre[0] -= rayon[0];
	centre[1] -= rayon[1];

	XDrawArc(xps_display,
		 xps_dessin[ps_current_window]->pixmap,
		 xps_dessin[ps_current_window]->xps_gc,
		 centre[0],centre[1],
		 2*rayon[0],2*rayon[1],
		 (int)(theta_1*64.),(int)(theta_2*64.));

	xps_dessin[ps_current_window]->xder = x;
	xps_dessin[ps_current_window]->yder = y;

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSarc			*
 *					*
 ****************************************
 * Permet de tracer arc cercle ellipse  *
 * angle dans le sens trigonometrique   *
 ****************************************/

void PSarc(float x, float y,
	   float rx, float ry,
	   float theta_1, float theta_2)
{
	int	centre[2],
		rayon[2];
	
#if XPS_DEBUG
	printf("PSarc \n");
#endif
	if (!xps_display)
		return;
#if 1	
	xps_pixel(x,y,centre);
	rayon[0] = rx * pl_conv;
	rayon[1] = ry * pl_conv;
	centre[0] -= rayon[0];
	centre[1] -= rayon[1];
	
	XDrawArc(xps_display,
		 xps_dessin[ps_current_window]->pixmap,
		 xps_dessin[ps_current_window]->xps_gc,
		 centre[0],centre[1],
		 2*rayon[0],2*rayon[1],
		 (int)(theta_1*64.),(int)(theta_2*64.));

	xps_dessin[ps_current_window]->xder = x;
	xps_dessin[ps_current_window]->yder = y;

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSfillarc		*
 *					*
 ****************************************
 * Permet de tracer arc cercle ellipse  *
 * remplis avec la couleur courante     *
 ****************************************/

void PSfillarc(float x, float y,
	       float rx, float ry,
	       float theta_1, float theta_2)
{
	int	centre[2],
		rayon[2];
	
#if XPS_DEBUG
	printf("PSarc \n");
#endif
	if (!xps_display)
		return;
#if 1	
	xps_pixel(x,y,centre);
	rayon[0] = rx * pl_conv;
	rayon[1] = ry * pl_conv;
	centre[0] -= rayon[0];
	centre[1] -= rayon[1];
	
	XFillArc(xps_display,
		 xps_dessin[ps_current_window]->pixmap,
		 xps_dessin[ps_current_window]->xps_gc,
		 centre[0],centre[1],
		 2*rayon[0],2*rayon[1],
		 (int)(theta_1*64.),(int)(theta_2*64.));

	xps_dessin[ps_current_window]->xder = x;
	xps_dessin[ps_current_window]->yder = y;

	PSflush();
#endif
}



/****************************************/
/*					*
 *		PSfill			*
 *					*
 ****************************************
 * Rempli une zone defini par n points  *
 ****************************************/

void PSfill(float *x_i, float *y_i, int npt)
{
	XPoint *points;
	int	pt[2],i;

#if XPS_DEBUG
	printf("PSfill \n");
#endif
	if (!xps_display)
		return;
#if 1	
	if ((points = (XPoint *)calloc(npt,sizeof(XPoint))) == NULL)
	{
		fprintf(stderr,"PLOT_XPS Warning: Allocation impossible dans PSfill\n");
		return;
	} /* fin du if */

	for (i=0; i<npt; i++)
	{
		xps_pixel(x_i[i],y_i[i],pt);
		points[i].x = pt[0];
		points[i].y = pt[1];
	} /* fin du for */

	XFillPolygon(xps_display,
		     xps_dessin[ps_current_window]->pixmap,
		     xps_dessin[ps_current_window]->xps_gc,
		     points, npt,
		     Complex, CoordModeOrigin);

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSclosepath		*
 *					*
 ****************************************
 * Trace un trait entre le point courant*
 * et le point initiale defini dans     *
 * PSnewpath				*
 ****************************************/

void PSclosepath()
{
	int pt1[2],pt2[2];
#if XPS_DEBUG
	printf("PSclosepath \n");
#endif
	if (!xps_display)
		return;
#if 1	
	xps_pixel(xps_dessin[ps_current_window]->xder,
		  xps_dessin[ps_current_window]->yder,
		  pt1);
	xps_pixel(xps_dessin[ps_current_window]->xini,
		  xps_dessin[ps_current_window]->yini,
		  pt2);

	XDrawLine(xps_display,
		  xps_dessin[ps_current_window]->pixmap,
		  xps_dessin[ps_current_window]->xps_gc,
		  pt1[0],pt1[1],
		  pt2[0],pt2[1]);

	xps_dessin[ps_current_window]->xder = xps_dessin[ps_current_window]->xini;
	xps_dessin[ps_current_window]->yder = xps_dessin[ps_current_window]->yini;

	PSflush();
#endif
}


/****************************************/
/*					*
 *		PSnewpath		*
 *					*
 ****************************************
 * Initialise un nouveau trajet         *
 ****************************************/

void PSnewpath()
{
#if XPS_DEBUG
	printf("PSnewpath \n");
#endif
#if 1
	xps_dessin[ps_current_window]->xini = xps_dessin[ps_current_window]->xder;
	xps_dessin[ps_current_window]->yini = xps_dessin[ps_current_window]->yder;
#endif
}
