/*
=============================================================================
	A C T I O N _ C A L L B A C K S _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for resarch purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- COMMON_ACTION : This callback is called for common actions.
			  It calls the action corresponding of the given tag.
	- SIMUL_ACTION : This callback is called for actions corresponding to
			 a simulation step. It calls the action corresponding 
			 of the given tag.
=============================================================================
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <X11/X.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include "global_def_visu.h"


/*
=============================================================================
Name and dates: COMMON_ACTION
---------------
	Creation : 20-04-93
	Last modification : 20-04-93

Functionnal description :
-------------------------
	This callback call the function corresponding of the given tag. This
	tag must be one of the constants declared in value section of UILs and
	defined in global_def_visu.h.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	w		Widget		Widget id of the caller
			tag		int *		The wanted action
			reason		XtPointer	The reason of the call
							-back. This could be too
							the address of the
							callback structure.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	Definition of the network
			CMAP_ID		Colormap	The colormap
							
Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

#define NETWORK_FILES 0
#define SIMUL_FILES   1


void CONTROL_BUTTON(Widget w, XtPointer TAG, XtPointer reason)
{
	switch((int)TAG)
	{
	case k_Button_Matrix:
	  {
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation],True);

		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Global_Activity],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation],False);

	    break;
	  }
	case k_Button_Linear:
	  {
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation],True);

		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Global_Activity],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation],False);

	    break;
	  }
	case k_Button_Global_Act:
	  {
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Global_Activity],True);

		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation],False);

	    break;
	  }
	case k_Button_Membr_Pot:
	  {
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation],True);

		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation],False);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Global_Activity],False);

	    break;
	  }

	}
	

}

void COMMON_ACTION(Widget w, XtPointer TAG, XtPointer reason)
{
	int *POSITION;
	static int *UNSELECT_NEURON;	/*array containning index of unselect neurons*/
	XmFileSelectionBoxCallbackStruct *FILE_CB_STRUCT;
	XmDrawingAreaCallbackStruct *DRAW_CB_STRUCT;
	XmListCallbackStruct *LIST_CB_STRUCT;
	char *FILE_NAME, *CHAR_PTR;
	Widget Wid;
	XmString PATTERN, STRING;
	Arg arglist[5];
	static int WICH_FILE;	/*flag : NETWORK_FILES or SIMUL_FILES*/
	static int NB_UNSELECT_NEURON;	/*size of the array of unselect neurons*/
	int i,n, NB_ITEM, NEURON;
	int TOP_VALUE, BOTTOM_VALUE;
	XEvent EVENT;
	char MESSAGE[500];	/*error message*/
	Boolean  argok=False;
	char buffer[100];

	switch((int)TAG)
	  {
/*
---------------------------------------------------------------------------------
display the About Window
---------------------------------------------------------------------------------
*/
		case k_Button_Help_Main_About:
			MANAGE(k_Window_About, "Window_About");
			break;

		case k_Button_Help_Browse:
			browse_manual(XNBC_VISU);
			break;

			
/*
---------------------------------------------------------------------------------
display the File Selection Window
---------------------------------------------------------------------------------
*/
		case k_Button_Open_File:
/*
                        printf("ID simul avant: %d\n",WIDGET_ARRAY[k_Window_Simul_Selection]);
*/
			MANAGE(k_Window_Network_Selection, "Window_Network_Selection");

			PATTERN = XmStringCreateSimple("*.len");
			n=0;

			XtSetArg(arglist[n], XmNdirMask,  XmStringCreateSimple(get_dir_name())); n++;
			XtSetArg(arglist[n], XmNpattern, PATTERN); n++;
			XtSetValues(WIDGET_ARRAY[k_Window_Network_Selection], arglist, n);
/*
                        printf("ID network: %d\n",WIDGET_ARRAY[k_Window_Network_Selection]);
*/
			XmStringFree(PATTERN);

			WICH_FILE = NETWORK_FILES;
			break;

/*
---------------------------------------------------------------------------------
display the File Selection Window
---------------------------------------------------------------------------------
*/
		case k_Button_Open_Simul_Files:
/*
			printf("ID simul avant: %d\n",WIDGET_ARRAY[k_Window_Simul_Selection]);
*/
			MANAGE(k_Window_Simul_Selection, "Window_Simul_Selection");

/*
                        printf("ID simul apres: %d\n",WIDGET_ARRAY[k_Window_Simul_Selection]);
*/
			PATTERN = XmStringCreateSimple("*.pmb");
			n=0;
			XtSetArg(arglist[n], XmNdirMask,  XmStringCreateSimple(get_dir_name())); n++;
			XtSetArg(arglist[n], XmNpattern, PATTERN); n++;
			XtSetValues(WIDGET_ARRAY[k_Window_Simul_Selection], arglist, n);
			XmStringFree(PATTERN);

			WICH_FILE = SIMUL_FILES;
			break;

/*
---------------------------------------------------------------------------------
Get back the wanted file
---------------------------------------------------------------------------------
*/
		case k_Button_Ok_File_Selection:
			XtSetSensitive(WIDGET_ARRAY[k_Button_Select_Neuron], True);
			switch(WICH_FILE)
			{
				case NETWORK_FILES :
                                        MESSAGE[0] = '\0';
					/*get back the callback struct*/
					FILE_CB_STRUCT = (XmFileSelectionBoxCallbackStruct *)reason;
					/*keep the directory*/
					MASK = FILE_CB_STRUCT->mask;
					/*get back the file name*/
					XmStringGetLtoR(FILE_CB_STRUCT->value, XmSTRING_DEFAULT_CHARSET,
							&FILE_NAME);
					/***/
					/***/

					/*get back the root of the file name*/
					if((CHAR_PTR = strrchr(FILE_NAME, '.')) != Null)
						*CHAR_PTR = '\0';
					if(FILE_NAME[0] == '\0')
					{
						DISPLAY_MESSAGE("You have not given a file name", ERROR);
						break;
					}
					/*set wait cursor*/
                                        /*	SET_WAIT_CURSOR(); */
					/*
					 * try to open the given file with .cnx and .anat extension and to 
					 * to create the corresponding network
					 */

					if(!CREATE_NETWORK(FILE_NAME, MESSAGE))
					{
						UNSET_WAIT_CURSOR();
						DISPLAY_MESSAGE(MESSAGE, ERROR);
					}
       					XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
#if DEBUG
					printf("File name: %s\n",FILE_NAME);
#endif
					XtFree(FILE_NAME);
					break;

				case SIMUL_FILES :
                                        MESSAGE[0] = '\0';
					/*get back the callback struct*/
					FILE_CB_STRUCT = (XmFileSelectionBoxCallbackStruct *)reason;
					/*keep the directory*/
					MASK = FILE_CB_STRUCT->mask;
					/*get back the file name*/
					XmStringGetLtoR(FILE_CB_STRUCT->value, XmSTRING_DEFAULT_CHARSET,
							&FILE_NAME);
#if DEBUG
					printf("File name: %s\n",FILE_NAME);
#endif
					/*get back the root of the file name*/
					if((CHAR_PTR = strrchr(FILE_NAME, '.')) != Null)
						*CHAR_PTR = '\0';
					if(FILE_NAME[0] == '\0')
					{
						DISPLAY_MESSAGE("You have not given a file name", ERROR);
						break;
					}

					/*set wait cursor*/
					SET_WAIT_CURSOR();
					/*try to open the given file with .def and .bmp extension and to 
					initializes the corresponding simulation*/
					if(!LOAD_SIMUL(FILE_NAME, MESSAGE))
					{
						UNSET_WAIT_CURSOR();
						DISPLAY_MESSAGE(MESSAGE, ERROR);
					}
					else
					{
					        /* GS add */
					        /* memorisation of the file name */
					        strcpy(LOAD_FILE_NAME, FILE_NAME);
					        /* allocation des tables */
					        INIT_ALLOC_TAB_SOURCE(FILE_NAME);
					        /* end GS add */

						XtSetSensitive(WIDGET_ARRAY[k_Button_Open_File], False);
						XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
						XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
						UNSET_WAIT_CURSOR();
					}
					XtFree(FILE_NAME);
					UPDATE_REPRESENTATION();
					break;
			}
			break;

/*
---------------------------------------------------------------------------------
Redraw the network, if it exists, in the expose window
---------------------------------------------------------------------------------
*/
		case k_Window_Drawing_Area_Main:
			if(NETWORK.FLAG.EXIST)
			{
/*
				printf("Redraw the network\n");
*/
				SET_WAIT_CURSOR();
				R_DRAW();
				/*Expose events filtering*/
				while(XCheckTypedWindowEvent(DISPLAY_VISU, 
				      XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]), 
				      Expose, &EVENT));
				UNSET_WAIT_CURSOR();
			}
/*
			else
				printf("Pas de Redraw the network\n");
*/
			break;

		case k_Draw_Affiche_Pmb0:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb0,
					    tab_source0,
					    X_pos[0], Y_pos[0],
					    Debut[0], Fin[0], Lecture_Memoire );
			      break;
			    }
			}
	        case k_Draw_Affiche_Pmb1:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb1,
					    tab_source1,
					    X_pos[1], Y_pos[1],
					    Debut[1], Fin[1], Lecture_Memoire );
			      break;
			    }
			}
		case k_Draw_Affiche_Pmb2:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb2,
					    tab_source2,
					    X_pos[2], Y_pos[2],
					    Debut[2], Fin[2], Lecture_Memoire );
			      break;
			    }
			}
		case k_Draw_Affiche_Pmb3:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb3,
					    tab_source3,   
					    X_pos[3], Y_pos[3],
					    Debut[3], Fin[3], Lecture_Memoire );
			      break;
			    }
			}
		case k_Draw_Affiche_Pmb4:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb4,
					    tab_source4,
					    X_pos[4], Y_pos[4],
					    Debut[4], Fin[4], Lecture_Memoire );
			      break;
			    }
			}
		case k_Draw_Affiche_Pmb5:
			{
			  if(NETWORK.FLAG.EXIST)
			    {
			      CREATE_WINDOW(k_Draw_Affiche_Pmb5,
					    tab_source5,
					    X_pos[5], Y_pos[5],
					    Debut[5], Fin[5], Lecture_Memoire );
			      break;
			    }
			}

/***********************************************************************/

		case k_Button_Print_Affiche_Pmb0:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 0;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
		case k_Button_Print_Affiche_Pmb1:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 1;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
		case k_Button_Print_Affiche_Pmb2:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 2;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
		case k_Button_Print_Affiche_Pmb3:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 3;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
		case k_Button_Print_Affiche_Pmb4:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 4;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
		case k_Button_Print_Affiche_Pmb5:
			{
			  sprintf(MESSAGE, "%s_v_aff.ps", NETWORK.NAME);
			  
			  STRING = XmStringCreateSimple(MESSAGE);
			  n=0;
			  XtSetArg(arglist[n], XmNtextString, STRING); n++;
			  XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			  XmStringFree(STRING);
			  FLAG_AFFICHE = 1;
			  INDEX_AFF = 5;
			  MANAGE(k_Window_Print, "Window_Print");
			  break;
			}
/*************************************************************************/

		case k_Button_Cancel_Affiche_Pmb0:
			{
			  UNMANAGE(k_Window_Affiche_Pmb0);
			  Tab_Pointer_Affiche[0]=0;
			  Debut[0] = Fin[0] = Null;
			  break;
			}
		case k_Button_Cancel_Affiche_Pmb1:
			{
			  UNMANAGE(k_Window_Affiche_Pmb1);
			  Tab_Pointer_Affiche[1]=0;
			  Debut[1] = Fin[1] = Null;
			  break;
			}
		case k_Button_Cancel_Affiche_Pmb2:
			{
			  UNMANAGE(k_Window_Affiche_Pmb2);
			  Tab_Pointer_Affiche[2]=0;
			  Debut[2] = Fin[2] = Null;
			  break;
			}
		case k_Button_Cancel_Affiche_Pmb3:
			{
			  UNMANAGE(k_Window_Affiche_Pmb3);
			  Tab_Pointer_Affiche[3]=0;
			  Debut[3] = Fin[3] = Null;
			  break;
			}
		case k_Button_Cancel_Affiche_Pmb4:
			{
			  UNMANAGE(k_Window_Affiche_Pmb4);
			  Tab_Pointer_Affiche[4]=0;
			  Debut[4] = Fin[4] = Null;
			  break;
			}
		case k_Button_Cancel_Affiche_Pmb5:
			{
			  UNMANAGE(k_Window_Affiche_Pmb5);
			  Tab_Pointer_Affiche[5]=0;
			  Debut[5] = Fin[5] = Null;
			  break;
			}


/*
---------------------------------------------------------------------------------
Switch the draw details flags and refresh the screen
---------------------------------------------------------------------------------
*/
		case k_Button_Refresh_Screen:			
			if(NETWORK.FLAG.EXIST)
			{
				SET_WAIT_CURSOR();
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
				UNSET_WAIT_CURSOR();
			}
			break;

		case k_Toggle_Show_Details:
			NETWORK.FLAG.SHOW_DETAILS = XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Show_Details]);
			if(NETWORK.FLAG.EXIST)
			{
				SET_WAIT_CURSOR();
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
				UNSET_WAIT_CURSOR();
			}
			break;

/*
---------------------------------------------------------------------------------
Switch the neuron selection flag
---------------------------------------------------------------------------------
*/
		case k_Button_Select_Neuron:
			NETWORK.FLAG.SELECTION = True;
			SET_ELECTRODE_CURSOR();
			MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");
			break;			
/*
---------------------------------------------------------------------------------
initializes and manage the window scale top value
---------------------------------------------------------------------------------
*/
		case k_Button_Change_Top_Value :
			MANAGE(k_Window_Scale_Top_Value, "Window_Scale_Top_Value");
			n=0;
			XtSetArg(arglist[n], XmNminimum, 
				(int)(SIMUL.CS_BOTTOM_VALUE + SIMUL.CS_TOP_VALUE)/2 
				+ 5); n++;
			XtSetArg(arglist[n], XmNvalue, (int)SIMUL.CS_TOP_VALUE); n++;
			XtSetValues(WIDGET_ARRAY[k_Scale_Top_Value], arglist, n);
			n=0;
			XtSetArg(arglist[n], XmNmaximum, 
				(int)(SIMUL.CS_BOTTOM_VALUE + SIMUL.CS_TOP_VALUE)/2 
				- 5); n++;
			XtSetArg(arglist[n], XmNvalue, (int)SIMUL.CS_BOTTOM_VALUE); n++;
			XtSetValues(WIDGET_ARRAY[k_Scale_Bottom_Value], arglist, n);
			break;

/*
---------------------------------------------------------------------------------
update scales bottom and top value
---------------------------------------------------------------------------------
*/
		case k_Scale_Bottom_Value:
		case k_Scale_Top_Value:
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Top_Value], &TOP_VALUE);
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Bottom_Value], &BOTTOM_VALUE);
			n=0;
			XtSetArg(arglist[n], XmNminimum, (BOTTOM_VALUE + TOP_VALUE)/2 
				+ 5); n++;
			XtSetValues(WIDGET_ARRAY[k_Scale_Top_Value], arglist, n);
			n=0;
			XtSetArg(arglist[n], XmNmaximum, (BOTTOM_VALUE + TOP_VALUE)/2 
				- 5); n++;
			XtSetValues(WIDGET_ARRAY[k_Scale_Bottom_Value], arglist, n);
			break;
/*
---------------------------------------------------------------------------------
actualizes values of color scale
---------------------------------------------------------------------------------
*/
		case k_Button_Ok_Scale_Top_Value :
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Top_Value], &TOP_VALUE);
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Bottom_Value], &BOTTOM_VALUE);
			SIMUL.CS_BOTTOM_VALUE = (float)BOTTOM_VALUE;
			SIMUL.CS_TOP_VALUE = (float)TOP_VALUE;
			SET_WAIT_CURSOR();
			ACTUALIZE_COLOR_SCALE();
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
			break;

/*
---------------------------------------------------------------------------------
manage the window time step
---------------------------------------------------------------------------------
*/
		case k_Button_Time_Step :
			MANAGE(k_Window_Time_Step, "Window_Time_Step");
			n=0;
			XtSetArg(arglist[n], XmNvalue, (int)SIMUL.TIME_STEP/100); n++;
			XtSetValues(WIDGET_ARRAY[k_Scale_Time_Step], arglist, n);
			break;

/*
---------------------------------------------------------------------------------
actualizes time step
---------------------------------------------------------------------------------
*/
		case k_Button_Ok_Time_Step :
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Time_Step], 
					&SIMUL.TIME_STEP);
			SIMUL.TIME_STEP *= 100;
			break;

/*
---------------------------------------------------------------------------------
manage the window number of iteration
---------------------------------------------------------------------------------
*/
		case k_Button_Number_Of_Iteration :
		        if (Nbr_Iteration == 0)
			  Nbr_Iteration = 10;
			MANAGE(k_Window_Number_It, "Window_Number_Iteration");
			XmScaleSetValue(WIDGET_ARRAY[k_Scale_Number_It], SIMUL.NB_ITERATION);
			if (Nbr_Iteration!=0)
			  { 
			    n = 0;
			    XtSetArg(arglist[0],XmNmaximum,Nbr_Iteration);n++;
			    if (WIDGET_ARRAY[k_Scale_Number_It])
			      {
				XtSetValues(WIDGET_ARRAY[k_Scale_Number_It],arglist,n);
			      }
			  }
			if (Nbr_Iteration<100)
			  {

			    n = 0;
			    XtSetArg(arglist[0],XmNvalue,Nbr_Iteration);n++;
			    if (WIDGET_ARRAY[k_Scale_Number_It])
			      {
				XtSetValues(WIDGET_ARRAY[k_Scale_Number_It],arglist,n);
			      }
			  }

			break;

/*
---------------------------------------------------------------------------------
actualizes number of visible iteration
---------------------------------------------------------------------------------
*/
		case k_Button_Ok_It :
			XmScaleGetValue(WIDGET_ARRAY[k_Scale_Number_It], 
					&SIMUL.NB_ITERATION);

			/* reset the value displayed on the boutons */

			sprintf(buffer,"Visible iterations (%d ms)",SIMUL.NB_ITERATION);
			n = 0;
			XtSetArg(arglist[n], XmNlabelString, 
				 CONVERT(w, buffer,
					 XmRXmString, 0, &argok)); if (argok) n++;
			XtSetValues(WIDGET_ARRAY[k_Button_Millisec], arglist, n);
			XtSetValues(WIDGET_ARRAY[k_Button_Number_Of_Iteration], arglist, n);

			/*recompute the entire representation if needed*/
			if(NETWORK.FLAG.REPRESENTATION == LINEAR_REPRESENTATION ||
			   NETWORK.FLAG.REPRESENTATION == PMB_REPRESENTATION ||
			   NETWORK.FLAG.REPRESENTATION == GA_REPRESENTATION || 
			   NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION)
			{
				SET_WAIT_CURSOR();
				UPDATE_REPRESENTATION();
				UNSET_WAIT_CURSOR();
			}
			break;

/*
---------------------------------------------------------------------------------
manage the window number of iteration
---------------------------------------------------------------------------------
*/
		case k_Button_Movie :
			flag_movie = !flag_movie;
			n = 0;
			if (flag_movie)
			{
			  if (Start_Images == 0)
			    Start_Images = 1;
			  if (Inter_Images == 0)
			    Inter_Images = 10;
			  if (Stop_Images == 0)
			    Stop_Images = 100;
			  MANAGE(k_Window_Inter_Images, "Window_Inter_Images");
			  if (WIDGET_ARRAY[k_Text_Start_Images])
			  {
			      sprintf(buffer,"%d",Start_Images);
			      XmTextSetString(WIDGET_ARRAY[k_Text_Start_Images], buffer);
			  }
			  if (WIDGET_ARRAY[k_Text_Inter_Images])
			  {
			      sprintf(buffer,"%d",Inter_Images);
			      XmTextSetString(WIDGET_ARRAY[k_Text_Inter_Images], buffer);
			  }
			  if (WIDGET_ARRAY[k_Text_Stop_Images])
			  {
			      sprintf(buffer,"%d",Stop_Images);
			      XmTextSetString(WIDGET_ARRAY[k_Text_Stop_Images], buffer);
			  }
			  XtSetArg(arglist[n], XmNlabelString, 
				   CONVERT(w, "Stop movie",
					   XmRXmString, 0, &argok)); if (argok) n++;
			  num_image = 0;
			}
			else
			{
			  XtSetArg(arglist[n], XmNlabelString, 
				   CONVERT(w, "Start movie...",
					   XmRXmString, 0, &argok)); if (argok) n++;
			}
			XtSetValues(WIDGET_ARRAY[k_Button_Movie], arglist, n);
			break;

/*
---------------------------------------------------------------------------------
actualizes number of visible iteration
---------------------------------------------------------------------------------
*/
		case k_Button_Ok_Inter_Images:
			CHAR_PTR = XmTextGetString(WIDGET_ARRAY[k_Text_Start_Images]);
			sscanf(CHAR_PTR,"%d",&Start_Images);
			CHAR_PTR = XmTextGetString(WIDGET_ARRAY[k_Text_Inter_Images]); 
			sscanf(CHAR_PTR,"%d",&Inter_Images);
			CHAR_PTR = XmTextGetString(WIDGET_ARRAY[k_Text_Stop_Images]); 
			sscanf(CHAR_PTR,"%d",&Stop_Images);
			UNMANAGE(k_Window_Inter_Images);
			break;

/*
---------------------------------------------------------------------------------
manage the window of selected neuron
---------------------------------------------------------------------------------
*/
		case k_Button_Unselect_Neuron :
			MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");
			break;
/*
---------------------------------------------------------------------------------
unmanage the window of selected neuron
---------------------------------------------------------------------------------
*/
		case k_Button_Close_List :
			XmListDeselectAllItems(WIDGET_ARRAY[k_List_Selected_Neuron]);
			if(UNSELECT_NEURON != Null)
				free(UNSELECT_NEURON);
			NB_UNSELECT_NEURON = 0;
			UNMANAGE(k_Window_List_Selected_Neuron);
			UNSET_ELECTRODE_CURSOR();
			break;
/*
---------------------------------------------------------------------------------
change focus for the window of selected neuron
---------------------------------------------------------------------------------
*/
		case k_Window_List_Selected_Neuron :
			XSetInputFocus(DISPLAY_VISU, NETWORK.WINDOW_ID, RevertToParent, CurrentTime);
			break;

/*
---------------------------------------------------------------------------------
Selection of neuron to deselect
---------------------------------------------------------------------------------
*/
		case k_List_Selected_Neuron :
			/*get back the callback struct*/
			LIST_CB_STRUCT = (XmListCallbackStruct *)reason;
			if(LIST_CB_STRUCT->reason == XmCR_DEFAULT_ACTION)
			{
				if(UNSELECT_NEURON != Null)
					free(UNSELECT_NEURON);
				NB_UNSELECT_NEURON = 0;
				NEURON = LIST_CB_STRUCT->item_position;
				DESELECT_NEURON(&NEURON, 1);
			}
			else if(LIST_CB_STRUCT->reason == XmCR_EXTENDED_SELECT)
			{
				if(UNSELECT_NEURON != Null)
					UNSELECT_NEURON = (int *)realloc(UNSELECT_NEURON,
						LIST_CB_STRUCT->selected_item_count *
						sizeof(int));
				else
					UNSELECT_NEURON = (int *)calloc(
						LIST_CB_STRUCT->selected_item_count, sizeof(int));
				/*copy unselected neuron*/
				for(n=0; n<LIST_CB_STRUCT->selected_item_count; n++)
					UNSELECT_NEURON[n] = 
						LIST_CB_STRUCT->selected_item_positions[n];
				NB_UNSELECT_NEURON = LIST_CB_STRUCT->selected_item_count;
			}
			break;

/*
---------------------------------------------------------------------------------
Deselection of neuron
---------------------------------------------------------------------------------
*/
		case k_Button_Unselect :
			if(NB_UNSELECT_NEURON == 0)
				XBell(DISPLAY_VISU, 100);
			else
				DESELECT_NEURON(UNSELECT_NEURON, NB_UNSELECT_NEURON);	
			break;
				
/*
---------------------------------------------------------------------------------
manage the window Print
---------------------------------------------------------------------------------
*/
		case k_Button_Print:
			switch (NETWORK.FLAG.REPRESENTATION)
			{
				case	GA_REPRESENTATION:
					sprintf(MESSAGE, "%s_v_ga.ps", NETWORK.NAME);
					break;
				case	MATRIX_REPRESENTATION:
					sprintf(MESSAGE, "%s_v_mat.ps", NETWORK.NAME);
					break;
				case	PMB_REPRESENTATION:
					sprintf(MESSAGE, "%s_v_pmb.ps", NETWORK.NAME);
					break;
				case	LINEAR_REPRESENTATION:
					sprintf(MESSAGE, "%s_v_lin.ps", NETWORK.NAME);
			} /* fin du switch */
#if 0
			printf("ps file: %s (%s)\n",MESSAGE,NETWORK.NAME);
#endif			
			STRING = XmStringCreateSimple(MESSAGE);
			n=0;
			XtSetArg(arglist[n], XmNtextString, STRING); n++;
			XtSetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
			XmStringFree(STRING);
			MANAGE(k_Window_Print, "Window_Print");
			break;
/*
---------------------------------------------------------------------------------
get back the selected queue name
---------------------------------------------------------------------------------
*/
		case k_List_Printer:
			LIST_CB_STRUCT = (XmListCallbackStruct *)reason;
			GET_QUEUE_NAME(LIST_CB_STRUCT->item);
			break;
/*
---------------------------------------------------------------------------------
unmanage the window Print when user cancels
---------------------------------------------------------------------------------
*/
		case k_Print_Box_Cancel:
			UNMANAGE(k_Window_Print);
			FLAG_AFFICHE = 0;
			break;

/*
---------------------------------------------------------------------------------
runs printing
---------------------------------------------------------------------------------
*/
		case k_Print_Box_Ok:
			SET_WAIT_CURSOR();
			PRINT();
			UNSET_WAIT_CURSOR();
			FLAG_AFFICHE = 0;
			break;
/*
---------------------------------------------------------------------------------
selects representation
---------------------------------------------------------------------------------
*/

		
		case k_Toggle_Matrix_Representation :
		  {
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation]) )
		      {
			TOP_VALUE = -30;
			BOTTOM_VALUE = -90;
			SIMUL.CS_BOTTOM_VALUE = (float)BOTTOM_VALUE;
			SIMUL.CS_TOP_VALUE = (float)TOP_VALUE;
			SET_WAIT_CURSOR();
			ACTUALIZE_COLOR_SCALE();
			NETWORK.FLAG.REPRESENTATION = MATRIX_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
		
		case k_Toggle_Linear_Representation :
		  {
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Linear_Representation]) )
		      {
			TOP_VALUE = -30;
			BOTTOM_VALUE = -90;
			SIMUL.CS_BOTTOM_VALUE = (float)BOTTOM_VALUE;
			SIMUL.CS_TOP_VALUE = (float)TOP_VALUE;
			SET_WAIT_CURSOR();
			ACTUALIZE_COLOR_SCALE();
			NETWORK.FLAG.REPRESENTATION = LINEAR_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
		case k_Toggle_Cluster_Representation :
		  {
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Cluster_Representation]))
		      {
			SET_WAIT_CURSOR();
			NETWORK.FLAG.REPRESENTATION = CLUSTER_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
		case k_Toggle_Anatomic_Represent :
		  {
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Anatomic_Represent]))
		      {
			SET_WAIT_CURSOR();
			NETWORK.FLAG.REPRESENTATION = ANATOMIC_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
		
		case k_Toggle_Global_Activity :
		  {
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Global_Activity]) )
		      {
			SET_WAIT_CURSOR();
			NETWORK.FLAG.REPRESENTATION = GA_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
		
		case k_Toggle_Pmb_Representation :
		  {
		    
		    if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation]) )
		      {
			TOP_VALUE = 90;
			BOTTOM_VALUE = -90;
			SIMUL.CS_BOTTOM_VALUE = (float)BOTTOM_VALUE;
			SIMUL.CS_TOP_VALUE = (float)TOP_VALUE;
			SET_WAIT_CURSOR();
			ACTUALIZE_COLOR_SCALE();
			NETWORK.FLAG.REPRESENTATION = PMB_REPRESENTATION;
			UPDATE_REPRESENTATION();
			UNSET_WAIT_CURSOR();
		      }
		    break;
		  }
	}
}


/*
=============================================================================
Name and dates: SIMUL_ACTION
---------------
	Creation : 20-04-93
	Last modification : 20-04-93

Functionnal description :
-------------------------
	This callback call the function corresponding of the given tag. This
	tag must be one of the constants declared in value section of UILs and
	defined in global_def_visu.h.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	w		Widget		Widget id of the caller
			tag		int *		The wanted action
			reason		XtPointer	The reason of the call
							-back. This could be too
							the address of the
							callback structure.

Explicits outputs :	None

Implicits inputs  :	CMAP_ID		Colormap	The colormap
			NETWORK		NETWORK_DEF	the definition of the network
			SIMUL	   SIMUL_PARAMETERS	The definition of the simulation
							
Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network
			SIMUL	   SIMUL_PARAMETERS	The definition of the simulation

Sides effects :		None
---------------

=============================================================================
*/

void SIMUL_ACTION(Widget w, XtPointer TAG, XtPointer reason)
{
	char *INDEX_STRING, *CHAR;
	static XtWorkProcId WORK_PROC;
	int STATE;
	int i, VALUE;
        char name_image[256];
	char buffer[256];
	char STRING[100];
	
	switch((int)TAG)
	{
/*
-------------------------------------------------------------------------------------------
run forward one step of the simulation
-------------------------------------------------------------------------------------------
*/
		case k_Button_Play_Step_Main :
			/*sets wait cursor*/
			SET_WAIT_CURSOR();
			/*refresh display if we have stopped*/
			if(SIMUL.FLAG.SIMUL_STATE == STOPPED)
			{
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
			}
			/*updates simulation state*/
			SIMUL.FLAG.SIMUL_STATE = PAUSED;
			/*display neurons values*/
			STATE = FOREWARD;
			if(RUN_SIMUL((XtPointer)&STATE))
			{
				/*if we are at the end of the file*/
				XBell(DISPLAY_VISU, 100);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], False);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], False);
			}
			else
			{
				XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], True);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], True);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
			}
			/*unset wait cursor*/

			if(NETWORK.FLAG.EXIST)
			  {
			    XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
			    R_DRAW();
			  }
			UNSET_WAIT_CURSOR();

			break;

/*
-------------------------------------------------------------------------------------------
activate the goto index button (pas de break, c'est normal)
-------------------------------------------------------------------------------------------
*/
		case k_Text_Counter_Main :
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
/*
-------------------------------------------------------------------------------------------
run simulation to the specified index 
-------------------------------------------------------------------------------------------
*/
		case k_Button_Goto_Index_Main :
			/*sets wait cursor*/
			SET_WAIT_CURSOR();
			/*get back the index string*/
			INDEX_STRING = XmTextFieldGetString(WIDGET_ARRAY[k_Text_Counter_Main]);
			/*Verify validity of the input*/
			CHAR = INDEX_STRING;

			VALUE = True;
			do
			{
				if(!isdigit(*CHAR))
				{
					VALUE = False;
					/*Unset wait cursor*/
					UNSET_WAIT_CURSOR();
					break;
				}
			}while(*(++CHAR));
			/*If the value is not valid */
   			if(!VALUE) 
			{
				XBell(DISPLAY_VISU, 100);
				sprintf(STRING, "%d", SIMUL.ITERATION);
				XmTextFieldSetString(WIDGET_ARRAY[k_Text_Counter_Main], STRING);
			}
			/*valid value*/
    			else 			
			{
				/*get back the index value*/
				SIMUL.ITERATION = atoi(INDEX_STRING);
				if(SIMUL.ITERATION < 1) SIMUL.ITERATION = 1;
				/*reset simulation*/
				rewind(SIMUL.INPUT_PMB_FILE);
				/*free special memory*/
				R_FREE(SPECIAL);
				/*reinitializes pmb buffer*/
				if(SIMUL.POT != Null)
				{
					free(SIMUL.POT);
					SIMUL.POT = (NEURON_PMB *)calloc((size_t)NETWORK.NB_NEURONS, sizeof(NEURON_PMB));
				}
				/*refresh display */
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
				/*updates simulation state*/
				SIMUL.FLAG.SIMUL_STATE= PAUSED;
				/*display neurons values*/
				STATE = GOTO_INDEX;
				if(RUN_SIMUL((XtPointer)&STATE))
				{
					/*if we are at the end of the file*/
					XBell(DISPLAY_VISU, 100);
					XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
					XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], False);
					XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], False);
				}
				else
				{
					XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], True);
					XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], True);
					XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
				}
			}
			if(NETWORK.FLAG.EXIST)
			  {
			    XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
			    R_DRAW();
			  }
			/*Unset wait cursor*/
			UNSET_WAIT_CURSOR();
			break;
/*
-------------------------------------------------------------------------------------------
run the simulation until the user press stop or pause buttons, or until 
we reach the end of the file
-------------------------------------------------------------------------------------------
*/
		case k_Button_Play_Main :
			/*set wait cursor*/
			SET_WAIT_CURSOR();
			XtSetSensitive(WIDGET_ARRAY[k_Button_Stop_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Pause_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Change_Top_Value], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Refresh_Screen], False);
			XtSetSensitive(WIDGET_ARRAY[k_Toggle_Show_Details], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Time_Step], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Print], False);

			/*refresh display if we have stopped*/
			if(SIMUL.FLAG.SIMUL_STATE == STOPPED)
			{
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
				/* now we can print the result */
				XtSetSensitive(WIDGET_ARRAY[k_Button_Print], True);
			}
			/*updates simulation state*/
			SIMUL.FLAG.SIMUL_STATE = RUNNING;
			STATE = FOREWARD;
			if(SIMUL.TIME_STEP != 0)
				ACTIVATE_TIMER_SIMUL(&STATE);
			else
				WORK_PROC = XtAppAddWorkProc(APP_CONTEXT, RUN_SIMUL, &STATE);

			if(NETWORK.FLAG.EXIST)
			  {
			    XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
			    R_DRAW();
			    if (NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION)
			    {
			      if (flag_movie && !(SIMUL.ITERATION % Inter_Images))
			      {
				if (SIMUL.ITERATION >= Start_Images && SIMUL.ITERATION <= Stop_Images) 
				{
				  num_image++;
				  sprintf(name_image,"%s_anim_%d.ps",generic_name,num_image);
#if DEBUG
				  printf("A %d: %d: %s\n",SIMUL.ITERATION,Inter_Images, name_image);
#endif
				/*
				  ps_no_banner();
				*/
				  ps_iniplot(name_image,21.);
				  PRINT_MATRIX(1,21.);
				  ps_endplot(name_image,'f');
				/*
				  ps_banner();
				  
				*/
				  sprintf(buffer,"%s -sDEVICE=png256 -sOutputFile=%s_anim_%d.png -q -sPAPERSIZE=%s %s&", 
					  GHOSTSCRIPT, generic_name, num_image, PAPERSIZE, name_image);
				  system(buffer);
				}
			      }
			    }
			  }
			break;
/*
-------------------------------------------------------------------------------------------
stop the simulation and reinitializes it
-------------------------------------------------------------------------------------------
*/
		case k_Button_Stop_Main :
			STATE = STOPPED;
			if(SIMUL.TIME_STEP != 0)
				ACTIVATE_TIMER_SIMUL(&STATE);
			else
				XtRemoveWorkProc(WORK_PROC);

			/*unset the wait cursor which has been set in play*/
			UNSET_WAIT_CURSOR();

			SIMUL.ITERATION = 0;
			rewind(SIMUL.INPUT_PMB_FILE);
			/*free special memory*/
			R_FREE(SPECIAL);
			/*reinitializes pmb buffer*/
			free(SIMUL.POT);
			SIMUL.POT = (NEURON_PMB *)calloc((size_t)NETWORK.NB_NEURONS, sizeof(NEURON_PMB));
			/*updates simulation state*/
			SIMUL.FLAG.SIMUL_STATE = STOPPED;
			XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Change_Top_Value], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Refresh_Screen], True);
			XtSetSensitive(WIDGET_ARRAY[k_Toggle_Show_Details], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Pause_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Time_Step], True);

			break;
/*
-------------------------------------------------------------------------------------------
pause the simulation
-------------------------------------------------------------------------------------------
*/
		case k_Button_Pause_Main :
			STATE = PAUSED;
			if(SIMUL.TIME_STEP != 0)
				ACTIVATE_TIMER_SIMUL(&STATE);
			else
				XtRemoveWorkProc(WORK_PROC);
			/*updates simulation state*/
			SIMUL.FLAG.SIMUL_STATE = PAUSED;
			XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Change_Top_Value], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Refresh_Screen], True);
			XtSetSensitive(WIDGET_ARRAY[k_Toggle_Show_Details], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Time_Step], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Print], True);
			/*unset the wait cursor which has been set in play*/

			if(NETWORK.FLAG.EXIST)
			{
			  XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
			  R_DRAW();
			}
			UNSET_WAIT_CURSOR();

			break;
/*
-------------------------------------------------------------------------------------------
close the simulation
-------------------------------------------------------------------------------------------
*/
		case k_Button_Close_Simulation :
			/*updates simulation state*/
			SIMUL.FLAG.SIMUL_STATE = NO_SIMULATION;
			/*refresh display*/

			SIMUL.ITERATION = 0;
			fclose(SIMUL.INPUT_PMB_FILE);

			if(NETWORK.FLAG.EXIST)
			{
				SET_WAIT_CURSOR();
				XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
				R_DRAW();
				UNSET_WAIT_CURSOR();
			}


			NETWORK.FLAG.SELECTION = False;

			/*free special memory*/
			R_FREE(SPECIAL);
			/*free pmb buffer*/
			if(SIMUL.POT != Null) 
				free(SIMUL.POT);
			/*updates interface*/
			XtSetSensitive(WIDGET_ARRAY[k_Button_Stop_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Pause_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Text_Counter_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Label_Counter_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Text_Spikes_Count], False);
			XtSetSensitive(WIDGET_ARRAY[k_Label_Spikes_Count], False);
			XtSetSensitive(WIDGET_ARRAY[k_Label_Top_Value], False);
			XtSetSensitive(WIDGET_ARRAY[k_Label_Bottom_Value], False);
			XtSetSensitive(WIDGET_ARRAY[k_Label_Middle_Value], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Change_Top_Value], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Open_File], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Exit], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Time_Step], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Print], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Membr_Pot], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Select_Neuron], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Unselect_Neuron], False);
	}
}
