/*
=============================================================================
	G A _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- REALIZE_GA :  This function allocs memory and initializes the
			  global activity representation.
	- COMPUTE_GEOMETRY_GA : This function computes geometricals parameters
				    for clusters and details of the global activity 
				    representation.
	- DRAW_GA : This function draws the global activity representation.
	- DRAW_NEURONS_VALUES_GA : This function draws global activity of each cluster
				       at the current iteration.
	- PRINT_GA : this function prints the global activity representation.
	- FREE_GA : This function free the global activity representation.
	- INFO_GA : This function construct the information string for the 
		    global activity representation.
=============================================================================
*/

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */
#include <X11/X.h>

#include "global_def_visu.h"
#include <plot_ps.h>

/*forward declarations*/
extern void COMPUTE_GEOMETRY_GA();

/*locals constants*/
#define GRAPH_HEIGHT 265	/*size in pixels of a graphic*/
#define UPPER_OFFSET 10
#define LOWER_OFFSET 55
#define LEFT_OFFSET 40
#define RIGHT_OFFSET 20
#define Y_AXIS_LENGHT (GRAPH_HEIGHT-UPPER_OFFSET-LOWER_OFFSET)
#define X_AXIS_LENGHT (NETWORK.GA->WIN_WIDTH-LEFT_OFFSET-RIGHT_OFFSET)
#define REFERENCE_STEP ((float)SIMUL.NB_ITERATION / 14.)
#define Y_NB_SCALE 11
#define X_NB_SCALE 14



/*
=============================================================================
Name and dates: REALIZE_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function allocs memory and initializes the global activity representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	<return>	int		False => the representation can't be realized

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
int REALIZE_GA()
{
	Widget CLIP_WIN_ID;
	Arg arglist[2];
	int n, CLIP_WIN_SIZE;
	int SIZE_OF_SCREEN;
	
	/*allocs memory for the representation*/
	if((NETWORK.GA = (GA_GEOMETRY *)calloc(sizeof(GA_GEOMETRY), 1)) == Null)
		return(False);

	/*computes and sets window size*/
	SIZE_OF_SCREEN = DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));

#if 0
	/*actualize the size of the main window*/
	n=0;
	XtSetArg(arglist[n], XmNwidth, SIZE_OF_SCREEN); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);
#endif

#if 0

	/*get back the size of the clipping window*/
	n=0;
	XtSetArg(arglist[n], XmNclipWindow, &CLIP_WIN_ID); n++;
	XtGetValues(WIDGET_ARRAY[k_Scrolled_Window_Main], arglist, n);
	n=0;
	XtSetArg(arglist[n], XmNwidth, &CLIP_WIN_SIZE); n++;
	XtGetValues(CLIP_WIN_ID, arglist, n);
#endif

#if 0

	CLIP_WIN_SIZE = SIZE_OF_SCREEN - 245 - LEFT_OFFSET - RIGHT_OFFSET;   /*245 is corresponding to the window decoration*/
#endif
	CLIP_WIN_SIZE = 0;
	n = 0;
	XtSetArg(arglist[n], XmNwidth, &CLIP_WIN_SIZE); n++;
	XtGetValues(WIDGET_ARRAY[k_Scrolled_Window_Main], arglist, n);


	NETWORK.GA->GRAPH_STEP = (float)((float)CLIP_WIN_SIZE - (float)LEFT_OFFSET - (float)RIGHT_OFFSET)
	                          /(float)SIMUL.NB_ITERATION;

/*
	if(NETWORK.GA->GRAPH_STEP < 1.)
		NETWORK.GA->GRAPH_STEP = 1.;
*/
	/*adjust the number of displayed iteration to display the maximum of iteration*/

	Taille_Dessin = CLIP_WIN_SIZE/NETWORK.GA->GRAPH_STEP;

	NETWORK.GA->WIN_WIDTH = Taille_Dessin * NETWORK.GA->GRAPH_STEP + LEFT_OFFSET + RIGHT_OFFSET;

	NETWORK.GA->WIN_HEIGHT = GRAPH_HEIGHT * NETWORK.NB_CLUSTERS;

	/*Actualize the drawing area size*/
	n=0;
	XtSetArg(arglist[n], XmNheight, NETWORK.GA->WIN_HEIGHT); n++;
	XtSetArg(arglist[n], XmNwidth, NETWORK.GA->WIN_WIDTH); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Drawing_Area_Main], arglist, n);
	

	/*allocs memory to keep values*/
	if (NETWORK.GA->VALUES != NULL)
	  free(NETWORK.GA->VALUES);
	if((NETWORK.GA->VALUES = (XPoint *)calloc(sizeof(XPoint), 
						 (SIMUL.NB_ITERATION + 1)*NETWORK.NB_CLUSTERS)) == Null)
		return(False);
	/*allocs memory to keep coordinates*/
	if (NETWORK.GA->GRAPH_AXIS != NULL)
	  free(NETWORK.GA->GRAPH_AXIS);
	if((NETWORK.GA->GRAPH_AXIS = (XSegment *)calloc(sizeof(XSegment), (size_t)NETWORK.NB_CLUSTERS * 2)) == Null)
		return(False);
	if (NETWORK.GA->SCALE != NULL)
	  free(NETWORK.GA->SCALE);
	if((NETWORK.GA->SCALE = (XSegment *)calloc(sizeof(XSegment), Y_NB_SCALE + X_NB_SCALE)) == Null)
		return(False);

	/*creates and initializes GC for graphics*/
	NETWORK.GA->GA_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	XSetForeground(DISPLAY_VISU, NETWORK.GA->GA_GRAPH_CTX, LUT[C_BLACK]);

	/*computes geometry for graphics and details*/
	COMPUTE_GEOMETRY_GA();

	return(True);
}


/*
=============================================================================
Name and dates: COMPUTE_GEOMETRY_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function computes geometricals parameters for graphics and details 
	of the global activity representation.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
void COMPUTE_GEOMETRY_GA()
{
	int i, j;

	/*computes axis coordinates*/
	for(i=0; i<2*NETWORK.NB_CLUSTERS; i+=2)
	{
		(NETWORK.GA->GRAPH_AXIS + i)->x1 = LEFT_OFFSET;
		(NETWORK.GA->GRAPH_AXIS + i)->x2 = LEFT_OFFSET;
		(NETWORK.GA->GRAPH_AXIS + i)->y1 = UPPER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.GA->GRAPH_AXIS + i)->y2 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.GA->GRAPH_AXIS + i + 1)->x1 = LEFT_OFFSET;
		(NETWORK.GA->GRAPH_AXIS + i + 1)->x2 = NETWORK.GA->WIN_WIDTH - RIGHT_OFFSET;
		(NETWORK.GA->GRAPH_AXIS + i + 1)->y1 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.GA->GRAPH_AXIS + i + 1)->y2 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
	}
	/*computes scale coordinates*/
	/*Y axis*/
	for(i=0; i< Y_NB_SCALE; i++)
	{
		(NETWORK.GA->SCALE + i)->x1 = LEFT_OFFSET - 7;
		(NETWORK.GA->SCALE + i)->x2 = LEFT_OFFSET;
	}
	/*X axis*/
	for(i=Y_NB_SCALE; i<=(Y_NB_SCALE + X_NB_SCALE); i++)
		{
		  (NETWORK.GA->SCALE + i)->x1 = 
		    (NETWORK.GA->SCALE + i)->x2 = 
		      (int)( (float)LEFT_OFFSET + 
			     ((float)i-(float)Y_NB_SCALE) * 
			      (float)REFERENCE_STEP * 
			      (float)NETWORK.GA->GRAPH_STEP );
		}
	/*computes X and initials Y coordinates for values array*/
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
		for(j=0; j<=SIMUL.NB_ITERATION; j++)
		{
		  (NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = (int)(
		    (float)j * (float)NETWORK.GA->GRAPH_STEP + (float)LEFT_OFFSET );
		  (NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->y = (int)(
		    (float)(i+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET);
		}
}


/*
=============================================================================
Name and dates: DRAW_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws the global activity representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_GA()
{
	XmString STRING;
	EXPOSED_REGION REGION;
	int FIRST_CLUSTER, LAST_CLUSTER;
	int i, j;
	char STR[80];


	/*gets back the visible part of the darwing area*/
	GET_EXPOSED_REGION(&REGION);

	/*computes the first and the last cluster to draw*/
	FIRST_CLUSTER = (int)floor((double)REGION.Y/(double)GRAPH_HEIGHT);
	LAST_CLUSTER = (int)floor((double)(REGION.Y + REGION.HEIGHT)/(double)GRAPH_HEIGHT);	
	if(LAST_CLUSTER == NETWORK.NB_CLUSTERS) 
		LAST_CLUSTER--;

	/*draws axis*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.GA->GA_GRAPH_CTX, 3, LineSolid, CapButt,
			   JoinMiter);
	XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
		      NETWORK.GA->GRAPH_AXIS + (FIRST_CLUSTER * 2),
		      (LAST_CLUSTER - FIRST_CLUSTER + 1) * 2);

	/*draws details*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.GA->GA_GRAPH_CTX, 1, LineSolid, CapButt,
			   JoinMiter);
	for(i=FIRST_CLUSTER; i<=LAST_CLUSTER; i++)
	{
		/*draw scales*/
		/*first segments for Y axis and others for X axis*/
		/*Y axis*/
		for(j=0; j< Y_NB_SCALE; j++)
			(NETWORK.GA->SCALE + j)->y1 = (NETWORK.GA->SCALE + j)->y2 =
				(i+1)*GRAPH_HEIGHT - LOWER_OFFSET - j*Y_AXIS_LENGHT/10;
		/*X axis*/
		for(j=Y_NB_SCALE; j<Y_NB_SCALE + X_NB_SCALE; j++)
		{
			(NETWORK.GA->SCALE + j)->y1 = (i+1)*GRAPH_HEIGHT - LOWER_OFFSET;
			(NETWORK.GA->SCALE + j)->y2 = (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 7;
		}
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
		      	      NETWORK.GA->SCALE, Y_NB_SCALE + X_NB_SCALE);

		/*draw Y axis legend and cluster number*/
		sprintf(STR, "Activity in %% for cluster No %d", i+1);
		XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			    LEFT_OFFSET, i*GRAPH_HEIGHT + UPPER_OFFSET - 2, STR, strlen(STR));
		for(j=0; j<=10; j++)
		{
			sprintf(STR, "%3d", j*10);
			XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			    	    LEFT_OFFSET - 25, (i+1)*GRAPH_HEIGHT - LOWER_OFFSET 
				    - j*Y_AXIS_LENGHT/10, STR, strlen(STR));
		}
		
		/*draw X axis legend*/
		strcpy(STR, "Time in ms\0");
		XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			    LEFT_OFFSET, (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 30, STR, strlen(STR));
		
	}
	
	R_DRAW_NEURONS_VALUES(SAME_STEP);

}


/*
=============================================================================
Name and dates: DRAW_NEURONS_VALUES_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws global activity of each cluster at the current iteration.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	STATE		int		FOREWARD, GOTO_INEDX, or SAME_STEP.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display
			SIMUL	   SIMUL_PARAMETERS	Definition of the simulation
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_NEURONS_VALUES_GA(int STATE)
{
	Arg arglist[1];
	float PERCENT;
	EXPOSED_REGION REGION;
	static int PREV_MAX_IT;
	int NB_POINTS;
	int LAST_NEURON, FIRST_NEURON;
	int FIRST_CLUSTER, LAST_CLUSTER;
	int MIN_IT, MAX_IT;
	int i, j, n;
	float step;
	char STR[10];

	/*computes new values if needed*/
	if(SIMUL.FLAG.SIMUL_STATE != NO_SIMULATION && STATE != SAME_STEP)
	{
		/*computes new value for each cluster*/
		FIRST_NEURON = 0;
		for(i=0; i<NETWORK.NB_CLUSTERS; i++)
		{
			/*translates previous iterations if needed*/
			if(SIMUL.ITERATION > SIMUL.NB_ITERATION || STATE == GOTO_INDEX)
			{
				memmove(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1),
					NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + 1,
					SIMUL.NB_ITERATION * sizeof(XPoint)/sizeof(char));
				/*restore x coordinate*/
				for(j=0; j<=SIMUL.NB_ITERATION; j++)
					(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = j * NETWORK.GA->GRAPH_STEP + LEFT_OFFSET;
			}
			/*sum of spikes for the current cluster*/
			PERCENT = 0;
			LAST_NEURON = FIRST_NEURON + *(NETWORK.NB_NEURONS_PER_CLUSTER + i);
			for(j=FIRST_NEURON; j<LAST_NEURON; j++)
				if((SIMUL.POT + j)->EVENT == SPIKE)
					PERCENT += 1.;
			PERCENT = PERCENT/(float)(*(NETWORK.NB_NEURONS_PER_CLUSTER + i));
			/*next cluster*/
			FIRST_NEURON = LAST_NEURON;
		
			/*computes new point for each cluster*/
			if(SIMUL.ITERATION > SIMUL.NB_ITERATION)
				(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.NB_ITERATION)->y = 
					(i+1)*GRAPH_HEIGHT - Y_AXIS_LENGHT*PERCENT - LOWER_OFFSET;
			else
				(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.ITERATION)->y = 
					(i+1)*GRAPH_HEIGHT - Y_AXIS_LENGHT*PERCENT - LOWER_OFFSET;
		}
	}
	
	/*computes number of points to draw*/
	if(SIMUL.ITERATION > SIMUL.NB_ITERATION)
		NB_POINTS = SIMUL.NB_ITERATION + 1;
	else
		NB_POINTS = SIMUL.ITERATION + 1;

	/*gets back the visible part of the darwing area*/
	GET_EXPOSED_REGION(&REGION);

	/*resize the window if needed*/
	if(REGION.WIDTH < NETWORK.GA->WIN_WIDTH)
	{
		n=0;
		XtSetArg(arglist[n], XmNwidth, (DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))-20)); n++;
		XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);
	}

	/*computes the first and the last cluster to draw*/
	FIRST_CLUSTER = (int)floor((double)REGION.Y/(double)GRAPH_HEIGHT);
	LAST_CLUSTER = (int)floor((double)(REGION.Y + REGION.HEIGHT)/(double)GRAPH_HEIGHT);
	if(LAST_CLUSTER == NETWORK.NB_CLUSTERS) 
		LAST_CLUSTER--;

	/*computes max and min iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	MAX_IT = MIN_IT + SIMUL.NB_ITERATION;
	
	/*clean and redraw time scale and graphic area*/
	for(i=FIRST_CLUSTER; i<=LAST_CLUSTER; i++)
	{
		/*clear previous time scale if it has changed */
		if(PREV_MAX_IT != MAX_IT)

			XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, 
				   (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 7,
				   NETWORK.GA->WIN_WIDTH, 15, False);
		
		/*draw values*/
		if(STATE == FOREWARD && PREV_MAX_IT != MAX_IT)
		{
#if 0
			XCopyArea(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.WINDOW_ID,
				  NETWORK.GA->GA_GRAPH_CTX, LEFT_OFFSET + NETWORK.GA->GRAPH_STEP, 
				  i*GRAPH_HEIGHT + UPPER_OFFSET, NETWORK.GA->WIN_WIDTH, 
				  Y_AXIS_LENGHT - 1, LEFT_OFFSET, i*GRAPH_HEIGHT + UPPER_OFFSET);
			XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			   	   NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.NB_ITERATION - 1, 2, CoordModeOrigin);
#endif
			XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, LEFT_OFFSET + 2,
				   i*GRAPH_HEIGHT + UPPER_OFFSET,
				   NETWORK.GA->WIN_WIDTH, Y_AXIS_LENGHT - 1, False);

			XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			   	NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1),
			   	NB_POINTS, CoordModeOrigin);

		}
		else if(STATE == FOREWARD && PREV_MAX_IT == MAX_IT)
			XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			   	   NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.ITERATION - 1, 2, CoordModeOrigin);
		else if(STATE == SAME_STEP)
		{

			XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, LEFT_OFFSET + 2,
				   i*GRAPH_HEIGHT + UPPER_OFFSET,
				   NETWORK.GA->WIN_WIDTH, Y_AXIS_LENGHT - 1, False);

			XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
			   	NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1),
			   	NB_POINTS, CoordModeOrigin);
		}

		/*draw references*/
		for(step=MIN_IT; step<=MAX_IT; step+=REFERENCE_STEP)
		{
		  sprintf(STR, "%.1f", step);
		  XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.GA->GA_GRAPH_CTX, 
		              (int)( (float)LEFT_OFFSET + ((float)step-(float)MIN_IT)*(float)NETWORK.GA->GRAPH_STEP - 5.), 
			      (int)( ((float)i+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET + 15. ), STR, strlen(STR));
		}
	}

	/*keep for the next call*/
	PREV_MAX_IT = MAX_IT;

}


/*
=============================================================================
Name and dates: PRINT_GA
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function print the global activity representation in its actual
	state if there is a simulation running. This function use the
	PLOT_PS library.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	DRAW_ALL	int		Not used. We always draw the 
							entire network.
			PAPER_WIDTH	float		The width of the sheet

Explicits outputs :     None

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network
			CMAP_ID		Colormap	The colormap
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

void PRINT_GA(int DRAW_ALL, float PAPER_WIDTH)
{
	float MAX_HEIGHT = 25.;
	float SCALE;
	float GRAPHIC_HEIGHT;

	float step;
	EXPOSED_REGION REGION;
	int i, j, IT,k,page;
	char STRING[80];


	if(!SIMUL.ITERATION)
	  return;

	if(Coord_X != NULL)
		{
		  FREE_MEMORY(Coord_X);
		}

	if((Coord_X = (float *)calloc(SIMUL.ITERATION + 1, sizeof(float))) == NULL)
	  {
	    printf("Synapse structure allocation impossible\nTry later\n");
	    return;
	  } 	
	if(Coord_Y != NULL)
		{
		  FREE_MEMORY(Coord_Y);
		}

	if((Coord_Y = (float *)calloc(SIMUL.ITERATION + 1, sizeof(float))) == NULL)
	  {
	    printf("Synapse structure allocation impossible\nTry later\n");
	    return;
	  } 	


	/*computes scale for drawing.*/
	/*one user unit equal SCALE centimeters*/
	SCALE = (PAPER_WIDTH - 2.)/ (float)(NETWORK.GA->WIN_WIDTH); 
	/*computes height of a graphic*/
	/*we considere that the sheet drawable area is maximum 20. centimeters heigh*/
	/*and we draw 3 clusters maximum per sheet*/
	GRAPHIC_HEIGHT = MAX_HEIGHT/3./SCALE;

	/*if drawing is not possible*/
	if(SCALE < 0.008)
	{
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 1., 0., 0.);
		ps_writeah(0., 5., "XNBC : Visualization Printing Error", 0.7);
		ps_writeah(0., 4., NETWORK.NAME, 0.5);
		ps_writeah(0., 3., "is too large to be printed", 0.5);
	}
	/*drawing is possible*/
	else
	{
		/*set scale*/
		ps_echel(SCALE, SCALE, 0., 0., 1., 0.);
		/*draw comments for the first sheet*/
		ps_cms();
		ps_origin(0., MAX_HEIGHT + 0.2, 0., 0.);
		/*draw values*/
		sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", 
			SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
		ps_writeah(0., 0.2, STRING, 0.4);
		sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
		ps_writeah(0., 0.8, STRING, 0.4);
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., -MAX_HEIGHT - 0.2, 0., 0.);
		/*use user units*/
		ps_user();
		/*cluster loop*/

		k = 0;     /* ces deux variables sont utilise pour recaler */
		page = 0;  /* les dessin du fichier Ps pour les differentes page */


		for(i=0; i<NETWORK.NB_CLUSTERS; i++)
		{
		  k++;
		  if (k==4)
		    {
		      k=1;
		      page++;
		    }
			/* draw comments on each sheet*/
			if(i != 0 && i%3 == 0)
			{
				/*new page*/
				ps_newpage();
				ps_cms();
				ps_origin(0., MAX_HEIGHT + 0.2, 0., 0.);
				/*draw values*/
				sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", 
					SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
				ps_writeah(0., 0.2, STRING, 0.4);
				sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
				ps_writeah(0., 0.8, STRING, 0.4);
				ps_origin(0., -MAX_HEIGHT - 0.2, 0., 0.);
				ps_user();
			}

			/*draw axis*/ 
			ps_line_width(2.);
			Coord_Y[0] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			Coord_X[0] = LEFT_OFFSET;
			Coord_Y[1] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			Coord_X[1] = LEFT_OFFSET;
			Coord_Y[2] = Coord_Y[1];
			Coord_X[2] = LEFT_OFFSET + X_AXIS_LENGHT;
			/*X and Y axis*/
			ps_polyline(Coord_X, Coord_Y, 3, False, 0, 0.);
		
			/*draw ticked*/
			ps_line_width(1.);
			/*Y axis*/
			for(j=0; j<=10; j++)
			{
				Coord_X[0] = LEFT_OFFSET;
				Coord_Y[0] = MAX_HEIGHT/SCALE - 
				  ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET
				   - (float)Y_AXIS_LENGHT/10.*j)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				Coord_X[1] = LEFT_OFFSET - 7.;
				Coord_Y[1] = Coord_Y[0];
				ps_polyline(Coord_X, Coord_Y, 2, False, 0, 0.);
			}
			/*X axis*/
			for(step=0; step<SIMUL.NB_ITERATION; step+=REFERENCE_STEP)
			{
				Coord_X[0] = LEFT_OFFSET + step * NETWORK.GA->GRAPH_STEP;
				Coord_Y[0] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				Coord_X[1] = Coord_X[0];
				Coord_Y[1] = Coord_Y[0] - 5.0*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				ps_polyline(Coord_X, Coord_Y, 2, False, 0, 0.);
			}
			/*draw values*/
			for(j=0; j<SIMUL.NB_ITERATION; j++)
			{
				Coord_X[j] = LEFT_OFFSET + j * NETWORK.GA->GRAPH_STEP;
				Coord_Y[j] = (page + 1) * MAX_HEIGHT/SCALE - 
				       (NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION + 1) + j)->y * 
					GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			}
			ps_polyline(Coord_X, Coord_Y, SIMUL.NB_ITERATION, False, 0, 0.);

			/*draw legend and references*/
			ps_cms();
			/*Y legend*/
			sprintf(STRING, "Global activity for cluster %d in %%", i+1);
			ps_writeah((float)LEFT_OFFSET * SCALE, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE + 0.2, STRING, 0.3);
			/*X legend*/
			ps_writeah((float)LEFT_OFFSET * SCALE - 1.5, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE -0.4, "Time in", 0.3);
			ps_writeah((float)LEFT_OFFSET * SCALE - 1, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE - 0.7, "ms", 0.3);
			/*Y references*/
			for(j=0; j<=10; j++)
			{
				sprintf(STRING, "%3d", j*10);
				ps_writeah((float)(LEFT_OFFSET)*SCALE - 0.7,
					   MAX_HEIGHT - ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET - 
					   (float)Y_AXIS_LENGHT/10.*j) * 
				   	   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE, STRING, 0.3);
			}
			/*X references*/
			IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
			for(step=0; step<SIMUL.NB_ITERATION; step+=REFERENCE_STEP)
			{
				sprintf(STRING, "%d", IT + (int)step);
				ps_writeah((float)(LEFT_OFFSET + step*NETWORK.GA->GRAPH_STEP)*SCALE - 0.1,
					   MAX_HEIGHT - ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET + 5.) * 
				   	   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE - 0.3, STRING, 0.3);
			}

			ps_user();
		} /*end of cluster loop*/
	}/*end of drawing*/

}

/*
=============================================================================
Name and dates: FREE_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function free the global activity representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/

void FREE_GA(int ACTION)
{
	int n, i, j;

	/*free the entire representation*/
	if(NETWORK.GA != Null && ACTION == ALL)
	{
		FREE_GC(NETWORK.GA->GA_GRAPH_CTX);
		FREE_MEMORY(NETWORK.GA->VALUES);
		FREE_MEMORY(NETWORK.GA->GRAPH_AXIS);
		FREE_MEMORY(NETWORK.GA->SCALE);

		FREE_MEMORY(NETWORK.GA);
		NETWORK.GA = Null;
	}
	
	/*reinitializes values array*/
	if(NETWORK.GA != Null && ACTION == SPECIAL)
		/*computes X and initials Y coordinates for values array*/
		for(i=0; i<NETWORK.NB_CLUSTERS; i++)
			for(j=0; j<=SIMUL.NB_ITERATION; j++)
			{
				(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = j * NETWORK.GA->GRAPH_STEP + LEFT_OFFSET;
				(NETWORK.GA->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->y = (i+1)*GRAPH_HEIGHT - LOWER_OFFSET;
			}
}


/*
=============================================================================
Name and dates: INFO_GA
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function construct the information string for the global activity representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	X		short		X position of the pointer
			Y		short		Y position of the pointer

Explicits outputs :	STRING		char *		The information string
			INDEX		int *		Not Used

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			SIMUL		SIMUL_DEF	The definition of the simulation

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
/*static string locals to this file*/
static char STRING[256];

char *INFO_GA(short X, short Y, int *INDEX)
{	
	float VALUE;
	int CLUSTER, ITERATION;
	int i, TMP=0, MIN_IT;
	char POT_STR[80];
	char NAME[80];


	/*computes cluster number*/
	CLUSTER = Y/GRAPH_HEIGHT + 1;
	
	/*computes iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	ITERATION = MIN_IT + (X-LEFT_OFFSET)/NETWORK.GA->GRAPH_STEP;
	if(ITERATION < MIN_IT )
		ITERATION = MIN_IT;
	if(ITERATION > SIMUL.ITERATION)
		ITERATION = SIMUL.ITERATION;
	
	/*computes activity*/
	if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION || NETWORK.GA->VALUES == Null
	   || ITERATION > SIMUL.ITERATION)
		strcpy(POT_STR, ": Undefined");
	else
	{
		VALUE = ((float)CLUSTER * (float)GRAPH_HEIGHT
			 - (float)(NETWORK.GA->VALUES + (CLUSTER-1)*(SIMUL.NB_ITERATION+1)
			 + ITERATION - MIN_IT)->y
			 - (float)LOWER_OFFSET)/(float)Y_AXIS_LENGHT*100;
		sprintf(POT_STR, "= %3.2f%%", VALUE);
	}
	
	/*get cluster name*/
	if(NETWORK.INFO_CLUSTERS != Null)
		strcpy(NAME, NETWORK.INFO_CLUSTERS[CLUSTER-1].NAME);
	else
		sprintf(NAME, "CLUSTER_%d", CLUSTER);

	/*construct string*/
	sprintf(STRING, "TME : %d ms\nCLUSTER\n %s\n No %d\n with %d neurons\nC.G.A. %s",
	        ITERATION, NAME,
	        CLUSTER, *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1)), POT_STR);

	
	return(STRING);

}

/************************************************************
  SET_WAIT_CURSOR();
  NETWORK.FLAG.REPRESENTATION = PMB_REPRESENTATION;
  UPDATE_REPRESENTATION();
  UNSET_WAIT_CURSOR();
************************************************************/
