#define DEBUG 0
/*
=============================================================================
	I N I T I A L I Z E _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- INIT_VISUALIZATION : This function contents all the initialization
			       that Visualization needs at its beginning.

	- INIT_ALLOC_TAB_SOURCE: allocation of the 6 tab_source for the AFFICHE type window.		       
=============================================================================
*/

#include <stdlib.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <X11/X.h>
#include <X11/cursorfont.h>

#include "global_def_visu.h"
#include "wait_cursor_visu.h"
#include "electrode_cursor_visu.h"


/*
=============================================================================
Name and dates: INIT_VISUALIZATION
---------------
	Creation : 21-04-93
	Last modification : 21-04-93
	                    17-12-99
Functionnal description :
-------------------------
	This function contents all the initialization
	that Visualization needs at its beginning.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	DISPLAY_VISU	Display*	Application display
		    WAIT_CURSOR_BITS	WAIT_CURSOR_STRUCT
		    ELECTRODE_CURSOR_BITS	ELECTRODE_CURSOR_STRUCT
						Definition of the CURSORS
		TOPLEVEL_WIDGET_VISU	Widget
Implicits outputs :     WAIT_CURSOR	Cursor		Cursor when the
							application is working
			ELECTRODE_CURSOR	Cursor		Cursor when the
							application is working
			CMAP_ID		Colormap	The colormap
			NETWORK		NETWORK_DEF	Definition of the network
			SIMUL	   SIMUL_PARAMETERS	Definition of the simulation

Sides effects :		None
---------------

=============================================================================
*/


void INIT_VISUALIZATION()
{
	Visual *VISUAL;
	XGCValues GC_VALUES;
	Pixmap WAIT_CURSOR_PIXMAP,  WAIT_CURSOR_MASK; /*For the wait cursor*/
	Pixmap ELECTRODE_CURSOR_PIXMAP,  ELECTRODE_CURSOR_MASK; /*For the electrode cursor*/
	Arg arglist[1];
	XSetWindowAttributes COLOR_SCALE_ATTRIBUTES;
	XColor FOREGROUND, BACKGROUND;		      /*For the wait cursor*/
	XColor COLOR, COLOR_DEF, COLOR_TRUE;	      /*To alloc colors*/
	Colormap DEFAULT_CMAP_ID;		      /*the default colormap*/
	int i, n, status;
	

/*
--------------------------------------------------------------------------------
Create and initialize the WAIT_CURSOR
--------------------------------------------------------------------------------
*/
	BACKGROUND.red = 65535;	/*colors for cursor*/
	BACKGROUND.green = 65535;
	BACKGROUND.blue = 65535;
	FOREGROUND.red = 0;
	FOREGROUND.green = 0;
	FOREGROUND.blue = 0;
	
	WAIT_CURSOR_PIXMAP = XCreateBitmapFromData(DISPLAY_VISU,
			XtWindow(TOPLEVEL_WIDGET_VISU),
			WAIT_CURSOR_BITS.CURSOR_SHAPE, WAIT_CURSOR_WIDTH,
			WAIT_CURSOR_HEIGHT);
	WAIT_CURSOR_MASK = XCreateBitmapFromData(DISPLAY_VISU,
		        	XtWindow(TOPLEVEL_WIDGET_VISU),
			WAIT_CURSOR_BITS.CURSOR_MASK, WAIT_CURSOR_WIDTH,
			WAIT_CURSOR_HEIGHT);
	WAIT_CURSOR = XCreatePixmapCursor(DISPLAY_VISU, WAIT_CURSOR_PIXMAP,
				   WAIT_CURSOR_MASK, &FOREGROUND,
				   &BACKGROUND, WAIT_CURSOR_X_HOT,
				   WAIT_CURSOR_Y_HOT);

/*
--------------------------------------------------------------------------------
Create and initialize the ELECTRODE_CURSOR
--------------------------------------------------------------------------------
*/
	BACKGROUND.red = 0;	/*colors for cursor*/
	BACKGROUND.green = 0;
	BACKGROUND.blue = 0;
	FOREGROUND.red = 0;
	FOREGROUND.green = 0;
	FOREGROUND.blue = 0;
	
	ELECTRODE_CURSOR_PIXMAP = XCreateBitmapFromData(DISPLAY_VISU,
			XtWindow(TOPLEVEL_WIDGET_VISU),
			ELECTRODE_CURSOR_BITS.CURSOR_SHAPE, ELECTRODE_CURSOR_WIDTH,
			ELECTRODE_CURSOR_HEIGHT);
	ELECTRODE_CURSOR_MASK = XCreateBitmapFromData(DISPLAY_VISU,
		        	XtWindow(TOPLEVEL_WIDGET_VISU),
			ELECTRODE_CURSOR_BITS.CURSOR_MASK, ELECTRODE_CURSOR_WIDTH,
			ELECTRODE_CURSOR_HEIGHT);
	ELECTRODE_CURSOR = XCreatePixmapCursor(DISPLAY_VISU, ELECTRODE_CURSOR_PIXMAP,
				   ELECTRODE_CURSOR_MASK, &FOREGROUND,
				   &BACKGROUND, ELECTRODE_CURSOR_X_HOT,
				   ELECTRODE_CURSOR_Y_HOT);

/*
--------------------------------------------------------------------------------
Initializes the color scale window
--------------------------------------------------------------------------------
*/
	COLOR_SCALE_ATTRIBUTES.backing_store = Always;
	COLOR_SCALE_ATTRIBUTES.backing_planes = DefaultDepth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));
	XChangeWindowAttributes(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Drawing_Area_Legend_Main]),
				CWBackingStore | CWBackingPlanes, &COLOR_SCALE_ATTRIBUTES);

/*
--------------------------------------------------------------------------------
Get back the default colormap, create the private colormap and initialize colors
--------------------------------------------------------------------------------
*/

	/* creation de la palette de couleur	*/
	VISUAL = DefaultVisual(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));
	switch(VISUAL->class)
	{
		case GrayScale	:
		case DirectColor:
		case PseudoColor: COLORMAP_MODE = True; break;
		case StaticColor:
		case StaticGray :
		case TrueColor	: COLORMAP_MODE = False; break;
	} /* fin du switch */
#if DEBUG
	printf("COLORMAP_MODE = %d\nVisual class: %d\n",COLORMAP_MODE, VISUAL->class);
#endif

	DEFAULT_CMAP_ID = DefaultColormapOfScreen(DefaultScreenOfDisplay(DISPLAY_VISU));


	if(COLORMAP_MODE)
	{
		if(BlackPixel(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)) == 0)
		{
			C_BLACK = 0;
			C_WHITE = 1;
		}
		else
		{
			C_BLACK = 1;
			C_WHITE = 0;
		}
		CMAP_ID = XCreateColormap(DISPLAY_VISU, XtWindow(TOPLEVEL_WIDGET_VISU), 
						  DefaultVisual(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)),
						   AllocAll);
		/*Copy the default colormap in the private one*/
		for(i=0; i < CellsOfScreen(DefaultScreenOfDisplay(DISPLAY_VISU)); i++)
		{
			COLOR.pixel = i;
			XQueryColor(DISPLAY_VISU, DEFAULT_CMAP_ID, &COLOR);
			XStoreColor(DISPLAY_VISU, CMAP_ID, &COLOR);
		}

		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Black", C_BLACK, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "White", C_WHITE, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Violet", C_VIOLET, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Green", C_GREEN, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Red", C_RED, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Red", C_RED1, DoRed | DoGreen | DoBlue);
		XStoreNamedColor(DISPLAY_VISU, CMAP_ID, "Red", C_RED2, DoRed | DoGreen | DoBlue);

		XSetWindowColormap(DISPLAY_VISU, XtWindow(TOPLEVEL_WIDGET_VISU), CMAP_ID);

		for(i=0; i<256; i++)
			LUT[i] = i;
	}
	else
	{
		CMAP_ID = DEFAULT_CMAP_ID;
		
		C_BLACK = 0;
		C_WHITE = 1;
		
		LUT[C_BLACK] = BlackPixel(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));
		LUT[C_WHITE] = WhitePixel(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));
		LUT[C_GREEN] = 65280;
		LUT[C_RED] = 16711680;
		LUT[C_RED1] = 16711680;
		LUT[C_RED2] = 16711680;
	} 
/*
--------------------------------------------------------------------------------
Initializes background color of the drawing area
--------------------------------------------------------------------------------
*/
	n=0;
	XtSetArg(arglist[n], XmNbackground, LUT[C_WHITE]); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Drawing_Area_Main], arglist, n);

/*
--------------------------------------------------------------------------------
Initializes events handlers for the drawing area
--------------------------------------------------------------------------------
*/

	XtAddEventHandler(WIDGET_ARRAY[k_Window_Drawing_Area_Main], 
			  ButtonPressMask, 
			  False,
			  DISPLAY_RETICUL, 
			  &RETICUL_DATA);
	XtAddEventHandler(WIDGET_ARRAY[k_Window_Drawing_Area_Main], 
			  ButtonReleaseMask, 
			  False,
			  UNDISPLAY_RETICUL, 
			  &RETICUL_DATA);

	XtAddEventHandler(WIDGET_ARRAY[k_Window_Drawing_Area_Main], 
			  ButtonMotionMask, 
			  False,
			  TRACK_RETICUL, 
			  &RETICUL_DATA);

	
	

	/*the drawing area grab buttons when any button is pressed and the pointer
	is confined in the scrolled window*/
	XGrabButton(DISPLAY_VISU, AnyButton, AnyModifier, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]),
		    True, ButtonPressMask | ButtonMotionMask | ButtonReleaseMask,
		    GrabModeAsync, GrabModeAsync, XtWindow(WIDGET_ARRAY[k_Scrolled_Window_Main]),
		    XCreateFontCursor(DISPLAY_VISU, XC_crosshair));
	/*initializes GC for the reticul*/
	GC_VALUES.function = GXinvert;
	GC_VALUES.line_width = 3;
	RETICUL_DATA.GRAPH_CTX = XCreateGC(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]),
					   GCFunction | GCLineWidth, &GC_VALUES);
	
/*
--------------------------------------------------------------------------------
Initialize the Print Box
--------------------------------------------------------------------------------
*/
	INIT_PRINT_BOX();
	
/*
--------------------------------------------------------------------------------
Initialize the NETWORK definition
--------------------------------------------------------------------------------
*/
	/*Actions functions*/
	NETWORK.ACTION.REALIZE = Null;
	NETWORK.ACTION.DRAW = Null;
	NETWORK.ACTION.DRAW_NEURONS_VALUES = Null;
	NETWORK.ACTION.PRINT = Null;
	NETWORK.ACTION.FREE = Null;

	/*data*/
	NETWORK.NAME = Null;
	NETWORK.NB_NEURONS_PER_CLUSTER = Null;
	NETWORK.CONNEXION_MATRIX = Null;
	NETWORK.SYNAPTIC_MATRIX = Null;
	NETWORK.ANATOMY_MATRIX = Null;
	NETWORK.MATRIX = Null;
	NETWORK.SELECTED_NEURON = Null;
	NETWORK.WINDOW_ID = 0;
	NETWORK.NB_NEURONS = 0;
	NETWORK.NB_SELECTED_NEURON = 0;
	NETWORK.NB_CLUSTERS = 0;

	/*flags*/
	NETWORK.FLAG.SHOW_DETAILS = False;
	NETWORK.FLAG.EXIST = False;
	NETWORK.FLAG.SELECTION = False;
	NETWORK.FLAG.REPRESENTATION = LINEAR_REPRESENTATION;

/*
--------------------------------------------------------------------------------
Initialize the SIMUL definition
--------------------------------------------------------------------------------
*/
	/*data*/
	SIMUL.INPUT_PMB_FILE = Null;
	SIMUL.POT = Null;
	SIMUL.CS_BOTTOM_VALUE = -90.0;
	SIMUL.CS_TOP_VALUE = -30.0;
	SIMUL.CS_STEP = 0.0;
	SIMUL.ITERATION = 0;
	SIMUL.NB_ITERATION = 300;
	SIMUL.TIME_STEP = 1;
	SIMUL.SPIKE_COUNT = 0;

	/*flags*/
	SIMUL.FLAG.SIMUL_STATE = NO_SIMULATION;

} 

/* Pour la visualization du pot de membrane dans une fenetre separee */


void INIT_ALLOC_TAB_SOURCE(char *pmb_name)
{

/*
---------------------------------------------------------------------------------
Initialize the tab_source for the AFFICHE type window
---------------------------------------------------------------------------------
*/
	int       i, j, Nbr_Iteration;
	FILE      *INPUT_FILE;
	char      AFF_FILE_NAME[256], MESSAGE[500];
	char      STRING[81],string1[51],string2[31];

	
	if (!pmb_name[0]) 
	  return;

	for(i = 0 ; i <=strlen(pmb_name) ; i++ )
	  {
	    if ( pmb_name[i]=='.')
	      {
		if (pmb_name[i+1]!='.' && pmb_name[i+1]!='/')
		  {
		    break;
		  }
	      }
	  }

	/* copier de 0 au point puis ajouter l`extension pmb */

	pmb_name[i]='\0';

	sprintf(AFF_FILE_NAME, "%s.pmb", pmb_name);

#if 0	
	printf("AFF_FILE_NAME: %s\n",AFF_FILE_NAME);
#endif
	if((INPUT_FILE = fopen(AFF_FILE_NAME, "r")) == NULL)
	{
	    /* si on ne peut pas ouvrir un fichier */
	    sprintf(MESSAGE, "Unable to open\n%s\nVisualization of individual membrane potential impossible.",
		    AFF_FILE_NAME);
	    DISPLAY_MESSAGE(MESSAGE,ERROR);
	    return;
	}
	
	/* remplissage de la table */
	
	Nbr_Iteration = 0; /* i = l`iteration lue */
	while (fgets(STRING, 80, INPUT_FILE)!=NULL)
	  {
	    sscanf(STRING, "%s %s", string1, string2);
	    
	    if ( (strcmp (string2,"iteration"))==0 )
	      Nbr_Iteration++;
	  } /* fin du while */
	
	fclose(INPUT_FILE);
#if 0
	printf("allocation de %d en memoire\n",Nbr_Iteration+1);
#endif
	if(tab_source0)
	  {
	    free(tab_source0);
	  }
	if((tab_source0 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (0) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

	if(tab_source1)
	  {
	    free(tab_source1);
	  }
	if((tab_source1 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (1) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

	if(tab_source2)
	  {
	    free(tab_source2);
	  }
	if((tab_source2 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (2) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

	if(tab_source3)
	  {
	    free(tab_source3);
	  }
	if((tab_source3 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (3) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

	if(tab_source4)
	  {
	    free(tab_source4);
	  }
	if((tab_source4 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (4) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

	if(tab_source5)
	  {
	    free(tab_source5);
	  }
	if((tab_source5 =  (int * )calloc(( Nbr_Iteration + 1 ), sizeof(int ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Memory allocation (5) for membrane potential visualization impossible\nTry later\n", ERROR);
	    return;
	  } 	

  

}
