/*
=============================================================================
	L I N E A R _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- REALIZE_LINEAR :  This function allocs memory and initializes the
			  linear representation.
	- COMPUTE_GEOMETRY_LINEAR : This function computes geometricals parameters
				    for clusters and details of the linear 
				    representation.
	- DRAW_LINEAR : This function draws the linear representation.
	- DRAW_NEURONS_VALUES_LINEAR : This function draws neurons values
				       at the current iteration.
	- PRINT_LINEAR : this function prints the linear representation.
	- FREE_LINEAR : This function free the linear representation.
	- INFO_LINEAR : This function constructs the information string for the
			linear rpresentation.
=============================================================================
*/

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */
#include <X11/X.h>

#include "global_def_visu.h"
#include <plot_ps.h>

/*forward declarations*/
extern void COMPUTE_GEOMETRY_LINEAR();


/*
=============================================================================
Name and dates: REALIZE_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function allocs memory and initializes the linear representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	<return>	int		False => the representation can't be realized

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
0			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
int REALIZE_LINEAR()
{
	Widget CLIP_WIN_ID;
	Arg arglist[2];			/*not important stuff*/
	int n;
	unsigned int CLIP_WIN_SIZE;			
	int SIZE_OF_SCREEN;		/*To computes the neuron size*/

	/*allocates memory for the representation*/
	if((NETWORK.LINEAR = (LINEAR_GEOMETRY *)calloc(sizeof(LINEAR_GEOMETRY), 1)) == Null)
		return(False);

	/*allocates memory for geometry*/
	NETWORK.LINEAR->CLUSTER_GEOMETRY = (XSegment *)calloc(sizeof(XSegment), 
							      (size_t)NETWORK.NB_CLUSTERS + 1);

	/*Computes neuron and window height*/
	NETWORK.LINEAR->WIN_HEIGHT = NETWORK.NB_NEURONS * DEFAULT_NEURON_SIZE + 1; /*default size*/
	SIZE_OF_SCREEN = (DisplayHeight(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)) <=
			  DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))  ?
			  DisplayHeight(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)) :
			  DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))  );
	SIZE_OF_SCREEN -= 170; /*To count of the window decoration*/
	if(NETWORK.LINEAR->WIN_HEIGHT > SIZE_OF_SCREEN)
		NETWORK.LINEAR->NEURON_HEIGHT = DEFAULT_NEURON_SIZE;
	else
	{
		/*compute the neuron size*/
		NETWORK.LINEAR->NEURON_HEIGHT = (int)floor((double)SIZE_OF_SCREEN/
							 (double)NETWORK.NB_NEURONS);
		/*Recompute the size of the drawing*/
		NETWORK.LINEAR->WIN_HEIGHT = NETWORK.NB_NEURONS * NETWORK.LINEAR->NEURON_HEIGHT + 1;
	}


	/*computes neuron and window width*/
	SIZE_OF_SCREEN = DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));

#if 0	
	/*actualize the size of the main window */
	n=0;
	XtSetArg(arglist[n], XmNwidth, SIZE_OF_SCREEN); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);

	CLIP_WIN_SIZE = SIZE_OF_SCREEN - 245;   /* 245 is corresponding to the window decoration */
#endif
#if 0

	/*get back the size of the clipping window*/
	n=0;
	XtSetArg(arglist[n], XmNclipWindow, &CLIP_WIN_ID); n++;
	XtGetValues(WIDGET_ARRAY[k_Scrolled_Window_Main], arglist, n);
	n=0;
	XtSetArg(arglist[n], XmNwidth, &CLIP_WIN_SIZE); n++;
	XtGetValues(CLIP_WIN_ID, arglist, n);
#endif
	CLIP_WIN_SIZE = 0;
	n = 0;
	XtSetArg(arglist[n], XmNwidth, &CLIP_WIN_SIZE); n++;
	XtGetValues(WIDGET_ARRAY[k_Scrolled_Window_Main], arglist, n);

	/*we computes neuron width to display SIMUL.NB_ITERATION iterations*/

	NETWORK.LINEAR->NEURON_WIDTH = (int)((float)CLIP_WIN_SIZE/(float)SIMUL.NB_ITERATION);

	if(NETWORK.LINEAR->NEURON_WIDTH < 2)
		NETWORK.LINEAR->NEURON_WIDTH = 2;

	/*adjust the number of displayed iteration to display the maximum of iteration*/

	NETWORK.LINEAR->WIN_WIDTH = NETWORK.LINEAR->NEURON_WIDTH * SIMUL.NB_ITERATION;
#if 0

	printf("SIZE_OF_SCREEN = %d\n SIMUL.NB_ITERATION = %d\n CLIP_WIN_SIZE = %d\n NETWORK.LINEAR->NEURON_WIDTH = %d\nNETWORK.LINEAR->WIN_HEIGHT: %d\n"
	       ,SIZE_OF_SCREEN, SIMUL.NB_ITERATION, CLIP_WIN_SIZE, NETWORK.LINEAR->NEURON_WIDTH, NETWORK.LINEAR->WIN_HEIGHT);
#endif



/*Actualize the drawing area size*/
	n=0;
	XtSetArg(arglist[n], XmNheight, NETWORK.LINEAR->WIN_HEIGHT); n++;
	XtSetArg(arglist[n], XmNwidth, NETWORK.LINEAR->WIN_WIDTH); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Drawing_Area_Main], arglist, n);

	/*allocates memory for neurons colors array*/
	NETWORK.LINEAR->NEURONS_COLORS = (unsigned long *)calloc(sizeof(unsigned long),
					 (size_t)NETWORK.NB_NEURONS * SIMUL.NB_ITERATION);
	if(NETWORK.LINEAR->CLUSTER_GEOMETRY == Null ||
	   NETWORK.LINEAR->NEURONS_COLORS == Null)
		return(False);

	/*initialize neurons colors array*/
	/*first iteration*/
	for(n=0; n<NETWORK.NB_NEURONS; n++)
		*(NETWORK.LINEAR->NEURONS_COLORS + n)= C_GREEN;
	/*next iterations*/
	for(n=NETWORK.NB_NEURONS; n<NETWORK.NB_NEURONS*SIMUL.NB_ITERATION; n++)
		*(NETWORK.LINEAR->NEURONS_COLORS + n)= C_WHITE;

	/*Create GC for clusters*/
	NETWORK.LINEAR->CLUSTER_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*Create GC for neurons*/
	NETWORK.LINEAR->NEURON_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);

	/*initializes GC's*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.LINEAR->CLUSTER_GRAPH_CTX, 3, LineSolid, CapButt,
			   JoinMiter);
	XSetLineAttributes(DISPLAY_VISU, NETWORK.LINEAR->NEURON_GRAPH_CTX, 1, LineSolid, CapButt,
			   JoinMiter);
	XSetForeground(DISPLAY_VISU, NETWORK.LINEAR->CLUSTER_GRAPH_CTX, LUT[C_BLACK]);
	XSetForeground(DISPLAY_VISU, NETWORK.LINEAR->NEURON_GRAPH_CTX, LUT[C_GREEN]);

	/*computes geometry for clusters and details*/
	COMPUTE_GEOMETRY_LINEAR();

	return(True);
}


/*
=============================================================================
Name and dates: COMPUTE_GEOMETRY_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function computes geometricals parameters for clusters and details 
	of the linear representation.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
void COMPUTE_GEOMETRY_LINEAR()
{
	int i, X, Y;

	/*computes clusters geometry*/
	NETWORK.LINEAR->CLUSTER_GEOMETRY->y1 = NETWORK.LINEAR->CLUSTER_GEOMETRY->y2 = 1;
	(NETWORK.LINEAR->CLUSTER_GEOMETRY + NETWORK.NB_CLUSTERS)->y1 = 
	(NETWORK.LINEAR->CLUSTER_GEOMETRY + NETWORK.NB_CLUSTERS)->y2 = 
	NETWORK.LINEAR->WIN_HEIGHT - 2;
	Y=0;
	for(i=1; i<NETWORK.NB_CLUSTERS; i++)
	{
		Y = *(NETWORK.NB_NEURONS_PER_CLUSTER + i - 1) * NETWORK.LINEAR->NEURON_HEIGHT + Y;
		(NETWORK.LINEAR->CLUSTER_GEOMETRY + i)->y1 = 
		(NETWORK.LINEAR->CLUSTER_GEOMETRY + i)->y2 = Y;
	}
}


/*
=============================================================================
Name and dates: DRAW_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws the linear representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_LINEAR()
{
	XSegment *FIRST_CLUSTER;
	EXPOSED_REGION REGION;
	int i, NB_CLUSTER;

	/*gets back the visible part of the drawing area*/
	GET_EXPOSED_REGION(&REGION);

	/*draws neurons*/
	R_DRAW_NEURONS_VALUES(SAME_STEP);

	/*draw clusters*/
	/*find the first cluster to draw and count the number of cluster to draw*/
	FIRST_CLUSTER = Null;
	NB_CLUSTER = 0;
	for(i=0; i<=NETWORK.NB_CLUSTERS; i++)
	{
		if(FIRST_CLUSTER == Null && (NETWORK.LINEAR->CLUSTER_GEOMETRY + i)->y1 >= REGION.Y)
			FIRST_CLUSTER = NETWORK.LINEAR->CLUSTER_GEOMETRY + i;
		if(FIRST_CLUSTER != Null && (NETWORK.LINEAR->CLUSTER_GEOMETRY + i)->y1 <= REGION.Y + REGION.HEIGHT)
			NB_CLUSTER++;
	}
	/*initializes clusters*/
	for(i=0; i<NB_CLUSTER; i++)		
	{
		(FIRST_CLUSTER + i)->x1 = REGION.X;
		(FIRST_CLUSTER + i)->x2 = REGION.X + REGION.WIDTH;
	}
	if(NB_CLUSTER != 0)
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.LINEAR->CLUSTER_GRAPH_CTX,
			   FIRST_CLUSTER, NB_CLUSTER);
}


/*
=============================================================================
Name and dates: DRAW_NEURONS_VALUES_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws neurons values at the current iteration.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	STATE		int		FOREWARD, GOTO_INEDX, or SAME_STEP.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display
			SIMUL	   SIMUL_PARAMETERS	Definition of the simulation
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_NEURONS_VALUES_LINEAR(int STATE)
{
	Arg arglist[2];
	unsigned long COLOR_INDEX;
	float VALUE;
	static int PREV_MAX_IT;
	int MAX_IT, MIN_IT, IT;
	int i, j, n;
	int POSITION_X, POSITION_Y, SUP_LIMIT_X, INF_LIMIT_X, SUP_LIMIT_Y, INF_LIMIT_Y;
	EXPOSED_REGION REGION;

#if 0
	printf("DRAW_NEURONS_VALUES_LINEAR(%d)\n",STATE);
#endif

	/*gets back the exposed region*/
	GET_EXPOSED_REGION(&REGION);

	/*resize the window if needed*/
	if(REGION.WIDTH < NETWORK.LINEAR->WIN_WIDTH)
	{
		n=0;
		XtSetArg(arglist[n], XmNwidth, (DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))-20)); n++;
		XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);
	}

	/*computes max and min iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	MAX_IT = MIN_IT + SIMUL.NB_ITERATION;
	IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.NB_ITERATION - 1 : SIMUL.ITERATION - 1;

	/*translates previous iteration if needed*/
	if((SIMUL.ITERATION > SIMUL.NB_ITERATION && STATE != SAME_STEP) || STATE == GOTO_INDEX)
		memmove(NETWORK.LINEAR->NEURONS_COLORS,
			NETWORK.LINEAR->NEURONS_COLORS + NETWORK.NB_NEURONS,
			((SIMUL.NB_ITERATION - 1) * NETWORK.NB_NEURONS * sizeof(unsigned long)/sizeof(char)));

	/*computes neuron colors for new iteration*/
	if(STATE != SAME_STEP)
	{		
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			/*no simulation*/
			if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION)
				COLOR_INDEX = C_GREEN;
			/*spike*/
			else if((SIMUL.POT + i)->EVENT == SPIKE)
				COLOR_INDEX = C_RED;
			/*greater than color scale top value*/
			else if((SIMUL.POT + i)->PMB >= SIMUL.CS_TOP_VALUE)
				COLOR_INDEX = C_SCALE_COLOR_OFFSET + 99;
			/*lower than color scale bottom value*/
			else if((SIMUL.POT + i)->PMB <= SIMUL.CS_BOTTOM_VALUE)
				COLOR_INDEX = C_SCALE_COLOR_OFFSET;
			/*not special value*/
			else
			{
				VALUE = (float)SIMUL.CS_TOP_VALUE - (SIMUL.POT + i)->PMB;
				COLOR_INDEX = (unsigned long)C_SCALE_COLOR_OFFSET + 99 -
					      (unsigned long)(VALUE/SIMUL.CS_STEP);
			}
			/*store color*/
			*(NETWORK.LINEAR->NEURONS_COLORS + i + IT * NETWORK.NB_NEURONS) = COLOR_INDEX;
		}/*end of neuron loop*/
	}

	/*draw values*/
	if(STATE == FOREWARD)
	{
		/*translate previous iteration if needed*/
		if(PREV_MAX_IT != MAX_IT)
			XCopyArea(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.WINDOW_ID,
			  	NETWORK.LINEAR->NEURON_GRAPH_CTX, REGION.X, REGION.Y,
			  	REGION.WIDTH, REGION.HEIGHT, REGION.X 
				- NETWORK.LINEAR->NEURON_WIDTH,
			  	REGION.Y);
		/*draw new iteration*/
		INF_LIMIT_X = REGION.X - NETWORK.LINEAR->NEURON_WIDTH;
		SUP_LIMIT_X = REGION.X + REGION.WIDTH;
		INF_LIMIT_Y = REGION.Y - NETWORK.LINEAR->NEURON_HEIGHT;
		SUP_LIMIT_Y = REGION.Y + REGION.HEIGHT;
		POSITION_X =  IT * NETWORK.LINEAR->NEURON_WIDTH + 1;
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			POSITION_Y = i * NETWORK.LINEAR->NEURON_HEIGHT + 2;
			if(POSITION_X >= INF_LIMIT_X && POSITION_X <= SUP_LIMIT_X &&
			   POSITION_Y >= INF_LIMIT_Y && POSITION_Y <= SUP_LIMIT_Y)
			{
				XSetForeground(DISPLAY_VISU, NETWORK.LINEAR->NEURON_GRAPH_CTX,
					LUT[*(NETWORK.LINEAR->NEURONS_COLORS + i + NETWORK.NB_NEURONS 
					* IT)]);
				XFillRectangle(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.LINEAR->NEURON_GRAPH_CTX,
			 	      	POSITION_X, POSITION_Y, NETWORK.LINEAR->NEURON_WIDTH - 1, 
			  	     	NETWORK.LINEAR->NEURON_HEIGHT - 3);
			}
		}
	}
	else if(STATE == SAME_STEP)
	{
		XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, REGION.X, REGION.Y,
			   REGION.WIDTH, REGION.HEIGHT, False);
		/*redraw all*/
		INF_LIMIT_X = REGION.X - NETWORK.LINEAR->NEURON_WIDTH;
		SUP_LIMIT_X = REGION.X + REGION.WIDTH;
		INF_LIMIT_Y = REGION.Y - NETWORK.LINEAR->NEURON_HEIGHT;
		SUP_LIMIT_Y = REGION.Y + REGION.HEIGHT;
		for(j=0; j < SIMUL.NB_ITERATION; j++)
		{
			POSITION_X = j * NETWORK.LINEAR->NEURON_WIDTH + 1;
			for(i=0; i<NETWORK.NB_NEURONS; i++)
			{
				POSITION_Y = i * NETWORK.LINEAR->NEURON_HEIGHT + 2;
				if(POSITION_X >= INF_LIMIT_X && POSITION_X <= SUP_LIMIT_X &&
				   POSITION_Y >= INF_LIMIT_Y && POSITION_Y <= SUP_LIMIT_Y)
				{
					XSetForeground(DISPLAY_VISU, NETWORK.LINEAR->NEURON_GRAPH_CTX,
						       LUT[*(NETWORK.LINEAR->NEURONS_COLORS + i + NETWORK.NB_NEURONS * j)]);
					XFillRectangle(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.LINEAR->NEURON_GRAPH_CTX,
				 	      	POSITION_X, POSITION_Y, NETWORK.LINEAR->NEURON_WIDTH - 1, 
				  	     	NETWORK.LINEAR->NEURON_HEIGHT - 3);
				}
			}
		}
	}

	/*keep for the next call*/
	PREV_MAX_IT = MAX_IT;

}


/*
=============================================================================
Name and dates: PRINT_LINEAR
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function print the linear representation in its actual
	state if there is a simulation running. This function use the
	PLOT_PS library.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	DRAW_ALL	int		flag : True = all the network 
							       is printed else only the 
							       visible part is printed.
			PAPER_WIDTH	float		The width of the sheet

Explicits outputs :     None

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network
			CMAP_ID		Colormap	The colormap
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

#define REFERENCE_STEP 50

void PRINT_LINEAR(int DRAW_ALL, float PAPER_WIDTH)
{
	float NEURON_HEIGHT;
	float MAX_HEIGHT = 20.;
	float SCALE;
	float X_tab[4], Y_tab[4];
	int i, j, IT;
	EXPOSED_REGION REGION;
	XColor COLOR;
	char ACTION;
	char STRING[80];
	
	/*computes scale for drawing. A neuron is a rectangle with a width of one user unit*/
	/*one user unit equal SCALE centimeters*/
	SCALE = (PAPER_WIDTH - 2)/(float)SIMUL.NB_ITERATION;
	/*computes neuron height in user unit*/
	/*we considere that the sheet drawable area is maximum 20. centimeters heigh*/
	if(DRAW_ALL)
	{
		/*computes height of a neuron rectangle*/
		NEURON_HEIGHT = MAX_HEIGHT/(float)NETWORK.NB_NEURONS/SCALE;
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 2., 0., 0.);
	}
	else
	{
		/*get back visible part geometry*/
		GET_EXPOSED_REGION(&REGION);
		/*compute height of a neuron rectangle*/
		NEURON_HEIGHT = MAX_HEIGHT/((float)REGION.HEIGHT/(float)NETWORK.LINEAR->NEURON_HEIGHT)/SCALE;
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 2. - SCALE * (float)(NETWORK.LINEAR->WIN_HEIGHT - REGION.Y - REGION.HEIGHT)/
			  (float)NETWORK.LINEAR->NEURON_HEIGHT * NEURON_HEIGHT, 0., 0.);
	}

	/*if drawing is not possible*/
	if(SCALE < 0.008)
	{
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 1., 0., 0.);
		ps_writeah(0., 5., "XNBC : Visualization Printing Error", 0.7);
		ps_writeah(0., 4., NETWORK.NAME, 0.5);
		ps_writeah(0., 3., "is too large to be printed", 0.5);
	}
	/*drawing is possible*/
	else
	{
		/*set scale*/
		ps_echel(SCALE, SCALE, 0., 0., 1., 0.);
		/*use user units*/
		ps_user();

		/*draw neurons*/
		/*iteration loop*/
		for(j = 0; j < SIMUL.NB_ITERATION; j++)
		{
			/* neuron loop */
			for(i=0; i < NETWORK.NB_NEURONS; i++)
			{
				/* computes position */
				X_tab[0] = (float)j;      Y_tab[0] = (NETWORK.NB_NEURONS - i) * NEURON_HEIGHT;
				X_tab[1] = 1.+(float)j;    Y_tab[1] = Y_tab[0];
				X_tab[2] = X_tab[1];   Y_tab[2] = (NETWORK.NB_NEURONS - i - 1) * NEURON_HEIGHT;
				X_tab[3] = X_tab[0];   Y_tab[3] = Y_tab[2];

				/* sets color */
				COLOR.pixel = LUT[*(NETWORK.LINEAR->NEURONS_COLORS + j * NETWORK.NB_NEURONS + i)];
				XQueryColor(DISPLAY_VISU, CMAP_ID, &COLOR);
				ps_set_rgb_color((float)COLOR.red/65535, (float)COLOR.green/65535, 
						 (float)COLOR.blue/65535);
				/* draw neuron */
				ps_filled_area(X_tab, Y_tab, 4);
			}/*end of neuron loop*/
		} /* fin du for */


		/*draw clusters separations*/
		ps_set_rgb_color(0., 0., 0.);
		ps_line_width(3.);
		X_tab[0] = 0.;
		X_tab[1] = SIMUL.NB_ITERATION; 
		Y_tab[0] = NETWORK.NB_NEURONS * NEURON_HEIGHT;
		Y_tab[1] = Y_tab[0];
		ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		for(i=0; i<NETWORK.NB_CLUSTERS; i++)
		{
			Y_tab[0] = Y_tab[0] - (*(NETWORK.NB_NEURONS_PER_CLUSTER + i) * NEURON_HEIGHT);
			Y_tab[1] = Y_tab[0];
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}

#if 0
		/*draw neurons separations*/
		ps_line_width(1.);
		X_tab[0] = 0.;
		X_tab[1] = SIMUL.NB_ITERATION; 
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			Y_tab[0] = (NETWORK.NB_NEURONS - i) * NEURON_HEIGHT;
			Y_tab[1] = Y_tab[0];
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}

		/*draw iterations separations*/
		Y_tab[0] = NETWORK.NB_NEURONS * NEURON_HEIGHT;
		Y_tab[1] = 0.;
		for(i=0; i<SIMUL.NB_ITERATION; i++)
		{
			X_tab[0] = i;
			X_tab[1] = X_tab[0];
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}
#endif
		/*draw bouding box if not DRAW_ALL*/
		ps_cms();
		if(!DRAW_ALL)
		{
			ps_set_rgb_color(1., 1., 1.);
			ps_origin(0., SCALE * (float)(NETWORK.LINEAR->WIN_HEIGHT - REGION.Y - REGION.HEIGHT)/
			  (float)NETWORK.LINEAR->NEURON_HEIGHT * NEURON_HEIGHT, 0., 0.);

			/*lower rectangle*/
			X_tab[0] = -1.;
			Y_tab[0] = -0.1;
			X_tab[1] = PAPER_WIDTH;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = -5.;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
			/*upper rectangle*/
			X_tab[0] = -1.;
			Y_tab[0] = MAX_HEIGHT+0.1;
			X_tab[1] = PAPER_WIDTH;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = 50.;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
			/*left rectangle*/
			X_tab[0] = -0.1;
			Y_tab[0] = 0.;
			X_tab[1] = -0.1;
			Y_tab[1] = 50.;
			X_tab[2] = -2.;
			Y_tab[2] = 50.;
			X_tab[3] = -2.;
			Y_tab[3] = 0.;
			ps_filled_area(X_tab, Y_tab, 4);
			/*right rectangle*/
			X_tab[0] = PAPER_WIDTH - 1.9;
			Y_tab[0] = 0.;
			X_tab[1] = X_tab[0];
			Y_tab[1] = 50.;
			X_tab[2] = PAPER_WIDTH +2.;
			Y_tab[2] = 50.;
			X_tab[3] = X_tab[2];
			Y_tab[3] = 0.;
			ps_filled_area(X_tab, Y_tab, 4);
		}			
		/*draw comments*/
		ps_origin(0., MAX_HEIGHT + 0.1, 0., 0.);
		/*draw color scale*/
		for(i=0; i<100; i++)
		{
			COLOR.pixel = LUT[i + C_SCALE_COLOR_OFFSET];
			XQueryColor(DISPLAY_VISU, CMAP_ID, &COLOR);
			ps_set_rgb_color((float)COLOR.red/65535, (float)COLOR.green/65535, 
					 (float)COLOR.blue/65535);
			X_tab[0] = (float)i*0.1;
			Y_tab[0] = 1.;
			X_tab[1] = X_tab[0]+0.1;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = Y_tab[0] - 0.5;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
		}
		ps_set_rgb_color(0., 0., 0.);
		ps_writeah(0., 1.1, "Color Scale in mV", 0.3);
		/*draw spike color*/
		ps_writeah(10.5, 1.1, "Spike", 0.3);
		X_tab[0] = 10.5;
		Y_tab[0] = 1.;
		X_tab[1] = 11.;
		Y_tab[1] = Y_tab[0];
		X_tab[2] = X_tab[1];
		Y_tab[2] = 0.5;
		X_tab[3] = X_tab[0];
		Y_tab[3] = Y_tab[2];
		ps_set_rgb_color(1., 0., 0.);
		ps_filled_area(X_tab, Y_tab, 4);
		/*draw values*/
		ps_set_rgb_color(0., 0., 0.);
		ps_writeintah(0., 0.1, (int)SIMUL.CS_BOTTOM_VALUE, 0.3);
		ps_writeintah(9.8, 0.1, SIMUL.CS_TOP_VALUE, 0.3);
		sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
		ps_writeah(0., 1.6, STRING, 0.3);
		sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
		ps_writeah(0., 2., STRING, 0.3);

		/*draw time scale*/
		IT = (SIMUL.ITERATION > 50) ? SIMUL.ITERATION : 50;
		sprintf(STRING, "%d", IT);
		ps_origin(0., -MAX_HEIGHT - strlen(STRING)*0.25 - 0.1, 0., 0.);
		for(i=0; i<SIMUL.NB_ITERATION; i+=REFERENCE_STEP)
			ps_writeintav(PAPER_WIDTH - 2.0 - (i+0.1)*SCALE, 0., IT-i, 0.3);
		ps_writeintav(0.3, 0., IT - SIMUL.NB_ITERATION + 1, 0.3);
		ps_writeah(PAPER_WIDTH - 2.0 - 4.5*SCALE, -0.5, "(in ms)", 0.3);

	}/*end of drawing*/

}


/*
=============================================================================
Name and dates: FREE_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function free the linear representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/

void FREE_LINEAR(int ACTION)
{
	int n;

	/*free the entire representation*/
	if(NETWORK.LINEAR != Null && ACTION == ALL)
	{
		FREE_GC(NETWORK.LINEAR->NEURON_GRAPH_CTX);
		FREE_GC(NETWORK.LINEAR->CLUSTER_GRAPH_CTX);
		FREE_MEMORY(NETWORK.LINEAR->CLUSTER_GEOMETRY);

		FREE_MEMORY(NETWORK.LINEAR);
		NETWORK.LINEAR = Null;
	}
	
	/*reinitializes neurons colors array*/
	if(NETWORK.LINEAR != Null && ACTION == SPECIAL)
	{
		/*first iteration*/
		for(n=0; n<NETWORK.NB_NEURONS; n++)
			*(NETWORK.LINEAR->NEURONS_COLORS + n)= C_GREEN;
		/*next iterations*/
		for(n=NETWORK.NB_NEURONS; n<NETWORK.NB_NEURONS*SIMUL.NB_ITERATION; n++)
			*(NETWORK.LINEAR->NEURONS_COLORS + n)= C_WHITE;
	}
}


/*
=============================================================================
Name and dates: INFO_LINEAR
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function constructs the information string for the
	linear rpresentation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	X		short		X position of the pointer
			Y		short		Y position of the pointer

Explicits outputs :	STRING		char *		The information string
			INDEX		int *		Index of the selected neuron

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			SIMUL		SIMUL_DEF	The definition of the simulation

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
/*static string locals to this file*/
static char STRING[256];

char *INFO_LINEAR(short X, short Y, int *INDEX)
{
	unsigned long COLOR_INDEX;
	int NEURON, CLUSTER, ITERATION;
	int i, TMP=0, MIN_IT;
	char POT_STR[80];
	char NAME[80];

	/*computes neuron number*/
	NEURON = Y/NETWORK.LINEAR->NEURON_HEIGHT + 1;
	
	/*computes cluster number*/
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
	{
		TMP += *(NETWORK.NB_NEURONS_PER_CLUSTER + i);	
		if(NEURON <= TMP)
		{
			CLUSTER = i+1;
			break;
		}
	}

	/*computes iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	ITERATION = MIN_IT + X/NETWORK.LINEAR->NEURON_WIDTH + 1;
	
	/*computes potential*/
	if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION || NETWORK.LINEAR->NEURONS_COLORS == Null
	   || ITERATION > SIMUL.ITERATION)
		strcpy(POT_STR, ": Undefined");
	else
	{
		COLOR_INDEX = *(NETWORK.LINEAR->NEURONS_COLORS + 
				(ITERATION - MIN_IT -1)*NETWORK.NB_NEURONS + (NEURON - 1));
		if(COLOR_INDEX == C_RED)
			strcpy(POT_STR, ": Spike");
		else if(COLOR_INDEX == C_SCALE_COLOR_OFFSET + 99)
			sprintf(POT_STR, ">= %3.2f mV", SIMUL.CS_TOP_VALUE);
		else if(COLOR_INDEX == C_SCALE_COLOR_OFFSET)
			sprintf(POT_STR, "<= %3.2f mV", SIMUL.CS_BOTTOM_VALUE);
		else
			sprintf(POT_STR, "= %3.2f mV", SIMUL.CS_TOP_VALUE + (signed)(COLOR_INDEX - 
				C_SCALE_COLOR_OFFSET - 99) * (double)SIMUL.CS_STEP);
	}

	/*get cluster name*/
	if(NETWORK.INFO_CLUSTERS != Null)
		strcpy(NAME, NETWORK.INFO_CLUSTERS[CLUSTER-1].NAME);
	else
		sprintf(NAME, "CLUSTER_%d", CLUSTER);

	/*construct string*/
	sprintf(STRING, "TIME : %d ms\nCLUSTER\n %s\n No %d\n with %d neurons\nNEURON\n No %d/network\n No %d/cluster\nP.MB. %s",
	        ITERATION, NAME, CLUSTER, *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1)), NEURON, 
		NEURON - (TMP - *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1))), POT_STR);
	*INDEX = NEURON;
	return(STRING);
}
