/*
=============================================================================
	MAIN_VISU.C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================

	INIT_VISU :
		Initialization of the Visualization application. If
		MAIN_VISU is defined during the compilation, this module
		becomes the main module of the Visualization application.
	EXIT_VISU :
		Exit from application Visualization.

=============================================================================
	History:

		Version:
			0.0	25-feb-93	DLZ
			0.1	15-apr-93	DLZ
			1.0	2-nov-93	True color support DLZ
			1.1	9-may-95	OSF v3.2 port
						+ name on command line JFV
			1.2	10-may-95	electrode cursor JFV
			1.3	9-apr-96	UID environment variable added JFV
			2.0	7-jul-96	*.cnx file replaced by *.len and *.wgt files PD
			2.1	12-sep-96	Only read *.len, *.pmb read in the same time JFV
			2.2     1 mars 97       Correction de bugs dans print_visu JFV
			2.3     5 mars 97       Supression memory leaks JFV
			2.4     7 mars 97       Correction bugs introduits dans 2.3!! JFV
                        2.5     4 juin 97       Nouveaux boutons dans barre de menu     GS
			3.0     6 juin 97       Ajout de 6 fenetres de visualisations des P.mbs
			                        controlees pas la touche de gauche de la souris. GS
			3.1     11 juin 97	Correction bugs dans boutons de la barre de menu. GS
			3.2     19 juin 97      Correction bugs dans creation des fichiers .ps. GS
			3.3     23 juin 97      Amelioration de la fenetre ITERATION et correction
			                        de bugs pour les boutons de PAUSE et de PASaPAS. GS
			4.0     11 juillet 97   Ajout d`une fonction de Zoom sur les fenetres de type AFFICHE. GS
			4.1     31 juillet 97   Ajout d`une aide dans les fenetres Affiches et le main. GS
		        4.2     1 Aout 97       Ajout d`une taille variable aux fenetres AFFICHES. GS
		        4.3     2 sept 97       Prise en compte .xnbcrc JFV
		        4.4     20 dec 97       Prise en compte LessTiff (MrmOpenHierarchyPerDisplay) JFV
		        4.5     30 dec 97       Correction de bugs dus a la correction LessTiff JFV

			                        Version 5: integralement C
			5.0     26 jan 99       Suppression des UIL du fait des bugs de Lesstif  JFV			
			5.1     18 aout 99      Adaptation pour que ca tourne avec Lesstif ou Motif JFV 
			5.2     30 aout 99      Probleme de nom de fichier d'impression et impression en blanc JFV 
			5.3     1 sept 99       Message d'erreur plus precis pour impression JFV 
			5.4     2 sept 99       Prise en compte de la variable d'environnement 
			                        PRINT_PLOT_PS et du choix de l'imprimante
						Correction de bugs d'affichage et de rafraichissement
						Ajout du bouton de choix de ms Ms visibles
						(ABANDON de la version UIL a 4.9)
						Grace a Fabian Alvares (Montevideo)
						et son habilite a trouver les bugs...
						Cette version devient la version unique de VISU_NBC JFV
			5.5     20 dec 99       Correction d'un pb de taille de region visible 
                                                (marchait sur Motif, pas sur Lesstif)  JFV
			5.6     4 avril 2000    Affichage du nom de la simulation dans la barre de titre JFV
			5.7     ??
			5.8	17 jan 2001	Changement dans le global-visu.h (JFV)
			5.9     20 aout 2001    Adaptation a CYGNUS (JFV)
			5.10    13 sep 2001     Adaptation a Windws: lecture de .ntw au lieu de .lnk (JFV)
			5.11    16 nov 2001     Allows the reading of .lnk if .ntw is not found (for back compatibiliry) (JFV)
*/


#ifdef	DOCUMENTATION
title	VISU_NBC 	A tool to visualize biologicals neurals networks activity.

synopsis

	visu_nbc [-d display]

description

	This tool is also accessible from the xnbc menu : Applications->Visualization

	To visualize the network connectivity, visu_n_b_c needs a {\em .cnx} file
	(connectivity matrix).
	
	To visualize the evolution of the network activity, visu_n_b_c needs a
	a {\em .def} (simulation definition) and a {\em .pmb} file (menbrane potentials).

	The program outputs can be send on another X server with the 
	following switch :

	-d Display (terminal:display.screen)

	{\bf Remark :} In the xnbc version a dialog box prompts you to select another
	display (or to keep the default one) just after activation of 
	the {\bf Applications->Visualization} menu.

description of the user interface

	{\bf Menu File} 

	- Open Network Files : 	  choose a {\em .cnx} file.

	- Open Simulation Files : choose a {\em .def} file.

	- Close Simualtion : 	  close simulation files.

	- Exit : 		  exit visu_n_b_c.

	{\bf Menu Edit}

	- Show Details :   visualization of details for the selected.
			   representation (Button On/Off).

	- Refresh Screen : redraw the selected representation.

	- Print Network :  print the selected representation.

	{\bf Menu Options}

	- Scale Top Value : permits to change the top value of the menbrane
			    potentials color scale.

	- Time Step : 	    permits to modify the time between the visualization 
		            of each step of the simulation.

	- Type of representation : permits to select the type of representation.

	{\bf Menu Help}

	- On Context : 		      Not yet implemented.

	- On Window  : 		      Not yet implemented.

	- Overview of Visualization : Not yet implemented.

	- About Visualization       : General informations.
				    
	{\bf Color Scale}

	This scale associates a color at a value of the menbrane potential. This scale has
	two parts :

	The first one (red rectangle) indicates the potential for a spike. This value 
	is read in the {\em .def} file.

	The second one indicates a linear correspondance between color and potential values.
	The top value can be modified (the bottom value is read in the {\em .def} file as the
	{\bf Negative Reversal Potential}). By increasing this value, small
	variations of the potential can be filtered, and on the opposite by decreasing it
	small variations of the potential around the resting potential can be magnified.

	To change this value :

	- select the corresponding menu in the Options menu.

	- move the cursor of the scale in the dialog box to set the wanted value.

	- click on Ok to change the value or on Cancel to keep the previous value.

	{\bf Time Step}

	The time between representation of each step of the simulation can be modified.
	This time acts on the visualization speed not on the simulation time step increment. 
	The major purpose
	of this function is to slow down the visualization when the network is small or the computer
	too rapid.

	To change this time :

	- select the corresponding menu in the Options menu.

	- move the cursor of the scale in the dialog box to set the wanted time.

	- click on Ok to change the time or on Cancel to keep the previous time.

	{\bf Visualization buttons}

	These buttons are in the bottom left corner of the window. They control
	the visualization like a {\bf video tape player}.

	The first button at left stops the visualization and returns to the first
	iteration.

	The second button starts the visualization.

	The third button pauses the visualization at its current iteration. Visualization can be restarted 
	from this iteration by clicking the second button.

	The fourth button allows to go directly to a specific iteration. 
	Enter the selected iteration number in the 
	iteration text field, then press {\em return} or click on this button to go to the selected
	iteration.

	The fifth button runs one step of the visualization (step by step visualization).

	{\bf Informations fields}

	Several fields display informations on the window.

	-{\tt Iteration} : display the current iteration.
	This field allows too to enter an iteration for the "Goto Index" button 
	(see previous section).

	-{\tt N.G.A.} : display the Network Global Activity in percentage (number of spiking neurons / number
	of neurons in the network).

	-{\tt Network Information} : display number of clusters and neurons in the network.

	-{\tt Specials Informations} : display differents informations according to the selected
	representation. To obtain this information click in the window where the visualization
	is displayed. Then a hair-cross cursor, moving with pointer device, appears in the window.
	Move the cursor
	to point on which part of the visualization informations are needed.

	{\bf Print Network}

	To print the visualization at its current iteration (when the visualization is paused)
	select the corresponding menu in the Edit menu.

	Then choose the PostScript printer, if the file must be kept or not after printing, 
	the type of the printed part: entire representation or only the visible part 
	(what is currently seen in the window). Then click on
	Ok to print or on Cancel to cancel it.

	If the PostScript file is kept for further printing,
	a specific file name can be given (default name 
	is {\em visualization.ps}.

	{\bf Open Files}

	To open Network files and Simulation files select the corresponding menu in the file menu.
	Then select file in the dialog box and click on Ok to open it or on Cancel to cancel it.

how it works

	{\em {\bf visu_n_b_c}} is like a "video tape player".

	A normal session is as follow :

	- Open Network file ({\em .cnx}) then the representation of network appears in the window.
	Default representation is LINEAR REPRESENTATION (see below).

	- Open Simulation files ({\em .def}). visu_n_b_c tries to open the given file name
	with {\em .def} and {\em .pmb} extension. If it fails the visualization is impossible and an
	information message is sent.
	A file name without extension, with a {\em .def} extension (default int the file selection box) or
	with a {\em .pmb} extension can be given.

	- Now the visualization is ready to start.

	- At every moment the representation type can be changed, or special
	informations on the current simulation can be obtained.

	- To print the network, change time step or change color scale top value,
	the visualization must be paused.

	- At the end of the work, close the simulation.

	- Others Simulation files for the same network or an other network can be obtained.

description of representation

	{\bf Linear Representation}

	This representation is like a dot display but shows as a color graph the evolution of the 
	menbrane potential between spikes.

	Each colored square represents a neuron. Each column represents one iteration of the simulation.
	The black lines delimit clusters.

	{\bf Global Activity Representation}

	This representation displays for each cluster its global activity in percentage at each 
	iteration (number of spiking neurons / number of neurons in the cluster).

	{\bf Matrix Representation}

	This representation displays the network state at an iteration. The network is represented like a
	matrix.

	Each colored square represents a neuron. Each black big square represents a cluster. Small lines
	represent connections from a neuron to another. With {\bf Show Details} connection types can be 
	visualized.

	When a neuron fires the spike can be seen travelling along its axon toward others neurons.

author
	Denis Lambolez 
	CHU Saint-Antoine

	Please reports all remarks to {\em e-mail : vibert@b3e.jussieu.fr}
#endif



#include <stdlib.h>
#include <stdio.h>			/* For printf and so on. */
#include <string.h>
#include <Xm/Text.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#if (XmVersion >= 1002)
#include <Xm/RepType.h>
#endif

#include <X11/X.h>

#define MAIN_VISU_MODULE
#include "global_def_visu.h"
#include "def_principale.h"
#include "def_les_fenetres.h"
#undef MAIN_VISU_MODULE


/*
 * Convenience functions from utilities file.
 */
extern void RegisterBxConverters(XtAppContext);
extern XtPointer CONVERT(Widget, char *, char *, int, Boolean *);
extern XtPointer DOUBLE(double);
extern XtPointer SINGLE(float);
extern void MENU_POST(Widget, XtPointer, XEvent *, Boolean *);
extern Pixmap XPM_PIXMAP(Widget, char**);
extern void SET_BACKGROUND_COLOR(Widget, ArgList, Cardinal*, Pixel);

/*
-----------------------------------------------------------------------------
	Global data for the file
-----------------------------------------------------------------------------
*/


static Widget BUTTON_VISUALIZATION;	/*Place to keep the widget id*/


/*
-----------------------------------------------------------------------------
	Names and addresses of callback routines to register with Mrm
-----------------------------------------------------------------------------
*/


/*Icone of the application*/
#include "icone_visu.h"





#define GRAPH_HEIGHT 265     /*size in pixels of a graphic*/
#define LOWER_OFFSET 55
#define REFERENCE_STEP 50
#define Y_AXIS_LENGHT (GRAPH_HEIGHT-UPPER_OFFSET-LOWER_OFFSET)
#define X_AXIS_LENGHT (NETWORK.GA->WIN_WIDTH-LEFT_OFFSET-RIGHT_OFFSET)
#define Y_NB_SCALE 11
#define UPPER_OFFSET 10
#define RIGHT_OFFSET 20
#define LEFT_OFFSET 40

/*=============================================================================
Name and dates: main
---------------
	Creation : Feb 25, 1993
	Last modification : Dec 30, 1997

Functionnal description :
-------------------------
	Initialize the application and enter in the main loop.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :
   			argc		uns int		Number of arguments
							on the command line
			argv		char *[]	Array of arguments on
							the command line
							(size = argc)
							menu Visualization
Explicits outputs : None

Implicits inputs  : None

Implicits outputs :
		    TOP_LEVEL_WIDGET	 Widget		Widget id of the
							application shell
		    MRM_HIERARCHY_VISU	 MrmHierarchy	Mrm hierarchy of the
							application
		    DISPLAY_VISU	 Display *	Display of the
							application
		    APP_CONTEXT		 XtAppContext   Application context of
							Visualization
		    WIDGET_ARRAY	 Widget		The widget array

Sides effects : None
---------------

=============================================================================
*/

void main(int argc, char **argv)
                          /* Command line argument count. */
	                  /* Pointers to command line args. */
{
	Arg ARGLIST[4];	/*Not important stuff*/
	Widget W;

	int i, n, pourquoi;
	char MESSAGE[500];
	char	file_name[256], *uid_root;

    XtAppContext app;
    Arg          args[256];
    Cardinal     ac;
    Boolean      argok=False;
    Widget   topLevelShell;

    Widget   window_Main_Visu;
    Widget   window_Affiche_Pmb5;
    Widget   window_Affiche_Pmb4;
    Widget   window_Affiche_Pmb3;
    Widget   window_Affiche_Pmb2;
    Widget   window_Affiche_Pmb1;
    Widget   window_Affiche_Pmb0;
    Widget   window_Number_Iteration;
    Widget   window_Inter_Images;
    Widget   window_List_Selected_Neuron;
    Widget   window_Info_Message;
    Widget   window_Error_Message;
    Widget   window_About;
    Widget   window_Simul_Selection;
    Widget   window_Network_Selection;
    Widget   window_Scale_Top_Value;
    Widget   window_Time_Step;
    Widget   window_Print;

/*
Initialize the Mrm and the Motif  toolkits create the application context,
open and initialize the display and create the toplevel shell
*/



	XtToolkitInitialize();
	APP_CONTEXT = XtCreateApplicationContext();

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i],"-k") == 0)
			keep_ps_files = TRUE;
		else if (strcmp(argv[i], "-d") == 0)
		{
			i++;
			if((DISPLAY_VISU = XOpenDisplay(argv[i])) == NULL)
			{
				printf("Can't open display %s for Visualization\nOpen default display...\n",
				       argv[i]);
				if((DISPLAY_VISU = XOpenDisplay(NULL)) == NULL)
					FATAL_ERROR("Can't open default display for Visualization");
			}
		}
		else if (strcmp(argv[i], "--help") == 0)
		{
		  printf("visu_nbc [--help] [--version] [-k] [-d] [input_file]\n");
		  printf("\t--help: this help\n");
		  printf("\t--version: version number\n");
		  printf("\t-k: keeps postscript files\n");
		  printf("\t-d: open display d\n");
		  printf("\tinput_file: the prefix name of the simulation (no extension)\n\n");
		  printf("Report bugs to vibert@b3e.jussieu.fr\n");
		  exit(0);
		}
		else if (strcmp(argv[i], "--version") == 0)
		{
		  printf("visu_nbc (XNBC V8) %s\n",VERSION_VISU);
		  printf("Copyright (C) 1993 Dr J-F Vibert\n");
		  printf("visu_nbc comes with NO WARRANTY,\nto the extent permitted by law.\n");
		  printf("You may redistribute copyes of this program\n");
		  printf("under the terms of the GNU General Public Licence.\n");
		  printf("For more information about these matters\n");
		  printf("the files named COPYING\n");
		  exit(0);
		}
		else
			strcpy(generic_name,argv[i]);
	} /* fin du for */


	if(!DISPLAY_VISU)
	{
		if((DISPLAY_VISU = XOpenDisplay(NULL)) == NULL)
			FATAL_ERROR("Can't open default display for Visualization");
	}

	XtDisplayInitialize(APP_CONTEXT, DISPLAY_VISU, "Visualization",
	                    NULL, NULL, 0, &argc, argv);

	ac = 0;
	
	XtSetArg(args[ac], XmNx, 0); ac++;
	XtSetArg(args[ac], XmNy, 0); ac++;
	XtSetArg(args[ac], XmNwidth, 1131); ac++;
	XtSetArg(args[ac], XmNheight, 712); ac++;
	
	XtSetArg(args[ac], XmNallowShellResize, True);  ac++;
	TOPLEVEL_WIDGET_VISU = XtAppCreateShell("Visualization", "visu_nbc",
	    applicationShellWidgetClass, DISPLAY_VISU, args, ac);

/*Initialize the icone of Visualization and the title of the main window*/

	n=0;
	XtSetArg(ARGLIST[n], XmNiconPixmap,
		 XCreateBitmapFromData (DISPLAY_VISU, DefaultRootWindow(DISPLAY_VISU),
					icone_visu_bits, icone_visu_width, icone_visu_height)); n++;
	XtSetArg(ARGLIST[n], XmNiconName, "XNBC Visualization"); n++;
	sprintf(MESSAGE,"XNBC: Visualization V%s", VERSION_VISU);
	XtSetArg(ARGLIST[n], XmNtitle, MESSAGE); n++;
	XtSetValues(TOPLEVEL_WIDGET_VISU, ARGLIST, n);


    
    RegisterBxConverters(APP_CONTEXT);
#if (XmVersion >= 1002) 
    XmRepTypeInstallTearOffModelConverter();
#endif
    

/*
Initialize WIDGET_ARRAY
*/
	for(n=0; n < NB_REGISTERED_WIDGET; n++)
		WIDGET_ARRAY[n] = Null;

    window_Main_Visu = (Widget)Createwindow_Main_Visu(TOPLEVEL_WIDGET_VISU);
    REGISTER(window_Main_Visu, (XtPointer)k_Window_Main_Visu, (XtPointer)0);
    XtManageChild(window_Main_Visu);
    XtPopup(XtParent(window_Main_Visu), XtGrabNone);


    /* Begin user code block <create_dialogShell15> */
    /* End user code block <create_dialogShell15> */
    
    window_Affiche_Pmb5 = (Widget)Createwindow_Affiche_Pmb5(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell14> */
    /* End user code block <create_dialogShell14> */
    
    window_Affiche_Pmb4 = (Widget)Createwindow_Affiche_Pmb4(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell13> */
    /* End user code block <create_dialogShell13> */
    
    window_Affiche_Pmb3 = (Widget)Createwindow_Affiche_Pmb3(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell12> */
    /* End user code block <create_dialogShell12> */
    
    window_Affiche_Pmb2 = (Widget)Createwindow_Affiche_Pmb2(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell11> */
    /* End user code block <create_dialogShell11> */
    
    window_Affiche_Pmb1 = (Widget)Createwindow_Affiche_Pmb1(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell10> */
    /* End user code block <create_dialogShell10> */
    
    window_Affiche_Pmb0 = (Widget)Createwindow_Affiche_Pmb0(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell9> */
    /* End user code block <create_dialogShell9> */
    
    window_Number_Iteration = (Widget)Createwindow_Number_Iteration(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell8> */
    /* End user code block <create_dialogShell8> */
    
    window_Inter_Images = (Widget)Createwindow_Inter_Images(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell8> */
    /* End user code block <create_dialogShell8> */
    
    window_List_Selected_Neuron = (Widget)Createwindow_List_Selected_Neuron(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell7> */
    /* End user code block <create_dialogShell7> */
    
    window_About = (Widget)Createwindow_About(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell4> */
    /* End user code block <create_dialogShell4> */
    
    window_Simul_Selection = (Widget)Createwindow_Simul_Selection(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell3> */
    /* End user code block <create_dialogShell3> */
    
    window_Network_Selection = (Widget)Createwindow_Network_Selection(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell2> */
    /* End user code block <create_dialogShell2> */
    
    window_Scale_Top_Value = (Widget)Createwindow_Scale_Top_Value(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell1> */
    /* End user code block <create_dialogShell1> */
    
    window_Time_Step = (Widget)Createwindow_Time_Step(TOPLEVEL_WIDGET_VISU);
    
    /* Begin user code block <create_dialogShell> */
    /* End user code block <create_dialogShell> */
    
    window_Print = (Widget)Createwindow_Print(TOPLEVEL_WIDGET_VISU);

/* 
Realize the top level widget.  All managed children now become visible
*/

	ghostscript = getenv("GHOSTSCRIPT");
	if (ghostscript == NULL)
	  strcpy(GHOSTSCRIPT,"/usr/local/bin/gs");
	else
	  strcpy(GHOSTSCRIPT, ghostscript);
	papersize = getenv("PAPERSIZE");
	if (papersize == NULL)
	  strcpy(PAPERSIZE,"a4");
	else
	  strcpy(PAPERSIZE, papersize);

	XtRealizeWidget(TOPLEVEL_WIDGET_VISU);
/*
Initialization
*/

	INIT_VISUALIZATION();


	/* initializes the value displayed on the boutons */

	sprintf(MESSAGE,"Visible iterations (%d ms)",SIMUL.NB_ITERATION);
	n = 0;
	XtSetArg(ARGLIST[n], XmNlabelString, 
		 CONVERT(window_Main_Visu, MESSAGE,
			 XmRXmString, 0, &argok)); if (argok) n++;
	XtSetValues(WIDGET_ARRAY[k_Button_Millisec], ARGLIST, n);
	XtSetValues(WIDGET_ARRAY[k_Button_Number_Of_Iteration], ARGLIST, n);

	

	MASK = 	XmStringCreateSimple("./");

	XtSetSensitive(WIDGET_ARRAY[k_Button_Membr_Pot], False);
	
	if (generic_name[0])
	{
		XtSetSensitive(WIDGET_ARRAY[k_Button_Open_File], False);
		XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
		XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
		strcpy(file_name,get_network_name());
		
		strcpy(LOAD_FILE_NAME, file_name);

		if(!CREATE_NETWORK(file_name, MESSAGE))
		{
			DISPLAY_MESSAGE(MESSAGE, ERROR);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Open_File], True);
		}
		else
		{
			strcpy(file_name,get_simulation_name());
			if(!LOAD_SIMUL(file_name, MESSAGE))
			{
				DISPLAY_MESSAGE(MESSAGE, ERROR);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
			}
			else
			{
/*
printf("Avant INIT_ALLOC_TAB_SOURCE\n");
*/
                                INIT_ALLOC_TAB_SOURCE(file_name);
/*
printf("Apres INIT_ALLOC_TAB_SOURCE\n");
*/
				XtSetSensitive(WIDGET_ARRAY[k_Button_Open_File], False);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
				XtSetSensitive(WIDGET_ARRAY[k_Button_Open_Simul_Files], True);
			}
		} /* fin du else */
	} /* fin du if */

	sprintf(MESSAGE,"XNBC: Visualization V%s: %s", VERSION_VISU, file_name);
	XtSetArg(ARGLIST[n], XmNtitle, MESSAGE); n++;
	XtSetValues(TOPLEVEL_WIDGET_VISU, ARGLIST, n);
/* 
Enter the main loop
*/
	XtAppMainLoop(APP_CONTEXT);

}


/*
=============================================================================
Name and dates: EXIT_VISU
---------------
	Creation : Feb 25, 1993
	Last modification : Feb 25, 1993

Functionnal description :
-------------------------
	This callback exit from the application Visualization. It's called when the user
	press the Quit button.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  : None

Explicits outputs : None

Implicits inputs  : None

Implicits outputs : None

Sides effects : None
---------------

=============================================================================
*/


void EXIT_VISU ()
{

	if (SIMUL.POT) 
		free(SIMUL.POT);

	/*free allocated memory*/

	if (NETWORK.NAME)
	{
		FREE_MEMORY(NETWORK.NAME);
		FREE_MEMORY(NETWORK.NB_NEURONS_PER_CLUSTER);
		FREE_MEMORY(NETWORK.CONNEXION_MATRIX);
		FREE_MEMORY(NETWORK.SYNAPTIC_MATRIX);
		FREE_MEMORY(NETWORK.ANATOMY_MATRIX);
		FREE_MEMORY(NETWORK.INFO_CLUSTERS);
		FREE_MEMORY(NETWORK.SELECTED_NEURON);
	} /* fin du if */

	if (NETWORK.LINEAR)
	{
		FREE_GC(NETWORK.LINEAR->NEURON_GRAPH_CTX);
		FREE_GC(NETWORK.LINEAR->CLUSTER_GRAPH_CTX);
		FREE_MEMORY(NETWORK.LINEAR->CLUSTER_GEOMETRY);
		FREE_MEMORY(NETWORK.LINEAR->NEURONS_COLORS);
		FREE_MEMORY(NETWORK.LINEAR);
	}
	if (NETWORK.GA)
	{
		FREE_GC(NETWORK.GA->GA_GRAPH_CTX);
		FREE_MEMORY(NETWORK.GA->VALUES);
		FREE_MEMORY(NETWORK.GA->GRAPH_AXIS);
		FREE_MEMORY(NETWORK.GA->SCALE);
		FREE_MEMORY(NETWORK.GA);
	} /* fin du if */

	if (NETWORK.PMB)
	{
		FREE_GC(NETWORK.PMB->PMB_GRAPH_CTX);
		FREE_MEMORY(NETWORK.PMB->VALUES);
		FREE_MEMORY(NETWORK.PMB->GRAPH_AXIS);
		FREE_MEMORY(NETWORK.PMB->SCALE);
		FREE_MEMORY(NETWORK.PMB);
	}

	if (NETWORK.MATRIX)
	{
		FREE_GC(NETWORK.MATRIX->SPIKE_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->CLUSTER_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->NEURON_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->DENDRITE_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->DETAILS_GRAPH_CTX);
		FREE_MEMORY(NETWORK.MATRIX->CLUSTER_GEOMETRY);
		FREE_MEMORY(NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER);
		FREE_MEMORY(NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER);
		FREE_MEMORY(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER);
		FREE_MEMORY(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER);
		FREE_MEMORY(NETWORK.MATRIX);
	}
	
	if(tab_source0)
	{
	    free(tab_source0);
	}
        if(tab_source1)
	{
	    free(tab_source1);
	}
	if(tab_source2)
	{
	    free(tab_source2);
	}
	if(tab_source3)
	{
	    free(tab_source3);
	}
	if(tab_source4)
	{
	    free(tab_source4);
	}
	if(tab_source5)
	{
	    free(tab_source5);
	}


#if MEMSPY
	if (memspy_status())
		memspy_scan();
#endif
	exit(0);
}


/*===================================================================

	Cette fonction permet d`afficher dans une fenetre
	Affiche_Pmb et d`y representer le potentiel de membrane 
	d`un Neuron.

entrees: -fenetre       numero de la fenetre a creer.
         -*tab_source   tableau de points representant le Pmb.
	 - X et Y       valeur retournees par le pointeur souris.
	 - Debut et Fin valeur de debut et de Fin du zoom la representation
	                se fera de 0 a ITERATION si ces parametres sont Nuls.
	 - choix        si choix = Lecture_fichier alors on lit un fichier
	                si choix = Lecture_memoire alors on lit les tables deja remplies.

===================================================================*/

void CREATE_WINDOW(int fenetre , int *tab_source, int X, int Y, int Debut, int Fin, int choix)
{
	Display   *display;
	Window    window;
	GC        gc;
	int       n,i,j,MIN_IT,ITERATION,NEURON_INDEX;
	int       x_pos, y_pos;
	Arg       args[10];
	Pixel     fg, bg;
	XGCValues xgcv;
	char      MESSAGE[500];
	char      STRING[80],string1[50],string2[30];
	float     valeur;
	int       Ymax, Ymin, Tampon;
	Dimension largeur, hauteur;
	float     a,b;
	float     MAX_HEIGHT = 25.;
	float     SCALE;
	float     GRAPHIC_HEIGHT;
	FILE      *INPUT_FILE;
	char      AFF_FILE_NAME[256];

	display = XtDisplay(WIDGET_ARRAY[fenetre]);
	window = XtWindow(WIDGET_ARRAY[fenetre]);

#if 0
	switch(fenetre)
	{
	    case k_Draw_Affiche_Pmb0:
	      x_pos = 200;
	      y_pos = 500;
	      break;
	    case k_Draw_Affiche_Pmb1:
	      x_pos = 220;
	      y_pos = 520;
	      break;
	    case k_Draw_Affiche_Pmb2:
	      x_pos = 240;
	      y_pos = 540;
	      break;
	    case k_Draw_Affiche_Pmb3:
	      x_pos = 260;
	      y_pos = 560;
	      break;
	    case k_Draw_Affiche_Pmb4:
	      x_pos = 280;
	      y_pos = 580;
	      break;
	    case k_Draw_Affiche_Pmb5:
	      x_pos = 300;
	      y_pos = 600;
	      break;
	}
	n = 0;
	XtSetArg(args[n], XmNx, x_pos); n++;
	XtSetArg(args[n], XmNy, y_pos); n++;
	XtSetValues(WIDGET_ARRAY[fenetre], args, n);

	XMoveWindow(display,window,x_pos,y_pos);
#endif

	SET_WAIT_CURSOR();
	

	/* compter le nombre d`iteration a memoriser */
	R_INFO(X , Y, &NEURON_INDEX);
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? 
	  SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	ITERATION = MIN_IT + X/NETWORK.LINEAR->NEURON_WIDTH + 1;

	/* inverser les valeur si Debut>Fin */
	if ( Debut > Fin )
	  {
	    Tampon = Debut;
	    Debut = Fin; 
	    Fin = Tampon;
	  }

	/* allocation d`un tableau de points */
	if(tableau != NULL)
	  {
	    free(tableau);
	  }

	if((tableau =  (XPoint * )calloc(( Fin - Debut + 1 ), sizeof(XPoint ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Table allocation for membrane impossible\nTry later\n",ERROR);
	    return;
	  }	


	if ( choix == Lecture_Fichier )
	  {

	    /********* lecture du fichier ***********/
	    sprintf(AFF_FILE_NAME, "%s.pmb", LOAD_FILE_NAME);
	    if ((INPUT_FILE = fopen(AFF_FILE_NAME, "r")) == NULL)
	    {	    
	      /* si on ne peut pas ouvrir un fichier */
		sprintf(MESSAGE, "Unable to open\n%s\nVisualization is not possible.",
			AFF_FILE_NAME);
		DISPLAY_MESSAGE(MESSAGE, ERROR);
		return;
	    }
	    
	    /* remplissage de la table */
	    
	    i = -1; /* i = l`iteration lue */
	    
	    while (fgets(STRING, 80, INPUT_FILE)!=NULL)
	    {
		sscanf(STRING, "%s %s", string1, string2);
		
		if ( (strcmp (string2,"iteration"))==0 )
		  i++;
		else
		{
		    if( (atoi(string1))==NEURON_INDEX && i<=ITERATION  && 
		       (strcmp (string1,"\n"))!=0)
		      {
			tab_source[i] = atoi(string2);
#if 0
			printf("tab_source[%d]= %d\n",i,tab_source[i]);
#endif
		      }
		}
	    } /* fin du while */
	    
	    fclose(INPUT_FILE);
	    
	  }
	

	/* dessin du potentiel de membrane */

	n = 0;
	XtSetArg(args[n], XmNforeground, &xgcv.foreground); n++;
	XtSetArg(args[n], XmNbackground, &xgcv.background); n++;
	XtGetValues(WIDGET_ARRAY[fenetre], args, n);
	
	xgcv.foreground = xgcv.foreground ^ xgcv.background;  
	xgcv.line_style = LineOnOffDash;
	xgcv.function = GXxor;
	gc = XtGetGC(WIDGET_ARRAY[fenetre],
		     GCForeground | GCBackground | GCFunction | GCLineStyle,
		     &xgcv);
	
	n = 0;
	XtSetArg(args[n], XmNwidth, &largeur);n++;
	XtSetArg(args[n], XmNheight, &hauteur);n++;
	XtGetValues(WIDGET_ARRAY[fenetre], args, n);

	XClearArea(display, window, 0, 0, (int )largeur, (int )hauteur, False );
	
	XSetLineAttributes(display, gc, 2, LineSolid, CapButt, JoinMiter);


	Scr_Height = (int)hauteur - 40;
	Scr_Width = (int)largeur - 40;

	XDrawLine( display, window, gc, 20, 0, 20, Scr_Height);
	XDrawLine( display, window, gc, 20, Scr_Height, Scr_Width + 20, Scr_Height);

	for(i=1; i<15 ; i++)
	  {
	    a = ((float)i*((float)Scr_Width)/14.+20);
	    sprintf(string2, "%.0f",(float)(Fin*i - Debut*i)/14.+(float)Debut);
	    XDrawString(display, window, gc, (int)a, Scr_Height+20 , string2, 3);

	    XDrawLine( display, window, gc, (int)a, Scr_Height+5,
		                            (int)a, Scr_Height+1 );
	  }	

	XSetLineAttributes(display, gc, 1, LineSolid, CapButt, JoinMiter);

	/* dessin des valeurs */
	/* lecture le la table pour avoir le max et le min */
	Ymax = Ymin = 0;
	    

	for (i = 0 ; i <= ITERATION ; i++)
	  {
#if 0
	    printf("%d tab_source =%d\n",i,tab_source[i] );
#endif
	    if (Ymax < tab_source[i]) Ymax = tab_source[i];
	    if (Ymin > tab_source[i]) Ymin = tab_source[i];
	  }


	a = (float)(Ymax - Ymin)/(float)(Scr_Height);
	if (a==0.) a = 1.;

	if ((Fin - Debut)!=0)
	  b = (float)(Scr_Width)/(float)(Fin - Debut);	    
	else 
	  b=1.;

	for(i= 0 ; i<(Fin - Debut); i++)
	  {
	    tableau[i].x = (int)(b*(float)i)+20;
	    tableau[i].y = (int)( Ymax - tab_source[ Debut + i ] )/a;
	  }

	XDrawLines(display, window, gc, tableau, (Fin - Debut), CoordModeOrigin);

	XFlush(display);

	UNSET_WAIT_CURSOR();	

}

/**********************************************************

Cette fonction permet d`imprimer un potentiel de membrane
selectionne par une fenetre de type Affiche.

***********************************************************/


void PRINT_AFFICHE(int DRAW_ALL, float PAPER_WIDTH,
		   int *tab_source, int X, int Y, int Debut, int Fin )
{
	float     MAX_HEIGHT = 25.;
	float     a, b;
	float     GRAPHIC_HEIGHT;
	float     Y_STEP, STEP;
	EXPOSED_REGION REGION;
	int       i, j, IT, MIN_IT, ITERATION, NEURON_INDEX;
	char      INFO[80], STRING[250], string1[50], string2[30];
	float     Ymin,Ymax;
	char      POT_STR[250];


	R_INFO(X , Y, &NEURON_INDEX);
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? 
	  SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	ITERATION = MIN_IT + X/NETWORK.LINEAR->NEURON_WIDTH + 1;

	              
	/* allocation d`un tableau de points */

	if (Coord_X != NULL)
	  free(Coord_X);
	if((Coord_X =  (float * )calloc((Fin - Debut + 1 ), sizeof(float ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Table for printing: allocation impossible (X values)\nTry later\n", ERROR);
	    return;
	  } 	
	if (Coord_Y != NULL)
	  free(Coord_Y);
      	if((Coord_Y =  (float * )calloc((Fin - Debut + 1 ), sizeof(float ))) == NULL)
	  {
	    DISPLAY_MESSAGE("Table for printing: allocation impossible (Y values)\nTry later\n", ERROR);
	    return;
	  } 	

	/* Normalement, deja sorti... */
	if (Coord_X == NULL || Coord_Y == NULL)
	  return;


	for( i = 0 ; i<(Fin - Debut) ; i++ )
	  {
	    Coord_X[i] = i;
	    Coord_Y[i] = tab_source[ Debut + i ];
	  }


	ps_origin(1.5, 15., 0., 0.);

	ps_echel(18./(Fin - Debut), 10./(90.-(-90.)), 0., 0., 1., 0. );

	ps_user();

	/*draw values*/

	ps_pen(0);
	ps_polyline(Coord_X, Coord_Y, (Fin - Debut), False, 0, 0 );

	/* draw axis */
	ps_movea(0.,-90.);
	

	ps_ticked_axes((float)Debut, -90.,
		       (float)Debut, (float)Fin,
		       (float)Debut, (float)Fin,
		       -90., 90.,
		       -90., 90.,
		       .1, " Time (ms)","%.2f",
		       "(mV)", "%.0f", .3, 0 );


	/**** ecriture des informations sur le neuron */

	ps_movea(0.,0.);
	ps_writeah (5., 10.,"Evolution of Membrane Potential:", 0.5);

	ps_writeah (5., 9.,"Information on this Neuron:", 0.5);
	sprintf(STRING, "%s", R_INFO(X , Y, &NEURON_INDEX));
	ps_writeah (0., 8.,STRING, 0.3);

	sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", 
		SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
	ps_writeah(0., 7, STRING, 0.5);

	sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
	ps_writeah(0., 6, STRING, 0.5);

	if (Coord_X)
		free(Coord_X);
	if (Coord_Y)
		free(Coord_Y);
}



