#define DEBUG_FILE 0
/*
=============================================================================
	N E T W O R K _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- FREE_NETWORK     : This function free all allocated memory used by
			     the current network.
	- CREATE_NETWORK   : This function tries to open the given file with
			     .len .wgt, .ntw and .anat extensions and creates the 
			     NETWORK from these files.
	- UPDATE_REPRESENTATION : this function allows to the user to change 
				  the type of representation.
=============================================================================
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <Xm/Text.h>
#include <X11/X.h>

#include "global_def_visu.h"


/*
=============================================================================
Name and dates: FREE_NETWORK
---------------
	Creation : 29-04-93
	Last modification : 29-04-93

Functionnal description :
-------------------------
	This function free all the memory allocated for the current network and
	clear the drawing area.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  : None

Explicits outputs : None

Implicits inputs  : NETWORK	   NETWORK_DEF		The network definition
		    DISPLAY_VISU	Display *	The display
		    WIDGET_ARRAY	Widget		The widget array

Implicits outputs : NETWORK	   NETWORK_DEF 		The network definition

Sides effects : None
---------------

=============================================================================
*/


void FREE_NETWORK()
{
	/*free representation*/
	R_FREE(ALL);

	/*free allocated memory*/
	FREE_MEMORY(NETWORK.NAME);
	FREE_MEMORY(NETWORK.NB_NEURONS_PER_CLUSTER);
	FREE_MEMORY(NETWORK.CONNEXION_MATRIX);
	FREE_MEMORY(NETWORK.SYNAPTIC_MATRIX);
	FREE_MEMORY(NETWORK.ANATOMY_MATRIX);
	FREE_MEMORY(NETWORK.INFO_CLUSTERS);
	if(NETWORK.SELECTED_NEURON != Null)
		XmListDeleteAllItems(WIDGET_ARRAY[k_List_Selected_Neuron]);
	FREE_MEMORY(NETWORK.SELECTED_NEURON);
	
	/*Clear the drawing area*/
	XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));

	/* set insentitive representations toggle*/
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Matrix_Representation], False, False);
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Cluster_Representation], False, False);
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Anatomic_Represent], False, False);
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation], False, False);
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Global_Activity], False, False);
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Pmb_Representation], False, False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Matrix_Representation], False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Cluster_Representation], False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Anatomic_Represent], False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Linear_Representation], False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Global_Activity], False);
	XtSetSensitive(WIDGET_ARRAY[k_Button_Unselect_Neuron], False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Pmb_Representation], False);
	if(NETWORK.FLAG.REPRESENTATION == PMB_REPRESENTATION)
		NETWORK.FLAG.REPRESENTATION = LINEAR_REPRESENTATION;
	
	/*no network and no selected neurons*/
	NETWORK.FLAG.EXIST = False;
	NETWORK.NB_SELECTED_NEURON = 0;	  
}



/*
=============================================================================
Name and dates: CREATE_NETWORK
---------------
	Creation : 23-04-93
	Last modification : 23-04-93

Functionnal description :
-------------------------
	This function tries to open the given file with
	.cnx and .anat extensions and creates the NETWORK from these files.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  : 	ROOT_NAME	char *		The name of the files
							without extension

Explicits outputs :	<return>	int		Error Flag
							- 0 : Error
							- 1 : Success
			MESSAGE		char *		Error Message
							(max 500 carac)
			
			
Implicits inputs  :	WIDGET_ARRAY    Widget		The widget array
			DISPLAY_VISU    *Display	The display
			
Implicits outputs :	NETWORK		NETWORK_DEF	The head of the hierarchy
ides effects : None
---------------

=============================================================================
*/

#define MATRIX_REP	(1L<<0)
#define ANAT_REP	(1L<<1)
#define GA_REP		(1L<<2)
#define CLUSTER_REP	(1L<<3)
#define LINEAR_REP	(1L<<4)

#define LEN  0
#define WGT  1
#define ANAT 2
#define LNK  3
#define NB_FILE 4

int CREATE_NETWORK(char *ROOT_NAME, char *MESSAGE)
{
	void *TMP_PTR;
	FILE *INPUT_ANAT_FILE;
	FILE *INPUT_LNK_FILE;
	FILE *INPUT_LEN_FILE;
	FILE *INPUT_WGT_FILE;
	char *LEN_FILE_NAME;
	char *WGT_FILE_NAME;
	char *ANAT_FILE_NAME;
	char *LNK_FILE_NAME;
	char *TITLE;			/*Title of the main window*/
#if 0
	short *(TMP_NB_NEURONS_PER_CLUSTER[NB_FILE]);	/*to compare values*/
#endif
	short TMP_NB_NEURONS_PER_CLUSTER[NB_FILE][100];	/*to compare values*/
	short TMP_NB_CLUSTERS[NB_FILE];	/*to compare values*/
	FILE *INPUT_FILE[NB_FILE];
	char *FILE_NAME[NB_FILE];
	XmString XMSTRING;		/*not important stuff*/
	Arg ARGLIST[3];			/*not important stuff*/
	long REP_AVAILABLE = 0;		/*bit set => the representation is available*/
	int i, j, n;			/*not important stuff*/
	int INDEX[NB_FILE];
	int TMP_INT;
	char STRING[256];		/*not important stuff*/
	char CHAR_BUFFER[256], PARAMETRE[50], VALUE[256];
	short NEURON_ID;
	short NB_NEURONS_CONNECTED;
	short SOME_SHORT;
	short TARGET_NEURONS_ID;
	float SOME_FLOAT;
	
/*all representation are available*/
	REP_AVAILABLE |= (MATRIX_REP | LINEAR_REP | GA_REP | ANAT_REP | CLUSTER_REP);
	
/*If a network already exists we free it*/
	if(NETWORK.FLAG.EXIST)
		FREE_NETWORK();
/*
-----------------------------------------------------------------------------
Try to open .len, .wgt, .anat and .ntw files
-----------------------------------------------------------------------------
*/
	if ((LEN_FILE_NAME = (char *)calloc(sizeof(char), strlen(ROOT_NAME)+6)) == NULL)
	{
	  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
	  EXIT_VISU();
	}
	if ((WGT_FILE_NAME = (char *)calloc(sizeof(char), strlen(ROOT_NAME)+6)) == NULL)
	{
	  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
	  EXIT_VISU();
	}
	if ((ANAT_FILE_NAME = (char *)calloc(sizeof(char), strlen(ROOT_NAME)+7)) == NULL)
	{
	  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
	  EXIT_VISU();
	};
	if ((LNK_FILE_NAME = (char *)calloc(sizeof(char), strlen(ROOT_NAME)+6)) == NULL)
	{
	  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
	  EXIT_VISU();
	}

	sprintf(LEN_FILE_NAME, "%s.len", ROOT_NAME);
	sprintf(WGT_FILE_NAME, "%s.wgt", ROOT_NAME);
	sprintf(ANAT_FILE_NAME, "%s.anat", ROOT_NAME);
	sprintf(LNK_FILE_NAME, "%s.ntw", ROOT_NAME);

	INPUT_LEN_FILE = fopen(LEN_FILE_NAME, "r");
	INPUT_WGT_FILE = fopen(WGT_FILE_NAME, "r");
	INPUT_ANAT_FILE = fopen(ANAT_FILE_NAME, "r");
	if ((INPUT_LNK_FILE = fopen(LNK_FILE_NAME, "r")) == NULL)
	  {
	    /* Trying the old lnk files */
	    sprintf(LNK_FILE_NAME, "%s.lnk", ROOT_NAME);
	    INPUT_LNK_FILE = fopen(LNK_FILE_NAME, "r");
	  }

/*
-----------------------------------------------------------------------------
Verify conformity of all files and get number of clusters and number of 
neurons per cluster.
-----------------------------------------------------------------------------
*/
/*if we can't open files*/
	/*if we can't open any file*/
	if(INPUT_ANAT_FILE == Null &&
	   INPUT_LNK_FILE == Null &&
	   INPUT_LEN_FILE == Null &&
	   INPUT_WGT_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to read\n%s,\n%s,\n%s,\n%s.\nNo representation is available.",
			LNK_FILE_NAME,LEN_FILE_NAME,WGT_FILE_NAME, ANAT_FILE_NAME);
		REP_AVAILABLE = 0;
		free(LEN_FILE_NAME);
		free(WGT_FILE_NAME);
		free(ANAT_FILE_NAME);
		free(LNK_FILE_NAME);
		return(False);
	}
	if(INPUT_LEN_FILE == Null ||
	   INPUT_WGT_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to open %s or %s.\nThe Matrix representation is unavailable.",
			LEN_FILE_NAME, WGT_FILE_NAME);
		DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
		REP_AVAILABLE = REP_AVAILABLE & ~MATRIX_REP;
	}
	if(INPUT_ANAT_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to open %s.\nThe Anatomic representation is unavailable.",
			ANAT_FILE_NAME);
		DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
		REP_AVAILABLE = REP_AVAILABLE & ~ANAT_REP;
	}
	if(INPUT_LNK_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to open %s.\nThe Cluster representation is unavailable.",
			LNK_FILE_NAME);
		DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
		REP_AVAILABLE = REP_AVAILABLE & ~CLUSTER_REP;
	}
	else
	{
		/*test if we have a link configuration file*/
		fgets(CHAR_BUFFER, 255, INPUT_LNK_FILE);
		sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
		if(strcmp(PARAMETRE, "TypeOfFile:") ||
		   strcmp(VALUE, "LinkConfigurationFileVer2"))
		{
			sprintf(MESSAGE, "%s is not a link configuration file.\n\
				The Cluster representation is unavailable.",
				LNK_FILE_NAME);
			DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
			REP_AVAILABLE = REP_AVAILABLE & ~CLUSTER_REP;
			fclose(INPUT_LNK_FILE);
			INPUT_LNK_FILE = Null;
		}
	}
/*Read number of clusters and neurons in each file*/
	if(INPUT_LEN_FILE != Null &&
	   INPUT_WGT_FILE != Null)
	{
		/*Number of cluster*/
		fread(&(TMP_NB_CLUSTERS[LEN]), sizeof(short), 1, INPUT_LEN_FILE);
		fread(&(TMP_NB_CLUSTERS[WGT]), sizeof(short), 1, INPUT_WGT_FILE);

		/*Number of neurons per clusters*/
/*
		TMP_NB_NEURONS_PER_CLUSTER[LEN] = (short *)calloc(sizeof(short), 
						  (size_t)TMP_NB_CLUSTERS[LEN]);
		TMP_NB_NEURONS_PER_CLUSTER[WGT] = (short *)calloc(sizeof(short), 
						  (size_t)TMP_NB_CLUSTERS[WGT]);
*/
		fread(TMP_NB_NEURONS_PER_CLUSTER[LEN], sizeof(short), (size_t)TMP_NB_CLUSTERS[LEN],
		      INPUT_LEN_FILE);
		fread(TMP_NB_NEURONS_PER_CLUSTER[WGT], sizeof(short), (size_t)TMP_NB_CLUSTERS[WGT],
		      INPUT_WGT_FILE);
	}
	if(INPUT_ANAT_FILE != Null)
	{
		/*Number of cluster*/
		fread(&(TMP_NB_CLUSTERS[ANAT]), sizeof(short), 1, INPUT_ANAT_FILE);

		/*Number of neurons per clusters*/
/*
		TMP_NB_NEURONS_PER_CLUSTER[ANAT] = (short *)calloc(sizeof(short), 
						   (size_t)TMP_NB_CLUSTERS[ANAT]);
*/
		fread(TMP_NB_NEURONS_PER_CLUSTER[ANAT], sizeof(short), (size_t)TMP_NB_CLUSTERS[ANAT],
		      INPUT_ANAT_FILE);
	}
	if(INPUT_LNK_FILE != Null)
	{
		do
		{
			fgets(CHAR_BUFFER, 255, INPUT_LNK_FILE);
			sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
		} while (strcmp(PARAMETRE,"NumberOfNucleus:")); 	/*Number of cluster*/
		TMP_NB_CLUSTERS[LNK] = (short)atoi(VALUE);
		/*Number of neurons per clusters*/
/*
		TMP_NB_NEURONS_PER_CLUSTER[LNK] = (short *)calloc(sizeof(short), 
						   (size_t)TMP_NB_CLUSTERS[LNK]);
*/
		i=0;
		while(fgets(CHAR_BUFFER, 255, INPUT_LNK_FILE))
		{
			sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
			if(!strcmp(PARAMETRE, "NbNeurons:"))
			{
				TMP_NB_NEURONS_PER_CLUSTER[LNK][i] = (short)atoi(VALUE);
				i++;
			}
		}
	}
#if DEBUG_FILE
	printf("files read 1\n");
#endif

/*
-----------------------------------------------------------------------------
Compare values between each files.
-----------------------------------------------------------------------------
*/
	/*all files are available*/
	if(INPUT_LEN_FILE != Null &&
	   INPUT_WGT_FILE != Null &&
	   INPUT_ANAT_FILE != Null &&
	   INPUT_LNK_FILE != Null)
	{
#if DEBUG
		printf("LEN: %d ANAT: %d WGT: %d LNK: %d\n",
		       TMP_NB_CLUSTERS[LEN],
		       TMP_NB_CLUSTERS[ANAT],
		       TMP_NB_CLUSTERS[WGT],
		       TMP_NB_CLUSTERS[LNK]);
#endif
		if(TMP_NB_CLUSTERS[LEN] != TMP_NB_CLUSTERS[ANAT]  ||
	 	   TMP_NB_CLUSTERS[WGT] != TMP_NB_CLUSTERS[ANAT]  ||
	 	   TMP_NB_CLUSTERS[LEN] != TMP_NB_CLUSTERS[LNK]  ||
	 	   TMP_NB_CLUSTERS[WGT] != TMP_NB_CLUSTERS[LNK]  ||
	   	   TMP_NB_CLUSTERS[LNK] != TMP_NB_CLUSTERS[ANAT])
		{
			sprintf(MESSAGE, "ATTENTION:\n%s,\n%s,\n%s,\n%s \n hasn't same parameters!.\n\
				No representation is available.",
				LNK_FILE_NAME, LEN_FILE_NAME, WGT_FILE_NAME, ANAT_FILE_NAME);
			REP_AVAILABLE = 0;
			free(LEN_FILE_NAME);
			free(WGT_FILE_NAME);
			free(ANAT_FILE_NAME);
			free(LNK_FILE_NAME);
			return(False);
		}
#if DEBUG_FILE
	printf("files read 2\n");
#endif
	
		/*copy value*/
		NETWORK.NB_CLUSTERS = TMP_NB_CLUSTERS[LEN];

		for(i=0; i < NETWORK.NB_CLUSTERS; i++)
			if(TMP_NB_NEURONS_PER_CLUSTER[LEN][i] != TMP_NB_NEURONS_PER_CLUSTER[ANAT][i]  ||
	   	   	   TMP_NB_NEURONS_PER_CLUSTER[WGT][i] != TMP_NB_NEURONS_PER_CLUSTER[ANAT][i]  ||
	   	   	   TMP_NB_NEURONS_PER_CLUSTER[LEN][i] != TMP_NB_NEURONS_PER_CLUSTER[LNK][i]  ||
	   	   	   TMP_NB_NEURONS_PER_CLUSTER[WGT][i] != TMP_NB_NEURONS_PER_CLUSTER[LNK][i]  ||
	   	   	   TMP_NB_NEURONS_PER_CLUSTER[LNK][i] != TMP_NB_NEURONS_PER_CLUSTER[ANAT][i])
			{
				sprintf(MESSAGE, "%s,\n%s,\n%s,\n%s \n hasn't same parameters!.\n\
					No representation is available.",
					LNK_FILE_NAME, WGT_FILE_NAME, LEN_FILE_NAME, ANAT_FILE_NAME);
				REP_AVAILABLE = 0;
				free(LEN_FILE_NAME);
				free(WGT_FILE_NAME);
				free(ANAT_FILE_NAME);
				free(LNK_FILE_NAME);
				return(False);
			}
	
		/*copy values*/
		if ((NETWORK.NB_NEURONS_PER_CLUSTER = (short *)calloc(sizeof(short), 
						 (size_t)NETWORK.NB_CLUSTERS)) == NULL)
		{
		  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
		  EXIT_VISU();
		};
		memcpy((short *)NETWORK.NB_NEURONS_PER_CLUSTER, (short *)TMP_NB_NEURONS_PER_CLUSTER[LEN],
		       sizeof(short)/sizeof(char) * (size_t)NETWORK.NB_CLUSTERS);
/*
		for(i=0; i < NETWORK.NB_CLUSTERS; i++)
		{
			free(TMP_NB_NEURONS_PER_CLUSTER[LEN]);
			free(TMP_NB_NEURONS_PER_CLUSTER[WGT]);
			free(TMP_NB_NEURONS_PER_CLUSTER[ANAT]);
			free(TMP_NB_NEURONS_PER_CLUSTER[LNK]);
		}	
*/
	}
#if DEBUG_FILE
	printf("files read 3\n");
#endif
	
	/*two files are available*/
	INPUT_FILE[LEN] = INPUT_LEN_FILE;
	INPUT_FILE[WGT] = INPUT_WGT_FILE;
	INPUT_FILE[ANAT] = INPUT_ANAT_FILE;
	INPUT_FILE[LNK] = INPUT_LNK_FILE;
	FILE_NAME[LEN] = LEN_FILE_NAME;
	FILE_NAME[WGT] = WGT_FILE_NAME;
	FILE_NAME[ANAT] = ANAT_FILE_NAME;
	FILE_NAME[LNK] = LNK_FILE_NAME;
	INDEX[LEN] = LEN;
	INDEX[WGT] = WGT;
	INDEX[ANAT] = ANAT;
	INDEX[LNK] = LNK;
	for(j=0; j<NB_FILE; j++)
	{
		if(INPUT_FILE[0] != Null &&
	   	   INPUT_FILE[1] != Null &&
	   	   INPUT_FILE[2] == Null)
		{
			if(TMP_NB_CLUSTERS[INDEX[0]] != TMP_NB_CLUSTERS[INDEX[1]])
			{
				sprintf(MESSAGE, "%s,\n%s hasn't same parameters!.\n\
					No representation is available.",
					FILE_NAME[0], FILE_NAME[1]);
				REP_AVAILABLE = 0;
				free(LEN_FILE_NAME);
				free(WGT_FILE_NAME);
				free(ANAT_FILE_NAME);
				free(LNK_FILE_NAME);
				return(False);
			}
	
			/*copy value*/
			NETWORK.NB_CLUSTERS = TMP_NB_CLUSTERS[INDEX[0]];

			for(i=0; i < NETWORK.NB_CLUSTERS; i++)
				if(TMP_NB_NEURONS_PER_CLUSTER[INDEX[0]][i] != 
			   	   TMP_NB_NEURONS_PER_CLUSTER[INDEX[1]][i])
				{
					sprintf(MESSAGE, "%s,\n%s hasn't same parameters!.\n\
						No representation is available.",
						FILE_NAME[0], FILE_NAME[1]);
					REP_AVAILABLE = 0;
					free(LEN_FILE_NAME);
					free(WGT_FILE_NAME);
					free(ANAT_FILE_NAME);
					free(LNK_FILE_NAME);
					return(False);
				}
	
			/*copy values*/
			if ((NETWORK.NB_NEURONS_PER_CLUSTER = (short *)calloc(sizeof(short), 
						 	 (size_t)NETWORK.NB_CLUSTERS)) == NULL)
			{
			  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
			  EXIT_VISU();
			}
			memcpy((short *)NETWORK.NB_NEURONS_PER_CLUSTER, 
		       	       (short *)TMP_NB_NEURONS_PER_CLUSTER[INDEX[0]],
	       	       	       sizeof(short)/sizeof(char) * (size_t)NETWORK.NB_CLUSTERS);
/*
			for(i=0; i < NETWORK.NB_CLUSTERS; i++)
			{
				free(TMP_NB_NEURONS_PER_CLUSTER[INDEX[0]]);
				free(TMP_NB_NEURONS_PER_CLUSTER[INDEX[1]]);
			}
*/
		}	
		TMP_INT = INDEX[0];
		INDEX[0] = INDEX[1];
		INDEX[1] = INDEX[2];
		INDEX[2] = TMP_INT;
		TMP_PTR = FILE_NAME[0];
		FILE_NAME[0] = FILE_NAME[1];
		FILE_NAME[1] = FILE_NAME[2];
		FILE_NAME[2] = (char *)TMP_PTR;
		TMP_PTR = INPUT_FILE[0];
		INPUT_FILE[0] = INPUT_FILE[1];
		INPUT_FILE[1] = INPUT_FILE[2];
		INPUT_FILE[2] = (FILE *)TMP_PTR;
	}
#if DEBUG_FILE
	printf("files read 4\n");
#endif

	/*only one file is available*/
	if(INPUT_LEN_FILE != NULL &&
	   INPUT_WGT_FILE != NULL &&
	   INPUT_LNK_FILE == NULL &&
	   INPUT_ANAT_FILE == NULL)
	{
		/*copy value*/
		NETWORK.NB_CLUSTERS = TMP_NB_CLUSTERS[LEN];
		/*copy values*/
		if ((NETWORK.NB_NEURONS_PER_CLUSTER = (short *)calloc(sizeof(short), 
						 (size_t)NETWORK.NB_CLUSTERS)) == NULL)
		{
		  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
		  EXIT_VISU();
		}

		memcpy((short *)NETWORK.NB_NEURONS_PER_CLUSTER, (short *)TMP_NB_NEURONS_PER_CLUSTER[LEN],
		       sizeof(short)/sizeof(char) * (size_t)NETWORK.NB_CLUSTERS);
/*
		free(TMP_NB_NEURONS_PER_CLUSTER[LEN]);
		free(TMP_NB_NEURONS_PER_CLUSTER[WGT]);
*/
	}
	if(INPUT_LNK_FILE != NULL &&
	   INPUT_LEN_FILE != NULL &&
	   INPUT_WGT_FILE != NULL &&
	   INPUT_ANAT_FILE == NULL)
	{
		/*copy value*/
		NETWORK.NB_CLUSTERS = TMP_NB_CLUSTERS[LNK];
		/*copy values*/
		if ((NETWORK.NB_NEURONS_PER_CLUSTER = (short *)calloc(sizeof(short), 
						 (size_t)NETWORK.NB_CLUSTERS)) == NULL)
		{
		  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
		  EXIT_VISU();
	       	}

		memcpy((short *)NETWORK.NB_NEURONS_PER_CLUSTER, (short *)TMP_NB_NEURONS_PER_CLUSTER[LNK],
	       	sizeof(short)/sizeof(char) * (size_t)NETWORK.NB_CLUSTERS);
/*
		free(TMP_NB_NEURONS_PER_CLUSTER[LNK]);
*/
	}
	if(INPUT_LEN_FILE == NULL &&
	   INPUT_WGT_FILE == NULL &&
	   INPUT_LNK_FILE == NULL &&
	   INPUT_ANAT_FILE != NULL)
	{
		/*copy value*/
		NETWORK.NB_CLUSTERS = TMP_NB_CLUSTERS[ANAT];
		/*copy values*/
		if ((NETWORK.NB_NEURONS_PER_CLUSTER = (short *)calloc(sizeof(short), 
						 (size_t)NETWORK.NB_CLUSTERS)) == NULL)
		{
		  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
		  EXIT_VISU();
		}

		memcpy((short *)NETWORK.NB_NEURONS_PER_CLUSTER, (short *)TMP_NB_NEURONS_PER_CLUSTER[ANAT],
	       	sizeof(short)/sizeof(char) * (size_t)NETWORK.NB_CLUSTERS);
/*
		free(TMP_NB_NEURONS_PER_CLUSTER[ANAT]);
*/
	}
#if DEBUG_FILE
	printf("files read 5\n");
#endif

/*
-----------------------------------------------------------------------------
Initialize the network
-----------------------------------------------------------------------------
*/
	/*total of neurons*/
	for(i = 0, NETWORK.NB_NEURONS = 0; 
	    i < NETWORK.NB_CLUSTERS; 
	    NETWORK.NB_NEURONS += NETWORK.NB_NEURONS_PER_CLUSTER[i++]);

	/*alloc memory for synaptic and connexion matrix*/
	if(INPUT_LEN_FILE != NULL &&
	   INPUT_WGT_FILE != NULL)
	{
		NETWORK.CONNEXION_MATRIX = (short *)calloc(sizeof(short),
					   (size_t)(NETWORK.NB_NEURONS*NETWORK.NB_NEURONS));

		NETWORK.SYNAPTIC_MATRIX = (short *)calloc(sizeof(short),
					  (size_t)(NETWORK.NB_NEURONS*NETWORK.NB_NEURONS));

		if(NETWORK.CONNEXION_MATRIX == Null || NETWORK.SYNAPTIC_MATRIX == Null)
		{
			sprintf(MESSAGE, "Not enough memory to load the network for\n\
				the matrix representation.\nIt's unavailable.");
			DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
			REP_AVAILABLE = REP_AVAILABLE & ~MATRIX_REP;
		}
		else /*read matrix*/
		{			
			/* get the lenth data */
			for(i=0; i<NETWORK.NB_NEURONS; i++)
			{
				fread(&NEURON_ID, sizeof(short), 1, INPUT_LEN_FILE);
				fread(&SOME_SHORT, sizeof(short), 1, INPUT_LEN_FILE);
				fread(&NB_NEURONS_CONNECTED, sizeof(short), 1, INPUT_LEN_FILE);
				for (j = 0; j < NB_NEURONS_CONNECTED; j++)
				{
					fread(&TARGET_NEURONS_ID, sizeof(short), 1, INPUT_LEN_FILE);
					/*synaptic knobs*/
					fread(&SOME_FLOAT, sizeof(float), 1, INPUT_LEN_FILE);
					if (SOME_FLOAT < 1.) SOME_FLOAT = 1.;
					NETWORK.CONNEXION_MATRIX [(NEURON_ID - 1)* NETWORK.NB_NEURONS + (TARGET_NEURONS_ID - 1)]
						= (short) SOME_FLOAT;
				}
			}
			/* get the weight data */
			for(i=0; i<NETWORK.NB_NEURONS; i++)
			{
				fread(&NEURON_ID, sizeof(short), 1, INPUT_WGT_FILE);
				fread(&SOME_SHORT, sizeof(short), 1, INPUT_WGT_FILE);
				fread(&NB_NEURONS_CONNECTED, sizeof(short), 1, INPUT_WGT_FILE);
				for (j = 0; j < NB_NEURONS_CONNECTED; j++)
				{
					fread(&TARGET_NEURONS_ID, sizeof(short), 1, INPUT_WGT_FILE);
					/*connexions*/
					fread(&SOME_SHORT, sizeof(short), 1, INPUT_WGT_FILE);
					fread(&SOME_SHORT, sizeof(short), 1, INPUT_WGT_FILE);
					NETWORK.SYNAPTIC_MATRIX [(NEURON_ID - 1) * NETWORK.NB_NEURONS + (TARGET_NEURONS_ID - 1)]
						= SOME_SHORT;
				}
			}
		}
		/*close cnx file*/
		fclose(INPUT_LEN_FILE);	
		fclose(INPUT_WGT_FILE);	
	}
#if DEBUG_FILE
	printf("Allocation synapses OK\n");
#endif

	/*alloc memory for anatomy matrix*/
	if(INPUT_ANAT_FILE != NULL)
	{
		NETWORK.ANATOMY_MATRIX = (NEURON_ANATOMY *)calloc(sizeof(NEURON_ANATOMY),
					 (size_t)NETWORK.NB_NEURONS);
		if(NETWORK.ANATOMY_MATRIX == Null)
		{
			sprintf(MESSAGE, "Not enough memory to load the network for\n\
				the anatomic representation.\nIt's unavailable.");
			DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
			REP_AVAILABLE = REP_AVAILABLE & ~ANAT_REP;
		}
		else /*read matrix*/
			fread(NETWORK.ANATOMY_MATRIX, sizeof(NEURON_ANATOMY),
			      (size_t)NETWORK.NB_NEURONS, INPUT_ANAT_FILE);
		/*close anat file*/
		fclose(INPUT_ANAT_FILE);
	}
#if DEBUG_FILE
	printf("Allocation anatomy OK\n");
#endif

	/*alloc memory for cluster representation*/
	if(INPUT_LNK_FILE != NULL)
	{
		NETWORK.INFO_CLUSTERS = (CLUSTER_INFO *)calloc(sizeof(CLUSTER_INFO),
					 (size_t)NETWORK.NB_CLUSTERS);
		if(NETWORK.INFO_CLUSTERS == Null)
		{
			sprintf(MESSAGE, "Not enough memory to load the network for\n\
				the cluster representation.\nIt's unavailable.");
			DISPLAY_MESSAGE(MESSAGE, INFORMATIONAL);
			REP_AVAILABLE = REP_AVAILABLE & ~CLUSTER_REP;
		}
		else /*read info*/
		{
			/*return to the beginning of file*/
			rewind(INPUT_LNK_FILE);
			/*reread values*/
			fgets(CHAR_BUFFER, 100, INPUT_LNK_FILE);
			fgets(CHAR_BUFFER, 100, INPUT_LNK_FILE);
			/*read info*/
			i=0;
			while(fgets(CHAR_BUFFER, 100, INPUT_LNK_FILE))
			{
				sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
				if(!strcmp(PARAMETRE, "PlacementMode:"))
				{
					if(!strcmp(VALUE, "MANUAL_MODE"))
						NETWORK.PLACEMENT = MANUAL;
					else
						NETWORK.PLACEMENT = AUTOMATIC;
				}
				else if(!strcmp(PARAMETRE, "ClusterName:"))
				{
					sscanf(VALUE, "%s", NETWORK.INFO_CLUSTERS[i].NAME);
					fgets(CHAR_BUFFER, 100, INPUT_LNK_FILE);
					sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
					sscanf(VALUE, "%d", &(NETWORK.INFO_CLUSTERS[i].POS_X));
					fgets(CHAR_BUFFER, 100, INPUT_LNK_FILE);
					sscanf(CHAR_BUFFER, "%s %s", PARAMETRE, VALUE);
					sscanf(VALUE, "%d", &(NETWORK.INFO_CLUSTERS[i].POS_Y));
					i++;
				}
			}		
		}
		/*close lnk file*/
		fclose(INPUT_LNK_FILE);
	}
#if DEBUG_FILE
	printf("Allocation clusters OK\n");
#endif

	NETWORK.NAME= (char *)calloc(sizeof(char), 256);
	/*Keep the network name*/
	strcpy(NETWORK.NAME, ROOT_NAME);

	/*keep the window id*/
	NETWORK.WINDOW_ID = XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]);

	/*
	 * initialize to adress 0 the different representations in order to be
	 * able to know if the pointer was allocated
	 */
	NETWORK.LINEAR = (void *)0;
	NETWORK.GA = (void *)0;
	NETWORK.MATRIX = (void *)0;
	NETWORK.PMB = (void *)0;

	/*Initialize Main Window Title*/
	n=0;
	if ((TITLE = (char *)calloc(sizeof(char), strlen(ROOT_NAME)+30)) == NULL)
	{
	  DISPLAY_MESSAGE("Impossible to allocate memory, try later...\n",ERROR);
	  EXIT_VISU();
	}
	sprintf(TITLE, "XNBC: Visualization V%s: %s", VERSION_VISU, ROOT_NAME);
	XtSetArg(ARGLIST[n], XmNtitle, TITLE); n++;
	XtSetValues(TOPLEVEL_WIDGET_VISU, ARGLIST, n);

#if DEBUG_FILE
	printf("Title done\n");
#endif

	/*updates network informations*/
	sprintf(STRING, "-Clusters: %d", NETWORK.NB_CLUSTERS);
	XMSTRING = XmStringCreateSimple(STRING);
	n=0;
	XtSetArg(ARGLIST[n], XmNlabelString, XMSTRING); n++;
	XtSetValues(WIDGET_ARRAY[k_Label_Nb_Clusters], ARGLIST, n);
	XmStringFree(XMSTRING);

#if DEBUG_FILE
	printf("-Clusters info done\n");
#endif
	sprintf(STRING, "-Neurons: %d", NETWORK.NB_NEURONS);
	XMSTRING = XmStringCreateSimple(STRING);
	n=0;
	XtSetArg(ARGLIST[n], XmNlabelString, XMSTRING); n++;
	
	XtSetValues(WIDGET_ARRAY[k_Label_Nb_Neurons], ARGLIST, n);
	
	XmStringFree(XMSTRING);
	free(TITLE);

#if DEBUG_FILE
	printf("-Neuron info done\n");
#endif
	/*set sensitive representations toggle*/
	XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation], True, False);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Linear_Representation], True);
	XtSetSensitive(WIDGET_ARRAY[k_Toggle_Global_Activity], True);

	if((REP_AVAILABLE & MATRIX_REP) == MATRIX_REP)
		XtSetSensitive(WIDGET_ARRAY[k_Toggle_Matrix_Representation], True);

	/*Now a network exists*/
#if DEBUG_FILE
	printf("Network exists 0\n");
#endif
	NETWORK.FLAG.EXIST = True;
#if DEBUG_FILE
	printf("Network exists 00\n");
#endif
	/*Initialize the geometry*/
	if(!UPDATE_REPRESENTATION())
	{
		sprintf(MESSAGE, "Not enough memory to draw representation for %s",
			ROOT_NAME);
		FREE_NETWORK();
		if (LEN_FILE_NAME) free(LEN_FILE_NAME);
		if (WGT_FILE_NAME) free(WGT_FILE_NAME);
		if (ANAT_FILE_NAME) free(ANAT_FILE_NAME);
		if (LNK_FILE_NAME) free(LNK_FILE_NAME);
#if DEBUG_FILE
	printf("Network exists 1\n");
#endif
		return(False);
	}


	if (LEN_FILE_NAME) free(LEN_FILE_NAME);
	if (WGT_FILE_NAME) free(WGT_FILE_NAME);
	if (ANAT_FILE_NAME) free(ANAT_FILE_NAME);
	if (LNK_FILE_NAME) free(LNK_FILE_NAME);

#if DEBUG_FILE
	printf("Network exists 2\n");
#endif

	return(True);
} 


/*
=============================================================================
Name and dates: UPDATE_REPRESENTATION
---------------
	Creation : 23-04-93
	Last modification : 23-04-93

Functionnal description :
-------------------------
	This function permits to change the type of representation. If a simulation
	is running, the network is diplayed with the new representation at the
	current iteration.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  : 	None

Explicits outputs :	<return>	int		False => the representation can't be displayed
			
			
Implicits inputs  :	WIDGET_ARRAY    Widget		The widget array
			DISPLAY_VISU    *Display	The display
			NETWORK		NETWORK_DEF	The head of the hierarchy

Implicits outputs :	NETWORK		NETWORK_DEF	The head of the hierarchy

Sides effects : None
---------------

=============================================================================
*/

int UPDATE_REPRESENTATION()
{
	int TAG = k_Button_Goto_Index_Main;

	if(!NETWORK.FLAG.EXIST)
		return(True);

	/*Clear the drawing area*/
	XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));

	/*free the current representation*/
	if(NETWORK.ACTION.FREE != Null)
		R_FREE(ALL);

	/*Sets the new representation*/
	switch(NETWORK.FLAG.REPRESENTATION)
	{
		case MATRIX_REPRESENTATION :
			/*sets functions*/
			NETWORK.ACTION.REALIZE = REALIZE_MATRIX;
			NETWORK.ACTION.DRAW = DRAW_MATRIX;
			NETWORK.ACTION.DRAW_NEURONS_VALUES = DRAW_NEURONS_VALUES_MATRIX;
			NETWORK.ACTION.PRINT = PRINT_MATRIX;
			NETWORK.ACTION.FREE = FREE_MATRIX;
			NETWORK.ACTION.INFO = INFO_MATRIX;
			break;

		case LINEAR_REPRESENTATION :
			/*sets functions*/
			NETWORK.ACTION.REALIZE = REALIZE_LINEAR;
			NETWORK.ACTION.DRAW = DRAW_LINEAR;
			NETWORK.ACTION.DRAW_NEURONS_VALUES = DRAW_NEURONS_VALUES_LINEAR;
			NETWORK.ACTION.PRINT = PRINT_LINEAR;
			NETWORK.ACTION.FREE = FREE_LINEAR;
			NETWORK.ACTION.INFO = INFO_LINEAR;
			break;

		case CLUSTER_REPRESENTATION :
			/*sets functions*/
			NETWORK.ACTION.REALIZE = REALIZE_CLUSTER;
			break;

		case GA_REPRESENTATION :
			/*sets functions*/
			NETWORK.ACTION.REALIZE = REALIZE_GA;
			NETWORK.ACTION.DRAW = DRAW_GA;
			NETWORK.ACTION.DRAW_NEURONS_VALUES = DRAW_NEURONS_VALUES_GA;
			NETWORK.ACTION.PRINT = PRINT_GA;
			NETWORK.ACTION.FREE = FREE_GA;
			NETWORK.ACTION.INFO = INFO_GA;
			break;
		case PMB_REPRESENTATION :
			/*sets functions*/
			NETWORK.ACTION.REALIZE = REALIZE_PMB;
			NETWORK.ACTION.DRAW = DRAW_PMB;
			NETWORK.ACTION.DRAW_NEURONS_VALUES = DRAW_NEURONS_VALUES_PMB;
			NETWORK.ACTION.PRINT = PRINT_PMB;
			NETWORK.ACTION.FREE = FREE_PMB;
			NETWORK.ACTION.INFO = INFO_PMB;
			break;
	}

	/*realize the representation*/
	if(!R_REALIZE())
		return(False);
	
	/*draw the representation*/
	if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION)
		R_DRAW();	/*draw representation*/
	else
		SIMUL_ACTION(Null, (XtPointer)TAG, Null); /*draw representation at the current iteration*/

	/*everything is ok*/
	return(True);
}



