#ifndef PLOT_PS_H
#define PLOT_PS_H
/*
 * define here your SYSTEM and PostScript queue name
 *
 *	ATTENTION Only one must be at 1
 */

#define	UNIX	1
#define	VMS	0
#define	MS_DOS	0
#if UNIX
#	define PLOT_PS_QUEUE_NAME	("lpr -Ppostscript")
#endif
#if MS_DOS
#	define PLOT_PS_QUEUE_NAME	("postscript")
#endif
#if VMS
#	define PLOT_PS_QUEUE_NAME	("PRINT /QUEUE=LN03_PS")
#endif

/*
 *		P L O T _ P S . H
 *
 *
 *	Fichier entete des fonctions plot_ps
 *
 *		J-F Vibert
 *
 *	Version 0 was in fact PLOT_BENSON
 *	Version 1	sometime during winter 88...
 *	Version 2	juillet 90
 *	Version 3	31 juillet 90
 *	Version 3.1	6 oct 90
 *	Version 4	22 avril 91 (axes and ps_write, and colors)	 
 *	Version 4.1	26 avril 91 (ps_where)	 
 *	Version 4.2	11 oct 91 (variables globales externe dans le client, 
 *				afin de pouvoir	etre utilisable en Pascal)
 *	Version 4.3	29 aout 92 (ps_queue_name pour changer la queue par defaut)
 *	Version 4.4	9 juin 93 (ps_filled_area pour une zone coloree remplie
 *	Version 4.5	25 Nov 93 Chgt de taille du tableau de noms de hosts
 *	Version 5.0	12 juillet 94 Mise en flottant des coordonnees pour 600 dpi et +
 *	Version 5.1	21 avril 1995 ps_dash_line et ps_solid_line
 *	Version 5.2	3 mai 1995 date VMS
 *	...
 *	Version 5.7	1 aout 1997 Tres longs noms de fichiers
 *	Version 6.0	19 janvier 1999 Prototypage
 *	Version 6.1	1 sept 1999 prise en compte de PRINT_PLOT_PS (JFV)
 *	Version 6.2	6 oct 1999 Redefinition de M_PI (pas defini sur HPUX...) (JFV)
 *	Version 6.3	22 mar 2001 New function: ps_polydots (JFV)
 *      Version 6.4     5 apr 2001 New function ps_rectangle + void or int function types to avoid warnings (JFV)
 *
*	Auteur: Dr J-F VIBERT
*		B3E INSERM U263
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
*/
#ifdef	PLOT_PS
#	define	PS_CLASS	
#else
#	define	PS_CLASS extern
#endif
#define	PS_VERSION	"6.4"


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>

#ifndef M_PI
#  define M_PI 3.14159265358979323846
#endif

/*
 *	Fichier de definition des variables utilise par le traceur
 */
#define PL_PAS	28.3464566929134 /* Il y a 28.346... pas par centimetre	*/
#define X_PAS   28.3464566929134
#define MAX_PLT	200		/* longeur string maxi			*/
#define LL_X	15.		/* lower left X de la Bounding Box	*/
#define LL_Y	72.		/* lower left Y de la Bounding Box	*/
#define UR_X	576.		/* upper right X de la Bounding Box	*/
#define UR_Y	756.		/* upper right Y de la Bounding Box	*/
/*
 *	Fichier de travail
 */
PS_CLASS	FILE	*pl_spl;	/* Fichier de travail			*/

PS_CLASS	float	pl_rex,		/* Echelle exprimee en reel		*/
		pl_rey,
		hl_crte,	/* hauteur des lettres courante		*/
		pl_X_crt,	/* Position courante de la plume en cm  */
		pl_Y_crt,
		pl_xs,		/* Origine sujet en cm.			*/
		pl_ys,
		pl_ca,		/* Rotation de l'espace Sujet		*/
		pl_sa,
/*
 *	Espace Absolu 
 */
		pl_x,		/* Position de la plume en pas		*/
		pl_y,
		ex_x,		/* Position actuelle de la plume en pas	*/
		ex_y,
/*
 * BoundingBox
 */
		b_b_ll_x,	/* bounding box lower left X	*/
		b_b_ll_y,	/* bounding box lower left Y	*/
		b_b_ur_x,	/* bounding box upper right X	*/
		b_b_ur_y,	/* bounding box upper right Y	*/
/*
 *	Espace Sujet
 */
		pl_ex,		/* Echelle : Nombre de pas / Unite U	*/
		pl_ey;

PS_CLASS	int	pl_byt,		/* Nombre de caracteres dans le buffer	*/
		ps_page_banner 	/* Banniere de page (default to True) 			*/
	/* Changed by BaCh (bach@mwgdna.com): 
	   I don't want banners on the pages */
	/* and then JFV with the ps_page_banner flag */

#ifdef PUT_BANNER_ON_PAGES 
#  ifdef PLOT_PS
                = 1
#  endif
#endif                
                ,
		pl_ps_page,	/* page courante			*/
		pl_plume,	/* Plume utilisee : 0			*/
		pl_mode,	/* Mode de travail du traceur		*/
		pl_cms,		/* Flag de travail en espace cm		*/
		draw;		/* On est en train de tracer 		*/

PS_CLASS	char	pl_buf[MAX_PLT],/* Buffer de travail			*/
		pl_police[50],	/* Police courante			*/
		plot_file[256],	/* nom du fichier de plot		*/
		queue_name[50];	/* nom de la queue d'impression		*/
/*
 *	Code des deplacements de la plume
 */
#define PL_AH 0			/* Absolu Plume Haute	*/
#define PL_AB 1			/* Abslou Plume Basse	*/
#define PL_RH 2			/* Relatif Plume Haute	*/
#define PL_RB 3			/* Relatif Plume Basse	*/

#define PEN_AU 0		/* Absolute Pen Up	*/
#define PEN_AD 1		/* Absolute Pen Down	*/
#define PEN_RU 2		/* Relative Pen Up	*/
#define PEN_RD 3		/* Relative Pen Down	*/

/*
 * couleurs colors
 */

#define NOIR	0.
#define BLACK	0.
#define BLANC   1.
#define WHITE   1.

/*
 * Parametres 3D
 */
PS_CLASS float	ps_3D_angle,	/* Angle de rotation autour des Y */
		ps_3D_phi,	/* Angle de rotation autour des X */
		cps_3D_angle,	/* Cos de Angle de rotation autour des Y */
		cps_3D_phi,	/* Cos de Angle de rotation autour des X */
		sps_3D_angle,	/* Sin Angle de rotation autour des Y */
		sps_3D_phi,	/* Sin Angle de rotation autour des X */
		ps_3D_dist_1,	/* distance objet-ecran */
		ps_3D_dist_2;	/* distance oeil-ecran */

#define BOOLEAN	int
#define	FALSE	0
#define	TRUE	1


/*
 * Prototypage des fonctions
 */

void tras(float , float , int );
void traa(float , float , int );
int p_depl(float , float , int );
int p_puts(char *);
void ps_banner(void);
void ps_no_banner(void);
void plopen(char *);
void plclose(char *);
void ps_iniplot(char *, float);
void ps_endplot(char *, char);
void ps_queue_name(char *);
void ps_flush(void);
void ps_newpage(void);
int ps_echel(float , float, float, float, float, float);
int ps_pen(int );
void ps_scale(float, float);
void ps_size(float );
void ps_chara(float, float, int , char *, float , int );
void ps_chars(float, float, int , char *, float , int );
void ps_font(char *, float);
void ps_origin(float, float, float, float);
void ps_line_width(float );
void ps_dashed_line(float, float, float);
void ps_solid_line(void);
void ps_gray(float );
void ps_set_rgb_color(float , float , float);
void ps_set_hsb_color(float , float , float );
void ps_set_cmyk_color(float , float , float , float );
void ps_get_rgb_color(float *, float *, float *);
void ps_get_hsb_color(float *, float *, float *);
void ps_get_cmyk_color(float *, float *, float *, float *);
void ps_rotate(int );
void ps_arc(float , int , int );
void ps_circle(float );
void ps_disk(float , float);
void ps_ellipse(float , float );
void ps_ellip_disk(float , float , float );
void pmovba(float , float );
void pmovsa(float , float );
void pmovbr(float , float );
void pmovsr(float , float );
void plinba(float , float );
void plinsa(float , float );
void plinbr(float , float );
void plinsr(float , float );
void do_point(float *, float *, int, int, int, float);
void ps_cms(void);
void ps_benson(void);
int ps_is_cms(void);
int ps_where(float *, float *);
void ps_user(void);
void ps_drawa(float , float );
void ps_drawr(float , float );
void ps_movea(float , float );
void ps_mover(float , float );
void ps_home(void);
void ps_polyline(float [], float [], int , int , int, float );
void ps_polydots(float [], float [], int , int, float );
void ps_closed_curve(float [], float [], int , int , int , float );
void ps_filled_area(float [], float [], int);
void ps_rectangle(float, float, float, float);
void ps_bezier(float [], float [], int , int , float );
void ps_point(int , float );
void ps_frame(float , float );
void ps_write(char *);
void ps_writea(float , float , char *, int );
void ps_writer(float , float , char *, int );
void ps_writeah(float , float , char *, float );
void ps_writeav(float , float , char *, float );
void ps_writefloatav(float , float , float , float );
void ps_writefloatah(float , float , float , float );
void ps_writeintav(float , float , int , float );
void ps_writeintah(float , float , int , float );
int ps_include(char *);
void ps_code(char *);
void ps_axes(float , float , float , float , float , float , char *, char *, float , int );
void ps_ticked_axes(float , float , float , float , float , float , float , float , float , float ,float, char *, char *, char *, char *, float , int );
void ps_X_axis(float , float , float , float , float , char *, char *, float );
void ps_Y_axis(float , float , float , float , float , char *, char *, float );
void ps_histo(int *, float , float , float , int , int );
void ps_3Dsetup(float , float , float , float , float , float , float , float , float , float , float , float , float );
void ps_3Daxes(int , int );
void ps_3Dcalcul(float , float , float , float *, float *);
void ps_3Dframe(float , float , float , float , float , float );
void ps_3Dbar(float , float , float , float , float , float , int );
void ps_3Dbox(float , float , float , float , float , float , int );
void ps_3Dclosebox(float , float , float , float , float , float );
void couleur(void);
void stat_histo(float *, float *, float *, float *, float *, float *, int *, int);

#undef PS_CLASS
#undef PLOT_PS_H
#endif
