/*
=============================================================================
	P R I N T _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- INIT_PRINT : This function initialize the print box. It tries to read
		       the printcap file and to construct the printer list.
	- PRINT : This function unmanage the print box, gets back printing
		  parameters and run printing.
	- GET_QUEUE_NAME : This function store the selected queue name
=============================================================================
*/


#include <stdlib.h>
#include <stdio.h>
#include <X11/X.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include "global_def_visu.h"


static char *LA_QUEUE_NAME;



/*
=============================================================================
Name and dates: INIT_PRINT_BOX
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function initialize the print box. It tries to read
	the printcap file and to construct the printer list.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :     None

Implicits inputs  : 	WIDGET_ARRAY	Widget[]	The widget array
		  MRM_HIERARCHY_VISU	MrmHierarchy	the hierarchy
		TOPLEVEL_WIDGET_VISU    Widget		The toplevel
		
Implicits outputs :	None

Sides effects :		
---------------		

=============================================================================
*/
void INIT_PRINT_BOX()
{

	FILE *PRINTCAP;
	char *CHAR_PTR;
	XmString XMSTRING;
	Widget W;
	MrmType CLASS_ID;
	char STRING[256];
	
	/*fetch the print box*/
#if 0
	MrmFetchWidget(MRM_HIERARCHY_VISU, "Window_Print", TOPLEVEL_WIDGET_VISU, &W, 
	               &CLASS_ID);
	/*Now its widget id is register in the WIDGET_ARRAY by the MrmCreateCallback*/
#endif

	/*try to open the printcap file*/
	if((PRINTCAP = fopen("/etc/printcap", "r")) == Null)
	{
		DISPLAY_MESSAGE("Impossible to read the /etc/printcap file. Printing is possible only in a Print File!", INFORMATIONAL);
		NETWORK.FLAG.PRINT_ENABLE = False;
		XtSetSensitive(WIDGET_ARRAY[k_Toggle_To_Printer], False);
		XtSetSensitive(WIDGET_ARRAY[k_Toggle_Both], False);
		XmToggleButtonGadgetSetState(WIDGET_ARRAY[k_Toggle_To_Print_File], True, False);
	}
	else
	{
		NETWORK.FLAG.PRINT_ENABLE = True;
		XmToggleButtonGadgetSetState(WIDGET_ARRAY[k_Toggle_Both], True, False);
		
		/*read the file line by line*/
		while(fgets(STRING, 256, PRINTCAP) != Null)
		{
			/*test if the first character of the line is alpha*/
			if(isalpha(*STRING))
			{
				/*extract the printer name*/
				CHAR_PTR = strchr(STRING, ':');
				if(CHAR_PTR)
					*CHAR_PTR = '\0';
				/*extract the first printer name*/
				CHAR_PTR = strchr(STRING, '|');
				if(CHAR_PTR)
					*CHAR_PTR = '\0';
				/*keep the printer name in the print box*/
				XMSTRING = XmStringCreateSimple(STRING);
				XmListAddItem(WIDGET_ARRAY[k_List_Printer], XMSTRING, 0);
				XmStringFree(XMSTRING);
			}
		}
		/*select the first queue in the list*/
		XmListSelectPos(WIDGET_ARRAY[k_List_Printer], 1, False);
	}
}


/*
=============================================================================
Name and dates: PRINT
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function unmanages the print box, gets back printing parameters 
	and runs printing.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :     None

Implicits inputs  : 	WIDGET_ARRAY	Widget[]	The widget array

Implicits outputs :	None

Sides effects :		The print box is realized at the first call.
---------------		

=============================================================================
*/
void PRINT()
{
	FILE     *FICHIER;
	XmString XMSTRING;
	char     *FILE_NAME;
	Arg      arglist[2];
	int      n;
	char     PRINT_FLAG;


	/*gets back printing parameters*/
	/*file*/
	n=0;
	XtSetArg(arglist[n], XmNdirSpec, &XMSTRING); n++;
	XtGetValues(WIDGET_ARRAY[k_Print_File_Box], arglist, n);
	XmStringGetLtoR(XMSTRING, XmSTRING_DEFAULT_CHARSET, &FILE_NAME);
	/*test if the file name is correct*/
 	if((FICHIER = fopen(FILE_NAME, "wb+")) == Null)
	{


	        if (FILE_NAME)
		  FREE_MEMORY(FILE_NAME);

	        FILE_NAME = (char *)calloc(256, sizeof(char));
		switch (NETWORK.FLAG.REPRESENTATION)
		{
			case	GA_REPRESENTATION:
				sprintf(FILE_NAME, "%s_v_ga.ps", NETWORK.NAME);
				break;
			case	MATRIX_REPRESENTATION:
				sprintf(FILE_NAME, "%s_v_mat.ps", NETWORK.NAME);
				break;
			case	PMB_REPRESENTATION:
				sprintf(FILE_NAME, "%s_v_pmb.ps", NETWORK.NAME);
				break;
			case	LINEAR_REPRESENTATION:
				sprintf(FILE_NAME, "%s_v_lin.ps", NETWORK.NAME);
		} /* fin du switch */
		if (FLAG_AFFICHE) sprintf(FILE_NAME, "%s_v_aff.ps", NETWORK.NAME);

	}
	else
	{
#if 0
		printf("ps file: %s\n",FILE_NAME);
#endif
		fclose(FICHIER);
	}

	XmStringFree(XMSTRING);
	
	/*init plot_ps*/
	ps_iniplot(FILE_NAME, 21.);

	ps_queue_name(nom_de_printer);

	/*compute printing*/
	if (FLAG_AFFICHE) 
	  {
	    switch (INDEX_AFF)
	      {
	      case 0:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source0, 
				X_pos[0], Y_pos[0],				
				Debut[0], Fin[0] );
		  break;
		}
	      case 1:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source1, 
				X_pos[1], Y_pos[1],
				Debut[1], Fin[1] );
		  break;
		}
	      case 2:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source2, 
				X_pos[2], Y_pos[2],
				Debut[2], Fin[2] );
		  break;
		}
	      case 3:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source3, 
				X_pos[3], Y_pos[3],
				Debut[3], Fin[3] );
		  break;
		}
	      case 4:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source4, 
				X_pos[4], Y_pos[4], 
				Debut[4], Fin[4] );
		  break;
		}
	      case 5:
		{
		  PRINT_AFFICHE(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.,
				tab_source5, 
				X_pos[5], Y_pos[5],
				Debut[5], Fin[5] );
		  break;
		}
	      }

	  }
	else
	  {
	    R_PRINT(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Print_All]), 21.);
	  }

	/*computes printing action*/
	if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_To_Printer]))
	{
		PRINT_FLAG = 'd';	/*spool and delete*/
		if (keep_ps_files)
			PRINT_FLAG = 's';
	} /* fin du if */
	else if(XmToggleButtonGetState(WIDGET_ARRAY[k_Toggle_Both]))
		PRINT_FLAG = 's';	/*spool and keep*/
	else
		PRINT_FLAG = 'f';	/*no spool and keep*/

	/*close printing*/
	ps_endplot(FILE_NAME, PRINT_FLAG);

	/*unmanage the print window*/

	UNMANAGE(k_Window_Print);


}


/*
=============================================================================
Name and dates: GET_QUEUE_NAME
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function store the selected queue name.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	QUEUE		XmString	The queue name

Explicits outputs :     None

Implicits inputs  :	LA_QUEUE_NAME	char *		The queue name
		
Implicits outputs :	LA_QUEUE_NAME	char *		The queue name

Sides effects :		
---------------		

=============================================================================
*/
void GET_QUEUE_NAME(XmString QUEUE)
{
	char buffer[256];
	
	/*free previous value*/
	if(LA_QUEUE_NAME != Null)
		XtFree(LA_QUEUE_NAME);
	
	/*store the new value*/
	XmStringGetLtoR(QUEUE, XmSTRING_DEFAULT_CHARSET, &LA_QUEUE_NAME);

	sprintf(nom_de_printer,"lpr -P%s",LA_QUEUE_NAME);

}
