/*
=============================================================================
	S I M U L _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- LOAD_SIMUL : this function tries to open the given file with 
		       .pmb extension.
	- INIT_SIMUL : this function initializes the simulation.
	- ACTUALIZE_COLOR_SCALE : this fonction computes and displays limits of
				  the color scale.
	- RUN_SIMUL : this function run the simulation
	- RUN_TIMER_SIMUL : this function run the simulation at each time out.
	- ACTIVATE_TIMER_SIMUL : This function is used to activate or desactivate 
				 the timer for the simulation.
=============================================================================
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <X11/X.h>
#include <Xm/Text.h>

#include "global_def_visu.h"



/*
=============================================================================
Name and dates: ACTUALIZE_COLOR_SCALE
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function actualizes values for the color scale and display news values.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :     None

Implicits inputs  :	SIMUL	  SIMUL_PARAMETERS	The definition of the simulation
			WIDGET_ARRAY	Widget		The widget array
			DISPLAY_VISU    Display *	The display
Implicits outputs : 	none

Sides effects :		none
---------------

=============================================================================
*/

void ACTUALIZE_COLOR_SCALE()
{
	XmString XMSTRING;
	Arg arglist[3];
	int MIDDLE_VALUE;		/*the middle value of the color scale*/
	int n;
	char CHARSTRING[10];

	/*computes the middle value*/
	MIDDLE_VALUE = SIMUL.CS_TOP_VALUE +
		       (int)floor((double)(SIMUL.CS_BOTTOM_VALUE - SIMUL.CS_TOP_VALUE)/
				  (double)2);
	/*computes the step to make colors and values corresponding*/
	SIMUL.CS_STEP = (SIMUL.CS_BOTTOM_VALUE - (float)SIMUL.CS_TOP_VALUE)/(float)100;
	if(SIMUL.CS_STEP < 0)
		SIMUL.CS_STEP = -SIMUL.CS_STEP;

	/*Sets the top value*/
	sprintf(CHARSTRING, "%d", (int)SIMUL.CS_TOP_VALUE);
	XMSTRING = XmStringCreateSimple(CHARSTRING);
	n=0;
	XtSetArg(arglist[n], XmNlabelString, XMSTRING); n++;
	XtSetArg(arglist[n], XmNsensitive, True); n++;
	XtSetValues(WIDGET_ARRAY[k_Label_Top_Value], arglist, n);
	XmStringFree(XMSTRING);
	/*Sets the middlevalue*/
	sprintf(CHARSTRING, "%d", MIDDLE_VALUE);
	XMSTRING = XmStringCreateSimple(CHARSTRING);
	n=0;
	XtSetArg(arglist[n], XmNlabelString, XMSTRING); n++;
	XtSetArg(arglist[n], XmNsensitive, True); n++;
	XtSetValues(WIDGET_ARRAY[k_Label_Middle_Value], arglist, n);
	XmStringFree(XMSTRING);
	/*Sets the bottom value*/
	sprintf(CHARSTRING, "%d", (int)SIMUL.CS_BOTTOM_VALUE);
	XMSTRING = XmStringCreateSimple(CHARSTRING);
	n=0;
	XtSetArg(arglist[n], XmNlabelString, XMSTRING); n++;
	XtSetArg(arglist[n], XmNsensitive, True); n++;
	XtSetValues(WIDGET_ARRAY[k_Label_Bottom_Value], arglist, n);
	XmStringFree(XMSTRING);

}


/*
=============================================================================
Name and dates: INIT_SIMUL
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function initializes the simulation.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :     None

Implicits inputs  :	NETWORK         NETWORK_DEF	The head of the hierarchy
			SIMUL	  SIMUL_PARAMETERS	The definition of the simulation
			CMAP_ID		Colormap	The default colormap
			WIDGET_ARRAY	Widget		The widget array
			DISPLAY_VISU    Display *	The display
Implicits outputs :     None

Sides effects :		none
---------------

=============================================================================
*/

void INIT_SIMUL()
{
	XmString XMSTRING;
	Arg arglist[3];
	XColor COLOR, COLOR_INIT;
	GC GRAPH_CTX;
	int n, i, j;
	char CHARSTRING[10];
	
	/*Sets the bottom value*/
	sprintf(CHARSTRING, "%d", (int)SIMUL.CS_BOTTOM_VALUE);
	XMSTRING = XmStringCreateSimple(CHARSTRING);
	n=0;
	XtSetArg(arglist[n], XmNlabelString, XMSTRING); n++;
	XtSetArg(arglist[n], XmNsensitive, True); n++;
	XtSetValues(WIDGET_ARRAY[k_Label_Bottom_Value], arglist, n);
	XmStringFree(XMSTRING);

	ACTUALIZE_COLOR_SCALE();

	if(COLORMAP_MODE)
	{
	/*initializes color scale*/
		/*allocs blue colors*/
		COLOR.blue = 65535;
		COLOR_INIT.green = 32768;
		COLOR.red = 0;
		for(i=0; i<35; i++)
		{
				COLOR.green = COLOR_INIT.green + i*936;
				COLOR.pixel = i+C_SCALE_COLOR_OFFSET;
				COLOR.flags = DoRed | DoGreen | DoBlue;
				XStoreColor(DISPLAY_VISU, CMAP_ID, &COLOR);
		}
		/*allocs green colors*/
		COLOR_INIT.blue = 65535;
		COLOR.green = 65535;
		COLOR_INIT.red = 0;
		for(i=0; i<30; i++)
		{
			COLOR.blue =   COLOR_INIT.blue - i*2184;
			COLOR.red  =  COLOR_INIT.red + i*2184;
			COLOR.pixel = i+C_SCALE_COLOR_OFFSET + 35;
			COLOR.flags = DoRed | DoGreen | DoBlue;
			XStoreColor(DISPLAY_VISU, CMAP_ID, &COLOR);
		}
		/*allocs yellow colors*/
		COLOR.blue = 0;
		COLOR_INIT.green = 65535;
		COLOR.red = 65535;
		for(i=0; i<35; i++)
		{
			COLOR.green =  COLOR_INIT.green - i*936;
			COLOR.pixel = i+C_SCALE_COLOR_OFFSET + 65;
			COLOR.flags = DoRed | DoGreen | DoBlue;
			XStoreColor(DISPLAY_VISU, CMAP_ID, &COLOR);
		}
	}
	else
	{
	/*initializes color scale*/
		/*allocs blue colors*/
		COLOR.blue = 255;
		COLOR_INIT.green = 128;
		COLOR.red = 0;
		for(i=0; i<35; i++)
		{
			COLOR.green = COLOR_INIT.green + (int)(3.6*i);
			LUT[i+C_SCALE_COLOR_OFFSET] = COLOR.blue + COLOR.green * 256 + COLOR.red * 65536;
		}
		/*allocs green colors*/
		COLOR_INIT.blue = 255;
		COLOR.green = 255;
		COLOR_INIT.red = 0;
		for(i=0; i<30; i++)
		{
			COLOR.blue =   COLOR_INIT.blue - (int)(8.5*i);
			COLOR.red  =  COLOR_INIT.red + (int)(8.5*i);
			LUT[i+C_SCALE_COLOR_OFFSET + 35] = COLOR.blue + COLOR.green * 256 + COLOR.red * 65536;
		}
		/*allocs yellow colors*/
		COLOR.blue = 0;
		COLOR_INIT.green = 255;
		COLOR.red = 255;
		for(i=0; i<35; i++)
		{
			COLOR.green =  COLOR_INIT.green - (int)(3.6*i);
			LUT[i+C_SCALE_COLOR_OFFSET + 65] = COLOR.blue + COLOR.green * 256 + COLOR.red * 65536;
		}
	} /* fin du else */

/*Draw color scale*/
	GRAPH_CTX = XCreateGC(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Drawing_Area_Legend_Main]),
			      0, Null);
	XSetLineAttributes(DISPLAY_VISU, GRAPH_CTX, 3, LineSolid, CapButt, JoinMiter);

	for(i=1; i<=100; i++)
	{
		XSetForeground(DISPLAY_VISU, GRAPH_CTX, LUT[i-1+C_SCALE_COLOR_OFFSET]);
		XDrawLine(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Drawing_Area_Legend_Main]),
			  GRAPH_CTX, 0, 300 - i*3, 30, 300 - i*3);
	}

	XFreeGC(DISPLAY_VISU, GRAPH_CTX);
/*initializes time step to run simulation*/
	SIMUL.TIME_STEP = 0;
/*initializes simulation state*/
	SIMUL.FLAG.SIMUL_STATE = NO_SIMULATION;
/*initializes spike count*/
	SIMUL.SPIKE_COUNT = 0;
/*activate button for printing*/
	XtSetSensitive(WIDGET_ARRAY[k_Button_Print], True);

/*sets sensitive buttons*/
	n=0;
	XtSetArg(arglist[n], XmNsensitive, True); n++;
	XtSetValues(WIDGET_ARRAY[k_Button_Change_Top_Value], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Button_Play_Main], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Button_Play_Step_Main], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Label_Counter_Main], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Text_Counter_Main], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Label_Spikes_Count], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Text_Spikes_Count], arglist, n);
	XtSetValues(WIDGET_ARRAY[k_Button_Goto_Index_Main], arglist, n);
	/*
	XtSetSensitive(WIDGET_ARRAY[k_Button_Exit], False);
	*/
	XtSetSensitive(WIDGET_ARRAY[k_Button_Time_Step], True);
}




/*
=============================================================================
Name and dates:	LOAD_SIMUL
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function tries to open the given file with .def and .pmb extensions.
	Then it initializes the simulation.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :     ROOT_NAME       char *          The name of the files
                                                        without extension

Explicits outputs :	<return>        int             Error Flag
                                                        - 0 : Error
                                                        - 1 : Success
                        MESSAGE         char *          Error Message
                                                        (max 80 carac)

Implicits inputs  :	none

Implicits outputs :	NETWORK         NETWORK_STRUCT  The head of the hierarchy
			
Sides effects :		none
---------------

=============================================================================
*/

int LOAD_SIMUL(char *ROOT_NAME, char *MESSAGE)
{
	int TMP_NB_NEURONS = 0;		/*to compare with previous values*/
	char PMB_FILE_NAME[256];
	short i;
	char STRING[256],string1[30],string2[20];
	char CHAR;
	Arg  args[3];
	int  n;
/*
-----------------------------------------------------------------------------
Try to open and read .def and pmb
-----------------------------------------------------------------------------
*/

	sprintf(PMB_FILE_NAME, "%s.pmb", ROOT_NAME);



	/* preparer le Max du scale Iteration a la 
	   valeur du max d`iteration dans le fichier */

	/********** DEBUT DU COMPTAGE **********************/
	SIMUL.INPUT_PMB_FILE = fopen(PMB_FILE_NAME, "r");
	Nbr_Iteration = 0;
	if(SIMUL.INPUT_PMB_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to open\n%s\nSimulation is not possible.",
			PMB_FILE_NAME);
		return(0);
	}
	while (fgets(STRING, 256, SIMUL.INPUT_PMB_FILE)!=NULL)
	  {
	    sscanf(STRING, "%s %s", string1, string2);
	    if ( (strcmp (string2,"iteration"))==0 )
	      Nbr_Iteration++;
	  } /* fin du while */
	fclose(SIMUL.INPUT_PMB_FILE);
	/************ FIN DU COMPTAGE **********************/	


	SIMUL.INPUT_PMB_FILE = fopen(PMB_FILE_NAME, "r");

/*if we can't open files*/
	if(SIMUL.INPUT_PMB_FILE == Null)
	{
		sprintf(MESSAGE, "Unable to open\n%s\nSimulation is not possible.",
			PMB_FILE_NAME);
		return(False);
	}

/*Test if we have the same number of neurons in PMB file. 
  We test only the first iteration*/
	if(fgets(STRING, 256, SIMUL.INPUT_PMB_FILE) != Null)
	{
		fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
		sscanf(STRING, "%c", &CHAR);
		while(CHAR != '\n')
		{
			TMP_NB_NEURONS++;
			fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
			sscanf(STRING, "%c", &CHAR);
		}
		rewind(SIMUL.INPUT_PMB_FILE);
	}
	if(TMP_NB_NEURONS != NETWORK.NB_NEURONS)
	{
		sprintf(MESSAGE,
		"The given simulation :\n%s\nhasn't same parameters than the current network.\nSimulation is not possible.",
			PMB_FILE_NAME);
		return(False);
	}

/*allocs memory for pmbs*/
	if((SIMUL.POT = (NEURON_PMB *)calloc((size_t)NETWORK.NB_NEURONS, sizeof(NEURON_PMB))) == Null)
	{
		sprintf(MESSAGE,
		"Not enough memory to load simulation.\nTry later");
		return(False);
	}


/*initialize simulation*/
	INIT_SIMUL();


	return(True);
}


/*
=============================================================================
Name and dates:	RUN_SIMUL
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function runs on step of the simulation. It's used at a work procedure.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :     STATE		int*		FOREWARD or BACKWARD

Explicits outputs :	<return>	int		True => the work proc is not rerun 

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network.

Implicits outputs :	None

Sides effects :		none
---------------

=============================================================================
*/
Boolean RUN_SIMUL(XtPointer DATA)
{
	float TMP1, TMP2;
	int *STATE;
	int i, j, NEURON;
	char STRING[256];
        char name_image[256];
	char buffer[256];

#if 0
	printf("RUN_SIMUL\n");
#endif

	STATE = (int *)DATA;
	
	/*backward*/
	if(*STATE == GOTO_INDEX)
	{
		/*run without display simulation*/
		for(j=0; j<SIMUL.ITERATION; j++)
		{
			/*initializes spikes count*/
			SIMUL.SPIKE_COUNT = 0;
			/*if we are not at the end of file*/
			if(fgets(STRING, 256, SIMUL.INPUT_PMB_FILE) != Null)
			{
				/*read pmbs values for the current iteration*/
				for(i=0; i<NETWORK.NB_NEURONS; i++)
				{
					fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
					/*gets pmb*/
					sscanf(STRING, "%d %f", &NEURON, &((SIMUL.POT+i)->PMB));
					/*search if we have a spike*/
					/*for P_neuron and G_neuron model*/
					if(strchr(STRING, 'A') != Null || (SIMUL.POT+i)->PMB == SPIKE_TOP)
						(SIMUL.POT+i)->EVENT = SPIKE;
					else 
						/*no event*/
						(SIMUL.POT+i)->EVENT = NO_EVENT;
				}
				/*go to the next iteration*/
				fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
				
				/*Draws neurons values*/
				R_DRAW_NEURONS_VALUES(GOTO_INDEX);
			}
			else
			{
				XBell(DISPLAY_VISU, 100);
				SIMUL.ITERATION = j;
			}
		}
		/*Draws neurons values*/
		R_DRAW_NEURONS_VALUES(SAME_STEP);
		/*Updates time counter*/
		sprintf(STRING, "%d", SIMUL.ITERATION);
	        XmTextFieldSetString(WIDGET_ARRAY[k_Text_Counter_Main], STRING);

		return(False);
	}
	/*foreward*/
	else
	{
		/*read Iteration x...*/
	  
		if(fgets(STRING, 256, SIMUL.INPUT_PMB_FILE) != Null)
		{

		        /*Updates time counter*/
			SIMUL.ITERATION++;
			sprintf(STRING, "%d", SIMUL.ITERATION);
			XmTextFieldSetString(WIDGET_ARRAY[k_Text_Counter_Main], STRING);
			/*initializes spikes count*/
			SIMUL.SPIKE_COUNT = 0;
			/*read pmbs values for the current iteration*/
			for(i=0; i<NETWORK.NB_NEURONS; i++)
			{
				fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
				/*gets pmb*/
				sscanf(STRING, "%d %f", &NEURON, &((SIMUL.POT+i)->PMB));
				/*search if we have a spike*/
				/*for P_neuron and G_neuron model*/
				if(strchr(STRING, 'A') != Null || (SIMUL.POT+i)->PMB == SPIKE_TOP)
				{
					(SIMUL.POT+i)->EVENT = SPIKE;
					SIMUL.SPIKE_COUNT++;
				}
				else 
					/*no event*/
					(SIMUL.POT+i)->EVENT = NO_EVENT;
			}	
			/*go to the next iteration*/
			fgets(STRING, 256, SIMUL.INPUT_PMB_FILE);
			/*Draws neurons values*/
			R_DRAW_NEURONS_VALUES(FOREWARD);
			/*updates network global activity*/
			sprintf(STRING, "%3.2f%%", (float)SIMUL.SPIKE_COUNT*100/(float)NETWORK.NB_NEURONS);
	 	        XmTextFieldSetString(WIDGET_ARRAY[k_Text_Spikes_Count], STRING);
#if 0
			if(NETWORK.FLAG.REPRESENTATION != MATRIX_REPRESENTATION &&
			   NETWORK.FLAG.REPRESENTATION != LINEAR_REPRESENTATION) 
			  R_DRAW();
#endif
			if (NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION)
			{
			  if (flag_movie && !(SIMUL.ITERATION % Inter_Images))
			  {
			      if (SIMUL.ITERATION >= Start_Images && SIMUL.ITERATION <= Stop_Images) 
			      {
				num_image++;
				sprintf(name_image,"%s_anim_%d.ps",generic_name,num_image);
				/*
				  printf("B %d: %d: %s\n",SIMUL.ITERATION,Inter_Images, name_image);
				  ps_no_banner();
				*/
				ps_iniplot(name_image,21.);
				PRINT_MATRIX(1,21.);
				ps_endplot(name_image,'f');
				/*
				  ps_banner();
				  
				*/
				sprintf(buffer,"%s -sDEVICE=png256 -sOutputFile=%s_anim_%d.png -q -sPAPERSIZE=%s %s&", 
					GHOSTSCRIPT, generic_name, num_image, PAPERSIZE, name_image);
				system(buffer);
			      }
			  }
			}
			return(False);	/*the procedure is rerun*/
		}
		else
		{
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Play_Step_Main], False);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Goto_Index_Main], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Pause_Main], False);
			/*
			XtSetSensitive(WIDGET_ARRAY[k_Button_Exit], False);
			*/
			XtSetSensitive(WIDGET_ARRAY[k_Button_Close_Simulation], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Change_Top_Value], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Refresh_Screen], True);
			XtSetSensitive(WIDGET_ARRAY[k_Toggle_Show_Details], True);
			UNSET_WAIT_CURSOR();

			if(NETWORK.FLAG.EXIST)
			{
			    XClearWindow(DISPLAY_VISU, XtWindow(WIDGET_ARRAY[k_Window_Drawing_Area_Main]));
			    R_DRAW();
			    XtSetSensitive(WIDGET_ARRAY[k_Button_Print], True);
			    if (NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION)
			    {
			      if (flag_movie && !(SIMUL.ITERATION % Inter_Images))
			      {
				if (SIMUL.ITERATION >= Start_Images && SIMUL.ITERATION <= Stop_Images) 
				{
				    num_image++;
				    sprintf(name_image,"%s_anim_%d.ps",generic_name,num_image);
				/*
				  printf("C %d: %d: %s\n",SIMUL.ITERATION,Inter_Images, name_image);
				  ps_no_banner();
				*/
				    ps_iniplot(name_image,21.);
				    PRINT_MATRIX(1,21.);
				    ps_endplot(name_image,'f');
				/*
				  ps_banner();
				  
				*/
				    sprintf(buffer,"%s -sDEVICE=png256 -sOutputFile=%s_anim_%d.png -q -sPAPERSIZE=%s %s&", 
					    GHOSTSCRIPT, generic_name, num_image, PAPERSIZE, name_image);
				    system(buffer);
				}
			      }
			    }
			}
			return(True);	/*the procedure is not rerun*/
		}
	}
}


/*
=============================================================================
Name and dates:	RUN_TIMER_SIMUL
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function runs one step of the simulation by calling 
	RUN_SIMUL. It's used at a time out procedure.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :     STATE		int*		FOREWARD or STOPPED or PAUSED
			ID		XtIntervalId*   the id of timer

Explicits outputs :	None

Implicits inputs  :	None

Implicits outputs :	None

Sides effects :		none
---------------

=============================================================================
*/

void RUN_TIMER_SIMUL(XtPointer DATA, XtIntervalId *ID)
{
	int *STATE;
	int NEW_STATE;

	STATE = (int *)DATA;
	
	/*runs one step of the simulation*/
	if(RUN_SIMUL((XtPointer)STATE))
	{
		NEW_STATE = PAUSED;
		/* not rerun the timer => end of file */
		ACTIVATE_TIMER_SIMUL(&NEW_STATE);
	}
	else
	{
		NEW_STATE = FOREWARD;
		/*recalls timer*/
		ACTIVATE_TIMER_SIMUL(&NEW_STATE);
	}
}



/*
=============================================================================
Name and dates:	ACTIVATE_TIMER_SIMUL
---------------
	Creation : 12-05-93
	Last modification : 12-05-93

Functionnal description :
-------------------------
	This function is used to activate or desactivate the timer for the simulation.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :     STATE		int*		FOREWARD or STOPPED or PAUSED
			
Explicits outputs :	None

Implicits inputs  :	None

Implicits outputs :	None

Sides effects :		none
---------------

=============================================================================
*/

void ACTIVATE_TIMER_SIMUL(int *STATE)
{
	static XtIntervalId TIMER_ID;

	if(*STATE ==  FOREWARD)
		/*activates timer*/
		TIMER_ID = XtAppAddTimeOut(APP_CONTEXT, (unsigned long)SIMUL.TIME_STEP, 
					   RUN_TIMER_SIMUL, STATE);
	else if(TIMER_ID != 0)
	{
		/*kill timer*/
		XtRemoveTimeOut(TIMER_ID);
		TIMER_ID = 0;
	}
}




