/*
=============================================================================
	S T R U C T _ V I S U . H
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	-This file contents all the definition of the structures used by
	 Visualization.

=============================================================================
*/

#include "visu_typedefs.h"		/*For NEURON_ANATOMY*/

/*FOREWARDS DECLARATIONS*/

struct MATRIX_GEOMETRY_STRUCT;
struct SPIKE_GEOMETRY_STRUCT;
struct SIMUL_PARAMETERS_STRUCT;
struct SIMUL_FLAGS_STRUCT;
struct NETWORK_FLAGS_STRUCT;
struct REP_FUNCTION_STRUCT;
struct NEURON_PMB_STRUCT;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR NEURON MEMBRANE POTENTIAL
-----------------------------------------------------------------------------
*/
typedef struct NEURON_PMB_STRUCT
{
	float PMB;	/* the membrane potential */
	int   EVENT;	/* flag = SPIKE if there is a spike */
}NEURON_PMB;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR EXPOSED REGION OF THE DRAWING AREA
-----------------------------------------------------------------------------
*/
typedef struct EXPOSED_REGION_STRUCT
{
	int X, 		/*X coordinate of the upper left corner*/
	    Y, 		/*Y coordinate of the upper left corner*/
	    WIDTH, 	/*width of the region*/
            HEIGHT;	/*height of the region*/
}EXPOSED_REGION;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR THE RETICUL
-----------------------------------------------------------------------------
*/
typedef struct RETICUL_STRUCT
{
	GC	GRAPH_CTX;
	int	X, Y;
	unsigned WIDTH, HEIGHT;
}RETICUL;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR CLUSTERS INFORMATION
-----------------------------------------------------------------------------
*/
typedef struct CLUSTER_INFO_STRUCT
{
	int POS_X, POS_Y;	/*position of the cluster*/
	char NAME[40];		/*name of the cluster*/
}CLUSTER_INFO;
/*
-----------------------------------------------------------------------------
STRUCTURES FOR SIMULATION
-----------------------------------------------------------------------------
*/
/*SIMULATION FLAGS*/

typedef struct SIMUL_FLAGS_STRUCT
{
	int SIMUL_STATE;					/*state of the simulation*/
}SIMUL_FLAGS;

/*SIMULATION PARAMETERS*/

typedef struct SIMUL_PARAMETERS_STRUCT
{
	FILE *INPUT_PMB_FILE;				/*pmb file for simulation*/
	struct NEURON_PMB_STRUCT *POT;			/*pmb array for one iteration*/
	float CS_BOTTOM_VALUE;				/*the bottom value for the color scale*/
	float CS_TOP_VALUE;				/*the top value for the color scale*/
	float CS_STEP;					/*the scale step to display colors*/
	int ITERATION;					/*the current iteration of the simulation*/
	int NB_ITERATION;				/*the current number of visible iteration*/
	int TIME_STEP;					/*time step to run the simulation*/
	int SPIKE_COUNT;				/*number of spikes for the current iteration*/
	struct SIMUL_FLAGS_STRUCT FLAG;			/*flags for the simulation*/
}SIMUL_PARAMETERS;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR LINEAR REPRESENTATION
-----------------------------------------------------------------------------
*/

/*LINEAR GEOMETRY*/

typedef struct LINEAR_GEOMETRY_STRUCT
{
	GC CLUSTER_GRAPH_CTX;				/*Graphic context for clusters*/
	GC NEURON_GRAPH_CTX;				/*Graphic context for neurons*/
	XSegment *CLUSTER_GEOMETRY;			/*Geometry of clusters*/
	unsigned long *NEURONS_COLORS;			/*array to keep colors of displayed neurons*/
	int MAX_DISPLAYED_NEURONS;			/*maximum of displayed neurons*/
	int NEURON_HEIGHT;				/*Height of a neuron square*/
	int WIN_HEIGHT;					/*Height of the drawing area*/
	int NEURON_WIDTH;				/*Width of a neuron square*/
	int WIN_WIDTH;					/*Width of the drawing area*/
}LINEAR_GEOMETRY;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR GLOBAL ACTIVITY REPRESENTATION
-----------------------------------------------------------------------------
*/

/*GLOBAL ACTIVITY GEOMETRY*/

typedef struct GA_GEOMETRY_STRUCT
{
	GC GA_GRAPH_CTX;				/*Graphic context for graphics*/
	XPoint *VALUES;					/*array to keep coordinates of displayed values*/
	XSegment *GRAPH_AXIS;				/*array to keep axis coordinates*/
	XSegment *SCALE;				/*array to keep scale coordinates*/
	float GRAPH_STEP;				/*size in pixel of the step between two points*/
	unsigned WIN_HEIGHT;					/*Height of the drawing area*/
	unsigned WIN_WIDTH;					/*Width of the drawing area*/
}GA_GEOMETRY;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR PMB REPRESENTATION
-----------------------------------------------------------------------------
*/

/*PMB GEOMETRY*/

typedef struct PMB_GEOMETRY_STRUCT
{
	GC PMB_GRAPH_CTX;				/*Graphic context for graphics*/
	XPoint *VALUES;					/*array to keep coordinates of displayed values*/
	XSegment *GRAPH_AXIS;				/*array to keep axis coordinates*/
	XSegment *SCALE;				/*array to keep scale coordinates*/
	float GRAPH_STEP;				/*size in pixel of the step between two points*/
	unsigned WIN_HEIGHT;					/*Height of the drawing area*/
	unsigned WIN_WIDTH;					/*Width of the drawing area*/
}PMB_GEOMETRY;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR MATRIX REPRESENTATION
-----------------------------------------------------------------------------
*/
/*SPIKE GEOMETRY*/

typedef struct SPIKE_GEOMETRY_STRUCT
{
	struct SPIKE_GEOMETRY_STRUCT *NEXT_SPIKE;	/*Next spike in the queue*/
	struct SPIKE_GEOMETRY_STRUCT *PREV_SPIKE;	/*Previous spike in the queue*/
	Pixmap SAVE_PIXMAP;				/*Pixmap to save the screen under the spike*/
	int STEP;					/*Graphic step to draw spike at the next position*/
	int ITERATION;					/*The current iteration for the spike*/
	int LENGTH;					/*length of the axon*/
	int POS_X;					/*The current position of the spike*/
	int POS_Y;
}SPIKE_GEOMETRY;

/*MATRIX GEOMETRY*/

typedef struct MATRIX_GEOMETRY_STRUCT
{
	XRectangle *CLUSTER_GEOMETRY;			/*Geometry of all clusters*/
	XSegment *AXON_GEOMETRY_TO_LOWER;		/*Geometry of axons*/
	XSegment *AXON_GEOMETRY_TO_UPPER;		/*Geometry of axons*/
	XSegment *DENDRITE_GEOMETRY_TO_LOWER;		/*Geometry of dendrites*/
	XSegment *DENDRITE_GEOMETRY_TO_UPPER;		/*Geometry of dendrites*/
	struct SPIKE_GEOMETRY_STRUCT *FIRST_SPIKE;	/*First spike in the queue*/
	Pixmap SPIKE_ON;				/*pixmap to draw spike*/
	Pixmap SPIKE_OFF;				/*pixmap to restore spike underground*/
	GC SPIKE_GRAPH_CTX;				/*Graphic context for spikes*/
	GC CLUSTER_GRAPH_CTX;				/*Graphic context for clusters*/
	GC NEURON_GRAPH_CTX;				/*Graphic context for neurons*/
	GC DENDRITE_GRAPH_CTX;				/*Graphic context for dendrites*/
	GC DETAILS_GRAPH_CTX;				/*Graphic context for details of the representation*/
	int NB_DENDRITES_TO_LOWER;			/*Number of dendrites to a lower neuron*/
	int NB_DENDRITES_TO_UPPER;			/*Number of dendrites to a upper neuron*/
	int NB_AXONS_TO_LOWER;				/*Number of axons to lowers neurons*/
	int NB_AXONS_TO_UPPER;				/*Number of axons to uppers neurons*/
	int NEURON_SIZE;				/*Size of a neuron square*/
	int SIZE;					/*Size of the drawing area*/
	int STEP;					/*Coord step to draw dendrites*/
	int MAX_SYNAPSE;				/*The maximun of SYNAPSE on a neuron*/
}MATRIX_GEOMETRY;

/*
-----------------------------------------------------------------------------
STRUCTURES FOR NETWORK
-----------------------------------------------------------------------------
*/
/*REPRESENTATION FUNCTIONS*/
typedef struct REP_FUNCTION_STRUCT
{
	int (*REALIZE)();				/*memory allocation and initialization*/
	void (*DRAW)();					/*draw in the exposed region*/
	void (*DRAW_NEURONS_VALUES)(int);		/*draw neurons values and spikes in the exposed region*/
	char *(*INFO)(short, short, int *);		/*construct the information string with respect to the reticul position*/
  	void (*PRINT)(int, float);		  	/*print the current representation*/
	void (*FREE)(int);				/*free allocated memory*/
}REP_FUNCTION;

/*NETWORK FLAGS*/

typedef struct NETWORK_FLAGS_STRUCT
{
	int SHOW_DETAILS;		/*True => The details of the selected representation are drawn*/
	int EXIST;			/*True => a network already exists*/
	int REPRESENTATION;		/*Type of representation*/
	int SELECTION;			/*True => selected neuron are stored for pmb representation*/
	int PRINT_ENABLE;		/*true => The print function is enable*/
}NETWORK_FLAGS;

/*NETWORK*/

typedef struct NETWORK_STRUCT_DEF
{
	struct REP_FUNCTION_STRUCT ACTION;	/*all functions for the current representation*/
	char *NAME;				/*Name of the network*/
	short *NB_NEURONS_PER_CLUSTER;		/*Number of neurons per cluster*/
	short *CONNEXION_MATRIX;		/*Matrix of connexions*/
	short *SYNAPTIC_MATRIX;			/*Matrix of synaptic knobs*/
	NEURON_ANATOMY *ANATOMY_MATRIX;		/*Matrix of anatomy*/
	struct MATRIX_GEOMETRY_STRUCT *MATRIX;	/*geometry for the matrix representation*/
	struct LINEAR_GEOMETRY_STRUCT *LINEAR;	/*geometry for the linear representation*/
	struct GA_GEOMETRY_STRUCT *GA;		/*geometry for the global activity representation*/
	struct PMB_GEOMETRY_STRUCT *PMB;	/*geometry for the pmb representation*/
	struct CLUSTER_INFO_STRUCT *INFO_CLUSTERS;	/*information for each cluster*/
	int *SELECTED_NEURON;			/*array of selected neurons index*/
	struct NETWORK_FLAGS_STRUCT FLAG;	/*flags for network*/
	Window WINDOW_ID;			/*Window id of the drawing area*/
	int NB_NEURONS;				/*Number of neurons in the network*/
	int NB_SELECTED_NEURON;			/*Number of selected neurons*/
	int PLACEMENT;				/*placement mode for cluster*/
	short NB_CLUSTERS;			/*Number of clusters in the network*/
}NETWORK_DEF;

/*
-----------------------------------------------------------------------------
MACRO TO ACCES FUNCTIONS
-----------------------------------------------------------------------------
*/
#define R_REALIZE() 		 (*NETWORK.ACTION.REALIZE)()
#define R_DRAW() 	         (*NETWORK.ACTION.DRAW)()
#define R_DRAW_NEURONS_VALUES(a) (*NETWORK.ACTION.DRAW_NEURONS_VALUES)(a)
#define R_PRINT(a, b)  		 (*NETWORK.ACTION.PRINT)((a), (b))
#define R_FREE(a) 		 (*NETWORK.ACTION.FREE)(a)
#define R_INFO(a, b, c)		 (*NETWORK.ACTION.INFO)((a), (b), (c))







