/*
=============================================================================
	U T I L _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- REGISTER    : This callback registers widgets identifier in the
		        WIDGET_ARRAY. In general it's called by
		        MrmCreateCallback.
	- MANAGE      : This function manages or fetchs and manages a widget
		        depending of its state.
	- UNMANAGE    : This function unmanage a widget.
	- FATAL_ERROR : Displays the given message on the standard output and
			exit from the application -> the error is fatal.
	- SET_WAIT_CURSOR : This callback sets the WAIT_CURSOR for all the 
			    application.
	- UNSET_WAIT_CURSOR : This function sets the default cursor for all
			      the application.
	- SET_ELECTRODE_CURSOR : This callback sets the ELECTRODE_CURSOR for all the 
			    application.
	- UNSET_ELECTRODE_CURSOR : This function sets the default cursor for all
			      the application.
	- DISPLAY_MESSAGE : This function displays the given message with the
			    given severity. It waits until the user has read
			    the message.
	- DSETROY_MESSAGE : This callback destroy the message window which call
			    it.
	- GET_EXPOSED_REGION : This function returns geometry of the exposed
			       of the drawing area.
	- SELECT_NEURON : This function add a new neuron in the selected neuron
			  array.
	- DESELECT_NEURON : This function remove a new neuron in the selected neuron
			    array.
	- DISPLAY_RETICUL : This event handler display the reticul when a button
			    is pressed in the drawing area.
	- UNDISPLAY_RETICUL : This event handler undisplay the reticul when a button
			    is released in the drawing area.
	- TRACK_RETICUL : This event handler undisplay the reticul at the old 
			  position and redraw it at the new position.
=============================================================================
*/

#include <stdlib.h>
#include <X11/X.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>
#include <stdio.h>
#include "global_def_visu.h"


#define LEFT_OFFSET 40


/*
=============================================================================
Name and dates:	REGISTER
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This callback register the widget id of the
	widget which call it in the item of WIDGET_ARRAY corresponding
	of the WIDGET_INDEX of the callback.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	W		Widget		Widget id to register
			WIDGET_INDEX	int		Index in the
							WIDGET_ARRAY
			REASON		uns long *	Unused

Implicits outputs :     WIDGET_ARRAY	Widget		Array of widgets
							identifier.

Sides effects :		None
---------------

=============================================================================
*/
void REGISTER(Widget W, XtPointer WIDGET_INDEX, XtPointer REASON)
{

	WIDGET_ARRAY[(int)WIDGET_INDEX] = W;
} 


/*
=============================================================================
Name and dates:	MANAGE
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	If the item of WIDGET_ARRAY.WIDGET_ID corresponding to the given WIDGET_INDEX
	is Null the Widget corresponding to the given WIDGET_NAME is fetched from
	the hierarchy and managed
	else it's just managed.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	WIDGET_INDEX	int		Index of the widget in
							WIDGET_ARRAY.
			WIDGET_NAME	char *		Name of the widget in
							the hierarchy.

Explicits outputs :	None

Implicits inputs  :	WIDGET_ARRAY	Widget		Array of widgets
							identifier.
		     MRM_HIERARCHY_VISU	MrmHierarchy    Mrm hierarchy ID
		     	CMAP_ID		Colormap	The colormap
Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

void MANAGE(int WIDGET_INDEX, char *WIDGET_NAME)
{
    
    /*If the widget has never been fetched*/
    if(WIDGET_ARRAY[WIDGET_INDEX] == Null)
      {
	return;
      }
    
    /*Manage the widget*/    
    XtManageChild(WIDGET_ARRAY[WIDGET_INDEX]);
    /*sets colormap*/
    
    if(COLORMAP_MODE)
      /*the colormap has to be setting on the shell 
	window (thanks to ERIC BOUSSARD : the anti-bug)*/
      XSetWindowColormap(DISPLAY_VISU, XtWindow(XtParent(WIDGET_ARRAY[WIDGET_INDEX])), CMAP_ID);
        
}




/*
=============================================================================
Name and dates:	UNMANAGE
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	Unmanage the widget given by its WIDGET_INDEX in the WIDGET_ARRAY.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	WIDGET_INDEX	int		Index of the widget in
							WIDGET_ARRAY.

Explicits outputs :	None

Implicits inputs  :	WIDGET_ARRAY	Widget		Array of widgets
							identifier.
		     
Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/
void UNMANAGE(int WIDGET_INDEX)
{
	XtUnmanageChild(WIDGET_ARRAY[WIDGET_INDEX]);
} 


/*
=============================================================================
Name and dates:	FATAL_ERROR
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	Display the given message on the standard output and
	exit from the application -> the error is fatal.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	MESSAGE		char *		The message to display

Explicits outputs :	None

Implicits inputs  :	None
		     
Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

void FATAL_ERROR(char *MESSAGE)
{
    char buffer[256];

    sprintf(buffer,"Fatal error : %s\nForced exit\n", MESSAGE);
    DISPLAY_MESSAGE(buffer, ERROR);
    exit(0);
}


/*
=============================================================================
Name and dates: SET_WAIT_CURSOR
---------------
	Creation : 21-04-93
	Last modification : 21-0493

Functionnal description :
-------------------------
	This callback sets the WAIT_CURSOR.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  : TOPELEVEL_WIDGET_VISU Widget	The toplevel widget
			WAIT_CURSOR	Cursor		Shape of the wait cursor

Implicits outputs :	NUMBER_OF_GRAB	uns long	Buffer to count the number 
							of grab


Sides effects : None
---------------

=============================================================================
*/

/*use only by set and unset wait cursor*/
static long NUMBER_OF_GRAB = 0;

void SET_WAIT_CURSOR()
{

/*
-----------------------------------------------------------------------------
Now displayed and grab the wait cursor
-----------------------------------------------------------------------------
*/	
#ifdef DEBUG
	return;
#endif

	XtGrabPointer(TOPLEVEL_WIDGET_VISU, True, None, GrabModeAsync, 
		      GrabModeAsync, None, WAIT_CURSOR, CurrentTime);
	/*one more grab*/
	NUMBER_OF_GRAB++;
}


/*
=============================================================================
Name and dates: UNSET_WAIT_CURSOR
---------------
	Creation : 21-04-93
	Last modification : 21-04-93

Functionnal description :
-------------------------
	This function restore the default cursor .
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	TOPELEVEL_WIDGET_VISU Widget	The toplevel widget
			NUMBER_OF_GRAB	uns long	Buffer to count the number 
							of grab
	
Implicits outputs :	NUMBER_OF_GRAB	uns long	Buffer to count the number 
							of grab


Sides effects : None
---------------

=============================================================================
*/

void UNSET_WAIT_CURSOR()
{
#ifdef DEBUG
	return;
#endif
	if(NUMBER_OF_GRAB == 1)
/*Restore the default cursor*/
		XtUngrabPointer(TOPLEVEL_WIDGET_VISU, CurrentTime);
	NUMBER_OF_GRAB--;
	if(NUMBER_OF_GRAB < 0)
		NUMBER_OF_GRAB = 0;

} 


/*
=============================================================================
Name and dates: SET_ELECTRODE_CURSOR
---------------
	Creation : 10-05-95
	Last modification : 10-05-95

Functionnal description :
-------------------------
	This callback sets the ELECTRODE_CURSOR.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  : TOPELEVEL_WIDGET_VISU Widget	The toplevel widget
			ELECTRODE_CURSOR	Cursor		Shape of the electrode cursor

Implicits outputs :	NUMBER_OF_E_GRAB	uns long	Buffer to count the number 
							of grab


Sides effects : None
---------------

=============================================================================
*/

/*use only by set and unset wait cursor*/
static long NUMBER_OF_E_GRAB = 0;

void SET_ELECTRODE_CURSOR()
{

/*
-----------------------------------------------------------------------------
Now displayed and grab the electrode cursor
-----------------------------------------------------------------------------
*/	
#ifdef DEBUG
	return;
#endif

	XtGrabPointer(TOPLEVEL_WIDGET_VISU, True, None, GrabModeAsync, 
		      GrabModeAsync, None, ELECTRODE_CURSOR, CurrentTime);
	/*one more grab*/
	NUMBER_OF_E_GRAB++;
}


/*
=============================================================================
Name and dates: UNSET_ELECTRODE_CURSOR
---------------
	Creation : 10-05-95
	Last modification : 10-05-95

Functionnal description :
-------------------------
	This function restore the default cursor .
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	TOPELEVEL_WIDGET_VISU Widget	The toplevel widget
			NUMBER_OF_E_GRAB	uns long	Buffer to count the number 
							of grab
	
Implicits outputs :	NUMBER_OF_E_GRAB	uns long	Buffer to count the number 
							of grab


Sides effects : None
---------------

=============================================================================
*/

void UNSET_ELECTRODE_CURSOR()
{
#ifdef DEBUG
	return;
#endif
	if(NUMBER_OF_E_GRAB == 1)
/*Restore the default cursor*/
		XtUngrabPointer(TOPLEVEL_WIDGET_VISU, CurrentTime);
	NUMBER_OF_E_GRAB--;
	if(NUMBER_OF_E_GRAB < 0)
		NUMBER_OF_E_GRAB = 0;

} 


/*
=============================================================================
Name and dates: DISPLAY_MESSAGE
---------------
	Creation : 23-04-93
	Last modification : 23-04-93

Functionnal description :
-------------------------
	This function displays the given message and sets the icone
	of the window corresponding to the given severity. This window
	is FULL_APPLICATION_MODAL so the user must activate the ACKNOWLEDGE
	button to continue the execution.
	
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	MESSAGE		char *		The text wich is to
							be displayed
			SEVERITY	int		the severity of
							the message :
							- INFORMATIONAL
							- ERROR
Explicits outputs :	None

Implicits inputs  :	WIDGET_ARRAY	Widget		Array of widgets
							identifiers
		  MRM_HIERARCHY_VISU    MrmHierarchy	The hierarchy of
							Visualization
		TOPLEVEL_WIDGET_VISU	Widget		The Toplevel Widget
			CMAP_ID		Colormap	The colormap
Implicits outputs :	None

Sides effects : None
---------------

=============================================================================
*/
void DISPLAY_MESSAGE(char *MESSAGE, int SEVERITY)
{
	Arg arglist[22];
	Widget W, WID_TEXT;		/*place to keep returns of MrmFetchWidget*/
	int n;


/*Sets the icone*/
	switch(SEVERITY)
	{
		case INFORMATIONAL :
			/*create the window*/

			W = (Widget)Createwindow_Info_Message(TOPLEVEL_WIDGET_VISU);


			/*Sets message*/
			n=0;
        		XtSetArg(arglist[n], XmNcolumns, 1); n++;
        		XtSetArg(arglist[n], XmNleftAttachment, XmATTACH_FORM); n++;
        	        XtSetArg(arglist[n], XmNleftOffset, 59); n++;
        		XtSetArg(arglist[n], XmNtopAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNtopOffset, 10); n++;
        		XtSetArg(arglist[n], XmNrightAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNrightOffset, 5); n++;
        		XtSetArg(arglist[n], XmNautoShowCursorPosition, False); n++;
        		XtSetArg(arglist[n], XmNcursorPositionVisible, False); n++;
        		XtSetArg(arglist[n], XmNeditable, False); n++;
        		XtSetArg(arglist[n], XmNresizeHeight, True); n++;
        		XtSetArg(arglist[n], XmNwordWrap, True); n++;
        		XtSetArg(arglist[n], XmNwidth, 250); n++;
        		XtSetArg(arglist[n], XmNvalue, MESSAGE); n++;
        		XtSetArg(arglist[n], XmNresizeWidth, False); n++;
        		XtSetArg(arglist[n], XmNborderWidth, 0); n++;
        		XtSetArg(arglist[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
        		XtSetArg(arglist[n], XmNshadowThickness, 0); n++;
        		XtSetArg(arglist[n], XmNbottomAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNbottomOffset, 50); n++;
        		XtSetArg(arglist[n], XmNheight, 60); n++;
			WID_TEXT =  (Widget)XmCreateText(W, "Text_Information_Message", arglist, n);
		        break;
		case ERROR :
			/*create the window*/

			W = (Widget)Createwindow_Error_Message(TOPLEVEL_WIDGET_VISU);

			/*Sets message*/
			n=0;
        		XtSetArg(arglist[n], XmNcolumns, 1); n++;
        		XtSetArg(arglist[n], XmNleftAttachment, XmATTACH_FORM); n++;
        	        XtSetArg(arglist[n], XmNleftOffset, 59); n++;
        		XtSetArg(arglist[n], XmNtopAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNtopOffset, 10); n++;
        		XtSetArg(arglist[n], XmNrightAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNrightOffset, 5); n++;
        		XtSetArg(arglist[n], XmNautoShowCursorPosition, False); n++;
        		XtSetArg(arglist[n], XmNcursorPositionVisible, False); n++;
        		XtSetArg(arglist[n], XmNeditable, False); n++;
        		XtSetArg(arglist[n], XmNresizeHeight, True); n++;
        		XtSetArg(arglist[n], XmNwordWrap, True); n++;
        		XtSetArg(arglist[n], XmNwidth, 250); n++;
        		XtSetArg(arglist[n], XmNvalue, MESSAGE); n++;
        		XtSetArg(arglist[n], XmNresizeWidth, False); n++;
        		XtSetArg(arglist[n], XmNborderWidth, 0); n++;
        		XtSetArg(arglist[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
        		XtSetArg(arglist[n], XmNshadowThickness, 0); n++;
        		XtSetArg(arglist[n], XmNbottomAttachment, XmATTACH_FORM); n++;
        		XtSetArg(arglist[n], XmNbottomOffset, 50); n++;
        		XtSetArg(arglist[n], XmNheight, 60); n++;
			WID_TEXT = (Widget)XmCreateText(W, "Text_Error_Message", arglist, n);
	} 

/*Manage widgets*/    
    XtManageChild(WID_TEXT);
    XtManageChild(W);

/*sets colormap*/
    if(COLORMAP_MODE)
	/*the colormap has to be setting on the shell window (thanks to ERIC BOUSSARD : the anti-bug)*/
	XSetWindowColormap(DISPLAY_VISU, XtWindow(XtParent(W)), CMAP_ID);
}


/*
=============================================================================
Name and dates:	DESTROY_MESSAGE
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This callback destroy the message window which call it.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	W		Widget		Widget id of the caller
			DATA		void *		Unused
			REASON		uns long *	Unused

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void DESTROY_MESSAGE(Widget W, XtPointer DATA, XtPointer REASON)
{
	/*the widget id is the id of the pushbutton, so we destroy its mother*/
	XtDestroyWidget(XtParent(W));
} 


/*
=============================================================================
Name and dates:	GET_EXPOSED_REGION
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This function returns geometry of the exposed of the drawing area.
	It uses scroll bar values.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	REGION	    EXPOSED_REGION *	The geometry

Explicits outputs :	REGION	    EXPOSED_REGION *	The geometry

Implicits inputs  :	WIDGET_ARRAY	Widget		Array of widgets
							identifier.

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void GET_EXPOSED_REGION(EXPOSED_REGION * REGION)
{
	Widget WH, WV;
	Arg arglist[4];
	int n, width;
	int INCREMENT, PAGE_INCREMENT;

	/*gets back widget id of scrollbars*/	
	n=0;
	XtSetArg(arglist[n], XmNhorizontalScrollBar, &WH); n++;
	XtSetArg(arglist[n], XmNverticalScrollBar, &WV); n++;
	XtSetArg(arglist[n], XmNwidth,&width); n++;
	XtGetValues(WIDGET_ARRAY[k_Scrolled_Window_Main], arglist, n);

	/*gets back scrollbars values*/ 
	XmScrollBarGetValues(WH, &(REGION->X), &(REGION->WIDTH), &INCREMENT, &PAGE_INCREMENT);


	
	if (REGION->WIDTH <= 200)
	{
	  REGION->WIDTH = width;
	}
#if DEBUG
	printf("REGION->X: %d;  REGION->WIDTH: %d; INCREMENT: %d; PAGE_INCREMENT: %d\n",
       (REGION->X), (REGION->WIDTH), INCREMENT, PAGE_INCREMENT);
#endif
	XmScrollBarGetValues(WV, &(REGION->Y), &(REGION->HEIGHT), &INCREMENT, &PAGE_INCREMENT);

#if DEBUG
printf("REGION->Y: %d;  REGION->HEIGHT: %d; INCREMENT: %d; PAGE_INCREMENT: %d\n",
       (REGION->Y), (REGION->HEIGHT), INCREMENT, PAGE_INCREMENT);
#endif

}


/*
=============================================================================
Name and dates:	SELECT_NEURON
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This function add a neuron in the array of selected neurons.
	It first test if the neuron is not already selected and then insert
	the neuron in the array and sort the new array.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	NEURON_INDEX	int		Index of the new selected neuron in
							in the network.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_STRUCT	The network definition.

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void SELECT_NEURON(int NEURON_INDEX)
{
	int *TMP_PTR;
	XmString STRING;
	int i, j, TMP;
	int CLUSTER, NEURON_POS,LIST_POS;
	char BUFFER[80];
	
	/*compute cluster and neuron position in the cluster*/
	TMP = 0;
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
	{
		TMP += *(NETWORK.NB_NEURONS_PER_CLUSTER + i);	
		if(NEURON_INDEX <= TMP)
		{
			CLUSTER = i+1;
			break;
		}
	}
	NEURON_POS = NEURON_INDEX - (TMP - *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1)));	

	if(NETWORK.NB_SELECTED_NEURON == 0)
	{
		if((NETWORK.SELECTED_NEURON = (int *)calloc(1, sizeof(int))) == Null)
			DISPLAY_MESSAGE("Not enough memory to select neuron!", ERROR);
		else
		{
			/*manage the list of selected neuron if necessary*/
			if(WIDGET_ARRAY[k_Window_List_Selected_Neuron] == Null)
				MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");
			else if(!XtIsManaged(WIDGET_ARRAY[k_Window_List_Selected_Neuron]))
				MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");

			NETWORK.SELECTED_NEURON[0] = NEURON_INDEX;
			NETWORK.NB_SELECTED_NEURON++;


			/*add the neuron in the list*/
			sprintf(BUFFER, "Neuron %d/Network and %d/Cluster No %d",
				NEURON_INDEX, NEURON_POS, CLUSTER);
			STRING = XmStringCreateSimple(BUFFER);
			XmListAddItem(WIDGET_ARRAY[k_List_Selected_Neuron], STRING, 0);
			XmStringFree(STRING);

			/*the pmb representation is now avialable*/
			XtSetSensitive(WIDGET_ARRAY[k_Toggle_Pmb_Representation], True);
			/*and you can unselect neuron*/
			XtSetSensitive(WIDGET_ARRAY[k_Button_Unselect_Neuron], True);
			XtSetSensitive(WIDGET_ARRAY[k_Button_Membr_Pot], True);
		}
	}
	else
	{
		/*test if the neuron is not already selected*/
		for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
		{
			if(NETWORK.SELECTED_NEURON[i] == NEURON_INDEX)
			{
				/*if already selected we quit the function*/
				sprintf(BUFFER, "Neuron %d/Network is already selected.", NEURON_INDEX);
				DISPLAY_MESSAGE(BUFFER, ERROR);
				return;
			}
		}
		/*realloc one more selected neuron*/
		if((TMP_PTR = (int *)realloc(NETWORK.SELECTED_NEURON, 
		   (size_t)(NETWORK.NB_SELECTED_NEURON+1)*sizeof(int))) == Null)
			DISPLAY_MESSAGE("Not enough memory to select one more neuron!", ERROR);
		else 
		{
			/*manage the list of selected neuron if necessary*/
			if(WIDGET_ARRAY[k_Window_List_Selected_Neuron] == Null)
				MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");
			else if(!XtIsManaged(WIDGET_ARRAY[k_Window_List_Selected_Neuron]))
				MANAGE(k_Window_List_Selected_Neuron, "Window_List_Selected_Neuron");

			NETWORK.SELECTED_NEURON = TMP_PTR;
			NETWORK.NB_SELECTED_NEURON++;
			NETWORK.SELECTED_NEURON[NETWORK.NB_SELECTED_NEURON - 1] = NEURON_INDEX;

			/*sort the array from lower to upper neuron index*/
			for(i=0; i<NETWORK.NB_SELECTED_NEURON - 1; i++)
				for(j=i+1; j<NETWORK.NB_SELECTED_NEURON; j++)
					if(NETWORK.SELECTED_NEURON[j] < NETWORK.SELECTED_NEURON[i])
					{
						TMP = NETWORK.SELECTED_NEURON[i];
						NETWORK.SELECTED_NEURON[i] = NETWORK.SELECTED_NEURON[j];
						NETWORK.SELECTED_NEURON[j] = TMP;
					}

			/*add the neuron in the list*/
			for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
			{
				if(NETWORK.SELECTED_NEURON[i] == NEURON_INDEX)
				{
					LIST_POS = i+1;
					break;
				}
			}
			sprintf(BUFFER, "Neuron %d/Network and %d/Cluster No %d",
				NEURON_INDEX, NEURON_POS, CLUSTER);
			STRING = XmStringCreateSimple(BUFFER);
			XmListAddItem(WIDGET_ARRAY[k_List_Selected_Neuron], STRING, LIST_POS);
			XmStringFree(STRING);
		}
	}
}


/*
=============================================================================
Name and dates:	DESELECT_NEURON
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This function remove a neuron in the array of selected neurons.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	NEURON_INDEX	int *		Array of index in the selected neuron
							list of neuron to deselect.
							(+1 / array of selected neuron)
			NB_NEURON	int		Number of neuron to deselect.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_STRUCT	The network definition.

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void DESELECT_NEURON(int *NEURON_INDEX, int NB_NEURON)
{
	int i, j;

	for(i = NB_NEURON - 1; i>=0; i--)
	{
		XmListDeletePos(WIDGET_ARRAY[k_List_Selected_Neuron], NEURON_INDEX[i]);
		/*update array of selected neuron by copying to the beginning of the array*/
		for(j=NEURON_INDEX[i]; j<NETWORK.NB_SELECTED_NEURON; j++)
			NETWORK.SELECTED_NEURON[j-1] = NETWORK.SELECTED_NEURON[j];
	}

	NETWORK.NB_SELECTED_NEURON -= NB_NEURON;
	if(NETWORK.NB_SELECTED_NEURON == 0)
	{
		free(NETWORK.SELECTED_NEURON);
		/*the pmb representation is unavialable*/
		XtSetSensitive(WIDGET_ARRAY[k_Toggle_Pmb_Representation], False);
		/*and you can't unselect neuron*/
		XtSetSensitive(WIDGET_ARRAY[k_Button_Unselect_Neuron], False);
		XtSetSensitive(WIDGET_ARRAY[k_Button_Membr_Pot], False);
	}
	else	
		NETWORK.SELECTED_NEURON = (int *)realloc(NETWORK.SELECTED_NEURON, 
						 NETWORK.NB_SELECTED_NEURON * sizeof(int));
	if(NETWORK.FLAG.REPRESENTATION == PMB_REPRESENTATION)
	{
		SET_WAIT_CURSOR();
		UPDATE_REPRESENTATION();
		UNSET_WAIT_CURSOR();
	}
}



/*
=============================================================================
Name and dates:	DISPLAY_RETICUL
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This event handler display the reticul when a button
	is pressed in the drawing area. It initialize the reticul
	structure.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	W		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure

Explicits outputs :	None

Implicits inputs  :	DISPLAY_VISU	Display *	The display

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void DISPLAY_RETICUL(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT)
{
	RETICUL *DATA;
	Arg arglist[2];
	int n, NEURON_INDEX;
	XSegment SEGMENTS[2];
	short WIDTH, HEIGHT;

	DATA = (RETICUL *)CLIENT_DATA;
	
	if(!NETWORK.FLAG.EXIST)
		return;

	Flag_Button_1 = 0;

	/*gets back width and height of the drawing area*/
	n=0;
	XtSetArg(arglist[n], XmNwidth, &WIDTH); n++;
	XtSetArg(arglist[n], XmNheight, &HEIGHT); n++;
	XtGetValues(W, arglist, n);

	/*keep actual position and size of the drawing area*/
	DATA->X = EVENT->xbutton.x;
	DATA->Y = EVENT->xbutton.y;
	DATA->WIDTH = WIDTH;
	DATA->HEIGHT = HEIGHT;

	/*horizontal line*/
	SEGMENTS[0].x1 = 0;
	SEGMENTS[0].y1 = DATA->Y;
	SEGMENTS[0].x2 = DATA->WIDTH;
	SEGMENTS[0].y2 = DATA->Y;
	/*vertical line*/
	SEGMENTS[1].x1 = DATA->X;
	SEGMENTS[1].y1 = 0;
	SEGMENTS[1].x2 = DATA->X;
	SEGMENTS[1].y2 = DATA->HEIGHT;
	
	/*draw lines*/
	XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);

	/*updates informations*/
	XmTextSetString(WIDGET_ARRAY[k_Text_Info], R_INFO(DATA->X, DATA->Y, &NEURON_INDEX));
}




/*
=============================================================================
Name and dates:	UNDISPLAY_RETICUL
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This event handler undisplay the reticul when a button
	is released in the drawing area. We redraw at the old position.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	W		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure

Explicits outputs :	None

Implicits inputs  :	DISPLAY_VISU	Display *	The display

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void UNDISPLAY_RETICUL(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT )
{
	RETICUL *DATA;
	int NEURON_INDEX,i,n,MIN_IT,ITERATION;
	XSegment SEGMENTS[2];

	char *STRING;	
	char string[40];
	DATA = (RETICUL *)CLIENT_DATA;
	
	if(!NETWORK.FLAG.EXIST)
		return;

	Flag_Button_1 = 0;

	/*horizontal line*/
	SEGMENTS[0].x1 = 0;
	SEGMENTS[0].y1 = DATA->Y;
	SEGMENTS[0].x2 = DATA->WIDTH;
	SEGMENTS[0].y2 = DATA->Y;
	/*vertical line*/
	SEGMENTS[1].x1 = DATA->X;
	SEGMENTS[1].y1 = 0;
	SEGMENTS[1].x2 = DATA->X;
	SEGMENTS[1].y2 = DATA->HEIGHT;
	
	/*draw lines*/
	XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	XFlush(DISPLAY_VISU);
	/*updates informations*/
	XmTextSetString(WIDGET_ARRAY[k_Text_Info], "");


	if((EVENT->xbutton.button)==2)
		if(NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION ||
	   	   NETWORK.FLAG.REPRESENTATION == LINEAR_REPRESENTATION)
		{
			NETWORK.FLAG.SELECTION = True;
			R_INFO(DATA->X, DATA->Y, &NEURON_INDEX);
			SELECT_NEURON(NEURON_INDEX);
		}
		else
			DISPLAY_MESSAGE("You can't select a neuron in this representation!",
					ERROR);
	else
	  {
	    if((EVENT->xbutton.button)==3) 
	      if(NETWORK.FLAG.REPRESENTATION == MATRIX_REPRESENTATION ||
		 NETWORK.FLAG.REPRESENTATION == LINEAR_REPRESENTATION)
		{
		  /* selection du neuron */
		  STRING = R_INFO(DATA->X, DATA->Y, &NEURON_INDEX);
		  
		  MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? 
		    SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
		  ITERATION = MIN_IT + DATA->X/NETWORK.LINEAR->NEURON_WIDTH + 1;
		  

		  if (!SIMUL.ITERATION) return;
		  if (ITERATION > SIMUL.ITERATION) return;

		  /* ouvrir une fenetre pour la representation du potentiel de membrane */
		  
		  /* recherche de la premiere fenetre libre parmi les 6 existantes */
		  
		  for (i=0 ; i<=5 ; i++)
		    {
		      if (Tab_Pointer_Affiche[i]==0)
			{
			  Tab_Pointer_Affiche[i]=1;
			  sprintf(string,"Window_Affiche_Pmb%d",i);
			  MANAGE(k_Window_Affiche_Pmb0+i*6, string);
			  break;
			}
		      
		      
		    }
		  if ((i==6) && (Tab_Pointer_Affiche[i-1]=1))
		    {
		      return;
		    }
		  


		
		switch (i)
		  {
		  case 0:
		    {
		      X_pos[0] = DATA->X;
		      Y_pos[0] = DATA->Y;
		      Debut[0] = Debut_Hold[0] = Rep_Zoom[0] = 0;
		      Fin[0] = Fin_Hold[0] = Init[0] = ITERATION;
		      
		      CREATE_WINDOW(k_Draw_Affiche_Pmb0, 
				    tab_source0,
				    X_pos[0], Y_pos[0],
				    Debut[0], Fin[0], Lecture_Fichier );
		      break;
		    }
		  case 1:
		    {
		      X_pos[1] = DATA->X;
		      Y_pos[1] = DATA->Y;		    
		      Debut[1] = Debut_Hold[1] = Rep_Zoom[1] = 0;
		      Fin[1] = Fin_Hold[1] = Init[1] = ITERATION;
		      CREATE_WINDOW(k_Draw_Affiche_Pmb1, 
				    tab_source1,
				    X_pos[1], Y_pos[1],
				    Debut[1], Fin[1], Lecture_Fichier );
		      break;
		    }
		  case 2:
		    {
		      X_pos[2] = DATA->X;
		      Y_pos[2] = DATA->Y;		    
		      Debut[2] = Debut_Hold[2] = Rep_Zoom[2] = 0;
		      Fin[2] = Fin_Hold[2] = Init[2] = ITERATION;
		      CREATE_WINDOW(k_Draw_Affiche_Pmb2, 
				    tab_source2,
				    X_pos[2], Y_pos[2],
				    Debut[2], Fin[2], Lecture_Fichier );
		      break;
		    }
		  case 3:
		    {
		      X_pos[3] = DATA->X;
		      Y_pos[3] = DATA->Y;		    
		      Debut[3] = Debut_Hold[3] = Rep_Zoom[3] = 0;
		      Fin[3] = Fin_Hold[3] = Init[3] = ITERATION;
		      CREATE_WINDOW(k_Draw_Affiche_Pmb3, 
				    tab_source3,
				    X_pos[3], Y_pos[3],
				    Debut[3], Fin[3], Lecture_Fichier );
		      break;
		    }
		  case 4:
		    {
		      X_pos[4] = DATA->X;
		      Y_pos[4] = DATA->Y;		    
		      Debut[4] = Debut_Hold[4] = Rep_Zoom[4] = 0;
		      Fin[4] = Fin_Hold[4] = Init[4] = ITERATION;
		      CREATE_WINDOW(k_Draw_Affiche_Pmb4, 
				    tab_source4,
				    X_pos[4], Y_pos[4],
				    Debut[4], Fin[4], Lecture_Fichier );
		      break;
		    }
		  case 5:
		    {
		      X_pos[5] = DATA->X;
		      Y_pos[5] = DATA->Y;		    
		      Debut[5] = Debut_Hold[5] = Rep_Zoom[5] = 0;
		      Fin[5] = Fin_Hold[5] = Init[5] = ITERATION;
		      CREATE_WINDOW(k_Draw_Affiche_Pmb5, 
				    tab_source5,
				    X_pos[5], Y_pos[5],
				    Debut[5], Fin[5], Lecture_Fichier );
		      break;
		    }
		  } /* fin du switch */


		  STRING = R_INFO(DATA->X, DATA->Y, &NEURON_INDEX);
		  for (n=0 ; n<strlen(STRING) ; n++)
		    {
		      if (STRING[n]=='\n')
			{
			  STRING[n]='|';
			}
		    }

		  XmTextSetString(WIDGET_ARRAY[k_Info_Affiche_Pmb0+i*6], STRING);

		  
		  /* ajouter une gestion d`evenement de souris a la fenetre */
		  
		  XtAddEventHandler(WIDGET_ARRAY[k_Draw_Affiche_Pmb0+i*6], 
				    ButtonPressMask, 
				    False,
				    BOUTON_ENFONCE, 
				    &RETICUL_DATA);
		  XtAddEventHandler(WIDGET_ARRAY[k_Draw_Affiche_Pmb0+i*6], 
				    ButtonReleaseMask, 
				    False,
				    BOUTON_RELACHE, 
				    &RETICUL_DATA);
		  XtAddEventHandler(WIDGET_ARRAY[k_Draw_Affiche_Pmb0+i*6], 
				    ButtonMotionMask, 
				    False,
				    DEPLACEMENT, 
				    &RETICUL_DATA);

		  /* ajouter un affichage de Help_On_Line */
/*
		  XtAddEventHandler(WIDGET_ARRAY[k_Draw_Affiche_Pmb0+i*6], 
*/				    
	      }
	  }
}


/*
=============================================================================
Name and dates:	TRACK_RETICUL
---------------
	Creation : Apr 15, 1993
	Last modification : Apr 15, 1993

Functionnal description :
-------------------------
	This event handler undisplay the reticul at the old 
	position and redraw it at the new position when the pointer is drag over
	the darwing area.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	W		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure

Explicits outputs :	None

Implicits inputs  :	DISPLAY_VISU	Display *	The display

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/
void TRACK_RETICUL(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT)
{
	RETICUL *DATA;
	int NEURON_INDEX;
	XSegment SEGMENTS[2];

	DATA = (RETICUL *)CLIENT_DATA;

	if(!NETWORK.FLAG.EXIST)
		return;

	Flag_Button_1 = 0;

	/*horizontal line*/
	SEGMENTS[0].x1 = 0;
	SEGMENTS[0].y1 = DATA->Y;
	SEGMENTS[0].x2 = DATA->WIDTH;
	SEGMENTS[0].y2 = DATA->Y;
	/*vertical line*/
	SEGMENTS[1].x1 = DATA->X;
	SEGMENTS[1].y1 = 0;
	SEGMENTS[1].x2 = DATA->X;
	SEGMENTS[1].y2 = DATA->HEIGHT;
	
	/*undraw lines*/
	XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);

	/*keep actual position*/
	DATA->X = EVENT->xbutton.x;
	DATA->Y = EVENT->xbutton.y;

	/*horizontal line*/
	SEGMENTS[0].x1 = 0;
	SEGMENTS[0].y1 = DATA->Y;
	SEGMENTS[0].x2 = DATA->WIDTH;
	SEGMENTS[0].y2 = DATA->Y;
	/*vertical line*/
	SEGMENTS[1].x1 = DATA->X;
	SEGMENTS[1].y1 = 0;
	SEGMENTS[1].x2 = DATA->X;
	SEGMENTS[1].y2 = DATA->HEIGHT;
	
	/*draw lines*/
	XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);

	/*updates informations*/

	XmTextSetString(WIDGET_ARRAY[k_Text_Info], R_INFO(DATA->X, DATA->Y, &NEURON_INDEX));

}





/*
---------------------------------------------------------------------------------------------

FONCTION DE CONTROLE DE ENFONCEMENT D`UN BOUTON DE LA SOURIS DANS LES FENETRES DE TYPE AFFICHE


  Cette fonction permet de gerer l`enfoncement
  d`un bouton de la souris, dans une fenetre de type "AFFICHE"


entree: 		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure


sortie :                rien.


---------------------------------------------------------------------------------------------
*/
void BOUTON_ENFONCE(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT)
{
  RETICUL *DATA;
  Arg arglist[2];
  int n, NEURON_INDEX, VALEUR;
  XSegment SEGMENTS[1];
  short WIDTH, HEIGHT;
  
  DATA = (RETICUL *)CLIENT_DATA;
  
  if(!NETWORK.FLAG.EXIST)
    return;

  
  if((EVENT->xbutton.button)==3)
    {
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb0])
	{
	  Debut[0] = 0;
	  Fin[0] = Init[0];
	  Rep_Zoom[0] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb0,
			tab_source0,
			X_pos[0], Y_pos[0],
			Debut[0], Fin[0] );
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb1])
	{
	  Debut[1] = 0;
	  Fin[1] = Init[1];
	  Rep_Zoom[1] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb1,
			tab_source1,
			X_pos[1], Y_pos[1],
			Debut[1], Fin[1] );
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb2])
	{
	  Debut[2] = 0;
	  Fin[2] = Init[2];
	  Rep_Zoom[2] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb2,
			tab_source2,
			X_pos[2], Y_pos[2],
			Debut[2], Fin[2] );
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb3])
	{
	  Debut[3] = 0;
	  Fin[3] = Init[3];
	  Rep_Zoom[3] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb3,
			tab_source3,
			X_pos[3], Y_pos[3],
			Debut[3], Fin[3] );
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb4])
	{
	  Debut[4] = 0;
	  Fin[4] = Init[4];
	  Rep_Zoom[4] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb4,
			tab_source4,
			X_pos[4], Y_pos[4],
			Debut[4], Fin[4] );
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb5])
	{
	  Debut[5] = 0;
	  Fin[5] = Init[5];
	  Rep_Zoom[5] = 0;
	  CREATE_WINDOW(k_Draw_Affiche_Pmb5,
			tab_source5,
			X_pos[5], Y_pos[5],
			Debut[5], Fin[5] );
	}

#if 0
      printf("allumer le zoom\n");
#endif
    }
  
  
  
  if((EVENT->xbutton.button)==2)
    {
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb0])
	if (Rep_Zoom[0]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb0,
			  tab_source0,
			  X_pos[0], Y_pos[0],
			  Debut[0], Fin[0] );
	    Rep_Zoom[0]=0;
#if 0
	    printf("effectuer un zoom 0 entre %d et %d\n",Debut[0],Fin[0]);
#endif
	  }
     if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb1])
	if (Rep_Zoom[1]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb1,
			  tab_source1,
			  X_pos[1], Y_pos[1],
			  Debut[1], Fin[1] );
	    Rep_Zoom[1]=0;

	  }
     if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb2])
	if (Rep_Zoom[2]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb2,
			  tab_source2,
			  X_pos[2], Y_pos[2],
			  Debut[2], Fin[2] );
	    Rep_Zoom[2]=0;

	  }
     if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb3])
	if (Rep_Zoom[3]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb3,
			  tab_source3,
			  X_pos[3], Y_pos[3],
			  Debut[3], Fin[3] );
	    Rep_Zoom[3]=0;

	  }
     if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb4])
	if (Rep_Zoom[4]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb4,
			  tab_source4,
			  X_pos[4], Y_pos[4],
			  Debut[4], Fin[4] );
	    Rep_Zoom[4]=0;

	  }
     if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb5])
	if (Rep_Zoom[5]==2)
	  {
	    CREATE_WINDOW(k_Draw_Affiche_Pmb5,
			  tab_source5,
			  X_pos[5], Y_pos[5],
			  Debut[5], Fin[5] );
	    Rep_Zoom[5]=0;

	  }
    }
  
  

  if((EVENT->xbutton.button)==1)
    {
      Flag_Button_1 = 1;      
      /*keep actual position*/
      DATA->X = EVENT->xbutton.x;
      DATA->Y = EVENT->xbutton.y;
      
      /*keep actual position*/
      DATA->X = EVENT->xbutton.x;
      DATA->Y = EVENT->xbutton.y;

      /*vertical line*/
      SEGMENTS[1].x1 = DATA->X;
      SEGMENTS[1].y1 = 0;
      SEGMENTS[1].x2 = DATA->X;
      SEGMENTS[1].y2 = DATA->HEIGHT;
      
      /*draw lines*/
      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
      

      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb0])
	{
	  VALEUR = Debut_Hold[0]+(DATA->X-20)*(Fin_Hold[0] - Debut_Hold[0])/(float)(Scr_Width);

	  if (Rep_Zoom[0]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb0, 
			    tab_source0,
			    X_pos[0], Y_pos[0],
			    Debut_Hold[0], Fin_Hold[0], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}

      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb1])
	{
	  VALEUR = Debut_Hold[1]+(DATA->X-20)*(Fin_Hold[1] - Debut_Hold[1])/(float)(Scr_Width);

	  if (Rep_Zoom[1]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb1, 
			    tab_source1,
			    X_pos[1], Y_pos[1],
			    Debut_Hold[1], Fin_Hold[1], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb2])
	{
	  VALEUR = Debut_Hold[2]+(DATA->X-20)*(Fin_Hold[2] - Debut_Hold[2])/(float)(Scr_Width);

	  if (Rep_Zoom[2]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb2, 
			    tab_source2,
			    X_pos[2], Y_pos[2],
			    Debut_Hold[2], Fin_Hold[2], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb3])
	{
	  VALEUR = Debut_Hold[3]+(DATA->X-20)*(Fin_Hold[3] - Debut_Hold[3])/(float)(Scr_Width);

	  if (Rep_Zoom[3]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb3, 
			    tab_source3,
			    X_pos[3], Y_pos[3],
			    Debut_Hold[3], Fin_Hold[3], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb4])
	{
	  VALEUR = Debut_Hold[4]+(DATA->X-20)*(Fin_Hold[4] - Debut_Hold[4])/(float)(Scr_Width);

	  if (Rep_Zoom[4]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb4, 
			    tab_source4,
			    X_pos[4], Y_pos[4],
			    Debut_Hold[4], Fin_Hold[4], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb5])
	{
	  VALEUR = Debut_Hold[5]+(DATA->X-20)*(Fin_Hold[5] - Debut_Hold[5])/(float)(Scr_Width);

	  if (Rep_Zoom[5]==2)  /* on annule le coup */
	    {
	      CREATE_WINDOW(k_Draw_Affiche_Pmb5, 
			    tab_source5,
			    X_pos[5], Y_pos[5],
			    Debut_Hold[5], Fin_Hold[5], Lecture_Memoire );

	      /*keep actual position*/
	      DATA->X = EVENT->xbutton.x;
	      DATA->Y = EVENT->xbutton.y;
	      
	      /*vertical line*/
	      SEGMENTS[1].x1 = DATA->X;
	      SEGMENTS[1].y1 = 0;
	      SEGMENTS[1].x2 = DATA->X;
	      SEGMENTS[1].y2 = DATA->HEIGHT;
      
	      /*draw lines*/
	      XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	    }
	}

    }
}


/*
---------------------------------------------------------------------------------------------
   
   FONCTION DE CONTROLE DE RELACHEMENT D`UN BOUTON DE LA SOURIS DANS LES FENETRES DE TYPE AFFICHE
   
  Cette fonction permet de gerer le relachement
  d`un bouton de la souris, dans une fenetre de type "AFFICHE"


entree: 		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure


sortie :                rien.


---------------------------------------------------------------------------------------------
*/


void BOUTON_RELACHE(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT)
{
  RETICUL *DATA;
  int NEURON_INDEX,i,n;
  XSegment SEGMENTS[2];
  int  VALEUR;
  char *STRING;	
  char string[40];
  DATA = (RETICUL *)CLIENT_DATA;
  
  
  if(!NETWORK.FLAG.EXIST)
    return;


  if((EVENT->xbutton.button)==1)
    {	
      Flag_Button_1 = 0;

      /* pour la fenetre 0 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb0])
	{
	  VALEUR = Debut_Hold[0]+(DATA->X-20)*(Fin_Hold[0] - Debut_Hold[0])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[0]==2) /* on retourne a la selection de debut */
	    {
#if 0
	      printf("rep 2 retouner, pas de memo des Holds et memo Debut\n");
#endif
	      Debut[0] = VALEUR;
	      Rep_Zoom[0]= 1;
#if 0
	      printf("Fin[0]  :%d   Fin_Hold[0]  :%d\n",Fin[0],Fin_Hold[0]);
	      printf("Debut[0]:%d   Debut_Hold[0]:%d\n",Debut[0],Debut_Hold[0]);
#endif
	    }
	  if (Rep_Zoom[0]==1) /* Selection de Fin[0] ( Debut existe deja ) */
	    {
#if 0
	      printf("rep 1 test si valeur=debut\n");
#endif
	      if (Debut[0]!=VALEUR)
		{
		  
		  Fin[0] = VALEUR;
		  Rep_Zoom[0]= 2;
#if 0
		  printf("Fin[0]:%d   Fin_Hold[0]:%d\n",Fin[0],Fin_Hold[0]);
#endif
		}
	    }
	  if (Rep_Zoom[0]==0) /* !Debut[0] && !Fin[0] */
	    {
#if 0
	      printf("rep 0 memo des Holds et de Debut\n");
#endif
	      Debut_Hold[0] = Debut[0];
	      Fin_Hold[0] = Fin[0];

	      VALEUR = Debut_Hold[0]+(DATA->X-20)*(Fin_Hold[0] - Debut_Hold[0])/(float)(Scr_Width);

	      Debut[0] = VALEUR;
	      Rep_Zoom[0]= 1;
#if 0
	      printf("Debut[0]:%d  Debut_Hold[0]:%d \n",Debut[0],Debut_Hold[0]);
#endif
	    }
	}
      
      /* pour la fenetre 1 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb1])
	{
	  VALEUR = Debut_Hold[1]+(DATA->X-20)*(Fin_Hold[1] - Debut_Hold[1])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[1]==2) /* on retourne a la selection de debut */
	    {
	      Debut[1] = VALEUR;
	      Rep_Zoom[1]= 1;
	    }
	  if (Rep_Zoom[1]==1) /* Selection de Fin[1] ( Debut existe deja ) */
	    {
	      if (Debut[1]!=VALEUR)
		{
		  
		  Fin[1] = VALEUR;
		  Rep_Zoom[1]= 2;
		}
	    }
	  if (Rep_Zoom[1]==0) /* !Debut[1] && !Fin[1] */
	    {
	      Debut_Hold[1] = Debut[1];
	      Fin_Hold[1] = Fin[1];

	      VALEUR = Debut_Hold[1]+(DATA->X-20)*(Fin_Hold[1] - Debut_Hold[1])/(float)(Scr_Width);

	      Debut[1] = VALEUR;
	      Rep_Zoom[1]= 1;
	    }
	}
      
      /* pour la fenetre 2 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb2])
	{
	  VALEUR = Debut_Hold[2]+(DATA->X-20)*(Fin_Hold[2] - Debut_Hold[2])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[2]==2) /* on retourne a la selection de debut */
	    {
	      Debut[2] = VALEUR;
	      Rep_Zoom[2]= 1;
	    }
	  if (Rep_Zoom[2]==1) /* Selection de Fin[2] ( Debut existe deja ) */
	    {
	      if (Debut[2]!=VALEUR)
		{
		  
		  Fin[2] = VALEUR;
		  Rep_Zoom[2]= 2;
		}
	    }
	  if (Rep_Zoom[2]==0) /* !Debut[2] && !Fin[2] */
	    {
	      Debut_Hold[2] = Debut[2];
	      Fin_Hold[2] = Fin[2];

	      VALEUR = Debut_Hold[2]+(DATA->X-20)*(Fin_Hold[2] - Debut_Hold[2])/(float)(Scr_Width);

	      Debut[2] = VALEUR;
	      Rep_Zoom[2]= 1;
	    }
	}
      
      /* pour la fenetre 3 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb3])
	{
	  VALEUR = Debut_Hold[3]+(DATA->X-20)*(Fin_Hold[3] - Debut_Hold[3])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[3]==2) /* on retourne a la selection de debut */
	    {
	      Debut[3] = VALEUR;
	      Rep_Zoom[3]= 1;
	    }
	  if (Rep_Zoom[3]==1) /* Selection de Fin[3] ( Debut existe deja ) */
	    {
	      if (Debut[3]!=VALEUR)
		{
		  
		  Fin[3] = VALEUR;
		  Rep_Zoom[3]= 2;
		}
	    }
	  if (Rep_Zoom[3]==0) /* !Debut[3] && !Fin[3] */
	    {
	      Debut_Hold[3] = Debut[3];
	      Fin_Hold[3] = Fin[3];

	      VALEUR = Debut_Hold[3]+(DATA->X-20)*(Fin_Hold[3] - Debut_Hold[3])/(float)(Scr_Width);

	      Debut[3] = VALEUR;
	      Rep_Zoom[3]= 1;
	    }
	}
      
      /* pour la fenetre 4 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb4])
	{
	  VALEUR = Debut_Hold[4]+(DATA->X-20)*(Fin_Hold[4] - Debut_Hold[4])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[4]==2) /* on retourne a la selection de debut */
	    {
	      Debut[4] = VALEUR;
	      Rep_Zoom[4]= 1;
	    }
	  if (Rep_Zoom[4]==1) /* Selection de Fin[4] ( Debut existe deja ) */
	    {
	      if (Debut[4]!=VALEUR)
		{
		  
		  Fin[4] = VALEUR;
		  Rep_Zoom[4]= 2;
		}
	    }
	  if (Rep_Zoom[4]==0) /* !Debut[4] && !Fin[4] */
	    {
	      Debut_Hold[4] = Debut[4];
	      Fin_Hold[4] = Fin[4];

	      VALEUR = Debut_Hold[4]+(DATA->X-20)*(Fin_Hold[4] - Debut_Hold[4])/(float)(Scr_Width);

	      Debut[4] = VALEUR;
	      Rep_Zoom[4]= 1;
	    }
	}
      
      /* pour la fenetre 5 */
      if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb5])
	{
	  VALEUR = Debut_Hold[5]+(DATA->X-20)*(Fin_Hold[5] - Debut_Hold[5])/(float)(Scr_Width);
	  
	  if (Rep_Zoom[5]==2) /* on retourne a la selection de debut */
	    {
	      Debut[5] = VALEUR;
	      Rep_Zoom[5]= 1;
	    }
	  if (Rep_Zoom[5]==1) /* Selection de Fin[5] ( Debut existe deja ) */
	    {
	      if (Debut[5]!=VALEUR)
		{
		  
		  Fin[5] = VALEUR;
		  Rep_Zoom[5]= 2;
		}
	    }
	  if (Rep_Zoom[5]==0) /* !Debut[5] && !Fin[5] */
	    {
	      Debut_Hold[5] = Debut[5];
	      Fin_Hold[5] = Fin[5];

	      VALEUR = Debut_Hold[5]+(DATA->X-20)*(Fin_Hold[5] - Debut_Hold[5])/(float)(Scr_Width);

	      Debut[5] = VALEUR;
	      Rep_Zoom[5]= 1;
	    }
	}
      
    }
}
  
/*
---------------------------------------------------------------------------------------------
     
     FONCTION DE CONTROL DE DEPLACEMENT D`UN BOUTON DE LA SOURIS DANS LES FENETRES DE TYPE AFFICHE

  Cette fonction permet de gerer le deplacement
  d`un bouton de la souris, dans une fenetre de type "AFFICHE"


entree: 		Widget		Widget id of the caller
			DATA		RETICUL	*	Data for the reticul.
			EVENT		XEvent *	Event structure


sortie :                rien.


     
---------------------------------------------------------------------------------------------
*/

void DEPLACEMENT(Widget W, XtPointer CLIENT_DATA, XEvent *EVENT, Boolean *CONT)
{
  RETICUL *DATA;
  int NEURON_INDEX, VALEUR;
  XSegment SEGMENTS[1];
  
  DATA = (RETICUL *)CLIENT_DATA;
  
  if(!NETWORK.FLAG.EXIST)
    return;
  
  
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb0])
    {
      /* pour la fenetre 0 */
      
      VALEUR = Debut_Hold[0]+(DATA->X-20)*(Fin_Hold[0] - Debut_Hold[0])/(float)(Scr_Width);

      if (Flag_Button_1) /* VALEUR!= Debut[0] && VALEUR!= Fin[0] )*/
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb1])
    {
      /* pour la fenetre 1 */
      
      VALEUR = Debut_Hold[1]+(DATA->X-20)*(Fin_Hold[1] - Debut_Hold[1])/(float)(Scr_Width);

      if (Flag_Button_1)
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb2])
    {
      /* pour la fenetre 2 */
      
      VALEUR = Debut_Hold[2]+(DATA->X-20)*(Fin_Hold[2] - Debut_Hold[2])/(float)(Scr_Width);

      if (Flag_Button_1) 
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb3])
    {
      /* pour la fenetre 3 */
      
      VALEUR = Debut_Hold[0]+(DATA->X-20)*(Fin_Hold[0] - Debut_Hold[0])/(float)(Scr_Width);

      if (Flag_Button_1) 
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb4])
    {
      /* pour la fenetre 4 */
      
      VALEUR = Debut_Hold[4]+(DATA->X-20)*(Fin_Hold[4] - Debut_Hold[4])/(float)(Scr_Width);

      if (Flag_Button_1) 
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  if ((W)==WIDGET_ARRAY[k_Draw_Affiche_Pmb5])
    {
      /* pour la fenetre 0 */
      
      VALEUR = Debut_Hold[5]+(DATA->X-20)*(Fin_Hold[5] - Debut_Hold[5])/(float)(Scr_Width);

      if (Flag_Button_1)
	{

	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*undraw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	  
	  /*keep actual position*/
	  DATA->X = EVENT->xbutton.x;
	  DATA->Y = EVENT->xbutton.y;
	  
	  /*vertical line*/
	  SEGMENTS[1].x1 = DATA->X;
	  SEGMENTS[1].y1 = 0;
	  SEGMENTS[1].x2 = DATA->X;
	  SEGMENTS[1].y2 = DATA->HEIGHT;
	  
	  /*draw lines*/
	  XDrawSegments(DISPLAY_VISU, XtWindow(W), DATA->GRAPH_CTX, SEGMENTS, 2);
	}
    }
  
  
}

