/*
************************************************************************************
*
*	Callbacks for CAA
*
*
*
*
*
*
*
*
*******************************************************************************
*/

/*
 * Standard includes for builtins.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

/*
**	MOTIF INCLUDE FILES
*/

#include <Xm/Xm.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Text.h> 
/*
**      Personal Motif variables
*/

#define PRINTED_EXT ".ps"              /* extension of printed files */
#define _CAA_CALLBACK
#include "caa_unix.h"
#include "caa_interface_creation.h"
#include "caa_interface_callbacks.h"
#include "find_files.h"
#undef _CAA_CALLBACK

/*
**	prototypes
*/
Widget	CreateprintMain_Form (Widget);
Widget	CreatehelpMain_Form (Widget);
void	caa_MainDispHelpText (Widget widget, XtPointer client_data, XtPointer call_data);
Widget	CreatefctparaMain_Form (Widget);

/*
**	extern functions
*/

extern char     on_garde[10];
extern int	read_data();
extern int arrondi(float f);
extern void Page_de_garde();

/*
 * Macros to make code look nicer between ANSI and K&R.
 */
#ifndef ARGLIST
#if (NeedFunctionPrototypes == 0)
#define PROTOTYPE(p)	()
#define ARGLIST(p)	p
#define ARG(a, b)	a b;
#define GRA(a, b)	a b;
#define UARG(a, b)      a b;
#define GRAU(a, b)      a b;
#else
#define PROTOTYPE(p)	p
#define ARGLIST(p)	(
#define ARG(a, b)	a b,
#define GRA(a, b)	a b)
#ifdef __cplusplus
#define UARG(a, b)      a,
#define GRAU(a, b)      a)
#else
#define UARG(a, b)      a b,
#define GRAU(a, b)      a b)
#endif
#endif
#endif

/*
**	Global variables
*/

extern Boolean  FirstUse;
extern int      spooled;
extern Widget TheAppShell;
extern Widget main_TopLevelShell;
extern Widget parameters_TopLevelShell;
extern Widget fileSelection_TopLevelShell;
extern Widget filter_TopLevelShell;
extern Widget histogram_TopLevelShell;
extern Widget arrows_TopLevelShell;
extern Widget poincare_TopLevelShell;
Widget fctpara_TopLevelShell;
Widget		ThePopupBox;

Widget		BxFindTopShell PROTOTYPE((Widget));
WidgetList	BxWidgetIdsFromNames PROTOTYPE((Widget, char*, char*));

/*******************************************************************************************
 *      Function Name:	BxSetSensitiveCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then set the widget sensitive.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the list of widget names.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *                    * BxSetSensitiveCB expects the application shell to
 *			be AppShell as defined as a global value as in the
 *			default main-{c, uil}.c generated by the
 *			Builder Xcessory.
 ******************************************************************************************
 */

/* ARGSUSED */
void
BxSetSensitiveCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxSetSensitiveCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	XtSetSensitive(widgets[i], True);
	i++;
    }
    XtFree((char *)widgets);
}

/*      Function Name:	BxSetInsensitiveCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then set the widget insensitive.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the list of widget names.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *                    * BxSetInsensitiveCB expects the application shell to
 *			be AppShell as defined as a global value as in the
 *			default main-{c, uil}.c generated by the
 *			Builder Xcessory.
 */

/* ARGSUSED */
void
BxSetInsensitiveCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxSetInsensitiveCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	XtSetSensitive(widgets[i], False);
	i++;
    }
    XtFree((char *)widgets);
}

/*      Function Name:	BxPopdownCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then popdown the widgets WITHOUT any grab.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the string of widget names to
 *						popup.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *		      * BxPopdownCB can only work on Shell widgets.  It will
 *			not work on other object types.  This is because
 *			popping down can only be done to a shell.  A check
 *			is made using XtIsShell() and an appropriate error
 *			is output if the passed object is not a Shell.
 */

/* ARGSUSED */
void
BxPopdownCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxPopdownCB", (String)client);
    
    i = 0;
    while ( widgets && widgets[i] != NULL )
    {
	if ( XtIsShell(widgets[i]) )
	{
	    XtPopdown(widgets[i]);
	}
	else
	{
	    printf("Callback Error (BxPopdownCB):\n\t\
Object %s is not a Shell\n", XtName(widgets[i]));
	}
	i++;
    }
    XtFree((char *)widgets);
}

/*      Function Name:	BxPopupCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then popup the widgets WITHOUT any grab.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the string of widget names to
 *						popup.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *		      * BxPopupCB can only work on Shell widgets.  It will not
 *			work on other object types.  This is because popping up
 *			can only be done to a shell.  A check is made using
 *			XtIsShell() and an appropriate error is output if the
 *			passed object is not a Shell.
 */

/* ARGSUSED */
void
BxPopupCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxPopupCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	if ( XtIsShell(widgets[i]) )
	{
	    XtPopup(widgets[i], XtGrabNone);
	}
	else
	{
	    printf("Callback Error (BxPopupCB):\n\t\
Object %s is not a Shell\n", XtName(widgets[i]));
	}
	i++;
    }
    XtFree((char *)widgets);
}

/*
*******************************************************************************************
*
*
*
*	Functions not provided by Bx
*
*
*
*
*
*
*******************************************************************************************
*/

void caa_browse_help  ()
{
	browse_manual(XNBC_XCAA);
} /* fin du void */

/*
************************************************************************************
*
*	Function:
*		caa_ForceUpdate (TheWidget)
*	Description:
*		Force the fetching and displaying of the widget content
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*		june the 19th of 1996
*
*************************************************************************************
*/

void caa_ForceUpdate (Widget TheWidget)
{
/*
**	Variables
*/
	Widget			diashell, topshell;
	Window			diawindow, topwindow;
	XtAppContext		cxt;
	Display			*dpy;
	XWindowAttributes	xwa;
	XEvent			event;
	
/*
**	Code
*/
	cxt = XtWidgetToApplicationContext (TheWidget);

	/*** Locate the shell we are interested in ***/

	for (diashell = TheWidget; !XtIsShell (diashell); diashell = XtParent (diashell))
		;
	/*** Locate its primary window's shell (wich may be the same) ***/

	for (topshell = diashell; !XtIsTopLevelShell(topshell); topshell = XtParent (topshell))
		;

	/***  ***/

	if (XtIsRealized (diashell) && XtIsRealized (topshell))
	{
		dpy = XtDisplay (topshell);
		diawindow = XtWindow (diashell);
		topwindow = XtWindow (topshell);

		/*** Wait for the dialog to be mapped. ***/

		while (XGetWindowAttributes (dpy, diawindow, &xwa) && xwa.map_state != IsViewable)
		{
			if (XGetWindowAttributes (dpy, diawindow, &xwa) && xwa.map_state != IsViewable)
			{
				break;
			}
			XtAppNextEvent (cxt, &event);
			XtDispatchEvent (&event);
		}
	}

	/*** The nest XSync will get an expose event. ***/

	XmUpdateDisplay (topshell);
}
/*
**************************************************************
*
*	Function:
*		DispMessageDialog (Parent, TexteToDisplay, TitleOfDialog, TypeOfDialog)
*	Description:
*		Display a Message Box
*
*	Inputs arguments:
*		Widget		TheParent	:	Widget parent of the MessageBox
*		char*		TextToDisplay	:	...
*		char*		TitleOfDialog	:	...
*		unsigned char	TypeOfDialog	:	...
*		
*	Revision:
*		june the 18th of 1996
*
****************************************************************
*/
void DispMessageDialog (Widget TheParent,char *TextToDisplay, char *TitleOfDialog,unsigned char TypeOfDialog)
{
/*
**	Variables
*/
	static Widget	TheMessageBox;
	XmString	StringText,
			StringTitle;
	Arg		args[3];
	int		n;
/*
**	Code
*/
	n = 0;
	
	StringText = XmStringCreateLtoR (TextToDisplay, XmFONTLIST_DEFAULT_TAG);
	StringTitle = XmStringCreateLocalized (TitleOfDialog);
	
	if (!TheMessageBox)
	{
		XtSetArg (args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
		TheMessageBox = XmCreateMessageDialog (TheParent, "TheMessageBox", args, n);
		XtUnmanageChild (XmMessageBoxGetChild (TheMessageBox, XmDIALOG_CANCEL_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (TheMessageBox, XmDIALOG_HELP_BUTTON));
	}

	XtVaSetValues (TheMessageBox,
		       XmNdialogType, TypeOfDialog,
		       XmNmessageString, StringText,
#ifndef linux
		       XmNdialogTitle, StringTitle,
#endif
		       NULL);
	XmStringFree (StringText);
	XmStringFree (StringTitle);
	
	XtManageChild (TheMessageBox);
	XtPopup (XtParent(TheMessageBox), XtGrabNone);
	caa_ForceUpdate (TheMessageBox);
}
/*
**************************************************************
*
*	Function:
*		DispPopupDialog (Parent, TexteToDisplay, TitleOfDialog)
*	Description:
*		Display a Message Box with no buttons : just uesd to ask the user to wait
*		while something is going on
*
*	Inputs:
*		Widget	TheParent	:	Widget parent of the PopupDialog
*		char*	TestToDisplay	:	...
*		char*	TitleOfDialog	:	...
*	Revision:
*		june the 18th of 1996
*
****************************************************************
*/
void DispPopupDialog (Widget TheParent,char *TextToDisplay, char *TitleOfDialog)
{
/*
**	Variables
*/
	XmString	StringText,
			StringTitle;

	Arg		args[5];
	int		n;
/*
**	Code
*/
	n = 0;
/*
	printf("%s\n%s\n",TextToDisplay,TitleOfDialog);
*/	
	StringText = XmStringCreateLtoR (TextToDisplay, XmFONTLIST_DEFAULT_TAG);
	StringTitle = XmStringCreateLocalized (TitleOfDialog);
	
	if (!ThePopupBox)
	{
		XtSetArg (args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
		XtSetArg (args[n], XmNdialogType, XmDIALOG_INFORMATION); n++;
		ThePopupBox = XmCreateMessageDialog (TheParent, "ThePopupBox", args, n);
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_OK_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_CANCEL_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_HELP_BUTTON));
	}
	XtVaSetValues (ThePopupBox,
		       XmNmessageString, StringText,
#ifndef linux
		       XmNdialogTitle, StringTitle,
#endif
		       NULL);

	XtManageChild (ThePopupBox);

	XmStringFree (StringText);
	XmStringFree (StringTitle);
	
	XtPopup(XtParent(ThePopupBox), XtGrabNone);
	caa_ForceUpdate ( ThePopupBox );
}
/*
**************************************************************
*
*	Function:
*		caa_MainExitButtonCB (widget, client_data, call_data)
*	Description:
*		Exit correctly of the all application
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 19th of 1996
*
****************************************************************
*/

void caa_MainExitButtonCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

/*
**	Code
*/
	XtPopdown (main_TopLevelShell);
	XtPopdown (parameters_TopLevelShell);
	XtPopdown (fileSelection_TopLevelShell);
#if 0
	printf ("Thank you for using XCAA.\n");
#endif
	exit(0);
}

/*
************************************************************************************
*
*	Function:
*		caa_MainPrintOnCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList)
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 20th of 1996
*
*************************************************************************************
*/

void caa_MainPrintOnCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	FILE		*PrintcapFile;
	int		Index;
	char		FileLine[256];
	char		*SomeChar;
	static Widget	printMain_Form;
	int		PrinterCount;
	XmString	SomeString;
/*
**	Code
*/
	PrinterCount = 0;

#ifdef linux
	caa_MainPrintOnOKCB (widget, client_data, call_data);
	return;
#endif
	/*** create the print window the first time ***/

	if (!printMain_Form)
	{
		printMain_Form = (Widget)CreateprintMain_Form(TheAppShell);
	}

	/*** init the printer interface ***/

	XmListDeleteAllItems(printPrinter_List);

	for (Index = 0; Index < PRINTACTIONS_NB; Index++)
	{
		XtSetSensitive (printActions_ToggleButton[Index], True);
	}

	/*** open the printcap file ***/
	
	PrintcapFile = fopen ("/etc/printcap", "r");
	if (PrintcapFile == NULL)
	{
		DispMessageDialog (widget,
				   "Impossible to read /etc/printcap file\n\
				   You can only print in the file",
				   "Error : file not opened",
				   XmDIALOG_ERROR);
		XtSetSensitive (printActions_ToggleButton[0], False);
		XtSetSensitive (printActions_ToggleButton[2], False);
		XmToggleButtonSetState (printActions_ToggleButton[1], True, False);
	}
	else
	{
		XmToggleButtonSetState (printActions_ToggleButton[2], True, False);

		/*** Read the printcap and display the available printers ***/

		PrinterCount = 0;
		while ( fgets (FileLine, 256, PrintcapFile) != NULL)
		{
			/* look for alpha character */
			
			if (isalpha (*FileLine))
			{
				SomeChar = strchr (FileLine, ':');
				if (SomeChar)
				{
					*SomeChar = '\0';
				}
				SomeChar = strchr (FileLine, '|');
				if (SomeChar)
				{
					*SomeChar = '\0';
				}
				SomeString = XmStringCreateSimple (FileLine);
				XmListAddItem (printPrinter_List, SomeString, 0);
				XmStringFree(SomeString);
				PrinterCount++;
			}
		}
		XmListSelectPos (printPrinter_List, 1, False);
	}
	XtVaSetValues (printPrinter_List,
		       XmNvisibleItemCount, PrinterCount,
		       NULL);
	XtManageChild (printMain_Form);
	fclose (PrintcapFile);
}

/*
************************************************************************************
*
*	Function:
*		caa_MainPrintOnOKCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList)
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 20th of 1996
*
*************************************************************************************
*/

void caa_MainPrintOnOKCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
#ifdef linux
	char		PrinterName[80];
#else
	char		*PrinterName;
#endif
	Boolean		Founded;
	int		*PrinterPos;
	int		PrinterCount;
	XmString	*AllPrinterName;
	int		Index;
/*
**	Code
*/
#ifdef linux
	printf("Printer name: ");
	scanf("%s",PrinterName);
#else
	PrinterPos = NULL;
	PrinterName = NULL;
	/*** Get the printer name ***/

	Founded = XmListGetSelectedPos (printPrinter_List,
					&PrinterPos,
					&PrinterCount);
	if (!PrinterCount)
	{
		DispMessageDialog (widget,
				   "No printer has been selected.\n\
				   Please choose a printer",
				   "Error : No printer found",
				   XmDIALOG_ERROR);
		return;
	}

	XtVaGetValues (printPrinter_List,
		       XmNitems, &AllPrinterName,
		       XmNitemCount, &PrinterCount,
		       NULL);
		       
	XmStringGetLtoR (AllPrinterName[PrinterPos[0] - 1], XmSTRING_DEFAULT_CHARSET, &PrinterName);
#endif
	/*** Set the printer queue for ps routines ***/
	
	ps_queue_name (PrinterName);

#ifndef linux
	if (XmToggleButtonGetState (printActions_ToggleButton[0]))
	{
		print_file(widget, (XtPointer) 'd',call_data);
	}
	else if (XmToggleButtonGetState (printActions_ToggleButton[1]))
	{
		print_file(widget, (XtPointer) 's',call_data);
	}
	else if (XmToggleButtonGetState (printActions_ToggleButton[2]))
	{
		print_file(widget, (XtPointer) 'f',call_data);
	}
	if (PrinterPos)
	{
		XtFree (PrinterPos);
	}

	for (Index = 0; Index < PrinterCount; Index++)
	{
		XmStringFree (AllPrinterName[Index]);
	}
	if (PrinterName)
	{
		XtFree (PrinterName);
	}
#endif
}
/*
************************************************************************************
*
*	Function:
*		caa_MainDisplayHelpCB (Flag)
*	Description:
*		fills the Topic menu with items available.
*
*	Inputs:
*
*	Revision:
*		june the 20th of 1996
*
*************************************************************************************
*/

void caa_MainDisplayHelpCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	static Widget	helpMain_Form;
/*
**	Code
*/

	if (!helpMain_Form)
	{
		helpMain_Form = (Widget)CreatehelpMain_Form (TheAppShell);
		caa_MainDispHelpText (widget, (XtPointer) 0, call_data);
	}
	XtManageChild (helpMain_Form);
}
/*
************************************************************************************
*
*	Function:
*		caa_MainDispHelpText (widget, client_data, call_data)
*	Description:
*		create, the first time, the Topic menu's items
*		fills the Topic menu with items available.
*
*	Inputs:
*		int	client_data	the help item number
*
*	Revision:
*		june the 20th of 1996		
*
*************************************************************************************
*/
void caa_MainDispHelpText (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/	
	FILE		*fp;
	char		*pt;
	char		file_name[256];
	XmString	string;
	char		buffer[256];
	static		char **help_files;
	static		char **topics;
	Widget		w;
	static		int nb_items;
	char		*text;
	int		i;
	long		l;
	int		HelpNumber;
/*
**	Code
*/
#ifdef __CYGWIN__
#	define SEP_DIR "\\"
#	define CR 2
#else
#	define SEP_DIR "/"
#	define CR 1
#endif

	HelpNumber = (int) client_data;

	XtInitializeWidgetClass(xmPushButtonWidgetClass);

	if(HelpNumber == -1)
	{
		for(i=0 ; i<nb_items ; i++)
		{
			free(help_files[i]);
			free(topics[i]);
		}
		free(help_files);
		free(topics);
		return;
	}
	
	if(!nb_items)
	{
		strcpy(file_name,help_directory);
		strcat(file_name,SEP_DIR);
		strcat(file_name,"xcaa_hlprc");
		
		if(!(fp = fopen(file_name,"r")))
		{
			sprintf(buffer,"Sorry, file %s not found...",file_name);
			DispMessageDialog (main_TopLevelShell,
					   buffer,
					   "Error : No help available!",
					   XmDIALOG_ERROR);
			return;
		}
		
		nb_items = 0;
		
		while(fgets(buffer,255,fp))
		{	
			pt = strrchr(buffer,':');
			if(!pt)
				continue;
			nb_items++;
			*pt = '\0';
			pt += 2;
			if(nb_items == 1)
			{
				help_files = (char **)calloc(1,nb_items*sizeof(char *));
				topics = (char **)calloc(1,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}
			else
			{
				help_files = (char **)realloc(help_files,nb_items*sizeof(char *));
				topics = (char **)realloc(topics,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}	
		}
		
		for(i=0 ; i<nb_items ; i++)
		{
			string = XmStringCreateLocalized(topics[i]);
			w = XtVaCreateWidget(topics[i],
					     xmPushButtonWidgetClass,
					     helpTopic_PulldownMenu,
					     NULL);
			XtAddCallback(w, XmNactivateCallback, caa_MainDispHelpText, (XtPointer)(i+1));
			XtManageChild(w);
			XmStringFree(string);
		}
		HelpNumber = 1;	
	}
	
	strcpy(file_name,help_directory);
	strcat(file_name,SEP_DIR);
	strcat(file_name,help_files[HelpNumber-1]);
		
	if(!(fp = fopen(file_name,"r")))
	{
		sprintf(buffer,"Sorry, file %s not found...",file_name);
		DispMessageDialog (main_TopLevelShell,
				   buffer,
				   "Error : Topic not available!",
				   XmDIALOG_ERROR);
		return;
	}
	
	fseek(fp, 0L, SEEK_END);
	l = ftell(fp)+1;
	if (!(text = (char *)calloc(1,l*sizeof(char))))
	{
		sprintf(buffer,"Sorry, not possible to load the help on %s...",file_name);
		DispMessageDialog (main_TopLevelShell,
				   buffer,
				   "Error : Not enough memory!",
				   XmDIALOG_ERROR);
		return;
	} /* fin du if */
	fseek(fp, 0L, SEEK_SET);
	fread(text, l*sizeof(char), 1, fp);
	text[l-1] = '\0';
	
	XmTextSetString (help_Text, text);
	
	string = XmStringCreateLocalized(topics[HelpNumber-1]);
	XtVaSetValues(helpTopic_Label,
		      XmNlabelString, string,
		      NULL);
	XmStringFree(string);	
	
	free(text);
}

/*
************************************************************************************
*
*	Function:
*		caa_MainInitMenuVar (widget, client_data, call_data)
*	Description:
*		Initialize the functions name variables
*
*	Inputs:
*
*	Revision:
*		june the 20th of 1996
*
*************************************************************************************
*/

void caa_MainInitMenuVar()
{
		/* Actions boards... */

	action_1[0] = fft_amp;
	action_1[1] = fft_phase;
	action_1[2] = state_space;
	action_1[3] = poincare;
	action_1[4] = triDstate_space;
	action_1[5] = amp_time;
	action_1[6] = auto_correlation;
	action_1[7] = cross_correlation;
}
/*
************************************************************************************
*
*	Function:
*		caa_MainPrintOptionCB (widget, client_data, call_data)
*	Description:
*		Register the choice of automatic spooling or not 
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 21st of 1996
*
*************************************************************************************
*/

void caa_MainPrintOptionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	XmToggleButtonCallbackStruct *ToggleState;

/*
**	Code
*/

	ToggleState = (XmToggleButtonCallbackStruct *) call_data;

	if (ToggleState->set)
	{
		driver = 's' ;
		spooled = TRUE;
		strcpy(on_garde," -k ");
	}
	else
	{
		driver = 'f' ;
		spooled = FALSE;
		strcpy(on_garde," ");
	}
}

/*
**************************************************************
*
*	Function:
*		caa_MainFunctionCB (widget, client_data, call_data)
*	Description:
*		create the fctpara window (only for the first call).
*		Prepare the fctpara window depending of the function selected in the main menu.
*		display the fctpara window.
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 20 of 1996
*		june the 10 of 1996 end of design
*
****************************************************************
*/

void caa_MainFunctionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	Arg		   args[6];
	int		   ac;
	int		   Index;
	Widget		   fctparaMain_Form;
	char		   SomeChar[250];
	char		   *ClientValue;
	char		   TheCalcul;
	short		   ActionNumber;
	static pCALCULSARG FunctionArg;
	Boolean		   JustOneWindow;
	char		   PrintfChar[70];
/*
**	Code
*/
	/*** Menus choices ***/


	strcpy(menu_1[0], " A Fast Fourier Transform: amplitude spectrum");
	strcpy(menu_1[1], " B Fast Fourier Transform: phase spectrum");
	strcpy(menu_1[2], " C State space of a cluster");
	strcpy(menu_1[3], " D State at t versus t+1 (poincare map)");
	strcpy(menu_1[4], " E State space versus time (3D)");
	strcpy(menu_1[5], " F Amplitude as a function of time");
	strcpy(menu_1[6], " G Amplitude autocorrelation function");
	strcpy(menu_1[7], " H Amplitude cross-correlation function");

	/*** dispatch actions ***/

	JustOneWindow = True;
	ClientValue = (char *) client_data;
	
	PrintfChar[0] = '\0';

	if (!fctpara_TopLevelShell)
	{
		strcpy (PrintfChar, VERSION);

		ac = 0;
		XtSetArg(args[ac], XmNtitle, PrintfChar); ac++;
		XtSetArg(args[ac], XmNiconName, PrintfChar); ac++;
		XtSetArg(args[ac], XmNx, 200); ac++;
		XtSetArg(args[ac], XmNy, 200); ac++;
		XtSetArg(args[ac], XmNwidth, 730); ac++;
		XtSetArg(args[ac], XmNheight, 680); ac++;
		fctpara_TopLevelShell = XtCreatePopupShell("fctpara_TopLevelShell",
							   topLevelShellWidgetClass,
							   TheAppShell,	
							   args, 
							   ac);
		fctparaMain_Form = (Widget)CreatefctparaMain_Form(fctpara_TopLevelShell);
		XtManageChild(fctparaMain_Form);

		FunctionArg = (pCALCULSARG) XtMalloc (sizeof (CALCULSARG));
	}
	else
	{
		if (XtIsRealized(fctpara_TopLevelShell))
		{
			caa_fctparaDismissCB (fctparaApply_PushButton, client_data, call_data);
		}
	}

	/*** Initialize the lists of widgets sensitive state ***/

	for (Index = 0; Index < FCTPARA_PARANB; Index++)
	{
		FunctionArg->FctparaList[Index] = False;
	}

	/*** empty all fctpara's text fields***/

	XmTextSetString (fctparaTheta_TextField, "");
	XmTextSetString (fctparaPhi_TextField, "");
	XmTextSetString (fctparaObsDist_TextField, "");
	XmTextSetString (fctparaProp_TextField, "");
	XmTextSetString (fctparaDuration_TextField, "");
	XmTextSetString (fctparaACluster_TextField, "");
	XmTextSetString (fctparaBCluster_TextField, "");

	/*** select wich functions to call for calculs ***/

	TheCalcul = ClientValue[0];
	ActionNumber = (int) TheCalcul - (int) 'A';
	strcpy(FunctionArg->action, ClientValue);

	/*** prepare the FctPara window ***/

	switch (TheCalcul)
	{
		case 'A':
		{
			JustOneWindow = False;
			break;
		}
		case 'B':
		{
			JustOneWindow = False;
			break;
		}
		case 'C':
		{
			JustOneWindow = False;
			break;
		}
		case 'D':
		{
			JustOneWindow = False;
			break;
		}
		case 'E':
		{
			FunctionArg->FctparaList[0] = True;
			FunctionArg->FctparaList[1] = True;
			FunctionArg->FctparaList[2] = True;
			FunctionArg->FctparaList[3] = True;
			
			/*** put the default values ***/

			sprintf (SomeChar,"%f", 30.);
			XmTextSetString (fctparaTheta_TextField, SomeChar);
			sprintf (SomeChar,"%f", 30.);
			XmTextSetString (fctparaPhi_TextField, SomeChar);
			sprintf (SomeChar,"%f", 20.);
			XmTextSetString (fctparaObsDist_TextField, SomeChar);
			sprintf (SomeChar,"%d", 1);
			XmTextSetString (fctparaProp_TextField, SomeChar);

			JustOneWindow = True;
			break;
		}
		case 'F':
		{
			JustOneWindow = False;
			break;
		}
		case 'G':
		{
			FunctionArg->FctparaList[4] = True;

			/*** put the default values ***/

			sprintf (SomeChar,"%d", 100);
			XmTextSetString (fctparaDuration_TextField, SomeChar);
			JustOneWindow = True;
			break;
		}
		case 'H':
		{
			FunctionArg->FctparaList[4] = True;
			FunctionArg->FctparaList[5] = True;
			FunctionArg->FctparaList[6] = True;

			/*** put the default values ***/

			sprintf (SomeChar,"%d", 100);
			XmTextSetString (fctparaDuration_TextField, SomeChar);
			sprintf (SomeChar,"%d", 1);
			XmTextSetString (fctparaACluster_TextField, SomeChar);
			sprintf (SomeChar,"%d", 0);
			XmTextSetString (fctparaBCluster_TextField, SomeChar);

			JustOneWindow = True;
			break;
		}
		default:
		{
			break;
		}
	}
	sprintf (PrintfChar,"%s - %s", VERSION, menu_1[ActionNumber]);
	XtVaSetValues (fctpara_TopLevelShell,
		       XmNtitle, PrintfChar,
		       NULL);
	FunctionArg->function = action_1[ActionNumber];

	XtAddCallback (fctparaCompute_PushButton,
		       XmNactivateCallback, caa_fctparaComputeCB, FunctionArg);
	XtAddCallback (fctparaApply_PushButton,
		       XmNactivateCallback, caa_fctparaApplyCB, FunctionArg);

	caa_fctparaSetSensitive (FunctionArg->FctparaList);
	if (JustOneWindow)
	  XtPopup (fctpara_TopLevelShell, XtGrabNone);
	else
	   caa_fctparaComputeCB(widget,(XtPointer)FunctionArg,(XtPointer)NULL);
	return;
}
/*
**************************************************************
*
*	Function:
*		caa_RegisterCAAFileCB (widget, client_data, call_data)
*	Description:
*		Register the file name and open the working file
*		Also create the Postscript file name
*	Inputs:
*		Widget		widget		:	the activating widget.
*		XtPointer	client_data	:	The selection of the FilsSelectionBox (the filename...)
*		XtPointer	call_data	:	the call data (unused).
*
*	Revision:
*		june the 20th of 1996
*
****************************************************************
*/

void caa_RegisterCAAFileCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

XmFileSelectionBoxCallbackStruct *CallStruct;
char				 *TmpFileName,
				 AMessage [FILENAME_SIZE + 20]; /* Message to display in the MessageDialog */
XmString			 CurrentPSName,
				 CurrentPSExtension;
int				 FileNameSize;
char				 *SomeChar;
/*
**	Code
*/
	TmpFileName = NULL;
	/* Initialisation */

	CallStruct = (XmFileSelectionBoxCallbackStruct *) call_data;
	AMessage[0] = '\0';

	/* Get the file name selected */

	if (!XmStringGetLtoR (CallStruct->value,
			     XmFONTLIST_DEFAULT_TAG,
			     &TmpFileName))
	{
		DispMessageDialog (widget,
				   "Unable to get the file name",
				  "Error : file not opened",
				  XmDIALOG_ERROR);
		return;
	}

	/* Test the file name */

	if (!TmpFileName[0])
	{
		DispMessageDialog (widget,
				   "Empty value for file name",
				   "Error : file not opened",
				   XmDIALOG_ERROR);

		return;
	}
	else
	{
		/* Create filename without path */

		/* Open the file */
		
		FileNameSize = strlen (TmpFileName);		
		strcpy (filename,TmpFileName);
/*
		printf("Filename: %s\n",filename);
*/
		if((data = fopen(TmpFileName,"r")) == NULL)
		{
			strcpy (AMessage, "impossible to open file ");
			strcat (AMessage, filename);
			DispMessageDialog (widget,
					   AMessage,
					   "Error : file not opened",
					   XmDIALOG_ERROR);
			return;
		}
		else

		/* File opened, ask for the postcript file name... */

		{

			/* Create postcript file name and generic name */

			strcpy (scriptname, filename);
			SomeChar = strrchr (scriptname, '.');
			*SomeChar = '\0';
			strcpy (file,scriptname);
			strcat (scriptname, PRINTED_EXT );

			fclose (data); /* File will be opened again in ParametersApplyCB callback...*/

/*
			XtUnmanageChild(widget);
*/

			/* modify the FileSelectionBox for the next call : the postcriptname */

                        CurrentPSName = XmStringCreateLocalized (scriptname);
                        CurrentPSExtension = XmStringCreate ("*.ps*",XmSTRING_DEFAULT_CHARSET);
			XtVaSetValues (widget,
                                       XmNpattern, CurrentPSExtension,
                                       XmNtextString, CurrentPSName,
				       NULL);

			XtRemoveCallback (widget, XmNokCallback,
					 caa_RegisterCAAFileCB, NULL);
                        XtAddCallback(widget, XmNokCallback, BxSetSensitiveCB,
                                      (XtPointer) "mainClose_PushButton, mainSavePS_PushButton, \n\
					mainPrint_PushButton,  mainOperations_CascadeButton, \n\
					mainOptions_CascadeButton");
                        XtAddCallback(widget, XmNokCallback, BxSetInsensitiveCB,
                                      (XtPointer) 
"mainOpA_PushButton, \
mainOpB_PushButton, \
mainOpC_PushButton, \
mainOpD_PushButton, \
mainOpE_PushButton, \
mainOpF_PushButton, \
mainOpG_PushButton, \
mainOpH_PushButton");


			XtAddCallback (widget, XmNokCallback,
				      caa_RegisterCAAScriptFileCB, NULL);
/*
			XtAddCallback (widget, XmNcancelCallback,
				      caa_RegisterCancelCB, NULL);
*/
			XtManageChild(widget);
		}
	}
	XmStringFree (CurrentPSName);
	XmStringFree (CurrentPSExtension);
	if (TmpFileName)
	{
		XtFree (TmpFileName);
	}
}
/*
**************************************************************
*
*	Function:
*		caa_RegisterCancelCB (widget, client_data, call_data)
*	Description:
*		change the FileSelectionBox's callbacks
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client_data (unused)
*		XtPointer	call_data:	the call data (unused).
*
*	Revision:
*		june the 20th of 1996
*
****************************************************************
*/
void caa_RegisterCancelCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	XmString	CurrentFileName;
	XmString	CurrentFileExt;
/*
**	Code
*/
	/*** prepare the FileSelectionBox ***/

	CurrentFileName = XmStringCreateLocalized (filename);
	CurrentFileExt = XmStringCreate ("*.sim",XmSTRING_DEFAULT_CHARSET);
	XtVaSetValues (widget,
		       XmNpattern, CurrentFileExt,
                       XmNtextString, CurrentFileName,
		       NULL);

	/*** replace FileSelectionBox's callbacks ***/

	XtRemoveAllCallbacks (widget, XmNokCallback);
	XtAddCallback (widget, XmNokCallback,
		       caa_RegisterCAAFileCB, NULL);
/*
	XtRemoveCallback(widget, XmNokCallback, BxSetSensitiveCB,
			 (XtPointer) "mainClose_PushButton, mainSavePS_PushButton, \n\
			mainPrint_PushButton, mainPrintOn_PushButton, mainOperations_CascadeButton, mainOptions_CascadeButton");
	XtRemoveCallback (widget, XmNokCallback,
			  caa_RegisterCAAScriptFileCB, NULL);
	XtRemoveCallback (widget, XmNcancelCallback,
			  caa_RegisterCancelCB, NULL);
*/
	XtManageChild(widget);
}
/*
**************************************************************
*
*	Function:
*		caa_RegisterCAAScriptFileCB (widget, client_data, call_data)
*	Description:
*		Register the postcript file name from the FileSelectionBox
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the selection of the FileSelectionBox (the postscript file name)
*		XtPointer	call_data:	the call data (unused).
*
*       Modified global variables:
*		char*		scriptname
*
*	Revision:
*		june the 20th of 1996
*
****************************************************************
*/
void caa_RegisterCAAScriptFileCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

	XmFileSelectionBoxCallbackStruct *CallStruct;
	char				 *TmpScriptName;
	XmString			 CurrentFileName;
	XmString			 CurrentFileExt;

/*
**	Code
*/
	/* Initialisation */

	CallStruct = NULL;
	TmpScriptName = NULL;

	CallStruct = (XmFileSelectionBoxCallbackStruct *) call_data;

	/* Get the postscript file name from the FileSelectionBox */

	if (!XmStringGetLtoR (CallStruct->value,
			     XmFONTLIST_DEFAULT_TAG,
			     &TmpScriptName))
	{
		DispMessageDialog (widget,
				   "Unable to get the Postscript file name",
				  "Error : filename not found",
				  XmDIALOG_ERROR);
		return;
	}
/*
	printf("TmpScriptName: %s\n", TmpScriptName);
*/
	if (*(strrchr(TmpScriptName,'/')+sizeof(char)))
		strcpy(scriptname, TmpScriptName);
/*
	printf("scriptname: %s\n", scriptname);
*/
	XtPopdown (XtParent(widget));
	BxPopupCB(widget,(XtPointer)"parameters_TopLevelShell",NULL);
	caa_OpenParametersCB(widget,NULL,NULL);

	/*** prepare the FileSelectionBox for next call ***/

	CurrentFileName = XmStringCreateLocalized (filename);
	CurrentFileExt = XmStringCreate ("*.sim",XmSTRING_DEFAULT_CHARSET);

	XtVaSetValues (widget,
		       XmNpattern, CurrentFileExt,
                       XmNtextString, CurrentFileName,
		       NULL);

	/*** replace FileSelectionBox callbacks ***/

	XtRemoveAllCallbacks (widget, XmNokCallback);
	XtAddCallback (widget, XmNokCallback,
		      caa_RegisterCAAFileCB, NULL);

	XmStringFree (CurrentFileName);
	XmStringFree (CurrentFileExt);
#ifndef linux
	if (CallStruct)
	{
		XtFree (CallStruct);
	}
#endif
	if (TmpScriptName)
	{
		XtFree (TmpScriptName);		
	}

}

/*
**************************************************************
*
*	Function:
*		caa_OpenParametersCB (widget, client_data, call_data)
*	Description:
*		Open the Parameters Winodw and file the textfield with filename and Postscript file name
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, not used
*		XtPointer	call_data:	the call data, not used
*
*	Revision:
*		june the 21th of 1996
*
****************************************************************
*/
void caa_OpenParametersCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	char  TmpFileName[FILENAME_SIZE];
	char  TmpScriptName[FILENAME_SIZE];
/*
**	Code
*/
	/* Initialisation */

	if (strrchr(filename,'/'))
	  strcpy(TmpFileName, (strrchr(filename,'/') + sizeof(char)) );     /* + size(char) to get the name without '/' */
	else
	  strcpy(TmpFileName,filename);
	if (strrchr(scriptname,'/'))
	  strcpy(TmpScriptName, (strrchr(scriptname,'/') + sizeof(char)) );
	else
	  strcpy(TmpScriptName,scriptname); 

	XtVaSetValues (paraProcessedFile_TextField,
		       XmNvalue, filename,
		       NULL);
	XtVaSetValues (paraPostscriptFile_TextField,
		       XmNvalue, scriptname,
		       NULL);
	XtVaSetValues (paraEventNb_TextField,
		       XmNvalue, "0",
		       NULL);
	XtVaSetValues (paraStartAna_TextField,
		       XmNvalue, "",
		       NULL);
	XtVaSetValues (paraStopAna_TextField,
		       XmNvalue, "",
		       NULL);
	XtVaSetValues (paraDurationAna_TextField,
		       XmNvalue, "",
		       NULL);

	XtSetSensitive( parametersOK_PushButton,False);
	XtManageChild ( paraProcessedFile_TextField );
	XtManageChild ( paraPostscriptFile_TextField );
	XtManageChild ( XtParent( paraPostscriptFile_TextField ) );
}

/*
**************************************************************
*
*	Function:
*		caa_ParametersOKCB (widget, client_data, call_data)
*	Description:
*		Register the event's nb and compute it's caracteristics from the Parameters Window
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 21th of 1996
*
****************************************************************
*/

void caa_ParametersOKCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

	Widget		TheParent;
	XmString	SomeString;

	char		*EventTyped;
	char		AMessage[250];
	char		*SomeChar;
	char		SprintfChar[80];
	char		ClientButton;

	Boolean		ValidData;
	int		Index;
	Boolean		CloseWin;
/*
**	Code
*/

	CloseWin = False;
	TheParent = (Widget) client_data;
	EventTyped = NULL;
	ValidData = False;
	SomeChar = NULL;
	EventTyped = NULL;
	
	/*** empty some TextFields and initialise some global variables***/

	nb_de_types = 0;
	nb_donnees = 0;
	cycle_marker = 0;

	SomeChar = (char *) XmTextGetString (paraEventNb_TextField);
	if (!strcmp(SomeChar,"#NF#") || !strcmp(SomeChar,""))
	{
		XtVaSetValues (paraEventNb_TextField,
			       XmNvalue, "",
			       NULL);
	}
	if (SomeChar)
	{
		XtFree (SomeChar);
	}
	
	/*** get the events selected ***/

	if((data = fopen(filename,"r")) == NULL)
	{
		strcpy (AMessage, "Impossible to open file ");
		strcat (AMessage, filename);
		DispMessageDialog (widget,
				   AMessage,
				   "Error : file not opened",
				   XmDIALOG_ERROR);
		XtPopdown (XtParent(widget));
		return;
	}

	EventTyped = (char *) XmTextGetString (paraEventNb_TextField);
	if (strcmp(EventTyped,""))
	{
		if (strcmp(EventTyped,"#NF#"))
		{
			cluster = atoi (EventTyped);
		}
	}
	if (EventTyped)
	{
		XtFree (EventTyped);
	}

	DispPopupDialog (widget,
			 "Reading input data...",
			 "Please wait." );
	for (Index = 0; Index < MAX_CLUSTER; Index++)
	{
		nb_data[Index] = 0;
	}
	read_data();
	fclose(data) ;

	SomeChar = (char *) XmTextGetString (paraStartAna_TextField);
	if (!strcmp(SomeChar, ""))
	{
		depart = 0.;
		sprintf(SprintfChar, "%.3f", depart);
		XtVaSetValues (paraStartAna_TextField,
			       XmNvalue, SprintfChar,
			       NULL);
	}
	else
	{
		depart = atof (SomeChar);
	}
	if (SomeChar)
	{
		XtFree (SomeChar);
	}
	SomeChar = (char *) XmTextGetString (paraStopAna_TextField);
	if (!strcmp(SomeChar, ""))
	{
		fin = (float) (nb_data[cluster] - 1);
		sprintf(SprintfChar, "%.3f", fin);
		XtVaSetValues (paraStopAna_TextField,
			       XmNvalue, SprintfChar,
			       NULL);
	}
	else
	{
		fin = atof (SomeChar);
	}
	if (SomeChar)
	{
		XtFree (SomeChar);
	}

	arrivee = fin;
	nb_donnees = arrondi (fin - debut) ;

	XtPopdown (XtParent(ThePopupBox));

	if (!nb_donnees)
	{
		DispMessageDialog (widget,
				   "Sorry, no matching data were found",
				   "Can't find datas",
				   XmDIALOG_INFORMATION	);
		return;
	}
	else
	{
		ValidData = True;
	}

	rep_data = 1;
	
	temps_max[cluster] = temps[nb_donnees - 1][cluster] ;

	if (arrivee > temps_max[cluster]) arrivee = temps_max[cluster] ;
	if (depart < temps[0][cluster]) depart=temps[0][cluster] ;

	sprintf(SprintfChar, "%.3f", (arrivee - depart));
	XtVaSetValues (paraDurationAna_TextField,
		       XmNvalue, SprintfChar,
		       NULL);
	Index = 0;

	ClientButton = (char) client_data;
	if (ClientButton == 'O')
	{
	
		if (!ValidData)
		{
			DispMessageDialog (widget,
					   "You didn't choose a valid cluster.\nPress Cancel or choose some data",
					   "No data",
					   XmDIALOG_INFORMATION	);
			return;
		}
		else
		{
			/*** Prepare the fctpara window ***/

			sprintf (SprintfChar, "%d", cluster);
			SomeString = XmStringCreateLocalized (SprintfChar);
			XtVaSetValues (fctparaTheClusterNb_Label,
				       XmNlabelString, SomeString,
				       NULL);
			XmStringFree (SomeString);

			/*** Prepare the Axis Window and the variables it changes. So if the user doesn't  ***
			***	use the Axis Window, variables are set to default now.                     ***/
			
			long_X_axis = LONG_X;
			long_Y_axis = LONG_Y;
			sprintf (SprintfChar, "%.3f", long_X_axis);
			XtVaSetValues (axisXLenght_TextField,
				       XmNvalue, SprintfChar,
				       NULL);
			sprintf (SprintfChar, "%.3f", long_Y_axis);
			XtVaSetValues (axisYLenght_TextField,
				       XmNvalue, SprintfChar,
				       NULL);
		
		        Logo_XCAA();

			/*** Close Parameters window... ***/
			XtPopdown ( XtParent( XtParent( widget ) ) );
			XRaiseWindow(XtDisplay(main_Form),XtWindow(main_Form));
		}
	}
	else
		XtSetSensitive( parametersOK_PushButton,True);

}
/*
**************************************************************
*
*	Function:
*		caa_AxisApplyCB (widget, client_data, call_data)
*	Description:
*		Checks if the Axis window datas are corrects and records the values
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 21th of 1996
*
****************************************************************
*/

void caa_AxisApplyCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	char    *SomeChar1,
		*SomeChar2,
		ErrorMessage1[120],
		ErrorMessage2[60];
	Boolean XAxisWrong,
		YAxisWrong;
/*
**	Code
*/
	XAxisWrong = False;
	YAxisWrong = False;
	SomeChar1 = NULL;
	SomeChar2 = NULL;

	SomeChar1 = (char *) XmTextGetString (axisXLenght_TextField);
	long_X_axis = atof (SomeChar1);	
	SomeChar2 = (char *) XmTextGetString (axisYLenght_TextField);
	long_Y_axis = atof (SomeChar2);

	if (long_X_axis == 0 || !strcmp(SomeChar2, ""))
	{
		long_X_axis = 18.;
		XAxisWrong = True;
		strcpy(ErrorMessage1, "X axis lenght musn't be null\n");
	}
	else if (long_X_axis > 18.)
	{
		long_X_axis = 18.;
		XAxisWrong = True;
		strcpy(ErrorMessage1, "X axis lenght limited to 18. cms (paper size constraint!\n");
	}
	if (long_Y_axis == 0 || !strcmp(SomeChar2, ""))
	{
		long_Y_axis = 20.;
		YAxisWrong = True;
		strcpy(ErrorMessage2, "Y axis lenght musn't be null");
	}
	else if (long_Y_axis > 20.)
	{
		long_Y_axis = 20.;
		YAxisWrong = True;
		strcpy(ErrorMessage2, "Y axis lenght limited to 18. cms (paper size constraint!)");
	}
	if (XAxisWrong || YAxisWrong)
	{
		sprintf (SomeChar1, "%.3f", long_X_axis);
		XmTextSetString (axisXLenght_TextField, SomeChar1);

		sprintf (SomeChar2, "%.3f", long_Y_axis);
		XmTextSetString (axisYLenght_TextField, SomeChar2);

		strcat (ErrorMessage1, ErrorMessage2);
		DispMessageDialog (widget,
				   ErrorMessage1,
				   "Axis lenght has been changed",
				   XmDIALOG_INFORMATION	);	
	}

	if (SomeChar1)
	{
		XtFree (SomeChar1);
	}
	if (SomeChar2)
	{
		XtFree (SomeChar2);
	}
}

/*
************************************************************************************
*
*	Function:
*		caa_VerifyForCompute (pCALCULSARG FunctionArg)
*	Description:
*		Verify selection in 'fctpara' window and register the selections
*	Inputs:
*
*	Outputs:
*		pCALCULSARG FunctionArg	:	caracteristics of the selection. only events number are
*						initialized
*	Revision:
*		may the 20 of 1996
*		june the 10 of 1996 end of design
*
*************************************************************************************
*/

int caa_VerifyForCompute (pCALCULSARG FunctionArg)
{
/*
**	Variables
*/
	char		*SomeChar;
	char		CharToPrint[250];
	short		Index;
/*
**	Code
*/
	Index = 0;
	SomeChar = NULL;

	if (FunctionArg->FctparaList[Index])
	{
		SomeChar = (char *) XmTextGetString (fctparaTheta_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%f", 30.);
			XmTextSetString (fctparaTheta_TextField, CharToPrint);
			FunctionArg->theta = atof (CharToPrint);
		}
		else
		{
			FunctionArg->theta = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index])
	{
		SomeChar = (char *) XmTextGetString (fctparaPhi_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%f", 30.);
			XmTextSetString (fctparaPhi_TextField, CharToPrint);
			FunctionArg->phi = atof (CharToPrint);
		}
		else
		{
			FunctionArg->phi = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index])
	{
		SomeChar = (char *) XmTextGetString (fctparaObsDist_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%f", 20.);
			XmTextSetString (fctparaObsDist_TextField, CharToPrint);
			FunctionArg->obsdist = atof (CharToPrint);
		}
		else
		{
			FunctionArg->obsdist = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index])
	{
		SomeChar = (char *) XmTextGetString (fctparaProp_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%f", 10.);
			XmTextSetString (fctparaProp_TextField, CharToPrint);
			FunctionArg->prop = atof (CharToPrint);
		}
		else
		{
			FunctionArg->prop = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index]) /* set duration parameter */
	{
		SomeChar = (char *) XmTextGetString (fctparaDuration_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%d", 10);
			XmTextSetString (fctparaDuration_TextField, CharToPrint);
			FunctionArg->duration = atof (CharToPrint);
		}
		else
		{
			FunctionArg->duration = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index]) /* set ACluster parameter */
	{
		SomeChar = (char *) XmTextGetString (fctparaACluster_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%d", 3);
			XmTextSetString (fctparaACluster_TextField, CharToPrint);
			FunctionArg->acluster = atof (CharToPrint);
		}
		else
		{
			FunctionArg->acluster = atof (SomeChar);
		}
	}
	Index++;
	if (FunctionArg->FctparaList[Index]) /* set BCluster parameter */
	{
		SomeChar = (char *) XmTextGetString (fctparaBCluster_TextField);
		if (!strcmp(SomeChar, ""))
		{
			sprintf (CharToPrint, "%d", 0);
			XmTextSetString (fctparaBCluster_TextField, CharToPrint);
			FunctionArg->bcluster = atof (CharToPrint);
		}
		else
		{
			FunctionArg->bcluster = atof (SomeChar);
		}
	}

	if (SomeChar)
	{
		XtFree (SomeChar);
	}
	return 1;

}
/*
************************************************************************************
*
*	Function:
*		caa_fctparaApplyCB (widget, client_data, call_data)
*	Description:
*		Verify the events selected in fctpara window and execute the calcul funtion
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the arguments of calcul functions (type : pCALCULSARG)
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 1st of 1996
*
*************************************************************************************
*/

void caa_fctparaApplyCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	pCALCULSARG	FunctionArg;
	int		TheStatus;
/*
**	Code
*/

	
	FunctionArg = (pCALCULSARG) client_data;

	/*** Verify that datas are present and that its have been selected in Window parameters ***/

	TheStatus = caa_VerifyForCompute (FunctionArg);

	if (TheStatus == 0)
	{
		DispMessageDialog (widget,
				   "You didn't select an event for calculs.\n\
				   Or the event you selected is not available.\n\
				   Please chose an event",
				   "Warning",
				   XmDIALOG_INFORMATION);
		return;
	}
	else if (TheStatus == -2)
	{
		DispMessageDialog (widget,
				   "Some parameters don't have a good value.\n Please change your selection",
				   "Warning",
				   XmDIALOG_ERROR);
		return;
	}

}
/*
************************************************************************************
*
*	Function:
*		caa_fctparaComputeCB (widget, client_data, call_data)
*	Description:
*
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	The arguments of calculs funtions (type : pCALCULSARG)
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 20th of 1996
*		june the 10th of 1996 end of design
*
*************************************************************************************
*/

void caa_fctparaComputeCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	pCALCULSARG	FunctionArg;
	int		TheStatus;
/*
**	Code
*/
	FunctionArg = (pCALCULSARG) client_data;
	
	/*** Verify that datas are present and that it have been selected in Window parameters ***/

	TheStatus = caa_VerifyForCompute (FunctionArg);

	if (TheStatus > 0)
	{
		if (FirstUse)
		{
			Page_de_garde();
			FirstUse = False;
		}
		(*FunctionArg->function)(FunctionArg);
	}
	else if (TheStatus == 0)
	{
		DispMessageDialog (widget,
				   "You didn't select an event for calculs.\n\
				   Or the event you selected is not available.\n\
				   Please chose an event",
				   "Warning",
				   XmDIALOG_INFORMATION);
	}
	else if (TheStatus == -2)
	{
		DispMessageDialog (widget,
				   "Some parameters don't have a good value.\n Please change your selection",
				   "Warning",
				   XmDIALOG_ERROR);
	}
	XtPopdown (XtParent(XtParent(widget)));
	XRaiseWindow(XtDisplay(main_Form),XtWindow(main_Form));

	/*** remove callbacks of all parameters window exepted fctpara window ***/

	XtRemoveAllCallbacks (fctparaCompute_PushButton, XmNactivateCallback);
	XtRemoveAllCallbacks (fctparaApply_PushButton, XmNactivateCallback);

}
/*
************************************************************************************
*
*	Function:
*		caa_fctparaDismissCB (widget, client_data, call_data)
*	Description:
*		undisplay the fctpara window and set default caracteristics for some fctpara window widgets
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		june the 1st of 1996
*
*************************************************************************************
*/

void caa_fctparaDismissCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
/*
**	Code
*/
	XtPopdown (XtParent(XtParent(widget)));

	/*** remove callbacks of all parameters window exepted fctpara window ***/

	XtRemoveAllCallbacks (fctparaCompute_PushButton, XmNactivateCallback);
	XtRemoveAllCallbacks (fctparaApply_PushButton, XmNactivateCallback);
}

/*
************************************************************************************
*
*	Function:
*		caa_FctparaSetSensitive (*SensitifList[])
*	Description:
*		set sensitif or unsensitif the fctpara window's widgets refering to
*		a boolean list.
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*	Revision:
*		june the 5th of 1996
*
*************************************************************************************
*/

void caa_fctparaSetSensitive (Boolean *SensitifList)
{
/*
**	Variables
*/
	int	Index;
/*
**	Code
*/
	Index = 0;
	XtVaSetValues (fctparaTheta_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (fctparaModified_Label[Index + 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);

	Index++;
	XtVaSetValues (fctparaPhi_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (fctparaModified_Label[Index + 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (fctparaObsDist_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (fctparaModified_Label[Index + 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (fctparaProp_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
       	XtVaSetValues (fctparaModified_Label[Index + 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (fctparaDuration_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
       	XtVaSetValues (fctparaModified_Label[Index + 2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (fctparaACluster_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
       	XtVaSetValues (fctparaModified_Label[Index + 2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (fctparaBCluster_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
       	XtVaSetValues (fctparaModified_Label[Index + 2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
}










