/*
to obtain an X version of caa, called xcaa, you must either
	#define XCAA
	or make with the -DXCAA flag
If XCAA is not defined, you will get a non X version
*/
/************************************************************************
*									*
*			C A A _ U N I X . C	(unix)                  *
*                                                                       *
*
* caa_unix.c programme permettant de traiter les donnees venant de
* NBC (*.sim) pour l'analyse des decharges globales des clusters
*
*
*		Logiciel d'analyse dynamique de donnees
*		Neurophysiologiques 
*
* Menu ou l'on se deplace avec les fleches ou donne une lettre
* correspondant au choix voulu.
*
*	Version menu du prgram neurdyn.c de Sylvain Hanneton
*
*	caa_unix.c pour le menu
*
*		Auteurs: Sylvain HANNETON et J-F VIBERT
*			Laboratoire de Physiologie
*			CHU Saint-Antoine
*			27 rue Chaligny
*			75571 Paris cedex 12
*			France
*
*		et
*			Ecole Normale Superieure (SH)
*			45 Rue d'ulm
*			75005 Paris
*
*	Modifies par
*		JF Vibert
*		B3E ESI INSERM U444
*               CHU Saint-Antoine
*               27 rue Chaligny
*               75571 Paris cedex 12
*               France
*
*	V3.0	Version Xwindow/Motif	P Duvinage et JF Vibert	23 jul 96
*       V4.0    Graph dans la fenetre principale JF Vibert 12 aout 96
*       V4.2    Adaptation a plot_xps queue name JF Vibert 10 avr 1997
*       V4.3    Prise en compte de .xnbcrc JF Vibert 10 avr 1997
*       V4.4    Changement de place des help JF Vibert 6 oct 1999
*       V4.5    Ajout d'un bouton de changet de nom de fichier JF Vibert 8 oct 1999
*
*
*	Auteur: Dr J-F VIBERT
*		B3E ESI INSERM U444
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*		France
*
*	Copyright Dr JF Vibert 
*
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for research purpose
*	and is not, in any way usable for commercial purpose.
*
*/
#ifdef	DOCUMENTATION

title  CAA	Cluster Activity Analysis of NBC data

index	  	Cluster Activity Analysis of NBC data

synopsis
	
	
	caa [-s|k] [-d] [-v] file_in[.sim]

description

	CAA (General Discharge Analysis) allows to analyze the NBC cluster
	behavior.

	CAA can be compiled for use of Xwindow. It should be installed as XCAA
	It is a special version of CAA showing the output graphs during
	the analysis in a second X window. The terminal emulation window
	must be positionned at the right part of the sreen, in order to be
	visible when the graphic window will be automatically opened on the
	left side.  Otherwise, XCAA has exactly the same fonctionalities and
	interface that CAA, except the post analysis visualisation using the
	local PostScript previewier, since graphs are seen in real time.

	\nofill
	{\tt -v}: verbose switch
	{\tt -d}: default values switch
	{\tt -s|k}: keep PostScript file switch
	\fill



	CAA is menu driven.

	Moving in the menu is done using the arrow down and up arrows keys
	that allow to move from field to field. <CR> key allow to activates
	the menu item in reverse video. Menu item can also be activated by
	keying the capital letter. In this case the <CR> key has not to be
	depressed.

	Results are given as PostScript graphics, either displayed using the
	local PostScript previewer (dxpsview for exemple), and/or
	automatically spooled (or if asked from the menu area).

	The main menu propose some general actions and the following submenus:

	{\tt
	\nofill
	    S Spooling or not of the PostScript file (toggle)
	    V Visualization on screen (Workstation only) 
	    P Send to printer 
	
	    A Fast Fourier Transform: amplitude spectrum
	    B Fast Fourier Transform: phase spectrum
	    C State space of a cluster.
	    D State at t, (t+1) (Poincare map)
	    E Amplitude as a function of time
	    F Amplitude autocorrelation function
	    G Amplitude cross-correlation function
	    
	    Q Quit CAA 
	\fill
	}

	Start and End analysis are asked.

	For each analysis, specific question are asked.  All questions admit
	a default answer indicated at the prompt.
	
	An immediate <CR> sends this default value.

	A movement with one of the editing keys (see below) allows to edit
	this default value in order to change it.

	Any other new entry annulates the default value and takes the new
	key strokes as the beginning of a new value.

	Editing keys can be used at any moment. The arrow keys have their
	natural role: left arrow to go backward character by character,
	rigtharrow to go forward character by character. The special
	VT100/VT2xx/VT3xx PFn keys are significant:

	\nofill
	LEFT ARROW:     backward one character
	RIGHT ARROW:    forward one character
	PF1:    backward to beginning of line
	PF2:    backward one word
	PF3:    forward one word
	PF4:    forward to end of line
	UP ARROW:       backward in a 10 item ring buffer history
	DOWN ARROW:     forward in a 10 item ring buffer history
	CTRL R or Find:  considers  the string already  typed  as the beginning of a memorized string, and recalls the full string if any, or beeps if not.
	?  or Help:  gives an help text in reverse video characters, generaly at the top  of  the  screen,  or a fugace message {\bf No Help} at the current position.
	CTRL G: aborts the entry and calls back the default value.
	CTRL L: repaints the line if an external message is received.
	\fill

	PF2 and PF3 are only valid when entering strings.

	Literal characters are refused (with beep) when integer or
	floating values are awaited (eE are permitted for floats).

	Integer or float values are refused if they are external to
	given limits. If so, a message is given with the authorized limits,
	and a replacement value awaited.

	The  10 last values entered for int, float  or  string  (not
	for single characters) are memorized in 3 different ring buffers.

	Results are given as PostScript graphics, either displayed using the
	local PostScript previewer (dxpsview), and/or automatically spooled (or if
	asked from the menu area).

Files

	Input: file_in.sim, file_in.def

	Output: file_in.ps
	
author

	Sylvain Hanneton, Jean-Francois Vibert

	Ecole Normale Superieure, INSERM U263
	Paris
	
see also

	tms and xtms

bugs

	This is a beta test version, the interface will be rewritten.
	No known bugs in the computations and graphs.

#endif

/*
**
**	STANDART INCLUDE FILES
**
*/

#include  <stdio.h>	/* standard I/O files */
#include  <stdlib.h>
#include  <signal.h>
#include  <string.h>
#include  <ctype.h>	/* macro et globales pour fonctions classification de
			   caracteres */

#include  <sys/types.h>
#include  <math.h>
#include  <complex.h>

#include "find_files.h"

/*
**
**	MOTIF INCLUDE FILES
**
*/

#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#if (XmVersion >= 1002)
#include <Xm/RepType.h>
#endif

/*
**
*/

#define	TRUE	1
#define FALSE	0

#define WRK_FILE ""
#define ALT_FILE "xCAA.sim"

#define ESC	0x1b
#define RETURN	0x0d
#define PRINTER_PS	"postscript"
#ifdef __osf__
#	define	PREVIEWER_PS	"dxvdoc -f ps "
#endif
#ifdef ultrix
#	define	PREVIEWER_PS	"dxpsview -geometry 700x800 "
#endif
#ifdef gnu
#	define	PREVIEWER_PS	"ghostscript "
#endif	
#ifndef PREVIEWER_PS
#	define	PREVIEWER_PS	"ghostscript "
#endif
/*
**
**  Motif variables and parameters
**
*/

#define		CAA_APP_CLASS "b3e xcaa"
Widget		TheAppShell;
Widget		main_TopLevelShell;
Widget		parameters_TopLevelShell;
Widget		fileSelection_TopLevelShell;
Widget		axis_TopLevelShell;
extern Widget	fctpara_TopLevelShell;
extern Widget	print_DialogShell;
extern Widget   optionsOutput_ToggleButton;
extern Widget	ThePopupBox;

/*
**	PERSONAL INCLUDE FILES
*/

#define _CAA_MAIN
#include "caa_unix.h"
#undef _CAA_MAIN

/*** prototypes ***/

Widget	Createmain_Form (Widget);
Widget	CreateaxisMain_Form (Widget);
Widget	CreateparametersMain_Form (Widget);
Widget	CreatefctparaMain_Form (Widget);
Widget	CreatefileSelectionBox (Widget);
void Page_de_garde(void);
void Logo_XCAA(void);
void les_ps_ticked_axes(float tx, float ty, float mnx, float mxx,                         float mny, float mxy,
                        char legendx[60], char legendy[60],
		        float hy, int cadre);
void les_ps_axes(float tx, float ty, float mnx, float mxx, float mny, float mxy, int cadre);
void plotcarre(float x, float y);

/*** extern functions ***/

extern void caa_MainInitMenuVar();
extern void RegisterBxConverters();
extern void DispMessageDialog (Widget TheParent,char *TextToDisplay, char *TitleOfDialog,unsigned char TypeOfDialog);

/************************** VARIABLES GLOBALES DU MENU CAA *****************/

/* definition et initialisation des variables */
Boolean FirstUse = True,
        Window_open = False;

int	champ_deb = 10,	/* premiere ligne du menu */
	col_deb = 15,	/* premiere colonne du menu*/
	champ_fin,	/* derniere ligne du menu */
	pas = 1,	/* nombre de ligne entre deux choix */
	spooled = TRUE,	/* flag pour spooler ou non les dessins */
        largeur_pixmap = 600,
        hauteur_pixmap = 300,
        magnify = 100,
	champ ;		/* ligne courante */

char	work_file[250],
	work_file_type[250],
	prompt[250],
	on_garde[10];

FILE	*menu_def,	/* fichier du fichier par defaut! */
	*histo_file;	/* fichier des histogrammes en output ascii */
	
void	spawn(),	/* fonctions communes */
	retour(),
	fini();
	
	/*	MENU GENERAL */

void	chg_length(),
	print_file(Widget widget, XtPointer client_data, XtPointer call_data);
void    init_directories();

   /*******************
   * Pas d'impression *
   *******************/

void no_print()
{
	if (driver=='s')
	{
		driver = 'f' ;
	} /* fin du if */
} /* fin du autre */


int main(int argc, char **argv)
{
/*
**	Variables
*/
	/*** Motif Interface variables ***/

	XtAppContext	app;
	Arg		args[256];
	Cardinal	ac;

	Widget		parametersMain_Form;
	Widget		fileSelectionBox;
	Widget          axisMain_Form;
	char		SomeChar[250];

	/*** Divers ***/

	int	i,
		verbose;
		
	char	*f_n_l;
		
	int	memo_type(int);
	
	strcpy(on_garde," ");

	strcpy(work_file_type,".sim");

/*
**	Code
*/

	/*** Global variables initialisation ***/

	flag_dxpsview	 = 0;
	driver		 = 's';
	vient_de_nbc	 = FALSE;
	already_printed  = FALSE;
	filtre		 = FALSE;
	inside		 = TRUE;
	nb_demi_pages	 = 0;
	nb_de_types	 = 0;
	paper		 = 30.;
	long_X_axis	 = LONG_X;
	long_Y_axis	 = LONG_Y;
	horizontal_scale = 1.;
	vertical_scale	 = 1.;
	FileNameGiven	 = False;

	strcpy (filename,"");
	
	/*** Initialisation of Motif interface ***/

	caa_MainInitMenuVar();
	init_directories();

	/*
	* The applicationShell is created as an unrealized
	* parent for multiple topLevelShells.  The topLevelShells
	* are created as popup children of the applicationShell.
	* This is a recommendation of Paul Asente & Ralph Swick in
	* _X_Window_System_Toolkit_ p. 677.
	*/
	
	TheAppShell = XtVaAppInitialize ( &app, CAA_APP_CLASS, NULL, 0, 
#ifndef XlibSpecificationRelease
	(Cardinal *) &argc, 
#else
#if (XlibSpecificationRelease>=5)
	&argc, 
#else
	(Cardinal *) &argc, 
#endif
#endif
	argv, NULL, 
	NULL );
    
	magnify = 100;
	RegisterBxConverters(app);
#if (XmVersion >= 1002) 
	XmRepTypeInstallTearOffModelConverter();
#endif

   /***************************************
   *  traitement de la ligne de commande  *
   ***************************************/
    
	strcpy(file,"");
	verbose = FALSE;

	if (argc > 1)
	{
		for (i=1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf("%s [-v] [-mpct] [-s] [file]\n",argv[0]);
				printf("\t-v:\tverbose\n");
				printf("\t-mpct:\tmagnification in percentage (default 100%)\n");
				printf("\t-k:\tno spool of PS file\n");
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'm':
						magnify = atoi(&(argv[i][2]));
						break;
					case 's':
					case 'k':
						spooled = FALSE;
						driver = 'f';
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
						printf("%s [-v] [-mpct] [-s] [file]\n",argv[0]);
						printf("\t-v:\tverbose\n");
						printf("\t-mpct:\tmagnification in percentage (default 100%)\n");
						printf("\t-k:\tno spool of PS file\n");
				}
			}
			else
			{
			      if (argv[i][0] == '/')
			      {
				  strcpy(SomeChar,get_simulation_name());
				  f_n_l = strrchr(SomeChar,'/');
				  if (f_n_l)
				  {
					strcpy(filename,(f_n_l + sizeof(char)));
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,get_simulation_name());
					strcpy(file,get_simulation_name());		/* Nom generique */
					strcpy(filename,get_sim_name());
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory,get_dir_name());
				  }
				  else
				  {
					strcpy(filename,SomeChar);
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename); /* Nom generique */
					strcat(filename,".sim");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory, ".");
				  }
			      }
			      else
			      {
				    strcpy(SomeChar,argv[i]);
				    f_n_l = strrchr(SomeChar,'/');
				    if (f_n_l)
				    {
					strcpy(filename,(f_n_l + sizeof(char)));
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename);		/* Nom generique */
					strcat(filename,".sim");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					f_n_l = strrchr(SomeChar,'/');
					if (f_n_l)
					{
						f_n_l++;
						*f_n_l = 0;
						strcpy(project_directory, SomeChar);
						f_n_l = strrchr(SomeChar,'_');
					} /* fin du if */
				    }
				    else
				    {
					strcpy(filename,SomeChar);
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename); /* Nom generique */
					strcat(filename,".sim");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory, ".");
				    }
			      }
			}
		}
	}
#ifdef CAA_DEBUG
	printf("File name: %s -> %s\nCurrent directory: %s\nProject name: %s\n",
	       filename, scriptname, project_directory, file);

	/*** Create classes and widgets used in this program. ***/


    printf("CAA_UNIX: display: %x; &larg: %x; &haut: %x\n",
	   XtDisplay(TheAppShell),
	   &largeur_pixmap,
	   &hauteur_pixmap);
#endif

    ps_get_pixmap_size(XtDisplay(TheAppShell), &largeur_pixmap, &hauteur_pixmap, 210, 145, magnify);

#ifdef CAA_DEBUG
    printf("CAA_UNIX: larg: %d; haut: %d\n",
	   largeur_pixmap,
	   hauteur_pixmap);
#endif


	/*
	* Create classes and widgets used in this program. 
	*/
	
	sprintf (SomeChar,"%s: %s (%s)", VERSION, filename, scriptname);
	ac = 0;
	XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	sprintf (SomeChar,"%s", VERSION);
	XtSetArg(args[ac], XmNiconName, SomeChar); ac++;
	XtSetArg(args[ac], XmNx, 100); ac++;
	XtSetArg(args[ac], XmNy, 100); ac++;
	XtSetArg(args[ac], XmNwidth, largeur_pixmap); ac++;
	XtSetArg(args[ac], XmNheight, hauteur_pixmap); ac++;
	XtSetArg(args[ac], XmNbackground, WhitePixel(XtDisplay(TheAppShell),DefaultScreen(XtDisplay(TheAppShell)))); ac++;
	XtSetArg(args[ac], XmNforeground, BlackPixel(XtDisplay(TheAppShell),DefaultScreen(XtDisplay(TheAppShell)))); ac++;
	
	main_TopLevelShell = XtCreatePopupShell("main_TopLevelShell",
						topLevelShellWidgetClass,
						TheAppShell,
						args, 
						ac);
	main_Form = (Widget)Createmain_Form(main_TopLevelShell);
	XtManageChild(main_Form);
/*
	XtPopup(XtParent(main_Form), XtGrabNone);
*/
/*
	XtPopup(XtParent(fctparaMain_Form), XtGrabNone);
*/
	ac = 0;
	sprintf (SomeChar,"%s -  Axis size",VERSION);
	XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Axis size"); ac++;
	XtSetArg(args[ac], XmNx, 320); ac++;
	XtSetArg(args[ac], XmNy, 262); ac++;
	XtSetArg(args[ac], XmNwidth, 700); ac++;
	XtSetArg(args[ac], XmNheight, 270); ac++;
	axis_TopLevelShell = XtCreatePopupShell("axis_TopLevelShell",
						topLevelShellWidgetClass,
						TheAppShell,
						args, 
						ac);
	axisMain_Form = (Widget)CreateaxisMain_Form(axis_TopLevelShell);
	XtManageChild(axisMain_Form);
/*
	XtPopup(XtParent(axisMain_Form), XtGrabNone);
*/

	ac = 0;
	sprintf (SomeChar,"%s -  Parameters",VERSION);
	XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Parameters"); ac++;
	XtSetArg(args[ac], XmNx, 100); ac++;
	XtSetArg(args[ac], XmNy, 100); ac++;
	XtSetArg(args[ac], XmNwidth, 800); ac++;
	XtSetArg(args[ac], XmNheight, 450); ac++;
	parameters_TopLevelShell = XtCreatePopupShell("parameters_TopLevelShell",
						      topLevelShellWidgetClass,
						      TheAppShell,
						      args, 
						      ac);
	parametersMain_Form = (Widget)CreateparametersMain_Form(parameters_TopLevelShell);
	XtManageChild(parametersMain_Form);
/*
	XtPopup(XtParent(parametersMain_Form), XtGrabNone);
*/
	ac = 0;
	sprintf (SomeChar,"%s -  File selection",VERSION);
	XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	XtSetArg(args[ac], XmNiconName, "File selection"); ac++;
	XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
	XtSetArg(args[ac], XmNwidth, 430); ac++;
	XtSetArg(args[ac], XmNheight, 400); ac++;
	fileSelection_TopLevelShell = XtCreatePopupShell("fileSelection_TopLevelShell",
							 topLevelShellWidgetClass,
							 TheAppShell,
							 args, 
							 ac);
	fileSelectionBox = (Widget)CreatefileSelectionBox(fileSelection_TopLevelShell);
	XtManageChild(fileSelectionBox);

	ac = 0;
	sprintf (SomeChar,"%s -  Axis options",VERSION);
        XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Axis options"); ac++;
        XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
        XtSetArg(args[ac], XmNwidth, 801); ac++;
	XtSetArg(args[ac], XmNheight, 347); ac++;
        axis_TopLevelShell = XtCreatePopupShell("axis_TopLevelShell",
						topLevelShellWidgetClass,
						TheAppShell,
					        args,
						ac);
	axisMain_Form = (Widget)CreateaxisMain_Form(axis_TopLevelShell);
        XtManageChild(axisMain_Form);

	CreatefctparaMain_Form(main_Form);
	
	 /*** Display the main window ***/

	XtPopup(XtParent(main_Form), XtGrabNone);

	already_printed = True;

	if (FileNameGiven)
	{
	  BxPopupCB(XtParent(parametersMain_Form),(XtPointer)"parameters_TopLevelShell",NULL);
	  BxSetSensitiveCB(main_Form,"mainClose_PushButton, mainSavePS_PushButton,\
                           mainPrint_PushButton, mainOperations_CascadeButton, mainOptions_CascadeButton", NULL);

	}
	XtAppMainLoop(app);
	return (0);

} /* fin du main */


/*
*
*	READ_DATA() lit les donnees 
*	sur le fichier pour connaitre le nombre de 
*	donnees et les max et min
*	Le bruit sup est egalement lu en 3 eme position s'il y a lieu
*
*/
int read_data()
{
	int	i, j, nb_graph;
	float	x_i, y_i, bkg_i;
	char	le_buffer[80];

	bkg_i = 0.;
	nb_graph = -1;
	i = 0;
	j = 0;
	
	while (fgets(le_buffer,80,data))
	{
		if (le_buffer[0] == '\"')
		{
			le_buffer[strlen(buffer)-1] = 0;
			i = 0;
			nb_graph++;
		} /* fin du if */
		else
		{
			if (le_buffer[0] == '\n') continue;
			if (i > MAX_DATA) continue;
			bkg_i = -1.;
			sscanf(le_buffer,"%f %f %f",&x_i,&y_i,&bkg_i);
			j = nb_graph;
			temps[i][j] = x_i ;
			etat[i][j]  = y_i ;
			bkg[i][j]   = bkg_i ;
			nb_data[j]++;
			i++;
			  
		} /* fin du else */
	} /* fin du while */
	nb_graph++;
	return(nb_graph);
}

void print_file(Widget widget, XtPointer client_data, XtPointer call_data)
{

/*
**      Variables
*/
        char    PrintType;
	char SomeChar[250];
/*
**      Code
*/

        PrintType = (char) client_data;
#if DEBUG
	printf("PrintType: %c\n",PrintType);
#endif
        if (!already_printed)
        {
                ps_endplot_on(PrintType) ;
                already_printed = TRUE;
	} /* fin du if */

        strcat(scriptname,"+");
        temps_max[cluster] = temps[nb_donnees-1][cluster] ;
        if (arrivee > temps_max[cluster]) 
	  arrivee = temps_max[cluster] ;
        if (depart < temps[0][cluster]) 
	  depart=temps[0][cluster] ;
	sprintf (SomeChar,"%s: %s (%s)", VERSION, filename, scriptname);
	XtVaSetValues(main_TopLevelShell, XmNtitle, SomeChar,NULL); 
	ps_re_init_hard_copy(scriptname);
	already_printed = False;
	Page_de_garde();
} /* fin du void */


void fini()
{

   /*********************
   * fermeture fichiers *
   *********************/
	if (!already_printed)
		ps_endplot_on(driver) ;
	exit(0);
}

/************************** CAA **********************/




		/*******************************************
		* Fonctions utilisees dans la mise en page *
		*******************************************/
/*****************************************************************************/


/* place le curseur dans la zone message et fait clignoter
la legende 'message, */

void Message(char *chaine)
{
  return;
} /* fin du Message */

void Message_plotting()
{
	ps_code("%% *** DEBUT DU GRAPH ***\n");
}
/*
 * annonce le trace d'un dessin postscipt
 */

void Fin_plotting(int *data,	/* tableau a sauver */
		  int nb,	/* nb d'elements */
		  char *ext)	/* extension du fichier */
{
/*
**	Variables
*/
	char	out_file[250];
	char	SomeChar[250];
	int	i;
	Boolean SaveOutput;
/*
**	Code
*/
	ps_code("%% *** FIN DU GRAPHE ***\n");
	ps_flush();
	if (data)
	{
		XtVaGetValues (optionsOutput_ToggleButton,
			       XmNset, &SaveOutput,
			       NULL);
		if (SaveOutput)
		{
			strcpy(out_file,file);
			strcat(out_file,ext);
			if ((histo_file = fopen(out_file,"w")) == NULL)
			{
				sprintf (SomeChar, "Impossible to open %s, sorry\n",out_file);
				DispMessageDialog (fctpara_TopLevelShell,
						   SomeChar,
						   "Error : datas not saved",
						   XmDIALOG_ERROR);
			}
			else
			{
				for(i = 0; i < nb; i++)
					fprintf(histo_file,"%d\n",data[i]);
				fclose(histo_file);
			}
		} /* fin du if */
	} /* fin du if */
} /* fin du Fin */

/*
 * decide si il faut ou non tourner la page
 */

void tourne_page()
{
  /* d'abord sur le papier, pas sur l ecran, avec 2 dessins par page */
	ps_no_display(ps_current_window);
	nb_demi_pages = nb_demi_pages + 1 ;
	if ( nb_demi_pages  == 2)
	{
		ps_newpage() ;
		ps_home();
		ps_origin(1.5,13.,0.,0.);
		ps_home();
		nb_demi_pages = 0 ;
	} /* fin du if */
	else
	{
		ps_origin(0.,-12.,0.,0.) ;
		ps_home();
	} /* fin du else */
	ps_display(ps_current_window);
	/* puis sur l ecran, pas sur le papier, avec 1 dessin par page */ 
	ps_no_print();
	ps_newpage();
	ps_home();
	ps_origin(1.5,1.,0.,0.);
	ps_home();
	ps_print();
} /* fin du tourne */

void bounding_box(int nb_axes)
{
	float	bbllx,
		bblly,
		bburx,
		bbury;

	bbllx = X_PAS;
	if (nb_demi_pages)	/* On est en bas de la page */
	{
		bblly = 1.8 * X_PAS;
	} /* fin du if */
	else			/* on est en haut de la page */
	{
		bblly = 13.8 * X_PAS;
	} /* fin du else */
	bburx = bbllx +
		((1.5 + long_X_axis + ((nb_axes - 1)?2.:0.)) * X_PAS);
	bbury = bblly +
		((1.6 + long_Y_axis) * X_PAS);
	sprintf(buffer,"%%%%BoundingBox: %.1f %.1f %.1f %.1f\n",
		bbllx, bblly, bburx, bbury);
	ps_code(buffer);
} /* fin du int */
/*
 * Imprime la page de garde du dossier
 */
void Page_de_garde()
{
/*
**	Variables
*/
	char	titre[250];
	int	i ;
/*
**	Code
*/
	ps_no_display(ps_current_window);
	nb_demi_pages = 0 ;

	ps_origin(1.5,13.,0.,0.);
	sprintf(buffer,"%s",VERSION);

	ps_writeah(1.,11.,buffer,1.0);
	i = 0 ;

	ps_writeah(0.5,10.,"Cluster Activity Analysis",.3) ;
	sprintf(titre,"Number of analyzed events: %d",
		nb_donnees);
	ps_writeah(1.,6.5,titre,.5);
	sprintf(titre,"Analysis duration: %.3f secondes",
		arrivee-depart) ;	
	ps_writeah(1.,6.,titre,.5);
	ps_display(ps_current_window);

} /* fin du Page */

/*
 * Ouvre le dessin et imprime la page de garde du dossier sur le display
 */
void Logo_XCAA()
{
/*
**	Variables
*/
	Display *TheDisplay;
	char	titre[256];
	int	i ;
/*
**	Code
*/
	TheDisplay = XtDisplay (main_Form);
	already_printed = False;
#ifdef DEBUG
	printf("Avant ps_iniplot_on\n");
#endif
	if (!Window_open)
	{
	  ps_iniplot_on(TheDisplay, XtWindow(main_TopLevelShell),XtWindow(main_Form), scriptname, 645, 35, 210, 145, magnify, VERSION);
	  Window_open = True;
	}
#ifdef DEBUG
	printf("Apres ps_iniplot_on\n");
#endif
	ps_no_watch_progress();

	ps_no_print();
	ps_newpage();
	ps_origin(1.5,0.,0.,0.);
	sprintf(buffer,"%s",VERSION);

	ps_writeah(1.,3.,buffer,3.0);
	ps_gray(0.5);
	ps_writeah(0.5,2.,"Cluster Activity Analysis",1.) ;
	ps_gray(0.);
	ps_flush();
	ps_print();

} /* fin du Page */


   /**************************
   * retourne les n premiers *
   * pics d'une courbe       *
   **************************/

void get_pic(float *value,
	int nb_value,		/* nombre de valeurs          */
	int nb_pic)		/* nombre de pics a retourner */
{

	/* declarations  */
	int  indice = 0 ,
	     *picinter ,
	     i,j,k,
	     maxpic = 0 ;
	float  maxvalue = 0.,
	     *valueinter;

	picinter = int_table;
	valueinter = float_table_pic;

	/*initialisationa */
	for (i=0;i < (nb_value-1);i++)
	{
		picinter[i] = 0 ;
		valueinter[i]= value[i] ;
	}
	for (i=1;i < (nb_value - 1);i++)
	if ((value[i] > value[i-1] ) && ( value[i] > value[i+1] ))
	{
		indice = indice + 1 ;
		picinter[indice] = i ;
	} /* fin du if */
	for (i=0; i < nb_pic ;i++)
	{
		/* recherche du maximum */
		for (j=1; j < (nb_value - 1) ; j++)
		{
			k = picinter[j] ;
			if ((k != 0) && (valueinter[k] > maxvalue))
			{
				maxvalue = valueinter[k] ;
				maxpic   = k ;
			} /* fin du if */

		} /* fin du for */

		pic[i] = maxpic ;
		valueinter[maxpic] = 0.;
		maxvalue = 0.;
		
	} /* fin du for */
}

   /**************************************
   * calcul la valeur absolue d'un float *
   **************************************/
   
float float_abs(float f)
{
	if (f < 0.)
	  return(-f) ;
	else 
	  return(f);

} /* fin du float */

   /*******************
   * arrondi un float *
   *******************/

int arrondi(float f)
{
	int inter;

	inter = f ;
	if ((f-inter)>.5) 
	  inter = inter + 1 ;
	return(inter);
	
} /* fin du arrondi */



		/****************************
		* transformee de Fourier *
		****************************/
		
void fft_amp(pCALCULSARG ArgsOfFunction)
{

    /*******************
    *    declarations  *
    *******************/
    
	int   interint,nb_pic,i,j,k,time;
	float max,
	      min,
	      amp[MAX_DATA],
	      borne,
	      x1,y1,x,y,kx,ky;
	double sample[MAX_DATA],
	       fourier[MAX_DATA]; /*************** /
	       

    /***********************
    * dialogue utilisateur *
    ***********************/

	DispPopupDialog (main_TopLevelShell,
	                 "Fast Fourier Transform: amplitude spectrum",
			 "Please wait." );
	/* OUVERTURE DES FICHIERS a lire et ou ecrire */
	time = (int)debut;

    /**********************
    * debut de la lecture *
    **********************/
	for (i=0;i<= ( nb_donnees);i++)
	{
		if (rep_data == 1)
		sample[i] = etat[i+time][cluster] ;
		else
		sample[i] = bkg[i+time][cluster];

	} /* fin du for */

	realfft(sample,nb_donnees+1,fourier) ;

	/* on obtient seulement n/2 resultats car selon les proprietes
	   des transformees de fourier des signaux reels,on a :

	   R[F(-v)] = R[F(v)]
	   I[F(-v)] =-I[F(v)]

	*/
    /********************
    *  INITIALISATIONS  *
    ********************/

	/* calcul des amplitudes et phases */
	if ((nb_donnees % 2) != 0.) nb_donnees=nb_donnees-1 ;
	borne=nb_donnees/2. ;
	for (i= 1;i <= borne;i++)
	{
		amp[i]= sqrt((fourier[2*i]*fourier[2*i])+(fourier[2*i-1]*fourier[2*i-1]))/2.;
		
	} /* fin du for */

	amp[0]=0.;
	if (fourier[0] < 0.) fourier[0] = -fourier[0];
	j = cluster ;
	min = 0. ;
	max = 0. ;
       	for(i=0;i< (borne + 1);i++)
       	{
       		if (amp[i] > max) max=amp[i] ;
       		if (amp[i] < min) min=amp[i] ;
       	} /* fin du for */

	kx=10./(nb_donnees/2.);
	ky=7./(max-min);

	/* reperage des pics frequentiels */
	interint = borne ;
	nb_pic = 10 ;

	get_pic(amp,interint,nb_pic);

    /**************************
    * Trace de la transformee *
    **************************/
	
	tourne_page();
	Message_plotting();
	/* Mise en place des legendes */

	ps_writeah(0.,9.,"Fast Fourier Transform: amplitude spectrum",.4);

	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(14.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */

	/* TRACE de la courbe des amplitudes */

	les_ps_ticked_axes(10.,7.,0.,nb_donnees/(2.*fin),min,max,"Frequency","Amplitude",.1,0) ;

	/* changement d'origine */
	ps_origin(0.,float_abs(min)*ky,0.,0.) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	for (i=0 ;i < borne ;i++)
	{
		x = i * kx ;
		y = amp[i]*ky;
		x1 =(i+1) * kx ;
		y1 =amp[i+1]*ky ;
		ps_movea(x,y);
                ps_drawa(x1,y1);
		for (k=0;k<nb_pic;k++)
		{
			if ((i) == pic[k])
			ps_writeintah(-.1+x,y+.2,k+1,.3);
			
		} /* fin du for */

		
	} /* fin du for */

	ps_origin(0.,-float_abs(min)*ky,0.,0.) ;
		
	ps_writeah(11.5,8.,"Peak             period        frequency",.3);
	ps_font("Courier",.3);
	for (i=0;i<nb_pic;i++)
	{
		ps_writeintah(11.8,7.5-(i+1)*.4,i+1,.3) ;

		ps_writefloatah(12.5,7.5-(i+1)*.4,fin/pic[i],.3) ;
		ps_writefloatah(15.,7.5-(i+1)*.4,pic[i]*(1./fin),.3) ;
	} /* fin du for */
	ps_home();
	ps_font("Times-Roman",.3);

	Fin_plotting(NULL,0,NULL);

        XtPopdown (XtParent(ThePopupBox));
	
}

void fft_phase(pCALCULSARG ArgsOfFunction)
{

    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,time;
	float maxphase,
	      minphase,
	      phasemax,
	      *phase,
	      borne,
	      x1,y1,x,y,kx,kyp;
	double *sample,
	       *fourier;

	/*********************/
	
	phase = float_table;
	sample = double_table_1;
	fourier = double_table_2;

       
    /***********************
    * dialogue utilisateur *
    ***********************/
	DispPopupDialog (main_TopLevelShell,
	                 "Fast Fourier Transform: amplitude spectrum",
			 "Please wait." );

	/* OUVERTURE DES FICHIERS a lire et ou ecrire */

    /**********************
    * debut de la lecture *
    **********************/
	time = (int)debut;
	for (i=0;i<=(nb_donnees);i++)
	{
		if ( rep_data == 1)
		sample[i] = etat[i+time][cluster];
		else
		sample[i] = bkg[i+time][cluster];/**************/
	} /* fin du for */

	realfft(sample,nb_donnees+1,fourier) ;

	/* on obtient seulement n/2 resultats car selon les proprietes
	   des transformees de fourier des signaux reels,on a :

	   R[F(-v)] = R[F(v)]
	   I[F(-v)] =-I[F(v)]

	*/
	

    /********************
    *  INITIALISATIONS  *
    ********************/
	/* calcul des amplitudes et phases */
	if ((nb_donnees % 2) != 0.) nb_donnees=nb_donnees-1 ;
	borne=nb_donnees/2. ;

	for (i=1;i<=(borne);i++)/****************/
	{

		phase[i+time] = atan(-fourier[2*i]/fourier[2*i-1]) ;
	} /* fin du for */

	if (fourier[0]<0) fourier[0] = -fourier[0];
	phase[time] = 0.0000;
	j = cluster ;
	minphase = 0. ;
	maxphase = 0. ;
		for(i=time;i<=(borne+time);i++)
		{
			if (phase[i] > maxphase) maxphase=phase[i] ;
			if (phase[i] < minphase) minphase=phase[i] ;
			
		} /* fin du for */
	kx=10./(nb_donnees/2.);
	
	if (minphase<0) minphase = -minphase;
	if (minphase>maxphase) phasemax=minphase; else phasemax=maxphase;
	kyp=7./(2.*phasemax);
	
	
    /**************************
    * Trace de la transformee *
    **************************/

	tourne_page();
	/* Mise en place des legendes */
	ps_writeah(0.,9.,"Fast Fourier Transform: phase spectrum",.4);
	
	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(13.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */

	/* trace de la courbe des phases */
	les_ps_ticked_axes(10.,7.,0.,1./temps[1][cluster],-phasemax,phasemax,"Frequency","Phase",.1,0) ;

	
	/* changement d'origine */
	ps_origin(0.,phasemax*kyp,0.,0.) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	Message_plotting();
	
	for (i=time;i < (borne+ time);i++)
	{
		x = temps[i-time][cluster] * kx ;
		y = phase[i] * kyp;
		x1 = temps[i+1-time][cluster] * kx ;
		y1 = phase[i+1] * kyp ;
		ps_movea(x,y);
                ps_drawa(x1,y1);
	} /* fin du for */
	ps_origin(0.,-phasemax*kyp,0.,0.);
	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));
	
}

			/***********************
			* etat(t+1)=f(etat(t)) *
			***********************/
void poincare(pCALCULSARG ArgsOfFunction)
{


    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,time;
	float max,
	      min,
	      x,y,
	      x1,y1,
	      kx;

    /********************
    *  INITIALISATIONS  *
    ********************/
	DispPopupDialog (main_TopLevelShell,
	                 "Poincare Map",
			 "Please wait." );

	j = cluster ;
	time = (int)debut;
	min = 0. ;
	max = 1. ;
	for(i=time;i<(nb_donnees+1+time);i++)
	{
		if (rep_data == 1)
		{
			if (etat[i][j] > max)
				max = etat[i][j] ;
			if (etat[i][j] < min)
				min = etat[i][j] ;
		} /* fin du if */
		else
		{
			if (bkg[i][j] > max )
			max = bkg[i][j];
			if (bkg[i][j] < min )
			min = bkg[i][j];
			
		} /* fin du else */
	} /* fin du for */
	kx = (6./(max-min))      ; /* facteur de proportionnalite */
	
    /**********************************
    *  creation du fichier postscript *
    **********************************/
    
	Message_plotting();
	tourne_page();
	ps_cms() ;

	/* LEGENDES */
	ps_writeah(0.,9.,"Poincare Map",.4);
	
	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(14.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */

	/* trace des axes verticaux et horizontaux */
	les_ps_axes(6.,6.,min,max,min,max,1) ;

	/* changement d'origine */
	ps_origin(float_abs(min) * kx, float_abs(min) * kx, 0., 0.) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	for (i=time;(temps[i][j] < fin );i++)
	{
                if ( temps[i][j] >= debut )
		{
			if (rep_data == 1)
			{
				x = (etat[i][j] * kx);
				y = (etat[i+1][j] * kx);
				plotcarre(x,y);
			} /* fin du if */
			else
			{
				x = (bkg[i][j] * kx);
				y = (bkg[i+1][j] * kx);
				plotcarre(x,y);
			} /* fin du else */
		}/* fin du if*/
	} /* fin du for */

	/* plotting des droites */
	ps_origin(-float_abs(min) * kx + 8., -float_abs(min) * kx, 0., 0.) ;
	les_ps_axes(6.,6.,min,max,min,max,1) ;	
        ps_origin(float_abs(min)*kx,float_abs(min)*kx,0.,0.) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;
	/* plotting des ps_points */
	Message_plotting();
	for (i=time;(temps[i+1][j] < fin );i++)
	{
                if ( temps[i][j] >= debut )
		{
			if (rep_data == 1)
			{
				x = (etat[i][j] * kx)         ;
				y = (etat[i+1][j] * kx)       ;
				x1 = (etat[i+1][j] * kx)      ;
				y1 = (etat[i+2][j] * kx)      ;
				ps_movea(x,y)                 ;
				ps_drawa(x1,y1)               ;
				
			} /* fin du if */
			else
			{
				x = (bkg[i][j] * kx)         ;
				y = (bkg[i+1][j] * kx)       ;
				x1 = (bkg[i+1][j] * kx)      ;
				y1 = (bkg[i+2][j] * kx)      ;
				ps_movea(x,y)                ;
				ps_drawa(x1,y1)              ;
			} /* fin du else */
    		}/* fin du if */
	} /* fin du for */
	ps_origin(-float_abs(min)*kx-8.,-float_abs(min)*kx,0.,0.);
	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));
	
} /* fin du iter */

			/*************
			* statespace *
			*************/

void state_space(pCALCULSARG ArgsOfFunction)
{
    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,time;
	float max,maxspeed,
	      min,minspeed,
	      x,y,
	      x1,y1,
	      kx,ky,
	      *speed;

	speed = float_table;
	
    /********************
    *  INITIALISATIONS  *
    ********************/
	DispPopupDialog (main_TopLevelShell,
	                 "Phases space",
			 "Please wait." );

	time = (int)debut;
	for (i=(time+1); i < (nb_donnees + time);i++)
	{
		if (rep_data == 1)
		  speed[i]=(etat[i][cluster]-etat[i-1][cluster]) ;
		else
		  speed[i]=(bkg[i][cluster]-bkg[i-1][cluster]) ;
	} /* fin du for */
	j = cluster ;
	minspeed = -100. ; 
	min = 0. ;
	maxspeed = 100. ;
	max = 100. ;

#if 0
		for(i=(time+1);i<(nb_donnees+time);i++)
		{
			if (speed[i] > maxspeed) maxspeed=speed[i];
			if (speed[i] < minspeed) minspeed=speed[i];
			if (rep_data == 1)
			{
				if (etat[i][j] > max) max=etat[i][j] ;
				if (etat[i][j] < min) min=etat[i][j] ;
				
			} 
			else
			{
				if (bkg[i][j] > max) max=bkg[i][j] ;
				if (bkg[i][j] < min) min=bkg[i][j] ;
				
			} 
			

		} 
#endif
		
	kx = (6./(max-min)) ;           /* facteur de proportionnalite */
	ky = (6./(maxspeed-minspeed)) ;
	
    /**********************************
    *  creation du fichier postscript *
    **********************************/

	Message_plotting() ;
	tourne_page();
	ps_cms() ;
	/* LEGENDES */
	ps_writeah(0.,9.,"Phases space",.4);
	
	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(14.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */

	/* trace des axes verticaux et horizontaux */
	les_ps_axes(6.,6.,min,max,minspeed,maxspeed,1) ;

	/* changement d'origine */
	ps_origin( float_abs(min) * kx, float_abs(minspeed) * ky, 0. , 0. ) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	for (i=(time+1);temps[i][j] < fin;i++)
	{
	  if ( temps[i][j] >= debut )
	  {
	      if (rep_data == 1)
	      {
		  x = (etat[i][j]  * kx);
	      } /* fin du if */
	      else
	      {
		  x = (bkg[i][j]  * kx);
	      }
	      y = (speed[i] * ky);
              plotcarre(x,y);
	   }
	} /* fin du for */

	/* plotting des droites  */
	ps_origin(-float_abs(min)*kx+8.,-float_abs(minspeed)*ky,0.,0.) ;
	les_ps_axes(6.,6.,min,max,minspeed,maxspeed,1) ;
	ps_origin(float_abs(min)*kx,float_abs(minspeed)*ky,0.,0.);
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	for (i=1+time;(temps[i+2][j] < fin );i++)
	{
                if ( temps[i][j] >= debut )
		{
			if (rep_data == 1)
			{
				x = (etat[i][j] * kx);		
				x1 = (etat[i+1][j] * kx);
			} /* fin du if */
			else
			{
				x = (bkg[i][j] * kx); 
				x1 = (bkg[i+1][j] * kx);
			} /* fin du else */

			y = (speed[i] * ky)     ;
			y1 = (speed[i+1] * ky)  ;
			ps_movea(x,y)              ;
			ps_drawa(x1,y1)            ;
		}
	} /* fin du for */
	ps_origin(-float_abs(min)*kx-8.,-float_abs(minspeed)*ky,0.,0.) ;
	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));
	
} /* fin du statespace() */

/************
* Amplitude *
************/

void amp_time(pCALCULSARG ArgsOfFunction)
{
    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,time ;
	float max,
	      min,
	      maxy,miny,
	      moy,
	      x,y,
	      x1,y1,
	      kx,ky;       

    /********************
    *  INITIALISATIONS  *
    ********************/
	DispPopupDialog (main_TopLevelShell,
	                 "Amplitude as a function of time",
			 "Please wait." );

	j = cluster ;
	time = (int)debut;
	min = debut;
	max = fin;
	maxy = 1. ;
	miny = 0. ;
	moy  = 0. ;
	for (i = time; i <=(nb_donnees+time) ;i++)
	{
		if (rep_data == 1)
		{
			if (etat[i][j] > maxy) maxy=etat[i][j] ;
			if (etat[i][j] < miny) miny=etat[i][j] ;
			
		} /* fin du if */
		else
		{
			if (bkg[i][j] > maxy) maxy=bkg[i][j] ;
			if (bkg[i][j] < miny) miny=bkg[i][j] ;
			
		} /* fin du else */
		moy = moy + etat[i][j];
	} /* fin du for */
	moy = moy / nb_donnees;	
	kx = (long_X_axis/(max-min)) ;   /* facteur de proportionnalite */
	ky = (long_Y_axis/(maxy-miny)) ;
	
    /**********************************
    *  creation du fichier postscript *
    **********************************/
    
	Message_plotting() ;
	tourne_page();
	ps_cms() ;

	/* LEGENDES */
	ps_writeah(0.,9.,"Amplitude Spectrum",.4);
	
	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(14.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */
	
	/* trace des axes verticaux et horizontaux */

	les_ps_ticked_axes(long_X_axis,long_Y_axis,0.,max,miny,maxy,"times","Amplitude",.1,0) ;
	
	/* changement d'origine */
	if (min<0.)  min = - min ;
	if (miny<0.)  miny = - miny ;

	ps_origin(0.,miny*ky,0.,0.) ;
	ps_writeintah(-.5,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	for (i=time;(temps[i][j] < fin );i++)
	{
                if ( temps[i][j] >= debut )
		{
			if (rep_data == 1)
			{
				y = (etat[i][j] * ky)       ;	  
				y1 = (etat[i+1][j] * ky)    ;
			} /* fin du if */
			else
			{
				y = (bkg[i][j] * ky)       ;
				y1 = (bkg[i+1][j] * ky)    ;
			} /* fin du else */
			x = (temps[i-time][j] * kx)      ;
			x1 = (temps[i+1-time][j] * kx)   ;
			ps_movea(x,y)                 ;
			ps_drawa(x1,y1)               ;
		}/* fin du if*/
	} /* fin du for */
        ps_writefloatah(15.5,7.5 * .4,moy,.3) ;
				
	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));

} /* fin du amplitude() */


/**************************************
* FONCTIONS UTILISEES DANS statespace *
**************************************/

void plotcarre(float x, float y)
{
	ps_movea(x,y)   ;
	ps_point(2,0.05) ;
} /* fin du carre() */


void pcadrer(float z)
{
	ps_frame(z,z);
} /* fin du pcadre */

/************************************************
* represente l'espace des etats d'un cluster    *
************************************************/
void triDstate_space(pCALCULSARG ArgsOfFunction)
{

    /*******************
    *    declarations  *
    *******************/
    
	
	int   largeur,i,j,time;
	float inter1,inter2,inter3,
	      max,maxspeed,
	      min,minspeed,
	      x,y,
	      x1,y1,
	      angle,phi,
	      *speed;

	speed = float_table;

    /***********************
    * dialogue utilisateur *
    ***********************/

	angle = (2*M_PI/360.)*ArgsOfFunction->theta ;
	phi = (2*M_PI/360.)*ArgsOfFunction->phi ;

    /**********************
    * debut de la lecture *
    **********************/

	DispPopupDialog (main_TopLevelShell,
	                 "Computing 3D phase space state",
			 "Please wait." );
	j = cluster ;
	time=(int)debut;
	for (i=(time+1); i < (nb_donnees+time);i++)
	{
		if (rep_data == 1)
		speed[i]=(etat[i][j]-etat[i-1][j]) ;
		else
		speed[i]=(bkg[i][j]-bkg[i-1][j]) ;
	} /* fin du for */
		
    /********************
    *  INITIALISATIONS  *
    ********************/
	minspeed = 0. ; 
	min = 0. ;
	maxspeed = 0. ;
	max = 0. ;
	for(i=(time+1);i<(nb_donnees+time);i++)
	{
		if (speed[i] > maxspeed) maxspeed = speed[i] ;
		if (speed[i] < minspeed) minspeed = speed[i] ;
		if (rep_data == 1)
		{
			if (etat[i][j] >  max) max = etat[i][j] ;
			if (etat[i][j] <  min) min = etat[i][j] ;
		} /* fin du if */
		else
		{
			if (bkg[i][j] >  max) max = bkg[i][j] ;
			if (bkg[i][j] <  min) min = bkg[i][j] ;
		} /* fin du else */
	} /* fin du for */
/*
	printf("Max: %f; Min: %f\n",max,min);
*/
    /**********************************
    *  creation du fichier postscript *
    **********************************/
	Message_plotting();
	if (nb_demi_pages ==0 ) nb_demi_pages = 1;
	ps_newpage() ;
	nb_demi_pages = 2 ;
	ps_origin(18.5,1.,0.,0.);
	ps_rotate(90) ;
	ps_home();
	nb_demi_pages = 1;
	
	/* LEGENDES */
	ps_writeah(10.,18.5,"Phase Space as function of time",.4);

	if (cluster != 0)
	{
		ps_writeah(12.,0.,"Cluster :",.5);
		ps_writeintah(14.,0.,cluster,.7) ;
	} /* fin du if */
	else
	{
		ps_writeah(12.,0.,"Total Activity",.5);

	} /* fin du else */
	
	/* petit repere */
	ps_origin(3.5,1.,0.,0.);
	ps_3Dsetup(3.,3.,3.,0.,1.,0.,1.,0.,1.,angle,phi,ArgsOfFunction->obsdist,ArgsOfFunction->obsdist);
/*
	ps_3Daxes(1,0) ;
*/
	ps_origin(-3.5,-1.,0.,0.);

	ps_writeah(18.,0.,"View Angle:",.3);
	ps_writeah(18.,0.5,"Distance:",.3);
	ps_writefloatah(20.5,0.5,ArgsOfFunction->obsdist,.3);
	ps_writefloatah(20.5,0.,ArgsOfFunction->theta,.3);
	ps_writefloatah(20.5,-.5,ArgsOfFunction->phi,.3);
	ps_writeah(22.5,0.,"around Y",.25) ;
	ps_writeah(22.5,-.5,"around X",.25) ;
	ps_writeah(0.,1.," X : time ",.3);
	ps_writeah(0.,.5," Y : dA/dt ",.3);
	ps_writeah(0.,0.," Z : A ",.3);

	
	/* changement d'origine */
	ps_origin(6.,12.,0.,0.) ;
	
	/* trace des axes verticaux et horizontaux */

	ps_3Dsetup(4.,4.,4.,0.,fin,minspeed,maxspeed,min,max,angle,phi,ArgsOfFunction->obsdist,ArgsOfFunction->obsdist);
	ps_3Daxes(1,0) ;

	ps_gray(0.);

	/*  plotting des ps_points */

	for (i=time+1;(temps[i+1][j] < fin );i++)
	{
                if ( temps[i][j] >= debut )
		{
			ps_home();
			inter1 = temps[i][cluster];
			inter2 = speed[i];
			if (rep_data == 1)
				inter3 =  etat[i][j];
			else
				inter3 =  bkg[i][j];
			ps_3Dcalcul(inter1,inter2,inter3,&x,&y);
			if ((etat[i][j] != 0. || bkg[i][j] != 0.) && (ArgsOfFunction->prop != 0))
			{
				if (rep_data == 1)
					largeur= 5.- (4.* (max-etat[i][j]) / (max-min) ) ;
				else
					largeur= 5.- (4.* (max-bkg[i][j]) / (max-min) ) ;
				ps_line_width((float)largeur);
			} /* fin du if */

			inter1= temps[i+1][j];
			inter2= speed[i+1];
			if (rep_data == 1)
				inter3= etat[i+1][j];
			else
				inter3= bkg[i+1][j];

			ps_3Dcalcul(inter1,inter2,inter3,&x1,&y1);
			ps_movea(x,y);
			ps_drawa(x1,y1);
		
		} /* fin du if */

	} /* fin du for */
	ps_rotate(-90);

	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));
	
} /* fin du 3Dgraph */

/******************
* Autocorrelation *
******************/

void auto_correlation(pCALCULSARG ArgsOfFunction)
{
    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,t,k,s,nb_pic,time,ifin;

	float *autoco,
	      max,maxy,
	      min,miny,
	      x,y,
	      x1,y1,
	      kx,ky;       

	autoco = float_table;

    /********************
    *  INITIALISATIONS  *
    ********************/

	j = cluster ;
	time = (int)debut;
	min = debut;
	max = fin;
	miny = 10000000. ;
	maxy = -10000000. ;

	/* calcul de l'autocorrelation */

	DispPopupDialog (main_TopLevelShell,
	                 "Computing autocorrelation",
			 "Please wait." );
	ifin = (int)fin;
	for (s = 0 ; s <  ArgsOfFunction->duration ; s++)
	{
		autoco[s] = 0. ;
		for (t = time ; t < ( ifin - ArgsOfFunction->duration ) ; t++)
		{
			if ( rep_data == 1)
			autoco[s] += (etat[t][cluster] * etat[t+s][cluster]);
			else
			autoco[s] += (bkg[t][cluster] * bkg[t+s][cluster]);
		} /* fin du for */
		if (autoco[s] > maxy ) maxy = autoco[s];
		if (autoco[s] < miny ) miny = autoco[s];
		
	} /* fin du for */
	kx = long_X_axis/ArgsOfFunction->duration      ; 
	ky = long_Y_axis;
	for (i = 0 ; i < ArgsOfFunction->duration ; i++)
	{
		autoco[i] = autoco[i] / maxy; /* normalisation */
	} /* fin du for */

	/* reperage des pics frequentiels */
	nb_pic = 10 ;
	get_pic(autoco,ArgsOfFunction->duration,nb_pic);

    /**********************************
    *  creation du fichier postscript *
    **********************************/
    
	Message_plotting() ;
	tourne_page();
	ps_cms() ;

	/* LEGENDES */
	ps_writeah(0.,9.5,"Autocorrelation Function",.4);
	
	if (cluster != 0)
	{
		ps_writeah(12,9.,"Cluster :",.4);
		ps_writeintah(14.,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total Activity",.4);
		
	} /* fin du else */
	
	/* trace des axes verticaux et horizontaux */
	les_ps_ticked_axes(long_X_axis,long_Y_axis,0.,(float)ArgsOfFunction->duration,0.,1.,"Time lag = ","Autocorrelation",.1,0) ;

	/* changement d'origine */
	ps_origin(float_abs(0.)*kx,float_abs(0.)*ky,0.,0.) ;
	ps_writeah(-.5,-.5,"0",.4) ;
	ps_writeintah(6.,-.8,ArgsOfFunction->duration,.4);

	/* plotting des ps_points */
	for (i=0; i  < ArgsOfFunction->duration - 1 ; i++)
	{
		x = i * kx              ;
		y = (autoco[i] * ky)                ;
		x1 = (i+1) * kx           ;
		y1 = (autoco[i+1] * ky)             ;
		ps_movea(x,y)                         ;
		ps_drawa(x1,y1)                       ;

		for (k=0;k<nb_pic;k++)
			if ((i) == pic[k]) ps_writeintah(x,y+.2,k+1,.3);
		
	} /* fin du for */
	ps_origin(-float_abs(0.)*kx,-float_abs(0.)*ky,0.,0.) ;
	ps_writeah(11.5,8.,"Peak    time lag     value",.3);
	ps_font("Courier",.3) ;
	for (i=0;i<nb_pic;i++)
	{
		ps_writeintah(11.8,7.5-(i+1)*.4,i+1,.3) ;
		ps_writeintah(12.5,7.5-(i+1)*.4,pic[i],.3) ;
		ps_writefloatah(14.5,7.5-(i+1)*.4,autoco[pic[i]],.3) ;
		
	} /* fin du for */
	ps_home();
	ps_font("Times-Roman",.3) ;

	Fin_plotting(NULL,0,NULL);
#if 0
	Message("Attractor dimension...");
	for (i = 0 ; i < ArgsOfFunction->duration ; i++)
	autoco[i] = autoco[i] * maxy;
	attracteur(autoco,ArgsOfFunction->duration);
#endif

	XtPopdown (XtParent(ThePopupBox));

} /* fin du autococorrelation */
/************************************************************************/

void cross_correlation(pCALCULSARG ArgsOfFunction)
{
	int	i,j,t,k,s,nb_pic,time,
		ifin;

	float	*cross_co,
	        *signal_a,
	        *signal_b,
		max,maxy,
		min,miny,
		x,y,
		x1,y1,
		kx,ky;

	void	attracteur(float *, int);
	char	SomeChar[250];

	cross_co = float_table;
	signal_a = (float *) double_table_1;
	signal_b = (float *) double_table_2;

    /********************
    *  INITIALISATIONS  *
    ********************/

	sprintf(SomeChar, "Cross-correlation function between %d and %d",
			ArgsOfFunction->acluster, ArgsOfFunction->bcluster);
	DispPopupDialog (main_TopLevelShell,
	                 SomeChar,
			 "Please wait..." );

	j = cluster ;
	time = (int)debut;
	ifin = (int)fin;
	for ( i = 0 ; i < (ifin - time) ; i ++ )
	{
		if ( ArgsOfFunction->acluster == 3)
		  signal_a[i] = bkg[i+time][j];
		else
		  signal_a[i] = etat[i+time][ArgsOfFunction->acluster];
		signal_b[i] = etat[i+time][ArgsOfFunction->bcluster];
		
	} /* fin du for */

	min = - ArgsOfFunction->duration;
	max =   ArgsOfFunction->duration;
	miny = 100000. ;
	maxy = -100000. ;

	/* calcul de la cross-correlation */
	
	for (s = 0 ; s <  ArgsOfFunction->duration; s++)
	{
		cross_co[s+ArgsOfFunction->duration] = 0. ;
		for (t = ArgsOfFunction->duration ; t <  (ifin - time - ArgsOfFunction->duration ) ; t++)
		{
			cross_co[s+ArgsOfFunction->duration] += (signal_a[t] * signal_b[t+s]);
		} /* fin du for */
		if (cross_co[s] > maxy ) maxy = cross_co[s];
		if (cross_co[s] < miny ) miny = cross_co[s];
	} /* fin du for */

	for (s = ArgsOfFunction->duration ; s > 0 ; s--)
	{
		cross_co[s] = 0. ;
		for (t = ArgsOfFunction->duration ; t < ( ifin - time - ArgsOfFunction->duration ) ; t++)
		{
			cross_co[s] += (signal_a[t] * signal_b[t-s]);
		} /* fin du for */
		if (cross_co[s] > maxy ) maxy = cross_co[s];
		if (cross_co[s] < miny ) miny = cross_co[s];
	} /* fin du for */
	
	for (i = 0 ; i < ArgsOfFunction->duration *2 ; i++)
	{
	
		cross_co[i] = cross_co[i] / maxy; /* normalisation */
	
	} /* fin du for */
	kx = (long_X_axis/((float)ArgsOfFunction->duration * 2.))      ; 
	ky = long_Y_axis;
	/* reperage des pics frequentiels */
	nb_pic = 10 ;
	get_pic(cross_co,2 * ArgsOfFunction->duration,nb_pic);

    /**********************************
    *  creation du fichier postscript *
    **********************************/
    
	Message_plotting() ;
	tourne_page();
	ps_cms() ;

	/* LEGENDES */
	ps_writeah(0.,9.,"Cross-correlation function between",.4);
	if (ArgsOfFunction->acluster == 3)
		ps_writeah(0.,8.5," bgn supp ",.3);
	else
	{
		if ( ArgsOfFunction->acluster == 0)
		ps_writeah(1.8,8.5,"cluster",.3);
		else
		{
			ps_writeah(0.,8.5,"cluster Nb:",.3);
			ps_writeintah(1.8,8.5,ArgsOfFunction->acluster,.3);
			
		} /* fin du else */
		
		
	} /* fin du else */
	if ( ArgsOfFunction->bcluster == 0)
	  ps_writeah(1.8,8.5,"and network",.3);
	else
	{
		ps_writeah(2.,8.5,"and cluster Nb:",.3);
		ps_writeintah(4.5,8.5,ArgsOfFunction->bcluster,.3);
		
	} /* fin du else */
	
	if (cluster != 0)
	{
		ps_writeah(12.,9.,"Cluster :",.4);
		ps_writeintah(14.5,9.,cluster,.4);
	} /* fin du if */
	else
	{
		ps_writeah(12.,9.,"Total activity",.4);
		
	} /* fin du else */
	
	/* trace des axes verticaux et horizontaux */
	les_ps_ticked_axes(long_X_axis,long_Y_axis,-(float)ArgsOfFunction->duration,(float)ArgsOfFunction->duration,0.,1.,"Time lag","Cross-correlation",.1,0) ;

	/* changement d'origine */
	ps_origin(6.5,0.,0.,0.) ;
	ps_writeintah(.1,-.5,(int)debut,.3) ;

	/* plotting des ps_points */
	for (i=1; i  < ArgsOfFunction->duration * 2 - 1 ; i++)
	{
		x = (i - ArgsOfFunction->duration) * kx;
		y = (cross_co[i] * ky)                ;
		x1 = (i - ArgsOfFunction->duration + 1) * kx;
		y1 = (cross_co[i+1] * ky)             ;
		ps_movea(x,y)                         ;
		ps_drawa(x1,y1)                       ;

		for (k=0;k<nb_pic;k++)
			if (i == pic[k]) ps_writeintah(x,y+.2,k+1,.3);
		
	} /* fin du for */
	ps_origin(-5.5,0.,0.,0.) ;
	ps_writeah(11.5,8.,"Peak    time lag     value",.3);
	ps_font("Courier",.3) ;
	for (i=0;i<nb_pic;i++)
	{
		ps_writeintah(11.8,7.5-(i+1)*.4,i+1,.3) ;
		ps_writeintah(12.5,7.5-(i+1)*.4,pic[i],.3) ;
		ps_writefloatah(14.5,7.5-(i+1)*.4,cross_co[pic[i]],.3) ;
		
	} /* fin du for */
	ps_home();
	ps_font("Times-Roman",.3) ;

	Fin_plotting(NULL,0,NULL);

	XtPopdown (XtParent(ThePopupBox));
		
} /* fin du cross-correlation */

#if 0
void attracteur(float *autocorr, int duree)
{
    /*******************
    *    declarations  *
    *******************/
    
	int   i,j,t,k,m,
	      time,r,cpt;/* rajout de cpt = nb de points differents dans l'integrale de correlation*/
	            /* r coefficient pour calcul de */
                   /* la dimension attracteur      */
	float somme[1500], /* rajout pour le calcul de la dimension de l'attracteur */
	      phase[1500],
	      correlation[1500],
	      lg_autoco[1500],  /*           idem             */
	      X, teta,          /*           idem             */
	      max1,maxy1,/*         idem             */
	      min1,miny1,/*         idem             */
	      temp,temp1,nb,      /*         idem             */
	      kx1,ky1,            /*         idem             */
	      delta_x,
	      delta_y[1500],
	      delta; /*      idem             */
	      
    /********************
    *  INITIALISATIONS  *
    ********************/

	teta = 0.;
	min1 = 0.;/* recherche du minimum de log (autoco(r))*/
	miny1= 0.;/*  rechercher du minimun de log(corr(r))  */
	maxy1 = 0.;/* idem maximum pour plot de log(C(r)) en    */
	max1= 1.;/* fonction de log(r) */
	kx1 = 0.;
	ky1 = 0.;
	delta_x = 0.;
/*	delta_y = 0.; */
	delta = 0.;
	j = cluster;
	time = (int)debut;
	for ( i = 0 ; i < (nb_donnees) ; i++)
	{
		phase[i] = 0 ;
	} /* fin du for */

	/* calcul des modules des vecteurs de phases */
	Message(" Computing vectors ");


	for ( i = 0 ; i < nb_donnees ; i++)
	{
		for (t = i ; t < nb_donnees ;t++)
		if (rep_data == 1)
		phase[i] += etat[i+t+time][j] ;/* vi = v(ti)+v(ti+dt)+v(ti+2dt)...*/
		
		else
		phase[i] += bkg[i+t+time][j] ;
		printf(" phase [%d] = %f\n",i,phase[i]);
	} /* fin du for */

	/* calcul de l'integrale de correlation */

	Message("Computing correlation integral\n") ;

	
	for ( r = 0; r < duree ; r++)
	{
		somme[r]=0.;
		correlation[r]=0.;
		lg_autoco[r]=0.;
		for ( t = 0 ; t < (nb_donnees -1) ; t++)
		{
			X = autocorr[r] - fabs(phase[t] - phase[t+1]);
			if ( X < 0 ) teta = 0;
			if ( X >= 0 ) teta = 1;
			somme[r] = somme[r] + teta;
		} /* fin du for t */
		correlation[r] = (float)(somme[r] / (nb_donnees * nb_donnees));
		correlation[r] = log (correlation[r]);
		printf("autocorrelation[%d] = %f\n",r,autocorr[r+time]);
		lg_autoco[r] = log (autocorr[r+time]);
		if (correlation[r] < miny1) miny1 = correlation[r];
		if (correlation[r] > maxy1) maxy1 = correlation[r];
		if (lg_autoco[r] < min1 )   min1  = lg_autoco[r];
		if (lg_autoco[r] > max1 )   max1  = lg_autoco[r];
	} /* fin du for r = time */
	kx1 = 10./duree;
	ky1 = 7.;

	/* tri du tableau de valeur */
	Message("Sorting...\n") ;

	
	nb = (float)((duree) / 2);
	m = arrondi(nb);
	for (r = m ; r > 0 ; r /= 2)
	{
		for (i = r; i < (duree);i++)
		{
			for (k= i-r; k>=0 && lg_autoco[k] > lg_autoco[k+r] ; k -= r)
			{
		
				temp = lg_autoco[k];
				temp1= correlation[k];
				lg_autoco[k]=lg_autoco[k+r];
				correlation[k]=correlation[k+r];
				lg_autoco[k+r]=temp;
				correlation[k+r]=temp1;
			} /* fin du for de k */
		} /* fin du for de i*/
	} /* fin du for de r*/
	cpt = 0;
	for ( r = 0; r < duree-1 ; r ++)
	{
		printf("r = %d lg_autoco[r] = %f correl = %f\n",r,lg_autoco[r],correlation[r]);
		delta_x = fabs(lg_autoco[r+1] - lg_autoco[r]);
		delta_y[r] = fabs(correlation[r+1] - correlation[r]);
		if (r > 3 && delta_y[r] == delta_y[r-1] && delta_y[r-1] == delta_y[r-2] && delta_y[r-2]== delta_y[r-3])
		break; 
		if (delta_x != 0)
		{
			delta = delta + (delta_y[r]/ delta_x);
			cpt += 1;
		} /* fin du if */
		else
		delta = delta;
		printf("delta_y[r] = %f\n",delta_y[r]);
	} /* fin du for */
	delta /= cpt;
	printf("delta = % f cpt = %d\n",delta,cpt);

} /* fin du attracteur */
#endif

/*
*    Axes Sans Graduations
*/
void les_ps_axes(float tx, float ty, float mnx, float mxx, float mny, float mxy, int cadre)
{
	float px,py;
	
	/* mise a l'echelle */
	px = tx/(mxx-mnx);
	py = ty/(mxy-mny);
	mnx=mnx*px;
	mxx=mxx*px;
	mny=mny*py;
	mxy=mxy*py;
	if (mnx < 0. ) mnx = -mnx   ;
	if  (mny < 0. ) mny = -mny  ;
	if (mxx < 0. ) mxx = -mxx   ;
	if  (mxy < 0. ) mxy = -mxy  ;

	/* trace du cadre si demande */
	if (cadre != 0 )
	{
		ps_home() ;
		ps_drawa(0.,mxy+mny)   ;
		ps_drawa(mnx+mxx,mxy+mny)   ;
		ps_drawa((mxx+mnx),0.);
		ps_drawa(0.,0.);		
	} /* fin du if */

	/* trace des axes */
	ps_movea(0.,mny);
	ps_drawa((mxx+mnx),mny);
	ps_movea(mnx,0.);
	ps_drawa(mnx,mny+mxy);
	ps_home();

} /* fin du les_ps_axess */

/*
*    Axes avec graduations
*/
void les_ps_ticked_axes(float tx, float ty, float mnx, float mxx, float mny, float mxy,
		   char legendx[60], char legendy[60],
		   float hy, int cadre)
{
	float smxx,smnx,
	      smxy,smny,
	      px,py,
	      k1,k2,
	      pas1,pas2;
	int i;

	
  /* mise a l'echelle */
	smnx=mnx ;
	smxx=mxx ;
	smny=mny ;
	smxy=mxy ;
	px = tx/(mxx-mnx) ;
	py = ty/(mxy-mny) ;
	mnx=mnx*px ;
	mxx=mxx*px ;
	mny=mny*py ;
	mxy=mxy*py ;

	if (mnx < 0. ) mnx = -mnx   ;
	if  (mny < 0. ) mny = -mny  ;

	if (mxx < 0. ) mxx = -mxx   ;
	if  (mxy < 0. ) mxy = -mxy  ;
	
  /* trace du cadre si demande */
	if (cadre != 0 )
	{
		ps_home() ;
		ps_drawa(0.,mxy+mny)   ;
		ps_drawa(mnx+mxx,mxy+mny)   ;
		ps_drawa((mxx+mnx),0.);
		ps_drawa(0.,0.);
		
	} /* fin du if */

  /* trace des axes */
	ps_movea(0.,mny);
	ps_drawa((mxx+mnx),mny);
	ps_movea(mnx,0.);
	ps_drawa(mnx,mny+mxy);


  /* pose des legendes */
	ps_font("NewCenturySchlbk-BoldItalic",.4) ;
	ps_writeah(tx/4.,-3.*hy-.6,legendx,.4) ;
	ps_writeav(-.3*hy-.6,ty/4.,legendy,.4);
	ps_home();
	ps_font("NewCenturySchlbk-Roman",.4) ;
	
  /* Trace des graduations */
	/* horizontalement */
	
	k1 = (smxx-smnx);
	if (k1<1.)  while (k1<1.) k1 = k1 * 10. ;
	if (k1>10.)  while(k1>10.) k1 = k1 / 10. ;
	pas1 =px*(smxx-smnx)/(k1*10.) ;
	
	ps_movea(mnx,mny) ;
	for (i=1;(i*pas1) <= mxx;i++)
	{
		ps_movea(mnx+i*pas1,mny) ;
		if (i % 5  == 0 )
		{
			ps_drawr(0.,-2.*hy);
		} /* fin du if */
		if (i % 10 == 0 )
		{
			ps_drawr(0.,-3.*hy);
		} /* fin du if */
		else
		{
			ps_drawr(0.,-hy);
		} /* fin du else */
	} /* fin du for */
	if (mxx != 0.)
	ps_writefloatah(mnx+mxx-.9,mny-.4,smxx,.3) ;

	ps_movea(mnx,mny) ;
	for (i=0;(i*pas1) <= mnx;i++)
	{
		ps_movea(mnx-i*pas1,mny) ;
		if (i % 5  == 0 )
		{
			ps_drawr(0.,-2.*hy);
		} /* fin du if */
		if (i % 10 == 0 )
		{
			ps_drawr(0.,-3.*hy);
		} /* fin du if */
		else
		{
			ps_drawr(0.,-hy);
		} /* fin du else */
	} /* fin du for */
	if (mnx != 0. )
	ps_writefloatah(-.9,mny-.4,smxx,.3) ;

	/* verticalement */
	k2 = (smxy-smny) ;
	if (k2<1.)  while (k2<1.) k2 = k2 * 10. ;
	if (k2>10.)  while (k2>10.) k2 = k2 / 10. ;
	pas2 = py * (smxy-smny)/(k2 * 10.) ;
	
	ps_movea(mnx,mny) ;
	for (i=0;(i*pas2) <= mxy;i++)
	{
		ps_movea(mnx,i*pas2+mny) ;
		if (i % 5  == 0 )
		{
			ps_drawr(-2.*hy,0.);
		} /* fin du if */
		if (i % 10 == 0 )
		{
			ps_drawr(-3.*hy,0.);
		} /* fin du if */
		else
		{
			ps_drawr(-hy,0.);
		} /* fin du else */
	} /* fin du for */
	if (mxy != 0.)
	ps_writefloatah(mnx-.9,mny+mxy+.2,smxy,.3) ;

	ps_movea(mnx,mny) ;
	for (i=0;(i*pas2) <= mny;i++)
	{
		ps_movea(mnx,-i*pas2+mny) ;
		if (i % 5  == 0 )
		{
			ps_drawr(-2.*hy,0.);
		} /* fin du if */
		if (i % 10 == 0 )
		{
			ps_drawr(-3.*hy,0.);
		} /* fin du if */
		else
		{
			ps_drawr(-hy,0.);
		} /* fin du else */
	} /* fin du for */
	if (mny != 0. )
	ps_writefloatah(mnx-.4,-.9,smny,.3) ;
	
} /* fin du les_ps_axes */


/*
************************************************************************
*/
/*******************************************************************************
	Function for initializing global directory variables
*******************************************************************************/

void init_directories()
{
	FILE	*p_rc;
	char	*home;
	char	*caa_V8_bin;
	char	*caa_V8_help;
	char	buffer[256];

	home = caa_V8_bin = caa_V8_help = NULL; 
	home = getenv("HOME");
	if(home)
	{
		strcpy(buffer,home);
		strcat(buffer,"/.caarc");

		if (p_rc = fopen(buffer,"r"))
		{
			fscanf(p_rc,"%s",project_directory);
			strcpy(working_directory,project_directory);
			fclose(p_rc);
		} /* fin du if */
		else
		{
			strcpy(working_directory,".");
			strcpy(project_directory,".");
			strcat(project_directory,"/noname_nbc");
		} /* fin du else */
	}
	else
	{
		strcpy(working_directory,".");
		strcpy(project_directory,".");
		strcat(project_directory,"/noname_nbc");
	}

	caa_V8_bin = getenv("XNBC_BIN_DIR_TOOLS");
	if(caa_V8_bin)
	{
		strcpy(bin_directory,caa_V8_bin);
	}
	else
	{
		strcpy(bin_directory,"/usr/local/bin/xnbc.bin/V8");
		fprintf(stderr,"XNBC_BIN_DIR_TOOLS not defined, using %s\n",
			bin_directory);
	}
	caa_V8_help = getenv("XNBC_HELP_DIR");
	if(caa_V8_help)
	{
		strcpy(help_directory,caa_V8_help);
		strcat(help_directory,"/xcaa");
	}
	else
	{
		strcpy(help_directory,"/usr/local/bin/xnbc.bin/V8/help");
		fprintf(stderr,"XNBC_HELP_DIR not defined, using %s\n",
			help_directory);
		strcat(help_directory,"/xcaa");
	}
}




















