/*
*************************************************************************
*
*		TMS V6
*
*	This file contains all interface functions and callback functions
*
*
*
*
*
*
**************************************************************************
*/

/*
 * Standard includes for builtins.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

/*
**	MOTIF include files
*/

#include <Xm/Xm.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/List.h>

/*
**	Personal Motif variables
*/

#define PRINTED_EXT ".ps"              /* extension of printed files */
#define _TMS_CALLBACK
#include "tms_unix.h"
#include "tms_interface_creation.h"
#include "tms_interface_callbacks.h"
#include "find_files.h"
#undef _TMS_CALLBACK


/*
 * Macros to make code look nicer between ANSI and K&R.
 */
#ifndef ARGLIST
#if (NeedFunctionPrototypes == 0)
#define PROTOTYPE(p)	()
#define ARGLIST(p)	p
#define ARG(a, b)	a b;
#define GRA(a, b)	a b;
#define UARG(a, b)      a b;
#define GRAU(a, b)      a b;
#else
#define PROTOTYPE(p)	p
#define ARGLIST(p)	(
#define ARG(a, b)	a b,
#define GRA(a, b)	a b)
#ifdef __cplusplus
#define UARG(a, b)      a,
#define GRAU(a, b)      a)
#else
#define UARG(a, b)      a b,
#define GRAU(a, b)      a b)
#endif
#endif
#endif

/*
**	Global Variables
*/
/*** Min and Max parameters that identify how to calcul Min and Max ***/
#define FIRST_MM 1
#define SCND_MM 2
#define THURD_MM 3
#define FORTH_MM 4
#define FITH_MM 5
#define SIXTH_MM 6
#define SEVENTH_MM 7
#define EIGHTTH_MM 8
#define NINETH_MM 9

extern Widget TheAppShell;
extern Widget main_TopLevelShell;
extern Widget parameters_TopLevelShell;
extern Widget fileSelection_TopLevelShell;
extern Widget filter_TopLevelShell;
extern Widget histogram_TopLevelShell;
extern Widget arrows_TopLevelShell;
extern Widget poincare_TopLevelShell;
extern Widget fctparaTitle_Label;
extern Widget fileSelectionBox;
Widget fctpara_TopLevelShell;
Widget ThePopupBox;

/*** prototypes ***/

Widget CreateprintMain_Form (Widget);
Widget CreateFctPara_Form (Widget);

/*** extern functions... ***/

extern void	spawn(),	/* fonctions communes */
		retour(),
		fin(),
		no_op();

extern int	spooled;
extern char	on_garde[10];
		
Widget		BxFindTopShell PROTOTYPE((Widget));
WidgetList	BxWidgetIdsFromNames PROTOTYPE((Widget, char*, char*));

extern int compute_rate (int);
extern int compute_interval (int);
extern int filtre_les_data(int nombre_avant, int event);

/*
***************************************************************************************************
 *Function Name:	BxDestroyCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It tries to convert the widget names to Widget IDs and
 *			then destroy the specified widget.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the widget list to destroy.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 **************************************************************************************
 */
/* ARGSUSED */
void
BxDestroyCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxDestroyCB", (String)client);
    
    /* 
     * Loop through the list and destroy each widget.
     */
    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	XtDestroyWidget(widgets[i]);
	i++;
    }
    XtFree((char *)widgets);
}

/*
**************************************************************************************
 *Function Name:	BxSetSensitiveCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then set the widget sensitive.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the list of widget names.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *                    * BxSetSensitiveCB expects the application shell to
 *			be AppShell as defined as a global value as in the
 *			default main-{c, uil}.c generated by the
 *			Builder Xcessory.
 ***************************************************************************************8
 */

/* ARGSUSED */
void
BxSetSensitiveCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxSetSensitiveCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	XtSetSensitive(widgets[i], True);
	i++;
    }
    XtFree((char *)widgets);
}

/*
*********************************************************************************
 *Function Name:	BxSetInsensitiveCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then set the widget insensitive.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the list of widget names.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *                    * BxSetInsensitiveCB expects the application shell to
 *			be AppShell as defined as a global value as in the
 *			default main-{c, uil}.c generated by the
 *			Builder Xcessory.
 ************************************************************************************
 */

/* ARGSUSED */
void
BxSetInsensitiveCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxSetInsensitiveCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	XtSetSensitive(widgets[i], False);
	i++;
    }
    XtFree((char *)widgets);
}

/*
****************************************************************************************
 *Function Name:	BxPopdownCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then popdown the widgets WITHOUT any grab.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the string of widget names to
 *						popup.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *		      * BxPopdownCB can only work on Shell widgets.  It will
 *			not work on other object types.  This is because
 *			popping down can only be done to a shell.  A check
 *			is made using XtIsShell() and an appropriate error
 *			is output if the passed object is not a Shell.
********************************************************************************************
 */

/* ARGSUSED */
void
BxPopdownCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxPopdownCB", (String)client);
    
    i = 0;
    while ( widgets && widgets[i] != NULL )
    {
	if ( XtIsShell(widgets[i]) )
	{
	    XtPopdown(widgets[i]);
	}
	else
	{
	    printf("Callback Error (BxPopdownCB):\n\t\
Object %s is not a Shell\n", XtName(widgets[i]));
	}
	i++;
    }
    XtFree((char *)widgets);
}

/*
 **************************************************************************************
 *Function Name:	BxPopupCB
 *
 *      Description:   	This function accepts a string of the form:
 *			"(WL)[widgetName, widgetName, ...]"
 *			It attempts to convert the widget names to Widget IDs
 *			and then popup the widgets WITHOUT any grab.
 *
 *      Arguments:      Widget		w:	the activating widget.
 *			XtPointer	client:	the string of widget names to
 *						popup.
 *			XtPointer	call:	the call data (unused).
 *
 *      Notes:        * This function expects that there is an application
 *                      shell from which all other widgets are descended.
 *		      * BxPopupCB can only work on Shell widgets.  It will not
 *			work on other object types.  This is because popping up
 *			can only be done to a shell.  A check is made using
 *			XtIsShell() and an appropriate error is output if the
 *			passed object is not a Shell.
********************************************************************************************
 */

/* ARGSUSED */
void
BxPopupCB ARGLIST((w, client, call))
ARG( Widget, w)
ARG( XtPointer, client)
GRAU( XtPointer, call)
{
    WidgetList		widgets;
    int			i;

    /*
     * This function returns a NULL terminated WidgetList.  The memory for
     * the list needs to be freed when it is no longer needed.
     */
    widgets = BxWidgetIdsFromNames(w, "BxPopupCB", (String)client);

    i = 0;
    while( widgets && widgets[i] != NULL )
    {
	if ( XtIsShell(widgets[i]) )
	{
	    XtPopup(widgets[i], XtGrabNone);
	}
	else
	{
	    printf("Callback Error (BxPopupCB):Object %s is not a Shell\n", XtName(widgets[i]));
	}
	i++;
    }
    XtFree((char *)widgets);
}

/*
******************************************************************************
*
*	Callbacks for OSF-Motif interface that are not provided by buildXcessory
*
*	V0 april the 26 of  1996	file selection box managment
*
*
*
*
********************************************************************************
*/

void browse_help (Widget TheParent,char *TextToDisplay, char *TitleOfDialog,unsigned char TypeOfDialog)
{
	browse_manual(XNBC_XTMS);
} /* fin du void */
/*
************************************************************************************
*
*	Function:
*		tms_ForceUpdate (TheWidget)
*	Description:
*		Force the fetching and displaying of the widget content
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*		june the 19th of 1996
*
*************************************************************************************
*/

void tms_ForceUpdate (Widget TheWidget)
{
/*
**	Variables
*/
	Widget			diashell, topshell;
	Window			diawindow, topwindow;
	XtAppContext		cxt;
	Display			*dpy;
	XWindowAttributes	xwa;
	XEvent			event;
	
/*
**	Code
*/
	cxt = XtWidgetToApplicationContext (TheWidget);

	/*** Locate the shell we are interested in ***/

	for (diashell = TheWidget; !XtIsShell (diashell); diashell = XtParent (diashell))
		;
	/*** Locate its primary window's shell (wich may be the same) ***/

	for (topshell = diashell; !XtIsTopLevelShell(topshell); topshell = XtParent (topshell))
		;

	/***  ***/

	if (XtIsRealized (diashell) && XtIsRealized (topshell))
	{
		dpy = XtDisplay (topshell);
		diawindow = XtWindow (diashell);
		topwindow = XtWindow (topshell);

		/*** Wait for the dialog to be mapped. ***/

		while (XGetWindowAttributes (dpy, diawindow, &xwa) && xwa.map_state != IsViewable)
		{
			if (XGetWindowAttributes (dpy, diawindow, &xwa) && xwa.map_state != IsViewable)
			{
				break;
			}
			XtAppNextEvent (cxt, &event);
			XtDispatchEvent (&event);
		}
	}

	/*** The nest XSync will get an expose event. ***/

	XmUpdateDisplay (topshell);
}
/*
**************************************************************
*
*	Function:
*		DispMessageDialog (Parent, TexteToDisplay, TitleOfDialog, TypeOfDialog)
*	Description:
*		Display a Message Box
*
*	Inputs arguments:
*		Widget		TheParent	:	Widget parent of the MessageBox
*		char*		TextToDisplay	:	...
*		char*		TitleOfDialog	:	...
*		unsigned char	TypeOfDialog	:	...
*		
*	Revision:
*		April the 26 of 1996
*		May the 2nd of 1996	:	End of design
*
****************************************************************
*/
void DispMessageDialog (Widget TheParent,char *TextToDisplay, char *TitleOfDialog,unsigned char TypeOfDialog)
{
/*
**	Variables
*/
	static Widget	TheMessageBox;
	XmString	StringText,
			StringTitle;
	Arg		args[3];
	int		n;
/*
**	Code
*/
	n = 0;
	
	StringText = XmStringCreateLtoR (TextToDisplay, XmFONTLIST_DEFAULT_TAG);
	StringTitle = XmStringCreateLocalized (TitleOfDialog);
	
	if (!TheMessageBox)
	{
		XtSetArg (args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
		TheMessageBox = XmCreateMessageDialog (TheParent, "TheMessageBox", args, n);
		XtUnmanageChild (XmMessageBoxGetChild (TheMessageBox, XmDIALOG_CANCEL_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (TheMessageBox, XmDIALOG_HELP_BUTTON));
	}

	XtVaSetValues (TheMessageBox,
		       XmNdialogType, TypeOfDialog,
		       XmNmessageString, StringText,
#ifndef linux
		       XmNdialogTitle, StringTitle,
#endif
		       NULL);
	XmStringFree (StringText);
	XmStringFree (StringTitle);
	
	XtManageChild (TheMessageBox);
	XtPopup (XtParent(TheMessageBox), XtGrabNone);
	tms_ForceUpdate (TheMessageBox);
}
/*
**************************************************************
*
*	Function:
*		DispPopupDialog (Parent, TexteToDisplay, TitleOfDialog)
*	Description:
*		Display a Message Box with no buttons : just uesd to ask the user to wait
*		while something is going on
*
*	Inputs:
*		Widget	TheParent	:	Widget parent of the PopupDialog
*		char*	TestToDisplay	:	...
*		char*	TitleOfDialog	:	...
*	Revision:
*		April the 26 of 1996
*		May the 12th of 1996	:	End of design
*
****************************************************************
*/
void DispPopupDialog (Widget TheParent,char *TextToDisplay, char *TitleOfDialog)
{
/*
**	Variables
*/
	XmString	StringText,
			StringTitle;

	Arg		args[2];
	int		n;
/*
**	Code
*/

	n = 0;
	
	StringText = XmStringCreateLtoR (TextToDisplay, XmFONTLIST_DEFAULT_TAG);
	StringTitle = XmStringCreateLocalized (TitleOfDialog);
	
	if (!ThePopupBox)
	{
		XtSetArg (args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
		XtSetArg (args[n], XmNdialogType, XmDIALOG_INFORMATION); n++;
		ThePopupBox = XmCreateMessageDialog (TheParent, "ThePopupBox", args, n);
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_OK_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_CANCEL_BUTTON));
		XtUnmanageChild (XmMessageBoxGetChild (ThePopupBox, XmDIALOG_HELP_BUTTON));
	}
	XtVaSetValues (ThePopupBox,
		       XmNmessageString, StringText,
#ifndef linux
		       XmNdialogTitle, StringTitle,
#endif
		       NULL);
	XmStringFree (StringText);
	XmStringFree (StringTitle);
	
	XtManageChild (ThePopupBox);
	XtPopup(XtParent(ThePopupBox), XtGrabNone);
	tms_ForceUpdate ( ThePopupBox );
}
/*
**************************************************************
*
*	Function:
*		tms_FilterOKCB (widget, client_data, call_data)
*	Description:
*		Register the event's filters and compute the filters caracteristics (from the Filter Window)
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 10 of 1996
*
****************************************************************
*/

void tms_FilterOKCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	char    *SomeChar,
		*SomeOtherChar,
		SprintfChar[20];
	short	ClientButton;
	short   Index,
		Index2,
		FilterCounter;
	Boolean ToggleStatus;

	Widget  OpMenuSelection;
/*
**	Code
*/
	SomeChar = NULL;
	filtre = FALSE;

	/*** empty some TextFields and initialise some global variables***/

	for (Index = 0; Index < MAX_EVENT; Index++)
	{
		ToggleStatus = XmToggleButtonGetState(filterSelect_ToggleButton[Index]);
		if (!ToggleStatus)
		{
			XtVaSetValues (filterEventNbData_TextField[Index],
				       XmNvalue, "",
				       NULL);
			XtVaSetValues (filterMax_TextField[Index],
				       XmNvalue, "",
				       NULL);
			XtVaSetValues (filterMin_TextField[Index],
				       XmNvalue, "",
				       NULL);
			XtVaSetValues (filterNbDataAf_TextField[Index],
				       XmNvalue, "",
				       NULL);
			XtVaSetValues (filterNbDataBe_TextField[Index],
				       XmNvalue, "",
				       NULL);
		}
	}

	/*** Set the filters attributs when it is needed ***/

	FilterCounter = 0;
	for (Index = 0; Index < MAX_EVENT; Index++)
	{		
		SomeChar = (char *) XmTextGetString (filterEventNb_TextField[Index]);
		ToggleStatus = XmToggleButtonGetState(filterSelect_ToggleButton[Index]);

		for (Index2 = 0; Index2 < MAX_EVENT; Index2++)
		{
			sprintf(SprintfChar,"%d",event[Index2].numero);
			if (ToggleStatus && !strcmp (SomeChar,SprintfChar) && strcmp (SomeChar, "0"))
			{
		
				/*** This event has a filter action ***/
				
				FilterCounter++;
				filtre = True;
				SomeOtherChar = (char *) XmTextGetString (filterMin_TextField[Index]);
				if (!strcmp(SomeOtherChar,""))
				{
					event[Index2].min_interval_clip = 0.;
					sprintf(SprintfChar,"%.3f",event[Index2].min_interval_clip);
					XtVaSetValues (filterMin_TextField[Index],
						       XmNvalue, SprintfChar,
						       NULL);			
				}
				else
				{
					sscanf (SomeOtherChar, "%.3f",&event[Index2].min_interval_clip);
				
				}
				XtFree (SomeOtherChar);
				
				SomeOtherChar = (char *) XmTextGetString (filterMax_TextField[Index]);
				if (!strcmp(SomeOtherChar,""))
				{
					event[Index].max_interval_clip = 1.;
					sprintf(SprintfChar,"%.3f",event[Index2].max_interval_clip);
					XtVaSetValues (filterMax_TextField[Index],
						       XmNvalue, SprintfChar,
						       NULL);
				}
				else
				{
					sscanf (SomeOtherChar, "%.3f",&event[Index2].max_interval_clip);				
				}
				XtFree (SomeOtherChar);
				
				max_interval_clip = event[Index2].max_interval_clip;
				min_interval_clip = event[Index2].min_interval_clip;

				filtered_event = event[Index2].numero;
				XtVaGetValues (filterInOut_OptionMenu[Index],
					       XmNmenuHistory, &OpMenuSelection,
					       NULL);			
			
				if (OpMenuSelection == filterOutside_PushButton[Index])
				{
					event[Index2].inside = FALSE;
					}
				else
				{
					event[Index2].inside = TRUE;
					}

				sprintf(SprintfChar,"%d",nb_donnees);
				XtVaSetValues (filterNbDataBe_TextField[Index],
					       XmNvalue, SprintfChar,
					       NULL);
				       
				nb_events = filtre_les_data(nb_donnees,filtered_event);
			
				sprintf(SprintfChar,"%d",nb_events);
				XtVaSetValues (filterNbDataAf_TextField[Index],
					       XmNvalue, SprintfChar,
					       NULL);

				nb_donnees = nb_events;
				nb_events = compute_rate(filtered_event);
				compute_interval(filtered_event);
				event[Index2].nombre = nb_events;

				sprintf(SprintfChar,"%d",event[Index2].nombre);
				XtVaSetValues (filterEventNbData_TextField[Index],
					       XmNvalue, SprintfChar,
					       NULL);
				} /* fin du if */
		}
	} /* fin du for */

	ClientButton = (char) client_data;

	if (ClientButton == 'O')
	{
		if (FilterCounter == 0)
		{
			
			/*** Few line have not been filled completly 'Counter' shows how much***/
/*			
			sprintf (SprintfChar, "You didn't choose any event", Counter);
			DispMessageDialog (widget,
					   SprintfChar,
					   "No data",
					   XmDIALOG_INFORMATION	);
*/
		}
/*		XtPopdown (  XtParent ( XtParent( widget ) ) );		*/
		XtPopdown (  filter_TopLevelShell);		
	}

	if (SomeChar)
	{
		XtFree (SomeChar);		
	}
}

/*
**************************************************************
*
*	Function:
*		tms_AxisApplyCB (widget, client_data, call_data)
*	Description:
*		Checks if the Axis window datas are corrects and records the values
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 15 of 1996
*
****************************************************************
*/

void tms_AxisApplyCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	char    *SomeChar1,
		*SomeChar2,
		ErrorMessage1[120],
		ErrorMessage2[60];
	Boolean XAxisWrong,
		YAxisWrong;
/*
**	Code
*/
	SomeChar1 = NULL;
	SomeChar2 = NULL;
	XAxisWrong = False;
	YAxisWrong = False;

	SomeChar1 = (char *) XmTextGetString (axisXLenght_TextField);
	long_X_axis = atof (SomeChar1);	
	SomeChar2 = (char *) XmTextGetString (axisYLenght_TextField);
	long_Y_axis = atof (SomeChar2);

	if (long_X_axis == 0 || !strcmp(SomeChar2, ""))
	{
		long_X_axis = LONG_X;
		XAxisWrong = True;
		strcpy(ErrorMessage1, "X axis lenght musn't be null\n");
	}
	else if (long_X_axis > 18.)
	{
		long_X_axis = 18.;
		XAxisWrong = True;
		strcpy(ErrorMessage1, "X axis lenght limited to 18. cms (paper size constraint!\n");
	}
	if (long_Y_axis == 0 || !strcmp(SomeChar2, ""))
	{
		long_Y_axis = LONG_Y;
		YAxisWrong = True;
		strcpy(ErrorMessage2, "Y axis lenght musn't be null");
	}
	else if (long_Y_axis > 20.)
	{
		long_Y_axis = 20.;
		YAxisWrong = True;
		strcpy(ErrorMessage2, "Y axis lenght limited to 18. cms (paper size constraint!");
	}
	if (XAxisWrong || YAxisWrong)
	{
		sprintf (SomeChar1, "%.3f", long_X_axis);
		XmTextSetString (axisXLenght_TextField, SomeChar1);

		sprintf (SomeChar2, "%.3f", long_Y_axis);
		XmTextSetString (axisYLenght_TextField, SomeChar2);

		strcat (ErrorMessage1, ErrorMessage2);
		DispMessageDialog (widget,
				   ErrorMessage1,
				   "Axis lenght has been changed",
				   XmDIALOG_INFORMATION	);	
	}

	if (SomeChar1)
	{
		XtFree (SomeChar1);		
	}
	if (SomeChar2)
	{
		XtFree (SomeChar2);			
	}
}

/*
************************************************************************************
*
*	Function:
*		tms_histogramSetSensitive (SensitifList[])
*	Description:
*		set sensitif or unsensitif the Histogram window's widgets
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*		june the 5th of 1996
*
*************************************************************************************
*/

void tms_histogramSetSensitive (Boolean SensitifList[])
{
/*
**	Variables
*/
	int	Index;
/*
**	Code
*/
	Index = 0;

	XtVaSetValues (histogramLine1_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (histogramBinsNb_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (histogramModified_Label[Index - 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (histogramLine2_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (histogramModified_Label[Index - 1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (histogramScale_ToggleButton,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (histogramMax_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (histogramModified_Label[Index - 2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (histogramMin_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (histogramModified_Label[Index - 2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
}
/*
************************************************************************************
*
*	Function:
*		tms_arrowsSetSensitive (SensitifList[])
*	Description:
*		set sensitif or unsensitif the Histogram window's widgets
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*		june the 5th of 1996
*
*************************************************************************************
*/

void tms_arrowsSetSensitive (Boolean SensitifList[])
{
/*
**	Variables
*/
	int	Index;

/*
**	Code
*/
	Index = 0;

	XtVaSetValues (arrowsQue_ToggleButton,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (arrowsPos_ToggleButton,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (arrowsZoneFreq_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (arrowsModified_Label[0],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (arrowsZoneSelect_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (arrowsZoneSelect_ScrolledList,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (arrowsModified_Label[1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (arrowsModified_Label[2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (arrowsModified_Label[3],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
}
/*
************************************************************************************
*
*	Function:
*		tms_arrowsRemoveCB (widget, client_data, call_data)
*	Description:
*		Remove the selected element of the arrows window's list
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client_data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*
*************************************************************************************
*/

void tms_arrowsRemoveCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	int	 *PositionsList;
	int	 PositionsCount;
	Boolean	 Founded;
/*
**	Code
*/

	Founded = XmListGetSelectedPos(arrowsZoneSelect_ScrolledList,
				       &PositionsList,
				       &PositionsCount);
	if (Founded)
	{
		XmListDeletePositions (arrowsZoneSelect_ScrolledList,
				       PositionsList,
				       PositionsCount);		
	}
	
}
/*
************************************************************************************
*
*	Function:
*		tms_arrowsAddCB (widget, client_data, call_data)
*	Description:
*		Add an element of the arrows window's list. The element added is the one
*		in the arrows window's TextField Select.
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the Id of the "Select" TextField widget
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*
*************************************************************************************
*/

void tms_arrowsAddCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	Widget		WidgetTextField;
	XmString	ItemToAdd;
	char		*CharItem;
	Boolean		ItemExist;
/*
**	Code
*/
	CharItem = NULL;
	WidgetTextField = ( Widget ) client_data;

	CharItem = (char *) XmTextGetString (WidgetTextField);

	if (strcmp (CharItem, ""))
	{
		XmTextSetString (WidgetTextField, "");
		ItemToAdd = XmStringCreateLocalized (CharItem);
		ItemExist = XmListItemExists (arrowsZoneSelect_ScrolledList,
					      ItemToAdd);
		if (ItemExist)
		{
			DispMessageDialog (widget,
					   "The arrow has been already added.\nYou can choose an arrow only once",
					   "Error : arrow already selected",
					   XmDIALOG_ERROR);			
		}
		else
		{
			XmListAddItem (arrowsZoneSelect_ScrolledList,
				       ItemToAdd,
				       0);
		}
		XmStringFree (ItemToAdd);
	}

	if (CharItem)
	{
		XtFree (CharItem);		
	}
}
/*
************************************************************************************
*
*	Function:
*		tms_arrowsArrowNeededCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList) depending
*		of the "Arrow needed" toggle button.
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client_data, unused.
*		XtPointer	call_data:	the call-data, unused.
*
*	Revision:
*
*************************************************************************************
*/

void tms_arrowsArrowNeededCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	Boolean	ArrowNeeded;
	Boolean ArrowsList[ARROWS_PARANB];
	int	Index;
/*
**	Code
*/
	for (Index = 0; Index < ARROWS_PARANB; Index++)
	{
		ArrowsList[Index] = False;
	}
	ArrowNeeded= XmToggleButtonGetState(arrowsQue_ToggleButton);
	if (ArrowNeeded)
	{
		ArrowsList[0] = True;
		ArrowsList[1] = True;
	}
	else
	{
		ArrowsList[0] = True;
	}
	tms_arrowsSetSensitive(ArrowsList);	
	if (ArrowNeeded)
	{
		tms_arrowsArrowAutoCB (widget, client_data, call_data);
	}
}
/*
************************************************************************************
*
*	Function:
*		tms_arrowsArrowAutoCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList)
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the Id of the "Automatic arow positionning" ToggleButton widget
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*
*************************************************************************************
*/

void tms_arrowsArrowAutoCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	Boolean	ArrowAuto;
	Boolean ArrowsList[ARROWS_PARANB];
	int	Index;
/*
**	Code
*/
	for (Index = 0; Index < ARROWS_PARANB; Index++)
	{
		ArrowsList[Index] = False;
	}
	ArrowAuto = XmToggleButtonGetState(arrowsPos_ToggleButton);
	if (ArrowAuto)
	{
		ArrowsList[0] = True;
		ArrowsList[1] = True;
		ArrowsList[2] = True;
	}
	else
	{
		ArrowsList[0] = True;
		ArrowsList[1] = True;
		ArrowsList[3] = True;
	}
	tms_arrowsSetSensitive(ArrowsList);	
}
/*
************************************************************************************
*
*	Function:
*		tms_PoincareSetSensitive (SensitifList[])
*	Description:
*		set sensitif or unsensitif the Histogram window's widgets
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*		june the 5th of 1996
*
*************************************************************************************
*/

void tms_poincareSetSensitive (Boolean SensitifList[])
{
/*
**	Variables
*/
	int Index;
/*
**	Code
*/
	Index = 0;
	XtVaSetValues (poincareDrawFrom_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (poincareModified_Label[0],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (poincareDrawTo_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (poincareModified_Label[1],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (poincareAxisSize_ToggleButton,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	XtVaSetValues (poincareModified_Label[2],
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;

}
/*
************************************************************************************
*
*	Function:
*		tms_poincareAxisOptionCB (widget, client_data, call_data)
*	Description:
*		Register the choice of square axis from the poincare window ToggleButton
*		and show the axis size in the poincare window.
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	The ToggleButton widget ID concern by the "square axis" option
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*	
*
*************************************************************************************
*/

void tms_poincareAxisOptionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	XmToggleButtonCallbackStruct	*ToggleState;
	char				SomeChar[250];
	XmString			SomeString;
	pCALCULSARG			FunctionArg;
/*
**	Code
*/
	FunctionArg = (pCALCULSARG) client_data;

	ToggleState = (XmToggleButtonCallbackStruct *) call_data;

	if (ToggleState->set)
	{
		FunctionArg->square = True;
		sprintf (SomeChar, "(%1.3f X %1.3f cm)", long_Y_axis, long_Y_axis);
		SomeString = XmStringCreateLocalized (SomeChar);
		XtVaSetValues (poincareModified_Label[2],
			       XmNlabelString, SomeString,
			       NULL);		
	}
	else
	{
		FunctionArg->square = False;
		sprintf (SomeChar, "(%1.3f X %1.3f cm)", long_X_axis, long_Y_axis);
		SomeString = XmStringCreateLocalized (SomeChar);
		XtVaSetValues (poincareModified_Label[2],
			       XmNlabelString, SomeString,
			       NULL);		
	}
	XmStringFree(SomeString);
}
/*
************************************************************************************
*
*	Function:
*		tms_threeDSetSensitive (SensitifList[])
*	Description:
*		set sensitif or unsensitif the ThreeD window's widgets
*
*	Inputs:
*		Boolean	SensitifList	:	list of sensitve state for window's widget that need so
*
*	Revision:
*
*************************************************************************************
*/

void tms_threeDSetSensitive (Boolean SensitifList[])
{
/*
**	Variables
*/
	int	Index;

/*
**	Code
*/
	Index = 0;

	XtVaSetValues (threeDTeta_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (threeDPhi_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (threeDScreen_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
	XtVaSetValues (threeDEye_TextField,
		       XmNsensitive, SensitifList[Index],
		       NULL);
	Index++;
}

/*
**************************************************************
*
*	Function:
*		tms_MainExitButtonCB (widget, client_data, call_data)
*	Description:
*		Exit correctly of the all application
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		april the 26 of 1996
*
****************************************************************
*/

void tms_MainExitButtonCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

/*
**	Code
*/
	fin();
	XtPopdown (main_TopLevelShell);
	XtPopdown (parameters_TopLevelShell);
	XtPopdown (fileSelection_TopLevelShell);
	XtPopdown (filter_TopLevelShell);

	exit(0);
}

/*
************************************************************************************
*
*	Function:
*		tms_MainPrintOnCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList)
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*
*************************************************************************************
*/

void tms_MainPrintOnCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	FILE		*PrintcapFile;
	int		Index;
	char		FileLine[256];
	char		*SomeChar;
	XmString	SomeString;
	static Widget	printMain_Form;
	int		PrinterCount;
/*
**	Code
*/
	PrinterCount = 0;

	/*** create the print window the first time ***/

	if (!printMain_Form)
	{
		printMain_Form = (Widget)CreateprintMain_Form(TheAppShell);
	}

	/*** init the printer interface ***/

	XmListDeleteAllItems(printPrinter_List);

	for (Index = 0; Index < PRINTACTIONS_NB; Index++)
	{
		XtSetSensitive (printActions_ToggleButton[Index], True);
	}

	/*** open the printcap file ***/
	
	PrintcapFile = fopen ("/etc/printcap", "r");
	if (PrintcapFile == NULL)
	{
		DispMessageDialog (widget,
				   "Impossible to read /etc/printcap file\n\
				   You can only print in the file",
				   "Error : file not opened",
				   XmDIALOG_ERROR);
		XtSetSensitive (printActions_ToggleButton[0], False);
		XtSetSensitive (printActions_ToggleButton[2], False);
		XmToggleButtonSetState (printActions_ToggleButton[1], True, False);
	}
	else
	{
		XmToggleButtonSetState (printActions_ToggleButton[2], True, False);

		/*** Read the printcap and display the available printers ***/

		PrinterCount = 0;
		while ( fgets (FileLine, 256, PrintcapFile) != NULL)
		{
			/* look for alpha character */
			
			if (isalpha (*FileLine))
			{
				SomeChar = strchr (FileLine, ':');
				if (SomeChar)
				{
					*SomeChar = '\0';
				}
				SomeChar = strchr (FileLine, '|');
				if (SomeChar)
				{
					*SomeChar = '\0';
				}
				SomeString = XmStringCreateSimple (FileLine);
				XmListAddItem (printPrinter_List, SomeString, 0);
				XmStringFree(SomeString);
				PrinterCount++;
			}
		}
		XmListSelectPos (printPrinter_List, 1, False);
	}
	XtVaSetValues (printPrinter_List,
		       XmNvisibleItemCount, PrinterCount,
		       NULL);
	XtManageChild (printMain_Form);
	fclose (PrintcapFile);
}

/*
************************************************************************************
*
*	Function:
*		tms_MainPrintOnOKCB (widget, client_data, call_data)
*	Description:
*		Set sensitive or not the others parameters (TextField, Toggle Button and ScrolledList)
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*
*************************************************************************************
*/

void tms_MainPrintOnOKCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	char		*PrinterName;
	Boolean		Founded;
	int		*PrinterPos;
	int		PrinterCount;
	XmString	*AllPrinterName;
	int		Index;
/*
**	Code
*/
	PrinterPos = NULL;
	PrinterName = NULL;
	
	/*** Get the printer name ***/

	Founded = XmListGetSelectedPos (printPrinter_List,
					&PrinterPos,
					&PrinterCount);
	if (!PrinterCount)
	{
		DispMessageDialog (widget,
				   "No printer has been selected.\n\
				   Please choose a printer",
				   "Error : No printer found",
				   XmDIALOG_ERROR);
		return;
	}

	XtVaGetValues (printPrinter_List,
		       XmNitems, &AllPrinterName,
		       XmNitemCount, &PrinterCount,
		       NULL);
		       
	XmStringGetLtoR (AllPrinterName[PrinterPos[0] - 1], XmSTRING_DEFAULT_CHARSET, &PrinterName);

	/*** Set the printer queue for ps routines ***/
	
	ps_queue_name (PrinterName);

	if (XmToggleButtonGetState (printActions_ToggleButton[0]))
	{
		print_file(widget, (XtPointer) 'd',call_data);
	}
	else if (XmToggleButtonGetState (printActions_ToggleButton[1]))
	{
		print_file(widget, (XtPointer) 's',call_data);
	}
	else if (XmToggleButtonGetState (printActions_ToggleButton[2]))
	{
		print_file(widget, (XtPointer) 'f',call_data);
	}

	if (PrinterPos)
	{
		XtFree ((char *)PrinterPos);		
	}
	for (Index = 0; Index < PrinterCount; Index++)
	{
		XmStringFree (AllPrinterName[Index]);
	}
	if (PrinterName)
	{
		XtFree (PrinterName);			
	}
}

/*
************************************************************************************
*
*	Function:
*		tms_MainInitMenuVar (widget, client_data, call_data)
*	Description:
*		Initialize the functions name variables
*
*	Inputs:
*
*	Revision:
*
*************************************************************************************
*/

void tms_MainInitMenuVar()
{
		/* Actions boards... */

	action_2[0] = freqgram;
	action_2[1] = freqgram_dots;
	action_2[2] = freqgram2;
	action_2[3] = freqhisto;
	action_2[4] = interv_time;
	action_2[5] = interv_time2;
	action_2[6] = phase_time;
	action_2[7] = phase_inter;
	
	action_3[0] = interv_rank;
	action_3[1] = phase_rank;
	action_3[2] = fourier_order;

	action_4[0] = autocorrelation;
	action_4[1] = crosscorrelation;

	action_5[0] = poincare;
	action_5[1] = poincare_2;
	action_5[2] = poincare_int_cyclic;
	action_5[3] = poincare_phases;
	action_5[4] = poincare_pha_cyclic;

	action_6[0] = inter_event_histo;
	action_6[1] = dot_display;
	action_6[2] = lissajous;
	action_6[3] = post_event_rate_hist;
	action_6[4] = post_event_int_pooled;
	action_6[5] = post_event_rate_pooled;
	action_6[6] = post_event_phase_pooled;
	action_6[7] = scatter_diagram;

	action_7[0] = inter_3D;
	action_7[1] = phase_3D;
	
	action_8[0] = derive_inter;
	action_8[1] = exp_derive_inter;
	action_8[2] = derive_phase;

}

/*
**************************************************************
*
*	Function:
*		tms_MainFunctionCB (widget, client_data, call_data)
*	Description:
*		create the fctpara window (only for the first call).
*		Prepare the fctpara window depending of the function selected in the main menu.
*		display the fctpara window.
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 20 of 1996
*		june the 10 of 1996 end of design
*
****************************************************************
*/

void tms_MainFunctionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	Arg		   args[6];
	int		   ac;
	int		   Index;
	int		   TheStatus;
	Widget		   fctpara_Form;
	char		   SomeChar[250];
	XmString	   SomeString;
	char		   *ClientValue;
	char		   TypeOfCalcul;
	char		   TheCalcul;
	short		   ActionNumber;
	static pCALCULSARG FunctionArg;
	float		   SomeFloat;
	Boolean		   JustOneWindow;
	Boolean            NeedHisto;
	Boolean            NeedPoincare;
	Boolean            NeedArrows;
	Boolean            NeedThreeD;
	char		   PrintfChar[80];
/*
**	Code
*/
	ClientValue = NULL;

	/*** Menus choices ***/


	menu_2[0] = " A Instantaneous rate = f(time)    (1 event)    [line drawing]";
	menu_2[1] = " B Instantaneous rate = f(time)    (1 event)    [dots drawing]";
	menu_2[2] = " C Instantaneous rate = f(time)    (2 events)   [dots drawing]";
	menu_2[3] = " D Rate as funtion of running time (1 event)    [histogram] ";
	menu_2[4] = " E Interval = f(time)              (1 event)    [dots drawing]";
	menu_2[5] = " F Interval = f(time)              (2 events)   [dots drawing]";
	menu_2[6] = " G Phase = f(time)                 (2 events)   [dots drawing]";
	menu_2[7] = " H Phase = f(interval)             (2 events)   [dots drawing]";

	menu_3[0] = " A interval = f(rank)      (1 event)       [dots drawing] ";
	menu_3[1] = " B phase = f(rank)         (2 events)      [dots drawing] ";
	menu_3[2] = " C power spectrum of rank  (1 event)       [histogram] ";

	menu_4[0] = " A autocorrelation          (1 event)       [histogram] ";
	menu_4[1] = " B crosscorrelation         (2 events)      [histogram] ";

	menu_5[0] = " A Poincare\'s map of intervals     (1 event)       [dot drawing] ";
	menu_5[1] = " B 2 Poincare\'s maps of intervals  (2 events)      [dot drawing] ";
	menu_5[2] = " C Poincare\'s map of int for cycle (2 or 3 events) [dot drawing] ";
	menu_5[3] = " D Poincare\'s map of phases        (2 events)      [dot drawing] ";
	menu_5[4] = " E Poincare\'s map of pha for cycle (3 events)      [dot drawing] ";


	menu_6[0] = " A Inter event histogram      (1 event)       [histogram] ";
	menu_6[1] = " B Dot display                (2 events)      [dot drawing] ";
	menu_6[2] = " C Lissajous of cyclic data   (2 events)      [dot drawing] ";
	menu_6[3] = " D Post event histogram       (2 or 3 events) [histogram] ";
	menu_6[4] = " E Post event interv pooled   (2 or 3 events) [dot drawing] ";
	menu_6[5] = " F Post event rate pooled     (2 or 3 events) [dot drawing] ";
	menu_6[6] = " G Post event phase pooled    (3 events)      [dot drawing] ";
	menu_6[7] = " H Scatter diagram            (3 events)      [dot drawing] ";

	menu_7[0] = " A Interval I1 = f(I2,I3)       (1 event)        [dot drawing] ";
	menu_7[1] = " B Phase P1 = f(P2,P3)          (2 events)       [dot drawing] ";

	menu_8[0] = " A d(interval)/dt = f(Interval)       (1 event)  [dot drawing] ";
	menu_8[1] = " B exp(d(interval)/dt) = f(Interval)  (1 event)  [dot drawing] ";
	menu_8[2] = " C d(phase)/dt = f(phase)             (2 event)  [dot drawing] ";

	/*** dispatch actions ***/

	JustOneWindow = True;
	NeedHisto = NeedPoincare = NeedArrows = NeedThreeD = False;

	ClientValue = (char *) client_data;

	if (!fctpara_TopLevelShell)
	{
		strcpy (PrintfChar, VERSION);
		ac = 0;
		XtSetArg(args[ac], XmNtitle, PrintfChar); ac++;
		XtSetArg(args[ac], XmNiconName, PrintfChar); ac++;
		XtSetArg(args[ac], XmNx, 200); ac++;
		XtSetArg(args[ac], XmNy, 200); ac++;
		XtSetArg(args[ac], XmNwidth, 730); ac++;
		XtSetArg(args[ac], XmNheight, 660); ac++;
		fctpara_TopLevelShell = XtCreatePopupShell("fctpara_TopLevelShell",
							   topLevelShellWidgetClass,
							   TheAppShell,	
							   args, 
							   ac);
		fctpara_Form = (Widget)CreateFctPara_Form(fctpara_TopLevelShell);
		XtManageChild(fctpara_Form);

		FunctionArg = (pCALCULSARG) XtMalloc (sizeof (CALCULSARG));
	}
	else
	{
		if (XtIsRealized(fctpara_TopLevelShell))
		{
			tms_fctparaDismissCB (fctparaApply_PushButton, client_data, call_data);
		}
	}

	/*** Initialize the lists of widgets sensitive state ***/

	for (Index = 0; Index < FCTPARA_PARANB; Index++)
	{
		FunctionArg->FctparaList[Index] = False;
	}
	for (Index = 0; Index < HISTOGRAM_PARANB; Index++)
	{
		FunctionArg->HistogramList[Index] = False;
	}
	for (Index = 0; Index < ARROWS_PARANB; Index++)
	{
		FunctionArg->ArrowsList[Index] = False;
	}
	for (Index = 0; Index < POINCARE_PARANB; Index++)
	{
		FunctionArg->PoincareList[Index] = False;
	}
	for (Index = 0; Index < THREED_PARANB; Index++)
	{
		FunctionArg->ThreeDList[Index] = False;
	}

	/*** empty all fctpara's text fields***/

	XmTextSetString (fctparaBoardNb_TextField[DATA1], "");
	XmTextSetString (fctparaBoardMin_TextField[DATA1], "");
	XmTextSetString (fctparaBoardMax_TextField[DATA1], "");
	XmTextSetString (fctparaBoardNb_TextField[DATA2], "");
	XmTextSetString (fctparaBoardMin_TextField[DATA2], "");
	XmTextSetString (fctparaBoardMax_TextField[DATA2], "");
	XmTextSetString (fctparaBoardNb_TextField[DATA3], "");
	XmTextSetString (fctparaBoardMin_TextField[DATA3], "");
	XmTextSetString (fctparaBoardMax_TextField[DATA3], "");
	XmTextSetString (fctparaBin_TextField, "");
	XmTextSetString (fctparaDuration_TextField, "");
	XmTextSetString (fctparaOrder_TextField, "");
	XmTextSetString (fctparaCycle_TextField, "");

	SomeString = XmStringCreateLocalized ("Analysis duration:");
	XtVaSetValues (fctparaModified_Label[3],
		       XmNlabelString, SomeString,
		       NULL);
	XmStringFree(SomeString);

	SomeString = XmStringCreateLocalized ("Cycle marker:");
	XtVaSetValues (fctparaModified_Label[1],
		       XmNlabelString, SomeString,
		       NULL);
	XmStringFree(SomeString);

	/*** select wich functions to call for calculs ***/

	TypeOfCalcul = ClientValue[0];
	TheCalcul = ClientValue[1];
	ActionNumber = (int) TheCalcul - (int) 'A';
	strcpy(FunctionArg->action, ClientValue);

	switch (TypeOfCalcul)
	{
		case 'A': /*** f(Time) drawings ***/
		{
			/*** prepare the FctPara window ***/

			switch (TheCalcul)
			{
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'B':
				{
					FunctionArg->FctparaList[DATA1] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'C':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'D':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[5] = True;	/* activate Binwidth widget*/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%.6f", user_binwidth);
					XmTextSetString (fctparaBin_TextField, SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'F':
				{
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->HistogramList[3] = NeedHisto = True;
					
					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);

					/*** calculs the interval for second event ***/
					/*** The first one will be calculated in part 'E' ***/
					
					FunctionArg->events[DATA2] = pre_synaptic;
					
					/*** Add callback to fctpara compute to call Histogram options window ***/
					/*** Add callback to histoOp OK button to execute the function ***/
					/***	and prepare the Histogramm Options window ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"histogram_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"histogram_TopLevelShell");
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					JustOneWindow = False;
				}
				case 'E':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[3] = True; /* set sensitif Order widget*/
					
					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);
					
					/*** calculs the interval for first event ***/
					
					if (TheCalcul == 'E')
					{
						JustOneWindow = True;
					}
					break;
				}
				case 'G':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True; /* set sensitif Order widget*/

					/*** put the default values ***/

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					JustOneWindow = True;
					break;
				}
				case 'H':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True; /* set sensitif Order widget*/

					/*** put the default values ***/

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					JustOneWindow = True;
					break;
				}
				default:
				{
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_2[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_2[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_2[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'B': /*** f(rank) drawings ***/
		{
			/*** prepare the FctPara window ***/

			switch (TheCalcul)
			{
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[3] = True; /* set sensitif Order widget*/

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					JustOneWindow = True;
					break;
				}
				case 'B':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True; /* set sensitif Order widget*/

					/*** put the default values ***/

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					JustOneWindow = True;
					break;
				}
				case 'C':
				{
					FunctionArg->FctparaList[DATA1] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					JustOneWindow = True;
					break;
				}
				default:
				{
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_3[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_3[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_3[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'C': /*** Correlation analysis ***/
		{
			/*** prepare the FctPara window ***/

			switch (TheCalcul)
			{
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[5] = True;
					FunctionArg->FctparaList[6] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%5f", user_duration_AC);
					XmTextSetString (fctparaDuration_TextField, SomeChar);
					
					sprintf (SomeChar,"%5f", user_binwidth_AC);
					XmTextSetString (fctparaBin_TextField, SomeChar);					
					JustOneWindow = True;
					break;
				}
				case 'B':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[5] = True;
					FunctionArg->FctparaList[6] = True;
					
					/*** put the default values ***/

					SomeString = XmStringCreateLocalized ("Demi duration of analysis:");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree(SomeString);

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					
					sprintf (SomeChar,"%5f", user_duration_CC);
					XmTextSetString (fctparaDuration_TextField, SomeChar);
					sprintf (SomeChar,"%5f", user_binwidth_CC);
					
					XmTextSetString (fctparaBin_TextField, SomeChar);
					
					JustOneWindow = True;
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_4[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_4[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_4[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'D': /*** Poincare's map ***/
		{
			/*** prepare the FctPara window ***/

			switch (TheCalcul)
			{
				case 'B':
				{
					FunctionArg->FctparaList[DATA2] = True;

					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					XmTextSetString (axisLetter_TextField, "i");
				}
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[3] = True;
					FunctionArg->PoincareList[2] = True;
					
					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					if (TheCalcul == 'A')
					{
						XmTextSetString (axisLetter_TextField, "n");
					}

					sprintf (SomeChar, "(%.1f X %.1f cm)", long_Y_axis, long_Y_axis);
					SomeString = XmStringCreateLocalized (SomeChar);
					XtVaSetValues (poincareModified_Label[2],
						       XmNlabelString, SomeString,
						       NULL);
					XtVaSetValues (poincareAxisSize_ToggleButton,
						       XmNset, True,
						       NULL);
					XmStringFree (SomeString);
										
					/*** Add callback to fctpara compute to call Poincare options window ***/
					/*** Add callback to poincare OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (poincareAxisSize_ToggleButton,
						       XmNvalueChangedCallback, tms_poincareAxisOptionCB, FunctionArg);

					JustOneWindow = False;
					NeedPoincare = True;
					break;
				}
				case 'C':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[6] = True;
					FunctionArg->PoincareList[0] = True;
					FunctionArg->PoincareList[1] = True;
					FunctionArg->PoincareList[2] = True;
					
					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					

					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					XmTextSetString (axisLetter_TextField, "i");
					
					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					
					sprintf (SomeChar, "%f",duree_cycle);
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Scanning Window (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);

					sprintf (SomeChar, "(%.1f X %.1f cm)", long_Y_axis, long_Y_axis);
					SomeString = XmStringCreateLocalized (SomeChar);
					XtVaSetValues (poincareModified_Label[2],
						       XmNlabelString, SomeString,
						       NULL);

					XtVaSetValues (poincareAxisSize_ToggleButton,
						       XmNset, True,
						       NULL);

					XmStringFree (SomeString);
					XmStringFree (SomeString);
					
					/*** Add callback to fctpara compute to call Poincare options window ***/
					/*** Add callback to poincare OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (poincareAxisSize_ToggleButton,
						       XmNvalueChangedCallback, tms_poincareAxisOptionCB, FunctionArg);
					JustOneWindow = False;
					NeedPoincare = True;
					break;
				}
				case 'D':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True;
					FunctionArg->PoincareList[2] = True;
					
					/*** put the default values ***/

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);

					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);
					
					sprintf (SomeChar, "(%.1f X %.1f cm)", long_Y_axis, long_Y_axis);
					SomeString = XmStringCreateLocalized (SomeChar);
					XtVaSetValues (poincareModified_Label[2],
						       XmNlabelString, SomeString,
						       NULL);

					XtVaSetValues (poincareAxisSize_ToggleButton,
						       XmNset, True,
						       NULL);

					XmTextSetString (axisLetter_TextField, "i");

					/*** Add callback to fctpara compute to call Poincare options window ***/
					/*** Add callback to poincare OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (poincareAxisSize_ToggleButton,
						       XmNvalueChangedCallback, tms_poincareAxisOptionCB, FunctionArg);
					JustOneWindow = False;
					NeedPoincare = True;
					break;
				}
				case 'E':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[3] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[6] = True;
					FunctionArg->PoincareList[0] = True;
					FunctionArg->PoincareList[1] = True;
					FunctionArg->PoincareList[2] = True;
					
					/*** put the default values ***/
					
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);

					XmTextSetString (axisLetter_TextField, "i");
					
					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					
					sprintf (SomeChar, "%f",duree_cycle);
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Scanning Window (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);

					sprintf (SomeChar, "(%.1f X %.1f cm)", long_Y_axis, long_Y_axis);
					SomeString = XmStringCreateLocalized (SomeChar);
					XtVaSetValues (poincareModified_Label[2],
						       XmNlabelString, SomeString,
						       NULL);

					XtVaSetValues (poincareAxisSize_ToggleButton,
						       XmNset, True,
						       NULL);

					XmStringFree (SomeString);
					XmStringFree (SomeString);
					
					/*** Add callback to fctpara compute to call Poincare options window ***/
					/*** Add callback to poincare OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (poincareAxisSize_ToggleButton,
						       XmNvalueChangedCallback, tms_poincareAxisOptionCB, FunctionArg);
					JustOneWindow = False;
					NeedPoincare = True;
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_5[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_5[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_5[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'E': /*** Inter and Peri Event Analisys ***/
		{
			switch (TheCalcul)
			{
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[3] = True;
					FunctionArg->FctparaList[5] = True;
					FunctionArg->FctparaList[6] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", user_order);
					XmTextSetString (fctparaOrder_TextField, SomeChar);
					sprintf (SomeChar,"%.5f", user_duration_IS);
					XmTextSetString (fctparaDuration_TextField, SomeChar);
					sprintf (SomeChar,"%.5f", user_binwidth_IS);
					XmTextSetString (fctparaBin_TextField, SomeChar);

					JustOneWindow = True;					
					break;
				}
				case 'B':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[6] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);

					SomeString = XmStringCreateLocalized ("Scanning window (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree(SomeString);
					sprintf (SomeChar,"%.5f", user_duration_PST);
					XmTextSetString (fctparaDuration_TextField, SomeChar);

					JustOneWindow = True;					
					break;
				}
				case 'C':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[5] = True;
					FunctionArg->FctparaList[6] = True;
					FunctionArg->PoincareList[2] = True;
					FunctionArg->ArrowsList[0] = True;
					FunctionArg->ArrowsList[1] = True;
					FunctionArg->ArrowsList[2] = True;
					
					/*** put the default values ***/
					
					/* fctapara */
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);

					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					SomeString = XmStringCreateLocalized ("Cycle event:");
					XtVaSetValues (fctparaModified_Label[1],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					sprintf (SomeChar,"%f", user_binwidth_PST);
					XmTextSetString (fctparaBin_TextField, SomeChar);

					sprintf (SomeChar, "%f", user_duration_PST);
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Cycle duration (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					/* poincare window */

					sprintf (SomeChar, "(%.1f X %.1f cm)", long_Y_axis, long_Y_axis);
					SomeString = XmStringCreateLocalized (SomeChar);
					XtVaSetValues (poincareModified_Label[2],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);
					XtVaSetValues (poincareAxisSize_ToggleButton,
						       XmNset, True,
						       NULL);

					/*** Add callback to fctpara compute to call Poincare options window ***/
					/*** Add callback to poincare OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
						       
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"poincare_TopLevelShell");
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"arrows_TopLevelShell");
						       
					XtAddCallback (poincareOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (arrowsOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"arrows_TopLevelShell");
					XtAddCallback (arrowsOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
					XtAddCallback (poincareAxisSize_ToggleButton,
						       XmNvalueChangedCallback, tms_poincareAxisOptionCB, FunctionArg);

					JustOneWindow = False;
					NeedPoincare = True;
					NeedArrows = True;
					break;
				}
				case 'D':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[5] = True;
					FunctionArg->FctparaList[6] = True;
					FunctionArg->FctparaList[0] = True;
					FunctionArg->HistogramList[0] = True;
					FunctionArg->HistogramList[1] = True;
					FunctionArg->HistogramList[2] = True;
					FunctionArg->HistogramList[3] = True;

					/*** put the default values ***/

					/* fctpara window */
					
					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
/*
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
*/
					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					SomeString = XmStringCreateLocalized ("Reference event:");
					XtVaSetValues (fctparaModified_Label[1],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					sprintf (SomeChar, "%f", user_duration_PST);
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Histogram duration (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					sprintf (SomeChar,"%f", user_binwidth_PST);
					XmTextSetString (fctparaBin_TextField, SomeChar);

					/* histogram window */

					sprintf (SomeChar,"%d", 3);
					XmTextSetString (histogramLine1_TextField, SomeChar);

					sprintf (SomeChar,"%d", 1);
					XmTextSetString (histogramLine2_TextField, SomeChar);

					sprintf (SomeChar,"%d", ((int)(user_duration_PST/user_binwidth_PST)));
					XmTextSetString (histogramBinsNb_TextField, SomeChar);

					/*** Add callback to fctpara compute to call Historgam options window ***/
					/*** Add callback to histogram OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"histogram_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
						       
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"histogram_TopLevelShell");
					XtAddCallback (histogramOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);

					JustOneWindow = False;
					NeedHisto = True;
					break;					
				}
				case 'G':
				case 'F':
				case 'E':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[6] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					if (TheCalcul == 'G')
					{
					  sprintf (SomeChar,"%d", pre_synaptic);
					  XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					}
					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					SomeString = XmStringCreateLocalized ("Reference event:");
					XtVaSetValues (fctparaModified_Label[1],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					sprintf (SomeChar, "%f", user_duration_PST );
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Histogram duration (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					JustOneWindow = True;
					break;					

				}
				case 'H':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;
					FunctionArg->FctparaList[4] = True;
					FunctionArg->FctparaList[6] = True;

					/*** put the default values ***/

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);

					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);

					sprintf (SomeChar, "%d",cycle_marker);
					XmTextSetString (fctparaCycle_TextField, SomeChar);
					SomeString = XmStringCreateLocalized ("Reference event:");
					XtVaSetValues (fctparaModified_Label[1],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					sprintf (SomeChar, "%f", user_duration_PST );
					XmTextSetString (fctparaDuration_TextField, SomeChar);					
					SomeString = XmStringCreateLocalized ("Histogram duration (s):");
					XtVaSetValues (fctparaModified_Label[3],
						       XmNlabelString, SomeString,
						       NULL);
					XmStringFree (SomeString);

					XtVaSetValues (histogramScale_ToggleButton,
						       XmNset, True,
						       NULL);

					JustOneWindow = True;
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_6[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_6[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_6[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'F': /*** 3D drawings ***/
		{
			switch (TheCalcul)
			{
				case 'B':
				{
					FunctionArg->FctparaList[DATA2] = True;					

					/*** put the default values ***/

					/* fctpara window */

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
				}
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->ThreeDList[0] = True;
					FunctionArg->ThreeDList[1] = True;
					FunctionArg->ThreeDList[2] = True;
					FunctionArg->ThreeDList[3] = True;

					/*** put the default values ***/

					/* fctpara window */

					if(TheCalcul == 'A')
					{
						sprintf (SomeChar,"%d", post_synaptic);
					}
					else
					{
						sprintf (SomeChar,"%d", pre_synaptic);
					}
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);						

					/* threeD window */

					sprintf (SomeChar,"%f", -.75);
					XmTextSetString (threeDTeta_TextField, SomeChar);

					sprintf (SomeChar,"%f", .75);
					XmTextSetString (threeDPhi_TextField, SomeChar);

					sprintf (SomeChar,"%f", 20.);
					XmTextSetString (threeDScreen_TextField, SomeChar);

					sprintf (SomeChar,"%f", 10.);
					XmTextSetString (threeDEye_TextField, SomeChar);

					/*** Add callback to fctpara compute to call Historgam options window ***/
					/*** Add callback to histogram OK button to execute the function ***/

					XtAddCallback (fctparaCompute_PushButton,
						       XmNactivateCallback, BxPopupCB, (XtPointer)"threeD_TopLevelShell");
					XtAddCallback (fctparaApply_PushButton,
						       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);
						       
					XtAddCallback (threeDOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"threeD_TopLevelShell");
					XtAddCallback (threeDOK_PushButton,
						       XmNactivateCallback, BxPopdownCB, (XtPointer)"fctpara_TopLevelShell");
					XtAddCallback (threeDOK_PushButton,
						       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);

					JustOneWindow = False;
					NeedThreeD = True;
					break;
				}
			}
			sprintf (PrintfChar,"%s - %s", VERSION, menu_7[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_7[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_7[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			break;
		}
		case 'G': /*** Phase map analysis ***/
		{
			sprintf (PrintfChar,"%s - %s", VERSION, menu_8[ActionNumber]);
			XtVaSetValues (fctpara_TopLevelShell,
				       XmNtitle, PrintfChar,
				       NULL);
			FunctionArg->function = action_8[ActionNumber];
			SomeString = XmStringCreateLocalized (menu_8[ActionNumber]);
			XtVaSetValues (fctparaTitle_Label,
			      	       XmNlabelString, SomeString,
		       		       NULL);
	       		XmStringFree(SomeString);
			switch(TheCalcul)
			{
				case 'A':
				{
					FunctionArg->FctparaList[DATA1] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'B':
				{
					FunctionArg->FctparaList[DATA1] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					JustOneWindow = True;
					break;
				}
				case 'C':
				{
					FunctionArg->FctparaList[DATA1] = True;
					FunctionArg->FctparaList[DATA2] = True;

					sprintf (SomeChar,"%d", post_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA1], SomeChar);
					sprintf (SomeChar,"%d", pre_synaptic);
					XmTextSetString (fctparaBoardNb_TextField[DATA2], SomeChar);
					JustOneWindow = True;
					break;
				}
			}

		}
		default:
		{
			printf ("tms_MainFunctionCB : not yet implemented\n");
			break;
		}
	}

	tms_fctparaSetSensitive (FunctionArg->FctparaList);
	tms_histogramSetSensitive (FunctionArg->HistogramList);
	tms_arrowsSetSensitive (FunctionArg->ArrowsList);
	tms_poincareSetSensitive (FunctionArg->PoincareList);
	tms_threeDSetSensitive (FunctionArg->ThreeDList);

	if (JustOneWindow)
	{
	  if (use_defaults)
	  {
	        tms_fctparaComputeCB(widget, (XtPointer)FunctionArg, NULL);
	  }
	  else
	  { 
	        XtAddCallback (fctparaCompute_PushButton,
		 	       XmNactivateCallback, tms_fctparaComputeCB, FunctionArg);
	        XtAddCallback (fctparaApply_PushButton,
			       XmNactivateCallback, tms_fctparaApplyCB, FunctionArg);			      
		XtPopup (fctpara_TopLevelShell, XtGrabNone);
	  }
	}
	else
	{
	        if (use_defaults)
	        {
		  tms_fctparaComputeCB(widget, (XtPointer)FunctionArg, NULL);
	        }
		else
		{
		  XtPopup (fctpara_TopLevelShell, XtGrabNone);
		}
	}

#ifndef linux
	if (ClientValue)
	{
		XtFree (ClientValue);			
	}
	return;
#endif
}
/*
************************************************************************************
*
*	Function:
*		tms_MainPrintOptionCB (widget, client_data, call_data)
*	Description:
*		Register the choice of automatic spooling or not 
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		may the 10 of 1996
*
*************************************************************************************
*/

void tms_MainPrintOptionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	XmToggleButtonCallbackStruct *ToggleState;

/*
**	Code
*/

	ToggleState = (XmToggleButtonCallbackStruct *) call_data;

	if (ToggleState->set)
	{
		driver = 's' ;
		spooled = TRUE;
		strcpy(on_garde," -k ");
	}
	else
	{
		driver = 'f' ;
		spooled = FALSE;
		strcpy(on_garde," ");
	}
}
/*
************************************************************************************
*
*	Function:
*		tms_MainDefaultsOptionCB (widget, client_data, call_data)
*	Description:
*		Register the choice of using defaults or not 
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		august the 28 of 1996
*
*************************************************************************************
*/

void tms_MainDefaultsOptionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/
	XmToggleButtonCallbackStruct *ToggleState;

/*
**	Code
*/

	ToggleState = (XmToggleButtonCallbackStruct *) call_data;

	if (ToggleState->set)
	{
		use_defaults = TRUE ;
	}
	else
	{
		use_defaults = FALSE ;
	}
}
/*
************************************************************************************
*
*	Function:
*		tms_MainStdDefaultsOptionCB (widget, client_data, call_data)
*	Description:
*		Replace user values by default values 
*
*	Inputs:
*		Widget		widget:		the activating widget.
*		XtPointer	client_data:	the client data, unused
*		XtPointer	call_data:	the call data, unused
*
*	Revision:
*		august the 28 of 1996
*
*************************************************************************************
*/

void tms_MainStdDefaultsOptionCB (Widget widget, XtPointer client_data, XtPointer call_data)
{
/*
**	Variables
*/

/*
**	Code
*/

     user_order    = 1;
     user_post_synaptic = post_synaptic = 1;
     user_pre_synaptic = pre_synaptic = 2;
     user_cycle_marker = cycle_marker = 4;
     user_binwidth_AC = 0.001;
     user_binwidth_CC = 0.001;
     user_binwidth_IS = 0.001;
     user_duration_AC = 0.05;
     user_duration_CC = 0.025;
     user_duration_IS = 0.1;

}













