#define XPS_DEBUG	1
#define UNIX            1
/*
*	                P  L  O  T  1  _  D  P  S
*	                ======================
*
*		   Routines de dessin sur la LN03R PostScript.
*
*	   NIVEAU 1 : Ce jeu de routine regroupe les routines de base
*		appelables par l'utilisateur de base
*
*		Mise en forme Version 2
*			Juillet 90
*		Version 3	Nouveaux noms de fonctions ps_*
*			31 juillet 90
*		Version 4
*			21 mars 91 / plot_dps V0
*		PLOT_DPS
*		Version 1.0
*			3 Avril 1991
*		Version 1.1
*			28 Mai 1991
*		Version 1.2
*			30 Oct 1991
*		PLOT_XPS
*		Version 1.0
*			3 Decembre 1992
*		Version 2.0
*			11 juillet 1994
*		Version 2.2
*			3 mai 1995
*		Version 2.5
*			9 dec 1995
*
*	Auteur: Dr J-F VIBERT
*		Laboratoire de Physiologie
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*			France
*
*	Copyright Dr JF Vibert 
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for resarch purpose
*	and is in any way usable for commercial purpose.
*
title	PLOT1	Niveau 1 des routines PLOT_XPS pour C
index		Niveau 1 des routines PLOT_XPS pour C

description

		   Routines de dessin sur la PLOT_XPS.

	NIVEAU 1 : Ce jeu de routine regroupe les routines 
		appelables par l'utilisateur


author

	Jean-Francois Vibert

	CHU Saint-Antoine CNRS UA1162 Universite Paris VI

See also

	PLOT0_XPS, PLOT2_XPS, PLOT3_XPS et PLOT3D_XPS

newpage
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef PLOT_XPS
#    undef PLOT_XPS
#endif

#include <plot_xps.h>
#include <Xm/MessageB.h>

extern Widget main_TopLevelShell;

/*
*
*	PS_INIPLOT() permet d'initialiser le dessin et la 1ere fenetre.
*	Il appelle plopen() et dessine la baniere
*
*/

#if UNIX
#	include <limits.h>
#	include <sys/utsname.h>
#	include <sys/types.h>
#	include <time.h>
#endif


void ps_get_pixmap_size(Display *le_display,
			 int *l_pixels,
			 int *h_pixels,
			 int large_cm,
			 int haut_cm,
			 int la_scale)
{
  int l, h;
  int screen;
  float sc_width,
        sc_cm;

#if XPS_DEBUG
    printf("Display: %x; &larg: %x; &haut: %x\n",
	   le_display,
	   l_pixels,
	   h_pixels);

        printf("ps 1: l: %d; h: %d; sc: %d \n",
	       large_cm, haut_cm, la_scale);
#endif
 	screen = (int) DefaultScreen(le_display);
#if XPS_DEBUG
    printf("ps 2: screen: %d\n",screen);
#endif

	sc_width =   (float)DisplayWidth(le_display,screen);
#if XPS_DEBUG
  printf("width: %f\n",sc_width);
#endif
	sc_cm =   (float)DisplayWidthMM(le_display,screen)/10.;
#if XPS_DEBUG
  printf("width cm: %f\n",sc_cm);
#endif
	pl_conv = (sc_width / sc_cm) / X_PAS;
#if XPS_DEBUG
    printf("ps 3\npl_conv: %f\n",pl_conv);
#endif

	pl_conv *= ((float)la_scale/100.);

#if XPS_DEBUG
    printf("ps 4\npl_conv: %f\n",pl_conv);
#endif

	l = (int)(pl_conv * X_PAS * ((float)large_cm/10.));

	h = (int)(pl_conv * X_PAS * ((float)haut_cm/10.));
        *(l_pixels) = l;
        *(h_pixels) = h;
#if XPS_DEBUG
    printf("ps 5\nl: %x %d %d\n",l_pixels,*l_pixels,l);
    printf("ps 6\nh: %x %d %d\n",h_pixels,*h_pixels,h);
#endif

}

int ps_iniplot_on(Display *display,
		  Window la_fenetre_mere,
		  Window la_fenetre,
		  char *la_plot_file,
		  int x_ps_xps,
		  int y_ps_xps,
		  int largeur_mm,
		  int hauteur_mm,
		  int scale_mm,
		  char *window_name)
{
	float   l_cm_xps, h_cm_xps, scale;  
	int	nb_char;
	int	l_ps_xps, h_ps_xps;
	char	proprio[64],
		moment[26],
		page[5];
	
	Visual  *visual;
	XGCValues values;
	XSizeHints zhints;

#if UNIX
	struct utsname sys_name, *p_sys_name;
	time_t tim, *p_tim;

	p_sys_name = &sys_name;
	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
	uname(p_sys_name);
	sprintf(proprio,"%s@%s",getlogin(),p_sys_name->nodename);

	/* Protection de la memoire vive en cas de ctrlC */
	signal(SIGINT,ps_controle_c);
	signal(SIGQUIT,ps_controle_c);

#endif
#if vms
	$DESCRIPTOR(dscDate, "12345678901234567890123");
	/* la chaine doit faire 23 caracteres de long */
	lib$date_time(&dscDate);
	dscDate.dsc$a_pointer[17]='\0';
	/* on tronque les secondes et les centiemes */
	strcpy(moment, dscDate.dsc$a_pointer);
	strcpy(proprio, "VAX/VMS");
#endif
#if MS_DOS
	strcpy(moment,"now...");
	strcpy(proprio,"MS/DOS");
#endif

/*
 * Initialisations generales
 */
	l_cm_xps = (float) largeur_mm / 10.;
	h_cm_xps = (float) hauteur_mm / 10.;
	scale = (float) scale_mm / 100.;

	strcpy(queue_name,PLOT_XPS_QUEUE_NAME);

	if ((xps_dessin[0] =
	     (struct ps_dessin *)calloc(1,sizeof(struct ps_dessin))) == NULL)
	{
		fprintf(stderr,"\007PLOT_XPS: Impossible to allocate space for a new drawing\n");
		return(PS_NO_ROOM_DESSIN);
	} /* fin du if */
	ps_current_window = 0;


	xps_dessin[0]->pl_ex = xps_dessin[0]->pl_ey = PL_PAS;
	xps_dessin[0]->pl_rex = xps_dessin[0]->pl_rey = X_PAS;
	xps_dessin[0]->pl_ps_page = xps_dessin[0]->pl_cms = 1;
	xps_dessin[0]->pl_ca = 1.;
	xps_dessin[0]->pl_sa = 0.;

	nb_char = strlen(la_plot_file);
	if (nb_char)
	{
		xps_dessin[0]->ps_hard_copy = TRUE;
		if ((xps_dessin[0]->plot_file = (char *)calloc(nb_char+1,sizeof(char))) == NULL)
		{
			fprintf(stderr,"\007PLOT_XPS: No room for the name of the file %s\n",
				la_plot_file);
			xps_dessin[0]->ps_hard_copy = FALSE;
		} /* fin du if */
	} /* fin du if */
	else
		xps_dessin[0]->ps_hard_copy = FALSE;

	xps_dessin[0]->ps_xps = FALSE;

	if (xps_dessin[0]->ps_hard_copy)
	{

		strcpy(xps_dessin[0]->plot_file,la_plot_file);	/* pour pl_newpage */
		plopen(xps_dessin[0]->plot_file);
		fprintf(xps_dessin[0]->pl_spl,"%%!PS-Adobe-2.0 EPSF-1.2\n");
		fprintf(xps_dessin[0]->pl_spl,"%%PLOT_XPS V%s\n%%%%Creator: %s\n",PS_VERSION,proprio);
		fprintf(xps_dessin[0]->pl_spl,"%%%%Title: %s\n",xps_dessin[0]->plot_file);
		fprintf(xps_dessin[0]->pl_spl,"%%%%CreationDate: %s\n",moment);
		fprintf(xps_dessin[0]->pl_spl,"%%%%Pages: (atend)\n");
		fprintf(xps_dessin[0]->pl_spl,"%%%%EndComments\n");
		fprintf(xps_dessin[0]->pl_spl,"save\n");
		fprintf(xps_dessin[0]->pl_spl,"/M { moveto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/rM { rmoveto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/L { lineto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/rL { rlineto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/S { show } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/GSG { gsave stroke grestore } bind def\n");
	
		fprintf(xps_dessin[0]->pl_spl,"%%%%Beginning of the flag page\n");

		/*	Le cadre de flag-page */

		traa(0.8,0.8,PL_AH);
		traa(19.5,0.,PL_RB);
		traa(0.,1.,PL_RB);
		traa(-19.5,0.,PL_RB);
		traa(0.,-1.,PL_RB);
		traa(18.5,0.,PL_RH);
		traa(0.,1.,PL_RB);
	
		traa(1.,1.15,PL_AH);

		/* le texte de la baniere */
		ps_font("Times-Roman",0.4);
		ps_chara(0.,0.,PL_RH,moment,0.4,0);
		traa(7.,1.1,PL_AH);
		ps_font("NewCenturySchlbk-BoldItalic",0.5);
		ps_chara(0.,0.,PL_RH,xps_dessin[0]->plot_file,0.5,0);
		traa(14.,1.15,PL_AH);
		ps_font("Times-Roman",0.4);
		ps_chara(0.,0.,PL_RH,proprio,0.4,0);
		traa(19.5,1.15,PL_AH);
		sprintf(page,"%2d",xps_dessin[0]->pl_ps_page);
		ps_chara(0.,0.,PL_RH,page,0.4,0);

		traa(0.,0.,PL_AH);

		fprintf(xps_dessin[0]->pl_spl,"%%End of the flag page\n");
	}
	else
	{
		fprintf(stderr,"\007PLOT_XPS: Hard Copy was not previously required\n");
		fprintf(stderr,"          No Post_script file will be produced\n");
		if (xps_dessin[ps_current_window]->ps_xps)
			fprintf(stderr,"          Only an Xwindow display will be done\n");
	} /* fin du else */

	/*
	 * Maintenant on ouvre une fenetre X s'il y a lieu
	 */
	xps_dessin[0]->ps_xps = TRUE;
	if (l_cm_xps == 0. && h_cm_xps == 0.)
		xps_dessin[0]->ps_xps = FALSE;
	else
		xps_dessin[0]->ps_xps = TRUE;
	xps_open_display = FALSE;
	if (xps_dessin[ps_current_window]->ps_xps)
	{
		ps_window_max = 0;
		if (display)
		{
			xps_display = display;
			xps_open_display = FALSE;
		}
		else
		{
			xps_display = XOpenDisplay(NULL);
			xps_open_display = TRUE;
		}
		if (xps_display)
		{
#if XPS_DEBUG
			printf("xps_display open\n");
#endif
			xps_screen = DefaultScreen(xps_display);
			ps_current_window = 0;
			xps_dessin[0]->ps_auto_flush = TRUE;
			xps_dessin[0]->ps_wait_page = FALSE;
			pl_conv = (float)((DisplayWidth(xps_display,xps_screen))
				    /(((DisplayWidthMM(xps_display,xps_screen))
				      /10)))
				 / X_PAS;
			pl_conv *= scale;

			l_ps_xps = (int)(pl_conv * X_PAS * l_cm_xps);
			h_ps_xps = (int)(pl_conv * X_PAS * h_cm_xps);
			
			visual = DefaultVisual(xps_display,xps_screen);
			switch(visual->class)
			{
				case StaticGray : type_ecran = 0; break;
				case StaticColor: type_ecran = 1; break;
				case GrayScale  : ;
				case PseudoColor: type_ecran = 2; break;
				case TrueColor  : ;
				case DirectColor: type_ecran = 3; break;
			}
			if (type_ecran == 2)
			{
				couleur();
				XSetWindowColormap(xps_display,la_fenetre_mere,colormap);
				XSetWindowBorder(xps_display,la_fenetre_mere,BlackPixel(xps_display,xps_screen));
				XSetWindowBackground(xps_display,la_fenetre_mere,WhitePixel(xps_display,xps_screen));
			}
       			xps_dessin[0]->xps_window = la_fenetre;

			xps_dessin[0]->fontinfo = NULL;

			xps_dessin[0]->alpha = 0;
			
			xps_dessin[0]->width = l_ps_xps;
			xps_dessin[0]->height = h_ps_xps;
			
#if XPS_DEBUG
			printf("xps_window open (%d x %d)\n",
			       xps_dessin[0]->width,
			       xps_dessin[0]->height);
#endif
			xps_dessin[0]->cmt[0][0] = pl_conv;
			xps_dessin[0]->cmt[1][0] = 0.;
			xps_dessin[0]->cmt[0][1] = 0.;
			xps_dessin[0]->cmt[1][1] = -pl_conv;
			xps_dessin[0]->cmt[0][2] = 0.;
			xps_dessin[0]->cmt[1][2] = (float)xps_dessin[0]->height;

			xps_dessin[0]->xini = 0.;
			xps_dessin[0]->yini = 0.;
			xps_dessin[0]->xder = 0.;
			xps_dessin[0]->xder = 0.;

			XSelectInput(xps_display, xps_dessin[0]->xps_window,
				     ExposureMask | ButtonPressMask | KeyPressMask);

			xps_dessin[0]->xps_gc = XCreateGC(xps_display,
							  xps_dessin[0]->xps_window,
							  0,NULL);

#if XPS_DEBUG
			printf("xps_gc open\n");
#endif

			xps_dessin[0]->pixmap = XCreatePixmap(xps_display,
						 xps_dessin[0]->xps_window,
						 l_ps_xps, h_ps_xps,
						 DefaultDepth(xps_display,xps_screen));
#if XPS_DEBUG
			printf("pixmap open\n");
#endif
			XSetForeground(xps_display,
				       xps_dessin[0]->xps_gc,
				       WhitePixel(xps_display,xps_screen));
#if XPS_DEBUG
			printf("SetForeground 1 done\n");
#endif


			XFillRectangle(xps_display,
				       xps_dessin[0]->pixmap,
				       xps_dessin[0]->xps_gc,
				       0,0,
				       l_ps_xps, h_ps_xps);
#if XPS_DEBUG
			printf("FillRectangle done\n");
#endif

			XSetWindowBackgroundPixmap(xps_display,
						   xps_dessin[0]->xps_window,
						   xps_dessin[0]->pixmap);
#if XPS_DEBUG
			printf("SetWindowBackgroundPixmap done\n");
#endif
			XSetForeground(xps_display,
				       xps_dessin[0]->xps_gc,
				       BlackPixel(xps_display,xps_screen));
#if XPS_DEBUG
			printf("SetForeground 2 done\n");
#endif


			xps_dessin[0]->red = 0.;
			xps_dessin[0]->green = 0.;
			xps_dessin[0]->blue = 0.;
/*			
			zhints.x = x_ps_xps;
			zhints.y = y_ps_xps;
			zhints.width = l_ps_xps;
			zhints.height = h_ps_xps;
			zhints.min_width = l_ps_xps;
			zhints.max_width = l_ps_xps;
			zhints.min_height = h_ps_xps;
			zhints.min_height = h_ps_xps;
			zhints.flags |= PMinSize | PMaxSize;
			XSetStandardProperties(xps_display,
					       xps_dessin[0]->xps_window,
					       window_name,window_name,
					       None,NULL,0,
					       &zhints);

#if XPS_DEBUG
			printf("SetStandardProperties done\n");
#endif

			if ((xps_dessin[0]->ps_window_name =
			     (char *)calloc((strlen(window_name)+1),sizeof(char))) == NULL)
			{
				fprintf(stderr,"\007PLOT_XPS: Impossible to allocate space for the window name\n");
				return(PS_NO_ROOM_NAME);
			} 

			strcpy(xps_dessin[0]->ps_window_name,"");
#if XPS_DEBUG
			printf("Strcpy done\n");
#endif

*/

			XMapWindow(xps_display, xps_dessin[0]->xps_window);
#if XPS_DEBUG
			printf("XMap done\n");
#endif


			XFlush(xps_display);
#if XPS_DEBUG
			printf("XFlush done\n");
#endif

			ps_current_window = 0;
			ps_window_max++;
#if XPS_DEBUG
			printf("%d fenetre mapee (la %d)\n",ps_window_max,ps_current_window);
#endif
		} /* fin du if */
		else
		{
			xps_dessin[0]->ps_xps = FALSE;
			ps_current_window = PS_NO_WINDOW;
			ps_window_max = 0;
			fprintf(stderr,"\007PLOT_XPS: Not an Xwindow terminal or set display\n");
			if (xps_dessin[0]->ps_hard_copy)
				fprintf(stderr,"      Only a printed output will be available\n");
			else
			{
				fprintf(stderr,"      A printed output was not required\n");
				fprintf(stderr,"\007      NO IMAGE will be done!!!!\n");
			} /* fin du else */
		}
		
	} /* fin du if */
	else
	{
		fprintf(stderr,"\007PLOT_XPS: No window asked\n");
		if (xps_dessin[0]->ps_hard_copy)
			fprintf(stderr,"      Only a printed output will be available\n");
		else
		{
			fprintf(stderr,"      A printed output was not required\n");
			fprintf(stderr,"\007      NO IMAGE will be done!!!!\n");
		} /* fin du else */
	}/* fin du else */
#if XPS_DEBUG
	printf("xps_done, on attaque...\n");
#endif
	/*
	 * Mise en place pour le dessin
	 */

	ps_origin(1.,2.,0.,0.);
	ps_font("Times-Roman",0.4);

#if XPS_DEBUG
	printf("ps_origin fait\n");
#endif

	/* Initialisation de la Bounding Box aux valeurs inverses */

	xps_dessin[0]->b_b_ll_x = UR_X;
	xps_dessin[0]->b_b_ll_y = UR_Y;
	xps_dessin[0]->b_b_ur_x = LL_X;
	xps_dessin[0]->b_b_ur_y = LL_Y;

	xps_dessin[0]->ps_show = TRUE;
	return (0);
}


/*
*
*	PS_ENDPLOT_ON() permet de terminer et spooler la laser. Il appelle plclose()
*	et spoole eventuellement en effacant toutes les fenetres non deja fermees
*	par PS_CLOSE_WINDOW
*       C'est ce qui doit fermer une session ouverte par ps_ini_plot_on
*
*/

ps_endplot_on(char action)
		/* Action to take about the plot file
		 *	s: spool and keep
		 *	d: spool and delete
		 *	f: freeze (no spool)
		 */ 
{
	char ligne[250];
	char *print_command = NULL;
	register len, status, i;

	if (!(print_command = getenv("PRINT_PLOT_PS")))
	{
	  strcpy(ligne,"PRINT_PLOT_PS not defined\nUsing a standard way on the printer <<postscript>>");
	  DispMessageDialog(main_TopLevelShell, ligne,"WARNING",XmDIALOG_MESSAGE);
	}

	i = 0;
	if (ps_window_max == 0)
	{
		ps_code("GSG\n");
		if (xps_dessin[0]->ps_hard_copy)
		{
			plclose(xps_dessin[0]->plot_file);	/* fermeture du fichier */
			switch (action)
			{
				case 'd':
#if UNIX
			                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[0]->plot_file);	  
					else
					  sprintf(ligne,"lpr -P%s %s",queue_name,xps_dessin[0]->plot_file);
					system(ligne);
					sprintf(ligne,"rm %s",xps_dessin[0]->plot_file);
#endif
#if vms
			                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[0]->plot_file);	  
					else
					  sprintf(ligne,"PRINT /QUEUE=%s %s /DELETE",queue_name,xps_dessin[0]->plot_file);
					system(ligne);
#endif
					break;
				case 'f':
					break;
				case 's':
				default:
#if UNIX
					if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[0]->plot_file);	  
					else
					  sprintf(ligne,"lpr -P%s %s",queue_name,xps_dessin[0]->plot_file);
#endif
#if vms
		       	                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[0]->plot_file);	  
					else
					  sprintf(ligne,"PRINT /QUEUE=%s %s",queue_name,xps_dessin[0]->plot_file);
#endif
#if MS_DOS
      			                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[0]->plot_file);	  
					else
					  sprintf(ligne,"PRINT %s",xps_dessin[0]->plot_file);
#endif
					system(ligne);
			}
		}
		return;
	} /* fin du if */
	for (i = 0; i < ps_window_max; i++)
	{
		ps_code("GSG\n");
		if (xps_dessin[i]->xps_window) /* Fenetre ouverte */
		{
			if (xps_dessin[i]->ps_hard_copy)
			{
				plclose(xps_dessin[i]->plot_file);	/* fermeture du fichier */
				switch (action)
				{
					case 'd':
#if UNIX
	       		                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[i]->plot_file);	  
					else
					  sprintf(ligne,"lpr -P%s %s",queue_name,xps_dessin[i]->plot_file);
						system(ligne);
						sprintf(ligne,"rm %s",xps_dessin[i]->plot_file);
#endif
#if vms
	      		                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[i]->plot_file);	  
					else
					  sprintf(ligne,"PRINT /QUEUE=%s %s /DELETE",queue_name,xps_dessin[i]->plot_file);
						system(ligne);
#endif
						break;
					case 'f':
						break;
					case 's':
					default:
#if UNIX
       			                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[i]->plot_file);	  
					else
					  sprintf(ligne,"lpr -P%s %s",queue_name,xps_dessin[i]->plot_file);
#endif
#if vms
	      		                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[i]->plot_file);	  
					else
					  sprintf(ligne,"PRINT /QUEUE=%s %s",queue_name,xps_dessin[i]->plot_file);
#endif
#if MS_DOS
	       		                if (print_command)
					  sprintf(ligne,"%s %s",print_command,xps_dessin[i]->plot_file);	  
					else
					  sprintf(ligne,"PRINT %s",xps_dessin[i]->plot_file);
#endif
						system(ligne);
				}
			}
			if (xps_dessin[i]->ps_xps)
			{
#if XPS_DEBUG
			  printf("dans ps_close_on (i = %d)\n",i);
#endif
			  if (i)
			  {
				XDestroyWindow(xps_display, xps_dessin[i]->xps_window);
				XFreePixmap(xps_display,xps_dessin[i]->pixmap);
			  }
			} /* fin du if */
		} /* fin du if */
		if (i)
		  free(xps_dessin[i]);	/* fenetre detruite */
	} /* fin du for */
/*
	if(type_ecran == 2 && i)
		XFreeColormap(xps_display,colormap);
*/
}

int ps_re_init_hard_copy(char *la_plot_file)
{
	int	nb_char;
	int	l_ps_xps, h_ps_xps;
	char	proprio[64],
		moment[26],
		page[5];
	
#if UNIX
	struct utsname sys_name, *p_sys_name;
	time_t tim, *p_tim;

	p_sys_name = &sys_name;
	p_tim = &tim;
	tim = time(0);
	sprintf(moment,"%s",ctime(p_tim));
	moment[(strlen(moment)-1)] = 0;	/* enleve le '\n' final */
	uname(p_sys_name);
	sprintf(proprio,"%s@%s",getlogin(),p_sys_name->nodename);

	/* Protection de la memoire vive en cas de ctrlC */
	signal(SIGINT,ps_controle_c);
	signal(SIGQUIT,ps_controle_c);

#endif
#if vms
	$DESCRIPTOR(dscDate, "12345678901234567890123");
	/* la chaine doit faire 23 caracteres de long */
	lib$date_time(&dscDate);
	dscDate.dsc$a_pointer[17]='\0';
	/* on tronque les secondes et les centiemes */
	strcpy(moment, dscDate.dsc$a_pointer);
	strcpy(proprio, "VAX/VMS");
#endif
#if MS_DOS
	strcpy(moment,"now...");
	strcpy(proprio,"MS/DOS");
#endif

/*
 * Initialisations generales
 */

	nb_char = strlen(la_plot_file);
	if (nb_char)
	{
		xps_dessin[0]->ps_hard_copy = TRUE;
		if ((xps_dessin[0]->plot_file = (char *)calloc(nb_char+1,sizeof(char))) == NULL)
		{
			fprintf(stderr,"\007PLOT_XPS: No room for the name of the file %s\n",
				la_plot_file);
			xps_dessin[0]->ps_hard_copy = FALSE;
		} /* fin du if */
	} /* fin du if */
	else
		xps_dessin[0]->ps_hard_copy = FALSE;


	if (xps_dessin[0]->ps_hard_copy)
	{

		strcpy(xps_dessin[0]->plot_file,la_plot_file);	/* pour pl_newpage */
		plopen(xps_dessin[0]->plot_file);
		fprintf(xps_dessin[0]->pl_spl,"%%!PS-Adobe-2.0 EPSF-1.2\n");
		fprintf(xps_dessin[0]->pl_spl,"%%PLOT_XPS V%s\n%%%%Creator: %s\n",PS_VERSION,proprio);
		fprintf(xps_dessin[0]->pl_spl,"%%%%Title: %s\n",xps_dessin[0]->plot_file);
		fprintf(xps_dessin[0]->pl_spl,"%%%%CreationDate: %s\n",moment);
		fprintf(xps_dessin[0]->pl_spl,"%%%%Pages: (atend)\n");
		fprintf(xps_dessin[0]->pl_spl,"%%%%EndComments\n");
		fprintf(xps_dessin[0]->pl_spl,"save\n");
		fprintf(xps_dessin[0]->pl_spl,"/M { moveto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/rM { rmoveto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/L { lineto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/rL { rlineto } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/S { show } bind def\n");
		fprintf(xps_dessin[0]->pl_spl,"/GSG { gsave stroke grestore } bind def\n");
	
		fprintf(xps_dessin[0]->pl_spl,"%%%%Beginning of the flag page\n");

		/*	Le cadre de flag-page */

		traa(0.8,0.8,PL_AH);
		traa(19.5,0.,PL_RB);
		traa(0.,1.,PL_RB);
		traa(-19.5,0.,PL_RB);
		traa(0.,-1.,PL_RB);
		traa(18.5,0.,PL_RH);
		traa(0.,1.,PL_RB);
	
		traa(1.,1.15,PL_AH);

		/* le texte de la baniere */
		ps_font("Times-Roman",0.4);
		ps_chara(0.,0.,PL_RH,moment,0.4,0);
		traa(7.,1.1,PL_AH);
		ps_font("NewCenturySchlbk-BoldItalic",0.5);
		ps_chara(0.,0.,PL_RH,xps_dessin[0]->plot_file,0.5,0);
		traa(14.,1.15,PL_AH);
		ps_font("Times-Roman",0.4);
		ps_chara(0.,0.,PL_RH,proprio,0.4,0);
		traa(19.5,1.15,PL_AH);
		sprintf(page,"%2d",xps_dessin[0]->pl_ps_page);
		ps_chara(0.,0.,PL_RH,page,0.4,0);

		traa(0.,0.,PL_AH);

		fprintf(xps_dessin[0]->pl_spl,"%%End of the flag page\n");
	}
	else
	{
		fprintf(stderr,"\007PLOT_XPS: Hard Copy was not previously required\n");
		fprintf(stderr,"          No Post_script file will be produced\n");
		if (xps_dessin[ps_current_window]->ps_xps)
			fprintf(stderr,"          Only an Xwindow display will be done\n");
	} /* fin du else */

	/*
	 * Mise en place pour le dessin
	 */

	ps_origin(1.,2.,0.,0.);
	ps_font("Times-Roman",0.4);

#if XPS_DEBUG
	printf("ps_origin fait\n");
#endif

	/* Initialisation de la Bounding Box aux valeurs inverses */

	xps_dessin[0]->b_b_ll_x = UR_X;
	xps_dessin[0]->b_b_ll_y = UR_Y;
	xps_dessin[0]->b_b_ur_x = LL_X;
	xps_dessin[0]->b_b_ur_y = LL_Y;

	xps_dessin[0]->ps_show = TRUE;
	return (0);
}










