#define TMS_DEBUG 1 
/************************************************************************
*									*
*			T M S _ U N I X . C	(unix)                  *
*                                                                       *
*
* tms_unix.c programme permettant de traiter les donnees venant de
* UCLA (.po) ou de NBC (*.tms) pour les times series
*
*
*		Logiciel d'analyse dynamique de donnees
*		Neurophysiologiques (a partir de sequences
*		d'evenements dates) ou Point Process.
*
* Menu ou l'on se deplace avec les fleches ou donne une lettre
* correspondant au choix voulu.
*
*	Version C pour ULTRIX a partir de nbc_unic.c
*
*	nbc_unix.c pour le menu
*		10 Avril 1991
*		V5	1 mars 92
*		Keep file	25 Juin 1992
*
*		TMS V3.0	21 juillet 1992
*		TMS V3.1	6 octobre 1992
*		TMS V3.2	12 octobre 1992	(pb Lissajous + histogrammes de cycle)
*		TMS V3.3	15 octobre 1992	(Ajout possibilite de filtrer les donnees)
*		TMS V3.4	24 octobre 1992 (Ajout des bounding boxes a chaque
*						dessin + choix des longueur d'axe
*						apres visualisation + warning si figures
*						habituellement carrees a axes differents)
*		TMS V3.5	28 octobre 1992 (Ajout des fleches pour les Lissajous)
*						(fleches: K Pakdaman)
*		TMS V3.6	21 novembre 1992 Correction de bugs de
*							lecture de nom fichier
*		TMS V3.7	1er mai 1993	Possibilite de ne prendre q'un morceau
*						du cycle dans les Poincare
*		TMS V3.8	11 aout 1993	Correction des echelles des correlation
*		TMS V3.9	10 sept 1993	Correction des poincare cycliques
*		TMS V4.0	16 nov 1993	Chgt de philosophie du recueil des donnees
*						pour permettre l'analyse de tres longues
*						series comportant de tres nombreux
*						evenements, comme dans NBC.
*						Prise en compte du previewer PostScript
*						dans un #define.
*						Code unique pour tms et xtms
*						(choix par compilation conditionnelle)
*		TMS V4.1	25 nov 1993	Mise a niveau avec plot_xps V1.5
*		TMS V4.2	26 nov 1993	Blindage des entrees vides
*	 					et chgt des defauts + chgt axes par menu
*		TMS V4.3	 2 dec 1992	Mise en conformit avec plot_xps 1.6
*               TMS V4.4        28 jan 1994     Sauvetage des valeurs des histogrammes
*                               10 fev 1994     Version VMS (tms_vms.c) VAX (Thessalonique)
*                                               (prb avec la compilation sur VMS/AXP)
*               TMS V4.5         6 dec 1995	Correction de petits bugs
*		XTMS V5.0	9 juil 96	Reecriture de l'interface ascii en interface X
*						(Pierre Duvinage)
*		XTMS V5.1	1 aout 96	Pointage sur le bon directory de projet (JFV)
*		XTMS V6.0	18 aout 96	dessin dans la fenetre principale + correction de bugs (JFV)
*		XTMS V6.1	25 aout 96	correction de bugs (JFV)
*		XTMS V6.2	28 aout 96	amelioration de l'ergonomie (JFV)
*		XTMS V6.3	20 sept 96	amelioration de l'ergonomie (JFV)
*		XTMS V6.4	11 dec 96	amelioration de la presentation des crosscorrelations (JFV)
*		XTMS V6.5	13 mar 97	amelioration de la presentation des CTH (JFV)
*		XTMS V6.6	10 avr 97	Adaptation au chgt de plot_xps queue name (JFV)
*		XTMS V6.7	10 avr 97	Correction de bug si 1 seul event (JFV)
*		XTMS V6.8	4 sep 97	Prise en compte de .xnbcrc (JFV)
*		XTMS V6.9	10 nov 97	Prise en compte du temps de depart (JFV)
*		XTMS V6.10	14 jan 98	Recalcul des moyennes (JFV)
*		XTMS V6.11	22 jan 98	creation de .xtmsrc (JFV)
*                                               changement de curseurs (JFV)
*                                               defaults standards (JFV)
*               XTMS V6.12      11 jan 99       Fond de fenetre blanc + modifs pour Lesstif (JFV)
*               XTMS V6.12      6 oct 99        Chgt de place des helps (JFV)
*               XTMS V6.13      8 oct 99        Ajout d'un bouton de changmt de nom de fichier (JFV)
*               XTMS V6.14      17 sept 2001    Different colors when two events. (JFV)
*
*
*		Il incorpore des fonctions venant de
*
*		            TMS1, TMS2 et TMS3
*
*		Auteurs: Sylvain HANNETON et J-F VIBERT
*			Laboratoire de Physiologie
*			CHU Saint-Antoine
*			27 rue Chaligny
*			75571 Paris cedex 12
*			France
*
*		et
*			Ecole Normale Superieure (SH)
*			45 Rue d'ulm
*			75005 Paris
*
*	Modifies par
*		JF Vibert
*		B3E ESI INSERM U444
*               CHU Saint-Antoine
*               27 rue Chaligny
*               75571 Paris cedex 12
*               France
*
*	Interface X par Pierre Duvinage modifie par JF Vibert
*
*
*
*	Auteur: Dr J-F VIBERT
*		B3E ESI INSERM U444
*		CHU Saint-Antoine
*		27 rue Chaligny
*		75571 Paris cedex 12
*		France
*
*	Copyright Dr JF Vibert 
*
*	All rights reserved
*	Access limited to authorized persons	
*	This software was developped only for research purpose
*	and is not, in any way usable for commercial purpose.
*
*/
#ifdef	DOCUMENTATION

title  XTMS	Time series analysis

index	  	Time series analysis

synopsis
	
	xtms [-s|k] [-mpct] file_in[.tms|.po]

description

	TMS allows to make time series analysis from files storing events as
	point process. It can analyze the NBC cluster behavior from the NBC
	*.tms files. The default input file extension is .tms, the ouput
	file is a PostScript file.

	TMS can be compiled for use of Xwindow. It should be installed as XTMS
	It is a special version of TMS showing the output graphs during
	the analysis in a second X window. The terminal emulation window
	must be positionned at the right part of the sreen, in order to be
	visible when the graphic window will be automatically opened on the
	left side.  Otherwise, XTMS has exactly the same fonctionalities and
	interface that TMS, except the post analysis visualisation using the
	local PostScript previewier, since graphs are seen in real time.

	\nofill
	{\tt -v}: verbose switch
	{\tt -mpct}: change the size values switch
	{\tt -s|k}: keep PostScript file switch
	\fill

	The input file format is:

	{\tt
	\nofill
		1 2
		2 3
		3 3
		3 2
		3 1
		5 1
		...
	\fill
	}
	
	were the first value is the time (in increasing order), that can be
	a float or an integer. It is supposed to be given in ms.  The second
	is the event number. TMS can process simultaneously up to 10
	different events. More can be present in the file, but only 10 can
	be chosen among them, which is largely enough, since only 1 to 3 are
	relevant for each processing. Up to 30,000 such pairs can be stored and
	processed. Only the data corresponding to the chosen events are stored. 

	Start and End analysis are asked.

	Data can be filtered before processing (interval limits to be
	included or excluded).

	TMS is menu driven.

	Moving in the menu is done using the arrow down and up arrows keys
	that allow to move from field to field. <CR> key allow to activates
	the menu item in reverse video. Menu item can also be activated by
	keying the capital letter. In this case the <CR> key has not to be
	depressed.

	Results are given as PostScript graphics, either displayed using the
	local PostScript previewer (dxpsview for exemple), and/or
	automatically spooled (or if asked from the menu area).

	The main menu propose some general actions and the following submenus:

	{\tt
	\nofill
	    S Spooling or not of the PostScript file (toggle)
	    V Visualization on screen (Workstation only) 
	    P Send to printer 
	        SUB-MENUS    
	    A f(Time) drawings 
	    B f(Rank) drawings 
	    C Correlation analysis 
	    D Poincare Maps 
	    E Inter and Peri Event Analysis 
	    F 3D drawings 
	    G Phase map analysis [dx/dt = f(x)]

	    Q Quit TMS 
	\fill
	}
	    And the submenus contain the following analyses:

	\begin{verbatim}

	        SUB-MENU A: f(Time) drawings 
		
 	    A Instantaneous rate = f(time)    (1 event)    [line drawing]
	    B Instantaneous rate = f(time)    (1 event)    [dots drawing]
	    C Instantaneous rate = f(time)    (2 events)   [dots drawing]
 	    D Rate as funtion of running time (1 event)    [histogram] 
	    E Interval = f(time)              (1 event)    [dots drawing]
	    F Interval = f(time)              (2 events)   [dots drawing]
	    G Phase = f(time)                 (2 events)   [dots drawing]
	    H Phase = f(interval)             (2 events)   [dots drawing]
		
	        SUB-MENU B: f(Rank) drawings 
		
 	    A interval = f(rank)      (1 event)       [dots drawing] 
	    B phase = f(rank)         (2 events)      [dots drawing] 
	    C power spectrum of rank  (1 event)       [histogram]
		
	        SUB-MENU C: Correlation analysis
		
 	    A autocorrelation          (1 event)       [histogram] 
	    B crosscorrelation         (2 events)      [histogram] 
		
	        SUB-MENU D: Poincare Maps
		
	    A Poincare map of intervals     (1 event)       [dot drawing] 
	    B 2 Poincare maps of intervals  (2 events)      [dot drawing] 
	    C Poincare map of phases        (2 events)      [dot drawing] 
	    D Poincare map of cyclic data   (2 or 3 events) [dot drawing] 
		
	        SUB-MENU E: Inter and Peri Event Analysis
		
	    A Inter event histogram      (1 event)       [histogram] 
	    B Dot display                (2 events)      [dot drawing] 
	    C Lissajous of cyclic data   (2 events)      [dot drawing] 
	    D Post event histogram       (2 or 3 events) [histogram] 
	    E Post event interv pooled   (2 or 3 events) [dot drawing] 
	    F Post event rate pooled     (2 or 3 events) [dot drawing] 
	    G Post event phase pooled    (3 events)      [dot drawing] 
	    H Scatter diagram            (3 events)      [dot drawing] 
		
	        SUB-MENU F: 3D drawings
		
 	    A Interval I1 = f(I2,I3)       (1 event)        [dot drawing] 
 	    B Phase P1 = f(P2,P3)          (2 events)       [dot drawing] 
		
	        SUB-MENU G: Phase map analysis [dx/dt = f(x)]
		
 	    A d(interval)/dt = f(Interval)       (1 event)  [dot drawing] 
 	    B exp(d(interval)/dt) = f(Interval)  (1 event)  [dot drawing] 
 	    C d(phase)/dt = f(phase)             (2 event)  [dot drawing]
		
	\end{verbatim}

	For each analysis, specific question are asked.  All questions admit
	a default answer indicated at the prompt.
	
	An immediate <CR> sends this default value.

	A movement with one of the editing keys (see below) allows to edit
	this default value in order to change it.

	Any other new entry annulates the default value and takes the new
	key strokes as the beginning of a new value.

	Editing keys can be used at any moment. The arrow keys have their
	natural role: left arrow to go backward character by character,
	rigtharrow to go forward character by character. The special
	VT100/VT2xx/VT3xx PFn keys are significant:

	\nofill
	LEFT ARROW:     backward one character
	RIGHT ARROW:    forward one character
	PF1:    backward to beginning of line
	PF2:    backward one word
	PF3:    forward one word
	PF4:    forward to end of line
	UP ARROW:       backward in a 10 item ring buffer history
	DOWN ARROW:     forward in a 10 item ring buffer history
	CTRL R or Find:  considers  the string already  typed  as the beginning of a memorized string, and recalls the full string if any, or beeps if not.
	?  or Help:  gives an help text in reverse video characters, generaly at the top  of  the  screen,  or a fugace message {\bf No Help} at the current position.
	CTRL G: aborts the entry and calls back the default value.
	CTRL L: repaints the line if an external message is received.
	\fill

	PF2 and PF3 are only valid when entering strings.

	Literal characters are refused (with beep) when integer or
	floating values are awaited (eE are permitted for floats).

	Integer or float values are refused if they are external to
	given limits. If so, a message is given with the authorized limits,
	and a replacement value awaited.

	The  10 last values entered for int, float  or  string  (not
	for single characters) are memorized in 3 different ring buffers.

	It is possible to read other exotic (personal data file -.po); the
	last extension file type used is stored in the file .tmsrc 

Files

	Input: file_in.tms .tmsrc

	Output: file_in.ps .tmsrc
	
author

	Jean-Franois Vibert\\
	B3E, INSERM U444, Universite Paris 6
	
	Sylvain Hanneton\\
	Ecole Normale Superieure
	Paris

see also


#endif

/*
**	STANDART INCLUDE FILES
*/

#include  <stdio.h>	/* standard I/O files */
#include  <signal.h>
#include  <stdlib.h>
#include  <string.h>
#include  <ctype.h>	/* macro et globales pour fonctions classification de
			   caracteres */
#include  <sys/types.h>
#include  <math.h>

/*
**	PLOT INCLUDE FILES
*/

#include "find_files.h"

/*
**	Motif INCLUDE FILES
*/

#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#if (XmVersion >= 1002)
#	include <Xm/RepType.h>
#endif
#include <Xm/MwmUtil.h>

/*
**	Bx FILES
*/

extern void RegisterBxConverters();
extern void BxSetSensitiveCB ();
extern void BxSetInsensitiveCB ();

/*
**
*/

#define	TRUE	1
#define FALSE	0

#define WRK_FILE ""
#define ALT_FILE "tms.tms"

#define ESC	0x1b
#define RETURN	0x0d

#define PRINTER_PS  "postscript"
#define PRINTED_EXT ".ps"              /* extension of printed files */

#ifdef __osf__
#	define	PREVIEWER_PS	"dxvdoc -f ps "
#endif
#ifdef ultrix
#	define	PREVIEWER_PS	"dxpsview -geometry 700x800 "
#endif
#ifdef gnu
#	define	PREVIEWER_PS	"ghostscript "
#endif
#ifndef PREVIEWER_PS
#	define  PREVIEWER_PS    "ghostscript "
#endif

/*
**
**  Motif variables and parameters
**
*/

#define		TMS_APP_CLASS "b3e xtms"

Widget		TheAppShell;
Widget		main_TopLevelShell;
Widget		parameters_TopLevelShell;
Widget		fileSelection_TopLevelShell;
Widget		filter_TopLevelShell;
Widget		axis_TopLevelShell;
Widget		histogram_TopLevelShell;
Widget		arrows_TopLevelShell;
Widget		poincare_TopLevelShell;
Widget		fileSelectionBox;
extern Widget	fctpara_TopLevelShell;
extern Widget	optionsOutput_ToggleButton;
Widget		threeD_TopLevelShell;
extern Widget	print_DialogShell;
extern Widget	ThePopupBox;

/*
**	PERSONAL INCLUDE FILES
*/

#define _TMS_MAIN
#include "tms_unix.h"
#undef _TMS_MAIN

/************************** VARIABLES GLOBALES DU MENU TMS *****************/


/* definition et initialisation des variables */
Boolean FirstUse = True,
        Window_open = False;

int	champ_deb = 10,	/* premiere ligne du menu */
	col_deb = 15,	/* premiere colonne du menu*/
	champ_fin,	/* derniere ligne du menu */
	pas = 1,	/* nombre de ligne entre deux choix */
	spooled = TRUE,	/* flag pour spooler ou non les dessins */
        largeur_pixmap = 600,
        hauteur_pixmap = 300,
        magnify = 100,
	champ ;		/* ligne courante */

char	work_file[250],
	work_file_type[250],
	prompt[80],
	on_garde[10];

FILE	*menu_def,	/* fichier du fichier par defaut! */
	*histo_file;	/* fichier des histogrammes en output ascii */
	
void	print_file(Widget widget, XtPointer client_data, XtPointer call_data);
void	init_directories();
void    Page_de_garde(void);
void	Logo_XTMS(void);
void	read_user_defaults(void);


/************************** Main ****************************/

int main(int argc,char **argv)
{
/*
**	Variables
*/
	/*** Motif Interface variables ***/

	XtAppContext	app;
	Arg		args[256];
	Cardinal	ac;
	Widget		parametersMain_Form;
	Widget		filterMain_Form;
	Widget          axisMain_Form;
	Widget		arrowsMain_Form;
	Widget		histogramMain_Form;
	Widget		poincareMain_Form;
	Widget		threeDMain_Form;
	Boolean		FileNameGiven;
	char		SprintfChar[250];
	char		SomeChar[250];

	/*** Divers ***/



	int	i,
		verbose;
		
	char	*f_n_l;

/*
**	Code
*/

	/*** Global variables initialisation ***/

	driver		 = 's';
	vient_de_nbc	 = FALSE;
	already_printed  = True;
	filtre		 = FALSE;
	inside		 = TRUE;
	use_defaults	 = TRUE;
	nb_demi_pages	 = 0;
	nb_de_types	 = 0;
	paper		 = 30.;
	long_X_axis	 = LONG_X;
	long_Y_axis	 = LONG_Y;
	horizontal_scale = 1.;
	vertical_scale	 = 1.;
	temps_max	 = 0.;
	user_order	 = 1;
	user_post_synaptic = post_synaptic = 1;
	user_pre_synaptic = pre_synaptic = 2;
	user_cycle_marker = cycle_marker = 4;
	user_binwidth_AC = 0.001;
	user_binwidth_CC = 0.001;
	user_binwidth_IS = 0.001;
	user_duration_AC = 0.05;
	user_duration_CC = 0.025;
	user_duration_IS = 0.1;

	strcpy (filename,"");
	
	/*** Initialisation of Motif interface ***/

	tms_MainInitMenuVar();
	init_directories();

	FileNameGiven = False;
	
	/*
	* The applicationShell is created as an unrealized
	* parent for multiple topLevelShells.  The topLevelShells
	* are created as popup children of the applicationShell.
	* This is a recommendation of Paul Asente & Ralph Swick in
	* _X_Window_System_Toolkit_ p. 677.
	*/
	
	TheAppShell = XtVaAppInitialize ( &app, TMS_APP_CLASS, NULL, 0, 
#ifndef XlibSpecificationRelease
	(Cardinal *) &argc, 
#else
#if (XlibSpecificationRelease>=5)
	&argc, 
#else
	(Cardinal *) &argc, 
#endif
#endif
	argv, NULL, 
	NULL );
    
	RegisterBxConverters(app);
#if (XmVersion >= 1002) 
	XmRepTypeInstallTearOffModelConverter();
#endif


        magnify = 100;
	strcpy(file,"");
	verbose = FALSE;
	if (argc > 1)
	{
		for (i = 1; i < argc;i++)
		{
			if (argv[i][0] == '?')
			{
				printf("%s [-v] [-mpct] [-k] [file]\n",argv[0]);
				printf("\t-v:\tverbose\n");
				printf("\t-mpct:\tmagnification in percentage (default 100%)\n");
				printf("\t-k:\tno spool of PS file\n");
				exit(1);
			}
			if (argv[i][0] == '-')
			{
				switch (tolower(argv[i][1]))
				{
					case 'v':
						verbose = TRUE;
						break;
					case 'm':
						magnify = atoi(&(argv[i][2]));
						break;
					case 's':
					case 'k':
						spooled = FALSE;
						break;
					default:
						printf("%s --> -%c illegal switch\n",argv[0],argv[i][1]);
						printf("%s [-v] [-mpct] [-k] [file]\n",argv[0]);
						printf("\t-v:\tverbose\n");
						printf("\t-mpct:\tmagnification in percentage (default 100%)\n");
						printf("\t-k:\tno spool of PS file\n");
					}
			}
			else
			{
			      if (argv[i][0] == '/')
			      {
				  strcpy(SomeChar,get_simulation_name());
				  f_n_l = strrchr(SomeChar,'/');
				  if (f_n_l)
				  {
					strcpy(filename,(f_n_l + sizeof(char)));
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,get_simulation_name());
					strcpy(file,get_simulation_name());		/* Nom generique */
					strcpy(filename,get_tms_name());
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory,get_dir_name());
				  }
				  else
				  {
					strcpy(filename,SomeChar);
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename); /* Nom generique */
					strcat(filename,".tms");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory, ".");
				  }
			      }
			      else
			      {
				    strcpy(SomeChar,argv[i]);
				    f_n_l = strrchr(SomeChar,'/');
				    if (f_n_l)
				    {
					strcpy(filename,(f_n_l + sizeof(char)));
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename);		/* Nom generique */
					strcat(filename,".tms");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					f_n_l = strrchr(SomeChar,'/');
					if (f_n_l)
					{
						f_n_l++;
						*f_n_l = 0;
						strcpy(project_directory, SomeChar);
						f_n_l = strrchr(SomeChar,'_');
					} /* fin du if */
				    }
				    else
				    {
					strcpy(filename,SomeChar);
					f_n_l = strrchr(filename,'.');
					if (f_n_l)
					{
						*f_n_l = 0; /* Extension removed */
					}
					strcpy(scriptname,filename);
					strcpy(file,filename); /* Nom generique */
					strcat(filename,".tms");
					strcat(scriptname,".ps");
					FileNameGiven = True;
					strcpy(project_directory, ".");
				    }
			      }
			}
		}
	}
#if TMS_DEBUG
	printf("File name: %s -> %s\nCurrent directory: %s\nProject name: %s\n",
	       filename, scriptname, project_directory, file);

	/*** Create classes and widgets used in this program. ***/


    printf("TMS_UNIX: display: %x; &larg: %x; &haut: %x\n",
	   XtDisplay(TheAppShell),
	   &largeur_pixmap,
	   &hauteur_pixmap);
#endif

    ps_get_pixmap_size(XtDisplay(TheAppShell), &largeur_pixmap, &hauteur_pixmap, 210, 145, magnify);

#if TMS_DEBUG
    printf("TMS_UNIX: larg: %d; haut: %d\n",
	   largeur_pixmap,
	   hauteur_pixmap);
#endif

	strcpy (SprintfChar, VERSION);

    /*** Create classes and widgets used in this program. ***/

	sprintf (SomeChar,"%s: %s (%s)", VERSION, filename, scriptname);
	ac = 0;
	XtSetArg(args[ac], XmNtitle, SomeChar); ac++;
	XtSetArg(args[ac], XmNiconName, SprintfChar); ac++;
	XtSetArg(args[ac], XmNx, 100); ac++;
	XtSetArg(args[ac], XmNy, 100); ac++;
	XtSetArg(args[ac], XmNwidth, largeur_pixmap); ac++;
	XtSetArg(args[ac], XmNheight, hauteur_pixmap); ac++;
	XtSetArg(args[ac], XmNbackground, WhitePixel(XtDisplay(TheAppShell),DefaultScreen(XtDisplay(TheAppShell)))); ac++;
	XtSetArg(args[ac], XmNforeground, BlackPixel(XtDisplay(TheAppShell),DefaultScreen(XtDisplay(TheAppShell)))); ac++;
 	main_TopLevelShell = XtCreatePopupShell("main_TopLevelShell",
						topLevelShellWidgetClass,
						TheAppShell,	
						args, 
						ac);
	main_Form = (Widget) Createmain_Form(main_TopLevelShell, FileNameGiven);
	XtManageChild(main_Form);
    
	ac = 0;
	sprintf (SprintfChar,"%s - Parameters", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Parameters"); ac++;
	XtSetArg(args[ac], XmNx, 100); ac++;
	XtSetArg(args[ac], XmNy, 100); ac++;
	XtSetArg(args[ac], XmNwidth, 730); ac++;
	XtSetArg(args[ac], XmNheight, 610); ac++;
	parameters_TopLevelShell = XtCreatePopupShell("parameters_TopLevelShell",
						      topLevelShellWidgetClass,
						      TheAppShell,
						      args, 
						      ac);
	parametersMain_Form = (Widget)CreateparametersMain_Form(parameters_TopLevelShell);
	XtManageChild(parametersMain_Form);
	
	ac = 0;
	sprintf (SprintfChar,"%s - File selection", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "File selection"); ac++;
	XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
	XtSetArg(args[ac], XmNwidth, 430); ac++;
	XtSetArg(args[ac], XmNheight, 400); ac++;
	fileSelection_TopLevelShell = XtCreatePopupShell("fileSelection_TopLevelShell",
							 topLevelShellWidgetClass,
							 TheAppShell,
							 args, 
							 ac);
	fileSelectionBox = (Widget)CreatefileSelectionBox(fileSelection_TopLevelShell);
	XtManageChild(fileSelectionBox);

	ac = 0;
	sprintf (SprintfChar,"%s - Filters", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Filters"); ac++;
	XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
	XtSetArg(args[ac], XmNwidth, 800); ac++;
	XtSetArg(args[ac], XmNheight, 500); ac++;
	filter_TopLevelShell = XtCreatePopupShell("filter_TopLevelShell",
						  topLevelShellWidgetClass,
						  TheAppShell,
						  args, 
						  ac);
	filterMain_Form = (Widget)CreatefilterMain_Form(filter_TopLevelShell);
	XtManageChild(filterMain_Form);

	ac = 0;
	sprintf (SprintfChar,"%s - Axis options", VERSION);
        XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Axis options"); ac++;
        XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
        XtSetArg(args[ac], XmNwidth, 801); ac++;
	XtSetArg(args[ac], XmNheight, 360); ac++;
        axis_TopLevelShell = XtCreatePopupShell("axis_TopLevelShell",
						topLevelShellWidgetClass,
						TheAppShell,
					        args,
					        ac);
	axisMain_Form = (Widget)CreateaxisMain_Form(axis_TopLevelShell);
        XtManageChild(axisMain_Form);

	ac = 0;
	sprintf (SprintfChar,"%s - Arrows options", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Arrows options"); ac++;
	XtSetArg(args[ac], XmNx, 120); ac++;
	XtSetArg(args[ac], XmNy, 120); ac++;
	XtSetArg(args[ac], XmNwidth, 650); ac++;
	XtSetArg(args[ac], XmNheight, 472); ac++;
	arrows_TopLevelShell = XtCreatePopupShell("arrows_TopLevelShell",
						  topLevelShellWidgetClass,
						  TheAppShell,
						  args, 
						  ac);
	arrowsMain_Form = (Widget)CreatearrowsMain_Form(arrows_TopLevelShell);
	XtManageChild(arrowsMain_Form);
	
	ac = 0;
	sprintf (SprintfChar,"%s - Poincare\'s map options", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Poincare\'s map options"); ac++;
	XtSetArg(args[ac], XmNx, 140); ac++;
	XtSetArg(args[ac], XmNy, 140); ac++;
	XtSetArg(args[ac], XmNwidth, 700); ac++;
	XtSetArg(args[ac], XmNheight, 357); ac++;
	poincare_TopLevelShell = XtCreatePopupShell("poincare_TopLevelShell",
						    topLevelShellWidgetClass,
						    TheAppShell,
						    args, 
						    ac);
	poincareMain_Form = (Widget)CreatepoincareMain_Form(poincare_TopLevelShell);
	XtManageChild(poincareMain_Form);

	ac = 0;
	sprintf (SprintfChar,"%s - Histogram options", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNiconName, "Histogram options"); ac++;
	XtSetArg(args[ac], XmNx, 140); ac++;
	XtSetArg(args[ac], XmNy, 140); ac++;
	XtSetArg(args[ac], XmNwidth, 500); ac++;
	XtSetArg(args[ac], XmNheight, 550); ac++;
	histogram_TopLevelShell = XtCreatePopupShell("histogram_TopLevelShell",
						     topLevelShellWidgetClass,
						     TheAppShell,
						     args, 
						     ac);
	histogramMain_Form = (Widget)CreatehistogramMain_Form(histogram_TopLevelShell);
	XtManageChild(histogramMain_Form);
	
	ac = 0;
	sprintf (SprintfChar,"%s - 3D options", VERSION);
	XtSetArg(args[ac], XmNtitle, SprintfChar); ac++;
	XtSetArg(args[ac], XmNx, 140); ac++;
	XtSetArg(args[ac], XmNy, 140); ac++;
	XtSetArg(args[ac], XmNwidth, 566); ac++;
	XtSetArg(args[ac], XmNheight, 337); ac++;
	threeD_TopLevelShell = XtCreatePopupShell("threeD_TopLevelShell",
						  topLevelShellWidgetClass,
						  TheAppShell,
						  args, 
						  ac);
	threeDMain_Form = (Widget)CreatethreeDMain_Form(threeD_TopLevelShell);
	XtManageChild(threeDMain_Form);

	wait_cursor = XCreateFontCursor(XtDisplay(TheAppShell), XC_watch);
    
	 /*** Display the main window ***/

	XtPopup(XtParent(main_Form), XtGrabNone);

	already_printed = True;

	read_user_defaults();

	if (FileNameGiven)
	{
	  BxPopupCB(XtParent(parametersMain_Form),(XtPointer)"parameters_TopLevelShell",NULL);
	  BxSetSensitiveCB(main_Form,"mainClose_PushButton, mainSavePS_PushButton,\
                           mainPrint_PushButton, mainOperations_CascadeButton, mainOptions_CascadeButton", NULL);

	}
	XtAppMainLoop(app);
	return (0);
} /* findu main */


/*
***************************************************************************************************8
*/
void print_file(Widget widget, XtPointer client_data, XtPointer call_data)
{

/*
**      Variables
*/
        char    PrintType;
	char SomeChar[250];
/*
**      Code
*/

        PrintType = (char) client_data;
#if DEBUG
	printf("PrintType: %c\n",PrintType);
#endif
        if (!already_printed)
        {
                ps_endplot_on(PrintType) ;
                already_printed = TRUE;
	} /* fin du if */

        strcat(scriptname,"+");

        temps_max = temps[nb_donnees-1] ;
        if (arrivee > temps_max) arrivee = temps_max ;
        if (depart < temps[0]) depart=temps[0] ;

	sprintf (SomeChar,"%s: %s (%s)", VERSION, filename, scriptname);
	XtVaSetValues(main_TopLevelShell, XmNtitle, SomeChar,NULL); 
	ps_re_init_hard_copy(scriptname);
	already_printed = False;
	Page_de_garde();
} /* fin du void */

void	fin(void)
{
	FILE *p_rc;
        char *home;
	char buffer[256];

   /*********************
   * fermeture fichiers *
   *********************/
	if (!already_printed)
		ps_endplot_on(driver) ;

  /*******************************
   * sauvegarde fichiers .xtmsrc *
   ******************************/

	home = getenv("HOME");
	if(home)
	{
		strcpy(buffer,home);
		strcat(buffer,"/.xtmsrc");
	}
	else
	{
		strcpy(buffer,"./.xtmsrc");
	}

	if ((p_rc = fopen(buffer,"w")) == NULL)
	{
	        printf("Impossible to create the %s user\'s default file, sorry\n",buffer);
		return;
	} /* fin du if */

	/* valeurs par defaut */
	fprintf(p_rc,"Order: %d\n", user_order);
	fprintf(p_rc,"CycleMarker: %d\n", user_cycle_marker);
	fprintf(p_rc,"PreSynaptic: %d\n", user_pre_synaptic);
	fprintf(p_rc,"PostSynaptic: %d\n", user_post_synaptic);
	fprintf(p_rc,"LongXAxis: %f\n", long_X_axis);
	fprintf(p_rc,"LongYAxis: %f\n", long_Y_axis);
	fprintf(p_rc,"HorizontalScale: %f\n", horizontal_scale);
	fprintf(p_rc,"VertcalScale: %f\n", vertical_scale);
	fprintf(p_rc,"BinWidth: %f\n", user_binwidth);
	fprintf(p_rc,"BinWidthAC: %f\n", user_binwidth_AC);
	fprintf(p_rc,"BinWidthCC: %f\n", user_binwidth_CC);
	fprintf(p_rc,"BinWidthIS: %f\n", user_binwidth_IS);
	fprintf(p_rc,"BinWidthPST: %f\n", user_binwidth_PST);
	fprintf(p_rc,"Duration: %f\n", user_duration);
	fprintf(p_rc,"DurationAC: %f\n", user_duration_AC);
	fprintf(p_rc,"DurationCC: %f\n", user_duration_CC);
	fprintf(p_rc,"DurationIS: %f\n", user_duration_IS);
	fprintf(p_rc,"DurationPST: %f\n", user_duration_PST);
	fprintf(p_rc,"DurationCycle: %f\n", duree_cycle);

        fclose(p_rc);

	XFreeCursor( XtDisplay(TheAppShell), wait_cursor);
}

void read_user_defaults(void)
{
	FILE *p_rc;
        char *home;
	char buffer[256];
	char parametre[80];
	char value[40];

  /*******************************
   * sauvegarde fichiers .xtmsrc *
   ******************************/

	home = getenv("HOME");
	if(home)
	{
		strcpy(buffer,home);
		strcat(buffer,"/.xtmsrc");
	}
	else
	{
		strcpy(buffer,"./.xtmsrc");
	}

	if ((p_rc = fopen(buffer,"r")) == NULL)
	{
	        printf("Impossible to read the %s user\'s default file, sorry\n",buffer);
		return;
	} /* fin du if */

	/* valeurs par defaut */

	while(fgets(buffer,256,p_rc))
	{
	    sscanf(buffer,"%s %s",parametre,value);

	    if (!strcmp(parametre,"Order:"))		
	    {
		sscanf(value,"%d",&user_order);
		break;
	    }
	    if (!strcmp(parametre,"Order:"))
	    {
	        sscanf(value,"%d", &user_order);
		break;
	    }
	    if (!strcmp(parametre,"CycleMarker:"))
	    {
	        sscanf(value,"%d", &user_cycle_marker);
		break;
	    }
	    if (!strcmp(parametre,"PreSynaptic:"))
	    {
	        sscanf(value,"%d", &user_pre_synaptic);
		break;
	    }
	    if (!strcmp(parametre,"PostSynaptic:"))
	    {
	        sscanf(value,"%d", &user_post_synaptic);
		break;
	    }
	    if (!strcmp(parametre,"LongXAxis:"))
	    {
	        sscanf(value,"%f", &long_X_axis);
		break;
	    }
	    if (!strcmp(parametre,"LongYAxis:"))
	    {
	        sscanf(value,"%f", &long_Y_axis);
		break;
	    }
	    if (!strcmp(parametre,"HorizontalScale:"))
	    {
	        sscanf(value,"%f", &horizontal_scale);
		break;
	    }
	    if (!strcmp(parametre,"VertcalScale:"))
	    {
	        sscanf(value,"%f", &vertical_scale);
		break;
	    }
	    if (!strcmp(parametre,"BinWidth:"))
	    {
	        sscanf(value,"%f", &user_binwidth);
		break;
	    }
	    if (!strcmp(parametre,"BinWidthAC:"))
	    {
	        sscanf(value,"%f", &user_binwidth_AC);
		break;
	    }
	    if (!strcmp(parametre,"BinWidthCC:"))
	    {
	        sscanf(value,"%f", &user_binwidth_CC);
		break;
	    }
	    if (!strcmp(parametre,"BinWidthIS:"))
	    {
	        sscanf(value,"%f", &user_binwidth_IS);
		break;
	    }
	    if (!strcmp(parametre,"BinWidthPST:"))
	    {
	        sscanf(value,"%f", &user_binwidth_PST);
		break;
	    }
	    if (!strcmp(parametre,"Duration:"))
	    {
	        sscanf(value,"%f", &user_duration);
		break;
	    }
	    if (!strcmp(parametre,"DurationAC:"))
	    {
	        sscanf(value,"%f", &user_duration_AC);
		break;
	    }
	    if (!strcmp(parametre,"DurationCC:"))
	    {
	        sscanf(value,"%f", &user_duration_CC);
		break;
	    }
	    if (!strcmp(parametre,"DurationIS:"))
	    {
	        sscanf(value,"%f", &user_duration_IS);
		break;
	    }
	    if (!strcmp(parametre,"DurationPST:"))
	    {
	        sscanf(value,"%f", &user_duration_PST);
		break;
	    }
	    if (!strcmp(parametre,"DurationCycle:"))
	    {
	        sscanf(value,"%f", &duree_cycle);
		break;
	    }
	}
        fclose(p_rc);
}

void	no_op()
{
	return;
}

/************************** TMS General **********************/


/* place le curseur dans la zone message et fait clignoter
la legende 'message, */


void Message(char *chaine)
{
	return;
}

/* fin du Message */

/*
 * annonce le trace d'un dessin postscipt
 */

void Message_plotting()
{
	ps_code("%% *** DEBUT DU GRAPHE ***\n");
} /* fin du Message */

void Fin_plotting(int *data,	/* tableau a sauver */
		  int nb,	/* nb d'elements */
		  char *ext)	/* extension du fichier */
{
/*
**	Variables
*/
	char	out_file[250];
	char	SomeChar[250];
	int	i;
	Boolean SaveOutput;
/*
**	Code
*/

	ps_code("%% *** FIN DU GRAPHE ***\n");
	ps_flush();
	if (data)
	{
		XtVaGetValues (optionsOutput_ToggleButton,
			       XmNset, &SaveOutput,
			       NULL);
		if (SaveOutput)
		{
			strcpy(out_file,file);
			strcat(out_file,ext);
			if ((histo_file = fopen(out_file,"w")) == NULL)
			{
				sprintf (SomeChar, "Impossible to open %s, sorry\n",out_file);
				DispMessageDialog (fctpara_TopLevelShell,
						   SomeChar,
						   "Error : datas not saved",
						   XmDIALOG_ERROR);
			}
			else
			{
				for(i = 0; i < nb; i++)
					fprintf(histo_file,"%d\n",data[i]);
				fclose(histo_file);
			}
		} /* fin du if */
	} /* fin du if */
} /* fin du Fin */

/*
 * decide si il faut ou non tourner la page
 */

void tourne_page()
{
  /* d'abord sur le papier, pas sur l ecran, avec 2 dessins par page */
	ps_no_display(ps_current_window);
	nb_demi_pages = nb_demi_pages + 1 ;
	if ( nb_demi_pages  == 2)
	{
		ps_newpage() ;
		ps_home();
		ps_origin(1.5,13.,0.,0.);
		ps_home();
		nb_demi_pages = 0 ;
	} /* fin du if */
	else
	{
		ps_origin(0.,-12.,0.,0.) ;
		ps_home();
	} /* fin du else */
	ps_display(ps_current_window);
	/* puis sur l ecran, pas sur le papier, avec 1 dessin par page */ 
	ps_no_print();
	ps_newpage();
	ps_home();
	ps_origin(1.5,1.,0.,0.);
	ps_home();
	ps_print();
} /* fin du tourne */



void bounding_box(int nb_axes) /* nb d'axes sur X */
{
	float	bbllx,
		bblly,
		bburx,
		bbury;

	bbllx = X_PAS;
	if (nb_demi_pages)	/* On est en bas de la page */
	{
		bblly = 1.8 * X_PAS;
	} /* fin du if */
	else			/* on est en haut de la page */
	{
		bblly = 13.8 * X_PAS;
	} /* fin du else */
	bburx = bbllx +
		((1.5 + long_X_axis + ((nb_axes - 1)?2.:0.)) * X_PAS);
	bbury = bblly +
		((1.7 + long_Y_axis) * X_PAS);
	sprintf(buffer,"%%%%BoundingBox: %.1f %.1f %.1f %.1f\n",
		bbllx, bblly, bburx, bbury);
	ps_code(buffer);
} /* fin du int */


/*
 * Imprime la page de garde du dossier
 */
void Page_de_garde()
{
/*
**	Variables
*/
	char	titre[250];
	int	i ;
/*
**	Code
*/
	ps_no_display(ps_current_window);
	nb_demi_pages = 0 ;

	ps_origin(1.5,13.,0.,0.);
	sprintf(buffer,"%s",VERSION);

	ps_writeah(1.,11.,buffer,1.0);
	ps_writeah(0.5,10.,"Time Series Analysis of Point Process",.3) ;

	if (filtre)
	{
		for (i = 0; i < nb_de_types; i++)
		{
			if (event[i].filtre)
			{
				sprintf(titre,
					"Filtered data for event %s (%d)",
					event[i].nom,
					event[i].numero);
				ps_writeah(1.,(7.5-0.5*i),titre,.5);
				sprintf(titre,
					"Intervals %s between %.3f and %.3f s",
					((event[i].inside)?"kept":"rejected"),
					event[i].max_interval_clip,
					event[i].min_interval_clip);
				ps_writeah(1.,(7.-0.5*i),titre,.5);
			} /* fin du if */
		} /* fin du for */
	} /* fin du if */
	else
	{
		sprintf(titre,"Unfiltered data");
		ps_writeah(1.,7.5,titre,.5);
	} /* fin du if */
	sprintf(titre,"Number of analyzed events: %d",
		nb_donnees);
	ps_writeah(1.,6.5,titre,.5);
	sprintf(titre,"Analysis duration: %.3f secondes",
		arrivee-depart) ;	
	ps_writeah(1.,6.,titre,.5);

	ps_display(ps_current_window);

} /* fin du Page_de_garde */

/*
 * Ouvre le dessin et imprime la page de garde du dossier sur le display
 */
void Logo_XTMS()
{
/*
**	Variables
*/
	Display *TheDisplay;
	char	titre[256];
	int	i ;
/*
**	Code
*/
	TheDisplay = XtDisplay (main_Form);
	already_printed = FALSE;
#ifdef DEBUG
	printf("Avant ps_iniplot_on\n");
#endif
	if (!Window_open)
	{
	  ps_iniplot_on(TheDisplay, XtWindow(main_TopLevelShell), XtWindow(main_Form), scriptname, 645, 35, 210, 145, magnify, VERSION);
	  Window_open = True;
	}
#ifdef DEBUG
	printf("Apres ps_iniplot_on\n");
#endif
	ps_no_watch_progress();
	nb_demi_pages = 0 ;

	ps_no_print();
	ps_newpage();
	ps_origin(1.5,0.,0.,0.);
	sprintf(buffer,"%s",VERSION);
	ps_writeah(1.,3.,buffer,3.0);
	ps_gray(0.5);
	ps_writeah(0.5,2.,"Time Series Analysis of Point Process",1.);
	ps_gray(0.);
	ps_flush();
	ps_print();

} /* fin du Logo_XTMS */


/*
 * Calcule la frequence instantanee pour l'ensemble des evenements et filtre
 * s'il y a lieu
 */
   
int compute_rate(int event)
{
	int	i = 0,
		indice = 0;
	float	temps_precedent = 0.;
	float   inst_rate_prec = 1.;
	char	SomeChar[250];
	Boolean CloseWin;
/*
**	Code
*/
	CloseWin = False;
	while (i < nb_donnees)
	{
		if ( type[i] == event)
		{
			if ((temps[i]-temps_precedent) == 0.)
			{
				sprintf (SomeChar, "Twice the same event at data %d",i);
				DispMessageDialog (TheAppShell,
						   SomeChar,
						   "Computation of rate",
						   XmDIALOG_INFORMATION	);	
			/*
				DispPopupDialog (TheAppShell,
						 SomeChar,
						 "Please wait." );
				CloseWin = True;
			 */
				inst_rate[i] = inst_rate_prec ;
			} /* fin du if */
			else
			{
				inst_rate[i] = 1./(temps[i] - temps_precedent) ;
				temps_precedent = temps[i] ;
				inst_rate_prec = inst_rate[i];
			} /* fin du else */
			indice++;
		} /* fin du if */
		i++;
	} /* fin du while */
	return(indice) ;
	if (CloseWin)
	{
		XtPopdown (XtParent(ThePopupBox));
	}

} /* fin du compute_rate */


/*
 * Calcule la phase pour l'ensemble des evenements 
 */
   
int compute_phase(int event_1, int event_2)
{
	int	i = 0,
		j,
		indice = 0;
	float	temps_reference = 0.;

	while (i < nb_donnees)
	{
		if ( type[i] == event_1)
		{
			temps_reference = temps[i];
			for (j = i+1; j < nb_donnees; j++)
			{
				if ( type[j] == event_2)
				{
					phase_interval[j] = temps[j] - temps_reference;
					indice++;
				}
				if (type[j] == event_1)
					break;
			} /* fin du for */
		}
		i++;
	} /* fin du while */
	return(indice) ;
} /* fin du  */

/*
 * Calcule l'intervalle pour l'ensemble des evenements 
 */
   
int compute_interval(int event)
{
	int	i = 0,
		indice = 0;
	float	temps_precedent = 0.;
	float   inst_interval_prec = 1.;
	char	SomeChar[50];
	Boolean CloseWin;
/*
**	Code
*/
	CloseWin = False;
	while (i < nb_donnees)
	{
		if ( type[i] == event)
		{
			if ((temps[i]-temps_precedent) == 0.)
			{
				sprintf (SomeChar, "Twice the same event at data %d",i);
				DispMessageDialog (TheAppShell,
						   SomeChar,
						   "Computation of interval",
						   XmDIALOG_INFORMATION	);	
				/*
				DispPopupDialog (TheAppShell,
						 SomeChar,
						 "Please wait." );
				CloseWin = True;
				*/
				inst_interval[i] = inst_interval_prec ;
			} /* fin du if */
			else
			{
				inst_interval[i] = (temps[i] - temps_precedent) ;
				temps_precedent = temps[i] ;
				inst_interval_prec = inst_interval[i];
			} /* fin du else */
			indice++;
		} /* fin du if */
		i++;
	} /* fin du while */
	return(indice) ;
	if (CloseWin)
	{
		XtPopdown (XtParent(ThePopupBox));
	}

} /* fin du compute_interval */

int filtre_les_data(int nombre_avant, int event)
{
	int	i, j;

	j = 0;
	for (i = 0; i < nombre_avant; i++)
	{
		if (type[i] == event)
		{
			if (inside)
			{
				if ((inst_interval[i] > min_interval_clip)
				    &&
				    (inst_interval[i] < max_interval_clip))
				{
					temps[j] = temps[i];
					type[j] = type[i];
					inst_interval[j] = inst_interval[i];
					phase_interval[j] = phase_interval[i];
					inst_rate[j] = inst_rate[i];
					j++;
				} /* fin du if */
			} /* fin du if */
			else
			{
				if ((inst_interval[i] < min_interval_clip)
				    ||
				    (inst_interval[i] > max_interval_clip))
				{
					temps[j] = temps[i];
					type[j] = type[i];
					inst_interval[j] = inst_interval[i];
					phase_interval[j] = phase_interval[i];
					inst_rate[j] = inst_rate[i];
					j++;
				} /* fin du if */
			} /* fin du else */
		} /* fin du if */
		else
		{
			temps[j] = temps[i];
			type[j] = type[i];
			inst_interval[j] = inst_interval[i];
			phase_interval[j] = phase_interval[i];
			inst_rate[j] = inst_rate[i];
			j++;
		} /* fin du else */
	} /* fin du for */
	for (i = j; i < nombre_avant; i++)
	{
			temps[i] = 0.;
			type[i] = 0;
			inst_interval[i] = 0;
			phase_interval[i] = 0;
			inst_rate[i] = 0;
	} /* fin du for */
	return(j);
} /* fin du int */

   /**************************
   * retourne les n premiers *
   * pics d'une courbe       *
   **************************/

void get_pic(float value[1000],
	     int nb_value,	/* nombre de valeurs          */
	     int pic[100],	/* indices designant les pics */
	     int nb_pic)	/* nombre de pics a retourner */
{
	/* declarations  */
	int	indice = 0 ,
		picinter[1000] ,
		i,j,k,
		maxpic = 0 ;

	float	maxvalue = 0.,
		valueinter[1000];

	/*initialisations */
	for (i=0;i < (nb_value-1);i++)
	{
		picinter[i] = 0 ;
		valueinter[i]= value[i] ;
	}
	for (i=1;i < (nb_value - 1);i++)
	{
		if ((value[i] > value[i-1] ) && ( value[i] > value[i+1] ))
		{
			indice = indice + 1 ;
			picinter[indice] = i ;
		} /* fin du if */
	}
	for (i=0; i < nb_pic ;i++)
	{
		/* recherche du maximum */
		for (j=1; j < (nb_value - 1) ; j++)
		{
			k = picinter[j] ;
			if ((k != 0) && (valueinter[k] > maxvalue))
			{
				maxvalue = valueinter[k] ;
				maxpic   = k ;
			} /* fin du if */

		} /* fin du for */

		pic[i] = maxpic ;
		valueinter[maxpic] = 0.;
		maxvalue = 0.;
		
	} /* fin du for */
}

   /**************************************
   * calcul la valeur absolue d'un float *
   **************************************/
   
float float_abs(float f)
{
	if (f < 0.)
		return(-f) ;
	else 
		return(f);

} /* fin du float */

   /*******************
   * arrondi un float *
   *******************/

int arrondi(float f)
{
	int inter;

	inter = f ;
	if ((f - inter) > .5) 
		inter = inter + 1 ;
	return(inter);
	
} /* fin du arrondi */

   /*************
   *  ps_line *
   *************/

void ps_line(float xdata[MAX_DATA],
	     float ydata[MAX_DATA],
	     int nb_val,
	     float start,
	     float stop,
	     float tx,
	     float ty)
{
	int i;
	float px,py,
	      maxx,minx,
	      maxy,miny;

   /* calcul des maxi et mini */
	minx = miny = maxx = maxy = 0. ;
	i = 0 ;
	while ( i < (nb_val-1) )
	{
		if (ydata[i] > maxy) maxy = ydata[i] ;
		if (ydata[i] < miny) miny = ydata[i] ;
		i++ ;
	} /* fin du while */
	
   /* changement d'echelle */
	maxx = stop ;
	minx = start ;
	px = tx / (stop - start) ;
	py = ty / ( maxy-miny) ;

   /* trace de la courbe */
	i = 0 ;
	while ( xdata[i++] < start );
	ps_movea( (xdata[i]-minx)  * px, (ydata[i]-miny) * py);
	while (  (xdata[i] >= start) && (xdata[i] < stop) )
	{
		ps_drawa( (xdata[i]-minx) * px,(ydata[i]-miny) * py);
		i++;
	} /* fin du while */
	ps_movea(0.,0.);
	
} /* fin du ps_line */


   /*******************************************
   * Calculs statistiques: esperance,variance *
   *******************************************/

void statis_histo(float *esperanceh,
		  float *varianceh,
		  float *ecart_typeh,
		  float *esperancev,
		  float *variancev,
		  float *ecart_typev,
		  int *histo,
		  int nb_max)
{
	int i,maxy,miny,total;
	float esperance_x2h = 0.,esperance_x2v = 0.;

	*esperancev = 0.;
	*esperanceh = 0.;
	total = 0 ;
	for (i=0;i<nb_max;i++)
	{
		if (histo[i]<miny) miny=histo[i] ;
		if (histo[i]>maxy) maxy=histo[i] ;
		total += histo[i] ;
	} /* fin du for */
	if (miny>0) miny = 0;

	if (!total)
		return;
		
	for (i=0;i<nb_max;i++)
	{
		*esperanceh += (float)(histo[i]*(i+1)) ;
		esperance_x2h += (float)(histo[i]*(i+1)*(i+1));
		*esperancev += (float)(histo[i]) ;
		esperance_x2v += ((float)(histo[i])*(float)(histo[i]) );
	} /* fin du for */

	*esperanceh = *esperanceh / (float)(total) ;
	*esperancev = *esperancev / (float)(nb_max) ;
	esperance_x2h = esperance_x2h / (float)(total) ;
	esperance_x2v = esperance_x2v / (float)(nb_max) ;
	*variancev = esperance_x2v - (*esperancev * *esperancev) ;
	*varianceh = esperance_x2h - (*esperanceh * *esperanceh) ;	
	*ecart_typeh = sqrt(*varianceh) ;
	*ecart_typev = sqrt(*variancev) ;

	if (*esperancev == 0.) *esperancev = 1.;
	if (*esperanceh == 0.) *esperanceh = 1.;

} /* fin du statis_histo */


/*******************************************************************************
	Function for initializing global directory variables
*******************************************************************************/

void init_directories(void)
{
	FILE	*p_rc;
	char	*home;
	char	*tms_V8_bin;
	char	*tms_V8_help;
	char	un_buffer[256];

	tms_V8_bin = tms_V8_help = home = NULL;

	home = getenv("HOME");
	if(home)
	{
		strcpy(un_buffer,home);
		strcat(un_buffer,"/.xnbcrc");

		if (p_rc = fopen(un_buffer,"r"))
		{
			fscanf(p_rc,"%s",project_directory);
			strcpy(working_directory,project_directory);
			fclose(p_rc);
		} /* fin du if */
		else
		{
			strcpy(working_directory,".");
			strcpy(project_directory,".");
			strcat(project_directory,"/noname_nbc");
		} /* fin du else */
	}
	else
	{
		strcpy(working_directory,".");
		strcpy(project_directory,"./noname_nbc");
	}

	tms_V8_bin = getenv("XNBC_BIN_DIR_TOOLS");
	if(tms_V8_bin)
	{
		strcpy(bin_directory,tms_V8_bin);
	}
	else
	{
		strcpy(bin_directory,"/usr/local/bin/xnbc.bin/V8");
		fprintf(stderr,"XNBC_BIN_DIR_TOOLS not defined, using %s\n",
			bin_directory);
	}
	tms_V8_help = getenv("XNBC_HELP_DIR");
	if(tms_V8_help)
	{
		strcpy(help_directory,tms_V8_help);
		strcat(help_directory,"/xtms");
	}
	else
	{
		strcpy(help_directory,"/usr/local/bin/xnbc.bin/V8/help");
		fprintf(stderr,"XNBC_HELP_DIR not defined, using %s\n",
			help_directory);
		strcat(help_directory,"/xtms");
	}
}






