#ifndef _SIM_FUNCTIONS_2_
#define _SIM_FUNCTIONS_2_


#define ESSAI_DRUG 0

#include "creation-simu.h"
#include "sim_init.h"


#ifndef MAXFLOAT
#define MAXFLOAT 1000000000.
#endif

/*******************************************************************************
	Function for applying nucleus changes
*******************************************************************************/
int apply_nucleus_changes(void)
{
	float float_value;
	int int_value;
	int w_id;
	char *value;
	char *out_buf;
	char buffer[256];
	
	if(!current_nucleus)
	{
		fprintf(stderr,"Bad nucleus in init nucleus changes !!!\n");
		exit(-1);
	}
	
	if(XtIsManaged(widget_array[ID_form_nucl_para_other]))
	{
		w_id = ID_form_nucl_para_other;
	}
	else if(XtIsManaged(widget_array[ID_form_nucl_modulation]))
	{
		w_id = ID_form_nucl_modulation;
	}
	else if(XtIsManaged(widget_array[ID_form_nucl_para_noise]))
	{
		w_id = ID_form_nucl_para_noise;
	}
	else if(XtIsManaged(widget_array[ID_form_nucl_para_input]))
	{
		w_id = ID_form_nucl_para_input;
	}
	else if(XtIsManaged(widget_array[ID_form_nucl_drug]))
	{
		w_id = ID_form_nucl_drug;
	}
	else 
		return -1;
	
	out_buf = NULL;
	
	switch(w_id)
	{
	
		case ID_form_nucl_para_other:
			if(XmToggleButtonGetState(widget_array[ID_toggleB_make_alive]))
			{
				if(current_nucleus->alive == 'n')
				{
				  sprintf(buffer,"\tmaking %s alive.\n",current_nucleus->name);
				  XtSetSensitive(widget_array[ID_label_prob_spike],True);
				  XtSetSensitive(widget_array[ID_textF_prob_spike],True);
				  XtSetSensitive(widget_array[ID_label_old_last_spike],True);
				  XtSetSensitive(widget_array[ID_textF_old_last_spike],True);
				  XtSetSensitive(widget_array[ID_label_recent_last_spike],True);
				  XtSetSensitive(widget_array[ID_textF_recent_last_spike],True);
				  XtSetSensitive(widget_array[ID_label_low_mb_pot],True);
				  XtSetSensitive(widget_array[ID_textF_low_mb_pot],True);
				  XtSetSensitive(widget_array[ID_label_up_mb_pot],True);
				  XtSetSensitive(widget_array[ID_textF_up_mb_pot],True);
				  out_buf = bufferize(out_buf,buffer);
				}
				current_nucleus->alive = 'y';
			}
			else
			{
				if(current_nucleus->alive == 'y')
				{
				  sprintf(buffer,"\tmaking %s dead.\n",current_nucleus->name);
				  XtSetSensitive(widget_array[ID_label_prob_spike],False);
				  XtSetSensitive(widget_array[ID_textF_prob_spike],False);
				  XtSetSensitive(widget_array[ID_label_old_last_spike],False);
				  XtSetSensitive(widget_array[ID_textF_old_last_spike],False);
				  XtSetSensitive(widget_array[ID_label_recent_last_spike],False);
				  XtSetSensitive(widget_array[ID_textF_recent_last_spike],False);
				  XtSetSensitive(widget_array[ID_label_low_mb_pot],False);
				  XtSetSensitive(widget_array[ID_textF_low_mb_pot],False);
				  XtSetSensitive(widget_array[ID_label_up_mb_pot],False);
				  XtSetSensitive(widget_array[ID_textF_up_mb_pot],False);
				  out_buf = bufferize(out_buf,buffer);
				}
				current_nucleus->alive = 'n';
			}
			if(XmToggleButtonGetState(widget_array[ID_toggleB_weight_fluct]))
			{
				if(current_nucleus->law != 'h')
				{
					sprintf(buffer,"\tweight fluctuation activated.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d WF_ON\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					current_nucleus->date = simu->nb_it;
				}
				XtSetSensitive(widget_array[ID_label_learning_factor],True);
				XtSetSensitive(widget_array[ID_textF_hebb_factor],True);
				current_nucleus->law = 'h';
			}
			else
			{
				if(current_nucleus->law != 0)
				{
					sprintf(buffer,"\tweight fluctuation desactivated.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d WF_OFF\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					current_nucleus->date = 0;
					
				}
				XtSetSensitive(widget_array[ID_label_learning_factor],False);
				XtSetSensitive(widget_array[ID_textF_hebb_factor],False);
				current_nucleus->law = 0;
			}
			value = XmTextFieldGetString(widget_array[ID_textF_hebb_factor]);
			sscanf(value,"%f",&float_value);
			if(current_nucleus->hebb != float_value)
			{
				sprintf(buffer,"\thebb factor updated from %.2f to %.2f.\n",current_nucleus->hebb,
					float_value);
				out_buf = bufferize(out_buf,buffer);
				if(simu->nb_it > 0)
				{
					sprintf(buffer,"%lu nuc %d HEBB %.2f > %.2f\n",simu->nb_it,
						current_nucleus->id,current_nucleus->hebb,
						float_value);
					mod_buffer = bufferize(mod_buffer,buffer);
				}
				
			}
			current_nucleus->hebb = float_value;
			free(value);

			value = XmTextFieldGetString(widget_array[ID_textF_prob_spike]);
			sscanf(value,"%d",&int_value);
			if(current_nucleus->prob_spike != int_value)
			{
				sprintf(buffer,"\tPercentage of spiking neurons changed from %d to %d.\n",current_nucleus->prob_spike,
					int_value);
				out_buf = bufferize(out_buf,buffer);				
			}
			current_nucleus->prob_spike = int_value;
			free(value);

			value = XmTextFieldGetString(widget_array[ID_textF_old_last_spike]);
			sscanf(value,"%d",&int_value);
			if(current_nucleus->old_last_spike != int_value)
			{
				sprintf(buffer,"\tOldest spike changed from %d to %d ms.\n",current_nucleus->old_last_spike,
					int_value);
				out_buf = bufferize(out_buf,buffer);				
			}
			current_nucleus->old_last_spike = int_value;
			free(value);

			value = XmTextFieldGetString(widget_array[ID_textF_recent_last_spike]);
			sscanf(value,"%d",&int_value);
			if(current_nucleus->recent_last_spike != int_value)
			{
				sprintf(buffer,"\tRecentest spike changed from %d to %d ms.\n",current_nucleus->recent_last_spike,
					int_value);
				out_buf = bufferize(out_buf,buffer);				
			}
			current_nucleus->recent_last_spike = int_value;
			free(value);
			
			value = XmTextFieldGetString(widget_array[ID_textF_low_mb_pot]);
			sscanf(value,"%f",&float_value);
			if(current_nucleus->low_mb_pot != float_value)
			{
				sprintf(buffer,"\tLow limit of membrane potential changed from %.2f to %.2f mV.\n",current_nucleus->low_mb_pot,
					float_value);
				out_buf = bufferize(out_buf,buffer);				
			}
			current_nucleus->low_mb_pot = float_value;
			free(value);

			value = XmTextFieldGetString(widget_array[ID_textF_up_mb_pot]);
			sscanf(value,"%f",&float_value);
			if(current_nucleus->up_mb_pot != float_value)
			{
				sprintf(buffer,"\tHigh limit of membrane potential changed from %.2f to %.2f mV.\n",current_nucleus->up_mb_pot,
					float_value);
				out_buf = bufferize(out_buf,buffer);				
			}
			current_nucleus->up_mb_pot = float_value;
			free(value);

			if(out_buf)
			{
				sprintf(buffer,"\n\nGeneral parameter changes for %s...",current_nucleus->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			}
			
			break;
			
		case ID_form_nucl_modulation:
			if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_modul_sin]))
			{
				if(current_nucleus->modul_type != 's')
				{
					sprintf(buffer,"\tsinusoidal modulation actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d MOD_S\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_type = 's';
				XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], True);
			}
			else if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_modul_triangle]))
			{
				
				if(current_nucleus->modul_type != 't')
				{
					sprintf(buffer,"\ttriangle modulation actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d MOD_T\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_type = 't';
				XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], True);
			}
			else if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_modul_rect]))
			{
				if(current_nucleus->modul_type != 'r')
				{
					sprintf(buffer,"\trectangle modulation actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d MOD_R\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_type = 'r';
				XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], True);
			}
			else if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_modul_pulse]))
			{
				if(current_nucleus->modul_type != 'p')
				{
					sprintf(buffer,"\tsingle pulse modulation actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d MOD_P\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_type = 'p';
				XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], True);
			}
			else
			{
				if(current_nucleus->modul_type != 0)
				{
					sprintf(buffer,"\tmodulation desactived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d NO_MOD\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_type = 0;
				XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], False);
			}
			
			if (current_nucleus->modul_type == 'p')
			{
			    value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_period]);
			    sscanf(value,"%f",&float_value);
			    if((float_value >= 0.0) && (float_value < 1000000.0))
			    {	
				    if(current_nucleus->modul_period != float_value)
				    {
					    sprintf(buffer,"\tpulse duration changed from %.2f to %.2f ms.\n",
						    current_nucleus->modul_period,float_value);
					    out_buf = bufferize(out_buf,buffer);
					    if(simu->nb_it > 0)
					    {
						    sprintf(buffer,"%lu nuc %d Mod_T %.2f > %.2f\n",simu->nb_it,
							    current_nucleus->id,current_nucleus->modul_period,
								    float_value);
						    mod_buffer = bufferize(mod_buffer,buffer);
					    }

				    }
				    current_nucleus->modul_period = float_value;
				    current_nucleus->pulse_down = (unsigned long) float_value;
			    }
			    else
			    {
				    sprintf(buffer,"The pulse duration cannot be %.2f...",float_value);
				    display_error(widget_array[ID_form_nucl_modulation],buffer,
					    "Bad Value !");
				    free(value);
				    return -1;
			    }
			    free(value);

			    value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_phase]);
			    sscanf(value,"%f",&float_value);
			    if((float_value >= 0.0))  
			    {	
				    if(current_nucleus->modul_phase != float_value)
				    {
					    sprintf(buffer,"\tPulse interval changed from %.2f to %.2f ms.\n",
						    current_nucleus->modul_phase,float_value);
					    out_buf = bufferize(out_buf,buffer);
					    if(simu->nb_it > 0)
					    {
						    sprintf(buffer,"%lu nuc %d Mod_P %.2f > %.2f\n",simu->nb_it,
							    current_nucleus->id,current_nucleus->modul_phase,
								    float_value);
						    mod_buffer = bufferize(mod_buffer,buffer);
					    }

				    }
				    current_nucleus->modul_phase = float_value;
				    current_nucleus->pulse_up = (unsigned long) float_value;
			    }
			    else
			    {
				    sprintf(buffer,"The pulse interval cannot be %.2f...\nIt is necessarilly positive.",float_value);
				    display_error(widget_array[ID_form_nucl_modulation],buffer,
					    "Bad Value !");
				    free(value);	
				    return -1;
			    }
			    free(value);

			    value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_nb]);
			    sscanf(value,"%d",&int_value);
			    if((int_value >= 0))  
			    {	
				    if(current_nucleus->nb_pulses != int_value)
				    {
					    sprintf(buffer,"\tPulses number changed from %d to %d.\n",
						    current_nucleus->nb_pulses,int_value);
					    out_buf = bufferize(out_buf,buffer);
					    if(simu->nb_it > 0)
					    {
						    sprintf(buffer,"%lu nuc %d Mod_nb %d > %d\n",simu->nb_it,
							    current_nucleus->id,current_nucleus->nb_pulses,
								    int_value);
						    mod_buffer = bufferize(mod_buffer,buffer);
					    }

				    }
				    current_nucleus->nb_pulses = int_value;
				    current_nucleus->pulses_restants = int_value;
			    }
			    else
			    {
				    sprintf(buffer,"The pulses number cannot be %2d...\nIt is necessarilly positive.",int_value);
				    display_error(widget_array[ID_form_nucl_modulation],buffer,
					    "Bad Value !");
				    free(value);	
				    return -1;
			    }
			    free(value);
			}
			else
			{
			    value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_period]);
			    sscanf(value,"%f",&float_value);
			    if((float_value >= 0.0) && (float_value < 1000000.0))
			    {	
				    if(current_nucleus->modul_period != float_value)
				    {
					    sprintf(buffer,"\tmodulation period changed from %.2f to %.2f ms.\n",
						    current_nucleus->modul_period,float_value);
					    out_buf = bufferize(out_buf,buffer);
					    if(simu->nb_it > 0)
					    {
						    sprintf(buffer,"%lu nuc %d Mod_T %.2f > %.2f\n",simu->nb_it,
							    current_nucleus->id,current_nucleus->modul_period,
								    float_value);
						    mod_buffer = bufferize(mod_buffer,buffer);
					    }

				    }
				    current_nucleus->modul_period = float_value;
			    }
			    else
			    {
				    sprintf(buffer,"The modulation period cannot be %.2f...",float_value);
				    display_error(widget_array[ID_form_nucl_modulation],buffer,
					    "Bad Value !");
				    free(value);
				    return -1;
			    }
			    free(value);

			    value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_phase]);
			    sscanf(value,"%f",&float_value);
			    if((float_value >= 0.0) && (float_value <= current_nucleus->modul_period))  
			    {	
				    if(current_nucleus->modul_phase != float_value)
				    {
					    sprintf(buffer,"\tmodulation phase changed from %.2f to %.2f ms.\n",
						    current_nucleus->modul_phase,float_value);
					    out_buf = bufferize(out_buf,buffer);
					    if(simu->nb_it > 0)
					    {
						    sprintf(buffer,"%lu nuc %d Mod_P %.2f > %.2f\n",simu->nb_it,
							    current_nucleus->id,current_nucleus->modul_phase,
								    float_value);
						    mod_buffer = bufferize(mod_buffer,buffer);
					    }

				    }
				    current_nucleus->modul_phase = float_value;
			    }
			    else
			    {
				    sprintf(buffer,"The modulation phase cannot be %.2f...\nIt is necessarilly between 0.0 and %.2f.",float_value,
					    current_nucleus->modul_period);
				    display_error(widget_array[ID_form_nucl_modulation],buffer,
					    "Bad Value !");
				    free(value);	
				    return -1;
			    }
			    free(value);
			}
			
			value = XmTextFieldGetString(widget_array[ID_textF_nucl_modul_amp]);
			sscanf(value,"%f",&float_value);
			if((float_value >= -2000.0) && (float_value < 2000.0))  
			{	
				if(current_nucleus->modul_amp != float_value)
				{
					sprintf(buffer,"\tmodulation amplitude changed from %.2f to %.2f mV.\n",
						current_nucleus->modul_amp,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d Mod_A %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->modul_amp,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->modul_amp = float_value;
				if (float_value)
				  XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], True);
				else
				  XtSetSensitive(stim_modul_w[(current_nucleus->number - 1)], False);
			}
			else
			{
				sprintf(buffer,"This modulation amplitude (%.2f) is probably wrong...",float_value);
				display_error(widget_array[ID_form_nucl_modulation],buffer,
					"Bizarre Value !");
				free(value);	
				return -1;
			}
			free(value);
			
			if(out_buf)
			{
				sprintf(buffer,"\n\nModulation parameter changes for %s...",current_nucleus->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			}
			break;
			
			
		case ID_form_nucl_para_noise:
			if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_noise_active]))
			{
				if(current_nucleus->noise_active != 1)
				{
					sprintf(buffer,"\tGaussian noise actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d BNoise_ON\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->noise_active = 1;
				XtSetSensitive(stim_noise_w[(current_nucleus->number - 1)], True);
			}
			else
			{
				if(current_nucleus->noise_active != 0)
				{
					sprintf(buffer,"\tnoise desactived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d BNoise_OFF\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->noise_active = 0;
				XtSetSensitive(stim_noise_w[(current_nucleus->number - 1)], False);
			}
			
			value = XmTextFieldGetString(widget_array[ID_textF_nucl_noise_m]);
			sscanf(value,"%f",&float_value);
			if((float_value >= - 200.0) && (float_value < 200.0))
			{	
				if(current_nucleus->backgrd_noise_m != float_value)
				{
					sprintf(buffer,"\tnoise mean changed from %.2f to %.2f mv.\n",
						current_nucleus->backgrd_noise_m,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d BNoise_m %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->backgrd_noise_m,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->backgrd_noise_m = float_value;

				/*ATTENTION bruit nucleus pour LIM pas traite */

			}
			else
			{
				sprintf(buffer,"The noise mean cannot be %.2f...",float_value);
				display_error(widget_array[ID_form_nucl_para_noise],buffer,
					"Bad Value !");
				free(value);
				return -1;
			}
			free(value);
			
			value = XmTextFieldGetString(widget_array[ID_textF_nucl_noise_sd]);
			sscanf(value,"%f",&float_value);
			if((float_value >= 0.0) && (float_value <  200.0))  
			{	
				if(current_nucleus->backgrd_noise_sd != float_value)
				{
					sprintf(buffer,"\tnoise standard deviation changed from %.2f to %.2f mv.\n",
						current_nucleus->backgrd_noise_sd,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d BNoise_sd %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->backgrd_noise_sd,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->backgrd_noise_sd = float_value;
				current_nucleus->backgrd_n_sd = current_nucleus->backgrd_noise_sd * sqrt(simu->time_step);
				if (current_nucleus->noise_active && (current_nucleus->backgrd_noise_sd || current_nucleus->backgrd_noise_m)) 
				  XtSetSensitive(stim_noise_w[(current_nucleus->number - 1)], True);
				else
				  XtSetSensitive(stim_noise_w[(current_nucleus->number - 1)], False);
			}
			else
			{
				sprintf(buffer,"The noise standard deviation cannot be %.2f...",float_value);
				
				display_error(widget_array[ID_form_nucl_para_noise],buffer,
					"Bad Value !");
				free(value);	
				return -1;
			}
			free(value);
			
			if(out_buf)
			{
				sprintf(buffer,"\n\nNoise parameter changes for %s...",current_nucleus->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			}
			break;
		
	        /*
		 * Pour les pulses maintenant
		 */
		case ID_form_nucl_para_input:
			if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_reg]))
			{
				if(current_nucleus->input_type != 'r')
				{
					sprintf(buffer,"\tregular input actived.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_R\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->input_type = 'r';
				XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], True);
			}
			else
			{
				if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_gauss]))
				{
					if(current_nucleus->input_type != 'n')
					{
						sprintf(buffer,"\tgaussian input actived.\n");
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu nuc %d INP_N\n",simu->nb_it,
								current_nucleus->id);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
						
					}
					current_nucleus->input_type = 'n';
					XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], True);
				}
				else
				{
					if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_gamma]))
					{
						if(current_nucleus->input_type != 'g')
						{
							sprintf(buffer,"\tgamma input actived.\n");
							out_buf = bufferize(out_buf,buffer);
							if(simu->nb_it > 0)
							{
								sprintf(buffer,"%lu nuc %d INP_G\n",simu->nb_it,
									current_nucleus->id);
								mod_buffer = bufferize(mod_buffer,buffer);
							}
							
						}
						current_nucleus->input_type = 'g';
						XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], True);
					}
					else
					{
						if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_poisson]))
						{
							if(current_nucleus->input_type != 'p')
							{
								sprintf(buffer,"\tPoisson input actived.\n");
								out_buf = bufferize(out_buf,buffer);
								if(simu->nb_it > 0)
								{
									sprintf(buffer,"%lu nuc %d INP_P\n",simu->nb_it,
										current_nucleus->id);
									mod_buffer = bufferize(mod_buffer,buffer);
								}
								
							}
							current_nucleus->input_type = 'p';
							XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], True);
						}
						else
						{
	
							if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_none]))
							{
								if(current_nucleus->input_type != 0)
								{
									sprintf(buffer,"\tinput desactived.\n");
									out_buf = bufferize(out_buf,buffer);
									if(simu->nb_it > 0)
									{
										sprintf(buffer,"%lu nuc %d NO_INP\n",simu->nb_it,
											current_nucleus->id);
										mod_buffer = bufferize(mod_buffer,buffer);
									}
									
								}
								current_nucleus->input_type = 0;
								XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], False);
							}
						} /* fin du else */
					} /* fin du else */
				} /* fin du else */
			} /* fin du else */
				
			if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_inhib]))
			{
				if(current_nucleus->inhib != True)
				{
					sprintf(buffer,"\tset inhibition input.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_INH\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->inhib = True;
			}
			else
			{
				if(current_nucleus->inhib != False)
				{
					sprintf(buffer,"\tset excitation input.\n");
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_EXC\n",simu->nb_it,
							current_nucleus->id);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->inhib = False;
			}
			
			
			value = XmTextFieldGetString(widget_array[ID_textF_input_period_m]);
			sscanf(value,"%f",&float_value);
			if((float_value >= 0.0) && (float_value < 1000000.0))
			{	
				
				if(current_nucleus->input_mean != float_value)
				{
					sprintf(buffer,"\tinput mean changed from %.2f to %.2f ms.\n",
						current_nucleus->input_mean,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_m %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->input_mean,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->input_mean = float_value;
			}
			else
			{
				sprintf(buffer,"The input period mean cannot be %.2f...",float_value);
				display_error(widget_array[ID_form_nucl_para_input],buffer,
					"Bad Value !");
				free(value);
				return -1;
			}
			free(value);
			
			value = XmTextFieldGetString(widget_array[ID_textF_input_sd]);
			sscanf(value,"%f",&float_value);
			if((float_value >= 0.0) && (float_value <= current_nucleus->input_mean))
			{	
				if(current_nucleus->input_sd != float_value)
				{
					sprintf(buffer,"\tinput standard deviation changed from %.2f to %.2f ms.\n",
						current_nucleus->input_sd,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_sd %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->input_sd,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->input_sd = float_value;
			}
			else
			{
				sprintf(buffer,"The standard deviation cannot be %.2f...",float_value);
				display_error(widget_array[ID_form_nucl_para_input],buffer,
					"Bad Value !");
				free(value);
				return -1;
			}
			free(value);
			
			value = XmTextFieldGetString(widget_array[ID_textF_input_scale_factor]);
			sscanf(value,"%f",&float_value);
			if((float_value >= 0.0) && (float_value < 1000000.0))  
			{	
				if(current_nucleus->scale_f != float_value)
				{
					sprintf(buffer,"\tinput scale factor changed from %.2f to %.2f.\n",
						current_nucleus->scale_f,float_value);
					out_buf = bufferize(out_buf,buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu nuc %d INP_a %.2f > %.2f\n",simu->nb_it,
							current_nucleus->id,current_nucleus->scale_f,
								float_value);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
					
				}
				current_nucleus->scale_f = float_value;
				if (float_value)
				  XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], True);
				else
				  XtSetSensitive(stim_pulse_w[(current_nucleus->number - 1)], False);
			}
			else
			{
				sprintf(buffer,"The input scale factor cannot be %.2f...",float_value);
				
				display_error(widget_array[ID_form_nucl_para_input],buffer,
					"Bad Value !");
				free(value);	
				return -1;
			}
			free(value);
			
			if(out_buf)
			{
				sprintf(buffer,"\n\nInput parameter changes for %s...",current_nucleus->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			}	
			break;

	        /*
		 * Pour les drogues maintenant
		 */
	       	case ID_form_nucl_drug:
		          out_buf = NULL;
			  /* si on a la selection sur drug */
			  if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_drug]))
			  {
#if ESSAI_DRUG
			      printf("il y a des drogues il faut lire la selection et agir en consequence\n");
#endif
			      sprintf(buffer,"\tDrugs enabled for nucleus %d.\n",current_nucleus->number);
			      out_buf = bufferize(out_buf,buffer);
			      current_nucleus->drug_enabled = True;
			      XtSetSensitive(stim_drug_w[(current_nucleus->number - 1)], True);

			  }

			  /* si on a la selection sur No drug */ 
			  if(XmToggleButtonGetState(widget_array[ID_toggleB_nucl_input_No_drug]))
			  {
#if ESSAI_DRUG
			      printf("Pas de drogues\n");
#endif
			      sprintf(buffer,"\tDrugs disabled for nucleus %d.\n",current_nucleus->number);
			      out_buf = bufferize(out_buf,buffer);
			      current_nucleus->drug_enabled = False;
			      XtSetSensitive(stim_drug_w[(current_nucleus->number - 1)], False);

			  }
			  if(out_buf)
			  {
				sprintf(buffer,"\n\nDrug status for %s...",current_nucleus->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			  }	
			  break;
		       
      	      default:
			break;
	
	}

	return 0;

}

/*******************************************************************************
	Function for buffering message for output
*******************************************************************************/
char *bufferize(char *old,char *msg)
{
	char *new;
	
	if(msg && *msg)
	{
		if(!old)
		{
			new = (char *)calloc(1,(strlen(msg)+1)*sizeof(msg));
			strcpy(new,msg);
		}
		else
		{
			new = (char *)realloc(old,(strlen(msg)+strlen(old)+1)*sizeof(msg));	
			strcat(new,msg);
		}
		return new;
	}
	else
		return old;


}
/*******************************************************************************
	Function for buffering data from specified size
*******************************************************************************/
char *data_bufferize(char *old,char *data,unsigned long old_size,
	unsigned long data_size)
{
	char *new;
	
	if(data)
	{
		if(!old)
		{
			new = (char *)calloc(1,data_size*sizeof(char));
			memcpy(new,data,data_size);
		}
		else
		{
			new = (char *)realloc(old,(old_size+data_size)*sizeof(char));	
			memcpy(&new[old_size],data,data_size);
		}
		return new;
	}
	else
		return old;


}
/*******************************************************************************
	Function for buffering and flushing dynamically data from specified size
*******************************************************************************/
char *bufferize_and_flush(char *old,char *data,unsigned long *size,
	unsigned long data_size,FILE **fp)
{
	char *new;
	
	
	if(data)
	{
		if(!old)
		{
			new = (char *)calloc(1,_BUFFER_SIZE_);
			memcpy(new,data,data_size);
			*size = data_size;
		}
		else if(*size+data_size < _BUFFER_SIZE_)
		{	
			new = old;
			memcpy(&new[*size],data,data_size);
			*size += data_size;
		}
		else if(!(*fp))
		{
			*fp = tmpfile();
			fwrite(old,1,*size,*fp);
			fwrite(data,1,_BUFFER_SIZE_ - *size,*fp);
			new = old;
			memcpy(new,&data[_BUFFER_SIZE_ - *size],
				data_size - _BUFFER_SIZE_ + *size);
			*size = data_size - _BUFFER_SIZE_ + *size;
		}
		else
		{
			fwrite(old,1,*size,*fp);
			fwrite(data,1,_BUFFER_SIZE_ - *size,*fp);
			new = old;
			memcpy(new,&data[_BUFFER_SIZE_ - *size],
				data_size - _BUFFER_SIZE_ + *size);
			*size = data_size - _BUFFER_SIZE_ + *size;
		}
		return new;
	}
	else
		return old;


}

/*******************************************************************************
	Function for modifying cluster simulation parameters
*******************************************************************************/
void modify_cluster(int number)
{

	CLUSTER *cluster_tmp;
	static int last_modif = 0;
	static int last_nb = 0;
	char buffer[256];
	
	cluster_tmp = sim_clusters[number];
	
	if(!cluster_tmp)
	{
		fprintf(stderr,"Bad cluster in modify cluster !!!\n");
		exit(-1);
	}
	else
		current_cluster = cluster_tmp;

	if(number != last_nb)
	{
		if(last_modif && XtIsManaged(widget_array[last_modif]))
				XtUnmanageChild(widget_array[last_modif]);
		last_modif = 0;
	}


	switch(last_modif)
	{
	
		case 0:
		case ID_form_cluster_general:
			if(XtIsManaged(widget_array[ID_form_cluster_general]))
				XtUnmanageChild(widget_array[ID_form_cluster_general]);
			if(!XtIsManaged(widget_array[ID_form_cluster_general]))
				XtManageChild(widget_array[ID_form_cluster_general]);
			init_cluster_changes(ID_form_cluster_general);
			last_modif = ID_form_cluster_general;
			break;

		
		default:
			break;	
	
	}

	sprintf(buffer,"Simulation Parameters for %s",cluster_tmp->name);
	XtVaSetValues(XtParent(widget_array[ID_form_cluster_parameters]),XmNtitle,buffer,NULL);

	if(!XtIsManaged(widget_array[ID_form_cluster_parameters]))
		XtManageChild(widget_array[ID_form_cluster_parameters]);
	else
		XtPopup(XtParent(widget_array[ID_form_cluster_parameters]),XtGrabNone);

	last_nb = number;


}
/**********************************************************************************
	Function for initializing cluster parameter changes
**********************************************************************************/
void init_cluster_changes(int w_id)
{

	char buffer[256];
	PUM *pum;
	CBM *cbm;
	LIM *lim;
	BUM *bum;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	XXM *xxm;
/***********************       STOP     ***********************/
#endif
	XmString string;
	
	if(!current_cluster)
	{
		fprintf(stderr,"Bad cluster in init cluster changes !!!\n");
		exit(-1);
	}
	
	switch(w_id)
	{
	
		case ID_form_cluster_general:
			
			XmTextFieldRemove(widget_array[ID_textF_cluster_file_name]);
			sprintf(buffer,"%s",current_cluster->file_name);
			XmTextReplace(widget_array[ID_textF_cluster_file_name],0,
			XmTextGetLastPosition(widget_array[ID_textF_cluster_file_name]),
				buffer);
			
			sprintf(buffer,"%d neuron(s)",current_cluster->cluster_size);
			string =XmStringCreateLocalized(buffer);
			XtVaSetValues(widget_array[ID_label_clu_nb_neurons],XmNlabelString,string,NULL);
			XmStringFree(string);
			
			if(current_cluster->model == 'V')
			{
				
				XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					False);
				
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.2f",0.0);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					False);	
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.2f",0.0);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					False);	
			
			}
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			else if(current_cluster->model == 'X')
			{
				pum = (XXM *)current_cluster->clu_data;
				if(!xxm->noise_active)
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				else
				{
				  if(!xxm->filter_noise)
				        XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_active],True,True);
				  else
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_filter],True,True);
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.3f",xxm->backgrd_noise_m);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
						
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.3f",xxm->backgrd_noise_sd);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);				
			}
/***********************       STOP     ***********************/
#endif
			else if(current_cluster->model == 'L')
			{
				lim = (LIM *)current_cluster->clu_data;
				if(!lim->noise_active)
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				else
				{
				  if(!lim->filter_noise)
				        XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_active],True,True);
				  else
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_filter],True,True);
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.3f",lim->backgrd_noise_m);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
						
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.3f",lim->backgrd_noise_sd);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);				
			}
			else if(current_cluster->model == 'P')
			{
				pum = (PUM *)current_cluster->clu_data;
				if(!pum->noise_active)
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				else
				{
				  if(!pum->filter_noise)
				        XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_active],True,True);
				  else
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_filter],True,True);
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.3f",pum->backgrd_noise_m);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
						
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.3f",pum->backgrd_noise_sd);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);				
				XtSetSensitive(widget_array[ID_toggleB_clu_noise_filter],True);				
			}
			else if(current_cluster->model == 'B')
			{
				bum = (BUM *)current_cluster->clu_data;
				if(!bum->noise_active)
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				else
				{
				  if(!bum->filter_noise)
				        XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_active],True,True);
				  else
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_filter],True,True);
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.3f",bum->backgrd_noise_m);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
						
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.3f",bum->backgrd_noise_sd);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);				
				XtSetSensitive(widget_array[ID_toggleB_clu_noise_filter],True);				
			}
                        else if(current_cluster->model == 'G')
			{
				cbm = (CBM *)current_cluster->clu_data;
				if(!cbm->noise_active)
					XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_none],True,True);
				else
				        XmToggleButtonSetState(widget_array[ID_toggleB_clu_noise_active],True,True);

				XtSetSensitive(widget_array[ID_toggleB_clu_noise_filter],False);				

				XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
				sprintf(buffer,"%.3f",cbm->backgrd_noise_m);
				XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
					
				XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
				sprintf(buffer,"%.3f",cbm->backgrd_noise_sd);
				XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
				XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
					
			}
			
			break;
			
		default:
			break;
	}
	
}
/*******************************************************************************
	Function for applying cluster changes
*******************************************************************************/
int apply_cluster_changes(void)
{
	char buffer[256];
	int w_id;
	char *value;
	char *out_buf;
	float float_value;
	char old_file[256];
	int i,j;
	PUM *pum_clu;
	PUM old_pum;
	BUM *bum_clu;
	BUM old_bum;
	CBM *cbm_clu;
	CBM old_cbm;
	LIM *lim_clu;
	LIM old_lim;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	XXM *xxm_clu;
	XXM old_xxm;
/***********************       STOP     ***********************/
#endif
	XmString string;
	
	if(!current_cluster)
	{
		fprintf(stderr,"Bad cluster in init cluster changes !!!\n");
		exit(-1);
	}
	
	if(XtIsManaged(widget_array[ID_form_cluster_general]))
	{
		w_id = ID_form_cluster_general;
		
	}
	else 
		return -1;
		
		
	out_buf = NULL;

	switch(w_id)
	{
	
		case ID_form_cluster_general:
		
			strcpy(old_file,current_cluster->file_name);
			value = XmTextFieldGetString(widget_array[ID_textF_cluster_file_name]);
#if 0
			for(j=0 ; j<simu->nb_clusters ; j++)
			{
				if((j!=(current_cluster->number-1)) && !strcmp(sim_clusters[j]->file_name,value))
				{
			
					sprintf(buffer,"File %s already used by cluster %d...",
						value,sim_clusters[j]->number);
					display_error(widget_array[ID_sim_mainW],buffer,"Invalid File Name !");
					return -1;
				}
			}
#endif
			if(strcmp(value,old_file))
			{
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				if(current_cluster->model == 'X')
				{
					memcpy(&old_xxm,current_cluster->clu_data,sizeof(XXM));
				}
/***********************       STOP     ***********************/
#endif
				if(current_cluster->model == 'L')
				{
					memcpy(&old_lim,current_cluster->clu_data,sizeof(LIM));
				}
				if(current_cluster->model == 'B')
				{
					memcpy(&old_bum,current_cluster->clu_data,sizeof(BUM));
				}
				if(current_cluster->model == 'P')
				{
					memcpy(&old_pum,current_cluster->clu_data,sizeof(PUM));
				}
				if(current_cluster->model == 'G')
				{
					memcpy(&old_cbm,current_cluster->clu_data,sizeof(CBM));
				}
				if(current_cluster->model == 'V')
				{
					if(((VIRTUAL *)current_cluster->clu_data)->fp_tms)
					{
						fclose(((VIRTUAL *)current_cluster->clu_data)->fp_tms);
						((VIRTUAL *)current_cluster->clu_data)->fp_tms = NULL;
					}
				}
				free(current_cluster->clu_data);
				current_cluster->clu_data = NULL;
						
				if(read_clu_unit(value,current_cluster->number - 1))
				{
					sprintf(buffer,"File %s cannot be read...",
							value);
					display_error(widget_array[ID_sim_mainW],buffer,"Invalid File Name !");
						
					read_clu_unit(old_file,current_cluster->number - 1);
					XmTextFieldRemove(widget_array[ID_textF_cluster_file_name]);
					sprintf(buffer,"%s",current_cluster->file_name);
					XmTextReplace(widget_array[ID_textF_cluster_file_name],0,
					XmTextGetLastPosition(widget_array[ID_textF_cluster_file_name]),
						buffer);
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
					if(current_cluster->model == 'X')
					{
						((XXM *)current_cluster->clu_data)->noise_active = old_xxm.noise_active;
						((XXM *)current_cluster->clu_data)->filter_noise = old_xxm.filter_noise;
						if(!((XXM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((XXM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_xxm.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((XXM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((XXM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((XXM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_xxm.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((XXM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
/***********************       STOP     ***********************/
#endif
					if(current_cluster->model == 'L')
					{
						((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
						((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_lim.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_lim.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'B')
					{
						((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
						((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_bum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_bum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'P')
					{
						((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
						((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_pum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_pum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'G')
					{
						((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
						((CBM *)current_cluster->clu_data)->filter_noise = old_cbm.filter_noise;
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_cbm.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_cbm.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					return -1;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_cluster_file_name]);
					sprintf(buffer,"%s",current_cluster->file_name);
					XmTextReplace(widget_array[ID_textF_cluster_file_name],0,
					XmTextGetLastPosition(widget_array[ID_textF_cluster_file_name]),
						buffer);
					if(simu->nb_it > 0)
					{
						sprintf(buffer,"%lu clu %d NEURON_FILE\n",simu->nb_it,
							current_cluster->number);
						mod_buffer = bufferize(mod_buffer,buffer);
					}
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/

					if(current_cluster->model == 'P')
					{
						((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
						((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_pum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_pum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}

/***********************       STOP     ***********************/
#endif

					if(current_cluster->model == 'L')
					{
						((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
						((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_lim.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_lim.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}

					if(current_cluster->model == 'B')
					{
						((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
						((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_bum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_bum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'P')
					{
						((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
						((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_pum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_pum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'G')
					{
						((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
						((CBM *)current_cluster->clu_data)->filter_noise = old_cbm.filter_noise;
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_cbm.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_cbm.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					sprintf(buffer,"%s (%s)",current_cluster->name,current_cluster->file_name);
					string = XmStringCreateLocalized(buffer);
					XtVaSetValues(clu_w[current_cluster->number - 1],XmNlabelString,string,NULL);
					XmStringFree(string);
					
					sprintf(buffer,"\n\nNeuron file from %s changed...\n\tfrom %s \n\tto %s.\n",
						current_cluster->name,old_file,current_cluster->file_name);
					out_buf = bufferize(out_buf,buffer);
				}
				free(value);
			}			
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			if(current_cluster->model == 'X')
			{
				xxm_clu = (XXM *)current_cluster->clu_data;
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_none]))
				{
					if((xxm_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise in %s desactived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_OFF\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					xxm_clu->noise_active = False;
					xxm_clu->filter_noise = False;
				}
				else
				{
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_active]))
				    {    
					if((!xxm_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tGaussian white noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					xxm_clu->noise_active = True;
					xxm_clu->filter_noise = False;
				    }
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_filter]))
				    {    
					if((!xxm_clu->filter_noise) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tKohn colored noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					xxm_clu->noise_active = True;
					xxm_clu->filter_noise = True;
				    }
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_m]);
				sscanf(value,"%f",&float_value);
				if((float_value >= -200.0) && (float_value < 200.0))
				{
					if((xxm_clu->backgrd_noise_m != float_value) 
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise mean changed from %.3f to %.3f mv in %s.\n",xxm_clu->backgrd_noise_m,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_m %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,xxm_clu->backgrd_noise_m,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					xxm_clu->backgrd_noise_m = float_value;
				}
				else
				{
					sprintf(buffer,"noise mean cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_sd]);
				sscanf(value,"%f",&float_value);
				if((float_value >= 0.0) && (float_value <= 200.0))
				{
					if((xxm_clu->backgrd_noise_sd != float_value)
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise standard deviation changed from %.3f to %.3f mv in %s.\n",xxm_clu->backgrd_noise_sd,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_sd %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,xxm_clu->backgrd_noise_sd,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					xxm_clu->backgrd_noise_sd = float_value;
				}
				else
				{
					sprintf(buffer,"noise standard deviation cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);
			}

/***********************       STOP     ***********************/
#endif
			if(current_cluster->model == 'L')
			{
				lim_clu = (LIM *)current_cluster->clu_data;
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_none]))
				{
					if((lim_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise in %s desactived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_OFF\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					lim_clu->noise_active = False;
					lim_clu->filter_noise = False;
				}
				else
				{
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_active]))
				    {    
					if((!lim_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tGaussian white noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					lim_clu->noise_active = True;
					lim_clu->filter_noise = False;
				    }
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_filter]))
				    {    
					if((!lim_clu->filter_noise) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tKohn colored noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					lim_clu->noise_active = True;
					lim_clu->filter_noise = True;
				    }
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_m]);
				sscanf(value,"%f",&float_value);
				if((float_value >= -200.0) && (float_value < 200.0))
				{
					if((lim_clu->backgrd_noise_m != float_value) 
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise mean changed from %.3f to %.3f mv in %s.\n",lim_clu->backgrd_noise_m,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_m %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,lim_clu->backgrd_noise_m,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					lim_clu->backgrd_noise_m = float_value;
					lim_clu->backgrd_n_m = lim_clu->backgrd_noise_m * simu->time_step;
				}
				else
				{
					sprintf(buffer,"noise mean cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_sd]);
				sscanf(value,"%f",&float_value);
				if((float_value >= 0.0) && (float_value <= 200.0))
				{
					if((lim_clu->backgrd_noise_sd != float_value)
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise standard deviation changed from %.3f to %.3f mv in %s.\n",lim_clu->backgrd_noise_sd,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_sd %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,lim_clu->backgrd_noise_sd,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					lim_clu->backgrd_noise_sd = float_value;
					lim_clu->backgrd_n_sd = lim_clu->backgrd_noise_sd * sqrt(simu->time_step);
				}
				else
				{
					sprintf(buffer,"noise standard deviation cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);
			}

			if(current_cluster->model == 'B')
			{
				bum_clu = (BUM *)current_cluster->clu_data;
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_none]))
				{
					if((bum_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise in %s desactived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_OFF\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					bum_clu->noise_active = False;
					bum_clu->filter_noise = False;
				}
				else
				{
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_active]))
				    {    
					if((!bum_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tGaussian white noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					bum_clu->noise_active = True;
					bum_clu->filter_noise = False;
				    }
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_filter]))
				    {    
					if((!bum_clu->filter_noise) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tKohn`s colored noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					bum_clu->noise_active = True;
					bum_clu->filter_noise = True;
				    }
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_m]);
				sscanf(value,"%f",&float_value);
				if((float_value >= -200.0) && (float_value < 200.0))
				{
					if((bum_clu->backgrd_noise_m != float_value) 
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise mean changed from %.3f to %.3f mv in %s.\n",bum_clu->backgrd_noise_m,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_m %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,bum_clu->backgrd_noise_m,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					bum_clu->backgrd_noise_m = float_value;
				}
				else
				{
					sprintf(buffer,"noise mean cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_sd]);
				sscanf(value,"%f",&float_value);
				if((float_value >= 0.0) && (float_value <= 200.0))
				{
					if((bum_clu->backgrd_noise_sd != float_value)
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise standard deviation changed from %.3f to %.3f mv in %s.\n",bum_clu->backgrd_noise_sd,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_sd %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,bum_clu->backgrd_noise_sd,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					bum_clu->backgrd_noise_sd = float_value;
					bum_clu->backgrd_n_sd = bum_clu->backgrd_noise_sd * sqrt(simu->time_step);
				}
				else
				{
					sprintf(buffer,"noise standard deviation cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);
			}
			if(current_cluster->model == 'P')
			{
				pum_clu = (PUM *)current_cluster->clu_data;
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_none]))
				{
					if((pum_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise in %s desactived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_OFF\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					pum_clu->noise_active = False;
					pum_clu->filter_noise = False;
				}
				else
				{
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_active]))
				    {    
					if((!pum_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tGaussian white noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					pum_clu->noise_active = True;
					pum_clu->filter_noise = False;
				    }
				    if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_filter]))
				    {    
					if((!pum_clu->filter_noise) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tKohn`s colored noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					pum_clu->noise_active = True;
					pum_clu->filter_noise = True;
				    }
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_m]);
				sscanf(value,"%f",&float_value);
				if((float_value >= -200.0) && (float_value < 200.0))
				{
					if((pum_clu->backgrd_noise_m != float_value) 
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise mean changed from %.3f to %.3f mv in %s.\n",pum_clu->backgrd_noise_m,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_m %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,pum_clu->backgrd_noise_m,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					pum_clu->backgrd_noise_m = float_value;
				}
				else
				{
					sprintf(buffer,"noise mean cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_sd]);
				sscanf(value,"%f",&float_value);
				if((float_value >= 0.0) && (float_value <= 200.0))
				{
					if((pum_clu->backgrd_noise_sd != float_value)
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise standard deviation changed from %.3f to %.3f mv in %s.\n",pum_clu->backgrd_noise_sd,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_sd %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,pum_clu->backgrd_noise_sd,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					pum_clu->backgrd_noise_sd = float_value;
					pum_clu->backgrd_n_sd = pum_clu->backgrd_noise_sd * sqrt(simu->time_step);
				}
				else
				{
					sprintf(buffer,"noise standard deviation cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);
			}
			if(current_cluster->model == 'G')
			{
				cbm_clu = (CBM *)current_cluster->clu_data;
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_none]))
				{
					if((cbm_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise in %s desactived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_OFF\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					cbm_clu->noise_active = False;
					cbm_clu->filter_noise = False;
				}
				else
				{
					if((!cbm_clu->noise_active) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tGaussian white noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					cbm_clu->noise_active = True;
					cbm_clu->filter_noise = False;
				}
				if(XmToggleButtonGetState(widget_array[ID_toggleB_clu_noise_filter]))
				{    
					if((!cbm_clu->filter_noise) && !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tKohn colored noise in %s actived.\n",current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_ON\n",simu->nb_it,
								current_cluster->number);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					cbm_clu->noise_active = True;
					cbm_clu->filter_noise = True;
				}
				XtSetSensitive(XtParent(widget_array[ID_toggleB_clu_noise_none]),
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_m]);
				sscanf(value,"%f",&float_value);
				if((float_value >= -200.0) && (float_value < 200.0))
				{
					if((cbm_clu->backgrd_noise_m != float_value) 
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise mean changed from %.3f to %.3f mv in %s.\n",cbm_clu->backgrd_noise_m,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_m %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,cbm_clu->backgrd_noise_m,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					cbm_clu->backgrd_noise_m = float_value;
				}
				else
				{
					sprintf(buffer,"noise mean cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_m],
					True);
				value = XmTextFieldGetString(widget_array[ID_textF_clu_noise_sd]);
				sscanf(value,"%f",&float_value);
				if((float_value >= 0.0) && (float_value <= 200.0))
				{
					if((cbm_clu->backgrd_noise_sd != float_value)
						&& !strstr(old_file,".tms"))
					{
						sprintf(buffer,"\tnoise standard deviation changed from %.3f to %.3f mv in %s.\n",cbm_clu->backgrd_noise_sd,
							float_value,current_cluster->name);
						out_buf = bufferize(out_buf,buffer);
						if(simu->nb_it > 0)
						{
							sprintf(buffer,"%lu clu %d CBNoise_sd %.3f > %.3f\n",simu->nb_it,
								current_cluster->number,cbm_clu->backgrd_noise_sd,
									float_value);
							mod_buffer = bufferize(mod_buffer,buffer);
						}
					}
					cbm_clu->backgrd_noise_sd = float_value;
				}
				else
				{
					sprintf(buffer,"noise standard deviation cannot be %.3f...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return -1;
				}
				free(value);
				XtSetSensitive(widget_array[ID_textF_clu_noise_sd],
					True);
			}
			if(current_cluster->model == 'V')
			{
				/* default values */
				init_cluster_changes(ID_form_cluster_general);
			}
			
			if(out_buf)
			{
				sprintf(buffer,"\n\nGeneral parameter changes for %s...",current_cluster->name);
				_output(buffer);
				_output(out_buf);
				free(out_buf);
				out_buf = NULL;
			}
			
			break;
	
		default:
			return -1;
	}
	
	return 0;	
	
}

/*********************************************************************************
	Function for changing type of cluster
*********************************************************************************/
void change_cluster_type(void)
{

	char *new_file;
	char mask[10];
	char buffer[256];
	char old_file[256];
	char message[256];
	int i,j;
	XmString string;
	PUM old_pum;
	CBM old_cbm;
       	LIM old_lim;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
       	XXM old_xxm;
/***********************       STOP     ***********************/
#endif
	BUM old_bum;
	
	
	strcpy(old_file,current_cluster->file_name);
	
	
	switch(current_cluster->model)
	{
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
		case 'X':
			strcpy(mask,"*.X_unit");
			memcpy(&old_xxm,current_cluster->clu_data,sizeof(XXM));
			break;
/***********************       STOP     ***********************/
#endif
		case 'L':
			strcpy(mask,"*.L_unit");
			memcpy(&old_lim,current_cluster->clu_data,sizeof(LIM));
			break;
		case 'B':
			strcpy(mask,"*.B_unit");
			memcpy(&old_bum,current_cluster->clu_data,sizeof(BUM));
			break;
		case 'P':
			strcpy(mask,"*.P_unit");
			memcpy(&old_pum,current_cluster->clu_data,sizeof(PUM));
			break;
		case 'G':
			strcpy(mask,"*.G_unit");
			memcpy(&old_cbm,current_cluster->clu_data,sizeof(CBM));
			break;
		case 'V':
			strcpy(mask,"*.tms");
		default:
			strcpy(mask,"*");
			break;
	}
	
	sprintf(buffer,"Choose a new neuron file for %s:",current_cluster->name);

	new_file = display_fileselect(widget_array[ID_form_cluster_general],0,mask,buffer,get_dir_name());
	
	if (verification(new_file)==-1)
	  {
	    sprintf(message,"\007Invalid selection!...");
	    display_error(widget_array[ID_sim_mainW],message,"Access Error");
	    return ;
	  }

	if(!new_file)
		return;
	
	for(j = 0 ; j < simu->nb_clusters ; j++)
	{
		if((j!=i) && !strcmp(sim_clusters[j]->file_name,new_file))
		{
			
			sprintf(buffer,"File %s already used by cluster %d...",
				new_file,sim_clusters[j]->number);
			display_error(widget_array[ID_sim_mainW],buffer,"Invalid File Name !");
			return;
		}
	}
	if(current_cluster->model == 'V')
	{
		if(((VIRTUAL *)current_cluster->clu_data)->fp_tms)
		{
			fclose(((VIRTUAL *)current_cluster->clu_data)->fp_tms);
			((VIRTUAL *)current_cluster->clu_data)->fp_tms = NULL;
		}
	}
	free(current_cluster->clu_data);
	current_cluster->clu_data = NULL;
	
	while(read_clu_unit(new_file,current_cluster->number - 1))
	{

		sprintf(buffer,"Neuron file %s doesn't exist. \nDo you want to specify an other existing file ?",
			new_file);
		if(display_question(widget_array[ID_sim_mainW],buffer,
			"Neuron File Recovery") == YES_ANSWER)
		{
			sprintf(buffer,"Choose an existing neuron file:");
			new_file = display_fileselect(widget_array[ID_sim_mainW],0,mask,buffer,
				get_dir_name());
			if(!new_file)
			{
				read_clu_unit(old_file,current_cluster->number - 1);
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				if(current_cluster->model == 'X')
				{
					((XXM *)current_cluster->clu_data)->noise_active = old_xxm.noise_active;
					((XXM *)current_cluster->clu_data)->filter_noise = old_xxm.filter_noise;
					if(!((XXM *)current_cluster->clu_data)->backgrd_noise_m)
					{
						((XXM *)current_cluster->clu_data)->backgrd_noise_m = 
							old_xxm.backgrd_noise_m;
					}
					else
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
						sprintf(buffer,"%.3f",
						((XXM *)current_cluster->clu_data)->backgrd_noise_m);
						XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
						buffer);
					}
					if(!((XXM *)current_cluster->clu_data)->backgrd_noise_sd)
					{
						((XXM *)current_cluster->clu_data)->backgrd_noise_sd = 
							old_xxm.backgrd_noise_sd;


					}
					else 
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
						sprintf(buffer,"%.3f",
						((XXM *)current_cluster->clu_data)->backgrd_noise_sd);
						XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
						buffer);
					}
				}

/***********************       STOP     ***********************/
#endif
				if(current_cluster->model == 'L')
				{
					((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
					((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
					if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
					{
						((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
							old_lim.backgrd_noise_m;
					}
					else
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
						sprintf(buffer,"%.3f",
						((LIM *)current_cluster->clu_data)->backgrd_noise_m);
						XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
						buffer);
					}
					if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
					{
						((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
							old_lim.backgrd_noise_sd;

					}
					else 
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
						sprintf(buffer,"%.3f",
						((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
						XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
						buffer);
					}
				}

				if(current_cluster->model == 'B')
				{
					((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
					((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
					if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
					{
						((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
							old_bum.backgrd_noise_m;
					}
					else
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
						sprintf(buffer,"%.3f",
						((BUM *)current_cluster->clu_data)->backgrd_noise_m);
						XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
						buffer);
					}
					if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
					{
						((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
							old_bum.backgrd_noise_sd;


					}
					else 
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
						sprintf(buffer,"%.3f",
						((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
						XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
						buffer);
					}
				}
				if(current_cluster->model == 'P')
				{
					((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
					((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
					if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
					{
						((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
							old_pum.backgrd_noise_m;
					}
					else
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
						sprintf(buffer,"%.3f",
						((PUM *)current_cluster->clu_data)->backgrd_noise_m);
						XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
						buffer);
					}
					if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
					{
						((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
							old_pum.backgrd_noise_sd;


					}
					else 
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
						sprintf(buffer,"%.3f",
						((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
						XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
						buffer);
					}
				}
				if(current_cluster->model == 'G')
				{
					((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
					if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
					{
						((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
							old_cbm.backgrd_noise_m;
					}
					else
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
						sprintf(buffer,"%.3f",
						((CBM *)current_cluster->clu_data)->backgrd_noise_m);
						XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
						buffer);
					}
					if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
					{
						((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
							old_cbm.backgrd_noise_sd;
					}
					else 
					{
						XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
						sprintf(buffer,"%.3f",
						((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
						XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
						XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
						buffer);
					}
				}
				return;
			}
			for(j=0 ; j<simu->nb_clusters ; j++)
			{
				if((j!=i) && !strcmp(sim_clusters[j]->file_name,new_file))
				{
					read_clu_unit(old_file,current_cluster->number - 1);
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
					if(current_cluster->model == 'X')
					{
						((XXM *)current_cluster->clu_data)->noise_active = old_xxm.noise_active;
						((XXM *)current_cluster->clu_data)->filter_noise = old_xxm.filter_noise;
						if(!((XXM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((XXM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_xxm.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((XXM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((XXM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((XXM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_xxm.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((XXM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}

/***********************       STOP     ***********************/
#endif					
					if(current_cluster->model == 'L')
					{
						((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
						((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_lim.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_lim.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}

					if(current_cluster->model == 'B')
					{
						((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
						((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_bum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_bum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'P')
					{
						((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
						((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_pum.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_pum.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					if(current_cluster->model == 'G')
					{
						((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
								old_cbm.backgrd_noise_m;
						}
						else
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_m);
							XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
							buffer);
						}
						if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
						{
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
								old_cbm.backgrd_noise_sd;
						}
						else 
						{
							XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
							sprintf(buffer,"%.3f",
							((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
							XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
							XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
							buffer);
						}
					}
					return;
				}
			}
		}
		else
		{
			read_clu_unit(old_file,current_cluster->number - 1);
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			if(current_cluster->model == 'X')
			{
				((XXM *)current_cluster->clu_data)->noise_active = old_xxm.noise_active;
				((XXM *)current_cluster->clu_data)->filter_noise = old_xxm.filter_noise;
				if(!((XXM *)current_cluster->clu_data)->backgrd_noise_m)
				{
					((XXM *)current_cluster->clu_data)->backgrd_noise_m = 
						old_xxm.backgrd_noise_m;
				}
				else
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
					sprintf(buffer,"%.3f",
					((XXM *)current_cluster->clu_data)->backgrd_noise_m);
					XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				}
				if(!((XXM *)current_cluster->clu_data)->backgrd_noise_sd)
				{
					((XXM *)current_cluster->clu_data)->backgrd_noise_sd = 
						old_xxm.backgrd_noise_sd;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
					sprintf(buffer,"%.3f",
					((XXM *)current_cluster->clu_data)->backgrd_noise_sd);
					XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				}
			}

/***********************       STOP     ***********************/
#endif 

			if(current_cluster->model == 'L')
			{
				((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
				((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
				if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
				{
					((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
						old_lim.backgrd_noise_m;
				}
				else
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
					sprintf(buffer,"%.3f",
					((LIM *)current_cluster->clu_data)->backgrd_noise_m);
					XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				}
				if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
				{
					((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
						old_lim.backgrd_noise_sd;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
					sprintf(buffer,"%.3f",
					((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
					XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				}
			}

			if(current_cluster->model == 'B')
			{
				((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
				((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
				if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
				{
					((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
						old_bum.backgrd_noise_m;
				}
				else
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
					sprintf(buffer,"%.3f",
					((BUM *)current_cluster->clu_data)->backgrd_noise_m);
					XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				}
				if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
				{
					((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
						old_bum.backgrd_noise_sd;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
					sprintf(buffer,"%.3f",
					((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
					XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				}
			}
			if(current_cluster->model == 'P')
			{
				((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
				((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
				if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
				{
					((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
						old_pum.backgrd_noise_m;
				}
				else
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
					sprintf(buffer,"%.3f",
					((PUM *)current_cluster->clu_data)->backgrd_noise_m);
					XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				}
				if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
				{
					((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
						old_pum.backgrd_noise_sd;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
					sprintf(buffer,"%.3f",
					((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
					XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				}
			}
			if(current_cluster->model == 'G')
			{
				((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
				if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
				{
					((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
						old_cbm.backgrd_noise_m;
				}
				else
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
					sprintf(buffer,"%.3f",
					((CBM *)current_cluster->clu_data)->backgrd_noise_m);
					XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
					buffer);
				}
				if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
				{
					((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
						old_cbm.backgrd_noise_sd;
				}
				else 
				{
					XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
					sprintf(buffer,"%.3f",
					((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
					XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
					XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
					buffer);
				}
			}
			return;
		}
	}
	XmTextFieldRemove(widget_array[ID_textF_cluster_file_name]);
	sprintf(buffer,"%s",current_cluster->file_name);
	XmTextReplace(widget_array[ID_textF_cluster_file_name],0,
	XmTextGetLastPosition(widget_array[ID_textF_cluster_file_name]),
		buffer);
	
	sprintf(buffer,"%s (%s)",current_cluster->name,current_cluster->file_name);
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(clu_w[current_cluster->number - 1],XmNlabelString,string,NULL);
	XmStringFree(string);
	
	sprintf(buffer,"\n\nNeuron file from %s changed...\n\tfrom %s \n\tto %s.",
		current_cluster->name,old_file,current_cluster->file_name);
	
	_output(buffer);

	if(simu->nb_it > 0)
	{
		sprintf(buffer,"%lu clu %d NEURON_FILE\n",simu->nb_it,
			current_cluster->number);
		mod_buffer = bufferize(mod_buffer,buffer);
	}
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	if(current_cluster->model == 'X')
	{
		((XXM *)current_cluster->clu_data)->noise_active = old_xxm.noise_active;
		((XXM *)current_cluster->clu_data)->filter_noise = old_xxm.filter_noise;
		if(!((XXM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((XXM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_xxm.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((XXM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((XXM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((XXM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_xxm.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((XXM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}
/***********************       STOP     ***********************/
#endif

	if(current_cluster->model == 'L')
	{
		((LIM *)current_cluster->clu_data)->noise_active = old_lim.noise_active;
		((LIM *)current_cluster->clu_data)->filter_noise = old_lim.filter_noise;
		if(!((LIM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((LIM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_lim.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((LIM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((LIM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((LIM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_lim.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((LIM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}

	if(current_cluster->model == 'B')
	{
		((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
		((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
		if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_bum.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((BUM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_bum.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}

	if(current_cluster->model == 'P')
	{
		((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
		((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
		if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_pum.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((PUM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_pum.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}

	if(current_cluster->model == 'G')
	{
		((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
		if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_cbm.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((CBM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_cbm.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}
}

/*******************************************************************************
	Function for changing simulation general run time parameters
*******************************************************************************/
void change_simu_parameter(int msg)
{

	NEURON *neuron_tmp;
	char *answer;
	char *pt;
	char buffer[256];
	char value[256];
	char unit[256];
	float float_value;
	int l_value, i, j, i_r;
	CBM *clu_cbm;
	PUM *clu_pum;
	LIM *clu_lim;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	XXM *clu_xxm;
/***********************       STOP     ***********************/
#endif
	BUM *clu_bum;

#if 0
	if (!simu->time_step == 0.)
		simu->time_step = 1.;
#endif

	switch(msg)
	{
		case ID_pushB_it_count:
			sprintf(value,"%lu it",simu->count_show_it);
			sprintf(buffer,"Type an new value (in ms (m) or iteration (i)):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Show iteration counter each: ");
			if(!answer)
				return;
			
			sscanf(answer,"%s %s",value,unit);
			if(pt = strrchr(value,'.'))
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->count_show_it  = (unsigned long)nintf(float_value / simu->time_step);
				simu->count_show_ms  = (unsigned long)float_value;
				simu->look_at_it_count = FALSE;
				simu->look_at_ms_count = TRUE;
			}
			else
			{
				sscanf(value,"%lu",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->count_show_it = (unsigned long)l_value;
				simu->count_show_ms = (float)simu->count_show_it * simu->time_step;
				simu->look_at_it_count = TRUE;
				simu->look_at_ms_count = FALSE;
			}
			sprintf(buffer,"\nIteration number shown each  %lu iterations (%.3f ms).",
				simu->count_show_it, simu->count_show_ms);
			_output(buffer);
			break;
			
		
		case ID_pushB_watch_sampling:
			sprintf(value,"%.3f",simu->nb_step_watch_ms);
			sprintf(buffer,"Type an new value (in ms (m) or iteration (i)):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Watch activity each: ");
			if(!answer)
				return;

			if(pt = strrchr(value,'.'))
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->nb_step_watch_it  = (unsigned long)nintf(float_value / simu->time_step);
				simu->nb_step_watch_ms  = (unsigned long)float_value;
			}
			else
			{
				sscanf(value,"%d",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->nb_step_watch_it = (unsigned long)l_value;
				simu->nb_step_watch_ms = (float)simu->nb_step_watch_it * simu->time_step;
				sprintf(buffer,"\nActivity shown each  %.3f ms (%lu it).",
					simu->nb_step_save_ms, simu->nb_step_save_it);
				_output(buffer);
			}
			break;

		case ID_pushB_save_pmb:
			sprintf(value,"%.3f",simu->nb_step_save_ms);
			sprintf(buffer,"Type an new value (in ms (m) or iteration (i)):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Save membrane potential each: ");
			if(!answer)
				return;
			
			sscanf(answer,"%s %s",value,unit);
			if(pt = strrchr(value,'.'))
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->nb_step_save_it  = (unsigned long)nintf(float_value / simu->time_step);
				simu->nb_step_save_ms  = (unsigned long)float_value;
			}
			else
			{
				sscanf(value,"%d",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->nb_step_save_it = (unsigned long)l_value;
				simu->nb_step_save_ms = (float)simu->nb_step_save_it * simu->time_step;
				_output(buffer);
			}
			sprintf(buffer,"\nMembrane potential saved each  %.3f ms (%lu it).",
				simu->nb_step_save_ms, simu->nb_step_save_it);
			_output(buffer);
			break;			

		case ID_pushB_first_stop:
			sprintf(value,"%u iterations (%.3f ms)",simu->nb_next_it,
				                         (simu->nb_next_it * simu->time_step));
			sprintf(buffer,"Type an new value (in ms or iteration):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Time between two Stops");
			if(!answer)
				return;
			
			sscanf(answer,"%s %s",value,unit);
			if(pt = strrchr(value,'.'))  /* Valeur en ms */
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}

				simu->nb_next_it = (unsigned long)(float_value / simu->time_step);
				simu->next_stop += simu->nb_next_it;
				simu->stop_at_ms = (float)(simu->next_stop) * simu->time_step;
				sprintf(buffer,"\nTime between stops changed: %.3f ms <=> %lu iterations.",
					float_value,simu->nb_next_it);
				_output(buffer);
			}
			else  /* Valeur en iterations */
			{
				sscanf(value,"%d",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				simu->nb_next_it = (unsigned long)l_value;
				simu->next_stop += simu->nb_next_it;
				simu->stop_at_ms = (float)(simu->next_stop) * simu->time_step;
				sprintf(buffer,"\nSimulation next stop changed: %.3f ms <=> %lu iterations.",
					simu->stop_at_ms,simu->next_stop);
				_output(buffer);
			}
			if(widget_array[ID_text_stop_main])
			{
			        XmTextFieldRemove(widget_array[ID_text_stop_main]);
				sprintf(buffer,"%.3f ms",simu->stop_at_ms);
				XmTextReplace(widget_array[ID_text_stop_main],0,
					      XmTextGetLastPosition(widget_array[ID_text_stop_main]), buffer);
			}
			
			break;
    	case ID_pushB_last_stop:
    		sprintf(value,"%.3f ms",simu->end_at_ms);
			sprintf(buffer,"Type an new value (in ms or iteration):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Simulation Last Stop");
			if(!answer)
				return;
			
			sscanf(answer,"%s %s",value,unit);
			if(pt = strrchr(value,'.'))
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				if(float_value <= simu->time_ms)
				{
					sprintf(buffer,"Sorry, %.3f is out of range...",float_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return;
				}
				simu->end_at_ms = float_value;
				simu->last_it = (unsigned long)nintf(simu->end_at_ms / simu->time_step);
				sprintf(buffer,"\nSimulation last stop changed: %.3f ms <=> %lu iterations.",
					simu->end_at_ms, simu->last_it);
				_output(buffer);
			}
			else
			{
				sscanf(value,"%d",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				if(l_value <= simu->nb_it) 
				{
					sprintf(buffer,"Sorry, %d is out of range...",l_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return;
				}
				simu->last_it = (unsigned long)l_value;
				simu->end_at_ms = (float)simu->last_it * simu->time_step;
				sprintf(buffer,"\nSimulation last stop changed: %.3f ms <=> %lu iterations.",
					simu->end_at_ms,simu->last_it);
				_output(buffer);
			}
			
			break;
    	case ID_pushB_time_step:
    		sprintf(value,"%f ms",simu->time_step);
		sprintf(buffer,"Type an new value (xx.xx m for ms for time by step,\nor xx i for iteration by ms):");
		answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
			"Simulation Time Step");
		if(!answer)
			return;

		for (j = 0; j < simu->nb_nucleus; j++)
		{
			sim_nucleus[j]->backgrd_noise_sd *= sqrt(simu->time_step);
		} /* fin du for */

		sscanf(answer,"%s %s",value,unit);
		if(pt = strrchr(value,'.'))
		{
			sscanf(value,"%f",&float_value);
			if(tolower(*unit) == 'i')
			{
				sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
				display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
				return;
			}
			else if(tolower(*unit) != 'm')
			{
				sprintf(buffer,"Is this float value in ms ?");
				if(display_question(widget_array[ID_sim_mainW],buffer,
					"Unit Confirmation")== NO_ANSWER)
				{
					sprintf(buffer,"Sorry, float value is only compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
			}
			if((float_value <= 0.0) || (float_value>1.0))
			{
				sprintf(buffer,"Sorry, %.3f is out of range...",float_value);
				display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
				return;
			}

			simu->time_step = float_value;

			simu->next_stop = (unsigned long)nintf(simu->stop_at_ms / simu->time_step);
			simu->last_it = (unsigned long)nintf(simu->end_at_ms / simu->time_step);

			sprintf(buffer,"\nSimulation time step changed: %.3f ms/it <=> %.3f it/ms.", simu->time_step, (1./simu->time_step));
			_output(buffer);
			sprintf(buffer,"Simulation next stop correspondance: %.3f ms <=> %lu iterations.",
				simu->stop_at_ms, simu->next_stop);
			_output(buffer);
			sprintf(buffer,"Simulation last stop correspondance: %.3f ms <=> %lu iterations.",
				simu->end_at_ms, simu->last_it);
			_output(buffer);
		}
		else
		{
			sscanf(value,"%d",&l_value);
			simu->time_step = (float)l_value;

			simu->next_stop = (unsigned long)nintf(simu->stop_at_ms / simu->time_step);
			simu->last_it = (unsigned long)nintf(simu->end_at_ms / simu->time_step);
			
			sprintf(buffer,"\nSimulation time step changed: %f ms/it <=> %.3f it/ms.",
				simu->time_step,1./simu->time_step);
			_output(buffer);
			sprintf(buffer,"Simulation next stop correspondance: %.3f ms <=> %lu iterations.",
				simu->stop_at_ms,simu->next_stop);
			_output(buffer);
			sprintf(buffer,"Simulation last stop correspondance: %.3f ms <=> %lu iterations.",
				simu->end_at_ms,simu->last_it);
			_output(buffer);

		}
#if 0
		simu->nb_next_it = simu->next_stop - simu->nb_it;
#endif
		simu->nb_step_spk = (unsigned long)nintf(simu->spike_width / simu->time_step);
		simu->nb_step_save_it = 1;
		simu->nb_step_watch_it = 1;
		simu->count_show_it = 1;
		simu->nb_step_save_ms = simu->time_step;
		simu->nb_step_watch_ms = simu->time_step;
		simu->count_show_ms = simu->time_step;

		/*
		* normalisation des cstes de temps sur le nb de pas
		* d'iteration par ms
		*/
		for (j = 0; j < simu->nb_clusters; j++)
		{
			switch(sim_clusters[j]->model)
			{
				case 'G':
					clu_cbm = (CBM *)sim_clusters[j]->clu_data;

				/*	clu_cbm->backgrd_noise_sd /= sqrt(simu->time_step); */

					clu_cbm->tau_a_epsp = clu_cbm->tau_attack_epsp / simu->time_step;
					clu_cbm->tau_d_epsp = clu_cbm->tau_decay_epsp / simu->time_step;
					clu_cbm->tau_a_ipsp = clu_cbm->tau_attack_ipsp / simu->time_step;
					clu_cbm->tau_d_ipsp = clu_cbm->tau_decay_ipsp / simu->time_step;
					clu_cbm->tau_a_NMDA = clu_cbm->tau_attack_NMDA / simu->time_step;
					clu_cbm->tau_d_NMDA = clu_cbm->tau_decay_NMDA / simu->time_step;


					clu_cbm->i_tau_attack_epsp = 1./clu_cbm->tau_a_epsp;
					clu_cbm->i_tau_decay_epsp = 1./clu_cbm->tau_d_epsp;
					clu_cbm->i_tau_attack_ipsp = 1./clu_cbm->tau_a_ipsp;
					clu_cbm->i_tau_decay_ipsp = 1./clu_cbm->tau_d_ipsp;
					clu_cbm->pente_spike = clu_cbm->spike_slope * simu->time_step;
					for (i_r = 0; i_r < clu_cbm->nb_receptors; i_r++)
					{
						clu_cbm->Receptors[i_r].tau_a_psp = clu_cbm->Receptors[i_r].tau_attack / simu->time_step;
						clu_cbm->Receptors[i_r].tau_d_psp = clu_cbm->Receptors[i_r].tau_decay / simu->time_step;
					}

					break;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				case 'X':
					clu_xxm = (XXM *)sim_clusters[j]->clu_data;
					
					clu_xxm->backgrd_n_sd = clu_xxm->backgrd_noise_sd * sqrt(simu->time_step);

					clu_xxm->tau_a_epsp = clu_xxm->tau_attack_epsp / simu->time_step;
					clu_xxm->tau_d_epsp = clu_xxm->tau_decay_epsp / simu->time_step;
					clu_xxm->tau_a_ipsp = clu_xxm->tau_attack_ipsp / simu->time_step;
					clu_xxm->tau_d_ipsp = clu_xxm->tau_decay_ipsp / simu->time_step;

					clu_xxm->tau_i_mb_pot = clu_xxm->tau_mb_pot / simu->time_step;
					clu_xxm->tau_i_seuil = clu_xxm->tau_seuil / simu->time_step;

					clu_xxm->i_tau_attack_epsp = 1./clu_xxm->tau_a_epsp;
					clu_xxm->i_tau_decay_epsp = 1./clu_xxm->tau_d_epsp;
					clu_xxm->i_tau_attack_ipsp = 1./clu_xxm->tau_a_ipsp;
					clu_xxm->i_tau_decay_ipsp = 1./clu_xxm->tau_d_ipsp;

					clu_xxm->i_tau_mb_pot = 1./clu_xxm->tau_i_mb_pot;
					clu_xxm->i_tau_seuil = 1./clu_xxm->tau_i_seuil;
					break;

/***********************       STOP     ***********************/
#endif
				case 'L':
					clu_lim = (LIM *)sim_clusters[j]->clu_data;
					
					clu_lim->backgrd_n_m = clu_lim->backgrd_noise_m * simu->time_step;
					clu_lim->backgrd_n_sd = clu_lim->backgrd_noise_sd * sqrt(simu->time_step);
					clu_lim->i_tau_mb_pot = 1. - (simu->time_step / clu_lim->tau_mb_pot);
					clu_lim->i_tau_i_mb_pot = clu_lim->rest_pot * 
								(simu->time_step / clu_lim->tau_mb_pot);
					clu_lim->tau_i_seuil = clu_lim->tau_seuil / simu->time_step;
					clu_lim->i_tau_seuil = 1./clu_lim->tau_i_seuil;
					break;

				case 'B':
					clu_bum = (BUM *)sim_clusters[j]->clu_data;
					
					clu_bum->backgrd_n_sd = clu_bum->backgrd_noise_sd * sqrt(simu->time_step);

					clu_bum->tau_a_epsp = clu_bum->tau_attack_epsp / simu->time_step;
					clu_bum->tau_d_epsp = clu_bum->tau_decay_epsp / simu->time_step;
					clu_bum->tau_a_ipsp = clu_bum->tau_attack_ipsp / simu->time_step;
					clu_bum->tau_d_ipsp = clu_bum->tau_decay_ipsp / simu->time_step;

					clu_bum->tau_i_mb_pot = clu_bum->tau_mb_pot / simu->time_step;
					clu_bum->tau_i_seuil = clu_bum->tau_seuil / simu->time_step;
					clu_bum->shunt_i_tau = clu_bum->shunt_tau / simu->time_step;

					clu_bum->i_tau_attack_epsp = 1./clu_bum->tau_a_epsp;
					clu_bum->i_tau_decay_epsp = 1./clu_bum->tau_d_epsp;
					clu_bum->i_tau_attack_ipsp = 1./clu_bum->tau_a_ipsp;
					clu_bum->i_tau_decay_ipsp = 1./clu_bum->tau_d_ipsp;

					clu_bum->i_tau_mb_pot = 1./clu_bum->tau_i_mb_pot;
					clu_bum->i_tau_seuil = 1./clu_bum->tau_i_seuil;

					clu_bum->duree_b_min = clu_bum->duree_burst_min / simu->time_step;
					clu_bum->duree_b_max = clu_bum->duree_burst_max / simu->time_step;
					clu_bum->burst_range = clu_bum->duree_b_max - clu_bum->duree_b_min;
					clu_bum->freq_b_spike = clu_bum->freq_spike * simu->time_step;
					break;
				case 'P':
					clu_pum = (PUM *)sim_clusters[j]->clu_data;
					
					clu_pum->backgrd_n_sd = clu_pum->backgrd_noise_sd * sqrt(simu->time_step);

					clu_pum->tau_a_epsp = clu_pum->tau_attack_epsp / simu->time_step;
					clu_pum->tau_d_epsp = clu_pum->tau_decay_epsp / simu->time_step;
					clu_pum->tau_a_ipsp = clu_pum->tau_attack_ipsp / simu->time_step;
					clu_pum->tau_d_ipsp = clu_pum->tau_decay_ipsp / simu->time_step;

					clu_pum->tau_i_mb_pot = clu_pum->tau_mb_pot / simu->time_step;
					clu_pum->tau_i_seuil = clu_pum->tau_seuil / simu->time_step;
					clu_pum->shunt_i_tau = clu_pum->shunt_tau / simu->time_step;

					clu_pum->i_tau_attack_epsp = 1./clu_pum->tau_a_epsp;
					clu_pum->i_tau_decay_epsp = 1./clu_pum->tau_d_epsp;
					clu_pum->i_tau_attack_ipsp = 1./clu_pum->tau_a_ipsp;
					clu_pum->i_tau_decay_ipsp = 1./clu_pum->tau_d_ipsp;

					clu_pum->i_tau_mb_pot = 1./clu_pum->tau_i_mb_pot;
					clu_pum->i_tau_seuil = 1./clu_pum->tau_i_seuil;

					break;
				}
			} /* fin du for */
			for (j = 0; j < simu->nb_nucleus; j++)
			{
				sim_nucleus[j]->backgrd_n_sd = sim_nucleus[j]->backgrd_noise_sd * sqrt(simu->time_step);
				sim_nucleus[j]->pulse_up = (unsigned long)(sim_nucleus[j]->modul_phase * simu->time_step);
				sim_nucleus[j]->pulse_down = (unsigned long)(sim_nucleus[j]->modul_period * simu->time_step);
			} /* fin du for */
			/* neuron data allocation and initialization */
			for(i = 0 ; i < simu->nb_neurons ; i++)
			{
				neuron_tmp = sim_neurons[i];

				for(j=0 ; j<neuron_tmp->connected_neurons ; j++)
				{
					neuron_tmp->axone_step[j] = (unsigned long)
					(neuron_tmp->axone_length[j] / simu->time_step);
				}
			}
			break;
    	case ID_pushB_spike_duration:
    		sprintf(value,"%f ms",simu->spike_width);
			sprintf(buffer,"Type a new value (xxx.xx m for ms \n or xxx i for iteration):");
			answer = display_prompt(widget_array[ID_sim_mainW],buffer,value,
				"Simulation Spike Duration");
			if(!answer)
				return;
			
			sscanf(answer,"%s %s",value,unit);
			if(pt = strrchr(value,'.'))
			{
				sscanf(value,"%f",&float_value);
				if(tolower(*unit) == 'i')
				{
					sprintf(buffer,"Sorry, float value isn't compatible with iteration...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				if(tolower(*unit) != 'm')
				{
					sprintf(buffer,"Is this float value in ms ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, float value is only compatible with ms...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				if(float_value < simu->time_step)
				{
					sprintf(buffer,"Sorry, the spike width must be greater than %.3f ...",simu->time_step);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return;
				}
				
				simu->spike_width = float_value;
				simu->nb_step_spk = (unsigned long)nintf(simu->spike_width / simu->time_step);
				sprintf(buffer,"\nSimulation spike duration changed: %f ms <=> %lu iterations.",
					simu->spike_width, simu->nb_step_spk);
				_output(buffer);
			}
			else
			{
				sscanf(value,"%d",&l_value);
				if(tolower(*unit) == 'm')
				{
					sprintf(buffer,"Sorry, integer value isn't compatible with ms...");
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
					return;
				}
				else if(tolower(*unit) != 'i')
				{
					sprintf(buffer,"Is this integer value in iteration ?");
					if(display_question(widget_array[ID_sim_mainW],buffer,
						"Unit Confirmation")== NO_ANSWER)
					{
						sprintf(buffer,"Sorry, integer value is only compatible with iteration...");
						display_error(widget_array[ID_sim_mainW],buffer,"Bad Unit !");
						return;
					}
				}
				if(l_value <= simu->nb_it) 
				{
					sprintf(buffer,"Sorry, %d is out of range...",l_value);
					display_error(widget_array[ID_sim_mainW],buffer,"Bad Value !");
					return;
				}
				
				simu->nb_step_spk = (unsigned long)l_value;
				simu->spike_width = (float)simu->nb_step_spk * simu->time_step;
				sprintf(buffer,"\nSimulation spike duration changed: %.3f ms <=> %lu iterations.",
					simu->spike_width,simu->nb_step_spk);
				_output(buffer);
			}
    		break;
    	default:
    		break;
	
	}


}

/*******************************************************************************
	Function for changing cluster type
*******************************************************************************/



void reload_cluster_type(void)
{

	char *new_file;
	char mask[10];
	char buffer[256];
	char old_file[256];
	int i,j;
	XmString string;
	PUM old_pum;
	CBM old_cbm;
	BUM old_bum;
	
	strcpy(old_file,current_cluster->file_name);
	current_cluster->clu_data = NULL;
	read_clu_unit(old_file,current_cluster->number - 1);
	
	XmTextFieldRemove(widget_array[ID_textF_cluster_file_name]);
	sprintf(buffer,"%s",current_cluster->file_name);
	XmTextReplace(widget_array[ID_textF_cluster_file_name],0,
		      XmTextGetLastPosition(widget_array[ID_textF_cluster_file_name]),
		      buffer);
	
	sprintf(buffer,"%s (%s)",current_cluster->name,current_cluster->file_name);
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(clu_w[current_cluster->number - 1],XmNlabelString,string,NULL);
	XmStringFree(string);
	
	sprintf(buffer,"\n\nNeuron file from %s changed...\n\tfrom %s \n\tto %s.",
		current_cluster->name,old_file,current_cluster->file_name);
	
	_output(buffer);
	
	if(simu->nb_it > 0)
	{
		sprintf(buffer,"%lu clu %d NEURON_FILE\n",simu->nb_it,
			current_cluster->number);
		mod_buffer = bufferize(mod_buffer,buffer);
	}
	if(current_cluster->model == 'B')
	{
		((BUM *)current_cluster->clu_data)->noise_active = old_bum.noise_active;
		((BUM *)current_cluster->clu_data)->filter_noise = old_bum.filter_noise;
		if(!((BUM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((BUM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_bum.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((BUM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((BUM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((BUM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_bum.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((BUM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}
	if(current_cluster->model == 'P')
	{
		((PUM *)current_cluster->clu_data)->noise_active = old_pum.noise_active;
		((PUM *)current_cluster->clu_data)->filter_noise = old_pum.filter_noise;
		if(!((PUM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((PUM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_pum.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((PUM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((PUM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((PUM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_pum.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((PUM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}
	if(current_cluster->model == 'G')
	{
		((CBM *)current_cluster->clu_data)->noise_active = old_cbm.noise_active;
		if(!((CBM *)current_cluster->clu_data)->backgrd_noise_m)
		{
			((CBM *)current_cluster->clu_data)->backgrd_noise_m = 
				old_cbm.backgrd_noise_m;
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_m]);
			sprintf(buffer,"%.3f",
			((CBM *)current_cluster->clu_data)->backgrd_noise_m);
			XmTextReplace(widget_array[ID_textF_clu_noise_m],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_m]),
			buffer);
		}
		if(!((CBM *)current_cluster->clu_data)->backgrd_noise_sd)
		{
			((CBM *)current_cluster->clu_data)->backgrd_noise_sd = 
				old_cbm.backgrd_noise_sd;
		}
		else 
		{
			XmTextFieldRemove(widget_array[ID_textF_clu_noise_sd]);
			sprintf(buffer,"%.3f",
			((CBM *)current_cluster->clu_data)->backgrd_noise_sd);
			XmTextReplace(widget_array[ID_textF_clu_noise_sd],0,
			XmTextGetLastPosition(widget_array[ID_textF_clu_noise_sd]),
			buffer);
		}
	}
			

}

/*******************************************************************************
	Function for configuring neurons according to the pre-defined 
	nucleus and neurons parameters
*******************************************************************************/
void configure_neurons(int first_config)
{

	/* first_config == True => first config */

	int i, j, k, ij;
	float  v_moins;
	NEURON *neuron_tmp;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	XXM_DYNAMIC *xxm_dyn;
	XXM *clu_xxm;
/***********************       STOP     ***********************/
#endif
	LIM_DYNAMIC *lim_dyn;
	LIM *clu_lim;
	BUM_DYNAMIC *bum_dyn;
	BUM *clu_bum;
	PUM_DYNAMIC *pum_dyn;
	PUM *clu_pum;
	CBM_DYNAMIC *cbm_dyn;
	CBM *clu_cbm;
	float axone_start;
	long set1;
	long set2;

	set1 = time(0);
	
	/* allocate, initialize neuron data according to cluster type */
	/* give the appropriate updating function for each cluster */
	/* give the appriopriate learning fonction in case of weight fluc. */
	/* make corresponding neurons alive if necessary */		
	
	if(!batch)
		status("Running (press Stop to interrupt)...");

	if(first_config)
	{
		simu->nb_it = 0;
		simu->time_ms = 0.0;
	}
	else
	        return;
	
	/* update function */
	for(i=0 ; i<simu->nb_clusters ; i++)
	{
		switch(sim_clusters[i]->model)
		{
			case 'V':
				sim_clusters[i]->update_soma = update_V_res;
				break;
			case 'G':
				if (simu->use_ntr)
				{
					if (((CBM *)sim_clusters[i]->clu_data)->model_type == HH_SIMPLE_MODEL)
						sim_clusters[i]->update_soma = update_G_res_use_ntr_simple;
					else
						sim_clusters[i]->update_soma = update_G_res_use_ntr_full;
				}
				else
				{ 
					if (((CBM *)sim_clusters[i]->clu_data)->model_type == HH_SIMPLE_MODEL)
						sim_clusters[i]->update_soma = update_G_res_no_ntr_simple;
					else
						sim_clusters[i]->update_soma = update_G_res_no_ntr_full;
				}
				break;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			case 'X':
				sim_clusters[i]->update_soma = update_X_res;
				/* .... */
				break;
/***********************       STOP     ***********************/
#endif
			case 'L':
				sim_clusters[i]->update_soma = update_L_res;
				/* quelques mesures de prudence... */
				clu_lim = (LIM *)sim_clusters[i]->clu_data;
				break;
			case 'B':
				sim_clusters[i]->update_soma = update_B_res;
				/* quelques mesures de prudence... */
				clu_bum = (BUM *)sim_clusters[i]->clu_data;
				if (clu_bum->rev_pot_pos - clu_bum->rest_pot)
					clu_bum->K_rev_p = ((1. /
					(clu_bum->rev_pot_pos - clu_bum->rest_pot)));
				else
					clu_bum->K_rev_p = 1.;
				if (clu_bum->rev_pot_neg - clu_bum->rest_pot)
					clu_bum->K_rev_n = ((1. /
					(clu_bum->rev_pot_neg - clu_bum->rest_pot)));
				else
					clu_bum->K_rev_n = 1.;	
				
				clu_bum->burst_overshoot = clu_bum->burst_post_spike - clu_bum->seuil_burst;

				clu_bum->b_v_plus = (clu_bum->rev_pot_pos - clu_bum->seuil_burst);
				v_moins = (clu_bum->rev_pot_neg - clu_bum->seuil_burst);
				clu_bum->b_A = (1./ clu_bum->b_v_plus) - (1. / v_moins);
				clu_bum->b_v_rap = clu_bum->b_v_plus / v_moins;
				
				break;
			case 'P':
				sim_clusters[i]->update_soma = update_P_res;
				/* quelques mesures de prudence... */
				clu_pum = (PUM *)sim_clusters[i]->clu_data;
				if (clu_pum->rev_pot_pos - clu_pum->rest_pot)
					clu_pum->K_rev_p = ((1. /
					(clu_pum->rev_pot_pos - clu_pum->rest_pot)));
				else
					clu_pum->K_rev_p = 1.;
				if (clu_pum->rev_pot_neg - clu_pum->rest_pot)
					clu_pum->K_rev_n = ((1. /
					(clu_pum->rev_pot_neg - clu_pum->rest_pot)));
				else
					clu_pum->K_rev_n = 1.;	
				
				clu_pum->v_plus = (clu_pum->rev_pot_pos - clu_pum->rest_pot);
				v_moins = (clu_pum->rev_pot_neg - clu_pum->rest_pot);
				clu_pum->A = (1./ clu_pum->v_plus) - (1. / v_moins);
				clu_pum->v_rap = clu_pum->v_plus / v_moins;
				break;
		}
		sim_clusters[i]->spike_duration = (float)simu->nb_step_spk;
	}

	/* learning function */
	for(i=0 ; i<simu->nb_nucleus ; i++)
	{
		
		switch(sim_nucleus[i]->law)
		{
			case 'h':
				sim_nucleus[i]->learning = hebb_learn;
				break;
			
			default:
				break;
		}
		init_inp_interval(i);
	}
/*	
	if(!first_config)
		return;
*/		
	set2 = set1;
	set1 = -set1;
	ran1(&set1);
	/* neuron data allocation and initialization */
	for(i=0 ; i<simu->nb_neurons ; i++)
	{
		neuron_tmp = sim_neurons[i];
		axone_start = 0.0;
/*
		neuron_tmp->radius = neuron_tmp->its_cluster->soma_diam_m / 2.;
*/
		for(j=0 ; j<neuron_tmp->connected_neurons ; j++)
		{
			axone_start += neuron_tmp->axone_length[j];
			neuron_tmp->axone_step[j] = (unsigned long)
				(neuron_tmp->axone_length[j] / simu->time_step);
		}
		if(neuron_tmp->connected_neurons)
		{
			axone_start /= neuron_tmp->connected_neurons;
			axone_start /= simu->time_step;
		}
		switch(neuron_tmp->its_cluster->model)
		{
			case 'V':
				break;
			case 'G':
				if(!neuron_tmp->neuron_data)
				{
					neuron_tmp->neuron_data = (CBM_DYNAMIC *)calloc(1, sizeof(CBM_DYNAMIC));
					cbm_dyn = (CBM_DYNAMIC *)neuron_tmp->neuron_data;
				}
				break;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			case 'X':
				if(!neuron_tmp->neuron_data)
				{
					neuron_tmp->neuron_data = (XXM_DYNAMIC *)calloc(1, sizeof(XXM_DYNAMIC));
					xxm_dyn = (XXM_DYNAMIC *)neuron_tmp->neuron_data;
				}
				break;
/***********************       STOP     ***********************/
#endif
			case 'L':
				if(!neuron_tmp->neuron_data)
				{
					neuron_tmp->neuron_data = (LIM_DYNAMIC *)calloc(1, sizeof(LIM_DYNAMIC));
					lim_dyn = (LIM_DYNAMIC *)neuron_tmp->neuron_data;
				}
				break;
			case 'B':
				if(!neuron_tmp->neuron_data)
				{
					neuron_tmp->neuron_data = (BUM_DYNAMIC *)calloc(1, sizeof(BUM_DYNAMIC));
					bum_dyn = (BUM_DYNAMIC *)neuron_tmp->neuron_data;
				}
				break;
			case 'P':
				if(!neuron_tmp->neuron_data)
				{
					neuron_tmp->neuron_data = (PUM_DYNAMIC *)calloc(1, sizeof(PUM_DYNAMIC));
					pum_dyn = (PUM_DYNAMIC *)neuron_tmp->neuron_data;
				}
				break;
		}
		if(!neuron_tmp->neuron_data)
			continue;
		if(first_config && (tolower(neuron_tmp->its_nucleus->alive) == 'y'))
		{
			neuron_tmp->new_state = ran1(&set2) < (float)neuron_tmp->its_nucleus->prob_spike/100. ? SPIKE : REPOS;

			if(neuron_tmp->new_state)
			{
				simu->total_active_neurons++;
				neuron_tmp->its_cluster->nb_act_clu++;
				neuron_tmp->its_nucleus->nb_act_nucl++;
				neuron_tmp->decharge_lag = 1.;
				neuron_tmp->spike_time[0] = 0;
				neuron_tmp->tete = 0;
				neuron_tmp->queue = 0;
				for (j = 0; j < neuron_tmp->connected_neurons; j++)
					neuron_tmp->next_spike[j] = 0;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				if(neuron_tmp->its_cluster->model == 'X')
				{
				     /* here initialize all is necessary at the neuron level */
					xxm_dyn = (XXM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_xxm = (XXM *)neuron_tmp->its_cluster->clu_data;
					xxm_dyn->soma = clu_xxm->pot_post_spike;
					xxm_dyn->soma_post_spike = xxm_dyn->soma;
					xxm_dyn->seuil_peak = clu_xxm->seuil_post_spike 
						+ xxm_dyn->threshold;
					xxm_dyn->delta_seuil = xxm_dyn->seuil_peak - clu_xxm->seuil;
					xxm_dyn->threshold = xxm_dyn->seuil_peak;
				}

/***********************       STOP     ***********************/
#endif

				if(neuron_tmp->its_cluster->model == 'B')
				{
					bum_dyn = (BUM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_bum = (BUM *)neuron_tmp->its_cluster->clu_data;
					bum_dyn->soma = clu_bum->pot_post_spike;
					bum_dyn->soma_post_spike = bum_dyn->soma;
					bum_dyn->seuil_peak = clu_bum->seuil_post_spike 
						+ bum_dyn->threshold;
					bum_dyn->delta_seuil = bum_dyn->seuil_peak - clu_bum->seuil;
					bum_dyn->threshold = bum_dyn->seuil_peak;
					for (ij = 0; ij < MAX_BURST; ij++)
					{
					    bum_dyn->sablier[ij] = 0;
					}
					bum_dyn->nb_sabliers = 0;
					bum_dyn->in_burst = FALSE;
				}

				if(neuron_tmp->its_cluster->model == 'L')
				{
					lim_dyn = (LIM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_lim = (LIM *)neuron_tmp->its_cluster->clu_data;
					lim_dyn->soma = clu_lim->pot_post_spike;
					lim_dyn->soma_post_spike = lim_dyn->soma;
					lim_dyn->seuil_peak = clu_lim->seuil_post_spike 
						+ lim_dyn->threshold;
					lim_dyn->threshold = lim_dyn->seuil_peak;
				}

				if(neuron_tmp->its_cluster->model == 'P')
				{
					pum_dyn = (PUM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_pum = (PUM *)neuron_tmp->its_cluster->clu_data;
					pum_dyn->soma = SPIKE_TOP;
					pum_dyn->soma_post_spike = clu_pum->pot_post_spike;
					pum_dyn->seuil_peak = clu_pum->seuil_post_spike 
						+ pum_dyn->threshold;
					pum_dyn->delta_seuil = pum_dyn->seuil_peak - clu_pum->seuil;
					pum_dyn->threshold = pum_dyn->seuil_peak;
					neuron_tmp->spike = neuron_tmp->its_cluster->spike_duration;
#if DEBUG
					printf("neuron %d: %f %f\n",i+1,pum_dyn->soma,pum_dyn->threshold);
#endif

				}
				if(neuron_tmp->its_cluster->model == 'G')
				{
					cbm_dyn = (CBM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_cbm = (CBM *)neuron_tmp->its_cluster->clu_data;
				        init_G_neuron(i);
				}
#if DEBUG
				printf("Neurone %d initialise (alive avec spike)\n",i);
#endif
			}
			else
			{
				/*
				 * pour les cellules ne dechargeant pas, on prend une
				 * valeur aleatoire de duree depuis laquelle elles
				 * ont decharge
				 */
				 
				neuron_tmp->decharge_lag = (float)((int)(ran1(&set2) * 
									 (neuron_tmp->its_nucleus->old_last_spike
									  - neuron_tmp->its_nucleus->recent_last_spike)));
				neuron_tmp->decharge_lag += neuron_tmp->its_nucleus->recent_last_spike;
     	       			if(!neuron_tmp->decharge_lag)
					neuron_tmp->decharge_lag = neuron_tmp->its_nucleus->recent_last_spike;

				neuron_tmp->spike_time[0] = (unsigned long)(neuron_tmp->decharge_lag);
				neuron_tmp->tete = 0;
				neuron_tmp->queue = 1;
				for (j = 0; j < neuron_tmp->connected_neurons; j++)
					neuron_tmp->next_spike[j] = 0;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				if(neuron_tmp->its_cluster->model == 'X')
				{
					xxm_dyn = (XXM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_xxm = (XXM *)neuron_tmp->its_cluster->clu_data;
					xxm_dyn->center_noise = 0.;
					if (neuron_tmp->its_nucleus->low_mb_pot == 0.)
					  neuron_tmp->its_nucleus->low_mb_pot = clu_xxm->rest_pot - 20.;
					if (neuron_tmp->its_nucleus->up_mb_pot == 0.)
					  neuron_tmp->its_nucleus->up_mb_pot = clu_xxm->rest_pot;
					xxm_dyn->soma = neuron_tmp->its_nucleus->low_mb_pot 
					  + (fabs(neuron_tmp->its_nucleus->low_mb_pot - neuron_tmp->its_nucleus->up_mb_pot)
					     * ran1(&set2));

					/*
					xxm_dyn->soma = (clu_xxm->overshoot
							  * exp((-neuron_tmp->decharge_lag) 
							  * clu_xxm->i_tau_mb_pot)) 
							  + clu_xxm->rest_pot;
					*/
					xxm_dyn->soma_post_spike = clu_xxm->pot_post_spike;
					xxm_dyn->threshold = clu_xxm->seuil_post_spike
							* exp((-neuron_tmp->decharge_lag) 
							* clu_xxm->i_tau_seuil) /* deja un inverse */
							+ clu_xxm->seuil;
					xxm_dyn->seuil_peak = clu_xxm->seuil_post_spike
								+ clu_xxm->seuil;
				}

/***********************       STOP     ***********************/
#endif
				if(neuron_tmp->its_cluster->model == 'B')
				{
					bum_dyn = (BUM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_bum = (BUM *)neuron_tmp->its_cluster->clu_data;
					bum_dyn->center_noise = 0.;
					/*
					if (neuron_tmp->its_nucleus->low_mb_pot == 0.)
					  neuron_tmp->its_nucleus->low_mb_pot = clu_bum->pot_post_spike;
					if (neuron_tmp->its_nucleus->up_mb_pot == 0.)
					  neuron_tmp->its_nucleus->up_mb_pot = clu_bum->rest_pot;
					bum_dyn->soma = neuron_tmp->its_nucleus->low_mb_pot 
					  + (fabs(neuron_tmp->its_nucleus->low_mb_pot - neuron_tmp->its_nucleus->up_mb_pot)
					     * ran1(&set2));
					*/

					bum_dyn->soma = (clu_bum->overshoot
							  * exp((-neuron_tmp->decharge_lag) 
							  * clu_bum->i_tau_mb_pot))
							  + clu_bum->rest_pot;
					bum_dyn->soma_post_spike = clu_bum->pot_post_spike;
					bum_dyn->threshold = clu_bum->seuil_post_spike
							* exp((-neuron_tmp->decharge_lag) 
							* clu_bum->i_tau_seuil) /* deja un inverse */
							+ clu_bum->seuil;
					bum_dyn->seuil_peak = clu_bum->seuil_post_spike
								+ clu_bum->seuil;
				}

				if(neuron_tmp->its_cluster->model == 'L')
				{
					lim_dyn = (LIM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_lim = (LIM *)neuron_tmp->its_cluster->clu_data;
					lim_dyn->center_noise = 0.;
					if (neuron_tmp->its_nucleus->low_mb_pot == 0.)
					  neuron_tmp->its_nucleus->low_mb_pot = clu_lim->pot_post_spike;
					if (neuron_tmp->its_nucleus->up_mb_pot == 0.)
					  neuron_tmp->its_nucleus->up_mb_pot = clu_lim->rest_pot;
					lim_dyn->soma = neuron_tmp->its_nucleus->low_mb_pot 
					  + (fabs(neuron_tmp->its_nucleus->low_mb_pot - neuron_tmp->its_nucleus->up_mb_pot)
					     * ran1(&set2));
					/*
					lim_dyn->soma = clu_lim->rest_pot + 
					  (ran1(&set2) * (clu_lim->pot_post_spike - clu_lim->rest_pot));

					lim_dyn->soma = ((clu_lim->rest_pot - clu_lim->pot_post_spike)
							  * exp((-neuron_tmp->decharge_lag) 
							  * clu_lim->i_tau_mb_pot))
							  + clu_lim->rest_pot;
					*/
					lim_dyn->soma_post_spike = clu_lim->pot_post_spike;
					lim_dyn->threshold = clu_lim->seuil_post_spike
							* exp((-neuron_tmp->decharge_lag) 
							* clu_lim->i_tau_seuil) /* deja un inverse */
							+ clu_lim->seuil;
					lim_dyn->soma_post_spike = clu_lim->pot_post_spike;
					lim_dyn->seuil_peak = clu_lim->seuil_post_spike
								+ clu_lim->seuil;
				}

				if(neuron_tmp->its_cluster->model == 'P')
				{
					pum_dyn = (PUM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_pum = (PUM *)neuron_tmp->its_cluster->clu_data;
					pum_dyn->center_noise = 0.;

					pum_dyn->soma = (clu_pum->overshoot
							  * exp((-neuron_tmp->decharge_lag) 
							  * clu_pum->i_tau_mb_pot))
							  + clu_pum->rest_pot;

					pum_dyn->soma_post_spike = clu_pum->pot_post_spike;
					pum_dyn->threshold = clu_pum->seuil_post_spike
							* exp((-neuron_tmp->decharge_lag) 
							* clu_pum->i_tau_seuil) /* deja un inverse */
							+ clu_pum->seuil;
					pum_dyn->seuil_peak = clu_pum->seuil_post_spike
								+ clu_pum->seuil;

#if DEBUG
					printf("neuron %d: %f %f\n",i+1,pum_dyn->soma,pum_dyn->threshold);
#endif
				}
				if(neuron_tmp->its_cluster->model == 'G')
				{
					cbm_dyn = (CBM_DYNAMIC *)neuron_tmp->neuron_data;
					clu_cbm = (CBM *)neuron_tmp->its_cluster->clu_data;
				        init_G_neuron(i);
					clu_cbm = (CBM *)neuron_tmp->its_cluster->clu_data;
					if (neuron_tmp->its_nucleus->low_mb_pot == 0.)
					  neuron_tmp->its_nucleus->low_mb_pot = clu_cbm->rest_pot - 20.;
					if (neuron_tmp->its_nucleus->up_mb_pot == 0.)
					  neuron_tmp->its_nucleus->up_mb_pot = clu_cbm->rest_pot;
					cbm_dyn->soma = neuron_tmp->its_nucleus->low_mb_pot 
					  + (fabs(neuron_tmp->its_nucleus->low_mb_pot - neuron_tmp->its_nucleus->up_mb_pot)
					     * ran1(&set2));
				}
			}
#if DEBUG
			printf("Neurone %d initialise (alive sans spike)\n",i);
#endif
		}
		else if(first_config)
		{
			neuron_tmp->last_state = REPOS;
			neuron_tmp->decharge_lag = 100000.;
			neuron_tmp->spike_time[0] = -32000;
			neuron_tmp->tete = 0;
			neuron_tmp->queue = 0;
			for (j = 0; j < neuron_tmp->connected_neurons; j++)
				neuron_tmp->next_spike[j] = 0;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
			if(neuron_tmp->its_cluster->model == 'X')
			{
				xxm_dyn = (XXM_DYNAMIC *)neuron_tmp->neuron_data;
				clu_xxm = (XXM *)neuron_tmp->its_cluster->clu_data;

				xxm_dyn->soma = clu_xxm->rest_pot;
				xxm_dyn->soma_post_spike = clu_xxm->pot_post_spike;
				xxm_dyn->threshold = clu_xxm->seuil;
				xxm_dyn->seuil_peak = clu_xxm->seuil_post_spike
						+ clu_xxm->seuil;
			}

/***********************       STOP     ***********************/
#endif
			if(neuron_tmp->its_cluster->model == 'B')
			{
				bum_dyn = (BUM_DYNAMIC *)neuron_tmp->neuron_data;
				clu_bum = (BUM *)neuron_tmp->its_cluster->clu_data;

				bum_dyn->soma = clu_bum->rest_pot;
				bum_dyn->soma_post_spike = clu_bum->pot_post_spike;
				bum_dyn->threshold = clu_bum->seuil;
				bum_dyn->seuil_peak = clu_bum->seuil_post_spike
						+ clu_bum->seuil;
			}

			if(neuron_tmp->its_cluster->model == 'L')
			{
				lim_dyn = (LIM_DYNAMIC *)neuron_tmp->neuron_data;
				clu_lim = (LIM *)neuron_tmp->its_cluster->clu_data;

				lim_dyn->soma = clu_lim->rest_pot;
				lim_dyn->soma_post_spike = clu_lim->pot_post_spike;
				lim_dyn->threshold = clu_lim->seuil;
				lim_dyn->seuil_peak = clu_lim->seuil_post_spike
						+ clu_lim->seuil;
			}

			if(neuron_tmp->its_cluster->model == 'P')
			{
				pum_dyn = (PUM_DYNAMIC *)neuron_tmp->neuron_data;
				clu_pum = (PUM *)neuron_tmp->its_cluster->clu_data;

				pum_dyn->soma = clu_pum->rest_pot;
				pum_dyn->soma_post_spike = clu_pum->pot_post_spike;
				pum_dyn->threshold = clu_pum->seuil;
				pum_dyn->seuil_peak = clu_pum->seuil_post_spike
						+ clu_pum->seuil;
			}
			if(neuron_tmp->its_cluster->model == 'G')
			{
			        init_G_neuron(i);
			}
#if DEBUG
			printf("Neurone %d initialise (dead 1ere config)\n",i);
#endif
		}
		else
		{
			/* case first_config == False => some nucleus or cluster parameters
			may have changed... */			
		}
		
	}


}

/*******************************************************************************
	Function for running simulation
*******************************************************************************/
void simulate(void)
{	
	char buffer[256];
	Boolean  argok=False;
	int  i;


	if (!simu->started)
	{
		XtSetSensitive(widget_array[ID_pushB_time_step],False);
		XtSetSensitive(widget_array[ID_pushB_spike_duration],False);
	
		R_temp_ZF = R * (simu->Temp + 273.15) / (ZF);
	
			
		for(i=0 ; i<simu->nb_neurons ; i++)
		{
			if (sim_neurons[i]->its_cluster->model == 'G')
				init_G_neuron(i);
		}
	}
	running = True;
	stopped = False;
	simu->started = True;

	sprintf(buffer,
	"\n\nRunning simulation %s for %d iterations (press Stop to interrupt)...\n",
		simu->name,(simu->nb_next_it?simu->nb_next_it:1));
	_output(buffer);
	background_displaying = XtAppAddWorkProc(app_context,display_simulation,
		(XtPointer)(&running));
	background_running = XtAppAddWorkProc(app_context,run_simulation,
		(XtPointer)(&running));
}

/*******************************************************************************
	Function for stopping simulation while running
*******************************************************************************/
void stop_simulation(void)
{
	char buffer[256];
	Boolean  argok=False;
	
	running = False;
	stopped = True;

	status("Stopped (press Run to continue)...");

	simu->next_stop = simu->nb_it + (simu->nb_next_it? simu->nb_next_it: 1) ;
	simu->stop_at_ms = (float)simu->next_stop * simu->time_step;

	if(widget_array[ID_text_stop_main])
	{
	  XmTextFieldRemove(widget_array[ID_text_stop_main]);
	  sprintf(buffer,"%.3f ms",simu->stop_at_ms);
	  XmTextReplace(widget_array[ID_text_stop_main],0,
			XmTextGetLastPosition(widget_array[ID_text_stop_main]), buffer);
	}

}

/*******************************************************************************
	Function that do background computation for simulation
*******************************************************************************/
Boolean run_simulation(int * msg)
{
	char buffer[256];
	char	*buf;
	int	nb_star;
	int *_status = (int *)msg;
	int i,j;
	int x_i, x_r, y_i, y_r;
	Boolean	argok;
	XmString	xmstring;
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
	XXM_DYNAMIC *xxm_dyn;
	XXM *clu_xxm;
/***********************       STOP     ***********************/
#endif
	LIM_DYNAMIC *lim_dyn;
	LIM *clu_lim;
	BUM_DYNAMIC *bum_dyn;
	BUM *clu_bum;
	PUM *clu_pum;
	PUM_DYNAMIC *pum_dyn;
	CBM *clu_cbm;
	CBM_DYNAMIC *cbm_dyn;
	
	if(*_status == False)
		return True;
	
	buf = buffer;
	
	if(!simu->nb_it)
	{
		if (simu->output_sim_f)
		{
		  fprintf(sim_fp,"\n\"Total Act.\n");
		  for(i=0 ; i<simu->nb_nucleus ; i++)
		    {
		      fprintf(sim_fp,"\"%d %s\n",sim_nucleus[i]->id,sim_nucleus[i]->name);
		    }
		}
	}

#if NBC_X_DEBUG	
	if(debug)
	{
		fprintf(fdbg,"------------------------ITERATION No %lu-----------------------\n",
			simu->nb_it);
	}
#endif	

	/* here is the code for one simulation iteration */
	
	/*
	 *  modulation for each nucleus
	 */
	for(i = 0; i < simu->nb_nucleus; i++) 
	{
		
		if (sim_nucleus[i]->modul_type != 0)
		{
			sim_nucleus[i]->modul_value = modulation(sim_nucleus[i]);
#if DEBUG
			printf("%f\n",sim_nucleus[i]->modul_value);
#endif
#if NBC_X_DEBUG	
			if(debug)
			{
				fprintf(fdbg,"modul value for nucleus %s: %f\n",
					sim_nucleus[i]->name,sim_nucleus[i]->modul_value);
			}						 
#endif			
		} 
		else
			sim_nucleus[i]->modul_value = 0.;
	}
	
	/* update soma, neuron state and eventually .tms */
	
	for (i = 0; i < simu->nb_neurons ; i++)
	{
		(*sim_neurons[i]->its_cluster->update_soma)(i);
#if NBC_X_DEBUG
		if(debug)
		{
			fprintf(fdbg,"\tstate for neuron %ld: %s\n",
				sim_neurons[i]->id,(sim_neurons[i]->new_state? "!!!SPIKE!!!":"Repos"));
		}
#endif
	}

	if (simu->nb_it == simu->it_to_save)
	{
	        text_buffer = NULL;

		if(simu->nb_it)
		{
			if (simu->output_pmb_f)
			  fprintf(pmb_fp,"\n");
		}
		if (simu->output_pmb_f)
		  fprintf(pmb_fp,"\" iteration %.3f\n",simu->time_ms);

		/* reset active neurons */
		simu->total_active_neurons = 0;

		for(i = 0 ; i < simu->nb_nucleus ; i++)
			sim_nucleus[i]->nb_act_nucl = 0;
		for(i = 0 ; i < simu->nb_clusters ; i++)
			sim_clusters[i]->nb_act_clu = 0;
		
		/* update .pmb buffer */
			
		for(i = 0 ; i < simu->nb_neurons ; i++)
		{
			if(sim_neurons[i]->new_state == SPIKE)
			{
				sim_neurons[i]->its_nucleus->nb_act_nucl++;
				sim_neurons[i]->its_cluster->nb_act_clu++;
				simu->total_active_neurons++;

				switch(sim_neurons[i]->its_cluster->model)
				{
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				  /* not always necessary, decide, it for the graphs in visu */
				     case 'X':
					xxm_dyn = (XXM_DYNAMIC *)sim_neurons[i]->neuron_data;
					sprintf(buffer,"%ld %.1f %.1f %.1f\n",
						sim_neurons[i]->id,
						SPIKE_TOP,
						xxm_dyn->threshold,
						xxm_dyn->soma_post_spike);
					break;
/***********************       STOP     ***********************/
#endif
				     case 'B':
					bum_dyn = (BUM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f %.1f %.1f\n",
						  sim_neurons[i]->id,
						  SPIKE_TOP,
						  bum_dyn->threshold,
						  bum_dyn->soma_post_spike);
					break;
				
				     case 'L':
					lim_dyn = (LIM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f %.1f %.1f\n",
						  sim_neurons[i]->id,
						  SPIKE_TOP,
						  lim_dyn->threshold,
						  lim_dyn->soma_post_spike);
					break;

				     case 'P':
					pum_dyn = (PUM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f %.1f %.1f\n",
						  sim_neurons[i]->id,
						  SPIKE_TOP,
						  pum_dyn->threshold,
						  pum_dyn->soma_post_spike);
					break;

				     case 'G':
					cbm_dyn = (CBM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f\n",
						  sim_neurons[i]->id,
						  cbm_dyn->soma);
					break;

				     case 'V':
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f -50.0 -85.0\n",
						  sim_neurons[i]->id,
						  SPIKE_TOP);
				}
			} 
			else
			{
#if ADD_NEW_MODEL
/***************** TO ADD A MODEL COPY AND ADAPT **************/
				if(sim_neurons[i]->its_cluster->model == 'X')
				{
					xxm_dyn = (XXM_DYNAMIC *)sim_neurons[i]->neuron_data;
					sprintf(buffer,"%ld %.1f\n",
						sim_neurons[i]->id,
						xxm_dyn->soma);
				}

/***********************       STOP     ***********************/
#endif
				if(sim_neurons[i]->its_cluster->model == 'L')
				{
					lim_dyn = (LIM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f\n",
						  sim_neurons[i]->id,
						  lim_dyn->soma);
				}

				if(sim_neurons[i]->its_cluster->model == 'B')
				{
					bum_dyn = (BUM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f\n",
						  sim_neurons[i]->id,
						  bum_dyn->soma);
				}

				if(sim_neurons[i]->its_cluster->model == 'P')
				{
					pum_dyn = (PUM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f\n",
						  sim_neurons[i]->id,
						  pum_dyn->soma);
				}

				if(sim_neurons[i]->its_cluster->model == 'G')
				{
					cbm_dyn = (CBM_DYNAMIC *)sim_neurons[i]->neuron_data;
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld %.1f\n",
						  sim_neurons[i]->id,
						  cbm_dyn->soma);
				}
				if(sim_neurons[i]->its_cluster->model == 'V')
				{
					if (simu->output_pmb_f)
					  fprintf(pmb_fp,"%ld -70.\n",
						  sim_neurons[i]->id);
				}
			}
		}
		
		/* update .sim buffers */
	

		if (simu->output_sim_f)
		{
		  fprintf(sim_fp,"%lu\t%d\t0\n",simu->nb_it,simu->total_active_neurons);
		  for(i=0 ; i<simu->nb_nucleus ; i++)
		  {
		      fprintf(sim_fp,"%lu\t%d\t0\n",simu->nb_it,sim_nucleus[i]->nb_act_nucl);
		  }
		}

		if (simu->output_gnp_f)
       		{
		  fprintf(gnp_fp,"%lu\t%d",simu->nb_it,simu->total_active_neurons);
		  for(i=0 ; i<simu->nb_nucleus ; i++)
		  {
		      fprintf(gnp_fp,"\t%d",sim_nucleus[i]->nb_act_nucl);
		  }
		  fprintf(gnp_fp,"\n");
		}
	}

	/* AFFICHAGES / DISPLAYS */

	y_r = simu->nb_neurons + 100;
	if (simu->nb_it == simu->it_to_watch && simu->watch_sim)
	{
	        x_i = simu->nb_it%graph_area_width;
		x_r = x_i + 6;

/*	Dessine la ligne noire - inutile au debut, la lgne blanche suffit ensuite */

		XDrawLine(the_display,the_drawable,gc_red,x_r,y_r,x_r,0);
		XDrawLine(the_display,the_drawable,gc_white,x_i,y_r,x_i,0);

		if (simu->look_at_dot_display)
		{
		  for(i = 0 ; i < simu->nb_neurons ; i++)
		  {
		    y_i = i + 101;
		    if(sim_neurons[i]->new_state == SPIKE)
		    {
		      XDrawLine(the_display,the_drawable,gc_dot,x_i,y_i,x_i+1,y_i);
		    }
		  }
		}
		if (simu->look_at_glob_act)
		{
		  nb_star = (simu->total_active_neurons * 100) / simu->nb_neurons;
		  XDrawLine(the_display,the_drawable,gc_black,x_i,100,x_i,(100-nb_star));

		} /* fin du if */
	}

	if (simu->nb_it == simu->it_to_show)
	{
		if (simu->look_at_ms_count)
		{
			sprintf(buffer,"%.3f ms", simu->time_ms);
		}
		else
		{
			sprintf(buffer,"%lu it", simu->nb_it);
		}
		xmstring = XmStringCreateLocalized(buffer);
		XtVaSetValues(widget_array[ID_label_iterations],
			      XmNlabelString, xmstring, NULL);
		XmStringFree(xmstring);
	} /* fin du if */

	/* update text display */


	/* time to stop ? */
		
	if(simu->nb_it == simu->next_stop)
	{
		simu->next_stop = simu->nb_it + (simu->nb_next_it? simu->nb_next_it: 1);
		simu->stop_at_ms = (float)simu->next_stop * simu->time_step;
		running = False;
		stopped = True;
		if(!batch)
		{
			status("Finished (press Run for next run)...");
			XtVaSetValues(widget_array[ID_pushB_STOP],
				      XmNlabelString, CONVERT(widget_array[ID_pushB_STOP], "RUN", XmRXmString, 0, &argok),
				      XmNsensitive,True,
				      XtVaTypedArg, XmNbackground, XmRString, "green", 6,
				      NULL);
			XtRemoveCallback(widget_array[ID_pushB_STOP],XmNactivateCallback,sim_activate_CB,
					 (XtPointer)ID_pushB_interrupt);
			XtAddCallback(widget_array[ID_pushB_STOP],XmNactivateCallback,sim_activate_CB,
				      (XtPointer)ID_pushB_run_simulation);
			if(widget_array[ID_text_stop_main])
			{
			        XmTextFieldRemove(widget_array[ID_text_stop_main]);
				sprintf(buffer,"%.3f ms",simu->stop_at_ms);
				XmTextReplace(widget_array[ID_text_stop_main],0,
					      XmTextGetLastPosition(widget_array[ID_text_stop_main]), buffer);
			}
		}
	}
	for(i = 0 ; i < simu->nb_neurons ; i++)
	{
	  sim_neurons[i]->last_state = sim_neurons[i]->new_state;
	  sim_neurons[i]->new_state = REPOS; 
	}

	if (simu->nb_it == simu->it_to_save)
	        simu->it_to_save += simu->nb_step_save_it;
	if (simu->nb_it == simu->it_to_watch)
	        simu->it_to_watch += simu->nb_step_watch_it;
	if (simu->nb_it == simu->it_to_show)
	        simu->it_to_show += simu->count_show_it;
	simu->nb_it++;
	simu->t_nb_it = (float)simu->nb_it;
	simu->time_ms = simu->time_ms + simu->time_step;

	return False;

}

/*******************************************************************************
	Function that do background displaying of simulation
*******************************************************************************/
Boolean display_simulation(XtPointer msg)
{
	char buffer[256];
	int *status = (int *)msg;

	if(text_buffer)	
	{
		_output(text_buffer);
		free(text_buffer);
	}
	text_buffer = NULL;
	
	if(*status == False)
		return True;
			
	return False;
	
}
/*******************************************************************************
	Function for managing and displaying the help
*******************************************************************************/
void display_help(int flag)
{
	
	int count;
	int *pos;
	FILE *fp;
	char *pt;
	char file_name[256];
	XmString string;
	char buffer[256];
	static char **help_files;
	static char **topics;
	Widget w;
	static int nb_items;
	char *text;
	int i;
	long l;

#ifdef __CYGWIN__
#	define SEP_DIR "\\"
#	define CR 2
#else
#	define SEP_DIR "/"
#	define CR 1
#endif
	
	if(flag == -1)
	{
		for(i=0 ; i<nb_items ; i++)
		{
			free(help_files[i]);
			free(topics[i]);
		}
		free(help_files);
		free(topics);
		return;
	}
	
	if(!nb_items)
	{
		strcpy(file_name,help_directory);
		strcat(file_name,SEP_DIR);
		strcat(file_name,"sim_hlprc");
		
		if(!(fp = fopen(file_name,"r")))
		{
#if DEBUG
			sprintf(buffer,"Sorry, file %s not found...",file_name);
			display_error(widget_array[ID_sim_mainW],buffer,
				"No Help Available !");
#endif
			return;
		}
		
		nb_items = 0;
		
		while(fgets(buffer,255,fp))
		{	
			pt = strrchr(buffer,':');
			if(!pt)
				continue;
			nb_items++;
			*pt = '\0';
			pt += 2;
			if(nb_items == 1)
			{
				help_files = (char **)calloc(1,nb_items*sizeof(char *));
				topics = (char **)calloc(1,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}
			else
			{
				help_files = (char **)realloc(help_files,nb_items*sizeof(char *));
				topics = (char **)realloc(topics,nb_items*sizeof(char *));
				topics[nb_items - 1] = (char *)calloc(1,(strlen(buffer)+1)*sizeof(char));
				strcpy(topics[nb_items - 1],buffer);
				help_files[nb_items - 1] = (char *)calloc(1,(strlen(pt)+1)*sizeof(char));
				strcpy(help_files[nb_items - 1],pt);
				help_files[nb_items - 1][strlen(pt)- CR] = '\0';
			}	
		}
		
		for(i=0 ; i<nb_items ; i++)
		{
			string = XmStringCreateLocalized(topics[i]);
			w = XtVaCreateWidget(topics[i],
				xmPushButtonWidgetClass,
				widget_array[ID_topic_pulldownMenu],NULL);
		
			XtAddCallback(w, XmNactivateCallback, help_CB, (XtPointer)(i+1));
			XtManageChild(w);
			XmStringFree(string);
		}
		flag = 1;	
	}

	strcpy(file_name,help_directory);
	strcat(file_name,SEP_DIR);
	strcat(file_name,help_files[flag-1]);
		
	if(!(fp = fopen(file_name,"r")))
	{
		sprintf(buffer,"Sorry, file %s not found...",file_name);
		display_error(widget_array[ID_help_form],buffer,
			"Topic Not Available !");
		return;
	}
	
	
	fseek(fp, 0L, SEEK_END);
	l = ftell(fp)+1;
	if (!(text = (char *)calloc(1,l*sizeof(char))))
	{
		sprintf(buffer,"Sorry, not possible to load the help on %s...",file_name);
		display_error(widget_array[ID_help_form],buffer,
			"Not enough memory !");
		return;
	} /* fin du if */
	fseek(fp, 0L, SEEK_SET);
	fread(text, l*sizeof(char), 1, fp);
	text[l-1] = '\0';
	
	XmTextSetString(widget_array[ID_help_text],text);
	
	string = XmStringCreateLocalized(topics[flag-1]);
	XtVaSetValues(widget_array[ID_topic_label],XmNlabelString,string,NULL);
	XmStringFree(string);
	
	
	free(text);		

}


#endif


